@@ -137,7 +137,7 @@ LWRaise(
     {
         shortMsg = "Undocumented exception";
     }
-    if ((ceError = CTAllocateString(desc, &shortMsg)));
+    if ((ceError = CTAllocateString(desc, &shortMsg)))
     {
 	*dest = CreateException(ceError, __FILE__, __LINE__, NULL, NULL);
 	return;
@@ -152,7 +152,7 @@ LWRaise(
 	*dest = CreateException(ceError, __FILE__, __LINE__, NULL, NULL);
 	return;
     }
-
+    
     *dest = CreateException(code, NULL, 0, shortMsg, longMsg);
 }
 
@@ -287,7 +287,7 @@ LWHandle(
         {
             LWStackFrame* frame;
             LWStackFrame* nextFrame;
-
+            
             for (frame = (*exc)->stack.down; frame; frame = nextFrame)
             {
                 nextFrame = frame->down;
