@@ -92,7 +92,8 @@ bind_server(
 
 static void usage()
 {
-    printf("usage: echo_server [-e endpoint] [-n] [-u] [-t]\n");
+    printf("usage: echo_server [-a name] [-e endpoint] [-n] [-u] [-t]\n");
+    printf("         -a:  specify authentication identity\n");
     printf("         -e:  specify endpoint\n");
     printf("         -n:  use named pipe protocol\n");
     printf("         -u:  use UDP protocol\n");
@@ -111,15 +112,19 @@ int main(int argc, char *argv[])
     char * endpoint = NULL;
     int c;
     char * function = NULL;
+    char * spn = NULL;
 
     /*
      * Process the cmd line args
      */
 
-    while ((c = getopt(argc, argv, "e:nut")) != EOF)
+    while ((c = getopt(argc, argv, "a:e:nut")) != EOF)
     {
         switch (c)
         {
+        case 'a':
+            spn = optarg;
+            break;
         case 'e':
             endpoint = optarg;
             break;
@@ -151,6 +156,21 @@ int main(int argc, char *argv[])
     }
 #endif
 
+    if (spn)
+    {
+        rpc_server_register_auth_info(
+            spn,
+            rpc_c_authn_gss_negotiate,
+            NULL,
+            NULL,
+            &status);
+        if (status)
+        {
+            printf ("Couldn't set auth info. exiting.\n");
+            exit(1);
+        }
+    }
+
     /*
      * Register the Interface with the local endpoint mapper (rpcd)
      */
@@ -276,6 +296,13 @@ ReverseIt(
     unsigned result_size;
     args * result;
     unsigned32 i,j,l;
+#if 0
+    rpc_transport_info_handle_t transport_info = NULL;
+#endif
+    unsigned32 rpcstatus = 0;
+    unsigned char* sesskey = NULL;
+    unsigned32 sesskey_len = 0;
+    unsigned char* principal_name = NULL;
 
     /*
      * Get some info about the client binding
@@ -287,6 +314,24 @@ ReverseIt(
         printf ("ReverseIt() called by client: %s\n", binding_info);
     }
 
+#if 0
+    rpc_binding_inq_transport_info(h, &transport_info, &rpcstatus);
+
+    if (transport_info)
+    {
+        rpc_smb_transport_info_inq_session_key(transport_info, &sesskey, &sesskey_len);
+
+        printf ("Session key: ");
+
+        for (i = 0; i < sesskey_len; i++)
+        {
+            printf("%X", sesskey[i]);
+        }
+
+        printf ("\n");
+    }
+#endif
+
     if (in_text == NULL) return 0;
 
     /*
