@@ -125,11 +125,15 @@ static void CSPELL_constant_def
                 fprintf (fid, "NULL");
                 break;
             case AST_int_const_k:
-					 /* handle the signed-ness of the constant */
-					 if (cp->int_signed)
-                	fprintf (fid, "(%ld)", cp->value.int_val);
-					 else
-						fprintf (fid, "(0x%lx)", cp->value.int_val); /* prevent signed-edness problems with cxx */
+                /* handle the signed-ness of the constant */
+                if (cp->int_signed)
+                    fprintf (fid, "(%ld)", cp->value.int_val);
+                else if (sizeof(long) == 4)
+                    fprintf (fid, "(0x%lx)", (unsigned long) cp->value.int_val);
+                else if (sizeof(int) == 4)
+                    fprintf (fid, "(0x%x)", (unsigned int) cp->value.int_val);
+                else
+                    abort();
                 break;
             case AST_hyper_int_const_k:
                 fprintf (fid, "{%ld,%lu}",
@@ -167,7 +171,7 @@ static void CSPELL_operation_def
     char op_internal_name[3 * MAX_ID];
     AST_type_n_t       func_type_node;
     NAMETABLE_id_t emitted_name;
-
+    
     func_type_node = *BE_function_p;
     func_type_node.type_structure.function = op;
 
@@ -183,7 +187,7 @@ static void CSPELL_operation_def
     {
         sprintf(op_internal_name, "%s%s", sstub_pref, BE_get_name(op->name));
         emitted_name = NAMETABLE_add_id(op_internal_name);
-
+        
         fprintf (fid, "extern ");
         CSPELL_typed_name (fid, &func_type_node, emitted_name, NULL, false, true,
                            (AST_ENCODE_SET(op) || AST_DECODE_SET(op)));
