@@ -139,7 +139,7 @@ void rpc_ss_init_callee_ctx_tables(
 ** The memset below has the same effect as this more descriptive loop.
 ** 
 **     for (i = 0; i < CALLEE_CONTEXT_TABLE_SIZE; i++) {
-**         uuid_create_nil(&context_table[i].uuid, &status);
+**         dce_uuid_create_nil(&context_table[i].uuid, &status);
 **         context_table[i].next_context = NULL;
 **     }
 ** 
@@ -164,7 +164,7 @@ void rpc_ss_create_callee_context
 #ifdef IDL_PROTOTYPES
 (
     rpc_ss_context_t callee_context,/* The user's local form of the context */
-    uuid_t    *p_uuid,              /* Pointer to the equivalent UUID */
+    dce_uuid_t    *p_uuid,              /* Pointer to the equivalent UUID */
     handle_t h,                     /* Binding handle */
     ctx_rundown_fn_p_t ctx_rundown, /* Pointer to context rundown routine */
     error_status_t *result     /* Function result */
@@ -172,7 +172,7 @@ void rpc_ss_create_callee_context
 #else
 (callee_context, p_uuid, h, ctx_rundown, result)
     rpc_ss_context_t callee_context;/* The user's local form of the context */
-    uuid_t    *p_uuid;              /* Pointer to the equivalent UUID */
+    dce_uuid_t    *p_uuid;              /* Pointer to the equivalent UUID */
     handle_t h;                     /* Binding handle */
     ctx_rundown_fn_p_t ctx_rundown; /* Pointer to context rundown routine */
     error_status_t *result;    /* Function result */
@@ -196,9 +196,9 @@ void rpc_ss_create_callee_context
 
     RPC_SS_THREADS_MUTEX_LOCK(&rpc_ss_context_table_mutex);
     DPRINT(("Seized context tables\n"));
-    this_link = &context_table[uuid_hash(p_uuid,(error_status_t *)result)
+    this_link = &context_table[dce_uuid_hash(p_uuid,(error_status_t *)result)
                                                % CALLEE_CONTEXT_TABLE_SIZE];
-    if ( uuid_is_nil(&this_link->uuid, (error_status_t *)result) )
+    if ( dce_uuid_is_nil(&this_link->uuid, (error_status_t *)result) )
     {
         /* Home slot in the hash table is empty */
         new_link = this_link;
@@ -223,7 +223,7 @@ void rpc_ss_create_callee_context
     memcpy(
         (char *)&new_link->uuid,
         (char *)p_uuid,
-        sizeof(uuid_t)
+        sizeof(dce_uuid_t)
     );
     new_link->user_context = callee_context;
     new_link->rundown = ctx_rundown;
@@ -257,13 +257,13 @@ void rpc_ss_update_callee_context
 #ifdef IDL_PROTOTYPES
 (
     rpc_ss_context_t    callee_context, /* The user's local form of the context */
-    uuid_t              *p_uuid,        /* Pointer to the equivalent UUID */
+    dce_uuid_t              *p_uuid,        /* Pointer to the equivalent UUID */
     error_status_t      *result         /* Function result */
 )
 #else
 (callee_context, p_uuid, result)
     rpc_ss_context_t    callee_context; /* The user's local form of the context */
-    uuid_t              *p_uuid;        /* Pointer to the equivalent UUID */
+    dce_uuid_t              *p_uuid;        /* Pointer to the equivalent UUID */
     error_status_t      *result;        /* Function result */
 #endif
 {
@@ -275,9 +275,9 @@ void rpc_ss_update_callee_context
 
     RPC_SS_THREADS_MUTEX_LOCK(&rpc_ss_context_table_mutex);
     DPRINT(("Seized context tables\n"));
-    this_link = &context_table[uuid_hash(p_uuid,result)
+    this_link = &context_table[dce_uuid_hash(p_uuid,result)
                                                % CALLEE_CONTEXT_TABLE_SIZE];
-    while ( ! uuid_equal(p_uuid,&this_link->uuid,result) )
+    while ( ! dce_uuid_equal(p_uuid,&this_link->uuid,result) )
     {
         this_link = this_link->next_context;
         if (this_link == NULL)
@@ -319,7 +319,7 @@ void rpc_ss_ee_ctx_from_wire
     volatile error_status_t *p_st;
 #endif
 {
-    uuid_t *p_uuid;    /* Pointer to the UUID that has come off the wire */
+    dce_uuid_t *p_uuid;    /* Pointer to the UUID that has come off the wire */
     callee_context_entry_t *this_link;
 
 #ifdef PERFMON
@@ -334,7 +334,7 @@ void rpc_ss_ee_ctx_from_wire
 #endif
 
     *p_st = error_status_ok;
-    if ( uuid_is_nil(p_uuid, (error_status_t *)p_st) )
+    if ( dce_uuid_is_nil(p_uuid, (error_status_t *)p_st) )
     {
         *p_context = NULL;
 
@@ -346,9 +346,9 @@ void rpc_ss_ee_ctx_from_wire
     }
     RPC_SS_THREADS_MUTEX_LOCK(&rpc_ss_context_table_mutex);
     DPRINT(("Seized context tables\n"));
-    this_link = &context_table[uuid_hash(p_uuid, (error_status_t *)p_st)
+    this_link = &context_table[dce_uuid_hash(p_uuid, (error_status_t *)p_st)
                                                % CALLEE_CONTEXT_TABLE_SIZE];
-    while ( ! uuid_equal(p_uuid,&this_link->uuid, (error_status_t *)p_st) )
+    while ( ! dce_uuid_equal(p_uuid,&this_link->uuid, (error_status_t *)p_st) )
     {
         this_link = this_link->next_context;
         if (this_link == NULL)
@@ -383,13 +383,13 @@ void rpc_ss_ee_ctx_from_wire
 void rpc_ss_destroy_callee_context
 #ifdef IDL_PROTOTYPES
 (
-    uuid_t *p_uuid,             /* Pointer to UUID of context to be destroyed */
+    dce_uuid_t *p_uuid,             /* Pointer to UUID of context to be destroyed */
     handle_t  h,                /* Binding handle */
     error_status_t *result /* Function result */
 )    /* Returns error_status_ok unless the UUID is not in the lookup table */
 #else
 (p_uuid, h, result)
-    uuid_t *p_uuid;             /* Pointer to UUID of context to be destroyed */
+    dce_uuid_t *p_uuid;             /* Pointer to UUID of context to be destroyed */
     handle_t  h;                /* Binding handle */
     error_status_t *result;/* Function result */
 #endif
@@ -424,14 +424,14 @@ void rpc_ss_destroy_callee_context
 void rpc_ss_lkddest_callee_context
 #ifdef IDL_PROTOTYPES
 (
-    uuid_t *p_uuid,    /* Pointer to UUID of context to be destroyed */
+    dce_uuid_t *p_uuid,    /* Pointer to UUID of context to be destroyed */
     rpc_client_handle_t *p_close_client,
                                 /* Ptr to NULL or client to stop monitoring */
     error_status_t *result /* Function result */
 )    /* Returns error_status_ok unless the UUID is not in the lookup table */
 #else
 (p_uuid, p_close_client, result)
-    uuid_t *p_uuid;    /* Pointer to UUID of context to be destroyed */
+    dce_uuid_t *p_uuid;    /* Pointer to UUID of context to be destroyed */
     rpc_client_handle_t *p_close_client;
                                 /* Ptr to NULL or client to stop monitoring */
     error_status_t *result;/* Function result */
@@ -443,17 +443,17 @@ void rpc_ss_lkddest_callee_context
     RPC_SS_LKDDEST_CALLEE_CONTEXT_N;
 #endif
 
-    this_link = &context_table[uuid_hash(p_uuid,(error_status_t *) result)
+    this_link = &context_table[dce_uuid_hash(p_uuid,(error_status_t *) result)
                                                % CALLEE_CONTEXT_TABLE_SIZE];
     next_link = this_link->next_context;
-    if ( uuid_equal(p_uuid,&this_link->uuid, (error_status_t *) result) )
+    if ( dce_uuid_equal(p_uuid,&this_link->uuid, (error_status_t *) result) )
     {
         /* Context to be destroyed is in home slot */
         rpc_ss_take_from_callee_client(this_link,p_close_client,result);
         if (next_link == NULL)
         {
             /* There is no chain from the home slot */
-            uuid_create_nil(&this_link->uuid, (error_status_t *) result);
+            dce_uuid_create_nil(&this_link->uuid, (error_status_t *) result);
         }
         else
         {
@@ -461,7 +461,7 @@ void rpc_ss_lkddest_callee_context
             memcpy(
                 (char *)&this_link->uuid,
                 (char *)&next_link->uuid,
-                sizeof(uuid_t)
+                sizeof(dce_uuid_t)
             );
             this_link->user_context = next_link->user_context;
             this_link->rundown = next_link->rundown;
@@ -502,7 +502,7 @@ void rpc_ss_lkddest_callee_context
             last_link = this_link;
             this_link = next_link;
             next_link = this_link->next_context;
-            if ( uuid_equal(p_uuid,&this_link->uuid,(error_status_t *)result) )
+            if ( dce_uuid_equal(p_uuid,&this_link->uuid,(error_status_t *)result) )
             {
                 rpc_ss_take_from_callee_client(this_link,p_close_client,result);
                 /* Relink chain to omit found entry */
@@ -534,7 +534,7 @@ void dump_context_table()
 
     for (i=0; i<CALLEE_CONTEXT_TABLE_SIZE; i++)
     {
-        if ( ! uuid_is_nil(&context_table[i].uuid, &status) )
+        if ( ! dce_uuid_is_nil(&context_table[i].uuid, &status) )
         {
             this_link = &context_table[i];
             printf("Context chain for context_slot %d\n",i);
@@ -572,7 +572,7 @@ static int debug_context_lookup(uuid_p)
     }
 
     fprintf(debug_fid, "L");
-    for (j=0; j<sizeof(uuid_t); j++)
+    for (j=0; j<sizeof(dce_uuid_t); j++)
     {
         k = *uuid_p++;
         fprintf(debug_fid, " %02x", k);
@@ -610,7 +610,7 @@ static int debug_context_table()
 
     for (i=0; i<CALLEE_CONTEXT_TABLE_SIZE; i++)
     {
-        if ( ! uuid_is_nil(&context_table[i].uuid, &status) )
+        if ( ! dce_uuid_is_nil(&context_table[i].uuid, &status) )
         {
             at_home = ndr_true;
             this_link = &context_table[i];
@@ -695,7 +695,7 @@ void rpc_ss_ee_ctx_to_wire
     RPC_SS_EE_CTX_TO_WIRE_N;
 #endif
 
-    wire    = in_out && !uuid_is_nil(
+    wire    = in_out && !dce_uuid_is_nil(
                 &p_wire_context->context_handle_uuid, (error_status_t *)p_st
             )? 4: 0;
     context = callee_context? 2: 0;
@@ -703,7 +703,7 @@ void rpc_ss_ee_ctx_to_wire
 
     switch (wire | context | in_out) {
     case 0:
-        uuid_create_nil(
+        dce_uuid_create_nil(
             &p_wire_context->context_handle_uuid, (error_status_t *)p_st
         );
         break;
@@ -712,7 +712,7 @@ void rpc_ss_ee_ctx_to_wire
         break;
     case 2:
     case 3:
-        uuid_create(
+        dce_uuid_create(
             &p_wire_context->context_handle_uuid, (error_status_t *)p_st
         );
         rpc_ss_create_callee_context(
@@ -725,7 +725,7 @@ void rpc_ss_ee_ctx_to_wire
             &p_wire_context->context_handle_uuid, h, (error_status_t *)p_st
         );
         if (*p_st != error_status_ok) break;
-        uuid_create_nil(
+        dce_uuid_create_nil(
             &p_wire_context->context_handle_uuid, (error_status_t *)p_st
         );
         break;
