@@ -167,7 +167,6 @@ void rpc_ss_ndr_unmar_cs_array
     idl_ulong_int cs_type_defn_index;
     idl_byte *cs_type_defn_ptr;
     idl_ulong_int routine_index;
-    idl_byte ln_type;           /* Data type of [length_is] item */
     idl_ulong_int ln_index;     /* Index in shadow of [length_is] item */
     idl_ulong_int sz_index;     /* Index in shadow of [size_is] item */
     void (**routine_ptr)();
@@ -218,7 +217,6 @@ void rpc_ss_ndr_unmar_cs_array
         idl_ulong_int lower;
         /* Skip over lower data limit entirely and upper data limit kind */
         array_defn_ptr += IDL_DATA_LIMIT_PAIR_WIDTH/2 + 1;
-        ln_type = *array_defn_ptr;
         IDL_GET_LONG_FROM_VECTOR(ln_index, array_defn_ptr);
         ln_index--;
         IDL_UNMAR_ULONG(&lower);     /* A-value */
@@ -808,7 +806,6 @@ void rpc_ss_alloc_out_cs_conf_array
     idl_byte array_type;        /* DT_CONF_ARRAY or DT_OPEN_ARRAY */
     idl_ulong_int array_defn_index;
     idl_byte *array_defn_ptr;
-    idl_ulong_int dimensionality;
     idl_ulong_int Z_value;
     idl_byte sz_type;           /* Data type of [size_is] item */
     idl_ulong_int szp_index;    /* Index in parameter list of [size_is] item */
@@ -822,8 +819,7 @@ void rpc_ss_alloc_out_cs_conf_array
                                             /* Discard full array definition */
     IDL_GET_LONG_FROM_VECTOR(array_defn_index,type_vec_ptr);
     array_defn_ptr = IDL_msp->IDL_type_vec + array_defn_index;
-    dimensionality = (idl_ulong_int)*array_defn_ptr;
-    array_defn_ptr++;
+    array_defn_ptr++; /* Skip dimensionality */
 
     /* Skip over lower bound entirely and upper bound kind */
     array_defn_ptr += IDL_CONF_BOUND_PAIR_WIDTH/2 + 1;
