@@ -39,8 +39,6 @@
 **
 */
 
-#include <npctypes.h>
-
 
 /******************************************************************************/
 /*
@@ -88,7 +86,7 @@ static void RPC_CN_ASSOC_ACB_DEC_REF(rpc_cn_assoc_t *assoc)
  */
 #if (uuid_c_version == 1)
 #define RPC_CN_ASSOC_SYNTAX_EQUAL(s1, s2, st)\
-    ((memcmp (&((s1)->id), &((s2)->id), sizeof (uuid_t)) == 0) &&\
+    ((memcmp (&((s1)->id), &((s2)->id), sizeof (dce_uuid_t)) == 0) &&\
      (((s1)->version & 0xFFFF) == ((s2)->version & 0xFFFF)) &&\
      (((s1)->version >> 16) == ((s2)->version >> 16)))
 #else
@@ -149,9 +147,6 @@ error "***Make sure memcmp works on this version of UUIDs***"
     RPC_CALL_UNLOCK (((rpc_call_rep_t *) assoc->call_rep));\
 }
  
-PRIVATE NPC_TOKEN_ID rpc__get_current_token_id(
-    unsigned32 *st);
-
 
 /******************************************************************************/
 /*
@@ -404,7 +399,7 @@ PRIVATE void rpc__cn_assoc_acb_dealloc _DCE_PROTOTYPE_ ((rpc_cn_assoc_p_t/* asso
 
 PRIVATE rpc_cn_local_id_t rpc__cn_assoc_grp_alloc _DCE_PROTOTYPE_ ((
     rpc_addr_p_t             /* rpc_addr */,
-    NPC_TOKEN_ID             /* token_id */,
+    rpc_transport_info_p_t    /* prot_info */,
     unsigned32               /* type */,
     unsigned32               /* rem_id */,                                                             
     unsigned32              * /* st */ ));
@@ -438,7 +433,7 @@ PRIVATE void rpc__cn_assoc_grp_rem_assoc _DCE_PROTOTYPE_ ((
 
 PRIVATE rpc_cn_local_id_t rpc__cn_assoc_grp_lkup_by_addr _DCE_PROTOTYPE_ ((
     rpc_addr_p_t                 /* rpc_addr */,
-    NPC_TOKEN_ID                 /* imp_token */,
+    rpc_transport_info_p_t       /* transport_info */,
     unsigned32                   /* type */,
     unsigned32                  * /* st */ ));
 
@@ -459,6 +454,7 @@ PRIVATE rpc_cn_local_id_t rpc__cn_assoc_grp_lkup_by_remid _DCE_PROTOTYPE_ ((
 PRIVATE rpc_cn_local_id_t rpc__cn_assoc_grp_lkup_by_id _DCE_PROTOTYPE_ ((
     rpc_cn_local_id_t            /* grp_id */,
     unsigned32                   /* type */,
+    rpc_transport_info_p_t  transport_info,
     unsigned32                  * /* st */ ));
 
 /*
