@@ -1,5 +1,5 @@
 /*
- *
+ * 
  * (c) Copyright 1990 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1990 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1990 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- *
+ * 
  */
 /*
  */
@@ -72,7 +72,7 @@
 
 /*
  * Macro to forward, if the first frag has been sent, and queued
- * cancels.
+ * cancels. 
  */
 #define RPC_CN_FORWARD_QUEUED_CANCELS(call_r, st) \
 { \
@@ -80,7 +80,7 @@
     { \
         rpc__cn_call_forward_cancel (call_r, st); \
     } \
-}
+}    
 
 /*
  * Macro to check for a pending cancel and forward, if the first frag
@@ -91,7 +91,7 @@
     RPC_CN_CHECK_FOR_CANCEL (call_r); \
     RPC_CN_FORWARD_QUEUED_CANCELS (call_r, st); \
 }
-
+    
 INTERNAL unsigned32 rpc__cn_call_cvt_from_nca_st _DCE_PROTOTYPE_ ((
         unsigned32      /*a_st*/
     ));
@@ -133,7 +133,7 @@ INTERNAL boolean rpc__cn_call_cancel_timer _DCE_PROTOTYPE_ ((
 **
 **  This is the NCA connection oriented protocol specific routine
 **  which begins a remote procedure call.  It returns the information
-**  needed to marshal input arguments.  This routine is intended
+**  needed to marshal input arguments.  This routine is intended 
 **  for use by the client stub only.
 **
 **  INPUTS:
@@ -169,7 +169,7 @@ INTERNAL boolean rpc__cn_call_cancel_timer _DCE_PROTOTYPE_ ((
 **--
 **/
 
-PRIVATE rpc_call_rep_t *rpc__cn_call_start
+PRIVATE rpc_call_rep_t *rpc__cn_call_start 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r,
@@ -275,7 +275,7 @@ unsigned32              *st;
     if (((rpc_cn_binding_rep_t *)binding_r)->being_resolved)
     {
         current_thread_id = dcethread_self ();
-        resolving_thread = dcethread_equal
+        resolving_thread = dcethread_equal 
             (((rpc_cn_binding_rep_t *)binding_r)->resolving_thread_id,
              current_thread_id);
         if (!resolving_thread)
@@ -283,7 +283,7 @@ unsigned32              *st;
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                     ("CN: call_rep->none waiting on binding resolution\n"));
             /*
-             * Serialize calls on this binding handle until the handle
+             * Serialize calls on this binding handle until the handle 
              * has an endpoint.
              */
             RPC_BINDING_CALL_START ((rpc_binding_rep_t *) binding_r);
@@ -324,7 +324,7 @@ unsigned32              *st;
             ((rpc_cn_binding_rep_t *)binding_r)->resolving_thread_id = dcethread_self ();
 
             RPC_CN_UNLOCK ();
-            rpc_ep_resolve_binding ((rpc_binding_handle_t) binding_r,
+            rpc_ep_resolve_binding ((rpc_binding_handle_t) binding_r, 
                                     (rpc_if_handle_t) ifspec_r, st);
             RPC_CN_LOCK ();
             ((rpc_cn_binding_rep_t *)binding_r)->being_resolved = false;
@@ -339,10 +339,10 @@ unsigned32              *st;
             }
 
             /*
-             * If the attempt to resolve the binding handle was
+             * If the attempt to resolve the binding handle was 
              * unsuccessful return.
              */
-            if (*st != rpc_s_ok)
+            if (*st != rpc_s_ok) 
             {
                 RPC_CN_UNLOCK ();
                 return (NULL);
@@ -354,7 +354,7 @@ unsigned32              *st;
      * At this point, we have a fully-bound binding.
      * We can now allocate a call_rep.
      */
-    call_rep = (rpc_cn_call_rep_p_t)
+    call_rep = (rpc_cn_call_rep_p_t) 
         rpc__list_element_alloc (&rpc_g_cn_call_lookaside_list, true);
     if (call_rep == NULL)
     {
@@ -408,7 +408,7 @@ unsigned32              *st;
     call_rep->call_executed = false;
 
     /*
-     * Initialize some cancel state information.
+     * Initialize some cancel state information. 
      */
     call_rep->u.client.cancel.server_is_accepting = false;
     call_rep->u.client.cancel.server_had_pending = false;
@@ -471,7 +471,7 @@ unsigned32              *st;
     iov_p = &(RPC_CN_CREP_IOV (call_rep)[0]);
 
     /*
-     * The start of stub data will depend upon whether the object
+     * The start of stub data will depend upon whether the object 
      * UUID field is present.
      */
     if (memcmp (&binding_r->obj, &uuid_g_nil_uuid, sizeof (dce_uuid_t)) != 0)
@@ -482,10 +482,10 @@ unsigned32              *st;
         RPC_CN_PKT_OBJECT(header_p) = binding_r->obj;
         RPC_CN_PKT_FLAGS (header_p) |= RPC_C_CN_FLAGS_OBJECT_UUID;
         RPC_CN_CREP_SIZEOF_HDR (call_rep) = RPC_CN_PKT_SIZEOF_RQST_HDR_W_OBJ;
-        RPC_CN_CREP_FREE_BYTES (call_rep) =
+        RPC_CN_CREP_FREE_BYTES (call_rep) = 
             RPC_C_CN_SMALL_FRAG_SIZE - RPC_CN_PKT_SIZEOF_RQST_HDR_W_OBJ;
         RPC_CN_CREP_ACC_BYTCNT (call_rep) = RPC_CN_PKT_SIZEOF_RQST_HDR_W_OBJ;
-        RPC_CN_CREP_FREE_BYTE_PTR(call_rep) =
+        RPC_CN_CREP_FREE_BYTE_PTR(call_rep) = 
         RPC_CN_PKT_RQST_STUB_DATA_W_OBJ (header_p);
         iov_p->data_len = RPC_CN_PKT_SIZEOF_RQST_HDR_W_OBJ;
     }
@@ -495,7 +495,7 @@ unsigned32              *st;
          * There is no object UUID (it is nil) in the binding rep.
          */
         RPC_CN_CREP_SIZEOF_HDR (call_rep) = RPC_CN_PKT_SIZEOF_RQST_HDR_NO_OBJ;
-        RPC_CN_CREP_FREE_BYTES (call_rep) =
+        RPC_CN_CREP_FREE_BYTES (call_rep) = 
             RPC_C_CN_SMALL_FRAG_SIZE - RPC_CN_PKT_SIZEOF_RQST_HDR_NO_OBJ;
         RPC_CN_CREP_ACC_BYTCNT (call_rep) = RPC_CN_PKT_SIZEOF_RQST_HDR_NO_OBJ;
         RPC_CN_CREP_FREE_BYTE_PTR(call_rep) =
@@ -505,13 +505,13 @@ unsigned32              *st;
 
     /*
      * Evaluate the RPC_REQUEST event.
-     * If we return successfully, an association should be allocated
+     * If we return successfully, an association should be allocated 
      * and we should be in the call_pending state.
      */
 
-    RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_START_CALL,
+    RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_START_CALL, 
                             ifspec_r,
-                            call_rep,
+                            call_rep, 
                             *st);
 
     if (*st == rpc_s_ok)
@@ -521,7 +521,7 @@ unsigned32              *st;
          * placed there by the rpc__cn_assoc_alloc routine.
          */
         RPC_CN_PKT_PRES_CONT_ID (header_p) = call_rep->context_id;
-
+        
         /*
          * Use the negotiated minor version number from the association.
          */
@@ -535,7 +535,7 @@ unsigned32              *st;
         {
             rpc_cn_auth_tlr_t       *auth_tlr;
             unsigned32              auth_value_len;
-
+            
             /*
              * Allocate a small fragbuf to contain the authentication trailer.
              */
@@ -559,7 +559,7 @@ unsigned32              *st;
         {
             RPC_CN_PKT_AUTH_LEN (header_p) = 0;
         }
-
+        
         /*
          * Return the negotiated transfer syntax placed in the call
          * rep by rpc__cn_assoc_alloc.
@@ -630,7 +630,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_transmit
+PRIVATE void rpc__cn_call_transmit 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -670,10 +670,10 @@ unsigned32              *st;
         for (i = 0; i < call_args->num_elt; i++)
         {
             RPC_DBG_PRINTF (rpc_e_dbg_cn_pkt, RPC_C_CN_DBG_PKT,
-                            ("        elt[%d]: elt.flags->%x args.buff_len->%d args.data_len->%d\n",
-                             i,
+                            ("        elt[%d]: elt.flags->%x args.buff_len->%d args.data_len->%d\n", 
+                             i, 
                              call_args->elt[i].flags,
-                             call_args->elt[i].buff_len,
+                             call_args->elt[i].buff_len, 
                              call_args->elt[i].data_len));
         }
     }
@@ -696,7 +696,7 @@ unsigned32              *st;
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                         ("CN: call_rep->%x call orphaned...\n",
                          call_rep));
-
+        
         iov_elt_p = call_args->elt;
         for (i = 1; i <= call_args->num_elt; i++, iov_elt_p++)
         {
@@ -709,9 +709,9 @@ unsigned32              *st;
     }
     else
     {
-        RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_SEND,
+        RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_SEND, 
                                 call_args,
-                                call_rep,
+                                call_rep, 
                                 *st);
         if (((call_rep->call_state.cur_state == RPC_C_CLIENT_CALL_CALL_FAILED)
 	    || (call_rep->call_state.cur_state == RPC_C_CLIENT_CALL_CFDNE))
@@ -748,7 +748,7 @@ unsigned32              *st;
 		frag_buf->data_p = (pointer_t)
 				   (RPC_CN_PKT_FAULT_STUB_DATA(header_p));
 
-		/*
+	 	/* 
 		 * We had conservatively assumed that the call has executed
 		 * as soon as the call started. We can now update that
 		 * information.
@@ -848,7 +848,7 @@ unsigned32              *st;
 **  to transmit a vector of marshalled arguments to the remote
 **  thread.  It uses the call rep as the identifier of the RPC
 **  being performed.  Block until the first buffer of marshalled
-**  output arguments has been received.  This routine is intended
+**  output arguments has been received.  This routine is intended 
 **  for use by the client stub only.
 **
 **  INPUTS:
@@ -887,7 +887,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_transceive
+PRIVATE void rpc__cn_call_transceive 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -936,10 +936,10 @@ unsigned32              *st;
         for (i = 0; i < in_call_args->num_elt; i++)
         {
             RPC_DBG_PRINTF (rpc_e_dbg_cn_pkt, RPC_C_CN_DBG_PKT,
-                            ("        elt[%d]: elt.flags->%x in_args.buff_len->%d in_args.data_len->%d\n",
-                             i,
+                            ("        elt[%d]: elt.flags->%x in_args.buff_len->%d in_args.data_len->%d\n", 
+                             i, 
                              in_call_args->elt[i].flags,
-                             in_call_args->elt[i].buff_len,
+                             in_call_args->elt[i].buff_len, 
                              in_call_args->elt[i].data_len));
         }
     }
@@ -961,7 +961,7 @@ unsigned32              *st;
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                         ("CN: call_rep->%x call orphaned...\n",
                          call_rep));
-
+        
         out_call_args->buff_dealloc = NULL;
         out_call_args->data_addr = (byte_p_t) NULL;
         out_call_args->data_len = 0;
@@ -993,8 +993,8 @@ unsigned32              *st;
      * Note that we might get NULL data.
      */
     RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_LAST_TRANSMIT_REQ,
-                           in_call_args,
-                           call_rep,
+                           in_call_args, 
+                           call_rep, 
                            temp_status);
     if (temp_status != rpc_s_ok)
     {
@@ -1038,10 +1038,10 @@ unsigned32              *st;
                 valid_fragbuf = false;
                 while (!valid_fragbuf)
                 {
-                    rpc__cn_assoc_receive_frag (call_rep->assoc,
-                                                &frag_buf,
+                    rpc__cn_assoc_receive_frag (call_rep->assoc, 
+                                                &frag_buf, 
                                                 st);
-
+                    
                     if (*st != rpc_s_ok)
                     {
                         out_call_args->buff_dealloc = NULL;
@@ -1090,7 +1090,7 @@ unsigned32              *st;
                      * has executed as soon as the call started.
                      * We can now update that information.
                      */
-                    if (RPC_CN_PKT_FLAGS (header_p) &
+                    if (RPC_CN_PKT_FLAGS (header_p) & 
                         RPC_C_CN_FLAGS_DID_NOT_EXECUTE)
                     {
                         call_rep->call_executed = false;
@@ -1115,7 +1115,7 @@ unsigned32              *st;
                          * won't be returning it to the client stub.
                          */
                         (* frag_buf->fragbuf_dealloc) (frag_buf);
-
+                        
                         /*
                          * We translate the on-wire fault code into
                          * a local status code & return it.
@@ -1131,9 +1131,9 @@ unsigned32              *st;
                     }
                     else
                     {
-                        /*
-                         * On a fault, we store the address of the fragment
-                         * buffer in the call_rep for later retrieval via
+                        /* 
+                         * On a fault, we store the address of the fragment 
+                         * buffer in the call_rep for later retrieval via 
                          * rpc__receive_fault.
                          */
                         call_rep->u.client.fault_data = frag_buf;
@@ -1163,7 +1163,7 @@ unsigned32              *st;
                      * Set up the iovector element to point to the received data.
                      */
                     out_call_args->buff_dealloc =
-                        (rpc_buff_dealloc_fn_t)frag_buf->fragbuf_dealloc;
+                        (rpc_buff_dealloc_fn_t)frag_buf->fragbuf_dealloc; 
                     out_call_args->buff_addr = (byte_p_t) frag_buf;
                     out_call_args->buff_len = frag_buf->max_data_size;
                     out_call_args->data_addr = (byte_p_t) frag_buf->data_p;
@@ -1195,9 +1195,9 @@ unsigned32              *st;
     RPC_CN_UNLOCK ();
 
     RPC_DBG_PRINTF (rpc_e_dbg_cn_pkt, RPC_C_CN_DBG_PKT,
-        ("PACKET: call transceive out_elt.flags->%x out_elt.buff_len->%d out_elt.data_len->%d\n",
+        ("PACKET: call transceive out_elt.flags->%x out_elt.buff_len->%d out_elt.data_len->%d\n", 
         out_call_args->flags,
-        out_call_args->buff_len,
+        out_call_args->buff_len, 
         out_call_args->data_len));
     RPC_LOG_CN_CALL_TRANSCEIVE_XIT;
 }
@@ -1245,7 +1245,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_receive
+PRIVATE void rpc__cn_call_receive 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -1290,7 +1290,7 @@ unsigned32              *st;
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                         ("CN: call_rep->%x call orphaned...\n",
                          call_rep));
-
+        
         call_args->buff_dealloc = NULL;
         call_args->data_addr = (byte_p_t) NULL;
         call_args->data_len = 0;
@@ -1333,8 +1333,8 @@ unsigned32              *st;
     valid_fragbuf = false;
     while (!valid_fragbuf)
     {
-        rpc__cn_assoc_receive_frag (call_rep->assoc,
-                                    &frag_buf,
+        rpc__cn_assoc_receive_frag (call_rep->assoc, 
+                                    &frag_buf, 
                                     st);
         if (*st != rpc_s_ok)
         {
@@ -1374,7 +1374,7 @@ unsigned32              *st;
     {
         if (RPC_CN_PKT_OBJ_UUID_PRESENT (header_p))
         {
-                frag_buf->data_p = (pointer_t)
+                frag_buf->data_p = (pointer_t) 
                     RPC_CN_PKT_RQST_STUB_DATA_W_OBJ (header_p);
         }
         else
@@ -1425,9 +1425,9 @@ unsigned32              *st;
         }
         else
         {
-            /*
-             * On a fault, we store the address of the fragment
-             * buffer in the call_rep for later retrieval via
+            /* 
+             * On a fault, we store the address of the fragment 
+             * buffer in the call_rep for later retrieval via 
              * rpc__receive_fault.
              */
             call_rep->u.client.fault_data = frag_buf;
@@ -1461,7 +1461,7 @@ unsigned32              *st;
         call_args->data_addr = (byte_p_t) frag_buf->data_p;
         call_args->data_len = frag_buf->data_size;
         call_args->buff_dealloc =
-                (rpc_buff_dealloc_fn_t)frag_buf->fragbuf_dealloc;
+                (rpc_buff_dealloc_fn_t)frag_buf->fragbuf_dealloc; 
     }
     else
     {
@@ -1487,7 +1487,7 @@ unsigned32              *st;
     {
         RPC_CN_CHK_AND_FWD_CANCELS (call_rep, st);
     }
-
+    
     /*
      * Release the CN global mutex to allow other threads to
      * run.
@@ -1496,9 +1496,9 @@ unsigned32              *st;
     RPC_CN_UNLOCK ();
 
     RPC_DBG_PRINTF (rpc_e_dbg_cn_pkt, RPC_C_CN_DBG_PKT,
-        ("PACKET: call receive args.flags->%x args.buff_len->%d args.data_len->%d\n",
+        ("PACKET: call receive args.flags->%x args.buff_len->%d args.data_len->%d\n", 
         call_args->flags,
-        call_args->buff_len,
+        call_args->buff_len, 
         call_args->data_len));
     RPC_LOG_CN_CALL_RECEIVE_XIT;
 }
@@ -1545,7 +1545,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_block_until_free
+PRIVATE void rpc__cn_call_block_until_free 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -1627,7 +1627,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_alert
+PRIVATE void rpc__cn_call_alert 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -1709,7 +1709,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_end
+PRIVATE void rpc__cn_call_end 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        *call_r,
@@ -1748,12 +1748,12 @@ unsigned32              *st;
 
     if (call_rep != NULL)
     {
-        /*
+        /* 
          * Evaluate the state machine with the call_end event
          */
         RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_END,
-                                NULL,
-                                call_rep,
+                                NULL, 
+                                call_rep, 
                                 *st);
 
         /*
@@ -1786,7 +1786,7 @@ unsigned32              *st;
                     (*fault_fbp->fragbuf_dealloc) (fault_fbp);
                 }
             }
-
+            
             /*
              * Cancel this thread if:
              *
@@ -1806,7 +1806,7 @@ unsigned32              *st;
                                ("(rpc__cn_call_end) call_rep->%x reposting cancel\n", call_rep));
                 dcethread_interrupt_throw (dcethread_self());
             }
-
+            
             /*
              * Stop the cancel timer if one was running.
              */
@@ -1820,7 +1820,7 @@ unsigned32              *st;
          */
         if (RPC_CN_CREP_IOVLEN (call_rep) > 0)
         {
-            for (cur_iov_index = 0;
+            for (cur_iov_index = 0; 
                  cur_iov_index < RPC_CN_CREP_IOVLEN (call_rep);
                  cur_iov_index++)
             {
@@ -1845,7 +1845,7 @@ unsigned32              *st;
             rpc__cn_smfragbuf_free (call_rep->prot_tlr);
         }
 
-        /*
+        /* 
          * Free the call_rep itself.
          */
         rpc__list_element_free (&rpc_g_cn_call_lookaside_list,
@@ -1905,7 +1905,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_transmit_fault
+PRIVATE void rpc__cn_call_transmit_fault 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -1948,7 +1948,7 @@ unsigned32              *st;
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                         ("CN: call_rep->%x call orphaned...\n",
                          call_rep));
-
+        
         iov_elt_p = call_fault_info->elt;
         for (i = 1; i <= call_fault_info->num_elt; i++, iov_elt_p++)
         {
@@ -1961,12 +1961,12 @@ unsigned32              *st;
     }
     else
     {
-        /*
+        /* 
          * Evaluate the state machine with the call_fault event.
          */
         RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_FAULT,
-                                call_fault_info,
-                                call_rep,
+                                call_fault_info, 
+                                call_rep, 
                                 *st);
     }
 
@@ -2006,7 +2006,7 @@ unsigned32              *st;
 **
 **  IMPLICIT INPUTS:    This routine assumes that the prototype
 **                      send header in the call_rep has been
-**                      set up.
+**                      set up. 
 **
 **  IMPLICIT OUTPUTS:   none
 **
@@ -2017,7 +2017,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_reject
+PRIVATE void rpc__cn_call_reject 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -2031,7 +2031,7 @@ unsigned32              l_st;
 #endif
 {
     rpc_cn_call_rep_p_t         call_rep;
-    unsigned32                  st;
+    unsigned32                  st ATTRIBUTE_UNUSED;
 
     RPC_CN_DBG_RTN_PRINTF(rpc__cn_call_reject);
 
@@ -2055,9 +2055,9 @@ unsigned32              l_st;
      * architected status code in the call rep and pass a null
      * fault_data.
      */
-    RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_FAULT_DNE,
+    RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_FAULT_DNE, 
                             NULL,
-                            call_rep,
+                            call_rep, 
                             st);
 }
 
@@ -2107,7 +2107,7 @@ unsigned32              l_st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_receive_fault
+PRIVATE void rpc__cn_call_receive_fault 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -2162,7 +2162,7 @@ unsigned32              *st;
          * We now copy the descriptors into the iovector.
          * The stub will deallocate the fragment buffer.
          */
-        call_fault_info->buff_dealloc =
+        call_fault_info->buff_dealloc = 
             (rpc_buff_dealloc_fn_t)fault_fragp->fragbuf_dealloc;
         call_fault_info->buff_addr = (byte_p_t) fault_fragp;
         call_fault_info->buff_len = fault_fragp->max_data_size;
@@ -2220,7 +2220,7 @@ unsigned32              *st;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **      boolean32       true if the manager routine has begun, false otherwise.
 **
@@ -2229,7 +2229,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE boolean32 rpc__cn_call_did_mgr_execute
+PRIVATE boolean32 rpc__cn_call_did_mgr_execute 
 #ifdef _DCE_PROTO_
 (
   rpc_call_rep_p_t        call_r,
@@ -2293,7 +2293,7 @@ unsigned32              *st;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **      unsigned32      The runtime status code corresponding to
 **                      the nca status code.
@@ -2303,7 +2303,7 @@ unsigned32              *st;
 **--
 **/
 
-INTERNAL unsigned32 rpc__cn_call_cvt_from_nca_st
+INTERNAL unsigned32 rpc__cn_call_cvt_from_nca_st 
 #ifdef _DCE_PROTO_
 (
   unsigned32 a_st
@@ -2356,7 +2356,7 @@ unsigned32              a_st;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **      unsigned32      The architected (nca_s...) status code.
 **
@@ -2365,7 +2365,7 @@ unsigned32              a_st;
 **--
 **/
 
-INTERNAL unsigned32 rpc__cn_call_cvt_to_nca_st
+INTERNAL unsigned32 rpc__cn_call_cvt_to_nca_st 
 #ifdef _DCE_PROTO_
 (
   unsigned32 l_st
@@ -2432,7 +2432,7 @@ unsigned32              l_st;
 **--
 **/
 
-PRIVATE void rpc__cn_call_no_conn_ind
+PRIVATE void rpc__cn_call_no_conn_ind 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t call_rep ATTRIBUTE_UNUSED
@@ -2487,7 +2487,7 @@ rpc_cn_call_rep_p_t     call_rep;
 **--
 **/
 
-PRIVATE void rpc__cn_call_ccb_create
+PRIVATE void rpc__cn_call_ccb_create 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     ccb
@@ -2541,7 +2541,7 @@ rpc_cn_call_rep_p_t     ccb;
 **
 **  This routine will free the fragment buffer contained in an call
 **  control block before the rpc__list_element_free routine returns
-**  it to heap storage.
+**  it to heap storage. 
 **
 **  INPUTS:
 **
@@ -2562,7 +2562,7 @@ rpc_cn_call_rep_p_t     ccb;
 **--
 **/
 
-PRIVATE void rpc__cn_call_ccb_free
+PRIVATE void rpc__cn_call_ccb_free 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     ccb
@@ -2627,7 +2627,7 @@ rpc_cn_call_rep_p_t     ccb;
 **--
 **/
 
-PRIVATE void rpc__cn_call_local_cancel
+PRIVATE void rpc__cn_call_local_cancel 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -2646,11 +2646,11 @@ unsigned32              *status;
     CODING_ERROR (status);
 
     RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
-                   ("(rpc__cn_call_local_cancel) call_rep->%x local cancel caught\n",
+                   ("(rpc__cn_call_local_cancel) call_rep->%x local cancel caught\n", 
                     call_rep));
     /*
      * If the call rep is NULL, this is the server side of a
-     * call and it has been orphaned. We were in a cancellable pipe
+     * call and it has been orphaned. We were in a cancellable pipe 
      * routine. Return the rpc_s_call_cancelled status code.
      */
     if (call_rep == NULL)
@@ -2723,7 +2723,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void rpc__cn_call_check_for_cancel
+INTERNAL void rpc__cn_call_check_for_cancel 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep
@@ -2743,16 +2743,16 @@ rpc_cn_call_rep_p_t     call_rep;
      */
     if (call_rep->u.client.cancel.timeout_time == 0)
     {
-        DCETHREAD_TRY
-        {
-            dcethread_checkinterrupt ();
-        }
-        DCETHREAD_CATCH (dcethread_interrupt_e)
-        {
+        DCETHREAD_TRY 
+        { 
+            dcethread_checkinterrupt (); 
+        } 
+        DCETHREAD_CATCH (dcethread_interrupt_e) 
+        { 
             RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
                             ("(rpc__cn_call_check_for_cancel) call_rep->%x local cancel detected\n", call_rep));
-            (call_rep)->u.client.cancel.local_count++;
-        }
+            (call_rep)->u.client.cancel.local_count++; 
+        } 
         DCETHREAD_ENDTRY
     }
 }
@@ -2790,7 +2790,7 @@ rpc_cn_call_rep_p_t     call_rep;
 **--
 **/
 
-INTERNAL void rpc__cn_call_forward_cancel
+INTERNAL void rpc__cn_call_forward_cancel 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -2803,7 +2803,7 @@ rpc_cn_call_rep_p_t     call_rep;
 unsigned32              *status;
 #endif
 {
-    unsigned32          temp_status;
+    unsigned32          temp_status ATTRIBUTE_UNUSED;
 
     RPC_CN_DBG_RTN_PRINTF(rpc__cn_call_forward_cancel);
     CODING_ERROR (status);
@@ -2820,8 +2820,8 @@ unsigned32              *status;
             RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
                            ("(rpc__cn_call_forward_cancel) call_rep-> %x forwarding cancel\n", call_rep));
             RPC_CN_CALL_EVAL_EVENT (RPC_C_CALL_LOCAL_ALERT,
-                                    NULL,
-                                    call_rep,
+                                    NULL, 
+                                    call_rep, 
                                     temp_status);
         }
     }
@@ -2853,28 +2853,28 @@ unsigned32              *status;
 **      binding_r       The binding rep access to which is being serialized
 **      cancel_timeout  The thread's cancel timeout value
 **
-**  INPUTS/OUTPUTS:
+**  INPUTS/OUTPUTS:     
 **
 **      cancel_cnt      The number of cancels detected while waiting
-**                      for the binding rep to become free.
+**                      for the binding rep to become free. 
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
-**      st              The RPC runtime status code
+**      st              The RPC runtime status code 
 **
 **
 **  IMPLICIT INPUTS:    none
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-INTERNAL void rpc__cn_call_binding_serialize
+INTERNAL void rpc__cn_call_binding_serialize 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r,
@@ -2897,7 +2897,7 @@ unsigned32              *st;
     volatile struct timespec     zero_delta, delta, abstime, curtime;
 
     RPC_CN_DBG_RTN_PRINTF(rpc__cn_call_binding_serialize);
-    RPC_LOCK_ASSERT(0);
+    RPC_LOCK_ASSERT(0);        
     CODING_ERROR (st);
 
     *st = rpc_s_ok;
@@ -2910,10 +2910,10 @@ unsigned32              *st;
     delta.tv_sec = cancel_timeout;
 
     /*
-     * Wait while the binding is being resolved by someone else
-     * (this routine is called only by threads not doing the
+     * Wait while the binding is being resolved by someone else 
+     * (this routine is called only by threads not doing the 
      * resolving) and we haven't timed out.
-     *
+     * 
      * If the user attempts to cancel the call,
      * don't wait longer than the cancel timeout time.
      */
@@ -2929,7 +2929,7 @@ unsigned32              *st;
             else
             {
                 RPC_BINDING_COND_TIMED_WAIT ((struct timespec *) &abstime);
-                dcethread_get_expiration ((struct timespec *) &zero_delta,
+                dcethread_get_expiration ((struct timespec *) &zero_delta, 
                                           (struct timespec *) &curtime);
                 if (curtime.tv_sec >= abstime.tv_sec)
                 {
@@ -2940,7 +2940,7 @@ unsigned32              *st;
         DCETHREAD_CATCH (dcethread_interrupt_e)
         {
             /*
-             * Track the cancels and setup a cancel timeout value
+             * Track the cancels and setup a cancel timeout value 
              * (if appropriate).
              */
             RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
@@ -2967,8 +2967,8 @@ unsigned32              *st;
                         RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
                                        ("(rpc__cn_call_binding_serialize) binding_rep->%x %d sec cancel timeout setup\n",
                                         binding_r, delta.tv_sec));
-
-                        dcethread_get_expiration ((struct timespec *) (&delta),
+                        
+                        dcethread_get_expiration ((struct timespec *) (&delta), 
                                                   (struct timespec *) (&abstime));
                     }
                     is_awaiting_timeout = true;
@@ -2976,8 +2976,8 @@ unsigned32              *st;
             }
             *cancel_cnt += 1;
 
-        /*
-         * Any other type of exception is something serious; just let it
+        /* 
+         * Any other type of exception is something serious; just let it 
          * propagate and we die in our usual unclean fashion.
          */
         }
@@ -3016,23 +3016,23 @@ unsigned32              *st;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
-**      st              The RPC runtime status code
+**      st              The RPC runtime status code 
 **
 **
 **  IMPLICIT INPUTS:    none
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_call_start_cancel_timer
+PRIVATE void rpc__cn_call_start_cancel_timer 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_r,
@@ -3063,7 +3063,7 @@ unsigned32              *st;
             (call_r->u.client.cancel.timeout_time != (typeof(call_r->u.client.cancel.timeout_time))(rpc_c_cancel_infinite_timeout)))
         {
             RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
-                           ("(rpc__cn_call_start_cancel_timer) call_rep->%x starting cancel timer - %d seconds\n",
+                           ("(rpc__cn_call_start_cancel_timer) call_rep->%x starting cancel timer - %d seconds\n", 
                             call_r, call_r->u.client.cancel.timeout_time));
             call_r->u.client.cancel.timer_running = true;
             call_r->u.client.cancel.thread_h = dcethread_self ();
@@ -3076,7 +3076,7 @@ unsigned32              *st;
     else
     {
         RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
-                       ("(rpc__cn_call_start_cancel_timer) call_rep->%x timer expired ... returning rpc_s_cancel_timeout\n",
+                       ("(rpc__cn_call_start_cancel_timer) call_rep->%x timer expired ... returning rpc_s_cancel_timeout\n", 
                         call_r));
     }
 }
@@ -3099,23 +3099,23 @@ unsigned32              *st;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
-**      st              The RPC runtime status code
+**      st              The RPC runtime status code 
 **
 **
 **  IMPLICIT INPUTS:    none
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_call_stop_cancel_timer
+PRIVATE void rpc__cn_call_stop_cancel_timer 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_r
@@ -3154,16 +3154,16 @@ rpc_cn_call_rep_p_t     call_r;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
-**      st              The RPC runtime status code
+**      st              The RPC runtime status code 
 **
 **
 **  IMPLICIT INPUTS:    none
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **      true            the routine should be rescheduled
 **      false           the routine should not be rescheduled
@@ -3173,7 +3173,7 @@ rpc_cn_call_rep_p_t     call_r;
 **--
 **/
 
-INTERNAL boolean rpc__cn_call_cancel_timer
+INTERNAL boolean rpc__cn_call_cancel_timer 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_r
