@@ -1,5 +1,5 @@
 /*
- *
+ * 
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- *
+ * 
  */
 /*
  */
@@ -134,7 +134,7 @@ INTERNAL pointer_t rpc__cn_network_init_desc _DCE_PROTOTYPE_ ((
 **--
 **/
 
-INTERNAL void rpc__cn_network_desc_inq_ep
+INTERNAL void rpc__cn_network_desc_inq_ep 
 #ifdef _DCE_PROTO_
 (
   rpc_socket_t            desc,
@@ -152,7 +152,7 @@ unsigned32              *status;
 {
     rpc_addr_vector_p_t rpc_addr_vec;
     unsigned32          temp_status;
-
+    
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_desc_inq_ep);
     CODING_ERROR (status);
 
@@ -161,7 +161,7 @@ unsigned32              *status;
      */
     rpc__naf_desc_inq_addr (pseq_id, desc, &rpc_addr_vec, status);
     if (*status != rpc_s_ok) return;
-
+    
     if (rpc_addr_vec->len == 0)
     {
         rpc__naf_addr_vector_free (&rpc_addr_vec, &temp_status);
@@ -208,7 +208,7 @@ unsigned32              *status;
 **      max_calls       The min number of concurrent call requests
 **                      the server wishes to be able to handle.
 **      rpc_addr        The rpc_addr containing the endpoint and use
-**                      to bind to the socket.
+**                      to bind to the socket. 
 **      endpoint        The endpoint referred to above.
 **
 **  INPUTS/OUTPUTS:     none
@@ -229,7 +229,7 @@ unsigned32              *status;
 **--
 **/
 
-PRIVATE void rpc__cn_network_use_protseq
+PRIVATE void rpc__cn_network_use_protseq 
 #ifdef _DCE_PROTO_
 (
   rpc_protseq_id_t        pseq_id,
@@ -273,7 +273,7 @@ unsigned32              *status;
 #endif
 
     CODING_ERROR (status);
-
+    
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_use_protseq);
 
     /*
@@ -296,7 +296,7 @@ unsigned32              *status;
          */
         if ((endpoint != NULL) && (max_calls > RPC_C_LISTEN_BACKLOG))
         {
-           *status = rpc_s_calls_too_large_for_wk_ep;
+           *status = rpc_s_calls_too_large_for_wk_ep; 
            return;
        }
     }
@@ -324,15 +324,15 @@ unsigned32              *status;
          * RPC address given.
          */
         auto_naf_id = rpc_addr->sa.family;
-
+        
         /*
          * If this call succeeds, we can safely assume stdin is a socket and
          * we will also have all the information about the socket that we need.
          */
-        rpc__naf_desc_inq_network (0,
-                                   &auto_naf_id,
-                                   &auto_network_if_id,
-                                   &auto_prot_id,
+        rpc__naf_desc_inq_network (0, 
+                                   &auto_naf_id, 
+                                   &auto_network_if_id, 
+                                   &auto_prot_id, 
                                    &auto_status);
         spawned_checked = true;
         spawned = (auto_status == rpc_s_ok);
@@ -376,7 +376,7 @@ unsigned32              *status;
     {
         if (spawned)
         {
-            spawned_same_address = !strcmp ((char *) endpoint,
+            spawned_same_address = !strcmp ((char *) endpoint, 
                                             (char *) auto_endpoint);
         }
     }
@@ -397,22 +397,22 @@ unsigned32              *status;
      */
     num_socks = (max_calls + RPC_C_LISTEN_BACKLOG - 1)/RPC_C_LISTEN_BACKLOG;
     RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                    ("(rpc__cn_network_use_protseq) Creating %d sockets\n",
+                    ("(rpc__cn_network_use_protseq) Creating %d sockets\n", 
                      num_socks));
-    RPC_MEM_ALLOC (sock_list,
-                   rpc_socket_t *,
+    RPC_MEM_ALLOC (sock_list, 
+                   rpc_socket_t *, 
                    ((num_socks) * sizeof (rpc_socket_t)),
                    RPC_C_MEM_SOCKET_LIST,
                    RPC_C_MEM_WAITOK);
-    for (i = 0, backlog = max_calls;
-         i < num_socks;
+    for (i = 0, backlog = max_calls; 
+         i < num_socks; 
          i++, backlog -= RPC_C_LISTEN_BACKLOG)
     {
         /*
          * First, create a socket for this RPC Protocol Sequence.
          */
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                        ("(rpc__cn_network_use_protseq) Created socket #%d\n",
+                        ("(rpc__cn_network_use_protseq) Created socket #%d\n", 
                          (i + 1)));
 #ifdef AUTOSTART
         /*
@@ -450,7 +450,7 @@ unsigned32              *status;
             RPC_SOCKET_CLOSE (sock);
             break;
         }
-
+        
         serr = rpc__socket_bind (sock, rpc_addr);
         if (RPC_SOCKET_IS_ERR (serr))
         {
@@ -477,13 +477,13 @@ unsigned32              *status;
          * Finally, add the socket to the listener thread's select
          * pool of sockets.
          */
-        rpc__network_add_desc (sock,
-                               true,
-                               (endpoint == NULL),
-                               pseq_id,
-                               priv_info,
+        rpc__network_add_desc (sock, 
+                               true, 
+                               (endpoint == NULL), 
+                               pseq_id, 
+                               priv_info, 
                                status);
-
+        
         if (*status != rpc_s_ok)
         {
             RPC_SOCKET_CLOSE (sock);
@@ -504,8 +504,8 @@ unsigned32              *status;
      */
     if (i != num_socks)
     {
-        for (created_sock_index = 0;
-             created_sock_index < i;
+        for (created_sock_index = 0; 
+             created_sock_index < i; 
              created_sock_index++)
         {
             rpc__network_remove_desc (sock_list[created_sock_index], &temp_status);
@@ -575,7 +575,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL pointer_t rpc__cn_network_init_desc
+INTERNAL pointer_t rpc__cn_network_init_desc 
 #ifdef _DCE_PROTO_
 (
   rpc_socket_t            *desc,
@@ -598,12 +598,11 @@ unsigned32              *status;
     rpc_addr_p_t        rpc_addr;
     unsigned_char_t     *endpoint;
     rpc_socket_error_t  serr;
-    rpc_cn_assoc_t      *assoc;
     unsigned32          temp_status;
     unsigned32          ssize, rsize;
-
+    
     CODING_ERROR (status);
-
+    
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_init_desc);
 
     /*
@@ -689,7 +688,7 @@ unsigned32              *status;
     /*
      * Setup the socket's send and receive buffering
      */
-    serr = rpc__socket_set_bufs (*desc,
+    serr = rpc__socket_set_bufs (*desc, 
                                  rpc_g_cn_socket_read_buffer,
                                  rpc_g_cn_socket_write_buffer,
                                  &ssize,
@@ -741,7 +740,7 @@ unsigned32              *status;
          * block which comes back will have all mutexes and condition
          * variables created. Also the receiver thread will be created.
          */
-        assoc = rpc__cn_assoc_listen (connected_desc, endpoint, status);
+        (void) rpc__cn_assoc_listen (connected_desc, endpoint, status);
         if (*status != rpc_s_ok)
         {
             rpc_string_free (&endpoint, &temp_status);
@@ -756,7 +755,7 @@ unsigned32              *status;
     if (RPC_SOCKET_IS_ERR(serr))
     {
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
-                        ("(rpc__cn_network_init_desc) desc->%x rpc__socket_listen failed, error = %d\n",
+                        ("(rpc__cn_network_init_desc) desc->%x rpc__socket_listen failed, error = %d\n", 
                          *desc, RPC_SOCKET_ETOI(serr)));
         rpc_string_free (&endpoint, &temp_status);
         *status = rpc_s_cant_listen_sock;
@@ -776,7 +775,7 @@ unsigned32              *status;
 **  SCOPE:              PRIVATE - declared in cnnet.h
 **
 **  DESCRIPTION:
-**
+**      
 **  Return the version number of the NCA CN protocol currently in use.
 **
 **  INPUTS:             none
@@ -802,7 +801,7 @@ unsigned32              *status;
 **--
 **/
 
-PRIVATE void rpc__cn_network_inq_prot_vers
+PRIVATE void rpc__cn_network_inq_prot_vers 
 #ifdef _DCE_PROTO_
 (
   unsigned8               *prot_id,
@@ -873,7 +872,7 @@ unsigned32              *status;
 **--
 **/
 
-PRIVATE void rpc__cn_network_select_dispatch
+PRIVATE void rpc__cn_network_select_dispatch 
 #ifdef _DCE_PROTO_
 (
   rpc_socket_t            desc,
@@ -890,21 +889,20 @@ unsigned32              *st;
 #endif
 {
     rpc_socket_t        newdesc;
-    rpc_cn_assoc_t      *assoc;
     rpc_socket_error_t  serr;
-
+    
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_select_dispatch);
     CODING_ERROR(st);
-
+    
     RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                     ("CN: desc->%x connection request received\n", desc));
-
+    
     /*
      * We have been called by the network listener thread because a
      * network event occured on the descriptor passed in. Since the
      * only descriptors the network listener thread is listening on for us
      * are listening sockets we should just be able to accept the
-     * connection.
+     * connection. 
      */
     serr = rpc__socket_accept (desc, NULL, &newdesc);
     if (RPC_SOCKET_IS_ERR(serr))
@@ -912,7 +910,7 @@ unsigned32              *st;
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
                         ("(rpc__cn_network_select_dispatch) desc->%x rpc__socket_accept failed, error = %d\n",
                          desc, RPC_SOCKET_ETOI(serr)));
-
+        
         *st = rpc_s_cannot_accept;
         dcethread_yield ();
     }
@@ -928,11 +926,11 @@ unsigned32              *st;
             rpc__naf_desc_inq_protseq_id (newdesc,
                                           RPC_C_PROTOCOL_ID_NCACN,
                                           &protseq_id,
-                                          &dbg_status);
+                                          &dbg_status); 
 
             if (dbg_status == rpc_s_ok)
             {
-                rpc__naf_desc_inq_peer_addr (newdesc,
+                rpc__naf_desc_inq_peer_addr (newdesc, 
                                              protseq_id,
                                              &rpc_addr,
                                              &dbg_status);
@@ -980,7 +978,7 @@ unsigned32              *st;
              * Set the socket to close itself if the process execs.
              */
             rpc__socket_set_close_on_exec (newdesc);
-
+            
 #ifndef NODELAY_BROKEN
             /*
              * The application is listening for new calls.
@@ -989,7 +987,7 @@ unsigned32              *st;
             rpc__naf_set_pkt_nodelay (newdesc, NULL, st);
             if (*st != rpc_s_ok)
             {
-                /*
+                /* 
                  * The set option failed. We'll continue anyway.
                  */
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
@@ -997,7 +995,7 @@ unsigned32              *st;
                                  newdesc, *st));
             }
 #endif
-
+            
             /*
              * Set the keepalive socket option for this connection.
              */
@@ -1014,7 +1012,7 @@ unsigned32              *st;
 
             {
                 struct timeval tmo;
-
+                
                 tmo.tv_sec = RPC_C_SOCKET_SERVER_RECV_TIMER;
                 tmo.tv_usec = 0;
 
@@ -1039,12 +1037,12 @@ unsigned32              *st;
              * condition variables created. Also the receiver thread
              * will be created.
              */
-            assoc = rpc__cn_assoc_listen (newdesc,
-                                          (unsigned_char_t *) priv_info,
+            (void) rpc__cn_assoc_listen (newdesc,
+                                          (unsigned_char_t *) priv_info, 
                                           st);
             if (*st != rpc_s_ok)
             {
-                /*
+                /* 
                  * The association listen failed. Close the connection.
                  */
                 serr = RPC_SOCKET_CLOSE (newdesc);
@@ -1082,7 +1080,7 @@ unsigned32              *st;
 **      call_r          The call rep data structure to be linked to
 **                      the association control block. This may be
 **                      NULL if this association is being allocated
-**                      on the server side.
+**                      on the server side. 
 **
 **  INPUTS/OUTPUTS:     none
 **
@@ -1103,7 +1101,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_req_connect
+PRIVATE void rpc__cn_network_req_connect 
 #ifdef _DCE_PROTO_
 (
   rpc_addr_p_t            rpc_addr,
@@ -1128,10 +1126,10 @@ unsigned32              *st;
 
     //DO_NOT_CLOBBER(serr);
     //DO_NOT_CLOBBER(connect_completed);
-
+	 
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_req_connect);
     CODING_ERROR(st);
-
+    
     /*
      * Now we need to create a connection to the server address
      * contained in the RPC address given. First create a socket to
@@ -1148,7 +1146,7 @@ unsigned32              *st;
                          assoc,
                          assoc->cn_ctlblk.cn_sock,
                          RPC_SOCKET_ETOI(serr)));
-
+        
         *st = rpc_s_cant_create_sock;
     }
     else
@@ -1167,17 +1165,17 @@ unsigned32              *st;
                             ("(rpc__cn_network_req_connect) call_rep->%x assoc->%x desc->%x Can't set socket bufs, error=%d\n",
                              assoc->call_rep,
                              assoc,
-                             assoc->cn_ctlblk.cn_sock,
+                             assoc->cn_ctlblk.cn_sock, 
                              RPC_SOCKET_ETOI (serr)));
         }
-
+        
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_BUFFS,
                         ("(rpc__cn_network_req_connect) desc->%x desired_sndbuf %u, desired_rcvbuf %u\n",
                          assoc->cn_ctlblk.cn_sock, rpc_g_cn_socket_read_buffer, rpc_g_cn_socket_write_buffer));
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_BUFFS,
                        ("(rpc__cn_network_req_connect) desc->%x actual sndbuf %lu, actual rcvbuf %lu\n",
                         assoc->cn_ctlblk.cn_sock, ssize, rsize));
-
+        
         if (rsize < RPC_C_ASSOC_MUST_RECV_FRAG_SIZE)
         {
 	    /*
@@ -1219,8 +1217,8 @@ unsigned32              *st;
                  * The socket close failed.
                  */
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
-                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n",
-                                 assoc->cn_ctlblk.cn_sock,
+                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n", 
+                                 assoc->cn_ctlblk.cn_sock, 
                                  RPC_SOCKET_ETOI(serr)));
             }
             return;
@@ -1235,10 +1233,10 @@ unsigned32              *st;
         {
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
                             ("(rpc__cn_network_req_connect) desc->%x rpc__socket_bind failed, error = %d\n",
-                             assoc->cn_ctlblk.cn_sock,
+                             assoc->cn_ctlblk.cn_sock, 
                              RPC_SOCKET_ETOI(serr)));
             *st = rpc_s_cant_bind_sock;
-
+            
             /*
              * The bind request failed. Close the socket just created
              * and free the association control block.
@@ -1250,8 +1248,8 @@ unsigned32              *st;
                  * The socket close failed.
                  */
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
-                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n",
-                                 assoc->cn_ctlblk.cn_sock,
+                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n", 
+                                 assoc->cn_ctlblk.cn_sock, 
                                  RPC_SOCKET_ETOI(serr)));
             }
             return;
@@ -1261,13 +1259,13 @@ unsigned32              *st;
          * Indicate that the connection is being attempted.
          */
         assoc->cn_ctlblk.cn_state = RPC_C_CN_CONNECTING;
-
+        
         /*
          * Since the connect call will block we will release the CN
          * global mutex before the call and reqaquire it after the call.
          */
         RPC_CN_UNLOCK ();
-
+        
         /*
          * Now actually do the connect to the server.
          */
@@ -1278,7 +1276,7 @@ unsigned32              *st;
          * Only exit the while loop on success, failure, or
          * when you've received a cancel and the cancel timer
          * has expired.
-         * If it is just a cancel, set cancel_pending and
+         * If it is just a cancel, set cancel_pending and 
          * start the cancel timer.
          */
         connect_completed = false;
@@ -1326,7 +1324,7 @@ unsigned32              *st;
                                            &retry_op,
                                            st);
                 RPC_DBG_PRINTF (rpc_e_dbg_cancel, RPC_C_CN_DBG_CANCEL,
-                                ("(rpc__cn_network_req_connect) call_rep->%x assoc->%x desc->%x cancel caught before association setup\n",
+                                ("(rpc__cn_network_req_connect) call_rep->%x assoc->%x desc->%x cancel caught before association setup\n", 
                                  assoc->call_rep,
                                  assoc,
                                  assoc->cn_ctlblk.cn_sock));
@@ -1342,7 +1340,7 @@ unsigned32              *st;
 
         }
 
-        /*
+        /* 
          * The connect completed; see if it completed successfully.
          */
         RPC_CN_LOCK ();
@@ -1361,11 +1359,11 @@ unsigned32              *st;
         {
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
                             ("(rpc__cn_network_req_connect) desc->%x rpc__socket_connect failed, error = %d\n",
-                             assoc->cn_ctlblk.cn_sock,
+                             assoc->cn_ctlblk.cn_sock, 
                              RPC_SOCKET_ETOI(serr)));
-
+            
             rpc__cn_network_serr_to_status (serr, st);
-
+            
             /*
              * The connect request failed. Close the socket just created
              * and free the association control block.
@@ -1377,8 +1375,8 @@ unsigned32              *st;
                  * The socket close failed.
                  */
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
-                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n",
-                                 assoc->cn_ctlblk.cn_sock,
+                                ("(rpc__cn_network_req_connect) desc->%x RPC_SOCKET_CLOSE failed, error = %d\n", 
+                                 assoc->cn_ctlblk.cn_sock, 
                                  RPC_SOCKET_ETOI(serr)));
             }
         }
@@ -1389,20 +1387,20 @@ unsigned32              *st;
              * Set the packet no delay option on the connection.
              */
             rpc__naf_set_pkt_nodelay (assoc->cn_ctlblk.cn_sock,
-                                      rpc_addr,
+                                      rpc_addr, 
                                       st);
             if (*st != rpc_s_ok)
             {
-                /*
+                /* 
                  * The set option failed. We'll continue anyway.
                  */
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
                                 ("(rpc__cn_network_req_connect) desc->%x rpc__naf_set_pkt_nodelay failed, error = %d\n",
-                                 assoc->cn_ctlblk.cn_sock,
+                                 assoc->cn_ctlblk.cn_sock, 
                                  *st));
             }
 #endif
-
+            
             /*
              * Update the transport information by querying the socket
              */
@@ -1411,7 +1409,7 @@ unsigned32              *st;
             {
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
                                 ("(rpc__cn_network_req_connect) desc->%x rpc__socket_inq_transport_info failed, error = %d\n",
-                                 assoc->cn_ctlblk.cn_sock,
+                                 assoc->cn_ctlblk.cn_sock, 
                                  serr));
             }
             else
@@ -1424,17 +1422,17 @@ unsigned32              *st;
              * Indicate that there is a valid connection.
              */
             assoc->cn_ctlblk.cn_state = RPC_C_CN_OPEN;
-
+            
             /*
              * A connection is now set up. Tell the receiver thread to begin
              * receiving on the connection.
              */
             if (assoc->cn_ctlblk.cn_rcvr_waiters)
             {
-                RPC_COND_SIGNAL (assoc->cn_ctlblk.cn_rcvr_cond,
+                RPC_COND_SIGNAL (assoc->cn_ctlblk.cn_rcvr_cond, 
                                  rpc_g_global_mutex);
             }
-
+            
             /*
              * Set the keepalive socket option for this connection.
              */
@@ -1454,7 +1452,7 @@ unsigned32              *st;
 
             {
                 struct timeval tmo;
-
+                
                 tmo.tv_sec = RPC_C_SOCKET_CLIENT_RECV_TIMER;
                 tmo.tv_usec = 0;
 
@@ -1468,7 +1466,7 @@ unsigned32              *st;
             }
 
             /*
-             * Everything went OK. Return a successful status code.
+             * Everything went OK. Return a successful status code. 
              */
             *st = rpc_s_ok;
         }
@@ -1491,11 +1489,11 @@ unsigned32              *st;
 **  INPUTS:
 **
 **      assoc           The association control block to which this connection
-**                      is attached.
+**                      is attached. 
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
 **      st              The return status of this routine.
 **                      rpc_s_ok
@@ -1511,7 +1509,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_close_connect
+PRIVATE void rpc__cn_network_close_connect 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_assoc_p_t        assoc,
@@ -1526,7 +1524,7 @@ unsigned32              *st;
 
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_close_connect);
     CODING_ERROR (st);
-
+    
     /*
      * Closing the connection will cause the receiver thread to stop
      * trying to receive on and and return to waiting on a condition
@@ -1593,7 +1591,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_mon
+PRIVATE void rpc__cn_network_mon 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r ATTRIBUTE_UNUSED,
@@ -1614,10 +1612,10 @@ unsigned32              *st;
 
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_mon);
     CODING_ERROR(st);
-
+    
     /*
      * Get the association group using the group id provided as a
-     * client handle.
+     * client handle. 
      */
     grp_id.all = (unsigned long) client_h;
     grp_id = rpc__cn_assoc_grp_lkup_by_id (grp_id,
@@ -1682,7 +1680,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_stop_mon
+PRIVATE void rpc__cn_network_stop_mon 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r ATTRIBUTE_UNUSED,
@@ -1698,13 +1696,13 @@ unsigned32              *st;
 {
     rpc_cn_assoc_grp_t  *assoc_grp;
     rpc_cn_local_id_t   grp_id;
-
+    
     CODING_ERROR(st);
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_stop_mon);
-
+    
     /*
      * Get the association group using the group id provided as a
-     * client handle.
+     * client handle. 
      */
     grp_id.all = (unsigned long) client_h;
     grp_id = rpc__cn_assoc_grp_lkup_by_id (grp_id,
@@ -1768,7 +1766,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_maint
+PRIVATE void rpc__cn_network_maint 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r,
@@ -1785,7 +1783,7 @@ unsigned32              *st;
 
     CODING_ERROR(st);
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_maint);
-
+    
     /*
      * Get the association group using the group id contained in the
      * binding handle.
@@ -1794,8 +1792,8 @@ unsigned32              *st;
                                             (binding_r))->grp_id,
                                            RPC_C_CN_ASSOC_GRP_CLIENT,
                                            binding_r->transport_info,
-                                           st);
-
+                                           st);  
+    
     /*
      * If the association group control block can't be found
      * return an error.
@@ -1850,7 +1848,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE void rpc__cn_network_stop_maint
+PRIVATE void rpc__cn_network_stop_maint 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r,
@@ -1867,7 +1865,7 @@ unsigned32              *st;
 
     CODING_ERROR(st);
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_stop_maint);
-
+    
     /*
      * Get the association group using the group id contained in the
      * binding handle.
@@ -1876,8 +1874,8 @@ unsigned32              *st;
                                             (binding_r))->grp_id,
                                            RPC_C_CN_ASSOC_GRP_CLIENT,
                                            binding_r->transport_info,
-                                           st);
-
+                                           st); 
+    
     /*
      * If the association group control block can't be found
      * return an error.
@@ -1931,7 +1929,7 @@ unsigned32              *st;
 **--
 **/
 
-PRIVATE boolean32 rpc__cn_network_connect_fail
+PRIVATE boolean32 rpc__cn_network_connect_fail 
 #ifdef _DCE_PROTO_
 (
 unsigned32              st
@@ -1962,7 +1960,7 @@ unsigned32              st;
         {
             return (true);
         }
-
+        
         default:
         {
             return (false);
@@ -1991,7 +1989,7 @@ unsigned32              st;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
 **      st              The status code.
 **
@@ -2006,7 +2004,7 @@ unsigned32              st;
 **--
 **/
 
-INTERNAL void rpc__cn_network_serr_to_status
+INTERNAL void rpc__cn_network_serr_to_status 
 #ifdef _DCE_PROTO_
 (
   rpc_socket_error_t      serr,
@@ -2016,66 +2014,66 @@ INTERNAL void rpc__cn_network_serr_to_status
 (serr, st)
 rpc_socket_error_t      serr;
 unsigned32              *st;
-#endif
+#endif 
 {
     switch (serr)
     {
         case RPC_C_SOCKET_ETIMEDOUT:
         *st = rpc_s_connect_timed_out;
         break;
-
+        
         case RPC_C_SOCKET_ECONNREFUSED:
         *st = rpc_s_connect_rejected;
         break;
-
+        
         case RPC_C_SOCKET_ENETUNREACH:
         *st = rpc_s_network_unreachable;
         break;
-
+        
         case RPC_C_SOCKET_ENOSPC:
         *st = rpc_s_connect_no_resources;
         break;
-
+        
         case RPC_C_SOCKET_ENETDOWN:
         *st = rpc_s_rem_network_shutdown;
         break;
-
+        
 #ifdef ETOOMANYREFS
         case RPC_C_SOCKET_ETOOMANYREFS:
         *st = rpc_s_too_many_rem_connects;
         break;
-#endif
+#endif        
 
         case RPC_C_SOCKET_ESRCH:
         *st = rpc_s_no_rem_endpoint;
         break;
-
-#ifdef EHOSTDOWN
+      
+#ifdef EHOSTDOWN  
         case RPC_C_SOCKET_EHOSTDOWN:
         *st = rpc_s_rem_host_down;
         break;
-#endif
+#endif        
 
         case RPC_C_SOCKET_EHOSTUNREACH:
         *st = rpc_s_host_unreachable;
         break;
-
+        
         case RPC_C_SOCKET_EACCESS:
         *st = rpc_s_invalid_credentials;
         break;
-
+        
         case RPC_C_SOCKET_ECONNABORTED:
         *st = rpc_s_loc_connect_aborted;
         break;
-
+        
         case RPC_C_SOCKET_ECONNRESET:
         *st = rpc_s_connect_closed_by_rem;
         break;
-
+        
         case RPC_C_SOCKET_ENETRESET:
         *st = rpc_s_rem_host_crashed;
         break;
-
+        
         case RPC_C_SOCKET_ENOEXEC:
         *st = rpc_s_invalid_endpoint_format;
         break;
@@ -2089,7 +2087,7 @@ unsigned32              *st;
         case LW_STATUS_KDC_CERT_REVOKED:
         *st = rpc_s_auth_tkt_expired;
         break;
-
+        
         case -1:
         *st = rpc_s_unknown_status_code;
         break;
@@ -2102,7 +2100,7 @@ unsigned32              *st;
         *st = rpc_s_cannot_connect;
         break;
     }
-}
+}    
 
 
 /*
@@ -2121,7 +2119,7 @@ unsigned32              *st;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
 **      rsize           The receive buffer size (rpc_g_cn_socket_read_buffer)
 **
@@ -2172,7 +2170,7 @@ rpc__cn_inq_sock_buffsize(
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:            
 **
 **      st              The status code.
 **
@@ -2236,7 +2234,7 @@ rpc__cn_set_sock_buffsize(
 **--
 **/
 
-PRIVATE void rpc__cn_network_close
+PRIVATE void rpc__cn_network_close 
 #ifdef _DCE_PROTO_
 (
   rpc_binding_rep_p_t     binding_r,
@@ -2253,7 +2251,7 @@ unsigned32              *st;
 
     CODING_ERROR(st);
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_close);
-
+    
     /*
      * Get the association group using the group id contained in the
      * binding handle.
@@ -2262,8 +2260,8 @@ unsigned32              *st;
                                             (binding_r))->grp_id,
                                            RPC_C_CN_ASSOC_GRP_CLIENT,
                                            binding_r->transport_info,
-                                           st);
-
+                                           st); 
+    
     /*
      * If the association group control block can't be found
      * return an error.
