@@ -531,6 +531,7 @@ typedef struct
 {
     unsigned32                  grp_id;
     rpc_cn_syntax_t             *pres_context;
+    boolean                     reuse_context;
     rpc_cn_sec_context_t        *sec_context;
 } rpc_cn_assoc_sm_work_t, *rpc_cn_assoc_sm_work_p_t;
 
@@ -624,6 +625,7 @@ struct rpc_cn_assoc_s_t
     rpc_cn_fragbuf_p_t                  raw_packet_p;
     rpc_cn_assoc_sm_work_p_t            assoc_sm_work;
     unsigned32                          bind_packets_sent;
+    signed long long                    alter_call_id;
 };
 // rpc_cn_assoc_t and rpc_cn_assoc_p_t are declared in comsoc.h
 
