@@ -1,5 +1,5 @@
 /*
- *
+ * 
  * (c) Copyright 1990 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1990 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1990 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- *
+ * 
  */
 /*
  */
@@ -28,11 +28,11 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC)
+**      Remote Procedure Call (RPC) 
 **
 **  ABSTRACT:
 **
-**  Entrypoints to support buffered data transfer within the
+**  Entrypoints to support buffered data transfer within the 
 **  Connection-oriented protocol services component of the RPC
 **  runtime.
 **
@@ -54,7 +54,7 @@
  */
 
 INTERNAL void rpc__cn_prep_next_iovector_elmt _DCE_PROTOTYPE_ ((
-        rpc_cn_call_rep_p_t /*call_rep*/,
+        rpc_cn_call_rep_p_t /*call_rep*/, 
         unsigned32     * /*status*/
     ));
 
@@ -67,7 +67,7 @@ INTERNAL void rpc__cn_prep_next_iovector_elmt _DCE_PROTOTYPE_ ((
 **  SCOPE:              PRIVATE
 **
 **  DESCRIPTION:
-**
+**      
 **  Copies an iovector element to the iovector array in the
 **  call rep.  This routine will buffer data until the total
 **  byte count reaches the max segment size for the transport
@@ -91,14 +91,14 @@ INTERNAL void rpc__cn_prep_next_iovector_elmt _DCE_PROTOTYPE_ ((
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_copy_buffer
+PRIVATE void rpc__cn_copy_buffer 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -117,14 +117,12 @@ unsigned32              *status;
     unsigned32              bytes_left_to_xfer;
     unsigned32              cur_iov_index;
     byte_p_t                src;
-    rpc_cn_packet_p_t       header_p;
 
     src = iov_elt_p->data_addr;
     bytes_left_to_xfer = iov_elt_p->data_len;
-    header_p = (rpc_cn_packet_p_t) RPC_CN_CREP_SEND_HDR (call_rep);
     *status = rpc_s_ok;
 
-    bytes_to_segment_size = call_rep->max_seg_size -
+    bytes_to_segment_size = call_rep->max_seg_size - 
                             RPC_CN_CREP_ACC_BYTCNT (call_rep);
 
     cur_iov_index = RPC_CN_CREP_CUR_IOV_INDX (call_rep);
@@ -136,12 +134,12 @@ unsigned32              *status;
          */
         if (bytes_to_segment_size == 0)
         {
-            /*
+            /* 
              * Transmit all the data buffered thus far.
              */
             rpc__cn_transmit_buffers (call_rep, status);
             rpc__cn_dealloc_buffered_data (call_rep);
-
+            
             /*
              * Fix up the iovector in the call rep so
              * that we again have only the cached protocol
@@ -154,7 +152,7 @@ unsigned32              *status;
                 return;
             }
         }
-        /*
+        /* 
          * Check to see if the current iovector element is full.
          */
         else if (RPC_CN_CREP_FREE_BYTES (call_rep) == 0)
@@ -166,7 +164,7 @@ unsigned32              *status;
              */
             if (RPC_CN_CREP_IOVLEN (call_rep) >= RPC_C_MAX_IOVEC_LEN)
             {
-                /*
+                /* 
                  * Transmit all the data buffered thus far.
                  */
                 rpc__cn_transmit_buffers (call_rep, status);
@@ -220,7 +218,7 @@ unsigned32              *status;
         src += xfer_size;
         RPC_CN_CREP_FREE_BYTE_PTR (call_rep) += xfer_size;
         RPC_CN_CREP_FREE_BYTES (call_rep) -= xfer_size;
-        RPC_CN_CREP_IOV (call_rep) [cur_iov_index].data_len +=
+        RPC_CN_CREP_IOV (call_rep) [cur_iov_index].data_len += 
             xfer_size;
     }
 }
@@ -238,7 +236,7 @@ unsigned32              *status;
 **  Append the specified buffer as a new iovector element to
 **  the iovector in the call rep.  This routine will transfer
 **  data as necessary over the association until the total
-**  accumulated data in the call rep is less than the
+**  accumulated data in the call rep is less than the 
 **  negotiated segment size.
 **
 **  INPUTS:
@@ -249,7 +247,7 @@ unsigned32              *status;
 **                      data to add.
 **
 **  INPUTS/OUTPUTS:     none
-**
+**          
 **  OUTPUTS:
 **
 **      status          The completion status
@@ -258,14 +256,14 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_add_new_iovector_elmt
+PRIVATE void rpc__cn_add_new_iovector_elmt 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -280,15 +278,9 @@ unsigned32              *status;
 #endif
 {
     unsigned32              bytes_to_segment_size;
-    unsigned32              bytes_left_to_xfer;
     unsigned32              cur_iov_index;
-    byte_p_t                src;
     rpc_iovector_elt_p_t    iov_p;
-    rpc_cn_packet_p_t       header_p;
 
-    src = iov_elt_p->data_addr;
-    bytes_left_to_xfer = iov_elt_p->data_len;
-    header_p = (rpc_cn_packet_p_t) RPC_CN_CREP_SEND_HDR (call_rep);
     *status = rpc_s_ok;
 
     /*
@@ -298,7 +290,7 @@ unsigned32              *status;
      */
     if (RPC_CN_CREP_IOVLEN (call_rep) >= RPC_C_MAX_IOVEC_LEN)
     {
-        /*
+        /* 
          * Transmit all the data buffered thus far.
          */
         rpc__cn_transmit_buffers (call_rep, status);
@@ -344,8 +336,8 @@ unsigned32              *status;
     bytes_to_segment_size = call_rep->max_seg_size -
                             RPC_CN_CREP_ACC_BYTCNT (call_rep);
 
-    /*
-     * Only invoke rpc__cn_transmit_buffers() if iov_elt_p->data_len is
+    /* 
+     * Only invoke rpc__cn_transmit_buffers() if iov_elt_p->data_len is 
      * greater than bytes_to_segment_size.
      */
     while (iov_elt_p->data_len > bytes_to_segment_size)
@@ -403,9 +395,9 @@ unsigned32              *status;
          * during the next iteration of this while loop.
          */
         RPC_CN_CREP_ACC_BYTCNT (call_rep) = RPC_CN_CREP_SIZEOF_HDR (call_rep);
-        RPC_CN_CREP_IOV(call_rep)[0].data_len =
+        RPC_CN_CREP_IOV(call_rep)[0].data_len = 
                 RPC_CN_CREP_SIZEOF_HDR (call_rep);
-
+        
         *iov_p = *iov_elt_p;
 
         bytes_to_segment_size = call_rep->max_seg_size -
@@ -424,7 +416,7 @@ unsigned32              *status;
      * element would have length = 0 (since all the data
      * would have been transmitted in the while loop).
      * Free the element in this case.
-     *
+     * 
      */
     if (iov_p->data_len == 0)
     {
@@ -475,7 +467,7 @@ unsigned32              *status;
 **      call_rep        The call rep.
 **
 **  INPUTS/OUTPUTS:     none
-**
+**          
 **  OUTPUTS:
 **
 **      status          The completion status
@@ -484,14 +476,14 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_flush_buffers
+PRIVATE void rpc__cn_flush_buffers 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -527,7 +519,7 @@ unsigned32              *status;
              i < RPC_CN_CREP_IOVLEN (call_rep);
              i++)
         {
-            rpc__cn_copy_buffer (call_rep,
+            rpc__cn_copy_buffer (call_rep, 
                                  &RPC_CN_CREP_IOV (call_rep)[i],
                                  status);
         }
@@ -562,14 +554,14 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-PRIVATE void rpc__cn_transmit_buffers
+PRIVATE void rpc__cn_transmit_buffers 
 #ifdef _DCE_PROTO_
 (
  rpc_cn_call_rep_p_t     call_rep,
@@ -582,7 +574,7 @@ unsigned32              *status;
 #endif
 {
     rpc_cn_packet_p_t   header_p;
-
+    
     /*
      * Write the bytecount accumulated thus far into the fragment
      * length field of the cached protocol header.
@@ -616,7 +608,7 @@ unsigned32              *status;
             }
             else
             {
-                DCETHREAD_TRY
+                DCETHREAD_TRY 
                 {
                     dcethread_checkinterrupt ();
                 }
@@ -663,15 +655,15 @@ unsigned32              *status;
          * trailer must start on a 4-byte boundary. Pad the user, or
          * stub, data to make it so. The amount of padding is
          * contained in the auth trailer so that the receiver can
-         * determine the real user data size.
+         * determine the real user data size. 
          */
         auth_tlr = (rpc_cn_auth_tlr_t *)call_rep->prot_tlr->data_p;
-        auth_tlr->stub_pad_length =
+        auth_tlr->stub_pad_length = 
             (4 - ((RPC_CN_CREP_ACC_BYTCNT (call_rep) -
-                   call_rep->prot_tlr->data_size) & 0x03)) & 0x03;
-        (RPC_CN_CREP_IOV(call_rep)[RPC_CN_CREP_IOVLEN(call_rep) - 2]).data_len +=
+                   call_rep->prot_tlr->data_size) & 0x03)) & 0x03; 
+        (RPC_CN_CREP_IOV(call_rep)[RPC_CN_CREP_IOVLEN(call_rep) - 2]).data_len += 
                                    auth_tlr->stub_pad_length;
-        RPC_CN_PKT_FRAG_LEN (header_p) +=
+        RPC_CN_PKT_FRAG_LEN (header_p) += 
             auth_tlr->stub_pad_length -
             RPC_CN_CREP_SIZEOF_TLR_PAD (call_rep);
 
@@ -681,7 +673,7 @@ unsigned32              *status;
          */
         iov_p = &(RPC_CN_CREP_IOV(call_rep)[RPC_CN_CREP_IOVLEN(call_rep) - 1]);
         iov_p->buff_dealloc = NULL;
-        iov_p->data_len =
+        iov_p->data_len = 
             call_rep->prot_tlr->data_size -
             RPC_CN_CREP_SIZEOF_TLR_PAD (call_rep) ;
         iov_p->data_addr = (byte_p_t) call_rep->prot_tlr->data_p;
@@ -690,8 +682,8 @@ unsigned32              *status;
     /*
      * Send the buffers in the iovector out over the association.
      */
-    rpc__cn_assoc_send_frag (call_rep->assoc,
-                             &(call_rep->buffered_output.iov),
+    rpc__cn_assoc_send_frag (call_rep->assoc, 
+                             &(call_rep->buffered_output.iov), 
                              call_rep->sec,
                              status);
 
@@ -717,7 +709,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL
 **
 **  DESCRIPTION:
-**
+**      
 **  Prepare a new iovector element from the call rep for use
 **  in buffering data.
 **
@@ -736,14 +728,14 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
 **--
 **/
 
-INTERNAL void rpc__cn_prep_next_iovector_elmt
+INTERNAL void rpc__cn_prep_next_iovector_elmt 
 #ifdef _DCE_PROTO_
 (
   rpc_cn_call_rep_p_t     call_rep,
@@ -797,7 +789,7 @@ unsigned32              *status;
 **
 **  DESCRIPTION:
 **
-**  Deallocates all the elements of an iovector (except for the
+**  Deallocates all the elements of an iovector (except for the 
 **  first element).  The first element is assumed to contain the
 **  protocol header which will be reused on subsequent transfers.
 **
@@ -817,7 +809,7 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     none
+**  FUNCTION VALUE:     none       
 **
 **  SIDE EFFECTS:       none
 **
@@ -848,7 +840,7 @@ rpc_cn_call_rep_p_t     call_rep;
 	iov_elmnts--;
     }
 
-    for (cur_iov_index = 1;
+    for (cur_iov_index = 1; 
          cur_iov_index < iov_elmnts;
          cur_iov_index++)
     {
