@@ -76,15 +76,16 @@ INTERNAL void rpc__auth_info_cache_remove _DCE_PROTOTYPE_ ((
     ));
 
 
+
 /*
  * Macro to assign through a pointer iff the pointer is non-NULL.  Note
  * that we depend on the fact that "val" is not evaluated if "ptr" is
  * NULL (i.e., this must be a macro, not a function).
  */
 #define ASSIGN(ptr, val) \
-( \
-    (ptr) != NULL ? *(ptr) = (val) : 0 \
-)
+{ \
+    if ((ptr) != NULL) *(ptr) = (val); \
+}
 
 #define ASSIGN_COPY(buffer, length, val) do { \
         char* _val = (char*) (val);                             \
@@ -475,7 +476,7 @@ rpc_auth_info_p_t       *info;
 #endif
 {
     rpc_auth_info_p_t auth_info = *info;
-    char *info_type;
+    char ATTRIBUTE_UNUSED *info_type;
     
     if (auth_info == NULL)
     {
@@ -1207,19 +1208,19 @@ unsigned32              *st;
     RPC_BINDING_VALIDATE_SERVER(binding_rep, st);
     if (*st != rpc_s_ok)
         return;
-
+    
     auth_info = ((rpc_binding_rep_p_t)binding_h)->auth_info;
-
+    
     if (auth_info == NULL)
     {
         *st = rpc_s_binding_has_no_auth;
         return;
     }
-
+    
     assert(auth_info->is_server);
-
+    
     ASSIGN(privs,               auth_info->u.s.privs);
-
+    
     if (server_princ_name != NULL)
     {
         if (auth_info->server_princ_name == NULL) 
@@ -1579,9 +1580,9 @@ rpc_authn_protocol_id_t             authn_protocol;
              ((server_princ_name == NULL)
               && (auth_info->server_princ_name == NULL))
              ||
-             (server_princ_name
+             (server_princ_name 
 	      && auth_info->server_princ_name
-	      && (strcmp ((char *) server_princ_name,
+	      && (strcmp ((char *) server_princ_name, 
 			  (char *) auth_info->server_princ_name) == 0))
 	    )
             &&
@@ -1709,7 +1710,7 @@ INTERNAL void rpc__auth_info_cache_remove
 rpc_auth_info_p_t       auth_info;
 #endif
 {
-    char *info_type;
+    char ATTRIBUTE_UNUSED *info_type;
 
     assert (!auth_info->is_server);
 
