@@ -89,7 +89,7 @@ typedef int rpc_socket_error_t;
  *
  * New types that allow for socket handles that are more abstract than just a file descriptor
  */
-
+    
 typedef boolean (*rpc_socket_enum_iface_fn_p_t) (
     rpc_socket_t sock,
     rpc_addr_p_t ip_addr,
@@ -271,7 +271,7 @@ typedef struct rpc_socket_vtbl_s
         rpc_access_token_p_t* token
         );
 } rpc_socket_vtbl_t, *rpc_socket_vtbl_p_t;
-
+    
 typedef struct rpc_socket_handle_s
 {
     rpc_socket_vtbl_t* vtbl;
@@ -282,7 +282,7 @@ typedef struct rpc_socket_handle_s
         void* pointer;
     } data;
 } rpc_socket_handle_t, *rpc_socket_handle_p_t;
-
+    
 typedef int rpc_socket_basic_t;
 
 #include <comsoc_sys.h>
@@ -766,7 +766,14 @@ rpc__transport_info_equal(
 
 #define RPC_SOCKET_ETOI(serr)       (serr)
 
-#define RPC_SOCKET_CLOSE(s) \
-    ({ int __err = rpc__socket_close(s); s = NULL; __err; })
+static inline int
+__rpc_socket_close(rpc_socket_t* sock)
+{
+    int __err = rpc__socket_close(*sock);
+    *sock = NULL;
+    return __err;
+}
+
+#define RPC_SOCKET_CLOSE(s) __rpc_socket_close((rpc_socket_t*)(void*)&(s))
 
 #endif /* _COMSOC_H */
