@@ -64,7 +64,7 @@
 /* Bizarre hack for HP-UX ia64 where a system header
  * makes reference to a kernel-only data structure
  */
-#if defined(__hpux) && defined(__ia64)
+#if defined(__hpux) && defined(__ia64) && !defined(_DEFINED_MPINFOU)
 union mpinfou {};
 #endif
 #include <net/if.h>
@@ -361,9 +361,11 @@ rpc__bsd_socket_construct(
     if (!lrpc)
     {
         serr = ENOMEM;
-	goto error;
+        goto error;
     }
 
+    sock->data.pointer = (void*) lrpc;
+
     lrpc->fd                      = -1;
     lrpc->info.peer_uid           = -1;
     lrpc->info.peer_gid           = -1;
@@ -383,16 +385,13 @@ rpc__bsd_socket_construct(
         goto error;
     }
 
-    sock->data.pointer = (void*) lrpc;
-
 done:
+
     return serr;
 
 error:
-    if (lrpc)
-    {
-        rpc__bsd_socket_destruct(sock);
-    }
+
+    rpc__bsd_socket_destruct(sock);
 
     goto done;
 }
@@ -465,27 +464,28 @@ rpc_socket_t        sock;
     rpc_socket_error_t  serr = RPC_C_SOCKET_OK;
     rpc_bsd_socket_p_t lrpc = (rpc_bsd_socket_p_t) sock->data.pointer;
 
-    if (lrpc && lrpc->fd > 0)
-    {
-
-        RPC_LOG_SOCKET_CLOSE_NTR;
-        RPC_SOCKET_DISABLE_CANCEL;
-        serr = (close(lrpc->fd) == -1) ? errno : RPC_C_SOCKET_OK;
-        RPC_SOCKET_RESTORE_CANCEL;
-        RPC_LOG_SOCKET_CLOSE_XIT;
-    }
-
     if (lrpc)
     {
+        if (lrpc->fd > 0)
+        {
+            RPC_LOG_SOCKET_CLOSE_NTR;
+            RPC_SOCKET_DISABLE_CANCEL;
+            serr = (close(lrpc->fd) == -1) ? errno : RPC_C_SOCKET_OK;
+            RPC_SOCKET_RESTORE_CANCEL;
+            RPC_LOG_SOCKET_CLOSE_XIT;
+        }
+
         if (lrpc->info.session_key.data)
         {
             free(lrpc->info.session_key.data);
             lrpc->info.session_key.length = 0;
         }
 
-	free(lrpc);
+        free(lrpc);
     }
 
+    sock->data.pointer = NULL;
+
     return serr;
 }
 
@@ -566,7 +566,7 @@ rpc_addr_p_t        addr;
                 // Ignore any errors from this function.
                 unlink((const char*)addr->sa.data);
             }
-            serr =
+            serr = 
                 (bind(lrpc->fd, (struct sockaddr *)&addr->sa, addr->len) == -1) ?
                       errno : RPC_C_SOCKET_OK;
         }
@@ -1816,7 +1816,7 @@ gid_t		    *egid;
         serr = RPC_C_SOCKET_EACCESS;
         goto error;
     }
-
+    
     cmsg = CMSG_FIRSTHDR(&msg);
     if (!cmsg ||
 	!(cmsg->cmsg_type == SCM_RIGHTS) ||
@@ -1912,7 +1912,7 @@ unsigned16        *session_key_len;
 
 #if 0
     for (i = 0; i < (key_len / sizeof(rand)); i++)
-    {
+    {    
         rand = RPC_RANDOM_GET(1, 0xffffffff);
 	offset = i * sizeof(rand);
 
@@ -2619,7 +2619,7 @@ rpc__bsd_socket_transport_info_equal(
     rpc_bsd_transport_info_p_t bsd_info1 = (rpc_bsd_transport_info_p_t) info1;
     rpc_bsd_transport_info_p_t bsd_info2 = (rpc_bsd_transport_info_p_t) info2;
 
-    return
+    return 
         (bsd_info2 == NULL) ||
         (bsd_info2 != NULL &&
          bsd_info1->session_key.length == bsd_info2->session_key.length &&
@@ -2648,7 +2648,7 @@ rpc__bsd_socket_transport_inq_access_token(
         lrpc_info->peer_uid,
         lrpc_info->peer_gid);
     if (status) goto error;
-
+    
 error:
 
     LwMapSecurityFreeContext(&context);
