@@ -18,6 +18,14 @@
 
 #define SMB_SOCKET_LOCK(sock) (rpc__smb_socket_lock(sock))
 #define SMB_SOCKET_UNLOCK(sock) (rpc__smb_socket_unlock(sock))
+#define GE(err)                                 \
+    do                                          \
+    {                                           \
+        if ((err))                              \
+            goto error;                         \
+    } while (0)
+#define RETRY_INTERVAL 10
+
 
 typedef struct rpc_smb_transport_info_s
 {
@@ -28,7 +36,6 @@ typedef struct rpc_smb_transport_info_s
         unsigned char* data;
     } session_key;
     PIO_CREDS creds;
-    boolean schannel;
 } rpc_smb_transport_info_t, *rpc_smb_transport_info_p_t;
 
 typedef enum rpc_smb_state_e
@@ -47,14 +54,25 @@ typedef struct rpc_smb_buffer_s
     unsigned char* end_cursor;
 } rpc_smb_buffer_t, *rpc_smb_buffer_p_t;
 
+typedef struct rpc_smb_close_context_s
+{
+    IO_STATUS_BLOCK io_status;
+    IO_ASYNC_CONTROL_BLOCK io_async;
+} rpc_smb_close_context_t, *rpc_smb_close_context_p_t;
+
 typedef struct rpc_smb_socket_s
 {
     rpc_smb_state_t volatile state;
     rpc_np_addr_t peeraddr;
     rpc_np_addr_t localaddr;
     rpc_smb_transport_info_t info;
-    PIO_CONTEXT context;
+    IO_FILE_NAME filename;
     IO_FILE_HANDLE np;
+    rpc_smb_close_context_p_t close_context;
+    IO_STATUS_BLOCK io_status;
+    IO_ASYNC_CONTROL_BLOCK io_async;
+    rpc_timer_t retry_timer;
+    unsigned retry_timer_set:1;
     rpc_smb_buffer_t sendbuffer;
     rpc_smb_buffer_t recvbuffer;
     boolean received_last;
@@ -67,13 +85,38 @@ typedef struct rpc_smb_socket_s
     } accept_backlog;
     dcethread* listen_thread;
     dcethread_mutex lock;
+    unsigned lock_init:1;
     dcethread_cond event;
+    unsigned event_init:1;
 } rpc_smb_socket_t, *rpc_smb_socket_p_t;
 
+INTERNAL
+void
+rpc__smb_socket_destroy(
+    rpc_smb_socket_p_t sock
+    );
+
+INTERNAL
+void
+rpc__smb_socket_create_named_pipe_complete(
+    void* data
+    );
+
+INTERNAL
+void
+rpc__smb_socket_connect_named_pipe_complete(
+    void* data
+    );
+
+INTERNAL
+int
+rpc__smb_socket_accept_async(
+    rpc_smb_socket_p_t smb
+    );
+
 void
 rpc_smb_transport_info_from_lwio_creds(
     void* creds,
-    boolean schannel,
     rpc_transport_info_handle_t* info,
     unsigned32* st
     )
@@ -81,7 +124,7 @@ rpc_smb_transport_info_from_lwio_creds(
     rpc_smb_transport_info_p_t smb_info = NULL;
 
     smb_info = calloc(1, sizeof(*smb_info));
-
+    
     if (!smb_info)
     {
         *st = rpc_s_no_memory;
@@ -94,8 +137,6 @@ rpc_smb_transport_info_from_lwio_creds(
         goto error;
     }
 
-    smb_info->schannel = schannel;
-
     *info = (rpc_transport_info_handle_t) smb_info;
 
     *st = rpc_s_ok;
@@ -120,7 +161,7 @@ rpc__smb_transport_info_destroy(
     {
         LwIoDeleteCreds(smb_info->creds);
     }
-
+    
     if (smb_info->session_key.data)
     {
         free(smb_info->session_key.data);
@@ -168,12 +209,10 @@ rpc__smb_transport_info_equal(
     rpc_smb_transport_info_p_t smb_info1 = (rpc_smb_transport_info_p_t) info1;
     rpc_smb_transport_info_p_t smb_info2 = (rpc_smb_transport_info_p_t) info2;
 
-    return
+    return 
         (smb_info2 == NULL
-         && smb_info1->creds == NULL
-         && smb_info1->schannel == FALSE) ||
+         && smb_info1->creds == NULL) ||
         (smb_info2 != NULL &&
-         smb_info1->schannel == smb_info2->schannel &&
          ((smb_info1->creds == NULL && smb_info2->creds == NULL) ||
           (smb_info1->creds != NULL && smb_info2->creds != NULL &&
            LwIoCompareCredss(smb_info1->creds, smb_info2->creds))));
@@ -230,28 +269,28 @@ rpc__smb_buffer_ensure_available(
             serr = ENOMEM;
             goto error;
         }
-
+        
         buffer->end_cursor = buffer->start_cursor = buffer->base;
     }
-
+    
     if (space > rpc__smb_buffer_available(buffer))
     {
         while (space > rpc__smb_buffer_available(buffer))
         {
             buffer->capacity *= 2;
         }
-
+            
         new_base = realloc(buffer->base, buffer->capacity);
-
+        
         if (!new_base)
         {
             serr = ENOMEM;
             goto error;
         }
-
+        
         buffer->start_cursor = new_base + (buffer->start_cursor - buffer->base);
         buffer->end_cursor = new_base + (buffer->end_cursor - buffer->base);
-
+        
         buffer->base = new_base;
     }
 
@@ -301,7 +340,8 @@ rpc__smb_buffer_packet_is_last(
 {
     rpc_cn_common_hdr_p_t packet = (rpc_cn_common_hdr_p_t) buffer->start_cursor;
 
-    return (packet->flags & RPC_C_CN_FLAGS_LAST_FRAG) == RPC_C_CN_FLAGS_LAST_FRAG;
+    return ((packet->flags & RPC_C_CN_FLAGS_LAST_FRAG) == RPC_C_CN_FLAGS_LAST_FRAG &&
+	    packet->ptype != RPC_C_CN_PKT_AUTH3);
 }
 
 INTERNAL
@@ -314,7 +354,7 @@ rpc__smb_buffer_append(
     )
 {
     rpc_socket_error_t serr = RPC_C_SOCKET_OK;
-
+ 
     serr = rpc__smb_buffer_ensure_available(buffer, data_size);
     if (serr)
     {
@@ -326,7 +366,7 @@ rpc__smb_buffer_append(
     buffer->end_cursor += data_size;
 
 error:
-
+    
     return serr;
 }
 
@@ -387,6 +427,13 @@ rpc__smb_socket_create(
         goto done;
     }
 
+    sock->close_context = calloc(1, sizeof(*sock->close_context));
+
+    if (!sock->close_context)
+    {
+        err = ENOMEM;
+        goto done;
+    }
     sock->accept_backlog.selectfd[0] = -1;
     sock->accept_backlog.selectfd[1] = -1;
 
@@ -397,35 +444,50 @@ rpc__smb_socket_create(
     sock->localaddr.sa.sun_path[0] = '\0';
     sock->localaddr.remote_host[0] = '\0';
 
-    dcethread_mutex_init_throw(&sock->lock, NULL);
-    dcethread_cond_init_throw(&sock->event, NULL);
+    DCETHREAD_TRY
+    {
+        dcethread_mutex_init_throw(&sock->lock, NULL);
+        sock->lock_init = TRUE;
+        dcethread_cond_init_throw(&sock->event, NULL);
+        sock->event_init = TRUE;
+    }
+    DCETHREAD_CATCH_ALL(e)
+    {
+        err = ENOMEM;
+    }
+    DCETHREAD_ENDTRY;
 
-    err = LwNtStatusToErrno(LwIoOpenContextShared(&sock->context));
     if (err)
     {
-        goto error;
+        goto done;
     }
 
     *out = sock;
 
 done:
 
-    return err;
-
-error:
-
-    if (sock)
+    if (err && sock)
     {
-        if (sock->context)
-        {
-            LwIoCloseContext(sock->context);
-        }
-
-        dcethread_mutex_destroy_throw(&sock->lock);
-        dcethread_cond_destroy_throw(&sock->event);
+        rpc__smb_socket_destroy(sock);
     }
 
-    goto done;
+    return err;
+}
+
+INTERNAL
+void
+rpc__smb_close_file_complete(
+    void* context
+    )
+{
+    rpc_smb_close_context_p_t close_context = context;
+
+    if (close_context->io_async.AsyncCancelContext)
+    {
+        LwNtDereferenceAsyncCancelContext(&close_context->io_async.AsyncCancelContext);
+    }
+
+    free(close_context);
 }
 
 INTERNAL
@@ -444,24 +506,29 @@ rpc__smb_socket_destroy(
             {
                 if (sock->accept_backlog.queue[i])
                 {
-                    NtCtxCloseFile(sock->context, sock->accept_backlog.queue[i]);
+                    NtCloseFile(sock->accept_backlog.queue[i]);
                 }
             }
-
+            
             close(sock->accept_backlog.selectfd[0]);
             close(sock->accept_backlog.selectfd[1]);
 
             free(sock->accept_backlog.queue);
         }
 
-        if (sock->np && sock->context)
+        if (sock->close_context)
         {
-            NtCtxCloseFile(sock->context, sock->np);
-        }
+            sock->close_context->io_async.Callback = rpc__smb_close_file_complete;
+            sock->close_context->io_async.CallbackContext = sock->close_context;
 
-        if (sock->context)
-        {
-            LwIoCloseContext(sock->context);
+            if (!sock->np ||
+                LwNtAsyncCloseFile(
+                    sock->np,
+                    &sock->close_context->io_async,
+                    &sock->close_context->io_status) != STATUS_PENDING)
+            {
+                free(sock->close_context);
+            }
         }
 
         if (sock->sendbuffer.base)
@@ -473,15 +540,27 @@ rpc__smb_socket_destroy(
         {
             free(sock->recvbuffer.base);
         }
+        
+        if (sock->retry_timer_set)
+        {
+            rpc__timer_clear(&sock->retry_timer);
+        }
 
         rpc__smb_transport_info_destroy(&sock->info);
 
-        dcethread_mutex_destroy_throw(&sock->lock);
-        dcethread_cond_destroy_throw(&sock->event);
+        if (sock->lock_init)
+        {
+            dcethread_mutex_destroy_throw(&sock->lock);
+        }
+
+        if (sock->event_init)
+        {
+            dcethread_cond_destroy_throw(&sock->event);
+        }
 
         free(sock);
     }
-
+    
     return;
 }
 
@@ -565,14 +644,12 @@ rpc__smb_socket_construct(
                 goto error;
             }
         }
-
-        smb_sock->info.schannel = smb_info->schannel;
     }
 
     sock->data.pointer = (void*) smb_sock;
 
 done:
-
+    
     return serr;
 
 error:
@@ -633,7 +710,7 @@ rpc__smb_socket_connect(
     USHORT sesskeylen = 0;
     IO_FILE_NAME filename = { 0 };
     IO_STATUS_BLOCK io_status = { 0 };
-    static const ACCESS_MASK pipeAccess =
+    static const ACCESS_MASK pipeAccess = 
         READ_CONTROL | FILE_WRITE_ATTRIBUTES | FILE_READ_ATTRIBUTES |
         FILE_WRITE_EA | FILE_READ_EA | FILE_READ_DATA | FILE_APPEND_DATA |
         FILE_WRITE_DATA;
@@ -647,7 +724,7 @@ rpc__smb_socket_connect(
     rpc__naf_addr_inq_endpoint (addr,
                                 (unsigned_char_t**) &endpoint,
                                 &dbg_status);
-
+    
     if (!strncmp(endpoint, "\\pipe\\", sizeof("\\pipe\\") - 1) ||
         !strncmp(endpoint, "\\PIPE\\", sizeof("\\PIPE\\") - 1))
     {
@@ -661,15 +738,15 @@ rpc__smb_socket_connect(
 
     serr = NtStatusToErrno(
         LwRtlCStringAllocatePrintf(
-            &smbpath,
-            "\\rdr\\%s\\IPC$\\%s",
+            &smbpath, 
+            "/rdr/%s/IPC$/%s",
             (char*) netaddr,
             (char*) pipename));
     if (serr)
     {
         goto error;
     }
-
+    
     serr = NtStatusToErrno(
         LwRtlWC16StringAllocateFromCString(
             &filename.FileName,
@@ -680,9 +757,7 @@ rpc__smb_socket_connect(
     }
 
     serr = NtStatusToErrno(
-        NtCtxCreateFile(
-            smb->context,                            /* IO context */
-            smb->info.creds,                         /* Security token */
+        NtCreateFile(
             &smb->np,                                /* Created handle */
             NULL,                                    /* Async control block */
             &io_status,                              /* Status block */
@@ -692,21 +767,20 @@ rpc__smb_socket_connect(
             pipeAccess,                              /* Access mode */
             0,                                       /* Allocation size */
             0,                                       /* File attributes */
-            FILE_SHARE_READ | FILE_SHARE_WRITE,      /* Sharing mode */
+            FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE,      /* Sharing mode */
             FILE_OPEN,                               /* Create disposition */
-            FILE_CREATE_TREE_CONNECTION,             /* Create options */
+            FILE_NON_DIRECTORY_FILE | FILE_OPEN_NO_RECALL,             /* Create options */
             NULL,                                    /* EA buffer */
             0,                                       /* EA buffer length */
-            NULL                                     /* ECP List */
-            ));
+            NULL,                                    /* ECP List */
+            smb->info.creds));                       /* Security token */
     if (serr)
     {
         goto error;
     }
 
     serr = NtStatusToErrno(
-        LwIoCtxGetSessionKey(
-            smb->context,
+        LwIoGetSessionKey(
             smb->np,
             &sesskeylen,
             &sesskey));
@@ -777,10 +851,12 @@ rpc__smb_socket_accept(
     char c = 0;
     BYTE clientaddr[16] = {0};
     USHORT clientaddrlen = sizeof(clientaddr);
+    boolean locked = false;
 
     *newsock = NULL;
 
     SMB_SOCKET_LOCK(smb);
+    locked = true;
 
     while (smb->accept_backlog.length == 0)
     {
@@ -810,6 +886,9 @@ rpc__smb_socket_accept(
         }
     }
 
+    SMB_SOCKET_UNLOCK(smb);
+    locked = false;
+
     serr = rpc__socket_open(sock->pseq_id, NULL, &npsock);
     if (serr)
     {
@@ -820,7 +899,7 @@ rpc__smb_socket_accept(
 
     npsmb->np = np;
     np = NULL;
-
+    
     npsmb->state = SMB_STATE_RECV;
 
     memcpy(&npsmb->localaddr, &smb->localaddr, sizeof(npsmb->localaddr));
@@ -830,8 +909,7 @@ rpc__smb_socket_accept(
 
     /* Query for client address */
     serr = NtStatusToErrno(
-        LwIoCtxGetPeerAddress(
-            npsmb->context,
+        LwIoGetPeerAddress(
             npsmb->np,
             clientaddr,
             &clientaddrlen));
@@ -873,8 +951,7 @@ rpc__smb_socket_accept(
     }
 
      serr = NtStatusToErrno(
-        LwIoCtxGetPeerAccessToken(
-            npsmb->context,
+        LwIoGetPeerAccessToken(
             npsmb->np,
             &npsmb->info.access_token));
     if (serr)
@@ -882,10 +959,8 @@ rpc__smb_socket_accept(
         goto error;
     }
 
-
     serr = NtStatusToErrno(
-        LwIoCtxGetSessionKey(
-            npsmb->context,
+        LwIoGetSessionKey(
             npsmb->np,
             &npsmb->info.session_key.length,
             &npsmb->info.session_key.data));
@@ -893,185 +968,216 @@ rpc__smb_socket_accept(
     {
         goto error;
     }
-
+    
     *newsock = npsock;
+    npsock = NULL;
 
 error:
 
     if (np)
     {
-        NtCtxCloseFile(smb->context, np);
+        NtCloseFile(np);
     }
 
-    SMB_SOCKET_UNLOCK(smb);
+    if (npsock)
+    {
+        rpc__socket_close(npsock);
+    }
+
+    if (locked)
+    {
+        SMB_SOCKET_UNLOCK(smb);
+    }
 
     return serr;
 }
 
 INTERNAL
-void*
-rpc__smb_socket_listen_thread(void* data)
+void
+rpc__smb_socket_retry(void* data)
 {
     int serr = RPC_C_SOCKET_OK;
     rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) data;
-    IO_STATUS_BLOCK status_block = { 0 };
-    char *endpoint = NULL;
-    char *pipename = NULL;
-    unsigned32 dbg_status = 0;
-    PSTR smbpath = NULL;
-    IO_FILE_NAME filename = { 0 };
-    size_t i;
-    char c = 0;
-    LONG64 default_timeout = 0;
 
     SMB_SOCKET_LOCK(smb);
 
-    while (smb->state != SMB_STATE_LISTEN)
+    rpc__timer_clear(&smb->retry_timer);
+    smb->retry_timer_set = false;
+
+    serr = rpc__smb_socket_accept_async(smb);
+    GE(serr);
+
+error:
+
+    SMB_SOCKET_UNLOCK(smb);
+
+    return;
+}
+
+INTERNAL
+void
+rpc__smb_socket_retry_in_seconds(rpc_smb_socket_p_t smb, int seconds)
+{
+    rpc__timer_set(
+        &smb->retry_timer,
+        rpc__smb_socket_retry,
+        smb,
+        RPC_CLOCK_SEC(seconds));
+    smb->retry_timer_set = true;
+}
+
+INTERNAL
+int
+rpc__smb_socket_accept_async(
+    rpc_smb_socket_p_t smb
+    )
+{
+    int serr = RPC_C_SOCKET_OK;
+    LONG64 default_timeout = 0;
+    NTSTATUS status = STATUS_SUCCESS;
+
+    smb->io_async.Callback = rpc__smb_socket_create_named_pipe_complete;
+    smb->io_async.CallbackContext = smb;
+
+    status = LwNtCreateNamedPipeFile(
+        &smb->np,                                /* NP handle */
+        &smb->io_async,                          /* Async control */
+        &smb->io_status,                         /* IO status block */
+        &smb->filename,                          /* Filename */
+        NULL,                                    /* Security descriptor */
+        NULL,                                    /* Security QOS */
+        GENERIC_READ | GENERIC_WRITE,            /* Desired access mode */
+        FILE_SHARE_READ | FILE_SHARE_WRITE,      /* Share access mode */
+        FILE_CREATE,                             /* Create disposition */
+        0,                                       /* Create options */
+        0,                                       /* Named pipe type */
+        0,                                       /* Read mode */
+        0,                                       /* Completion mode */
+        smb->accept_backlog.capacity,            /* Maximum instances */
+        0,                                       /* Inbound quota */
+        0,                                       /* Outbound quota */
+        &default_timeout);                       /* Default timeout */
+    if (status == STATUS_PENDING)
+    {
+        status = STATUS_SUCCESS;
+    }
+    serr = NtStatusToErrno(status);
+    GE(serr);
+
+error:
+
+    if (serr)
+    {
+        rpc__smb_socket_retry_in_seconds(smb, RETRY_INTERVAL);
+    }
+
+    return serr;
+}
+
+INTERNAL
+void
+rpc__smb_socket_create_named_pipe_complete(void* data)
+{
+    int serr = RPC_C_SOCKET_OK;
+    rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) data;
+    NTSTATUS status = STATUS_SUCCESS;
+
+    SMB_SOCKET_LOCK(smb);
+
+    LwNtDereferenceAsyncCancelContext(&smb->io_async.AsyncCancelContext);
+
+    serr = NtStatusToErrno(smb->io_status.Status);
+    GE(serr);
+    
+    smb->io_async.Callback = rpc__smb_socket_connect_named_pipe_complete;
+    smb->io_async.CallbackContext = smb;
+
+    status = LwIoConnectNamedPipe(
+        smb->np,
+        &smb->io_async,
+        &smb->io_status);
+
+    if (status == STATUS_PENDING)
+    {
+        status = STATUS_SUCCESS;
+    }
+
+    serr = NtStatusToErrno(status);
+    GE(serr);
+
+error:
+
+    if (serr)
+    {
+        rpc__smb_socket_retry_in_seconds(smb, RETRY_INTERVAL);
+    }
+
+    SMB_SOCKET_UNLOCK(smb);
+
+    return;
+}
+
+INTERNAL
+void
+rpc__smb_socket_connect_named_pipe_complete(void* data)
+{
+    int serr = RPC_C_SOCKET_OK;
+    rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) data;
+    size_t i = 0;
+    char c = 0;
+
+    SMB_SOCKET_LOCK(smb);
+
+    LwNtDereferenceAsyncCancelContext(&smb->io_async.AsyncCancelContext);
+
+    serr = NtStatusToErrno(smb->io_status.Status);
+    GE(serr);
+
+    /* Wait for a slot to open in the accept queue */
+    while (smb->accept_backlog.length == smb->accept_backlog.capacity)
     {
         if (smb->state == SMB_STATE_ERROR)
         {
             goto error;
         }
-
+        
         rpc__smb_socket_wait(smb);
     }
-
-    /* Extract endpoint */
-    rpc__naf_addr_inq_endpoint ((rpc_addr_p_t) &smb->localaddr,
-                                (unsigned_char_t**) &endpoint,
-                                &dbg_status);
-
-    if (!strncmp(endpoint, "\\pipe\\", sizeof("\\pipe\\") - 1) ||
-        !strncmp(endpoint, "\\PIPE\\", sizeof("\\PIPE\\") - 1))
+    
+    /* Put the handle into the accept queue */
+    for (i = 0; i < smb->accept_backlog.capacity; i++)
     {
-        pipename = endpoint + sizeof("\\pipe\\") - 1;
-    }
-    else
-    {
-        serr = EINVAL;
-        goto error;
-    }
-
-    serr = NtStatusToErrno(
-        LwRtlCStringAllocatePrintf(
-            &smbpath,
-            "\\npfs\\%s",
-            (char*) pipename));
-    if (serr)
-    {
-        goto error;
-    }
-
-    serr = NtStatusToErrno(
-        LwRtlWC16StringAllocateFromCString(
-            &filename.FileName,
-            smbpath));
-    if (serr)
-    {
-        goto error;
-    }
-
-    while (smb->state == SMB_STATE_LISTEN)
-    {
-        SMB_SOCKET_UNLOCK(smb);
-
-        serr = NtStatusToErrno(
-            LwNtCtxCreateNamedPipeFile(
-                smb->context,                            /* IO context */
-                NULL,                                    /* Security token */
-                &smb->np,                                /* NP handle */
-                NULL,                                    /* Async control */
-                &status_block,                           /* IO status block */
-                &filename,                               /* Filename */
-                NULL,                                    /* Security descriptor */
-                NULL,                                    /* Security QOS */
-                GENERIC_READ | GENERIC_WRITE,            /* Desired access mode */
-                FILE_SHARE_READ | FILE_SHARE_WRITE,      /* Share access mode */
-                FILE_CREATE,                             /* Create disposition */
-                0,                                       /* Create options */
-                0,                                       /* Named pipe type */
-                0,                                       /* Read mode */
-                0,                                       /* Completion mode */
-                smb->accept_backlog.capacity,            /* Maximum instances */
-                0,                                       /* Inbound quota */
-                0,                                       /* Outbound quota */
-                &default_timeout                         /* Default timeout */
-                ));
-        if (serr)
+        if (smb->accept_backlog.queue[i] == NULL)
         {
-            SMB_SOCKET_LOCK(smb);
-            goto error;
-        }
-
-        serr = NtStatusToErrno(
-            LwIoCtxConnectNamedPipe(
-                smb->context,
-                smb->np,
-                NULL,
-                &status_block));
-        if (serr)
-        {
-            SMB_SOCKET_LOCK(smb);
-            goto error;
-        }
-
-        SMB_SOCKET_LOCK(smb);
-
-        /* Wait for a slot to open in the accept queue */
-        while (smb->accept_backlog.length == smb->accept_backlog.capacity)
-        {
-            if (smb->state == SMB_STATE_ERROR)
+            smb->accept_backlog.queue[i] = smb->np;
+            smb->np = NULL;
+            smb->accept_backlog.length++;
+            if (write(smb->accept_backlog.selectfd[1], &c, sizeof(c)) != sizeof(c))
             {
+                serr = errno;
                 goto error;
             }
-
-            rpc__smb_socket_wait(smb);
-        }
-
-        /* Put the handle into the accept queue */
-        for (i = 0; i < smb->accept_backlog.capacity; i++)
-        {
-            if (smb->accept_backlog.queue[i] == NULL)
-            {
-                smb->accept_backlog.queue[i] = smb->np;
-                smb->np = NULL;
-                smb->accept_backlog.length++;
-                if (write(smb->accept_backlog.selectfd[1], &c, sizeof(c)) != sizeof(c))
-                {
-                    serr = errno;
-                    goto error;
-                }
-                dcethread_cond_broadcast_throw(&smb->event);
-                break;
-            }
+            dcethread_cond_broadcast_throw(&smb->event);
+            break;
         }
     }
+    
+    /* Do it all again */
+    serr = rpc__smb_socket_accept_async(smb);
+    GE(serr);
 
 error:
-    if (filename.FileName)
-    {
-        RtlMemoryFree(filename.FileName);
-    }
-
-    if (smbpath)
-    {
-        RtlMemoryFree(smbpath);
-    }
-
-    // rpc_string_free handles when *ptr is NULL
-    rpc_string_free((unsigned_char_t**) &endpoint, &dbg_status);
 
     if (serr)
     {
-        rpc__smb_socket_change_state(smb, SMB_STATE_ERROR);
+        rpc__smb_socket_retry_in_seconds(smb, RETRY_INTERVAL);
     }
 
     SMB_SOCKET_UNLOCK(smb);
 
-    return NULL;
+    return;
 }
 
-
 INTERNAL
 rpc_socket_error_t
 rpc__smb_socket_listen(
@@ -1081,6 +1187,10 @@ rpc__smb_socket_listen(
 {
     rpc_socket_error_t serr = RPC_C_SOCKET_OK;
     rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) sock->data.pointer;
+    char* endpoint = NULL;
+    char* pipename = NULL;
+    char* smbpath = NULL;
+    unsigned32 dbg_status = 0;
 
     SMB_SOCKET_LOCK(smb);
 
@@ -1100,12 +1210,50 @@ rpc__smb_socket_listen(
         goto error;
     }
 
+    /* Extract endpoint */
+    rpc__naf_addr_inq_endpoint ((rpc_addr_p_t) &smb->localaddr,
+                                (unsigned_char_t**) &endpoint,
+                                &dbg_status);
+    
+    if (!strncmp(endpoint, "\\pipe\\", sizeof("\\pipe\\") - 1) ||
+        !strncmp(endpoint, "\\PIPE\\", sizeof("\\PIPE\\") - 1))
+    {
+        pipename = endpoint + sizeof("\\pipe\\") - 1;
+    }
+    else
+    {
+        serr = EINVAL;
+        GE(serr);
+    }
+
+    serr = NtStatusToErrno(
+        LwRtlCStringAllocatePrintf(
+            &smbpath, 
+            "\\npfs\\%s",
+            (char*) pipename));
+    GE(serr);
+    
+    serr = NtStatusToErrno(
+        LwRtlWC16StringAllocateFromCString(
+            &smb->filename.FileName,
+            smbpath));
+    GE(serr);
+
     smb->state = SMB_STATE_LISTEN;
 
-    dcethread_create_throw(&smb->listen_thread, NULL, rpc__smb_socket_listen_thread, smb);
+    serr = rpc__smb_socket_accept_async(smb);
+    GE(serr);
 
 error:
 
+    if (smbpath)
+    {
+        RtlMemoryFree(smbpath);
+    }
+
+    // rpc_string_free handles when *ptr is NULL
+    rpc_string_free((unsigned_char_t**) &endpoint, &dbg_status);
+
     SMB_SOCKET_UNLOCK(smb);
 
     return serr;
@@ -1126,8 +1274,7 @@ rpc__smb_socket_do_send(
     do
     {
         serr = NtStatusToErrno(
-            NtCtxWriteFile(
-                smb->context,                          /* IO context */
+            NtWriteFile(
                 smb->np,                               /* File handle */
                 NULL,                                  /* Async control block */
                 &io_status,                            /* IO status block */
@@ -1151,7 +1298,7 @@ rpc__smb_socket_do_send(
     rpc__smb_buffer_settle(&smb->sendbuffer);
 
 error:
-
+    
     return serr;
 }
 
@@ -1179,12 +1326,11 @@ rpc__smb_socket_do_recv(
         {
             goto error;
         }
-
+        
         bytes_read = 0;
         bytes_requested = rpc__smb_buffer_available(&smb->recvbuffer);
 
-        status = NtCtxReadFile(
-            smb->context,                 /* IO context */
+        status = NtReadFile(
             smb->np,                      /* File handle */
             NULL,                         /* Async control block */
             &io_status,                   /* IO status block */
@@ -1193,7 +1339,7 @@ rpc__smb_socket_do_recv(
             NULL,                         /* Byte offset */
             NULL                          /* Key */
             );
-
+        
         if (status == STATUS_END_OF_FILE)
         {
             serr = 0;
@@ -1210,7 +1356,7 @@ rpc__smb_socket_do_recv(
     } while (status == STATUS_SUCCESS && bytes_read == bytes_requested);
 
 error:
-
+    
     return serr;
 }
 
@@ -1353,7 +1499,7 @@ rpc__smb_socket_recvmsg(
         /* Reset cursor back to start to begin disperal into scatter buffer */
         smb->recvbuffer.start_cursor = smb->recvbuffer.base;
     }
-
+    
     for (i = 0; i < iov_len; i++)
     {
         pending = rpc__smb_buffer_pending(&smb->recvbuffer);
@@ -1367,7 +1513,7 @@ rpc__smb_socket_recvmsg(
         else
         {
             memcpy(iov[i].iov_base, smb->recvbuffer.start_cursor, pending);
-
+            
             *cc += pending;
 
             /* Reset buffer because we have emptied it */
@@ -1480,13 +1626,13 @@ rpc__smb_socket_getpeername(
     rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) sock->data.pointer;
 
     SMB_SOCKET_LOCK(smb);
-
+    
     if (!smb->np)
     {
         serr = EINVAL;
         goto error;
     }
-
+    
     memcpy(addr, &smb->peeraddr, sizeof(smb->peeraddr));
 
 error:
@@ -1517,7 +1663,7 @@ rpc__smb_socket_set_keepalive(
     )
 {
     rpc_socket_error_t serr = RPC_C_SOCKET_OK;
-
+    
     return serr;
 }
 
@@ -1529,7 +1675,7 @@ rpc__smb_socket_nowriteblock_wait(
     )
 {
     rpc_socket_error_t serr = ENOTSUP;
-
+    
     fprintf(stderr, "WARNING: unsupported smb socket function %s\n", __FUNCTION__);
 
     return serr;
@@ -1601,15 +1747,13 @@ rpc__smb_socket_inq_transport_info(
     rpc_smb_transport_info_p_t smb_info = NULL;
 
     smb_info = calloc(1, sizeof(*smb_info));
-
+    
     if (!smb_info)
     {
         serr = ENOMEM;
         goto error;
     }
 
-    smb_info->schannel = smb->info.schannel;
-
     if (smb->info.creds)
     {
         serr = NtStatusToErrno(LwIoCopyCreds(smb->info.creds, &smb_info->creds));
@@ -1651,7 +1795,7 @@ error:
             rpc_smb_transport_info_free((rpc_transport_info_handle_t) smb_info);
         }
     }
-
+    
     return serr;
 }
 
@@ -1664,7 +1808,7 @@ rpc__smb_socket_transport_inq_access_token(
 {
     rpc_smb_transport_info_p_t smb_info = (rpc_smb_transport_info_p_t) info;
     NTSTATUS status = STATUS_SUCCESS;
-
+    
     RtlReferenceAccessToken(smb_info->access_token);
     *token = smb_info->access_token;
 
