@@ -52,7 +52,7 @@
 #include <comprot.h>
 
 #include <comp.h>
-void rpc__module_init_func(void)
+void rpc__dg_init_func(void)
 {
 	static rpc_protocol_id_elt_t prot[1] = {
 		{
