@@ -1,5 +1,5 @@
 /*
- *
+ * 
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- *
+ * 
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC)
+**      Remote Procedure Call (RPC) 
 **
 **  ABSTRACT:
 **
@@ -54,7 +54,7 @@ INTERNAL void use_protseq _DCE_PROTOTYPE_((
         unsigned_char_p_t  /*endpoint*/,
         rpc_dg_sock_pool_elt_p_t * /*sock_pool_elt*/,
         unsigned32 * /*st*/
-    ));
+    )); 
 
 /* ========================================================================= */
 
@@ -79,7 +79,7 @@ rpc_dg_sock_pool_elt_p_t *sp;
     boolean32 is_private_socket = (*sp)->is_private;
 
     RPC_DG_SOCK_POOL_LOCK_ASSERT(0);
-
+           
     /*
      * Close the socket desc.
      */
@@ -129,7 +129,7 @@ rpc_dg_sock_pool_elt_p_t *sp;
     }
 
     rpc_g_dg_sock_pool.num_entries--;
-
+    
     /*
      * Free up the elt's memory.
      */
@@ -138,7 +138,7 @@ rpc_dg_sock_pool_elt_p_t *sp;
     *sp = NULL;
 }
 
-
+           
 /*
  * U S E _ P R O T S E Q
  *
@@ -174,17 +174,17 @@ unsigned32 *st;
     unsigned32 sndbuf, rcvbuf;
     unsigned32 priv_sock_count = 0;
     unsigned32 desired_sndbuf, desired_rcvbuf;
-
+    
     *st = rpc_s_ok;
 
     RPC_DG_SOCK_POOL_LOCK(0);
-
+    
     /*
      * For clients, first see if there's already an open socket of the
      * correct type and if so, return a ref to it. Note: servers always
      * create new entries.
      */
-
+                                    
     if (! is_server)
     {
         unsigned32 max_priv_socks = RPC_C_DG_SOCK_MAX_PRIV_SOCKS;
@@ -261,20 +261,20 @@ unsigned32 *st;
                 }
             }
         }
-    }
-
+    }  
+    
 
     RPC_DBG_PRINTF(rpc_e_dbg_dg_sockets, 3, (
             "(use_protseq) allocating a %s socket\n",
             creating_private_socket ? "private" : "shared"));
 
-    /*
+    /* 
      * Allocate a new pool entry and initialize it.
-     */
-    RPC_MEM_ALLOC(eltp, rpc_dg_sock_pool_elt_p_t,
-        sizeof(rpc_dg_sock_pool_elt_t), RPC_C_MEM_DG_SOCK_POOL_ELT,
+     */ 
+    RPC_MEM_ALLOC(eltp, rpc_dg_sock_pool_elt_p_t, 
+        sizeof(rpc_dg_sock_pool_elt_t), RPC_C_MEM_DG_SOCK_POOL_ELT, 
         RPC_C_MEM_NOWAIT);
-
+    
     /*
      * Create a network descriptor for this RPC Protocol Sequence.
      */
@@ -288,10 +288,10 @@ unsigned32 *st;
         goto CLEANUP;
     }
     sock_open = true;
-
+     
     /*
      * Bind the socket (Network descriptor) to the RPC address.
-     */
+     */ 
 
     rpc_addr->rpc_protseq_id = pseq_id;
 
@@ -323,7 +323,7 @@ unsigned32 *st;
     serr = rpc__socket_set_bufs(socket_desc,
                                 desired_sndbuf, desired_rcvbuf,
                                 &sndbuf, &rcvbuf);
-
+            
     if (RPC_SOCKET_IS_ERR(serr))
     {
         RPC_DBG_GPRINTF((
@@ -337,7 +337,7 @@ unsigned32 *st;
     RPC_DBG_PRINTF(rpc_e_dbg_general, 3, (
         "(use_protseq) actual sndbuf %lu, actual rcvbuf %lu\n",
         sndbuf, rcvbuf));
-
+      
     /*
      * For shared sockets, set the socket to do non-blocking IO.
      */
@@ -387,7 +387,7 @@ unsigned32 *st;
         eltp->rqe->more_data  = NULL;
         eltp->rqe->frag_len   = 0;
         eltp->rqe->hdrp   = NULL;
-
+ 
         pkt->u.rqe.sock_ref = eltp;
         eltp->rqe_available = true;
     }
@@ -477,11 +477,11 @@ CLEANUP:
     return;
 }
 
-
+         
 /*
  * R P C _ _ D G _ N E T W O R K _ U S E _ P R O T S E Q _ S V
  *
- * Server side entry point into socket pool allocation routine.
+ * Server side entry point into socket pool allocation routine. 
  */
 
 PRIVATE void rpc__dg_network_use_protseq_sv
@@ -501,9 +501,9 @@ rpc_addr_p_t rpc_addr;
 unsigned_char_p_t endpoint;
 unsigned32 *st;
 #endif
-{
+{   
     rpc_dg_sock_pool_elt_p_t sp_elt;
-
+        
     use_protseq(true, pseq_id, rpc_addr, endpoint, &sp_elt, st);
     if (*st == rpc_s_ok)
         rpc__dg_network_sock_release(&sp_elt);
@@ -513,7 +513,7 @@ unsigned32 *st;
 /*
  * R P C _ _ D G _ N E T W O R K _ U S E _ P R O T S E Q _ C L
  *
- * Client side entry point into socket pool allocation routine.
+ * Client side entry point into socket pool allocation routine. 
  * Create a NULL address for the bind, and call the internal
  * use_protseq routine.
  */
@@ -529,10 +529,10 @@ PRIVATE void rpc__dg_network_use_protseq_cl
 rpc_protseq_id_t pseq_id;
 rpc_dg_sock_pool_elt_p_t *sp;
 #endif
-{
+{                          
     rpc_addr_il_t addr;
     unsigned32 st;
-
+        
     *sp = NULL;
 
     /*
@@ -542,12 +542,12 @@ rpc_dg_sock_pool_elt_p_t *sp;
     addr.len = sizeof addr.sa;
     memset((char *) &addr.sa, 0, addr.len);
     addr.sa.family = rpc_g_protseq_id[pseq_id].naf_id;
-
+        
     use_protseq(false, pseq_id, (rpc_addr_p_t) &addr, NULL, sp, &st);
 }
 
 
-/*
+/* 
  * R P C _ _ D G _ N E T W O R K _ D I S A B L E _ D E S C
  *
  * This routine is responsible for removing a socket from the pool of
@@ -566,11 +566,11 @@ PRIVATE void rpc__dg_network_disable_desc
 (sp)
 rpc_dg_sock_pool_elt_p_t sp;
 #endif
-{
+{                                               
     unsigned32 st;
     boolean is_private;
 
-    RPC_DG_SOCK_POOL_LOCK(0);
+    RPC_DG_SOCK_POOL_LOCK(0); 
 
     is_private = sp->is_private;
 
@@ -579,16 +579,16 @@ rpc_dg_sock_pool_elt_p_t sp;
      */
     if (sp->is_disabled)
     {
-        RPC_DG_SOCK_POOL_UNLOCK(0);
+        RPC_DG_SOCK_POOL_UNLOCK(0); 
         return;
     }
-
+       
     RPC_DBG_GPRINTF(("(rpc__dg_network_disable_desc) Disabing socket, error=%d\n",
         sp->sock));
 
     sp->is_disabled = true;
 
-    RPC_DG_SOCK_POOL_UNLOCK(0);
+    RPC_DG_SOCK_POOL_UNLOCK(0); 
 
     /*
      * For shared sockets, remove the sock from the listener's database
@@ -608,7 +608,7 @@ rpc_dg_sock_pool_elt_p_t sp;
 /*
  * R P C _ _ D G _ N E T W O R K _ I N I T
  *
- * This routine is called as part of the rpc__init startup
+ * This routine is called as part of the rpc__init startup 
  * initializations.  It creates the mutex that protects the
  * socket pool.
  */
@@ -620,7 +620,7 @@ PRIVATE void rpc__dg_network_init(void)
 
 /*
  * R P C _ _ D G _ N E T W O R K _ F O R K _ H A N D L E R
- *
+ * 
  * This routine handles socket pool related fork processing.
  */
 
@@ -633,9 +633,8 @@ PRIVATE void rpc__dg_network_fork_handler
 (stage)
 rpc_fork_stage_id_t stage;
 #endif
-{
+{ 
     rpc_dg_sock_pool_elt_p_t eltp, neltp;
-    rpc_socket_error_t serr;
 
     switch ((int)stage)
     {
@@ -643,7 +642,7 @@ rpc_fork_stage_id_t stage;
             break;
         case RPC_C_POSTFORK_PARENT:
             break;
-        case RPC_C_POSTFORK_CHILD:
+        case RPC_C_POSTFORK_CHILD:  
             /*
              * In the child of a fork, scan the socket pool
              * for in_use socket descriptors.
@@ -656,7 +655,7 @@ rpc_fork_stage_id_t stage;
             for (eltp = rpc_g_dg_sock_pool.private_sockets; eltp != NULL;
                  eltp = neltp)
             {
-                serr = RPC_SOCKET_CLOSE(eltp->sock);
+                (void) RPC_SOCKET_CLOSE(eltp->sock);
                 neltp = eltp->next;
                 RPC_MEM_FREE(eltp, RPC_C_MEM_DG_SOCK_POOL_ELT);
             }
@@ -665,7 +664,7 @@ rpc_fork_stage_id_t stage;
             for (eltp = rpc_g_dg_sock_pool.shared_sockets; eltp != NULL;
                  eltp = neltp)
             {
-                serr = RPC_SOCKET_CLOSE(eltp->sock);
+                (void) RPC_SOCKET_CLOSE(eltp->sock);
                 neltp = eltp->next;
                 RPC_MEM_FREE(eltp, RPC_C_MEM_DG_SOCK_POOL_ELT);
             }
@@ -682,11 +681,11 @@ rpc_fork_stage_id_t stage;
  * Increment the reference count associated with an entry in the socket pool.
  */
 PRIVATE void rpc__dg_network_sock_reference(sp)
-rpc_dg_sock_pool_elt_p_t sp;
-{
-    RPC_DG_SOCK_POOL_LOCK(0);
-    sp->refcnt++;
-    RPC_DG_SOCK_POOL_UNLOCK(0);
+rpc_dg_sock_pool_elt_p_t sp;  
+{  
+    RPC_DG_SOCK_POOL_LOCK(0); 
+    sp->refcnt++; 
+    RPC_DG_SOCK_POOL_UNLOCK(0); 
 }
 
 
@@ -705,17 +704,17 @@ PRIVATE void rpc__dg_network_sock_release
 )
 #else
 (sp)
-rpc_dg_sock_pool_elt_p_t *sp;
+rpc_dg_sock_pool_elt_p_t *sp;  
 #endif
-{
-    RPC_DG_SOCK_POOL_LOCK(0);
+{  
+    RPC_DG_SOCK_POOL_LOCK(0); 
 
     if (--((*sp)->refcnt) == 1 && (*sp)->is_disabled)
-        sock_free(sp);
+        sock_free(sp); 
     else
         (*sp)->ccall = NULL;
 
-    RPC_DG_SOCK_POOL_UNLOCK(0);
+    RPC_DG_SOCK_POOL_UNLOCK(0); 
     *sp = NULL;
 }
 
