@@ -32,7 +32,7 @@
 **
 **  ABSTRACT:
 **
-**      Client-side support of kerberos module.
+**      Client-side support of gss-spnego and gss-krb5 module.
 **
 **
 */
@@ -742,28 +742,32 @@ INTERNAL void rpc__gssauth_inq_access_token(
     PLW_MAP_SECURITY_CONTEXT context = NULL;
     rpc_gssauth_info_p_t gssauth_info = NULL;
     rpc_gssauth_cn_info_p_t gssauth_cn_info = NULL;
-
+    
     gssauth_info = (rpc_gssauth_info_p_t)auth_info;
     gssauth_cn_info = gssauth_info->cn_info;
-
+    
     status = LwMapSecurityCreateContext(&context);
     if (status) goto error;
-
+    
     status = LwMapSecurityCreateAccessTokenFromGssContext(
         context,
         token,
         gssauth_cn_info->gss_ctx);
     if (status) goto error;
-
+    
     *stp = rpc_s_ok;
 
 cleanup:
+    if (context)
+    {
+	    LwMapSecurityFreeContext(&context);
+    }
 
     return;
 
 error:
 
     *stp = -1;
-
+    
     goto cleanup;
 }
