@@ -2,7 +2,7 @@
 /*
  * Copyright (c) 2007, Novell, Inc.
  * All rights reserved.
- *
+ * 
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -28,7 +28,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- *
+ * 
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -45,7 +45,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- *
+ * 
  */
 /*
  */
@@ -57,7 +57,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC)
+**      Remote Procedure Call (RPC) 
 **
 **  ABSTRACT:
 **
@@ -185,7 +185,7 @@ INTERNAL boolean addr_compare _DCE_PROTOTYPE_ ((
         rpc_addr_p_t                 /*addr2*/,
         unsigned32                  * /*status*/
     ));
-
+      
 INTERNAL void inq_max_pth_unfrag_tpdu _DCE_PROTOTYPE_ ((
         rpc_addr_p_t                 /*rpc_addr*/,
         rpc_network_if_id_t          /*iftype*/,
@@ -206,7 +206,7 @@ INTERNAL void set_pkt_nodelay _DCE_PROTOTYPE_ ((
         rpc_socket_t                 /*desc*/,
         unsigned32                  * /*status*/
     ));
-
+              
 INTERNAL boolean is_connect_closed _DCE_PROTOTYPE_ ((
         rpc_socket_t                 /*desc*/,
         unsigned32                  * /*status*/
@@ -260,7 +260,7 @@ INTERNAL void inq_max_frag_size _DCE_PROTOTYPE_ ((
 **  SCOPE:              PRIVATE - EPV declared in npnaf.h
 **
 **  DESCRIPTION:
-**
+**      
 **  Named Pipe Family Initialization routine, rpc__np_init, is
 **  calld only once, by the Communications Service initialization
 **  procedure,  at the time RPC is initialized.  If the Communications
@@ -319,12 +319,12 @@ PRIVATE void rpc__np_naf_init_func(void)
          * descriptors.
          */
 	static rpc_tower_prot_ids_t prot_ids[2] = {
-        { RPC_C_PROTSEQ_ID_NCACN_NP,   3,
+        { RPC_C_PROTSEQ_ID_NCACN_NP,   3, 
           { {0x0B,   { 0, 0, 0, 0, 0, {0} }}, /* Connection-oriented */
             {0x0F,   { 0, 0, 0, 0, 0, {0} }}, /* SMB Named Pipes */
             {0x11,   { 0, 0, 0, 0, 0, {0} }}, /* NetBIOS host */
             {0x00,   { 0, 0, 0, 0, 0, {0} }} } },
-        { RPC_C_PROTSEQ_ID_NCALRPC,   2,
+        { RPC_C_PROTSEQ_ID_NCALRPC,   2, 
           { {0x0B,   { 0, 0, 0, 0, 0, {0} }}, /* Connection-oriented */
             {0x20,   { 0, 0, 0, 0, 0, {0} }}, /* socket pathname */
             {0x00,   { 0, 0, 0, 0, 0, {0} }} } }
@@ -336,7 +336,7 @@ PRIVATE void rpc__np_naf_init_func(void)
         RPC_C_PROTSEQ_ID_NCACN_NP,
         RPC_C_PROTOCOL_ID_NCACN,
         RPC_C_NAF_ID_UXD,
-        RPC_C_NETWORK_PROTOCOL_ID_NP,
+        RPC_C_NETWORK_PROTOCOL_ID_NP, 
         RPC_C_NETWORK_IF_ID_STREAM,
         RPC_PROTSEQ_NCACN_NP,
         (rpc_port_restriction_list_p_t) NULL,
@@ -362,19 +362,19 @@ PRIVATE void rpc__np_naf_init_func(void)
 }
 #endif
 
-PRIVATE void  rpc__np_init
+PRIVATE void  rpc__np_init 
 #ifdef _DCE_PROTO_
 (
     rpc_naf_epv_p_t         *naf_epv,
     unsigned32              *status
 )
 #else
-(naf_epv, status)
+(naf_epv, status)  
 rpc_naf_epv_p_t         *naf_epv;
 unsigned32              *status;
 #endif
 {
-    /*
+    /*  
      * The Internal Entry Point Vectors for the Internet Protocol Family
      * Extension service routines.  At RPC startup time, the IP init routine,
      * rpc__np_init, is responsible for inserting  a pointer to this EPV into
@@ -408,7 +408,7 @@ unsigned32              *status;
         set_port_restriction,
         get_next_restricted_port,
         inq_max_frag_size
-    };
+    };      
     unsigned32 lstatus;
 
     rpc__np_init_local_addr_vec (&lstatus);
@@ -429,7 +429,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Create a copy of an RPC address. Allocate memory for a variable
 **  length RPC address, for the NP service.  Insert the Internet
 **  address and endpoint along with the overall length of the allocated
@@ -457,7 +457,7 @@ unsigned32              *status;
 **
 **      rpc_addr        The address of a pointer to an RPC address -
 **                      returned with the address of the memory
-**                      allocated by this routine.
+**                      allocated by this routine. 
 **
 **  OUTPUTS:
 **
@@ -481,7 +481,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_alloc
+INTERNAL void addr_alloc 
 #ifdef _DCE_PROTO_
 (
     rpc_protseq_id_t        rpc_protseq_id,
@@ -504,7 +504,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-
+    
     /*
      * allocate memory for the new RPC address
      */
@@ -547,7 +547,7 @@ unsigned32              *status;
      */
     addr_set_netaddr (netaddr, rpc_addr, status);
     if (*status != rpc_s_ok) return;
-
+    
     *status = rpc_s_ok;
 }
 
@@ -559,7 +559,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Obtain the length from the source RPC address.  Allocate memory for a
 **  new, destination  RPC address. Do a byte copy from the surce address
 **  to the destination address.
@@ -568,13 +568,13 @@ unsigned32              *status;
 **
 **     src_rpc_addr     The address of a pointer to an RPC address to be
 **                      copied.  It must be the correct format for Internet
-**                      Protocol.
+**                      Protocol. 
 **
 **  INPUTS/OUTPUTS:
 **
 **     dst_rpc_addr     The address of a pointer to an RPC address -returned
 **                      with the address of the memory allocated by
-**                      this routine.
+**                      this routine. 
 **
 **  OUTPUTS:
 **
@@ -599,13 +599,13 @@ unsigned32              *status;
 **  SIDE EFFECTS:
 **
 **           In the event, the addres of a of memory segment contained in
-**           rpc_addr, is not valid or the length isn't as long as is
+**           rpc_addr, is not valid or the length isn't as long as is 
 **           indicated, a memory fault may result.
 **
 **--
 **/
 
-INTERNAL void addr_copy
+INTERNAL void addr_copy 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            src_rpc_addr,
@@ -620,7 +620,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-
+    
     /*
      * if the source RPC address looks valid - IP family ok
      */
@@ -666,7 +666,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Free the memory for the RPC address pointed to by the argument
 **  address pointer rpc_addr.  Null the address pointer.  The memory
 **  must have been previously allocated by RPC_MEM_ALLC.
@@ -691,14 +691,14 @@ unsigned32              *status;
 **
 **  FUNCTION VALUE:     none
 **
-**  SIDE EFFECTS:
+**  SIDE EFFECTS:       
 **           In the event, the segment of memory refered to by pointer
 **           rpc_addr, is allocated by means other than RPC_MEM_ALLOC,
 **           unpredictable results will occur when this routine is called.
 **--
 **/
 
-INTERNAL void addr_free
+INTERNAL void addr_free 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            *rpc_addr,
@@ -711,7 +711,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-
+    
     /*
      * free memory of RPC addr
      */
@@ -733,7 +733,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **    Receive the null terminated ascii character string,  rpc_endpoint
 **    and convert to the Internet Protocol byte order format.  Insert
 **    into the RPC address, pointed to by argument rpc_addr.  The only
@@ -755,14 +755,14 @@ unsigned32              *status;
 **
 **          rpc_s_ok                 The call was successful.
 **
-**          rpc_s_invalid_naf_id  Argument, endpoint contains an
+**          rpc_s_invalid_naf_id  Argument, endpoint contains an 
 **                                  unauthorized pointer value.
-**          rpc_s_invalid_endpoint_format Endpoint Argument can not be
+**          rpc_s_invalid_endpoint_format Endpoint Argument can not be 
 **                                  converted (not numeric).
 **
 **  IMPLICIT INPUTS:
 **
-**      A NULL, (first byte NULL), endpoint string is an indicator to
+**      A NULL, (first byte NULL), endpoint string is an indicator to 
 **      the routine to delete the endpoint from the RPC address. indicated.
 **
 **  IMPLICIT OUTPUTS:   none
@@ -774,7 +774,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_set_endpoint
+INTERNAL void addr_set_endpoint 
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       endpoint,
@@ -791,9 +791,9 @@ unsigned32              *status;
     rpc_np_addr_p_t     np_addr = (rpc_np_addr_p_t) *rpc_addr;
     unsigned_char_p_t   p;
     size_t              req_len;
-
+ 
     CODING_ERROR (status);
-
+    
     /*
      * check to see if this is a request to remove the endpoint
      */
@@ -805,7 +805,7 @@ unsigned32              *status;
     }
 
     if (np_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCACN_NP)
-    {
+    { 
         if (strncasecmp((char *)endpoint, "\\PIPE\\", 6) != 0)
         {
             *status = rpc_s_invalid_endpoint_format;
@@ -835,12 +835,12 @@ unsigned32              *status;
      * Convert backslashes to forward slashes to conform to
      * UNIX filesystem convention.
      */
-    if (np_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCACN_NP)
+    if (np_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCALRPC)
     {
         for (p = (unsigned char*) &np_addr->sa.sun_path[RPC_C_NP_DIR_LEN]; *p != '\0'; p++) {
             if (*p == '\\')
                 *p = '/';
-        }
+        } 
     }
 
     *status = rpc_s_ok;
@@ -854,7 +854,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **    From the RPC address indicated by arg., rpc_addr, examine the
 **    endpoint.  Convert the endopint value to a NULL terminated asci
 **    character string to be returned in the memory segment pointed to
@@ -897,7 +897,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_endpoint
+INTERNAL void addr_inq_endpoint 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr,
@@ -924,7 +924,7 @@ unsigned32              *status;
     /*
      * if no endpoint present, return null string. Otherwise,
      * return the endpoint in Internet "dot" notation.
-     */
+     */    
     if (sun_path[0] == 0)
     {
         RPC_MEM_ALLOC(
@@ -968,7 +968,7 @@ unsigned32              *status;
         }
     }
 
-    *status = rpc_s_ok;
+    *status = rpc_s_ok;    
 }
 
 /*
@@ -979,7 +979,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **    Receive the null terminated ascii character string, netaddr,
 **    and convert to the Internet Protocol Network Address format.  Insert
 **    into the RPC address, indicated by argument rpc_addr.
@@ -1015,7 +1015,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_set_netaddr
+INTERNAL void addr_set_netaddr 
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       netaddr,
@@ -1031,7 +1031,7 @@ unsigned32              *status;
 {
     rpc_np_addr_p_t     np_addr = (rpc_np_addr_p_t) *rpc_addr;
 
-    CODING_ERROR (status);
+    CODING_ERROR (status);  
 
     /*
      * check to see if this is a request to remove the netaddr
@@ -1043,7 +1043,8 @@ unsigned32              *status;
         return;
     }
 
-    strncpy((char*) &np_addr->remote_host, (char*) netaddr, sizeof(np_addr->remote_host));
+    strncpy((char*) &np_addr->remote_host, (char*) netaddr, sizeof(np_addr->remote_host) - 1);
+    np_addr->remote_host[sizeof(np_addr->remote_host) - 1] = '\0';
 
     *status = rpc_s_ok;
 }
@@ -1056,7 +1057,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **    From the RPC address indicated by arg., rpc_addr, examine the
 **    IP network address.  Convert the network address from its network
 **    format  to a NULL terminated ascii character string in IP dot
@@ -1091,7 +1092,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_netaddr
+INTERNAL void addr_inq_netaddr 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr,
@@ -1144,7 +1145,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **     Receive a NULL terminated network options string and insert
 **     into the RPC address indicated by art., rpc_addr.
 **
@@ -1178,7 +1179,7 @@ unsigned32              *status;
 **/
 
 
-INTERNAL void addr_set_options
+INTERNAL void addr_set_options 
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       network_options ATTRIBUTE_UNUSED,
@@ -1203,7 +1204,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **      Extract the network options from the RPC address pointed to
 **      by rpc_addr and convert to a NULL terminated string placed
 **      in a buffer indicated by the options arg.
@@ -1239,7 +1240,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_options
+INTERNAL void addr_inq_options 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr ATTRIBUTE_UNUSED,
@@ -1272,7 +1273,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1284,7 +1285,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      max_tsdu
 **
@@ -1301,7 +1302,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_tsdu
+INTERNAL void inq_max_tsdu 
 #ifdef _DCE_PROTO_
 (
     rpc_naf_id_t            naf_id ATTRIBUTE_UNUSED,
@@ -1342,7 +1343,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Determine if two address are equal.
 **
 **  INPUTS:
@@ -1353,7 +1354,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1361,7 +1362,7 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:    
 **
 **      return          Boolean; true if address are the same.
 **
@@ -1370,10 +1371,10 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL boolean addr_compare
+INTERNAL boolean addr_compare 
 #ifdef _DCE_PROTO_
 (
-    rpc_addr_p_t            addr1,
+    rpc_addr_p_t            addr1, 
     rpc_addr_p_t            addr2,
     unsigned32              *status ATTRIBUTE_UNUSED
 )
@@ -1409,7 +1410,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1421,7 +1422,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      max_tpdu
 **
@@ -1438,7 +1439,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_pth_unfrag_tpdu
+INTERNAL void inq_max_pth_unfrag_tpdu 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr ATTRIBUTE_UNUSED,
@@ -1479,7 +1480,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1491,7 +1492,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      max_tpdu
 **
@@ -1508,7 +1509,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_loc_unfrag_tpdu
+INTERNAL void inq_max_loc_unfrag_tpdu 
 #ifdef _DCE_PROTO_
 (
     rpc_naf_id_t            naf_id ATTRIBUTE_UNUSED,
@@ -1549,7 +1550,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  This routine is responsible for "reverse-engineering" the parameters to
 **  the original socket call that was made to create the socket "desc".
 **
@@ -1580,7 +1581,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void desc_inq_network
+INTERNAL void desc_inq_network 
 #ifdef _DCE_PROTO_
 (
     rpc_socket_t              desc,
@@ -1606,7 +1607,7 @@ unsigned32                *status;
      * Get the socket type.
      */
     *protocol_id = 0;
-
+ 
     serr = rpc__socket_get_if_id (desc, socket_type);
     if (RPC_SOCKET_IS_ERR (serr))
     {
@@ -1634,9 +1635,9 @@ unsigned32                *status;
     /* XXX should check for arbitrary number of delimiters. */
     is_np = (strncmp(addr.sa.sun_path, RPC_C_NP_DIR "//PIPE", RPC_C_NP_DIR_LEN + 6) == 0) ||
             (strncmp(addr.sa.sun_path, RPC_C_NP_DIR "/PIPE", RPC_C_NP_DIR_LEN + 5) == 0);
-
+ 
     /*
-     * This is a disgusting hack due to the tight binding between
+     * This is a disgusting hack due to the tight binding between 
      * the BSD socket API and the OSF runtime. Domain sockets use
      * the "IP" protocol and named pipes use "ICMP". Yuck.
      */
@@ -1672,7 +1673,7 @@ unsigned32                *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  INPUTS:
 **
 **      desc            The network descriptor to apply the nodelay
@@ -1680,7 +1681,7 @@ unsigned32                *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1695,7 +1696,7 @@ unsigned32                *status;
 **--
 **/
 
-INTERNAL void set_pkt_nodelay
+INTERNAL void set_pkt_nodelay 
 #ifdef _DCE_PROTO_
 (
     rpc_socket_t            desc,
@@ -1726,14 +1727,14 @@ unsigned32              *status;
 **      sequenced sockets the routine is a no-op. "true" is returned
 **      because zero bytes received on a stream socket does mean the
 **      connection is closed.
-**
+**      
 **  INPUTS:
 **
 **      desc            The network descriptor representing the connection.
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1741,7 +1742,7 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:
+**  FUNCTION VALUE:     
 **
 **      boolean         true if the connection is closed, false otherwise.
 **
@@ -1750,7 +1751,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL boolean is_connect_closed
+INTERNAL boolean is_connect_closed 
 #ifdef _DCE_PROTO_
 (
     rpc_socket_t            desc ATTRIBUTE_UNUSED,
@@ -1775,7 +1776,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Creates the lower tower floors from an RPC addr.
 **
 **  INPUTS:
@@ -1801,7 +1802,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void tower_flrs_from_addr
+INTERNAL void tower_flrs_from_addr 
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t       rpc_addr,
@@ -1815,8 +1816,6 @@ twr_p_t            *lower_flrs;
 unsigned32         *status;
 #endif
 {
-    unsigned32    net_prot_id;
-
     CODING_ERROR (status);
 
 #ifdef RPC_NO_TOWER_SUPPORT
@@ -1832,7 +1831,7 @@ unsigned32         *status;
      * their own memory allocation. In that case, twr_* allocates
      * memory for returning the lower towers, and we must copy from
      * twr_* memory into our (rpc) memory. After the copy, we free
-     * the twr_* allocated memory.
+     * the twr_* allocated memory. 
      *
      * For now, twr_* routines also use RPC_MEM_ALLOC and RPC_MEM_FREE,
      * so we'll skip the extra copy.
@@ -1840,29 +1839,23 @@ unsigned32         *status;
     twr_p_t     temp_lower_flrs;
 #endif
 
-    /*
-     * Get the network protocol id (aka transport layer protocol)
-     * for this RPC addr.
-     */
-    net_prot_id = RPC_PROTSEQ_INQ_NET_PROT_ID(rpc_addr->rpc_protseq_id);
-
     /*
      * Convert sockaddr to lower tower floors.
      */
     if (rpc_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCACN_NP)
-        twr_np_lower_flrs_from_sa ((sockaddr_t *) &(rpc_addr->sa),
+        twr_np_lower_flrs_from_sa ((sockaddr_t *) &(rpc_addr->sa), 
 #if 0
-        &temp_lower_flrs,
+        &temp_lower_flrs, 
 #else
-        lower_flrs,
+        lower_flrs, 
 #endif
         status);
     else
         twr_uxd_lower_flrs_from_sa ((sockaddr_t *) &(rpc_addr->sa),
 #if 0
-        &temp_lower_flrs,
+        &temp_lower_flrs, 
 #else
-        lower_flrs,
+        lower_flrs, 
 #endif
         status);
 
@@ -1876,18 +1869,18 @@ unsigned32         *status;
      * Allocate a tower structure to hold the wire (and nameservice)
      * representation of the lower tower floors returned from twr_*().
      *
-     * The size includes the sizof twr_t + length of the tower floors
+     * The size includes the sizof twr_t + length of the tower floors 
      * returned from twr_np_lower_flrs_from_sa - 1 (for tower_octet_string[0].
      */
     RPC_MEM_ALLOC (
-        *lower_flrs,
-        twr_p_t,
+        *lower_flrs, 
+        twr_p_t, 
         sizeof(twr_t) + temp_lower_flrs->tower_length - 1,
-        RPC_C_MEM_TOWER,
+        RPC_C_MEM_TOWER, 
         RPC_C_MEM_WAITOK );
 
     /*
-     * Set the tower length to the length of the tower flrs returnd from
+     * Set the tower length to the length of the tower flrs returnd from 
      * twr_np_lower_flrs_from_sa.
      */
     (*lower_flrs)->tower_length = temp_lower_flrs->tower_length;
@@ -1895,7 +1888,7 @@ unsigned32         *status;
     /*
      * Copy the lower tower floors to the tower octet string.
      */
-    memcpy ((*lower_flrs)->tower_octet_string,
+    memcpy ((*lower_flrs)->tower_octet_string, 
         temp_lower_flrs->tower_octet_string,
         temp_lower_flrs->tower_length);
 
@@ -1918,7 +1911,7 @@ unsigned32         *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  Creates an RPC addr from a protocol tower.
 **
 **  INPUTS:
@@ -1947,7 +1940,7 @@ unsigned32         *status;
 **--
 **/
 
-INTERNAL void tower_flrs_to_addr
+INTERNAL void tower_flrs_to_addr 
 #ifdef _DCE_PROTO_
 (
     byte_p_t           tower_octet_string,
@@ -1990,7 +1983,7 @@ unsigned32         *status;
 
     if (*status != rpc_s_ok)
         return;
-
+ 
     /*
      * Call the common NAF routine to create an RPC addr from a sockaddr.
      * (rpc__naf_addr_from_sa doesn't dispatch to a naf-specific routine.)
@@ -1998,7 +1991,7 @@ unsigned32         *status;
     rpc__naf_addr_from_sa (sa, sa_len, rpc_addr, status);
 
     /*
-     * Always free the twr_np_lower_flrs_to_sa allocated memory - regardless
+     * Always free the twr_np_lower_flrs_to_sa allocated memory - regardless 
      * of the status from rpc__naf_addr_from_sa.
      */
     RPC_MEM_FREE (sa, RPC_C_MEM_SOCKADDR);
@@ -2020,7 +2013,7 @@ unsigned32         *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **    Receive a socket descriptor which is queried to obtain family,
 **    remote endpoint and remote network address.  If this information appears valid
 **    for an DECnet IV address,  space is allocated for an RPC address which
@@ -2067,7 +2060,7 @@ unsigned32         *status;
 **--
 **/
 
-INTERNAL void desc_inq_peer_addr
+INTERNAL void desc_inq_peer_addr 
 #ifdef _DCE_PROTO_
 (
     rpc_protseq_id_t        protseq_id,
@@ -2149,21 +2142,21 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL
 **
 **  DESCRIPTION:
-**
+**      
 **  Builds an rpc_port_restriction_list_t and glues it into the
-**  rpc_protseq_id_elt_t in the rpc_g_protseq_id[] list.
-**
+**  rpc_protseq_id_elt_t in the rpc_g_protseq_id[] list.  
+**  
 **  INPUTS:
 **
 **      protseq_id
 **                      The protocol sequence id to set port restriction
-**                      on.
+**                      on. 
 **      n_elements
 **                      The number of port ranges passed in.
 **
 **      first_port_name_list
 **                      An array of pointers to strings containing the
-**                      lower bound port names.
+**                      lower bound port names. 
 **
 **      last_port_name_list
 **                      An array of pointers to strings containing the
@@ -2196,7 +2189,7 @@ INTERNAL void set_port_restriction
      unsigned_char_p_t           *last_port_name_list,
      unsigned32                  *status
 )
-#else
+#else 
 (protseq_id, n_elements, first_port_name_list, last_port_name_list, status)
 rpc_protseq_id_t            protseq_id;
 unsigned32                  n_elements;
@@ -2224,15 +2217,15 @@ unsigned32                  *status;
 **  SCOPE:              INTERNAL
 **
 **  DESCRIPTION:
-**
+**      
 **  Returns the next restricted port in a sequence.  There is no guarantee
 **  that the port is available, that is up to bind() to determine.
-**
+**  
 **  INPUTS:
 **
 **      protseq_id
-**                      The protocol sequence id to get the port on.
-**
+**                      The protocol sequence id to get the port on. 
+**      
 **  INPUTS/OUTPUTS:     none
 **
 **  OUTPUTS:
@@ -2271,7 +2264,7 @@ unsigned32                 *status;
 	protseq_id = 0;
 	port_name = NULL;
 
-	/*
+	/* 
 	 * Return an error to tell the caller that there is no range
 	 * restriction on this protocol sequence.
 	 */
@@ -2288,7 +2281,7 @@ unsigned32                 *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**
+**      
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -2300,7 +2293,7 @@ unsigned32                 *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:
+**  OUTPUTS:                        
 **
 **      max_tpdu
 **
