@@ -183,9 +183,11 @@
 #define RPC_C_MEM_SCHNAUTH_INFO    98
 #define RPC_C_MEM_SCHNAUTH_CN_INFO 99
 #define RPC_C_MEM_NAMED_PIPE_INFO  100      /* rpc_np_auth_info_t */
+#define RPC_C_MEM_NTLMAUTH_INFO    101      /* rpc_ntlmauth_info_t */
+#define RPC_C_MEM_NTLMAUTH_CN_INFO 102      /* rpc_ntlmauth_cn_info_t */
 
 /* can only use up to "rpc_c_mem_maxtypes - 1" without upping it */
-#define RPC_C_MEM_MAX_TYPES        101       /* i.e. 0 : (max_types - 1)     */
+#define RPC_C_MEM_MAX_TYPES        103       /* i.e. 0 : (max_types - 1)     */
 
 
 /*
