@@ -203,7 +203,7 @@ unsigned32        *status;
     gethostname(&hostname[2], MAXHOSTNAMELEN - 3);
     hostname[MAXHOSTNAMELEN - 1] = '\0';
     for (p = &hostname[2]; *p != '\0'; p++)
-        *p = toupper(*p);
+        *p = toupper((int)*p);
     related_data_size[1] = strlen(hostname) + 1;
     related_data_ptr[1] = (unsigned char*) hostname;
 
@@ -386,6 +386,8 @@ unsigned32        *status;
     CODING_ERROR (status);
     RPC_VERIFY_INIT ();
 
+    id_size = 0;
+
     /* 
      * Make sure we have a pointer to some data structure.
      */
