@@ -116,10 +116,12 @@ INTERNAL void register_ifs
         error_status_t  *status
     ));
 
+#if 0
 INTERNAL void use_protseqs
     _DCE_PROTOTYPE_ ((
         error_status_t  *status
     ));
+#endif
 
 INTERNAL void init
     _DCE_PROTOTYPE_ ((
@@ -195,6 +197,10 @@ GLOBAL   boolean32      dflag = false;
 INTERNAL boolean32      foreground = false;
 
 
+/* Dcerpcd was asked to shutdown and all threads should exit */
+INTERNAL boolean32      shutdown_threads = false;
+
+
 INTERNAL char           rpcd_version_str[] =
 #ifdef ENABLE_DCOM
 	"rpcd/orpcd version freedce 1.1";
@@ -344,6 +350,7 @@ char            *argv[];
         }
 
         protseq[num_protseq++] = (unsigned_char_p_t) argv[i];
+        use_all_protseqs = false;
     }
 }
 
@@ -356,7 +363,7 @@ error_status_t  *status;
     ept_v3_0_epv_t* _epv = &ept_v3_0_mgr_epv;
     rpc_mgr_epv_t epv = (rpc_mgr_epv_t) _epv;
 
-    rpc_server_register_if(ept_v3_0_s_ifspec, (dce_uuid_p_t)NULL,
+    rpc_server_register_if(ept_v3_0_s_ifspec, (dce_uuid_p_t)NULL, 
             epv, status);
     if (check_st_bad("Unable to rpc_server_register_if for ept", status))
         return;
@@ -377,6 +384,7 @@ error_status_t  *status;
 	 
 }
 
+#if 0
 /*
  * Arrange to handle calls on the protocol sequences of interest.
  * Note that while both interfaces specify well know endpoints,
@@ -443,6 +451,7 @@ error_status_t  *status;
             return;
     }
 }
+#endif
 
 /*
  * Do some server database, ... initialization
@@ -614,231 +623,6 @@ error_status_t          *status;
 
 //#if defined(UNIX) || defined(unix)
 
-#define RPCD_PID_FILE "/var/run/dcerpcd.pid" 
-#define PID_FILE_CONTENTS_SIZE ((9 * 2) + 2)
-#define RPCD_DAEMON_NAME "dcerpcd"
-
-static
-void
-StripLeadingWhitespace(
-    char* str
-    )
-{
-    char* pszNew = str;
-    char* pszTmp = str;
-
-    if (!str || !*str || !isspace((int)*str)) {
-        return;
-    }
-
-    while (pszTmp != NULL && *pszTmp != '\0' && isspace((int)*pszTmp)) {
-        pszTmp++;
-    }
-
-    while (pszTmp != NULL && *pszTmp != '\0') {
-        *pszNew++ = *pszTmp++;
-    }
-    *pszNew = '\0';
-}
-
-static
-void
-StripTrailingWhitespace(
-    char* str
-    )
-{
-    char* pszLastSpace = NULL;
-    char* pszTmp = str;
-
-    if (!str || !*str) {
-        return;
-    }
-
-    while (pszTmp != NULL && *pszTmp != '\0') {
-        pszLastSpace = (isspace((int)*pszTmp) ? (pszLastSpace ? pszLastSpace : pszTmp) : NULL);
-        pszTmp++;
-    }
-
-    if (pszLastSpace != NULL) {
-        *pszLastSpace = '\0';
-    }
-}
-
-static
-void
-StripWhitespace(
-    char* str
-    )
-{
-    if (!str || !*str)
-       return;
-    StripLeadingWhitespace(str);
-    StripTrailingWhitespace(str);
-}
-
-static
-int
-MatchProgramToPID(
-    const char* pszProgramName,
-    pid_t pid
-    )
-{
-    int ceError = 0;
-    char szBuf[PATH_MAX+1];
-    FILE* pFile = NULL;
-
-#if defined(__MACH__) && defined(__APPLE__)
-    sprintf(szBuf, "ps -p %d -o command= | grep %s", pid, pszProgramName);
-#else
-    sprintf(szBuf, "UNIX95=1 ps -p %ld -o comm= | grep %s", (long)pid, pszProgramName);
-#endif
-
-    pFile = popen(szBuf, "r");
-    if (pFile == NULL) {
-        ceError = errno;
-        goto error;
-    }
-
-    while (TRUE) {
-
-        if (NULL == fgets(szBuf, PATH_MAX, pFile)) {
-            if (feof(pFile))
-                break;
-            else {
-                ceError = errno;
-                goto error;
-            }
-        }
-
-        StripWhitespace(szBuf);
-        if (*szBuf) {
-            ceError = 0;
-            break;
-        }
-
-    }
-
-error:
-
-    if (pFile)
-        fclose(pFile);
-
-    return ceError;
-}
-
-static
-pid_t
-pid_from_pid_file()
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(RPCD_PID_FILE, O_RDONLY, 0644);
-    if (fd < 0) {
-        goto error;
-    }
-
-    result = dcethread_read(fd, contents, sizeof(contents)-1);
-    if (result < 0) {
-        goto error;
-    } else if (result == 0) {
-        unlink(RPCD_PID_FILE);
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result < 0) {
-        result = -1;
-        goto error;
-    } else if (result == 0) {
-        unlink(RPCD_PID_FILE);
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH) {
-        unlink(RPCD_PID_FILE);
-        pid = 0;
-    } else {
-        // Verify that the peer process is a rpc daemon
-        if (MatchProgramToPID(RPCD_DAEMON_NAME, pid) != 0) {
-            unlink(RPCD_PID_FILE);
-            pid = 0;
-        }
-    }
-
-error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    return pid;
-}
-
-static
-void
-delete_pid_file()
-{
-    pid_t pid;
-
-    pid = pid_from_pid_file();
-    if (pid == getpid()) {
-        unlink(RPCD_PID_FILE);
-    }
-}
-
-static
-void
-create_pid_file()
-{
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-
-    pid = pid_from_pid_file();
-    if (pid > 0) {
-        fprintf(stderr, "Daemon already running as %d", (int) pid);
-        result = -1;
-        goto error;
-    }
-
-    fd = open(RPCD_PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0) {
-        fprintf(stderr, "Could not create pid file: %s", strerror(errno));
-        result = 1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) dcethread_write(fd, contents, len);
-    if ( result != (int) len ) {
-        fprintf(stderr, "Could not write to pid file: %s", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
-error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    if (result < 0) {
-        exit(1);
-    }
-}
-
 
 static
 void
@@ -850,8 +634,8 @@ attach_log_file(
     char *fname;
     char *p;
 
-    if ((fname = malloc(strlen(rpcd_c_database_name_prefix1) +
-                        strlen(rpcd_c_database_name_prefix2) +
+    if ((fname = malloc(strlen(rpcd_c_database_name_prefix1) + 
+                        strlen(rpcd_c_database_name_prefix2) + 
                         strlen(rpcd_c_logfile_name) + 1)) != NULL)
     {
         sprintf((char *) fname, "%s%s%s", rpcd_c_database_name_prefix1,
@@ -939,8 +723,6 @@ start_as_daemon(
         }
     }
 
-    atexit(delete_pid_file);
-    create_pid_file();
     attach_log_file();
 }
 
@@ -966,6 +748,8 @@ rpcd_network_thread(
 {
     error_status_t status = 0;
     int index = 0;
+    int jndex = 0;
+    boolean32 use_protseq = false;
     static const struct timespec retry_interval = {5, 0};
     static const char* network_protseqs[] =
     {
@@ -974,18 +758,37 @@ rpcd_network_thread(
         NULL
     };
 
-    while (network_protseqs[index])
+    while (network_protseqs[index] && !shutdown_threads)
     {
-        rpc_server_use_protseq_if(
-            (unsigned char*) network_protseqs[index],
-            0,
-            ept_v3_0_s_ifspec,
-            &status);
-
-        if (!STATUS_OK(&status))
+        use_protseq = false;
+        if (!use_all_protseqs)
         {
-            printf("(rpcd) Could not listen on %s: %x.  Retrying in %i seconds\n",
-                   network_protseqs[index], status, (int) retry_interval.tv_sec);
+            for (jndex = 0; jndex < num_protseq; jndex++)
+            {
+                if (!strcmp(network_protseqs[index], (char*) protseq[jndex]))
+                {
+                    use_protseq = true;
+                    break;
+                }
+            }
+        }
+        else
+        {
+            use_protseq = true;
+        }
+        if (use_protseq)
+        {
+            rpc_server_use_protseq_if(
+                (unsigned char*) network_protseqs[index],
+                0,
+                ept_v3_0_s_ifspec,
+                &status);
+        }
+
+        if (!STATUS_NOT_NULL_OK(&status))
+        {
+            printf("(rpcd) Could not listen on %s: %lx.  Retrying in %i seconds\n",
+                   network_protseqs[index], (long) status, (int) retry_interval.tv_sec);
             dcethread_delay(&retry_interval);
         }
         else
@@ -1165,10 +968,10 @@ int main(int argc, char *argv[])
      * Initialize the database and other misc stuff.
      */
     init(&status);
-    if (! STATUS_OK(&status)) exit(1);
+    if (! STATUS_NOT_NULL_OK(&status)) exit(1);
 
     register_ifs(&status);
-    if (! STATUS_OK(&status)) exit(1);
+    if (! STATUS_NOT_NULL_OK(&status)) exit(1);
 
     /*
      * Ensure existence/permissions on named pipe socket directory
@@ -1189,7 +992,7 @@ int main(int argc, char *argv[])
      */
     rpc_server_use_protseq_if((unsigned char*) "ncalrpc", 0, ept_v3_0_s_ifspec, &status);
 
-    if (!STATUS_OK(&status))
+    if (!STATUS_NOT_NULL_OK(&status))
     {
         printf("(rpcd) could not listen on ncalrpc\n");
         exit(1);
@@ -1214,7 +1017,7 @@ int main(int argc, char *argv[])
     {
         is_listening = rpc_mgmt_is_server_listening(NULL, &status);
 
-        if (!STATUS_OK(&status))
+        if (!STATUS_NOT_NULL_OK(&status))
         {
             printf("(rpcd) could not determine if listener is running\n");
             exit(1);
@@ -1227,7 +1030,7 @@ int main(int argc, char *argv[])
     if ((sm_notify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notify_fd = atoi(sm_notify);
-
+        
         do
         {
             ret = dcethread_write(notify_fd, &notify_code, sizeof(notify_code));
@@ -1262,7 +1065,7 @@ int main(int argc, char *argv[])
     /*
      * Tell listener thread to stop
      */
-
+    
     rpc_mgmt_stop_server_listening(NULL, &status);
 
     if (check_st_bad("Error stopping server", &status))
@@ -1275,6 +1078,8 @@ int main(int argc, char *argv[])
      */
     dcethread_join_throw(listen_thread, NULL);
 
+    shutdown_threads = true;
+
     /*
      * Cancel network detection thread
      */
