@@ -327,7 +327,7 @@ error_status_t  *status;
     if (! STATUS_OK(status))
     {
         /* Try deleting and recreating the database */
-        unlink(pathname);
+        unlink((char*) pathname);
         db_open(h, pathname, epdb_c_file_version, status);
         if (! STATUS_OK(status))
         {
@@ -580,7 +580,7 @@ rpc_addr_p_t    addr;
             dce_uuid_equal(&tfp->interface.uuid, &entp->interface.uuid, &tmp_st) &&
             (tfp->interface.vers_major == entp->interface.vers_major) && 
             (addr->rpc_protseq_id == entp->addr->rpc_protseq_id) &&
-            dce_uuid_equal(&tfp->data_rep.id, &entp->data_rep_id, &tmp_st) &&
+            dce_uuid_equal(&tfp->data_rep.id, &entp->data_rep_id, &tmp_st) && 
             (data_rep_vers_major == entp->data_rep_vers_major) &&
             (data_rep_vers_minor == entp->data_rep_vers_minor) &&
             (tfp->rpc_protocol == entp->rpc_protocol) &&
@@ -666,7 +666,7 @@ error_status_t  *status;
         if (epdb_is_replace_candidate(entp, object, tfp, addr))
         {
             rpc__naf_addr_inq_netaddr(entp->addr, &netaddr2, &tmp_st);
-            if (! STATUS_OK(&tmp_st)) continue;
+            if (! STATUS_NOT_NULL_OK(&tmp_st)) continue;
 
             if (strcmp((char *) netaddr, (char *) netaddr2) == 0)
             {
@@ -824,13 +824,13 @@ error_status_t  *status;
      * Parse xentry's tower into twr_fields
      */
     tower_to_fields(xentry->tower, &twr_fields, &tmp_st);
-    if (!STATUS_OK(&tmp_st)) return;
+    if (!STATUS_NOT_NULL_OK(&tmp_st)) return;
 
     tower_to_addr(xentry->tower, &addr, &tmp_st);
-    if (!STATUS_OK(&tmp_st)) return;
+    if (!STATUS_NOT_NULL_OK(&tmp_st)) return;
 
     epdb_chk_entry(xentry, &twr_fields, addr, &tmp_st);
-    if (!STATUS_OK(&tmp_st))
+    if (!STATUS_NOT_NULL_OK(&tmp_st))
     {
         if (addr != NULL) rpc__naf_addr_free(&addr, &tmp_st);
         return;
@@ -1636,12 +1636,12 @@ unsigned32              *status;
         if (entp->delete_flag) continue;
 
         if  (((object_nil && entp->object_nil) || 
-                dce_uuid_equal(object, &entp->object, &tmp_st)) &&
+                dce_uuid_equal(object, &entp->object, &tmp_st)) && 
             dce_uuid_equal(&interface->uuid, &entp->interface.uuid, &tmp_st) &&
             (interface->vers_major == entp->interface.vers_major) && 
             (interface->vers_minor <= entp->interface.vers_minor) &&
             (protseq == entp->addr->rpc_protseq_id) &&
-            dce_uuid_equal(&data_rep->id, &entp->data_rep_id, &tmp_st) &&
+            dce_uuid_equal(&data_rep->id, &entp->data_rep_id, &tmp_st) && 
             (data_rep_vers_major == entp->data_rep_vers_major) &&
             (data_rep_vers_minor <= entp->data_rep_vers_minor) &&
             (rpc_protocol == entp->rpc_protocol) &&
@@ -1783,9 +1783,9 @@ unsigned32              *status;
 
         if (entp->delete_flag) continue;
 
-        if (dce_uuid_equal(object, &entp->object, &tmp_st) &&
+        if (dce_uuid_equal(object, &entp->object, &tmp_st) && 
             (protseq == entp->addr->rpc_protseq_id) &&
-            dce_uuid_equal(&data_rep->id, &entp->data_rep_id, &tmp_st) &&
+            dce_uuid_equal(&data_rep->id, &entp->data_rep_id, &tmp_st) && 
             (data_rep_vers_major == entp->data_rep_vers_major) &&
             (data_rep_vers_minor <= entp->data_rep_vers_minor) &&
             (rpc_protocol == entp->rpc_protocol) &&
