@@ -541,7 +541,7 @@ unsigned32              *status;
 **--
 **/
 
-void dce_uuid_create
+void dce_uuid_create 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_t              *uuid,
@@ -679,7 +679,7 @@ unsigned32              *status;
 **--
 **/
 
-void dce_uuid_create_nil
+void dce_uuid_create_nil 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_t              *uuid,
@@ -736,7 +736,7 @@ unsigned32          *status;
 **--
 **/
 
-void dce_uuid_to_string
+void dce_uuid_to_string 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_p_t            uuid,
@@ -788,7 +788,7 @@ unsigned32              *status;
     UUID_SPRINTF(
         (char *) *uuid_string,
         "%08lx-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
-        uuid->time_low, uuid->time_mid, uuid->time_hi_and_version,
+        (unsigned long) uuid->time_low, uuid->time_mid, uuid->time_hi_and_version,
         uuid->clock_seq_hi_and_reserved, uuid->clock_seq_low,
         (unsigned8) uuid->node[0], (unsigned8) uuid->node[1],
         (unsigned8) uuid->node[2], (unsigned8) uuid->node[3],
@@ -836,7 +836,7 @@ unsigned32              *status;
 **--
 **/
 
-void dce_uuid_from_string
+void dce_uuid_from_string 
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       uuid_string,
@@ -1032,7 +1032,7 @@ unsigned32              *status;
 **--
 **/
 
-boolean32 dce_uuid_equal
+boolean32 dce_uuid_equal 
 #ifdef _DCE_PROTO_
 (
     register dce_uuid_p_t            uuid1,
@@ -1112,7 +1112,7 @@ register unsigned32              *status;
 **--
 **/
 
-boolean32 dce_uuid_is_nil
+boolean32 dce_uuid_is_nil 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_p_t        uuid,
@@ -1196,7 +1196,7 @@ unsigned32          *status;
 **--
 **/
 
-signed32 dce_uuid_compare
+signed32 dce_uuid_compare 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_p_t            uuid1,
@@ -1351,7 +1351,7 @@ unsigned32              *status;
 **--
 **/
 
-unsigned16 dce_uuid_hash
+unsigned16 dce_uuid_hash 
 #ifdef _DCE_PROTO_
 (
     dce_uuid_p_t            uuid,
@@ -1599,10 +1599,9 @@ unsigned64_t        *prodPtr;
 
 static void true_random_init (void)
 {
-    dce_uuid_time_t     t;
+    union { dce_uuid_time_t t; unsigned16 u16; } u;
     unsigned16          *seedp, seed=0;
 
-
     /*
      * optimal/recommended starting values according to the reference
      */
@@ -1627,8 +1626,8 @@ static void true_random_init (void)
      * independent.  Then for good measure to ensure a unique seed when there
      * are multiple processes creating UUID's on a system, we add in the PID.
      */
-    uuid__get_os_time(&t);
-    seedp = (unsigned16 *)(&t);
+    uuid__get_os_time(&u.t);
+    seedp = (unsigned16 *)(&u.u16);
     seed ^= *seedp++;
     seed ^= *seedp++;
     seed ^= *seedp++;
