@@ -66,18 +66,15 @@ ShowUsage()
                     "                                               to console).\n");
     fprintf(stdout, "    --loglevel {error|warning|info|verbose}    Adjusts how much logging is\n"
                     "                                               produced by domainjoin.\n");
-    fprintf(stdout, "    --nodcerpcd                                prevents dcerpcd from being\n"
-                    "                                               started, along with any daemons\n"
-                    "                                               that depend on it.\n");
     fprintf(stdout, "\n");
     fprintf(stdout, "  and commands are:\n\n");
     fprintf(stdout, "    query\n");
     fprintf(stdout, "    setname <computer name>\n");
-    fprintf(stdout, "    join [--notimesync] [--enable <module> --disable <module> ...] [--ou <organizationalUnit>] <domain name> <user name> [<password>]\n");
+    fprintf(stdout, "    join [--notimesync] [--enable <module> --disable <module> ...] [--ou <organizationalUnit>] [--multiple] <domain name> <user name> [<password>]\n");
     fprintf(stdout, "    join [--advanced] --preview [--ou <organizationalUnit>] <domain name>\n");
-    fprintf(stdout, "    join [--assumeDefaultDomain] [--userDomainPrefix <short domain name>] [--ou <organizationalUnit>] <domain name>\n");
+    fprintf(stdout, "    join [--assumeDefaultDomain {yes|no}] [--userDomainPrefix <short domain name>] [--ou <organizationalUnit>] <domain name>\n");
     fprintf(stdout, "    join [--ou <organizationalUnit>] --details <module> <domain name>\n");
-    fprintf(stdout, "    leave [--enable <module> --disable <module> ...] [user name] [password]\n");
+    fprintf(stdout, "    leave [--enable <module> --disable <module> ...] [--multiple <domain name>] [user name] [password]\n");
     fprintf(stdout, "    leave [--advanced] --preview [user name] [password]\n");
     fprintf(stdout, "    leave --details <module>\n\n");
 
@@ -255,8 +252,15 @@ void PrintJoinHeader(const JoinProcessOptions *options, LWException **exc)
     }
     else
     {
-        LW_TRY(exc, QueryInformation(&pDomainJoinInfo, &LW_EXC));
-        domain = pDomainJoinInfo->pszDomainName;
+        if (options->domainName)
+        {
+            domain = options->domainName;
+        }
+        else
+        {
+            LW_TRY(exc, QueryInformation(&pDomainJoinInfo, &LW_EXC));
+            domain = pDomainJoinInfo->pszDomainName;
+        }
         if(domain == NULL)
             domain = "(unknown)";
         fprintf(stdout, "Leaving AD Domain:   %s\n", domain);
@@ -323,6 +327,8 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
             options.ignorePam = TRUE;
         else if(!strcmp(argv[0], "--notimesync"))
             options.disableTimeSync = TRUE;
+        else if(!strcmp(argv[0], "--multiple"))
+            options.enableMultipleJoins = TRUE;
         else if(!strcmp(argv[0], "--nohosts"))
         {
             PCSTR module = "hostname";
@@ -362,14 +368,35 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
         else if(!strcmp(argv[0], "--assumeDefaultDomain"))
         {
             DJ_LOG_INFO("Domainjoin invoked with option --assumeDefaultDomain");
-            options.assumeDefaultDomain = TRUE;
+            options.setAssumeDefaultDomain = TRUE;
+
+            if (!strcasecmp(argv[1], "yes") || !strcasecmp(argv[1], "true") ||
+                !strcasecmp(argv[1], "on"))
+            {
+                options.assumeDefaultDomain = TRUE;
+            }
+            else if (!strcasecmp(argv[1], "no") ||
+                !strcasecmp(argv[1], "false") ||
+                !strcasecmp(argv[1], "off"))
+            {
+                options.assumeDefaultDomain = FALSE;
+            }
+            else
+            {
+                LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
+                goto cleanup;
+            }
+            argv++;
+            argc--;
         }
         else if(!strcmp(argv[0], "--userDomainPrefix"))
         {
             DJ_LOG_INFO("Domainjoin invoked with option --userDomainPrefix %s", argv[1]);
+            options.setAssumeDefaultDomain = TRUE;
             options.assumeDefaultDomain = TRUE;
             CT_SAFE_FREE_STRING(options.userDomainPrefix);
             LW_CLEANUP_CTERR(exc, CTStrdup(argv[1], &options.userDomainPrefix));
+            CTStrToUpper(options.userDomainPrefix);
             argv++;
             argc--;
         }
@@ -548,6 +575,13 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
             argv++;
             argc--;
         }
+        else if(!strcmp(argv[0], "--multiple"))
+        {
+            options.enableMultipleJoins = TRUE;
+            LW_CLEANUP_CTERR(exc, CTStrdup(argv[1], &options.domainName));
+            argv++;
+            argc--;
+        }
         else
         {
             LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
@@ -679,6 +713,8 @@ void DoConfigure(int argc, char **argv, LWException **exc)
     PCSTR testPrefix = NULL;
     PCSTR longDomain = NULL;
     PCSTR shortDomain = NULL;
+    JoinProcessOptions options = { 0 };
+
     while(argc > 0 && CTStrStartsWith(argv[0], "--"))
     {
         if(!strcmp(argv[0], "--autoenable"))
@@ -725,16 +761,20 @@ void DoConfigure(int argc, char **argv, LWException **exc)
         goto cleanup;
     }
 
+    options.joiningDomain = GetEnableBoolean(dwEnable);
+    options.warningCallback = PrintWarning;
+
     if(!strcmp(argv[0], "pam"))
         LW_TRY(exc, DJNewConfigurePamForADLogin(testPrefix, NULL, PrintWarning, GetEnableBoolean(dwEnable), &LW_EXC));
     else if(!strcmp(argv[0], "nsswitch"))
-        LW_CLEANUP_CTERR(exc, DJConfigureNameServiceSwitch(testPrefix,
-                GetEnableBoolean(dwEnable)));
+    {
+        LW_TRY(exc, DoNsswitch(&options, &LW_EXC));
+    }
     else if(!strcmp(argv[0], "ssh"))
-        LW_TRY(exc, DJConfigureSshForADLogin(testPrefix, GetEnableBoolean(dwEnable), NULL, &LW_EXC));
+        LW_TRY(exc, DJConfigureSshForADLogin(GetEnableBoolean(dwEnable), NULL, &LW_EXC));
     else if(!strcmp(argv[0], "krb5"))
         LW_CLEANUP_CTERR(exc, DJModifyKrb5Conf(testPrefix,
-            GetEnableBoolean(dwEnable), longDomain, shortDomain, NULL));
+            GetEnableBoolean(dwEnable), longDomain, shortDomain, NULL, NULL));
     else if(!strcmp(argv[0], "eventfwdd"))
         LW_CLEANUP_CTERR(exc, DJConfigureEventFwd(testPrefix, GetEnableBoolean(dwEnable)));
     else if(!strcmp(argv[0], "reapsysld"))
@@ -755,7 +795,7 @@ void DoGetDistroInfo(int argc, char **argv, LWException **exc)
     PSTR str = NULL;
     PCSTR requestType = argv[0];
     PCSTR testPrefix = NULL;
-    DistroInfo distro = {0};
+    LwDistroInfo distro = {0};
 
     argc--;
     argv++;
@@ -808,16 +848,16 @@ int main(
 {
     LWException *exc = NULL;
     int columns;
-    PSTR pszLogFilePath = "/tmp/lwidentity.join.log";
+    PSTR pszLogFilePath = NULL;
     BOOLEAN bNoLog = FALSE;
     PSTR logLevel = "warning";
     DWORD dwLogLevel;
     BOOLEAN showHelp = FALSE;
     BOOLEAN showInternalHelp = FALSE;
-    BOOLEAN bEnableDcerpcd = TRUE;
     int remainingArgs = argc;
     char **argPos = argv;
     int i;
+    BOOLEAN directoryExists = FALSE;
 
     if(CTGetTerminalWidth(fileno(stdout), &columns))
         columns = -1;
@@ -849,8 +889,6 @@ int main(
             logLevel = (++argPos)[0];
             remainingArgs--;
         }
-        else if (!strcmp(argPos[0], "--nodcerpcd"))
-            bEnableDcerpcd = FALSE;
         else
             break;
         remainingArgs--;
@@ -882,10 +920,24 @@ int main(
         LW_CLEANUP_CTERR(&exc, LW_ERROR_INVALID_LOG_LEVEL);
     }
 
+    if (pszLogFilePath == NULL)
+    {
+        // Determine the default log path
+        LW_CLEANUP_CTERR(&exc,
+                CTCheckDirectoryExists("/var/log", &directoryExists));
+        if (directoryExists)
+        {
+            pszLogFilePath = "/var/log/likewise-join.log";
+        }
+        else
+        {
+            pszLogFilePath = "/var/adm/likewise-join.log";
+        }
+    }
+
     if (bNoLog) {
         LW_CLEANUP_CTERR(&exc, dj_disable_logging());
-    } else if (pszLogFilePath == NULL ||
-               !strcmp(pszLogFilePath, ".")) {
+    } else if (!strcmp(pszLogFilePath, ".")) {
         LW_CLEANUP_CTERR(&exc, dj_init_logging_to_console(dwLogLevel));
     } else {
         DWORD ceError = dj_init_logging_to_file(dwLogLevel, pszLogFilePath);
@@ -896,6 +948,13 @@ int main(
             ceError = ERROR_SUCCESS;
             LW_CLEANUP_CTERR(&exc, dj_disable_logging());
         }
+        else if (ceError == ERROR_FILE_NOT_FOUND)
+        {
+            fprintf(stderr, "Warning: parent directory of log file %s does not exist. To enable logging, please specify a different filename with --logfile <file>.\n",
+                    pszLogFilePath);
+            ceError = ERROR_SUCCESS;
+            LW_CLEANUP_CTERR(&exc, dj_disable_logging());
+        }
         LW_CLEANUP_CTERR(&exc, ceError);
     }
 
@@ -926,26 +985,22 @@ int main(
             goto cleanup;
         }
         //Needed so that DJGetMachineSid does not call winbind
-        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DJSetComputerName(argPos[0], NULL, &LW_EXC));
     }
     else if(!strcmp(argPos[0], "join"))
     {
         argPos++;
         remainingArgs--;
-        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoJoin(remainingArgs, argPos, columns, &LW_EXC));
     }
     else if(!strcmp(argPos[0], "leave"))
     {
         argPos++;
         remainingArgs--;
-        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoLeaveNew(remainingArgs, argPos, columns, &LW_EXC));
     }
     else if(!strcmp(argPos[0], "query"))
     {
-        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoQuery(&LW_EXC));
     }
     else if(!strcmp(argPos[0], "fixfqdn"))
@@ -990,7 +1045,6 @@ cleanup:
         return 1;
     }
 
-    DJNetShutdown(NULL);
     dj_close_log();
 
     return 0;
