@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -54,7 +54,7 @@ typedef struct _USER_LIST
     char * pszUserGID;
     char * pszUserHomeDir;
     struct _USER_LIST * pNext;
-
+    
 } USER_LIST, * PUSER_LIST;
 
 // Helper routines for this class
@@ -71,7 +71,7 @@ long
 GetLocalUserList(
     PUSER_LIST * ppLocalUsers
     );
-
+    
 void
 FreeLocalUserList(
     PUSER_LIST pLocalUsers
@@ -113,7 +113,7 @@ class DomainMigrateWindow : public TWindow
         virtual void Close();
 
     protected:
-
+    
         virtual Boolean HandleCommand( const HICommandExtended& inCommand );
         void ClearLocalUsersCombo();
         void AddUserToLocalUsersCombo(const std::string& value);
@@ -125,6 +125,8 @@ class DomainMigrateWindow : public TWindow
         std::string GetADUserUID();
         std::string GetADUserGID();
         bool IsMoveOptionSelected();
+        bool IsDeleteOptionSelected();
+        bool IsKeepAdminOptionSelected();
         void HandleMigration();
         bool HandleValidateUser();
         bool ConfirmMigration(const std::string& localUserName,
@@ -133,11 +135,15 @@ class DomainMigrateWindow : public TWindow
                               const std::string& adUserHomeDir,
                               const std::string& adUserUID,
                               const std::string& adUserGID,
-                              bool  bMoveProfile);
-        int CallMigrateCommand(const std::string& localUserHomeDir,
+                              bool  bMoveProfile,
+                              bool  bDeleteAccount,
+                              bool  bKeepAdmin);
+        int CallMigrateCommand(const std::string& localUserName,
                                const std::string& adUserName,
                                const std::string& logFileName,
                                bool bMoveProfile,
+                               bool bDeleteAccount,
+                               bool bKeepAdmin,
                                char ** ppszOutput);
         void ShowMigrateCompleteDialog(const std::string& value);
         void ShowMigrateCompleteErrorDialog(const std::string& value, int code, const std::string& resultMessage);
@@ -145,7 +151,7 @@ class DomainMigrateWindow : public TWindow
         void ShowMigrateProgressBar();
 
     protected:
-
+	
         static const int LOCAL_USER_COMBO_ID;
         static const int LOCAL_USER_PATH_ID;
         static const int LOCAL_USER_REAL_NAME_ID;
@@ -158,6 +164,8 @@ class DomainMigrateWindow : public TWindow
         static const int AD_USER_GID_ID;
         static const int COPY_RADIO_ID;
         static const int MOVE_RADIO_ID;
+        static const int DELETE_ACCOUNT_ID;
+        static const int KEEP_ADMIN_ID;
         static const int VALIDATE_ID;
         static const int CANCEL_ID;
         static const int MIGRATE_ID;
@@ -170,7 +178,7 @@ class DomainMigrateWindow : public TWindow
         static const int VALIDATE_CMD_ID;
         static const int CANCEL_CMD_ID;
         static const int MIGRATE_CMD_ID;
-
+		
     private:
 
         std::string _localUsersFirstItem;
