@@ -28,6 +28,7 @@
  * license@likewisesoftware.com
  */
 
+#include "DomainJoinConfig.h"
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 
@@ -114,7 +115,10 @@ log_copy(const char* dest)
 
     while ((amount = fread(buffer, 1, sizeof(buffer), log_handle)) > 0)
     {
-        fwrite(buffer, 1, amount, destfile);
+        if (fwrite(buffer, 1, amount, destfile) != amount)
+        {
+           break;
+        }
     }
 
     clearerr(log_handle);
@@ -141,7 +145,7 @@ safe_free(void** ptr)
     *ptr = NULL;
 }
 
-#define SAFE_FREE(var) safe_free((void**) &var);
+#define SAFE_FREE(var) safe_free((void**)(void*) &var);
 
 static void
 fill_state(JoinDialog* dialog)
@@ -207,7 +211,7 @@ free_state_password()
 /* Dirty, horrible hacks to allow use of new file chooser dialog widget
    despite old RHEL3 build machine */
 
-#if GTK_MINOR_VERSION < 6
+#if GTK_MINOR_VERSION < 6 && !defined(HAVE_GTK_FILE_CHOOSER_DIALOG_NEW)
 #define GTK_FILE_CHOOSER_ACTION_SAVE 1
 typedef int GtkFileChooserAction;
 static GtkWidget *(*gtk_file_chooser_dialog_new)(const gchar *title,
@@ -222,6 +226,7 @@ static const gchar* (*gtk_file_chooser_get_filename)(void*);
 static void
 show_error_dialog(GtkWindow* parent, LWException* exc)
 {
+#if !defined(HAVE_GTK_FILE_CHOOSE_DIALOG_NEW)
     if(gtk_minor_version < 6)
     {
         GtkDialog* dialog = GTK_DIALOG(
@@ -238,11 +243,12 @@ show_error_dialog(GtkWindow* parent, LWException* exc)
         return;
     }
     else
+#endif
     {
         JoinErrorDialog* error_dialog = joinerror_new(parent, exc);
         int result;
 
-#if GTK_MINOR_VERSION < 6
+#if GTK_MINOR_VERSION < 6 && !defined(HAVE_GTK_FILE_CHOOSER_DIALOG_NEW)
         {
             void* handle = dlopen(NULL, RTLD_LAZY);
 
@@ -420,6 +426,7 @@ do_join(JoinDialog* dialog, LWException** exc)
             info.options.ouName = NULL;
         info.options.domainName = safe_strdup(join_state.domain);
         info.options.joiningDomain = TRUE;
+        info.options.setAssumeDefaultDomain = TRUE;
         info.options.assumeDefaultDomain = join_state.prefix != NULL;
         info.options.userDomainPrefix = safe_strdup(join_state.prefix);
 
@@ -572,7 +579,7 @@ ensure_gtk_version(int major, int minor, int micro, LWException** exc)
     if ((msg = gtk_check_version(major, minor, micro)))
     {
         LW_RAISE_EX(exc, ERROR_DLL_MIGHT_BE_INCOMPATIBLE,
-            "Incompatible library detected",
+            "Incompatible library detected", 
             "%s.  Likewise does not support graphical domain joins on this platform.  "
             "Please use the command-line domain join application instead.",
             msg);
@@ -597,8 +604,6 @@ main(int argc, char** argv)
 
     gtk_init(&argc, &argv);
 
-    LW_TRY(&exc, DJNetInitialize(TRUE, &LW_EXC));
-
     do
     {
         char* computer = NULL;
@@ -644,9 +649,6 @@ main(int argc, char** argv)
 
     } while (!quit);
 
-    // Try to shutdown the net api and report any failures
-    LW_TRY(&exc, DJNetShutdown(&LW_EXC));
-
 cleanup:
 
     if (exc)
@@ -654,10 +656,6 @@ cleanup:
         show_error_dialog(NULL, exc);
     }
 
-    // Shutdown the net api without reporting failures (it is ok if
-    // DJNetShutdown is called twice).
-    DJNetShutdown(NULL);
-
     close_stale_dialogs(NULL);
 
     gdk_threads_leave();
