@@ -31,17 +31,9 @@
 #ifndef __DJ_AUTHINFO_H__
 #define __DJ_AUTHINFO_H__
 
-//The answer is non-authoritative
-void
-DJGetDomainRwDC(PCSTR domain, PSTR *dc, LWException **exc);
-
 void
 DJGetComputerDN(PSTR *dn, LWException **exc);
 
-void DJNetInitialize(BOOLEAN bEnableDcerpcd, LWException **exc);
-
-void DJNetShutdown(LWException **exc);
-
 void DJCreateComputerAccount(
                 PSTR *shortDomainName,
                 JoinProcessOptions *options,
