@@ -70,9 +70,6 @@ struct _JoinProcessOptions
     PSTR shortDomainName;
     PSTR computerName;
     PSTR ouName;
-    // Optional alternate user domain prefix used with assumeDefaultDomain (true) setting.
-    // If null, the computer domain name prefix is assumed.
-    PSTR userDomainPrefix;
     PSTR username;
     PSTR password;
     void *userData;
@@ -80,11 +77,18 @@ struct _JoinProcessOptions
     BOOLEAN joiningDomain;
     BOOLEAN showTraces;
     BOOLEAN disableTimeSync;
+    BOOLEAN enableMultipleJoins;
     BOOLEAN ignorePam;
+
+    BOOLEAN setAssumeDefaultDomain;
     // TRUE means that lsass will be configured to append a domain prefix so user logons
     // can be with short account names.
     // FALSE means that logons will require a UPN or NT4 style domain user account.
     BOOLEAN assumeDefaultDomain;
+    // Optional alternate user domain prefix used with assumeDefaultDomain (true) setting.
+    // If null, the computer domain name prefix is assumed.
+    PSTR userDomainPrefix;
+
     WarningFunction warningCallback;
     /* Contains modules that are enabled and disabled by the user, but does
      * not contain NA modules. This list is populated from the moduleTable.
@@ -107,6 +111,47 @@ struct _JoinModule
     void (*FreeModuleData)(const JoinProcessOptions *, ModuleState *state);
 };
 
+DWORD
+DJInit(
+    VOID
+    );
+
+DWORD
+DJSetComputerNameEx(
+    PCSTR pszComputerName
+    );
+
+DWORD
+DJJoinDomain(
+    PCSTR pszDomain,
+    PCSTR pszOU,
+    PCSTR pszUsername,
+    PCSTR pszPassword
+    );
+
+DWORD
+DJQueryJoinInformation(
+    PSTR* ppszComputerName,
+    PSTR* ppszDomainName,
+    PSTR* ppszComptuterDN
+    );
+
+DWORD
+DJUnjoinDomain(
+    PCSTR pszUsername,
+    PCSTR pszPassword
+    );
+
+VOID
+DJFreeMemory(
+    PVOID pMemory
+    );
+
+DWORD
+DJShutdown(
+    VOID
+    );
+
 void DJZeroJoinProcessOptions(JoinProcessOptions *options);
 void DJFreeJoinProcessOptions(JoinProcessOptions *options);
 void DJRefreshModuleStates(JoinProcessOptions *options, LWException **err);
