@@ -32,15 +32,15 @@
 #define __DJNSSWITCH_H__
 
 
-DWORD
-UnConfigureNameServiceSwitch();
-
-DWORD
-ConfigureNameServiceSwitch();
-
 DWORD
 DJConfigureNameServiceSwitch(const char *testPrefix, BOOLEAN enable);
 
+void
+DoNsswitch(
+    JoinProcessOptions *options,
+    LWException **exc
+    );
+
 DWORD
 DJConfigureHostsEntry(const char *testPrefix);
 
