@@ -52,10 +52,6 @@
 #include "djconfig_mac.h"
 #endif
 
-#ifndef SCRIPTDIR
-#define SCRIPTDIR  PREFIXDIR"/bin"
-#endif
-
 #ifndef SIZE_MAX
 #define SIZE_MAX    ((size_t)-1)
 #endif
