@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -44,7 +44,7 @@ DJFindStanza(const DynamicArray *lines, PCSTR name)
     {
         PCSTR line = *(PCSTR *)CTArrayGetItem((DynamicArray*)lines, i,
                 sizeof(PCSTR));
-        while (*line != '\0' && isspace(*line))
+        while (*line != '\0' && isspace((int)*line))
             line++;
 
         if(!strncmp(line, name, namelen) && line[namelen] == ':')
@@ -56,7 +56,7 @@ DJFindStanza(const DynamicArray *lines, PCSTR name)
 ssize_t DJFindLine(const DynamicArray *lines, const char *stanza, const char *name)
 {
     ssize_t i = DJFindStanza(lines, stanza);
-
+    
     if(i == -1)
         return -1;
 
@@ -65,14 +65,14 @@ ssize_t DJFindLine(const DynamicArray *lines, const char *stanza, const char *na
         PCSTR line = *(PCSTR *)CTArrayGetItem((DynamicArray*)lines, i,
                 sizeof(PCSTR));
 
-        while (*line != '\0' && isspace(*line))
+        while (*line != '\0' && isspace((int)*line))
             line++;
 
         if(strncmp(line, name, strlen(name)))
             continue;
         line += strlen(name);
-
-        while (*line != '\0' && isspace(*line))
+       
+        while (*line != '\0' && isspace((int)*line))
             line++;
 
         if(*line != '=')
@@ -92,19 +92,19 @@ DWORD DJGetOptionValue(const DynamicArray *lines, PCSTR stanza, PCSTR name, PSTR
     PSTR _value = NULL;
 
     *value = NULL;
-
+    
     if(i == -1)
         GCE(ceError = ERROR_NOT_FOUND);
 
     line = *(PCSTR *)CTArrayGetItem((DynamicArray*)lines, i,
             sizeof(PCSTR));
 
-    while (*line != '\0' && isspace(*line))
+    while (*line != '\0' && isspace((int)*line))
         line++;
 
     line += strlen(name);
-
-    while (*line != '\0' && isspace(*line))
+   
+    while (*line != '\0' && isspace((int)*line))
         line++;
 
     if(*line != '=')
