@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -119,7 +119,7 @@ cleanup:
 static QueryResult QueryPamMode(const JoinProcessOptions *options, LWException **exc)
 {
     BOOLEAN bFileExists = FALSE;
-    DistroInfo distro;
+    LwDistroInfo distro;
     QueryResult result = NotApplicable;
     FILE* fp = NULL;
     DynamicArray lines;
@@ -129,7 +129,7 @@ static QueryResult QueryPamMode(const JoinProcessOptions *options, LWException *
     memset(&lines, 0, sizeof(lines));
     memset(&distro, 0, sizeof(distro));
 
-    if(!options->joiningDomain)
+    if(!options->joiningDomain || options->enableMultipleJoins)
         goto cleanup;
 
     LW_CLEANUP_CTERR(exc, DJGetDistroInfo(NULL, &distro));
