@@ -41,13 +41,14 @@
 #include "ctstrutils.h"
 #include <glob.h>
 #include <dlfcn.h>
-#include <lsa/join.h>
 #include <lsa/lsa.h>
 #include <lsa/ad.h>
 #include <lwnet.h>
 #include <eventlog.h>
-#include <lwps/lwps.h>
 #include <lwstr.h>
+#include <lwsm/lwsm.h>
+#include <lwmem.h>
+#include <assert.h>
 
 #define DOMAINJOIN_EVENT_CATEGORY   "Domain join"
 
@@ -154,7 +155,7 @@ QueryResult QueryCache(
 {
     ModuleState *state = DJGetModuleStateByName(options, "cache");
 
-    if(!options->joiningDomain)
+    if(!options->joiningDomain || options->enableMultipleJoins)
         return NotApplicable;
 
     //This module sets its moduleData after it is finished making changes. By
@@ -291,7 +292,7 @@ RemoveCacheFiles()
     (void) CTRemoveFiles(pszCachePath, FALSE, TRUE);
 
     /* Revert any system configuration files that may have been changed by previous domain GPOs */
-    ceError = CTCheckDirectoryExists( "/var/lib/likewise/grouppolicy",
+    ceError = CTCheckDirectoryExists( "/var/lib/likewise/grouppolicy", 
                                       &bDirExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
     if (bDirExists) {
@@ -306,7 +307,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/krb5.conf.lwidentity-gp.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/krb5.conf.lwidentity-gp.orig file to /etc/krb5.conf");
         ceError = CTMoveFile("/etc/krb5.conf.lwidentity-gp.orig", "/etc/krb5.conf");
@@ -343,12 +344,12 @@ RemoveCacheFiles()
         }
 
     }
-
+   
     /* /etc/motd */
     ceError = CTCheckFileExists("/etc/motd.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/motd.lwidentity.orig file to /etc/motd");
         ceError = CTMoveFile("/etc/motd.lwidentity.orig", "/etc/motd");
@@ -359,7 +360,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/syslog.conf.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/syslog.conf.lwidentity.orig file to /etc/syslog.conf");
         ceError = CTMoveFile("/etc/syslog.conf.lwidentity.orig", "/etc/syslog.conf");
@@ -371,7 +372,7 @@ RemoveCacheFiles()
         ceError = CTCheckFileExists("/etc/syslog-ng/syslog-ng.conf.lwidentity.orig", &bFileExists);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
-        if (bFileExists)
+        if (bFileExists) 
         {
             DJ_LOG_VERBOSE("Restoring /etc/syslog-ng/syslog-ng.conf.lwidentity.orig file to /etc/syslog-ng/syslog-ng.conf");
             ceError = CTMoveFile("/etc/syslog-ng/syslog-ng.conf.lwidentity.orig", "/etc/syslog-ng/syslog-ng.conf");
@@ -384,7 +385,7 @@ RemoveCacheFiles()
             ceError = CTCheckFileExists("/etc/rsyslog.conf.lwidentity.orig", &bFileExists);
             BAIL_ON_CENTERIS_ERROR(ceError);
 
-            if (bFileExists)
+            if (bFileExists) 
             {
                 DJ_LOG_VERBOSE("Restoring /etc/rsyslog.conf.lwidentity.orig file to /etc/rsyslog.conf");
                 ceError = CTMoveFile("/etc/rsyslog.conf.lwidentity.orig", "/etc/rsyslog.conf");
@@ -397,7 +398,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/crontab.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/crontab.lwidentity.orig file to /etc/crontab");
         ceError = CTMoveFile("/etc/crontab.lwidentity.orig", "/etc/crontab");
@@ -408,7 +409,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/logrotate.conf.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/logrotate.conf.lwidentity.orig file to /etc/logrotate.conf");
         ceError = CTMoveFile("/etc/logrotate.conf.lwidentity.orig", "/etc/logrotate.conf");
@@ -419,7 +420,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/issue.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/issue.lwidentity.orig file to /etc/issue");
         ceError = CTMoveFile("/etc/issue.lwidentity.orig", "/etc/issue");
@@ -430,7 +431,7 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/selinux/config.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/selinux/config.lwidentity.orig file to /etc/selinux/config.lwidentity.orig");
         ceError = CTMoveFile("/etc/selinux/config.lwidentity.orig", "/etc/selinux/config");
@@ -441,30 +442,30 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/fstab.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/fstab.lwidentity.orig file to /etc/fstab");
         ceError = CTMoveFile("/etc/fstab.lwidentity.orig", "/etc/fstab");
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
-    else
+    else 
     {
 
         ceError = CTCheckFileExists("/etc/vfstab.lwidentity.orig", &bFileExists);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
-        if (bFileExists)
+        if (bFileExists) 
         {
             DJ_LOG_VERBOSE("Restoring /etc/vfstab.lwidentity.orig file to /etc/vfstab");
             ceError = CTMoveFile("/etc/vfstab.lwidentity.orig", "/etc/vfstab");
             BAIL_ON_CENTERIS_ERROR(ceError);
         }
-        else
+        else 
         {
             ceError = CTCheckFileExists("/etc/filesystems.lwidentity.orig", &bFileExists);
             BAIL_ON_CENTERIS_ERROR(ceError);
 
-            if (bFileExists)
+            if (bFileExists) 
             {
                 DJ_LOG_VERBOSE("Restoring /etc/filesystems.lwidentity.orig file to /etc/filesystems");
                 ceError = CTMoveFile("/etc/filesystems.lwidentity.orig", "/etc/filesystems");
@@ -477,18 +478,18 @@ RemoveCacheFiles()
     ceError = CTCheckFileExists("/etc/auto.master.lwidentity.orig", &bFileExists);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    if (bFileExists)
+    if (bFileExists) 
     {
         DJ_LOG_VERBOSE("Restoring /etc/auto.master.lwidentity.orig file to /etc/auto.master");
         ceError = CTMoveFile("/etc/auto.master.lwidentity.orig", "/etc/auto.master");
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
-    else
+    else 
     {
         ceError = CTCheckFileExists("/etc/auto_master.lwidentity.orig", &bFileExists);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
-        if (bFileExists)
+        if (bFileExists) 
         {
             DJ_LOG_VERBOSE("Restoring /etc/auto_master.lwidentity.orig file to /etc/auto_master");
             ceError = CTMoveFile("/etc/auto_master.lwidentity.orig", "/etc/auto_master");
@@ -504,159 +505,293 @@ error:
     return ceError;
 }
 
+static
+BOOLEAN
+IsContainer(
+    IN PCSTR pName,
+    IN size_t Len
+    )
+{
+    PCSTR ppTopLevelContainers[] = {
+        "Builtin",
+        "Computers",
+        "ForeignSecurityPrincipals",
+        "LostAndFound",
+        "NTDS Quotas",
+        "Program Data",
+        "System",
+        "Users",
+        NULL
+    };
+    DWORD index = 0;
+
+    for (index = 0; ppTopLevelContainers[index]; index++)
+    {
+        if (Len == strlen(ppTopLevelContainers[index]) &&
+                !strncasecmp(pName, ppTopLevelContainers[index], Len))
+        {
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
 static
 DWORD
 CanonicalizeOrganizationalUnit(
-    PSTR* pszCanonicalizedOrganizationalUnit,
-    PCSTR pszOrganizationalUnit,
-    PCSTR pszDomainName
+    PCSTR pInputOu,
+    PCSTR pDomainName,
+    PSTR* ppCanonicalOu
     )
 {
     DWORD ceError = ERROR_SUCCESS;
-    int EE = 0;
-    PSTR comma;
-    PSTR current;
-    PSTR temp = NULL;
-    PSTR result = NULL;
-    PSTR dnsDomain = NULL;
+    PCSTR pPos = NULL;
+    PCSTR pNextComponent = NULL;
+    DWORD componentCount = 1;
+    PSTR pLdapDomainSuffix = NULL;
+    // Do not free
+    PSTR pOutputPos = NULL;
+    PSTR pOutputOu = NULL;
+    // Do not free
+    PSTR pUnescapePos = NULL;
+    size_t outputSize = 0;
+    // Do not free
+    PSTR pUnescapedComponentEnd = NULL;
+    int outputEscapeSpace = 0;
 
-    if (!pszOrganizationalUnit || !pszOrganizationalUnit[0])
+    if (!pDomainName || !pDomainName[0])
     {
-        result = NULL;
-        ceError = ERROR_SUCCESS;
-        GOTO_CLEANUP_EE(EE);
+        ceError = LW_ERROR_INVALID_DOMAIN;
+        BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
-    comma = strchr(pszOrganizationalUnit, ',');
-    if (!comma)
+    pPos = pDomainName;
+    while (TRUE)
     {
-        /* already in canonical "/" format */
-        ceError = CTAllocateString(pszOrganizationalUnit, &result);
-        GOTO_CLEANUP_EE(EE);
-    }
-
-    /* create a temporary buffer in which to party */
-    ceError = CTAllocateString(pszOrganizationalUnit, &temp);
-    CLEANUP_ON_DWORD_EE(ceError, EE);
-
-    CTStripWhitespace(temp);
-
-    current = temp;
-    comma = strchr(current, ',');
-
-    while (1)
-    {
-        PSTR equalSign;
-        PSTR type;
-        PSTR component;
-        BOOLEAN isDc;
-        BOOLEAN isOu;
-
-        if (comma)
+        pNextComponent = strchr(pPos, '.');
+        if (pNextComponent == pPos)
         {
-            comma[0] = 0;
+            // Two periods in a row
+            ceError = LW_ERROR_INVALID_DOMAIN;
+            BAIL_ON_CENTERIS_ERROR(ceError);
         }
-        equalSign = strchr(current, '=');
-        if (!equalSign)
+        if (pNextComponent && pNextComponent[1])
         {
-            ceError = LW_ERROR_INVALID_OU;
-            GOTO_CLEANUP_EE(EE);
-        }
-        equalSign[0] = 0;
-
-        type = current;
-        component = equalSign + 1;
-
-        isDc = !strcasecmp("dc", type);
-        isOu = !strcasecmp("ou", type) || !strcasecmp("cn", type);
-        if (!isDc && !isOu)
-        {
-            ceError = LW_ERROR_INVALID_OU;
-            GOTO_CLEANUP_EE(EE);
-        }
-        if (!isDc)
-        {
-            if (dnsDomain)
-            {
-                ceError = LW_ERROR_INVALID_OU;
-                GOTO_CLEANUP_EE(EE);
-            }
-            if (result)
-            {
-                PSTR newResult;
-                ceError = CTAllocateStringPrintf(&newResult, "%s/%s", component, result);
-                GOTO_CLEANUP_ON_DWORD_EE(ceError, EE);
-                CT_SAFE_FREE_STRING(result);
-                result = newResult;
-            }
-            else
-            {
-                ceError = CTAllocateString(component, &result);
-                GOTO_CLEANUP_ON_DWORD_EE(ceError, EE);
-            }
+            componentCount++;
+            pPos = pNextComponent + 1;
         }
         else
         {
-            if (!result)
-            {
-                ceError = LW_ERROR_INVALID_OU;
-                GOTO_CLEANUP_EE(EE);
-            }
-            if (dnsDomain)
-            {
-                PSTR newDnsDomain;
-                ceError = CTAllocateStringPrintf(&newDnsDomain, "%s.%s", dnsDomain, component);
-                GOTO_CLEANUP_ON_DWORD_EE(ceError, EE);
-                CT_SAFE_FREE_STRING(dnsDomain);
-                dnsDomain = newDnsDomain;
-            }
-            else
-            {
-                ceError = CTAllocateString(component, &dnsDomain);
-                GOTO_CLEANUP_ON_DWORD_EE(ceError, EE);
-            }
+            break;
         }
-        if (!comma)
+    }
+
+    // Length original string - periods + strlen(",dc=") * components + null
+    // Note there is one more component than number of periods
+    ceError = LwAllocateMemory(
+                    strlen(pDomainName) +
+                        (sizeof(",dc=") - 2) * componentCount + 2,
+                    PPCAST(&pLdapDomainSuffix));
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    pOutputPos = pLdapDomainSuffix;
+    pPos = pDomainName;
+    while (TRUE)
+    {
+        pNextComponent = strchr(pPos, '.');
+        if (!pNextComponent)
+        {
+            pNextComponent = pPos + strlen(pPos);
+        }
+        memcpy(pOutputPos, ",dc=", sizeof(",dc=") - 1);
+        pOutputPos += sizeof(",dc=") - 1;
+        memcpy(pOutputPos, pPos, pNextComponent - pPos);
+        pOutputPos += pNextComponent - pPos;
+
+        if (!pNextComponent[0] || !pNextComponent[1])
         {
             break;
         }
-        current = comma + 1;
-        comma = strchr(current, ',');
+        else
+        {
+            // Skip the .
+            pPos = pNextComponent + 1;
+        }
+    }
+    pOutputPos[0] = 0;
+
+    if (strlen(pInputOu) > strlen(pLdapDomainSuffix) &&
+            !strcasecmp(pInputOu + strlen(pInputOu) -
+                strlen(pLdapDomainSuffix), pLdapDomainSuffix))
+    {
+        // The input OU is in ldap format because ends with the right domain suffix
+        ceError = LwAllocateString(pInputOu, ppCanonicalOu);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+
+        goto cleanup;
     }
 
-    if (IsNullOrEmptyString(dnsDomain))
+    componentCount = 1;
+    pPos = pInputOu;
+
+    while(pPos[0])
     {
-        ceError = LW_ERROR_INVALID_OU;
-        GOTO_CLEANUP_EE(EE);
+        if (pPos[0] == '\\')
+        {
+            if (!pPos[1])
+            {
+                ceError = LW_ERROR_INVALID_OU;
+                BAIL_ON_CENTERIS_ERROR(ceError);
+            }
+            else if (pPos[1] == '/')
+            {
+                outputEscapeSpace--;
+            }
+            pPos++;
+        }
+        else if (pPos[0] == ',')
+        {
+            outputEscapeSpace++;
+        }
+        else if (pPos[0] == '/')
+        {
+            if (!pPos[1] || pPos[1] == '/')
+            {
+                ceError = LW_ERROR_INVALID_OU;
+                BAIL_ON_CENTERIS_ERROR(ceError);
+            }
+            componentCount++;
+        }
+        pPos++;
     }
 
-    if (IsNullOrEmptyString(result))
+    outputSize = strlen(pInputOu) + (sizeof("ou=") - 1) * componentCount +
+                    strlen(pLdapDomainSuffix) + outputEscapeSpace + 1;
+    ceError = LwAllocateMemory(
+                    outputSize,
+                    PPCAST(&pOutputOu));
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    pOutputPos = pOutputOu + outputSize;
+    memcpy(
+            pOutputPos - strlen(pLdapDomainSuffix) - 1,
+            pLdapDomainSuffix,
+            strlen(pLdapDomainSuffix) + 1);
+    pOutputPos -= strlen(pLdapDomainSuffix) + 1;
+    componentCount = 1;
+    pPos = pInputOu;
+
+    while (pPos[0])
     {
-        ceError = LW_ERROR_INVALID_OU;
-        GOTO_CLEANUP_EE(EE);
+        pNextComponent = pPos;
+        outputEscapeSpace = 0;
+        while(pNextComponent[0])
+        {
+            if (pNextComponent[0] == '\\')
+            {
+                if (!pNextComponent[1])
+                {
+                    ceError = LW_ERROR_INVALID_OU;
+                    BAIL_ON_CENTERIS_ERROR(ceError);
+                }
+                else if (pNextComponent[1] == '/')
+                {
+                    outputEscapeSpace--;
+                }
+                pNextComponent++;
+            }
+            else if (pNextComponent[0] == ',')
+            {
+                outputEscapeSpace++;
+            }
+            else if (pNextComponent[0] == '/')
+            {
+                break;
+            }
+            pNextComponent++;
+        }
+
+        if (componentCount > 1)
+        {
+            pOutputPos[-1] = ',';
+            pOutputPos--;
+        }
+
+        pUnescapedComponentEnd = pOutputPos;
+        pOutputPos -= (pNextComponent - pPos) + outputEscapeSpace;
+        pUnescapePos = pOutputPos;
+        while (pPos < pNextComponent)
+        {
+            if (pPos[0] == '\\')
+            {
+                if (!pPos[1])
+                {
+                    ceError = LW_ERROR_INVALID_OU;
+                    BAIL_ON_CENTERIS_ERROR(ceError);
+                }
+                else if (pPos[1] == '/')
+                {
+                    pUnescapePos[0] = pPos[1];
+                }
+                else
+                {
+                    pUnescapePos[0] = pPos[0];
+                    pUnescapePos[1] = pPos[1];
+                    pUnescapePos++;
+                }
+                pPos++;
+            }
+            else if (pPos[0] == ',')
+            {
+                pUnescapePos[0] = '\\';
+                pUnescapePos[1] = pPos[0];
+                pUnescapePos++;
+            }
+            else
+            {
+                pUnescapePos[0] = pPos[0];
+            }
+            pUnescapePos++;
+            pPos++;
+        }
+
+        if (componentCount == 1 &&
+                IsContainer(pOutputPos, pUnescapedComponentEnd - pOutputPos))
+        {
+            memcpy(pOutputPos - (sizeof("cn=") - 1), "cn=", sizeof("cn=") - 1);
+        }
+        else
+        {
+            // pOutputPos already points to the next byte to write, so back up
+            // one less byte
+            memcpy(pOutputPos - (sizeof("ou=") - 1), "ou=", sizeof("ou=") - 1);
+        }
+        pOutputPos -= (sizeof("ou=") - 1);
+        pPos = pNextComponent;
+        if (pPos[0] == '/')
+        {
+            pPos++;
+        }
+        componentCount++;
     }
 
-    if (strcasecmp(dnsDomain, pszDomainName))
-    {
-        ceError = LW_ERROR_INVALID_OU;
-        GOTO_CLEANUP_EE(EE);
-    }
+    LW_ASSERT(pOutputPos == pOutputOu);
+
+    *ppCanonicalOu = pOutputOu;
 
 cleanup:
     if (ceError)
     {
-        CT_SAFE_FREE_STRING(result);
+        *ppCanonicalOu = NULL;
+        LW_SAFE_FREE_STRING(pOutputOu);
     }
+    LW_SAFE_FREE_STRING(pLdapDomainSuffix);
 
-    CT_SAFE_FREE_STRING(dnsDomain);
-    CT_SAFE_FREE_STRING(temp);
-
-    *pszCanonicalizedOrganizationalUnit = result;
-    if (ceError)
-    {
-        DJ_LOG_VERBOSE("Error in CanonicalizeOrganizationalUnit: 0x%08x, EE = %d", ceError, EE);
-    }
     return ceError;
+
+error:
+    goto cleanup;
 }
 
 static QueryResult QueryDoJoin(const JoinProcessOptions *options, LWException **exc)
@@ -686,7 +821,7 @@ static void DoJoin(JoinProcessOptions *options, LWException **exc)
     PSTR pszCanonicalizedOU = NULL;
     ModuleState *state = DJGetModuleStateByName(options, "join");
 
-    if (!IsNullOrEmptyString(getenv("LD_LIBRARY_PATH")) ||
+    if (!IsNullOrEmptyString(getenv("LD_LIBRARY_PATH")) || 
         !IsNullOrEmptyString(getenv("LD_PRELOAD")))
     {
         if (options->warningCallback != NULL)
@@ -706,9 +841,10 @@ static void DoJoin(JoinProcessOptions *options, LWException **exc)
 
     if (options->ouName)
     {
-        LW_CLEANUP_CTERR(exc, CanonicalizeOrganizationalUnit(&pszCanonicalizedOU,
+        LW_CLEANUP_CTERR(exc, CanonicalizeOrganizationalUnit(
                                                  options->ouName,
-                                                 options->domainName));
+                                                 options->domainName,
+                                                 &pszCanonicalizedOU));
 
         CT_SAFE_FREE_STRING(options->ouName);
         options->ouName = pszCanonicalizedOU;
@@ -743,13 +879,50 @@ cleanup:
 
 void
 DJTestJoin(
+    PCSTR pszDomainName,
     BOOLEAN *isValid,
     LWException **exc
     )
 {
-    LW_CLEANUP_LSERR(exc, LsaNetTestJoinDomain(isValid));
+    DWORD dwError = 0;
+    BOOLEAN isJoined = FALSE;
+    HANDLE hLsaConnection = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
+
+    dwError = LsaOpenServer(&hLsaConnection);
+    LW_CLEANUP_LSERR(exc, dwError);
+
+    dwError = LsaAdGetMachineAccountInfo(hLsaConnection, pszDomainName, &pAccountInfo);
+    switch (dwError)
+    {
+        case NERR_SetupNotJoined:
+            isJoined = FALSE;
+            dwError = 0;
+            break;
+        case 0:
+            isJoined = TRUE;
+            break;
+        default:
+            LW_CLEANUP_LSERR(exc, dwError);
+    }
+
 cleanup:
-    ;
+    if (dwError)
+    {
+        isJoined = FALSE;
+    }
+
+    if (pAccountInfo)
+    {
+        LsaAdFreeMachineAccountInfo(pAccountInfo);
+    }
+
+    if (hLsaConnection)
+    {
+        LsaCloseServer(hLsaConnection);
+    }
+
+    *isValid = isJoined;
 }
 
 const JoinModule DJDoJoinModule = { TRUE, "join", "join computer to AD", QueryDoJoin, DoJoin, GetJoinDescription };
@@ -778,7 +951,7 @@ static QueryResult QueryLeave(const JoinProcessOptions *options, LWException **e
         return CannotConfigure;
     }
 
-    LW_TRY(exc, DJTestJoin(&joinValid, &LW_EXC));
+    LW_TRY(exc, DJTestJoin(options->domainName, &joinValid, &LW_EXC));
     if(!joinValid)
     {
         result = FullyConfigured;
@@ -846,23 +1019,37 @@ DJGetConfiguredDnsDomain(
     LWException **exc
     )
 {
-    DWORD _err = LsaGetDnsDomainName(ppszDomain);
-    if(_err)
-    {
-        LW_RAISE_LSERR(exc, _err);
-        if(exc != NULL)
-        {
-            switch(_err)
-            {
-                case LW_ERROR_NOT_JOINED_TO_AD:
-                    (*exc)->code = ERROR_NO_SUCH_DOMAIN;
-                    break;
-            }
-        }
-        goto cleanup;
-    }
+    DWORD dwError = 0;
+    PSTR pszDnsDomainName = NULL;
+    HANDLE hLsaConnection = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
+
+    dwError = LsaOpenServer(&hLsaConnection);
+    LW_CLEANUP_LSERR(exc, dwError);
+
+    dwError = LsaAdGetMachineAccountInfo(hLsaConnection, NULL, &pAccountInfo);
+    LW_CLEANUP_LSERR(exc, dwError);
+
+    dwError = LwAllocateString(pAccountInfo->DnsDomainName, &pszDnsDomainName);
+    LW_CLEANUP_LSERR(exc, dwError);
+
 cleanup:
-    ;
+    if (dwError)
+    {
+        LW_SAFE_FREE_STRING(pszDnsDomainName);
+    }
+
+    if (pAccountInfo)
+    {
+        LsaAdFreeMachineAccountInfo(pAccountInfo);
+    }
+
+    if (hLsaConnection)
+    {
+        LsaCloseServer(hLsaConnection);
+    }
+
+    *ppszDomain = pszDnsDomainName;
 }
 
 void
@@ -872,132 +1059,55 @@ DJGetConfiguredShortDomain(
     )
 {
     DWORD dwError = 0;
-    HANDLE hStore = 0;
-    PLWPS_PASSWORD_INFO pPassInfo = NULL;
-    PSTR pszDomain = NULL;
+    PSTR pszNetbiosDomainName = NULL;
+    HANDLE hLsaConnection = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
-    *ppszWorkgroup = NULL;
-
-    dwError = LwpsOpenPasswordStore(
-                LWPS_PASSWORD_STORE_DEFAULT,
-                &hStore);
+    dwError = LsaOpenServer(&hLsaConnection);
     LW_CLEANUP_LSERR(exc, dwError);
 
-    dwError = LwpsGetPasswordByCurrentHostName(
-                hStore,
-                &pPassInfo);
-    if (dwError || pPassInfo == NULL || pPassInfo->pwszDnsDomainName == NULL)
-    {
-        dwError = ERROR_NOT_JOINED;
-    }
+    dwError = LsaAdGetMachineAccountInfo(hLsaConnection, NULL, &pAccountInfo);
     LW_CLEANUP_LSERR(exc, dwError);
 
-    dwError = LwWc16sToMbs(
-                pPassInfo->pwszDomainName,
-                &pszDomain);
+    dwError = LwAllocateString(pAccountInfo->NetbiosDomainName, &pszNetbiosDomainName);
     LW_CLEANUP_LSERR(exc, dwError);
 
-    *ppszWorkgroup = pszDomain;
-    pszDomain = NULL;
-
 cleanup:
-    LW_SAFE_FREE_STRING(pszDomain);
-    if (pPassInfo)
+    if (dwError)
     {
-        LwpsFreePasswordInfo(hStore, pPassInfo);
+        LW_SAFE_FREE_STRING(pszNetbiosDomainName);
     }
-    if (hStore != (HANDLE)NULL)
-    {
-        LwpsClosePasswordStore(hStore);
-    }
-}
 
-void
-DJGetDomainRwDC(PCSTR domain, PSTR *dc, LWException **exc)
-{
-    LW_CLEANUP_LSERR(exc, LsaNetGetRwDCName(domain, dc));
-cleanup:
-    ;
+    if (pAccountInfo)
+    {
+        LsaAdFreeMachineAccountInfo(pAccountInfo);
+    }
+
+    if (hLsaConnection)
+    {
+        LsaCloseServer(hLsaConnection);
+    }
+
+    *ppszWorkgroup = pszNetbiosDomainName;
 }
 
 void
 DJGetComputerDN(PSTR *dn, LWException **exc)
 {
-    DWORD _err = 0;
+    DWORD dwError = 0;
+    HANDLE hLsaConnection = NULL;
 
-    LW_CLEANUP_LSERR(exc, LWNetExtendEnvironmentForKrb5Affinity(TRUE));
+    dwError = LsaOpenServer(&hLsaConnection);
+    LW_CLEANUP_LSERR(exc, dwError);
 
-    _err = LsaGetComputerDN(dn);
-    if(_err)
-    {
-        LW_RAISE_LSERR(exc, _err);
-        if(exc != NULL)
-        {
-            switch(_err)
-            {
-                case LW_ERROR_NOT_JOINED_TO_AD:
-                    (*exc)->code = ERROR_NO_SUCH_DOMAIN;
-                    break;
-            }
-        }
-        goto cleanup;
-    }
-cleanup:
-    ;
-}
-
-void DJNetInitialize(BOOLEAN bEnableDcerpcd, LWException **exc)
-{
-    BOOLEAN systemDcedExists = FALSE;
-    LWException *innerExc = NULL;
-    int firstStart = 0;
-    int firstStop = 0;
-    int stopLaterOffset = 0;
-    BOOLEAN exists = FALSE;
-
-#ifndef MINIMAL_JOIN
-
-    if (geteuid() == 0)
-    {
-        LW_CLEANUP_CTERR(exc, DJGetBaseDaemonPriorities(
-                    &firstStart,
-                    &firstStop,
-                    &stopLaterOffset));
-
-
-        LW_CLEANUP_CTERR(exc, CTCheckFileOrLinkExists("/usr/sbin/svcadm", &exists));
-
-        if (!exists)
-        {
-            LW_TRY(exc, DJManageDaemon("lwsmd", TRUE,
-                        firstStart - 2,
-                        firstStop + stopLaterOffset * 1,
-                        &LW_EXC));
-        }
-    }
-
-#if 0
-        LW_TRY(exc, DJManageDaemon("srvsvcd", TRUE,
-                    21, 9, &LW_EXC));
-#endif
-#endif
-
-        LW_CLEANUP_LSERR(exc, LsaNetJoinInitialize());
-#if 0
-        /* Do not enable debug logging in lsajoin because
-           it does not respect domainjoin logging settings
-           such as logfile */
-        if(gdjLogInfo.dwLogLevel >= LOG_LEVEL_VERBOSE)
-            lsaFunctions->pfnEnableDebugLog();
-#endif
+    dwError = LsaAdGetComputerDn(hLsaConnection, NULL, dn);
+    LW_CLEANUP_LSERR(exc, dwError);
 
 cleanup:
-    LWHandle(&innerExc);
-}
-
-void DJNetShutdown(LWException **exc)
-{
-    LsaNetJoinShutdown();
+    if (hLsaConnection)
+    {
+        LsaCloseServer(hLsaConnection);
+    }
 }
 
 void DJCreateComputerAccount(
@@ -1005,17 +1115,13 @@ void DJCreateComputerAccount(
                 JoinProcessOptions *options,
                 LWException **exc)
 {
-    DistroInfo distro;
+    LwDistroInfo distro;
     PSTR osName = NULL;
     PSTR tempDir = NULL;
-    PSTR origEnv = NULL;
-    // Do not free origEnvVarStr
-    PSTR origEnvVarStr = NULL;
     PSTR shortHostname = NULL;
     PSTR hostFqdn = NULL;
     // Do not free dnsDomain
     PSTR dnsDomain = NULL;
-    CHAR krb5ConfEnv[256];
     DWORD dwFlags = 0;
     DWORD dwError = 0;
 
@@ -1049,68 +1155,46 @@ void DJCreateComputerAccount(
 
     LW_TRY(exc, DJCopyKrb5ToRootDir(NULL, tempDir, &LW_EXC));
 
-    /*
-     * Setup krb5.conf with the domain as the Kerberos realm.
-     * We do this before doing the join. This is required for any
-     * authenticated LDAP connections used to acquire a TGT in order
-     * to move the computer account into the right OU.
-     */
-    LW_CLEANUP_CTERR(exc, DJModifyKrb5Conf(
-                                 tempDir,
-                                 TRUE,
-                                 options->domainName, NULL, NULL));
-
-    origEnv = getenv("KRB5_CONFIG");
-    if (origEnv && *origEnv)
-    {
-        LW_CLEANUP_CTERR(exc,
-                         CTAllocateStringPrintf(
-                         &origEnvVarStr,
-                         "KRB5_CONFIG=%s",
-                         origEnv));
-    }
-
-    sprintf(krb5ConfEnv, "KRB5_CONFIG=%s/etc/krb5.conf", tempDir);
-
-    if (putenv(krb5ConfEnv) != 0) {
-       LW_CLEANUP_CTERR(exc, LwMapErrnoToLwError(errno));
-    }
-
-    LW_CLEANUP_LSERR(exc, LWNetExtendEnvironmentForKrb5Affinity(TRUE));
-
-    if ( options->disableTimeSync )
+    if ( options->disableTimeSync || options->enableMultipleJoins )
     {
         dwFlags |= LSA_NET_JOIN_DOMAIN_NOTIMESYNC;
     }
 
+    if ( options->enableMultipleJoins )
+    {
+        dwFlags |= LSA_NET_JOIN_DOMAIN_MULTIPLE;
+    }
+
     LW_CLEANUP_CTERR(exc, DJGetFQDN(&shortHostname, &hostFqdn));
 
     if (hostFqdn && (strlen(hostFqdn) > (strlen(shortHostname) + 1)))
     {
         dnsDomain = hostFqdn + strlen(shortHostname) + 1;
     }
-
-    dwError = SetBooleanRegistryValue("Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
-                                   "AssumeDefaultDomain",
-                                   options->assumeDefaultDomain);
-    if (dwError)
+    else
     {
-        LW_RAISE_LSERR(exc, dwError);
-        goto cleanup;
-    }
-
-    dwError = SetStringRegistryValue("Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
-                                     "UserDomainPrefix",
-                                     options->userDomainPrefix);
-    if (dwError)
-    {
-        LW_RAISE_LSERR(exc, dwError);
-        goto cleanup;
+        dnsDomain = "";
     }
 
     LW_CLEANUP_LSERR(exc, LsaOpenServer(&lsa));
 
-    dwError = LsaAdJoinDomain(
+    if (options->setAssumeDefaultDomain)
+    {
+        dwError = SetBooleanRegistryValue("Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
+                                       "AssumeDefaultDomain",
+                                       options->assumeDefaultDomain);
+        LW_CLEANUP_LSERR(exc, dwError);
+
+        dwError = SetStringRegistryValue("Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
+                                         "UserDomainPrefix",
+                                         options->userDomainPrefix);
+        LW_CLEANUP_LSERR(exc, dwError);
+
+        dwError = LsaRefreshConfiguration(lsa);
+        LW_CLEANUP_LSERR(exc, dwError);
+    }
+
+    dwError = LsaAdJoinDomainDn(
                  lsa,
                  options->computerName,
                  dnsDomain,
@@ -1126,7 +1210,7 @@ void DJCreateComputerAccount(
     {
         switch(dwError)
         {
-            case ERROR_FILE_NOT_FOUND:
+            case LW_ERROR_LDAP_NO_SUCH_OBJECT:
                 LW_RAISE_EX(exc, LW_ERROR_INVALID_OU, "Lsass Error", "The OU is invalid.");
                 break;
             case ERROR_CRC:
@@ -1159,7 +1243,7 @@ void DJCreateComputerAccount(
         }
         goto cleanup;
     }
-
+    
     LW_TRY(exc, DJGuessShortDomainName(
                                  options->domainName,
                                  shortDomainName, &LW_EXC));
@@ -1185,12 +1269,6 @@ cleanup:
         CT_SAFE_FREE_STRING(tempDir);
     }
 
-    if (origEnvVarStr) {
-       putenv(origEnvVarStr);
-    } else {
-       putenv("KRB5_CONFIG=/etc/krb5.conf");
-    }
-
     CT_SAFE_FREE_STRING(likewiseProduct);
     CT_SAFE_FREE_STRING(likewiseVersion);
     CT_SAFE_FREE_STRING(likewiseBuild);
@@ -1210,7 +1288,7 @@ void DJDisableComputerAccount(PCSTR username,
     HANDLE lsa = NULL;
 
     LW_CLEANUP_LSERR(exc, LsaOpenServer(&lsa));
-    LW_CLEANUP_LSERR(exc, LsaAdLeaveDomain(lsa, username, password));
+    LW_CLEANUP_LSERR(exc, LsaAdLeaveDomain2(lsa, username, password, options->domainName, 0));
 
 cleanup:
 
@@ -1233,9 +1311,51 @@ void DJGuessShortDomainName(PCSTR longName,
                 PSTR *shortName,
                 LWException **exc)
 {
-    LW_CLEANUP_LSERR(exc, LsaNetGetShortDomainName(longName, shortName));
+    DWORD dwError = 0;
+    PSTR pszNetbiosDomainName = NULL;
+    PCSTR pszDnsDomainName = NULL;
+    PSTR pszFreeDnsDomainName = NULL;
+    PLWNET_DC_INFO pDcInfo = NULL;
+
+    if (LW_IS_NULL_OR_EMPTY_STR(longName))
+    {
+        LW_TRY(exc, DJGetConfiguredDnsDomain(&pszFreeDnsDomainName, &LW_EXC));
+        pszDnsDomainName = pszFreeDnsDomainName;
+    }
+    else
+    {
+        pszDnsDomainName = longName;
+    }
+
+    dwError = LWNetGetDCName(
+                    NULL,
+                    pszDnsDomainName,
+                    NULL,
+                    0,
+                    &pDcInfo);
+    LW_CLEANUP_LSERR(exc, dwError);
+
+    if (LW_IS_NULL_OR_EMPTY_STR(pDcInfo->pszNetBIOSDomainName))
+    {
+        dwError = LW_ERROR_NO_NETBIOS_NAME;
+        LW_CLEANUP_LSERR(exc, dwError);
+    }
+
+    dwError = LwAllocateString(
+                    pDcInfo->pszNetBIOSDomainName,
+                    &pszNetbiosDomainName);
+    LW_CLEANUP_LSERR(exc, dwError);
+
 cleanup:
-    ;
+    if (dwError)
+    {
+        LW_SAFE_FREE_STRING(pszNetbiosDomainName);
+    }
+
+    LW_SAFE_FREE_STRING(pszFreeDnsDomainName);
+    LWNET_SAFE_FREE_DC_INFO(pDcInfo);
+
+    *shortName = pszNetbiosDomainName;
 }
 
 DWORD
