@@ -102,9 +102,9 @@ DJGetDaemonStatus(
          */
         status = 1;
     }
-#ifdef __LWI_FREEBSD7__
-    /* FreeBSD 7.x, running the script nscd w/ status option returns 0 (and
-     * prints nothing) even when nscd is not running.
+#if defined(__LWI_FREEBSD__)
+    /* On various versions of FreeBSD, running the script nscd w/ status
+     * returns 0 (and prints nothing) even when nscd is not running.
      */
     else if (!strcmp(prefixedPath, "/etc/rc.d/nscd"))
     {
@@ -280,7 +280,6 @@ DJStartStopDaemon(
     PSTR pszDaemonPath = NULL;
     int retry;
 
-#if !defined(__LWI_MACOSX__)
     LW_TRY(exc, FindDaemonScript(pszDaemonName, &pszDaemonPath, &LW_EXC));
 
     if (bStatus) {
@@ -346,7 +345,6 @@ DJStartStopDaemon(
         }
         goto cleanup;
     }
-#endif
 
 cleanup:
 
@@ -587,7 +585,7 @@ DJConfigureForDaemonRestart(
     )
 {
     BOOLEAN bFileExists = FALSE;
-    DistroInfo distro;
+    LwDistroInfo distro;
     PSTR symlinkTarget = NULL;
     PSTR symlinkName = NULL;
     long status = 0;
@@ -727,6 +725,106 @@ DJConfigureForDaemonRestart(
     {
         BOOLEAN bUsesSVCS = FALSE;
 
+        LW_CLEANUP_CTERR(exc, CTCheckFileExists("/usr/sbin/svcadm", &bUsesSVCS));
+        if (bUsesSVCS)
+        {
+            /* On newer Solaris system, in order for a daemon to start in multi-user mode,
+             * we need to register our daemons with the service manager (svcs).
+             *
+             * To add:
+             * svccfg import /etc/likewise/svcs-solaris/<daemon name>.xml
+             * svcadm enable <daemon name>
+             *
+             * To remove:
+             * svcadm disable <daemon name>
+             * svccfg delete  <daemon name>
+             */
+            if(bStatus)
+            {
+                LW_CLEANUP_CTERR(exc, CTShell("/usr/sbin/svccfg import /etc/likewise/svcs-solaris/%daemonName.xml >%outputBuffer 2>&1; echo $? >%statusBuffer",
+                            CTSHELL_STRING(daemonName, pszDaemonName),
+                            CTSHELL_BUFFER(outputBuffer, &outputBuffer),
+                            CTSHELL_BUFFER(statusBuffer, &statusBuffer)));
+
+                status = atol(statusBuffer);
+
+                CT_SAFE_FREE_STRING(statusBuffer);
+                statusBuffer = NULL;
+
+                DJ_LOG_INFO("Daemon [%s]: svccfg import /etc/likewise/svcs-solaris/%s.xml status [%d]", pszDaemonName, pszDaemonName, status);
+                if (outputBuffer[0] != '\0')
+                {
+                    DJ_LOG_VERBOSE("svccfg command output: %s", outputBuffer);
+                }
+
+                CT_SAFE_FREE_STRING(outputBuffer);
+                outputBuffer = NULL;
+
+                if (status) {
+                    LW_RAISE_EX(exc, ERROR_INVALID_STATE, "svccfg import failed", "An error occurred while using svccfg to process the '%s' daemon. This daemon was being added to the list of processes to start on reboot.", pszDaemonName);
+                    goto cleanup;
+                }
+
+                LW_CLEANUP_CTERR(exc, CTShell("/usr/sbin/svcadm enable %daemonName >%outputBuffer 2>&1; echo $? >%statusBuffer",
+                            CTSHELL_STRING(daemonName, pszDaemonName),
+                            CTSHELL_BUFFER(outputBuffer, &outputBuffer),
+                            CTSHELL_BUFFER(statusBuffer, &statusBuffer)));
+
+                status = atol(statusBuffer);
+
+                DJ_LOG_INFO("Daemon [%s]: svcadm enable %s status [%d]", pszDaemonName, pszDaemonName, status);
+                if (outputBuffer[0] != '\0')
+                {
+                    DJ_LOG_VERBOSE("svcadm command output: %s", outputBuffer);
+                }
+
+                CT_SAFE_FREE_STRING(outputBuffer);
+                outputBuffer = NULL;
+
+                if (status) {
+                    LW_RAISE_EX(exc, ERROR_INVALID_STATE, "svcadm enable failed", "An error occurred while using svcadm to process the '%s' daemon. This daemon was being added to the list of processes to start on reboot.", pszDaemonName);
+                    goto cleanup;
+                }
+            }
+            else
+            {
+                LW_CLEANUP_CTERR(exc, CTShell("/usr/sbin/svcadm disable %daemonName >%outputBuffer 2>&1; echo $? >%statusBuffer",
+                            CTSHELL_STRING(daemonName, pszDaemonName),
+                            CTSHELL_BUFFER(outputBuffer, &outputBuffer),
+                            CTSHELL_BUFFER(statusBuffer, &statusBuffer)));
+
+                status = atol(statusBuffer);
+
+                CT_SAFE_FREE_STRING(statusBuffer);
+                statusBuffer = NULL;
+
+                DJ_LOG_INFO("Daemon [%s]: svcadm disable %s status [%d]", pszDaemonName, pszDaemonName, status);
+                if (outputBuffer[0] != '\0')
+                {
+                    DJ_LOG_VERBOSE("svcadm command output: %s", outputBuffer);
+                }
+
+                CT_SAFE_FREE_STRING(outputBuffer);
+                outputBuffer = NULL;
+
+                LW_CLEANUP_CTERR(exc, CTShell("/usr/sbin/svccfg delete %daemonName >%outputBuffer 2>&1; echo $? >%statusBuffer",
+                            CTSHELL_STRING(daemonName, pszDaemonName),
+                            CTSHELL_BUFFER(outputBuffer, &outputBuffer),
+                            CTSHELL_BUFFER(statusBuffer, &statusBuffer)));
+
+                status = atol(statusBuffer);
+
+                DJ_LOG_INFO("Daemon [%s]: svccfg delete %s status [%d]", pszDaemonName, pszDaemonName, status);
+                if (outputBuffer[0] != '\0')
+                {
+                    DJ_LOG_VERBOSE("svccfg command output: %s", outputBuffer);
+                }
+
+                CT_SAFE_FREE_STRING(outputBuffer);
+                outputBuffer = NULL;
+            }
+        }
+        else
         {
             /* On older Solaris system, in order for a daemon to start in multi-user mode,
              * a start symlink must be put in /etc/rc2.d, and a stop symlink
@@ -735,7 +833,7 @@ DJConfigureForDaemonRestart(
             if(bStatus)
             {
                 PCSTR relativeName = strrchr(pszDaemonName, '/');
-
+    
                 if (relativeName == NULL)
                 {
                     relativeName = pszDaemonName;
@@ -744,7 +842,7 @@ DJConfigureForDaemonRestart(
                 {
                     relativeName++;
                 }
-
+    
                 LW_TRY(exc, FindDaemonScript(pszDaemonName,
                             &symlinkTarget, &LW_EXC));
                 LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
@@ -752,19 +850,19 @@ DJConfigureForDaemonRestart(
                 LW_TRY(exc,
                         DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
                 CT_SAFE_FREE_STRING(symlinkName);
-
+    
                 LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
                         "/etc/rc1.d/K%02d%s", stopPriority, relativeName));
                 LW_TRY(exc,
                         DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
                 CT_SAFE_FREE_STRING(symlinkName);
-
+    
                 LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
                         "/etc/rcS.d/K%02d%s", stopPriority, relativeName));
                 LW_TRY(exc,
                         DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
                 CT_SAFE_FREE_STRING(symlinkName);
-
+    
                 LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
                         "/etc/rc2.d/S%02d%s", startPriority, relativeName));
                 LW_TRY(exc,
@@ -840,8 +938,7 @@ DJManageDaemon(
 {
     BOOLEAN bStarted = FALSE;
 
-#if !defined(__LWI_MACOSX__)
-    // Check for the existence of the daemon prior to doing anything.
+    // Check for the existence of the daemon prior to doing anything. 
     // notice that we are using the private version so that if we fail,
     // our inner exception will be the one that was tossed due to the failure.
     LW_TRY(exc, DJGetDaemonStatus(pszName, &bStarted, &LW_EXC));
@@ -861,7 +958,6 @@ DJManageDaemon(
     if (bStarted != bStatus) {
         LW_TRY(exc, DJStartStopDaemon(pszName, bStatus, &LW_EXC));
     }
-#endif
 
 cleanup:
     ;
