@@ -32,6 +32,8 @@
 #include "ctshell.h"
 #include "djauthinfo.h"
 #include "djdistroinfo.h"
+#include "lwmem.h"
+#include "lwstr.h"
 
 #define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
@@ -94,7 +96,13 @@ WriteHostnameToFiles(
                 ceError = LwMapErrnoToLwError(errno);
                 BAIL_ON_CENTERIS_ERROR(ceError);
             }
-            fprintf(fp, "%s\n", pszComputerName);
+
+            if (fprintf(fp, "%s\n", pszComputerName) < 0)
+            {
+                ceError = LwMapErrnoToLwError(errno);
+                BAIL_ON_CENTERIS_ERROR(ceError);
+            }
+
             fclose(fp);
             fp = NULL;
         }
@@ -246,10 +254,73 @@ error:
     return ceError;
 }
 
+DWORD
+SedEscapeLiteral(
+    PCSTR pInput,
+    PSTR* ppOutput
+    )
+{
+    PSTR pOutput = NULL;
+    DWORD outputIndex = 0;
+    DWORD index = 0;
+    DWORD dwError = 0;
+
+    // Calculate the length first
+    for (index = 0; pInput[index]; index++)
+    {
+        switch (pInput[index])
+        {
+            // See the \char section of gnu sed manual
+            case '$':
+            case '*':
+            case '.':
+            case '[':
+            case '\\':
+            case '^':
+                outputIndex += 2;
+                break;
+            default:
+                outputIndex += 1;
+        }
+    }
+
+    dwError = LwAllocateMemory(
+                    outputIndex + 1,
+                    PPCAST(&pOutput));
+    BAIL_ON_CENTERIS_ERROR(dwError);
+
+    outputIndex = 0;
+    for (index = 0; pInput[index]; index++)
+    {
+        switch (pInput[index])
+        {
+            // See the \char section of gnu sed manual
+            case '$':
+            case '*':
+            case '.':
+            case '[':
+            case '\\':
+            case '^':
+                pOutput[outputIndex++] = '\\';
+                break;
+        }
+        pOutput[outputIndex++] = pInput[index];
+    }
+    pOutput[outputIndex++] = 0;
+
+error:
+    *ppOutput = pOutput;
+    return dwError;
+}
+
 static
 DWORD
 WriteHostnameToSunFiles(
-    PSTR pszComputerName
+    PCSTR pOldShortHostname,
+    PCSTR pNewShortHostname,
+    PCSTR pDnsDomainName,
+    PCSTR pOldFqdnHostname,
+    PCSTR pNewFqdnHostname
     )
 {
     DWORD ceError = ERROR_SUCCESS;
@@ -257,52 +328,144 @@ WriteHostnameToSunFiles(
     PSTR* ppszHostfilePaths = NULL;
     DWORD nPaths = 0;
     DWORD iPath = 0;
-    PSTR contents = NULL;
-    LONG fileLen;
+    PSTR pRealPath = NULL;
+    PSTR pTempPath = NULL;
+    PSTR pOldEscapedShortHostname = NULL;
+    PSTR pOldEscapedFqdnHostname = NULL;
+    PSTR pOldSedExpression = NULL;
+    PSTR pSedExpression = NULL;
+    BOOLEAN isSame = FALSE;
 
-    DJ_LOG_INFO("Setting hostname to [%s]", pszComputerName);
+    DJ_LOG_INFO("Setting hostname to [%s]", pNewShortHostname);
 
-    fp = fopen("/etc/nodename", "w");
-    if (fp == NULL) {
+    ceError = CTGetFileTempPath(
+                        "/etc/nodename",
+                        &pRealPath,
+                        &pTempPath);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    ceError = CTOpenFile(
+            pTempPath,
+            "w",
+            &fp);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    if (fprintf(fp, "%s\n", pNewShortHostname) < 0)
+    {
         ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
-    fprintf(fp, "%s\n", pszComputerName);
-    fclose(fp);
-    fp = NULL;
 
+    ceError = CTSafeCloseFile(&fp);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    ceError = CTFileContentsSame(
+                    pTempPath,
+                    pRealPath,
+                    &isSame);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+    if (isSame)
+    {
+        BAIL_ON_CENTERIS_ERROR(ceError = CTRemoveFile(pTempPath));
+    }
+    else
+    {
+        ceError = CTSafeReplaceFile(pRealPath, pTempPath);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+    }
+
+    CT_SAFE_FREE_STRING(pRealPath);
+    CT_SAFE_FREE_STRING(pTempPath);
+
+    ceError = CTGetFileTempPath(
+                        "/etc/defaultdomain",
+                        &pRealPath,
+                        &pTempPath);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    ceError = CTOpenFile(
+            pTempPath,
+            "w",
+            &fp);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    fprintf(fp, "%s\n", pDnsDomainName);
+
+    ceError = CTSafeCloseFile(&fp);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    ceError = CTFileContentsSame(
+                    pTempPath,
+                    pRealPath,
+                    &isSame);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+    if (isSame)
+    {
+        BAIL_ON_CENTERIS_ERROR(ceError = CTRemoveFile(pTempPath));
+    }
+    else
+    {
+        ceError = CTSafeReplaceFile(pRealPath, pTempPath);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+    }
+
+    /* The first column of the /etc/hostname.<interface> files is the IP
+     * address of the interface. It may either be specified directly in the
+     * file, or the file may refer to an entry in /etc/hosts. This program is
+     * editing /etc/hosts, any old entries in the hostname files need to be
+     * fixed. */
     ceError = CTGetMatchingFilePathsInFolder("/etc",
                                              "hostname.*",
                                              &ppszHostfilePaths,
                                              &nPaths);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    for (iPath = 0; iPath < nPaths; iPath++) {
+    ceError = SedEscapeLiteral(
+                pOldShortHostname,
+                &pOldEscapedShortHostname);
+    BAIL_ON_CENTERIS_ERROR(ceError);
 
-        CTReadFile(*(ppszHostfilePaths+iPath), SIZE_MAX, &contents, &fileLen);
-        CTStripWhitespace(contents);
-        if(!strcasecmp(contents, pszComputerName))
-        {
-            //The machine hostname is already set
-            goto done;
-        }
+    ceError = LwAllocateStringPrintf(
+                &pSedExpression,
+                "s/^%s\\([ ].*\\)\\{0,1\\}$/%s\\1/",
+                pOldEscapedShortHostname,
+                pNewShortHostname);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    if (pOldFqdnHostname != NULL)
+    {
+        ceError = SedEscapeLiteral(
+                    pOldFqdnHostname,
+                    &pOldEscapedFqdnHostname);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+
+        pOldSedExpression = pSedExpression;
+        pSedExpression = NULL;
+
+        ceError = LwAllocateStringPrintf(
+                    &pSedExpression,
+                    "%s;s/^%s\\([ ].*\\)\\{0,1\\}$/%s\\1/",
+                    pOldSedExpression,
+                    pOldEscapedFqdnHostname,
+                    pNewFqdnHostname);
+        BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
-    // Only write the hostname in 1 file
-    for (iPath = 0; iPath < nPaths && iPath < 1; iPath++) {
-
-        fp = fopen(*(ppszHostfilePaths+iPath), "w");
-        if (fp == NULL) {
-            ceError = LwMapErrnoToLwError(errno);
+    for (iPath = 0; iPath < nPaths; iPath++)
+    {
+        if (!CTStrEndsWith(ppszHostfilePaths[iPath], ".lwidentity.bak") &&
+            !CTStrEndsWith(ppszHostfilePaths[iPath], ".lwidentity.orig"))
+        {
+            ceError = CTRunSedOnFile(
+                            ppszHostfilePaths[iPath],
+                            ppszHostfilePaths[iPath],
+                            FALSE,
+                            pSedExpression);
             BAIL_ON_CENTERIS_ERROR(ceError);
         }
-        fprintf(fp, "%s\n", pszComputerName);
-        fclose(fp);
-        fp = NULL;
     }
 
 error:
-done:
 
     if (ppszHostfilePaths)
         CTFreeStringArray(ppszHostfilePaths, nPaths);
@@ -311,7 +474,12 @@ done:
         fclose(fp);
     }
 
-    CT_SAFE_FREE_STRING(contents);
+    CT_SAFE_FREE_STRING(pRealPath);
+    CT_SAFE_FREE_STRING(pTempPath);
+    CT_SAFE_FREE_STRING(pOldEscapedShortHostname);
+    CT_SAFE_FREE_STRING(pOldEscapedFqdnHostname);
+    CT_SAFE_FREE_STRING(pOldSedExpression);
+    CT_SAFE_FREE_STRING(pSedExpression);
 
     return ceError;
 }
@@ -894,7 +1062,7 @@ static QueryResult QueryDescriptionSetHostname(const JoinProcessOptions *options
     BOOLEAN modified = FALSE;
     DWORD ceError = ERROR_SUCCESS;
 
-    if(!options->joiningDomain)
+    if(!options->joiningDomain || options->enableMultipleJoins)
     {
         if(changeDescription != NULL)
         {
@@ -946,32 +1114,16 @@ static QueryResult QueryDescriptionSetHostname(const JoinProcessOptions *options
     }
     else if(strcmp(newFqdn, oldFqdnHostname))
     {
-        if(strchr(oldFqdnHostname, '.') == NULL)
-        {
-            //The current fqdn does not match out minimal requirements
-            LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&newValue,
-"%s\n"
-"\tChange the fqdn from '%s' to '%s'. The current fqdn is invalid because it does not contain a dot in the name. Changing the fqdn could be done via DNS, but this program will change it with the following steps:\n"
-"\t\t* Making sure local comes before bind in nsswitch\n"
-"\t\t* Adding the fqdn before all entries in /etc/hosts that contain the short hostname and removing the old fqdn if it appears on the line\n"
-"\t\t* Restart nscd (if running) to flush the DNS cache",
-                required, oldFqdnHostname, newFqdn));
-            CT_SAFE_FREE_STRING(required);
-            required = newValue;
-        }
-        else
-        {
-            //The current fqdn does not match our ideal fqdn
-            LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&newValue,
+        //The current fqdn does not match our ideal fqdn
+        LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&newValue,
 "%s\n"
 "\tChange the fqdn from '%s' to '%s'. This could be done via DNS, but this program will do it with the following steps:\n"
 "\t\t* Making sure local comes before bind in nsswitch\n"
 "\t\t* Adding the fqdn before all entries in /etc/hosts that contain the short hostname and removing the old fqdn if it appears on the line\n"
 "\t\t* Restart nscd (if running) to flush the DNS cache",
-                optional, oldFqdnHostname, newFqdn));
-            CT_SAFE_FREE_STRING(optional);
-            optional = newValue;
-        }
+            optional, oldFqdnHostname, newFqdn));
+        CT_SAFE_FREE_STRING(optional);
+        optional = newValue;
         describedFqdn = TRUE;
     }
 
@@ -1210,8 +1362,9 @@ DJSetComputerName(
     PSTR oldShortHostname = NULL;
     PSTR oldFqdnHostname = NULL;
     PSTR pszComputerName_lower = NULL;
+    PSTR pNewFqdnHostname = NULL;
     PSTR ppszHostfilePaths[] = { "/etc/hostname", "/etc/HOSTNAME", NULL };
-    DistroInfo distro;
+    LwDistroInfo distro;
 
     memset(&distro, 0, sizeof(distro));
 
@@ -1262,6 +1415,13 @@ DJSetComputerName(
         oldShortHostname = NULL;
     }
 
+    ceError = LwAllocateStringPrintf(
+                &pNewFqdnHostname,
+                "%s.%s",
+                pszComputerName,
+                pszDnsDomainName);
+    LW_CLEANUP_CTERR(exc, ceError);
+
     ceError = DJCopyMissingHostsEntry("/etc/inet/ipnodes", "/etc/hosts",
             pszComputerName_lower, oldShortHostname);
     if(ceError == ERROR_FILE_NOT_FOUND)
@@ -1282,7 +1442,13 @@ DJSetComputerName(
     switch (distro.os)
     {
         case OS_SUNOS:
-            LW_CLEANUP_CTERR(exc, WriteHostnameToSunFiles(pszComputerName_lower));
+            LW_CLEANUP_CTERR(exc, WriteHostnameToSunFiles(
+                        oldShortHostname,
+                        pszComputerName_lower,
+                        pszDnsDomainName,
+                        oldFqdnHostname,
+                        pNewFqdnHostname
+                        ));
             break;
         case OS_AIX:
             LW_CLEANUP_CTERR(exc, SetAIXHostname(pszComputerName_lower));
@@ -1303,6 +1469,7 @@ cleanup:
     CT_SAFE_FREE_STRING(oldShortHostname);
     CT_SAFE_FREE_STRING(oldFqdnHostname);
     CT_SAFE_FREE_STRING(pszComputerName_lower);
+    CT_SAFE_FREE_STRING(pNewFqdnHostname);
     DJFreeDistroInfo(&distro);
 }
 
@@ -1378,7 +1545,7 @@ DJIsValidComputerName(
         LWHandle(&exc);
     }
 
-    if (ceError == ERROR_INVALID_COMPUTERNAME ||
+    if (ceError == ERROR_INVALID_COMPUTERNAME || 
             ceError == ERROR_INVALID_COMPUTERNAME)
     {
         ceError = ERROR_SUCCESS;
