@@ -38,6 +38,10 @@
 #define NSSWITCH_CONF_PATH "/etc/nsswitch.conf"
 #define NSSWITCH_LWIDEFAULTS "/etc/nsswitch.lwi_defaults"
 
+#ifndef SCRIPTDIR
+#define SCRIPTDIR  PREFIXDIR"/bin"
+#endif
+
 #define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
 typedef struct
@@ -294,7 +298,7 @@ static int FindEntry(const NsswitchConf *conf, int startLine, const char *name)
     return -1;
 }
 
-static DWORD AddEntry(NsswitchConf *conf, const DistroInfo *distro,
+static DWORD AddEntry(NsswitchConf *conf, const LwDistroInfo *distro,
         int *addedIndex, const char *name)
 {
     DWORD ceError = ERROR_SUCCESS;
@@ -337,7 +341,7 @@ cleanup:
     return ceError;
 }
 
-const char * GetModuleSeparator(NsswitchConf *conf, const DistroInfo *distro)
+const char * GetModuleSeparator(NsswitchConf *conf, const LwDistroInfo *distro)
 {
     int line;
     const NsswitchEntry *copy;
@@ -377,7 +381,7 @@ static int FindModuleOnLine(const NsswitchConf *conf, int line, const char *name
     return -1;
 }
 
-static DWORD InsertModule(NsswitchConf *conf, const DistroInfo *distro,
+static DWORD InsertModule(NsswitchConf *conf, const LwDistroInfo *distro,
         int line, int insertIndex, const char *name)
 {
     DWORD ceError = ERROR_SUCCESS;
@@ -443,7 +447,7 @@ cleanup:
     return ceError;
 }
 
-static DWORD RemoveModule(NsswitchConf *conf,
+static DWORD RemoveModule(NsswitchConf *conf, 
         int line, int moduleIndex)
 {
     DWORD ceError = ERROR_SUCCESS;
@@ -494,12 +498,6 @@ cleanup:
     return ceError;
 }
 
-DWORD
-UnConfigureNameServiceSwitch()
-{
-    return DJConfigureNameServiceSwitch(NULL, FALSE);
-}
-
 DWORD
 ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
         BOOLEAN allowFileCreate)
@@ -549,11 +547,12 @@ ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
             ceError = ReadNsswitchFile(conf, testPrefix, NSSWITCH_LWIDEFAULTS);
             GCE(ceError);
             CT_SAFE_FREE_STRING(conf->filename);
-            GCE(ceError = CTStrdup(NSSWITCH_CONF_PATH, &conf->filename));
             conf->modified = TRUE;
 
             if(allowFileCreate)
             {
+                GCE(ceError = CTStrdup(NSSWITCH_CONF_PATH, &conf->filename));
+
                 /* Copy over the original file. This way the user can more
                  * clearly see what we changed by comparing nsswitch.conf with
                  * nsswitch.conf.lwidentity.orig. Also, the permissions will be
@@ -565,6 +564,10 @@ ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
                 ceError = CTCopyFileWithOriginalPerms(defaultFilePath, copyDestPath);
                 GCE(ceError);
             }
+            else
+            {
+                GCE(ceError = CTStrdup(NSSWITCH_LWIDEFAULTS, &conf->filename));
+            }
         }
     }
 
@@ -580,23 +583,9 @@ cleanup:
     return ceError;
 }
 
-static DWORD UsingLsass(BOOLEAN *using)
-{
-    DWORD ceError = ERROR_SUCCESS;
-    ceError = CTFindFileInPath("lsassd", BINDIR ":" SBINDIR, NULL);
-    if(ceError == ERROR_FILE_NOT_FOUND)
-    {
-        *using = FALSE;
-        ceError = ERROR_SUCCESS;
-    }
-    else if(!ceError)
-        *using = TRUE;
-    return ceError;
-}
-
 static QueryResult RemoveCompat(NsswitchConf *conf, PSTR *description, LWException **exc)
 {
-    DistroInfo distro;
+    LwDistroInfo distro;
     int compatLine;
     int noncompatLine;
     int compatModIndex;
@@ -739,25 +728,13 @@ DWORD
 UpdateNsswitchConf(NsswitchConf *conf, BOOLEAN enable)
 {
     DWORD ceError = ERROR_SUCCESS;
-    DistroInfo distro;
+    LwDistroInfo distro;
     int line;
     int lwiIndex;
-    BOOLEAN usingLsass;
-    PCSTR preferredModule;
-    PCSTR oldModule;
+    static const char* moduleName = "lsass";
+    static const char* oldModule = "lwidentity";
 
     GCE(ceError = DJGetDistroInfo(NULL, &distro));
-    GCE(ceError = UsingLsass(&usingLsass));
-    if(usingLsass)
-    {
-        preferredModule = "lsass";
-        oldModule = "lwidentity";
-    }
-    else
-    {
-        preferredModule = "lwidentity";
-        oldModule = "lsass";
-    }
 
     line = FindEntry(conf, 0, "passwd");
     if(enable && line == -1)
@@ -766,10 +743,10 @@ UpdateNsswitchConf(NsswitchConf *conf, BOOLEAN enable)
         GCE(ceError = AddEntry(conf, &distro, &line, "passwd"));
         GCE(ceError = InsertModule(conf, &distro, line, -1, "files"));
     }
-    lwiIndex = FindModuleOnLine(conf, line, preferredModule);
+    lwiIndex = FindModuleOnLine(conf, line, moduleName);
     if(enable && lwiIndex == -1)
     {
-        GCE(ceError = InsertModule(conf, &distro, line, -1, preferredModule));
+        GCE(ceError = InsertModule(conf, &distro, line, -1, moduleName));
     }
     if(!enable && lwiIndex != -1)
     {
@@ -808,10 +785,10 @@ UpdateNsswitchConf(NsswitchConf *conf, BOOLEAN enable)
         GCE(ceError = AddEntry(conf, &distro, &line, groupName));
         GCE(ceError = InsertModule(conf, &distro, line, -1, "files"));
     }
-    lwiIndex = FindModuleOnLine(conf, line, preferredModule);
+    lwiIndex = FindModuleOnLine(conf, line, moduleName);
     if(enable && lwiIndex == -1)
     {
-        GCE(ceError = InsertModule(conf, &distro, line, -1, preferredModule));
+        GCE(ceError = InsertModule(conf, &distro, line, -1, moduleName));
     }
     if(!enable && lwiIndex != -1)
     {
@@ -868,7 +845,7 @@ cleanup:
     return ceError;
 }
 
-const char *GetNameOfHostsByFile(const NsswitchConf *conf, const DistroInfo *distro)
+const char *GetNameOfHostsByFile(const NsswitchConf *conf, const LwDistroInfo *distro)
 {
     int line = FindEntry(conf, 0, "hosts");
 
@@ -884,7 +861,7 @@ const char *GetNameOfHostsByFile(const NsswitchConf *conf, const DistroInfo *dis
         return "files";
 }
 
-const char *GetNameOfHostsByDns(const NsswitchConf *conf, const DistroInfo *distro)
+const char *GetNameOfHostsByDns(const NsswitchConf *conf, const LwDistroInfo *distro)
 {
     int line = FindEntry(conf, 0, "hosts");
 
@@ -907,7 +884,7 @@ DJConfigureHostsEntry(const char *testPrefix)
 {
     DWORD ceError = ERROR_SUCCESS;
     NsswitchConf conf;
-    DistroInfo distro;
+    LwDistroInfo distro;
     int line;
     const char *hostsByFile;
     const char *hostsByDns;
@@ -959,29 +936,6 @@ cleanup:
     return ceError;
 }
 
-DWORD
-ConfigureNameServiceSwitch()
-{
-    DWORD ceError = ERROR_SUCCESS;
-    DistroInfo distro;
-    GCE(ceError = DJGetDistroInfo(NULL, &distro));
-    GCE(ceError = DJConfigureNameServiceSwitch(NULL, TRUE));
-
-    /* By default, AIX will check a bind server before it will check its
-     * hosts file after checking DNS. This means that our FQDN setting code
-     * won't necessarily take effect. So instead we rewrite the line to put
-     * local in front.
-     */
-    if(distro.os == OS_AIX)
-    {
-        GCE(ceError = DJConfigureHostsEntry(NULL));
-    }
-
-cleanup:
-    DJFreeDistroInfo(&distro);
-    return ceError;
-}
-
 #define APPARMOR_NSSWITCH "/etc/apparmor.d/abstractions/nameservice"
 
 static DWORD
@@ -1114,12 +1068,12 @@ cleanup:
     CT_SAFE_FREE_STRING(finalName);
 }
 
-DWORD UnsupportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
+static DWORD UnsupportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
 {
     BOOLEAN hasSeLinux;
     DWORD ceError = ERROR_SUCCESS;
     PSTR output = NULL;
-    DistroInfo distro;
+    LwDistroInfo distro;
 
     *hasBadSeLinux = FALSE;
     memset(&distro, 0, sizeof(distro));
@@ -1192,9 +1146,18 @@ static QueryResult QueryNsswitch(const JoinProcessOptions *options, LWException
     BOOLEAN hasBadSeLinux;
     NsswitchConf conf;
     DWORD ceError = ERROR_SUCCESS;
+    uid_t uid = 0;
+    gid_t gid = 0;
+    mode_t mode = 0;
 
     memset(&conf, 0, sizeof(conf));
 
+    if (options->enableMultipleJoins)
+    {
+        result = NotApplicable;
+        goto cleanup;
+    }
+
     if (options->joiningDomain)
     {
         ceError = ReadNsswitchConf(&conf, NULL, FALSE);
@@ -1205,13 +1168,23 @@ static QueryResult QueryNsswitch(const JoinProcessOptions *options, LWException
             goto cleanup;
         }
         LW_CLEANUP_CTERR(exc, ceError);
-
+        
         LW_TRY(exc, result = RemoveCompat(&conf, NULL, &LW_EXC));
         if(result == CannotConfigure || result == NotConfigured)
         {
             goto cleanup;
         }
 
+        LW_CLEANUP_CTERR(exc, CTGetOwnerAndPermissions(
+            conf.filename, &uid, &gid, &mode));
+
+        if ((mode & 0444) != 0444)
+        {
+            // The user has to fix the permissions
+            result = CannotConfigure;
+            goto cleanup;
+        }
+
         LW_CLEANUP_CTERR(exc, UpdateNsswitchConf(&conf, TRUE));
         if(conf.modified)
         {
@@ -1222,7 +1195,7 @@ static QueryResult QueryNsswitch(const JoinProcessOptions *options, LWException
                 result = NotConfigured;
             goto cleanup;
         }
-
+        
         LW_CLEANUP_CTERR(exc, DJHasMethodsCfg(&exists));
 
         if(exists)
@@ -1272,7 +1245,7 @@ static void RestartDtloginIfRunning(JoinProcessOptions *options, LWException **e
     BOOLEAN doRestart;
     BOOLEAN inX;
     LWException *inner = NULL;
-    DistroInfo distro;
+    LwDistroInfo distro;
 
     memset(&distro, 0, sizeof(distro));
 
@@ -1349,7 +1322,11 @@ cleanup:
     DJFreeDistroInfo(&distro);
 }
 
-static void DoNsswitch(JoinProcessOptions *options, LWException **exc)
+void
+DoNsswitch(
+    JoinProcessOptions *options,
+    LWException **exc
+    )
 {
     LWException *restartException = NULL;
     NsswitchConf conf;
@@ -1402,7 +1379,7 @@ static void DoNsswitch(JoinProcessOptions *options, LWException **exc)
 
     LW_TRY(exc, RestartDtloginIfRunning(options, &LW_EXC));
 
-    if (options->warningCallback != NULL)
+    if (options->joiningDomain && options->warningCallback != NULL)
     {
         options->warningCallback(options,
                                  "System restart required",
@@ -1459,6 +1436,22 @@ static PSTR GetNsswitchDescription(const JoinProcessOptions *options, LWExceptio
         LW_CLEANUP_CTERR(exc, CTStrdup("", &compatDescription));
     }
 
+    if (options->joiningDomain)
+    {
+        uid_t uid = 0;
+        gid_t gid = 0;
+        mode_t mode = 0;
+        LW_CLEANUP_CTERR(exc, CTGetOwnerAndPermissions(
+            conf.filename, &uid, &gid, &mode));
+
+        if ((mode & 0444) != 0444)
+        {
+            LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&ret,
+"The permissions of 0%03o on %s are invalid. All users must have at least read permission for the file. You can fix this by running 'chmod a+r %s'.", (int)(mode&0777), conf.filename, conf.filename));
+            goto cleanup;
+        }
+    }
+
     conf.modified = FALSE;
     LW_CLEANUP_CTERR(exc, UpdateNsswitchConf(&conf, options->joiningDomain));
 
