@@ -43,7 +43,7 @@ FreeProcInfo(
     CTFreeProcInfo(pProcInfo);
 }
 
-CENTERROR
+DWORD
 DJSpawnProcessWithEnvironment(
     PCSTR pszCommand,
     const PSTR* ppszArgs,
@@ -65,7 +65,7 @@ DJSpawnProcessWithEnvironment(
         );
 }
 
-CENTERROR
+DWORD
 DJSpawnProcess(
     PCSTR pszCommand,
     const PSTR* ppszArgs,
@@ -75,138 +75,7 @@ DJSpawnProcess(
     return DJSpawnProcessWithEnvironment(pszCommand, ppszArgs, NULL, -1, -1, -1, ppProcInfo);
 }
 
-#ifdef NOT_YET
-CENTERROR
-DJSpawnProcessCaptureOutput(
-    PCSTR pszCommand,
-    PSTR* ppszArgs,
-    PPROCINFO* ppProcInfo,
-    PSTR* ppszStdout,
-    PSTR* ppszStderr)
-{
-    CENTERROR ceError = CENTERROR_SUCCESS;
-    BOOLEAN bFirst = FALSE;
-    PPROCINFO pProcInfo = NULL;
-    PROCBUFFER procBuffer;
-    DWORD iOutBufIdx = 0;
-    DWORD dwOutBufLen = 0;
-    DWORD dwOutBufAvailable = 0;
-    PSTR pszOutBuffer = NULL;
-    DWORD iErrBufIdx = 0;
-    DWORD dwErrBufLen = 0;
-    DWORD dwErrBufAvailable = 0;
-    PSTR pszErrBuffer = NULL;
-
-    ceError = DJSpawnProcess(pszCommand, ppszArgs, ppProcInfo);
-    BAIL_ON_CENTERIS_ERROR(ceError);
-
-    pProcInfo = *ppProcInfo;
-
-    do {
-
-        if (!bFirst) {
-            ceError = DJReadData(pProcInfo, &procBuffer);
-            BAIL_ON_CENTERIS_ERROR(ceError);
-        }
-
-        bFirst = FALSE;
-
-        if (procBuffer.dwOutBytesRead) {
-
-            while (1) {
-
-                if (procBuffer.dwOutBytesRead < dwOutBufAvailable) {
-
-                    memcpy(pszOutBuffer+iOutBufIdx,
-                           procBuffer.szOutBuf,
-                           procBuffer.dwOutBytesRead
-                        );
-
-
-                    iOutBufIdx+= procBuffer.dwOutBytesRead;
-                    dwOutBufAvailable -= procBuffer.dwOutBytesRead;
-
-                    *(pszOutBuffer+iOutBufIdx+1) = '\0';
-
-                    break;
-
-                } else {
-
-                    /*
-                     * TODO: Limit the amount of memory acquired
-                     */
-
-                    ceError = CTReallocMemory(pszOutBuffer,
-                                              (PVOID*)&pszOutBuffer,
-                                              dwOutBufLen+1024);
-                    BAIL_ON_CENTERIS_ERROR(ceError);
-
-                    dwOutBufLen += 1024;
-                    dwOutBufAvailable += 1024;
-                }
-            }
-        }
-
-        if (procBuffer.dwErrBytesRead) {
-
-            while (1) {
-
-                if (procBuffer.dwErrBytesRead < dwErrBufAvailable) {
-
-                    memcpy(pszErrBuffer+iErrBufIdx,
-                           procBuffer.szErrBuf,
-                           procBuffer.dwErrBytesRead
-                        );
-
-                    iErrBufIdx+= procBuffer.dwErrBytesRead;
-                    dwErrBufAvailable -= procBuffer.dwErrBytesRead;
-
-                    *(pszErrBuffer+iErrBufIdx+1) = '\0';
-
-                    break;
-
-                } else {
-
-                    /*
-                     * TODO: Limit the amount of memory acquired
-                     */
-                    ceError = CTReallocMemory(pszErrBuffer,
-                                              (PVOID*)&pszErrBuffer,
-                                              dwErrBufLen+1024);
-                    BAIL_ON_CENTERIS_ERROR(ceError);
-
-                    dwErrBufLen += 1024;
-                    dwErrBufAvailable += 1024;
-                }
-            }
-        }
-
-    } while (!procBuffer.bEndOfFile);
-
-    ceError = CTAllocateString(pszOutBuffer, ppszStdout);
-    BAIL_ON_CENTERIS_ERROR(ceError);
-
-    CT_SAFE_FREE_STRING(pszOutBuffer);
-
-    ceError = CTAllocateString(pszErrBuffer, ppszStderr);
-    BAIL_ON_CENTERIS_ERROR(ceError);
-
-    CT_SAFE_FREE_STRING(pszErrBuffer);
-
-error:
-
-    if (pszOutBuffer) {
-       *ppszStdout = pszOutBuffer;
-    }
-    if (pszErrBuffer) {
-       *ppszStderr = pszErrBuffer;
-    }
-
-    return ceError;
-}
-#endif /* NOT_YET */
-
-CENTERROR
+DWORD
 DJSpawnProcessWithFds(
     PCSTR pszCommand,
     PSTR* ppszArgs,
@@ -219,7 +88,7 @@ DJSpawnProcessWithFds(
     return CTSpawnProcessWithFds(pszCommand, ppszArgs, dwFdIn, dwFdOut, dwFdErr, ppProcInfo);
 }
 
-CENTERROR
+DWORD
 DJSpawnProcessSilent(
     PCSTR pszCommand,
     PSTR* ppArgs,
@@ -227,13 +96,13 @@ DJSpawnProcessSilent(
     )
 {
     int dwFdIn = -1, dwFdOut = -1, dwFdErr = -1;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
 
     dwFdIn = open("/dev/zero", O_RDONLY);
 
     if (dwFdIn < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -241,7 +110,7 @@ DJSpawnProcessSilent(
 
     if (dwFdOut < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -249,7 +118,7 @@ DJSpawnProcessSilent(
 
     if (dwFdErr < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -268,7 +137,7 @@ error:
     return ceError;
 }
 
-CENTERROR
+DWORD
 DJSpawnProcessOutputToFile(
     PCSTR pszCommand,
     PSTR* ppArgs,
@@ -277,13 +146,13 @@ DJSpawnProcessOutputToFile(
     )
 {
     int dwFdIn = -1, dwFdOut = -1, dwFdErr = -1;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
 
     dwFdIn = open("/dev/zero", O_RDONLY);
 
     if (dwFdIn < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -291,7 +160,7 @@ DJSpawnProcessOutputToFile(
 
     if (dwFdOut < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -299,7 +168,7 @@ DJSpawnProcessOutputToFile(
 
     if (dwFdErr < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -318,7 +187,7 @@ error:
     return ceError;
 }
 
-CENTERROR
+DWORD
 DJTimedReadData(
     PPROCINFO pProcInfo,
     PPROCBUFFER pProcBuffer,
@@ -326,7 +195,7 @@ DJTimedReadData(
     PBOOLEAN pbTimedout
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     PSTR pszBuf = NULL;
     DWORD dwBytesRead = 0;
     int maxfd;
@@ -369,7 +238,7 @@ DJTimedReadData(
             if (errno == EINTR)
                 continue;
 
-            ceError = CTMapSystemError(errno);
+            ceError = LwMapErrnoToLwError(errno);
             BAIL_ON_CENTERIS_ERROR(ceError);
 
         } else if (select_status == 0) {
@@ -393,7 +262,7 @@ DJTimedReadData(
                     if (dwBytesRead < 0) {
 
                         if (errno != EAGAIN && errno != EINTR) {
-                            ceError = CTMapSystemError(errno);
+                            ceError = LwMapErrnoToLwError(errno);
                             BAIL_ON_CENTERIS_ERROR(ceError);
                         }
 
@@ -420,7 +289,7 @@ error:
     return (ceError);
 }
 
-CENTERROR
+DWORD
 DJReadData(
     PPROCINFO pProcInfo,
     PPROCBUFFER pProcBuffer
@@ -430,14 +299,14 @@ DJReadData(
     return DJTimedReadData(pProcInfo, pProcBuffer, 5, &bTimedout);
 }
 
-CENTERROR
+DWORD
 DJWriteData(
     DWORD dwFd,
     PSTR pszBuf,
     DWORD dwLen
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     DWORD nWritten = 0;
     DWORD dwRemaining = 0;
     PSTR pStr;
@@ -451,7 +320,7 @@ DJWriteData(
         {
             if (errno != EAGAIN && errno != EINTR)
             {
-                ceError = CTMapSystemError(errno);
+                ceError = LwMapErrnoToLwError(errno);
                 BAIL_ON_CENTERIS_ERROR(ceError);
             }
         }
@@ -467,20 +336,20 @@ error:
     return (ceError);
 }
 
-CENTERROR
+DWORD
 DJGetProcessStatus(
     PPROCINFO pProcInfo,
     PLONG plstatus
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     int status = 0;
 
     do {
         if (waitpid(pProcInfo->pid, &status, 0) < 0) {
             if (errno == EINTR)
                 continue;
-            ceError = CTMapSystemError(errno);
+            ceError = LwMapErrnoToLwError(errno);
             BAIL_ON_CENTERIS_ERROR(ceError);
         }
 
@@ -500,12 +369,12 @@ error:
     return ceError;
 }
 
-CENTERROR
+DWORD
 DJKillProcess(
     PPROCINFO pProcInfo
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     DWORD dwFlag = 0;
     DWORD dwTimeout = 30;
     int status = 0;
@@ -520,18 +389,18 @@ DJKillProcess(
 #ifdef HAVE_SIGPROCMASK
     if(sigemptyset(&newset) < 0 || sigaddset(&newset, SIGALRM) < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
     if(sigprocmask(SIG_BLOCK, &newset, &oldset) < 0)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 #else
     if((long)(oldhandler = sigset(SIGALRM, WaitTimeout)) == -1)
     {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 #endif
@@ -562,14 +431,14 @@ error:
     if(oldhandlerset)
     {
 #ifdef HAVE_SIGPROCMASK
-        if(sigprocmask(SIG_SETMASK, &oldset, NULL) < 0 && CENTERROR_IS_OK(ceError))
+        if(sigprocmask(SIG_SETMASK, &oldset, NULL) < 0 && !ceError)
         {
-            ceError = CTMapSystemError(errno);
+            ceError = LwMapErrnoToLwError(errno);
         }
 #else
-        if(sigset(SIGALRM, oldhandler) == -1 && CENTERROR_IS_OK(ceError))
+        if(sigset(SIGALRM, oldhandler) == -1 && !ceError)
         {
-            ceError = CTMapSystemError(errno);
+            ceError = LwMapErrnoToLwError(errno);
         }
 #endif
     }
