@@ -48,7 +48,7 @@ static PCSTR ubuntuSedExpr = "s/^\\([ \t]*\\)PROMPT_COMMAND=\'echo -ne \"\\\\033
 
 static QueryResult QueryBash(const JoinProcessOptions *options, LWException **exc)
 {
-    DistroInfo distro;
+    LwDistroInfo distro;
     BOOLEAN exists;
     BOOLEAN changes;
     QueryResult result = SufficientlyConfigured;
@@ -118,12 +118,17 @@ static DWORD WritePromptFile(PCSTR filename, PCSTR contents)
     GCE(CTFileStreamWrite(file, contents, strlen(contents)));
 
 cleanup:
+    if (file != NULL)
+    {
+        CTCloseFile(file);
+        file = NULL;
+    }
     return ceError;
 }
 
 static void DoBash(JoinProcessOptions *options, LWException **exc)
 {
-    DistroInfo distro;
+    LwDistroInfo distro;
     BOOLEAN exists;
 
     memset(&distro, 0, sizeof(distro));
@@ -181,7 +186,7 @@ cleanup:
 
 static PSTR GetBashDescription(const JoinProcessOptions *options, LWException **exc)
 {
-    DistroInfo distro;
+    LwDistroInfo distro;
     QueryResult currentStatus;
     PSTR result = NULL;
 
