@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -38,15 +38,64 @@ typedef enum
     NSPSearchPath
 } SearchPolicyType;
 
-#define LWIDSPLUGIN_SYMLINK_PATH "/System/Library/Frameworks/DirectoryService.framework/Versions/Current/Resources/Plugins/LWIDSPlugin.dsplug"
-#define LWEDSPLUGIN_SYMLINK_PATH "/System/Library/Frameworks/DirectoryService.framework/Versions/Current/Resources/Plugins/LWEDSPlugin.dsplug"
-#define LWIDSPLUGIN_INSTALL_PATH LIBDIR "/LWIDSPlugin.dsplug"
-#define LWEDSPLUGIN_INSTALL_PATH LIBDIR "/LWEDSPlugin.dsplug"
 #define LWDSPLUGIN_NAME         "/Likewise - Active Directory"
 #define APPLEADDSPLUGIN_NAME    "^Active Directory"
 #define PID_FILE_CONTENTS_SIZE   ((9 * 2) + 2)
 #define CONFIGD_PID_FILE         "/var/run/configd.pid"
 
+#if 0
+static
+DWORD
+DJGetConfigDPID(
+    pid_t* ppid
+    )
+{
+    DWORD ceError = ERROR_SUCCESS;
+    BOOLEAN bFileExists = FALSE;
+    char contents[PID_FILE_CONTENTS_SIZE];
+    int  fd = -1;
+    int  len = 0;
+
+    ceError = CTCheckFileExists(CONFIGD_PID_FILE, &bFileExists);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
+    if (!bFileExists)
+    {
+       *ppid = 0;
+       goto error;
+    }
+
+    fd = open(CONFIGD_PID_FILE, O_RDONLY, 0644);
+    if (fd < 0)
+    {
+       ceError = LwMapErrnoToLwError(errno);
+       BAIL_ON_CENTERIS_ERROR(ceError);
+    }
+
+    if ((len = read(fd, contents, sizeof(contents)-1)) < 0)
+    {
+       ceError = LwMapErrnoToLwError(errno);
+       BAIL_ON_CENTERIS_ERROR(ceError);
+    }
+    else if (len == 0)
+    {
+       *ppid = 0;
+       goto error;
+    }
+    contents[len-1] = 0;
+
+    *ppid = atoi(contents);
+
+error:
+
+    if (fd >= 0)
+    {
+       close(fd);
+    }
+
+    return ceError;
+}
+#endif /* 0 */
 
 static
 DWORD
@@ -62,7 +111,7 @@ DJSetSearchPath(
 
     DJ_LOG_INFO("Setting search policy to %s", searchPolicyType == CSPSearchPath ? "Custom path" : "Automatic");
 
-    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)&ppszArgs);
+    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)(PVOID)&ppszArgs);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = CTAllocateString("/usr/bin/dscl", ppszArgs);
@@ -125,10 +174,10 @@ error:
     return ceError;
 }
 
-/* Use dscl to place the LWIDSPlugin in the authenticator list */
+/* Use dscl to place the DSPlugin in the authenticator list */
 static
 DWORD
-DJRegisterLWIDSPlugin()
+DJRegisterDSPlugin()
 {
     DWORD ceError = ERROR_SUCCESS;
     PPROCINFO pProcInfo = NULL;
@@ -137,12 +186,12 @@ DJRegisterLWIDSPlugin()
     LONG status = 0;
     DWORD retryCount = 3;
 
-    DJ_LOG_INFO("Registering LWIDSPlugin for Macintosh Directory Services Authentication");
+    DJ_LOG_INFO("Registering DSPlugin for Macintosh Directory Services Authentication");
 
     ceError = DJSetSearchPath(CSPSearchPath);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
-    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)&ppszArgs);
+    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)(PVOID)&ppszArgs);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = CTAllocateString("/usr/bin/dscl", ppszArgs);
@@ -205,10 +254,10 @@ error:
     return ceError;
 }
 
-/* Remove LWIDSPlugin from the authenticator list */
+/* Remove DSPlugin from the authenticator list */
 static
 DWORD
-DJUnregisterLWIDSPlugin()
+DJUnregisterDSPlugin()
 {
     DWORD ceError = ERROR_SUCCESS;
     PPROCINFO pProcInfo = NULL;
@@ -216,9 +265,9 @@ DJUnregisterLWIDSPlugin()
     DWORD nArgs = 7;
     LONG status = 0;
 
-    DJ_LOG_INFO("Unregistering LWIDSPlugin from Open Directory Authentication");
+    DJ_LOG_INFO("Unregistering DSPlugin from Open Directory Authentication");
 
-    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)&ppszArgs);
+    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)(PVOID)&ppszArgs);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = CTAllocateString("/usr/bin/dscl", ppszArgs);
@@ -315,12 +364,13 @@ error:
     return ceError;
 }
 
+
 DWORD
-DJConfigureLWIDSPlugin()
+DJConfigureDSPlugin()
 {
     DWORD ceError = ERROR_SUCCESS;
 
-    ceError = DJRegisterLWIDSPlugin();
+    ceError = DJRegisterDSPlugin();
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = DJFlushCache();
@@ -332,11 +382,11 @@ error:
 }
 
 DWORD
-DJUnconfigureLWIDSPlugin()
+DJUnconfigureDSPlugin()
 {
     DWORD ceError = ERROR_SUCCESS;
 
-    ceError = DJUnregisterLWIDSPlugin();
+    ceError = DJUnregisterDSPlugin();
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = DJFlushCache();
@@ -359,7 +409,7 @@ DJIsAppleADPluginInUse(BOOLEAN* pExists)
 
     DJ_LOG_INFO("Testing to see if Apple AD plugin is already in use");
 
-    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)&ppszArgs);
+    ceError = CTAllocateMemory(sizeof(PSTR)*nArgs, (PVOID*)(PVOID)&ppszArgs);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ceError = CTAllocateString("/usr/bin/dscl", ppszArgs);
@@ -416,6 +466,12 @@ static QueryResult QueryDSPlugin(const JoinProcessOptions *options, LWException
     PSTR valueStart = NULL;
     BOOLEAN bLikewisePresent = FALSE;
 
+    if (options->enableMultipleJoins)
+    {
+        result = NotApplicable;
+        goto cleanup;
+    }
+
     LW_CLEANUP_CTERR(exc, CTCheckFileOrLinkExists("/usr/bin/dscl", &exists));
     if(!exists)
     {
@@ -486,9 +542,9 @@ cleanup:
 static void DoDSPlugin(JoinProcessOptions *options, LWException **exc)
 {
     if(options->joiningDomain)
-        LW_CLEANUP_CTERR(exc, DJConfigureLWIDSPlugin());
+        LW_CLEANUP_CTERR(exc, DJConfigureDSPlugin());
     else
-        LW_CLEANUP_CTERR(exc, DJUnconfigureLWIDSPlugin());
+        LW_CLEANUP_CTERR(exc, DJUnconfigureDSPlugin());
 cleanup:
     ;
 }
