@@ -39,9 +39,6 @@
  * Eventlog Service (Process Utilities)
  *
  */
-#ifndef _POSIX_PTHREAD_SEMANTICS
-#define _POSIX_PTHREAD_SEMANTICS 1
-#endif
 
 #include "includes.h"
 
@@ -64,6 +61,7 @@ EVTSERVERINFO gServerInfo =
     0,                          /* Remove records older than*/
     0,                          /* Purge records at interval*/
     0,                          /* Enable/disable Remove records a boolean value TRUE or FALSE*/
+    TRUE,                       /* Register TCP/IP RPC endpoints*/
     NULL,
 };
 
@@ -87,6 +85,12 @@ EVTGetProcessExitCode(
     PDWORD pdwExitCode
     );
 
+static
+DWORD
+EVTGetRegisterTcpIp(
+    PBOOLEAN pbRegisterTcpIp
+    );
+
 static
 void
 EVTExitHandler(
@@ -408,6 +412,23 @@ EVTGetRemoveAsNeeded(
     return (dwError);
 }
 
+static
+DWORD
+EVTGetRegisterTcpIp(
+    PBOOLEAN pbRegisterTcpIp
+    )
+{
+    DWORD dwError = 0;
+
+    EVT_LOCK_SERVERINFO;
+
+    *pbRegisterTcpIp = gServerInfo.bRegisterTcpIp;
+
+    EVT_UNLOCK_SERVERINFO;
+
+    return (dwError);
+}
+
 DWORD
 EVTGetPrefixPath(
     PSTR* ppszPath
@@ -698,102 +719,6 @@ EVTStartAsDaemon()
     return (dwError);
 }
 
-#define DAEMON_NAME "eventlogd"
-#define PID_DIR "/var/run"
-#define PID_FILE PID_DIR "/" DAEMON_NAME ".pid"
-
-#define PID_FILE_CONTENTS_SIZE ((9 * 2) + 2)
-
-static
-pid_t
-pid_from_pid_file()
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(PID_FILE, O_RDONLY, 0644);
-    if (fd < 0) {
-        goto error;
-    }
-
-    result = read(fd, contents, sizeof(contents)-1);
-    if (result <= 0) {
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result <= 0) {
-        result = -1;
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH) {
-        unlink(PID_FILE);
-        pid = 0;
-    }
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    return pid;
-}
-
-static
-void
-EVTCreatePIDFile()
-{
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-
-    pid = pid_from_pid_file();
-    if (pid > 0) {
-        fprintf(stderr, "Daemon already running as %d\n", (int) pid);
-        result = -1;
-        goto error;
-    }
-
-    fd = open(PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0) {
-        fprintf(stderr, "Could not create pid file: %s\n", strerror(errno));
-        result = 1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) write(fd, contents, len);
-    if ( result != (int) len ) {
-        fprintf(stderr, "Could not write to pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    if (result < 0) {
-        exit(1);
-    }
-}
-
-
 static
 DWORD
 EVTSetConfigDefaults()
@@ -933,6 +858,15 @@ static EVT_CONFIG_TABLE gConfigDescription[] =
         NULL,
         &(gServerInfo.bRemoveAsNeeded)
     },
+    {
+        "RegisterTcpIp",
+        TRUE,
+        EVTTypeBoolean,
+        0,
+        -1,
+        NULL,
+        &(gServerInfo.bRegisterTcpIp)
+    },
     {
         "AllowReadTo",
         TRUE,
@@ -973,17 +907,19 @@ EVTLogConfigReload(
     dwError = EVTAllocateStringPrintf(
                  &pszDescription,
                  "     Current config settings are...\r\n" \
-                 "     Max Disk Usage :           %d\r\n" \
-                 "     Max Number Of Events:      %d\r\n" \
-                 "     Max Event Lifespan:        %d\r\n" \
-                 "     Remove Events As Needed:   %s\r\n" \
-                 "     Allow Read   To :          %s\r\n" \
-                 "     Allow Write  To :          %s\r\n" \
-                 "     Allow Delete To :          %s\r\n",
+                 "     Max Disk Usage :                 %d\r\n" \
+                 "     Max Number Of Events:            %d\r\n" \
+                 "     Max Event Lifespan:              %d\r\n" \
+                 "     Remove Events As Needed:         %s\r\n" \
+                 "     Register TCP/IP RPC endpoints:   %s\r\n" \
+                 "     Allow Read   To :                %s\r\n" \
+                 "     Allow Write  To :                %s\r\n" \
+                 "     Allow Delete To :                %s\r\n",
                  gServerInfo.dwMaxLogSize,
                  gServerInfo.dwMaxRecords,
                  gServerInfo.dwMaxAge,
                  gServerInfo.bRemoveAsNeeded? "true" : "false",
+                 gServerInfo.bRegisterTcpIp ? "true" : "false",
                  gServerInfo.pszAllowReadTo ?
                     gServerInfo.pszAllowReadTo: "",
                  gServerInfo.pszAllowWriteTo ?
@@ -1283,14 +1219,14 @@ EVTCreateAccessDescriptor(
     PSID pAdministrators = NULL;
     BOOLEAN bFullyResolved = TRUE;
 
-    dwError = LwCreateWellKnownSid(
+    dwError = LwAllocateWellKnownSid(
                     WinLocalSystemSid,
                     NULL,
                     &pLocalSystem,
                     NULL);
     BAIL_ON_EVT_ERROR(dwError);
 
-    dwError = LwCreateWellKnownSid(
+    dwError = LwAllocateWellKnownSid(
                     WinBuiltinAdministratorsSid,
                     NULL,
                     &pAdministrators,
@@ -1351,7 +1287,7 @@ EVTCreateAccessDescriptor(
                 &ppWriteList);
     BAIL_ON_EVT_ERROR(dwError);
 
-    if (dwReadCount < dwCount)
+    if (dwWriteCount < dwCount)
     {
         bFullyResolved = FALSE;
     }
@@ -1375,7 +1311,7 @@ EVTCreateAccessDescriptor(
                 &ppDeleteList);
     BAIL_ON_EVT_ERROR(dwError);
 
-    if (dwReadCount < dwCount)
+    if (dwDeleteCount < dwCount)
     {
         bFullyResolved = FALSE;
     }
@@ -1663,14 +1599,14 @@ EVTNetworkThread(
     };
     struct timespec delay = {5, 0};
     BOOLEAN *pbExitNow = (BOOLEAN *)pArg;
-
+ 
     while (endpoints[index].protocol && !*pbExitNow)
     {
         dwError = EVTRegisterEndpoint(
             "Likewise Eventlog Service",
             &endpoints[index]
             );
-
+        
         if (dwError)
         {
             dwError = 0;
@@ -1690,7 +1626,7 @@ EVTNetworkThread(
                                 endpoints[index].protocol,
                                 endpoints[index].endpoint);
             }
-
+                                
             index++;
         }
     }
@@ -1717,6 +1653,7 @@ main(
         {NULL, NULL}
     };
     BOOLEAN bExitNow = FALSE;
+    BOOLEAN bRegisterTcpIp = TRUE;
 
     dwError = EVTSetServerDefaults();
     BAIL_ON_EVT_ERROR(dwError);
@@ -1750,8 +1687,6 @@ main(
         BAIL_ON_EVT_ERROR(dwError);
     }
 
-    EVTCreatePIDFile();
-
     dwError = EVTInitLogging(get_program_name(argv[0]));
     BAIL_ON_EVT_ERROR(dwError);
 
@@ -1764,6 +1699,9 @@ main(
         dwError = 0;
     }
 
+    dwError = EVTGetRegisterTcpIp(&bRegisterTcpIp);
+    BAIL_ON_EVT_ERROR(dwError);
+
     dwError = SrvInitEventDatabase();
     BAIL_ON_EVT_ERROR(dwError);
 
@@ -1797,12 +1735,15 @@ main(
                                       NULL));
     BAIL_ON_EVT_ERROR(dwError);
 
-    dwError = LwMapErrnoToLwError(dcethread_create(
+    if (bRegisterTcpIp)
+    {
+        dwError = LwMapErrnoToLwError(dcethread_create(
                                       &networkThread,
                                       NULL,
                                       EVTNetworkThread,
                                       &bExitNow));
-    BAIL_ON_EVT_ERROR(dwError);
+        BAIL_ON_EVT_ERROR(dwError);
+    }
 
     while (!EVTIsListening())
     {
@@ -1811,7 +1752,7 @@ main(
     if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notifyFd = atoi(pszSmNotify);
-
+        
         do
         {
             ret = write(notifyFd, &notifyCode, sizeof(notifyCode));
@@ -1839,15 +1780,21 @@ main(
 
     dwError = EVTStopListen();
     BAIL_ON_EVT_ERROR(dwError);
-
-    dwError = LwMapErrnoToLwError(dcethread_interrupt(networkThread));
-    BAIL_ON_EVT_ERROR(dwError);
+    
+    if (bRegisterTcpIp)
+    {
+        dwError = LwMapErrnoToLwError(dcethread_interrupt(networkThread));
+        BAIL_ON_EVT_ERROR(dwError);
+    }
 
     dwError = LwMapErrnoToLwError(dcethread_join(listenThread, NULL));
     BAIL_ON_EVT_ERROR(dwError);
 
-    dwError = LwMapErrnoToLwError(dcethread_join(networkThread, NULL));
-    BAIL_ON_EVT_ERROR(dwError);
+    if (bRegisterTcpIp)
+    {
+        dwError = LwMapErrnoToLwError(dcethread_join(networkThread, NULL));
+        BAIL_ON_EVT_ERROR(dwError);
+    }
 
  cleanup:
 
