@@ -179,7 +179,7 @@ idl_long_int
 RpcLWIWriteEventLogRecords(
     handle_t bindingHandle,
     unsigned32 cRecords,
-    EVENT_LOG_RECORD* pEventRecords
+    EVENT_LOG_RECORD* pEventRecords 
     )
 {
     DWORD  dwError = 0;
@@ -395,7 +395,7 @@ bind_server(
 
     if (!pEndPoint->endpoint)
     {
-        rpc_server_use_protseq((unsigned char *)pEndPoint->protocol,
+        rpc_server_use_protseq((unsigned char *)pEndPoint->protocol, 
                                rpc_c_protseq_max_calls_default,
                                (unsigned32*)&dwRpcStatus);
         BAIL_ON_DCE_ERROR(dwError, dwRpcStatus);
@@ -408,7 +408,7 @@ bind_server(
             dwError = prepare_domain_socket(pEndPoint->endpoint);
             BAIL_ON_EVT_ERROR(dwError);
         }
-
+        
         rpc_server_use_protseq_ep((unsigned char *)pEndPoint->protocol,
                                   rpc_c_protseq_max_calls_default,
                                   (unsigned char *)pEndPoint->endpoint,
@@ -444,7 +444,7 @@ EVTRegisterInterface(
         dwRpcStatus = dcethread_exc_getstatus (THIS_CATCH);
     }
     ENDTRY;
-
+    
     BAIL_ON_DCE_ERROR(dwError, dwRpcStatus);
     BAIL_ON_EVT_ERROR(dwError);
 
@@ -463,8 +463,6 @@ EVTRegisterEndpoint(
     volatile DWORD dwRpcStatus = 0;
     unsigned32 tmpStatus = 0;
     rpc_binding_vector_p_t pServerBinding = NULL;
-    BOOLEAN bEPRegistered = FALSE;
-    BOOLEAN bBound = TRUE;
 
     TRY
     {
@@ -481,8 +479,6 @@ EVTRegisterEndpoint(
     BAIL_ON_DCE_ERROR(dwError, dwRpcStatus);
     BAIL_ON_EVT_ERROR(dwError);
 
-    bBound = TRUE;
-
     TRY
     {
         rpc_ep_register(eventlog_v1_0_s_ifspec,
@@ -506,7 +502,6 @@ EVTRegisterEndpoint(
     BAIL_ON_DCE_ERROR(dwError, dwRpcStatus);
     BAIL_ON_EVT_ERROR(dwError);
 
-    bEPRegistered = TRUE;
     EVT_LOG_INFO("RPC Endpoint registered successfully.");
 
 cleanup:
