@@ -2412,7 +2412,7 @@ struct _krb5_cc_ops {
 						   krb5_ccache *);
     krb5_error_code (KRB5_CALLCONV *ptcursor_free)(krb5_context,
 						   krb5_cc_ptcursor *);
-    krb5_error_code (KRB5_CALLCONV *move)(krb5_context, krb5_ccache,
+    krb5_error_code (KRB5_CALLCONV *move)(krb5_context, krb5_ccache, 
 						krb5_ccache);
     krb5_error_code (KRB5_CALLCONV *lastchange)(krb5_context,
 						krb5_ccache, krb5_timestamp *);
@@ -2575,13 +2575,13 @@ krb5_error_code KRB5_CALLCONV krb5_random_confounder
 	(size_t, krb5_pointer);
 
 krb5_error_code krb5_encrypt_data
-	(krb5_context context, krb5_keyblock *key,
-		krb5_pointer ivec, krb5_data *data,
+	(krb5_context context, krb5_keyblock *key, 
+		krb5_pointer ivec, krb5_data *data, 
 		krb5_enc_data *enc_data);
 
 krb5_error_code krb5_decrypt_data
-	(krb5_context context, krb5_keyblock *key,
-		krb5_pointer ivec, krb5_enc_data *data,
+	(krb5_context context, krb5_keyblock *key, 
+		krb5_pointer ivec, krb5_enc_data *data, 
 		krb5_data *enc_data);
 
 krb5_error_code
@@ -2626,7 +2626,7 @@ typedef struct
         krb5_int32 etype_count;
 } krb5_etypes_permitted;
 
-krb5_boolean krb5_is_permitted_enctype_ext
+krb5_boolean krb5_is_permitted_enctype_ext 
          ( krb5_context, krb5_etypes_permitted *);
 
 krb5_boolean KRB5_CALLCONV krb5_c_weak_enctype(krb5_enctype);
@@ -2700,6 +2700,9 @@ krb5_error_code KRB5_CALLCONV krb5_kt_free_entry
 	(krb5_context,
 		krb5_keytab_entry * );
 
+krb5_error_code k5_kt_get_principal(krb5_context context, krb5_keytab keytab,
+                                    krb5_principal *princ_out);
+
 krb5_error_code krb5_principal2salt_norealm
 	(krb5_context,
 		krb5_const_principal, krb5_data *);
@@ -2875,7 +2878,7 @@ krb5_auth_con_set_authdata_context
 
 krb5_error_code KRB5_CALLCONV
 krb5int_server_decrypt_ticket_keyblock
-	(krb5_context context,
+  	(krb5_context context,
                 const krb5_keyblock *key,
                 krb5_ticket  *ticket);
 
@@ -2931,10 +2934,10 @@ void KRB5_CALLCONV krb5_free_realm_string
 
 /* Internal principal function used by KIM to avoid code duplication */
 krb5_error_code KRB5_CALLCONV
-krb5int_build_principal_alloc_va(krb5_context context,
-                                 krb5_principal *princ,
-                                 unsigned int rlen,
-                                 const char *realm,
+krb5int_build_principal_alloc_va(krb5_context context, 
+                                 krb5_principal *princ, 
+                                 unsigned int rlen, 
+                                 const char *realm, 
                                  const char *first,
                                  va_list ap);
 
