@@ -218,7 +218,7 @@ static struct gss_config spnego_mechanism =
 #ifndef LEAN_CLIENT
 	spnego_gss_accept_sec_context,
 #else
-	NULL,
+	NULL,				
 #endif  /* LEAN_CLIENT */
 	NULL,				/* gss_process_context_token */
 	spnego_gss_delete_sec_context,	/* gss_delete_sec_context */
@@ -249,11 +249,11 @@ static struct gss_config spnego_mechanism =
 	spnego_gss_wrap_size_limit,	/* gss_wrap_size_limit */
 	NULL,				/* gss_export_name */
 	NULL,				/* gss_store_cred */
-	spnego_gss_inquire_sec_context_by_oid, /* gss_inquire_sec_context_by_oid */
-	spnego_gss_inquire_cred_by_oid,	/* gss_inquire_cred_by_oid */
-	spnego_gss_set_sec_context_option, /* gss_set_sec_context_option */
-	spnego_gssspi_set_cred_option,	/* gssspi_set_cred_option */
-	NULL,				/* gssspi_mech_invoke */
+ 	spnego_gss_inquire_sec_context_by_oid, /* gss_inquire_sec_context_by_oid */
+ 	spnego_gss_inquire_cred_by_oid,	/* gss_inquire_cred_by_oid */
+ 	spnego_gss_set_sec_context_option, /* gss_set_sec_context_option */
+ 	spnego_gssspi_set_cred_option,	/* gssspi_set_cred_option */
+ 	NULL,				/* gssspi_mech_invoke */
 	spnego_gss_wrap_aead,
 	spnego_gss_unwrap_aead,
 	spnego_gss_wrap_iov,
@@ -664,10 +664,12 @@ init_ctx_new(OM_uint32 *minor_status,
 	 */
 	sc->ctx_handle = GSS_C_NO_CONTEXT;
 	*ctx = (gss_ctx_id_t)sc;
+	sc = NULL;
 	*tokflag = INIT_TOKEN_SEND;
 	ret = GSS_S_CONTINUE_NEEDED;
 
 cleanup:
+	release_spnego_ctx(&sc);
 	gss_release_buffer(&tmpmin, &tmpoutput);
 	gss_delete_sec_context(&tmpmin, &tmpctx, GSS_C_NO_BUFFER);
 	gss_release_oid_set(&tmpmin, mechSet);
@@ -1059,9 +1061,9 @@ cleanup:
 		if (peerState == ACCEPT_COMPLETE) {
 		    *context_handle = (gss_union_ctx_id_t)(spnego_ctx->ctx_handle)->internal_ctx_id;
 		    if (actual_mech != NULL) {
-                        /*
+                        /* 
                          * krb5 can set actual_mech to the
-                         * requested mechanism which may be
+                         * requested mechanism which may be 
                          * spnego_ctx->internal_mech which
                          * causes a segfault when spnego_ctx is freed
                          */
@@ -1358,9 +1360,11 @@ acc_ctx_hints(OM_uint32 *minor_status,
 	*return_token = INIT_TOKEN_SEND;
 	sc->firstpass = 1;
 	*ctx = (gss_ctx_id_t)sc;
+	sc = NULL;
 	ret = GSS_S_COMPLETE;
 
 cleanup:
+	release_spnego_ctx(&sc);
 	gss_release_oid_set(&tmpmin, &supported_mechSet);
 
 	return ret;
@@ -1387,8 +1391,6 @@ acc_ctx_new(OM_uint32 *minor_status,
 	gss_OID mech_wanted;
 	spnego_gss_ctx_id_t sc = NULL;
 
-	*ctx = GSS_C_NO_CONTEXT;
-
 	ret = GSS_S_DEFECTIVE_TOKEN;
 	der_mechTypes.length = 0;
 	der_mechTypes.value = NULL;
@@ -2699,7 +2701,7 @@ get_available_mechs(OM_uint32 *minor_status,
 	 */
 	if (found > 0 && major_status == GSS_S_COMPLETE && creds != NULL) {
 		major_status = gss_acquire_cred(minor_status,
-						name, GSS_C_INDEFINITE,
+						name, GSS_C_INDEFINITE, 
 						*rmechs, usage, creds,
 						&goodmechs, NULL);
 
@@ -3849,9 +3851,9 @@ is_kerb_mech(gss_OID oid)
 	int answer = 0;
 	OM_uint32 minor;
 	extern const gss_OID_set_desc * const gss_mech_set_krb5_both;
-
+	
 	(void) gss_test_oid_set_member(&minor,
 		oid, (gss_OID_set)gss_mech_set_krb5_both, &answer);
-
+	
 	return (answer);
 }
