@@ -1058,7 +1058,7 @@ krb5_error_code pa_sam(krb5_context context,
 }
 
 #if APPLE_PKINIT
-/*
+/* 
  * PKINIT. One function to generate AS-REQ, one to parse AS-REP
  */
 #define  PKINIT_DEBUG    0
@@ -1073,13 +1073,13 @@ static krb5_error_code pa_pkinit_gen_req(
     krb5_kdc_req *request,
     krb5_pa_data *in_padata,
     krb5_pa_data **out_padata,
-    krb5_data *salt,
+    krb5_data *salt, 
     krb5_data *s2kparams,
     krb5_enctype *etype,
     krb5_keyblock *as_key,
-    krb5_prompter_fct prompter,
+    krb5_prompter_fct prompter, 
     void *prompter_data,
-    krb5_gic_get_as_key_fct gak_fct,
+    krb5_gic_get_as_key_fct gak_fct, 
     void *gak_data)
 {
     krb5_error_code		krtn;
@@ -1096,9 +1096,9 @@ static krb5_error_code pa_pkinit_gen_req(
     krb5_data			nonce_data = {0, 4, (char *)nonce_bytes};
     int				dex;
 
-    /*
+    /* 
      * Trusted CA list and specific KC cert optionally obtained via
-     * krb5_pkinit_get_server_certs(). All are DER-encoded certs.
+     * krb5_pkinit_get_server_certs(). All are DER-encoded certs. 
      */
     krb5_data *trusted_CAs = NULL;
     krb5_ui_4 num_trusted_CAs;
@@ -1121,7 +1121,7 @@ static krb5_error_code pa_pkinit_gen_req(
 	kdcPkinitDebug("No client cert; aborting PKINIT\n");
 	return krtn;
     }
-
+	
     /* optional platform-dependent CA list and KDC cert */
     krtn = krb5_unparse_name(context, request->server, &server_principal);
     if(krtn) {
@@ -1132,7 +1132,7 @@ static krb5_error_code pa_pkinit_gen_req(
     if(krtn) {
 	goto cleanup;
     }
-
+    
     /* checksum of the encoded KDC-REQ-BODY */
     krtn = encode_krb5_kdc_req_body(request, &der_req);
     if(krtn) {
@@ -1148,7 +1148,7 @@ static krb5_error_code pa_pkinit_gen_req(
     if(krtn) {
 	goto cleanup;
     }
-
+    
     /* cook up a random 4-byte nonce */
     krtn = krb5_c_random_make_octets(context, &nonce_data);
     if(krtn) {
@@ -1159,10 +1159,10 @@ static krb5_error_code pa_pkinit_gen_req(
 	nonce |= nonce_bytes[dex];
     }
 
-    krtn = krb5int_pkinit_as_req_create(context,
+    krtn = krb5int_pkinit_as_req_create(context, 
 	kctime, cusec, nonce, &cksum,
-	client_cert,
-	trusted_CAs, num_trusted_CAs,
+	client_cert, 
+	trusted_CAs, num_trusted_CAs, 
 	(kdc_cert.data ? &kdc_cert : NULL),
 	&out_data);
     if(krtn) {
@@ -1247,13 +1247,13 @@ static krb5_error_code pa_pkinit_parse_rep(
     krb5_kdc_req *request,
     krb5_pa_data *in_padata,
     krb5_pa_data **out_padata,
-    krb5_data *salt,
+    krb5_data *salt, 
     krb5_data *s2kparams,
     krb5_enctype *etype,
     krb5_keyblock *as_key,
-    krb5_prompter_fct prompter,
+    krb5_prompter_fct prompter, 
     void *prompter_data,
-    krb5_gic_get_as_key_fct gak_fct,
+    krb5_gic_get_as_key_fct gak_fct, 
     void *gak_data)
 {
     krb5int_cert_sig_status	sig_status = (krb5int_cert_sig_status)-999;
@@ -1289,11 +1289,11 @@ static krb5_error_code pa_pkinit_parse_rep(
 	kdcPkinitDebug("No client cert; aborting PKINIT\n");
 	return krtn;
     }
-
+    
     memset(&local_key, 0, sizeof(local_key));
     asRep.data = (char *)in_padata->contents;
     asRep.length = in_padata->length;
-    krtn = krb5int_pkinit_as_rep_parse(context, &asRep, client_cert,
+    krtn = krb5int_pkinit_as_rep_parse(context, &asRep, client_cert, 
 	&local_key, &as_req_checksum_rcd, &sig_status,
 	&signer_cert, NULL, NULL);
     if(krtn) {
@@ -1308,20 +1308,20 @@ static krb5_error_code pa_pkinit_parse_rep(
 		break;
 	    /* FALLTHROUGH */
 	default:
-	    kdcPkinitDebug("pa_pkinit_parse_rep: bad cert/sig status %d\n",
+	    kdcPkinitDebug("pa_pkinit_parse_rep: bad cert/sig status %d\n", 
 		(int)sig_status);
 	    krtn = KRB5KDC_ERR_PREAUTH_FAILED;
 	    goto error_out;
     }
-
-    /* calculate checksum of incoming AS-REQ using the decryption key
+    
+    /* calculate checksum of incoming AS-REQ using the decryption key 
      * we just got from the ReplyKeyPack */
     krtn = encode_krb5_as_req(request, &encoded_as_req);
     if(krtn) {
 	goto error_out;
     }
-    krtn = krb5_c_make_checksum(context, context->kdc_req_sumtype,
-	&local_key, KRB5_KEYUSAGE_TGS_REQ_AUTH_CKSUM,
+    krtn = krb5_c_make_checksum(context, context->kdc_req_sumtype, 
+	&local_key, KRB5_KEYUSAGE_TGS_REQ_AUTH_CKSUM, 
 	encoded_as_req, &as_req_checksum_gen);
     if(krtn) {
 	goto error_out;
@@ -1334,23 +1334,23 @@ static krb5_error_code pa_pkinit_parse_rep(
 	krtn = KRB5KDC_ERR_PREAUTH_FAILED;
 	goto error_out;
     }
-
+    
     /* We have the key; transfer to caller */
     if (as_key->length) {
 	krb5_free_keyblock_contents(context, as_key);
     }
     *as_key = local_key;
-
+    
     #if PKINIT_DEBUG
     fprintf(stderr, "pa_pkinit_parse_rep: SUCCESS\n");
     fprintf(stderr, "enctype %d keylen %d keydata %02x %02x %02x %02x...\n",
 	(int)as_key->enctype, (int)as_key->length,
-	as_key->contents[0], as_key->contents[1],
+	as_key->contents[0], as_key->contents[1], 
 	as_key->contents[2], as_key->contents[3]);
     #endif
-
+    
     krtn = 0;
-
+    
 error_out:
     if (signer_cert.data) {
 	    free(signer_cert.data);
@@ -1579,7 +1579,9 @@ krb5_error_code pa_sam_2(krb5_context context,
 
    cksum = sc2->sam_cksum;
    
-   while (*cksum) {
+   for (; *cksum; cksum++) {
+        if (!krb5_c_is_keyed_cksum((*cksum)->checksum_type))
+            continue;
 	/* Check this cksum */
 	retval = krb5_c_verify_checksum(context, as_key,
 			KRB5_KEYUSAGE_PA_SAM_CHALLENGE_CKSUM,
@@ -1593,7 +1595,6 @@ krb5_error_code pa_sam_2(krb5_context context,
 	}
 	if (valid_cksum)
 	   break;
-	cksum++;
    }
 
    if (!valid_cksum) {
@@ -2055,7 +2056,7 @@ krb5_do_preauth(krb5_context context,
 						   gak_fct, gak_data)))) {
                         if (paorder[h] == PA_INFO) {
 #ifdef DEBUG
-                            fprintf (stderr,
+                            fprintf (stderr, 
                                      "internal function for type %d, flag %d "
                                      "failed with err %d\n",
                                      in_padata[i]->pa_type, paorder[h], ret);
@@ -2063,7 +2064,7 @@ krb5_do_preauth(krb5_context context,
                             ret = 0;
                             continue; /* PA_INFO type failed, ignore */
                         }
-
+                        
 		      goto cleanup;
 		    }
 
