@@ -53,7 +53,7 @@ LsaOpenServer(
 {
     DWORD dwError = 0;
     PLSA_CLIENT_CONNECTION_CONTEXT pContext = NULL;
-    static LWMsgTime connectTimeout = {2, 0};
+    static LWMsgTime connectTimeout = {10, 0};
 
     BAIL_ON_INVALID_POINTER(phConnection);
 
@@ -132,7 +132,7 @@ LsaCloseServer(
 
     if (pContext->pAssoc)
     {
-        lwmsg_assoc_close(pContext->pAssoc);
+        (void) lwmsg_assoc_close(pContext->pAssoc);
         lwmsg_assoc_delete(pContext->pAssoc);
     }
 
@@ -353,9 +353,9 @@ LsaTransactAddUser2(
         dwError = LW_ERROR_INTERNAL;
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 cleanup:
-
+    
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &out);
@@ -403,7 +403,7 @@ LsaTransactAuthenticateUserPam(
             break;
         case LSA_R_AUTH_USER_PAM_FAILURE:
             pError = (PLSA_IPC_ERROR) out.data;
-            if (pParams->dwFlags & LSA_AUTH_USER_PAM_FLAG_RETURN_MESSAGE &&
+            if (pParams->dwFlags & LSA_AUTH_USER_PAM_FLAG_RETURN_MESSAGE && 
                     ppPamAuthInfo)
             {
                 dwError = LwAllocateMemory(sizeof(*ppPamAuthInfo),
@@ -439,14 +439,14 @@ error:
 DWORD
 LsaTransactAuthenticateUserEx(
     IN HANDLE hServer,
+    IN PCSTR pszTargetProvider,
     IN LSA_AUTH_USER_PARAMS* pParams,
     OUT PLSA_AUTH_USER_INFO* ppUserInfo
     )
 {
     DWORD dwError = 0;
-    LSA_AUTH_USER_PARAMS authUserExReq;
+    LSA_IPC_AUTH_USER_EX_REQ req = {0};
     PLSA_IPC_ERROR pError = NULL;
-
     LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
     LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -454,14 +454,15 @@ LsaTransactAuthenticateUserEx(
     dwError = LsaIpcAcquireCall(hServer, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
-    authUserExReq.AuthType = pParams->AuthType;
-    authUserExReq.pass = pParams->pass;
-    authUserExReq.pszAccountName = pParams->pszAccountName;
-    authUserExReq.pszDomain = pParams->pszDomain;
-    authUserExReq.pszWorkstation = pParams->pszWorkstation;
+    req.pszTargetProvider = pszTargetProvider;
+    req.authUserParams.AuthType = pParams->AuthType;
+    req.authUserParams.pass = pParams->pass;
+    req.authUserParams.pszAccountName = pParams->pszAccountName;
+    req.authUserParams.pszDomain = pParams->pszDomain;
+    req.authUserParams.pszWorkstation = pParams->pszWorkstation;
 
     in.tag = LSA_Q_AUTH_USER_EX;
-    in.data = &authUserExReq;
+    in.data = &req;
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LSA_ERROR(dwError);
@@ -873,12 +874,12 @@ LsaTransactFindObjects(
 
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     req.pszTargetProvider = pszTargetProvider;
     req.FindFlags = FindFlags;
     req.ObjectType = ObjectType;
     req.QueryType = QueryType;
-
+    
     switch (QueryType)
     {
     case LSA_QUERY_TYPE_BY_UNIX_ID:
@@ -965,7 +966,7 @@ LsaTransactOpenEnumObjects(
 
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     req.pszTargetProvider = pszTargetProvider;
     req.FindFlags = FindFlags;
     req.ObjectType = ObjectType;
@@ -1030,7 +1031,7 @@ LsaTransactEnumObjects(
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
-    req.hEnum = hEnum;
+    req.hEnum = hEnum;   
     req.dwMaxObjectsCount = dwMaxObjectsCount;
 
     in.tag = LSA2_Q_ENUM_OBJECTS;
@@ -1098,7 +1099,7 @@ LsaTransactOpenEnumMembers(
 
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     req.pszTargetProvider = pszTargetProvider;
     req.FindFlags = FindFlags;
     req.pszSid = pszSid;
@@ -1162,7 +1163,7 @@ LsaTransactEnumMembers(
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
-    req.hEnum = hEnum;
+    req.hEnum = hEnum;   
     req.dwMaxSidCount = dwMaxSidCount;
 
     in.tag = LSA2_Q_ENUM_MEMBERS;
@@ -1308,7 +1309,7 @@ LsaTransactFindGroupAndExpandedMembers(
     req.FindFlags = FindFlags;
     req.QueryType = QueryType;
     req.QueryItem = QueryItem;
-
+    
     switch(QueryType)
     {
     case LSA_QUERY_TYPE_BY_UNIX_ID:
@@ -1458,9 +1459,9 @@ LsaTransactGetSmartCardUserObject(
     default:
         BAIL_WITH_LSA_ERROR(LW_ERROR_INTERNAL);
     }
-
+    
 cleanup:
-
+    
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &out);
