@@ -170,7 +170,7 @@ LsaFindGroupByName(
     case 1:
         /* Fast path */
         QueryItem.pszString = pszGroupName;
-
+        
         dwError = LsaFindGroupAndExpandedMembers(
             hLsaConnection,
             NULL,
@@ -181,7 +181,7 @@ LsaFindGroupByName(
             &dwObjectCount,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LsaMarshalGroupInfo1(
             hLsaConnection,
             FindFlags,
@@ -194,7 +194,7 @@ LsaFindGroupByName(
         break;
     default:
         QueryList.ppszStrings = &pszGroupName;
-
+        
         dwError = LsaFindObjects(
             hLsaConnection,
             NULL,
@@ -205,13 +205,13 @@ LsaFindGroupByName(
             QueryList,
         &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (ppObjects[0] == NULL)
         {
             dwError = LW_ERROR_NO_SUCH_GROUP;
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         dwError = LsaMarshalGroupInfo(
             hLsaConnection,
             FindFlags,
@@ -273,7 +273,7 @@ LsaFindGroupById(
     case 1:
         /* Fast path */
         QueryItem.dwId = dwGid;
-
+        
         dwError = LsaFindGroupAndExpandedMembers(
             hLsaConnection,
             NULL,
@@ -284,7 +284,7 @@ LsaFindGroupById(
             &dwObjectCount,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LsaMarshalGroupInfo1(
             hLsaConnection,
             FindFlags,
@@ -297,7 +297,7 @@ LsaFindGroupById(
         break;
     default:
         QueryList.pdwIds = &dwGid;
-
+        
         dwError = LsaFindObjects(
             hLsaConnection,
             NULL,
@@ -308,13 +308,13 @@ LsaFindGroupById(
             QueryList,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (ppObjects[0] == NULL)
         {
             dwError = LW_ERROR_NO_SUCH_GROUP;
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         dwError = LsaMarshalGroupInfo(
             hLsaConnection,
             FindFlags,
@@ -330,7 +330,7 @@ error:
     {
         *ppGroupInfo = pGroupInfo;
     }
-
+    
     if (pGroup)
     {
         LsaFreeSecurityObject(pGroup);
@@ -363,7 +363,7 @@ LsaBeginEnumGroups(
     pEnum->dwGroupInfoLevel = dwGroupInfoLevel;
     pEnum->dwMaxNumGroups = dwMaxNumGroups;
     pEnum->FindFlags = FindFlags;
-
+    
     dwError = LsaOpenEnumObjects(
         hLsaConnection,
         NULL,
@@ -484,7 +484,7 @@ LsaEnumGroups(
 
         *pdwNumGroupsFound = 0;
         *pppGroupInfoList = NULL;
-
+        
         LW_SAFE_FREE_MEMORY(ppGroupInfo);
     }
     else
@@ -551,7 +551,7 @@ LsaDeleteGroupById(
     DWORD dwGid = (DWORD) gid;
 
     QueryList.pdwIds = &dwGid;
-
+    
     dwError = LsaFindObjects(
         hLsaConnection,
         NULL,
@@ -562,7 +562,7 @@ LsaDeleteGroupById(
         QueryList,
         &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (ppObjects[0] == NULL)
     {
         dwError = LW_ERROR_NO_SUCH_GROUP;
@@ -647,7 +647,7 @@ LsaGetGidsForUserByName(
     DWORD dwError = 0;
     PVOID pUserInfo = NULL;
     DWORD dwUserInfoLevel = 0;
-    DWORD dwGroupInfoLevel = 0;
+    static const DWORD dwGroupInfoLevel = 0;
     DWORD dwGroupFound = 0;
     gid_t* pGidResult = NULL;
     PVOID*  ppGroupInfoList = NULL;
@@ -681,20 +681,9 @@ LsaGetGidsForUserByName(
                     (PVOID*)&pGidResult);
     BAIL_ON_LSA_ERROR(dwError);
 
-    switch(dwGroupInfoLevel)
+    for (iGroup = 0; iGroup < dwGroupFound; iGroup++)
     {
-        case 0:
-
-            for (iGroup = 0; iGroup < dwGroupFound; iGroup++) {
-                   *(pGidResult+iGroup) = ((PLSA_GROUP_INFO_0)(*(ppGroupInfoList+iGroup)))->gid;
-            }
-
-            break;
-
-        default:
-            dwError = LW_ERROR_UNSUPPORTED_GROUP_LEVEL;
-            BAIL_ON_LSA_ERROR(dwError);
-            break;
+        *(pGidResult+iGroup) = ((PLSA_GROUP_INFO_0)(*(ppGroupInfoList+iGroup)))->gid;
     }
 
     *ppGidResults = pGidResult;
@@ -758,7 +747,7 @@ LsaGetGroupsForUserBySid(
     else
     {
         QueryList.ppszStrings = (PCSTR*) ppszGroupSids;
-
+        
         dwError = LsaFindObjects(
             hLsaConnection,
             NULL,
@@ -842,7 +831,7 @@ LsaGetGroupsForUserByName(
     LSA_QUERY_LIST QueryList;
 
     QueryList.ppszStrings = &pszUserName;
-
+    
     dwError = LsaFindObjects(
         hLsaConnection,
         NULL,
@@ -858,8 +847,8 @@ LsaGetGroupsForUserByName(
     {
         dwError = LW_ERROR_NO_SUCH_USER;
         BAIL_ON_LSA_ERROR(dwError);
-    }
-
+    }     
+    
     dwError = LsaGetGroupsForUserBySid(
         hLsaConnection,
         ppObjects[0]->pszObjectSid,
@@ -868,7 +857,7 @@ LsaGetGroupsForUserByName(
         pdwGroupsFound,
         pppGroupInfoList);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
 cleanup:
 
     if (ppObjects)
@@ -903,7 +892,7 @@ LsaGetGroupsForUserById(
     DWORD dwUid = (DWORD) uid;
 
     QueryList.pdwIds = &dwUid;
-
+    
     dwError = LsaFindObjects(
         hLsaConnection,
         NULL,
@@ -914,13 +903,13 @@ LsaGetGroupsForUserById(
         QueryList,
         &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (!ppObjects[0])
     {
         dwError = LW_ERROR_NO_SUCH_USER;
         BAIL_ON_LSA_ERROR(dwError);
-    }
-
+    }     
+    
     dwError = LsaGetGroupsForUserBySid(
         hLsaConnection,
         ppObjects[0]->pszObjectSid,
@@ -929,7 +918,7 @@ LsaGetGroupsForUserById(
         pdwGroupsFound,
         pppGroupInfoList);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
 cleanup:
 
     if (ppObjects)
