@@ -329,7 +329,7 @@ LsaMarshalGroupInfo(
     PVOID pGroupInfo = NULL;
     /* The variable represents pGroupInfo casted to different types. Do not
      * free these values directly, free pGroupInfo instead.
-     */
+     */   
     size_t sIndex = 0;
     size_t sEnabled = 0;
     DWORD dwMemberCount = 0;
@@ -361,11 +361,11 @@ LsaMarshalGroupInfo(
                             sizeof(LSA_GROUP_INFO_0),
                             (PVOID*)&pGroupInfo);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             pGroupInfo0 = (PLSA_GROUP_INFO_0) pGroupInfo;
-
+            
             pGroupInfo0->gid = pGroup->groupInfo.gid;
-
+            
             dwError = LwAllocateString(
                                 pGroup->groupInfo.pszUnixName,
                                 &pGroupInfo0->pszName);
@@ -375,12 +375,12 @@ LsaMarshalGroupInfo(
                                 pGroup->pszObjectSid,
                                 &pGroupInfo0->pszSid);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             break;
         }
         case 1:
         {
-            PLSA_GROUP_INFO_1 pGroupInfo1 = NULL;
+            PLSA_GROUP_INFO_1 pGroupInfo1 = NULL; 
 
             dwError = LsaQueryExpandedGroupMembers(
                 hLsa,
@@ -400,9 +400,9 @@ LsaMarshalGroupInfo(
                             sizeof(LSA_GROUP_INFO_1),
                             (PVOID*)&pGroupInfo);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             pGroupInfo1 = (PLSA_GROUP_INFO_1) pGroupInfo;
-
+     
             pGroupInfo1->gid = pGroup->groupInfo.gid;
 
             dwError = LwAllocateString(
@@ -415,7 +415,7 @@ LsaMarshalGroupInfo(
             dwError = LwStrDupOrNull(
                         pGroup->groupInfo.pszPasswd,
                         &pGroupInfo1->pszPasswd);
-            BAIL_ON_LSA_ERROR(dwError);
+            BAIL_ON_LSA_ERROR(dwError);        
 
             dwError = LwAllocateString(
                                 pGroup->pszObjectSid,
@@ -438,7 +438,7 @@ LsaMarshalGroupInfo(
                     {
                         sEnabled++;
                     }
-
+                    
                     if (ppMembers[sIndex]->type != LSA_OBJECT_TYPE_USER)
                     {
                         dwError = LW_ERROR_INVALID_PARAMETER;
@@ -455,7 +455,7 @@ LsaMarshalGroupInfo(
 
             sEnabled = 0;
 
-            for (sIndex = 0; sIndex < dwMemberCount; sIndex++)
+            for (sIndex = 0; sIndex < dwMemberCount; sIndex++)               
             {
                 if (ppMembers[sIndex])
                 {
@@ -468,7 +468,7 @@ LsaMarshalGroupInfo(
                     }
                 }
             }
-
+            
             break;
         }
 
@@ -477,26 +477,26 @@ LsaMarshalGroupInfo(
             BAIL_ON_LSA_ERROR(dwError);
             break;
     }
-
+    
     *ppGroupInfo = pGroupInfo;
-
+    
 cleanup:
-
+    
     if (ppMembers)
     {
         LsaUtilFreeSecurityObjectList(dwMemberCount, ppMembers);
     }
-
+    
     return dwError;
-
+    
 error:
     if (pGroupInfo) {
         LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
         pGroupInfo = NULL;
     }
-
+    
     *ppGroupInfo = NULL;
-
+    
     goto cleanup;
 }
 
@@ -516,7 +516,7 @@ LsaMarshalGroupInfo1(
     PLSA_GROUP_INFO_1 pGroupInfo1 = NULL;
     /* The variable represents pGroupInfo casted to different types. Do not
      * free these values directly, free pGroupInfo instead.
-     */
+     */   
     size_t sIndex = 0;
     size_t sEnabled = 0;
 
@@ -540,28 +540,28 @@ LsaMarshalGroupInfo1(
         sizeof(LSA_GROUP_INFO_1),
         (PVOID*)&pGroupInfo);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     pGroupInfo1 = (PLSA_GROUP_INFO_1) pGroupInfo;
-
+     
     pGroupInfo1->gid = pGroup->groupInfo.gid;
-
+    
     dwError = LwAllocateString(
         pGroup->groupInfo.pszUnixName,
         &pGroupInfo1->pszName);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     // Optional values use LwStrDupOrNull. Required values use
     // LwAllocateString.
     dwError = LwStrDupOrNull(
         pGroup->groupInfo.pszPasswd,
         &pGroupInfo1->pszPasswd);
-    BAIL_ON_LSA_ERROR(dwError);
-
+    BAIL_ON_LSA_ERROR(dwError);        
+    
     dwError = LwAllocateString(
         pGroup->pszObjectSid,
         &pGroupInfo1->pszSid);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (pGroup->pszDN)
     {
         dwError = LwAllocateString(
@@ -569,7 +569,7 @@ LsaMarshalGroupInfo1(
             &pGroupInfo1->pszDN);
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     for (sIndex = 0; sIndex < dwMemberCount; sIndex++)
     {
         if (ppMembers[sIndex])
@@ -578,7 +578,7 @@ LsaMarshalGroupInfo1(
             {
                 sEnabled++;
             }
-
+            
             if (ppMembers[sIndex]->type != LSA_OBJECT_TYPE_USER)
             {
                 dwError = LW_ERROR_INVALID_PARAMETER;
@@ -586,16 +586,16 @@ LsaMarshalGroupInfo1(
             }
         }
     }
-
+    
     dwError = LwAllocateMemory(
         //Leave room for terminating null pointer
         sizeof(PSTR) * (sEnabled+1),
         (PVOID*)&pGroupInfo1->ppszMembers);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     sEnabled = 0;
-
-    for (sIndex = 0; sIndex < dwMemberCount; sIndex++)
+    
+    for (sIndex = 0; sIndex < dwMemberCount; sIndex++)               
     {
         if (ppMembers[sIndex])
         {
@@ -608,21 +608,21 @@ LsaMarshalGroupInfo1(
             }
         }
     }
-
+    
     *ppGroupInfo = pGroupInfo;
-
+    
 cleanup:
-
+    
     return dwError;
-
+    
 error:
     if (pGroupInfo) {
         LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
         pGroupInfo = NULL;
     }
-
+    
     *ppGroupInfo = NULL;
-
+    
     goto cleanup;
 }
 
@@ -730,7 +730,7 @@ LsaMarshalUserModInfoToUserModInfo2(
     if (pModInfo1->actions.bSetPrimaryGroup)
     {
         QueryList.pdwIds = &dwGid;
-
+        
         dwError = LsaFindObjects(
             hLsa,
             NULL,
@@ -741,7 +741,7 @@ LsaMarshalUserModInfoToUserModInfo2(
             QueryList,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (ppObjects[0] == NULL)
         {
             dwError = LW_ERROR_NO_SUCH_GROUP;
@@ -818,16 +818,16 @@ LsaMarshalGroupModInfoToGroupModInfo2(
     LSA_QUERY_LIST QueryList;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     DWORD dwGid = (DWORD) pModInfo1->gid;
-
+    
     dwError = LwAllocateMemory(sizeof(*pModInfo2), OUT_PPVOID(&pModInfo2));
     BAIL_ON_LSA_ERROR(dwError);
 
     memcpy(&pModInfo2->actions, &pModInfo1->actions, sizeof(pModInfo2->actions));
     pModInfo2->dwAddMembersNum = pModInfo1->dwAddMembersNum;
     pModInfo2->dwRemoveMembersNum = pModInfo1->dwRemoveMembersNum;
-
+    
     QueryList.pdwIds = &dwGid;
-
+    
     dwError = LsaFindObjects(
         hLsa,
         NULL,
@@ -838,7 +838,7 @@ LsaMarshalGroupModInfoToGroupModInfo2(
         QueryList,
         &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (ppObjects[0] == NULL)
     {
         dwError = LW_ERROR_NO_SUCH_GROUP;
@@ -847,14 +847,14 @@ LsaMarshalGroupModInfoToGroupModInfo2(
 
     dwError = LwStrDupOrNull(ppObjects[0]->pszObjectSid, &pModInfo2->pszSid);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (pModInfo1->pAddMembers)
     {
         dwError = LwAllocateMemory(
             sizeof(*pModInfo2->ppszAddMembers) * pModInfo2->dwAddMembersNum,
             OUT_PPVOID(&pModInfo2->ppszAddMembers));
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         for (dwIndex = 0; dwIndex < pModInfo2->dwAddMembersNum; dwIndex++)
         {
             dwError = LwAllocateString(
@@ -914,7 +914,7 @@ LsaMarshalGroupInfo0ToGroupAddInfo(
     BAIL_ON_LSA_ERROR(dwError);
 
     pAddInfo->gid = pGroupInfo->gid;
-
+    
     dwError = LwStrDupOrNull(pGroupInfo->pszName, &pAddInfo->pszName);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -923,7 +923,7 @@ LsaMarshalGroupInfo0ToGroupAddInfo(
 cleanup:
 
     return dwError;
-
+    
 error:
 
     *ppAddInfo = NULL;
@@ -1002,7 +1002,7 @@ cleanup:
     LsaUtilFreeSecurityObjectList(dwCount, ppObjects);
 
     return dwError;
-
+    
 error:
 
     *ppAddInfo = NULL;
@@ -1036,9 +1036,6 @@ LsaMarshalUserInfo0ToUserAddInfo(
     dwError = LwStrDupOrNull(pUserInfo->pszName, &pAddInfo->pszName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwStrDupOrNull(pUserInfo->pszName, &pAddInfo->pszName);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = LwStrDupOrNull(pUserInfo->pszPasswd, &pAddInfo->pszPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -1081,7 +1078,7 @@ LsaMarshalUserInfo0ToUserAddInfo(
 cleanup:
 
     return dwError;
-
+    
 error:
 
     *ppAddInfo = NULL;
