@@ -77,7 +77,7 @@ NtlmInitialize(
         dwError = LwMapLwmsgStatusToLwError(
             lwmsg_client_new(NULL, gpProtocol, &gpClient));
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LwMapLwmsgStatusToLwError(
             lwmsg_client_set_endpoint(
                 gpClient,
@@ -152,10 +152,12 @@ NtlmTransactAcceptSecurityContext(
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
-    NTLM_IPC_ACCEPT_SEC_CTXT_REQ AcceptSecCtxtReq;
-
-    LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
-    LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
+    NTLM_IPC_ACCEPT_SEC_CTXT_REQ AcceptSecCtxtReq = {0};
+    // Do not free pResult and pError
+    PNTLM_IPC_ACCEPT_SEC_CTXT_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
+    LWMsgParams In = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams Out = LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
     dwError = NtlmIpcAcquireCall(&pCall);
@@ -166,10 +168,6 @@ NtlmTransactAcceptSecurityContext(
 
     memset(&AcceptSecCtxtReq, 0, sizeof(AcceptSecCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_ACCEPT_SEC_CTXT_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     AcceptSecCtxtReq.hCredential = *phCredential;
     if (phContext)
     {
@@ -183,7 +181,11 @@ NtlmTransactAcceptSecurityContext(
     AcceptSecCtxtReq.pInput = &pInput->pBuffers[0];
     AcceptSecCtxtReq.fContextReq = fContextReq;
     AcceptSecCtxtReq.TargetDataRep = TargetDataRep;
-    AcceptSecCtxtReq.hNewContext = *phNewContext;
+
+    if (phNewContext)
+    {
+        AcceptSecCtxtReq.hNewContext = *phNewContext;
+    }
 
     In.tag = NTLM_Q_ACCEPT_SEC_CTXT;
     In.data = &AcceptSecCtxtReq;
@@ -251,7 +253,7 @@ error:
             *phNewContext = NULL;
         }
     }
-
+    
     *pfContextAttr = 0;
     *ptsTimeStamp = 0;
 
@@ -271,7 +273,9 @@ NtlmTransactAcquireCredentialsHandle(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_ACQUIRE_CREDS_REQ AcquireCredsReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_ACQUIRE_CREDS_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -281,10 +285,6 @@ NtlmTransactAcquireCredentialsHandle(
 
     memset(&AcquireCredsReq, 0, sizeof(AcquireCredsReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_ACQUIRE_CREDS_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     AcquireCredsReq.pszPrincipal = pszPrincipal;
     AcquireCredsReq.pszPackage = pszPackage;
     AcquireCredsReq.fCredentialUse = fCredentialUse;
@@ -342,7 +342,9 @@ NtlmTransactDecryptMessage(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_DECRYPT_MSG_REQ DecryptMsgReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_DECRYPT_MSG_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -352,10 +354,6 @@ NtlmTransactDecryptMessage(
 
     memset(&DecryptMsgReq, 0, sizeof(DecryptMsgReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_DECRYPT_MSG_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     DecryptMsgReq.hContext = *phContext;
     DecryptMsgReq.pMessage = pMessage;
     DecryptMsgReq.MessageSeqNo = MessageSeqNo;
@@ -412,7 +410,8 @@ NtlmTransactDeleteSecurityContext(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_DELETE_SEC_CTXT_REQ DeleteSecCtxtReq;
-
+    // Do not free pError
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -422,9 +421,6 @@ NtlmTransactDeleteSecurityContext(
 
     memset(&DeleteSecCtxtReq, 0, sizeof(DeleteSecCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_ERROR pError = NULL;
-
     DeleteSecCtxtReq.hContext = *phContext;
 
     In.tag = NTLM_Q_DELETE_SEC_CTXT;
@@ -474,7 +470,9 @@ NtlmTransactEncryptMessage(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_ENCRYPT_MSG_REQ EncryptMsgReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_ENCRYPT_MSG_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -484,10 +482,6 @@ NtlmTransactEncryptMessage(
 
     memset(&EncryptMsgReq, 0, sizeof(EncryptMsgReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_ENCRYPT_MSG_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     EncryptMsgReq.hContext = *phContext;
     EncryptMsgReq.bEncrypt = bEncrypt;
     EncryptMsgReq.pMessage = pMessage;
@@ -546,7 +540,9 @@ NtlmTransactExportSecurityContext(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_EXPORT_SEC_CTXT_REQ ExportSecCtxtReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_EXPORT_SEC_CTXT_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -556,10 +552,6 @@ NtlmTransactExportSecurityContext(
 
     memset(&ExportSecCtxtReq, 0, sizeof(ExportSecCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_EXPORT_SEC_CTXT_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     ExportSecCtxtReq.hContext = *phContext;
     ExportSecCtxtReq.fFlags = fFlags;
 
@@ -615,7 +607,8 @@ NtlmTransactFreeCredentialsHandle(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_FREE_CREDS_REQ FreeCredsReq;
-
+    // Do not free pError
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -625,9 +618,6 @@ NtlmTransactFreeCredentialsHandle(
 
     memset(&FreeCredsReq, 0, sizeof(FreeCredsReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_ERROR pError = NULL;
-
     FreeCredsReq.hCredential = *phCredential;
 
     In.tag = NTLM_Q_FREE_CREDS;
@@ -677,7 +667,9 @@ NtlmTransactImportSecurityContext(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_IMPORT_SEC_CTXT_REQ ImportSecCtxtReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_IMPORT_SEC_CTXT_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -687,10 +679,6 @@ NtlmTransactImportSecurityContext(
 
     memset(&ImportSecCtxtReq, 0, sizeof(ImportSecCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_IMPORT_SEC_CTXT_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     ImportSecCtxtReq.pszPackage = pszPackage;
     ImportSecCtxtReq.pPackedContext = pPackedContext;
     ImportSecCtxtReq.pToken = pToken;
@@ -751,7 +739,9 @@ NtlmTransactInitializeSecurityContext(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_INIT_SEC_CTXT_REQ InitSecCtxtReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_INIT_SEC_CTXT_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -763,10 +753,6 @@ NtlmTransactInitializeSecurityContext(
 
     memset(&InitSecCtxtReq, 0, sizeof(InitSecCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_INIT_SEC_CTXT_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     if (phCredential)
     {
         InitSecCtxtReq.hCredential = *phCredential;
@@ -869,7 +855,7 @@ error:
             *phNewContext = NULL;
         }
     }
-
+    
     *pfContextAttr = 0;
     *ptsExpiry = 0;
     memset(pOutput, 0, sizeof(SecBufferDesc));
@@ -887,7 +873,9 @@ NtlmTransactMakeSignature(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_MAKE_SIGN_REQ MakeSignReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_MAKE_SIGN_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -899,10 +887,6 @@ NtlmTransactMakeSignature(
 
     memset(&MakeSignReq, 0, sizeof(MakeSignReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_MAKE_SIGN_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     MakeSignReq.hContext = *phContext;
     MakeSignReq.dwQop = dwQop;
     MakeSignReq.pMessage = pMessage;
@@ -960,7 +944,9 @@ NtlmTransactQueryContextAttributes(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_QUERY_CTXT_REQ QueryCtxtReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_QUERY_CTXT_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -972,10 +958,6 @@ NtlmTransactQueryContextAttributes(
 
     memset(&QueryCtxtReq, 0, sizeof(QueryCtxtReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_QUERY_CTXT_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     QueryCtxtReq.hContext = *phContext;
     QueryCtxtReq.ulAttribute = ulAttribute;
 
@@ -1065,7 +1047,9 @@ NtlmTransactQueryCredentialsAttributes(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_QUERY_CREDS_REQ QueryCredsReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_QUERY_CREDS_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -1077,10 +1061,6 @@ NtlmTransactQueryCredentialsAttributes(
 
     memset(&QueryCredsReq, 0, sizeof(QueryCredsReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_QUERY_CREDS_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     QueryCredsReq.hCredential = *phCredential;
     QueryCredsReq.ulAttribute = ulAttribute;
 
@@ -1099,11 +1079,9 @@ NtlmTransactQueryCredentialsAttributes(
             switch(pResultList->ulAttribute)
             {
             case SECPKG_CRED_ATTR_NAMES:
-                memcpy(
-                    pBuffer,
-                    pResultList->Buffer.pNames,
-                    sizeof(SecPkgContext_Names));
-                pResultList->Buffer.pNames = NULL;
+                ((PSecPkgCred_Names)pBuffer)->pUserName =
+                    pResultList->Buffer.pNames->pUserName;
+                pResultList->Buffer.pNames->pUserName = NULL;
                 break;
             default:
                 dwError = LW_ERROR_INTERNAL;
@@ -1134,6 +1112,75 @@ error:
     goto cleanup;
 }
 
+DWORD
+NtlmTransactSetCredentialsAttributes(
+    IN PNTLM_CRED_HANDLE phCredential,
+    IN DWORD ulAttribute,
+    IN PVOID pBuffer
+    )
+{
+    DWORD dwError = LW_ERROR_SUCCESS;
+    NTLM_IPC_SET_CREDS_REQ SetCredsReq;
+    // Do not free pResult and pError
+    PNTLM_IPC_ERROR pError = NULL;
+    LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    NtlmInitialize();
+
+    dwError = NtlmIpcAcquireCall(&pCall);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    memset(&SetCredsReq, 0, sizeof(SetCredsReq));
+
+    SetCredsReq.hCredential = *phCredential;
+    SetCredsReq.ulAttribute = ulAttribute;
+    switch (ulAttribute)
+    {
+        case SECPKG_CRED_ATTR_DOMAIN_NAME:
+            SetCredsReq.Buffer.pDomainName = (PSecPkgCred_DomainName)pBuffer;
+            break;
+        default:
+            dwError = LW_ERROR_INVALID_PARAMETER;
+            BAIL_ON_LSA_ERROR(dwError);
+            break;
+    }
+
+    In.tag = NTLM_Q_SET_CREDS;
+    In.data = &SetCredsReq;
+
+    dwError = MAP_LWMSG_ERROR(
+        lwmsg_call_dispatch(pCall, &In, &Out, NULL, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    switch (Out.tag)
+    {
+        case NTLM_R_SET_CREDS_SUCCESS:
+            break;
+        case NTLM_R_GENERIC_FAILURE:
+            pError = (PNTLM_IPC_ERROR) Out.data;
+            dwError = pError->dwError;
+            BAIL_ON_LSA_ERROR(dwError);
+            break;
+        default:
+            dwError = LW_ERROR_INTERNAL;
+            BAIL_ON_LSA_ERROR(dwError);
+    }
+
+cleanup:
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &Out);
+        lwmsg_call_release(pCall);
+    }
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
 DWORD
 NtlmTransactVerifySignature(
     IN PNTLM_CONTEXT_HANDLE phContext,
@@ -1144,7 +1191,9 @@ NtlmTransactVerifySignature(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     NTLM_IPC_VERIFY_SIGN_REQ VerifySignReq;
-
+    // Do not free pResult and pError
+    PNTLM_IPC_VERIFY_SIGN_RESPONSE pResultList = NULL;
+    PNTLM_IPC_ERROR pError = NULL;
     LWMsgParams In= LWMSG_PARAMS_INITIALIZER;
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
@@ -1156,10 +1205,6 @@ NtlmTransactVerifySignature(
 
     memset(&VerifySignReq, 0, sizeof(VerifySignReq));
 
-    // Do not free pResult and pError
-    PNTLM_IPC_VERIFY_SIGN_RESPONSE pResultList = NULL;
-    PNTLM_IPC_ERROR pError = NULL;
-
     VerifySignReq.hContext = *phContext;
     VerifySignReq.pMessage = pMessage;
     VerifySignReq.MessageSeqNo = MessageSeqNo;
