@@ -69,26 +69,6 @@ LsaAllocateAuditEventsInfo(
     );
 
 
-NTSTATUS
-LsaRpcInitMemory(
-    VOID
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
-NTSTATUS
-LsaRpcDestroyMemory(
-    VOID
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
 NTSTATUS
 LsaRpcAllocateMemory(
     OUT PVOID *ppOut,
@@ -240,7 +220,7 @@ LsaAllocateTranslatedSids3(
         {
             LWBUF_ALLOC_PSID(pBuffer, NULL);
         }
-        else
+        else 
         {
             ntStatus = STATUS_INVALID_SID;
             BAIL_ON_NT_STATUS(ntStatus);
@@ -338,14 +318,14 @@ LsaAllocateRefDomainList(
 
         ppDomains        = (PVOID*)pCursor;
         *ppDomains       = (PVOID)pDomains;
-        (*pdwSpaceLeft) -= (pDomains) ? dwDomainsSize : 0;
+        (*pdwSpaceLeft) -= (pDomains) ? LWBUF_ALIGN_SIZE(dwDomainsSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft) -= sizeof(PVOID);
     }
     else
     {
-        (*pdwSize) += dwDomainsSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwDomainsSize);
     }
 
     /* include size of the pointer */
@@ -586,7 +566,6 @@ LsaAllocateAuditEventsInfo(
     PVOID *ppBlob = NULL;
     DWORD dwBlobSize = 0;
     DWORD dwBlobSpaceLeft = 0;
-    DWORD dwBlobOffset = 0;
 
     LWBUF_ALLOC_DWORD(pBuffer, pIn->auditing_mode);
     LWBUF_ALIGN_PTR(pdwOffset, pdwSize, pdwSpaceLeft);
@@ -612,7 +591,6 @@ LsaAllocateAuditEventsInfo(
 
             dwBlobSpaceLeft = dwBlobSize;
             dwBlobSize      = 0;
-            dwBlobOffset    = 0;
 
             dwError = LwBufferAllocFixedBlob(
                                        pBlob,
@@ -626,14 +604,14 @@ LsaAllocateAuditEventsInfo(
 
         ppBlob           = (PVOID*)pCursor;
         *ppBlob          = (PVOID)pBlob;
-        (*pdwSpaceLeft) -= (pBlob) ? dwBlobSize : 0;
+        (*pdwSpaceLeft) -= (pBlob) ? LWBUF_ALIGN_SIZE(dwBlobSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft) -= sizeof(PVOID);
     }
     else
     {
-        (*pdwSize) += dwBlobSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwBlobSize);
     }
 
     /* include size of the pointer */
