@@ -170,26 +170,6 @@ NetrAllocateDomainTrustInfo(
     );
 
 
-NTSTATUS
-NetrInitMemory(
-    VOID
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
-NTSTATUS
-NetrDestroyMemory(
-    VOID
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
 NTSTATUS
 NetrAllocateMemory(
     OUT PVOID *ppOut,
@@ -550,14 +530,14 @@ NetrAllocateChallengeResponse(
 
         ppRespData       = (PBYTE*)pCursor;
         *ppRespData      = pRespData;
-        (*pdwSpaceLeft) -= (pRespData) ? dwResponseSize : 0;
+        (*pdwSpaceLeft) -= (pRespData) ? LWBUF_ALIGN_SIZE(dwResponseSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(PBYTE);
     }
     else
     {
-        (*pdwSize) += dwResponseSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwResponseSize);
     }
 
     /* include size of the pointer */
@@ -697,7 +677,7 @@ NetrAllocateRidWithAttributeArray(
     DWORD iRid = 0;
     PVOID pRids = NULL;
     PRID_WITH_ATTRIBUTE *ppRids = NULL;
-
+    
     BAIL_ON_INVALID_PTR(pdwOffset, ntStatus);
     BAIL_ON_INVALID_PTR(pIn, ntStatus);
     BAIL_ON_INVALID_PTR(pdwSize, ntStatus);
@@ -746,14 +726,14 @@ NetrAllocateRidWithAttributeArray(
 
         ppRids           = (PRID_WITH_ATTRIBUTE*)pCursor;
         *ppRids          = (PRID_WITH_ATTRIBUTE)pRids;
-        (*pdwSpaceLeft) -= (pRids) ? dwRidsSize : 0;
+        (*pdwSpaceLeft) -= (pRids) ? LWBUF_ALIGN_SIZE(dwRidsSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(PRID_WITH_ATTRIBUTE);
     }
     else
     {
-        (*pdwSize) += dwRidsSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwRidsSize);
     }
 
     /* include size of the pointer */
@@ -891,14 +871,14 @@ NetrAllocateSamInfo3(
 
         ppSids           = (NetrSidAttr**)pCursor;
         *ppSids          = (NetrSidAttr*)pSids;
-        (*pdwSpaceLeft) -= (pSids) ? dwSidsSize : 0;
+        (*pdwSpaceLeft) -= (pSids) ? LWBUF_ALIGN_SIZE(dwSidsSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(NetrSidAttr*);
     }
     else
     {
-        (*pdwSize) += dwSidsSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwSidsSize);
     }
 
     /* include size of the pointer */
@@ -936,9 +916,10 @@ NetrAllocateSidAttr(
     BAIL_ON_INVALID_PTR(pdwOffset, ntStatus);
     BAIL_ON_INVALID_PTR(pIn, ntStatus);
     BAIL_ON_INVALID_PTR(pdwSize, ntStatus);
-
+    
     LWBUF_ALLOC_PSID(pBuffer, pIn->sid);
     LWBUF_ALLOC_DWORD(pBuffer, pIn->attribute);
+    LWBUF_ALIGN(pdwOffset, pdwSize, pdwSpaceLeft);
 
 cleanup:
     if (ntStatus == STATUS_SUCCESS &&
@@ -1156,14 +1137,14 @@ NetrAllocateValidationInfo(
 
         ppInfo           = pCursor;
         *ppInfo          = pInfo;
-        (*pdwSpaceLeft) -= (pInfo) ? dwInfoSize : 0;
+        (*pdwSpaceLeft) -= (pInfo) ? LWBUF_ALIGN_SIZE(dwInfoSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(PVOID);
     }
     else
     {
-        (*pdwSize) += dwInfoSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwInfoSize);
     }
 
     /* include size of the pointer */
@@ -1278,14 +1259,14 @@ NetrAllocateDomainInfo(
 
         ppInfo           = (NetrDomainInfo1**)pCursor;
         *ppInfo          = (NetrDomainInfo1*)pInfo;
-        (*pdwSpaceLeft) -= (pInfo) ? dwInfoSize : 0;
+        (*pdwSpaceLeft) -= (pInfo) ? LWBUF_ALIGN_SIZE(dwInfoSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(NetrDomainInfo1*);
     }
     else
     {
-        (*pdwSize) += dwInfoSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwInfoSize);
     }
 
     /* include size of the pointer */
@@ -1391,14 +1372,14 @@ NetrAllocateDomainInfo1(
 
         ppTrusts         = (NetrDomainTrustInfo**)pCursor;
         *ppTrusts        = (NetrDomainTrustInfo*)pTrusts;
-        (*pdwSpaceLeft) -= (pTrusts) ? dwTrustsSize : 0;
+        (*pdwSpaceLeft) -= (pTrusts) ? LWBUF_ALIGN_SIZE(dwTrustsSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft)  -= sizeof(NetrDomainTrustInfo*);
     }
     else
     {
-        (*pdwSize) += dwTrustsSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwTrustsSize);
     }
 
     /* include size of the pointer */
