@@ -113,26 +113,6 @@ SamrAllocateDisplayEntryAscii(
     );
 
 
-NTSTATUS
-SamrInitMemory(
-    void
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
-NTSTATUS
-SamrDestroyMemory(
-    void
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    return ntStatus;
-}
-
-
 NTSTATUS
 SamrAllocateMemory(
     OUT PVOID  *ppOut,
@@ -513,7 +493,7 @@ SamrAllocateLogonHours(
 
             dwBlobSpaceLeft = dwLogonHoursSize;
             dwBlobOffset    = 0;
-
+            
             /* logon hours blob */
             dwError  = LwBufferAllocFixedBlob(pLogonHours,
                                               &dwBlobOffset,
@@ -526,14 +506,14 @@ SamrAllocateLogonHours(
 
         ppLogonHours     = (PVOID*)pCursor;
         *ppLogonHours    = pLogonHours;
-        (*pdwSpaceLeft) -= (pLogonHours) ? dwLogonHoursSize : 0;
+        (*pdwSpaceLeft) -= (pLogonHours) ? LWBUF_ALIGN_SIZE(dwLogonHoursSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft) -= sizeof(PVOID);
     }
     else
     {
-        (*pdwSize) += dwLogonHoursSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwLogonHoursSize);
     }
 
     /* include size of the pointer */
@@ -723,7 +703,6 @@ SamrAllocateDomainInfo(
     default:
         ntStatus = STATUS_INVALID_INFO_CLASS;
         BAIL_ON_NT_STATUS(ntStatus);
-        goto error;
     }
 
 cleanup:
@@ -1045,14 +1024,14 @@ SamrAllocateUserInfo21(
 
         ppBufBlob        = (PVOID*)pCursor;
         *ppBufBlob       = pBufBlob;
-        (*pdwSpaceLeft) -= (pBufBlob) ? dwBufBlobSize : 0;
+        (*pdwSpaceLeft) -= (pBufBlob) ? LWBUF_ALIGN_SIZE(dwBufBlobSize) : 0;
 
         /* recalculate space after setting the pointer */
         (*pdwSpaceLeft) -= sizeof(PVOID);
     }
     else
     {
-        (*pdwSize) += dwBufBlobSize;
+        (*pdwSize) += LWBUF_ALIGN_SIZE(dwBufBlobSize);
     }
 
     /* include size of the pointer */
