@@ -260,6 +260,7 @@ static LWMsgTypeSpec gLsaUserAddInfoSpec[] =
     LWMSG_MEMBER_UINT32(LSA_USER_ADD_INFO, uid),
     LWMSG_MEMBER_PSTR(LSA_USER_ADD_INFO, pszPrimaryGroupSid),
     LWMSG_MEMBER_PSTR(LSA_USER_ADD_INFO, pszPassword),
+    LWMSG_ATTR_SENSITIVE,
     LWMSG_MEMBER_PSTR(LSA_USER_ADD_INFO, pszGecos),
     LWMSG_MEMBER_PSTR(LSA_USER_ADD_INFO, pszShell),
     LWMSG_MEMBER_PSTR(LSA_USER_ADD_INFO, pszHomedir),
@@ -341,6 +342,7 @@ static LWMsgTypeSpec gLsaIPCUserModInfo2Spec[] =
     LWMSG_MEMBER_PSTR(LSA_USER_MOD_INFO_2, pszShell),
     LWMSG_MEMBER_PSTR(LSA_USER_MOD_INFO_2, pszGecos),
     LWMSG_MEMBER_PSTR(LSA_USER_MOD_INFO_2, pszPassword),
+    LWMSG_ATTR_SENSITIVE,
     LWMSG_MEMBER_POINTER_BEGIN(LSA_USER_MOD_INFO_2, pNtPasswordHash),
     LWMSG_TYPESPEC(gLsaIPCDataBlobSpec),
     LWMSG_POINTER_END,
@@ -728,6 +730,12 @@ static LWMsgTypeSpec gLsaIPCGetMetricsReqSpec[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gLsaIPCGetStatusReqSpec[] =
+{
+    LWMSG_PSTR,
+    LWMSG_TYPE_END
+};
+
 static LWMsgTypeSpec gLsaIPCCheckUserInListReqSpec[] =
 {
     LWMSG_STRUCT_BEGIN(LSA_IPC_CHECK_USER_IN_LIST_REQ),
@@ -774,6 +782,7 @@ static LWMsgTypeSpec gLsaIPCAuthClearTextParamSpec[] =
 {
     LWMSG_STRUCT_BEGIN(LSA_AUTH_CLEARTEXT_PARAM),
     LWMSG_MEMBER_PSTR(LSA_AUTH_CLEARTEXT_PARAM, pszPassword),
+    LWMSG_ATTR_SENSITIVE,
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
@@ -815,9 +824,10 @@ static LWMsgTypeSpec gLsaIPCAuthUserParamSpec[] =
 
 static LWMsgTypeSpec gLsaIPCAuthUserExReqSpec[] =
 {
-    LWMSG_POINTER_BEGIN,
-    LWMSG_TYPESPEC(gLsaIPCAuthUserParamSpec),
-    LWMSG_POINTER_END,
+    LWMSG_STRUCT_BEGIN(LSA_IPC_AUTH_USER_EX_REQ),
+    LWMSG_MEMBER_PSTR(LSA_IPC_AUTH_USER_EX_REQ, pszTargetProvider),
+    LWMSG_MEMBER_TYPESPEC(LSA_IPC_AUTH_USER_EX_REQ, authUserParams, gLsaIPCAuthUserParamSpec),
+    LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
 
@@ -1311,7 +1321,7 @@ static LWMsgProtocolSpec gLsaIPCSpec[] =
     LWMSG_MESSAGE(LSA_Q_GET_METRICS, gLsaIPCGetMetricsReqSpec),
     LWMSG_MESSAGE(LSA_R_GET_METRICS_SUCCESS, gLsaMetricPackSpec),
     LWMSG_MESSAGE(LSA_R_GET_METRICS_FAILURE, gLsaIPCErrorSpec),
-    LWMSG_MESSAGE(LSA_Q_GET_STATUS, NULL),
+    LWMSG_MESSAGE(LSA_Q_GET_STATUS, gLsaIPCGetStatusReqSpec),
     LWMSG_MESSAGE(LSA_R_GET_STATUS_SUCCESS, gLsaStatusPtrSpec),
     LWMSG_MESSAGE(LSA_R_GET_STATUS_FAILURE, gLsaIPCErrorSpec),
     LWMSG_MESSAGE(LSA_Q_REFRESH_CONFIGURATION, NULL),
