@@ -866,13 +866,14 @@ LsaGetSymlinkTarget(
 {
     DWORD dwError = 0;
     CHAR szBuf[PATH_MAX+1];
+    ssize_t len = 0;
     PSTR pszTargetPath = NULL;
 
     memset(szBuf, 0, sizeof(szBuf));
 
     while (1) {
 
-       if (readlink(pszPath, szBuf, PATH_MAX) < 0) {
+       if ((len = readlink(pszPath, szBuf, PATH_MAX)) < 0) {
           if (errno == EINTR)
              continue;
 
@@ -882,7 +883,7 @@ LsaGetSymlinkTarget(
 
        break;
     }
-
+    szBuf[len] = '\0';
     dwError = LwAllocateString(
                     szBuf,
                     &pszTargetPath);
@@ -916,7 +917,7 @@ LsaCreateSymlink(
 }
 
 DWORD
-LsaCopyDirectory(
+LsaCopySkeletonDirectory(
     PCSTR pszSourceDirPath,
     uid_t ownerUid,
     gid_t ownerGid,
@@ -952,7 +953,18 @@ LsaCopyDirectory(
             BAIL_ON_LSA_ERROR(dwError);
         }
 
+#ifdef __LWI_FREEBSD__
+        if (!strncmp(pDirEntry->d_name, "dot.", 4) && pDirEntry->d_name[4] != '\0' )
+        {
+            sprintf(szDstPath, "%s/%s", pszDestDirPath, pDirEntry->d_name + 3);
+        }
+        else
+        {
+            sprintf(szDstPath, "%s/%s", pszDestDirPath, pDirEntry->d_name);
+        }
+#else
         sprintf(szDstPath, "%s/%s", pszDestDirPath, pDirEntry->d_name);
+#endif
 
         if (S_ISDIR(statbuf.st_mode)) {
 
@@ -967,7 +979,7 @@ LsaCopyDirectory(
                             ownerGid);
             BAIL_ON_LSA_ERROR(dwError);
 
-            dwError = LsaCopyDirectory(
+            dwError = LsaCopySkeletonDirectory(
                             szSrcPath,
                             ownerUid,
                             ownerGid,
