@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -42,7 +42,7 @@
  *        Private Includes (Utils)
  *
  * Authors: Sriram Nambakam (snambakam@likewisesoftware.com)
- *
+ * 
  */
 
 #include <config.h>
@@ -60,12 +60,10 @@
 #include <lsautils.h>
 
 #include "lsalogger_p.h"
-#include "lsahash_p.h"
 #include "sysfuncs_p.h"
 #include "filelog_p.h"
 #include "consolelog_p.h"
 #include "syslog_p.h"
-#include "lsadns_p.h"
 #include "lsaerror_p.h"
 #include "lsasecurityidentifier_p.h"
 #include "lsaartefactinfo_p.h"
