@@ -1421,6 +1421,7 @@ LsaFreeAuthUserParams(
 LW_DWORD
 LsaAuthenticateUserEx(
     LW_IN LW_HANDLE hLsaConnection,
+    LW_IN LW_PCSTR pszTargetProvider,
     LW_IN LSA_AUTH_USER_PARAMS* pParams,
     LW_OUT PLSA_AUTH_USER_INFO* ppUserInfo
     );
@@ -1491,6 +1492,13 @@ LsaGetStatus(
     PLSASTATUS* ppLsaStatus
     );
 
+LW_DWORD
+LsaGetStatus2(
+    LW_HANDLE hLsaConnection,
+    LW_IN LW_PCSTR pszTargetProvider,
+    PLSASTATUS* ppLsaStatus
+    );
+
 LW_DWORD
 LsaRefreshConfiguration(
     LW_HANDLE hLsaConnection
@@ -1568,7 +1576,7 @@ LsaDataBlobAllocate(
     LW_DWORD dwSize
     );
 
-LW_DWORD
+LW_VOID
 LsaDataBlobFree(
     PLW_LSA_DATA_BLOB* ppBlob
     );
@@ -2004,6 +2012,13 @@ LsaGetSmartCardUserObject(
     LW_OUT LW_PSTR* ppszSmartCardReader
     );
 
+//
+// Provider Tags
+//
+
+#define LSA_PROVIDER_TAG_LOCAL "lsa-local-provider"
+#define LSA_PROVIDER_TAG_AD "lsa-activedirectory-provider"
+
 #endif /* __LSA_H__ */
 
 
