@@ -340,39 +340,6 @@ typedef struct __LSA_BIT_VECTOR
     PDWORD pVector;
 } LSA_BIT_VECTOR, *PLSA_BIT_VECTOR;
 
-typedef struct __LSA_HASH_ENTRY LSA_HASH_ENTRY;
-
-typedef int (*LSA_HASH_KEY_COMPARE)(PCVOID, PCVOID);
-typedef size_t (*LSA_HASH_KEY)(PCVOID);
-typedef void (*LSA_HASH_FREE_ENTRY)(const LSA_HASH_ENTRY *);
-typedef DWORD (*LSA_HASH_COPY_ENTRY)(const LSA_HASH_ENTRY *, LSA_HASH_ENTRY *);
-
-struct __LSA_HASH_ENTRY
-{
-    PVOID pKey;
-    PVOID pValue;
-    //Next value in chain
-    struct __LSA_HASH_ENTRY *pNext;
-};
-
-typedef struct __LSA_HASH_TABLE
-{
-    size_t sTableSize;
-    size_t sCount;
-    LSA_HASH_ENTRY **ppEntries;
-    LSA_HASH_KEY_COMPARE fnComparator;
-    LSA_HASH_KEY fnHash;
-    LSA_HASH_FREE_ENTRY fnFree;
-    LSA_HASH_COPY_ENTRY fnCopy;
-} LSA_HASH_TABLE, *PLSA_HASH_TABLE;
-
-typedef struct __LSA_HASH_ITERATOR
-{
-    LSA_HASH_TABLE *pTable;
-    size_t sEntryIndex;
-    LSA_HASH_ENTRY *pEntryPos;
-} LSA_HASH_ITERATOR;
-
 typedef struct __DLINKEDLIST
 {
     PVOID pItem;
@@ -639,109 +606,6 @@ LsaStackFree(
     PLSA_STACK pStack
     );
 
-DWORD
-LsaHashCreate(
-    size_t sTableSize,
-    LSA_HASH_KEY_COMPARE fnComparator,
-    LSA_HASH_KEY fnHash,
-    LSA_HASH_FREE_ENTRY fnFree, //optional
-    LSA_HASH_COPY_ENTRY fnCopy, //optional
-    LSA_HASH_TABLE** ppResult
-    );
-
-size_t
-LsaHashGetKeyCount(
-    PLSA_HASH_TABLE pTable
-    );
-
-void
-LsaHashRemoveAll(
-        LSA_HASH_TABLE* pResult);
-
-void
-LsaHashSafeFree(
-    LSA_HASH_TABLE** ppResult
-    );
-
-DWORD
-LsaHashSetValue(
-    LSA_HASH_TABLE *pTable,
-    PVOID  pKey,
-    PVOID  pValue
-    );
-
-//Returns ERROR_NOT_FOUND if pKey is not in the table
-DWORD
-LsaHashGetValue(
-    LSA_HASH_TABLE *pTable,
-    PCVOID  pKey,
-    PVOID* ppValue
-    );
-
-BOOLEAN
-LsaHashExists(
-    IN PLSA_HASH_TABLE pTable,
-    IN PCVOID pKey
-    );
-
-DWORD
-LsaHashCopy(
-    IN  LSA_HASH_TABLE *pTable,
-    OUT LSA_HASH_TABLE **ppResult
-    );
-
-//Invalidates all iterators
-DWORD
-LsaHashResize(
-    LSA_HASH_TABLE *pTable,
-    size_t sTableSize
-    );
-
-DWORD
-LsaHashGetIterator(
-    LSA_HASH_TABLE *pTable,
-    LSA_HASH_ITERATOR *pIterator
-    );
-
-// returns NULL after passing the last entry
-LSA_HASH_ENTRY *
-LsaHashNext(
-    LSA_HASH_ITERATOR *pIterator
-    );
-
-DWORD
-LsaHashRemoveKey(
-    LSA_HASH_TABLE *pTable,
-    PVOID  pKey
-    );
-
-int
-LsaHashCaselessStringCompare(
-    PCVOID str1,
-    PCVOID str2
-    );
-
-size_t
-LsaHashCaselessStringHash(
-    PCVOID str
-    );
-
-int
-LsaHashPVoidCompare(
-    IN PCVOID pvData1,
-    IN PCVOID pvData2
-    );
-
-size_t
-LsaHashPVoidHash(
-    IN PCVOID pvData
-    );
-
-VOID
-LsaHashFreeStringKey(
-    IN OUT const LSA_HASH_ENTRY *pEntry
-    );
-
 DWORD
 LsaRemoveFile(
     PCSTR pszPath
@@ -814,7 +678,7 @@ LsaRemoveDirectory(
     );
 
 DWORD
-LsaCopyDirectory(
+LsaCopySkeletonDirectory(
     PCSTR pszSourceDirPath,
     uid_t ownerUid,
     gid_t ownerGid,
