@@ -48,11 +48,6 @@
 #define _RPC_NETLOGON_H_
 
 
-#define NETLOGON_DEFAULT_PROT_SEQ   "ncacn_np"
-#define NETLOGON_DEFAULT_ENDPOINT   "\\PIPE\\netlogon"
-#define NETLOGON_LOCAL_ENDPOINT     CACHEDIR "/rpc/lsass"
-
-
 /*
  * Domain trust definitions
  */
@@ -471,7 +466,7 @@ typedef union netr_domain_info {
 #define DS_DNS_CONTROLLER                (0x20000000)
 #define DS_DNS_DOMAIN                    (0x40000000)
 #define DS_DNS_FOREST                    (0x80000000)
-
+ 
 
 typedef struct dsr_dc_name_info {
 #ifdef _DCE_IDL_
@@ -517,8 +512,7 @@ NTSTATUS
 NetrInitBindingDefault(
     OUT PNETR_BINDING   phNetrBinding,
     IN  PCWSTR          pwszHostname,
-    IN  LW_PIO_CREDS    pCreds,
-    IN  BOOLEAN         bIsSchannel
+    IN  LW_PIO_CREDS    pCreds
     );
 
 
@@ -530,11 +524,16 @@ NetrInitBindingFull(
     IN  PCWSTR          pszEndpoint,
     IN  PCWSTR          pszUuid,
     IN  PCWSTR          pszOptions,
-    IN  LW_PIO_CREDS    pCreds,
-    IN  BOOLEAN         bIsSchannel
+    IN  LW_PIO_CREDS    pCreds
     );
 
 
+#define NetrInitBindingFromBindingString(binding_ptr, binding_str, creds_ptr) \
+    RpcInitBindingFromBindingString((handle_t*)(binding_ptr),                 \
+                                    (binding_str),                            \
+                                    (creds_ptr));
+
+
 VOID
 NetrFreeBinding(
     IN OUT PNETR_BINDING phNetrBinding
@@ -548,6 +547,7 @@ NetrOpenSchannel(
     IN  PCWSTR            pwszHostname,
     IN  PCWSTR            pwszServer,
     IN  PCWSTR            pwszDomain,
+    IN  PCWSTR            pwszFqdn,
     IN  PCWSTR            pwszComputer,
     IN  PCWSTR            pwszMachinePassword,
     IN  NetrCredentials  *pCreds,
@@ -762,18 +762,6 @@ NetrNTLMv1EncryptChallenge(
     );
 
 
-NTSTATUS
-NetrInitMemory(
-    VOID
-    );
-
-
-NTSTATUS
-NetrDestroyMemory(
-    VOID
-    );
-
-
 VOID
 NetrFreeMemory(
     IN PVOID pPtr
