@@ -39,7 +39,7 @@
  *
  *        Remote Procedure Call (RPC) Client Interface
  *
- *        Samr library
+ *        Samr rpc client library
  *
  * Authors: Rafal Szczesniak (rafal@likewise.com)
  */
@@ -49,10 +49,6 @@
 #define _RPC_SAMR_H_
 
 
-#define SAMR_DEFAULT_PROT_SEQ   "ncacn_np"
-#define SAMR_DEFAULT_ENDPOINT   "\\pipe\\samr"
-#define SAMR_LOCAL_ENDPOINT     CACHEDIR "/rpc/lsass"
-
 /* Connect access mask flags */
 #define SAMR_ACCESS_CONNECT_TO_SERVER          0x00000001
 #define SAMR_ACCESS_SHUTDOWN_SERVER            0x00000002
@@ -747,6 +743,12 @@ SamrInitBindingFull(
     );
 
 
+#define SamrInitBindingFromBindingString(binding_ptr, binding_str, creds_ptr) \
+    RpcInitBindingFromBindingString((handle_t*)(binding_ptr),                 \
+                                    (binding_str),                            \
+                                    (creds_ptr));
+
+
 VOID
 SamrFreeBinding(
     IN  PSAMR_BINDING  phBinding
@@ -1085,16 +1087,6 @@ SamrOpenGroup(
     );
 
 
-NTSTATUS
-SamrInitMemory(
-    VOID
-    );
-
-NTSTATUS
-SamrDestroyMemory(
-    VOID
-    );
-
 VOID
 SamrFreeMemory(
     PVOID pPtr
