@@ -118,17 +118,17 @@ typedef struct _SecPkgCred_Names
     SEC_CHAR *pUserName;
 } SecPkgCred_Names, *PSecPkgCred_Names;
 
-typedef struct _SecPkgCred
+typedef struct _SecPkgCred_DomainName
+{
+    SEC_CHAR *pName;
+} SecPkgCred_DomainName, *PSecPkgCred_DomainName;
+
+typedef union _SecPkgCred
 {
     PSecPkgCred_Names pNames;
+    PSecPkgCred_DomainName pDomainName;
 } SecPkgCred, *PSecPkgCred;
 
-typedef struct _LUID
-{
-    DWORD LowPart;
-    INT HighPart;
-} LUID, *PLUID;
-
 typedef struct _SEC_WINNT_AUTH_IDENTITY
 {
     PCHAR User;
@@ -156,7 +156,8 @@ typedef struct _NTLM_SEC_BUFFER
     USHORT usLength;
     USHORT usMaxLength;
     DWORD dwOffset;
-} NTLM_SEC_BUFFER, *PNTLM_SEC_BUFFER;
+} __attribute__((__packed__))
+NTLM_SEC_BUFFER, *PNTLM_SEC_BUFFER;
 
 typedef struct _WIN_VERSION_INFO
 {
@@ -164,7 +165,8 @@ typedef struct _WIN_VERSION_INFO
     BYTE bMinor;
     SHORT sBuild;
     DWORD dwReserved;
-} WIN_VERSION_INFO, *PWIN_VERSION_INFO;
+} __attribute__((__packed__))
+WIN_VERSION_INFO, *PWIN_VERSION_INFO;
 
 struct _NTLM_CONTEXT;
 typedef struct _NTLM_CONTEXT *NTLM_CONTEXT_HANDLE, **PNTLM_CONTEXT_HANDLE;
@@ -202,7 +204,8 @@ typedef struct
             DWORD dwMsgSeqNum;
         } v2;
     };
-} NTLM_SIGNATURE, *PNTLM_SIGNATURE;
+} __attribute__((__packed__))
+NTLM_SIGNATURE, *PNTLM_SIGNATURE;
 
 //******************************************************************************
 //
@@ -260,18 +263,22 @@ typedef struct
 #define ISC_REQ_INTEGRITY           0x01
 #define ISC_REQ_CONFIDENTIALITY     0x02
 #define ISC_REQ_NULL_SESSION        0x04
+#define ISC_REQ_USE_DCE_STYLE       0x08
 
 #define ISC_RET_INTEGRITY           0x01
 #define ISC_RET_CONFIDENTIALITY     0x02
 #define ISC_RET_NULL_SESSION        0x04
+#define ISC_RET_USE_DCE_STYLE       0x08
 
 #define ASC_REQ_INTEGRITY           0x01
 #define ASC_REQ_CONFIDENTIALITY     0x02
 #define ASC_REQ_NULL_SESSION        0x04
+#define ASC_REQ_USE_DCE_STYLE       0x08
 
 #define ASC_RET_INTEGRITY           0x01
 #define ASC_RET_CONFIDENTIALITY     0x02
 #define ASC_RET_NULL_SESSION        0x04
+#define ASC_RET_USE_DCE_STYLE       0x08
 
 // Possible information to query our context for
 #define SECPKG_ATTR_ACCESS_TOKEN                1
@@ -298,6 +305,7 @@ typedef struct
 #define SECPKG_ATTR_CIPHER_STRENGTHS            21
 #define SECPKG_ATTR_SUPPORTED_PROTOCOLS         22
 #define SECPKG_ATTR_PAC_LOGON_INFO              100
+#define SECPKG_CRED_ATTR_DOMAIN_NAME            101
 
 //******************************************************************************
 //
@@ -413,6 +421,13 @@ NtlmClientQueryContextAttributes(
     OUT PVOID pBuffer
     );
 
+DWORD
+NtlmClientSetCredentialsAttributes(
+    IN PNTLM_CRED_HANDLE phCredential,
+    IN DWORD ulAttribute,
+    IN PVOID pBuffer
+    );
+
 DWORD
 NtlmClientVerifySignature(
     IN PNTLM_CONTEXT_HANDLE phContext,
