@@ -86,21 +86,6 @@ pam_sm_acct_mgmt(
                     &pPamContext);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (pPamContext->bPasswordExpired)
-    {
-        // If during pam_sm_authenticate,
-        // we detected that the password expired,
-        // we handle it here
-        pPamContext->bPasswordExpired = FALSE;
-        dwError = LW_ERROR_PASSWORD_EXPIRED;
-        LsaPamConverse(
-            pamh,
-            "Your password has expired",
-            PAM_ERROR_MSG,
-            NULL);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
     dwError = LsaPamGetLoginId(
                     pamh,
                     pPamContext,
@@ -118,18 +103,14 @@ pam_sm_acct_mgmt(
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaValidateUser(
-                    hLsaConnection,
-                    pszLoginId,
-                    NULL);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = LsaCheckUserInList(
                     hLsaConnection,
                     pszLoginId,
                     NULL);
     if (dwError)
     {
+        LSA_LOG_PAM_ERROR("User %s is denied access because they are not in the 'require membership of' list",
+                          LSA_SAFE_LOG_STRING(pszLoginId));
         if (!LW_IS_NULL_OR_EMPTY_STR(pConfig->pszAccessDeniedMessage))
         {
             LsaPamConverse(pamh,
@@ -140,6 +121,35 @@ pam_sm_acct_mgmt(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    dwError = LsaValidateUser(
+                    hLsaConnection,
+                    pszLoginId,
+                    NULL);
+    if (dwError == LW_ERROR_PASSWORD_EXPIRED)
+    {
+        dwError = 0;
+        pPamContext->bPasswordExpired = TRUE;
+    }
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (pPamContext->bPasswordExpired)
+    {
+        // If during pam_sm_authenticate,
+        // we detected that the password expired,
+        // we handle it here
+        if (!pPamContext->bPasswordMessageShown)
+        {
+            LsaPamConverse(
+                pamh,
+                "Your password has expired",
+                PAM_ERROR_MSG,
+                NULL);
+            pPamContext->bPasswordMessageShown = TRUE;
+        }
+        dwError = LW_ERROR_PASSWORD_EXPIRED;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     iPamError = pam_get_data(
         pamh,
         PAM_LSASS_EXPIRE_WARNING_DONE,
@@ -154,7 +164,7 @@ pam_sm_acct_mgmt(
         BAIL_ON_LSA_ERROR(dwError);
 
         if (pUserInfo->bPromptPasswordChange == TRUE &&
-            pUserInfo->bPasswordExpired == FALSE &&
+            pUserInfo->bPasswordExpired == FALSE && 
             pUserInfo->bPasswordNeverExpires == FALSE) {
 
             CHAR szMessage[512];
