@@ -93,12 +93,59 @@ pam_sm_authenticate(
     if (pPamContext->pamOptions.bSetDefaultRepository)
     {
 #ifdef HAVE_STRUCT_PAM_REPOSITORY
+        BOOLEAN bChangeRepository = FALSE;
         struct pam_repository *currentRepository = NULL;
         pam_get_item(
                 pamh,
                 PAM_REPOSITORY,
                 (PAM_GET_ITEM_TYPE)&currentRepository);
         if (currentRepository == NULL)
+        {
+            if (!pPamContext->pamOptions.bLsassUsersOnly)
+            {
+                bChangeRepository = TRUE;
+            }
+            else
+            {
+                dwError = LsaPamGetLoginId(
+                                pamh,
+                                pPamContext,
+                                &pszLoginId,
+                                TRUE);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                dwError = LsaOpenServer(&hLsaConnection);
+
+                if (dwError == ERROR_SUCCESS)
+                {
+                    dwError = LsaFindUserByName(
+                                    hLsaConnection,
+                                    pszLoginId,
+                                    dwUserInfoLevel,
+                                    (PVOID*)&pUserInfo);
+                    if (dwError == ERROR_SUCCESS)
+                    {
+                        bChangeRepository = FALSE;
+                    }
+                    if (dwError == LW_ERROR_NO_SUCH_USER ||
+                            dwError == LW_ERROR_NOT_HANDLED)
+                    {
+                        LSA_LOG_PAM_INFO("Not setting pam repository for unknown user %s", LSA_SAFE_LOG_STRING(pszLoginId));
+                        dwError = 0;
+                    }
+                    else
+                    {
+                        LSA_LOG_PAM_INFO("Error %d looking up user %s", dwError, LSA_SAFE_LOG_STRING(pszLoginId));
+                        dwError = 0;
+                    }
+                }
+                else
+                {
+                    dwError = 0;
+                }
+            }
+        }
+        if (bChangeRepository)
         {
             struct pam_repository files = { "files", NULL, 0 };
             iPamError = pam_set_item(pamh, PAM_REPOSITORY, &files);
@@ -430,20 +477,25 @@ pam_sm_authenticate(
             pPamContext->bOnlineLogon = TRUE;
         }
 
-        dwError = LsaCheckUserInList(
-                        hLsaConnection,
-                        pszLoginId,
-                        NULL);
-        if (dwError)
+        if (!pPamContext->pamOptions.bNoRequireMembership)
         {
-            if (!LW_IS_NULL_OR_EMPTY_STR(pConfig->pszAccessDeniedMessage))
+            dwError = LsaCheckUserInList(
+                            hLsaConnection,
+                            pszLoginId,
+                            NULL);
+            if (dwError)
             {
-                LsaPamConverse(pamh,
-                               pConfig->pszAccessDeniedMessage,
-                               PAM_TEXT_INFO,
-                               NULL);
+                LSA_LOG_PAM_ERROR("User %s is denied access because they are not in the 'require membership of' list",
+                                  LSA_SAFE_LOG_STRING(pszLoginId));
+                if (!LW_IS_NULL_OR_EMPTY_STR(pConfig->pszAccessDeniedMessage))
+                {
+                    LsaPamConverse(pamh,
+                                   pConfig->pszAccessDeniedMessage,
+                                   PAM_TEXT_INFO,
+                                   NULL);
+                }
+                BAIL_ON_LSA_ERROR(dwError);
             }
-            BAIL_ON_LSA_ERROR(dwError);
         }
 
         /* On Centos, the pam_console module will grab the username from pam
@@ -453,7 +505,7 @@ pam_sm_authenticate(
          *
          * We know that the username can be found in AD, and that
          * their password matches the AD user's password. At this point, it
-         * is very unlikely that we will mangle a local username.
+         * is very unlikely that we will mangle a local username. 
          */
 
         dwError = LsaFindUserByName(
@@ -519,7 +571,7 @@ cleanup:
         LsaFreeUserInfo(dwUserInfoLevel, (PVOID)pUserInfo);
     }
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
     LW_SAFE_FREE_STRING(pszLoginId);
     LW_SAFE_FREE_STRING(pszSmartCardReader);
     LW_SAFE_FREE_STRING(pszPINPrompt);
