@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -36,11 +36,11 @@
  *        pam-context.c
  *
  * Abstract:
- *
+ * 
  *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ * 
  *        Pluggable Authentication Module (PAM)
- *
+ * 
  *        Likewise Context API
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -50,9 +50,9 @@
 
 DWORD
 LsaPamGetContext(
-    pam_handle_t* pamh,
-    int           flags,
-    int           argc,
+    pam_handle_t* pamh, 
+    int           flags, 
+    int           argc, 
     const char**  argv,
     PPAMCONTEXT*  ppPamContext
     )
@@ -61,9 +61,9 @@ LsaPamGetContext(
     PPAMCONTEXT pPamContext = NULL;
     BOOLEAN     bFreeContext = FALSE;
     int         iPamError = 0;
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamGetContext::begin");
-
+    
     iPamError = pam_get_data(pamh, MODULE_NAME, (PAM_GET_DATA_TYPE)&pPamContext);
     dwError = LsaPamUnmapErrorCode(iPamError);
     if (dwError)
@@ -95,7 +95,7 @@ LsaPamGetContext(
 
     dwError = LsaPamGetLoginId(pamh, pPamContext, NULL, FALSE);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaPamGetOptions(
                     pamh,
                     flags,
@@ -103,9 +103,9 @@ LsaPamGetContext(
                     argv,
                     &pPamContext->pamOptions);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     *ppPamContext = pPamContext;
-
+    
 cleanup:
 
     LSA_LOG_PAM_DEBUG("LsaPamGetContext::end");
@@ -119,7 +119,7 @@ error:
     }
 
     *ppPamContext = NULL;
-
+    
     LSA_LOG_PAM_ERROR("LsaPamGetContext failed [error code: %u]", dwError);
 
     goto cleanup;
@@ -136,11 +136,11 @@ LsaPamGetOptions(
 {
     DWORD dwError = 0;
     int iArg = 0;
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamGetOptions::begin");
 
     memset(pPamOptions, 0, sizeof(PAMOPTIONS));
-
+    
     for (; iArg < argc; iArg++)
     {
         if (!strcasecmp(argv[iArg], "try_first_pass"))
@@ -167,12 +167,20 @@ LsaPamGetOptions(
         {
             pPamOptions->bSetDefaultRepository = TRUE;
         }
+        else if (!strcasecmp(argv[iArg], "lsass_users_only"))
+        {
+            pPamOptions->bLsassUsersOnly = TRUE;
+        }
         else if (!strcasecmp(argv[iArg], "smartcard_prompt"))
         {
             pPamOptions->bSmartCardPrompt = TRUE;
         }
+        else if (!strcasecmp(argv[iArg], "no_require_membership"))
+        {
+            pPamOptions->bNoRequireMembership = TRUE;
+        }
     }
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamGetOptions::end");
 
     return dwError;
@@ -190,10 +198,10 @@ LsaPamGetLoginId(
     PSTR pszLoginId = NULL;
     PSTR pszPamId = NULL;
     int iPamError = 0;
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamGetLoginId::begin");
 
-
+        
     iPamError = pam_get_item(
                     pamh,
                     PAM_USER,
@@ -232,7 +240,7 @@ LsaPamGetLoginId(
                 pszPamId,
                 &pszLoginId);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     LW_SAFE_FREE_STRING(pPamContext->pszLoginName);
     dwError = LwStrDupOrNull(pszPamId, &pPamContext->pszLoginName);
     BAIL_ON_LSA_ERROR(dwError);
@@ -255,12 +263,12 @@ cleanup:
 error:
 
     LW_SAFE_FREE_STRING(pszLoginId);
-
+    
     if (ppszLoginId)
     {
        *ppszLoginId = NULL;
     }
-
+    
     LSA_LOG_PAM_ERROR("LsaPamGetLoginId failed [error code: %u]", dwError);
 
     goto cleanup;
@@ -274,14 +282,14 @@ LsaPamCleanupContext(
     )
 {
     LSA_LOG_PAM_DEBUG("LsaPamCleanupContext::begin");
-
+    
     if (pData) {
         LsaPamFreeContext((PPAMCONTEXT)pData);
     }
 
     // Syslog must be reset before this pam module is unloaded
     LsaPamCloseLog();
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamCleanupContext::end");
 }
 
@@ -291,11 +299,11 @@ LsaPamFreeContext(
     )
 {
     LSA_LOG_PAM_DEBUG("LsaPamFreeContext::begin");
-
+    
     LW_SAFE_FREE_STRING(pPamContext->pszLoginName);
-
+    
     LwFreeMemory(pPamContext);
-
+    
     LSA_LOG_PAM_DEBUG("LsaPamFreeContext::end");
 }
 
@@ -308,7 +316,7 @@ LsaPamCleanupDataString(
 {
     PSTR pStr = data;
 
-    LW_SAFE_CLEAR_FREE_STRING(pStr);
+    LW_SECURE_FREE_STRING(pStr);
 }
 
 DWORD
@@ -324,13 +332,13 @@ LsaPamSetDataString(
 
     dwError = LwAllocateString(pszStr, &pszStrCopy);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     iPamError = pam_set_data(pamh, pszKey, pszStrCopy, LsaPamCleanupDataString);
     dwError = LsaPamUnmapErrorCode(iPamError);
     BAIL_ON_LSA_ERROR(dwError);
 
 error:
-
+    
     return dwError;
 }
 
