@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -36,11 +36,11 @@
  *        pam-context.h
  *
  * Abstract:
- *
+ * 
  *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ * 
  *        Pluggable Authentication Module (PAM)
- *
+ * 
  *        Likewise Context API
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -52,16 +52,18 @@
 
 typedef struct _PAMOPTIONS
 {
-
+    
     BOOLEAN bTryFirstPass;
     BOOLEAN bUseFirstPass;
     BOOLEAN bUseAuthTok;
     BOOLEAN bRememberChPass;
     BOOLEAN bSetDefaultRepository;
+    BOOLEAN bLsassUsersOnly;
     BOOLEAN bUnknownOK;
     BOOLEAN bSmartCardPrompt;
     BOOLEAN bPromptGecos;
-
+    BOOLEAN bNoRequireMembership;
+    
 } PAMOPTIONS, *PPAMOPTIONS;
 
 typedef struct _PAMCONTEXT
@@ -69,17 +71,18 @@ typedef struct _PAMCONTEXT
     PSTR       pszLoginName;
     PAMOPTIONS pamOptions;
     BOOLEAN    bPasswordExpired;
+    BOOLEAN    bPasswordMessageShown;
     BOOLEAN    bPasswordChangeFailed;
     BOOLEAN    bPasswordChangeSuceeded;
     BOOLEAN    bOnlineLogon;
-
+    
 } PAMCONTEXT, *PPAMCONTEXT;
 
 DWORD
 LsaPamGetContext(
-    pam_handle_t* pamh,
-    int           flags,
-    int           argc,
+    pam_handle_t* pamh, 
+    int           flags, 
+    int           argc, 
     const char**  argv,
     PPAMCONTEXT*  ppContext
     );
