@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -36,11 +36,11 @@
  *        pam-conv.h
  *
  * Abstract:
- *
+ * 
  *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ * 
  *        Pluggable Authentication Module
- *
+ * 
  *        Application Conversations
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -79,7 +79,7 @@ LsaPamConverseMulti(
     struct pam_message** pMsgs = NULL;
     int    iPamError = 0;
     int    i;
-
+    
     iPamError = pam_get_item(pamh, PAM_CONV, (PAM_GET_ITEM_TYPE)&pConv);
     dwError = LsaPamUnmapErrorCode(iPamError);
     BAIL_ON_LSA_ERROR(dwError);
@@ -187,7 +187,7 @@ error:
     {
         for (i = 0; i < numMessages; ++i)
         {
-            LW_SAFE_CLEAR_FREE_STRING(pResponses[i].resp);
+            LW_SECURE_FREE_STRING(pResponses[i].resp);
         }
     }
 
