@@ -144,13 +144,13 @@ error:
 
     if ((dwError == LW_ERROR_NO_SUCH_USER) || (dwError == LW_ERROR_NOT_HANDLED))
     {
-        LSA_LOG_PAM_WARNING("LsaPamNotifyUserLogon failed [login:%s][error code: %u]",
+        LSA_LOG_PAM_WARNING("LsaPamNotifyUserLogon failed [login:%s][error code: %u]", 
                             LSA_SAFE_LOG_STRING(pszLoginId),
                             dwError);
     }
     else
     {
-        LSA_LOG_PAM_INFO("LsaPamNotifyUserLogon failed [login:%s][error code: %u]",
+        LSA_LOG_PAM_INFO("LsaPamNotifyUserLogon failed [login:%s][error code: %u]", 
                          LSA_SAFE_LOG_STRING(pszLoginId),
                          dwError);
     }
@@ -190,7 +190,7 @@ error:
     }
     else
     {
-        LSA_LOG_PAM_ERROR("LsaPamNotifyUserLogoff error [error code:%u]", dwError);
+        LSA_LOG_PAM_INFO("LsaPamNotifyUserLogoff error [error code:%u]", dwError);
     }
 
     goto cleanup;
@@ -203,7 +203,6 @@ GPInitLibrary(
     )
 {
     DWORD dwError = 0;
-    char szGPLibPath[256] = { 0 };
 
     /* Test to see if we are already setup */
     if (gbGPLibInitialized == TRUE)
@@ -213,11 +212,9 @@ GPInitLibrary(
 
     gbGPLibInitialized = TRUE;
 
-    sprintf(szGPLibPath, "%s/%s", LIBDIR, GPAPI_DLL_NAME);
-
     dlerror();
 
-    gpGPLibHandle = dlopen(szGPLibPath, RTLD_LAZY);
+    gpGPLibHandle = dlopen(LIBDIR "/" GPAPI_DLL_NAME, RTLD_LAZY);
     if (gpGPLibHandle == NULL)
     {
         dwError = LW_ERROR_LOAD_LIBRARY_FAILED;
