@@ -128,7 +128,7 @@ cleanup:
         with PAM_UPDATE_AUTHTOK; AIX will think the password change has
         finished and was successful. It seems like any error code other
         than PAM_SUCCESS will convince AIX to call back.
-
+ 
         On AIX 5.3 TL6, PAM_SUCCESS can be returned, however most other
         error codes (including PAM_NEW_AUTHTOK_REQD) will cause AIX to
         think an unrecoverable error occurred.
@@ -229,7 +229,7 @@ cleanup:
     }
 
     LW_SAFE_FREE_STRING(pszLoginId);
-    LW_SAFE_CLEAR_FREE_STRING(pszOldPassword);
+    LW_SECURE_FREE_STRING(pszOldPassword);
     LW_SAFE_FREE_STRING(pszMessage);
 
     LSA_LOG_PAM_DEBUG("LsaPamCheckCurrentPassword::end");
@@ -320,7 +320,10 @@ LsaPamUpdatePassword(
     PSTR   pszOldPassword = NULL;
     PSTR   pszPassword = NULL;
     PSTR   pszLoginId = NULL;
+    PCSTR  pszConstLoginId = NULL;
     HANDLE hLsaConnection = (HANDLE)NULL;
+    LSA_QUERY_LIST query = { NULL };
+    PLSA_SECURITY_OBJECT* ppUser = NULL;
 
     LSA_LOG_PAM_DEBUG("LsaPamUpdatePassword::begin");
 
@@ -346,6 +349,26 @@ LsaPamUpdatePassword(
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
 
+    // Verify the user is known to lsass
+    pszConstLoginId = pszLoginId;
+    query.ppszStrings = &pszConstLoginId;
+    dwError = LsaFindObjects(
+                    hLsaConnection,
+                    NULL,
+                    LSA_FIND_FLAGS_NSS,
+                    LSA_OBJECT_TYPE_USER,
+                    LSA_QUERY_TYPE_BY_NAME,
+                    1,
+                    query,
+                    &ppUser);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (!ppUser[0])
+    {
+        dwError = LW_ERROR_NO_SUCH_USER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LsaPamGetOldPassword(
                    pamh,
                    pPamContext,
@@ -367,9 +390,15 @@ LsaPamUpdatePassword(
 
 cleanup:
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword);
-    LW_SAFE_CLEAR_FREE_STRING(pszOldPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszOldPassword);
     LW_SAFE_FREE_STRING(pszLoginId);
+    if (ppUser)
+    {
+        LsaFreeSecurityObjectList(
+            1,
+            ppUser);
+    }
 
     if (hLsaConnection != (HANDLE)NULL)
     {
@@ -503,7 +532,7 @@ cleanup:
 
 error:
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
 
     *ppszPassword = NULL;
 
@@ -599,7 +628,7 @@ LsaPamGetOldPassword(
        /* HP-UX clears PAM_OLDAUTHTOK between the two phases of chauthtok, so
           save a copy of the old password where we can find it later */
 
-       /* For Solaris, we read PAM_LSASS_OLDAUTHTOK instead of
+       /* For Solaris, we read PAM_LSASS_OLDAUTHTOK instead of 
           PAM_OLDAUTHTOK. */
        dwError = LsaPamSetDataString(pamh, PAM_LSASS_OLDAUTHTOK, pszPassword);
        BAIL_ON_LSA_ERROR(dwError);
@@ -616,7 +645,7 @@ cleanup:
 
 error:
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
 
     *ppszPassword = NULL;
 
@@ -691,8 +720,8 @@ LsaPamGetNewPassword(
            LsaPamConverse(pamh, "Passwords do not match", PAM_ERROR_MSG, NULL);
            dwError = 0;
 
-           LW_SAFE_CLEAR_FREE_STRING(pszPassword_1);
-           LW_SAFE_CLEAR_FREE_STRING(pszPassword_2);
+           LW_SECURE_FREE_STRING(pszPassword_1);
+           LW_SECURE_FREE_STRING(pszPassword_2);
 
        }
        else
@@ -712,7 +741,7 @@ LsaPamGetNewPassword(
 
 cleanup:
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword_2);
+    LW_SECURE_FREE_STRING(pszPassword_2);
 
     LSA_LOG_PAM_DEBUG("LsaPamGetNewPassword::end");
 
@@ -720,7 +749,7 @@ cleanup:
 
 error:
 
-    LW_SAFE_CLEAR_FREE_STRING(pszPassword_1);
+    LW_SECURE_FREE_STRING(pszPassword_1);
 
     *ppszPassword = NULL;
 
