@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright (c) Likewise Software.  All rights Reserved.
@@ -74,6 +74,12 @@ gss_OID_desc gGssCredOptionPasswordOidDesc = {
     };
 gss_OID gGssCredOptionPasswordOid = &gGssCredOptionPasswordOidDesc;
 
+gss_OID_desc gGssCredOptionDomainOidDesc = {
+    .length = GSS_CRED_OPT_DOMAIN_LEN,
+    .elements = GSS_CRED_OPT_DOMAIN
+    };
+gss_OID gGssCredOptionDomainOid = &gGssCredOptionDomainOidDesc;
+
 //
 // Since there is no GSSAPI mech plugin header, this must be kept in sync with
 // gss_mechanism in krb5/src/lib/gssapi/mglueP.h.
@@ -489,7 +495,7 @@ typedef struct _GSS_MECH_CONFIG
         gss_OID *,
         gss_buffer_set_t *
         );
-
+    
     OM_uint32
     (*gss_get_name_attribute)(
         OM_uint32 *,
@@ -540,7 +546,7 @@ typedef struct _GSS_MECH_CONFIG
         gss_name_t,
         gss_buffer_t,
         gss_any_t *
-        );
+        );    
 } GSS_MECH_CONFIG, *PGSS_MECH_CONFIG;
 
 typedef struct _NTLM_GSS_NAME
@@ -853,13 +859,13 @@ ntlm_gss_init_sec_context(
 
     InputToken.BufferType = SECBUFFER_TOKEN;
 
-    if(pInputToken)
+    if (pInputToken)
     {
         InputToken.cbBuffer = pInputToken->length;
         InputToken.pvBuffer = pInputToken->value;
     }
 
-    if( pContextHandle)
+    if (pContextHandle)
     {
         hContext = (NTLM_CONTEXT_HANDLE)*pContextHandle;
     }
@@ -913,6 +919,10 @@ ntlm_gss_init_sec_context(
     {
         dwNtlmFlags |= ISC_REQ_NULL_SESSION;
     }
+    if (nReqFlags & GSS_C_DCE_STYLE)
+    {
+        dwNtlmFlags |= ISC_REQ_USE_DCE_STYLE;
+    }
 
     MinorStatus = NtlmClientInitializeSecurityContext(
         &CredHandle,
@@ -950,6 +960,10 @@ ntlm_gss_init_sec_context(
     {
         RetFlags |= GSS_C_ANON_FLAG;
     }
+    if (dwOutNtlmFlags & ISC_RET_USE_DCE_STYLE)
+    {
+        RetFlags |= GSS_C_DCE_STYLE;
+    }
 
 cleanup:
     *pMinorStatus = MinorStatus;
@@ -1604,12 +1618,23 @@ ntlm_gss_wrap_iov(
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
 
-    MinorStatus = NtlmClientEncryptMessage(
-        &ContextHandle,
-        nEncrypt ? TRUE : FALSE,
-        &Message,
-        0
-        );
+    if (nEncrypt)
+    {
+        MinorStatus = NtlmClientEncryptMessage(
+            &ContextHandle,
+            TRUE,
+            &Message,
+            0
+            );
+    }
+    else
+    {
+        MinorStatus = NtlmClientMakeSignature(
+            &ContextHandle,
+            Qop,
+            &Message,
+            0);
+    }
     BAIL_ON_LSA_ERROR(MinorStatus);
 
     if (nEncrypt)
@@ -1802,7 +1827,7 @@ ntlm_gss_unwrap(
     NtlmBuffer[1].cbBuffer = dwBufferSize;
     NtlmBuffer[1].pvBuffer = pBuffer;
 
-    if (dwNtlmFlags & (ISC_RET_CONFIDENTIALITY | ISC_RET_INTEGRITY))
+    if (dwNtlmFlags & ISC_RET_CONFIDENTIALITY)
     {
         MinorStatus = NtlmClientDecryptMessage(
             &ContextHandle,
@@ -1874,6 +1899,8 @@ ntlm_gss_unwrap_iov(
     BOOLEAN bEncrypted = FALSE;
     DWORD dwIndex = 0;
     BOOLEAN bFoundHeader = FALSE;
+    DWORD dwNtlmFlags = 0;
+    DWORD dwQop = GSS_C_QOP_DEFAULT;
 
     if (cBuffers < 2)
     {
@@ -1890,7 +1917,7 @@ ntlm_gss_unwrap_iov(
     Message.pBuffers = NtlmBuffer;
 
     for (dwIndex = 0 ; dwIndex < cBuffers ; dwIndex++)
-    {
+    {    
         switch (GSS_IOV_BUFFER_TYPE(pBuffers[dwIndex].type))
         {
             case GSS_IOV_BUFFER_TYPE_HEADER:
@@ -1923,12 +1950,36 @@ ntlm_gss_unwrap_iov(
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
 
-    MinorStatus = NtlmClientDecryptMessage(
+    // Check the negotiated flags to find out if the message is expected
+    // to be encrypted or signed only.
+    MinorStatus = NtlmClientQueryContextAttributes(
         &ContextHandle,
-        &Message,
-        0,
-        &bEncrypted
-        );
+        SECPKG_ATTR_FLAGS,
+        &dwNtlmFlags);
+    BAIL_ON_LSA_ERROR(MinorStatus);
+
+    if (dwNtlmFlags & ISC_RET_CONFIDENTIALITY)
+    {
+        MinorStatus = NtlmClientDecryptMessage(
+            &ContextHandle,
+            &Message,
+            0,
+            &bEncrypted
+            );
+    }
+    else if (dwNtlmFlags & ISC_RET_INTEGRITY)
+    {
+        MinorStatus = NtlmClientVerifySignature(
+            &ContextHandle,
+            &Message,
+            0,
+            &dwQop
+            );
+    }
+    else
+    {
+        MinorStatus = LW_ERROR_INVALID_PARAMETER;
+    }
     BAIL_ON_LSA_ERROR(MinorStatus);
 
 cleanup:
@@ -2012,7 +2063,7 @@ ntlm_gss_display_status(
     BAIL_ON_LSA_ERROR(MinorStatus);
 
     pMsg->length = strlen(pszCommonStr);
-
+    
 cleanup:
     if (*pMinorStatus)
     {
@@ -2470,7 +2521,7 @@ ntlm_gss_get_name_attribute(
                 SECPKG_ATTR_PAC_LOGON_INFO,
                 &LogonInfo);
             BAIL_ON_LSA_ERROR(MinorStatus);
-
+            
             pValue->value = LogonInfo.pLogonInfo;
             pValue->length = LogonInfo.LogonInfoLength;
         }
@@ -2479,7 +2530,7 @@ ntlm_gss_get_name_attribute(
         {
             *pAuthenticate = 1;
         }
-
+        
         if (pComplete)
         {
             *pComplete = 1;
@@ -2530,6 +2581,8 @@ ntlm_gssspi_set_cred_option(
     OM_uint32 MinorStatus = LW_ERROR_SUCCESS;
     PNTLM_GSS_CREDS pCreds = (PNTLM_GSS_CREDS)GssCredHandle;
     PSEC_WINNT_AUTH_IDENTITY pAuthData = NULL;
+    PSTR pszDomain = NULL;
+    SecPkgCred_DomainName spcDomainName = {0};
 
     if (OptionOid->length == gGssCredOptionPasswordOid->length &&
         !memcmp(
@@ -2567,8 +2620,39 @@ ntlm_gssspi_set_cred_option(
             );
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
+    else if (OptionOid->length == gGssCredOptionDomainOid->length &&
+        !memcmp(
+            OptionOid->elements,
+            gGssCredOptionDomainOid->elements,
+            OptionOid->length))
+    {
+        MajorStatus = GSS_S_COMPLETE;
+
+        if (!Buffer || !Buffer->length || !Buffer->value)
+        {
+            MinorStatus = LW_ERROR_INVALID_PARAMETER;
+            BAIL_ON_LSA_ERROR(MinorStatus);
+        }
+
+        MinorStatus = LwStrndup(
+                          Buffer->value,
+                          Buffer->length,
+                          &pszDomain);
+        BAIL_ON_LSA_ERROR(MinorStatus);
+
+        spcDomainName.pName = pszDomain;
+
+        MinorStatus = NtlmClientSetCredentialsAttributes(
+                          &pCreds->CredHandle,
+                          SECPKG_CRED_ATTR_DOMAIN_NAME,
+                          &spcDomainName);
+        BAIL_ON_LSA_ERROR(MinorStatus);
+    }
 
 cleanup:
+
+    LW_SAFE_FREE_STRING(pszDomain);
+
     *pMinorStatus = MinorStatus;
 
     return MajorStatus;
