@@ -63,6 +63,7 @@
 #include "lsautils.h"
 #include <assert.h>
 #include "lsass-calls.h"
+#include "lsalocalprovider.h"
 
 
 typedef struct _LW_MAP_SECURITY_PLUGIN_CONTEXT {
@@ -133,11 +134,7 @@ LsaMapSecurityCloseConnection(
 
     if (hConnection)
     {
-        NTSTATUS status = STATUS_SUCCESS;
-        DWORD dwError = LW_ERROR_SUCCESS;
-
-        dwError = LsaCloseServer(hConnection);
-        status = LsaLsaErrorToNtStatus(dwError);
+        (void) LsaCloseServer(hConnection);
 
         *phConnection = NULL;
     }
@@ -295,7 +292,7 @@ LsaMapSecurityResolveObjectInfo(
 
         objectInfo.Uid = ppObjects[0]->userInfo.uid;
         objectInfo.Gid = ppObjects[0]->userInfo.gid;
-
+        
         status = RtlAllocateSidFromCString(&objectInfo.Sid, ppObjects[0]->pszObjectSid);
         GOTO_CLEANUP_ON_STATUS(status);
 
@@ -423,29 +420,123 @@ LsaMapSecurityConstructSid(
 {
     NTSTATUS status = STATUS_SUCCESS;
     PSID pSid = NULL;
+    ULONG ulNewSidLength = RtlLengthRequiredSid(pDomainSid->SubAuthorityCount + 1);
 
-    status = RTL_ALLOCATE(&pSid, SID, SID_MAX_SIZE);
-    BAIL_ON_NT_STATUS(status);
+    status = RTL_ALLOCATE(&pSid, SID, ulNewSidLength);
+    GOTO_CLEANUP_ON_STATUS(status);
 
-    status = RtlCopySid(SID_MAX_SIZE, pSid, pDomainSid);
-    BAIL_ON_NT_STATUS(status);
+    status = RtlCopySid(ulNewSidLength, pSid, pDomainSid);
+    GOTO_CLEANUP_ON_STATUS(status);
 
-    status = RtlAppendRidSid(SID_MAX_SIZE, pSid, rid);
-    BAIL_ON_NT_STATUS(status);
+    status = RtlAppendRidSid(ulNewSidLength, pSid, rid);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     *ppSid = pSid;
 
 cleanup:
 
+    if (!NT_SUCCESS(status))
+    {
+        *ppSid = NULL;
+        RTL_FREE(&pSid);
+    }
+    
     return status;
+}
 
-error:
+static
+NTSTATUS
+LsaMapSecurityCompleteObjectInfoFromSid(
+    IN HANDLE hLsaConnection,
+    IN OUT PLSA_MAP_SECURITY_OBJECT_INFO pObjectInfo
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    DWORD dwError = LW_ERROR_SUCCESS;
+    PSTR pszSid = NULL;
+    PLSA_SECURITY_OBJECT* ppObjects = NULL;
+    LSA_QUERY_LIST QueryList;
 
-    *ppSid = NULL;
+    status = RtlAllocateCStringFromSid(&pszSid, pObjectInfo->Sid);
+    GOTO_CLEANUP_ON_STATUS(status);
 
-    RTL_FREE(&pSid);
+    QueryList.ppszStrings = (PCSTR*) &pszSid;
 
-    goto cleanup;
+    dwError = LsaFindObjects(
+                    hLsaConnection,
+                    NULL,
+                    0,
+                    LSA_OBJECT_TYPE_UNDEFINED,
+                    LSA_QUERY_TYPE_BY_SID,
+                    1,
+                    QueryList,
+                    &ppObjects);
+    status = LwWin32ErrorToNtStatus(dwError);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    SetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_IS_USER);
+
+    if (ppObjects[0] && ppObjects[0]->enabled)
+    {
+        assert(ppObjects[0]->type == LSA_OBJECT_TYPE_USER);
+            
+        SetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID);
+        SetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID);
+        
+        pObjectInfo->Uid = ppObjects[0]->userInfo.uid;
+        pObjectInfo->Gid = ppObjects[0]->userInfo.gid;
+    }
+
+cleanup:
+
+    LW_SAFE_FREE_STRING(pszSid);
+    LsaUtilFreeSecurityObjectList(1, ppObjects);
+
+    return status;
+}
+
+static
+NTSTATUS
+LsaMapSecurityResolveObjectInfoFromAuthUserInfo(
+    IN HANDLE hLsaConnection,
+    IN PLSA_AUTH_USER_INFO pAuthUserInfo,
+    OUT PLSA_MAP_SECURITY_OBJECT_INFO pObjectInfo
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    LSA_MAP_SECURITY_OBJECT_INFO objectInfo = { 0 };
+    PSID pDomainSid = NULL;
+
+    status = RtlAllocateSidFromCString(&pDomainSid, pAuthUserInfo->pszDomainSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // Sid and PrimaryGroupSid
+    status = LsaMapSecurityConstructSid(pDomainSid, 
+                                        pAuthUserInfo->dwUserRid, 
+                                        &objectInfo.Sid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = LsaMapSecurityConstructSid(pDomainSid, 
+                                        pAuthUserInfo->dwPrimaryGroupRid, 
+                                        &objectInfo.PrimaryGroupSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // The rest
+    status = LsaMapSecurityCompleteObjectInfoFromSid(hLsaConnection, &objectInfo);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+cleanup:
+
+    if (!NT_SUCCESS(status))
+    {
+        LsaMapSecurityFreeObjectInfo(&objectInfo);
+    }
+
+    RTL_FREE(&pDomainSid);
+
+    *pObjectInfo = objectInfo;
+
+    return status;
 }
 
 static
@@ -457,56 +548,26 @@ LsaMapSecurityResolveObjectInfoFromPac(
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    DWORD dwError = LW_ERROR_SUCCESS;
     LSA_MAP_SECURITY_OBJECT_INFO objectInfo = { 0 };
-    PSTR pszSid = NULL;
-    PLSA_SECURITY_OBJECT* ppObjects = NULL;
-    LSA_QUERY_LIST QueryList;
-    HANDLE hConnection = NULL;
-
-    status = LsaMapSecurityConstructSid(pPac->info3.base.domain_sid, pPac->info3.base.rid, &objectInfo.Sid);
+    HANDLE hLsaConnection = NULL;
+    
+    status = LsaMapSecurityConstructSid(
+        pPac->info3.base.domain_sid, 
+        pPac->info3.base.rid, 
+        &objectInfo.Sid);
     GOTO_CLEANUP_ON_STATUS(status);
 
-    status = RtlAllocateCStringFromSid(&pszSid, objectInfo.Sid);
-    GOTO_CLEANUP_ON_STATUS(status);
-
-    status = LsaMapSecurityOpenConnection(Context, &hConnection);
-    GOTO_CLEANUP_ON_STATUS(status);
-
-    QueryList.ppszStrings = (PCSTR*) &pszSid;
-
-    dwError = LsaFindObjects(
-                    hConnection,
-                    NULL,
-                    0,
-                    LSA_OBJECT_TYPE_UNDEFINED,
-                    LSA_QUERY_TYPE_BY_SID,
-                    1,
-                    QueryList,
-                    &ppObjects);
-    status = LwWin32ErrorToNtStatus(dwError);
-    GOTO_CLEANUP_ON_STATUS(status);
-
-    LsaMapSecurityCloseConnection(Context, &hConnection);
-
     status = LsaMapSecurityConstructSid(
         pPac->info3.base.domain_sid,
         pPac->info3.base.primary_gid,
         &objectInfo.PrimaryGroupSid);
     GOTO_CLEANUP_ON_STATUS(status);
 
-    SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_IS_USER);
+    status = LsaMapSecurityOpenConnection(Context, &hLsaConnection);
+    GOTO_CLEANUP_ON_STATUS(status);
 
-    if (ppObjects[0] && ppObjects[0]->enabled)
-    {
-        assert(ppObjects[0]->type == LSA_OBJECT_TYPE_USER);
-
-        SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID);
-        SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID);
-
-        objectInfo.Uid = ppObjects[0]->userInfo.uid;
-        objectInfo.Gid = ppObjects[0]->userInfo.gid;
-    }
+    status = LsaMapSecurityCompleteObjectInfoFromSid(hLsaConnection, &objectInfo);
+    GOTO_CLEANUP_ON_STATUS(status);
 
 cleanup:
 
@@ -515,9 +576,7 @@ cleanup:
         LsaMapSecurityFreeObjectInfo(&objectInfo);
     }
 
-    LW_SAFE_FREE_STRING(pszSid);
-
-    LsaUtilFreeSecurityObjectList(1, ppObjects);
+    LsaMapSecurityCloseConnection(Context, &hLsaConnection);
 
     *pObjectInfo = objectInfo;
 
@@ -985,9 +1044,9 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfo(
                 &group->Sid,
                 ppObjects[i]->pszObjectSid);
             GOTO_CLEANUP_ON_STATUS(status);
-
+            
             group->Attributes = SE_GROUP_ENABLED;
-
+            
             createInformation->Groups->GroupCount++;
         }
     }
@@ -1056,6 +1115,8 @@ cleanup:
         RTL_FREE(&extraGidSidList[i]);
     }
 
+    RTL_FREE(&pszSid);
+
     *CreateInformation = createInformation;
 
     return status;
@@ -1080,10 +1141,10 @@ LsaMapSecurityMergeStringLists(
     BOOLEAN bFound = FALSE;
 
     status = RTL_ALLOCATE(&ppszMergedStrings, PSTR, sizeof(PSTR) * (dwCount1 + dwCount2));
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     memcpy(ppszMergedStrings, ppszStrings1, sizeof(*ppszStrings1) * dwCount1);
-
+    
     for (dwSecondIndex = 0; dwSecondIndex < dwCount2; dwSecondIndex++)
     {
         bFound = FALSE;
@@ -1107,13 +1168,12 @@ LsaMapSecurityMergeStringLists(
 
 cleanup:
 
+    if (!NT_SUCCESS(status))
+    {
+        RTL_FREE(ppszMergedStrings);
+    }
+
     return status;
-
-error:
-
-    RTL_FREE(ppszMergedStrings);
-
-    goto cleanup;
 }
 
 static
@@ -1138,7 +1198,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
     DWORD dwSuppGroupCount = 0;
     PSID pPrimaryGidSid = NULL;
     PSTR pszPrimaryGidSid = NULL;
-
+    
     /* Allocate array for string forms of sids, plus extra slots for user sid and sid from primary UNIX gid */
     status = RTL_ALLOCATE(&ppszInputSids, PSTR, sizeof(*ppszInputSids) * (dwInputSidCount + 2));
     GOTO_CLEANUP_ON_STATUS(status);
@@ -1167,7 +1227,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
 
         status = RtlAllocateCStringFromSid(&pszPrimaryGidSid, pPrimaryGidSid);
         GOTO_CLEANUP_ON_STATUS(status);
-
+        
         for (i = 0; i < dwInputSidCount; i++)
         {
             if (!strcasecmp(ppszInputSids[i], pszPrimaryGidSid))
@@ -1175,7 +1235,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
                 break;
             }
         }
-
+         
         if (i == dwInputSidCount)
         {
             ppszInputSids[dwInputSidCount++] = pszPrimaryGidSid;
@@ -1227,7 +1287,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
     // TOKEN_UNIX
     //
 
-    if (IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID) &&
+    if (IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID) && 
         IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID))
     {
         createInformation->Unix->Uid = pObjectInfo->Uid;
@@ -1241,7 +1301,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
     {
         createInformation->Unix = NULL;
     }
-
+    
 
     //
     // TOKEN_USER
@@ -1262,9 +1322,9 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
             &group->Sid,
             ppszGroupSids[i]);
         GOTO_CLEANUP_ON_STATUS(status);
-
+        
         group->Attributes = SE_GROUP_ENABLED;
-
+        
         createInformation->Groups->GroupCount++;
     }
 
@@ -1417,9 +1477,9 @@ LsaMapSecurityGetPacInfoFromGssContext(
     if (majorStatus != GSS_S_COMPLETE)
     {
         status = STATUS_UNSUCCESSFUL;
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     majorStatus = gss_get_name_attribute(
         &minorStatus,
         srcName,
@@ -1436,25 +1496,34 @@ LsaMapSecurityGetPacInfoFromGssContext(
         // be fair we were always returning STATUS_UNSUCCESSFUL here before.
 
         status = LwWin32ErrorToNtStatus(minorStatus);
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     if (pacData.value == NULL)
     {
         status = STATUS_INVALID_USER_BUFFER;
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
     }
 
     status = DecodePacLogonInfo(
         pacData.value,
         pacData.length,
         &pPac);
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     *ppPac = pPac;
 
 cleanup:
 
+    if (!NT_SUCCESS(status))
+    {
+        *ppPac = NULL;
+        if (pPac)
+        {
+            FreePacLogonInfo(pPac);
+        }
+    }
+
     if (pacData.value)
     {
         gss_release_buffer(&minorStatus, &pacData);
@@ -1471,17 +1540,6 @@ cleanup:
     }
 
     return status;
-
-error:
-
-    *ppPac = NULL;
-
-    if (pPac)
-    {
-        FreePacLogonInfo(pPac);
-    }
-
-    goto cleanup;
 }
 
 static
@@ -1537,16 +1595,16 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
         Context,
         &pPac,
         GssContext);
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     status = LsaMapSecurityResolveObjectInfoFromPac(Context, pPac, &objectInfo);
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
-    status = RTL_ALLOCATE(&ppInputSids, PSID, sizeof(PSID) *
+    status = RTL_ALLOCATE(&ppInputSids, PSID, sizeof(PSID) * 
                           (pPac->info3.base.groups.dwCount +
                            pPac->res_groups.dwCount +
                            pPac->info3.sidcount));
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     for (dwIndex = 0; dwIndex < pPac->info3.base.groups.dwCount; dwIndex++)
     {
@@ -1554,7 +1612,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
             pPac->info3.base.domain_sid,
             pPac->info3.base.groups.pRids[dwIndex].dwRid,
             &pSid);
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
 
         ppInputSids[dwInputSidCount++] = pSid;
         pSid = NULL;
@@ -1566,16 +1624,16 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
             pPac->res_group_dom_sid,
             pPac->res_groups.pRids[dwIndex].dwRid,
             &pSid);
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
 
         ppInputSids[dwInputSidCount++] = pSid;
         pSid = NULL;
     }
-
+   
     for (dwIndex = 0; dwIndex < pPac->info3.sidcount; dwIndex++)
     {
         status = RtlDuplicateSid(&pSid, pPac->info3.sids[dwIndex].sid);
-        BAIL_ON_NT_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);
 
         ppInputSids[dwInputSidCount++] = pSid;
         pSid = NULL;
@@ -1587,7 +1645,7 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
         &objectInfo,
         dwInputSidCount,
         ppInputSids);
-    BAIL_ON_NT_STATUS(status);
+    GOTO_CLEANUP_ON_STATUS(status);
 
 cleanup:
 
@@ -1605,13 +1663,9 @@ cleanup:
         FreePacLogonInfo(pPac);
     }
 
-    LsaMapSecurityFreeObjectInfo(&objectInfo);
-
+    LsaMapSecurityFreeObjectInfo(&objectInfo);    
+    
     return status;
-
-error:
-
-    goto cleanup;
 }
 
 static
@@ -1629,6 +1683,356 @@ LsaMapSecurityFreeContext(
     }
 }
 
+static
+VOID
+LsaMapSecurityFreeNtlmLogonResult(
+    IN PLW_MAP_SECURITY_PLUGIN_CONTEXT Context,
+    IN OUT PLW_MAP_SECURITY_NTLM_LOGON_RESULT* ppNtlmResult
+    )
+{
+    PLW_MAP_SECURITY_NTLM_LOGON_RESULT pNtlmResult = *ppNtlmResult;
+
+    if (pNtlmResult)
+    {
+        if (pNtlmResult->pszUsername)
+        {
+            LwRtlCStringFree(&pNtlmResult->pszUsername);
+        }
+        RTL_FREE(&pNtlmResult);
+
+        *ppNtlmResult = pNtlmResult;
+    }
+}
+
+static
+NTSTATUS
+LsaMapSecurityGetGuestAuthUserInfo(
+    IN HANDLE hLsaConnection,
+    OUT PLSA_AUTH_USER_INFO* ppUserInfo
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    DWORD dwError = LW_ERROR_SUCCESS;
+    PSTR pszQueryName = "Guest";
+    LSA_QUERY_LIST QueryList = {0};
+    PLSA_SECURITY_OBJECT* ppObjects = NULL;
+    DWORD dwGroupCount = 0;
+    PSTR* ppszGroupSids = NULL;
+    LSA_AUTH_USER_INFO userInfo = { 0 };
+    PLSA_AUTH_USER_INFO pUserInfo = NULL;
+    PSID pSid = NULL;
+    PSID pDomainSid = NULL;
+    int i = 0;
+
+    QueryList.ppszStrings = (PCSTR*)&pszQueryName;
+
+    dwError = LsaFindObjects(
+                  hLsaConnection,
+                  LSA_PROVIDER_TAG_LOCAL,
+                  0,
+                  LSA_OBJECT_TYPE_USER,
+                  LSA_QUERY_TYPE_BY_NAME,
+                  1,
+                  QueryList,
+                  &ppObjects);
+    status = LsaLsaErrorToNtStatus(dwError);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    if (ppObjects[0]->userInfo.bAccountDisabled)
+    {
+        status = STATUS_ACCOUNT_DISABLED;
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    // Build LSA_AUTH_USER_INFO from LSA_SECURITY_OBJECT, at least the info we
+    // need for NTLM logon
+    dwError = LsaQueryMemberOf(
+                    hLsaConnection,
+                    LSA_PROVIDER_TAG_LOCAL,
+                    0,
+                    1,
+                    &ppObjects[0]->pszObjectSid,
+                    &dwGroupCount,
+                    &ppszGroupSids);
+    if (IS_NOT_FOUND_ERROR(dwError))
+    {
+        dwError = 0;
+    }
+    else
+    {
+        status = LsaLsaErrorToNtStatus(dwError);
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    status = RTL_ALLOCATE(&pUserInfo, LSA_AUTH_USER_INFO, sizeof(*pUserInfo));
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    *pUserInfo = userInfo;  // zero all fields
+
+    // Fill pUserInfo only with data needed in the caller function
+    // Session key
+    dwError = LsaDataBlobAllocate(&pUserInfo->pSessionKey, NTLM_SESSION_KEY_SIZE);
+    status = LsaLsaErrorToNtStatus(dwError);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    RtlZeroMemory(pUserInfo->pSessionKey->pData, NTLM_SESSION_KEY_SIZE);
+
+    // UPN
+    if (ppObjects[0]->userInfo.pszUPN)
+    {
+        status = RtlCStringDuplicate(&pUserInfo->pszUserPrincipalName,
+                                     ppObjects[0]->userInfo.pszUPN);
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    // Account
+    if (ppObjects[0]->pszSamAccountName)
+    {
+        status = RtlCStringDuplicate(&pUserInfo->pszAccount,
+                                     ppObjects[0]->pszSamAccountName);
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    // UserRid
+    status = RtlAllocateSidFromCString(&pSid, ppObjects[0]->pszObjectSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = RtlGetRidSid(&pUserInfo->dwUserRid, pSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // Domain SID
+    status = RtlDuplicateSid(&pDomainSid, pSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    pDomainSid->SubAuthorityCount--;
+
+    status = RtlAllocateCStringFromSid(&pUserInfo->pszDomainSid, pDomainSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // Primary group RID
+    RTL_FREE(&pSid);
+    status = RtlAllocateSidFromCString(&pSid, ppObjects[0]->userInfo.pszPrimaryGroupSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = RtlGetRidSid(&pUserInfo->dwPrimaryGroupRid, pSid);
+    GOTO_CLEANUP_ON_STATUS(status);
+    
+    // Groups
+    pUserInfo->dwNumRids = 0;
+    pUserInfo->dwNumSids = dwGroupCount;
+
+    status = RTL_ALLOCATE(&pUserInfo->pSidAttribList, LSA_SID_ATTRIB, 
+                          sizeof(*pUserInfo->pSidAttribList) * dwGroupCount);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // for the case we fail to allocate somewhere in the
+    // middle of the next loop
+    RtlZeroMemory(pUserInfo->pSidAttribList, 
+                  sizeof(*pUserInfo->pSidAttribList) * dwGroupCount);    
+
+    for (i = 0; i < pUserInfo->dwNumSids; ++i)
+    {
+        status = RtlCStringDuplicate(&pUserInfo->pSidAttribList[i].pszSid,
+                                     ppszGroupSids[i]);
+        GOTO_CLEANUP_ON_STATUS(status);
+
+        pUserInfo->pSidAttribList[i].dwAttrib = 
+            LSA_SID_ATTR_GROUP_MANDATORY |
+            LSA_SID_ATTR_GROUP_ENABLED_BY_DEFAULT |
+            LSA_SID_ATTR_GROUP_ENABLED;        
+    }
+
+
+    *ppUserInfo = pUserInfo;
+
+cleanup:
+    if (!NT_SUCCESS(status))
+    {
+        LsaFreeAuthUserInfo(&pUserInfo);
+    }
+
+    RTL_FREE(&pSid);
+    RTL_FREE(&pDomainSid);
+    LwFreeStringArray(ppszGroupSids, dwGroupCount);
+    LsaUtilFreeSecurityObjectList(1, ppObjects);
+
+    return status;
+}
+
+static
+NTSTATUS
+LsaMapSecurityGetAccessTokenCreateInformationFromNtlmLogon(
+    IN PLW_MAP_SECURITY_PLUGIN_CONTEXT Context,
+    OUT PACCESS_TOKEN_CREATE_INFORMATION* ppCreateInformation,
+    IN PLW_MAP_SECURITY_NTLM_LOGON_INFO pNtlmInfo,
+    OUT PLW_MAP_SECURITY_NTLM_LOGON_RESULT* ppNtlmResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    DWORD dwError = LW_ERROR_SUCCESS;
+    HANDLE hLsaConnection = NULL;
+    LSA_AUTH_USER_PARAMS lsaUserParams = { 0 };
+    LW_LSA_DATA_BLOB Challenge = { 0 };
+    LW_LSA_DATA_BLOB LMResp = { 0 };
+    LW_LSA_DATA_BLOB NTResp = { 0 };
+    PLSA_AUTH_USER_INFO pUserInfo = NULL;
+    PLW_MAP_SECURITY_NTLM_LOGON_RESULT pNtlmResult = NULL;
+    PACCESS_TOKEN_CREATE_INFORMATION pCreateInformation = NULL;
+    LSA_MAP_SECURITY_OBJECT_INFO objectInfo = {0};
+    DWORD dwInputSidCount = 0;
+    PSID* ppInputSids = NULL;
+    PSID pDomainSid = NULL;
+    DWORD dwIndex = 0;
+    PSID pSid = NULL;
+
+    status = LsaMapSecurityOpenConnection(Context, &hLsaConnection);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // Create LSA_AUTH_USER_PARAMS
+    Challenge.dwLen = sizeof(pNtlmInfo->Challenge);
+    Challenge.pData = pNtlmInfo->Challenge;
+    LMResp.dwLen = pNtlmInfo->ulLmResponseSize;
+    LMResp.pData = pNtlmInfo->pLmResponse;
+    NTResp.dwLen = pNtlmInfo->ulNtResponseSize;
+    NTResp.pData = pNtlmInfo->pNtResponse;
+
+    status = LwRtlCStringAllocateFromWC16String(
+                &lsaUserParams.pszAccountName,
+                pNtlmInfo->pwszAccountName);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = LwRtlCStringAllocateFromWC16String(
+                &lsaUserParams.pszDomain,
+                pNtlmInfo->pwszDomain);
+    GOTO_CLEANUP_ON_STATUS(status);
+    
+    lsaUserParams.AuthType = LSA_AUTH_CHAP;
+    lsaUserParams.pszWorkstation = NULL;
+    lsaUserParams.pass.chap.pChallenge = &Challenge;
+    lsaUserParams.pass.chap.pLM_resp = &LMResp;
+    lsaUserParams.pass.chap.pNT_resp = &NTResp;
+
+    // Authenticate
+    dwError = LsaAuthenticateUserEx(
+                  hLsaConnection,
+                  NULL,
+                  &lsaUserParams,
+                  &pUserInfo);
+    if (dwError == LW_ERROR_NOT_HANDLED)
+    {
+        // Attempt to fallback to Guest access
+        status = LsaMapSecurityGetGuestAuthUserInfo(hLsaConnection, &pUserInfo);
+    }
+    else
+    {
+        status = LsaLsaErrorToNtStatus(dwError);
+    }
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    // Copy data to the result struct
+    status = RTL_ALLOCATE(&pNtlmResult, LW_MAP_SECURITY_NTLM_LOGON_RESULT, 
+                          sizeof(*pNtlmResult));
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    if (pUserInfo->pszUserPrincipalName)
+    {
+        status = LwRtlCStringDuplicate(
+                    &pNtlmResult->pszUsername, 
+                    pUserInfo->pszUserPrincipalName);
+    }
+    else if (pUserInfo->pszAccount)
+    {
+        status = LwRtlCStringDuplicate(
+                    &pNtlmResult->pszUsername, 
+                    pUserInfo->pszAccount);
+    }
+    GOTO_CLEANUP_ON_STATUS(status);
+    
+    assert(pUserInfo->pSessionKey->dwLen == NTLM_SESSION_KEY_SIZE);    
+    RtlCopyMemory(pNtlmResult->SessionKey, 
+                  pUserInfo->pSessionKey->pData, 
+                  NTLM_SESSION_KEY_SIZE);
+
+    pNtlmResult->bMappedToGuest = 
+        (pUserInfo->dwUserRid == DOMAIN_USER_RID_GUEST) ? TRUE : FALSE;
+    
+    // Create CreateInformation
+    status = LsaMapSecurityResolveObjectInfoFromAuthUserInfo(
+                hLsaConnection,
+                pUserInfo,
+                &objectInfo);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    if (pUserInfo->dwNumRids != 0 || pUserInfo->dwNumSids != 0)
+    {
+        status = RTL_ALLOCATE(&ppInputSids, PSID, 
+                              sizeof(*ppInputSids) * (pUserInfo->dwNumRids + pUserInfo->dwNumSids));
+        GOTO_CLEANUP_ON_STATUS(status);
+    
+        status = RtlAllocateSidFromCString(&pDomainSid, pUserInfo->pszDomainSid);
+        GOTO_CLEANUP_ON_STATUS(status);
+    
+        for (dwIndex = 0; dwIndex < pUserInfo->dwNumRids; dwIndex++)
+        {
+            status = LsaMapSecurityConstructSid(
+                pDomainSid,
+                pUserInfo->pRidAttribList[dwIndex].Rid,
+                &pSid);
+            GOTO_CLEANUP_ON_STATUS(status);
+    
+            ppInputSids[dwInputSidCount++] = pSid;
+            pSid = NULL;
+        }
+    
+        for (dwIndex = 0; dwIndex < pUserInfo->dwNumSids; dwIndex++)
+        {
+            status = RtlAllocateSidFromCString(&pSid, pUserInfo->pSidAttribList[dwIndex].pszSid);
+            GOTO_CLEANUP_ON_STATUS(status);
+    
+            ppInputSids[dwInputSidCount++] = pSid;
+            pSid = NULL;
+        }
+    }
+
+    status = LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
+                Context,
+                &pCreateInformation,
+                &objectInfo,
+                dwInputSidCount,
+                ppInputSids);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+cleanup:
+
+    if (!NT_SUCCESS(status))
+    {
+        LsaMapSecurityFreeNtlmLogonResult(Context, &pNtlmResult);
+        LsaMapSecurityFreeAccessTokenCreateInformation(Context, &pCreateInformation);
+    }
+
+    LwRtlCStringFree(&lsaUserParams.pszAccountName);
+    LwRtlCStringFree(&lsaUserParams.pszDomain);
+    LsaFreeAuthUserInfo(&pUserInfo);
+
+    RTL_FREE(&pDomainSid);
+    RTL_FREE(&pSid);
+
+    for (dwIndex = 0; dwIndex < dwInputSidCount; dwIndex++)
+    {
+        RTL_FREE(&ppInputSids[dwIndex]);
+    }
+
+    RTL_FREE(&ppInputSids);
+    
+    LsaMapSecurityCloseConnection(Context, &hLsaConnection);
+    LsaMapSecurityFreeObjectInfo(&objectInfo);    
+
+    *ppNtlmResult = pNtlmResult;
+    *ppCreateInformation = pCreateInformation;
+
+    return status;
+}
+
 static LW_MAP_SECURITY_PLUGIN_INTERFACE gLsaMapSecurityPluginInterface = {
     .FreeContext = LsaMapSecurityFreeContext,
     .GetIdFromSid = LsaMapSecurityGetIdFromSid,
@@ -1640,6 +2044,8 @@ static LW_MAP_SECURITY_PLUGIN_INTERFACE gLsaMapSecurityPluginInterface = {
     .GetAccessTokenCreateInformationFromUsername = LsaMapSecurityGetAccessTokenCreateInformationFromUsername,
     .GetAccessTokenCreateInformationFromGssContext = LsaMapSecurityGetAccessTokenCreateInformationFromGssContext,
     .FreeAccessTokenCreateInformation = LsaMapSecurityFreeAccessTokenCreateInformation,
+    .GetAccessTokenCreateInformationFromNtlmLogon = LsaMapSecurityGetAccessTokenCreateInformationFromNtlmLogon,
+    .FreeNtlmLogonResult = LsaMapSecurityFreeNtlmLogonResult,
 };
 
 NTSTATUS
@@ -1708,7 +2114,7 @@ cleanup:
     {
         LW_RTL_FREE(&pDacl);
     }
-
+    
     return status;
 }
 
