@@ -65,7 +65,7 @@ LsaNssClearEnumUsersState(
             );
         pState->ppUserInfoList = (HANDLE)NULL;
     }
-
+    
     if (hLsaConnection && pState->hResume != (HANDLE)NULL)
     {
         LsaEndEnumUsers(hLsaConnection, pState->hResume);
@@ -211,7 +211,7 @@ LsaNssCommonPasswdSetpwent(
             LsaNssCommonEnsureConnected(pConnection));
     BAIL_ON_NSS_ERROR(ret);
     hLsaConnection = pConnection->hLsaConnection;
-
+    
     LsaNssClearEnumUsersState(hLsaConnection, pEnumUsersState);
 
     ret = MAP_LSA_ERROR(NULL,
@@ -258,7 +258,7 @@ LsaNssCommonPasswdGetpwent(
                             LW_ERROR_INVALID_LSA_CONNECTION);
         BAIL_ON_NSS_ERROR(ret);
     }
-
+    
     if (!pEnumUsersState->bTryAgain)
     {
         if (!pEnumUsersState->idxUser ||
@@ -320,7 +320,7 @@ error:
     else
     {
         LsaNssClearEnumUsersState(hLsaConnection, pEnumUsersState);
-
+        
         if ( hLsaConnection != (HANDLE)NULL)
         {
             if (ret != NSS_STATUS_TRYAGAIN && ret != NSS_STATUS_NOTFOUND)
@@ -366,6 +366,11 @@ LsaNssCommonPasswdGetpwnam(
     PVOID pUserInfo = NULL;
     DWORD dwUserInfoLevel = 0;
 
+    if (LsaShouldIgnoreUser(pszLoginId))
+    {
+        ret = MAP_LSA_ERROR(NULL, LW_ERROR_NOT_HANDLED);
+        BAIL_ON_NSS_ERROR(ret);
+    }
 
     ret = MAP_LSA_ERROR(NULL,
             LsaNssCommonEnsureConnected(pConnection));
