@@ -74,7 +74,7 @@ LsaNssSolarisPasswdDestructor(
         &pLsaBackend->lsaConnection,
         pEnumUsersState);
     LsaNssCommonCloseConnection(&pLsaBackend->lsaConnection);
-    LsaFreeIgnoreLists();
+    LsaFreeIgnoreHashes();
     LwFreeMemory(pBackend);
 
     return ret;
