@@ -187,7 +187,7 @@ LsaNssSolarisNetgroupDestructor(
         LsaNssFreeNetgroupList(&pLsaBackend->pExpand);
 
         LsaNssCommonCloseConnection(&pLsaBackend->base.lsaConnection);
-        LsaFreeIgnoreLists();
+        LsaFreeIgnoreHashes();
         LW_SAFE_FREE_MEMORY(pLsaBackend->pBuffer);
     }
 
