@@ -52,6 +52,7 @@ typedef struct _LSA_SRV_ENUM_HANDLE
     LSA_OBJECT_TYPE ObjectType;
     PSTR pszDomainName;
     PSTR pszSid;
+    PSTR pszTargetInstance;
     PLSA_AUTH_PROVIDER pProvider;
     HANDLE hProvider;
     HANDLE hEnum;
@@ -70,6 +71,82 @@ typedef struct _LSA_SRV_MEMBER_OF_PASS
     } *pResults;
 } LSA_SRV_MEMBER_OF_PASS, *PLSA_SRV_MEMBER_OF_PASS;
 
+VOID
+LsaSrvInitializeLock(
+     PLSA_SRV_RWLOCK pLock
+     )
+{
+    int localError = 0;
+
+    localError = pthread_mutex_init(&pLock->stateMutex, NULL);
+    LSA_ASSERT(localError == 0);
+
+    localError = pthread_cond_init(&pLock->stateCond, NULL);
+    LSA_ASSERT(localError == 0);
+
+    pLock->readers = 0;
+}
+
+VOID
+LsaSrvAcquireRead(
+     PLSA_SRV_RWLOCK pLock
+     )
+{
+    int status = 0;
+
+    status = pthread_mutex_lock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+    pLock->readers++;
+    status = pthread_mutex_unlock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+}
+
+VOID
+LsaSrvAcquireWrite(
+     PLSA_SRV_RWLOCK pLock
+     )
+{
+    int status = 0;
+
+    status = pthread_mutex_lock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+    while (pLock->readers)
+    {
+        pthread_cond_wait(&pLock->stateCond, &pLock->stateMutex);
+    }
+}
+
+VOID
+LsaSrvReleaseRead(
+     PLSA_SRV_RWLOCK pLock
+     )
+{
+    int status = 0;
+
+    status = pthread_mutex_lock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+    pLock->readers--;
+    if (pLock->readers == 0)
+    {
+        pthread_cond_broadcast(&pLock->stateCond);
+    }
+    status = pthread_mutex_unlock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+}
+
+
+VOID
+LsaSrvReleaseWrite(
+     PLSA_SRV_RWLOCK pLock
+     )
+{
+    int status = 0;
+
+    LW_ASSERT(pLock->readers == 0);
+    status = pthread_mutex_unlock(&pLock->stateMutex);
+    LW_ASSERT(status == 0);
+}
+
 static
 DWORD
 LsaSrvInitMemberOfPass(
@@ -141,7 +218,7 @@ static
 DWORD
 LsaSrvMergeMemberOfPass(
     IN PLSA_SRV_MEMBER_OF_PASS pPass,
-    IN OUT PLSA_HASH_TABLE pSidHash
+    IN OUT PLW_HASH_TABLE pSidHash
     )
 {
     DWORD dwError = 0;
@@ -152,7 +229,7 @@ LsaSrvMergeMemberOfPass(
     {
         for (dwSidIndex = 0; dwSidIndex < pPass->pResults[dwResultIndex].dwSidCount; dwSidIndex++)
         {
-            dwError = LsaHashSetValue(pSidHash,
+            dwError = LwHashSetValue(pSidHash,
                                       pPass->pResults[dwResultIndex].ppszSids[dwSidIndex],
                                       pPass->pResults[dwResultIndex].ppszSids[dwSidIndex]);
             BAIL_ON_LSA_ERROR(dwError);
@@ -169,6 +246,7 @@ static
 DWORD
 LsaSrvMakeMemberOfFirstPass(
     HANDLE hServer,
+    PCSTR pszTargetInstance,
     LSA_FIND_FLAGS FindFlags,
     IN DWORD dwSidCount,
     IN PSTR* ppszSids,
@@ -195,15 +273,19 @@ LsaSrvMakeMemberOfFirstPass(
          pProvider;
          pProvider = pProvider->pNext, dwIndex++)
     {
-        if (pProvider->pFnTable2 == NULL)
+        if (pProvider->pFnTable == NULL)
         {
             continue;
         }
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnQueryMemberOf(
+        dwError = pProvider->pFnTable->pfnQueryMemberOf(
             hProvider,
             FindFlags,
             dwSidCount,
@@ -246,6 +328,7 @@ static
 DWORD
 LsaSrvMakeMemberOfTransferPass(
     HANDLE hServer,
+    PCSTR pszTargetInstance,
     LSA_FIND_FLAGS FindFlags,
     IN PLSA_SRV_MEMBER_OF_PASS pIn,
     OUT PLSA_SRV_MEMBER_OF_PASS pOut
@@ -270,7 +353,7 @@ LsaSrvMakeMemberOfTransferPass(
          pProvider;
          pProvider = pProvider->pNext, dwIndex++)
     {
-        if (pProvider->pFnTable2 == NULL)
+        if (pProvider->pFnTable == NULL)
         {
             continue;
         }
@@ -297,13 +380,17 @@ LsaSrvMakeMemberOfTransferPass(
                 dwBatchedSidCount += pIn->pResults[dwSourceIndex].dwSidCount;
             }
         }
-
+        
         if (dwBatchedSidCount)
         {
-            dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+            dwError = LsaSrvOpenProvider(
+                          hServer,
+                          pProvider,
+                          pszTargetInstance,
+                          &hProvider);
             BAIL_ON_LSA_ERROR(dwError);
 
-            dwError = pProvider->pFnTable2->pfnQueryMemberOf(
+            dwError = pProvider->pFnTable->pfnQueryMemberOf(
                 hProvider,
                 FindFlags,
                 dwBatchedSidCount,
@@ -327,7 +414,7 @@ LsaSrvMakeMemberOfTransferPass(
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = NULL;
         }
-
+        
         LW_SAFE_FREE_MEMORY(ppszBatchedSids);
         dwBatchedSidCount = 0;
     }
@@ -424,6 +511,8 @@ LsaSrvFindObjectsInternal(
     LSA_QUERY_LIST PartialQueryList;
     DWORD dwPartialCount = 0;
     BOOLEAN bFoundProvider = FALSE;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     memset(&PartialQueryList, 0, sizeof(PartialQueryList));
 
@@ -443,15 +532,24 @@ LsaSrvFindObjectsInternal(
         break;
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     for (pProvider = gpAuthProviderList;
          pProvider;
          pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider)
+        if (pszTargetProviderName)
         {
-            if (!strcmp(pszTargetProvider, pProvider->pszName))
+            if (!strcmp(pszTargetProviderName, pProvider->pszName))
             {
                 bFoundProvider = TRUE;
             }
@@ -461,7 +559,7 @@ LsaSrvFindObjectsInternal(
             }
         }
 
-        if (pProvider->pFnTable2 == NULL)
+        if (pProvider->pFnTable == NULL)
         {
             continue;
         }
@@ -480,10 +578,14 @@ LsaSrvFindObjectsInternal(
             break;
         }
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnFindObjects(
+        dwError = pProvider->pFnTable->pfnFindObjects(
             hProvider,
             FindFlags,
             ObjectType,
@@ -499,19 +601,19 @@ LsaSrvFindObjectsInternal(
         else
         {
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             LsaSrvMergePartialQueryResult(
                 dwCount,
                 ppPartialObjects,
                 ppCombinedObjects);
-        }
+        }   
 
         LW_SAFE_FREE_MEMORY(ppPartialObjects);
         LsaSrvCloseProvider(pProvider, hProvider);
         hProvider = NULL;
     }
-
-    if (pszTargetProvider && !bFoundProvider)
+        
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -525,6 +627,9 @@ cleanup:
     /* Note that this is safe regardless of the type of the query */
     LW_SAFE_FREE_MEMORY(PartialQueryList.ppszStrings);
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != NULL)
     {
         LsaSrvCloseProvider(pProvider, hProvider);
@@ -619,11 +724,8 @@ LsaSrvFindObjects(
                 &ppCombinedObjects[dwIndex]);
             BAIL_ON_LSA_ERROR(dwError);
 
-            if (pLoginInfo)
-            {
-                LsaSrvFreeNameInfo(pLoginInfo);
-                pLoginInfo = NULL;
-            }
+            LsaSrvFreeNameInfo(pLoginInfo);
+            pLoginInfo = NULL;
         }
         break;
     }
@@ -665,6 +767,8 @@ LsaSrvOpenEnumObjects(
     DWORD dwError = 0;
     PLSA_SRV_ENUM_HANDLE pEnum = NULL;
     PLSA_AUTH_PROVIDER pProvider = NULL;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     dwError = LwAllocateMemory(sizeof(*pEnum), OUT_PPVOID(&pEnum));
     BAIL_ON_LSA_ERROR(dwError);
@@ -679,15 +783,27 @@ LsaSrvOpenEnumObjects(
     pEnum->FindFlags = FindFlags;
     pEnum->ObjectType = ObjectType;
 
-    ENTER_AUTH_PROVIDER_LIST_READER_LOCK(pEnum->bReleaseLock);
-
     if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pEnum->pszTargetInstance = pszTargetInstance;
+        pszTargetInstance = NULL;
+    }
+
+    ENTER_AUTH_PROVIDER_LIST_READER_LOCK(pEnum->bReleaseLock);
+    
+    if (pszTargetProviderName)
     {
         for (pProvider = gpAuthProviderList;
              pProvider;
              pProvider = pProvider->pNext)
         {
-            if (!strcmp(pszTargetProvider, pProvider->pszName))
+            if (!strcmp(pszTargetProviderName, pProvider->pszName))
             {
                 pEnum->pProvider = pProvider;
                 break;
@@ -710,6 +826,9 @@ LsaSrvOpenEnumObjects(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     return dwError;
 
 error:
@@ -748,10 +867,10 @@ LsaSrvEnumObjects(
         sizeof(*ppCombinedObjects) * dwMaxObjectsCount,
         OUT_PPVOID(&ppCombinedObjects));
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     while (dwCombinedObjectCount < dwMaxObjectsCount && pEnum->pProvider != NULL)
     {
-        if (pEnum->pProvider->pFnTable2 == NULL)
+        if (pEnum->pProvider->pFnTable == NULL)
         {
             pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
             continue;
@@ -759,13 +878,17 @@ LsaSrvEnumObjects(
 
         if (!pEnum->hProvider)
         {
-            dwError = LsaSrvOpenProvider(hServer, pEnum->pProvider, &pEnum->hProvider);
+            dwError = LsaSrvOpenProvider(
+                          hServer,
+                          pEnum->pProvider,
+                          pEnum->pszTargetInstance,
+                          &pEnum->hProvider);
             BAIL_ON_LSA_ERROR(dwError);
         }
 
         if (!pEnum->hEnum)
         {
-            dwError = pEnum->pProvider->pFnTable2->pfnOpenEnumObjects(
+            dwError = pEnum->pProvider->pFnTable->pfnOpenEnumObjects(
                 pEnum->hProvider,
                 &pEnum->hEnum,
                 pEnum->FindFlags,
@@ -774,7 +897,7 @@ LsaSrvEnumObjects(
             if (dwError == LW_ERROR_NOT_HANDLED)
             {
                 dwError = LW_ERROR_SUCCESS;
-                pEnum->pProvider->pFnTable2->pfnCloseHandle(pEnum->hProvider);
+                pEnum->pProvider->pFnTable->pfnCloseHandle(pEnum->hProvider);
                 pEnum->hProvider = NULL;
                 pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
                 continue;
@@ -785,7 +908,7 @@ LsaSrvEnumObjects(
             }
         }
 
-        dwError = pEnum->pProvider->pFnTable2->pfnEnumObjects(
+        dwError = pEnum->pProvider->pFnTable->pfnEnumObjects(
             pEnum->hEnum,
             dwMaxObjectsCount - dwCombinedObjectCount,
             &dwPartialObjectCount,
@@ -793,9 +916,9 @@ LsaSrvEnumObjects(
         if (dwError == ERROR_NO_MORE_ITEMS)
         {
             dwError = LW_ERROR_SUCCESS;
-            pEnum->pProvider->pFnTable2->pfnCloseEnum(pEnum->hEnum);
+            pEnum->pProvider->pFnTable->pfnCloseEnum(pEnum->hEnum);
             pEnum->hEnum = NULL;
-            pEnum->pProvider->pFnTable2->pfnCloseHandle(pEnum->hProvider);
+            pEnum->pProvider->pFnTable->pfnCloseHandle(pEnum->hProvider);
             pEnum->hProvider = NULL;
             pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
             continue;
@@ -804,7 +927,7 @@ LsaSrvEnumObjects(
         {
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         memcpy(ppCombinedObjects + dwCombinedObjectCount,
                ppPartialObjects,
                sizeof(*ppPartialObjects) * dwPartialObjectCount);
@@ -852,28 +975,42 @@ LsaSrvOpenEnumMembers(
     IN PCSTR pszSid
     )
 {
-   DWORD dwError = 0;
-   PLSA_SRV_ENUM_HANDLE pEnum = NULL;
-   PLSA_AUTH_PROVIDER pProvider = NULL;
+    DWORD dwError = 0;
+    PLSA_SRV_ENUM_HANDLE pEnum = NULL;
+    PLSA_AUTH_PROVIDER pProvider = NULL;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
-   dwError = LwAllocateMemory(sizeof(*pEnum), OUT_PPVOID(&pEnum));
-   BAIL_ON_LSA_ERROR(dwError);
+    dwError = LwAllocateMemory(sizeof(*pEnum), OUT_PPVOID(&pEnum));
+    BAIL_ON_LSA_ERROR(dwError);
+   
+    dwError = LwAllocateString(pszSid, &pEnum->pszSid);
+    BAIL_ON_LSA_ERROR(dwError);
 
-   dwError = LwAllocateString(pszSid, &pEnum->pszSid);
-   BAIL_ON_LSA_ERROR(dwError);
-
-   pEnum->Type = LSA_SRV_ENUM_MEMBERS;
-   pEnum->FindFlags = FindFlags;
-
-   ENTER_AUTH_PROVIDER_LIST_READER_LOCK(pEnum->bReleaseLock);
+    pEnum->Type = LSA_SRV_ENUM_MEMBERS;
+    pEnum->FindFlags = FindFlags;
 
     if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pEnum->pszTargetInstance = pszTargetInstance;
+        pszTargetInstance = NULL;
+    }
+
+    ENTER_AUTH_PROVIDER_LIST_READER_LOCK(pEnum->bReleaseLock);
+    
+    if (pszTargetProviderName)
     {
         for (pProvider = gpAuthProviderList;
              pProvider;
              pProvider = pProvider->pNext)
         {
-            if (!strcmp(pszTargetProvider, pProvider->pszName))
+            if (!strcmp(pszTargetProviderName, pProvider->pszName))
             {
                 pEnum->pProvider = pProvider;
                 break;
@@ -896,6 +1033,9 @@ LsaSrvOpenEnumMembers(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     return dwError;
 
 error:
@@ -934,10 +1074,10 @@ LsaSrvEnumMembers(
         sizeof(*ppszCombinedSids) * dwMaxSidCount,
         OUT_PPVOID(&ppszCombinedSids));
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     while (dwCombinedSidCount < dwMaxSidCount && pEnum->pProvider != NULL)
     {
-        if (pEnum->pProvider->pFnTable2 == NULL)
+        if (pEnum->pProvider->pFnTable == NULL)
         {
             pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
             continue;
@@ -945,13 +1085,17 @@ LsaSrvEnumMembers(
 
         if (!pEnum->hProvider)
         {
-            dwError = LsaSrvOpenProvider(hServer, pEnum->pProvider, &pEnum->hProvider);
+            dwError = LsaSrvOpenProvider(
+                          hServer,
+                          pEnum->pProvider,
+                          pEnum->pszTargetInstance,
+                          &pEnum->hProvider);
             BAIL_ON_LSA_ERROR(dwError);
         }
 
         if (!pEnum->hEnum)
         {
-            dwError = pEnum->pProvider->pFnTable2->pfnOpenEnumGroupMembers(
+            dwError = pEnum->pProvider->pFnTable->pfnOpenEnumGroupMembers(
                 pEnum->hProvider,
                 &pEnum->hEnum,
                 pEnum->FindFlags,
@@ -962,7 +1106,7 @@ LsaSrvEnumMembers(
             case LW_ERROR_NO_SUCH_OBJECT:
             case LW_ERROR_NO_SUCH_GROUP:
                 dwError = LW_ERROR_SUCCESS;
-                pEnum->pProvider->pFnTable2->pfnCloseHandle(pEnum->hProvider);
+                pEnum->pProvider->pFnTable->pfnCloseHandle(pEnum->hProvider);
                 pEnum->hProvider = NULL;
                 pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
                 continue;
@@ -971,7 +1115,7 @@ LsaSrvEnumMembers(
             }
         }
 
-        dwError = pEnum->pProvider->pFnTable2->pfnEnumGroupMembers(
+        dwError = pEnum->pProvider->pFnTable->pfnEnumGroupMembers(
             pEnum->hEnum,
             dwMaxSidCount - dwCombinedSidCount,
             &dwPartialSidCount,
@@ -979,9 +1123,9 @@ LsaSrvEnumMembers(
         if (dwError == ERROR_NO_MORE_ITEMS)
         {
             dwError = LW_ERROR_SUCCESS;
-            pEnum->pProvider->pFnTable2->pfnCloseEnum(pEnum->hEnum);
+            pEnum->pProvider->pFnTable->pfnCloseEnum(pEnum->hEnum);
             pEnum->hEnum = NULL;
-            pEnum->pProvider->pFnTable2->pfnCloseHandle(pEnum->hProvider);
+            pEnum->pProvider->pFnTable->pfnCloseHandle(pEnum->hProvider);
             pEnum->hProvider = NULL;
             pEnum->pProvider = pEnum->bMergeResults ? pEnum->pProvider->pNext : NULL;
             continue;
@@ -990,7 +1134,7 @@ LsaSrvEnumMembers(
         {
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         memcpy(ppszCombinedSids + dwCombinedSidCount,
                ppszPartialSids,
                sizeof(*ppszPartialSids) * dwPartialSidCount);
@@ -1032,7 +1176,7 @@ error:
 static
 VOID
 LsaSrvFreeMemberOfHashEntry(
-    const LSA_HASH_ENTRY* pEntry
+    const LW_HASH_ENTRY* pEntry
     )
 {
     if (pEntry->pValue)
@@ -1045,6 +1189,7 @@ static
 DWORD
 LsaSrvQueryMemberOfMerged(
     IN HANDLE hServer,
+    IN OPTIONAL PCSTR pszTargetInstance,
     IN LSA_FIND_FLAGS FindFlags,
     IN DWORD dwSidCount,
     IN PSTR* ppszSids,
@@ -1061,15 +1206,15 @@ LsaSrvQueryMemberOfMerged(
     PLSA_SRV_MEMBER_OF_PASS pSourcePass = &pass1;
     PLSA_SRV_MEMBER_OF_PASS pDestPass = &pass2;
     PLSA_SRV_MEMBER_OF_PASS pTempPass = NULL;
-    PLSA_HASH_TABLE pHash = NULL;
-    LSA_HASH_ITERATOR hashIterator = {0};
-    LSA_HASH_ENTRY* pHashEntry = NULL;
+    PLW_HASH_TABLE pHash = NULL;
+    LW_HASH_ITERATOR hashIterator = {0};
+    LW_HASH_ENTRY* pHashEntry = NULL;
     DWORD dwIndex = 0;
-
-    dwError = LsaHashCreate(
+    
+    dwError = LwHashCreate(
         29,
-        LsaHashCaselessStringCompare,
-        LsaHashCaselessStringHash,
+        LwHashCaselessStringCompare,
+        LwHashCaselessStringHash,
         LsaSrvFreeMemberOfHashEntry,
         NULL,
         &pHash);
@@ -1079,6 +1224,7 @@ LsaSrvQueryMemberOfMerged(
 
     dwError = LsaSrvMakeMemberOfFirstPass(
         hServer,
+        pszTargetInstance,
         FindFlags,
         dwSidCount,
         ppszSids,
@@ -1089,6 +1235,7 @@ LsaSrvQueryMemberOfMerged(
     {
         dwError = LsaSrvMakeMemberOfTransferPass(
             hServer,
+            pszTargetInstance,
             FindFlags,
             pSourcePass,
             pDestPass);
@@ -1104,19 +1251,19 @@ LsaSrvQueryMemberOfMerged(
         pDestPass =  pTempPass;
     }
 
-    dwCombinedCount = (DWORD) LsaHashGetKeyCount(pHash);
-
+    dwCombinedCount = (DWORD) LwHashGetKeyCount(pHash);
+    
     if (dwCombinedCount)
     {
         dwError = LwAllocateMemory(
             sizeof(*ppszCombinedSids) * dwCombinedCount,
             OUT_PPVOID(&ppszCombinedSids));
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LsaHashGetIterator(pHash, &hashIterator);
+    
+        dwError = LwHashGetIterator(pHash, &hashIterator);
         BAIL_ON_LSA_ERROR(dwError);
-
-        for (dwIndex = 0; (pHashEntry = LsaHashNext(&hashIterator)) != NULL; dwIndex++)
+        
+        for (dwIndex = 0; (pHashEntry = LwHashNext(&hashIterator)) != NULL; dwIndex++)
         {
             ppszCombinedSids[dwIndex] = (PSTR) pHashEntry->pValue;
             pHashEntry->pValue = NULL;
@@ -1130,7 +1277,7 @@ cleanup:
 
     LsaSrvDestroyMemberOfPass(&pass1);
     LsaSrvDestroyMemberOfPass(&pass2);
-    LsaHashSafeFree(&pHash);
+    LwHashSafeFree(&pHash);
 
     LEAVE_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
@@ -1153,7 +1300,8 @@ static
 DWORD
 LsaSrvQueryMemberOfSingle(
     IN HANDLE hServer,
-    IN OPTIONAL PCSTR pszTargetProvider,
+    IN OPTIONAL PCSTR pszTargetProviderName,
+    IN OPTIONAL PCSTR pszTargetInstance,
     IN LSA_FIND_FLAGS FindFlags,
     IN DWORD dwSidCount,
     IN PSTR* ppszSids,
@@ -1172,17 +1320,21 @@ LsaSrvQueryMemberOfSingle(
          pProvider;
          pProvider = pProvider->pNext)
     {
-        if (pProvider->pFnTable2 == NULL)
+        if (pProvider->pFnTable == NULL)
         {
             continue;
         }
 
-        if (!strcmp(pProvider->pszName, pszTargetProvider))
+        if (!strcmp(pProvider->pszName, pszTargetProviderName))
         {
-            dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+            dwError = LsaSrvOpenProvider(
+                          hServer,
+                          pProvider,
+                          pszTargetInstance,
+                          &hProvider);
             BAIL_ON_LSA_ERROR(dwError);
-
-            dwError = pProvider->pFnTable2->pfnQueryMemberOf(
+            
+            dwError = pProvider->pFnTable->pfnQueryMemberOf(
                 hProvider,
                 FindFlags,
                 dwSidCount,
@@ -1226,7 +1378,19 @@ LsaSrvQueryMemberOf(
     )
 {
     DWORD dwError = 0;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    
     if (dwSidCount == 0)
     {
         *pdwGroupSidCount = 0;
@@ -1234,11 +1398,12 @@ LsaSrvQueryMemberOf(
         goto cleanup;
     }
 
-    if (pszTargetProvider)
+    if (pszTargetProviderName)
     {
         dwError = LsaSrvQueryMemberOfSingle(
             hServer,
-            pszTargetProvider,
+            pszTargetProviderName,
+            pszTargetInstance,
             FindFlags,
             dwSidCount,
             ppszSids,
@@ -1250,6 +1415,7 @@ LsaSrvQueryMemberOf(
     {
         dwError = LsaSrvQueryMemberOfMerged(
             hServer,
+            pszTargetInstance,
             FindFlags,
             dwSidCount,
             ppszSids,
@@ -1260,6 +1426,9 @@ LsaSrvQueryMemberOf(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     return dwError;
 
 error:
@@ -1278,7 +1447,7 @@ LsaSrvQueryExpandedGroupMembersInternal(
     IN LSA_FIND_FLAGS FindFlags,
     IN LSA_OBJECT_TYPE ObjectType,
     IN PCSTR pszSid,
-    IN OUT PLSA_HASH_TABLE pHash
+    IN OUT PLW_HASH_TABLE pHash
     )
 {
     DWORD dwError = 0;
@@ -1328,9 +1497,9 @@ LsaSrvQueryExpandedGroupMembersInternal(
 
         for (dwIndex = 0; dwIndex < dwEnumCount; dwIndex++)
         {
-            if (ppObjects[dwIndex] && !LsaHashExists(pHash, ppObjects[dwIndex]->pszObjectSid))
+            if (ppObjects[dwIndex] && !LwHashExists(pHash, ppObjects[dwIndex]->pszObjectSid))
             {
-                dwError = LsaHashSetValue(
+                dwError = LwHashSetValue(
                     pHash,
                     ppObjects[dwIndex]->pszObjectSid,
                     ppObjects[dwIndex]);
@@ -1395,7 +1564,7 @@ error:
 static
 VOID
 LsaFreeMemberHashEntry(
-    const LSA_HASH_ENTRY* pEntry
+    const LW_HASH_ENTRY* pEntry
     )
 {
     if (pEntry->pValue)
@@ -1418,18 +1587,18 @@ LsaSrvQueryExpandedGroupMembers(
 {
     DWORD dwError = 0;
     DWORD dwIndex = 0;
-    PLSA_HASH_TABLE pHash = NULL;
-    LSA_HASH_ITERATOR hashIterator = {0};
-    LSA_HASH_ENTRY* pHashEntry = NULL;
+    PLW_HASH_TABLE pHash = NULL;
+    LW_HASH_ITERATOR hashIterator = {0};
+    LW_HASH_ENTRY* pHashEntry = NULL;
     DWORD dwMemberCount = 0;
     DWORD dwFilteredMemberCount = 0;
     PLSA_SECURITY_OBJECT* ppMembers = NULL;
     PLSA_SECURITY_OBJECT pMember = NULL;
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
         29,
-        LsaHashCaselessStringCompare,
-        LsaHashCaselessStringHash,
+        LwHashCaselessStringCompare,
+        LwHashCaselessStringHash,
         LsaFreeMemberHashEntry,
         NULL,
         &pHash);
@@ -1443,19 +1612,19 @@ LsaSrvQueryExpandedGroupMembers(
         pszSid,
         pHash);
 
-    dwMemberCount = (DWORD) LsaHashGetKeyCount(pHash);
-
+    dwMemberCount = (DWORD) LwHashGetKeyCount(pHash);
+    
     if (dwMemberCount)
     {
         dwError = LwAllocateMemory(
             sizeof(*ppMembers) * dwMemberCount,
             OUT_PPVOID(&ppMembers));
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LsaHashGetIterator(pHash, &hashIterator);
+    
+        dwError = LwHashGetIterator(pHash, &hashIterator);
         BAIL_ON_LSA_ERROR(dwError);
-
-        for (dwIndex = 0; (pHashEntry = LsaHashNext(&hashIterator)) != NULL; dwIndex++)
+        
+        for (dwIndex = 0; (pHashEntry = LwHashNext(&hashIterator)) != NULL; dwIndex++)
         {
             pMember = pHashEntry->pValue;
 
@@ -1473,7 +1642,7 @@ LsaSrvQueryExpandedGroupMembers(
 
 cleanup:
 
-    LsaHashSafeFree(&pHash);
+    LwHashSafeFree(&pHash);
 
     return dwError;
 
@@ -1573,16 +1742,17 @@ LsaSrvCloseEnum(
     {
         if (pEnum->hEnum)
         {
-            pEnum->pProvider->pFnTable2->pfnCloseEnum(pEnum->hEnum);
+            pEnum->pProvider->pFnTable->pfnCloseEnum(pEnum->hEnum);
         }
 
         if (pEnum->hProvider)
         {
-            pEnum->pProvider->pFnTable2->pfnCloseHandle(pEnum->hProvider);
+            pEnum->pProvider->pFnTable->pfnCloseHandle(pEnum->hProvider);
         }
 
         LW_SAFE_FREE_STRING(pEnum->pszDomainName);
         LW_SAFE_FREE_STRING(pEnum->pszSid);
+        LW_SAFE_FREE_STRING(pEnum->pszTargetInstance);
         LEAVE_AUTH_PROVIDER_LIST_READER_LOCK(pEnum->bReleaseLock);
         LwFreeMemory(pEnum);
     }
@@ -1602,6 +1772,8 @@ LsaSrvAddUser2(
     PLSA_SRV_API_STATE pServerState = (PLSA_SRV_API_STATE)hServer;
     PLSA_AUTH_PROVIDER pProvider = NULL;
     HANDLE hProvider = (HANDLE)NULL;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));;
 
@@ -1611,6 +1783,15 @@ LsaSrvAddUser2(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     dwError = LW_ERROR_NOT_HANDLED;
@@ -1619,17 +1800,22 @@ LsaSrvAddUser2(
          pProvider;
          pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider && strcmp(pProvider->pszName, pszTargetProvider))
+        if (pszTargetProviderName &&
+            strcmp(pProvider->pszName, pszTargetProviderName))
         {
             continue;
         }
 
         bFoundProvider = TRUE;
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnAddUser(
+        dwError = pProvider->pFnTable->pfnAddUser(
                                         hProvider,
                                         pUserAddInfo);
         if (dwError == LW_ERROR_SUCCESS)
@@ -1647,7 +1833,7 @@ LsaSrvAddUser2(
         }
     }
 
-    if (pszTargetProvider && !bFoundProvider)
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -1655,6 +1841,9 @@ LsaSrvAddUser2(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != (HANDLE)NULL) {
         LsaSrvCloseProvider(pProvider, hProvider);
     }
@@ -1685,6 +1874,8 @@ LsaSrvModifyUser2(
     PLSA_SRV_API_STATE pServerState = (PLSA_SRV_API_STATE)hServer;
     PLSA_AUTH_PROVIDER pProvider = NULL;
     HANDLE hProvider = (HANDLE)NULL;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
@@ -1694,6 +1885,15 @@ LsaSrvModifyUser2(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     dwError = LW_ERROR_NOT_HANDLED;
@@ -1702,17 +1902,22 @@ LsaSrvModifyUser2(
          pProvider;
          pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider && strcmp(pProvider->pszName, pszTargetProvider))
+        if (pszTargetProviderName &&
+            strcmp(pProvider->pszName, pszTargetProviderName))
         {
             continue;
         }
 
         bFoundProvider = TRUE;
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnModifyUser(
+        dwError = pProvider->pFnTable->pfnModifyUser(
                                         hProvider,
                                         pUserModInfo);
         if (dwError == LW_ERROR_SUCCESS)
@@ -1721,7 +1926,7 @@ LsaSrvModifyUser2(
         }
         else if ((dwError == LW_ERROR_NOT_HANDLED ||
                   dwError == LW_ERROR_NO_SUCH_USER) &&
-                 !pszTargetProvider)
+                 !pszTargetProviderName)
         {
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = NULL;
@@ -1731,7 +1936,7 @@ LsaSrvModifyUser2(
         }
     }
 
-    if (pszTargetProvider && !bFoundProvider)
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -1739,6 +1944,9 @@ LsaSrvModifyUser2(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != (HANDLE)NULL) {
         LsaSrvCloseProvider(pProvider, hProvider);
     }
@@ -1766,9 +1974,11 @@ LsaSrvAddGroup2(
     DWORD dwTraceFlags[] = {LSA_TRACE_FLAG_USER_GROUP_ADMINISTRATION};
     PLSA_AUTH_PROVIDER pProvider = NULL;
     BOOLEAN bInLock = FALSE;
-    BOOLEAN bFoundProvider = TRUE;
+    BOOLEAN bFoundProvider = FALSE;
     HANDLE hProvider = (HANDLE)NULL;
     PLSA_SRV_API_STATE pServerState = (PLSA_SRV_API_STATE)hServer;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
@@ -1778,30 +1988,44 @@ LsaSrvAddGroup2(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider && strcmp(pProvider->pszName, pszTargetProvider))
+        if (pszTargetProviderName &&
+            strcmp(pProvider->pszName, pszTargetProviderName))
         {
             continue;
         }
 
         bFoundProvider = TRUE;
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnAddGroup(
+        dwError = pProvider->pFnTable->pfnAddGroup(
                                         hProvider,
                                         pGroupAddInfo);
         if (dwError == LW_ERROR_SUCCESS)
         {
             break;
         }
-        else if (dwError == LW_ERROR_NOT_HANDLED && !pszTargetProvider)
+        else if (dwError == LW_ERROR_NOT_HANDLED && !pszTargetProviderName)
         {
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = NULL;
@@ -1812,7 +2036,7 @@ LsaSrvAddGroup2(
         }
     }
 
-    if (pszTargetProvider && !bFoundProvider)
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -1820,6 +2044,9 @@ LsaSrvAddGroup2(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != NULL)
     {
         LsaSrvCloseProvider(pProvider, hProvider);
@@ -1851,6 +2078,8 @@ LsaSrvModifyGroup2(
     PLSA_SRV_API_STATE pServerState = (PLSA_SRV_API_STATE)hServer;
     PLSA_AUTH_PROVIDER pProvider = NULL;
     HANDLE hProvider = (HANDLE)NULL;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
@@ -1860,23 +2089,37 @@ LsaSrvModifyGroup2(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider && strcmp(pProvider->pszName, pszTargetProvider))
+        if (pszTargetProviderName &&
+            strcmp(pProvider->pszName, pszTargetProviderName))
         {
             continue;
         }
 
         bFoundProvider = TRUE;
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnModifyGroup(
+        dwError = pProvider->pFnTable->pfnModifyGroup(
                                         hProvider,
                                         pGroupModInfo);
         if (dwError == LW_ERROR_SUCCESS)
@@ -1885,7 +2128,7 @@ LsaSrvModifyGroup2(
         }
         else if ((dwError == LW_ERROR_NOT_HANDLED ||
              dwError == LW_ERROR_NO_SUCH_GROUP) &&
-            !pszTargetProvider)
+            !pszTargetProviderName)
         {
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = NULL;
@@ -1896,7 +2139,7 @@ LsaSrvModifyGroup2(
         }
     }
 
-    if (pszTargetProvider && !bFoundProvider)
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -1904,6 +2147,9 @@ LsaSrvModifyGroup2(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != (HANDLE)NULL)
     {
         LsaSrvCloseProvider(pProvider, hProvider);
@@ -1916,7 +2162,7 @@ cleanup:
     return dwError;
 
 error:
-
+    
     LSA_LOG_ERROR_API_FAILED(hServer, dwError, "modify group (sid %s)", pGroupModInfo->pszSid);
 
     goto cleanup;
@@ -1936,6 +2182,8 @@ LsaSrvDeleteObject(
     BOOLEAN bInLock = FALSE;
     BOOLEAN bFoundProvider = FALSE;
     PLSA_SRV_API_STATE pServerState = (PLSA_SRV_API_STATE)hServer;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
@@ -1945,23 +2193,37 @@ LsaSrvDeleteObject(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        if (pszTargetProvider && strcmp(pProvider->pszName, pszTargetProvider))
+        if (pszTargetProviderName &&
+            strcmp(pProvider->pszName, pszTargetProviderName))
         {
             continue;
         }
 
         bFoundProvider = TRUE;
 
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnDeleteObject(hProvider, pszSid);
+        dwError = pProvider->pFnTable->pfnDeleteObject(hProvider, pszSid);
         if (dwError == LW_ERROR_SUCCESS)
         {
             break;
@@ -1970,7 +2232,7 @@ LsaSrvDeleteObject(
                   dwError == LW_ERROR_NO_SUCH_OBJECT ||
                   dwError == LW_ERROR_NO_SUCH_USER ||
                   dwError == LW_ERROR_NO_SUCH_GROUP) &&
-                 !pszTargetProvider)
+                 !pszTargetProviderName)
         {
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = (HANDLE)NULL;
@@ -1981,7 +2243,7 @@ LsaSrvDeleteObject(
         }
     }
 
-    if (pszTargetProvider && !bFoundProvider)
+    if (pszTargetProviderName && !bFoundProvider)
     {
         dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
@@ -1989,6 +2251,9 @@ LsaSrvDeleteObject(
 
 cleanup:
 
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
     if (hProvider != (HANDLE)NULL) {
         LsaSrvCloseProvider(pProvider, hProvider);
     }
@@ -2026,10 +2291,14 @@ LsaSrvGetSmartCardUserObject(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnGetSmartCardUserObject(
+        dwError = pProvider->pFnTable->pfnGetSmartCardUserObject(
                                         hProvider,
                                         ppObject,
                                         ppszSmartCardReader);
