@@ -64,31 +64,35 @@ LsaSrvFindNSSArtefactByKey(
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
-    if (IsNullOrEmptyString(pszKeyName))
+    if (LW_IS_NULL_OR_EMPTY_STR(pszKeyName))
     {
-        dwError = LSA_ERROR_INVALID_NSS_KEY_NAME;
+        dwError = LW_ERROR_INVALID_NSS_KEY_NAME;
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    if (IsNullOrEmptyString(pszMapName))
+    if (LW_IS_NULL_OR_EMPTY_STR(pszMapName))
     {
-        dwError = LSA_ERROR_INVALID_NSS_MAP_NAME;
+        dwError = LW_ERROR_INVALID_NSS_MAP_NAME;
         BAIL_ON_LSA_ERROR(dwError);
     }
 
     if (!dwFlags)
     {
-        dwError = LSA_ERROR_INVALID_PARAMETER;
+        dwError = LW_ERROR_INVALID_PARAMETER;
         BAIL_ON_LSA_ERROR(dwError);
     }
 
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
-    dwError = LSA_ERROR_NOT_HANDLED;
+    dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
         dwError = pProvider->pFnTable->pfnLookupNSSArtefactByKey(
@@ -101,9 +105,9 @@ LsaSrvFindNSSArtefactByKey(
         if (!dwError) {
             break;
         }
-        else if ((dwError == LSA_ERROR_NOT_HANDLED) ||
-                   (dwError == LSA_ERROR_NO_SUCH_NSS_KEY) ||
-                   (dwError == LSA_ERROR_NO_SUCH_NSS_MAP)) {
+        else if ((dwError == LW_ERROR_NOT_HANDLED) ||
+                   (dwError == LW_ERROR_NO_SUCH_NSS_KEY) ||
+                   (dwError == LW_ERROR_NO_SUCH_NSS_MAP)) {
 
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = (HANDLE)NULL;
@@ -128,9 +132,12 @@ cleanup:
 
 error:
 
-    LSA_LOG_ERROR("Failed to find NIS Artefact [Map:%s][Key:%s]",
-                    LSA_SAFE_LOG_STRING(pszMapName),
-                    LSA_SAFE_LOG_STRING(pszKeyName));
+    LSA_LOG_ERROR_API_FAILED(
+            hServer,
+            dwError,
+            "find NIS Artefact by key (map = '%s', key = '%s')",
+            LSA_SAFE_LOG_STRING(pszMapName),
+            LSA_SAFE_LOG_STRING(pszKeyName));
 
     *ppNSSArtefactInfo = NULL;
 
@@ -144,17 +151,16 @@ LsaSrvBeginEnumNSSArtefacts(
     LSA_NIS_MAP_QUERY_FLAGS dwFlags,
     DWORD  dwNSSArtefactInfoLevel,
     DWORD  dwMaxNumNSSArtefacts,
-    PSTR*  ppszGUID
+    PHANDLE phState
     )
 {
     DWORD dwError = 0;
     DWORD dwTraceFlags[] = {LSA_TRACE_FLAG_USER_GROUP_QUERIES};
-    PLSA_SRV_RECORD_ENUM_STATE pEnumState = NULL;
-    PSTR pszGUID = NULL;
+    PLSA_SRV_ENUM_STATE pEnumState = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
-    dwError = LsaSrvAddNSSArtefactEnumState(
+    dwError = LsaSrvCreateNSSArtefactEnumState(
                     hServer,
                     pszMapName,
                     dwFlags,
@@ -163,10 +169,7 @@ LsaSrvBeginEnumNSSArtefacts(
                     &pEnumState);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaAllocateString(pEnumState->pszGUID, &pszGUID);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    *ppszGUID = pszGUID;
+    *phState = pEnumState;
 
 cleanup:
 
@@ -175,8 +178,11 @@ cleanup:
     return dwError;
 
 error:
-
-    *ppszGUID = NULL;
+    LSA_LOG_ERROR_API_FAILED(
+            hServer,
+            dwError,
+            "start enumerating NIS Artefacts (map = '%s')",
+            LSA_SAFE_LOG_STRING(pszMapName));
 
     goto cleanup;
 }
@@ -184,7 +190,7 @@ error:
 DWORD
 LsaSrvEnumNSSArtefacts(
     HANDLE  hServer,
-    PCSTR   pszGUID,
+    HANDLE  hState,
     PDWORD  pdwNSSArtefactInfoLevel,
     PVOID** pppNSSArtefactInfoList,
     PDWORD  pdwNumNSSArtefactsFound
@@ -192,7 +198,7 @@ LsaSrvEnumNSSArtefacts(
 {
     DWORD dwError = 0;
     DWORD dwTraceFlags[] = {LSA_TRACE_FLAG_USER_GROUP_QUERIES};
-    PLSA_SRV_RECORD_ENUM_STATE pEnumState = NULL;
+    PLSA_SRV_ENUM_STATE pEnumState = hState;
     PVOID* ppNSSArtefactInfoList_accumulate = NULL;
     DWORD  dwTotalNumNSSArtefactsFound = 0;
     PVOID* ppNSSArtefactInfoList = NULL;
@@ -202,12 +208,6 @@ LsaSrvEnumNSSArtefacts(
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
-    pEnumState = LsaSrvFindNSSArtefactEnumState(hServer, pszGUID);
-    if (!pEnumState) {
-        dwError = LSA_ERROR_INTERNAL;
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
     dwNSSArtefactInfoLevel = pEnumState->dwInfoLevel;
     dwNumNSSArtefactsRemaining = pEnumState->dwNumMaxRecords;
 
@@ -231,7 +231,7 @@ LsaSrvEnumNSSArtefacts(
 
 
         if (dwError) {
-           if (dwError != LSA_ERROR_NO_MORE_NSS_ARTEFACTS) {
+           if (dwError != LW_ERROR_NO_MORE_NSS_ARTEFACTS) {
               BAIL_ON_LSA_ERROR(dwError);
            }
         }
@@ -240,17 +240,18 @@ LsaSrvEnumNSSArtefacts(
 
         if (dwNumNSSArtefactsRemaining) {
            pEnumState->pCurProviderState = pEnumState->pCurProviderState->pNext;
-           if (dwError == LSA_ERROR_NO_MORE_NSS_ARTEFACTS){
+           if (dwError == LW_ERROR_NO_MORE_NSS_ARTEFACTS){
              dwError = 0;
            }
         }
 
-        dwError = LsaCoalesceNSSArtefactInfoList(
-                        &ppNSSArtefactInfoList,
-                        &dwNumNSSArtefactsFound,
+        dwError = LsaAppendAndFreePtrs(
+                        &dwTotalNumNSSArtefactsFound,
                         &ppNSSArtefactInfoList_accumulate,
-                        &dwTotalNumNSSArtefactsFound);
+                        &dwNumNSSArtefactsFound,
+                        &ppNSSArtefactInfoList);
         BAIL_ON_LSA_ERROR(dwError);
+
     }
 
     *pdwNSSArtefactInfoLevel = dwNSSArtefactInfoLevel;
@@ -264,6 +265,10 @@ cleanup:
     return(dwError);
 
 error:
+    LSA_LOG_ERROR_API_FAILED(
+            hServer,
+            dwError,
+            "continue enumerating NIS Artefacts");
 
     *pdwNSSArtefactInfoLevel = 0;
     *pppNSSArtefactInfoList = NULL;
@@ -284,22 +289,16 @@ error:
 DWORD
 LsaSrvEndEnumNSSArtefacts(
     HANDLE hServer,
-    PSTR   pszGUID
+    HANDLE hState
     )
 {
     DWORD dwError = 0;
     DWORD dwTraceFlags[] = {LSA_TRACE_FLAG_USER_GROUP_QUERIES};
-    PLSA_SRV_RECORD_ENUM_STATE pEnumState = NULL;
+    PLSA_SRV_ENUM_STATE pEnumState = hState;
     PLSA_SRV_PROVIDER_STATE pProviderState = NULL;
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
-    pEnumState = LsaSrvFindNSSArtefactEnumState(hServer, pszGUID);
-    if (!pEnumState) {
-        dwError = LSA_ERROR_INTERNAL;
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
     for (pProviderState = pEnumState->pProviderStateList;
          pProviderState;
          pProviderState = pProviderState->pNext)
@@ -309,22 +308,14 @@ LsaSrvEndEnumNSSArtefacts(
            HANDLE hProvider = pProviderState->hProvider;
            pProvider->pFnTable->pfnEndEnumNSSArtefacts(
                                        hProvider,
-                                       pszGUID);
+                                       pProviderState->hResume);
         }
     }
 
-    LsaSrvFreeNSSArtefactEnumState(
-                        hServer,
-                        pszGUID);
-
-cleanup:
+    LsaSrvFreeEnumState(pEnumState);
 
     LSA_TRACE_END_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
     return dwError;
-
-error:
-
-    goto cleanup;
 }
 
