@@ -69,14 +69,18 @@ LsaSrvAuthenticateUserPam(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
         if (pPamAuthInfo)
         {
             LsaFreeAuthUserPamInfo(pPamAuthInfo);
         }
-        dwError = pProvider->pFnTable2->pfnAuthenticateUserPam(
+        dwError = pProvider->pFnTable->pfnAuthenticateUserPam(
                                             hProvider,
                                             pParams,
                                             &pPamAuthInfo);
@@ -161,11 +165,11 @@ error:
     }
     else
     {
-        if (dwError == LW_ERROR_PASSWORD_MISMATCH && LsaSrvLogInvalidPasswords())
+        /* if (dwError == LW_ERROR_PASSWORD_MISMATCH && LsaSrvLogPasswords())
         {
             LSA_LOG_ERROR_API_FAILED(hServer, dwError, "authenticate user (name = '%s', password = '%s')", LSA_SAFE_LOG_STRING(pParams->pszLoginName), LSA_SAFE_LOG_STRING(pParams->pszPassword));
         }
-        else
+        else */
         {
             LSA_LOG_ERROR_API_FAILED(hServer, dwError, "authenticate user (name = '%s')", LSA_SAFE_LOG_STRING(pParams->pszLoginName));
         }
@@ -177,6 +181,7 @@ error:
 DWORD
 LsaSrvAuthenticateUserEx(
     HANDLE hServer,
+    PCSTR pszTargetProvider,
     PLSA_AUTH_USER_PARAMS pUserParams,
     PLSA_AUTH_USER_INFO *ppUserInfo
     )
@@ -188,6 +193,9 @@ LsaSrvAuthenticateUserEx(
     HANDLE hProvider = (HANDLE)NULL;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
     LSA_AUTH_USER_PARAMS localUserParams;
+    BOOLEAN bFoundProvider = FALSE;
+    PSTR pszTargetProviderName = NULL;
+    PSTR pszTargetInstance = NULL;
 
     BAIL_ON_INVALID_POINTER(pUserParams);
     BAIL_ON_INVALID_POINTER(ppUserInfo);
@@ -206,31 +214,35 @@ LsaSrvAuthenticateUserEx(
 	    PSTR pszAccountName = NULL;
 	    DWORD dwLen = 0;
             LSA_AUTH_USER_PAM_PARAMS params = { 0 };
+	    
+	    /* calculate length including '\' and terminating NULL */
 
-	    /* calculate length includeing '\' and terminating NULL */
-
-	    dwLen = strlen(pUserParams->pszDomain) + strlen(pUserParams->pszAccountName) + 2;
+            if (pUserParams->pszDomain)
+            {
+	        dwLen = strlen(pUserParams->pszDomain);
+            }
+	    dwLen += strlen(pUserParams->pszAccountName) + 2;    
 	    dwError = LwAllocateMemory(dwLen, (PVOID*)&pszAccountName);
 	    BAIL_ON_LSA_ERROR(dwError);
-
+    
 	    snprintf(pszAccountName, dwLen,
-		     "%s%s%s",
+		     "%s%s%s", 
 		     pUserParams->pszDomain,
-		     pUserParams->pszDomain ? "\\" : "",
+		     pUserParams->pszDomain ? "\\" : "", 
 		     pUserParams->pszAccountName);
 
             params.pszLoginName = pszAccountName;
             params.pszPassword = pUserParams->pass.clear.pszPassword;
 
 	    /* Pass off plain text auth to AuthenticateUser() */
-	    dwError = LsaSrvAuthenticateUserPam(hServer,
+	    dwError = LsaSrvAuthenticateUserPam(hServer, 
 					     &params,
                                              NULL);
-	    LW_SAFE_FREE_MEMORY(pszAccountName);
+	    LW_SAFE_FREE_MEMORY(pszAccountName);	    
 	    goto cleanup;
 	    break;
     }
-
+    
     case LSA_AUTH_CHAP:
 	    /* NTLM is what we'll do for the rest of the routine */
 	    break;
@@ -238,7 +250,7 @@ LsaSrvAuthenticateUserEx(
 	    /* Bad AuthType */
 	    dwError = LW_ERROR_INVALID_PARAMETER;
 	    goto cleanup;
-	    break;
+	    break;	    
     }
 
     /* Fix up the name.  This allows us to handle a UPN */
@@ -251,21 +263,46 @@ LsaSrvAuthenticateUserEx(
         /* If they gave us a UPN, our domain MUST be NULL */
         localUserParams.pszDomain = NULL;
     }
-
+    
     /* Do the NTLM authentication */
 
+    if (pszTargetProvider)
+    {
+        dwError = LsaSrvGetTargetElements(
+                      pszTargetProvider,
+                      &pszTargetProviderName,
+                      &pszTargetInstance);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        if (pszTargetProviderName)
+        {
+            if (!strcmp(pszTargetProviderName, pProvider->pszName))
+            {
+                bFoundProvider = TRUE;
+            }
+            else
+            {
+                continue;
+            }
+        }
+
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      pszTargetInstance,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnAuthenticateUserEx(
+        dwError = pProvider->pFnTable->pfnAuthenticateUserEx(
                                             hProvider,
                                             &localUserParams,
                                             ppUserInfo);
-        if (!dwError)
+        if (!dwError) 
         {
             if (LsaSrvEventlogEnabled())
             {
@@ -281,7 +318,7 @@ LsaSrvAuthenticateUserEx(
             break;
         }
         else if ((dwError == LW_ERROR_NOT_HANDLED) ||
-                 (dwError == LW_ERROR_NO_SUCH_USER))
+                 (dwError == LW_ERROR_NO_SUCH_USER)) 
         {
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = (HANDLE)NULL;
@@ -304,6 +341,12 @@ LsaSrvAuthenticateUserEx(
         }
     }
 
+    if (pszTargetProviderName && !bFoundProvider)
+    {
+        dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     if (pProvider == NULL) {
         dwError = LW_ERROR_NOT_HANDLED;
     }
@@ -311,7 +354,10 @@ LsaSrvAuthenticateUserEx(
 
 cleanup:
 
-    if (hProvider != (HANDLE)NULL)
+    LW_SAFE_FREE_STRING(pszTargetProviderName);
+    LW_SAFE_FREE_STRING(pszTargetInstance);
+
+    if (hProvider != (HANDLE)NULL) 
     {
         LsaSrvCloseProvider(pProvider, hProvider);
     }
@@ -323,7 +369,7 @@ cleanup:
         LsaSrvFreeNameInfo(pLoginInfo);
     }
 
-    if (!dwError)
+    if (!dwError) 
     {
         LsaSrvIncrementMetricValue(LsaMetricSuccessfulAuthentications);
     }
@@ -350,7 +396,7 @@ error:
                   pUserParams && pUserParams->pszAccountName ? pUserParams->pszAccountName : "");
     }
 
-    goto cleanup;
+    goto cleanup;	
 }
 
 DWORD
@@ -376,10 +422,14 @@ LsaSrvValidateUser(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnValidateUser(
+        dwError = pProvider->pFnTable->pfnValidateUser(
                                             hProvider,
                                             pszLoginId,
                                             pszPassword);
@@ -443,10 +493,14 @@ LsaSrvCheckUserInList(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnCheckUserInList(
+        dwError = pProvider->pFnTable->pfnCheckUserInList(
                                             hProvider,
                                             pszLoginId,
                                             pszListName);
@@ -536,10 +590,14 @@ LsaSrvChangePassword(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnChangePassword(
+        dwError = pProvider->pFnTable->pfnChangePassword(
                                         hProvider,
                                         pszLoginId,
                                         pszPassword,
@@ -624,10 +682,14 @@ LsaSrvSetPassword(
 
     for (pProvider = gpAuthProviderList; pProvider; pProvider = pProvider->pNext)
     {
-        dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
+        dwError = LsaSrvOpenProvider(
+                      hServer,
+                      pProvider,
+                      NULL,
+                      &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable2->pfnSetPassword(
+        dwError = pProvider->pFnTable->pfnSetPassword(
                                         hProvider,
                                         pszLoginId,
                                         pszPassword);
