@@ -49,72 +49,18 @@
 #ifndef __AUTH_PROVIDER_P_H__
 #define __AUTH_PROVIDER_P_H__
 
-#define LSA_CFG_TAG_AUTH_PROVIDER "auth provider:"
-
-typedef struct __LSA_AUTH_PROVIDER
-{
+typedef struct _LSA_AUTH_PROVIDER {
     PSTR pszId;
     PSTR pszProviderLibpath;
-    PSTR pszName;
     PVOID pLibHandle;
-    PFNSHUTDOWNPROVIDER pFnShutdown;
+    PCSTR pszName;
     PLSA_PROVIDER_FUNCTION_TABLE pFnTable;
-    
-    struct __LSA_AUTH_PROVIDER *pNext;
-    
+    struct _LSA_AUTH_PROVIDER *pNext;
 } LSA_AUTH_PROVIDER, *PLSA_AUTH_PROVIDER;
 
-VOID
-LsaSrvFreeAuthProvider(
-    PLSA_AUTH_PROVIDER pProvider
-    );
-
-VOID
-LsaSrvFreeAuthProviderStack(
-    PLSA_STACK pProviderStack
-    );
-
-VOID
-LsaSrvFreeAuthProviderList(
-    PLSA_AUTH_PROVIDER pProviderList
-    );
-
-DWORD
-LsaSrvValidateProvider(
-    PLSA_AUTH_PROVIDER pProvider
-    );
-
-DWORD
-LsaSrvInitAuthProvider(
-    PCSTR pszConfigFilePath,
-    PLSA_AUTH_PROVIDER pProvider
-    );
-
 DWORD
 LsaSrvInitAuthProviders(
-    PCSTR pszConfigFilePath
-    );
-
-DWORD
-LsaCfgFreeAuthProviderInStack(
-    PVOID pItem,
-    PVOID pUserData
-    );
-
-DWORD
-LsaSrvAuthProviderConfigStartSection(
-    PCSTR    pszSectionName,
-    PVOID    pData,
-    PBOOLEAN pbSkipSection,
-    PBOOLEAN pbContinue
-    );
-
-DWORD
-LsaSrvAuthProviderConfigNameValuePair(
-    PCSTR    pszName,
-    PCSTR    pszValue,
-    PVOID    pData,
-    PBOOLEAN pbContinue
+    IN OPTIONAL PLSA_STATIC_PROVIDER pStaticProviders
     );
 
 VOID
@@ -122,33 +68,6 @@ LsaSrvFreeAuthProviders(
     VOID
     );
 
-DWORD
-LsaSrvIsLocalDomain(
-    PCSTR pszDomain,
-    PBOOLEAN pbLocalDomain
-    );
-
-DWORD
-LsaSrvCrackName(
-    PCSTR pszId,
-    PSTR* ppszDomain,
-    PSTR* ppszName
-    );
-
-DWORD
-LsaSrvCrackUserName(
-    PCSTR pszLoginId,
-    PSTR* ppszDomain,
-    PSTR* ppszUserName
-    );
-
-DWORD
-LsaSrvCrackGroupName(
-    PCSTR pszGroup,
-    PSTR* ppszDomain,
-    PSTR* ppszGroupName
-    );
-
 DWORD
 LsaGetNumberOfProviders_inlock(
     VOID
