@@ -57,8 +57,8 @@ LsaFreeSecurityObjectPrivateAttrs(
     {
         if (pObject->type == LSA_OBJECT_TYPE_USER)
         {
-            LW_SAFE_FREE_MEMORY(pObject->userInfo.pLmHash);
-            LW_SAFE_FREE_MEMORY(pObject->userInfo.pNtHash);
+            LW_SECURE_FREE_MEMORY(pObject->userInfo.pLmHash, pObject->userInfo.dwLmHashLen);
+            LW_SECURE_FREE_MEMORY(pObject->userInfo.pNtHash, pObject->userInfo.dwNtHashLen);
         }
     }
 }
@@ -636,7 +636,7 @@ FreeAuthUserInfo(PLSA_AUTH_USER_INFO *pUserInfo)
     LsaDataBlobFree(&(*pUserInfo)->pSessionKey);
     LsaDataBlobFree(&(*pUserInfo)->pLmSessionKey);
 
-    LW_SAFE_FREE_MEMORY((*pUserInfo)->pSidAttribList);
+    LW_SAFE_FREE_MEMORY((*pUserInfo)->pSidAttribList);	
 
     LW_SAFE_FREE_MEMORY(*pUserInfo);
 
@@ -653,12 +653,13 @@ LsaSrvIpcAuthenticateUserEx(
     )
 {
     DWORD dwError = LW_ERROR_NOT_IMPLEMENTED;
-    PLSA_AUTH_USER_PARAMS pParams = (PLSA_AUTH_USER_PARAMS) pIn->data;
+    PLSA_IPC_AUTH_USER_EX_REQ pReq = (PLSA_IPC_AUTH_USER_EX_REQ) pIn->data;
     PLSA_AUTH_USER_INFO pUserInfo = NULL;
     PLSA_IPC_ERROR pError = NULL;
 
     dwError = LsaSrvAuthenticateUserEx(LsaSrvIpcGetSessionData(pCall),
-                                       pParams,
+                                       pReq->pszTargetProvider,
+                                       &pReq->authUserParams,
                                        &pUserInfo);
 
     if (!dwError)
@@ -1142,6 +1143,7 @@ LsaSrvIpcGetStatus(
 
     dwError = LsaSrvGetStatus(
                     LsaSrvIpcGetSessionData(pCall),
+                    (PSTR)pIn->data,
                     &pLsaStatus);
 
     if (!dwError)
@@ -1294,14 +1296,17 @@ LsaSrvIpcEnumTraceInfo(
         dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
         BAIL_ON_LSA_ERROR(dwError);
 
-        pOut->tag = LSA_R_ENUM_TRACE_INFO_FAILURE;;
+        pOut->tag = LSA_R_ENUM_TRACE_INFO_FAILURE;
         pOut->data = pError;
+        goto error;
     }
 
 cleanup:
+
     return MAP_LW_ERROR_IPC(dwError);
 
 error:
+
     if (pResult)
     {
         LW_SAFE_FREE_MEMORY(pResult->pTraceInfoArray);
@@ -1547,7 +1552,7 @@ LsaSrvIpcOpenEnumObjects(
             pReq->FindFlags,
             pReq->ObjectType,
             pReq->pszDomainName);
-
+    
     if (!dwError)
     {
         dwError = LsaSrvIpcRegisterHandle(
@@ -1661,7 +1666,7 @@ LsaSrvIpcOpenEnumMembers(
             &hEnum,
             pReq->FindFlags,
             pReq->pszSid);
-
+    
     if (!dwError)
     {
         dwError = LsaSrvIpcRegisterHandle(
@@ -1930,7 +1935,7 @@ LsaSrvIpcCloseEnum(
     {
         dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         pOut->tag = LSA2_R_ERROR;
         pOut->data = pError;
     }
