@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see 
+ * Public License along with this program.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -38,7 +38,7 @@
  * Abstract:
  *
  *        Likewise Security and Authentication Subsystem (LSASS)
- * 
+ *
  *        Metrics (Server)
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -58,39 +58,40 @@ LsaSrvGetMetrics(
     PVOID pMetricPack = NULL;
 
     BAIL_ON_INVALID_POINTER(ppMetricPack);
-    
+
     switch(dwInfoLevel)
     {
         case 0:
-            
+
             dwError = LsaSrvGetMetrics_0(
                             &pMetricPack);
             break;
-            
+
         case 1:
-            
+
             dwError = LsaSrvGetMetrics_1(
                             &pMetricPack);
             break;
-            
+
         default:
-            
-            dwError = LSA_ERROR_INVALID_METRIC_INFO_LEVEL;
+
+            dwError = LW_ERROR_INVALID_METRIC_INFO_LEVEL;
             break;
     }
     BAIL_ON_LSA_ERROR(dwError);
-    
+
     *ppMetricPack = pMetricPack;
-    
+
 cleanup:
 
     return dwError;
-    
+
 error:
+    LSA_LOG_ERROR_API_FAILED(hServer, dwError, "get metrics (level=%u)", dwInfoLevel);
 
     *ppMetricPack = NULL;
-    
-    LSA_SAFE_FREE_MEMORY(pMetricPack);
+
+    LW_SAFE_FREE_MEMORY(pMetricPack);
 
     goto cleanup;
 }
@@ -105,12 +106,12 @@ LsaSrvGetMetrics_0(
 
     pthread_rwlock_rdlock(&gPerfCounters_rwlock);
 
-    dwError = LsaAllocateMemory(
+    dwError = LwAllocateMemory(
                   sizeof(LSA_METRIC_PACK_0),
                   (PVOID*)&pMetricPack);
     BAIL_ON_LSA_ERROR(dwError);
 
-    pMetricPack->failedAuthentications = 
+    pMetricPack->failedAuthentications =
                  gPerfCounters[LsaMetricFailedAuthentications] ;
     pMetricPack->failedUserLookupsByName =
                  gPerfCounters[LsaMetricFailedUserLookupsByName];
@@ -126,23 +127,21 @@ LsaSrvGetMetrics_0(
                  gPerfCounters[LsaMetricFailedCloseSession];
     pMetricPack->failedChangePassword =
                  gPerfCounters[LsaMetricFailedChangePassword];
-    pMetricPack->unauthorizedAccesses =
-                 gPerfCounters[LsaMetricUnauthorizedAccesses];
-    
+
     *ppMetricPack = pMetricPack;
-    
+
 cleanup:
 
     pthread_rwlock_unlock(&gPerfCounters_rwlock);
 
     return dwError;
-    
+
 error:
 
     *ppMetricPack = NULL;
-    
-    LSA_SAFE_FREE_MEMORY(pMetricPack);
-    
+
+    LW_SAFE_FREE_MEMORY(pMetricPack);
+
     goto cleanup;
 }
 
@@ -156,7 +155,7 @@ LsaSrvGetMetrics_1(
 
     pthread_rwlock_rdlock(&gPerfCounters_rwlock);
 
-    dwError = LsaAllocateMemory(
+    dwError = LwAllocateMemory(
                   sizeof(LSA_METRIC_PACK_1),
                   (PVOID*)&pMetricPack);
     BAIL_ON_LSA_ERROR(dwError);
@@ -195,22 +194,20 @@ LsaSrvGetMetrics_1(
                  gPerfCounters[LsaMetricSuccessfulChangePassword];
     pMetricPack->failedChangePassword =
                  gPerfCounters[LsaMetricFailedChangePassword];
-    pMetricPack->unauthorizedAccesses =
-                 gPerfCounters[LsaMetricUnauthorizedAccesses];
-    
+
     *ppMetricPack = pMetricPack;
-    
+
 cleanup:
 
     pthread_rwlock_unlock(&gPerfCounters_rwlock);
 
     return dwError;
-    
+
 error:
 
     *ppMetricPack = NULL;
-    
-    LSA_SAFE_FREE_MEMORY(pMetricPack);
+
+    LW_SAFE_FREE_MEMORY(pMetricPack);
 
     goto cleanup;
 }
@@ -221,9 +218,32 @@ LsaSrvIncrementMetricValue(
     )
 {
     pthread_rwlock_wrlock(&gPerfCounters_rwlock);
-    
+
     gPerfCounters[metricType]++;
-    
+
     pthread_rwlock_unlock(&gPerfCounters_rwlock);
 }
 
+VOID
+LsaSrvFreeIpcMetriPack(
+    PLSA_METRIC_PACK pMetricPack
+    )
+{
+    if (pMetricPack)
+    {
+        switch (pMetricPack->dwInfoLevel)
+        {
+            case 0:
+                LW_SAFE_FREE_MEMORY(pMetricPack->pMetricPack.pMetricPack0);
+                break;
+            case 1:
+                LW_SAFE_FREE_MEMORY(pMetricPack->pMetricPack.pMetricPack1);
+                break;
+            default:
+                {
+                    LSA_LOG_ERROR("Unsupported Metric Pack Info Level [%u]", pMetricPack->dwInfoLevel);
+                }
+        }
+        LwFreeMemory(pMetricPack);
+    }
+}
