@@ -83,7 +83,7 @@ LsaCheckInvalidRpcServer(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-
+    
 cleanup:
     return dwError;
 
@@ -238,14 +238,13 @@ LsaStartRpcServers(
     PLSA_RPC_SERVER pRpcServerList
     )
 {
-    DWORD dwError = 0;
     PLSA_RPC_SERVER pRpc = NULL;
 
     while (pRpcServerList) {
         pRpc = pRpcServerList;
         pRpcServerList = pRpcServerList->pNext;
 
-        dwError = LsaStartRpcSrv(pRpc);
+        (void) LsaStartRpcSrv(pRpc);
     }
 }
 
@@ -277,14 +276,13 @@ LsaStopRpcServers(
     PLSA_RPC_SERVER pRpcServerList
     )
 {
-    DWORD dwError = 0;
     PLSA_RPC_SERVER pRpc = NULL;
 
     while (pRpcServerList) {
         pRpc = pRpcServerList;
         pRpcServerList = pRpcServerList->pNext;
 
-        dwError = LsaStopRpcSrv(pRpc);
+        (void) LsaStopRpcSrv(pRpc);
     }
 }
 
@@ -390,7 +388,7 @@ cleanup:
 
 error:
     LW_SAFE_FREE_STRING(pszPath);
-    if (pRpcSrv)
+    if (pRpcSrv) 
     {
         LsaSrvFreeRpcServer(pRpcSrv);
         pRpcSrv = NULL;
