@@ -49,6 +49,17 @@
 
 #include "adprovider.h"
 
+#define AD_PROVIDER_POLICY_REGKEY "Policy\\" AD_PROVIDER_REGKEY
+#define AD_PROVIDER_POLICY_DOMAINJOIN_REGKEY "Policy\\" AD_PROVIDER_DOMAINJOIN_REGKEY
+
+static
+DWORD
+AD_CheckList(
+    PCSTR pszNeedle,
+    PCSTR pszaHaystack,
+    PBOOLEAN pbFoundIt
+    );
+
 static
 DWORD
 AD_SetConfig_Umask(
@@ -131,6 +142,9 @@ AD_InitializeConfig(
     pConfig->DomainManager.ppszTrustExceptionList = NULL;
     pConfig->DomainManager.dwTrustExceptionCount = 0;
 
+    pConfig->bMultiTenancyEnabled = FALSE;
+    pConfig->bAddDomainToLocalGroupsEnabled = FALSE;
+
     dwError = LwAllocateString(
                     AD_DEFAULT_SHELL,
                     &pConfig->pszShell);
@@ -180,6 +194,8 @@ AD_FreeConfigContents(
     LW_SAFE_FREE_STRING(pConfig->pszHomedirTemplate);
     LW_SAFE_FREE_STRING(pConfig->pszShell);
     LW_SAFE_FREE_STRING(pConfig->pszSkelDirs);
+    LW_SAFE_FREE_MEMORY(pConfig->pszaIgnoreUserNameList);
+    LW_SAFE_FREE_MEMORY(pConfig->pszaIgnoreGroupNameList);
     LW_SAFE_FREE_STRING(pConfig->pszUserDomainPrefix);
 
     if (pConfig->pUnresolvedMemberList)
@@ -210,10 +226,13 @@ AD_FreeConfigMemberInList(
 
 DWORD
 AD_ReadRegistry(
-    PLSA_AD_CONFIG pConfig
+    IN OPTIONAL PCSTR pszDomainName,
+    OUT PLSA_AD_CONFIG pConfig
     )
 {
     DWORD dwError = 0;
+    PSTR pszDomainKey = NULL;
+    PSTR pszDomainPolicyKey = NULL;
     PSTR pszUmask = NULL;
     PSTR pszUnresolvedMemberList = NULL;
     DWORD dwMachinePasswordSyncLifetime = 0;
@@ -225,7 +244,7 @@ AD_ReadRegistry(
         "unprovisioned"
     };
 
-    const PCSTR CacheBackend[] =
+    const PCSTR CacheBackend[] = 
     {
         "sqlite",
         "memory"
@@ -512,6 +531,46 @@ AD_ReadRegistry(
             &pszIncludeTrustsListMultiString,
             NULL
         },
+        {
+            "IgnoreUserNameList",
+            TRUE,
+            LsaTypeMultiString,
+            0,
+            MAXDWORD,
+            NULL,
+            &StagingConfig.pszaIgnoreUserNameList,
+            NULL
+        },
+        {
+            "IgnoreGroupNameList",
+            TRUE,
+            LsaTypeMultiString,
+            0,
+            MAXDWORD,
+            NULL,
+            &StagingConfig.pszaIgnoreGroupNameList,
+            NULL
+        },
+        {
+            "MultiTenancyEnabled",
+            TRUE,
+            LsaTypeBoolean,
+            0,
+            MAXDWORD,
+            NULL,
+            &StagingConfig.bMultiTenancyEnabled,
+            NULL
+        },
+        {
+            "AddDomainToLocalGroupsEnabled",
+            TRUE,
+            LsaTypeBoolean,
+            0,
+            MAXDWORD,
+            NULL,
+            &StagingConfig.bAddDomainToLocalGroupsEnabled,
+            NULL
+        }
     };
 
     LSA_CONFIG LsaConfigDescription[] =
@@ -534,12 +593,36 @@ AD_ReadRegistry(
     dwMachinePasswordSyncLifetime = AD_MACHINE_PASSWORD_SYNC_DEFAULT_SECS;
 
     dwError = LsaProcessConfig(
-                "Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
-                "Policy\\Services\\lsass\\Parameters\\Providers\\ActiveDirectory",
+                AD_PROVIDER_REGKEY,
+                AD_PROVIDER_POLICY_REGKEY,
                 ADConfigDescription,
                 sizeof(ADConfigDescription)/sizeof(ADConfigDescription[0]));
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (pszDomainName)
+    {
+        dwError = LwAllocateStringPrintf(
+                     &pszDomainKey,
+                     "%s\\%s",
+                     AD_PROVIDER_DOMAINJOIN_REGKEY,
+                     pszDomainName);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        dwError = LwAllocateStringPrintf(
+                     &pszDomainPolicyKey,
+                     "%s\\%s",
+                     AD_PROVIDER_POLICY_DOMAINJOIN_REGKEY,
+                     pszDomainName);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        dwError = LsaProcessConfig(
+                      pszDomainKey,
+                      pszDomainPolicyKey,
+                      ADConfigDescription,
+                      sizeof(ADConfigDescription)/sizeof(ADConfigDescription[0]));
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LsaProcessConfig(
                 "Services\\lsass\\Parameters",
                 "Policy\\Services\\lsass\\Parameters",
@@ -571,8 +654,12 @@ AD_ReadRegistry(
     AD_TransferConfigContents(&StagingConfig, pConfig);
 
 cleanup:
+    LW_SAFE_FREE_STRING(pszDomainKey);
+    LW_SAFE_FREE_STRING(pszDomainPolicyKey);
     LW_SAFE_FREE_STRING(pszUmask);
     LW_SAFE_FREE_STRING(pszUnresolvedMemberList);
+    LW_SAFE_FREE_STRING(pszExcludeTrustsListMultiString);
+    LW_SAFE_FREE_STRING(pszIncludeTrustsListMultiString);
 
     AD_FreeConfigContents(&StagingConfig);
 
@@ -807,6 +894,7 @@ AD_SetConfig_DomainManager_TrustExceptionList(
 
 DWORD
 AD_GetUnprovisionedModeShell(
+    PLSA_AD_PROVIDER_STATE pState,
     PSTR* ppszUnprovisionedModeShell
     )
 {
@@ -814,12 +902,12 @@ AD_GetUnprovisionedModeShell(
     BOOLEAN bInLock = FALSE;
     PSTR pszUnprovisionedModeShell = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    if (!LW_IS_NULL_OR_EMPTY_STR(gpLsaAdProviderState->config.pszShell))
+    if (!LW_IS_NULL_OR_EMPTY_STR(pState->config.pszShell))
     {
         dwError = LwAllocateString(
-                        gpLsaAdProviderState->config.pszShell,
+                        pState->config.pszShell,
                         &pszUnprovisionedModeShell);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -828,7 +916,7 @@ AD_GetUnprovisionedModeShell(
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -841,6 +929,7 @@ error:
 
 DWORD
 AD_GetHomedirPrefixPath(
+    PLSA_AD_PROVIDER_STATE pState,
     PSTR* ppszPath
     )
 {
@@ -848,12 +937,12 @@ AD_GetHomedirPrefixPath(
     BOOLEAN bInLock = FALSE;
     PSTR  pszHomedirPrefixPath = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    if (!LW_IS_NULL_OR_EMPTY_STR(gpLsaAdProviderState->config.pszHomedirPrefix))
+    if (!LW_IS_NULL_OR_EMPTY_STR(pState->config.pszHomedirPrefix))
     {
         dwError = LwAllocateString(
-                        gpLsaAdProviderState->config.pszHomedirPrefix,
+                        pState->config.pszHomedirPrefix,
                         &pszHomedirPrefixPath
                         );
         BAIL_ON_LSA_ERROR(dwError);
@@ -863,7 +952,7 @@ AD_GetHomedirPrefixPath(
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -876,6 +965,7 @@ error:
 
 DWORD
 AD_GetUserDomainPrefix(
+    PLSA_AD_PROVIDER_STATE pState,
     PSTR* ppszPath
     )
 {
@@ -883,30 +973,33 @@ AD_GetUserDomainPrefix(
     BOOLEAN bInLock = FALSE;
     PSTR  pszValue = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    if (!LW_IS_NULL_OR_EMPTY_STR(gpLsaAdProviderState->config.pszUserDomainPrefix))
+    if (!LW_IS_NULL_OR_EMPTY_STR(pState->config.pszUserDomainPrefix))
     {
         dwError = LwAllocateString(
-                        gpLsaAdProviderState->config.pszUserDomainPrefix,
+                        pState->config.pszUserDomainPrefix,
                         &pszValue
                         );
         BAIL_ON_LSA_ERROR(dwError);
     }
-    else if (!LW_IS_NULL_OR_EMPTY_STR(gpADProviderData->szShortDomain))
+    else if (pState->pProviderData &&
+             !LW_IS_NULL_OR_EMPTY_STR(pState->pProviderData->szShortDomain))
     {
         dwError = LwAllocateString(
-                        gpADProviderData->szShortDomain,
+                        pState->pProviderData->szShortDomain,
                         &pszValue
                         );
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    LwStrToUpper(pszValue);
+
     *ppszPath = pszValue;
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -919,6 +1012,7 @@ error:
 
 DWORD
 AD_GetUnprovisionedModeHomedirTemplate(
+    PLSA_AD_PROVIDER_STATE pState,
     PSTR* ppszUnprovisionedModeHomedirTemplate
     )
 {
@@ -926,12 +1020,12 @@ AD_GetUnprovisionedModeHomedirTemplate(
     BOOLEAN bInLock = FALSE;
     PSTR pszUnprovisionedModeHomedirTemplate = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    if (!LW_IS_NULL_OR_EMPTY_STR(gpLsaAdProviderState->config.pszHomedirTemplate))
+    if (!LW_IS_NULL_OR_EMPTY_STR(pState->config.pszHomedirTemplate))
     {
         dwError = LwAllocateString(
-                        gpLsaAdProviderState->config.pszHomedirTemplate,
+                        pState->config.pszHomedirTemplate,
                         &pszUnprovisionedModeHomedirTemplate
                         );
         BAIL_ON_LSA_ERROR(dwError);
@@ -941,7 +1035,7 @@ AD_GetUnprovisionedModeHomedirTemplate(
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -954,68 +1048,69 @@ error:
 
 DWORD
 AD_GetMachinePasswordSyncPwdLifetime(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwMachinePasswordSyncPwdLifetime = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
-    dwMachinePasswordSyncPwdLifetime = gpLsaAdProviderState->config.dwMachinePasswordSyncLifetime;
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    dwMachinePasswordSyncPwdLifetime = pState->config.dwMachinePasswordSyncLifetime;
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwMachinePasswordSyncPwdLifetime;
 }
 
 DWORD
 AD_GetClockDriftSeconds(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwClockDriftSecs = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
-    dwClockDriftSecs = gpLsaAdProviderState->dwMaxAllowedClockDriftSeconds;
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    dwClockDriftSecs = pState->dwMaxAllowedClockDriftSeconds;
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwClockDriftSecs;
 }
 
 DWORD
 AD_GetCacheEntryExpirySeconds(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwResult = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    dwResult = gpLsaAdProviderState->config.dwCacheEntryExpirySecs;
+    dwResult = pState->config.dwCacheEntryExpirySecs;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwResult;
 }
 
 DWORD
 AD_GetUmask(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwResult = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
-    dwResult = gpLsaAdProviderState->config.dwUmask;
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    dwResult = pState->config.dwUmask;
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwResult;
 }
 
 DWORD
 AD_GetSkelDirs(
+    PLSA_AD_PROVIDER_STATE pState,
     PSTR* ppszSkelDirs
     )
 {
@@ -1023,12 +1118,12 @@ AD_GetSkelDirs(
     BOOLEAN bInLock = FALSE;
     PSTR pszSkelDirs = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    if (!LW_IS_NULL_OR_EMPTY_STR(gpLsaAdProviderState->config.pszSkelDirs))
+    if (!LW_IS_NULL_OR_EMPTY_STR(pState->config.pszSkelDirs))
     {
         dwError = LwAllocateString(
-                        gpLsaAdProviderState->config.pszSkelDirs,
+                        pState->config.pszSkelDirs,
                         &pszSkelDirs);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -1037,7 +1132,7 @@ AD_GetSkelDirs(
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -1050,17 +1145,17 @@ error:
 
 BOOLEAN
 AD_GetLDAPSignAndSeal(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bResult = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bResult = gpLsaAdProviderState->config.bLDAPSignAndSeal;
+    bResult = pState->config.bLDAPSignAndSeal;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bResult;
 }
@@ -1068,13 +1163,14 @@ AD_GetLDAPSignAndSeal(
 static
 BOOLEAN
 AD_IsInMembersList_InLock(
+    PLSA_AD_PROVIDER_STATE pState,
     PCSTR pszMember
     )
 {
     PDLINKEDLIST pIter = NULL;
     BOOLEAN      bInList = FALSE;
 
-    for (pIter = gpLsaAdProviderState->config.pUnresolvedMemberList;
+    for (pIter = pState->config.pUnresolvedMemberList;
          pIter;
          pIter = pIter->pNext)
     {
@@ -1091,13 +1187,14 @@ AD_IsInMembersList_InLock(
 static
 VOID
 AD_DeleteFromMembersList_InLock(
+    PLSA_AD_PROVIDER_STATE pState,
     PCSTR pszMember
     )
 {
     PDLINKEDLIST pIter = NULL;
     PVOID        pItem = NULL;
 
-    for (pIter = gpLsaAdProviderState->config.pUnresolvedMemberList;
+    for (pIter = pState->config.pUnresolvedMemberList;
          pIter;
          pIter = pIter->pNext)
     {
@@ -1110,7 +1207,7 @@ AD_DeleteFromMembersList_InLock(
 
     if (pItem)
     {
-        LsaDLinkedListDelete(&gpLsaAdProviderState->config.pUnresolvedMemberList,
+        LsaDLinkedListDelete(&pState->config.pUnresolvedMemberList,
                              pItem);
 
         LwFreeMemory(pItem);
@@ -1119,22 +1216,23 @@ AD_DeleteFromMembersList_InLock(
 
 VOID
 AD_DeleteFromMembersList(
+    PLSA_AD_PROVIDER_STATE pState,
     PCSTR pszMember
     )
 {
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_WRITER_LOCK(bInLock, pState);
 
-    AD_DeleteFromMembersList_InLock(pszMember);
+    AD_DeleteFromMembersList_InLock(pState, pszMember);
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_WRITER_LOCK(bInLock, pState);
 }
 
 static
 void
 AD_FreeHashStringKeyValue(
-    const LSA_HASH_ENTRY *pEntry)
+    const LW_HASH_ENTRY *pEntry)
 {
     PSTR pszKeyCopy = (PSTR)pEntry->pKey;
     LW_SAFE_FREE_STRING(pszKeyCopy);
@@ -1145,8 +1243,8 @@ AD_FreeHashStringKeyValue(
 static
 DWORD
 AD_CopyHashStringKeyValue(
-    const LSA_HASH_ENTRY *pEntry,
-    LSA_HASH_ENTRY       *pEntryCopy
+    const LW_HASH_ENTRY *pEntry,
+    LW_HASH_ENTRY       *pEntryCopy
     )
 {
     DWORD dwError = 0;
@@ -1178,9 +1276,10 @@ error:
 
 DWORD
 AD_AddAllowedMember(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR               pszSID,
     IN PSTR                pszMember,
-    IN OUT PLSA_HASH_TABLE *ppAllowedMemberList
+    IN OUT PLW_HASH_TABLE *ppAllowedMemberList
     )
 {
     DWORD dwError = 0;
@@ -1188,28 +1287,28 @@ AD_AddAllowedMember(
     PSTR  pszValue = NULL;
     PSTR  pszSIDCopy = NULL;
     PSTR  pszMemberCopy = NULL;
-    PLSA_HASH_TABLE pAllowedMemberList = *ppAllowedMemberList;
+    PLW_HASH_TABLE pAllowedMemberList = *ppAllowedMemberList;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_WRITER_LOCK(bInLock, pState);
 
-    if (!gpAllowedSIDs)
+    if (!pState->pAllowedSIDs)
     {
-        dwError = LsaHashCreate(
+        dwError = LwHashCreate(
                         11,
-                        LsaHashCaselessStringCompare,
-                        LsaHashCaselessStringHash,
+                        LwHashCaselessStringCompare,
+                        LwHashCaselessStringHash,
                         AD_FreeHashStringKeyValue,
                         AD_CopyHashStringKeyValue,
-                        &gpAllowedSIDs);
+                        &pState->pAllowedSIDs);
         BAIL_ON_LSA_ERROR(dwError);
     }
 
     if (!pAllowedMemberList)
     {
-        dwError = LsaHashCreate(
+        dwError = LwHashCreate(
                         11,
-                        LsaHashCaselessStringCompare,
-                        LsaHashCaselessStringHash,
+                        LwHashCaselessStringCompare,
+                        LwHashCaselessStringHash,
                         AD_FreeHashStringKeyValue,
                         AD_CopyHashStringKeyValue,
                         &pAllowedMemberList);
@@ -1226,7 +1325,7 @@ AD_AddAllowedMember(
                     &pszMemberCopy);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashSetValue(
+    dwError = LwHashSetValue(
                     pAllowedMemberList,
                     pszSIDCopy,
                     pszMemberCopy);
@@ -1235,10 +1334,10 @@ AD_AddAllowedMember(
     pszSIDCopy = NULL;
     pszMemberCopy = NULL;
 
-    if ( AD_IsInMembersList_InLock(pszMember) )
+    if ( AD_IsInMembersList_InLock(pState, pszMember) )
     {
-        dwError = LsaHashGetValue(
-                      gpAllowedSIDs,
+        dwError = LwHashGetValue(
+                      pState->pAllowedSIDs,
                       pszSID,
                       (PVOID*)&pszValue);
         if (dwError == ERROR_NOT_FOUND)
@@ -1253,8 +1352,8 @@ AD_AddAllowedMember(
                           &pszMemberCopy);
             BAIL_ON_LSA_ERROR(dwError);
 
-            dwError = LsaHashSetValue(
-                          gpAllowedSIDs,
+            dwError = LwHashSetValue(
+                          pState->pAllowedSIDs,
                           pszSIDCopy,
                           pszMemberCopy);
             BAIL_ON_LSA_ERROR(dwError);
@@ -1263,7 +1362,7 @@ AD_AddAllowedMember(
             pszMemberCopy = NULL;
         }
 
-        AD_DeleteFromMembersList_InLock(pszMember);
+        AD_DeleteFromMembersList_InLock(pState, pszMember);
     }
 
     *ppAllowedMemberList = pAllowedMemberList;
@@ -1273,7 +1372,7 @@ cleanup:
     LW_SAFE_FREE_STRING(pszSIDCopy);
     LW_SAFE_FREE_STRING(pszMemberCopy);
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_WRITER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -1281,17 +1380,114 @@ error:
 
     if ( ! *ppAllowedMemberList )
     {
-        LsaHashSafeFree(&pAllowedMemberList);
+        LwHashSafeFree(&pAllowedMemberList);
     }
 
     goto cleanup;
 }
 
+DWORD
+AD_CheckIgnoreUserNameList(
+    PLSA_AD_PROVIDER_STATE pState,
+    PCSTR pszUserName,
+    PBOOLEAN pbFoundIt
+    )
+{
+    DWORD dwError = 0;
+    BOOLEAN bInLock = FALSE;
+    BOOLEAN bFoundIt = FALSE;
+
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    dwError = AD_CheckList(
+                    pszUserName,
+                    pState->config.pszaIgnoreUserNameList,
+                    &bFoundIt);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    *pbFoundIt = bFoundIt;
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+DWORD
+AD_CheckIgnoreGroupNameList(
+    PLSA_AD_PROVIDER_STATE pState,
+    PCSTR pszGroupName,
+    PBOOLEAN pbFoundIt
+    )
+{
+    DWORD dwError = 0;
+    BOOLEAN bInLock = FALSE;
+    BOOLEAN bFoundIt = FALSE;
+
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    dwError = AD_CheckList(
+                    pszGroupName,
+                    pState->config.pszaIgnoreGroupNameList,
+                    &bFoundIt);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    *pbFoundIt = bFoundIt;
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+static
+DWORD
+AD_CheckList(
+    PCSTR pszNeedle,
+    PCSTR pszaHaystack,
+    PBOOLEAN pbFoundIt
+    )
+{
+    DWORD dwError = 0;
+    BOOLEAN bFoundIt = FALSE;
+    PCSTR pszCurrent = NULL;
+
+    BAIL_ON_INVALID_STRING(pszNeedle);
+
+    pszCurrent = pszaHaystack;
+    if (pszCurrent)
+    {
+        while (*pszCurrent)
+        {
+            if (!strcasecmp(pszCurrent, pszNeedle))
+            {
+                bFoundIt = TRUE;
+                goto cleanup;
+            }
+            pszCurrent = pszCurrent + strlen(pszCurrent) + 1;
+        }
+    }
+
+cleanup:
+    *pbFoundIt = bFoundIt;
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
 DWORD
 AD_GetMemberLists(
+    IN PLSA_AD_PROVIDER_STATE pState,
     PSTR** pppszMembers,
     PDWORD pdwNumMembers,
-    PLSA_HASH_TABLE* ppAllowedMemberList
+    PLW_HASH_TABLE* ppAllowedMemberList
     )
 {
     DWORD dwError = 0;
@@ -1299,11 +1495,11 @@ AD_GetMemberLists(
     DWORD dwNumMembers = 0;
     PDLINKEDLIST pIter = NULL;
     PSTR* ppszMembers = NULL;
-    PLSA_HASH_TABLE pAllowedMemberList = NULL;
+    PLW_HASH_TABLE pAllowedMemberList = NULL;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    for (pIter = gpLsaAdProviderState->config.pUnresolvedMemberList; pIter; pIter = pIter->pNext)
+    for (pIter = pState->config.pUnresolvedMemberList; pIter; pIter = pIter->pNext)
     {
         dwNumMembers++;
     }
@@ -1317,7 +1513,7 @@ AD_GetMemberLists(
                         (PVOID*)&ppszMembers);
         BAIL_ON_LSA_ERROR(dwError);
 
-        for (pIter = gpLsaAdProviderState->config.pUnresolvedMemberList;
+        for (pIter = pState->config.pUnresolvedMemberList;
              pIter;
              pIter = pIter->pNext, iMember++)
         {
@@ -1328,10 +1524,10 @@ AD_GetMemberLists(
         }
     }
 
-    if ( gpAllowedSIDs )
+    if (pState->pAllowedSIDs)
     {
-        dwError = LsaHashCopy(
-                      gpAllowedSIDs,
+        dwError = LwHashCopy(
+                      pState->pAllowedSIDs,
                       &pAllowedMemberList);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -1342,7 +1538,7 @@ AD_GetMemberLists(
 
 cleanup:
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 
@@ -1357,7 +1553,7 @@ error:
     *pdwNumMembers = 0;
     *ppAllowedMemberList = NULL;
 
-    LsaHashSafeFree(&pAllowedMemberList);
+    LwHashSafeFree(&pAllowedMemberList);
 
     goto cleanup;
 }
@@ -1365,12 +1561,12 @@ error:
 static
 BOOLEAN
 AD_ShouldFilterUserLoginsByGroup_InLock(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bFilter = FALSE;
 
-    if (gpAllowedSIDs || gpLsaAdProviderState->config.pUnresolvedMemberList)
+    if (pState->pAllowedSIDs || pState->config.pUnresolvedMemberList)
     {
         bFilter = TRUE;
     }
@@ -1380,33 +1576,34 @@ AD_ShouldFilterUserLoginsByGroup_InLock(
 
 BOOLEAN
 AD_ShouldFilterUserLoginsByGroup(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bFilter = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bFilter = AD_ShouldFilterUserLoginsByGroup_InLock();
+    bFilter = AD_ShouldFilterUserLoginsByGroup_InLock(pState);
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bFilter;
 }
 
 BOOLEAN
 AD_IsMemberAllowed(
+    IN PLSA_AD_PROVIDER_STATE pState,
     PCSTR           pszSID,
-    PLSA_HASH_TABLE pAllowedMemberList
+    PLW_HASH_TABLE pAllowedMemberList
     )
 {
     BOOLEAN bAllowed = FALSE;
     PSTR    pszValue = NULL;
 
-    if (!AD_ShouldFilterUserLoginsByGroup() ||
+    if (!AD_ShouldFilterUserLoginsByGroup(pState) ||
         (pAllowedMemberList &&
-         !LsaHashGetValue(
+         !LwHashGetValue(
                         pAllowedMemberList,
                         pszSID,
                         (PVOID*)&pszValue)))
@@ -1419,284 +1616,286 @@ AD_IsMemberAllowed(
 
 VOID
 AD_FreeAllowedSIDs_InLock(
-    VOID)
+    IN PLSA_AD_PROVIDER_STATE pState
+    )
 {
-    if (gpAllowedSIDs)
+    if (pState->pAllowedSIDs)
     {
-        LsaHashSafeFree(&gpAllowedSIDs);
+        LwHashSafeFree(&pState->pAllowedSIDs);
     }
 }
 
 BOOLEAN
 AD_ShouldAssumeDefaultDomain(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bAssumeDefaultDomain = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bAssumeDefaultDomain = gpLsaAdProviderState->config.bAssumeDefaultDomain;
+    bAssumeDefaultDomain = pState->config.bAssumeDefaultDomain;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bAssumeDefaultDomain;
 }
 
 BOOLEAN
 AD_ShouldSyncSystemTime(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bSyncSystemTime = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bSyncSystemTime = gpLsaAdProviderState->config.bSyncSystemTime;
+    bSyncSystemTime = pState->config.bSyncSystemTime;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bSyncSystemTime;
 }
 
 BOOLEAN
 AD_EventlogEnabled(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bResult = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bResult = gpLsaAdProviderState->config.bEnableEventLog;
+    bResult = pState->config.bEnableEventLog;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bResult;
 }
 
 BOOLEAN
 AD_ShouldLogNetworkConnectionEvents(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bResult = TRUE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bResult = gpLsaAdProviderState->config.bShouldLogNetworkConnectionEvents;
+    bResult = pState->config.bShouldLogNetworkConnectionEvents;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bResult;
 }
 
 BOOLEAN
 AD_ShouldCreateK5Login(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bResult = TRUE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bResult = gpLsaAdProviderState->config.bCreateK5Login;
+    bResult = pState->config.bCreateK5Login;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bResult;
 }
 
 BOOLEAN
 AD_ShouldCreateHomeDir(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bCreateHomeDir = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bCreateHomeDir = gpLsaAdProviderState->config.bCreateHomeDir;
+    bCreateHomeDir = pState->config.bCreateHomeDir;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bCreateHomeDir;
 }
 
 BOOLEAN
 AD_ShouldRefreshUserCreds(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN bRefreshUserCreds = FALSE;
     BOOLEAN bInLock          = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    bRefreshUserCreds = gpLsaAdProviderState->config.bRefreshUserCreds;
+    bRefreshUserCreds = pState->config.bRefreshUserCreds;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return bRefreshUserCreds;
 }
 
 AD_CELL_SUPPORT
 AD_GetCellSupport(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     AD_CELL_SUPPORT result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.CellSupport;
+    result = pState->config.CellSupport;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 AD_CACHE_BACKEND
 AD_GetCacheBackend(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     AD_CACHE_BACKEND result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.CacheBackend;
+    result = pState->config.CacheBackend;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 DWORD
 AD_GetCacheSizeCap(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwResult = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    dwResult = gpLsaAdProviderState->config.dwCacheSizeCap;
+    dwResult = pState->config.dwCacheSizeCap;
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwResult;
 }
 
 BOOLEAN
 AD_GetTrimUserMembershipEnabled(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.bTrimUserMembershipEnabled;
+    result = pState->config.bTrimUserMembershipEnabled;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 BOOLEAN
 AD_GetNssGroupMembersCacheOnlyEnabled(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.bNssGroupMembersCacheOnlyEnabled;
+    result = pState->config.bNssGroupMembersCacheOnlyEnabled;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 BOOLEAN
 AD_GetNssUserMembershipCacheOnlyEnabled(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.bNssUserMembershipCacheOnlyEnabled;
+    result = pState->config.bNssUserMembershipCacheOnlyEnabled;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 BOOLEAN
 AD_GetNssEnumerationEnabled(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     BOOLEAN result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
-    result = gpLsaAdProviderState->config.bNssEnumerationEnabled;
+    result = pState->config.bNssEnumerationEnabled;
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 DWORD
 AD_GetDomainManagerCheckDomainOnlineSeconds(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
-    result = gpLsaAdProviderState->config.DomainManager.dwCheckDomainOnlineSeconds;
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    result = pState->config.DomainManager.dwCheckDomainOnlineSeconds;
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 DWORD
 AD_GetDomainManagerUnknownDomainCacheTimeoutSeconds(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD result = FALSE;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
-    result = gpLsaAdProviderState->config.DomainManager.dwUnknownDomainCacheTimeoutSeconds;
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    result = pState->config.DomainManager.dwUnknownDomainCacheTimeoutSeconds;
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return result;
 }
 
 DWORD
 AD_GetDomainManagerTrustExceptionList(
+    IN PLSA_AD_PROVIDER_STATE pState,
     OUT PBOOLEAN pbIgnoreAllTrusts,
     OUT PSTR** pppszTrustsList,
     OUT PDWORD pdwTrustsCount
@@ -1705,14 +1904,64 @@ AD_GetDomainManagerTrustExceptionList(
     DWORD dwError = 0;
     BOOLEAN bInLock = FALSE;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
-    *pbIgnoreAllTrusts = gpLsaAdProviderState->config.DomainManager.bIgnoreAllTrusts;
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+    *pbIgnoreAllTrusts = pState->config.DomainManager.bIgnoreAllTrusts;
     dwError = LwDuplicateStringArray(
                     pppszTrustsList,
                     pdwTrustsCount,
-                    gpLsaAdProviderState->config.DomainManager.ppszTrustExceptionList,
-                    gpLsaAdProviderState->config.DomainManager.dwTrustExceptionCount);
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+                    pState->config.DomainManager.ppszTrustExceptionList,
+                    pState->config.DomainManager.dwTrustExceptionCount);
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
 
     return dwError;
 }
+
+BOOLEAN
+AD_GetAddDomainToLocalGroupsEnabled(
+    IN PLSA_AD_PROVIDER_STATE pState
+    )
+{
+    BOOLEAN result = FALSE;
+    BOOLEAN bInLock = FALSE;
+
+    ENTER_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    result = pState->config.bAddDomainToLocalGroupsEnabled;
+
+    LEAVE_AD_CONFIG_RW_READER_LOCK(bInLock, pState);
+
+    return result;
+}
+
+VOID
+AD_ConfigLockAcquireRead(
+    PLSA_AD_PROVIDER_STATE pState
+    )
+{
+    int status = 0;
+
+    status = pthread_rwlock_rdlock(pState->pConfigLock);
+    LW_ASSERT(status == 0);
+}
+
+VOID
+AD_ConfigLockAcquireWrite(
+    PLSA_AD_PROVIDER_STATE pState
+    )
+{
+    int status = 0;
+
+    status = pthread_rwlock_wrlock(pState->pConfigLock);
+    LW_ASSERT(status == 0);
+}
+
+void
+AD_ConfigLockRelease(
+    PLSA_AD_PROVIDER_STATE pState
+    )
+{
+    int status = 0;
+
+    status = pthread_rwlock_unlock(pState->pConfigLock);
+    LW_ASSERT(status == 0);
+}
