@@ -50,10 +50,14 @@
 #include "adprovider.h"
 #include "adnetapi.h"
 
-static NetrCredentials gSchannelCreds = { 0 };
-static NetrCredentials* gpSchannelCreds = NULL;
-static NETR_BINDING ghSchannelBinding = NULL;
-static pthread_mutex_t gSchannelLock = PTHREAD_MUTEX_INITIALIZER;
+typedef struct _LSA_SCHANNEL_STATE {
+    NetrCredentials SchannelCreds;
+    NetrCredentials *pSchannelCreds;
+    NETR_BINDING hSchannelBinding;
+    PSTR pszSchannelServer;
+    pthread_mutex_t SchannelLock;
+    pthread_mutex_t *pSchannelLock;
+} LSA_SCHANNEL_STATE;
 
 static
 BOOLEAN
@@ -82,52 +86,47 @@ AD_WinErrorIsConnectionError(
 static
 VOID
 AD_ClearSchannelStateInLock(
-    VOID
+    IN PLSA_SCHANNEL_STATE pSchannelState
     );
 
 static
 DWORD
 AD_GetSystemCreds(
-    LW_PIO_CREDS* ppCreds
+    IN PLSA_AD_PROVIDER_STATE pState,
+    OUT LW_PIO_CREDS* ppCreds
     )
 {
     LW_PIO_CREDS pCreds = NULL;
     DWORD dwError = 0;
-    PSTR pszUsername = NULL;
-    PSTR pszPassword = NULL;
-    PSTR pszDomainDnsName = NULL;
-    PSTR pszHostDnsDomain = NULL;
     PSTR pszMachPrincipal = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
-    dwError = LwKrb5GetMachineCreds(
-                    &pszUsername,
-                    &pszPassword,
-                    &pszDomainDnsName,
-                    &pszHostDnsDomain);
+    dwError = LsaPcacheGetMachineAccountInfoA(
+                  pState->pPcache,
+                  &pAccountInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwAllocateStringPrintf(
                     &pszMachPrincipal,
                     "%s@%s",
-                    pszUsername,
-                    pszDomainDnsName);
+                    pAccountInfo->SamAccountName,
+                    pAccountInfo->DnsDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwIoCreateKrb5CredsA(
                     pszMachPrincipal,
-                    LSASS_CACHE_PATH,
+                    pState->MachineCreds.pszCachePath,
                     &pCreds);
     BAIL_ON_LSA_ERROR(dwError);
 
     *ppCreds = pCreds;
 
 cleanup:
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_FREE_STRING(pszPassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
+
     LW_SAFE_FREE_STRING(pszMachPrincipal);
 
+    LsaPcacheReleaseMachineAccountInfoA(pAccountInfo);
+
     return dwError;
 
 error:
@@ -142,6 +141,7 @@ error:
 
 DWORD
 AD_SetSystemAccess(
+    IN PLSA_AD_PROVIDER_STATE pState,
     OUT OPTIONAL LW_PIO_CREDS* ppOldToken
     )
 {
@@ -149,7 +149,9 @@ AD_SetSystemAccess(
     LW_PIO_CREDS pSystemToken = NULL;
     DWORD dwError = 0;
 
-    dwError = AD_GetSystemCreds(&pSystemToken);
+    dwError = AD_GetSystemCreds(
+                  pState,
+                  &pSystemToken);
     BAIL_ON_LSA_ERROR(dwError);
 
     if (ppOldToken)
@@ -184,60 +186,60 @@ error:
     goto cleanup;
 }
 
-static
-VOID
-AD_ClearSchannelState(
-    VOID
-    );
-
 DWORD
-AD_NetInitMemory(
-    VOID
+AD_NetCreateSchannelState(
+    OUT PLSA_SCHANNEL_STATE* ppSchannelState
     )
 {
     DWORD dwError = 0;
+    PLSA_SCHANNEL_STATE pSchannelState = NULL;
 
-    dwError = LsaRpcInitMemory();
+    dwError = LwAllocateMemory(
+                  sizeof(*pSchannelState),
+                  (PVOID*)&pSchannelState);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = NetrInitMemory();
+    dwError = LwMapErrnoToLwError(pthread_mutex_init(&pSchannelState->SchannelLock, NULL));
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = SamrInitMemory();
-    BAIL_ON_LSA_ERROR(dwError);
+    pSchannelState->pSchannelLock = &pSchannelState->SchannelLock;
+
+    *ppSchannelState = pSchannelState;
+
+cleanup:
+
+    return dwError;
 
 error:
 
-    return dwError;
+    *ppSchannelState = NULL;
+
+    if (pSchannelState)
+    {
+        AD_NetDestroySchannelState(pSchannelState);
+    }
+
+    goto cleanup;
 }
 
-DWORD
-AD_NetShutdownMemory(
-    VOID
+VOID
+AD_NetDestroySchannelState(
+    IN PLSA_SCHANNEL_STATE pSchannelState
     )
 {
-    DWORD dwError = 0;
+    AD_ClearSchannelStateInLock(pSchannelState);
 
-    AD_ClearSchannelState();
+    if (pSchannelState->pSchannelLock)
+    {
+        pthread_mutex_destroy(pSchannelState->pSchannelLock);
+    }
 
-    dwError = SamrDestroyMemory();
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = NetrDestroyMemory();
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LsaRpcDestroyMemory();
-    BAIL_ON_LSA_ERROR(dwError);
-
-error:
-
-    return dwError;
+    LwFreeMemory(pSchannelState);
 }
 
 DWORD
 AD_NetUserChangePassword(
     PCSTR pszDomainName,
-    BOOLEAN bIsInOneWayTrustedDomain,
     PCSTR pszLoginId,
     PCSTR pszUserPrincipalName,
     PCSTR pszOldPassword,
@@ -255,7 +257,6 @@ AD_NetUserChangePassword(
     BAIL_ON_INVALID_STRING(pszLoginId);
 
     dwError = LsaSetSMBCreds(
-                    pszDomainName,
                     pszUserPrincipalName,
                     pszOldPassword,
                     FALSE,
@@ -300,14 +301,25 @@ AD_NetUserChangePassword(
                     pwszLoginId,
                     pwszOldPassword,
                     pwszNewPassword);
+    if (dwError == ERROR_ACCESS_DENIED)
+    {
+        // Try again using machine credentials
+        LsaFreeSMBCreds(&pFreeInfo);
+
+        dwError = LsaUserChangePassword(
+                        pwszDomainName,
+                        pwszLoginId,
+                        pwszOldPassword,
+                        pwszNewPassword);
+    }
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
 
     LW_SAFE_FREE_MEMORY(pwszDomainName);
     LW_SAFE_FREE_MEMORY(pwszLoginId);
-    LW_SAFE_FREE_MEMORY(pwszOldPassword);
-    LW_SAFE_FREE_MEMORY(pwszNewPassword);
+    LW_SECURE_FREE_WSTRING(pwszOldPassword);
+    LW_SECURE_FREE_WSTRING(pwszNewPassword);
     LsaFreeSMBCreds(&pFreeInfo);
 
     return AD_MapNetApiError(dwError);
@@ -355,6 +367,7 @@ GetObjectType(
 
 DWORD
 AD_NetLookupObjectSidByName(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszHostname,
     IN PCSTR pszObjectName,
     OUT PSTR* ppszObjectSid,
@@ -368,6 +381,7 @@ AD_NetLookupObjectSidByName(
     BOOLEAN bIsNetworkError = FALSE;
 
     dwError = AD_NetLookupObjectSidsByNames(
+                 pState,
                  pszHostname,
                  1,
                  (PSTR*)&pszObjectName,
@@ -415,6 +429,7 @@ error:
 
 DWORD
 AD_NetLookupObjectSidsByNames(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszHostname,
     IN DWORD dwNamesCount,
     IN PSTR* ppszNames,
@@ -448,7 +463,9 @@ AD_NetLookupObjectSidsByNames(
                   &pwcHost);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = AD_SetSystemAccess(&pOldToken);
+    dwError = AD_SetSystemAccess(
+                  pState,
+                  &pOldToken);
     BAIL_ON_LSA_ERROR(dwError);
     bChangedToken = TRUE;
 
@@ -711,6 +728,7 @@ error:
 
 DWORD
 AD_NetLookupObjectNameBySid(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszHostname,
     IN PCSTR pszObjectSid,
     OUT PSTR* ppszNT4Name,
@@ -724,6 +742,7 @@ AD_NetLookupObjectNameBySid(
     BOOLEAN bIsNetworkError = FALSE;
 
     dwError = AD_NetLookupObjectNamesBySids(
+                 pState,
                  pszHostname,
                  1,
                  (PSTR*)&pszObjectSid,
@@ -761,7 +780,7 @@ error:
     LW_SAFE_FREE_STRING(pszNT4Name);
     *pObjectType = LSA_OBJECT_TYPE_UNDEFINED;
 
-    LSA_LOG_ERROR("Failed to find user, group, or domain by sid (sid = '%s', searched host = '%s') -> error = %u, symbol = %s",
+    LSA_LOG_INFO("Failed to find user, group, or domain by sid (sid = '%s', searched host = '%s') -> error = %u, symbol = %s",
             LSA_SAFE_LOG_STRING(pszObjectSid),
             LSA_SAFE_LOG_STRING(pszHostname),
             dwError,
@@ -774,6 +793,7 @@ error:
 
 DWORD
 AD_NetLookupObjectNamesBySids(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszHostname,
     IN DWORD dwSidsCount,
     IN PSTR* ppszObjectSids,
@@ -810,7 +830,9 @@ AD_NetLookupObjectNamesBySids(
                   &pwcHost);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = AD_SetSystemAccess(&pOldToken);
+    dwError = AD_SetSystemAccess(
+                  pState,
+                  &pOldToken);
     BAIL_ON_LSA_ERROR(dwError);
     bChangedToken = TRUE;
 
@@ -1115,6 +1137,7 @@ error:
 
 DWORD
 AD_DsEnumerateDomainTrusts(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszDomainControllerName,
     IN DWORD dwFlags,
     OUT NetrDomainTrust** ppTrusts,
@@ -1137,7 +1160,9 @@ AD_DsEnumerateDomainTrusts(
     dwError = LwMbsToWc16s(pszDomainControllerName, &pwcDomainControllerName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = AD_SetSystemAccess(&pOldToken);
+    dwError = AD_SetSystemAccess(
+                  pState,
+                  &pOldToken);
     BAIL_ON_LSA_ERROR(dwError);
     bChangedToken = TRUE;
 
@@ -1147,8 +1172,7 @@ AD_DsEnumerateDomainTrusts(
 
     status = NetrInitBindingDefault(&netr_b,
                                     pwcDomainControllerName,
-                                    pCreds,
-                                    FALSE);
+                                    pCreds);
     if (status != 0)
     {
         LSA_LOG_DEBUG("Failed to bind to %s (error %u)",
@@ -1175,15 +1199,8 @@ AD_DsEnumerateDomainTrusts(
         }
         else
         {
-            switch (winError)
-            {
-            case ERROR_ACCESS_DENIED:
-                dwError = winError;
-                break;
-            default:
-                dwError = LW_ERROR_ENUM_DOMAIN_TRUSTS_FAILED;
-            }
-
+            dwError = winError;
+     
             if (AD_WinErrorIsConnectionError(winError))
             {
                 bIsNetworkError = TRUE;
@@ -1244,6 +1261,7 @@ AD_FreeDomainTrusts(
 
 DWORD
 AD_DsGetDcName(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszServerName,
     IN PCSTR pszDomainName,
     IN BOOLEAN bReturnDnsName,
@@ -1270,7 +1288,9 @@ AD_DsGetDcName(
     dwError = LwMbsToWc16s(pszServerName, &pwcServerName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = AD_SetSystemAccess(&pOldToken);
+    dwError = AD_SetSystemAccess(
+                  pState,
+                  &pOldToken);
     BAIL_ON_LSA_ERROR(dwError);
     bChangedToken = TRUE;
 
@@ -1280,8 +1300,7 @@ AD_DsGetDcName(
 
     status = NetrInitBindingDefault(&netr_b,
                                     pwcServerName,
-                                    pCreds,
-                                    FALSE);
+                                    pCreds);
     if (status != 0)
     {
         LSA_LOG_DEBUG("Failed to bind to %s (error %u)",
@@ -1443,13 +1462,14 @@ WinTimeToInt64(
 
 static DWORD
 LsaCopyNetrUserInfo3(
+    IN PLSA_SCHANNEL_STATE pSchannelState,
     OUT PLSA_AUTH_USER_INFO pUserInfo,
     IN NetrValidationInfo *pNetrUserInfo3
     )
 {
     DWORD dwError = LW_ERROR_INTERNAL;
     NetrSamBaseInfo *pBase = NULL;
-    NTSTATUS ntError = STATUS_UNSUCCESSFUL;
+    NTSTATUS ntError = STATUS_UNSUCCESSFUL;    
     RC4_KEY RC4Key;
 
     BAIL_ON_INVALID_POINTER(pUserInfo);
@@ -1478,10 +1498,10 @@ LsaCopyNetrUserInfo3(
 
     /* We have to decrypt the user session key before we can use it */
 
-    RC4_set_key(&RC4Key, 16, gpSchannelCreds->session_key);
-    RC4(&RC4Key,
+    RC4_set_key(&RC4Key, 16, pSchannelState->pSchannelCreds->session_key);
+    RC4(&RC4Key, 
         pUserInfo->pSessionKey->dwLen,
-        pUserInfo->pSessionKey->pData,
+        pUserInfo->pSessionKey->pData, 
         pUserInfo->pSessionKey->pData);
 
     dwError = LsaDataBlobStore(&pUserInfo->pLmSessionKey,
@@ -1489,10 +1509,10 @@ LsaCopyNetrUserInfo3(
                                pBase->lmkey.key);
     BAIL_ON_LSA_ERROR(dwError);
 
-    RC4_set_key(&RC4Key, 16, gpSchannelCreds->session_key);
-    RC4(&RC4Key,
+    RC4_set_key(&RC4Key, 16, pSchannelState->pSchannelCreds->session_key);
+    RC4(&RC4Key, 
         pUserInfo->pLmSessionKey->dwLen,
-        pUserInfo->pLmSessionKey->pData,
+        pUserInfo->pLmSessionKey->pData, 
         pUserInfo->pLmSessionKey->pData);
 
     pUserInfo->pszUserPrincipalName = NULL;
@@ -1570,25 +1590,54 @@ LsaCopyNetrUserInfo3(
 
     }
 
-    pUserInfo->dwNumSids = pNetrUserInfo3->sam3->sidcount;
-    if (pUserInfo->dwNumSids != 0)
+    if (pNetrUserInfo3->sam3->sidcount != 0)
     {
         int i = 0;
 
-        dwError = LwAllocateMemory(sizeof(LSA_SID_ATTRIB)*(pUserInfo->dwNumSids),
-                                    (PVOID*)&pUserInfo->pSidAttribList);
+        pUserInfo->dwNumSids = 0;        
+
+        dwError = LwAllocateMemory(
+                      sizeof(LSA_SID_ATTRIB)*(pNetrUserInfo3->sam3->sidcount),
+                      (PVOID*)&pUserInfo->pSidAttribList);
         BAIL_ON_LSA_ERROR(dwError);
 
-        for (i=0; i<pUserInfo->dwNumSids; i++)
+        for (i=0; i<pNetrUserInfo3->sam3->sidcount; i++)
         {
-            PLSA_SID_ATTRIB pSidAttrib = &(pUserInfo->pSidAttribList[i]);
+            PLSA_SID_ATTRIB pSidAttrib = &(pUserInfo->pSidAttribList[pUserInfo->dwNumSids]);
+
+            dwError = ERROR_SUCCESS;
 
             pSidAttrib->dwAttrib = pNetrUserInfo3->sam3->sids[i].attribute;
 
-            ntError = RtlAllocateCStringFromSid(&pSidAttrib->pszSid,
-                                                pNetrUserInfo3->sam3->sids[i].sid);
+            ntError = RtlAllocateCStringFromSid(
+                          &pSidAttrib->pszSid,
+                          pNetrUserInfo3->sam3->sids[i].sid);
             dwError = LwNtStatusToWin32Error(ntError);
-            BAIL_ON_LSA_ERROR(dwError);
+            if (dwError != ERROR_SUCCESS)
+            {
+                if (pNetrUserInfo3->sam3->sids[i].sid)
+                {
+                    LSA_LOG_DEBUG(
+                        "Ignoring invalid SID (User = %s\\%s, Attribute = 0x%x, Revision = %d, SubAuthorityCount = %d).\n",
+                        pUserInfo->pszDomain ? pUserInfo->pszDomain : "NULL",
+                        pUserInfo->pszAccount ? pUserInfo->pszAccount : "NULL",
+                        pNetrUserInfo3->sam3->sids[i].attribute,
+                        pNetrUserInfo3->sam3->sids[i].sid->Revision,
+                        pNetrUserInfo3->sam3->sids[i].sid->SubAuthorityCount);
+                }
+                else
+                {
+                    LSA_LOG_DEBUG(
+                        "Ignoring NULL SID (User = %s\\%s, Attribute = 0x%x).\n",
+                        pUserInfo->pszDomain ? pUserInfo->pszDomain : "NULL",
+                        pUserInfo->pszAccount ? pUserInfo->pszAccount : "NULL",
+                        pNetrUserInfo3->sam3->sids[i].attribute);
+                }
+                continue;
+            }
+
+            // Successfully added another SID
+            pUserInfo->dwNumSids++;
         }
     }
 
@@ -1605,6 +1654,7 @@ error:
 
 DWORD
 AD_NetlogonAuthenticationUserEx(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PSTR pszDomainController,
     IN PLSA_AUTH_USER_PARAMS pUserParams,
     OUT PLSA_AUTH_USER_INFO *ppUserInfo,
@@ -1612,17 +1662,17 @@ AD_NetlogonAuthenticationUserEx(
     )
 {
     DWORD dwError = LW_ERROR_INTERNAL;
+    PLSA_SCHANNEL_STATE pSchannelState = pState->hSchannelState;
     PWSTR pwszDomainController = NULL;
     PWSTR pwszServerName = NULL;
     PWSTR pwszShortDomain = NULL;
     PWSTR pwszPrimaryShortDomain = NULL;
+    PWSTR pwszPrimaryFqdn = NULL;
     PWSTR pwszUsername = NULL;
     PWSTR pwszComputer = NULL;
-    PSTR pszHostname = NULL;
-    HANDLE hPwdDb = (HANDLE)NULL;
     NTSTATUS status = 0;
     NETR_BINDING netr_b = NULL;
-    PLWPS_PASSWORD_INFO pMachAcctInfo = NULL;
+    PLSA_MACHINE_PASSWORD_INFO_W pMachinePasswordInfo = NULL;
     BOOLEAN bIsNetworkError = FALSE;
     NTSTATUS nt_status = STATUS_UNHANDLED_EXCEPTION;
     NetrValidationInfo  *pValidationInfo = NULL;
@@ -1639,21 +1689,19 @@ AD_NetlogonAuthenticationUserEx(
     BOOLEAN bChangedToken = FALSE;
     BOOLEAN bResetSchannel = FALSE;
     PLSA_AUTH_USER_INFO pUserInfo = NULL;
+    BOOLEAN passwordLocked = FALSE;
 
-    pthread_mutex_lock(&gSchannelLock);
+    pthread_mutex_lock(pSchannelState->pSchannelLock);
 
     /* Grab the machine password and account info */
 
-    dwError = LwpsOpenPasswordStore(LWPS_PASSWORD_STORE_DEFAULT,
-                                    &hPwdDb);
-    BAIL_ON_LSA_ERROR(dwError);
+    AD_LOCK_MACHINE_PASSWORD(
+                    pState->hMachinePwdState,
+                    passwordLocked);
 
-    dwError = LsaDnsGetHostInfo(&pszHostname);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LwpsGetPasswordByHostName(hPwdDb,
-                                        pszHostname,
-                                        &pMachAcctInfo);
+    dwError = LsaPcacheGetMachinePasswordInfoW(
+                  pState->pPcache,
+                  &pMachinePasswordInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
     /* Gather other Schannel params */
@@ -1669,10 +1717,13 @@ AD_NetlogonAuthenticationUserEx(
     dwError = LwMbsToWc16s(pszServerName, &pwszServerName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(pUserParams->pszDomain, &pwszShortDomain);
-    BAIL_ON_LSA_ERROR(dwError);
+    if (pUserParams->pszDomain)
+    {
+        dwError = LwMbsToWc16s(pUserParams->pszDomain, &pwszShortDomain);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
-    pwszComputer = wc16sdup(pMachAcctInfo->pwszMachineAccount);
+    pwszComputer = wc16sdup(pMachinePasswordInfo->Account.SamAccountName);
     if (!pwszComputer)
     {
         dwError = LW_ERROR_OUT_OF_MEMORY;
@@ -1682,18 +1733,34 @@ AD_NetlogonAuthenticationUserEx(
     // Remove $ from account name
     pwszComputer[wc16slen(pwszComputer) - 1] = 0;
 
-    if (!ghSchannelBinding)
+    if (pSchannelState->pszSchannelServer && strcasecmp(pSchannelState->pszSchannelServer, pszDomainController))
+    {
+        LSA_LOG_VERBOSE("Resetting schannel due to switching DC from '%s' to '%s'",
+                        pSchannelState->pszSchannelServer, pszDomainController);
+        AD_ClearSchannelStateInLock(pSchannelState);
+    }
+
+    if (!pSchannelState->hSchannelBinding)
     {
         dwError = LwMbsToWc16s(pszDomainController, &pwszDomainController);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LwMbsToWc16s(gpADProviderData->szShortDomain,
+        dwError = LwMbsToWc16s(pState->pProviderData->szShortDomain,
                                 &pwszPrimaryShortDomain);
         BAIL_ON_LSA_ERROR(dwError);
 
+        dwError = LwMbsToWc16s(pState->pProviderData->szDomain,
+                               &pwszPrimaryFqdn);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        dwError = LwWc16sToLower(pwszPrimaryFqdn);
+        BAIL_ON_LSA_ERROR(dwError);
+
         /* Establish the initial bind to \NETLOGON */
 
-        dwError = AD_SetSystemAccess(&pOldToken);
+        dwError = AD_SetSystemAccess(
+                      pState,
+                      &pOldToken);
         BAIL_ON_LSA_ERROR(dwError);
         bChangedToken = TRUE;
 
@@ -1701,7 +1768,7 @@ AD_NetlogonAuthenticationUserEx(
         dwError = LwNtStatusToErrno(status);
         BAIL_ON_LSA_ERROR(dwError);
 
-        status = NetrInitBindingDefault(&netr_b, pwszDomainController, pCreds, FALSE);
+        status = NetrInitBindingDefault(&netr_b, pwszDomainController, pCreds);
         if (status != 0)
         {
             LSA_LOG_DEBUG("Failed to bind to %s (error %u)",
@@ -1714,14 +1781,15 @@ AD_NetlogonAuthenticationUserEx(
         /* Now setup the Schannel session */
 
         nt_status = NetrOpenSchannel(netr_b,
-                                     pMachAcctInfo->pwszMachineAccount,
+                                     pMachinePasswordInfo->Account.SamAccountName,
                                      pwszDomainController,
                                      pwszServerName,
                                      pwszPrimaryShortDomain,
+                                     pwszPrimaryFqdn,
                                      pwszComputer,
-                                     pMachAcctInfo->pwszMachinePassword,
-                                     &gSchannelCreds,
-                                     &ghSchannelBinding);
+                                     pMachinePasswordInfo->Password,
+                                     &pSchannelState->SchannelCreds,
+                                     &pSchannelState->hSchannelBinding);
 
         if (nt_status != STATUS_SUCCESS)
         {
@@ -1732,6 +1800,11 @@ AD_NetlogonAuthenticationUserEx(
                 bIsNetworkError = TRUE;
                 dwError = LW_ERROR_KRB5KDC_ERR_TGT_REVOKED;
             }
+            else if (nt_status == STATUS_NO_TRUST_SAM_ACCOUNT)
+            {
+                bIsNetworkError = TRUE;
+                dwError = ERROR_NO_TRUST_SAM_ACCOUNT;
+            }
             else
             {
                 bResetSchannel = TRUE;
@@ -1741,11 +1814,23 @@ AD_NetlogonAuthenticationUserEx(
                     bIsNetworkError = TRUE;
                 }
             }
+        }
+        BAIL_ON_LSA_ERROR(dwError);
+
+        if (!pSchannelState->pszSchannelServer)
+        {
+            dwError = LwAllocateString(
+                          pszDomainController,
+                          &pSchannelState->pszSchannelServer);
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        gpSchannelCreds = &gSchannelCreds;
+        pSchannelState->pSchannelCreds = &pSchannelState->SchannelCreds;
     }
+    AD_UNLOCK_MACHINE_PASSWORD(
+                    pState->hMachinePwdState,
+                    passwordLocked);
+
 
     /* Time to do the authentication */
 
@@ -1767,7 +1852,7 @@ AD_NetlogonAuthenticationUserEx(
         NTRespLen = LsaDataBlobLength(pUserParams->pass.chap.pNT_resp);
     }
 
-    nt_status = NetrSamLogonNetworkEx(ghSchannelBinding,
+    nt_status = NetrSamLogonNetworkEx(pSchannelState->hSchannelBinding,
                                       pwszServerName,
                                       pwszShortDomain,
                                       pwszComputer,
@@ -1824,11 +1909,14 @@ AD_NetlogonAuthenticationUserEx(
     case STATUS_PASSWORD_EXPIRED:
         dwError = LW_ERROR_PASSWORD_EXPIRED;
         break;
+    case STATUS_PASSWORD_MUST_CHANGE:
+        dwError = ERROR_PASSWORD_MUST_CHANGE;
+        break;
     case STATUS_WRONG_PASSWORD:
         dwError = LW_ERROR_INVALID_PASSWORD;
         break;
     case STATUS_INVALID_ACCOUNT_NAME:
-        dwError = LW_ERROR_INVALID_ACCOUNT;
+        dwError = ERROR_INVALID_ACCOUNT_NAME;
         break;
     case STATUS_ACCOUNT_RESTRICTION:
     case STATUS_LOGON_FAILURE:
@@ -1846,7 +1934,7 @@ AD_NetlogonAuthenticationUserEx(
                       pUserParams->pszDomain, pUserParams->pszAccountName);
         break;
     }
-
+  
     BAIL_ON_LSA_ERROR(dwError);
 
     /* Translate the returned NetrValidationInfo to the
@@ -1855,19 +1943,15 @@ AD_NetlogonAuthenticationUserEx(
     dwError = LwAllocateMemory(sizeof(LSA_AUTH_USER_INFO), (PVOID*)&pUserInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaCopyNetrUserInfo3(pUserInfo, pValidationInfo);
+    dwError = LsaCopyNetrUserInfo3(pSchannelState, pUserInfo, pValidationInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
-    if (hPwdDb)
-    {
-        if (pMachAcctInfo) {
-            LwpsFreePasswordInfo(hPwdDb, pMachAcctInfo);
-        }
 
-        LwpsClosePasswordStore(hPwdDb);
-        hPwdDb = (HANDLE)NULL;
-    }
+    AD_UNLOCK_MACHINE_PASSWORD(
+                    pState->hMachinePwdState,
+                    passwordLocked);
+    LsaPcacheReleaseMachinePasswordInfoW(pMachinePasswordInfo);
 
     if (netr_b)
     {
@@ -1890,9 +1974,8 @@ cleanup:
 
     if (pValidationInfo) {
         NetrFreeMemory((void*)pValidationInfo);
-    }
+    }    
 
-    LW_SAFE_FREE_MEMORY(pszHostname);
     LW_SAFE_FREE_MEMORY(pszServerName);
 
     LW_SAFE_FREE_MEMORY(pwszUsername);
@@ -1900,9 +1983,10 @@ cleanup:
     LW_SAFE_FREE_MEMORY(pwszServerName);
     LW_SAFE_FREE_MEMORY(pwszShortDomain);
     LW_SAFE_FREE_MEMORY(pwszPrimaryShortDomain);
+    LW_SAFE_FREE_MEMORY(pwszPrimaryFqdn);
     LW_SAFE_FREE_MEMORY(pwszComputer);
 
-    pthread_mutex_unlock(&gSchannelLock);
+    pthread_mutex_unlock(pSchannelState->pSchannelLock);
 
     *ppUserInfo = pUserInfo;
 
@@ -1919,7 +2003,7 @@ error:
 
     if (bResetSchannel)
     {
-        AD_ClearSchannelStateInLock();
+        AD_ClearSchannelStateInLock(pSchannelState);
     }
 
     goto cleanup;
@@ -1928,33 +2012,24 @@ error:
 static
 VOID
 AD_ClearSchannelStateInLock(
-    VOID
+    IN PLSA_SCHANNEL_STATE pSchannelState
     )
 {
-    if (ghSchannelBinding)
+    if (pSchannelState->hSchannelBinding)
     {
-        NetrCloseSchannel(ghSchannelBinding);
+        NetrCloseSchannel(pSchannelState->hSchannelBinding);
 
-        ghSchannelBinding = NULL;
+        pSchannelState->hSchannelBinding = NULL;
 
-        memset(&gSchannelCreds, 0, sizeof(gSchannelCreds));
-        gpSchannelCreds = NULL;
+        memset(&pSchannelState->SchannelCreds,
+               0,
+               sizeof(pSchannelState->SchannelCreds));
+        pSchannelState->pSchannelCreds = NULL;
+
+        LW_SAFE_FREE_MEMORY(pSchannelState->pszSchannelServer);
     }
 }
 
-static
-VOID
-AD_ClearSchannelState(
-    VOID
-    )
-{
-    pthread_mutex_lock(&gSchannelLock);
-
-    AD_ClearSchannelStateInLock();
-
-    pthread_mutex_unlock(&gSchannelLock);
-}
-
 static
 BOOLEAN
 AD_NtStatusIsTgtRevokedError(
@@ -1983,12 +2058,16 @@ AD_NtStatusIsConnectionError(
 
     switch (status)
     {
+    case STATUS_CONNECTION_ABORTED:
+    case STATUS_CONNECTION_DISCONNECTED:
+    case STATUS_CONNECTION_REFUSED:
     case STATUS_CONNECTION_RESET:
     case STATUS_IO_TIMEOUT:
     case STATUS_ACCESS_DENIED:
     case STATUS_TIME_DIFFERENCE_AT_DC:
     case STATUS_INVALID_CONNECTION:
     case STATUS_PIPE_DISCONNECTED:
+    case STATUS_BAD_NETWORK_NAME:
         return TRUE;
     default:
         return FALSE;
@@ -2019,12 +2098,15 @@ AD_WinErrorIsConnectionError(
 {
     switch (winError)
     {
+        case ERROR_CONNECTION_ABORTED:
+        case ERROR_CONNECTION_REFUSED:
         case ERROR_NETNAME_DELETED:
         case ERROR_SEM_TIMEOUT:
         case ERROR_ACCESS_DENIED:
         case ERROR_TIME_SKEW:
         case ERROR_UNEXP_NET_ERR:
         case ERROR_PIPE_NOT_CONNECTED:
+        case ERROR_BAD_NET_NAME:
             return TRUE;
 
         default:
