@@ -61,7 +61,7 @@ typedef struct __AD_CELL_COOKIE_DATA
     // This hash table is used to ensure that the same user/group is not
     // returned twice through linked cells. It is only allocated if linked
     // cells are used in the computer's cell.
-    PLSA_HASH_TABLE pEnumeratedSids;
+    PLW_HASH_TABLE pEnumeratedSids;
 } AD_CELL_COOKIE_DATA, *PAD_CELL_COOKIE_DATA;
 
 static
@@ -76,7 +76,7 @@ LsaAdBatchFreeCellCookie(
     {
         LwFreeCookieContents(&pData->LdapCookie);
         LsaDmLdapClose(pData->pLdapConn);
-        LsaHashSafeFree(&pData->pEnumeratedSids);
+        LwHashSafeFree(&pData->pEnumeratedSids);
 
         LW_SAFE_FREE_MEMORY(pData);
     }
@@ -258,6 +258,7 @@ LsaAdBatchEnumGetQuery(
 static
 DWORD
 LsaAdBatchEnumProcessRealMessages(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszDnsDomainName,
     IN PCSTR pszNetbiosDomainName,
     IN DWORD dwDirectoryMode,
@@ -271,6 +272,7 @@ LsaAdBatchEnumProcessRealMessages(
     )
 {
     DWORD dwError = 0;
+    PAD_PROVIDER_DATA pProviderData = pState->pProviderData;
     LDAPMessage* pCurrentMessage = NULL;
     LDAP* pLd = LwLdapGetSession(hDirectory);
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
@@ -286,6 +288,7 @@ LsaAdBatchEnumProcessRealMessages(
     while (pCurrentMessage)
     {
         dwError = LsaAdBatchGatherRealObjectInternal(
+                        pProviderData,
                         &batchItem,
                         &dwDirectoryMode,
                         &adMode,
@@ -302,13 +305,16 @@ LsaAdBatchEnumProcessRealMessages(
         if (!AdIsSpecialDomainSidPrefix(batchItem.pszSid))
         {
             dwError = LsaAdBatchMarshal(
+                            pState,
                             pszDnsDomainName,
                             pszNetbiosDomainName,
                             &batchItem,
                             &ppObjects[dwObjectsCount]);
             BAIL_ON_LSA_ERROR(dwError);
 
-            if (ppObjects[dwObjectsCount])
+            // If the user was disabled, no error and null error would have been
+            // returned
+            if (ppObjects[dwObjectsCount] != NULL)
             {
                 dwObjectsCount++;
             }
@@ -338,6 +344,7 @@ error:
 static
 DWORD
 LsaAdBatchEnumProcessPseudoMessages(
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN PCSTR pszDnsDomainName,
     IN PCSTR pszNetbiosDomainName,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
@@ -395,7 +402,7 @@ LsaAdBatchEnumProcessPseudoMessages(
             dwError = LwAllocateString(pszSidFromKeywords,
                                         &ppszSids[dwSidsCount]);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             dwSidsCount++;
         }
         else
@@ -411,6 +418,7 @@ LsaAdBatchEnumProcessPseudoMessages(
     }
 
     dwError = LsaAdBatchFindObjects(
+                    pContext,
                     LSA_AD_BATCH_QUERY_TYPE_BY_SID,
                     dwSidsCount,
                     ppszSids,
@@ -419,6 +427,10 @@ LsaAdBatchEnumProcessPseudoMessages(
                     &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
 
+    // If orphaned objects are encountered, LsaAdBatchFindObjects does not
+    // return a NULL in the middle of the ppObjects, but instead sets
+    // dwObjectsCount less than dwSidsCount.
+
     // Ideally, do extra sanity check on object type.
     // In the future, find should take the object type
     // and restrict the search based on that.
@@ -444,6 +456,7 @@ error:
 static
 DWORD
 LsaAdBatchEnumProcessMessages(
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN PCSTR pszDnsDomainName,
     IN PCSTR pszNetbiosDomainName,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
@@ -457,6 +470,7 @@ LsaAdBatchEnumProcessMessages(
     )
 {
     DWORD dwError = 0;
+    PLSA_AD_PROVIDER_STATE pState = pContext->pState;
     DWORD dwCount = 0;
     LDAP* pLd = LwLdapGetSession(hDirectory);
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
@@ -483,6 +497,7 @@ LsaAdBatchEnumProcessMessages(
         (UNPROVISIONED_MODE == dwDirectoryMode))
     {
         dwError = LsaAdBatchEnumProcessRealMessages(
+                        pState,
                         pszDnsDomainName,
                         pszNetbiosDomainName,
                         dwDirectoryMode,
@@ -498,6 +513,7 @@ LsaAdBatchEnumProcessMessages(
     else
     {
         dwError = LsaAdBatchEnumProcessPseudoMessages(
+                        pContext,
                         pszDnsDomainName,
                         pszNetbiosDomainName,
                         ObjectType,
@@ -530,6 +546,7 @@ error:
 static
 DWORD
 LsaAdBatchEnumObjectsInCell(
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN PLSA_DM_LDAP_CONNECTION pConn,
     IN OUT PLW_SEARCH_COOKIE pCookie,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
@@ -543,6 +560,7 @@ LsaAdBatchEnumObjectsInCell(
     )
 {
     DWORD dwError = 0;
+    PLSA_AD_PROVIDER_STATE pState = pContext->pState;
     DWORD dwRemainingObjectsWanted = dwMaxObjectsCount;
     PSTR szBacklinkAttributeList[] =
     {
@@ -619,7 +637,8 @@ LsaAdBatchEnumObjectsInCell(
     pszQuery = LsaAdBatchEnumGetQuery(bIsByRealObject, bIsSchemaMode, ObjectType);
     LSA_ASSERT(pszQuery);
 
-    dwError = LsaDmWrapGetDomainName(pszDnsDomainName, NULL,
+    dwError = LsaDmWrapGetDomainName(pState->hDmState,
+                                     pszDnsDomainName, NULL,
                                      &pszNetbiosDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -638,6 +657,7 @@ LsaAdBatchEnumObjectsInCell(
         BAIL_ON_LSA_ERROR(dwError);
 
         dwError = LsaAdBatchEnumProcessMessages(
+                        pContext,
                         pszDnsDomainName,
                         pszNetbiosDomainName,
                         ObjectType,
@@ -696,6 +716,7 @@ error:
 static
 DWORD
 LsaAdBatchEnumObjectsInLinkedCells(
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN OUT PLW_SEARCH_COOKIE pCookie,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
     IN DWORD dwMaxObjectsCount,
@@ -704,13 +725,14 @@ LsaAdBatchEnumObjectsInLinkedCells(
     )
 {
     DWORD dwError = 0;
+    PLSA_AD_PROVIDER_STATE pState = pContext->pState;
     PLSA_SECURITY_OBJECT* ppObjectsInOneCell = NULL;
     DWORD dwObjectsCountInOneCell = 0;
     PLSA_SECURITY_OBJECT* ppObjects = *pppObjects;
     DWORD dwObjectsCount = *pdwObjectsCount;
     PAD_CELL_COOKIE_DATA pCookieData = NULL;
-    ADConfigurationMode adMode = gpADProviderData->adConfigurationMode;
-    BOOLEAN bIsDefaultCell = gpADProviderData->dwDirectoryMode == DEFAULT_MODE;
+    ADConfigurationMode adMode = pState->pProviderData->adConfigurationMode;
+    BOOLEAN bIsDefaultCell = pState->pProviderData->dwDirectoryMode == DEFAULT_MODE;
 
     LSA_ASSERT(pCookie->pfnFree == LsaAdBatchFreeCellCookie);
     pCookieData = (PAD_CELL_COOKIE_DATA)pCookie->pvData;
@@ -723,6 +745,7 @@ LsaAdBatchEnumObjectsInLinkedCells(
         if (pCookieData->pLdapConn == NULL)
         {
             dwError = LsaDmLdapOpenDc(
+                            pContext,
                             pCellInfo->pszDomain,
                             &pCookieData->pLdapConn);
             BAIL_ON_LSA_ERROR(dwError);
@@ -730,16 +753,19 @@ LsaAdBatchEnumObjectsInLinkedCells(
 
         // determine schema/non-schema mode in the current cell
         dwError = LsaAdBatchQueryCellConfigurationMode(
-                       gpADProviderData->szDomain,
+                       pContext,
+                       pState->pProviderData->szDomain,
                        pCellInfo->pszCellDN,
                        &adMode);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LsaAdBatchIsDefaultCell(pCellInfo->pszCellDN,
+        dwError = LsaAdBatchIsDefaultCell(pState->pProviderData,
+                                          pCellInfo->pszCellDN,
                                           &bIsDefaultCell);
         BAIL_ON_LSA_ERROR(dwError);
 
         dwError = LsaAdBatchEnumObjectsInCell(
+                        pContext,
                         pCookieData->pLdapConn,
                         &pCookieData->LdapCookie,
                         ObjectType,
@@ -789,12 +815,89 @@ error:
    goto cleanup;
 }
 
+DWORD
+LsaRemoveAlreadyEnumerated(
+    IN OUT PLW_HASH_TABLE pEnumeratedSids,
+    IN OUT PDWORD pObjectsCount,
+    IN OUT PLSA_SECURITY_OBJECT* ppObjects
+    )
+{
+    DWORD dwError = 0;
+    DWORD input = 0;
+    DWORD objectsCount = *pObjectsCount;
+    PSTR pszCopiedSid = NULL;
+    size_t sObjectsCount = 0;
+
+    // Remove any sids that have already been enumerated
+    if (pEnumeratedSids != NULL)
+    {
+        for (input = 0; input < objectsCount; input++)
+        {
+            dwError = LwHashGetValue(
+                        pEnumeratedSids,
+                        ppObjects[input]->pszObjectSid,
+                        NULL);
+            if (dwError == LW_ERROR_SUCCESS)
+            {
+                // The object is already in the hash
+                ADCacheSafeFreeObject(&ppObjects[input]);
+            }
+            else if (dwError == ERROR_NOT_FOUND)
+            {
+                // This is a new entry; let's track it in the hash
+
+                if (pEnumeratedSids->sCount * 2 >
+                    pEnumeratedSids->sTableSize)
+                {
+                    // Enlarge the hash table to avoid collisions
+                    dwError = LwHashResize(
+                                pEnumeratedSids,
+                                pEnumeratedSids->sCount * 4);
+                    BAIL_ON_LSA_ERROR(dwError);
+                }
+
+                dwError = LwAllocateString(
+                                ppObjects[input]->pszObjectSid,
+                                &pszCopiedSid);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                dwError = LwHashSetValue(
+                            pEnumeratedSids,
+                            pszCopiedSid,
+                            NULL);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                // This is now owned by the hash table
+                pszCopiedSid = NULL;
+            }
+            else
+            {
+                BAIL_ON_LSA_ERROR(dwError);
+            }
+        }
+    }
+
+    sObjectsCount = objectsCount;
+    AD_FilterNullEntries(
+        ppObjects,
+        &sObjectsCount);
+    *pObjectsCount = sObjectsCount;
+
+cleanup:
+    LW_SAFE_FREE_STRING(pszCopiedSid);
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
 // If this function returns with an error, the position stored in pCookie will
 // be advanced, even though no results are returned. pCookie will still need
 // to be freed outside of this function, even if this function returns with an
 // error.
 DWORD
 LsaAdBatchEnumObjects(
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN OUT PLW_SEARCH_COOKIE pCookie,
     IN LSA_OBJECT_TYPE AccountType,
     IN DWORD dwMaxObjectsCount,
@@ -803,15 +906,13 @@ LsaAdBatchEnumObjects(
     )
 {
     DWORD dwError = 0;
+    PLSA_AD_PROVIDER_STATE pState = pContext->pState;
     LSA_AD_BATCH_OBJECT_TYPE objectType = 0;
     DWORD dwObjectsCount = 0;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     PAD_CELL_COOKIE_DATA pCookieData = NULL;
     DWORD dwTotalObjectsCount = 0;
     PLSA_SECURITY_OBJECT* ppTotalObjects = NULL;
-    DWORD dwInput = 0;
-    DWORD dwOutput = 0;
-    PSTR pszCopiedSid = NULL;
 
     objectType = LsaAdBatchGetObjectTypeFromAccountType(AccountType);
     if (!LsaAdBatchIsUserOrGroupObjectType(objectType))
@@ -837,15 +938,15 @@ LsaAdBatchEnumObjects(
         BAIL_ON_LSA_ERROR(dwError);
 
         pCookieData = (PAD_CELL_COOKIE_DATA)pCookie->pvData;
-        if (gpADProviderData->pCellList != NULL)
+        if (pState->pProviderData->pCellList != NULL)
         {
             // There are linked cells, so we need to keep track of which
             // sids have been enumerated.
-            dwError = LsaHashCreate(
+            dwError = LwHashCreate(
                             10 * 1024,
-                            LsaHashCaselessStringCompare,
-                            LsaHashCaselessStringHash,
-                            LsaHashFreeStringKey,
+                            LwHashCaselessStringCompare,
+                            LwHashCaselessStringHash,
+                            LwHashFreeStringKey,
                             NULL,
                             &pCookieData->pEnumeratedSids);
             BAIL_ON_LSA_ERROR(dwError);
@@ -867,25 +968,34 @@ LsaAdBatchEnumObjects(
             if (pCookieData->pLdapConn == NULL)
             {
                 dwError = LsaDmLdapOpenDc(
-                                gpADProviderData->szDomain,
+                                pContext,
+                                pState->pProviderData->szDomain,
                                 &pCookieData->pLdapConn);
                 BAIL_ON_LSA_ERROR(dwError);
             }
 
             // First get the objects from the primary cell
             dwError = LsaAdBatchEnumObjectsInCell(
+                            pContext,
                             pCookieData->pLdapConn,
                             &pCookieData->LdapCookie,
                             objectType,
-                            gpADProviderData->dwDirectoryMode,
-                            gpADProviderData->adConfigurationMode,
-                            gpADProviderData->szDomain,
-                            gpADProviderData->cell.szCellDN,
+                            pState->pProviderData->dwDirectoryMode,
+                            pState->pProviderData->adConfigurationMode,
+                            pState->pProviderData->szDomain,
+                            pState->pProviderData->cell.szCellDN,
                             dwMaxObjectsCount - dwTotalObjectsCount,
                             &dwObjectsCount,
                             &ppObjects);
             BAIL_ON_LSA_ERROR(dwError);
 
+            dwError = LsaRemoveAlreadyEnumerated(
+                         pCookieData->pEnumeratedSids,
+                         &dwObjectsCount,
+                         ppObjects);
+            BAIL_ON_LSA_ERROR(dwError);
+
+
             dwError = LsaAppendAndFreePtrs(
                             &dwTotalObjectsCount,
                             (PVOID**)&ppTotalObjects,
@@ -898,7 +1008,7 @@ LsaAdBatchEnumObjects(
                 LwFreeCookieContents(&pCookieData->LdapCookie);
                 LsaDmLdapClose(pCookieData->pLdapConn);
                 pCookieData->pLdapConn = NULL;
-                pCookieData->pCurrentCell = gpADProviderData->pCellList;
+                pCookieData->pCurrentCell = pState->pProviderData->pCellList;
                 if (pCookieData->pCurrentCell == NULL)
                 {
                     pCookie->bSearchFinished = TRUE;
@@ -908,6 +1018,7 @@ LsaAdBatchEnumObjects(
         else
         {
             dwError = LsaAdBatchEnumObjectsInLinkedCells(
+                         pContext,
                          pCookie,
                          objectType,
                          dwMaxObjectsCount - dwTotalObjectsCount,
@@ -915,6 +1026,12 @@ LsaAdBatchEnumObjects(
                          &ppObjects);
             BAIL_ON_LSA_ERROR(dwError);
 
+            dwError = LsaRemoveAlreadyEnumerated(
+                         pCookieData->pEnumeratedSids,
+                         &dwObjectsCount,
+                         ppObjects);
+            BAIL_ON_LSA_ERROR(dwError);
+
             dwError = LsaAppendAndFreePtrs(
                             &dwTotalObjectsCount,
                             (PVOID**)&ppTotalObjects,
@@ -922,66 +1039,11 @@ LsaAdBatchEnumObjects(
                             (PVOID**)&ppObjects);
             BAIL_ON_LSA_ERROR(dwError);
         }
-
-        // Remove any sids that have already been enumerated
-        if (pCookieData->pEnumeratedSids != NULL)
-        {
-            for (; dwInput < dwTotalObjectsCount; dwInput++)
-            {
-                dwError = LsaHashGetValue(
-                            pCookieData->pEnumeratedSids,
-                            ppTotalObjects[dwInput]->pszObjectSid,
-                            NULL);
-                if (dwError == LW_ERROR_SUCCESS)
-                {
-                    // The object is already in the hash
-                    ADCacheSafeFreeObject(&ppTotalObjects[dwInput]);
-                }
-                else if (dwError == ERROR_NOT_FOUND)
-                {
-                    // This is a new entry; let's track it in the hash
-
-                    if (pCookieData->pEnumeratedSids->sCount * 2 >
-                        pCookieData->pEnumeratedSids->sTableSize)
-                    {
-                        // Enlarge the hash table to avoid collisions
-                        dwError = LsaHashResize(
-                                    pCookieData->pEnumeratedSids,
-                                    pCookieData->pEnumeratedSids->sCount * 4);
-                        BAIL_ON_LSA_ERROR(dwError);
-                    }
-
-                    dwError = LwAllocateString(
-                                    ppTotalObjects[dwInput]->pszObjectSid,
-                                    &pszCopiedSid);
-                    BAIL_ON_LSA_ERROR(dwError);
-
-                    dwError = LsaHashSetValue(
-                                pCookieData->pEnumeratedSids,
-                                pszCopiedSid,
-                                NULL);
-                    BAIL_ON_LSA_ERROR(dwError);
-
-                    // This is now owned by the hash table
-                    pszCopiedSid = NULL;
-
-                    ppTotalObjects[dwOutput++] = ppTotalObjects[dwInput];
-                }
-                else
-                {
-                    BAIL_ON_LSA_ERROR(dwError);
-                }
-            }
-            // The array is now smaller since duplicates have been removed
-            dwTotalObjectsCount = dwOutput;
-            dwInput = dwOutput;
-        }
     }
 
 cleanup:
     *pdwObjectsCount = dwTotalObjectsCount;
     *pppObjects = ppTotalObjects;
-    LW_SAFE_FREE_STRING(pszCopiedSid);
 
     return dwError;
 
