@@ -157,8 +157,18 @@ LsaAdBatchGatherSchemaModeUser(
                     pMessage,
                     AD_LDAP_GID_TAG,
                     &dwValue);
+    if (LW_ERROR_INVALID_LDAP_ATTR_VALUE == dwError)
+    {
+        SetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_DISABLED);
+        dwError = LW_ERROR_SUCCESS;
+    }
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (IsSetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_DISABLED))
+    {
+        goto cleanup;
+    }
+
     if (!dwValue)
     {
         LSA_LOG_DEBUG("gid must be non-zero for SID '%s'", pItem->pszSid);
@@ -515,7 +525,7 @@ LsaAdBatchGatherUnprovisionedModeUser(
     DWORD dwError = 0;
 
     // Check for a machine account.  Last character in name will be "$"
-    // In the future, it would be safer to either add a new
+    // In the future, it would be safer to either add a new 
     // LSA_AD_BATCH_OBJECT_TYPE_COMPUTER value or an additional
     // flag to distinguish a user from a computer.  For current
     // AD releases however, this is a sufficient check
@@ -524,7 +534,7 @@ LsaAdBatchGatherUnprovisionedModeUser(
     {
         SetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_DISABLED);
     }
-
+    
 
 #if 0
     if (dwError != 0)
@@ -681,6 +691,7 @@ error:
 
 DWORD
 LsaAdBatchGatherRealObject(
+    IN PAD_PROVIDER_DATA pProviderData,
     IN OUT PLSA_AD_BATCH_ITEM pItem,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
     IN OUT OPTIONAL PSTR* ppszSid,
@@ -689,6 +700,7 @@ LsaAdBatchGatherRealObject(
     )
 {
     return LsaAdBatchGatherRealObjectInternal(
+                pProviderData,
                 pItem,
                 NULL,
                 NULL,
@@ -700,6 +712,7 @@ LsaAdBatchGatherRealObject(
 
 DWORD
 LsaAdBatchGatherRealObjectInternal(
+    IN PAD_PROVIDER_DATA pProviderData,
     IN OUT PLSA_AD_BATCH_ITEM pItem,
     IN OPTIONAL PDWORD pdwDirectoryMode,
     IN OPTIONAL ADConfigurationMode* pAdMode,
@@ -710,8 +723,8 @@ LsaAdBatchGatherRealObjectInternal(
     )
 {
     DWORD dwError = 0;
-    DWORD dwDirectoryMode = pdwDirectoryMode == NULL ? gpADProviderData->dwDirectoryMode : *pdwDirectoryMode;
-    ADConfigurationMode adMode = pAdMode == NULL ? gpADProviderData->adConfigurationMode : *pAdMode;
+    DWORD dwDirectoryMode = pdwDirectoryMode == NULL ? pProviderData->dwDirectoryMode : *pdwDirectoryMode;
+    ADConfigurationMode adMode = pAdMode == NULL ? pProviderData->adConfigurationMode : *pAdMode;
 
     SetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_HAVE_REAL);
 
@@ -862,6 +875,7 @@ static
 DWORD
 LsaAdBatchGatherPseudoSid(
     OUT PSTR* ppszSid,
+    IN PAD_PROVIDER_DATA pProviderData,
     IN OPTIONAL DWORD dwKeywordValuesCount,
     IN OPTIONAL PSTR* ppszKeywordValues,
     IN HANDLE hDirectory,
@@ -871,7 +885,7 @@ LsaAdBatchGatherPseudoSid(
     DWORD dwError = 0;
     PSTR pszSid = NULL;
 
-    if (LsaAdBatchIsDefaultSchemaMode())
+    if (LsaAdBatchIsDefaultSchemaMode(pProviderData))
     {
         dwError = ADLdap_GetObjectSid(hDirectory, pMessage, &pszSid);
         BAIL_ON_LSA_ERROR(dwError);
@@ -907,6 +921,7 @@ error:
 
 DWORD
 LsaAdBatchGatherPseudoObject(
+    IN PAD_PROVIDER_DATA pProviderData,
     IN OUT PLSA_AD_BATCH_ITEM pItem,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
     IN BOOLEAN bIsSchemaMode,
@@ -918,7 +933,7 @@ LsaAdBatchGatherPseudoObject(
 {
     DWORD dwError = 0;
 
-    LSA_ASSERT(LSA_IS_XOR(LsaAdBatchIsDefaultSchemaMode(), ppszKeywordValues));
+    LSA_ASSERT(LSA_IS_XOR(LsaAdBatchIsDefaultSchemaMode(pProviderData), ppszKeywordValues));
 
     SetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_HAVE_PSEUDO);
 
@@ -929,6 +944,7 @@ LsaAdBatchGatherPseudoObject(
     {
         dwError = LsaAdBatchGatherPseudoSid(
                         &pItem->pszSid,
+                        pProviderData,
                         dwKeywordValuesCount,
                         ppszKeywordValues,
                         hDirectory,
@@ -983,6 +999,7 @@ error:
 
 DWORD
 LsaAdBatchGatherPseudoObjectSidFromGc(
+    IN PAD_PROVIDER_DATA pProviderData,
     IN OUT PLSA_AD_BATCH_ITEM pItem,
     IN LSA_AD_BATCH_OBJECT_TYPE ObjectType,
     IN OPTIONAL DWORD dwKeywordValuesCount,
@@ -993,7 +1010,7 @@ LsaAdBatchGatherPseudoObjectSidFromGc(
 {
     DWORD dwError = 0;
 
-    LSA_ASSERT(LSA_IS_XOR(LsaAdBatchIsDefaultSchemaMode(), ppszKeywordValues));
+    LSA_ASSERT(LSA_IS_XOR(LsaAdBatchIsDefaultSchemaMode(pProviderData), ppszKeywordValues));
 
     LSA_ASSERT(!IsSetFlag(pItem->Flags, LSA_AD_BATCH_ITEM_FLAG_HAVE_PSEUDO));
     LSA_ASSERT(!pItem->pszSid);
@@ -1005,6 +1022,7 @@ LsaAdBatchGatherPseudoObjectSidFromGc(
     {
         dwError = LsaAdBatchGatherPseudoSid(
                         &pItem->pszSid,
+                        pProviderData,
                         dwKeywordValuesCount,
                         ppszKeywordValues,
                         hDirectory,
