@@ -49,6 +49,7 @@
  */
 
 #include "adprovider.h"
+#include "lsadm_p.h"
 
 typedef struct _LSA_DM_WRAP_ENUM_ONE_DOMAIN_INFO_CALLBACK_CONTEXT {
     IN PCSTR pszDnsDomainName;
@@ -80,11 +81,14 @@ LsaDmWrappFilterExtraForestDomainsCallback(
 // However, this is not necessarily correct.
 DWORD
 LsaDmWrapEnumExtraForestTrustDomains(
+    IN LSA_DM_STATE_HANDLE hDmState,
     OUT PSTR** pppszDomainNames,
     OUT PDWORD pdwCount
     )
 {
-    return LsaDmEnumDomainNames(LsaDmWrappFilterExtraForestDomainsCallback,
+    return LsaDmEnumDomainNames(
+                                hDmState,
+                                LsaDmWrappFilterExtraForestDomainsCallback,
                                 NULL,
                                 pppszDomainNames,
                                 pdwCount);
@@ -117,11 +121,13 @@ LsaDmWrappFilterExtraTwoWayForestDomainsCallback(
 // However, this is not necessarily correct.
 DWORD
 LsaDmWrapEnumExtraTwoWayForestTrustDomains(
+    IN LSA_DM_STATE_HANDLE hDmState,
     OUT PSTR** pppszDomainNames,
     OUT PDWORD pdwCount
     )
 {
-    return LsaDmEnumDomainNames(LsaDmWrappFilterExtraTwoWayForestDomainsCallback,
+    return LsaDmEnumDomainNames(hDmState,
+                                LsaDmWrappFilterExtraTwoWayForestDomainsCallback,
                                 NULL,
                                 pppszDomainNames,
                                 pdwCount);
@@ -150,11 +156,13 @@ LsaDmWrappFilterInMyForestDomainsCallback(
 
 DWORD
 LsaDmWrapEnumInMyForestTrustDomains(
+    IN LSA_DM_STATE_HANDLE hDmState,
     OUT PSTR** pppszDomainNames,
     OUT PDWORD pdwCount
     )
 {
-    return LsaDmEnumDomainNames(LsaDmWrappFilterInMyForestDomainsCallback,
+    return LsaDmEnumDomainNames(hDmState,
+                                LsaDmWrappFilterInMyForestDomainsCallback,
                                 NULL,
                                 pppszDomainNames,
                                 pdwCount);
@@ -170,7 +178,7 @@ LsaDmWrappFilterFindDomainCallback(
     BOOLEAN bWantThis = FALSE;
     PLSA_DM_WRAP_ENUM_ONE_DOMAIN_INFO_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_ENUM_ONE_DOMAIN_INFO_CALLBACK_CONTEXT) pContext;
 
-    // Find a domain info given a specified domain name
+    // Find a domain info given a specified domain name   
     if (!strcasecmp(pDomainInfo->pszDnsDomainName, pCtx->pszDnsDomainName))
     {
         bWantThis = TRUE;
@@ -181,6 +189,7 @@ LsaDmWrappFilterFindDomainCallback(
 
 DWORD
 LsaDmWrapGetDomainEnumInfo(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     OUT PLSA_DM_ENUM_DOMAIN_INFO* ppDomainInfo
     )
@@ -191,7 +200,9 @@ LsaDmWrapGetDomainEnumInfo(
 
     context.pszDnsDomainName = pszDnsDomainName;
 
-    dwError = LsaDmEnumDomainInfo(LsaDmWrappFilterFindDomainCallback,
+    dwError = LsaDmEnumDomainInfo(
+                               hDmState,
+                               LsaDmWrappFilterFindDomainCallback,
                                &context,
                                &ppDomainInfoList,
                                NULL);
@@ -219,12 +230,14 @@ error:
 
 DWORD
 LsaDmWrapGetDomainName(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDomainName,
     OUT OPTIONAL PSTR* ppszDnsDomainName,
     OUT OPTIONAL PSTR* ppszNetbiosDomainName
     )
 {
-    return LsaDmQueryDomainInfo(pszDomainName,
+    return LsaDmQueryDomainInfo(hDmState,
+                                pszDomainName,
                                 ppszDnsDomainName,
                                 ppszNetbiosDomainName,
                                 NULL,
@@ -244,6 +257,7 @@ LsaDmWrapGetDomainName(
 
 DWORD
 LsaDmWrapGetDomainNameAndSidByObjectSid(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszObjectSid,
     OUT OPTIONAL PSTR* ppszDnsDomainName,
     OUT OPTIONAL PSTR* ppszNetbiosDomainName,
@@ -261,6 +275,7 @@ LsaDmWrapGetDomainNameAndSidByObjectSid(
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LsaDmQueryDomainInfoByObjectSid(
+                    hDmState,
                     pObjectSid,
                     ppszDnsDomainName ? &pszDnsDomainName : NULL,
                     ppszNetbiosDomainName ? &pszNetbiosDomainName : NULL,
@@ -322,18 +337,21 @@ typedef struct _LSA_DM_WRAP_LDAP_OPEN_DIRECORY_CALLBACK_CONTEXT {
 } LSA_DM_WRAP_LDAP_OPEN_DIRECORY_CALLBACK_CONTEXT, *PLSA_DM_WRAP_LDAP_OPEN_DIRECORY_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN PCSTR pszName;
     OUT PSTR pszSid;
     OUT LSA_OBJECT_TYPE ObjectType;
 } LSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT, *PLSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN PCSTR pszSid;
     OUT PSTR pszNT4Name;
     OUT LSA_OBJECT_TYPE ObjectType;
 } LSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT, *PLSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN DWORD dwSidCounts;
     IN PSTR* ppszSids;
     OUT DWORD dwFoundNamesCount;
@@ -341,6 +359,7 @@ typedef struct _LSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT {
 } LSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT, *PLSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN DWORD dwNameCounts;
     IN PSTR* ppszNames;
     OUT DWORD dwFoundSidsCount;
@@ -348,12 +367,14 @@ typedef struct _LSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT {
 } LSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT, *PLSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_ENUM_DOMAIN_TRUSTS_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN DWORD dwFlags;
     OUT NetrDomainTrust* pTrusts;
     OUT DWORD dwCount;
 } LSA_DM_WRAP_ENUM_DOMAIN_TRUSTS_CALLBACK_CONTEXT, *PLSA_DM_WRAP_ENUM_DOMAIN_TRUSTS_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN PCSTR pszDomainName;
     IN BOOLEAN bReturnDnsName;
     OUT PSTR pszDomainDnsOrFlatName;
@@ -361,6 +382,7 @@ typedef struct _LSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT {
 } LSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT, *PLSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT;
 
 typedef struct _LSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT {
+    IN PLSA_AD_PROVIDER_STATE pState;
     IN PLSA_AUTH_USER_PARAMS pUserParams;
     OUT PLSA_AUTH_USER_INFO  *ppUserInfo;
 } LSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT, *PLSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT;
@@ -397,11 +419,13 @@ LsaDmWrappLookupSidByNameCallback(
     DWORD dwError = 0;
     PLSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT) pContext;
 
-    dwError = AD_NetLookupObjectSidByName(pDcInfo->pszDomainControllerName,
-                                          pCtx->pszName,
-                                          &pCtx->pszSid,
-                                          &pCtx->ObjectType,
-                                          pbIsNetworkError);
+    dwError = AD_NetLookupObjectSidByName(
+                    pCtx->pState,
+                    pDcInfo->pszDomainControllerName,
+                    pCtx->pszName,
+                    &pCtx->pszSid,
+                    &pCtx->ObjectType,
+                    pbIsNetworkError);
     return dwError;
 }
 
@@ -417,6 +441,7 @@ LsaDmWrappLookupNameBySidCallback(
     PLSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT) pContext;
 
     return AD_NetLookupObjectNameBySid(
+                    pCtx->pState,
                     pDcInfo->pszDomainControllerName,
                     pCtx->pszSid,
                     &pCtx->pszNT4Name,
@@ -436,6 +461,7 @@ LsaDmWrappLookupNamesBySidsCallback(
     PLSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT) pContext;
 
     return AD_NetLookupObjectNamesBySids(
+                    pCtx->pState,
                     pDcInfo->pszDomainControllerName,
                     pCtx->dwSidCounts,
                     pCtx->ppszSids,
@@ -456,6 +482,7 @@ LsaDmWrappLookupSidsByNamesCallback(
     PLSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT) pContext;
 
     return AD_NetLookupObjectSidsByNames(
+                    pCtx->pState,
                     pDcInfo->pszDomainControllerName,
                     pCtx->dwNameCounts,
                     pCtx->ppszNames,
@@ -523,11 +550,13 @@ LsaDmWrappDsEnumerateDomainTrustsCallback(
                     &pszFqdnDC);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = AD_DsEnumerateDomainTrusts(pszFqdnDC,
-                                         pCtx->dwFlags,
-                                         &pCtx->pTrusts,
-                                         &pCtx->dwCount,
-                                         pbIsNetworkError);
+    dwError = AD_DsEnumerateDomainTrusts(
+                    pCtx->pState,
+                    pszFqdnDC,
+                    pCtx->dwFlags,
+                    &pCtx->pTrusts,
+                    &pCtx->dwCount,
+                    pbIsNetworkError);
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
@@ -550,12 +579,14 @@ LsaDmWrappDsGetDcNameCallback(
     DWORD dwError = 0;
     PLSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT) pContext;
 
-    dwError = AD_DsGetDcName(pDcInfo->pszDomainControllerName,
-                             pCtx->pszDomainName,
-                             pCtx->bReturnDnsName,
-                             &pCtx->pszDomainDnsOrFlatName,
-                             &pCtx->pszDomainForestDnsName,
-                             pbIsNetworkError);
+    dwError = AD_DsGetDcName(
+                    pCtx->pState,
+                    pDcInfo->pszDomainControllerName,
+                    pCtx->pszDomainName,
+                    pCtx->bReturnDnsName,
+                    &pCtx->pszDomainDnsOrFlatName,
+                    &pCtx->pszDomainForestDnsName,
+                    pbIsNetworkError);
     return dwError;
 }
 
@@ -565,12 +596,14 @@ LsaDmWrappDsGetDcNameCallback(
 
 DWORD
 LsaDmWrapLdapPingTcp(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName
     )
 {
     DWORD dwError = 0;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
                                       LsaDmWrappLdapPingTcpCallback,
@@ -580,6 +613,7 @@ LsaDmWrapLdapPingTcp(
 
 DWORD
 LsaDmWrapNetLookupObjectSidByName(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN PCSTR pszName,
     OUT PSTR* ppszSid,
@@ -589,9 +623,11 @@ LsaDmWrapNetLookupObjectSidByName(
     DWORD dwError = 0;
     LSA_DM_WRAP_LOOKUP_SID_BY_NAME_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.pszName = pszName;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
@@ -610,6 +646,7 @@ LsaDmWrapNetLookupObjectSidByName(
 
 DWORD
 LsaDmWrapNetLookupNameByObjectSid(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN  PCSTR pszDnsDomainName,
     IN  PCSTR pszSid,
     OUT PSTR* ppszName,
@@ -619,9 +656,11 @@ LsaDmWrapNetLookupNameByObjectSid(
     DWORD dwError = 0;
     LSA_DM_WRAP_LOOKUP_NAME_BY_SID_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.pszSid = pszSid;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
@@ -640,6 +679,7 @@ LsaDmWrapNetLookupNameByObjectSid(
 
 DWORD
 LsaDmWrapNetLookupNamesByObjectSids(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN DWORD dwSidCounts,
     IN PSTR* ppszSids,
@@ -650,10 +690,12 @@ LsaDmWrapNetLookupNamesByObjectSids(
     DWORD dwError = 0;
     LSA_DM_WRAP_LOOKUP_NAMES_BY_SIDS_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.ppszSids = ppszSids;
     context.dwSidCounts = dwSidCounts;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
@@ -668,6 +710,7 @@ LsaDmWrapNetLookupNamesByObjectSids(
 
 DWORD
 LsaDmWrapNetLookupObjectSidsByNames(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN DWORD dwNameCounts,
     IN PSTR* ppszNames,
@@ -678,10 +721,12 @@ LsaDmWrapNetLookupObjectSidsByNames(
     DWORD dwError = 0;
     LSA_DM_WRAP_LOOKUP_SIDS_BY_NAMES_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.ppszNames = ppszNames;
     context.dwNameCounts = dwNameCounts;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
@@ -696,6 +741,7 @@ LsaDmWrapNetLookupObjectSidsByNames(
 
 DWORD
 LsaDmWrapDsEnumerateDomainTrusts(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN DWORD dwFlags,
     OUT NetrDomainTrust** ppTrusts,
@@ -705,9 +751,11 @@ LsaDmWrapDsEnumerateDomainTrusts(
     DWORD dwError = 0;
     LSA_DM_WRAP_ENUM_DOMAIN_TRUSTS_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.dwFlags = dwFlags;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                       LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                       NULL,
@@ -722,6 +770,7 @@ LsaDmWrapDsEnumerateDomainTrusts(
 
 DWORD
 LsaDmWrapDsGetDcName(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN PCSTR pszFindDomainName,
     IN BOOLEAN bReturnDnsName,
@@ -732,10 +781,12 @@ LsaDmWrapDsGetDcName(
     DWORD dwError = 0;
     LSA_DM_WRAP_GET_DC_NAME_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.pszDomainName = pszFindDomainName;
     context.bReturnDnsName = bReturnDnsName;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                 pszDnsDomainName,
                                  LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
                                  LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
                                  NULL,
@@ -759,14 +810,15 @@ static
 DWORD
 LsaDmWrappAuthenticateUserExCallback(
     IN PCSTR pszDnsDomainOrForestName,
-    IN OPTIONAL PLWNET_DC_INFO pDcInfo,
-    IN OPTIONAL PVOID pContext,
+    IN PLWNET_DC_INFO pDcInfo,
+    IN PVOID pContext,
     OUT PBOOLEAN pbIsNetworkError
     )
 {
     PLSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT pCtx = (PLSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT) pContext;
 
     return AD_NetlogonAuthenticationUserEx(
+                    pCtx->pState,
                     pDcInfo->pszDomainControllerName,
                     pCtx->pUserParams,
                     pCtx->ppUserInfo,
@@ -775,6 +827,7 @@ LsaDmWrappAuthenticateUserExCallback(
 
 DWORD
 LsaDmWrapAuthenticateUserEx(
+    IN LSA_DM_STATE_HANDLE hDmState,
     IN PCSTR pszDnsDomainName,
     IN PLSA_AUTH_USER_PARAMS pUserParams,
     OUT PLSA_AUTH_USER_INFO *ppUserInfo
@@ -783,12 +836,15 @@ LsaDmWrapAuthenticateUserEx(
     DWORD dwError = 0;
     LSA_DM_WRAP_AUTH_USER_EX_CALLBACK_CONTEXT context = { 0 };
 
+    LsaDmpGetProviderState(hDmState, &context.pState);
     context.pUserParams = pUserParams;
     context.ppUserInfo = ppUserInfo;
 
-    dwError = LsaDmConnectDomain(pszDnsDomainName,
+    dwError = LsaDmConnectDomain(hDmState,
+                                      pszDnsDomainName,
                                       LSA_DM_CONNECT_DOMAIN_FLAG_AUTH |
-                                      LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO,
+                                      LSA_DM_CONNECT_DOMAIN_FLAG_DC_INFO |
+                                      LSA_DM_CONNECT_DOMAIN_FLAG_NETRSAMLOGON,
                                       NULL,
                                       LsaDmWrappAuthenticateUserExCallback,
                                       &context);
