@@ -125,6 +125,9 @@ typedef struct _LSA_UM_KSCHEDULES {
 /// Keeps track of all domain state.
 ///
 typedef struct _LSA_UM_STATE {
+    /// Provider state.
+    PLSA_AD_PROVIDER_STATE pProviderState;
+
     /// Linked list of users.
     PLSA_UM_USER_REFRESH_LIST UserList;
 
@@ -154,6 +157,7 @@ LsaUmpCheckUsers(
 static
 DWORD
 LsaUmpCreateKeys(
+    PLSA_AD_PROVIDER_STATE pState,
     PLSA_UM_KSCHEDULES kSchedules
     );
 
@@ -207,6 +211,7 @@ static
 DWORD
 LsaUmpRefreshUserCreds(
     LSA_UM_STATE_HANDLE       Handle,
+    PAD_PROVIDER_CONTEXT      pContext,
     PLSA_UM_USER_REFRESH_ITEM pItem
     );
 
@@ -405,6 +410,7 @@ LsaUmpStateDestroy(
 
 DWORD
 LsaUmpStateCreate(
+    IN PLSA_AD_PROVIDER_STATE pProviderState,
     OUT PLSA_UM_STATE_HANDLE pHandle
     )
 ///<
@@ -428,6 +434,8 @@ LsaUmpStateCreate(
                   (PVOID*)&pState);
     BAIL_ON_LSA_ERROR(dwError);
 
+    pState->pProviderState = pProviderState;
+
     pState->dwCheckUsersSeconds = LSA_UM_THREAD_MIN_PERIOD;
 
     dwError = LsaUmpCreateMutex(
@@ -457,7 +465,9 @@ LsaUmpStateCreate(
                   (PVOID*)&pState->kSchedules);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaUmpCreateKeys(pState->kSchedules);
+    dwError = LsaUmpCreateKeys(
+                  pProviderState,
+                  pState->kSchedules);
     BAIL_ON_LSA_ERROR(dwError);
 
     // Now that everything is set up, we need to initialize the thread.
@@ -526,6 +536,11 @@ LsaUmpThreadRoutine(
 
     LSA_LOG_VERBOSE("Started user manager credentials refresh thread");
 
+    dwError = LwKrb5SetThreadDefaultCachePath(
+                  pState->pProviderState->MachineCreds.pszCachePath,
+                  NULL);
+    BAIL_ON_LSA_ERROR(dwError);
+
     for (;;)
     {
         DWORD   dwCheckUsersSeconds = 0;
@@ -642,6 +657,7 @@ LsaUmpCheckUsers(
     PLSA_UM_USER_REFRESH_ITEM pItem = NULL;
     PLSA_UM_USER_REFRESH_ITEM pNextItem = NULL;
     DWORD                     dwTime = 0;
+    PAD_PROVIDER_CONTEXT      pProviderContext = NULL;
 
     LSA_LOG_VERBOSE("Lsa User Manager - checking user credentials refresh list");
 
@@ -692,16 +708,32 @@ LsaUmpCheckUsers(
         }
     }
 
-    ADSyncTimeToDC(gpADProviderData->szDomain);
+    ADSyncTimeToDC(
+        Handle->pProviderState,
+        Handle->pProviderState->pProviderData->szDomain);
 
-    bDomainIsOffline = LsaDmIsDomainOffline(gpADProviderData->szDomain);
-    bShouldRefreshCreds = AD_ShouldRefreshUserCreds();
+    bDomainIsOffline = LsaDmIsDomainOffline(
+                           Handle->pProviderState->hDmState,
+                           Handle->pProviderState->pProviderData->szDomain);
+    bShouldRefreshCreds = AD_ShouldRefreshUserCreds(Handle->pProviderState);
 
     if ( bDomainIsOffline )
     {
         LSA_LOG_DEBUG("LSA User Manager - domain is offline");
     }
 
+    if (bShouldRefreshCreds)
+    {
+        dwError = AD_CreateProviderContext(
+                      Handle->pProviderState->pszDomainName,
+                      Handle->pProviderState,
+                      &pProviderContext);
+        if (dwError)
+        {
+            bShouldRefreshCreds = FALSE;
+        }
+    }
+
     if ( pUserList )
     {
         for ( pItem = pUserList->pFirst ; pItem ; pItem = pNextItem )
@@ -737,12 +769,14 @@ LsaUmpCheckUsers(
                 // ignore errors because we want to process all users
                 LsaUmpRefreshUserCreds(
                     Handle,
+                    pProviderContext,
                     pItem);
             }
         }
     }
 
     LsaUmpFreeRequestList(pRequestList);
+    AD_DereferenceProviderContext(pProviderContext);
 
     return dwError;
 }
@@ -750,48 +784,50 @@ LsaUmpCheckUsers(
 static
 DWORD
 LsaUmpCreateKeys(
+    PLSA_AD_PROVIDER_STATE pState,
     PLSA_UM_KSCHEDULES kSchedules
     )
 {
     DWORD      dwError = 0;
     DWORD      dwCount = 0;
     DES_cblock Key;
-    PSTR       pszHostname = NULL;
-    PSTR       pszUsername = NULL;
-    PSTR       pszServicePassword = NULL;
-    PSTR       pszDomainDnsName = NULL;
-    PSTR       pszHostDnsDomain = NULL;
     DWORD      dwTime = 0;
     pid_t      Pid = 0;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
     if ( !RAND_status() )
     {
         LSA_LOG_DEBUG("LSA User Manager - randomness not seeded automatically");
-        dwError = LsaDnsGetHostInfo(&pszHostname);
+
+        dwError = LsaPcacheGetMachineAccountInfoA(
+                      pState->pPcache,
+                      &pAccountInfo);
         BAIL_ON_LSA_ERROR(dwError);
 
         RAND_seed(
-            pszHostname,
-            strlen(pszHostname));
-
-        LwStrToLower(pszHostname);
-
-        dwError = LwKrb5GetMachineCreds(
-                      &pszUsername,
-                      &pszServicePassword,
-                      &pszDomainDnsName,
-                      &pszHostDnsDomain);
-        BAIL_ON_LSA_ERROR(dwError);
-
+            pAccountInfo->DnsDomainName,
+            strlen(pAccountInfo->DnsDomainName));
         RAND_seed(
-            pszServicePassword,
-            strlen(pszServicePassword));
+            pAccountInfo->NetbiosDomainName,
+            strlen(pAccountInfo->NetbiosDomainName));
         RAND_seed(
-            pszUsername,
-            strlen(pszUsername));
+            pAccountInfo->DomainSid,
+            strlen(pAccountInfo->DomainSid));
         RAND_seed(
-            pszDomainDnsName,
-            strlen(pszDomainDnsName));
+            pAccountInfo->SamAccountName,
+            strlen(pAccountInfo->SamAccountName));
+        RAND_seed(
+            &pAccountInfo->AccountFlags,
+            sizeof(pAccountInfo->AccountFlags));
+        RAND_seed(
+            &pAccountInfo->KeyVersionNumber,
+            sizeof(pAccountInfo->KeyVersionNumber));
+        RAND_seed(
+            pAccountInfo->Fqdn,
+            strlen(pAccountInfo->Fqdn));
+        RAND_seed(
+            &pAccountInfo->LastChangeTime,
+            sizeof(pAccountInfo->LastChangeTime));
 
         dwTime = time(NULL);
         RAND_seed(
@@ -836,12 +872,7 @@ cleanup:
 
     memset(&Key, 0, sizeof(Key));
 
-    LW_SAFE_FREE_STRING(pszHostname);
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_CLEAR_FREE_STRING(pszServicePassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
-
+    LsaPcacheReleaseMachineAccountInfoA(pAccountInfo);
 
     return dwError;
 
@@ -991,7 +1022,7 @@ error:
 
     *ppszString = NULL;
 
-    LW_SAFE_CLEAR_FREE_STRING(pszString);
+    LW_SECURE_FREE_STRING(pszString);
 
     goto cleanup;
 }
@@ -1003,11 +1034,7 @@ LsaUmpFreePassword(
     DWORD dwPasswordLen
     )
 {
-    if ( pPassword )
-    {
-        memset(pPassword, 0, dwPasswordLen);
-        LW_SAFE_FREE_MEMORY(pPassword);
-    }
+    LW_SECURE_FREE_MEMORY(pPassword, dwPasswordLen);
 }
 #endif
 
@@ -1492,6 +1519,7 @@ static
 DWORD
 LsaUmpRefreshUserCreds(
     LSA_UM_STATE_HANDLE       Handle,
+    PAD_PROVIDER_CONTEXT      pProviderContext,
     PLSA_UM_USER_REFRESH_ITEM pUserItem
     )
 {
@@ -1515,7 +1543,7 @@ LsaUmpRefreshUserCreds(
     }
 
     dwError = AD_FindUserObjectById(
-                  (HANDLE)NULL,
+                  pProviderContext,
                   pUserItem->uUid,
                   &pUserInfo);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1527,10 +1555,11 @@ LsaUmpRefreshUserCreds(
         NULL);
 
     dwError = AD_OnlineCheckUserPassword(
-                    (HANDLE)NULL,
+                    pProviderContext,
                     pUserInfo,
                     pszPassword,
-                    &pUserItem->dwTgtEndTime);
+                    &pUserItem->dwTgtEndTime,
+                    NULL);
     BAIL_ON_LSA_ERROR(dwError);
 
     // At this point the user's TGT has been refreshed.
@@ -1539,7 +1568,7 @@ LsaUmpRefreshUserCreds(
     // post-processing fails.
     pUserItem->dwFailedCount = 0;
 
-    if (AD_EventlogEnabled())
+    if (AD_EventlogEnabled(Handle->pProviderState))
     {
         LsaUmpLogUserTGTRefreshSuccessEvent(pUserInfo->userInfo.pszUPN,
                                             pUserItem->uUid,
@@ -1560,7 +1589,7 @@ error:
     {
         pUserItem->dwFailedCount++;
 
-        if (AD_EventlogEnabled())
+        if (AD_EventlogEnabled(Handle->pProviderState))
         {
             LsaUmpLogUserTGTRefreshFailureEvent(pUserInfo->userInfo.pszUPN,
                                                 pUserItem->uUid,
