@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -38,7 +38,7 @@
  * Abstract:
  *
  *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ * 
  *        Machine Password API
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -55,47 +55,26 @@
 // Global Module State Type
 //
 
-typedef struct _AD_MACHINE_PASSWORD_SYNC_STATE {
+typedef struct _LSA_MACHINEPWD_STATE {
     BOOLEAN bThreadShutdown;
     DWORD dwThreadWaitSecs;
     pthread_t Thread;
-    pthread_mutex_t ThreadLock;
-    pthread_cond_t ThreadCondition;
     pthread_t* pThread;
-    HANDLE hPasswordStore;
+    pthread_mutex_t ThreadLock;
+    pthread_mutex_t *pThreadLock;
+    pthread_cond_t ThreadCondition;
+    pthread_cond_t *pThreadCondition;
     DWORD dwTgtExpiry;
     DWORD dwTgtExpiryGraceSeconds;
-} AD_MACHINE_PASSWORD_SYNC_STATE, *PAD_MACHINE_PASSWORD_SYNC_STATE;
-
-//
-// Global Module State Variable
-//
-
-//
-// ISSUE-2009/07/24-dalmeida -- Make the global state be
-// dynamically initialized rather than be statically
-// initialized as a global so we can easily restart.
-//
-
-static AD_MACHINE_PASSWORD_SYNC_STATE gAdMachinePasswordSyncState = {
-    .bThreadShutdown = FALSE,
-    .dwThreadWaitSecs = DEFAULT_THREAD_WAITSECS,
-    .ThreadLock = PTHREAD_MUTEX_INITIALIZER,
-    .ThreadCondition = PTHREAD_COND_INITIALIZER,
-    .pThread = NULL,
-    .hPasswordStore = NULL,
-    .dwTgtExpiry = 0,
-    .dwTgtExpiryGraceSeconds = 2 * DEFAULT_THREAD_WAITSECS,
-};
-
-//
-// Static Function Prototypes
-//
+    // Datalock protects dwTgtExpiry and dwTgtExpiryGraceSeconds
+    pthread_rwlock_t DataLock;
+    pthread_rwlock_t* pDataLock;
+} LSA_MACHINEPWD_STATE, *PLSA_MACHINEPWD_STATE;
 
 static
 BOOLEAN
 ADShouldRefreshMachineTGT(
-    VOID
+    IN PLSA_MACHINEPWD_STATE pMachinePwdState
     );
 
 static
@@ -131,6 +110,7 @@ ADLogMachineTGTRefreshFailureEvent(
 static
 VOID
 ADSetMachineTGTExpiryInternal(
+    PLSA_MACHINEPWD_STATE pMachinePwdState,
     DWORD dwGoodUntil,
     DWORD dwThreadWaitSecs
     );
@@ -141,39 +121,64 @@ ADSetMachineTGTExpiryInternal(
 
 DWORD
 ADInitMachinePasswordSync(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwError = 0;
+    PLSA_MACHINEPWD_STATE pMachinePwdState = NULL;
 
-    dwError = LwpsOpenPasswordStore(
-                    LWPS_PASSWORD_STORE_DEFAULT,
-                    &gAdMachinePasswordSyncState.hPasswordStore);
+    dwError = LwAllocateMemory(
+                  sizeof(*pMachinePwdState),
+                  (PVOID*)&pMachinePwdState);
     BAIL_ON_LSA_ERROR(dwError);
 
+    pMachinePwdState->bThreadShutdown = FALSE;
+    pMachinePwdState->dwThreadWaitSecs = DEFAULT_THREAD_WAITSECS;
+    pMachinePwdState->dwTgtExpiryGraceSeconds = 2 * DEFAULT_THREAD_WAITSECS;
+
+    dwError = LwMapErrnoToLwError(pthread_mutex_init(&pMachinePwdState->ThreadLock, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    pMachinePwdState->pThreadLock = &pMachinePwdState->ThreadLock;
+
+    dwError = LwMapErrnoToLwError(pthread_cond_init(&pMachinePwdState->ThreadCondition, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    pMachinePwdState->pThreadCondition = &pMachinePwdState->ThreadCondition;
+
+    dwError = LwMapErrnoToLwError(pthread_rwlock_init(&pMachinePwdState->DataLock, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    pMachinePwdState->pDataLock = &pMachinePwdState->DataLock;
+
+    pState->hMachinePwdState = pMachinePwdState;
+
 cleanup:
 
     return dwError;
 
 error:
 
+    ADShutdownMachinePasswordSync(&pMachinePwdState);
+
     goto cleanup;
 }
 
 DWORD
 ADStartMachinePasswordSync(
-    VOID
+    IN PLSA_AD_PROVIDER_STATE pState
     )
 {
     DWORD dwError = 0;
+    PLSA_MACHINEPWD_STATE pMachinePwdState = pState->hMachinePwdState;
 
-    dwError = pthread_create(&gAdMachinePasswordSyncState.Thread,
+    dwError = pthread_create(&pMachinePwdState->Thread,
                              NULL,
                              ADSyncMachinePasswordThreadRoutine,
-                             NULL);
+                             pState);
     BAIL_ON_LSA_ERROR(dwError);
 
-    gAdMachinePasswordSyncState.pThread = &gAdMachinePasswordSyncState.Thread;
+    pMachinePwdState->pThread = &pMachinePwdState->Thread;
 
 cleanup:
 
@@ -181,7 +186,53 @@ cleanup:
 
 error:
 
-    gAdMachinePasswordSyncState.pThread = NULL;
+    pMachinePwdState->pThread = NULL;
+
+    goto cleanup;
+}
+
+static
+DWORD
+ADChangeMachinePasswordInThreadLock(
+    IN PLSA_AD_PROVIDER_STATE pState
+    )
+{
+    DWORD dwError = 0;
+
+    LSA_LOG_VERBOSE("Changing machine password for %s", pState->pszDomainName);
+
+    dwError = AD_SetSystemAccess(pState, NULL);
+    if (dwError)
+    {
+        LSA_LOG_ERROR("Error: Failed to acquire credentials (error = %u)", dwError);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = LsaMachineChangePassword(pState->pszDomainName);
+    if (dwError)
+    {
+        LSA_LOG_ERROR("Error: Failed to change machine password for %s (error = %u)", pState->pszDomainName, dwError);
+
+        if (AD_EventlogEnabled(pState))
+        {
+            ADLogMachinePWUpdateFailureEvent(dwError);
+        }
+
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    LsaPcacheClearPasswordInfo(pState->pPcache);
+
+    if (AD_EventlogEnabled(pState))
+    {
+        ADLogMachinePWUpdateSuccessEvent();
+    }
+
+cleanup:
+
+    return dwError;
+
+error:
 
     goto cleanup;
 }
@@ -193,16 +244,22 @@ ADSyncMachinePasswordThreadRoutine(
     )
 {
     DWORD dwError = 0;
+    PLSA_AD_PROVIDER_STATE pState = (PLSA_AD_PROVIDER_STATE)pData;
+    PLSA_MACHINEPWD_STATE pMachinePwdState = (PLSA_MACHINEPWD_STATE)pState->hMachinePwdState;
     DWORD dwPasswordSyncLifetime = 0;
     struct timespec timeout = {0, 0};
-    PLWPS_PASSWORD_INFO pAcctInfo = NULL;
-    PSTR pszHostname = NULL;
-    PSTR pszDnsDomainName = NULL;
     DWORD dwGoodUntilTime = 0;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
+    UINT64 lastChangeTime = 0;
 
     LSA_LOG_INFO("Machine Password Sync Thread starting");
 
-    pthread_mutex_lock(&gAdMachinePasswordSyncState.ThreadLock);
+    pthread_mutex_lock(pMachinePwdState->pThreadLock);
+
+    dwError = LwKrb5SetThreadDefaultCachePath(
+                  pState->MachineCreds.pszCachePath,
+                  NULL);
+    BAIL_ON_LSA_ERROR(dwError);
 
     for (;;)
     {
@@ -210,42 +267,54 @@ ADSyncMachinePasswordThreadRoutine(
         DWORD dwCurrentPasswordAge = 0;
         BOOLEAN bRefreshTGT = FALSE;
 
-        dwError = LsaDnsGetHostInfo(&pszHostname);
+        if (pMachinePwdState->bThreadShutdown)
+        {
+           break;
+        }
+
+        if (!pState->pProviderData)
+        {
+            dwError = 0;
+            goto lsa_wait_resync;
+        }
+        ADSyncTimeToDC(pState, pState->pProviderData->szDomain);
+
+        dwError = LsaPcacheGetMachineAccountInfoA(
+                      pState->pPcache,
+                      &pAccountInfo);
         if (dwError)
         {
-            LSA_LOG_ERROR("Error: Failed to find hostname (Error code: %ld)",
-                          dwError);
+            LSA_LOG_ERROR("Error: Failed to get machine account information (error = %u)", dwError);
             dwError = 0;
             goto lsa_wait_resync;
         }
 
-        if (!gpADProviderData)
-        {
-            dwError = 0;
-            goto lsa_wait_resync;
-        }
-        ADSyncTimeToDC(gpADProviderData->szDomain);
-
-        dwError = LwpsGetPasswordByHostName(
-                        gAdMachinePasswordSyncState.hPasswordStore,
-                        pszHostname,
-                        &pAcctInfo);
+        dwError = ADConvertTimeNt2Unix(
+                        (UINT64) pAccountInfo->LastChangeTime,
+                        &lastChangeTime);
         if (dwError)
         {
-            LSA_LOG_ERROR("Error: Failed to re-sync machine account (Error code: %ld)", dwError);
+            LSA_LOG_DEBUG("Failed to convert time (error = %u)", dwError);
             dwError = 0;
             goto lsa_wait_resync;
         }
 
-        dwCurrentPasswordAge =
+        if (lastChangeTime != (time_t) lastChangeTime)
+        {
+            LSA_LOG_DEBUG("Overflow in converted time");
+            dwError = 0;
+            goto lsa_wait_resync;
+        }
+
+        dwCurrentPasswordAge = 
                          difftime(
                               time(NULL),
-                              pAcctInfo->last_change_time);
+                              (time_t) lastChangeTime);
 
-        dwPasswordSyncLifetime = AD_GetMachinePasswordSyncPwdLifetime();
+        dwPasswordSyncLifetime = AD_GetMachinePasswordSyncPwdLifetime(pState);
         dwReapingAge = dwPasswordSyncLifetime / 2;
 
-        dwError = AD_MachineCredentialsCacheInitialize();
+        dwError = AD_MachineCredentialsCacheInitialize(pState);
         if (dwError)
         {
             LSA_LOG_DEBUG("Failed to initialize credentials cache (error = %u)", dwError);
@@ -255,75 +324,59 @@ ADSyncMachinePasswordThreadRoutine(
 
         if ((dwReapingAge > 0) && (dwCurrentPasswordAge >= dwReapingAge))
         {
-            LSA_LOG_VERBOSE("Changing machine password");
+            PTHREAD_CALL_MUST_SUCCEED(
+                    pthread_rwlock_wrlock(pMachinePwdState->pDataLock));
+
+            dwError = ADChangeMachinePasswordInThreadLock(pState);
+
+            PTHREAD_CALL_MUST_SUCCEED(
+                    pthread_rwlock_unlock(pMachinePwdState->pDataLock));
 
-            dwError = AD_SetSystemAccess(NULL);
             if (dwError)
             {
-                LSA_LOG_ERROR("Error: Failed to acquire credentials, error = %u", dwError);
-
                 dwError = 0;
-                goto lsa_wait_resync;
             }
-
-            dwError = LsaMachineChangePassword();
-            if (dwError)
+            else
             {
-                LSA_LOG_ERROR("Error: Failed to re-sync machine account [Error code: %ld]", dwError);
-
-                if (AD_EventlogEnabled())
-                {
-                    ADLogMachinePWUpdateFailureEvent(dwError);
-                }
-
-                dwError = 0;
-                goto lsa_wait_resync;
+                bRefreshTGT = TRUE;
             }
-
-            if (AD_EventlogEnabled())
-            {
-                ADLogMachinePWUpdateSuccessEvent();
-            }
-
-            bRefreshTGT = TRUE;
         }
-        else
+
+        if (!bRefreshTGT)
         {
-            bRefreshTGT = ADShouldRefreshMachineTGT();
+            bRefreshTGT = ADShouldRefreshMachineTGT(pMachinePwdState);
         }
 
         if (bRefreshTGT)
         {
-            dwError = LwKrb5RefreshMachineTGT(&dwGoodUntilTime);
+            dwError = ADRefreshMachineTGT(pState, &dwGoodUntilTime);
             if (dwError)
             {
-                if (AD_EventlogEnabled())
+                if (AD_EventlogEnabled(pState))
                 {
                     ADLogMachineTGTRefreshFailureEvent(dwError);
                 }
 
-                LSA_LOG_ERROR("Error: Failed to refresh machine TGT [Error code: %ld]", dwError);
+                LSA_LOG_ERROR("Error: Failed to refresh machine TGT for %s (error = %u)", pState->pszDomainName, dwError);
 
                 if (dwError == LW_ERROR_DOMAIN_IS_OFFLINE)
                 {
-                    LW_SAFE_FREE_STRING(pszDnsDomainName);
-
-                    dwError = LwWc16sToMbs(pAcctInfo->pwszDnsDomainName, &pszDnsDomainName);
-                    BAIL_ON_LSA_ERROR(dwError);
-
-                    LsaDmTransitionOffline(pszDnsDomainName, FALSE);
+                    LsaDmTransitionOffline(
+                        pState->hDmState,
+                        pState->pszDomainName,
+                        FALSE);
                 }
 
-                ADSetMachineTGTExpiryError();
+                ADSetMachineTGTExpiryError(pMachinePwdState);
                 dwError = 0;
                 goto lsa_wait_resync;
             }
 
-            ADSetMachineTGTExpiry(dwGoodUntilTime);
+            ADSetMachineTGTExpiry(pMachinePwdState, dwGoodUntilTime);
 
-            LSA_LOG_VERBOSE("Machine TGT was refreshed successfully");
+            LSA_LOG_VERBOSE("Machine TGT was refreshed successfully for %s", pState->pszDomainName);
 
-            if (AD_EventlogEnabled())
+            if (AD_EventlogEnabled(pState))
             {
                 ADLogMachineTGTRefreshSuccessEvent();
             }
@@ -331,24 +384,19 @@ ADSyncMachinePasswordThreadRoutine(
 
 lsa_wait_resync:
 
-        if (pAcctInfo)
-        {
-            LwpsFreePasswordInfo(gAdMachinePasswordSyncState.hPasswordStore, pAcctInfo);
-            pAcctInfo = NULL;
-        }
+        LsaPcacheReleaseMachineAccountInfoA(pAccountInfo);
+        pAccountInfo = NULL;
 
-        LW_SAFE_FREE_STRING(pszHostname);
-
-        timeout.tv_sec = time(NULL) + gAdMachinePasswordSyncState.dwThreadWaitSecs;
+        timeout.tv_sec = time(NULL) + pMachinePwdState->dwThreadWaitSecs;
         timeout.tv_nsec = 0;
 
 retry_wait:
 
-        dwError = pthread_cond_timedwait(&gAdMachinePasswordSyncState.ThreadCondition,
-                                         &gAdMachinePasswordSyncState.ThreadLock,
+        dwError = pthread_cond_timedwait(pMachinePwdState->pThreadCondition,
+                                         pMachinePwdState->pThreadLock,
                                          &timeout);
 
-        if (gAdMachinePasswordSyncState.bThreadShutdown)
+        if (pMachinePwdState->bThreadShutdown)
         {
            break;
         }
@@ -364,19 +412,13 @@ retry_wait:
         }
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 cleanup:
 
-    if (pAcctInfo)
-    {
-        LwpsFreePasswordInfo(gAdMachinePasswordSyncState.hPasswordStore, pAcctInfo);
-    }
-
-    LW_SAFE_FREE_STRING(pszHostname);
-    LW_SAFE_FREE_STRING(pszDnsDomainName);
-
-    pthread_mutex_unlock(&gAdMachinePasswordSyncState.ThreadLock);
+    LsaPcacheReleaseMachineAccountInfoA(pAccountInfo);
 
+    pthread_mutex_unlock(pMachinePwdState->pThreadLock);
+    
     LSA_LOG_INFO("Machine Password Sync Thread stopping");
 
     return NULL;
@@ -390,6 +432,7 @@ error:
 
 VOID
 ADSyncTimeToDC(
+    PLSA_AD_PROVIDER_STATE pState,
     PCSTR pszDomainFQDN
     )
 {
@@ -397,14 +440,14 @@ ADSyncTimeToDC(
     LWNET_UNIX_TIME_T dcTime = 0;
     time_t ttDcTime = 0;
 
-    if ( !AD_ShouldSyncSystemTime() )
+    if ( !pState->bIsDefault || !AD_ShouldSyncSystemTime(pState) )
     {
         goto cleanup;
     }
 
     BAIL_ON_INVALID_STRING(pszDomainFQDN);
 
-    if (LsaDmIsDomainOffline(pszDomainFQDN))
+    if (LsaDmIsDomainOffline(pState->hDmState, pszDomainFQDN))
     {
         goto cleanup;
     }
@@ -413,18 +456,18 @@ ADSyncTimeToDC(
                     pszDomainFQDN,
                     &dcTime);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     ttDcTime = (time_t) dcTime;
-
-    if (labs(ttDcTime - time(NULL)) > AD_GetClockDriftSeconds()) {
+    
+    if (labs(ttDcTime - time(NULL)) > AD_GetClockDriftSeconds(pState)) {
         dwError = LsaSetSystemTime(ttDcTime);
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 cleanup:
 
     return;
-
+    
 error:
 
     LSA_LOG_ERROR("Failed to sync system time [error code: %u]", dwError);
@@ -433,100 +476,197 @@ error:
 }
 
 VOID
-ADShutdownMachinePasswordSync(
-    VOID
+ADLockMachinePassword(
+    IN LSA_MACHINEPWD_STATE_HANDLE hMachinePwdState
     )
 {
-    if (gAdMachinePasswordSyncState.pThread)
-    {
-        pthread_mutex_lock(&gAdMachinePasswordSyncState.ThreadLock);
-        gAdMachinePasswordSyncState.bThreadShutdown = TRUE;
-        pthread_cond_signal(&gAdMachinePasswordSyncState.ThreadCondition);
-        pthread_mutex_unlock(&gAdMachinePasswordSyncState.ThreadLock);
+    PTHREAD_CALL_MUST_SUCCEED(
+            pthread_rwlock_rdlock(hMachinePwdState->pDataLock));
+}
 
-        pthread_join(gAdMachinePasswordSyncState.Thread, NULL);
-        gAdMachinePasswordSyncState.pThread = NULL;
+VOID
+ADUnlockMachinePassword(
+    IN LSA_MACHINEPWD_STATE_HANDLE hMachinePwdState
+    )
+{
+    PTHREAD_CALL_MUST_SUCCEED(
+            pthread_rwlock_unlock(hMachinePwdState->pDataLock));
+}
+
+VOID
+ADShutdownMachinePasswordSync(
+    IN OUT LSA_MACHINEPWD_STATE_HANDLE* phMachinePwdState
+    )
+{
+    if (phMachinePwdState && *phMachinePwdState)
+    {
+        PLSA_MACHINEPWD_STATE pMachinePwdState = (PLSA_MACHINEPWD_STATE)*phMachinePwdState;
+
+        if (pMachinePwdState->pThread)
+        {   
+            pthread_mutex_lock(pMachinePwdState->pThreadLock);
+            pMachinePwdState->bThreadShutdown = TRUE;
+            pthread_cond_signal(pMachinePwdState->pThreadCondition);
+            pthread_mutex_unlock(pMachinePwdState->pThreadLock);
+
+            pthread_join(pMachinePwdState->Thread, NULL);
+            pMachinePwdState->pThread = NULL;
+            pMachinePwdState->bThreadShutdown = FALSE;
+        }
+
+        if (pMachinePwdState->pThreadCondition)
+        {
+            pthread_cond_destroy(pMachinePwdState->pThreadCondition);
+        }
+
+        if (pMachinePwdState->pThreadLock)
+        {
+            pthread_mutex_destroy(pMachinePwdState->pThreadLock);
+        }
+
+        if (pMachinePwdState->pDataLock)
+        {
+            pthread_rwlock_destroy(pMachinePwdState->pDataLock);
+        }
+
+        LwFreeMemory(pMachinePwdState);
+        *phMachinePwdState = NULL;
+    }
+}
+
+DWORD
+ADRefreshMachineTGT(
+    IN PLSA_AD_PROVIDER_STATE pState,
+    OUT OPTIONAL PDWORD pdwGoodUntilTime
+    )
+{
+    DWORD dwError = 0;
+    DWORD dwGoodUntilTime = 0;
+    PLSA_MACHINE_PASSWORD_INFO_A pPasswordInfo = NULL;
+    PSTR pszUserPrincipalName = NULL;
+
+    LSA_LOG_VERBOSE("Refreshing machine TGT");
+
+    dwError = LsaPcacheGetMachinePasswordInfoA(
+                    pState->pPcache,
+                    &pPasswordInfo);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LwAllocateStringPrintf(
+                    &pszUserPrincipalName,
+                    "%s@%s",
+                    pPasswordInfo->Account.SamAccountName,
+                    pPasswordInfo->Account.DnsDomainName);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LwKrb5InitializeCredentials(
+                    pszUserPrincipalName,
+                    pPasswordInfo->Password,
+                    pState->MachineCreds.pszCachePath,
+                    &dwGoodUntilTime);
+    BAIL_ON_LSA_ERROR(dwError);
+
+error:
+    if (dwError)
+    {
+        dwGoodUntilTime = 0;
     }
 
-    if (gAdMachinePasswordSyncState.hPasswordStore)
+    LW_SAFE_FREE_STRING(pszUserPrincipalName);
+    LsaPcacheReleaseMachinePasswordInfoA(pPasswordInfo);
+
+    if (pdwGoodUntilTime)
     {
-        LwpsClosePasswordStore(gAdMachinePasswordSyncState.hPasswordStore);
-        gAdMachinePasswordSyncState.hPasswordStore = NULL;
+        *pdwGoodUntilTime = dwGoodUntilTime;
     }
+
+    return dwError;
 }
 
 static
 BOOLEAN
 ADShouldRefreshMachineTGT(
-    VOID
+    IN PLSA_MACHINEPWD_STATE pMachinePwdState
     )
 {
     BOOLEAN bRefresh = FALSE;
-    BOOLEAN bInLock = FALSE;
+    int status = 0;
 
-    ENTER_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    status = pthread_rwlock_rdlock(pMachinePwdState->pDataLock);
+    LW_ASSERT(status == 0);
 
-    if (!gAdMachinePasswordSyncState.dwTgtExpiry ||
-        (difftime(gAdMachinePasswordSyncState.dwTgtExpiry, time(NULL)) <= gAdMachinePasswordSyncState.dwTgtExpiryGraceSeconds))
+    if (!pMachinePwdState->dwTgtExpiry ||
+        (difftime(pMachinePwdState->dwTgtExpiry, time(NULL)) <= pMachinePwdState->dwTgtExpiryGraceSeconds))
     {
         bRefresh = TRUE;
     }
 
-    LEAVE_AD_GLOBAL_DATA_RW_READER_LOCK(bInLock);
+    status = pthread_rwlock_unlock(pMachinePwdState->pDataLock);
+    LW_ASSERT(status == 0);
 
     return bRefresh;
 }
 
 VOID
 ADSetMachineTGTExpiry(
-    DWORD dwGoodUntil
+    IN LSA_MACHINEPWD_STATE_HANDLE hMachinePwdState,
+    IN DWORD dwGoodUntil
     )
 {
-    ADSetMachineTGTExpiryInternal(dwGoodUntil, DEFAULT_THREAD_WAITSECS);
+    ADSetMachineTGTExpiryInternal(
+        hMachinePwdState,
+        dwGoodUntil,
+        DEFAULT_THREAD_WAITSECS);
 }
 
 VOID
 ADSetMachineTGTExpiryError(
-    VOID
+    IN LSA_MACHINEPWD_STATE_HANDLE hMachinePwdState
     )
 {
-    ADSetMachineTGTExpiryInternal(0, ERROR_THREAD_WAITSECS);
+    ADSetMachineTGTExpiryInternal(
+        hMachinePwdState,
+        0,
+        ERROR_THREAD_WAITSECS);
 }
 
 static
 VOID
 ADSetMachineTGTExpiryInternal(
+    PLSA_MACHINEPWD_STATE pMachinePwdState,
     DWORD dwGoodUntil,
     DWORD dwThreadWaitSecs
     )
 {
-    BOOLEAN bInLock = FALSE;
     DWORD lifetime = 0;
+    int status = 0;
 
-    ENTER_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    status = pthread_rwlock_wrlock(pMachinePwdState->pDataLock);
+    LW_ASSERT(status == 0);
 
     if (dwGoodUntil)
     {
-        gAdMachinePasswordSyncState.dwTgtExpiry = dwGoodUntil;
+        pMachinePwdState->dwTgtExpiry = dwGoodUntil;
 
         lifetime = difftime(
-                       gAdMachinePasswordSyncState.dwTgtExpiry,
+                       pMachinePwdState->dwTgtExpiry,
                        time(NULL));
 
-        gAdMachinePasswordSyncState.dwTgtExpiryGraceSeconds =
+        pMachinePwdState->dwTgtExpiryGraceSeconds = 
             LW_MAX(lifetime / 2, 2 * DEFAULT_THREAD_WAITSECS);
     }
 
     if (dwThreadWaitSecs)
     {
-        gAdMachinePasswordSyncState.dwThreadWaitSecs = dwThreadWaitSecs;
+        pMachinePwdState->dwThreadWaitSecs = dwThreadWaitSecs;
     }
     else
     {
-        gAdMachinePasswordSyncState.dwThreadWaitSecs = DEFAULT_THREAD_WAITSECS;
+        pMachinePwdState->dwThreadWaitSecs = DEFAULT_THREAD_WAITSECS;
     }
 
-    LEAVE_AD_GLOBAL_DATA_RW_WRITER_LOCK(bInLock);
+    status = pthread_rwlock_unlock(pMachinePwdState->pDataLock);
+    LW_ASSERT(status == 0);
 }
 
 static
@@ -550,13 +690,13 @@ ADLogMachinePWUpdateSuccessEvent(
             PASSWORD_EVENT_CATEGORY,
             pszDescription,
             NULL);
-
+    
 cleanup:
 
     LW_SAFE_FREE_STRING(pszDescription);
 
     return;
-
+    
 error:
 
     goto cleanup;
@@ -578,24 +718,24 @@ ADLogMachinePWUpdateFailureEvent(
                  "     Authentication provider:   %s",
                  LSA_SAFE_LOG_STRING(gpszADProviderName));
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaGetErrorMessageForLoggingEvent(
                          dwErrCode,
                          &pszData);
-
+      
     LsaSrvLogServiceFailureEvent(
             LSASS_EVENT_FAILED_MACHINE_ACCOUNT_PASSWORD_UPDATE,
             PASSWORD_EVENT_CATEGORY,
             pszDescription,
             pszData);
-
+    
 cleanup:
 
     LW_SAFE_FREE_STRING(pszDescription);
     LW_SAFE_FREE_STRING(pszData);
 
     return;
-
+    
 error:
 
     goto cleanup;
@@ -622,13 +762,13 @@ ADLogMachineTGTRefreshSuccessEvent(
             KERBEROS_EVENT_CATEGORY,
             pszDescription,
             NULL);
-
+    
 cleanup:
 
     LW_SAFE_FREE_STRING(pszDescription);
 
     return;
-
+    
 error:
 
     goto cleanup;
@@ -650,24 +790,24 @@ ADLogMachineTGTRefreshFailureEvent(
                  "     Authentication provider:   %s",
                  LSA_SAFE_LOG_STRING(gpszADProviderName));
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaGetErrorMessageForLoggingEvent(
                          dwErrCode,
                          &pszData);
-
+      
     LsaSrvLogServiceFailureEvent(
             LSASS_EVENT_FAILED_MACHINE_ACCOUNT_TGT_REFRESH,
             KERBEROS_EVENT_CATEGORY,
             pszDescription,
             pszData);
-
+    
 cleanup:
 
     LW_SAFE_FREE_STRING(pszDescription);
     LW_SAFE_FREE_STRING(pszData);
 
     return;
-
+    
 error:
 
     goto cleanup;
