@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -52,6 +52,7 @@
 
 DWORD
 AD_OfflineGetGroupMembers(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN PCSTR pszGroupSid,
     OUT size_t* psMemberObjectsCount,
     OUT PLSA_SECURITY_OBJECT** pppMemberObjects
@@ -69,9 +70,9 @@ AD_OfflineGetGroupMembers(
     size_t sIndex = 0;
 
     dwError = ADCacheGetGroupMembers(
-        gpLsaAdProviderState->hCacheConnection,
+        pState->hCacheConnection,
         pszGroupSid,
-        AD_GetTrimUserMembershipEnabled(),
+        AD_GetTrimUserMembershipEnabled(pState),
         &sGroupMembershipsCount,
         &ppGroupMemberships);
     BAIL_ON_LSA_ERROR(dwError);
@@ -91,6 +92,7 @@ AD_OfflineGetGroupMembers(
     }
 
     dwError = AD_OfflineFindObjectsBySidList(
+        pState,
         sMemberSidsCount,
         ppszMemberSids,
         &ppObjects);
@@ -122,6 +124,7 @@ error:
 
 DWORD
 AD_OfflineFindObjectsBySidList(
+    IN PLSA_AD_PROVIDER_STATE pState,
     IN size_t sCount,
     IN PSTR* ppszSidList,
     OUT PLSA_SECURITY_OBJECT** pppObjects
@@ -130,12 +133,12 @@ AD_OfflineFindObjectsBySidList(
     DWORD dwError = LW_ERROR_SUCCESS;
     PLSA_SECURITY_OBJECT *ppObjects = NULL;
 
-    /*
+    /* 
      * Lookup users and groups from the cache.
      */
 
     dwError = ADCacheFindObjectsBySidList(
-                    gpLsaAdProviderState->hCacheConnection,
+                    pState->hCacheConnection,
                     sCount,
                     ppszSidList,
                     &ppObjects);
@@ -249,10 +252,10 @@ error:
 }
 
 struct _LSA_AD_GROUP_EXPANSION_DATA {
-    PLSA_HASH_TABLE pGroupsToExpand;
-    PLSA_HASH_TABLE pExpandedGroups;
-    PLSA_HASH_TABLE pUsers;
-    LSA_HASH_ITERATOR GroupsToExpandIterator;
+    PLW_HASH_TABLE pGroupsToExpand;
+    PLW_HASH_TABLE pExpandedGroups;
+    PLW_HASH_TABLE pUsers;
+    LW_HASH_ITERATOR GroupsToExpandIterator;
     BOOLEAN bIsIteratorInitialized;
     BOOLEAN bDiscardedDueToDepth;
     DWORD dwMaxDepth;
@@ -274,7 +277,7 @@ AD_GroupExpansionDataCreate(
                 (PVOID*) &pExpansionData);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
                 sNumberOfBuckets,
                 AD_CompareObjectSids,
                 AD_HashObjectSid,
@@ -283,7 +286,7 @@ AD_GroupExpansionDataCreate(
                 &pExpansionData->pGroupsToExpand);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
                 sNumberOfBuckets,
                 AD_CompareObjectSids,
                 AD_HashObjectSid,
@@ -292,7 +295,7 @@ AD_GroupExpansionDataCreate(
                 &pExpansionData->pExpandedGroups);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
                 sNumberOfBuckets,
                 AD_CompareObjectSids,
                 AD_HashObjectSid,
@@ -339,7 +342,7 @@ AD_GroupExpansionDataAddExpansionResults(
     if ((sMembersCount + pExpansionData->pUsers->sCount) * 2 >
             pExpansionData->pUsers->sTableSize)
     {
-        dwError = LsaHashResize(
+        dwError = LwHashResize(
                         pExpansionData->pUsers,
                         (sMembersCount +
                              pExpansionData->pUsers->sCount + 10) * 3);
@@ -349,7 +352,7 @@ AD_GroupExpansionDataAddExpansionResults(
     if ((sMembersCount + pExpansionData->pGroupsToExpand->sCount) * 2 >
             pExpansionData->pGroupsToExpand->sTableSize)
     {
-        dwError = LsaHashResize(
+        dwError = LwHashResize(
                         pExpansionData->pGroupsToExpand,
                         (sMembersCount +
                              pExpansionData->pGroupsToExpand->sCount + 10) * 3);
@@ -367,10 +370,10 @@ AD_GroupExpansionDataAddExpansionResults(
 
         if (pCurrentMember->type == LSA_OBJECT_TYPE_USER)
         {
-            if (!LsaHashExists(pExpansionData->pUsers,
+            if (!LwHashExists(pExpansionData->pUsers,
                                ppMembers[sMembersCount-1]))
             {
-                dwError = LsaHashSetValue(
+                dwError = LwHashSetValue(
                     pExpansionData->pUsers,
                     ppMembers[sMembersCount-1],
                     (PVOID)(size_t)dwExpandedGroupDepth);
@@ -389,16 +392,16 @@ AD_GroupExpansionDataAddExpansionResults(
                 pExpansionData->bDiscardedDueToDepth = TRUE;
                 ADCacheSafeFreeObject(&ppMembers[sMembersCount-1]);
             }
-            else if (LsaHashExists(pExpansionData->pExpandedGroups,
+            else if (LwHashExists(pExpansionData->pExpandedGroups,
                                    pCurrentMember) ||
-                     LsaHashExists(pExpansionData->pGroupsToExpand,
+                     LwHashExists(pExpansionData->pGroupsToExpand,
                                    pCurrentMember))
             {
                 ADCacheSafeFreeObject(&ppMembers[sMembersCount-1]);
             }
             else
             {
-                dwError = LsaHashSetValue(
+                dwError = LwHashSetValue(
                             pExpansionData->pGroupsToExpand,
                             ppMembers[sMembersCount-1],
                             (PVOID)(size_t)dwExpandedGroupDepth);
@@ -441,7 +444,7 @@ AD_GroupExpansionDataGetNextGroupToExpand(
     DWORD dwError = 0;
     PLSA_SECURITY_OBJECT pGroupToExpand = NULL;
     DWORD dwGroupToExpandDepth = 0;
-    const LSA_HASH_ENTRY* pHashEntry = NULL;
+    const LW_HASH_ENTRY* pHashEntry = NULL;
 
     dwError = pExpansionData->dwLastError;
     BAIL_ON_LSA_ERROR(dwError);
@@ -454,21 +457,21 @@ AD_GroupExpansionDataGetNextGroupToExpand(
 
     if (pExpansionData->bIsIteratorInitialized)
     {
-        pHashEntry = LsaHashNext(&pExpansionData->GroupsToExpandIterator);
+        pHashEntry = LwHashNext(&pExpansionData->GroupsToExpandIterator);
     }
 
     if (!pHashEntry)
     {
         // Either the iterator is not initialized or we
         // reached the end of the hash table and need to start over.
-        dwError = LsaHashGetIterator(
+        dwError = LwHashGetIterator(
                     pExpansionData->pGroupsToExpand,
                     &pExpansionData->GroupsToExpandIterator);
         BAIL_ON_LSA_ERROR(dwError);
 
         pExpansionData->bIsIteratorInitialized = TRUE;
 
-        pHashEntry = LsaHashNext(&pExpansionData->GroupsToExpandIterator);
+        pHashEntry = LwHashNext(&pExpansionData->GroupsToExpandIterator);
         if (!pHashEntry)
         {
             dwError = LW_ERROR_INTERNAL;
@@ -485,12 +488,12 @@ AD_GroupExpansionDataGetNextGroupToExpand(
     // the "to expand" list.  It does not hurt to track it in the
     // "expanded" list.
 
-    dwError = LsaHashSetValue(pExpansionData->pExpandedGroups,
+    dwError = LwHashSetValue(pExpansionData->pExpandedGroups,
                               pGroupToExpand,
                               (PVOID)(size_t)dwGroupToExpandDepth);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashRemoveKey(pExpansionData->pGroupsToExpand, pGroupToExpand);
+    dwError = LwHashRemoveKey(pExpansionData->pGroupsToExpand, pGroupToExpand);
     if (dwError)
     {
         LSA_LOG_DEBUG("ASSERT: cannot fail");
@@ -523,8 +526,8 @@ AD_GroupExpansionDataGetResults(
     )
 {
     DWORD dwError = 0;
-    LSA_HASH_ITERATOR hashIterator;
-    LSA_HASH_ENTRY* pHashEntry = NULL;
+    LW_HASH_ITERATOR hashIterator;
+    LW_HASH_ENTRY* pHashEntry = NULL;
     size_t sHashCount = 0;
     PLSA_SECURITY_OBJECT* ppUserMembers = NULL;
     size_t sUserMembersCount = 0;
@@ -540,16 +543,16 @@ AD_GroupExpansionDataGetResults(
                 (PVOID*)&ppUserMembers);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaHashGetIterator(pExpansionData->pUsers, &hashIterator);
+    dwError = LwHashGetIterator(pExpansionData->pUsers, &hashIterator);
     BAIL_ON_LSA_ERROR(dwError);
 
     for (sUserMembersCount = 0;
-         (pHashEntry = LsaHashNext(&hashIterator)) != NULL;
+         (pHashEntry = LwHashNext(&hashIterator)) != NULL;
          sUserMembersCount++)
     {
         PLSA_SECURITY_OBJECT pUser = (PLSA_SECURITY_OBJECT) pHashEntry->pKey;
 
-        dwError = LsaHashRemoveKey(pExpansionData->pUsers, pUser);
+        dwError = LwHashRemoveKey(pExpansionData->pUsers, pUser);
         BAIL_ON_LSA_ERROR(dwError);
 
         ppUserMembers[sUserMembersCount] = pUser;
@@ -608,9 +611,9 @@ AD_GroupExpansionDataDestroy(
         {
             pExpansionData->pUsers->fnFree = AD_FreeHashObject;
         }
-        LsaHashSafeFree(&pExpansionData->pGroupsToExpand);
-        LsaHashSafeFree(&pExpansionData->pExpandedGroups);
-        LsaHashSafeFree(&pExpansionData->pUsers);
+        LwHashSafeFree(&pExpansionData->pGroupsToExpand);
+        LwHashSafeFree(&pExpansionData->pExpandedGroups);
+        LwHashSafeFree(&pExpansionData->pUsers);
         LwFreeMemory(pExpansionData);
     }
 }
