@@ -52,15 +52,15 @@
 
 BOOLEAN
 AD_IsOffline(
-    VOID
+    PLSA_AD_PROVIDER_STATE pState
     )
 {
-    return LsaDmIsDomainOffline(NULL);
+    return LsaDmIsDomainOffline(pState->hDmState, NULL);
 }
 
 DWORD
 AD_OfflineAuthenticateUserPam(
-    HANDLE hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     LSA_AUTH_USER_PAM_PARAMS* pParams,
     PLSA_AUTH_USER_PAM_INFO* ppPamAuthInfo
     )
@@ -79,12 +79,13 @@ AD_OfflineAuthenticateUserPam(
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = AD_FindUserObjectByName(
-                hProvider,
+                pContext,
                 pParams->pszLoginName,
                 &pUserInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = AD_VerifyUserAccountCanLogin(
+                pContext,
                 pUserInfo);
     if (dwError == LW_ERROR_PASSWORD_EXPIRED)
     {
@@ -98,7 +99,7 @@ AD_OfflineAuthenticateUserPam(
     }
 
     dwError = ADCacheGetPasswordVerifier(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
                 pUserInfo->pszObjectSid,
                 &pVerifier
                 );
@@ -129,22 +130,24 @@ AD_OfflineAuthenticateUserPam(
         pUserInfo->pszSamAccountName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaUmAddUser(
-                  pUserInfo->userInfo.uid,
-                  pszNT4UserName,
-                  pParams->pszPassword,
-                  0);
-    BAIL_ON_LSA_ERROR(dwError);
+    if (pContext->pState->bIsDefault)
+    {
+        dwError = LsaUmAddUser(
+                      pUserInfo->userInfo.uid,
+                      pszNT4UserName,
+                      pParams->pszPassword,
+                      0);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     pPamAuthInfo->bOnlineLogon = FALSE;
 
     *ppPamAuthInfo = pPamAuthInfo;
 
 cleanup:
-
     ADCacheSafeFreeObject(&pUserInfo);
     LSA_DB_SAFE_FREE_PASSWORD_VERIFIER(pVerifier);
-    LW_SAFE_FREE_STRING(pszEnteredPasswordVerifier);
+    LW_SECURE_FREE_STRING(pszEnteredPasswordVerifier);
     LW_SAFE_FREE_MEMORY(pbHash);
     LW_SAFE_FREE_STRING(pszNT4UserName);
 
@@ -152,6 +155,7 @@ cleanup:
 
 error:
     *ppPamAuthInfo = NULL;
+
     if (pPamAuthInfo)
     {
         LsaFreeAuthUserPamInfo(pPamAuthInfo);
@@ -162,7 +166,7 @@ error:
 
 DWORD
 AD_OfflineEnumUsers(
-    HANDLE  hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     HANDLE  hResume,
     DWORD   dwMaxNumUsers,
     PDWORD  pdwUsersFound,
@@ -172,88 +176,9 @@ AD_OfflineEnumUsers(
     return LW_ERROR_NO_MORE_USERS;
 }
 
-DWORD
-AD_OfflineGetUserGroupObjectMembership(
-    IN HANDLE hProvider,
-    IN PLSA_SECURITY_OBJECT pUserInfo,
-    OUT size_t* psNumGroupsFound,
-    OUT PLSA_SECURITY_OBJECT** pppResult
-    )
-{
-    DWORD dwError = LW_ERROR_SUCCESS;
-    size_t sUserGroupMembershipsCount = 0;
-    PLSA_GROUP_MEMBERSHIP* ppUserGroupMemberships = NULL;
-    size_t sParentSidsCount = 0;
-    // Do not free ppszParentSids at it points to ppUserGroupMemberships data.
-    PSTR* ppszParentSids = NULL;
-    size_t sGroupObjectsCount = 0;
-    PLSA_SECURITY_OBJECT* ppGroupObjects = NULL;
-
-    dwError = ADCacheGetGroupsForUser(
-                gpLsaAdProviderState->hCacheConnection,
-                pUserInfo->pszObjectSid,
-                AD_GetTrimUserMembershipEnabled(),
-                &sUserGroupMembershipsCount,
-                &ppUserGroupMemberships);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    //
-    // Gather up the SIDs.
-    //
-
-    dwError = AD_GatherSidsFromGroupMemberships(
-                TRUE,
-                NULL,
-                sUserGroupMembershipsCount,
-                ppUserGroupMemberships,
-                &sParentSidsCount,
-                &ppszParentSids);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    //
-    // Resolve the SIDs to objects.
-    //
-
-    dwError = AD_OfflineFindObjectsBySidList(
-                sParentSidsCount,
-                ppszParentSids,
-                &ppGroupObjects);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    //
-    // Filter out any objects that could not be resolved.
-    //
-
-    sGroupObjectsCount = sParentSidsCount;
-    AD_FilterNullEntries(ppGroupObjects, &sGroupObjectsCount);
-
-    *pppResult = ppGroupObjects;
-    *psNumGroupsFound = sGroupObjectsCount;
-
-cleanup:
-    ADCacheSafeFreeGroupMembershipList(sUserGroupMembershipsCount, &ppUserGroupMemberships);
-    LW_SAFE_FREE_MEMORY(ppszParentSids);
-
-    return dwError;
-
-error:
-
-    *pppResult = NULL;
-    *psNumGroupsFound = 0;
-
-    LSA_LOG_ERROR("Failed to find memberships for user '%s\\%s' (error = %u)",
-                  pUserInfo->pszNetbiosDomainName,
-                  pUserInfo->pszSamAccountName,
-                  dwError);
-
-    ADCacheSafeFreeObjectList(sGroupObjectsCount, &ppGroupObjects);
-
-    goto cleanup;
-}
-
 DWORD
 AD_OfflineEnumGroups(
-    HANDLE  hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     HANDLE  hResume,
     DWORD   dwMaxGroups,
     PDWORD  pdwGroupsFound,
@@ -265,7 +190,7 @@ AD_OfflineEnumGroups(
 
 DWORD
 AD_OfflineChangePassword(
-    HANDLE hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     PCSTR pszUserName,
     PCSTR pszPassword,
     PCSTR pszOldPassword
@@ -276,7 +201,7 @@ AD_OfflineChangePassword(
 
 DWORD
 AD_OfflineFindNSSArtefactByKey(
-    HANDLE hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     PCSTR  pszKeyName,
     PCSTR  pszMapName,
     DWORD  dwInfoLevel,
@@ -291,7 +216,7 @@ AD_OfflineFindNSSArtefactByKey(
 
 DWORD
 AD_OfflineEnumNSSArtefacts(
-    HANDLE  hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     HANDLE  hResume,
     DWORD   dwMaxNSSArtefacts,
     PDWORD  pdwNSSArtefactsFound,
@@ -304,19 +229,21 @@ AD_OfflineEnumNSSArtefacts(
 DWORD
 AD_OfflineInitializeOperatingMode(
     OUT PAD_PROVIDER_DATA* ppProviderData,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN PCSTR pszDomain,
     IN PCSTR pszHostName
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
+    PLSA_AD_PROVIDER_STATE pState = pContext->pState;
     PAD_PROVIDER_DATA pProviderData = NULL;
     PDLINKEDLIST pDomains = NULL;
     const DLINKEDLIST* pPos = NULL;
     const LSA_DM_ENUM_DOMAIN_INFO* pDomain = NULL;
 
     dwError = ADState_GetDomainTrustList(
-        gpLsaAdProviderState->hStateConnection,
-        &pDomains);
+                  pState->pszDomainName,
+                  &pDomains);
     BAIL_ON_LSA_ERROR(dwError);
 
     pPos = pDomains;
@@ -325,6 +252,7 @@ AD_OfflineInitializeOperatingMode(
         pDomain = (const LSA_DM_ENUM_DOMAIN_INFO*)pPos->pItem;
 
         dwError = LsaDmAddTrustedDomain(
+            pState->hDmState,
             pDomain->pszDnsDomainName,
             pDomain->pszNetbiosDomainName,
             pDomain->pSid,
@@ -348,8 +276,8 @@ AD_OfflineInitializeOperatingMode(
     }
 
     dwError = ADState_GetProviderData(
-                gpLsaAdProviderState->hStateConnection,
-                &pProviderData);
+                  pState->pszDomainName,
+                  &pProviderData);
     BAIL_ON_LSA_ERROR(dwError);
 
     *ppProviderData = pProviderData;
@@ -374,7 +302,7 @@ error:
 static
 DWORD
 AD_OfflineFindObjectsByName(
-    IN HANDLE hProvider,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN OPTIONAL LSA_OBJECT_TYPE ObjectType,
     IN LSA_QUERY_TYPE QueryType,
@@ -390,10 +318,6 @@ AD_OfflineFindObjectsByName(
     DWORD dwIndex = 0;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     LSA_QUERY_TYPE type = LSA_QUERY_TYPE_UNDEFINED;
-    PSTR pszDefaultPrefix = NULL;
-
-    dwError = AD_GetUserDomainPrefix(&pszDefaultPrefix);
-    BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwAllocateMemory(sizeof(*ppObjects) * dwCount, OUT_PPVOID(&ppObjects));
     BAIL_ON_LSA_ERROR(dwError);
@@ -409,7 +333,7 @@ AD_OfflineFindObjectsByName(
             pszLoginId_copy,
             LsaSrvSpaceReplacement(),
             ' ');
-
+        
         dwError = LsaSrvCrackDomainQualifiedName(
             pszLoginId_copy,
             &pUserNameInfo);
@@ -438,26 +362,30 @@ AD_OfflineFindObjectsByName(
         {
         case LSA_OBJECT_TYPE_USER:
             dwError = ADCacheFindUserByName(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
+                pContext->pState,
                 pUserNameInfo,
                 &pCachedUser);
             break;
         case LSA_OBJECT_TYPE_GROUP:
             dwError = ADCacheFindGroupByName(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
+                pContext->pState,
                 pUserNameInfo,
                 &pCachedUser);
             break;
         default:
             dwError = ADCacheFindUserByName(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
+                pContext->pState,
                 pUserNameInfo,
                 &pCachedUser);
             if (dwError == LW_ERROR_NO_SUCH_USER  ||
                 dwError == LW_ERROR_NOT_HANDLED)
             {
                 dwError = ADCacheFindGroupByName(
-                    gpLsaAdProviderState->hCacheConnection,
+                    pContext->pState->hCacheConnection,
+                    pContext->pState,
                     pUserNameInfo,
                     &pCachedUser);
             }
@@ -475,63 +403,6 @@ AD_OfflineFindObjectsByName(
         case LW_ERROR_NO_SUCH_GROUP:
         case LW_ERROR_NO_SUCH_OBJECT:
             dwError = LW_ERROR_SUCCESS;
-
-            if (QueryType == LSA_QUERY_TYPE_BY_ALIAS &&
-                AD_ShouldAssumeDefaultDomain())
-            {
-                LW_SAFE_FREE_STRING(pszLoginId_copy);
-                LsaSrvFreeNameInfo(pUserNameInfo);
-                pUserNameInfo = NULL;
-
-                dwError = LwAllocateStringPrintf(
-                    &pszLoginId_copy,
-                    "%s%c%s",
-                    pszDefaultPrefix,
-                    LsaSrvDomainSeparator(),
-                    QueryList.ppszStrings[dwIndex]);
-                BAIL_ON_LSA_ERROR(dwError);
-
-                LwStrCharReplace(
-                    pszLoginId_copy,
-                    LsaSrvSpaceReplacement(),
-                    ' ');
-
-                dwError = LsaSrvCrackDomainQualifiedName(
-                    pszLoginId_copy,
-                    &pUserNameInfo);
-                BAIL_ON_LSA_ERROR(dwError);
-
-                if (ObjectType != LSA_OBJECT_TYPE_GROUP)
-                {
-                    dwError = ADCacheFindUserByName(
-                        gpLsaAdProviderState->hCacheConnection,
-                        pUserNameInfo,
-                        &pCachedUser);
-                }
-                if (ObjectType == LSA_OBJECT_TYPE_GROUP ||
-                    (dwError == LW_ERROR_NO_SUCH_USER  ||
-                        dwError == LW_ERROR_NOT_HANDLED))
-                {
-                    dwError = ADCacheFindGroupByName(
-                        gpLsaAdProviderState->hCacheConnection,
-                        pUserNameInfo,
-                        &pCachedUser);
-                }
-                switch (dwError)
-                {
-                case LW_ERROR_SUCCESS:
-                    break;
-                case LW_ERROR_NOT_HANDLED:
-                case LW_ERROR_NO_SUCH_USER:
-                case LW_ERROR_NO_SUCH_GROUP:
-                case LW_ERROR_NO_SUCH_OBJECT:
-                    dwError = LW_ERROR_SUCCESS;
-                    break;
-                default:
-                    BAIL_ON_LSA_ERROR(dwError);
-                }
-                ppObjects[dwIndex] = pCachedUser;
-            }
             break;
         default:
             BAIL_ON_LSA_ERROR(dwError);
@@ -546,7 +417,6 @@ AD_OfflineFindObjectsByName(
 
 cleanup:
 
-    LW_SAFE_FREE_STRING(pszDefaultPrefix);
     LW_SAFE_FREE_STRING(pszLoginId_copy);
 
     if (pUserNameInfo)
@@ -569,7 +439,7 @@ error:
 static
 DWORD
 AD_OfflineFindObjectsById(
-    IN HANDLE hProvider,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN OPTIONAL LSA_OBJECT_TYPE ObjectType,
     IN LSA_QUERY_TYPE QueryType,
@@ -592,13 +462,13 @@ AD_OfflineFindObjectsById(
         {
         case LSA_OBJECT_TYPE_USER:
             dwError = ADCacheFindUserById(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
                 QueryList.pdwIds[dwIndex],
                 &pCachedUser);
             break;
         case LSA_OBJECT_TYPE_GROUP:
             dwError = ADCacheFindGroupById(
-                gpLsaAdProviderState->hCacheConnection,
+                pContext->pState->hCacheConnection,
                 QueryList.pdwIds[dwIndex],
                 &pCachedUser);
             break;
@@ -642,7 +512,7 @@ error:
 
 DWORD
 AD_OfflineFindObjects(
-    IN HANDLE hProvider,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN OPTIONAL LSA_OBJECT_TYPE ObjectType,
     IN LSA_QUERY_TYPE QueryType,
@@ -660,7 +530,7 @@ AD_OfflineFindObjects(
     {
     case LSA_QUERY_TYPE_BY_SID:
         dwError = ADCacheFindObjectsBySidList(
-                    gpLsaAdProviderState->hCacheConnection,
+                    pContext->pState->hCacheConnection,
                     dwCount,
                     (PSTR*) QueryList.ppszStrings,
                     &ppObjects);
@@ -668,7 +538,7 @@ AD_OfflineFindObjects(
         break;
     case LSA_QUERY_TYPE_BY_DN:
         dwError = ADCacheFindObjectsByDNList(
-            gpLsaAdProviderState->hCacheConnection,
+            pContext->pState->hCacheConnection,
             dwCount,
             (PSTR*) QueryList.ppszStrings,
             &ppObjects);
@@ -678,7 +548,7 @@ AD_OfflineFindObjects(
     case LSA_QUERY_TYPE_BY_UPN:
     case LSA_QUERY_TYPE_BY_ALIAS:
         dwError = AD_OfflineFindObjectsByName(
-            hProvider,
+            pContext,
             FindFlags,
             ObjectType,
             QueryType,
@@ -689,7 +559,7 @@ AD_OfflineFindObjects(
         break;
     case LSA_QUERY_TYPE_BY_UNIX_ID:
         dwError = AD_OfflineFindObjectsById(
-            hProvider,
+            pContext,
             FindFlags,
             ObjectType,
             QueryType,
@@ -744,10 +614,10 @@ error:
 static
 DWORD
 AD_OfflineQueryMemberOfForSid(
-    IN HANDLE hProvider,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN PSTR pszSid,
-    IN OUT PLSA_HASH_TABLE pGroupHash
+    IN OUT PLW_HASH_TABLE pGroupHash
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
@@ -759,6 +629,7 @@ AD_OfflineQueryMemberOfForSid(
     DWORD dwIndex = 0;
 
     dwError = AD_OfflineFindObjectsBySidList(
+                  pContext->pState,
                   1,
                   &pszSid,
                   &ppUserObject);
@@ -771,9 +642,9 @@ AD_OfflineQueryMemberOfForSid(
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = ADCacheGetGroupsForUser(
-                    gpLsaAdProviderState->hCacheConnection,
+                    pContext->pState->hCacheConnection,
                     pszSid,
-                    AD_GetTrimUserMembershipEnabled(),
+                    AD_GetTrimUserMembershipEnabled(pContext->pState),
                     &sMembershipCount,
                     &ppMemberships);
     BAIL_ON_LSA_ERROR(dwError);
@@ -781,18 +652,18 @@ AD_OfflineQueryMemberOfForSid(
     for (dwIndex = 0; dwIndex < sMembershipCount; dwIndex++)
     {
         if (ppMemberships[dwIndex]->pszParentSid &&
-            !LsaHashExists(pGroupHash, ppMemberships[dwIndex]->pszParentSid))
+            !LwHashExists(pGroupHash, ppMemberships[dwIndex]->pszParentSid))
         {
             dwError = LwAllocateString(
                 ppMemberships[dwIndex]->pszParentSid,
                 &pszGroupSid);
             BAIL_ON_LSA_ERROR(dwError);
-
-            dwError = LsaHashSetValue(pGroupHash, pszGroupSid, pszGroupSid);
+            
+            dwError = LwHashSetValue(pGroupHash, pszGroupSid, pszGroupSid);
             BAIL_ON_LSA_ERROR(dwError);
 
             dwError = AD_OfflineQueryMemberOfForSid(
-                hProvider,
+                pContext,
                 FindFlags,
                 pszGroupSid,
                 pGroupHash);
@@ -817,7 +688,7 @@ error:
 static
 VOID
 AD_OfflineFreeMemberOfHashEntry(
-    const LSA_HASH_ENTRY* pEntry
+    const LW_HASH_ENTRY* pEntry
     )
 {
     if (pEntry->pValue)
@@ -828,7 +699,7 @@ AD_OfflineFreeMemberOfHashEntry(
 
 DWORD
 AD_OfflineQueryMemberOf(
-    IN HANDLE hProvider,
+    PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN DWORD dwSidCount,
     IN PSTR* ppszSids,
@@ -838,44 +709,44 @@ AD_OfflineQueryMemberOf(
 {
     DWORD dwError = 0;
     DWORD dwIndex = 0;
-    PLSA_HASH_TABLE   pGroupHash = NULL;
-    LSA_HASH_ITERATOR hashIterator = {0};
-    LSA_HASH_ENTRY*   pHashEntry = NULL;
+    PLW_HASH_TABLE   pGroupHash = NULL;
+    LW_HASH_ITERATOR hashIterator = {0};
+    LW_HASH_ENTRY*   pHashEntry = NULL;
     DWORD dwGroupSidCount = 0;
     PSTR* ppszGroupSids = NULL;
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
         13,
-        LsaHashCaselessStringCompare,
-        LsaHashCaselessStringHash,
+        LwHashCaselessStringCompare,
+        LwHashCaselessStringHash,
         AD_OfflineFreeMemberOfHashEntry,
         NULL,
         &pGroupHash);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     for (dwIndex = 0; dwIndex < dwSidCount; dwIndex++)
     {
         dwError = AD_OfflineQueryMemberOfForSid(
-            hProvider,
+            pContext,
             FindFlags,
             ppszSids[dwIndex],
             pGroupHash);
         BAIL_ON_LSA_ERROR(dwError);
     }
-
-    dwGroupSidCount = (DWORD) LsaHashGetKeyCount(pGroupHash);
-
+    
+    dwGroupSidCount = (DWORD) LwHashGetKeyCount(pGroupHash);
+    
     if (dwGroupSidCount)
     {
         dwError = LwAllocateMemory(
             sizeof(*ppszGroupSids) * dwGroupSidCount,
             OUT_PPVOID(&ppszGroupSids));
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LsaHashGetIterator(pGroupHash, &hashIterator);
+    
+        dwError = LwHashGetIterator(pGroupHash, &hashIterator);
         BAIL_ON_LSA_ERROR(dwError);
-
-        for(dwIndex = 0; (pHashEntry = LsaHashNext(&hashIterator)) != NULL; dwIndex++)
+        
+        for(dwIndex = 0; (pHashEntry = LwHashNext(&hashIterator)) != NULL; dwIndex++)
         {
             ppszGroupSids[dwIndex] = (PSTR) pHashEntry->pValue;
             pHashEntry->pValue = NULL;
@@ -887,7 +758,7 @@ AD_OfflineQueryMemberOf(
 
 cleanup:
 
-    LsaHashSafeFree(&pGroupHash);
+    LwHashSafeFree(&pGroupHash);
 
     return dwError;
 
@@ -903,7 +774,7 @@ error:
 
 DWORD
 AD_OfflineGetGroupMemberSids(
-    IN HANDLE hProvider,
+    IN PAD_PROVIDER_CONTEXT pContext,
     IN LSA_FIND_FLAGS FindFlags,
     IN PCSTR pszSid,
     OUT PDWORD pdwSidCount,
@@ -919,9 +790,9 @@ AD_OfflineGetGroupMemberSids(
     DWORD dwIndex = 0;
 
     dwError = ADCacheGetGroupMembers(
-                    gpLsaAdProviderState->hCacheConnection,
+                    pContext->pState->hCacheConnection,
                     pszSid,
-                    AD_GetTrimUserMembershipEnabled(),
+                    AD_GetTrimUserMembershipEnabled(pContext->pState),
                     &sMembershipCount,
                     &ppMemberships);
     BAIL_ON_LSA_ERROR(dwError);
@@ -931,7 +802,7 @@ AD_OfflineGetGroupMemberSids(
         dwError = LwAllocateMemory(
             sizeof(*ppszSids) * sMembershipCount,
             OUT_PPVOID(&ppszSids));
-
+        
         for (dwIndex = 0; dwIndex < sMembershipCount; dwIndex++)
         {
             if (ppMemberships[dwIndex]->pszChildSid)
@@ -948,7 +819,7 @@ AD_OfflineGetGroupMemberSids(
 cleanup:
 
     ADCacheSafeFreeGroupMembershipList(sMembershipCount, &ppMemberships);
-
+    
     return dwError;
 
 error:
