@@ -101,6 +101,7 @@ static
 DWORD
 LsaDbOpen(
     IN PCSTR pszDbPath,
+    IN PLSA_AD_PROVIDER_STATE pState,
     OUT PLSA_DB_HANDLE phDb
     )
 {
@@ -150,6 +151,8 @@ LsaDbOpen(
                     (PVOID*)&pConn);
     BAIL_ON_LSA_ERROR(dwError);
 
+    pConn->pProviderState = pState;
+
     dwError = pthread_rwlock_init(&pConn->lock, NULL);
     BAIL_ON_LSA_ERROR(dwError);
     bLockCreated = TRUE;
@@ -664,6 +667,7 @@ LsaDbFindUserByName(
     {
        case NameType_UPN:
             dwError = LsaDmQueryDomainInfo(
+                            pConn->pProviderState->hDmState,
                             pUserNameInfo->pszDomain,
                             &pszDnsDomain,
                             NULL,
@@ -1364,6 +1368,10 @@ LsaDbUnpackGroupInfo(
         piColumnPos,
         "Gid",
         (DWORD*)&pResult->groupInfo.gid);
+    if (dwError == LW_ERROR_INVALID_PARAMETER)
+    {
+        dwError = LW_ERROR_DATA_ERROR;
+    }
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LsaSqliteReadString(
@@ -2546,7 +2554,7 @@ LsaDbQueryObjectMulti(
     }
     else
     {
-        iColumnPos += 21; // This is the number of fields in the userInfo section of lsadb.h (LSA_SECURITY_OBJECT)
+        iColumnPos += 21; // Number of fields after ObjectSid in LSA_DB_TABLE_NAME_USERS in sqlcache_create.h
     }
 
     if (pObject->type == LSA_OBJECT_TYPE_GROUP && pObject->enabled)
@@ -2806,7 +2814,7 @@ LsaDbQueryObject(
     }
     else
     {
-        iColumnPos += 21; // This is the number of fields in the userInfo section of lsadb.h (LSA_SECURITY_OBJECT)
+        iColumnPos += 21; // Number of fields after ObjectSid in LSA_DB_TABLE_NAME_USERS in sqlcache_create.h
     }
 
     if (pObject->type == LSA_OBJECT_TYPE_GROUP && pObject->enabled)
@@ -2814,6 +2822,14 @@ LsaDbQueryObject(
         dwError = LsaDbUnpackGroupInfo(pstQuery,
                 &iColumnPos,
                 pObject);
+        if (dwError == LW_ERROR_DATA_ERROR)
+        {
+            LSA_LOG_ERROR("The group attributes in the cache data for '%s\\%s' are invalid. The cache database or group data in Active Directory could be corrupt.",
+                LSA_SAFE_LOG_STRING(pObject->pszNetbiosDomainName),
+                LSA_SAFE_LOG_STRING(pObject->pszSamAccountName));
+            // Pretend like the whole object is not in the database.
+            dwError = LW_ERROR_NOT_HANDLED;
+        }
         BAIL_ON_LSA_ERROR(dwError);
     }
 
