@@ -50,7 +50,6 @@
 #include <lsasystem.h>
 #include <lsadef.h>
 #include <lsa/lsa.h>
-#include <lsa/lsa2.h>
 #include <reg/reg.h>
 
 #include <lwio/lwio.h>
@@ -69,6 +68,7 @@
 #include "lwtime.h"
 #include "lwsecurityidentifier.h"
 #include "lwsid.h"
+#include <lwhash.h>
 
 #include "lsautils.h"
 #include "lsasrvutils.h"
