@@ -93,7 +93,6 @@ LocalAuthenticateUserExInternal(
     PSTR     pszAccountName = NULL;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     PLSA_AUTH_USER_INFO pUserInfo = NULL;
-    BOOLEAN bUsingNTLMv2 = FALSE;
     BOOLEAN bAcceptNTLMv1 = TRUE;
     PLSA_DATA_BLOB pSessionKey = NULL;
     LSA_QUERY_LIST QueryList;
@@ -105,30 +104,14 @@ LocalAuthenticateUserExInternal(
     dwError = LocalCheckForQueryAccess(hProvider);
     BAIL_ON_LSA_ERROR(dwError);
 
-    /* Assume the local domain (localhost) if we don't have one */
-
-    if (pUserParams->pszDomain)
-    {
-        pszDomain = pUserParams->pszDomain;
-    }
-    else
-    {
-        LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
-        pszDomain = gLPGlobals.pszLocalDomain;
-    }
-
-    /* Allow the next provider to continue if we don't handle this domain */
-
-    if (!LocalServicesDomain(pszDomain))
-    {
-        dwError = LW_ERROR_NOT_HANDLED;
-        BAIL_ON_LSA_ERROR(dwError);
-    }
+    /* Always assume authenticating a user from the local domain */
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+    pszDomain = gLPGlobals.pszLocalDomain;
 
     dwError = LwAllocateStringPrintf(&pszAccountName,
-                                      "%s\\%s",
-                                      pszDomain,
-                                      pUserParams->pszAccountName);
+                                     "%s\\%s",
+                                     pszDomain,
+                                     pUserParams->pszAccountName);
     BAIL_ON_LSA_ERROR(dwError);
 
     QueryList.ppszStrings = (PCSTR*) &pszAccountName;
@@ -168,8 +151,6 @@ LocalAuthenticateUserExInternal(
 
     if (LsaDataBlobLength(pUserParams->pass.chap.pNT_resp) == 24)
     {
-        bUsingNTLMv2 = FALSE;
-
         if (!bAcceptNTLMv1)
         {
             dwError = ERROR_INVALID_LOGON_TYPE;
@@ -183,8 +164,6 @@ LocalAuthenticateUserExInternal(
     }
     else
     {
-        bUsingNTLMv2 = TRUE;
-
         dwError = AuthenticateNTLMv2(pUserParams,
                                      ppObjects[0],
                                      &pSessionKey);
@@ -201,10 +180,10 @@ LocalAuthenticateUserExInternal(
     BAIL_ON_LSA_ERROR(dwError);
 
     pUserInfo->pSessionKey = pSessionKey;
-    pSessionKey = NULL;
+    pSessionKey = NULL;    
 
     *ppUserInfo = pUserInfo;
-    pUserInfo = NULL;
+    pUserInfo = NULL;    
 
 cleanup:
     LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
@@ -215,7 +194,7 @@ cleanup:
 
     if (pSessionKey) {
         LsaDataBlobFree(&pSessionKey);
-    }
+    }    
 
     LW_SAFE_FREE_MEMORY(pszAccountName);
 
@@ -229,7 +208,8 @@ error:
 /********************************************************
  *******************************************************/
 
-static DWORD
+static
+DWORD
 AuthenticateNTLMv1(
     IN PLSA_AUTH_USER_PARAMS pUserParams,
     IN PLSA_SECURITY_OBJECT pObject,
@@ -242,7 +222,7 @@ AuthenticateNTLMv1(
     PBYTE    pChal = NULL;
     PBYTE    pNTresp = NULL;
     PBYTE    pSessKeyBuf = NULL;
-    PLSA_DATA_BLOB pSessKeyBlob = NULL;
+    PLSA_DATA_BLOB pSessKeyBlob = NULL;    
 
     /* Authenticate */
 
@@ -272,30 +252,30 @@ AuthenticateNTLMv1(
     /* Calculate the session Key */
 
     dwError = LsaDataBlobAllocate(&pSessKeyBlob, 16);
-    BAIL_ON_LSA_ERROR(dwError);
+    BAIL_ON_LSA_ERROR(dwError);    
 
     pSessKeyBuf = LsaDataBlobBuffer(pSessKeyBlob);
 
     MD4(pObject->userInfo.pNtHash, 16, pSessKeyBuf);
 
     *ppSessionKeyBlob = pSessKeyBlob;
-    pSessKeyBlob = NULL;
+    pSessKeyBlob = NULL;    
 
     /* Done */
 
-    dwError = LW_ERROR_SUCCESS;
+    dwError = LW_ERROR_SUCCESS;    
 
 cleanup:
 
     if (pSessKeyBlob) {
         LsaDataBlobFree(&pSessKeyBlob);
-    }
+    }    
 
     return dwError;
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 
@@ -314,7 +294,7 @@ AuthenticateNTLMv2(
     PWSTR pwszAccountName = NULL;
     PWSTR pwszDestination = NULL;
     BYTE pNTLMv2Hash[16];
-    DWORD dwNTLMv2HashLen = 16;
+    DWORD dwNTLMv2HashLen = 16;    
     PBYTE pBuffer = NULL;
     DWORD dwBufferLen = 0;
     DWORD dwAcctNameSize = 0;
@@ -327,9 +307,9 @@ AuthenticateNTLMv2(
     PBYTE pData = NULL;
     BYTE pNTLMv2Resp[EVP_MAX_MD_SIZE];
     DWORD dwNTLMv2RespLen = 0;
-    PLSA_DATA_BLOB pSessKeyBlob = NULL;
+    PLSA_DATA_BLOB pSessKeyBlob = NULL;    
     PBYTE pSessKeyBuf = NULL;
-    DWORD dwSessKeyLen = 0;
+    DWORD dwSessKeyLen = 0;    
 
     /* Sanity Check */
 
@@ -345,7 +325,7 @@ AuthenticateNTLMv2(
     dwNTrespLen = LsaDataBlobLength(pUserParams->pass.chap.pNT_resp);
 
     memset(pNTLMv2Hash, 0, 16);
-
+    
     /* First calculate the NTLMv2 Hash */
 
     dwError = LwMbsToWc16s(pUserParams->pszAccountName,
@@ -365,7 +345,7 @@ AuthenticateNTLMv2(
     dwAcctNameSize = RtlWC16StringNumChars(pwszAccountName) * sizeof(WCHAR);
     dwDestNameSize = RtlWC16StringNumChars(pwszDestination) * sizeof(WCHAR);
 
-    dwBufferLen = dwAcctNameSize + dwDestNameSize;
+    dwBufferLen = dwAcctNameSize + dwDestNameSize;    
     dwError = LwAllocateMemory(dwBufferLen, (PVOID*)&pBuffer);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -375,13 +355,13 @@ AuthenticateNTLMv2(
     HMAC(EVP_md5(),
          pObject->userInfo.pNtHash, 16,
          pBuffer, dwBufferLen,
-         pNTLMv2Hash, &dwNTLMv2HashLen);
+         pNTLMv2Hash, &dwNTLMv2HashLen);    
 
     /* grab the client nonce (starts at offset 16 following
        the HMAC) */
 
     pClientNonce = pNTresp + 16;
-    dwClientNonceLen = dwNTrespLen - 16;
+    dwClientNonceLen = dwNTrespLen - 16;    
 
     /* generate the Response for comparing the MAC */
 
@@ -397,7 +377,7 @@ AuthenticateNTLMv2(
          pData, dwClientNonceLen + 8,
          pNTLMv2Resp, &dwNTLMv2RespLen);
 
-    if (memcmp(pNTLMv2Resp, pNTresp, 16) != 0) {
+    if (memcmp(pNTLMv2Resp, pNTresp, 16) != 0) {        
         dwError = LW_ERROR_PASSWORD_MISMATCH;
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -405,7 +385,7 @@ AuthenticateNTLMv2(
     /* Calculate the session Key */
 
     dwError = LsaDataBlobAllocate(&pSessKeyBlob, 16);
-    BAIL_ON_LSA_ERROR(dwError);
+    BAIL_ON_LSA_ERROR(dwError);    
 
     pSessKeyBuf = LsaDataBlobBuffer(pSessKeyBlob);
 
@@ -419,8 +399,8 @@ AuthenticateNTLMv2(
 
     /* Done */
 
-    dwError = LW_ERROR_SUCCESS;
-
+    dwError = LW_ERROR_SUCCESS;    
+    
 
 cleanup:
 
@@ -431,13 +411,13 @@ cleanup:
 
     if (pSessKeyBlob) {
         LsaDataBlobFree(&pSessKeyBlob);
-    }
-
+    }    
+    
     return dwError;
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 
@@ -450,10 +430,10 @@ SidSplitString(
     OUT PDWORD pdwRid
     )
 {
-    DWORD dwError = LW_ERROR_INTERNAL;
+    DWORD dwError = LW_ERROR_INTERNAL;    
     PSTR p = NULL;
-    PSTR q = NULL;
-    DWORD dwRid = 0;
+    PSTR q = NULL;    
+    DWORD dwRid = 0;    
 
     BAIL_ON_INVALID_POINTER(pszSidString);
 
@@ -471,7 +451,7 @@ SidSplitString(
         dwError = LW_ERROR_INVALID_SID;
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     /* Split the string now that we know the RID is valid */
 
     *pdwRid = dwRid;
@@ -479,15 +459,15 @@ SidSplitString(
     p--;
     *p = '\0';
 
-    dwError = LW_ERROR_SUCCESS;
+    dwError = LW_ERROR_SUCCESS;    
 
 cleanup:
 
-    return dwError;
+    return dwError;    
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 /********************************************************
@@ -521,9 +501,9 @@ FillAuthUserInfo(
     pAuthInfo->dwAcctFlags = ACB_NORMAL;
     if (pObject->userInfo.bAccountDisabled) {
         pAuthInfo->dwAcctFlags |= ACB_DISABLED;
-    }
+    }    
     if (pObject->userInfo.bAccountExpired) {
-        pAuthInfo->dwAcctFlags |= ACB_PW_EXPIRED;
+        pAuthInfo->dwAcctFlags |= ACB_PW_EXPIRED;        
     }
     if (pObject->userInfo.bPasswordNeverExpires) {
         pAuthInfo->dwAcctFlags |= ACB_PWNOEXP;
@@ -532,19 +512,19 @@ FillAuthUserInfo(
     /* Copy strings */
 
     dwError = LwStrDupOrNull(pObject->userInfo.pszUnixName,
-                              &pAuthInfo->pszAccount);
+                              &pAuthInfo->pszAccount);    
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwStrDupOrNull(pObject->userInfo.pszGecos,
                               &pAuthInfo->pszFullName);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LwStrDupOrNull(pszMachineName,
-                              &pAuthInfo->pszDomain);
+                              &pAuthInfo->pszDomain);    
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LwStrDupOrNull(pObject->pszObjectSid,
-                              &pAuthInfo->pszDomainSid);
+                              &pAuthInfo->pszDomainSid);    
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = SidSplitString(pAuthInfo->pszDomainSid,
@@ -557,7 +537,7 @@ FillAuthUserInfo(
     dwError = SidSplitString(pObject->userInfo.pszPrimaryGroupSid,
                              &pAuthInfo->dwPrimaryGroupRid);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     /* Since all the groups we get back have full SIDs,
        place them in the SidAttributeList rather that
        filtering the ones in the host's own SAM domain
@@ -576,13 +556,13 @@ FillAuthUserInfo(
                                   &pAuthInfo->pSidAttribList[i].pszSid);
         BAIL_ON_LSA_ERROR(dwError);
 
-        pAuthInfo->pSidAttribList[i].dwAttrib =
+        pAuthInfo->pSidAttribList[i].dwAttrib = 
             LSA_SID_ATTR_GROUP_MANDATORY |
             LSA_SID_ATTR_GROUP_ENABLED_BY_DEFAULT |
-            LSA_SID_ATTR_GROUP_ENABLED;
+            LSA_SID_ATTR_GROUP_ENABLED;        
     }
-
-    dwError = LW_ERROR_SUCCESS;
+    
+    dwError = LW_ERROR_SUCCESS;    
 
 cleanup:
 
@@ -592,7 +572,7 @@ cleanup:
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 
