@@ -107,6 +107,7 @@ LocalCfgInitialize(
     pConfig->bCreateHomedir = LOCAL_CFG_DEFAULT_CREATE_HOMEDIR;
     pConfig->bAcceptNTLMv1 = LOCAL_CFG_DEFAULT_ACCEPT_NTLMV1;
     pConfig->dwHomedirUMask = LOCAL_CFG_DEFAULT_HOMEDIR_UMASK;
+    pConfig->EnableUnixIds = LOCAL_CFG_DEFAULT_ENABLE_UNIX_IDS;
 
     dwError = LwAllocateString(
                     pszDefaultSkelDirs,
@@ -193,86 +194,6 @@ LocalCfgGetMinPwdLength(
     return dwError;
 }
 
-DWORD
-LocalCfgGetMinUid(
-    PDWORD pdwMinUid
-    )
-{
-    DWORD   dwError = 0;
-    DWORD   dwMinUid = LOWEST_UID;
-    BOOLEAN bInLock = FALSE;
-
-    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    dwMinUid = gLPGlobals.dwMinUid;
-
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    *pdwMinUid = dwMinUid;
-
-    return dwError;
-}
-
-DWORD
-LocalCfgGetMinGid(
-    PDWORD pdwMinGid
-    )
-{
-    DWORD   dwError = 0;
-    DWORD   dwMinGid = LOWEST_GID;
-    BOOLEAN bInLock = FALSE;
-
-    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    dwMinGid = gLPGlobals.dwMinGid;
-
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    *pdwMinGid = dwMinGid;
-
-    return dwError;
-}
-
-DWORD
-LocalCfgGetMaxUid(
-    PDWORD pdwMaxUid
-    )
-{
-    DWORD   dwError = 0;
-    DWORD   dwMaxUid = HIGHEST_UID;
-    BOOLEAN bInLock = FALSE;
-
-    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    dwMaxUid = gLPGlobals.dwMaxUid;
-
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    *pdwMaxUid = dwMaxUid;
-
-    return dwError;
-}
-
-DWORD
-LocalCfgGetMaxGid(
-    PDWORD pdwMaxGid
-    )
-{
-    DWORD   dwError = 0;
-    DWORD   dwMaxGid = HIGHEST_GID;
-    BOOLEAN bInLock = FALSE;
-
-    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    dwMaxGid = gLPGlobals.dwMaxGid;
-
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
-
-    *pdwMaxGid = dwMaxGid;
-
-    return dwError;
-}
-
 DWORD
 LocalCfgGetPasswordChangeWarningTime(
     PLONG64 pllPasswdChangeWarningTime
@@ -514,6 +435,22 @@ error:
     goto cleanup;
 }
 
+DWORD
+LocalCfgGetEnableUnixIds(
+    PBOOLEAN pbResult
+    )
+{
+    BOOLEAN bInLock = FALSE;
+
+    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
+
+    *pbResult = gLPGlobals.cfg.EnableUnixIds;
+
+    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
+
+    return 0;
+}
+
 VOID
 LocalCfgFree(
     PLOCAL_CONFIG pConfig
@@ -618,6 +555,16 @@ LocalCfgReadRegistry(
             &StagingConfig.bAcceptNTLMv1,
             NULL
         },
+        {
+            "EnableUnixIds",
+            TRUE,
+            LsaTypeBoolean,
+            0,
+            MAXDWORD,
+            NULL,
+            &StagingConfig.EnableUnixIds,
+            NULL
+        },
     };
 
     LSA_CONFIG LsaConfigDescription[] =
