@@ -81,26 +81,6 @@ LocalCfgGetMinPwdLength(
     PDWORD pdwMinPwdLength
     );
 
-DWORD
-LocalCfgGetMinUid(
-    PDWORD pdwMinUid
-    );
-
-DWORD
-LocalCfgGetMinGid(
-    PDWORD pdwMinGid
-    );
-
-DWORD
-LocalCfgGetMaxUid(
-    PDWORD pdwMaxUid
-    );
-
-DWORD
-LocalCfgGetMaxGid(
-    PDWORD pdwMaxGid
-    );
-
 DWORD
 LocalCfgGetPasswordChangeWarningTime(
     PLONG64 pllPasswdChangeWarningTime
@@ -151,6 +131,11 @@ LocalCfgGetSkeletonDirs(
     PSTR* ppszSkelDirs
     );
 
+DWORD
+LocalCfgGetEnableUnixIds(
+    PBOOLEAN pbResult
+    );
+
 VOID
 LocalCfgFree(
     PLOCAL_CONFIG pConfig
