@@ -84,18 +84,9 @@ LocalSyncDomainInfo(
     DWORD dwNewLockoutThreshold = LOCAL_LOCKOUT_THRESHOLD;
     DWORD dwNewLockoutDuration = LOCAL_LOCKOUT_DURATION;
     DWORD dwNewLockoutWindow = LOCAL_LOCKOUT_WINDOW;
-    DWORD dwMinUid = LOWEST_UID;;
-    DWORD dwNewMinUid = LOWEST_UID;;
-    DWORD dwMaxUid = HIGHEST_UID;;
-    DWORD dwNewMaxUid = HIGHEST_UID;;
-    DWORD dwMinGid = LOWEST_GID;;
-    DWORD dwNewMinGid = LOWEST_GID;;
-    DWORD dwMaxGid = HIGHEST_GID;;
-    DWORD dwNewMaxGid = HIGHEST_GID;;
     HANDLE hDirectory = NULL;
     PSTR pszFilterFmt = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
     DWORD dwDomainObjectClass = LOCAL_OBJECT_CLASS_DOMAIN;
-    PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     WCHAR wszAttrDn[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrDomain[] = LOCAL_DIR_ATTR_DOMAIN;
@@ -205,46 +196,6 @@ LocalSyncDomainInfo(
             NULL,
             &dwNewLockoutWindow,
             NULL
-        },
-        {
-            "MaxUid",
-            TRUE,
-            LsaTypeDword,
-            0,
-            MAXDWORD,
-            NULL,
-            &dwNewMaxUid,
-            NULL
-        },
-        {
-            "MaxGid",
-            TRUE,
-            LsaTypeDword,
-            0,
-            MAXDWORD,
-            NULL,
-            &dwNewMaxGid,
-            NULL
-        },
-        {
-            "MinUid",
-            TRUE,
-            LsaTypeDword,
-            0,
-            MAXDWORD,
-            NULL,
-            &dwNewMinUid,
-            NULL
-        },
-        {
-            "MinGid",
-            TRUE,
-            LsaTypeDword,
-            0,
-            MAXDWORD,
-            NULL,
-            &dwNewMinGid,
-            NULL
         }
     };
 
@@ -387,17 +338,12 @@ LocalSyncDomainInfo(
                     ulMethod);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwAllocateStringPrintf(
-                    &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
                     pszFilterFmt,
                     dwDomainObjectClass);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-                    pszFilter,
-                    &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
                     hDirectory,
                     NULL,
@@ -428,14 +374,14 @@ LocalSyncDomainInfo(
                     DIRECTORY_ATTR_TYPE_UNICODE_STRING,
                     &pwszDomainName);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = DirectoryGetEntryAttrValueByName(
                     pEntry,
                     wszAttrNetBIOSName,
                     DIRECTORY_ATTR_TYPE_UNICODE_STRING,
                     &pwszNetBIOSName);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = DirectoryGetEntryAttrValueByName(
                     pEntry,
                     wszAttrObjectSID,
@@ -587,34 +533,6 @@ LocalSyncDomainInfo(
         llLockoutWindow = llNewLockoutWindow;
     }
 
-    /* MaxUid */
-    if (dwMaxUid != dwNewMaxUid)
-    {
-        // Currently not stored in domain database, so we'll just update the global for now
-        dwMaxUid = dwNewMaxUid;
-    }
-
-    /* MaxGid */
-    if (dwMaxGid != dwNewMaxGid)
-    {
-        // Currently not stored in domain database, so we'll just update the global for now
-        dwMaxGid = dwNewMaxGid;
-    }
-
-    /* MinUid */
-    if (dwMinUid != dwNewMinUid)
-    {
-        // Currently not stored in domain database, so we'll just update the global for now
-        dwMinUid = dwNewMinUid;
-    }
-
-    /* MinGid */
-    if (dwMinGid != dwNewMinGid)
-    {
-        // Currently not stored in domain database, so we'll just update the global for now
-        dwMinGid = dwNewMinGid;
-    }
-
     /*
      * Update domain object with current SAM configuration if
      * there was any change found
@@ -657,10 +575,6 @@ LocalSyncDomainInfo(
     pGlobals->dwLockoutThreshold   = dwLockoutThreshold;
     pGlobals->llLockoutDuration    = llLockoutDuration;
     pGlobals->llLockoutWindow      = llLockoutWindow;
-    pGlobals->dwMaxUid             = dwMaxUid;
-    pGlobals->dwMaxGid             = dwMaxGid;
-    pGlobals->dwMinUid             = dwMinUid;
-    pGlobals->dwMinGid             = dwMinGid;
 
 cleanup:
     if (hDirectory)
@@ -673,7 +587,6 @@ cleanup:
         DirectoryFreeEntries(pEntries, dwNumEntries);
     }
 
-    LW_SAFE_FREE_MEMORY(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (dwError == ERROR_SUCCESS &&
@@ -705,7 +618,6 @@ LocalGetDomainInfo(
     HANDLE hDirectory  = NULL;
     DWORD  objectClass = LOCAL_OBJECT_CLASS_DOMAIN;
     PCSTR  pszFilterTemplate = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
-    PSTR   pszFilter = NULL;
     PWSTR  pwszFilter = NULL;
     wchar16_t wszAttrNameDomain[]      = LOCAL_DIR_ATTR_DOMAIN;
     wchar16_t wszAttrNameNetBIOSName[] = LOCAL_DIR_ATTR_NETBIOS_NAME;
@@ -734,17 +646,12 @@ LocalGetDomainInfo(
     LONG64 llMaxPwdAge = 0;
     LONG64 llPwdChangeTime = 0;
 
-    dwError = LwAllocateStringPrintf(
-                    &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
                     pszFilterTemplate,
                     objectClass);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-                    pszFilter,
-                    &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectoryOpen(&hDirectory);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -858,9 +765,7 @@ cleanup:
         DirectoryClose(hDirectory);
     }
 
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_STRING(pszDomainSID);
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     return dwError;
@@ -921,6 +826,89 @@ error:
     goto cleanup;
 }
 
+
+DWORD
+LocalGetSequenceNumber(
+    IN HANDLE hProvider,
+    OUT PLONG64 pllSequenceNumber
+    )
+{
+    wchar_t wszDomainFilterFmt[] = L"%ws = %u";
+    DWORD dwError = ERROR_SUCCESS;
+    PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
+    DWORD dwFilterLen = 0;
+    PWSTR pwszFilter = NULL;
+    WCHAR wszAttrNameObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrNameSeqNumber[] = LOCAL_DIR_ATTR_SEQUENCE_NUMBER;
+    PDIRECTORY_ENTRY pEntry = NULL;
+    DWORD dwNumEntries = 0;
+    LONG64 llSequenceNumber = 0;
+
+    PWSTR wszAttrs[] = {
+        wszAttrNameSeqNumber,
+        NULL
+    };
+
+    dwFilterLen = ((sizeof(wszAttrNameSeqNumber)/sizeof(wszAttrNameSeqNumber[0])) +
+                   10 +
+                   (sizeof(wszDomainFilterFmt)/sizeof(wszDomainFilterFmt[0])));
+
+    dwError = LwAllocateMemory(sizeof(pwszFilter[0]) * dwFilterLen,
+                               OUT_PPVOID(&pwszFilter));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (sw16printfw(pwszFilter, dwFilterLen, wszDomainFilterFmt,
+                    wszAttrNameObjectClass, LOCAL_OBJECT_CLASS_DOMAIN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectorySearch(
+        pContext->hDirectory,
+        NULL,
+        0,
+        pwszFilter,
+        wszAttrs,
+        FALSE,
+        &pEntry,
+        &dwNumEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumEntries != 1)
+    {
+        dwError = LW_ERROR_SAM_DATABASE_ERROR;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                          pEntry,
+                          wszAttrNameSeqNumber,
+                          DIRECTORY_ATTR_TYPE_LARGE_INTEGER,
+                          &llSequenceNumber);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    *pllSequenceNumber = llSequenceNumber;
+
+cleanup:
+    if (pEntry)
+    {
+        DirectoryFreeEntries(pEntry, dwNumEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+
+    return dwError;
+
+error:
+    if (pllSequenceNumber)
+    {
+        *pllSequenceNumber = 0;
+    }
+
+    goto cleanup;
+}
+
 static
 DWORD
 LocalGetSingleLargeIntegerAttrValue(
@@ -957,7 +945,7 @@ LocalDirSetDomainName(
     IN PCSTR  pszNewDomainName
     )
 {
-    const wchar_t wszDomainFilterFmt[] = L"%ws=%d";
+    const wchar_t wszDomainFilterFmt[] = L"%ws=%u";
     const DWORD dwInt32StrSize = 10;
 
     DWORD dwError = 0;
@@ -1180,7 +1168,7 @@ LocalDirSetDomainSid(
     IN PCSTR  pszSid
     )
 {
-    const wchar_t wszDomainFilterFmt[] = L"%ws=%d";
+    const wchar_t wszDomainFilterFmt[] = L"%ws=%u";
     const DWORD dwInt32StrSize = 10;
 
     DWORD dwError = 0;
