@@ -73,6 +73,12 @@ LocalGetDomainInfo(
     PLONG64 pllPwdChangeTime
     );
 
+DWORD
+LocalGetSequenceNumber(
+    IN HANDLE hProvider,
+    OUT PLONG64 pllSequenceNumber
+    );
+
 DWORD
 LocalDirSetDomainName(
     IN PCSTR  pszNewName
