@@ -50,9 +50,9 @@
 
 #include "includes.h"
 
-PCSTR gpszLocalProviderName = LOCAL_CFG_TAG_LOCAL_PROVIDER;
+PCSTR gpszLocalProviderName = LSA_PROVIDER_TAG_LOCAL;
 
-LSA_PROVIDER_FUNCTION_TABLE_2 gLocalProviderAPITable2 =
+LSA_PROVIDER_FUNCTION_TABLE gLocalProviderAPITable =
 {
     .pfnFindObjects                  = LocalFindObjects,
     .pfnOpenEnumObjects              = LocalOpenEnumObjects,
