@@ -76,7 +76,7 @@ static
 DWORD
 LocalInitializeProvider(
     OUT PCSTR* ppszProviderName,
-    OUT PLSA_PROVIDER_FUNCTION_TABLE_2* ppFunctionTable
+    OUT PLSA_PROVIDER_FUNCTION_TABLE* ppFunctionTable
     )
 {
     DWORD dwError = 0;
@@ -123,7 +123,7 @@ LocalInitializeProvider(
     }
 
     *ppszProviderName = gpszLocalProviderName;
-    *ppFunctionTable = &gLocalProviderAPITable2;
+    *ppFunctionTable = &gLocalProviderAPITable;
 
 cleanup:
     if (dwError == ERROR_SUCCESS &&
@@ -154,6 +154,7 @@ error:
 DWORD
 LocalOpenHandle(
     HANDLE hServer,
+    PCSTR pszInstance,
     PHANDLE phProvider
     )
 {
@@ -220,8 +221,21 @@ LocalCloseHandle(
     }
 }
 
-BOOLEAN
+DWORD
 LocalServicesDomain(
+    PCSTR pszDomain,
+    BOOLEAN *pbServicesDomain
+    )
+{
+    DWORD dwError = 0;
+
+    *pbServicesDomain = LocalServicesDomainInternal(pszDomain);
+
+    return dwError;
+}
+
+BOOLEAN
+LocalServicesDomainInternal(
     PCSTR pszDomain
     )
 {
@@ -326,7 +340,7 @@ LocalAuthenticateUserPam(
                         NULL,
                         NULL);
         BAIL_ON_LSA_ERROR(dwUpdateError);
-
+        
     }
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -340,7 +354,7 @@ cleanup:
     LsaUtilFreeSecurityObject(pObject);
 
     LW_SAFE_FREE_MEMORY(pwszUserDN);
-    LW_SAFE_FREE_MEMORY(pwszPassword);
+    LW_SECURE_FREE_WSTRING(pwszPassword);
 
     if (dwUpdateError != ERROR_SUCCESS)
     {
@@ -407,7 +421,7 @@ LocalValidateUser(
         dwError = LW_ERROR_PASSWORD_EXPIRED;
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 cleanup:
 
     LsaUtilFreeSecurityObject(pObject);
@@ -445,7 +459,7 @@ LocalChangePassword(
     PWSTR pwszOldPassword = NULL;
     PWSTR pwszNewPassword = NULL;
     PLSA_SECURITY_OBJECT pObject = NULL;
-
+  
     BAIL_ON_INVALID_HANDLE(hProvider);
 
     dwError = LocalCheckForQueryAccess(hProvider);
@@ -495,8 +509,8 @@ cleanup:
 
     LsaUtilFreeSecurityObject(pObject);
 
-    LW_SAFE_FREE_MEMORY(pwszNewPassword);
-    LW_SAFE_FREE_MEMORY(pwszOldPassword);
+    LW_SECURE_FREE_WSTRING(pwszNewPassword);
+    LW_SECURE_FREE_WSTRING(pwszOldPassword);
     LW_SAFE_FREE_MEMORY(pwszUserDN);
 
     return dwError;
@@ -517,7 +531,7 @@ LocalSetPassword(
     PWSTR pwszUserDN = NULL;
     PWSTR pwszNewPassword = NULL;
     PLSA_SECURITY_OBJECT pObject = NULL;
-
+  
     BAIL_ON_INVALID_HANDLE(hProvider);
 
     dwError = LocalDirFindObjectByGenericName(
@@ -553,7 +567,7 @@ cleanup:
 
     LsaUtilFreeSecurityObject(pObject);
 
-    LW_SAFE_FREE_MEMORY(pwszNewPassword);
+    LW_SECURE_FREE_WSTRING(pwszNewPassword);
     LW_SAFE_FREE_MEMORY(pwszUserDN);
 
     return dwError;
@@ -644,12 +658,12 @@ LocalDeleteObject(
         dwError = LW_ERROR_NO_SUCH_USER;
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     dwError = LwMbsToWc16s(
         ppObjects[0]->pszDN,
         &pwszDN);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
 
     switch (ppObjects[0]->type)
     {
@@ -744,7 +758,7 @@ LocalOpenSession(
     PLSA_SECURITY_OBJECT pObject = NULL;
     DWORD dwLogonCount = 0;
     LONG64 llLastLogonTime = 0;
-
+  
     dwError = LocalCheckForQueryAccess(hProvider);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -757,6 +771,12 @@ LocalOpenSession(
         pszLoginId,
         &pObject);
     BAIL_ON_LSA_ERROR(dwError);
+    
+    if (pObject && !pObject->enabled)
+    {
+        dwError = ERROR_NO_SUCH_USER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     // Allow directory creation only if this is
     //
@@ -829,6 +849,12 @@ LocalCloseSession(
         &pObject);
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (pObject && !pObject->enabled)
+    {
+        dwError = ERROR_NO_SUCH_USER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LwGetNtTime((PULONG64)&llLastLogoffTime);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -1295,9 +1321,9 @@ LocalShutdownProvider(
 }
 
 DWORD
-LsaInitializeProvider2(
+LsaInitializeProvider(
     OUT PCSTR* ppszProviderName,
-    OUT PLSA_PROVIDER_FUNCTION_TABLE_2* ppFunctionTable
+    OUT PLSA_PROVIDER_FUNCTION_TABLE* ppFunctionTable
     )
 {
     return LocalInitializeProvider(ppszProviderName, ppFunctionTable);
