@@ -755,6 +755,7 @@ LocalMarshalEntryToSecurityObject(
     LONG64 llPasswordLastSet = 0;
     DWORD  dwUid = 0;
     DWORD  dwGid = 0;
+    BOOLEAN enableUnixIds = TRUE;
 
     dwError = LocalMarshalAttrToInteger(
                     pEntry,
@@ -768,11 +769,14 @@ LocalMarshalEntryToSecurityObject(
     BAIL_ON_LSA_ERROR(dwError);
 
     pObject->bIsLocal = TRUE;
-    pObject->enabled = TRUE;
+
+    dwError = LocalCfgGetEnableUnixIds(&enableUnixIds);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    pObject->enabled = enableUnixIds;
 
     switch (dwObjectClass)
     {
-    case LOCAL_OBJECT_CLASS_GROUP_MEMBER:
     case LOCAL_OBJECT_CLASS_USER:
         pObject->type = LSA_OBJECT_TYPE_USER;
 
@@ -781,23 +785,23 @@ LocalMarshalEntryToSecurityObject(
             wszAttrNameUID,
             &dwUid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         pObject->userInfo.uid = (uid_t) dwUid;
-
+        
         dwError = LocalMarshalAttrToInteger(
             pEntry,
             wszAttrNamePrimaryGroup,
             &dwGid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         pObject->userInfo.gid = (gid_t) dwGid;
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameSamAccountName,
             &pObject->pszSamAccountName);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNamePassword,
@@ -807,7 +811,7 @@ LocalMarshalEntryToSecurityObject(
             dwError = 0;
         }
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameGecos,
@@ -817,25 +821,25 @@ LocalMarshalEntryToSecurityObject(
             dwError = 0;
         }
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameShell,
             &pObject->userInfo.pszShell);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
                     pEntry,
                     wszAttrNameHomedir,
                     &pObject->userInfo.pszHomedir);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameObjectSID,
             &pObject->pszObjectSid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameUPN,
@@ -845,7 +849,7 @@ LocalMarshalEntryToSecurityObject(
             dwError = 0;
         }
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
 		    wszAttrNameDN,
@@ -857,19 +861,19 @@ LocalMarshalEntryToSecurityObject(
             wszAttrNameNetBIOSDomain,
             &pObject->pszNetbiosDomainName);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToInteger(
             pEntry,
             wszAttrNameAccountFlags,
             &dwAccountFlags);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToLargeInteger(
             pEntry,
             wszAttrNameAccountExpiry,
             &llAccountExpiry);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToLargeInteger(
             pEntry,
             wszAttrNamePasswdLastSet,
@@ -900,7 +904,7 @@ LocalMarshalEntryToSecurityObject(
                 pObject->pszNetbiosDomainName);
             BAIL_ON_LSA_ERROR(dwError);
             LwStrToUpper(pObject->userInfo.pszUPN + strlen(pObject->pszSamAccountName) + 1);
-
+            
             pObject->userInfo.bIsGeneratedUPN = TRUE;
         }
 
@@ -914,7 +918,7 @@ LocalMarshalEntryToSecurityObject(
             dwError = 0;
         }
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToOctetStream(
             pEntry,
             wszAttrNameLMHash,
@@ -934,27 +938,27 @@ LocalMarshalEntryToSecurityObject(
             wszAttrNameGID,
             &dwGid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         pObject->groupInfo.gid = dwGid;
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameSamAccountName,
             &pObject->pszSamAccountName);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
 		    wszAttrNameDN,
 		    &pObject->pszDN);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameObjectSID,
             &pObject->pszObjectSid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
             pEntry,
             wszAttrNameNetBIOSDomain,
