@@ -73,25 +73,19 @@ LocalFindObjectByName(
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
     PCSTR pszFilterTemplate =
-                    LOCAL_DB_DIR_ATTR_SAM_ACCOUNT_NAME " = \"%s\"" \
-                    " AND " LOCAL_DB_DIR_ATTR_DOMAIN   " = \"%s\"";
-    PSTR pszFilter = NULL;
+                    LOCAL_DB_DIR_ATTR_SAM_ACCOUNT_NAME " = %Q" \
+                    " AND " LOCAL_DB_DIR_ATTR_DOMAIN   " = %Q";
     PWSTR pwszFilter = NULL;
     PWSTR pwszObjectDN = NULL;
     DWORD dwObjectClass = LOCAL_OBJECT_CLASS_UNKNOWN;
 
-    dwError = LwAllocateStringPrintf(
-                    &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
                     pszFilterTemplate,
                     pszName,
                     pszDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-                    pszFilter,
-                    &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
                     pContext->hDirectory,
                     NULL,
@@ -137,7 +131,6 @@ LocalFindObjectByName(
 
 cleanup:
 
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (pEntries)
@@ -173,27 +166,21 @@ LocalDirResolveUserObjectPrimaryGroupSid(
         NULL
     };
     PCSTR pszTemplate = LOCAL_DB_DIR_ATTR_GID " = %u";
-    PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
-
+    
     if (pUserObject->type != LSA_OBJECT_TYPE_USER)
     {
         goto cleanup;
     }
 
-    dwError = LwAllocateStringPrintf(
-        &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+        &pwszFilter,
         pszTemplate,
         pUserObject->userInfo.gid);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-        pszFilter,
-        &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
         pContext->hDirectory,
         NULL,
@@ -218,8 +205,6 @@ LocalDirResolveUserObjectPrimaryGroupSid(
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
-
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (pEntry)
@@ -292,20 +277,23 @@ LocalDirFindObjectsInternal(
     PDIRECTORY_ENTRY pEntries = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
-    PCSTR pszFilterTemplateQualified =
-        LOCAL_DB_DIR_ATTR_NETBIOS_NAME " = \"%s\"" \
-        " AND " LOCAL_DB_DIR_ATTR_SAM_ACCOUNT_NAME " = \"%s\"%s";
-    PCSTR pszFilterTemplateString = "%s = \"%s\"%s";
+    PCSTR pszFilterTemplateQualified = 
+        LOCAL_DB_DIR_ATTR_NETBIOS_NAME " = %Q" \
+        " AND " LOCAL_DB_DIR_ATTR_SAM_ACCOUNT_NAME " = %Q%s";
+    PCSTR pszFilterTemplateString = "%s = %Q%s";
     PCSTR pszFilterTemplateDword = "%s = %u%s";
     PCSTR pszFilterTemplateType = " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
+    PCSTR pszFilterTemplateUserOrGroup = " AND (" \
+            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u OR " \
+            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u)";
     PCSTR pszFilterBy = NULL;
     PSTR pszFilterType = NULL;
-    PSTR  pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     DWORD dwObjectClass = LOCAL_OBJECT_CLASS_UNKNOWN;
     DWORD dwIndex = 0;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
     BOOLEAN bLocked = FALSE;
+    BOOLEAN bFoundInvalidObject = FALSE;
 
     /* FIXME: support generic queries */
     switch (ObjectType)
@@ -353,7 +341,16 @@ LocalDirFindObjectsInternal(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    if (dwObjectClass != LOCAL_OBJECT_CLASS_UNKNOWN)
+    if (dwObjectClass == LOCAL_OBJECT_CLASS_UNKNOWN)
+    {
+        dwError = LwAllocateStringPrintf(
+            &pszFilterType,
+            pszFilterTemplateUserOrGroup,
+            LOCAL_OBJECT_CLASS_USER,
+            LOCAL_OBJECT_CLASS_GROUP);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+    else
     {
         dwError = LwAllocateStringPrintf(
             &pszFilterType,
@@ -361,13 +358,11 @@ LocalDirFindObjectsInternal(
             dwObjectClass);
         BAIL_ON_LSA_ERROR(dwError);
     }
-    else
-    {
-        pszFilterType = NULL;
-    }
 
     for (dwIndex = 0; dwIndex < dwCount; dwIndex++)
     {
+        bFoundInvalidObject = FALSE;
+
         switch (QueryType)
         {
         case LSA_QUERY_TYPE_BY_ALIAS:
@@ -379,38 +374,40 @@ LocalDirFindObjectsInternal(
 
             if (!pLoginInfo->pszDomain)
             {
-                LOCAL_LOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
+                LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
 
                 dwError = LwAllocateString(
                                 gLPGlobals.pszNetBIOSName,
                                 &pLoginInfo->pszDomain);
                 BAIL_ON_LSA_ERROR(dwError);
 
-                LOCAL_UNLOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
+                LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
             }
 
-            dwError = LwAllocateStringPrintf(
-                &pszFilter,
+            dwError = DirectoryAllocateWC16StringFilterPrintf(
+                &pwszFilter,
                 pszFilterTemplateQualified,
                 pLoginInfo->pszDomain,
                 pLoginInfo->pszName,
                 pszFilterType ? pszFilterType : "");
             BAIL_ON_LSA_ERROR(dwError);
             break;
+
         case LSA_QUERY_TYPE_BY_DN:
         case LSA_QUERY_TYPE_BY_SID:
         case LSA_QUERY_TYPE_BY_UPN:
-            dwError = LwAllocateStringPrintf(
-                &pszFilter,
+            dwError = DirectoryAllocateWC16StringFilterPrintf(
+                &pwszFilter,
                 pszFilterTemplateString,
                 pszFilterBy,
                 QueryList.ppszStrings[dwIndex],
                 pszFilterType ? pszFilterType : "");
             BAIL_ON_LSA_ERROR(dwError);
             break;
+
         case LSA_QUERY_TYPE_BY_UNIX_ID:
-            dwError = LwAllocateStringPrintf(
-                &pszFilter,
+            dwError = DirectoryAllocateWC16StringFilterPrintf(
+                &pwszFilter,
                 pszFilterTemplateDword,
                 pszFilterBy,
                 QueryList.pdwIds[dwIndex],
@@ -421,12 +418,7 @@ LocalDirFindObjectsInternal(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
-
-        dwError = LwMbsToWc16s(
-            pszFilter,
-            &pwszFilter);
-        BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = DirectorySearch(
             pContext->hDirectory,
             NULL,
@@ -437,7 +429,7 @@ LocalDirFindObjectsInternal(
             &pEntries,
             &dwNumEntries);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (dwNumEntries > 1)
         {
             dwError = LW_ERROR_DATA_ERROR;
@@ -446,25 +438,39 @@ LocalDirFindObjectsInternal(
         else if (dwNumEntries == 1)
         {
             pEntry = &pEntries[0];
-
+            
             dwError = LocalMarshalEntryToSecurityObject(
                 pEntry,
                 &ppObjects[dwIndex]);
-            BAIL_ON_LSA_ERROR(dwError);
-
-            dwError = LocalDirResolveUserObjectPrimaryGroupSid(
-                hProvider,
-                ppObjects[dwIndex]);
-            BAIL_ON_LSA_ERROR(dwError);
+            if (dwError)
+            {
+                if (dwError == LW_ERROR_NO_SUCH_OBJECT)
+                {
+                    bFoundInvalidObject = TRUE;
+                    dwError = 0;
+                }
+                else
+                {
+                    BAIL_ON_LSA_ERROR(dwError);
+                }
+            }
+            else
+            {
+                dwError = LocalDirResolveUserObjectPrimaryGroupSid(
+                    hProvider,
+                    ppObjects[dwIndex]);
+                BAIL_ON_LSA_ERROR(dwError);
+            }
         }
-        else if (dwNumEntries == 0 && QueryType == LSA_QUERY_TYPE_BY_UPN)
+
+        if ((dwNumEntries == 0 || bFoundInvalidObject) && QueryType == LSA_QUERY_TYPE_BY_UPN)
         {
             /* UPN lookup might fail because the UPN is generated, so try
                again as an NT4 lookup */
             LSA_QUERY_LIST Single;
 
             Single.ppszStrings = &QueryList.ppszStrings[dwIndex];
-
+            
             dwError = LocalDirFindObjectsInternal(
                 hProvider,
                 FindFlags,
@@ -476,7 +482,6 @@ LocalDirFindObjectsInternal(
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        LW_SAFE_FREE_STRING(pszFilter);
         LW_SAFE_FREE_MEMORY(pwszFilter);
 
         if (pEntries)
@@ -493,9 +498,8 @@ LocalDirFindObjectsInternal(
     }
 
 cleanup:
-    LOCAL_UNLOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
     LW_SAFE_FREE_STRING(pszFilterType);
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (pEntries)
@@ -573,6 +577,8 @@ typedef struct _LOCAL_ENUM_HANDLE
     DWORD dwCount;
     PDIRECTORY_ENTRY pEntries;
     DWORD dwIndex;
+    LONG64 llSequenceNumber;
+
 } *LOCAL_ENUM_HANDLE, **PLOCAL_ENUM_HANDLE;
 
 DWORD
@@ -627,7 +633,6 @@ LocalDirOpenEnumObjects(
         NULL
     };
     PSTR pszTypeFilter = NULL;
-    PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     DWORD dwObjectClass = LOCAL_OBJECT_CLASS_UNKNOWN;
     LOCAL_ENUM_HANDLE hEnum = NULL;
@@ -675,24 +680,21 @@ LocalDirOpenEnumObjects(
 
     if (pszDomainName)
     {
-        dwError = LwAllocateStringPrintf(
-            &pszFilter,
-            LOCAL_DB_DIR_ATTR_NETBIOS_NAME " = \"%s\" AND (%s)",
+        dwError = DirectoryAllocateWC16StringFilterPrintf(
+            &pwszFilter,
+            LOCAL_DB_DIR_ATTR_NETBIOS_NAME " = %Q AND (%s)",
             pszDomainName,
             pszTypeFilter);
         BAIL_ON_LSA_ERROR(dwError);
     }
     else
     {
-        pszFilter = pszTypeFilter;
-        pszTypeFilter = NULL;
+        dwError = LwMbsToWc16s(
+            pszTypeFilter,
+            &pwszFilter);
+        BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwError = LwMbsToWc16s(
-        pszFilter,
-        &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
         pContext->hDirectory,
         NULL,
@@ -704,12 +706,15 @@ LocalDirOpenEnumObjects(
         &hEnum->dwCount);
     BAIL_ON_LSA_ERROR(dwError);
 
+    dwError = LocalGetSequenceNumber(
+        pContext,
+        &hEnum->llSequenceNumber);
+    BAIL_ON_LSA_ERROR(dwError);
+
     *phEnum = hEnum;
-
+        
 cleanup:
-
     LW_SAFE_FREE_STRING(pszTypeFilter);
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     return dwError;
@@ -734,6 +739,8 @@ LocalDirEnumObjects(
 {
     DWORD dwError = 0;
     LOCAL_ENUM_HANDLE pEnum = (LOCAL_ENUM_HANDLE) hEnum;
+    PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT) pEnum->hProvider;
+    LONG64 llSequenceNumber = 0;
     DWORD dwAllocCount = 0;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     DWORD dwIndex = 0;
@@ -745,6 +752,17 @@ LocalDirEnumObjects(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    dwError = LocalGetSequenceNumber(
+        pContext,
+        &llSequenceNumber);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (llSequenceNumber != pEnum->llSequenceNumber)
+    {
+        dwError = ERROR_INVALID_DATA;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwAllocCount = dwMaxObjectsCount;
     if (dwAllocCount > pEnum->dwCount - pEnum->dwIndex)
     {
@@ -832,7 +850,6 @@ LocalDirOpenEnumMembers(
         NULL
     };
 
-    PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     LOCAL_ENUM_HANDLE hEnum = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
@@ -845,17 +862,12 @@ LocalDirOpenEnumMembers(
     hEnum->hProvider = hProvider;
     hEnum->type = LOCAL_ENUM_HANDLE_MEMBERS;
 
-    dwError = LwAllocateStringPrintf(
-        &pszFilter,
-        LOCAL_DB_DIR_ATTR_OBJECT_SID " = \"%s\"",
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+        &pwszFilter,
+        LOCAL_DB_DIR_ATTR_OBJECT_SID " = %Q",
         pszSid);
     BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LwMbsToWc16s(
-        pszFilter,
-        &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = DirectorySearch(
         pContext->hDirectory,
         NULL,
@@ -893,13 +905,17 @@ LocalDirOpenEnumMembers(
             &hEnum->pEntries,
             &hEnum->dwCount);
         BAIL_ON_LSA_ERROR(dwError);
+
+        dwError = LocalGetSequenceNumber(
+            pContext,
+            &hEnum->llSequenceNumber);
+        BAIL_ON_LSA_ERROR(dwError);
     }
 
     *phEnum = hEnum;
-
+        
 cleanup:
 
-    LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
     LW_SAFE_FREE_MEMORY(pwszDN);
 
@@ -930,6 +946,8 @@ LocalDirEnumMembers(
 {
     DWORD dwError = 0;
     LOCAL_ENUM_HANDLE pEnum = (LOCAL_ENUM_HANDLE) hEnum;
+    PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT) pEnum->hProvider;
+    LONG64 llSequenceNumber = 0;
     DWORD dwAllocCount = 0;
     PSTR* ppszMemberSids = NULL;
     DWORD dwIndex = 0;
@@ -942,6 +960,17 @@ LocalDirEnumMembers(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    dwError = LocalGetSequenceNumber(
+        pContext,
+        &llSequenceNumber);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (llSequenceNumber != pEnum->llSequenceNumber)
+    {
+        dwError = ERROR_INVALID_DATA;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwAllocCount = dwMaxMemberSidCount;
     if (dwAllocCount > pEnum->dwCount - pEnum->dwIndex)
     {
@@ -988,7 +1017,7 @@ LocalDirQueryMemberOfInternal(
     IN HANDLE hProvider,
     IN LSA_FIND_FLAGS FindFlags,
     IN PSTR pszSid,
-    IN OUT PLSA_HASH_TABLE pGroupHash
+    IN OUT PLW_HASH_TABLE pGroupHash
     );
 
 static
@@ -997,7 +1026,7 @@ LocalDirQueryMemberOfDN(
     IN HANDLE hProvider,
     IN LSA_FIND_FLAGS FindFlags,
     IN PWSTR pwszDN,
-    IN OUT PLSA_HASH_TABLE pGroupHash
+    IN OUT PLW_HASH_TABLE pGroupHash
     )
 {
     DWORD dwError = 0;
@@ -1023,7 +1052,7 @@ LocalDirQueryMemberOfDN(
         &pEntries,
         &dwNumEntries);
     BAIL_ON_LSA_ERROR(dwError);
-
+            
     for (dwIndex = 0; dwIndex < dwNumEntries; dwIndex++)
     {
         dwError = LocalMarshalAttrToANSIFromUnicodeString(
@@ -1031,19 +1060,19 @@ LocalDirQueryMemberOfDN(
             wszAttrNameObjectSID,
             &pszGroupSid);
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LsaHashGetValue(
+        
+        dwError = LwHashGetValue(
             pGroupHash,
             pszGroupSid,
             OUT_PPVOID(&pszPreviousGroupSid));
         if (dwError == ERROR_NOT_FOUND)
         {
-            dwError = LsaHashSetValue(
+            dwError = LwHashSetValue(
                 pGroupHash,
                 pszGroupSid,
                 pszGroupSid);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             dwError = LocalDirQueryMemberOfInternal(
                 hProvider,
                 FindFlags,
@@ -1056,7 +1085,7 @@ LocalDirQueryMemberOfDN(
         {
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         LW_SAFE_FREE_MEMORY(pszGroupSid);
     }
 
@@ -1064,7 +1093,7 @@ cleanup:
 
     LW_SAFE_FREE_MEMORY(pwszFilter);
     LW_SAFE_FREE_MEMORY(pszFilter);
-
+    
     if (pEntries)
     {
         DirectoryFreeEntries(pEntries, dwNumEntries);
@@ -1083,7 +1112,7 @@ LocalDirQueryMemberOfInternal(
     IN HANDLE hProvider,
     IN LSA_FIND_FLAGS FindFlags,
     IN PSTR pszSid,
-    IN OUT PLSA_HASH_TABLE pGroupHash
+    IN OUT PLW_HASH_TABLE pGroupHash
     )
 {
     DWORD dwError = 0;
@@ -1096,21 +1125,15 @@ LocalDirQueryMemberOfInternal(
     };
     PDIRECTORY_ENTRY pEntries = NULL;
     DWORD dwNumEntries = 0;
-    PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     PWSTR pwszDN = NULL;
 
-    dwError = LwAllocateStringPrintf(
-        &pszFilter,
-        LOCAL_DB_DIR_ATTR_OBJECT_SID " = \"%s\"",
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+        &pwszFilter,
+        LOCAL_DB_DIR_ATTR_OBJECT_SID " = %Q",
         pszSid);
     BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LwMbsToWc16s(
-        pszFilter,
-            &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = DirectorySearch(
         pContext->hDirectory,
         NULL,
@@ -1121,7 +1144,7 @@ LocalDirQueryMemberOfInternal(
         &pEntries,
         &dwNumEntries);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     if (dwNumEntries > 1)
     {
         dwError = LW_ERROR_DATA_ERROR;
@@ -1134,7 +1157,7 @@ LocalDirQueryMemberOfInternal(
             wszAttrNameDN,
             &pwszDN);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LocalDirQueryMemberOfDN(
             hProvider,
             FindFlags,
@@ -1142,13 +1165,12 @@ LocalDirQueryMemberOfInternal(
             pGroupHash);
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 cleanup:
 
     LW_SAFE_FREE_MEMORY(pwszDN);
     LW_SAFE_FREE_MEMORY(pwszFilter);
-    LW_SAFE_FREE_MEMORY(pszFilter);
-
+    
     if (pEntries)
     {
         DirectoryFreeEntries(pEntries, dwNumEntries);
@@ -1181,16 +1203,16 @@ LocalDirQueryMemberOf(
 {
     DWORD dwError = 0;
     DWORD dwIndex = 0;
-    PLSA_HASH_TABLE   pGroupHash = NULL;
-    LSA_HASH_ITERATOR hashIterator = {0};
-    LSA_HASH_ENTRY*   pHashEntry = NULL;
+    PLW_HASH_TABLE   pGroupHash = NULL;
+    LW_HASH_ITERATOR hashIterator = {0};
+    LW_HASH_ENTRY*   pHashEntry = NULL;
     DWORD dwGroupSidCount = 0;
     PSTR* ppszGroupSids = NULL;
 
-    dwError = LsaHashCreate(
+    dwError = LwHashCreate(
                     13,
-                    LsaHashCaselessStringCompare,
-                    LsaHashCaselessStringHash,
+                    LwHashCaselessStringCompare,
+                    LwHashCaselessStringHash,
                     NULL,
                     NULL,
                     &pGroupHash);
@@ -1206,19 +1228,19 @@ LocalDirQueryMemberOf(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwGroupSidCount = (DWORD) LsaHashGetKeyCount(pGroupHash);
-
+    dwGroupSidCount = (DWORD) LwHashGetKeyCount(pGroupHash);
+    
     if (dwGroupSidCount)
     {
         dwError = LwAllocateMemory(
             sizeof(*ppszGroupSids) * dwGroupSidCount,
             OUT_PPVOID(&ppszGroupSids));
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LsaHashGetIterator(pGroupHash, &hashIterator);
+    
+        dwError = LwHashGetIterator(pGroupHash, &hashIterator);
         BAIL_ON_LSA_ERROR(dwError);
-
-        for(dwIndex = 0; (pHashEntry = LsaHashNext(&hashIterator)) != NULL; dwIndex++)
+        
+        for(dwIndex = 0; (pHashEntry = LwHashNext(&hashIterator)) != NULL; dwIndex++)
         {
             ppszGroupSids[dwIndex] = (PSTR) pHashEntry->pValue;
             pHashEntry->pValue = NULL;
@@ -1232,15 +1254,15 @@ cleanup:
 
     if (pGroupHash)
     {
-        if (LsaHashGetIterator(pGroupHash, &hashIterator) == 0)
+        if (LwHashGetIterator(pGroupHash, &hashIterator) == 0)
         {
-            while ((pHashEntry = LsaHashNext(&hashIterator)))
+            while ((pHashEntry = LwHashNext(&hashIterator)))
             {
                 LW_SAFE_FREE_MEMORY(pHashEntry->pValue);
             }
         }
 
-        LsaHashSafeFree(&pGroupHash);
+        LwHashSafeFree(&pGroupHash);
     }
 
     return dwError;
@@ -1323,7 +1345,7 @@ LocalDirFindObjectByGenericName(
         }
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     *ppObject = ppObjects[0];
     ppObjects[0] = NULL;
 
@@ -1342,3 +1364,13 @@ error:
 
     goto cleanup;
 }
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
