@@ -98,6 +98,7 @@ typedef struct __LOCAL_CONFIG
     DWORD     dwHomedirUMask;
     PSTR      pszSkelDirs;
     BOOLEAN   bAcceptNTLMv1;
+    BOOLEAN   EnableUnixIds;
 } LOCAL_CONFIG, *PLOCAL_CONFIG;
 
 typedef struct _LOCAL_PROVIDER_GLOBALS
@@ -116,10 +117,6 @@ typedef struct _LOCAL_PROVIDER_GLOBALS
     DWORD             dwLockoutThreshold;
     LONG64            llLockoutDuration;
     LONG64            llLockoutWindow;
-    DWORD           dwMinUid;
-    DWORD           dwMaxUid;
-    DWORD           dwMinGid;
-    DWORD           dwMaxGid;
 
     PLW_MAP_SECURITY_CONTEXT pSecCtx;
 
