@@ -80,25 +80,19 @@ LocalDirGetUserInfoFlags(
     PCSTR pszFilterTemplate =
                         LOCAL_DB_DIR_ATTR_UID " = %u" \
                         " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
-    PSTR  pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     PDIRECTORY_ENTRY pEntries = NULL;
     DWORD dwNumEntries = 0;
     DWORD dwUserInfoFlags = 0;
 
-    dwError = LwAllocateStringPrintf(
-                    &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
                     pszFilterTemplate,
                     uid,
                     LOCAL_OBJECT_CLASS_USER);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-                    pszFilter,
-                    &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
                     pContext->hDirectory,
                     NULL,
@@ -143,7 +137,6 @@ cleanup:
     }
 
     LW_SAFE_FREE_MEMORY(pwszFilter);
-    LW_SAFE_FREE_STRING(pszFilter);
 
     return dwError;
 
@@ -166,6 +159,7 @@ LocalDirAddUser(
     BOOLEAN bEventlogEnabled = FALSE;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
     PWSTR pwszUserDN = NULL;
+    PSTR pszUserDn = NULL;
     enum AttrValueIndex {
         LOCAL_DAU0_IDX_GID = 0,
         LOCAL_DAU0_IDX_OBJECTCLASS,
@@ -324,7 +318,6 @@ LocalDirAddUser(
     };
     DIRECTORY_MOD mods[LOCAL_DAU0_IDX_SENTINEL + 1];
 
-    BOOLEAN bLocked = 0;
     DWORD iMod = 0;
     PWSTR pwszSamAccountName = NULL;
     PWSTR pwszGecos = NULL;
@@ -339,9 +332,8 @@ LocalDirAddUser(
     PWSTR pwszGroupDN = NULL;
     PWSTR pwszBase = NULL;
     ULONG ulScope = 0;
-    wchar_t wszFilterFmt[] = L"%ws = \'%ws\'";
+    PCSTR filterFormat = LOCAL_DB_DIR_ATTR_DISTINGUISHED_NAME " = %Q";
     PWSTR pwszFilter = NULL;
-    DWORD dwFilterLen = 0;
     PDIRECTORY_ENTRY pMember = NULL;
     DWORD dwNumEntries = 0;
     BOOLEAN bUserAdded = FALSE;
@@ -354,6 +346,7 @@ LocalDirAddUser(
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     LSA_QUERY_LIST QueryList;
     DWORD dwGid = -1;
+    BOOLEAN bLocked = FALSE;
 
     PWSTR wszMemberAttrs[] = {
         wszAttrObjectClass,
@@ -387,7 +380,7 @@ LocalDirAddUser(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    if (!LocalServicesDomain(pLoginInfo->pszDomain))
+    if (!LocalServicesDomainInternal(pLoginInfo->pszDomain))
     {
         dwError = LW_ERROR_NOT_HANDLED;
         BAIL_ON_LSA_ERROR(dwError);
@@ -547,21 +540,16 @@ LocalDirAddUser(
 
     bUserAdded = TRUE;
 
-    dwFilterLen = (sizeof(wszAttrDistinguishedName) - 2) +
-                  (wc16slen(pwszUserDN) * sizeof(WCHAR)) +
-                   sizeof(wszFilterFmt);
-
-    dwError = LwAllocateMemory(
-                    dwFilterLen,
-                    (PVOID*)&pwszFilter);
+    dwError = LwWc16sToMbs(
+                    pwszUserDN,
+                    &pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (sw16printfw(pwszFilter, dwFilterLen/sizeof(WCHAR), wszFilterFmt,
-                    wszAttrDistinguishedName, pwszUserDN) < 0)
-    {
-        dwError = LwErrnoToWin32Error(errno);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
+                    filterFormat,
+                    pszUserDn);
+    BAIL_ON_LSA_ERROR(dwError);
 
     dwError = DirectorySearch(
                     pContext->hDirectory,
@@ -701,6 +689,7 @@ cleanup:
         LsaSrvFreeNameInfo(pLoginInfo);
     }
 
+    LW_SAFE_FREE_MEMORY(pszUserDn);
     LW_SAFE_FREE_MEMORY(pwszUserDN);
     LW_SAFE_FREE_MEMORY(pwszGroupDN);
     LW_SAFE_FREE_MEMORY(pwszSamAccountName);
@@ -709,12 +698,11 @@ cleanup:
     LW_SAFE_FREE_STRING(pszShell);
     LW_SAFE_FREE_MEMORY(pwszHomedir);
     LW_SAFE_FREE_STRING(pszHomedir);
-    LW_SAFE_FREE_MEMORY(pwszPassword);
+    LW_SECURE_FREE_WSTRING(pwszPassword);
     LW_SAFE_FREE_MEMORY(pwszDomain);
     LW_SAFE_FREE_MEMORY(pwszNetBIOSDomain);
     RTL_FREE(&pGroupSID);
     RTL_FREE(&pUserSID);
-
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (pMember)
@@ -757,25 +745,12 @@ LocalDirValidateUID(
     )
 {
     DWORD dwError = 0;
-    DWORD dwUidMin = LOWEST_UID;
-    DWORD dwUidMax = HIGHEST_UID;
-    DWORD dwUid = (DWORD) uid;
-
-    dwError = LocalCfgGetMinUid(&dwUidMin);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LocalCfgGetMaxUid(&dwUidMax);
-    BAIL_ON_LSA_ERROR(dwError);
 
     /* Check whether account uid is within permitted range */
-    if (dwUid < dwUidMin) {
+    if (uid < LOWEST_UID) {
         BAIL_WITH_LSA_ERROR(LW_ERROR_UID_TOO_LOW);
     }
 
-    if (dwUid > dwUidMax) {
-        BAIL_WITH_LSA_ERROR(LW_ERROR_UID_TOO_HIGH);
-    }
-
 cleanup:
     return dwError;
 
@@ -867,7 +842,7 @@ LocalDirModifyUser(
     };
 
     QueryList.ppszStrings = (PCSTR*) &pUserModInfo->pszSid;
-
+        
     dwError = LocalDirFindObjects(
         hProvider,
         0,
@@ -877,13 +852,13 @@ LocalDirModifyUser(
         QueryList,
         &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
-
+        
     if (ppObjects[0] == NULL)
     {
         dwError = LW_ERROR_NO_SUCH_USER;
         BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     dwError = LwMbsToWc16s(
         ppObjects[0]->pszDN,
         &pwszUserDN);
@@ -1002,7 +977,7 @@ LocalDirModifyUser(
     if (pUserModInfo->actions.bSetPrimaryGroup)
     {
         QueryList.ppszStrings = (PCSTR*) &pUserModInfo->pszPrimaryGroupSid;
-
+        
         dwError = LocalDirFindObjects(
             hProvider,
             0,
@@ -1012,7 +987,7 @@ LocalDirModifyUser(
             QueryList,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (ppObjects[0] == NULL)
         {
             dwError = LW_ERROR_NO_SUCH_GROUP;
@@ -1120,15 +1095,15 @@ LocalDirModifyUser(
             pUserModInfo->pszRemoveFromGroups,
             &pObject);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LwMbsToWc16s(
             pObject->pszDN,
             &pwszGroupDN_remove);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         LsaUtilFreeSecurityObject(pObject);
         pObject = NULL;
-
+        
         dwError = DirectoryRemoveFromGroup(
             pContext->hDirectory,
             pwszGroupDN_remove,
@@ -1145,7 +1120,7 @@ LocalDirModifyUser(
             pUserModInfo->pszAddToGroups,
             &pObject);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LwMbsToWc16s(
             pObject->pszDN,
             &pwszGroupDN_add);
@@ -1187,7 +1162,7 @@ cleanup:
 
     LW_SAFE_FREE_MEMORY(pwszGroupDN_remove);
     LW_SAFE_FREE_MEMORY(pwszGroupDN_add);
-    LW_SAFE_FREE_MEMORY(pwszPassword);
+    LW_SECURE_FREE_WSTRING(pwszPassword);
     LW_SAFE_FREE_MEMORY(pwszUserDN);
     LW_SAFE_FREE_MEMORY(pwszPrimaryGroupDn);
 
@@ -1204,12 +1179,13 @@ LocalDirDeleteUser(
     PWSTR  pwszUserDN
     )
 {
-    const wchar_t wszFilterTemplate[] = L"%ws=%u AND %ws='%ws'";
+    PCSTR filterTemplate = LOCAL_DB_DIR_ATTR_OBJECT_CLASS "=%u AND "
+                           LOCAL_DB_DIR_ATTR_DISTINGUISHED_NAME "=%Q";
     DWORD dwError = 0;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
-    WCHAR wszAttrNameDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrNameObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
     WCHAR wszAttrNameObjectSid[] = LOCAL_DIR_ATTR_OBJECT_SID;
+    PSTR pszUserDn = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
@@ -1223,11 +1199,16 @@ LocalDirDeleteUser(
         NULL
     };
 
-    dwError = LwAllocateWc16sPrintfW(
+    dwError = LwWc16sToMbs(
+                    pwszUserDN,
+                    &pszUserDn);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
                     &pwszFilter,
-                    wszFilterTemplate,
-                    wszAttrNameObjectClass, LOCAL_OBJECT_CLASS_USER,
-                    wszAttrNameDN, pwszUserDN);
+                    filterTemplate,
+                    LOCAL_OBJECT_CLASS_USER,
+                    pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = DirectorySearch(
@@ -1282,6 +1263,7 @@ cleanup:
         DirectoryFreeEntries(pEntry, dwNumEntries);
     }
 
+    LW_SAFE_FREE_MEMORY(pszUserDn);
     LW_SAFE_FREE_MEMORY(pwszFilter);
     LW_SAFE_FREE_MEMORY(pUserSid);
 
@@ -1299,7 +1281,7 @@ LocalDirChangePassword(
     PWSTR  pwszNewPassword
     )
 {
-    const wchar_t wszUserFilter[] = L"%ws = '%ws'";
+    PCSTR userFilter = LOCAL_DB_DIR_ATTR_DISTINGUISHED_NAME " = %Q";
     DWORD dwError = 0;
     NTSTATUS ntStatus = STATUS_SUCCESS;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
@@ -1307,14 +1289,11 @@ LocalDirChangePassword(
     PACCESS_TOKEN pUserToken = NULL;
     PWSTR pwszBase = NULL;
     DWORD dwScope = 0;
+    PSTR pszUserDn = NULL;
     PWSTR pwszFilter = NULL;
-    DWORD dwFilterLen = 0;
-    size_t sUserDnLen = 0;
-    WCHAR wszAttrDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrUid[] = LOCAL_DIR_ATTR_UID;
     WCHAR wszAttrGid[] = LOCAL_DIR_ATTR_GID;
     WCHAR wszAttrSecurityDescriptor[] = LOCAL_DIR_ATTR_SECURITY_DESCRIPTOR;
-
     PWSTR wszAttributes[] = {
         wszAttrUid,
         wszAttrGid,
@@ -1330,25 +1309,17 @@ LocalDirChangePassword(
     GENERIC_MAPPING GenericMapping = {0};
     DWORD dwAccessGranted = 0;
 
-    dwError = LwWc16sLen(pwszUserDN, &sUserDnLen);
+    dwError = LwWc16sToMbs(
+                      pwszUserDN,
+                      &pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwFilterLen = (sizeof(wszAttrDN)/sizeof(wszAttrDN[0]) +
-                   sUserDnLen +
-                   sizeof(wszUserFilter));
-
-    dwError = LwAllocateMemory(sizeof(pwszFilter[0]) * dwFilterLen,
-                               OUT_PPVOID(&pwszFilter));
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
+                    userFilter,
+                    pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (sw16printfw(pwszFilter, dwFilterLen,
-                    wszUserFilter,
-                    wszAttrDN, pwszUserDN) < 0)
-    {
-        dwError = LwErrnoToWin32Error(errno);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
     dwError = DirectorySearch(
                     pContext->hDirectory,
                     pwszBase,
@@ -1379,7 +1350,7 @@ LocalDirChangePassword(
 
     dwError = DirectoryGetEntryAttrValueByName(
                     pUserEntry,
-                    wszAttrUid,
+                    wszAttrGid,
                     DIRECTORY_ATTR_TYPE_INTEGER,
                     &dwGid);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1427,6 +1398,7 @@ cleanup:
         DirectoryFreeEntries(pUserEntry, dwNumEntries);
     }
 
+    LW_SAFE_FREE_MEMORY(pszUserDn);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     DirectoryFreeEntrySecurityDescriptor(&pSecDesc);
@@ -1593,7 +1565,7 @@ LocalProvisionHomeDir(
 
         if (bExists)
         {
-            dwError = LsaCopyDirectory(
+            dwError = LsaCopySkeletonDirectory(
                         pszSkelPath,
                         ownerUid,
                         ownerGid,
@@ -1646,17 +1618,17 @@ LocalCheckIsGuest(
     if (ulRid == DOMAIN_USER_RID_GUEST)
     {
         bUserIsGuest = TRUE;
-    }
+    }    
 
 cleanup:
 
     if (pUserSid)
     {
         LW_RTL_FREE(&pUserSid);
-    }
+    }    
 
     *pbUserIsGuest = bUserIsGuest;
-
+    
     return dwError;
 
 error:
@@ -1747,8 +1719,7 @@ LocalGetUserLogonInfo(
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
     WCHAR wszAttrNameLogonCount[] = LOCAL_DIR_ATTR_LOGON_COUNT;
     WCHAR wszAttrNameBadPasswordCount[] = LOCAL_DIR_ATTR_BAD_PASSWORD_COUNT;
-    PCSTR pszFilterTemplate =  LOCAL_DB_DIR_ATTR_DISTINGUISHED_NAME " = '%s'";
-    PSTR  pszFilter = NULL;
+    PCSTR pszFilterTemplate = LOCAL_DB_DIR_ATTR_DISTINGUISHED_NAME " = %Q";
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     PDIRECTORY_ENTRY pEntries = NULL;
@@ -1765,17 +1736,12 @@ LocalGetUserLogonInfo(
     BAIL_ON_INVALID_POINTER(hProvider);
     BAIL_ON_INVALID_POINTER(pszUserDn);
 
-    dwError = LwAllocateStringPrintf(
-                    &pszFilter,
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                    &pwszFilter,
                     pszFilterTemplate,
                     pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMbsToWc16s(
-                    pszFilter,
-                    &pwszFilter);
-    BAIL_ON_LSA_ERROR(dwError);
-
     dwError = DirectorySearch(
                     pContext->hDirectory,
                     NULL,
@@ -1828,7 +1794,6 @@ cleanup:
     }
 
     LW_SAFE_FREE_MEMORY(pwszFilter);
-    LW_SAFE_FREE_MEMORY(pszFilter);
 
     return dwError;
 
@@ -2118,7 +2083,7 @@ LocalDirCheckIfAdministrator(
                     hProvider,
                     &pszBuiltinAdminSid);
                 BAIL_ON_LSA_ERROR(dwError);
-
+                
                 for (; iGroup < dwNumGroups; iGroup++)
                 {
                     if (!strcmp(ppszGroupSids[iGroup], pszBuiltinAdminSid))
@@ -2166,7 +2131,7 @@ LocalDirGetBuiltinAdministratorsSid(
     if (!pszBuiltinAdminSid)
     {
         QueryList.ppszStrings = &pszGroupName;
-
+        
         dwError = LocalDirFindObjects(
             hProvider,
             0,
@@ -2176,7 +2141,7 @@ LocalDirGetBuiltinAdministratorsSid(
             QueryList,
             &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (ppObjects[0] == NULL)
         {
             dwError = LW_ERROR_NO_SUCH_GROUP;
