@@ -401,6 +401,12 @@ DirectoryFreeEntries(
     DWORD            dwNumEntries
     );
 
+VOID
+DirectoryFreeAttributes(
+    PDIRECTORY_ATTRIBUTE pAttributes,
+    DWORD                dwNumAttributes
+    );
+
 VOID
 DirectoryFreeAttributeValues(
     PATTRIBUTE_VALUE pAttrValues,
@@ -492,6 +498,13 @@ DirectoryAllocateString(
     PSTR* ppszOutputString
     );
 
+DWORD
+DirectoryAllocateWC16StringFilterPrintf(
+    OUT PWSTR* pOutput,
+    IN PCSTR Format,
+    ...
+    );
+
 VOID
 DirectoryFreeStringW(
     PWSTR pwszString
