@@ -166,7 +166,7 @@
 #define LSASS_EVENT_INFO_NETWORK_DOMAIN_ONLINE_TRANSITION            1700
 #define LSASS_EVENT_WARNING_NETWORK_DOMAIN_OFFLINE_TRANSITION        1701
 
-
+	
 typedef struct _LSA_CONFIG_REG
 {
     HANDLE  hConnection;
@@ -220,6 +220,12 @@ typedef struct __LSA_LOGIN_NAME_INFO
     } while(0);
 
 
+#define PTHREAD_CALL_MUST_SUCCEED(Call) \
+    do { \
+        int localError = Call; \
+        LSA_ASSERT(localError == 0); \
+    } while (0)
+	
 //Convert to seconds string of form ##s, ##m, ##h, or ##d
 //where s,m,h,d = seconds, minutes, hours, days.
 DWORD
