@@ -67,36 +67,15 @@ extern DWORD LsaInitializeProvider_Local(PCSTR*, PLSA_PROVIDER_FUNCTION_TABLE_2*
 static LSA_STATIC_PROVIDER gStaticProviders[] =
 {
 #ifdef ENABLE_AD
-    { "lsa-activedirectory-provider", LsaInitializeProvider_ActiveDirectory },
+    { LSA_PROVIDER_TAG_AD, LsaInitializeProvider_ActiveDirectory },
 #endif
 #ifdef ENABLE_LOCAL
-    { "lsa-local-provider", LsaInitializeProvider_Local },
+    { LSA_PROVIDER_TAG_LOCAL, LsaInitializeProvider_Local },
 #endif
     { 0 }
 };
 #endif // ENABLE_STATIC_PROVIDERS
 
-#ifdef ENABLE_PIDFILE
-static
-VOID
-LsaSrvCreatePIDFile(
-    VOID
-    );
-
-static
-pid_t
-LsaSrvGetPidFromPidFile(
-    VOID
-    );
-
-static
-VOID
-LsaSrvRemovePidFile(
-    VOID
-    );
-
-#endif
-
 int
 lsassd_main(
     int argc,
@@ -144,9 +123,6 @@ lsassd_main(
        BAIL_ON_LSA_ERROR(dwError);
     }
 
-    // Ignore any errors returned by this function.
-    LsaSrvRaiseMaxFiles(1024);
-
     dwError = LWNetExtendEnvironmentForKrb5Affinity(TRUE);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -160,10 +136,6 @@ lsassd_main(
     dwError = LsaSrvStartEventLoggingThread();
     BAIL_ON_LSA_ERROR(dwError);
 
-#ifdef ENABLE_PIDFILE
-    LsaSrvCreatePIDFile();
-#endif
-
     dwError = LwDsCacheAddPidException(getpid());
     if (dwError == LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK)
     {
@@ -186,7 +158,7 @@ lsassd_main(
     if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notifyFd = atoi(pszSmNotify);
-
+        
         do
         {
             ret = write(notifyFd, &notifyCode, sizeof(notifyCode));
@@ -237,10 +209,6 @@ cleanup:
 
     LsaShutdownTracing_r();
 
-#ifdef ENABLE_PIDFILE
-    LsaSrvRemovePidFile();
-#endif
-
     return dwError;
 
 error:
@@ -258,10 +226,10 @@ LsaSrvStartupPreCheck(
     )
 {
     DWORD dwError = 0;
+#ifdef __LWI_DARWIN__
     PSTR  pszHostname = NULL;
     int  iter = 0;
 
-#ifdef __LWI_DARWIN__
     // Make sure that the local hostname has been setup by the system
     for (iter = 0; iter < STARTUP_PRE_CHECK_WAIT; iter++)
     {
@@ -289,193 +257,17 @@ LsaSrvStartupPreCheck(
                       dwError);
         BAIL_ON_LSA_ERROR(dwError);
     }
-#endif
 
-    for (iter = 0; iter < STARTUP_NETLOGON_WAIT; iter++)
-    {
-        dwError = LsaSrvVerifyNetLogonStatus();
-
-        if (dwError)
-        {
-            sleep(1);
-        }
-        else
-        {
-            /* NetLogon is responsive */
-            LSA_LOG_INFO("LSA Process start up check for NetLogon complete");
-            break;
-        }
-    }
-
-    if (iter >= STARTUP_NETLOGON_WAIT)
-    {
-        dwError = LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
-        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use NetLogonD after %u seconds of waiting [Code:%u]",
-                      STARTUP_NETLOGON_WAIT,
-                      dwError);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
-    for (iter = 0; iter < STARTUP_LWIO_WAIT; iter++)
-    {
-        dwError = LsaSrvVerifyLwIoStatus();
-
-        if (dwError)
-        {
-            sleep(1);
-        }
-        else
-        {
-            /* LwIo is responsive */
-            LSA_LOG_INFO("LSA Process start up check for LwIo complete");
-            break;
-        }
-    }
-
-    if (iter >= STARTUP_LWIO_WAIT)
-    {
-        dwError = LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
-        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use LwIoD after %u seconds of waiting [Code:%u]",
-                      STARTUP_LWIO_WAIT,
-                      dwError);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
-#if defined (__LWI_DARWIN_)
     // Now that we are running, we need to flush the DirectoryService process of any negative cache entries
     dwError = LsaSrvFlushSystemCache();
     BAIL_ON_LSA_ERROR(dwError);
-#endif
 
-cleanup:
+error:
 
     LW_SAFE_FREE_STRING(pszHostname);
+#endif
 
     return dwError;
-
-error:
-
-    LSA_LOG_ERROR("LSA Process exiting due to error checking hostname at startup [Code:%u]", dwError);
-
-    goto cleanup;
-}
-
-DWORD
-LsaSrvVerifyNetLogonStatus(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    PSTR pszDomain = NULL;
-
-    dwError = LWNetGetCurrentDomain(&pszDomain);
-    LSA_LOG_INFO("LsaSrvVerifyNetLogonStatus call to LWNet API returned %u", dwError);
-    BAIL_ON_LSA_ERROR(dwError);
-
-cleanup:
-
-    if (pszDomain)
-    {
-        LWNetFreeString(pszDomain);
-    }
-
-    return dwError;
-
-error:
-
-    if (dwError == ERROR_NOT_JOINED)
-    {
-        dwError = 0;
-    }
-
-    goto cleanup;
-}
-
-DWORD
-LsaSrvVerifyLwIoStatus(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    PLWIO_LOG_INFO pLogInfo = NULL;
-    PIO_CONTEXT pContext = NULL;
-
-    dwError = LwIoOpenContext(&pContext);
-    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %u", dwError);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = LwIoGetLogInfo(
-                  (HANDLE) pContext,
-                  &pLogInfo);
-    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %u", dwError);
-    BAIL_ON_LSA_ERROR(dwError);
-
-cleanup:
-
-    if (pLogInfo)
-    {
-        LwIoFreeLogInfo(pLogInfo);
-    }
-
-    if (pContext != NULL)
-    {
-        LwIoCloseContext(pContext);
-    }
-
-    return dwError;
-
-error:
-
-    goto cleanup;
-}
-
-DWORD
-LsaSrvRaiseMaxFiles(
-    DWORD dwMaxFiles
-    )
-{
-    DWORD dwError = 0;
-    struct rlimit maxFiles = {0};
-    BOOLEAN bRaised = FALSE;
-
-    if (getrlimit(RLIMIT_NOFILE, &maxFiles) < 0)
-    {
-        dwError = LwMapErrnoToLwError(errno);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-
-    if (maxFiles.rlim_cur < dwMaxFiles)
-    {
-        LSA_LOG_INFO("Raising soft max fd limit from %lu to %u",
-                maxFiles.rlim_cur,
-                dwMaxFiles);
-        maxFiles.rlim_cur = dwMaxFiles;
-        bRaised = TRUE;
-    }
-    if (maxFiles.rlim_max < dwMaxFiles)
-    {
-        LSA_LOG_INFO("Raising hard max fd limit from %lu to %u",
-                maxFiles.rlim_max,
-                dwMaxFiles);
-        maxFiles.rlim_max = dwMaxFiles;
-        bRaised = TRUE;
-    }
-
-    if (bRaised)
-    {
-        if (setrlimit(RLIMIT_NOFILE, &maxFiles) < 0)
-        {
-            dwError = LwMapErrnoToLwError(errno);
-            LSA_LOG_ERROR("Raising max fd limit failed");
-            BAIL_ON_LSA_ERROR(dwError);
-        }
-    }
-
-cleanup:
-    return dwError;
-
-error:
-    goto cleanup;
 }
 
 DWORD
@@ -982,113 +774,6 @@ LsaSrvGetPrefixPath(
     goto cleanup;
 }
 
-#ifdef ENABLE_PIDFILE
-VOID
-LsaSrvCreatePIDFile(
-    VOID
-    )
-{
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-
-    pid = LsaSrvGetPidFromPidFile();
-    if (pid > 0) {
-        fprintf(stderr, "Daemon already running as %d\n", (int) pid);
-        result = -1;
-        goto error;
-    }
-
-    fd = open(PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0) {
-        fprintf(stderr, "Could not create pid file: %s\n", strerror(errno));
-        result = 1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) write(fd, contents, len);
-    if ( result != (int) len ) {
-        fprintf(stderr, "Could not write to pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    if (result < 0) {
-        exit(1);
-    }
-}
-
-static
-pid_t
-LsaSrvGetPidFromPidFile(
-    VOID
-    )
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(PID_FILE, O_RDONLY, 0644);
-    if (fd < 0) {
-        goto error;
-    }
-
-    result = read(fd, contents, sizeof(contents)-1);
-    if (result <= 0) {
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result <= 0) {
-        result = -1;
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH) {
-        unlink(PID_FILE);
-        pid = 0;
-    }
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    return pid;
-}
-
-static
-VOID
-LsaSrvRemovePidFile(
-    VOID
-    )
-{
-    if (LsaSrvGetPidFromPidFile() == getpid())
-    {
-        unlink(PID_FILE);
-    }
-}
-
-#endif
-
 DWORD
 LsaBlockSelectedSignals(
     VOID
