@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see 
+ * Public License along with this program.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -49,8 +49,12 @@
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
 #include "lsaipc.h"
+#include "ntlmipc.h"
 #include "lsalog_r.h"
 #include "lsaserver.h"
 #include "lsasrvapi.h"
@@ -60,10 +64,10 @@
 #include "listener.h"
 #include "servermain.h"
 #include "sighandler.h"
+#include "backtrace.h"
 
 #include "externs_p.h"
 
-#include "lsaunistr.h"
-#include "ntlmgsssrv.h"
 #include <locale.h>
 
+#include "lwio/lwio.h"
