@@ -53,25 +53,17 @@ main(
     char* argv[]
     );
 
+int
+lsassd_main(
+    int argc,
+    char* argv[]
+    );
+
 DWORD
 LsaSrvStartupPreCheck(
     VOID
     );
 
-#if defined (__LWI_DARWIN__)
-DWORD
-LsaSrvFlushDirectoryServiceCache(
-    VOID
-    );
-#endif
-
-#if defined (__LWI_DARWIN__)
-DWORD
-LsaSrvVerifyNetLogonStatus(
-    VOID
-    );
-#endif
-
 DWORD
 LsaSrvSetDefaults(
     VOID
@@ -139,16 +131,6 @@ LsaSrvGetPrefixPath(
     PSTR* ppszPath
     );
 
-VOID
-LsaSrvCreatePIDFile(
-    VOID
-    );
-
-pid_t
-LsaSrvGetPidFromPidFile(
-    VOID
-    );
-
 DWORD
 LsaSrvInitLogging(
     PCSTR pszProgramName,
@@ -171,6 +153,16 @@ LsaSrvSetProcessToExit(
     BOOLEAN bExit
     );
 
+VOID
+LsaSrvLogProcessStartedEvent(
+    VOID
+    );
+
+VOID
+LsaSrvLogProcessStoppedEvent(
+    DWORD dwExitCode
+    );
+
 VOID
 LsaSrvLogProcessFailureEvent(
     DWORD dwErrCode
