@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -94,6 +94,12 @@ NtlmServerAcceptSecurityContext(
         pNegMsg = pInput->pvBuffer;
         dwMessageSize = pInput->cbBuffer;
 
+        if (dwMessageSize < sizeof(*pNegMsg))
+        {
+            dwError = ERROR_INVALID_PARAMETER;
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+
         dwError = NtlmCreateChallengeContext(
             pNegMsg,
             hCred,
@@ -123,6 +129,7 @@ NtlmServerAcceptSecurityContext(
 
         dwError = NtlmValidateResponse(
             Handle,
+            hCred,
             pRespMsg,
             dwMessageSize,
             pNtlmCtxtChlng,
@@ -136,7 +143,7 @@ NtlmServerAcceptSecurityContext(
                 dwError = NtlmCreateValidatedContext(
                     pRespMsg,
                     dwMessageSize,
-                    pNtlmCtxtChlng->NegotiatedFlags,
+                    pNtlmCtxtChlng,
                     SessionKey,
                     NTLM_SESSION_KEY_SIZE,
                     hCred,
@@ -200,6 +207,7 @@ NtlmCreateChallengeContext(
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
+    PNTLM_CREDENTIALS pCred = (PNTLM_CREDENTIALS)hCred;
     PNTLM_CONTEXT pNtlmContext = NULL;
     DWORD dwMessageSize = 0;
     PNTLM_CHALLENGE_MESSAGE pMessage = NULL;
@@ -214,6 +222,7 @@ NtlmCreateChallengeContext(
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = NtlmGetNameInformation(
+        pCred ? pCred->pszDomainName : NULL,
         &pServerName,
         &pDomainName,
         &pDnsServerName,
@@ -240,7 +249,7 @@ NtlmCreateChallengeContext(
 
     BAIL_ON_LSA_ERROR(dwError);
 
-    pNtlmContext->NegotiatedFlags = pMessage->NtlmFlags;
+    pNtlmContext->NegotiatedFlags = LW_LTOH32(pMessage->NtlmFlags);
     pOutput->cbBuffer = dwMessageSize;
     pOutput->BufferType = SECBUFFER_TOKEN;
     pOutput->pvBuffer = pMessage;
@@ -323,7 +332,7 @@ NtlmCreateGuestContext(
                   "%s\\%s",
                   ppObjects[0]->pszNetbiosDomainName,
                   ppObjects[0]->pszSamAccountName);
-
+ 
     // NULL session key
     memset(pNtlmContext->SessionKey, 0x0, NTLM_SESSION_KEY_SIZE);
     pNtlmContext->cbSessionKeyLen = NTLM_SESSION_KEY_SIZE;
@@ -356,7 +365,7 @@ DWORD
 NtlmCreateValidatedContext(
     IN PNTLM_RESPONSE_MESSAGE_V1 pNtlmRespMsg,
     IN DWORD dwMsgSize,
-    IN DWORD NegotiatedFlags,
+    IN PNTLM_CONTEXT pNtlmCtxtChlng,
     IN PBYTE pSessionKey,
     IN DWORD dwSessionKeyLen,
     IN NTLM_CRED_HANDLE hCred,
@@ -377,50 +386,38 @@ NtlmCreateValidatedContext(
 
     pNtlmContext->NtlmState = NtlmStateResponse;
 
-    pNtlmContext->NegotiatedFlags = NegotiatedFlags;
-
-    dwError = NtlmGetUserNameFromResponse(
-        pNtlmRespMsg,
-        NegotiatedFlags & NTLM_FLAG_UNICODE,
-        &pUserName);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    dwError = NtlmGetDomainNameFromResponse(
-        pNtlmRespMsg,
-        NegotiatedFlags & NTLM_FLAG_UNICODE,
-        &pDomainName);
-    BAIL_ON_LSA_ERROR(dwError);
+    pNtlmContext->NegotiatedFlags = pNtlmCtxtChlng->NegotiatedFlags;
 
     dwError = LwAllocateStringPrintf(
-        &pNtlmContext->pszClientUsername,
-        "%s\\%s",
-        pDomainName,
-        pUserName);
+                    &pNtlmContext->pszClientUsername,
+                    "%s\\%s",
+                    pNtlmCtxtChlng->pUserInfo->pszDomain,
+                    pNtlmCtxtChlng->pUserInfo->pszAccount);
     BAIL_ON_LSA_ERROR(dwError);
 
     memcpy(pNtlmContext->SessionKey, pSessionKey, NTLM_SESSION_KEY_SIZE);
     pNtlmContext->cbSessionKeyLen = dwSessionKeyLen;
     pNtlmContext->bInitiatedSide = FALSE;
 
-    if (NegotiatedFlags & NTLM_FLAG_KEY_EXCH)
+    if (pNtlmContext->NegotiatedFlags & NTLM_FLAG_KEY_EXCH)
     {
         pV2Message = (PNTLM_RESPONSE_MESSAGE_V2)pNtlmRespMsg;
         if (dwMsgSize < sizeof(*pV2Message))
         {
-            dwError = LW_ERROR_INVALID_PARAMETER;
+            dwError = ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        if (pV2Message->SessionKey.dwOffset +
-                pV2Message->SessionKey.usLength > dwMsgSize)
+        if (LW_LTOH32(pV2Message->SessionKey.dwOffset) +
+                LW_LTOH16(pV2Message->SessionKey.usLength) > dwMsgSize)
         {
-            dwError = LW_ERROR_INVALID_PARAMETER;
+            dwError = ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        if (pV2Message->SessionKey.usLength != NTLM_SESSION_KEY_SIZE)
+        if (LW_LTOH16(pV2Message->SessionKey.usLength) != NTLM_SESSION_KEY_SIZE)
         {
-            dwError = LW_ERROR_INVALID_PARAMETER;
+            dwError = ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
 
@@ -430,7 +427,7 @@ NtlmCreateValidatedContext(
                 pNtlmContext->SessionKey);
         RC4(&Rc4Key,
                 NTLM_SESSION_KEY_SIZE,
-                pV2Message->SessionKey.dwOffset + (PBYTE)pV2Message,
+                LW_LTOH32(pV2Message->SessionKey.dwOffset) + (PBYTE)pV2Message,
                 pNtlmContext->SessionKey);
     }
 
@@ -471,11 +468,11 @@ NtlmCreateSubkey(
 
     MD5_Init(&ctx);
     MD5_Update(
-        &ctx,
+        &ctx, 
         pMasterKey,
         dwMasterKeyLen);
     MD5_Update(
-        &ctx,
+        &ctx, 
         pszSubkeyMagic,
         strlen(pszSubkeyMagic) + 1); //Include the null
     MD5_Final(md5Result, &ctx);
@@ -522,11 +519,11 @@ NtlmInitializeKeys(
 
         MD5_Init(&ctx);
         MD5_Update(
-            &ctx,
+            &ctx, 
             pNtlmContext->SessionKey,
             pNtlmContext->cbSessionKeyLen);
         MD5_Update(
-            &ctx,
+            &ctx, 
             clientServerSign,
             strlen(clientServerSign) + 1); //Include the null
         MD5_Final(
@@ -536,11 +533,11 @@ NtlmInitializeKeys(
 
         MD5_Init(&ctx);
         MD5_Update(
-            &ctx,
+            &ctx, 
             pNtlmContext->SessionKey,
             pNtlmContext->cbSessionKeyLen);
         MD5_Update(
-            &ctx,
+            &ctx, 
             serverClientSign,
             strlen(serverClientSign) + 1); //Include the null
         MD5_Final(
@@ -593,7 +590,7 @@ NtlmInitializeKeys(
             sizeof(*pNtlmContext->pSealKey),
             OUT_PPVOID(&pNtlmContext->pSealKey));
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         RC4_set_key(
             pNtlmContext->pSealKey,
             pNtlmContext->cbSessionKeyLen,
@@ -611,6 +608,7 @@ error:
 DWORD
 NtlmValidateResponse(
     IN HANDLE Handle,
+    IN NTLM_CRED_HANDLE hCred,
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
     IN DWORD dwRespMsgSize,
     IN PNTLM_CONTEXT pChlngCtxt,
@@ -618,6 +616,7 @@ NtlmValidateResponse(
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
+    PNTLM_CREDENTIALS pCred = (PNTLM_CREDENTIALS)hCred;
     LSA_AUTH_USER_PARAMS Params;
     PLSA_AUTH_USER_INFO pUserInfo = NULL;
     PBYTE pLMRespBuffer = NULL;
@@ -628,6 +627,7 @@ NtlmValidateResponse(
     PSTR pUserName = NULL;
     PSTR pDomainName = NULL;
     PSTR pWorkstation = NULL;
+    PSTR pDomainInstance = NULL;
     BYTE sessionNonce[MD5_DIGEST_LENGTH];
     BYTE sessionHashUntrunc[MD5_DIGEST_LENGTH];
 
@@ -643,13 +643,19 @@ NtlmValidateResponse(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    if (dwRespMsgSize < sizeof(*pRespMsg))
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LwAllocateMemory(
-        pRespMsg->LmResponse.usLength,
+        LW_LTOH16(pRespMsg->LmResponse.usLength),
         OUT_PPVOID(&pLMRespBuffer));
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwAllocateMemory(
-        pRespMsg->NtResponse.usLength,
+        LW_LTOH16(pRespMsg->NtResponse.usLength),
         OUT_PPVOID(&pNTRespBuffer));
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -660,39 +666,61 @@ NtlmValidateResponse(
 
     dwError = NtlmGetUserNameFromResponse(
         pRespMsg,
+        dwRespMsgSize,
         pChlngCtxt->NegotiatedFlags & NTLM_FLAG_UNICODE,
         &pUserName);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = NtlmGetDomainNameFromResponse(
         pRespMsg,
+        dwRespMsgSize,
         pChlngCtxt->NegotiatedFlags & NTLM_FLAG_UNICODE,
         &pDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (pDomainName[0] == 0)
+    {
+        LW_SAFE_FREE_STRING(pDomainName);
+    }
+
     dwError = NtlmGetWorkstationFromResponse(
         pRespMsg,
+        dwRespMsgSize,
         pChlngCtxt->NegotiatedFlags & NTLM_FLAG_UNICODE,
         &pWorkstation);
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (LW_LTOH32(pRespMsg->LmResponse.dwOffset) + LW_LTOH16(pRespMsg->LmResponse.usLength) >
+            dwRespMsgSize)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     memcpy(
         pLMRespBuffer,
-        (PBYTE)pRespMsg + pRespMsg->LmResponse.dwOffset,
-        pRespMsg->LmResponse.usLength);
+        (PBYTE)pRespMsg + LW_LTOH32(pRespMsg->LmResponse.dwOffset),
+        LW_LTOH16(pRespMsg->LmResponse.usLength));
+
+    if (LW_LTOH32(pRespMsg->NtResponse.dwOffset) + LW_LTOH16(pRespMsg->NtResponse.usLength) >
+            dwRespMsgSize)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     memcpy(
         pNTRespBuffer,
-        (PBYTE)pRespMsg + pRespMsg->NtResponse.dwOffset,
-        pRespMsg->NtResponse.usLength);
+        (PBYTE)pRespMsg + LW_LTOH32(pRespMsg->NtResponse.dwOffset),
+        LW_LTOH16(pRespMsg->NtResponse.usLength));
 
-    if (pRespMsg->NtResponse.usLength == 24 &&
+    if (LW_LTOH16(pRespMsg->NtResponse.usLength) == 24 &&
             pChlngCtxt->NegotiatedFlags & NTLM_FLAG_NTLM2)
     {
         // The client sent an NTLM2 session response. That means we need to
         // calculate the challenge the client used.
 
-        if (pRespMsg->LmResponse.usLength < 8)
+        if (LW_LTOH16(pRespMsg->LmResponse.usLength) < 8)
         {
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
@@ -713,10 +741,10 @@ NtlmValidateResponse(
         Challenge.pData = pChlngCtxt->Challenge;
     }
 
-    LMResp.dwLen = pRespMsg->LmResponse.usLength;
+    LMResp.dwLen = LW_LTOH16(pRespMsg->LmResponse.usLength);
     LMResp.pData = pLMRespBuffer;
 
-    NTResp.dwLen = pRespMsg->NtResponse.usLength;
+    NTResp.dwLen = LW_LTOH16(pRespMsg->NtResponse.usLength);
     NTResp.pData = pNTRespBuffer;
 
     Params.AuthType = LSA_AUTH_CHAP;
@@ -731,8 +759,18 @@ NtlmValidateResponse(
 
     Params.pszWorkstation = pWorkstation;
 
+    if (pCred && pCred->pszDomainName)
+    {
+        dwError = LwAllocateStringPrintf(
+                      &pDomainInstance,
+                      ":%s",
+                      pCred->pszDomainName);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LsaSrvAuthenticateUserEx(
         Handle,
+        pDomainInstance,
         &Params,
         &pUserInfo
         );
@@ -740,7 +778,7 @@ NtlmValidateResponse(
 
     LW_ASSERT(pUserInfo->pSessionKey->dwLen == NTLM_SESSION_KEY_SIZE);
 
-    if (pRespMsg->NtResponse.usLength == 24 &&
+    if (LW_LTOH16(pRespMsg->NtResponse.usLength) == 24 &&
             pChlngCtxt->NegotiatedFlags & NTLM_FLAG_NTLM2)
     {
         HMAC(
@@ -772,6 +810,7 @@ cleanup:
     LW_SAFE_FREE_STRING(pUserName);
     LW_SAFE_FREE_STRING(pDomainName);
     LW_SAFE_FREE_STRING(pWorkstation);
+    LW_SAFE_FREE_STRING(pDomainInstance);
 
     return dwError;
 error:
@@ -782,6 +821,7 @@ error:
 DWORD
 NtlmGetDomainNameFromResponse(
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
+    IN DWORD dwRespMsgSize,
     IN BOOLEAN bUnicode,
     OUT PSTR* ppDomainName
     )
@@ -791,12 +831,23 @@ NtlmGetDomainNameFromResponse(
     DWORD dwNameLength = 0;
     PBYTE pBuffer = NULL;
     PNTLM_SEC_BUFFER pSecBuffer = &pRespMsg->AuthTargetName;
-    DWORD nIndex = 0;
 
     *ppDomainName = NULL;
 
-    dwNameLength = pSecBuffer->usLength;
-    pBuffer = pSecBuffer->dwOffset + (PBYTE)pRespMsg;
+    if (dwRespMsgSize < sizeof(*pRespMsg))
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    if (LW_LTOH32(pSecBuffer->dwOffset) + LW_LTOH16(pSecBuffer->usLength) > dwRespMsgSize)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwNameLength = LW_LTOH16(pSecBuffer->usLength);
+    pBuffer = LW_LTOH32(pSecBuffer->dwOffset) + (PBYTE)pRespMsg;
 
     if (!bUnicode)
     {
@@ -807,15 +858,11 @@ NtlmGetDomainNameFromResponse(
     }
     else
     {
-        dwNameLength = dwNameLength / sizeof(WCHAR);
-
-        dwError = LwAllocateMemory(dwNameLength + 1, OUT_PPVOID(&pName));
+        dwError = NtlmGetCStringFromUnicodeBuffer(
+                      pBuffer,
+                      dwNameLength,
+                      &pName);
         BAIL_ON_LSA_ERROR(dwError);
-
-        for (nIndex = 0; nIndex < dwNameLength; nIndex++)
-        {
-            pName[nIndex] = pBuffer[nIndex * sizeof(WCHAR)];
-        }
     }
 
 cleanup:
@@ -830,6 +877,7 @@ error:
 DWORD
 NtlmGetWorkstationFromResponse(
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
+    IN DWORD dwRespMsgSize,
     IN BOOLEAN bUnicode,
     OUT PSTR* ppWorkstation
     )
@@ -839,12 +887,23 @@ NtlmGetWorkstationFromResponse(
     DWORD dwNameLength = 0;
     PBYTE pBuffer = NULL;
     PNTLM_SEC_BUFFER pSecBuffer = &pRespMsg->Workstation;
-    DWORD nIndex = 0;
 
     *ppWorkstation = NULL;
 
-    dwNameLength = pSecBuffer->usLength;
-    pBuffer = pSecBuffer->dwOffset + (PBYTE)pRespMsg;
+    if (dwRespMsgSize < sizeof(*pRespMsg))
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    if (LW_LTOH32(pSecBuffer->dwOffset) + LW_LTOH16(pSecBuffer->usLength) > dwRespMsgSize)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwNameLength = LW_LTOH16(pSecBuffer->usLength);
+    pBuffer = LW_LTOH32(pSecBuffer->dwOffset) + (PBYTE)pRespMsg;
 
     if (!bUnicode)
     {
@@ -855,15 +914,12 @@ NtlmGetWorkstationFromResponse(
     }
     else
     {
-        dwNameLength = dwNameLength / sizeof(WCHAR);
-
-        dwError = LwAllocateMemory(dwNameLength + 1, OUT_PPVOID(&pName));
+        dwError = NtlmGetCStringFromUnicodeBuffer(
+                      pBuffer,
+                      dwNameLength,
+                      &pName);
         BAIL_ON_LSA_ERROR(dwError);
 
-        for (nIndex = 0; nIndex < dwNameLength; nIndex++)
-        {
-            pName[nIndex] = pBuffer[nIndex * sizeof(WCHAR)];
-        }
     }
 
 cleanup:
