@@ -159,7 +159,7 @@ NtlmServerAcquireCredentialsHandle(
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
-    LW_SAFE_FREE_STRING(pPassword);
+    LW_SECURE_FREE_STRING(pPassword);
     LW_SAFE_FREE_STRING(pUserName);
     LW_SAFE_FREE_STRING(pNT4UserName);
 
@@ -185,16 +185,16 @@ error:
 
 DWORD
 NtlmGetNameInformation(
-    PSTR* ppszServerName,
-    PSTR* ppszDomainName,
-    PSTR* ppszDnsServerName,
-    PSTR* ppszDnsDomainName
+    IN OPTIONAL PCSTR pszJoinedDomain,
+    OUT PSTR* ppszServerName,
+    OUT PSTR* ppszDomainName,
+    OUT PSTR* ppszDnsServerName,
+    OUT PSTR* ppszDnsDomainName
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
     CHAR FullDomainName[HOST_NAME_MAX + 1];
     PCHAR pSymbol = NULL;
-    PSTR pHostName = NULL;
     PSTR pServerName = NULL;
     PSTR pDomainName = NULL;
     PSTR pDnsServerName = NULL;
@@ -202,53 +202,36 @@ NtlmGetNameInformation(
     PSTR pName = NULL;
     struct hostent* pHost = NULL;
     DWORD dwHostSize = 0;
-    PLWPS_PASSWORD_INFO pMachineAcctInfo = NULL;
-    HANDLE hPasswordStore = NULL;
-
-    // First try pstore
-    dwError = LwpsOpenPasswordStore(
-                  LWPS_PASSWORD_STORE_DEFAULT,
-                  &hPasswordStore);
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
+    dwError = LsaSrvProviderGetMachineAccountInfoA(
+                  LSA_PROVIDER_TAG_AD,
+                  pszJoinedDomain,
+                  &pAccountInfo);
     if (dwError == LW_ERROR_SUCCESS)
     {
-        dwError = LwpsGetPasswordByCurrentHostName(
-                      hPasswordStore,
-                      &pMachineAcctInfo);
-    }
-
-    if (dwError == LW_ERROR_SUCCESS)
-    {
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszMachineAccount,
-            &pServerName);
+        dwError = LwAllocateString(
+                      pAccountInfo->SamAccountName,
+                      &pServerName);
         BAIL_ON_LSA_ERROR(dwError);
 
+        // Remove trailing $
         pServerName[strlen(pServerName) - 1] = 0;
 
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszHostname,
-            &pHostName);
+        dwError = LwAllocateString(
+                      pAccountInfo->NetbiosDomainName,
+                      &pDomainName);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszDomainName,
-            &pDomainName);
+        dwError = LwAllocateString(
+                      pAccountInfo->Fqdn,
+                      &pDnsServerName);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszHostDnsDomain,
-            &pDnsDomainName);
+        dwError = LwAllocateString(
+                      pAccountInfo->DnsDomainName,
+                      &pDnsDomainName);
         BAIL_ON_LSA_ERROR(dwError);
-
-        dwError = LwAllocateStringPrintf(
-                       &pDnsServerName,
-                       "%s.%s",
-                       pHostName,
-                       pDnsDomainName);
-        BAIL_ON_LSA_ERROR(dwError);
-
-        LwStrToLower(pDnsServerName);
     }
     else
     {
@@ -351,17 +334,7 @@ cleanup:
         LW_SAFE_FREE_STRING(pDnsDomainName);
     }
 
-    LW_SAFE_FREE_STRING(pHostName);
-
-    if (pMachineAcctInfo)
-    {
-        LwpsFreePasswordInfo(hPasswordStore, pMachineAcctInfo);
-    }
-
-    if (hPasswordStore)
-    {
-        LwpsClosePasswordStore(hPasswordStore);
-    }
+    LsaSrvFreeMachineAccountInfoA(pAccountInfo);
 
     return dwError;
 error:
