@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -102,7 +102,7 @@ NtlmServerEncryptMessage(
     {
         pData = &pMessage->pBuffers[dwIndex];
 
-        if (pData->BufferType == SECBUFFER_DATA)
+        if (pData->BufferType == SECBUFFER_DATA) 
         {
             if (!pData->pvBuffer)
             {
@@ -125,3 +125,13 @@ cleanup:
 error:
     goto cleanup;
 }
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
