@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -64,11 +64,11 @@ NtlmServerInitializeSecurityContext(
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
+    PNTLM_CREDENTIALS pCred = (PNTLM_CREDENTIALS)hCredential;
     PNTLM_CONTEXT pNtlmContext = NULL;
     PSTR pWorkstation = NULL;
     PSTR pDomain = NULL;
     PNTLM_CHALLENGE_MESSAGE pMessage = NULL;
-    DWORD dwMessageSize = 0;
 
     pOutput->pvBuffer = NULL;
 
@@ -80,6 +80,7 @@ NtlmServerInitializeSecurityContext(
     if (!pNtlmContext)
     {
         dwError = NtlmGetNameInformation(
+                pCred ? pCred->pszDomainName : NULL,
                 &pWorkstation,
                 &pDomain,
                 NULL,
@@ -110,7 +111,6 @@ NtlmServerInitializeSecurityContext(
         }
 
         pMessage = pInput->pvBuffer;
-        dwMessageSize = pInput->cbBuffer;
 
         dwError = NtlmCreateResponseContext(
             pMessage,
@@ -131,7 +131,7 @@ NtlmServerInitializeSecurityContext(
             pfContextAttr,
             NULL,
             NULL);
-    }
+     }
 
 
 cleanup:
@@ -175,7 +175,7 @@ NtlmCreateNegotiateContext(
     DWORD dwMessageSize = 0;
     PNTLM_NEGOTIATE_MESSAGE_V1 pMessage = NULL;
     NTLM_CONFIG config;
-    DWORD dwOptions =
+    DWORD dwDefaultOptions =
             // Always do signing and sealing since they cannot be turned off on
             // Windows
             NTLM_FLAG_SIGN                  |
@@ -185,6 +185,13 @@ NtlmCreateNegotiateContext(
             NTLM_FLAG_NTLM                  |
             NTLM_FLAG_DOMAIN                |
             0;
+    DWORD dwDceStyleOptions =
+            NTLM_FLAG_OEM                   |
+            NTLM_FLAG_REQUEST_TARGET        |
+            NTLM_FLAG_NTLM                  |
+            NTLM_FLAG_DOMAIN                |
+            0;
+    DWORD dwOptions = 0;
 
     *ppNtlmContext = NULL;
 
@@ -194,6 +201,15 @@ NtlmCreateNegotiateContext(
     dwError = NtlmReadRegistry(&config);
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (fContextReq & ISC_REQ_USE_DCE_STYLE)
+    {
+        dwOptions = dwDceStyleOptions;
+    }
+    else
+    {
+        dwOptions = dwDefaultOptions;
+    }
+
     if (config.bSupportUnicode)
     {
         dwOptions |= NTLM_FLAG_UNICODE;
@@ -215,6 +231,16 @@ NtlmCreateNegotiateContext(
         dwOptions |= NTLM_FLAG_128;
     }
 
+    if (fContextReq & ISC_REQ_INTEGRITY)
+    {
+        dwOptions |= NTLM_FLAG_SIGN;
+    }
+
+    if (fContextReq & ISC_REQ_CONFIDENTIALITY)
+    {
+        dwOptions |= NTLM_FLAG_SEAL;
+    }
+
     if (fContextReq & ISC_REQ_NULL_SESSION)
     {
         pNtlmContext->bDoAnonymous = TRUE;
@@ -344,7 +370,7 @@ NtlmCreateResponseContext(
         // TODO: the correct thing is to use LMv2
         dwLmRespType = NTLM_RESPONSE_TYPE_LM;
     }
-    else if(pChlngMsg->NtlmFlags & NTLM_FLAG_NTLM2)
+    else if(LW_LTOH32(pChlngMsg->NtlmFlags) & NTLM_FLAG_NTLM2)
     {
         dwLmRespType = NTLM_RESPONSE_TYPE_NTLM2;
         dwNtRespType = NTLM_RESPONSE_TYPE_NTLM2;
@@ -375,7 +401,7 @@ NtlmCreateResponseContext(
 
     pMasterKey = NtlmUserSessionKey;
 
-    if (pChlngMsg->NtlmFlags & NTLM_FLAG_LM_KEY)
+    if (LW_LTOH32(pChlngMsg->NtlmFlags) & NTLM_FLAG_LM_KEY)
     {
         NtlmGenerateLanManagerSessionKey(
             pMessage,
@@ -385,7 +411,7 @@ NtlmCreateResponseContext(
         pMasterKey = LanManagerSessionKey;
     }
 
-    if (pChlngMsg->NtlmFlags & NTLM_FLAG_KEY_EXCH)
+    if (LW_LTOH32(pChlngMsg->NtlmFlags) & NTLM_FLAG_KEY_EXCH)
     {
         // This is the key we will use for session security...
         dwError = NtlmGetRandomBuffer(
@@ -410,7 +436,7 @@ NtlmCreateResponseContext(
 
     memcpy(pNtlmContext->SessionKey, pMasterKey, NTLM_SESSION_KEY_SIZE);
 
-    pNtlmContext->NegotiatedFlags = pChlngMsg->NtlmFlags;
+    pNtlmContext->NegotiatedFlags = LW_LTOH32(pChlngMsg->NtlmFlags);
     pOutput->cbBuffer = dwMessageSize;
     pOutput->BufferType = SECBUFFER_TOKEN;
     pOutput->pvBuffer = pMessage;
@@ -442,3 +468,13 @@ error:
 
     goto cleanup;
 }
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
