@@ -75,10 +75,11 @@
 #include <lsalist.h>
 #include <lwsecurityidentifier.h>
 #include <lsautils.h>
+#include <lsaadprovider.h>
 #include <lwdef.h>
 #include <lwkrb5.h>
+#include <gssapi/gssapi_ext.h>
 #include <lwmem.h>
-#include <lwps/lwps.h>
 #include <lwstr.h>
 #include <wc16str.h>
 #include <uuid/uuid.h>
