@@ -73,10 +73,11 @@ NtlmServerAcquireCredentialsHandle(
 
 DWORD
 NtlmGetNameInformation(
-    PSTR* ppszServerName,
-    PSTR* ppszDomainName,
-    PSTR* ppszDnsServerName,
-    PSTR* ppszDnsDomainName
+    IN OPTIONAL PCSTR pszJoinedDomain,
+    OUT PSTR* ppszServerName,
+    OUT PSTR* ppszDomainName,
+    OUT PSTR* ppszDnsServerName,
+    OUT PSTR* ppszDnsDomainName
     );
 
 DWORD
@@ -178,6 +179,13 @@ NtlmServerQueryContextAttributes(
     OUT PVOID pBuffer
     );
 
+DWORD
+NtlmServerSetCredentialsAttributes(
+    IN PNTLM_CRED_HANDLE phCredential,
+    IN DWORD ulAttribute,
+    IN PSecPkgCred pCred
+    );
+
 DWORD
 NtlmServerVerifySignature(
     IN PNTLM_CONTEXT_HANDLE phContext,
@@ -217,6 +225,12 @@ NtlmServerQueryCredNameAttribute(
     OUT PSecPkgCred_Names *ppNames
     );
 
+DWORD
+NtlmServerSetCredDomainNameAttribute(
+    IN NTLM_CRED_HANDLE hCred,
+    IN PSecPkgCred_DomainName pDomainName
+    );
+
 VOID
 NtlmReleaseContext(
     IN PNTLM_CONTEXT_HANDLE phContext
@@ -316,19 +330,27 @@ NtlmCreateNegotiateMessage(
     OUT PNTLM_NEGOTIATE_MESSAGE_V1* ppNegMsg
     );
 
-VOID
+DWORD
+NtlmAddTargetInfoBuffer(
+    IN SHORT InfoType,
+    IN PWSTR pInput,
+    IN OUT PBYTE* ppBufferPos
+    );
+
+DWORD
 NtlmCopyStringToSecBuffer(
-    IN PCSTR pszInput,
-    IN DWORD dwFlags,
+    IN PVOID pInput,
+    IN DWORD Length,
     IN PBYTE pBufferStart,
     IN OUT PBYTE* ppBufferPos,
     OUT PNTLM_SEC_BUFFER pSec
     );
 
 DWORD
-NtlmGetStringProtocolSize(
-    DWORD dwFlags,
-    PCSTR pszString
+NtlmGetCStringFromUnicodeBuffer(
+    IN PBYTE pBuffer,
+    IN DWORD Length,
+    OUT PSTR *ppString
     );
 
 DWORD
@@ -373,17 +395,12 @@ NtlmWeakenSessionKey(
     OUT PDWORD pcbKeyLength
     );
 
-DWORD
-NtlmGetAuthTargetNameFromChallenge(
-    IN PNTLM_CHALLENGE_MESSAGE pChlngMsg,
-    OUT PCHAR* ppAuthTargetName
-    );
-
 DWORD
 NtlmBuildResponse(
     IN PNTLM_CHALLENGE_MESSAGE pChlngMsg,
     IN PCSTR pUserName,
     IN PCSTR pPassword,
+    IN PCSTR pTarget,
     IN DWORD dwResponseType,
     OUT PDWORD pdwBufferSize,
     OUT PBYTE pUserSessionKey,
@@ -413,6 +430,7 @@ NtlmBuildNtlmV2Response(
     IN PNTLM_CHALLENGE_MESSAGE pChlngMsg,
     IN PCSTR pUserName,
     IN PCSTR pPassword,
+    IN PCSTR pTarget,
     OUT PDWORD pdwResponseSize,
     OUT PBYTE pUserSessionKey,
     OUT PBYTE* ppResponse
@@ -490,7 +508,7 @@ DWORD
 NtlmCreateValidatedContext(
     IN PNTLM_RESPONSE_MESSAGE_V1 pNtlmRespMsg,
     IN DWORD dwMsgSize,
-    IN DWORD NegotiatedFlags,
+    IN PNTLM_CONTEXT pNtlmCtxtChlng,
     IN PBYTE pSessionKey,
     IN DWORD dwSessionKeyLen,
     IN NTLM_CRED_HANDLE hCred,
@@ -513,6 +531,7 @@ NtlmInitializeKeys(
 DWORD
 NtlmValidateResponse(
     IN HANDLE Handle,
+    IN NTLM_CRED_HANDLE hCred,
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
     IN DWORD dwRespMsgSize,
     IN PNTLM_CONTEXT pChlngCtxt,
@@ -522,6 +541,7 @@ NtlmValidateResponse(
 DWORD
 NtlmGetUserNameFromResponse(
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
+    IN DWORD dwRespMsgSize,
     IN BOOLEAN bUnicode,
     OUT PSTR* ppUserName
     );
@@ -529,6 +549,7 @@ NtlmGetUserNameFromResponse(
 DWORD
 NtlmGetDomainNameFromResponse(
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
+    IN DWORD dwRespMsgSize,
     IN BOOLEAN bUnicode,
     OUT PSTR* ppDomainName
     );
@@ -536,6 +557,7 @@ NtlmGetDomainNameFromResponse(
 DWORD
 NtlmGetWorkstationFromResponse(
     IN PNTLM_RESPONSE_MESSAGE_V1 pRespMsg,
+    IN DWORD dwRespMsgSize,
     IN BOOLEAN bUnicode,
     OUT PSTR* ppWorkstation
     );
