@@ -370,10 +370,10 @@ NtlmSetUnicodeStringEx(
     {
         dwError = LwMbsToWc16s(pszStr, &pUnicode->Buffer);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         dwError = LwWc16sLen(pUnicode->Buffer, &len);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         pUnicode->Length = (UINT16) (len * sizeof(WCHAR));
         pUnicode->MaximumLength = (UINT16) ((len + 1) * sizeof(WCHAR));
     }
@@ -435,7 +435,7 @@ NtlmServerMarshalUserInfoToEncodedPac(
     pInfo3->base.primary_gid = pUserInfo->dwPrimaryGroupRid;
     pInfo3->base.user_flags = pUserInfo->dwUserFlags;
     pInfo3->base.acct_flags = pUserInfo->dwAcctFlags;
-
+    
     pInfo3->base.groups.dwCount = pUserInfo->dwNumRids;
     pInfo3->base.groups.pRids = (PRID_WITH_ATTRIBUTE) pUserInfo->pRidAttribList;
 
@@ -497,7 +497,7 @@ NtlmServerMarshalUserInfoToEncodedPac(
     {
         dwError = LwAllocateMemory(pInfo3->sidcount * sizeof(NetrSidAttr), OUT_PPVOID(&pInfo3->sids));
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         for (i = 0; i < pInfo3->sidcount; i++)
         {
             pInfo3->sids[i].attribute = pUserInfo->pSidAttribList[i].dwAttrib;
@@ -509,11 +509,11 @@ NtlmServerMarshalUserInfoToEncodedPac(
         }
 
     }
-
+    
     if (EncodePacLogonInfo(
             &pac,
             pdwEncodedPacSize,
-            ppEncodedPac) != 0)
+            OUT_PPVOID(ppEncodedPac)) != 0)
     {
             dwError = LW_ERROR_INTERNAL;
             BAIL_ON_LSA_ERROR(dwError);
