@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -51,11 +51,11 @@
 #include <lw/rpc/netlogon.h>
 #include <lw/rpc/dssetup.h>
 #include <lwnet.h>
-#include <lwps/lwps.h>
 
 #include <lsa/lsa.h>
 #include <lsarpcsrv.h>
 #include <lsasrvutils.h>
+#include <lsasrvapi.h>
 #include <rpcctl-register.h>
 #include <directory.h>
 
