@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -71,7 +71,7 @@ LsaSrvInitAuthInfo(
 
     ntStatus = LwRpcStatusToNtStatus(rpcStatus);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
-
+     
     rpc_binding_inq_transport_info(hBinding,
                                    &hTransportInfo,
                                    &rpcStatus);
@@ -167,7 +167,7 @@ LsaSrvFreeAuthInfo(
 
     if (pPolCtx->pSessionKey)
     {
-        LW_SAFE_FREE_MEMORY(pPolCtx->pSessionKey);
+        LW_SECURE_FREE_MEMORY(pPolCtx->pSessionKey, pPolCtx->dwSessionKeyLen);
         pPolCtx->pSessionKey     = NULL;
         pPolCtx->dwSessionKeyLen = 0;
     }
@@ -182,40 +182,43 @@ LsaSrvGetSystemCreds(
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
     LW_PIO_CREDS pCreds = NULL;
-    PSTR pszUsername = NULL;
-    PSTR pszPassword = NULL;
-    PSTR pszDomainDnsName = NULL;
-    PSTR pszHostDnsDomain = NULL;
     PSTR pszMachinePrincipal = NULL;
+    PSTR pszCachePath = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
-    dwError = LwKrb5GetMachineCreds(
-                    &pszUsername,
-                    &pszPassword,
-                    &pszDomainDnsName,
-                    &pszHostDnsDomain);
+    dwError = LsaSrvProviderGetMachineAccountInfoA(
+                  LSA_PROVIDER_TAG_AD,
+                  NULL,
+                  &pAccountInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwAllocateStringPrintf(
                     &pszMachinePrincipal,
                     "%s@%s",
-                    pszUsername,
-                    pszDomainDnsName);
+                    pAccountInfo->SamAccountName,
+                    pAccountInfo->DnsDomainName);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LwAllocateStringPrintf(
+                    &pszCachePath,
+                    "%s.%s",
+                    LSASS_KRB5_CACHE_PATH,
+                    pAccountInfo->DnsDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwIoCreateKrb5CredsA(
                     pszMachinePrincipal,
-                    LSASS_KRB5_CACHE_PATH,
+                    pszCachePath,
                     &pCreds);
     BAIL_ON_LSA_ERROR(dwError);
 
     *ppCreds = pCreds;
 
 cleanup:
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_FREE_STRING(pszPassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
     LW_SAFE_FREE_STRING(pszMachinePrincipal);
+    LW_SAFE_FREE_STRING(pszCachePath);
+
+    LsaSrvFreeMachineAccountInfoA(pAccountInfo);
 
     if (ntStatus == STATUS_SUCCESS &&
         dwError != ERROR_SUCCESS)
