@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -58,7 +58,6 @@ typedef struct lsa_generic_context
 {
     enum LsaContextType  Type;
     LONG                 refcount;
-    BOOLEAN              bCleanClose;
 
 } LSA_GENERIC_CONTEXT, *PLSA_GENERIC_CONTEXT;
 
@@ -82,8 +81,8 @@ typedef struct lsa_policy_context
     PWSTR                pwszDomainName;
     PSID                 pDomainSid;
     PWSTR                pwszDcName;
-
-    PLSA_HASH_TABLE      pDomains;
+    
+    PLW_HASH_TABLE      pDomains;
     DWORD                dwDomainsNum;
 
 } POLICY_CONTEXT, *PPOLICY_CONTEXT;
