@@ -118,7 +118,6 @@ LsaSrvLookupNames3(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    DWORD dwError = ERROR_SUCCESS;
     PPOLICY_CONTEXT pPolCtx = NULL;
     PACCOUNT_NAMES pAccountNames = NULL;
     DWORD dwUnknownNamesNum = 0;
@@ -291,12 +290,6 @@ LsaSrvLookupNames3(
 cleanup:
     LsaSrvFreeAccountNames(pAccountNames);
 
-    if (ntStatus == STATUS_SUCCESS &&
-        dwError != ERROR_SUCCESS)
-    {
-        ntStatus = LwWin32ErrorToNtStatus(dwError);
-    }
-
     return ntStatus;
 
 error:
@@ -330,7 +323,6 @@ LsaSrvLookupDomainNames(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    DWORD dwError = ERROR_SUCCESS;
     PDOMAIN_ENTRY pDomEntry = NULL;
     handle_t hLsaBinding = NULL;
     POLICY_HANDLE hDcPolicy = NULL;
@@ -469,12 +461,6 @@ cleanup:
         LsaRpcFreeMemory(pSids);
     }
 
-    if (ntStatus == STATUS_SUCCESS &&
-        dwError != ERROR_SUCCESS)
-    {
-        ntStatus = LwWin32ErrorToNtStatus(dwError);
-    }
-
     return ntStatus;
 
 error:
@@ -882,10 +868,10 @@ LsaSrvLookupBuiltinNames(
                                          wszBuiltinDomainName);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
-    dwError = LwCreateWellKnownSid(WinBuiltinDomainSid,
-                                   NULL,
-                                   &pBuiltinDomainSid,
-                                   NULL);
+    dwError = LwAllocateWellKnownSid(WinBuiltinDomainSid,
+                                     NULL,
+                                     &pBuiltinDomainSid,
+                                     NULL);
     BAIL_ON_LSA_ERROR(dwError);
 
     ntStatus = LsaSrvDuplicateSid(&pBuiltinDomainInfo->sid,
