@@ -105,7 +105,6 @@ LsaSrvLookupSids2(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    DWORD dwError = ERROR_SUCCESS;
     DWORD dwLevel = level;
     PPOLICY_CONTEXT pPolCtx = NULL;
     PACCOUNT_SIDS pAccountSids = NULL;
@@ -259,12 +258,6 @@ cleanup:
         LsaSrvFreeAccountSids(pAccountSids);
     }
 
-    if (ntStatus == STATUS_SUCCESS &&
-        dwError != ERROR_SUCCESS)
-    {
-        ntStatus = LwWin32ErrorToNtStatus(dwError);
-    }
-
     return ntStatus;
 
 error:
@@ -640,6 +633,11 @@ LsaSrvLookupForeignDomainSids(
         }
         Sids.dwNumSids = 0;
 
+        if (pDomEntry)
+        {
+            LsaSrvDomainEntryFree(&pDomEntry);
+        }
+
         if (pForeignDomains)
         {
             LsaRpcFreeMemory(pForeignDomains);
@@ -687,6 +685,11 @@ cleanup:
     LW_SAFE_FREE_MEMORY(ForeignSids.pdwIndices);
     LW_SAFE_FREE_MEMORY(Sids.pSids);
 
+    if (pDomEntry)
+    {
+        LsaSrvDomainEntryFree(&pDomEntry);
+    }
+
     if (pForeignDomains)
     {
         LsaRpcFreeMemory(pForeignDomains);
@@ -878,10 +881,10 @@ LsaSrvLookupBuiltinSids(
     dwBuiltinDomIndex  = pDomains->count;
     pBuiltinDomainInfo = &(pDomains->domains[dwBuiltinDomIndex]);
 
-    dwError = LwCreateWellKnownSid(WinBuiltinDomainSid,
-                                   NULL,
-                                   &pBuiltinDomainSid,
-                                   NULL);
+    dwError = LwAllocateWellKnownSid(WinBuiltinDomainSid,
+                                     NULL,
+                                     &pBuiltinDomainSid,
+                                     NULL);
     BAIL_ON_LSA_ERROR(dwError);
 
     ntStatus = LsaSrvInitUnicodeStringEx(&pBuiltinDomainInfo->name,
