@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -86,10 +86,8 @@ LsaSrvOpenPolicy2(
     DWORD dwSidLength = 0;
     PSID pDomainSid = NULL;
     DOMAIN_HANDLE hDomain = (DOMAIN_HANDLE)NULL;
-    PSTR pszDomainFqdn = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
     PSTR pszDcFqdn = NULL;
-    HANDLE hPassStore = NULL;
-    PLWPS_PASSWORD_INFO pPassInfo = NULL;
 
     dwError = LwAllocateMemory(sizeof(*pPolCtx),
                                OUT_PPVOID(&pPolCtx));
@@ -211,14 +209,17 @@ LsaSrvOpenPolicy2(
         }
     }
 
-    dwError = LWNetGetCurrentDomain(&pszDomainFqdn);
+    dwError = LsaSrvProviderGetMachineAccountInfoA(
+                    LSA_PROVIDER_TAG_AD,
+                    NULL,
+                    &pAccountInfo);
     if (dwError == ERROR_SUCCESS)
     {
         /*
          * LSASS is a domain member so get some basic information
          * about the domain
          */
-        dwError = LWNetGetDomainController(pszDomainFqdn,
+        dwError = LWNetGetDomainController(pAccountInfo->DnsDomainName,
                                            &pszDcFqdn);
         if (dwError == DNS_ERROR_BAD_PACKET)
         {
@@ -238,27 +239,15 @@ LsaSrvOpenPolicy2(
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        dwError = LwpsOpenPasswordStore(LWPS_PASSWORD_STORE_DEFAULT,
-                                        &hPassStore);
+        dwError = LwMbsToWc16s(pAccountInfo->NetbiosDomainName,
+                               &pPolCtx->pwszDomainName);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LwpsGetPasswordByCurrentHostName(
-                                            hPassStore,
-                                            &pPassInfo);
-        BAIL_ON_LSA_ERROR(dwError);
-
-        if (pPassInfo)
-        {
-            dwError = LwAllocateWc16String(&pPolCtx->pwszDomainName,
-                                           pPassInfo->pwszDomainName);
-            BAIL_ON_LSA_ERROR(dwError);
-
-            ntStatus = RtlAllocateSidFromWC16String(&pPolCtx->pDomainSid,
-                                                    pPassInfo->pwszSID);
-            BAIL_ON_NTSTATUS_ERROR(ntStatus);
-        }
+        ntStatus = RtlAllocateSidFromCString(&pPolCtx->pDomainSid,
+                                             pAccountInfo->DomainSid);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
     }
-    else if (dwError == ERROR_NOT_JOINED)
+    else if (dwError == NERR_SetupNotJoined)
     {
         /*
          * LSASS is a standalone server
@@ -295,21 +284,7 @@ cleanup:
         LWNetFreeString(pszDcFqdn);
     }
 
-    if (pszDomainFqdn)
-    {
-        LWNetFreeString(pszDomainFqdn);
-    }
-
-    if (hPassStore && pPassInfo)
-    {
-        LwpsFreePasswordInfo(hPassStore,
-                             pPassInfo);
-    }
-
-    if (hPassStore)
-    {
-        LwpsClosePasswordStore(hPassStore);
-    }
+    LsaSrvFreeMachineAccountInfoA(pAccountInfo);
 
     LW_SAFE_FREE_MEMORY(pwszSystemName);
     LW_SAFE_FREE_MEMORY(pszSamrLpcSocketPath);
