@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -164,16 +164,13 @@ LsaQueryDomainInfo(
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
-    HANDLE hStore = NULL;
-    PLWPS_PASSWORD_INFO pPassInfo = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_W pAccountInfo = NULL;
 
-    ntStatus = LwpsOpenPasswordStore(LWPS_PASSWORD_STORE_DEFAULT,
-                                     &hStore);
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
-
-    dwError = LwpsGetPasswordByCurrentHostName(hStore,
-                                               &pPassInfo);
-    if (dwError == LWPS_ERROR_INVALID_ACCOUNT)
+    dwError = LsaSrvProviderGetMachineAccountInfoW(
+                  LSA_PROVIDER_TAG_AD,
+                  NULL,
+                  &pAccountInfo);
+    if (dwError == NERR_SetupNotJoined)
     {
         /* No password info means we're not joined */
         ntStatus = STATUS_INVALID_INFO_CLASS;
@@ -184,27 +181,19 @@ LsaQueryDomainInfo(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    if (pPassInfo)
+    if (pAccountInfo)
     {
         ntStatus = LsaSrvInitUnicodeStringEx(&pInfo->name,
-                                             pPassInfo->pwszDomainName);
-        BAIL_ON_NTSTATUS_ERROR(ntStatus);
+                                             pAccountInfo->NetbiosDomainName);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus); 
 
         ntStatus = LsaSrvAllocateSidFromWC16String(&pInfo->sid,
-                                                   pPassInfo->pwszSID);
+                                                   pAccountInfo->DomainSid);
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
     }
 
 cleanup:
-    if (pPassInfo != NULL)
-    {
-        LwpsFreePasswordInfo(hStore, pPassInfo);
-    }
-
-    if (hStore != NULL)
-    {
-        LwpsClosePasswordStore(hStore);
-    }
+    LsaSrvFreeMachineAccountInfoW(pAccountInfo);
 
     if (ntStatus == STATUS_SUCCESS &&
         dwError != ERROR_SUCCESS)
@@ -255,8 +244,7 @@ LsaQueryDnsDomainInfo(
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
-    HANDLE hStore = NULL;
-    PLWPS_PASSWORD_INFO pPassInfo = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_W pAccountInfo = NULL;
     PWSTR pwszDnsForest = NULL;
     PSTR pszDomainFqdn = NULL;
     PSTR pszDcFqdn = NULL;
@@ -264,13 +252,12 @@ LsaQueryDnsDomainInfo(
     DWORD dwFlags = 0;
     PLWNET_DC_INFO pDcInfo = NULL;
 
-    ntStatus = LwpsOpenPasswordStore(LWPS_PASSWORD_STORE_DEFAULT,
-                                     &hStore);
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
-    dwError = LwpsGetPasswordByCurrentHostName(hStore,
-                                               &pPassInfo);
-    if (dwError == LWPS_ERROR_INVALID_ACCOUNT)
+    dwError = LsaSrvProviderGetMachineAccountInfoW(
+                  LSA_PROVIDER_TAG_AD,
+                  NULL,
+                  &pAccountInfo);
+    if (dwError == NERR_SetupNotJoined)
     {
         ntStatus = STATUS_INVALID_INFO_CLASS;
         BAIL_ON_NT_STATUS(ntStatus);
@@ -280,24 +267,24 @@ LsaQueryDnsDomainInfo(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    if (pPassInfo)
+    if (pAccountInfo)
     {
         ntStatus = LsaSrvInitUnicodeStringEx(&pInfo->name,
-                                             pPassInfo->pwszHostname);
+                                             pAccountInfo->NetbiosDomainName);
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = LsaSrvInitUnicodeStringEx(&pInfo->dns_domain,
-                                             pPassInfo->pwszDnsDomainName);
+                                             pAccountInfo->DnsDomainName);
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = LsaSrvAllocateSidFromWC16String(&pInfo->sid,
-                                                   pPassInfo->pwszSID);
+                                                   pAccountInfo->DomainSid);
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
-        dwError = LwWc16sToMbs(pPassInfo->pwszDnsDomainName,
+        dwError = LwWc16sToMbs(pAccountInfo->DnsDomainName,
                                &pszDomainFqdn);
         BAIL_ON_LSA_ERROR(dwError);
-
+    
         dwError = LWNetGetDCName(pszDcFqdn,
                                  pszDomainFqdn,
                                  pszSiteName,
@@ -317,15 +304,7 @@ LsaQueryDnsDomainInfo(
     }
 
 cleanup:
-    if (pPassInfo)
-    {
-        LwpsFreePasswordInfo(hStore, pPassInfo);
-    }
-
-    if (hStore)
-    {
-        LwpsClosePasswordStore(hStore);
-    }
+    LsaSrvFreeMachineAccountInfoW(pAccountInfo);
 
     if (pDcInfo)
     {
