@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -73,6 +73,8 @@
 #include <lsautils.h>
 #include <rpcctl-register.h>
 #include <directory.h>
+#include <lsaadprovider.h>
+#include <lsasrvapi.h>
 
 #include "samr_cfg.h"
 #include "samr_srv.h"
