@@ -72,7 +72,7 @@ SamrSrvInitAuthInfo(
 
     ntStatus = LwRpcStatusToNtStatus(rpcStatus);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
-
+     
     rpc_binding_inq_transport_info(hBinding,
                                    &hTransportInfo,
                                    &rpcStatus);
@@ -168,7 +168,7 @@ SamrSrvFreeAuthInfo(
 
     if (pConnCtx->pSessionKey)
     {
-        LW_SAFE_FREE_MEMORY(pConnCtx->pSessionKey);
+        LW_SECURE_FREE_MEMORY(pConnCtx->pSessionKey, pConnCtx->dwSessionKeyLen);
         pConnCtx->pSessionKey     = NULL;
         pConnCtx->dwSessionKeyLen = 0;
     }
@@ -183,40 +183,43 @@ SamrSrvGetSystemCreds(
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
     LW_PIO_CREDS pCreds = NULL;
-    PSTR pszUsername = NULL;
-    PSTR pszPassword = NULL;
-    PSTR pszDomainDnsName = NULL;
-    PSTR pszHostDnsDomain = NULL;
     PSTR pszMachinePrincipal = NULL;
+    PSTR pszCachePath = NULL;
+    PLSA_MACHINE_ACCOUNT_INFO_A pAccountInfo = NULL;
 
-    dwError = LwKrb5GetMachineCreds(
-                    &pszUsername,
-                    &pszPassword,
-                    &pszDomainDnsName,
-                    &pszHostDnsDomain);
+    dwError = LsaSrvProviderGetMachineAccountInfoA(
+                  LSA_PROVIDER_TAG_AD,
+                  NULL,
+                  &pAccountInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwAllocateStringPrintf(
                     &pszMachinePrincipal,
                     "%s@%s",
-                    pszUsername,
-                    pszDomainDnsName);
+                    pAccountInfo->SamAccountName,
+                    pAccountInfo->DnsDomainName);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LwAllocateStringPrintf(
+                    &pszCachePath,
+                    "%s.%s",
+                    LSASS_KRB5_CACHE_PATH,
+                    pAccountInfo->DnsDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwIoCreateKrb5CredsA(
                     pszMachinePrincipal,
-                    LSASS_KRB5_CACHE_PATH,
+                    pszCachePath,
                     &pCreds);
     BAIL_ON_LSA_ERROR(dwError);
 
     *ppCreds = pCreds;
 
 cleanup:
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_FREE_STRING(pszPassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
     LW_SAFE_FREE_STRING(pszMachinePrincipal);
+    LW_SAFE_FREE_STRING(pszCachePath);
+
+    LsaSrvFreeMachineAccountInfoA(pAccountInfo);
 
     if (ntStatus == STATUS_SUCCESS &&
         dwError != ERROR_SUCCESS)
