@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -329,14 +329,13 @@ SamrSrvQueryUserInfo(
     /* [out] */ UserInfo **info
     )
 {
-    wchar_t wszFilterFmt[] = L"%ws='%ws'";
+    PCSTR filterFormat = "%s=%Q";
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = 0;
     PACCOUNT_CONTEXT pAcctCtx = NULL;
     PDOMAIN_CONTEXT pDomCtx = NULL;
     PCONNECT_CONTEXT pConnCtx = NULL;
     PWSTR pwszBase = NULL;
-    WCHAR wszAttrDn[] = DS_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrObjectSid[] = DS_ATTR_OBJECT_SID;
     WCHAR wszAttrSamAccountName[] = DS_ATTR_SAM_ACCOUNT_NAME;
     WCHAR wszAttrFullName[] = DS_ATTR_FULL_NAME;
@@ -363,9 +362,10 @@ SamrSrvQueryUserInfo(
     WCHAR wszAttrAccountExpiry[] = DS_ATTR_ACCOUNT_EXPIRY;
     WCHAR wszAttrLmHash[] = DS_ATTR_LM_HASH;
     WCHAR wszAttrNtHash[] = DS_ATTR_NT_HASH;
+    CHAR szAttrDn[] = DS_ATTR_DISTINGUISHED_NAME;
     DWORD dwScope = 0;
+    PSTR pszDn = NULL;
     PWSTR pwszFilter = NULL;
-    DWORD dwFilterLen = 0;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwEntriesNum = 0;
     UserInfo *pUserInfo = NULL;
@@ -577,16 +577,15 @@ SamrSrvQueryUserInfo(
 
     pwszBase = pDomCtx->pwszDn;
 
-    dwFilterLen = ((sizeof(wszAttrDn)/sizeof(WCHAR)) - 1) +
-                  wc16slen(pAcctCtx->pwszDn) +
-                  (sizeof(wszFilterFmt)/sizeof(wszFilterFmt[0]));
+    dwError = LwWc16sToMbs(pAcctCtx->pwszDn,
+                           &pszDn);
+    BAIL_ON_LSA_ERROR(dwError);
 
-    ntStatus = SamrSrvAllocateMemory((void**)&pwszFilter,
-                                   dwFilterLen * sizeof(WCHAR));
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
-
-    sw16printfw(pwszFilter, dwFilterLen, wszFilterFmt,
-                wszAttrDn, pAcctCtx->pwszDn);
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                              &pwszFilter,
+                              filterFormat,
+                              szAttrDn, pszDn);
+    BAIL_ON_LSA_ERROR(dwError);
 
     dwError = DirectorySearch(pConnCtx->hDirectory,
                               pwszBase,
@@ -742,15 +741,14 @@ SamrSrvQueryUserInfo(
     default:
         ntStatus = STATUS_INVALID_INFO_CLASS;
     }
-
+    
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
     *info = pUserInfo;
-
+    
 cleanup:
-    if (pwszFilter) {
-        SamrSrvFreeMemory(pwszFilter);
-    }
+    LW_SAFE_FREE_MEMORY(pszDn);
+    LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (pEntry) {
         DirectoryFreeEntries(pEntry, dwEntriesNum);
@@ -1353,10 +1351,10 @@ SamrFillUserInfo21(
 
     GET_UNICODE_STRING_VALUE(wszAttrHomeDrive, pInfo21->home_drive);
     pInfo21->fields_present |= SAMR_FIELD_HOME_DRIVE;
-
+    
     GET_UNICODE_STRING_VALUE(wszAttrLogonScript, pInfo21->logon_script);
     pInfo21->fields_present |= SAMR_FIELD_LOGON_SCRIPT;
-
+    
     GET_UNICODE_STRING_VALUE(wszAttrProfilePath, pInfo21->profile_path);
     pInfo21->fields_present |= SAMR_FIELD_PROFILE_PATH;
 
