@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -154,7 +154,7 @@ SamrSrvSetUserInfo(
             SET_NTTIME_VALUE(0, (idx), (mod));                    \
         }                                                         \
     } while (0);
-
+                               
 
 
 NTSTATUS
@@ -200,7 +200,7 @@ SamrSrvSetUserInfoInternal(
         ATTR_VAL_IDX_HOME_DIRECTORY,
         ATTR_VAL_IDX_HOME_DRIVE,
         ATTR_VAL_IDX_LOGON_SCRIPT,
-        ATTR_VAL_IDX_PROFILE_PATH,
+        ATTR_VAL_IDX_PROFILE_PATH, 
         ATTR_VAL_IDX_DESCRIPTION,
         ATTR_VAL_IDX_WORKSTATIONS,
         ATTR_VAL_IDX_COMMENT,
@@ -791,11 +791,7 @@ SamrSrvSetUserInfoInternal(
     }
 
 cleanup:
-    if (pwszPassword)
-    {
-        memset(pwszPassword, 0, dwPasswordLen * sizeof(WCHAR));
-        LW_SAFE_FREE_MEMORY(pwszPassword);
-    }
+    LW_SECURE_FREE_WSTRING(pwszPassword);
 
     for (i = ATTR_VAL_IDX_FULL_NAME;
          i < ATTR_VAL_IDX_SENTINEL;
@@ -825,18 +821,17 @@ SamrSrvCheckPasswordPolicy(
     IN  PWSTR            pwszPassword
     )
 {
-    wchar_t wszFilterFmt[] = L"%ws='%ws'";
+    PCSTR filterFormat = "%s=%Q";
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
     PDOMAIN_CONTEXT pDomCtx = NULL;
     PCONNECT_CONTEXT pConnCtx = NULL;
-    WCHAR wszAttrDn[] = DS_ATTR_DISTINGUISHED_NAME;
+    CHAR szAttrDn[] = DS_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrPasswordLastSet[] = DS_ATTR_PASSWORD_LAST_SET;
     PWSTR pwszBase = 0;
     DWORD dwScope = 0;
-    size_t sDnLen = 0;
-    DWORD dwFilterLen = 0;
     PWSTR pwszFilter = NULL;
+    PSTR pszDn = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
     LONG64 llLastPasswordChange = 0;
@@ -852,24 +847,16 @@ SamrSrvCheckPasswordPolicy(
     pConnCtx = pDomCtx->pConnCtx;
     pwszBase = pDomCtx->pwszDn;
 
-    dwError = LwWc16sLen(pAcctCtx->pwszDn, &sDnLen);
+    dwError = LwWc16sToMbs(pAcctCtx->pwszDn,
+                           &pszDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwFilterLen = ((sizeof(wszAttrDn)/sizeof(WCHAR)) - 1) +
-                  sDnLen +
-                  (sizeof(wszFilterFmt)/sizeof(wszFilterFmt[0]));
-
-    dwError = LwAllocateMemory(sizeof(WCHAR) * dwFilterLen,
-                               OUT_PPVOID(&pwszFilter));
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                              &pwszFilter,
+                              filterFormat,
+                              szAttrDn, pszDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (sw16printfw(pwszFilter, dwFilterLen, wszFilterFmt,
-                    wszAttrDn, pAcctCtx->pwszDn) < 0)
-    {
-        ntStatus = LwErrnoToNtStatus(errno);
-        BAIL_ON_NTSTATUS_ERROR(ntStatus);
-    }
-
     dwError = DirectorySearch(pConnCtx->hDirectory,
                               pwszBase,
                               dwScope,
@@ -929,6 +916,7 @@ cleanup:
         DirectoryFreeEntries(pEntry, dwNumEntries);
     }
 
+    LW_SAFE_FREE_MEMORY(pszDn);
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
     if (ntStatus == STATUS_SUCCESS &&
