@@ -70,34 +70,45 @@ DirectoryFreeEntries(
 
         if (pEntry->pAttributes)
         {
-            DWORD iDirAttr = 0;
-
-            for (; iDirAttr < pEntry->ulNumAttributes; iDirAttr++)
-            {
-                PDIRECTORY_ATTRIBUTE pDirAttr = NULL;
-
-                pDirAttr = &pEntry->pAttributes[iDirAttr];
-
-                if (pDirAttr->pwszName)
-                {
-                    DirectoryFreeStringW(pDirAttr->pwszName);
-                }
-
-                if (pDirAttr->pValues)
-                {
-                    DirectoryFreeAttributeValues(
-                            pDirAttr->pValues,
-                            pDirAttr->ulNumValues);
-                }
-            }
-
-            DirectoryFreeMemory(pEntry->pAttributes);
+            DirectoryFreeAttributes(
+                    pEntry->pAttributes,
+                    pEntry->ulNumAttributes);
         }
     }
 
     DirectoryFreeMemory(pEntries);
 }
 
+VOID
+DirectoryFreeAttributes(
+    PDIRECTORY_ATTRIBUTE pAttributes,
+    DWORD                dwNumAttributes
+    )
+{
+    DWORD iDirAttr = 0;
+
+    for (; iDirAttr < dwNumAttributes; iDirAttr++)
+    {
+        PDIRECTORY_ATTRIBUTE pDirAttr = NULL;
+
+        pDirAttr = &pAttributes[iDirAttr];
+
+        if (pDirAttr->pwszName)
+        {
+            DirectoryFreeStringW(pDirAttr->pwszName);
+        }
+
+        if (pDirAttr->pValues)
+        {
+            DirectoryFreeAttributeValues(
+                    pDirAttr->pValues,
+                    pDirAttr->ulNumValues);
+        }
+    }
+
+    DirectoryFreeMemory(pAttributes);
+}
+
 VOID
 DirectoryFreeAttributeValues(
     PATTRIBUTE_VALUE pAttrValues,
@@ -561,15 +572,15 @@ DirectorySetEntrySecurityDescriptor(
     PSECURITY_DESCRIPTOR_ABSOLUTE  pSecDesc
     )
 {
-    const wchar_t wszFilterFmt[] = L"%ws='%ws'";
+    PCSTR filterFormat = "%s=%Q";
     DWORD dwError = ERROR_SUCCESS;
     NTSTATUS ntStatus = STATUS_SUCCESS;
     WCHAR wszAttrDn[] = DIRECTORY_ATTR_DISTINGUISHED_NAME;
     WCHAR wszAttrSecurityDescriptor[] = DIRECTORY_ATTR_SECURITY_DESCRIPTOR;
+    CHAR szAttrDn[] = DIRECTORY_ATTR_DISTINGUISHED_NAME;
     PWSTR pwszBase = NULL;
     DWORD dwScope = 0;
-    size_t sDnLen = 0;
-    DWORD dwFilterLen = 0;
+    PSTR pszDn = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
     DWORD dwNumEntries = 0;
@@ -613,25 +624,15 @@ DirectorySetEntrySecurityDescriptor(
         BAIL_ON_DIRECTORY_ERROR(dwError);
     }
 
-    dwError = LwWc16sLen(pwszDn, &sDnLen);
+    dwError = LwWc16sToMbs(pwszDn, &pszDn);
     BAIL_ON_DIRECTORY_ERROR(dwError);
 
-    dwFilterLen = ((sizeof(wszAttrDn) / sizeof(WCHAR)) - 1) +
-                  sDnLen +
-                  (sizeof(wszFilterFmt) / sizeof(wszFilterFmt[0]));
-
-    dwError = LwAllocateMemory(dwFilterLen * sizeof(pwszFilter[0]),
-                               OUT_PPVOID(&pwszFilter));
+    dwError = DirectoryAllocateWC16StringFilterPrintf(
+                           &pwszFilter,
+                           filterFormat,
+                           szAttrDn, pszDn);
     BAIL_ON_DIRECTORY_ERROR(dwError);
 
-    if (sw16printfw(pwszFilter, dwFilterLen, wszFilterFmt,
-                    wszAttrDn,
-                    pwszDn) < 0)
-    {
-        ntStatus = LwErrnoToNtStatus(errno);
-        BAIL_ON_NTSTATUS_ERROR(ntStatus);
-    }
-
     dwError = DirectorySearch(hDirectory,
                               pwszBase,
                               dwScope,
@@ -691,6 +692,7 @@ DirectorySetEntrySecurityDescriptor(
     BAIL_ON_DIRECTORY_ERROR(dwError);
 
 cleanup:
+    DIRECTORY_FREE_MEMORY(pszDn);
     LW_SAFE_FREE_MEMORY(pwszFilter);
     LW_SAFE_FREE_MEMORY(pSecDescRel);
 
