@@ -330,6 +330,10 @@ SamDbAddObject(
                     modifications);
     BAIL_ON_SAMDB_ERROR(dwError);
 
+    dwError = SamDbIncrementSequenceNumber(
+                    pDirectoryContext);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
 cleanup:
 
    return dwError;
