@@ -142,6 +142,7 @@ SamDbGetNextAvailableUID(
     )
 {
     DWORD dwError = 0;
+    PSAM_DIRECTORY_CONTEXT pDirectoryContext = hDirectory;
     PCSTR pszQueryTemplate =
         "SELECT UIDCounter FROM " SAM_DB_CONFIG_TABLE \
         "; UPDATE " SAM_DB_CONFIG_TABLE               \
@@ -162,6 +163,10 @@ SamDbGetNextAvailableUID(
                         &dwUID);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        dwError = SamDbIncrementSequenceNumber_inlock(
+                        pDirectoryContext);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
         dwError = SamDbIsAvailableId(
                         hDirectory,
                         pszUIDQueryTemplate,
@@ -202,6 +207,7 @@ SamDbGetNextAvailableGID(
     )
 {
     DWORD dwError = 0;
+    PSAM_DIRECTORY_CONTEXT pDirectoryContext = hDirectory;
     PCSTR pszQueryTemplate =
         "SELECT GIDCounter FROM " SAM_DB_CONFIG_TABLE \
         "; UPDATE " SAM_DB_CONFIG_TABLE               \
@@ -222,6 +228,10 @@ SamDbGetNextAvailableGID(
                         &dwGID);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        dwError = SamDbIncrementSequenceNumber_inlock(
+                        pDirectoryContext);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
         dwError = SamDbIsAvailableId(
                         hDirectory,
                         pszGIDQueryTemplate,
@@ -325,6 +335,10 @@ SamDbGetNextAvailableRID(
                         &dwRID);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        dwError = SamDbIncrementSequenceNumber_inlock(
+                        pDirContext);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
         dwError = SamDbIsAvailableId(
                         hDirectory,
                         pszObjectRIDQueryTemplate,
@@ -536,18 +550,15 @@ cleanup:
     return dwError;
 
 error:
-
-    *pdwId = 0;
+    if (pdwId)
+    {
+        *pdwId = 0;
+    }
 
     SAMDB_LOG_DEBUG("Sqlite3 Error (code: %u): %s",
                     dwError,
                     LSA_SAFE_LOG_STRING(pszError));
 
-    if (pszError)
-    {
-        sqlite3_free(pszError);
-    }
-
     goto cleanup;
 }
 
