@@ -1667,7 +1667,7 @@ SamDbSetupLocalGroupMemberships(
         "Guest",
         NULL
     };
-
+    
     struct group_members {
         PCSTR pszGroupName;
         PCSTR *ppszMembers;
@@ -1804,7 +1804,7 @@ SamDbSetupLocalGroupMemberships(
                 LW_SAFE_FREE_MEMORY(pwszMemberName);
             }
         }
-
+                                                       
         if (pGroupEntries)
         {
             for (i = 0; i < dwGroupEntriesNum; i++)
@@ -2260,6 +2260,9 @@ SamDbFixLocalAccounts(
                                         pwszUserObjectDN,
                                         Mods);
             BAIL_ON_SAMDB_ERROR(dwError);
+
+            iMod = 0;
+            memset(&Mods, 0, sizeof(Mods));
         }
     }
 
