@@ -274,6 +274,10 @@ SamDbModifyObject(
                     modifications);
     BAIL_ON_SAMDB_ERROR(dwError);
 
+    dwError = SamDbIncrementSequenceNumber(
+                    pDirectoryContext);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
 cleanup:
 
     DIRECTORY_FREE_STRING(pszObjectDN);
@@ -1002,7 +1006,7 @@ SamDbModifyDomainName(
         pszAccountDN        = NULL;
         pszAccountQuery     = NULL;
     }
-
+    
     /*
      * Get all objects and update DN wherever it includes "DC=MACHINE"
      */
@@ -1352,7 +1356,6 @@ SamDbModifyDomainSid(
     sqlite3_stmt* pSqlStatement = NULL;
     DWORD iModified = 0;
     DWORD iMod = 0;
-    DWORD dwNumMods = 0;
     PWSTR pwszNewDomainSid = NULL;
     PSID pNewDomainSid = NULL;
     DWORD dwDomainFilterLen = 0;
@@ -1468,8 +1471,6 @@ SamDbModifyDomainSid(
         iMod++;
     }
 
-    dwNumMods = iMod;
-
     /*
      * Validate the SID
      */
@@ -1903,7 +1904,6 @@ SamDbModifyObjectSid(
     DWORD dwObjectClass = 0;
     PWSTR pwszNewObjectSid = NULL;
     PSID pNewObjectSid = NULL;
-    DWORD dwNumMods = 0;
     DWORD iMod = 0;
     DWORD dwDomainFilterLen = 0;
     PWSTR pwszDomainFilter = NULL;
@@ -2012,8 +2012,6 @@ SamDbModifyObjectSid(
         iMod++;
     }
 
-    dwNumMods = iMod;
-
     /*
      * Validate the SID
      */
@@ -2087,7 +2085,7 @@ SamDbModifyObjectSid(
      */
     if ((!RtlIsPrefixSid(pDomainSid, pNewObjectSid)) ||
         (pNewObjectSid->SubAuthorityCount != pDomainSid->SubAuthorityCount + 1))
-
+        
     {
         dwError = LW_ERROR_INVALID_SID;
         BAIL_ON_SAMDB_ERROR(dwError);
