@@ -108,11 +108,8 @@ SamDbSearchObject(
     )
 {
     DWORD dwError = 0;
-    PSAM_DIRECTORY_CONTEXT pDirectoryContext = NULL;
     BOOLEAN bInLock = FALSE;
 
-    pDirectoryContext = (PSAM_DIRECTORY_CONTEXT)hDirectory;
-
     SAMDB_LOCK_RWMUTEX_SHARED(bInLock, &gSamGlobals.rwLock);
 
     dwError = SamDbSearchObject_inlock(
@@ -796,6 +793,11 @@ error:
     *ppDirectoryEntries = NULL;
     *pdwNumEntries = 0;
 
+    if (pAttrs)
+    {
+        DirectoryFreeAttributes(pAttrs, dwNumAttrs);
+    }
+
     if (pDirectoryEntries)
     {
         DirectoryFreeEntries(pDirectoryEntries, dwTotalEntries);
