@@ -83,7 +83,13 @@ SamDbSetPassword(
                     hBindHandle,
                     pwszUserDN,
                     pwszPassword);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
+    dwError = SamDbIncrementSequenceNumber_inlock(
+                    pDirectoryContext);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
+error:
     SAMDB_UNLOCK_RWMUTEX(bInLock, &gSamGlobals.rwLock);
 
     return dwError;
@@ -206,7 +212,7 @@ cleanup:
         sqlite3_finalize(pSqlStatement);
     }
 
-    DIRECTORY_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
     DIRECTORY_FREE_STRING(pszUserDN);
 
     return dwError;
@@ -225,11 +231,8 @@ SamDbChangePassword(
     )
 {
     DWORD dwError = 0;
-    PSAM_DIRECTORY_CONTEXT pDirectoryContext = NULL;
     BOOLEAN bInLock = FALSE;
 
-    pDirectoryContext = (PSAM_DIRECTORY_CONTEXT)hBindHandle;
-
     SAMDB_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &gSamGlobals.rwLock);
 
     dwError = SamDbVerifyPassword_inlock(
@@ -263,11 +266,8 @@ SamDbVerifyPassword(
     )
 {
     DWORD dwError = 0;
-    PSAM_DIRECTORY_CONTEXT pDirectoryContext = NULL;
     BOOLEAN bInLock = FALSE;
 
-    pDirectoryContext = (PSAM_DIRECTORY_CONTEXT)hBindHandle;
-
     SAMDB_LOCK_RWMUTEX_SHARED(bInLock, &gSamGlobals.rwLock);
 
     dwError = SamDbVerifyPassword_inlock(
@@ -422,7 +422,7 @@ cleanup:
         sqlite3_finalize(pSqlStatement);
     }
 
-    DIRECTORY_FREE_STRING(pszPassword);
+    LW_SECURE_FREE_STRING(pszPassword);
     DIRECTORY_FREE_STRING(pszUserDN);
 
     return dwError;
