@@ -150,7 +150,7 @@ main(
 error:
     if (dwError)
     {
-        printf("Finished with error code: %d\n", dwError);
+        printf("Finished with error code: %u\n", dwError);
         LwGetErrorString(dwError, Error, 256);
         printf("%s\n", Error);
     }
