@@ -51,6 +51,9 @@
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
 
 DWORD
@@ -101,14 +104,14 @@ main(
         fprintf(stderr, "Mismatch between original string and output string:\n");
         fprintf(stderr, "Original: %s\n", pszSIDStr);
         fprintf(stderr, "Output:   %s\n", pszSIDStrFeedback);
-        dwError = LSA_ERROR_INVALID_SID;
+        dwError = LW_ERROR_INVALID_SID;
     }
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LsaGetSecurityIdentifierBinary(pSID, &pucSIDByteArr, &dwSIDByteCount);
     BAIL_ON_LSA_ERROR(dwError);
 
-    printf("LsaGetSecurityIdentifierBinary() successful, byte count = %d\n",
+    printf("LsaGetSecurityIdentifierBinary() successful, byte count = %u\n",
            dwSIDByteCount);
 
     dwError = LsaByteArrayToHexStr(pucSIDByteArr, dwSIDByteCount, &pszSIDHexStr);
@@ -120,9 +123,9 @@ main(
 
 cleanup:
 
-    LSA_SAFE_FREE_STRING(pszSIDHexStr);
-    LSA_SAFE_FREE_STRING(pszSIDStr);
-    LSA_SAFE_FREE_MEMORY(pucSIDByteArr);
+    LW_SAFE_FREE_STRING(pszSIDHexStr);
+    LW_SAFE_FREE_STRING(pszSIDStr);
+    LW_SAFE_FREE_MEMORY(pucSIDByteArr);
 
     if (pSID)
     {
@@ -175,7 +178,7 @@ ParseArgs(
                 else
                 {
 
-                    dwError = LsaAllocateString(pszArg, &pszSIDStr);
+                    dwError = LwAllocateString(pszArg, &pszSIDStr);
                     BAIL_ON_LSA_ERROR(dwError);
                 }
                 break;
@@ -184,7 +187,7 @@ ParseArgs(
 
     } while (iArg < argc);
 
-    if (IsNullOrEmptyString(pszSIDStr)) {
+    if (LW_IS_NULL_OR_EMPTY_STR(pszSIDStr)) {
        fprintf(stderr, "Please specify a SID string, i.e. S-1-5-532.\n");
        ShowUsage();
        exit(1);
@@ -198,7 +201,7 @@ cleanup:
 
 error:
 
-    LSA_SAFE_FREE_STRING(pszSIDStr);
+    LW_SAFE_FREE_STRING(pszSIDStr);
 
     *ppszSIDStr = NULL;
 
