@@ -51,6 +51,9 @@
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
 
 static
@@ -106,18 +109,18 @@ main(
 
     while ((pUser = getpwent()) != NULL)
     {
-          printf("User #%d", ++iUser);
+          printf("User #%u", ++iUser);
           printf("==========\n");
           printf("Name:     %s\n",
-                  IsNullOrEmptyString(pUser->pw_name) ? "<null>" : pUser->pw_name);
+                  LW_IS_NULL_OR_EMPTY_STR(pUser->pw_name) ? "<null>" : pUser->pw_name);
           printf("Uid:      %u\n", (unsigned int)pUser->pw_uid);
           printf("Gid:      %u\n", (unsigned int)pUser->pw_gid);
           printf("Gecos:    %s\n", 
-                  IsNullOrEmptyString(pUser->pw_gecos) ? "<null>" : pUser->pw_gecos);
+                  LW_IS_NULL_OR_EMPTY_STR(pUser->pw_gecos) ? "<null>" : pUser->pw_gecos);
           printf("Shell:    %s\n",
-                  IsNullOrEmptyString(pUser->pw_shell) ? "<null>" : pUser->pw_shell);
+                  LW_IS_NULL_OR_EMPTY_STR(pUser->pw_shell) ? "<null>" : pUser->pw_shell);
           printf("Home dir: %s\n",
-                  IsNullOrEmptyString(pUser->pw_dir) ? "<null>" : pUser->pw_dir);
+                  LW_IS_NULL_OR_EMPTY_STR(pUser->pw_dir) ? "<null>" : pUser->pw_dir);
     }
 
 cleanup:
