@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -76,6 +76,7 @@ ParseArgs(
     int    argc,
     char*  argv[],
     DWORD* pdwAction,
+    PSTR*  ppszDomainName,
     PSTR*  ppszName,
     uid_t* pUID,
     gid_t* pGID,
@@ -92,6 +93,7 @@ static
 DWORD
 EnumerateUsers(
     HANDLE hLsaConnection,
+    PCSTR pszDomainName,
     DWORD  dwBatchSize
     );
 
@@ -99,6 +101,7 @@ static
 DWORD
 EnumerateGroups(
     HANDLE hLsaConnection,
+    PCSTR pszDomainName,
     DWORD  dwBatchSize
     );
 
@@ -108,12 +111,6 @@ MapErrorCode(
     DWORD dwError
     );
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    );
-
 int
 ad_cache_main(
     int argc,
@@ -126,6 +123,7 @@ ad_cache_main(
     BOOLEAN bPrintOrigError = TRUE;
     PSTR    pszOperation = "complete operation";
     DWORD   dwAction = ACTION_NONE;
+    PSTR    pszDomainName = NULL;
     PSTR    pszName = NULL;
     uid_t   uid = 0;
     gid_t   gid = 0;
@@ -149,6 +147,7 @@ ad_cache_main(
                   argc,
                   argv,
                   &dwAction,
+                  &pszDomainName,
                   &pszName,
                   &uid,
                   &gid,
@@ -163,7 +162,9 @@ ad_cache_main(
         case ACTION_DELETE_ALL:
             pszOperation = "empty cache";
 
-            dwError = LsaAdEmptyCache(hLsaConnection);
+            dwError = LsaAdEmptyCache(
+                          hLsaConnection,
+                          pszDomainName);
             BAIL_ON_LSA_ERROR(dwError);
 
             fprintf(stdout, "The cache has been emptied successfully.\n");
@@ -177,6 +178,7 @@ ad_cache_main(
             {
                 dwError = LsaAdRemoveUserByNameFromCache(
                               hLsaConnection,
+                              pszDomainName,
                               pszName);
                 BAIL_ON_LSA_ERROR(dwError);
             }
@@ -184,6 +186,7 @@ ad_cache_main(
             {
                 dwError = LsaAdRemoveUserByIdFromCache(
                               hLsaConnection,
+                              pszDomainName,
                               uid);
                 BAIL_ON_LSA_ERROR(dwError);
             }
@@ -199,6 +202,7 @@ ad_cache_main(
             {
                 dwError = LsaAdRemoveGroupByNameFromCache(
                               hLsaConnection,
+                              pszDomainName,
                               pszName);
                 BAIL_ON_LSA_ERROR(dwError);
             }
@@ -206,6 +210,7 @@ ad_cache_main(
             {
                 dwError = LsaAdRemoveGroupByIdFromCache(
                               hLsaConnection,
+                              pszDomainName,
                               gid);
                 BAIL_ON_LSA_ERROR(dwError);
             }
@@ -219,6 +224,7 @@ ad_cache_main(
 
             dwError = EnumerateUsers(
                           hLsaConnection,
+                          pszDomainName,
                           dwBatchSize);
             BAIL_ON_LSA_ERROR(dwError);
 
@@ -229,6 +235,7 @@ ad_cache_main(
 
             dwError = EnumerateGroups(
                           hLsaConnection,
+                          pszDomainName,
                           dwBatchSize);
             BAIL_ON_LSA_ERROR(dwError);
 
@@ -246,22 +253,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(stderr,
@@ -273,10 +280,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(stderr,
@@ -285,7 +292,7 @@ error:
                 dwError,
                 LW_PRINTF_STRING(LwWin32ExtErrorToName(dwError)));
     }
-
+    
     goto cleanup;
 }
 
@@ -319,6 +326,7 @@ ParseArgs(
     int    argc,
     char*  argv[],
     DWORD* pdwAction,
+    PSTR*  ppszDomainName,
     PSTR*  ppszName,
     uid_t* pUID,
     gid_t* pGID,
@@ -327,6 +335,7 @@ ParseArgs(
 {
     typedef enum {
             PARSE_MODE_OPEN = 0,
+            PARSE_MODE_DOMAIN_NAME,
             PARSE_MODE_NAME,
             PARSE_MODE_UID,
             PARSE_MODE_GID,
@@ -339,6 +348,7 @@ ParseArgs(
     PSTR pszArg = NULL;
     ParseMode parseMode = PARSE_MODE_OPEN;
     DWORD dwAction = ACTION_NONE;
+    PSTR  pszDomainName = NULL;
     PSTR  pszName = NULL;
     uid_t uid = 0;
     gid_t gid = 0;
@@ -363,7 +373,6 @@ ParseArgs(
                 }
                 else if (!strcmp(pszArg, "--delete-all")) {
                     dwAction = ACTION_DELETE_ALL;
-                    parseMode = PARSE_MODE_DONE;
                 }
                 else if (!strcmp(pszArg, "--delete-user")) {
                     dwAction = ACTION_DELETE_USER;
@@ -377,6 +386,9 @@ ParseArgs(
                 else if (!strcmp(pszArg, "--enum-groups")) {
                     dwAction = ACTION_ENUM_GROUPS;
                 }
+                else if (!strcmp(pszArg, "--domain")) {
+                    parseMode = PARSE_MODE_DOMAIN_NAME;
+                }
                 else if (!strcmp(pszArg, "--name")) {
                     parseMode = PARSE_MODE_NAME;
                 }
@@ -396,6 +408,13 @@ ParseArgs(
                 }
                 break;
 
+            case PARSE_MODE_DOMAIN_NAME:
+                dwError = LwAllocateString(pszArg, &pszDomainName);
+                BAIL_ON_LSA_ERROR(dwError);
+                parseMode = PARSE_MODE_OPEN;
+
+                break;
+
             case PARSE_MODE_NAME:
                 dwError = LwAllocateString(pszArg, &pszName);
                 BAIL_ON_LSA_ERROR(dwError);
@@ -435,7 +454,7 @@ ParseArgs(
                     exit(1);
                 }
                 dwBatchSize = atoi(pszArg);
-                if ((dwBatchSize < 0) ||
+                if ((dwBatchSize == 0) ||
                     (dwBatchSize > 1000)) {
                     fprintf(stderr, "Please enter a valid batch size.\n");
                     ShowUsage(GetProgramName(argv[0]));
@@ -486,6 +505,7 @@ ParseArgs(
     }
 
     *pdwAction = dwAction;
+    *ppszDomainName = pszDomainName;
     *ppszName = pszName;
     *pUID = uid;
     *pGID = gid;
@@ -498,12 +518,14 @@ cleanup:
 error:
 
     *pdwAction = ACTION_NONE;
+    *ppszDomainName = NULL;
     *ppszName = NULL;
     *pUID = 0;
     *pGID = 0;
     *pdwBatchSize = 0;
 
     LW_SAFE_FREE_STRING(pszName);
+    LW_SAFE_FREE_STRING(pszDomainName);
 
     goto cleanup;
 }
@@ -514,11 +536,11 @@ ShowUsage(
     PCSTR pszProgramName
     )
 {
-    fprintf(stdout, "Usage: %s --delete-all\n", pszProgramName);
-    fprintf(stdout, "       %s --delete-user {--name <user login id> | --uid <uid>} \n", pszProgramName);
-    fprintf(stdout, "       %s --delete-group {--name <group name> | --gid <gid>} \n", pszProgramName);
-    fprintf(stdout, "       %s --enum-users {--batchsize [1..1000]}\n", pszProgramName);
-    fprintf(stdout, "       %s --enum-groups {--batchsize [1..1000]}\n\n", pszProgramName);
+    fprintf(stdout, "Usage: %s --delete-all [--domain domain]\n", pszProgramName);
+    fprintf(stdout, "       %s --delete-user [--domain domain] {--name <user login id> | --uid <uid>} \n", pszProgramName);
+    fprintf(stdout, "       %s --delete-group [--domain domain] {--name <group name> | --gid <gid>} \n", pszProgramName);
+    fprintf(stdout, "       %s --enum-users [--domain domain] {--batchsize [1..1000]}\n", pszProgramName);
+    fprintf(stdout, "       %s --enum-groups [--domain domain] {--batchsize [1..1000]}\n\n", pszProgramName);
     fprintf(stdout, "\t--delete-all        Deletes everything from the cache\n");
     fprintf(stdout, "\t--delete-user       Deletes one user from the cache\n");
     fprintf(stdout, "\t--delete-group      Deletes one group from the cache\n");
@@ -530,6 +552,7 @@ static
 DWORD
 EnumerateUsers(
     HANDLE hLsaConnection,
+    PCSTR pszDomainName,
     DWORD  dwBatchSize
     )
 {
@@ -548,11 +571,12 @@ EnumerateUsers(
         ppObjects = NULL;
 
         dwError = LsaAdEnumUsersFromCache(
-                    hLsaConnection,
-                    &pszResume,
-                    dwBatchSize,
-                    &dwNumUsersFound,
-                    &ppObjects);
+                      hLsaConnection,
+                      pszDomainName,
+                      &pszResume,
+                      dwBatchSize,
+                      &dwNumUsersFound,
+                      &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
 
         if (!dwNumUsersFound)
@@ -588,6 +612,7 @@ static
 DWORD
 EnumerateGroups(
     HANDLE hLsaConnection,
+    PCSTR pszDomainName,
     DWORD  dwBatchSize
     )
 {
@@ -606,11 +631,12 @@ EnumerateGroups(
         ppObjects = NULL;
 
         dwError = LsaAdEnumGroupsFromCache(
-                    hLsaConnection,
-                    &pszResume,
-                    dwBatchSize,
-                    &dwNumGroupsFound,
-                    &ppObjects);
+                      hLsaConnection,
+                      pszDomainName,
+                      &pszResume,
+                      dwBatchSize,
+                      &dwNumGroupsFound,
+                      &ppObjects);
         BAIL_ON_LSA_ERROR(dwError);
 
         if (!dwNumGroupsFound)
@@ -648,98 +674,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
-
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    )
-{
-    typedef enum {
-        PARSE_MODE_LEADING_SPACE = 0,
-        PARSE_MODE_INTEGER,
-        PARSE_MODE_TRAILING_SPACE
-    } ParseMode;
-
-    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
-    BOOLEAN bIsUnsignedInteger = TRUE;
-    INT iLength = 0;
-    INT iCharIdx = 0;
-    CHAR cNext = '\0';
-
-    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
-    {
-        bIsUnsignedInteger = FALSE;
-        goto error;
-    }
-
-    iLength = strlen(pszIntegerCandidate);
-
-    do {
-
-      cNext = pszIntegerCandidate[iCharIdx++];
-
-      switch(parseMode) {
-
-          case PARSE_MODE_LEADING_SPACE:
-          {
-              if (isdigit((int)cNext))
-              {
-                  parseMode = PARSE_MODE_INTEGER;
-              }
-              else if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_INTEGER:
-          {
-              if (isspace((int)cNext))
-              {
-                  parseMode = PARSE_MODE_TRAILING_SPACE;
-              }
-              else if (!isdigit((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_TRAILING_SPACE:
-          {
-              if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-      }
-
-    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
-
-
-error:
-
-    return bIsUnsignedInteger;
-}
-
