@@ -54,12 +54,6 @@ MapErrorCode(
     DWORD dwError
     );
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    );
-
 static
 PSTR
 LsaGetProgramName(
@@ -280,20 +274,7 @@ GetGroupId(
                     &pGroupInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    switch(dwGroupInfoLevel)
-    {
-        case 0:
-        {
-            gid = ((PLSA_GROUP_INFO_0)pGroupInfo)->gid;
-            break;
-        }
-        default:
-        {
-            dwError = LW_ERROR_INVALID_GROUP_INFO_LEVEL;
-            BAIL_ON_LSA_ERROR(dwError);
-            break;
-        }
-    }
+    gid = ((PLSA_GROUP_INFO_0)pGroupInfo)->gid;
 
     *pGid = gid;
 
@@ -418,6 +399,10 @@ cleanup:
         LsaCloseServer(hLsaConnection);
     }
 
+    if (pUserInfo) {
+        LsaFreeUserInfo(dwUserInfoLevel, pUserInfo);
+    }
+
     return dwError;
 
 error:
@@ -566,83 +551,6 @@ MapErrorCode(
     return dwError2;
 }
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    )
-{
-    typedef enum {
-        PARSE_MODE_LEADING_SPACE = 0,
-        PARSE_MODE_INTEGER,
-        PARSE_MODE_TRAILING_SPACE
-    } ParseMode;
-
-    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
-    BOOLEAN bIsUnsignedInteger = TRUE;
-    INT iLength = 0;
-    INT iCharIdx = 0;
-    CHAR cNext = '\0';
-
-    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
-    {
-        bIsUnsignedInteger = FALSE;
-        goto error;
-    }
-
-    iLength = strlen(pszIntegerCandidate);
-
-    do {
-
-      cNext = pszIntegerCandidate[iCharIdx++];
-
-      switch(parseMode) {
-
-          case PARSE_MODE_LEADING_SPACE:
-          {
-              if (isdigit((int)cNext))
-              {
-                  parseMode = PARSE_MODE_INTEGER;
-              }
-              else if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_INTEGER:
-          {
-              if (isspace((int)cNext))
-              {
-                  parseMode = PARSE_MODE_TRAILING_SPACE;
-              }
-              else if (!isdigit((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_TRAILING_SPACE:
-          {
-              if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-      }
-
-    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
-
-
-error:
-
-    return bIsUnsignedInteger;
-}
-
-
 /*
 local variables:
 mode: c
