@@ -43,7 +43,6 @@
 #include "lsadef.h"
 #include "lsaclient.h"
 #include <lsa/lsa.h>
-#include <lsa/lsa2.h>
 #include <lwmem.h>
 #include <lwerror.h>
 
@@ -67,7 +66,7 @@ PrintSecurityObject(
         }
         else
         {
-            printf("Group object [%u] (%s)\n", dwObjectNumber+1, SAFE_STRING(pObject->pszObjectSid));
+            printf("Group object [%u] (%s)\n", dwObjectNumber+1, SAFE_STRING(pObject->pszObjectSid));   
         }
         printf("============\n");
         printf("Enabled: %s\n", pObject->enabled ? "yes" : "no");
@@ -102,7 +101,7 @@ PrintSecurityObject(
         }
         printf("Alias: %s\n", SAFE_STRING(pObject->userInfo.pszAliasName));
         printf("UNIX name: %s\n", SAFE_STRING(pObject->userInfo.pszUnixName));
-        printf("GECOS: %s\n", SAFE_STRING(pObject->userInfo.pszGecos));
+        printf("GECOS: %s\n", SAFE_STRING(pObject->userInfo.pszGecos));        
         printf("Shell: %s\n", SAFE_STRING(pObject->userInfo.pszShell));
         printf("Home directory: %s\n", SAFE_STRING(pObject->userInfo.pszHomedir));
         printf("UID: %lu\n", (unsigned long) pObject->userInfo.uid);
@@ -111,7 +110,7 @@ PrintSecurityObject(
         if (pObject->userInfo.bIsAccountInfoKnown)
         {
             printf("Password expired: %s\n", pObject->userInfo.bPasswordExpired ? "yes" : "no");
-            printf("Password nevers expires: %s\n", pObject->userInfo.bPasswordNeverExpires ? "yes" : "no");
+            printf("Password never expires: %s\n", pObject->userInfo.bPasswordNeverExpires ? "yes" : "no");
             printf("Change password on next logon: %s\n", pObject->userInfo.bPromptPasswordChange ? "yes" : "no");
             printf("User can change password: %s\n", pObject->userInfo.bUserCanChangePassword ? "yes" : "no");
             printf("Account disabled: %s\n", pObject->userInfo.bAccountDisabled ? "yes" : "no");
@@ -141,3 +140,113 @@ Basename(
         return pszPath;
     }
 }
+
+VOID
+PrintErrorMessage(
+    IN DWORD ErrorCode
+    )
+{
+    PCSTR pszErrorName = LwWin32ExtErrorToName(ErrorCode);
+    PSTR pszErrorMessage = NULL;
+    DWORD size = LwGetErrorString(ErrorCode, NULL, 0);
+
+    if (size > 0)
+    {
+        DWORD dwError = LwAllocateMemory(size, OUT_PPVOID(&pszErrorMessage));
+        if (!dwError)
+        {
+            (void) LwGetErrorString(ErrorCode, pszErrorMessage, size);
+        }
+    }
+
+    if (!LW_IS_NULL_OR_EMPTY_STR(pszErrorMessage))
+    {
+        fprintf(stderr,
+                "Error code %u (%s).\n%s\n",
+                ErrorCode,
+                LW_PRINTF_STRING(pszErrorName),
+                pszErrorMessage);
+    }
+    else
+    {
+        fprintf(stderr,
+                "Error code %u (%s).\n",
+                ErrorCode,
+                LW_PRINTF_STRING(pszErrorName));
+    }
+
+    LW_SAFE_FREE_STRING(pszErrorMessage);
+}
+
+BOOLEAN
+IsUnsignedInteger(
+    PCSTR pszIntegerCandidate
+    )
+{
+    typedef enum {
+        PARSE_MODE_LEADING_SPACE = 0,
+        PARSE_MODE_INTEGER,
+        PARSE_MODE_TRAILING_SPACE
+    } ParseMode;
+
+    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
+    BOOLEAN bIsUnsignedInteger = TRUE;
+    INT iLength = 0;
+    INT iCharIdx = 0;
+    CHAR cNext = '\0';
+
+    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
+    {
+        bIsUnsignedInteger = FALSE;
+        goto error;
+    }
+
+    iLength = strlen(pszIntegerCandidate);
+
+    do {
+
+      cNext = pszIntegerCandidate[iCharIdx++];
+
+      switch(parseMode) {
+          case PARSE_MODE_LEADING_SPACE:
+          {
+              if (isdigit((int)cNext))
+              {
+                  parseMode = PARSE_MODE_INTEGER;
+              }
+              else if (!isspace((int)cNext))
+              {
+                  bIsUnsignedInteger = FALSE;
+              }
+              break;
+          }
+
+          case PARSE_MODE_INTEGER:
+          {
+              if (isspace((int)cNext))
+              {
+                  parseMode = PARSE_MODE_TRAILING_SPACE;
+              }
+              else if (!isdigit((int)cNext))
+              {
+                  bIsUnsignedInteger = FALSE;
+              }
+              break;
+          }
+          case PARSE_MODE_TRAILING_SPACE:
+          {
+              if (!isspace((int)cNext))
+              {
+                  bIsUnsignedInteger = FALSE;
+              }
+              break;
+          }
+      }
+
+    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
+
+
+error:
+
+    return bIsUnsignedInteger;
+}
