@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -54,12 +54,6 @@ MapErrorCode(
     DWORD dwError
     );
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    );
-
 static
 PSTR
 LsaGetProgramName(
@@ -117,15 +111,15 @@ ParseArgs(
     PSTR pArg = NULL;
     PSTR pszGid = NULL;
     PSTR pszGroup = NULL;
-
+    
     do {
 
       pArg = argv[iArg++];
-
+      
       if (pArg == NULL || *pArg == '\0') {
         break;
       }
-
+      
       switch(parseMode) {
 
           case PARSE_MODE_OPEN:
@@ -143,27 +137,27 @@ ParseArgs(
                   BAIL_ON_LSA_ERROR(dwError);
                   parseMode = PARSE_MODE_DONE;
               }
-
+              
               break;
           }
-
+    
           case PARSE_MODE_GID:
-          {
+          {          
               if (!IsUnsignedInteger(pArg))
               {
                 fprintf(stderr, "Please specifiy a gid that is an unsigned integer\n");
                 ShowUsage(LsaGetProgramName(argv[0]));
                 exit(1);
-              }
-
+              }         
+              
               dwError = LwAllocateString(pArg, &pszGid);
               BAIL_ON_LSA_ERROR(dwError);
-
+    
               parseMode = PARSE_MODE_OPEN;
-
+    
               break;
           }
-
+            
           case PARSE_MODE_DONE:
           {
               ShowUsage(LsaGetProgramName(argv[0]));
@@ -176,18 +170,18 @@ ParseArgs(
     if (parseMode != PARSE_MODE_OPEN && parseMode != PARSE_MODE_DONE)
     {
         ShowUsage(LsaGetProgramName(argv[0]));
-        exit(1);
+        exit(1);  
     }
 
     if (!pszGid && !pszGroup)
     {
         ShowUsage(LsaGetProgramName(argv[0]));
-        exit(1);
+        exit(1);  
     }
-
+    
     *ppszGid = pszGid;
     *ppszGroup = pszGroup;
-
+    
 cleanup:
 
     return dwError;
@@ -199,7 +193,7 @@ error:
 
     *ppszGid = NULL;
     *ppszGroup = NULL;
-
+    
     goto cleanup;
 }
 
@@ -215,7 +209,7 @@ LsaDelGroupMain(
     BOOLEAN bPrintOrigError = TRUE;
     PSTR pszGid = NULL;
     PSTR pszGroup = NULL;
-
+    
     if (geteuid() != 0) {
         fprintf(stderr, "This program requires super-user privileges.\n");
         dwError = LW_ERROR_ACCESS_DENIED;
@@ -228,16 +222,16 @@ LsaDelGroupMain(
                     &pszGid,
                     &pszGroup);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
 
     if (!LW_IS_NULL_OR_EMPTY_STR(pszGid))
     {
         int   nRead = 0;
         gid_t gid   = 0;
-
+        
         nRead = sscanf(pszGid, "%u", (unsigned int*)&gid);
         if ((nRead == EOF) || (nRead == 0))
         {
@@ -264,7 +258,7 @@ LsaDelGroupMain(
         ShowUsage(LsaGetProgramName(argv[0]));
         exit(1);
     }
-
+    
     fprintf(stdout, "The group has been deleted successfully.\n");
 
 cleanup:
@@ -273,27 +267,30 @@ cleanup:
         LsaCloseServer(hLsaConnection);
     }
 
+    LW_SAFE_FREE_STRING(pszGid);
+    LW_SAFE_FREE_STRING(pszGroup);
+
     return dwError;
 
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(stderr,
@@ -304,10 +301,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(stderr,
@@ -326,97 +323,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
-
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    )
-{
-    typedef enum {
-        PARSE_MODE_LEADING_SPACE = 0,
-        PARSE_MODE_INTEGER,
-        PARSE_MODE_TRAILING_SPACE
-    } ParseMode;
-
-    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
-    BOOLEAN bIsUnsignedInteger = TRUE;
-    INT iLength = 0;
-    INT iCharIdx = 0;
-    CHAR cNext = '\0';
-
-    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
-    {
-        bIsUnsignedInteger = FALSE;
-        goto error;
-    }
-
-    iLength = strlen(pszIntegerCandidate);
-
-    do {
-
-      cNext = pszIntegerCandidate[iCharIdx++];
-
-      switch(parseMode) {
-
-          case PARSE_MODE_LEADING_SPACE:
-          {
-              if (isdigit((int)cNext))
-              {
-                  parseMode = PARSE_MODE_INTEGER;
-              }
-              else if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_INTEGER:
-          {
-              if (isspace((int)cNext))
-              {
-                  parseMode = PARSE_MODE_TRAILING_SPACE;
-              }
-              else if (!isdigit((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_TRAILING_SPACE:
-          {
-              if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-      }
-
-    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
-
-
-error:
-
-    return bIsUnsignedInteger;
-}
