@@ -45,7 +45,6 @@
 #include "common.h"
 
 #include <lsa/lsa.h>
-#include <lsa/lsa2.h>
 #include <lwmem.h>
 #include <lwerror.h>
 
@@ -58,7 +57,7 @@ static struct
     DWORD dwCount;
     LSA_QUERY_LIST QueryList;
     BOOLEAN bShowUsage;
-} gState =
+} gState = 
 {
     .pszTargetProvider = NULL,
     .FindFlags = 0,
@@ -85,7 +84,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.pdwIds,
             OUT_PPVOID(&NewList.pdwIds),
-            sizeof(NewList.pdwIds) * (gState.dwCount + 1));
+            sizeof(*NewList.pdwIds) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.pdwIds[gState.dwCount] = (DWORD) id;
         gState.QueryList.pdwIds = NewList.pdwIds;
@@ -95,7 +94,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.ppszStrings,
             OUT_PPVOID(&NewList.ppszStrings),
-            sizeof(NewList.ppszStrings) * (gState.dwCount + 1));
+            sizeof(*NewList.ppszStrings) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.ppszStrings[gState.dwCount] = pszArg;
         gState.QueryList.ppszStrings = NewList.ppszStrings;
@@ -141,7 +140,7 @@ ShowUsage(
     printf(
         "Usage: %s [ --<object type> ] [ --by-<key> ] [ <flags> ] [ --provider name ] groups ...\n",
         Basename(pszProgramName));
-
+    
     if (bFull)
     {
         printf(
@@ -230,7 +229,7 @@ ParseArguments(
         else if (!strcmp(ppszArgv[i], "--provider"))
         {
             i++;
-
+                
             if (i >= argc)
             {
                 dwError = LW_ERROR_INVALID_PARAMETER;
@@ -288,7 +287,7 @@ ResolveSid(
             SingleList.ppszStrings = &QueryList.ppszStrings[dwIndex];
             break;
         }
-
+        
         dwError = LsaFindObjects(
             hLsa,
             gState.pszTargetProvider,
@@ -308,7 +307,7 @@ ResolveSid(
     }
 
 error:
-
+    
     if (ppObjects)
     {
         LsaFreeSecurityObjectList(1, ppObjects);
@@ -398,7 +397,7 @@ EnumMembers(
                 gState.FindFlags,
                 pszSid);
             BAIL_ON_LSA_ERROR(dwError);
-
+            
             for (;;)
             {
                 dwError = LsaEnumMembers(
@@ -413,12 +412,12 @@ EnumMembers(
                     break;
                 }
                 BAIL_ON_LSA_ERROR(dwError);
-
+                
                 dwError = ResolveMembers(hLsa, dwCount, ppszMembers, &ppObjects);
                 BAIL_ON_LSA_ERROR(dwError);
-
+                
                 for (dwIndex = 0; dwIndex < dwCount; dwIndex++, dwTotalIndex++)
-                {
+                {                  
                     if (ppObjects[dwIndex])
                     {
                         if (ppObjects[dwIndex]->type == gState.ObjectType || gState.ObjectType == LSA_OBJECT_TYPE_UNDEFINED)
@@ -438,7 +437,7 @@ EnumMembers(
                 LsaFreeSidList(dwCount, ppszMembers);
                 ppszMembers = NULL;
             }
-
+            
             LW_SAFE_FREE_MEMORY(pszSid);
             LsaCloseEnum(hLsa, hEnum);
         }
@@ -457,7 +456,7 @@ cleanup:
     {
         LsaFreeSidList(dwCount, ppszMembers);
     }
-
+    
     if (hEnum)
     {
         LsaCloseEnum(hLsa, hEnum);
@@ -499,7 +498,7 @@ EnumMembersMain(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         dwError = EnumMembers();
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -518,6 +517,13 @@ error:
     if (dwError)
     {
         printf("Error: %s (%x)\n", LwWin32ExtErrorToName(dwError), dwError);
+
+        if (dwError == ERROR_INVALID_DATA)
+        {
+            fprintf(stderr, "The members list has changed while enumerating. "
+                    "Try again.\n");
+        }
+
         return 1;
     }
     else
@@ -525,3 +531,13 @@ error:
         return 0;
     }
 }
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
