@@ -44,7 +44,6 @@
 #include "lsaclient.h"
 #include "common.h"
 #include <lsa/lsa.h>
-#include <lsa/lsa2.h>
 #include <lwmem.h>
 #include <lwerror.h>
 
@@ -59,7 +58,7 @@ static struct
     DWORD dwCount;
     LSA_QUERY_LIST QueryList;
     BOOLEAN bShowUsage;
-} gState =
+} gState = 
 {
     .pszTargetProvider = NULL,
     .FindFlags = 0,
@@ -86,7 +85,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.pdwIds,
             OUT_PPVOID(&NewList.pdwIds),
-            sizeof(NewList.pdwIds) * (gState.dwCount + 1));
+            sizeof(*NewList.pdwIds) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.pdwIds[gState.dwCount] = (DWORD) id;
         gState.QueryList.pdwIds = NewList.pdwIds;
@@ -96,7 +95,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.ppszStrings,
             OUT_PPVOID(&NewList.ppszStrings),
-            sizeof(NewList.ppszStrings) * (gState.dwCount + 1));
+            sizeof(*NewList.ppszStrings) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.ppszStrings[gState.dwCount] = pszArg;
         gState.QueryList.ppszStrings = NewList.ppszStrings;
@@ -142,7 +141,7 @@ ShowUsage(
     printf(
         "Usage: %s [ --<object type> ] [ --by-<key> ] [ <flags> ] [ --provider name ] keys ...\n",
         Basename(pszProgramName));
-
+    
     if (bFull)
     {
         printf(
@@ -168,7 +167,7 @@ ShowUsage(
             "\n");
     }
 }
-
+    
 static
 DWORD
 ParseArguments(
@@ -231,7 +230,7 @@ ParseArguments(
         else if (!strcmp(ppszArgv[i], "--provider"))
         {
             i++;
-
+                
             if (i >= argc)
             {
                 dwError = LW_ERROR_INVALID_PARAMETER;
@@ -298,7 +297,7 @@ FindObjects(
                        (unsigned long) gState.QueryList.pdwIds[dwIndex]);
                 break;
             default:
-                printf("Not found [%u of %u]: %s\n",
+                printf("Not found [%u of %u]: %s\n", 
                        dwIndex + 1, gState.dwCount,
                        gState.QueryList.ppszStrings[dwIndex]);
                 break;
