@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,16 +37,14 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Test Program for exercising LsaFindUserById
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
@@ -54,6 +52,7 @@
 
 #include "lsaclient.h"
 #include "lsaipc.h"
+#include "common.h"
 
 #define LW_PRINTF_STRING(x) ((x) ? (x) : "<null>")
 
@@ -97,12 +96,6 @@ MapErrorCode(
     DWORD dwError
     );
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    );
-
 int
 find_user_by_id_main(
     int argc,
@@ -117,12 +110,12 @@ find_user_by_id_main(
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
     BOOLEAN bAllowedLogon = TRUE;
-
+    
     ParseArgs(argc, argv, &uid, &dwInfoLevel);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaFindUserById(
                     hLsaConnection,
                     uid,
@@ -138,7 +131,7 @@ find_user_by_id_main(
     {
         bAllowedLogon = FALSE;
     }
-
+    
     switch(dwInfoLevel)
     {
         case 0:
@@ -154,7 +147,7 @@ find_user_by_id_main(
                             bAllowedLogon);
             break;
         default:
-
+            
             fprintf(stderr, "Error: Invalid user info level [%u]\n", dwInfoLevel);
             break;
     }
@@ -164,7 +157,7 @@ cleanup:
     if (pUserInfo) {
        LsaFreeUserInfo(dwInfoLevel, pUserInfo);
     }
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
@@ -174,22 +167,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(stderr,
@@ -200,10 +193,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(stderr,
@@ -242,11 +235,11 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
@@ -262,9 +255,9 @@ ParseArgs(
                     {
                         fprintf(stderr, "Please enter a uid which is an unsigned integer.\n");
                         ShowUsage();
-                        exit(1);
+                        exit(1); 
                     }
-
+                    
                     int nRead = sscanf(pszArg, "%u", (unsigned int*)&uid);
                     if ((nRead == EOF) || (nRead == 0)) {
                        fprintf(stderr, "A valid uid was not specified\n");
@@ -275,33 +268,33 @@ ParseArgs(
                     parseMode = PARSE_MODE_DONE;
                 }
                 break;
-
+                
             case PARSE_MODE_LEVEL:
                 if (!IsUnsignedInteger(pszArg))
                 {
                     fprintf(stderr, "Please enter an info level which is an unsigned integer.\n");
                     ShowUsage();
-                    exit(1);
+                    exit(1); 
                 }
                 dwInfoLevel = atoi(pszArg);
                 parseMode = PARSE_MODE_OPEN;
-
+                
                 break;
-
+                
             case PARSE_MODE_DONE:
                 ShowUsage();
-                exit(1);
-
+                exit(1);     
+                
         }
-
+        
     } while (iArg < argc);
 
     if (parseMode != PARSE_MODE_OPEN && parseMode != PARSE_MODE_DONE)
     {
         ShowUsage();
-        exit(1);
+        exit(1);  
     }
-
+    
     if (!bUidSpecified) {
         fprintf(stderr, "Please specify a uid to query for.\n");
         ShowUsage();
@@ -423,96 +416,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
-
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    )
-{
-    typedef enum {
-        PARSE_MODE_LEADING_SPACE = 0,
-        PARSE_MODE_INTEGER,
-        PARSE_MODE_TRAILING_SPACE
-    } ParseMode;
-
-    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
-    BOOLEAN bIsUnsignedInteger = TRUE;
-    INT iLength = 0;
-    INT iCharIdx = 0;
-    CHAR cNext = '\0';
-
-    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
-    {
-        bIsUnsignedInteger = FALSE;
-        goto error;
-    }
-
-    iLength = strlen(pszIntegerCandidate);
-
-    do {
-
-      cNext = pszIntegerCandidate[iCharIdx++];
-
-      switch(parseMode) {
-
-          case PARSE_MODE_LEADING_SPACE:
-          {
-              if (isdigit((int)cNext))
-              {
-                  parseMode = PARSE_MODE_INTEGER;
-              }
-              else if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_INTEGER:
-          {
-              if (isspace((int)cNext))
-              {
-                  parseMode = PARSE_MODE_TRAILING_SPACE;
-              }
-              else if (!isdigit((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_TRAILING_SPACE:
-          {
-              if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-      }
-
-    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
-
-
-error:
-
-    return bIsUnsignedInteger;
-}
