@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,16 +37,14 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Tool to get the LSASS log information
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
@@ -91,18 +89,18 @@ get_log_info_main(
     HANDLE hLsaConnection = (HANDLE)NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
-
+    
     dwError = ParseArgs(argc, argv);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaGetLogInfo(
                     hLsaConnection,
                     &pLogInfo);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = PrintLogInfo(pLogInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -111,7 +109,7 @@ cleanup:
     if (pLogInfo) {
        LsaFreeLogInfo(pLogInfo);
     }
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
@@ -121,22 +119,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(
@@ -148,10 +146,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(
@@ -160,7 +158,7 @@ error:
             dwError,
             LW_PRINTF_STRING(LwWin32ExtErrorToName(dwError)));
     }
-
+    
     goto cleanup;
 }
 
@@ -173,7 +171,7 @@ ParseArgs(
     typedef enum {
             PARSE_MODE_OPEN = 0
         } ParseMode;
-
+        
     DWORD dwError = 0;
     int iArg = 1;
     PSTR pszArg = NULL;
@@ -185,11 +183,11 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
@@ -203,9 +201,9 @@ ParseArgs(
                 }
                 break;
         }
-
+        
     } while (iArg < argc);
-
+    
     return dwError;
 }
 
@@ -221,7 +219,7 @@ PrintLogInfo(
     )
 {
     DWORD dwError = 0;
-
+    
     fprintf(stdout, "Current log settings:\n");
     fprintf(stdout, "=================\n");
     switch(pLogInfo->logTarget)
@@ -243,7 +241,7 @@ PrintLogInfo(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     fprintf(stdout, "Maximum allowed log level: ");
     switch(pLogInfo->maxAllowedLogLevel)
     {
@@ -269,7 +267,7 @@ PrintLogInfo(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 error:
 
     return dwError;
@@ -281,21 +279,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
