@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,16 +37,14 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Tool to get performance metrics from LSA Server
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
@@ -54,6 +52,7 @@
 
 #include "lsaclient.h"
 #include "lsaipc.h"
+#include "common.h"
 
 #define LW_PRINTF_STRING(x) ((x) ? (x) : "<null>")
 
@@ -87,12 +86,6 @@ MapErrorCode(
     DWORD dwError
     );
 
-static
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    );
-
 int
 get_metrics_main(
     int argc,
@@ -105,18 +98,18 @@ get_metrics_main(
     HANDLE hLsaConnection = (HANDLE)NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
-
+    
     ParseArgs(argc, argv, &dwInfoLevel);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaGetMetrics(
                     hLsaConnection,
                     dwInfoLevel,
                     &pMetricPack);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     switch (dwInfoLevel)
     {
         case 0:
@@ -138,7 +131,7 @@ get_metrics_main(
 cleanup:
 
     LW_SAFE_FREE_MEMORY(pMetricPack);
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
@@ -148,22 +141,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(
@@ -175,10 +168,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(
@@ -203,7 +196,7 @@ ParseArgs(
             PARSE_MODE_LEVEL,
             PARSE_MODE_DONE
         } ParseMode;
-
+        
     int iArg = 1;
     PSTR pszArg = NULL;
     ParseMode parseMode = PARSE_MODE_OPEN;
@@ -215,11 +208,11 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
@@ -236,36 +229,36 @@ ParseArgs(
                 }
 
                 break;
-
+                
             case PARSE_MODE_LEVEL:
-
+                
                 if (!IsUnsignedInteger(pszArg))
                 {
                     fprintf(stderr, "Please enter an info level which is an unsigned integer.\n");
                     ShowUsage();
-                    exit(1);
+                    exit(1); 
                 }
-
+                
                 dwInfoLevel = atoi(pszArg);
                 parseMode = PARSE_MODE_DONE;
-
+                
                 break;
-
+                
             case PARSE_MODE_DONE:
-
+                
                 ShowUsage();
                 exit(1);
 
         }
-
+        
     } while (iArg < argc);
 
     if (parseMode != PARSE_MODE_OPEN && parseMode != PARSE_MODE_DONE)
     {
         ShowUsage();
-        exit(1);
+        exit(1);  
     }
-
+    
     *pdwInfoLevel = dwInfoLevel;
 }
 
@@ -320,97 +313,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
-
-BOOLEAN
-IsUnsignedInteger(
-    PCSTR pszIntegerCandidate
-    )
-{
-    typedef enum {
-        PARSE_MODE_LEADING_SPACE = 0,
-        PARSE_MODE_INTEGER,
-        PARSE_MODE_TRAILING_SPACE
-    } ParseMode;
-
-    ParseMode parseMode = PARSE_MODE_LEADING_SPACE;
-    BOOLEAN bIsUnsignedInteger = TRUE;
-    INT iLength = 0;
-    INT iCharIdx = 0;
-    CHAR cNext = '\0';
-
-    if (LW_IS_NULL_OR_EMPTY_STR(pszIntegerCandidate))
-    {
-        bIsUnsignedInteger = FALSE;
-        goto error;
-    }
-
-    iLength = strlen(pszIntegerCandidate);
-
-    do {
-
-      cNext = pszIntegerCandidate[iCharIdx++];
-
-      switch(parseMode) {
-
-          case PARSE_MODE_LEADING_SPACE:
-          {
-              if (isdigit((int)cNext))
-              {
-                  parseMode = PARSE_MODE_INTEGER;
-              }
-              else if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_INTEGER:
-          {
-              if (isspace((int)cNext))
-              {
-                  parseMode = PARSE_MODE_TRAILING_SPACE;
-              }
-              else if (!isdigit((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-
-          case PARSE_MODE_TRAILING_SPACE:
-          {
-              if (!isspace((int)cNext))
-              {
-                  bIsUnsignedInteger = FALSE;
-              }
-              break;
-          }
-      }
-
-    } while (iCharIdx < iLength && bIsUnsignedInteger == TRUE);
-
-
-error:
-
-    return bIsUnsignedInteger;
-}
-
