@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,23 +37,24 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Tool to get status from LSA Server
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lsa/ad.h"
 
 #include "lsaclient.h"
 #include "lsaipc.h"
+#include "lsaadprovider.h"
+#include "lsalocalprovider.h"
 
 #define LSA_MODE_STRING_UNKNOWN          "Unknown"
 #define LSA_MODE_STRING_UNPROVISIONED    "Un-provisioned"
@@ -80,8 +81,10 @@
 static
 VOID
 ParseArgs(
-    int    argc,
-    char*  argv[]
+    IN int    argc,
+    IN char*  argv[],
+    OUT PBOOLEAN pbAll,
+    OUT PCSTR* ppszDomainName
     );
 
 static
@@ -91,7 +94,8 @@ ShowUsage();
 static
 VOID
 PrintStatus(
-    PLSASTATUS pStatus
+    PLSASTATUS pStatus,
+    PBOOLEAN bpPrintedServerStatus
     );
 
 static
@@ -136,50 +140,132 @@ get_status_main(
     HANDLE hLsaConnection = (HANDLE)NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
-
-    ParseArgs(argc, argv);
+    BOOLEAN bAll = FALSE;
+    PSTR pszProviderInstance = NULL;
+    PSTR* ppszDomains = NULL;
+    DWORD dwNumDomains = 0;
+    DWORD dwIndex = 0;
+    BOOLEAN bPrintedServerStatus = FALSE;
+    PCSTR pszDomainName = NULL;
+    
+    ParseArgs(argc, argv, &bAll, &pszDomainName);
 
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
+    
+    if (bAll)
+    {
+        dwError = LsaAdGetJoinedDomains(
+                      hLsaConnection,
+                      &dwNumDomains,
+                      &ppszDomains);
+        BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaGetStatus(
-                    hLsaConnection,
-                    &pLsaStatus);
-    BAIL_ON_LSA_ERROR(dwError);
+        if (dwNumDomains)
+        {
+            for (dwIndex = 0 ; dwIndex < dwNumDomains ; dwIndex++)
+            {
+                dwError = LwAllocateStringPrintf(
+                              &pszProviderInstance,
+                              "%s:%s",
+                              LSA_PROVIDER_TAG_AD,
+                              ppszDomains[dwIndex]);
+                BAIL_ON_LSA_ERROR(dwError);
 
-    PrintStatus(pLsaStatus);
+                dwError = LsaGetStatus2(
+                              hLsaConnection,
+                              pszProviderInstance,
+                              &pLsaStatus);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                PrintStatus(pLsaStatus, &bPrintedServerStatus);
+
+                LsaFreeStatus(pLsaStatus);
+                pLsaStatus = NULL;
+
+                LW_SAFE_FREE_STRING(pszProviderInstance);
+            }
+        }
+        else
+        {
+            dwError = LsaGetStatus2(
+                          hLsaConnection,
+                          LSA_PROVIDER_TAG_AD,
+                          &pLsaStatus);
+            BAIL_ON_LSA_ERROR(dwError);
+
+            PrintStatus(pLsaStatus, &bPrintedServerStatus);
+
+            LsaFreeStatus(pLsaStatus);
+            pLsaStatus = NULL;
+
+            LW_SAFE_FREE_STRING(pszProviderInstance);
+        }
+
+        dwError = LsaGetStatus2(
+                      hLsaConnection,
+                      LSA_PROVIDER_TAG_LOCAL,
+                      &pLsaStatus);
+        BAIL_ON_LSA_ERROR(dwError);
+    
+        PrintStatus(pLsaStatus, &bPrintedServerStatus);
+    }
+    else
+    {
+        if(pszDomainName)
+        {
+            dwError = LwAllocateStringPrintf(
+                          &pszProviderInstance,
+                          "%s:%s",
+                          LSA_PROVIDER_TAG_AD,
+                          pszDomainName);
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+
+        dwError = LsaGetStatus2(
+                      hLsaConnection,
+                      pszProviderInstance,
+                      &pLsaStatus);
+        BAIL_ON_LSA_ERROR(dwError);
+    
+        PrintStatus(pLsaStatus, &bPrintedServerStatus);
+    }
 
 cleanup:
 
     if (pLsaStatus) {
        LsaFreeStatus(pLsaStatus);
     }
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
 
+    LwFreeStringArray(ppszDomains, dwNumDomains);
+
+    LW_SAFE_FREE_STRING(pszProviderInstance);
+
     return (dwError);
 
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(
@@ -191,10 +277,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(
@@ -209,17 +295,21 @@ error:
 
 VOID
 ParseArgs(
-    int    argc,
-    char*  argv[]
+    IN int    argc,
+    IN char*  argv[],
+    OUT PBOOLEAN pbAll,
+    OUT PCSTR* ppszDomainName
     )
 {
     typedef enum {
             PARSE_MODE_OPEN = 0
         } ParseMode;
-
+        
     int iArg = 1;
     PSTR pszArg = NULL;
     ParseMode parseMode = PARSE_MODE_OPEN;
+    BOOLEAN bAll = FALSE;
+    PCSTR pszDomainName = NULL;
 
     do {
         pszArg = argv[iArg++];
@@ -227,72 +317,90 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
                     ShowUsage();
                     exit(0);
                 }
+                else if (strcmp(pszArg, "--all") == 0)
+                {
+                    bAll = TRUE;
+                }
                 else
                 {
-                    ShowUsage();
-                    exit(1);
+                    if (pszDomainName)
+                    {
+                        ShowUsage();
+                        exit(1);
+                    }
+
+                    pszDomainName = pszArg;
                 }
 
                 break;
         }
-
+        
     } while (iArg < argc);
+
+    *pbAll = bAll;
+    *ppszDomainName = pszDomainName;
 }
 
 void
 ShowUsage()
 {
-    printf("Usage: lw-get-status\n");
+    printf("Usage: lw-get-status [--all] [domain]\n");
 }
 
 VOID
 PrintStatus(
-    PLSASTATUS pStatus
+    PLSASTATUS pStatus,
+    PBOOLEAN pbPrintedServerStatus
     )
 {
     DWORD iCount = 0;
     DWORD dwDays = pStatus->dwUptime/LSA_SECONDS_IN_DAY;
     DWORD dwHours = (pStatus->dwUptime - (dwDays * LSA_SECONDS_IN_DAY))/LSA_SECONDS_IN_HOUR;
-    DWORD dwMins = (pStatus->dwUptime -
-                    (dwDays * LSA_SECONDS_IN_DAY) -
+    DWORD dwMins = (pStatus->dwUptime - 
+                    (dwDays * LSA_SECONDS_IN_DAY) - 
                     (dwHours * LSA_SECONDS_IN_HOUR))/LSA_SECONDS_IN_MINUTE;
-    DWORD dwSecs = (pStatus->dwUptime -
-                    (dwDays * LSA_SECONDS_IN_DAY) -
-                    (dwHours * LSA_SECONDS_IN_HOUR) -
+    DWORD dwSecs = (pStatus->dwUptime - 
+                    (dwDays * LSA_SECONDS_IN_DAY) - 
+                    (dwHours * LSA_SECONDS_IN_HOUR) - 
                     (dwMins * LSA_SECONDS_IN_MINUTE));
 
-    printf("LSA Server Status:\n\n");
-    printf("Compiled daemon version: %u.%u.%u.%u\n",
+    if (!*pbPrintedServerStatus)
+    {
+        printf("LSA Server Status:\n\n");
+        printf("Compiled daemon version: %u.%u.%u.%u\n",
                     pStatus->lsassVersion.dwMajor,
                     pStatus->lsassVersion.dwMinor,
                     pStatus->lsassVersion.dwBuild,
                     pStatus->lsassVersion.dwRevision);
-    printf("Packaged product version: %u.%u.%u.%u\n",
+        printf("Packaged product version: %u.%u.%u.%u\n",
                     pStatus->productVersion.dwMajor,
                     pStatus->productVersion.dwMinor,
                     pStatus->productVersion.dwBuild,
                     pStatus->productVersion.dwRevision);
-    printf("Uptime:        %u days %u hours %u minutes %u seconds\n", dwDays, dwHours, dwMins, dwSecs);
+        printf("Uptime:        %u days %u hours %u minutes %u seconds\n", dwDays, dwHours, dwMins, dwSecs);
 
+        *pbPrintedServerStatus = TRUE;
+    }
+    
     for (iCount = 0; iCount < pStatus->dwCount; iCount++)
     {
         PLSA_AUTH_PROVIDER_STATUS pProviderStatus =
             &pStatus->pAuthProviderStatusList[iCount];
-
-        printf("\n[Authentication provider: %s]\n\n",
+        
+        printf("\n[Authentication provider: %s]\n\n", 
                         LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszId) ? "" : pProviderStatus->pszId);
-
+        
         printf("\tStatus:        %s\n", GetStatusString(pProviderStatus->status));
         printf("\tMode:          %s\n", GetModeString(pProviderStatus->mode));
 
@@ -301,57 +409,57 @@ PrintStatus(
             case LSA_PROVIDER_MODE_LOCAL_SYSTEM:
                 printf("\tDomain:        %s\n", LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszDomain) ? "" : pProviderStatus->pszDomain);
                 break;
-
+            
             case LSA_PROVIDER_MODE_UNPROVISIONED:
             case LSA_PROVIDER_MODE_DEFAULT_CELL:
             case LSA_PROVIDER_MODE_NON_DEFAULT_CELL:
-
+                
                 printf("\tDomain:        %s\n", LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszDomain) ? "" : pProviderStatus->pszDomain);
                 printf("\tForest:        %s\n", LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszForest) ? "" : pProviderStatus->pszForest);
                 printf("\tSite:          %s\n", LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszSite) ? "" : pProviderStatus->pszSite);
                 printf("\tOnline check interval:  %u seconds\n", pProviderStatus->dwNetworkCheckInterval);
 
                 break;
-
+                
             default:
-
+                
                 break;
         }
-
+        
         switch (pProviderStatus->mode)
         {
             case LSA_PROVIDER_MODE_DEFAULT_CELL:
-
+                
                 printf("\tSub mode:      %s\n", GetSubmodeString(pProviderStatus->subMode));
-
+                
                 break;
-
+                
             case LSA_PROVIDER_MODE_NON_DEFAULT_CELL:
-
+                
                 printf("\tSub mode:      %s\n", GetSubmodeString(pProviderStatus->subMode));
-
+                
                 printf("\tCell:          %s\n", LW_IS_NULL_OR_EMPTY_STR(pProviderStatus->pszCell) ? "" : pProviderStatus->pszCell);
-
+                
                 break;
-
+                
             default:
-
+                
                 break;
         }
-
+        
         if (pProviderStatus->pTrustedDomainInfoArray)
         {
             DWORD iDomain = 0;
-
+            
             printf("\t[Trusted Domains: %u]\n\n", pProviderStatus->dwNumTrustedDomains);
-
+            
             for (; iDomain < pProviderStatus->dwNumTrustedDomains; iDomain++)
             {
                 PLSA_TRUSTED_DOMAIN_INFO pDomainInfo =
                     &pProviderStatus->pTrustedDomainInfoArray[iDomain];
-
+                
                 printf("\n\t[Domain: %s]\n\n", LW_IS_NULL_OR_EMPTY_STR(pDomainInfo->pszNetbiosDomain) ? "" : pDomainInfo->pszNetbiosDomain);
-
+                
                 printf("\t\tDNS Domain:       %s\n", LW_IS_NULL_OR_EMPTY_STR(pDomainInfo->pszDnsDomain) ? "" : pDomainInfo->pszDnsDomain);
                 printf("\t\tNetbios name:     %s\n", LW_IS_NULL_OR_EMPTY_STR(pDomainInfo->pszNetbiosDomain) ? "" : pDomainInfo->pszNetbiosDomain);
                 printf("\t\tForest name:      %s\n", LW_IS_NULL_OR_EMPTY_STR(pDomainInfo->pszForestName) ? "" : pDomainInfo->pszForestName);
@@ -400,12 +508,12 @@ PrintStatus(
                     case LSA_TRUST_DIRECTION_TWO_WAY:
                         printf("Twoway Trust\n");
                         break;
-
+                    
                     case LSA_TRUST_DIRECTION_SELF:
                         printf("Primary Domain\n");
                         break;
                     default:
-                        printf("Unknown trust direction\n");
+                        printf("Unknown trust direction\n");                
                 }
                 printf("\t\tTrust Mode:       ");
                 switch (pDomainInfo->dwTrustMode)
@@ -420,9 +528,9 @@ PrintStatus(
                         printf("In other forest Trust (OFT)\n");
                         break;
                     default:
-                        printf("Unknown trust mode\n");
+                        printf("Unknown trust mode\n");                
                 }
-
+                
                 printf("\t\tDomain flags:     [0x%.04x]\n", pDomainInfo->dwDomainFlags);
                 if (pDomainInfo->dwDomainFlags & LSA_DM_DOMAIN_FLAG_PRIMARY)
                     printf("\t\t                  [0x%.04x - Primary]\n", LSA_DM_DOMAIN_FLAG_PRIMARY);
@@ -436,7 +544,7 @@ PrintStatus(
                     printf("\t\t                  [0x%.04x - Forest root]\n", LSA_DM_DOMAIN_FLAG_FOREST_ROOT);
                 if (pDomainInfo->dwDomainFlags & LSA_DM_DOMAIN_FLAG_GC_OFFLINE)
                     printf("\t\t                  [0x%.04x - GC offline]\n", LSA_DM_DOMAIN_FLAG_GC_OFFLINE);
-
+                
                 if (pDomainInfo->pDCInfo)
                 {
                     printf("\n\t\t[Domain Controller (DC) Information]\n\n");
@@ -453,9 +561,9 @@ PrintStatus(
                     printf("\t\t\tDC is Global Catalog: %s\n",
                                     (pDomainInfo->pDCInfo->dwFlags & LSA_DS_GC_FLAG) ? "yes" : "no");
                     printf("\t\t\tDC is running KDC:    %s\n",
-                                    (pDomainInfo->pDCInfo->dwFlags & LSA_DS_KDC_FLAG) ? "yes" : "no");
+                                    (pDomainInfo->pDCInfo->dwFlags & LSA_DS_KDC_FLAG) ? "yes" : "no");         
                 }
-
+                
                 if (pDomainInfo->pGCInfo)
                 {
                     printf("\n\t\t[Global Catalog (GC) Information]\n\n");
@@ -470,7 +578,7 @@ PrintStatus(
                     printf("\t\t\tGC has writeable DS:  %s\n",
                                     (pDomainInfo->pGCInfo->dwFlags & LSA_DS_WRITABLE_FLAG) ? "yes" : "no");
                     printf("\t\t\tGC is running KDC:    %s\n",
-                                    (pDomainInfo->pGCInfo->dwFlags & LSA_DS_KDC_FLAG) ? "yes" : "no");
+                                    (pDomainInfo->pGCInfo->dwFlags & LSA_DS_KDC_FLAG) ? "yes" : "no");  
                 }
             }
         }
@@ -483,28 +591,28 @@ GetStatusString(
     )
 {
     PCSTR pszStatusString = NULL;
-
+    
     switch (status)
     {
         case LSA_AUTH_PROVIDER_STATUS_ONLINE:
-
+            
             pszStatusString = LSA_STATUS_STRING_ONLINE;
-
+            
             break;
-
+            
         case LSA_AUTH_PROVIDER_STATUS_OFFLINE:
-
+            
             pszStatusString = LSA_STATUS_STRING_OFFLINE;
-
+            
             break;
-
+            
         default:
-
+            
             pszStatusString = LSA_STATUS_STRING_UNKNOWN;
-
+            
             break;
     }
-
+    
     return pszStatusString;
 }
 
@@ -514,40 +622,40 @@ GetModeString(
     )
 {
     PCSTR pszModeString = NULL;
-
+    
     switch (mode)
-    {
+    {    
         case LSA_PROVIDER_MODE_UNPROVISIONED:
-
+            
             pszModeString = LSA_MODE_STRING_UNPROVISIONED;
-
+            
             break;
-
+            
         case LSA_PROVIDER_MODE_DEFAULT_CELL:
-
+            
             pszModeString = LSA_MODE_STRING_DEFAULT_CELL;
-
+            
             break;
-
+            
         case LSA_PROVIDER_MODE_NON_DEFAULT_CELL:
-
+            
             pszModeString = LSA_MODE_STRING_NON_DEFAULT_CELL;
-
+            
             break;
-
+            
         case LSA_PROVIDER_MODE_LOCAL_SYSTEM:
-
+            
             pszModeString = LSA_MODE_STRING_LOCAL;
-
+            
             break;
-
+            
         default:
-
+            
             pszModeString = LSA_MODE_STRING_UNKNOWN;
-
+            
             break;
     }
-
+    
     return pszModeString;
 }
 
@@ -557,25 +665,25 @@ GetSubmodeString(
     )
 {
     PCSTR pszSubmodeString = NULL;
-
+    
     switch(subMode)
     {
         case LSA_AUTH_PROVIDER_SUBMODE_SCHEMA:
-
+            
             pszSubmodeString = LSA_SUBMODE_STRING_SCHEMA;
-
+            
             break;
-
+            
         case LSA_AUTH_PROVIDER_SUBMODE_NONSCHEMA:
-
+            
             pszSubmodeString = LSA_SUBMODE_STRING_NON_SCHEMA;
-
+            
             break;
-
+            
         default:
-
+            
             pszSubmodeString = LSA_SUBMODE_STRING_UNKNOWN;
-
+            
             break;
     }
 
@@ -588,30 +696,30 @@ GetTrustTypeString(
     )
 {
     PCSTR pszTrustTypeString = NULL;
-
+    
     switch (dwTrustType)
     {
         case LSA_TRUST_TYPE_DOWNLEVEL:
             pszTrustTypeString = LSA_TRUST_TYPE_STRING_DOWNLEVEL;
             break;
-
+             
         case LSA_TRUST_TYPE_UPLEVEL:
             pszTrustTypeString = LSA_TRUST_TYPE_STRING_UPLEVEL;
             break;
-
+            
         case LSA_TRUST_TYPE_MIT:
             pszTrustTypeString = LSA_TRUST_TYPE_STRING_MIT;
             break;
-
+            
         case LSA_TRUST_TYPE_DCE:
             pszTrustTypeString = LSA_TRUST_TYPE_STRING_DCE;
             break;
-
+            
         default:
             pszTrustTypeString = LSA_TRUST_TYPE_STRING_UNKNOWN;
             break;
     }
-
+    
     return pszTrustTypeString;
 }
 
@@ -621,21 +729,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
