@@ -17,6 +17,7 @@ static main_entry entry_list[] =
 {
     {"lsa", lsa_main},
     {"ad-cache", ad_cache_main},
+    {"ad-get-machine", ad_get_machine_main},
     {"add-group", add_group_main},
     {"add-user", add_user_main},
     {"del-group", del_group_main},
@@ -37,6 +38,7 @@ static main_entry entry_list[] =
     {"mod-group", mod_group_main},
     {"mod-user", mod_user_main},
     {"refresh-configuration", refresh_configuration_main},
+    {"set-default-domain", set_default_domain_main},
     {"set-log-level", set_log_level_main},
     {"set-machine-sid", set_machine_sid_main},
     {"set-machine-name", set_machine_name_main},
@@ -90,7 +92,7 @@ lsa_main(
     {
         fprintf(stderr, "Usage: lsa mode ...\n\n");
         fprintf(stderr, "Available modes:\n");
-
+        
         for (i = 0; entry_list[i].name; i++)
         {
             fprintf(stderr, "    %s\n", entry_list[i].name);
