@@ -16,6 +16,12 @@ ad_cache_main(
     char* argv[]
     );
 
+int
+ad_get_machine_main(
+    int argc,
+    char* argv[]
+    );
+
 int
 add_group_main(
     int argc,
@@ -142,6 +148,12 @@ refresh_configuration_main(
     char* argv[]
     );
 
+int
+set_default_domain_main(
+    int argc,
+    char* argv[]
+    );
+
 int
 set_log_level_main(
     int argc,
