@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -63,14 +63,16 @@ DWORD
 ParseArgs(
     int argc,
     char* argv[],
-    PSTR* ppszLoginId,
+    PSTR* ppszGid,
+    PSTR* ppszGroupName,
     PDLINKEDLIST* ppTaskList
     );
 
 static
 BOOLEAN
 ValidateArgs(
-    PCSTR        pszLoginId,
+    PCSTR        pszGid,
+    PCSTR        pszGroupName,
     PDLINKEDLIST pTaskList
     );
 
@@ -102,7 +104,8 @@ ShowUsage(
 static
 DWORD
 ModifyGroup(
-    PSTR pszLoginId,
+    PCSTR pszGid,
+    PCSTR pszGroupName,
     PDLINKEDLIST pTaskList
     );
 
@@ -145,7 +148,8 @@ LsaModGroupMain(
 {
     DWORD dwError = 0;
     PDLINKEDLIST pTaskList = NULL;
-    PSTR pszLoginId = NULL;
+    PSTR pszGid = NULL;
+    PSTR pszGroupName = NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
 
@@ -155,11 +159,17 @@ LsaModGroupMain(
          BAIL_ON_LSA_ERROR(dwError);
      }
 
-     dwError = ParseArgs(argc, argv, &pszLoginId, &pTaskList);
+     dwError = ParseArgs(
+                     argc,
+                     argv,
+                     &pszGid,
+                     &pszGroupName,
+                     &pTaskList);
      BAIL_ON_LSA_ERROR(dwError);
 
      dwError = ModifyGroup(
-                     pszLoginId,
+                     pszGid,
+                     pszGroupName,
                      pTaskList);
      BAIL_ON_LSA_ERROR(dwError);
 
@@ -170,7 +180,8 @@ LsaModGroupMain(
          LsaDLinkedListFree(pTaskList);
      }
 
-     LW_SAFE_FREE_STRING(pszLoginId);
+     LW_SAFE_FREE_STRING(pszGid);
+     LW_SAFE_FREE_STRING(pszGroupName);
 
      return dwError;
 
@@ -218,11 +229,13 @@ LsaModGroupMain(
      goto cleanup;
 }
 
+static
 DWORD
 ParseArgs(
     int   argc,
     char* argv[],
-    PSTR* ppszLoginId,
+    PSTR* ppszGid,
+    PSTR* ppszGroupName,
     PDLINKEDLIST* ppTaskList
     )
 {
@@ -230,6 +243,7 @@ ParseArgs(
         PARSE_MODE_OPEN = 0,
         PARSE_MODE_ADD_TO_GROUPS,
         PARSE_MODE_REMOVE_FROM_GROUPS,
+        PARSE_MODE_GID,
         PARSE_MODE_DONE
     } ParseMode;
 
@@ -239,7 +253,8 @@ ParseArgs(
     PSTR pArg = NULL;
     PDLINKEDLIST pTaskList = NULL;
     PGROUP_MOD_TASK pTask = NULL;
-    PSTR pszLoginId = NULL;
+    PSTR pszGid = NULL;
+    PSTR pszGroupName = NULL;
     PSTR pszAddMembers = NULL;
     PSTR pszRemoveMembers = NULL;
 
@@ -266,9 +281,13 @@ ParseArgs(
                 {
                     parseMode = PARSE_MODE_REMOVE_FROM_GROUPS;
                 }
+                else if (!strcmp(pArg, "--gid"))
+                {
+                    parseMode = PARSE_MODE_GID;
+                }
                 else
                 {
-                    dwError = LwAllocateString(pArg, &pszLoginId);
+                    dwError = LwAllocateString(pArg, &pszGroupName);
                     BAIL_ON_LSA_ERROR(dwError);
                     parseMode = PARSE_MODE_DONE;
                 }
@@ -315,6 +334,23 @@ ParseArgs(
                  break;
             }
 
+            case PARSE_MODE_GID:
+            {
+                if (!IsUnsignedInteger(pArg))
+                {
+                    fprintf(stderr, "Please specifiy a gid that is an unsigned integer\n");
+                    ShowUsage(GetProgramName(argv[0]));
+                    exit(1);
+                }
+
+                dwError = LwAllocateString(pArg, &pszGid);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                parseMode = PARSE_MODE_OPEN;
+
+                break;
+            }
+
             case PARSE_MODE_DONE:
             {
 
@@ -331,12 +367,13 @@ ParseArgs(
         exit(1);
     }
 
-    if (!ValidateArgs(pszLoginId, pTaskList)) {
+    if (!ValidateArgs(pszGid, pszGroupName, pTaskList)) {
         ShowUsage(GetProgramName(argv[0]));
         exit(1);
     }
 
-    *ppszLoginId = pszLoginId;
+    *ppszGid = pszGid;
+    *ppszGroupName = pszGroupName;
     *ppTaskList = pTaskList;
 
 cleanup:
@@ -356,14 +393,17 @@ error:
         FreeTask(pTask);
     }
 
-    LW_SAFE_FREE_STRING(pszLoginId);
+    LW_SAFE_FREE_STRING(pszGid);
+    LW_SAFE_FREE_STRING(pszGroupName);
 
     goto cleanup;
 }
 
+static
 BOOLEAN
 ValidateArgs(
-    PCSTR        pszLoginId,
+    PCSTR        pszGid,
+    PCSTR        pszGroupName,
     PDLINKEDLIST pTaskList
     )
 {
@@ -383,7 +423,9 @@ ValidateArgs(
         }
     }
 
-    if (LW_IS_NULL_OR_EMPTY_STR(pszLoginId)) {
+    if (LW_IS_NULL_OR_EMPTY_STR(pszGid) &&
+        LW_IS_NULL_OR_EMPTY_STR(pszGroupName))
+    {
         fprintf(stderr, "Error: A valid group id or group name must be specified.\n");
         goto cleanup;
     }
@@ -395,6 +437,7 @@ cleanup:
     return bValid;
 }
 
+static
 VOID
 FreeTasksInList(
     PVOID pTask,
@@ -406,6 +449,7 @@ FreeTasksInList(
     }
 }
 
+static
 VOID
 FreeTask(
     PGROUP_MOD_TASK pTask
@@ -417,6 +461,7 @@ FreeTask(
     LwFreeMemory(pTask);
 }
 
+static
 PSTR
 GetProgramName(
     PSTR pszFullProgramPath
@@ -440,12 +485,13 @@ GetProgramName(
     return pszNameStart;
 }
 
+static
 VOID
 ShowUsage(
     PCSTR pszProgramName
     )
 {
-    fprintf(stdout, "Usage: %s {modification options} ( group name | gid )\n\n", pszProgramName);
+    fprintf(stdout, "Usage: %s {modification options} ( <group name> | --gid <gid> )\n\n", pszProgramName);
 
     fprintf(stdout, "\nModification options:\n");
     fprintf(stdout, "{ --help }\n");
@@ -453,9 +499,11 @@ ShowUsage(
     fprintf(stdout, "{ --remove-members nt4-style-name }\n");
 }
 
+static
 DWORD
 ModifyGroup(
-    PSTR pszGroupId,
+    PCSTR pszGid,
+    PCSTR pszGroupName,
     PDLINKEDLIST pTaskList
     )
 {
@@ -463,31 +511,51 @@ ModifyGroup(
     PLSA_GROUP_MOD_INFO pGroupModInfo = NULL;
     gid_t gid = 0;
     int   nRead = 0;
-    LSA_FIND_FLAGS dwFindFlags = LSA_FIND_FLAGS_NSS;
-    PVOID pGroupInfo = NULL;
-    DWORD dwGroupInfoLevel = 0;
+    LSA_FIND_FLAGS findFlags = 0;
     HANDLE hLsaConnection = (HANDLE)NULL;
-
-    BAIL_ON_INVALID_STRING(pszGroupId);
+    LSA_QUERY_LIST Query = {0};
+    PLSA_SECURITY_OBJECT* ppObjects = NULL;
 
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
 
-    nRead = sscanf(pszGroupId, "%u", (unsigned int*)&gid);
-    if ((nRead == EOF) || (nRead == 0)) {
+    if (!LW_IS_NULL_OR_EMPTY_STR(pszGid))
+    {
+        nRead = sscanf(pszGid, "%u", (unsigned int*)&gid);
+        if ((nRead == EOF) || (nRead == 0)) {
+            fprintf(stderr, "An invalid group id [%s] was specified.", pszGid);
 
-       dwError = LsaFindGroupByName(
+            dwError = LW_ERROR_INVALID_PARAMETER;
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+    }
+    else if (!LW_IS_NULL_OR_EMPTY_STR(pszGroupName))
+    {
+        Query.ppszStrings = &pszGroupName;
+
+        dwError = LsaFindObjects(
                        hLsaConnection,
-                       pszGroupId,
-                       dwFindFlags,
-                       dwGroupInfoLevel,
-                       &pGroupInfo);
-       BAIL_ON_LSA_ERROR(dwError);
+                       NULL,
+                       findFlags,
+                       LSA_OBJECT_TYPE_GROUP,
+                       LSA_QUERY_TYPE_BY_NAME,
+                       1,
+                       Query,
+                       &ppObjects);
+        BAIL_ON_LSA_ERROR(dwError);
 
-       gid = ((PLSA_GROUP_INFO_0)pGroupInfo)->gid;
+        if (ppObjects[0] == NULL)
+        {
+            dwError = LW_ERROR_NO_SUCH_GROUP;
+            BAIL_ON_LSA_ERROR(dwError);
+        }
 
-       LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
-       pGroupInfo = NULL;
+        gid = ppObjects[0]->groupInfo.gid;
+    }
+    else
+    {
+        dwError = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
     }
 
     dwError = ResolveNames(
@@ -506,7 +574,9 @@ ModifyGroup(
                     pGroupModInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    fprintf(stdout, "Successfully modified group %s\n", pszGroupId);
+    fprintf(stdout,
+            "Successfully modified group %s\n",
+            pszGroupName ? pszGroupName : pszGid);
 
 cleanup:
 
@@ -514,8 +584,9 @@ cleanup:
         LsaFreeGroupModInfo(pGroupModInfo);
     }
 
-    if (pGroupInfo) {
-        LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
+    if (ppObjects)
+    {
+        LsaFreeSecurityObjectList(1, ppObjects);
     }
 
     if (hLsaConnection != (HANDLE)NULL) {
@@ -529,6 +600,7 @@ error:
     goto cleanup;
 }
 
+static
 DWORD
 ResolveNames(
     HANDLE hLsaConnection,
@@ -538,12 +610,10 @@ ResolveNames(
     DWORD dwError = 0;
     NTSTATUS ntStatus = STATUS_SUCCESS;
     PDLINKEDLIST pListMember = pTaskList;
-    DWORD dwGroupInfoLevel = 1;
-    PLSA_GROUP_INFO_1 pGroupInfo = NULL;
-    DWORD dwUserInfoLevel = 1;
-    PLSA_USER_INFO_1 pUserInfo = NULL;
+    LSA_QUERY_LIST Query = {0};
+    PLSA_SECURITY_OBJECT* ppObjects = NULL;
     PGROUP_MOD_TASK pTask = NULL;
-    PSTR pszName = NULL;
+    PCSTR pszName = NULL;
     PSTR pszSID = NULL;
     PSID pSid = NULL;
 
@@ -553,7 +623,7 @@ ResolveNames(
         if (pTask->taskType == GroupModTask_AddMembers ||
             pTask->taskType == GroupModTask_RemoveMembers)
         {
-            pszName = (PSTR)pTask->pData;
+            pszName = (PCSTR)pTask->pData;
 
             ntStatus = RtlAllocateSidFromCString(&pSid, pszName);
             if (ntStatus == STATUS_SUCCESS)
@@ -564,31 +634,52 @@ ResolveNames(
             }
             else
             {
-                dwError = LsaFindGroupByName(hLsaConnection,
-                                             pszName,
-                                             LSA_FIND_FLAGS_NSS,
-                                             dwGroupInfoLevel,
-                                             (PVOID*)&pGroupInfo);
-                if (dwError == 0)
+                Query.ppszStrings = &pszName;
+
+                dwError = LsaFindObjects(
+                                  hLsaConnection,
+                                  NULL,
+                                  0,
+                                  LSA_OBJECT_TYPE_GROUP,
+                                  LSA_QUERY_TYPE_BY_NAME,
+                                  1,
+                                  Query,
+                                  &ppObjects);
+                BAIL_ON_LSA_ERROR(dwError);
+                                         
+                if (ppObjects[0])
                 {
-                    dwError = LwAllocateString(pGroupInfo->pszSid, &pszSID);
+                    dwError = LwAllocateString(
+                                     ppObjects[0]->pszObjectSid,
+                                     &pszSID);
                     BAIL_ON_LSA_ERROR(dwError);
 
                 }
-                else if (dwError == LW_ERROR_NO_SUCH_GROUP)
-                {
-                    dwError = LsaFindUserByName(hLsaConnection,
-                                                pszName,
-                                                dwUserInfoLevel,
-                                                (PVOID*)&pUserInfo);
-                    BAIL_ON_LSA_ERROR(dwError);
-
-                    dwError = LwAllocateString(pUserInfo->pszSid, &pszSID);
-                    BAIL_ON_LSA_ERROR(dwError);
-                }
                 else
                 {
+                    dwError = LsaFindObjects(
+                                      hLsaConnection,
+                                      NULL,
+                                      0,
+                                      LSA_OBJECT_TYPE_USER,
+                                      LSA_QUERY_TYPE_BY_NAME,
+                                      1,
+                                      Query,
+                                      &ppObjects);
                     BAIL_ON_LSA_ERROR(dwError);
+
+                    if (ppObjects[0])
+                    {
+                        dwError = LwAllocateString(
+                                         ppObjects[0]->pszObjectSid,
+                                         &pszSID);
+                        BAIL_ON_LSA_ERROR(dwError);
+                    }
+                    else
+                    {
+                        dwError = ERROR_INVALID_ACCOUNT_NAME;
+                        BAIL_ON_LSA_ERROR(dwError);
+                    }
                 }
             }
 
@@ -597,16 +688,10 @@ ResolveNames(
 
             pTask->pData = pszSID;
 
-            if (pGroupInfo)
+            if (ppObjects)
             {
-                LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
-                pGroupInfo = NULL;
-            }
-
-            if (pUserInfo)
-            {
-                LsaFreeUserInfo(dwUserInfoLevel, pUserInfo);
-                pUserInfo = NULL;
+                LsaFreeSecurityObjectList(1, ppObjects);
+                ppObjects = NULL;
             }
 
             RTL_FREE(&pSid);
@@ -614,14 +699,10 @@ ResolveNames(
     }
 
 cleanup:
-    if (pGroupInfo)
+    if (ppObjects)
     {
-        LsaFreeGroupInfo(dwGroupInfoLevel, pGroupInfo);
-    }
-
-    if (pUserInfo)
-    {
-        LsaFreeUserInfo(dwUserInfoLevel, pUserInfo);
+        LsaFreeSecurityObjectList(1, ppObjects);
+        ppObjects = NULL;
     }
 
     RTL_FREE(&pSid);
@@ -634,6 +715,7 @@ error:
     goto cleanup;
 }
 
+static
 DWORD
 BuildGroupModInfo(
     gid_t        gid,
