@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -60,7 +60,8 @@ MapErrorCode(
 static
 DWORD
 ModifyUser(
-    PSTR pszLoginId,
+    PCSTR pszUid,
+    PCSTR pszLoginId,
     PDLINKEDLIST pTaskList
     );
 
@@ -69,6 +70,7 @@ DWORD
 ParseArgs(
     int   argc,
     char* argv[],
+    PSTR* ppszUid,
     PSTR* ppszLoginId,
     PDLINKEDLIST* ppTaskList
     );
@@ -101,7 +103,8 @@ ShowUsage(
 static
 BOOLEAN
 ValidateArgs(
-    PCSTR        pszLoginId,
+    PCSTR pszUid,
+    PCSTR pszLoginId,
     PDLINKEDLIST pTaskList
     );
 
@@ -122,6 +125,7 @@ LsaModUserMain(
 {
     DWORD dwError = 0;
     PDLINKEDLIST pTaskList = NULL;
+    PSTR pszUid = NULL;
     PSTR pszLoginId = NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
@@ -132,10 +136,16 @@ LsaModUserMain(
          BAIL_ON_LSA_ERROR(dwError);
      }
 
-     dwError = ParseArgs(argc, argv, &pszLoginId, &pTaskList);
+     dwError = ParseArgs(
+                     argc,
+                     argv,
+                     &pszUid,
+                     &pszLoginId,
+                     &pTaskList);
      BAIL_ON_LSA_ERROR(dwError);
 
      dwError = ModifyUser(
+                     pszUid,
                      pszLoginId,
                      pTaskList);
      BAIL_ON_LSA_ERROR(dwError);
@@ -147,6 +157,7 @@ LsaModUserMain(
          LsaDLinkedListFree(pTaskList);
      }
 
+     LW_SAFE_FREE_STRING(pszUid);
      LW_SAFE_FREE_STRING(pszLoginId);
 
      return dwError;
@@ -200,6 +211,7 @@ DWORD
 ParseArgs(
     int   argc,
     char* argv[],
+    PSTR* ppszUid,
     PSTR* ppszLoginId,
     PDLINKEDLIST* ppTaskList
     )
@@ -216,6 +228,7 @@ ParseArgs(
         PARSE_MODE_SET_HOMEDIR,
         PARSE_MODE_SET_SHELL,
         PARSE_MODE_SET_GECOS,
+        PARSE_MODE_UID,
         PARSE_MODE_DONE
     } ParseMode;
 
@@ -225,6 +238,7 @@ ParseArgs(
     PSTR pArg = NULL;
     PDLINKEDLIST pTaskList = NULL;
     PUSER_MOD_TASK pTask = NULL;
+    PSTR    pszUid = NULL;
     PSTR    pszLoginId = NULL;
     PSTR    pszPassword = NULL;
 
@@ -353,6 +367,9 @@ ParseArgs(
                 {
                     parseMode = PARSE_MODE_SET_PRIMARY_GROUP;
                 }
+                else if (!strcmp(pArg, "--uid")) {
+                    parseMode = PARSE_MODE_UID;
+                }
                 else
                 {
                     dwError = LwAllocateString(pArg, &pszLoginId);
@@ -542,6 +559,23 @@ ParseArgs(
                 break;
             }
 
+            case PARSE_MODE_UID:
+            {
+                if (!IsUnsignedInteger(pArg))
+                {
+                    fprintf(stderr, "Please specifiy a uid that is an unsigned integer\n");
+                    ShowUsage(GetProgramName(argv[0]));
+                    exit(1);
+                }
+
+                dwError = LwAllocateString(pArg, &pszUid);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                parseMode = PARSE_MODE_OPEN;
+
+                break;
+            }
+
             case PARSE_MODE_DONE:
             {
 
@@ -558,17 +592,18 @@ ParseArgs(
         exit(1);
     }
 
-    if (!ValidateArgs(pszLoginId, pTaskList)) {
+    if (!ValidateArgs(pszUid, pszLoginId, pTaskList)) {
         ShowUsage(GetProgramName(argv[0]));
         exit(1);
     }
 
+    *ppszUid = pszUid;
     *ppszLoginId = pszLoginId;
     *ppTaskList = pTaskList;
 
 cleanup:
 
-    LW_SAFE_FREE_STRING (pszPassword);
+    LW_SECURE_FREE_STRING (pszPassword);
 
     return dwError;
 
@@ -585,6 +620,7 @@ error:
         FreeTask(pTask);
     }
 
+    LW_SAFE_FREE_STRING(pszUid);
     LW_SAFE_FREE_STRING(pszLoginId);
 
     goto cleanup;
@@ -593,7 +629,8 @@ error:
 static
 BOOLEAN
 ValidateArgs(
-    PCSTR        pszLoginId,
+    PCSTR pszUid,
+    PCSTR pszLoginId,
     PDLINKEDLIST pTaskList
     )
 {
@@ -668,7 +705,9 @@ ValidateArgs(
         goto cleanup;
     }
 
-    if (LW_IS_NULL_OR_EMPTY_STR(pszLoginId)) {
+    if (LW_IS_NULL_OR_EMPTY_STR(pszLoginId) &&
+        LW_IS_NULL_OR_EMPTY_STR(pszUid))
+    {
         fprintf(stderr, "Error: A valid user id or user login id must be specified.\n");
         goto cleanup;
     }
@@ -760,7 +799,7 @@ ShowUsage(
     PCSTR pszProgramName
     )
 {
-    fprintf(stdout, "Usage: %s {modification options} ( user login id | uid )\n\n", pszProgramName);
+    fprintf(stdout, "Usage: %s {modification options} ( <user login id> | --uid <uid> )\n\n", pszProgramName);
 
     fprintf(stdout, "\nModification options:\n");
     fprintf(stdout, "{ --help }\n");
@@ -792,7 +831,8 @@ ShowUsage(
 static
 DWORD
 ModifyUser(
-    PSTR pszLoginId,
+    PCSTR pszUid,
+    PCSTR pszLoginId,
     PDLINKEDLIST pTaskList
     )
 {
@@ -800,29 +840,51 @@ ModifyUser(
     PLSA_USER_MOD_INFO pUserModInfo = NULL;
     uid_t uid = 0;
     int   nRead = 0;
-    PVOID pUserInfo = NULL;
-    DWORD dwUserInfoLevel = 0;
+    LSA_FIND_FLAGS findFlags = 0;
     HANDLE hLsaConnection = (HANDLE)NULL;
-
-    BAIL_ON_INVALID_STRING(pszLoginId);
+    LSA_QUERY_LIST Query = {0};
+    PLSA_SECURITY_OBJECT* ppObjects = NULL;
 
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
 
-    nRead = sscanf(pszLoginId, "%u", (unsigned int*)&uid);
-    if ((nRead == EOF) || (nRead == 0)) {
+    if (!LW_IS_NULL_OR_EMPTY_STR(pszUid))
+    {
+        nRead = sscanf(pszUid, "%u", (unsigned int*)&uid);
+        if ((nRead == EOF) || (nRead == 0)) {
+            fprintf(stderr, "An invalid user id [%s] was specified.", pszUid);
 
-       dwError = LsaFindUserByName(
+            dwError = LW_ERROR_INVALID_PARAMETER;
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+    }
+    else if (!LW_IS_NULL_OR_EMPTY_STR(pszLoginId))
+    {
+        Query.ppszStrings = &pszLoginId;
+
+        dwError = LsaFindObjects(
                        hLsaConnection,
-                       pszLoginId,
-                       dwUserInfoLevel,
-                       &pUserInfo);
-       BAIL_ON_LSA_ERROR(dwError);
+                       NULL,
+                       findFlags,
+                       LSA_OBJECT_TYPE_USER,
+                       LSA_QUERY_TYPE_BY_NAME,
+                       1,
+                       Query,
+                       &ppObjects);
+        BAIL_ON_LSA_ERROR(dwError);
 
-       uid = ((PLSA_USER_INFO_0)pUserInfo)->uid;
+        if (ppObjects[0] == NULL)
+        {
+            dwError = LW_ERROR_NO_SUCH_USER;
+            BAIL_ON_LSA_ERROR(dwError);
+        }
 
-       LsaFreeUserInfo(dwUserInfoLevel, pUserInfo);
-       pUserInfo = NULL;
+        uid = ppObjects[0]->userInfo.uid;
+    }
+    else
+    {
+        dwError = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
     }
 
     dwError = BuildUserModInfo(
@@ -836,7 +898,9 @@ ModifyUser(
                     pUserModInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    fprintf(stdout, "Successfully modified user %s\n", pszLoginId);
+    fprintf(stdout,
+            "Successfully modified user %s\n",
+            pszLoginId ? pszLoginId : pszUid);
 
 cleanup:
 
@@ -844,8 +908,9 @@ cleanup:
         LsaFreeUserModInfo(pUserModInfo);
     }
 
-    if (pUserInfo) {
-       LsaFreeUserInfo(dwUserInfoLevel, pUserInfo);
+    if (ppObjects)
+    {
+        LsaFreeSecurityObjectList(1, ppObjects);
     }
 
     if (hLsaConnection != (HANDLE)NULL) {
