@@ -45,7 +45,6 @@
 #include "common.h"
 
 #include <lsa/lsa.h>
-#include <lsa/lsa2.h>
 #include <lwmem.h>
 #include <lwerror.h>
 
@@ -58,7 +57,7 @@ static struct
     DWORD dwCount;
     LSA_QUERY_LIST QueryList;
     BOOLEAN bShowUsage;
-} gState =
+} gState = 
 {
     .pszTargetProvider = NULL,
     .FindFlags = 0,
@@ -85,7 +84,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.pdwIds,
             OUT_PPVOID(&NewList.pdwIds),
-            sizeof(NewList.pdwIds) * (gState.dwCount + 1));
+            sizeof(*NewList.pdwIds) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.pdwIds[gState.dwCount] = (DWORD) id;
         gState.QueryList.pdwIds = NewList.pdwIds;
@@ -95,7 +94,7 @@ ParseQueryItem(
         dwError = LwReallocMemory(
             gState.QueryList.ppszStrings,
             OUT_PPVOID(&NewList.ppszStrings),
-            sizeof(NewList.ppszStrings) * (gState.dwCount + 1));
+            sizeof(*NewList.ppszStrings) * (gState.dwCount + 1));
         BAIL_ON_LSA_ERROR(dwError);
         NewList.ppszStrings[gState.dwCount] = pszArg;
         gState.QueryList.ppszStrings = NewList.ppszStrings;
@@ -141,7 +140,7 @@ ShowUsage(
     printf(
         "Usage: %s [ --<object type> ] [ --by-<key> ] [ <flags> ] [ --provider name ] objects ...\n",
         Basename(pszProgramName));
-
+    
     if (bFull)
     {
         printf(
@@ -230,7 +229,7 @@ ParseArguments(
         else if (!strcmp(ppszArgv[i], "--provider"))
         {
             i++;
-
+                
             if (i >= argc)
             {
                 dwError = LW_ERROR_INVALID_PARAMETER;
@@ -288,7 +287,7 @@ ResolveSid(
             SingleList.ppszStrings = &QueryList.ppszStrings[dwIndex];
             break;
         }
-
+        
         dwError = LsaFindObjects(
             hLsa,
             gState.pszTargetProvider,
@@ -308,7 +307,7 @@ ResolveSid(
     }
 
 error:
-
+    
     if (ppObjects)
     {
         LsaFreeSecurityObjectList(1, ppObjects);
@@ -333,7 +332,7 @@ ResolveGroups(
 
     dwError = LsaFindObjects(
         hLsa,
-        NULL, //gState.pszTargetProvider,
+        gState.pszTargetProvider,
         gState.FindFlags,
         LSA_OBJECT_TYPE_GROUP,
         LSA_QUERY_TYPE_BY_SID,
@@ -359,7 +358,6 @@ QueryMemberOf(
 {
     DWORD dwError = 0;
     HANDLE hLsa = NULL;
-    PSTR* ppszMembers = NULL;
     DWORD dwIndex = 0;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     PSTR pszSid = NULL;
@@ -367,7 +365,7 @@ QueryMemberOf(
     DWORD dwSidCount = 0;
     DWORD dwGroupSidCount = 0;
     PSTR* ppszGroupSids = NULL;
-
+       
     dwError = LsaOpenServer(&hLsa);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -378,7 +376,7 @@ QueryMemberOf(
     {
         dwError = ResolveSid(hLsa, gState.QueryType, gState.QueryList, dwIndex, &pszSid);
         BAIL_ON_LSA_ERROR(dwError);
-
+        
         if (!pszSid)
         {
             switch(gState.QueryType)
@@ -397,7 +395,7 @@ QueryMemberOf(
             pszSid = NULL;
         }
     }
-
+    
     dwError = LsaQueryMemberOf(
                 hLsa,
                 gState.pszTargetProvider,
@@ -410,7 +408,7 @@ QueryMemberOf(
 
     dwError = ResolveGroups(hLsa, dwGroupSidCount, ppszGroupSids, &ppObjects);
     BAIL_ON_LSA_ERROR(dwError);
-
+            
     for (dwIndex = 0; dwIndex < dwGroupSidCount; dwIndex++)
     {
         if (ppObjects[dwIndex])
@@ -421,7 +419,7 @@ QueryMemberOf(
         else
         {
             printf("Unresolvable SID [%u of %u] (%s)\n\n",
-                   dwIndex + 1, dwGroupSidCount, ppszMembers[dwIndex]);
+                   dwIndex + 1, dwGroupSidCount, ppszGroupSids[dwIndex]);
         }
     }
 
@@ -437,8 +435,8 @@ cleanup:
     if (ppszSids)
     {
         LsaFreeSidList(dwSidCount, ppszSids);
-    }
-
+    }   
+    
     if (hLsa)
     {
         LsaCloseServer(hLsa);
@@ -480,7 +478,7 @@ QueryMemberOfMain(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
         }
-
+        
         dwError = QueryMemberOf();
         BAIL_ON_LSA_ERROR(dwError);
     }
