@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,16 +37,14 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Tool to get status from LSA Server
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
@@ -84,7 +82,7 @@ refresh_configuration_main(
     HANDLE hLsaConnection = (HANDLE)NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
-
+    
     if (geteuid() != 0) {
         fprintf(stderr, "This program requires super-user privileges.\n");
         dwError = LW_ERROR_ACCESS_DENIED;
@@ -92,19 +90,19 @@ refresh_configuration_main(
     }
 
     ParseArgs(argc, argv);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaRefreshConfiguration(
                     hLsaConnection
                     );
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     printf("Configuration successfully loaded from disk.\n");
 
 cleanup:
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
@@ -114,22 +112,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(
@@ -141,10 +139,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(
@@ -167,7 +165,7 @@ ParseArgs(
     typedef enum {
             PARSE_MODE_OPEN = 0
         } ParseMode;
-
+        
     int iArg = 1;
     PSTR pszArg = NULL;
     ParseMode parseMode = PARSE_MODE_OPEN;
@@ -178,11 +176,11 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
@@ -197,7 +195,7 @@ ParseArgs(
 
                 break;
         }
-
+        
     } while (iArg < argc);
 }
 
@@ -215,21 +213,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
