@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -37,16 +37,14 @@
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
- *
+ *        Likewise Security and Authentication Subsystem (LSASS) 
+ *        
  *        Tool to set the LSASS Log Level at runtime
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
  *          Sriram Nambakam (snambakam@likewisesoftware.com)
  */
 
-#define _POSIX_PTHREAD_SEMANTICS 1
-
 #include "config.h"
 #include "lsasystem.h"
 #include "lsadef.h"
@@ -95,7 +93,7 @@ set_log_level_main(
     PLSA_LOG_INFO pLogInfo = NULL;
     size_t dwErrorBufferSize = 0;
     BOOLEAN bPrintOrigError = TRUE;
-
+    
     if (geteuid() != 0) {
         fprintf(stderr, "This program requires super-user privileges.\n");
         dwError = LW_ERROR_ACCESS_DENIED;
@@ -104,22 +102,22 @@ set_log_level_main(
 
     dwError = ParseArgs(argc, argv, &logLevel);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = LsaSetLogLevel(
                     hLsaConnection,
                     logLevel);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     fprintf(stdout, "The log level was set successfully\n\n");
-
+    
     dwError = LsaGetLogInfo(
                     hLsaConnection,
                     &pLogInfo);
     BAIL_ON_LSA_ERROR(dwError);
-
+    
     dwError = PrintLogInfo(pLogInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -129,7 +127,7 @@ cleanup:
     {
         LsaFreeLogInfo(pLogInfo);
     }
-
+    
     if (hLsaConnection != (HANDLE)NULL) {
         LsaCloseServer(hLsaConnection);
     }
@@ -139,22 +137,22 @@ cleanup:
 error:
 
     dwError = MapErrorCode(dwError);
-
+    
     dwErrorBufferSize = LwGetErrorString(dwError, NULL, 0);
-
+    
     if (dwErrorBufferSize > 0)
     {
         DWORD dwError2 = 0;
         PSTR   pszErrorBuffer = NULL;
-
+        
         dwError2 = LwAllocateMemory(
                     dwErrorBufferSize,
                     (PVOID*)&pszErrorBuffer);
-
+        
         if (!dwError2)
         {
             DWORD dwLen = LwGetErrorString(dwError, pszErrorBuffer, dwErrorBufferSize);
-
+            
             if ((dwLen == dwErrorBufferSize) && !LW_IS_NULL_OR_EMPTY_STR(pszErrorBuffer))
             {
                 fprintf(stderr,
@@ -165,10 +163,10 @@ error:
                 bPrintOrigError = FALSE;
             }
         }
-
+        
         LW_SAFE_FREE_STRING(pszErrorBuffer);
     }
-
+    
     if (bPrintOrigError)
     {
         fprintf(stderr,
@@ -191,7 +189,7 @@ ParseArgs(
     typedef enum {
             PARSE_MODE_OPEN = 0
         } ParseMode;
-
+        
     DWORD dwError = 0;
     int iArg = 1;
     PSTR pszArg = NULL;
@@ -205,11 +203,11 @@ ParseArgs(
         {
             break;
         }
-
+        
         switch (parseMode)
         {
             case PARSE_MODE_OPEN:
-
+        
                 if ((strcmp(pszArg, "--help") == 0) ||
                     (strcmp(pszArg, "-h") == 0))
                 {
@@ -256,9 +254,9 @@ ParseArgs(
                 }
                 break;
         }
-
+        
     } while (iArg < argc);
-
+    
     if (!bLogLevelSpecified)
     {
         ShowUsage();
@@ -266,7 +264,7 @@ ParseArgs(
     }
 
     *pLogLevel = logLevel;
-
+    
     return dwError;
 }
 
@@ -284,7 +282,7 @@ PrintLogInfo(
     )
 {
     DWORD dwError = 0;
-
+    
     fprintf(stdout, "Current log settings:\n");
     fprintf(stdout, "=================\n");
     switch(pLogInfo->logTarget)
@@ -306,7 +304,7 @@ PrintLogInfo(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
     fprintf(stdout, "Maximum allowed log level: ");
     switch(pLogInfo->maxAllowedLogLevel)
     {
@@ -332,7 +330,7 @@ PrintLogInfo(
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LSA_ERROR(dwError);
     }
-
+    
 error:
 
     return dwError;
@@ -345,21 +343,21 @@ MapErrorCode(
     )
 {
     DWORD dwError2 = dwError;
-
+    
     switch (dwError)
     {
         case ECONNREFUSED:
         case ENETUNREACH:
         case ETIMEDOUT:
-
+            
             dwError2 = LW_ERROR_LSA_SERVER_UNREACHABLE;
-
+            
             break;
-
+            
         default:
-
+            
             break;
     }
-
+    
     return dwError2;
 }
