@@ -8,7 +8,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -62,8 +62,8 @@ static const struct krb5err_lwerr krb5err_lwerr_map[] = {
     LWERR(KRB5KDC_ERR_BAD_PVNO, LW_ERROR_KRB5KDC_ERR_BAD_PVNO),
     LWERR(KRB5KDC_ERR_C_OLD_MAST_KVNO, LW_ERROR_KRB5KDC_ERR_C_OLD_MAST_KVNO),
     LWERR(KRB5KDC_ERR_S_OLD_MAST_KVNO, LW_ERROR_KRB5KDC_ERR_S_OLD_MAST_KVNO),
-    LWERR(KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN, LW_ERROR_INVALID_ACCOUNT),
-    LWERR(KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, LW_ERROR_KRB5_S_PRINCIPAL_UNKNOWN),
+    LWERR(KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN, LW_ERROR_KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN),
+    LWERR(KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, LW_ERROR_KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN),
     LWERR(KRB5KDC_ERR_PRINCIPAL_NOT_UNIQUE, LW_ERROR_KRB5KDC_ERR_PRINCIPAL_NOT_UNIQUE),
     LWERR(KRB5KDC_ERR_NULL_KEY, LW_ERROR_KRB5KDC_ERR_NULL_KEY),
     LWERR(KRB5KDC_ERR_CANNOT_POSTDATE, LW_ERROR_KRB5KDC_ERR_CANNOT_POSTDATE),
