@@ -107,13 +107,13 @@
 #define LW_ERROR_DUPLICATE_GROUPNAME                       40065
 #define LW_ERROR_DUPLICATE_CELLNAME                        40066
 #define LW_ERROR_STRING_CONV_FAILED                        40067
-#define LW_ERROR_INVALID_ACCOUNT                           40068
+// Obsolete LW_ERROR_INVALID_ACCOUNT                       40068
 #define LW_ERROR_INVALID_PASSWORD                          40069
 #define LW_ERROR_QUERY_CREATION_FAILED                     40070
 #define LW_ERROR_NO_SUCH_OBJECT                            40071
 #define LW_ERROR_DUPLICATE_USER_OR_GROUP                   40072
 #define LW_ERROR_INVALID_KRB5_CACHE_TYPE                   40073
-#define LW_ERROR_NOT_JOINED_TO_AD                          40074
+// Obsolete LW_ERROR_NOT_JOINED_TO_AD                      40074
 #define LW_ERROR_FAILED_TO_SET_TIME                        40075
 #define LW_ERROR_NO_NETBIOS_NAME                           40076
 #define LW_ERROR_INVALID_NETLOGON_RESPONSE                 40077
@@ -193,7 +193,7 @@
 #define LW_ERROR_NO_ATTRIBUTE_VALUE                        40151
 #define LW_ERROR_MEMBER_IN_LOCAL_GROUP                     40152
 #define LW_ERROR_MEMBER_NOT_IN_LOCAL_GROUP                 40153
-#define LW_ERROR_KRB5_S_PRINCIPAL_UNKNOWN                  40154
+// Obsolete LW_ERROR_KRB5_S_PRINCIPAL_UNKNOWN              40154
 #define LW_ERROR_INVALID_GROUP                             40155
 #define LW_WARNING_CONTINUE_NEEDED                         40157
 #define LW_ERROR_ACCESS_DENIED                             40158
@@ -413,9 +413,6 @@
 
 /*Range 40700 - 41200 is reserved for registry specific error*/
 
-#define _LW_ERROR_REG_BASE                                 40700
-#define _LW_ERROR_REG_MAX                                  41200
-
 /* Range 41201 - 41700 is reserved for service manager errors */
 #define LW_ERROR_INVALID_SERVICE_TRANSITION                41201
 #define LW_ERROR_SERVICE_DEPENDENCY_UNMET                  41202
@@ -456,6 +453,8 @@
 #define LW_ERROR_KRB5KDC_ERR_BAD_PVNO                      41734
 #define LW_ERROR_KRB5KDC_ERR_C_OLD_MAST_KVNO               41735
 #define LW_ERROR_KRB5KDC_ERR_S_OLD_MAST_KVNO               41736
+#define LW_ERROR_KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN           41737
+#define LW_ERROR_KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN           41738
 #define LW_ERROR_KRB5KDC_ERR_PRINCIPAL_NOT_UNIQUE          41739
 #define LW_ERROR_KRB5KDC_ERR_NULL_KEY                      41740
 #define LW_ERROR_KRB5KDC_ERR_CANNOT_POSTDATE               41741
@@ -745,6 +744,11 @@
 #define LW_ERROR_DOMAINJOIN_WARNING                        42507
 #define LW_ERROR_FAILED_ADMIN_PRIVS                        42508
 
+// Range 42600 - 42560 is reserved for autoenroll
+#define LW_ERROR_AUTOENROLL_FAILED                         42600
+#define LW_ERROR_AUTOENROLL_HTTP_REQUEST_FAILED            42601
+#define LW_ERROR_AUTOENROLL_SUBJECT_NAME_REQUIRED          42602
+
 //
 // 49900-49999 reserved for internal PAM error code mapping
 //
