@@ -11,7 +11,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -101,6 +101,15 @@ LwGetOwnerAndPermissions(
     mode_t * mode
     );
 
+DWORD
+LwFindFilesInPaths(
+    IN PCSTR pszName,
+    IN LWFILE_TYPE type,
+    IN const PCSTR* ppszSearchPaths,
+    OUT PDWORD pdwFoundCount,
+    OUT PSTR** pppszFoundPaths
+    );
+
 DWORD
 LwCheckFileTypeExists(
     PCSTR pszPath,
@@ -114,6 +123,12 @@ LwCreateDirectory(
     mode_t dwFileMode
     );
 
+DWORD
+LwRemoveDuplicateInodes(
+    IN OUT PDWORD pdwFoundCount,
+    IN OUT PSTR* ppszFoundPaths
+    );
+
 LW_END_EXTERN_C
 
 
