@@ -11,7 +11,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -46,21 +46,10 @@
 #ifndef __LWLDAP_H__
 #define __LWLDAP_H__
 
-#ifndef KRB5_PRIVATE
-#define KRB5_PRIVATE 1
-#ifndef KRB5_DEPRECATED
-#define KRB5_DEPRECATED 1
-#include <krb5.h>
-#endif
-#endif
-#include <gssapi/gssapi.h>
-#include <gssapi/gssapi_generic.h>
-#include <gssapi/gssapi_krb5.h>
 #ifndef LDAP_DEPRECATED
 #define LDAP_DEPRECATED 1
 #include <ldap.h>
 #endif
-#include <lber.h>
 
 //maximum length of LDAP query, in bytes.
 #define MAX_LDAP_QUERY_LENGTH 4096
@@ -93,20 +82,6 @@ LwLdapPingTcp(
     DWORD dwTimeoutSeconds
     );
 
-DWORD
-LwLdapOpenDirectoryDomain(
-    IN PCSTR pszDnsDomainName,
-    IN DWORD dwFlags,
-    OUT PHANDLE phDirectory
-    );
-
-DWORD
-LwLdapOpenDirectoryGc(
-    IN PCSTR pszDnsForestName,
-    IN DWORD dwFlags,
-    OUT PHANDLE phDirectory
-    );
-
 DWORD
 LwLdapOpenDirectoryServer(
     IN PCSTR pszServerAddress,
@@ -114,6 +89,19 @@ LwLdapOpenDirectoryServer(
     IN DWORD dwFlags,
     OUT PHANDLE phDirectory
     );
+///<
+/// Open a connection to an LDAP server.
+///
+/// @param[in] pszServerAddress - Server's address.
+/// @param[in] pszServerName - Server's name (used for authentication).
+/// @param[in] dwFlags - LW_LDAP_OPT_* flags.
+/// @param[out] phDirectory - Returns handle to LDAP connection.
+///
+/// @return Windows error code
+/// @retval ERROR_SUCCESS on success
+/// @retval SEC_E_NO_CREDENTIALS if no usable credentials
+/// @retval !ERROR_SUCCESS on failure
+///
 
 DWORD
 LwLdapConvertDomainToDN(
@@ -132,14 +120,6 @@ LwLdapCloseDirectory(
     HANDLE hDirectory
     );
 
-DWORD
-LwLdapReadObject(
-    HANDLE hDirectory,
-    PCSTR  pszObjectDN,
-    PSTR*  ppszAttributeList,
-    LDAPMessage **ppMessage
-    );
-
 DWORD
 LwLdapGetParentDN(
     PCSTR pszObjectDN,
@@ -303,6 +283,22 @@ LwLdapGetStringsWithExtDnResult(
     OUT PDWORD pdwNumValues
     );
 
+DWORD
+LwLdapPutString(
+    HANDLE hDirectory,
+    PCSTR  pszDN,
+    PCSTR  pszFieldName,
+    PSTR   pszValue
+    );
+
+DWORD
+LwLdapPutUInt32(
+    HANDLE hDirectory,
+    PCSTR  pszDN,
+    PCSTR  pszFieldName,
+    DWORD  dwValue
+    );
+
 DWORD
 LwLdapEscapeString(
     PSTR *ppszResult,
@@ -344,6 +340,13 @@ LwLdapBindDirectoryAnonymous(
     HANDLE hDirectory
     );
 
+DWORD
+LwLdapBindDirectorySasl(
+    LDAP *ld,
+    PCSTR pszServerName,
+    BOOLEAN bSeal
+    );
+
 LW_END_EXTERN_C
 
 
