@@ -11,7 +11,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -56,6 +56,16 @@
         } \
     } while (0)
 
+#define LW_SECURE_FREE_MEMORY(mem, len) \
+    do { \
+        if (mem) \
+        { \
+            memset(mem, 0, len); \
+            LwFreeMemory(mem); \
+            (mem) = NULL; \
+        } \
+    } while (0)
+
 
 LW_BEGIN_EXTERN_C
 
