@@ -11,7 +11,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -36,8 +36,8 @@
  *
  * Abstract:
  *
- *        Likewise Advanced API (lwadvapi)
- *
+ *        Likewise Advanced API (lwadvapi) 
+ *        
  *        Kerberos 5 API
  *
  * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
@@ -48,9 +48,208 @@
  */
 
 #include "includes.h"
+#include <gssapi/gssapi_krb5.h>
+
+#define LW_GSS_LOG_CALL_FORMAT \
+    "GSS API error calling %s(): majorStatus = 0x%08x, minorStatus = 0x%08x"
+
+#define _LW_GSS_LOG_CALL_ERROR(GssFunctionName, MajorStatus, MinorStatus) \
+    do { \
+        PSTR pszLocalErrorMessage = NULL; \
+        DWORD dwLocalError = LwGssGetErrorMessage(&pszLocalErrorMessage, GssFunctionName, MajorStatus, MinorStatus); \
+        if (dwLocalError) \
+        { \
+            LW_LOG_ERROR(LW_GSS_LOG_CALL_FORMAT, GssFunctionName, MajorStatus, MinorStatus); \
+        } \
+        else \
+        { \
+            LW_LOG_ERROR("%s", pszLocalErrorMessage); \
+            LwFreeString(pszLocalErrorMessage); \
+        } \
+    } while (0)
+
+#define _LW_GSS_LOG_CALL_DEBUG(GssFunctionName, MajorStatus, MinorStatus) \
+    do { \
+        PSTR pszLocalErrorMessage = NULL; \
+        DWORD dwLocalError = LwGssGetErrorMessage(&pszLocalErrorMessage, GssFunctionName, MajorStatus, MinorStatus); \
+        if (dwLocalError) \
+        { \
+            LW_LOG_DEBUG(LW_GSS_LOG_CALL_FORMAT, GssFunctionName, MajorStatus, MinorStatus); \
+        } \
+        else \
+        { \
+            LW_LOG_DEBUG("%s", pszLocalErrorMessage); \
+            LwFreeString(pszLocalErrorMessage); \
+        } \
+    } while (0)
+
+
+#define LW_GSS_LOG_IF_NOT_COMPLETE(GssFunctionName, MajorStatus, MinorStatus) \
+    do { \
+        switch (MajorStatus) \
+        { \
+            case GSS_S_COMPLETE: \
+                break; \
+            default: \
+                _LW_GSS_LOG_CALL_ERROR(GssFunctionName, MajorStatus, MinorStatus); \
+        } \
+    } while (0)
+
+#define LW_GSS_LOG_IF_NOT_COMPLETE_OR_CONTINUE(GssFunctionName, MajorStatus, MinorStatus) \
+    do { \
+        switch (MajorStatus) \
+        { \
+            case GSS_S_COMPLETE: \
+                break; \
+            case GSS_S_CONTINUE_NEEDED: \
+                _LW_GSS_LOG_CALL_DEBUG(GssFunctionName, MajorStatus, MinorStatus); \
+                break; \
+            default: \
+                _LW_GSS_LOG_CALL_ERROR(GssFunctionName, MajorStatus, MinorStatus); \
+        } \
+    } while (0)
+
+#define BAIL_ON_GSS_ERROR(dwError, MajorStatus, MinorStatus) \
+    do { \
+        if (((MajorStatus) != GSS_S_COMPLETE) && \
+            ((MajorStatus) != GSS_S_CONTINUE_NEEDED)) \
+        { \
+            LW_LOG_DEBUG("[%s() %s:%d] GSS API error: majorStatus = 0x%08x, minorStatus = 0x%08x", \
+                         __FUNCTION__, \
+                         __FILE__, \
+                         __LINE__, \
+                         MajorStatus, MinorStatus); \
+            dwError = LW_ERROR_GSS_CALL_FAILED; \
+            goto error; \
+        } \
+    } while (0)
+
+//
+// KG_EMPTY_CCACHE is defined inside of gssapi but it is not exposed
+// externally.  It means that the credentials cache does not have a
+// TGT inside of it.
+//
+        
+#define KG_EMPTY_CCACHE 0x25ea10c
+        
 
 static volatile LONG glLibraryRefCount = 0;
 
+static
+DWORD
+LwGssGetSingleErrorMessage(
+    OUT PSTR* ppszErrorMessage,
+    IN OM_uint32 Status,
+    IN BOOLEAN IsMajor
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszErrorMessage = NULL;
+    OM_uint32 majorStatus = 0;
+    OM_uint32 minorStatus = 0;
+    gss_buffer_desc message = GSS_C_EMPTY_BUFFER;
+    OM_uint32 messageContext = 0;
+    int statusType = IsMajor ? GSS_C_GSS_CODE : GSS_C_MECH_CODE;
+
+    do {
+        majorStatus = gss_display_status(
+                            &minorStatus,
+                            Status,
+                            statusType,
+                            GSS_C_NULL_OID,
+                            &messageContext,
+                            &message);
+        if (majorStatus != GSS_S_COMPLETE)
+        {
+            LW_LOG_ERROR("Call to gss_display_status() failed with "
+                         "majorStatus = 0x%08x, minorStatus = 0x%08x",
+                         majorStatus, minorStatus);
+
+            dwError = ERROR_INTERNAL_ERROR;
+            BAIL_ON_LW_ERROR(dwError);
+        }
+
+        if (!pszErrorMessage)
+        {
+            dwError = LwAllocateString((PSTR)message.value, &pszErrorMessage);
+            BAIL_ON_LW_ERROR(dwError);
+        }
+        else
+        {
+            PSTR pszNewErrorMessage = NULL;
+
+            dwError = LwAllocateStringPrintf(&pszNewErrorMessage,
+                                             "%s; %s",
+                                             pszErrorMessage,
+                                             (PSTR)message.value);
+            BAIL_ON_LW_ERROR(dwError);
+
+            LW_SAFE_FREE_STRING(pszErrorMessage);
+            pszErrorMessage = pszNewErrorMessage;
+        }
+
+        majorStatus = gss_release_buffer(&minorStatus, &message);
+    } while (messageContext);
+
+error:
+    if (dwError)
+    {
+        LW_SAFE_FREE_STRING(pszErrorMessage);
+    }
+
+    if (message.value)
+    {
+        majorStatus = gss_release_buffer(&minorStatus, &message);
+    }
+
+    *ppszErrorMessage = pszErrorMessage;
+
+    return dwError;
+}
+
+DWORD
+LwGssGetErrorMessage(
+    OUT PSTR* ppszErrorMessage,
+    IN OPTIONAL PCSTR pszGssFunction,
+    IN OM_uint32 MajorStatus,
+    IN OM_uint32 MinorStatus
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszErrorMessage = NULL;
+    PSTR pszMajorMessage = NULL;
+    PSTR pszMinorMessage = NULL;
+
+    dwError = LwGssGetSingleErrorMessage(&pszMajorMessage, MajorStatus, TRUE);
+    BAIL_ON_LW_ERROR(dwError);
+
+    dwError = LwGssGetSingleErrorMessage(&pszMinorMessage, MinorStatus, FALSE);
+    BAIL_ON_LW_ERROR(dwError);
+
+    dwError = LwAllocateStringPrintf(&pszErrorMessage,
+                                     "GSS API error calling %s(): "
+                                     "majorStatus = 0x%08x (%s), "
+                                     "minorStatus = 0x%08x (%s)",
+                                     pszGssFunction,
+                                     MajorStatus, pszMajorMessage,
+                                     MinorStatus, pszMinorMessage);
+    BAIL_ON_LW_ERROR(dwError);
+
+error:
+    if (dwError)
+    {
+        LW_SAFE_FREE_STRING(pszErrorMessage);
+    }
+
+    LW_SAFE_FREE_STRING(pszMajorMessage);
+    LW_SAFE_FREE_STRING(pszMinorMessage);
+
+    *ppszErrorMessage = pszErrorMessage;
+
+    return dwError;
+}
+
+
 DWORD
 LwKrb5GetDefaultRealm(
     PSTR* ppszRealm
@@ -73,7 +272,7 @@ LwKrb5GetDefaultRealm(
     BAIL_ON_LW_ERROR(dwError);
 
     *ppszRealm = pszRealm;
-
+    
 cleanup:
 
     if (pszKrb5Realm)
@@ -88,7 +287,7 @@ cleanup:
 error:
 
     *ppszRealm = NULL;
-
+    
     LW_SAFE_FREE_STRING(pszRealm);
 
     goto cleanup;
@@ -96,8 +295,8 @@ error:
 
 
 DWORD
-LwKrb5GetSystemCachePath(
-    PSTR*         ppszCachePath
+LwKrb5GetDefaultCachePath(
+    OUT PSTR* ppszCachePath
     )
 {
     DWORD dwError = 0;
@@ -105,7 +304,7 @@ LwKrb5GetSystemCachePath(
     krb5_context ctx = NULL;
     const char *pszKrbDefault = NULL;
     krb5_error_code ret = 0;
-
+    
     ret = krb5_init_context(&ctx);
     BAIL_ON_KRB_ERROR(ctx, ret);
 
@@ -115,9 +314,9 @@ LwKrb5GetSystemCachePath(
                 pszKrbDefault,
                 &pszCachePath);
     BAIL_ON_LW_ERROR(dwError);
-
+    
     *ppszCachePath = pszCachePath;
-
+    
 cleanup:
     if (ctx)
     {
@@ -125,11 +324,11 @@ cleanup:
     }
 
     return dwError;
-
+    
 error:
 
     *ppszCachePath = NULL;
-
+    
     goto cleanup;
 }
 
@@ -143,55 +342,55 @@ LwKrb5GetUserCachePath(
 {
     DWORD dwError = 0;
     PSTR  pszCachePath = NULL;
-
+    
     switch (cacheType)
     {
         case KRB5_InMemory_Cache:
-
+            
             dwError = LwAllocateStringPrintf(
                         &pszCachePath,
                         "MEMORY:krb5cc_%ld",
                         (long)uid);
             BAIL_ON_LW_ERROR(dwError);
-
+            
             break;
-
+            
         case KRB5_File_Cache:
-
+            
             dwError = LwAllocateStringPrintf(
                         &pszCachePath,
                         "FILE:/tmp/krb5cc_%ld",
                         (long)uid);
             BAIL_ON_LW_ERROR(dwError);
-
+            
             break;
-
+            
         default:
-
+            
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_LW_ERROR(dwError);
-
+            
             break;
     }
-
+    
     *ppszCachePath = pszCachePath;
-
+    
 cleanup:
 
     return dwError;
-
+    
 error:
 
     *ppszCachePath = NULL;
-
+    
     goto cleanup;
 }
 
 
 DWORD
-LwKrb5SetDefaultCachePath(
-    PCSTR pszCachePath,
-    PSTR* ppszOrigCachePath
+LwKrb5SetThreadDefaultCachePath(
+    IN PCSTR pszCachePath,
+    OUT PSTR* ppszPreviousCachePath
     )
 {
     DWORD dwError       = 0;
@@ -203,77 +402,35 @@ LwKrb5SetDefaultCachePath(
     dwMajorStatus = gss_krb5_ccache_name(
                             (OM_uint32 *)&dwMinorStatus,
                             pszCachePath,
-                            (ppszOrigCachePath) ? (const char**)&pszOrigCachePath : NULL);
-    BAIL_ON_SEC_ERROR(dwMajorStatus);
+                            (ppszPreviousCachePath) ? (const char**)&pszOrigCachePath : NULL);
+    BAIL_ON_GSS_ERROR(dwError, dwMajorStatus, dwMinorStatus);
 
     LW_LOG_DEBUG("Switched gss krb5 credentials path from %s to %s",
             LW_SAFE_LOG_STRING(pszOrigCachePath),
             LW_SAFE_LOG_STRING(pszCachePath));
-
-    if (ppszOrigCachePath) {
+    
+    if (ppszPreviousCachePath) {
         if (!LW_IS_NULL_OR_EMPTY_STR(pszOrigCachePath)) {
-            dwError = LwAllocateString(pszOrigCachePath, ppszOrigCachePath);
+            dwError = LwAllocateString(pszOrigCachePath, ppszPreviousCachePath);
             BAIL_ON_LW_ERROR(dwError);
         } else {
-            *ppszOrigCachePath = NULL;
+            *ppszPreviousCachePath = NULL;
         }
     }
-
+    
 cleanup:
 
     return dwError;
-
+    
 error:
 
-    if (ppszOrigCachePath) {
-        *ppszOrigCachePath = NULL;
+    if (ppszPreviousCachePath) {
+        *ppszPreviousCachePath = NULL;
     }
 
     goto cleanup;
 }
 
-
-DWORD
-LwKrb5GetSystemKeytabPath(
-    PSTR* ppszKeytabPath
-    )
-{
-    DWORD dwError = LW_ERROR_SUCCESS;
-    krb5_error_code ret = 0;
-    krb5_context ctx = NULL;
-    PSTR pszPath = NULL;
-    size_t size = 64;
-
-    ret = krb5_init_context(&ctx);
-    BAIL_ON_KRB_ERROR(ctx, ret);
-
-    do {
-        LW_SAFE_FREE_STRING(pszPath);
-
-        size *= 2;
-        dwError = LwAllocateMemory(size, OUT_PPVOID(&pszPath));
-        BAIL_ON_LW_ERROR(dwError);
-
-        ret = krb5_kt_default_name(ctx, pszPath, size);
-    } while (ret == KRB5_CONFIG_NOTENUFSPACE);
-
-    BAIL_ON_KRB_ERROR(ctx, ret);
-    *ppszKeytabPath = pszPath;
-
-cleanup:
-    if (ctx) {
-        krb5_free_context(ctx);
-    }
-
-    return dwError;
-
-error:
-    LW_SAFE_FREE_STRING(pszPath);
-    *ppszKeytabPath = NULL;
-
-    goto cleanup;
-}
-
 DWORD
 LwKrb5SetProcessDefaultCachePath(
     PCSTR pszCachePath
@@ -313,158 +470,70 @@ LwKrb5SetProcessDefaultCachePath(
 
 error:
     LW_SAFE_FREE_STRING(pszEnvironmentEntry);
-
+    
     if (bLocked)
     {
         pthread_mutex_unlock(&lock);
     }
-
+    
     return dwError;
 }
 
 DWORD
-LwSetupMachineSession(
-    PCSTR  pszSamAccountName,
-    PCSTR  pszPassword,
-    PCSTR  pszRealm,
-    PCSTR  pszDomain,
-    PDWORD pdwGoodUntilTime
+LwKrb5InitializeCredentials(
+    IN PCSTR pszUserPrincipalName,
+    IN PCSTR pszPassword,
+    IN PCSTR pszCachePath,
+    OUT OPTIONAL PDWORD pdwGoodUntilTime
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
-    PSTR pszHostKeytabFile = NULL;
-    PSTR pszKrb5CcPath = NULL;
-    PSTR pszKrb5CcPathNew = NULL;
-    PSTR pszDomname = NULL;
-    PSTR pszRealmCpy = NULL;
-    PSTR pszMachPrincipal = NULL;
     DWORD dwGoodUntilTime = 0;
+    PSTR pszTempCachePath = NULL;
 
-    dwError = LwKrb5GetSystemKeytabPath(&pszHostKeytabFile);
-    BAIL_ON_LW_ERROR(dwError);
-
-    dwError = LwKrb5GetSystemCachePath(&pszKrb5CcPath);
-    BAIL_ON_LW_ERROR(dwError);
-
-    if (!strncmp(pszKrb5CcPath, "FILE:", sizeof("FILE:") - 1))
+    if (!pszCachePath)
     {
-        dwError = LwAllocateStringPrintf(&pszKrb5CcPathNew, "%s.new",
-                                          pszKrb5CcPath);
+        dwError = ERROR_INVALID_PARAMETER;
         BAIL_ON_LW_ERROR(dwError);
     }
 
-    dwError = LwAllocateString(pszRealm, &pszRealmCpy);
-    BAIL_ON_LW_ERROR(dwError);
-    LwStrToUpper(pszRealmCpy);
-
-    dwError = LwAllocateStringPrintf(&pszMachPrincipal, "%s@%s",
-                                      pszSamAccountName, pszRealm);
-    BAIL_ON_LW_ERROR(dwError);
-
-    dwError = LwAllocateString(pszDomain, &pszDomname);
-    BAIL_ON_LW_ERROR(dwError);
-    LwStrToLower(pszDomname);
+    if (!strncmp(pszCachePath, "FILE:", sizeof("FILE:") - 1))
+    {
+        dwError = LwAllocateStringPrintf(&pszTempCachePath, "%s.new",
+                                         pszCachePath);
+        BAIL_ON_LW_ERROR(dwError);
+    }
 
     dwError = LwKrb5GetTgt(
-                  pszMachPrincipal,
-                  pszPassword,
-                  pszKrb5CcPathNew ? pszKrb5CcPathNew : pszKrb5CcPath,
-                  &dwGoodUntilTime);
+                    pszUserPrincipalName,
+                    pszPassword,
+                    pszTempCachePath ? pszTempCachePath : pszCachePath,
+                    &dwGoodUntilTime);
     BAIL_ON_LW_ERROR(dwError);
 
-    if (pszKrb5CcPathNew)
+    if (pszTempCachePath)
     {
-        dwError = LwMoveFile(pszKrb5CcPathNew + sizeof("FILE:") - 1,
-                        pszKrb5CcPath + sizeof("FILE:") - 1);
+        dwError = LwMoveFile(pszTempCachePath + sizeof("FILE:") - 1,
+                             pszCachePath + sizeof("FILE:") - 1);
         BAIL_ON_LW_ERROR(dwError);
     }
 
+error:
+    if (dwError)
+    {
+        dwGoodUntilTime = 0;
+    }
+
+    LW_SAFE_FREE_STRING(pszTempCachePath);
+
     if (pdwGoodUntilTime)
     {
         *pdwGoodUntilTime = dwGoodUntilTime;
     }
 
-cleanup:
-
-    LW_SAFE_FREE_STRING(pszMachPrincipal);
-    LW_SAFE_FREE_STRING(pszDomname);
-    LW_SAFE_FREE_STRING(pszRealmCpy);
-    LW_SAFE_FREE_STRING(pszKrb5CcPath);
-    LW_SAFE_FREE_STRING(pszHostKeytabFile);
-    LW_SAFE_FREE_STRING(pszKrb5CcPathNew);
-
-    return (dwError);
-
-error:
-
-    if (pdwGoodUntilTime)
-    {
-        *pdwGoodUntilTime = 0;
-    }
-
-    goto cleanup;
-}
-
-DWORD
-LwKrb5CleanupMachineSession(
-    VOID
-    )
-{
-    DWORD dwError = LW_ERROR_SUCCESS;
-    PSTR pszKrb5CcPath = NULL;
-    krb5_error_code ret = 0;
-    krb5_context ctx = NULL;
-    krb5_ccache cc = NULL;
-
-    dwError = LwKrb5GetSystemCachePath(&pszKrb5CcPath);
-    BAIL_ON_LW_ERROR(dwError);
-
-    ret = krb5_init_context(&ctx);
-    BAIL_ON_KRB_ERROR(ctx, ret);
-
-    ret = krb5_cc_resolve(ctx, pszKrb5CcPath, &cc);
-    if (KRB5_FCC_NOFILE == ret)
-    {
-        goto cleanup;
-    }
-    BAIL_ON_KRB_ERROR(ctx, ret);
-
-    ret = krb5_cc_destroy(ctx, cc);
-    // This always frees the cc reference, even on error.
-    cc = NULL;
-    if (KRB5_FCC_NOFILE == ret)
-    {
-        goto cleanup;
-    }
-    BAIL_ON_KRB_ERROR(ctx, ret);
-
-cleanup:
-    LW_SAFE_FREE_STRING(pszKrb5CcPath);
-
-    if (cc)
-    {
-        // ctx must be valid.
-        krb5_cc_close(ctx, cc);
-    }
-
-    if (ctx)
-    {
-        krb5_free_context(ctx);
-    }
-
     return dwError;
-
-error:
-    goto cleanup;
 }
 
-#define BAIL_ON_DCE_ERROR(dest, status)                   \
-    if ((status) != 0) {                    \
-        LW_LOG_ERROR("DCE Error [Code:%d]", (status));   \
-        (dest) = LW_ERROR_DCE_CALL_FAILED;               \
-        goto error;                                       \
-    }
-
 DWORD
 LwKrb5CopyFromUserCache(
                 krb5_context ctx,
@@ -497,7 +566,7 @@ LwKrb5CopyFromUserCache(
                     KRB5_File_Cache,
                     &pszCachePath);
     BAIL_ON_LW_ERROR(dwError);
-
+    
     ret = krb5_cc_resolve(
             ctx,
             pszCachePath,
@@ -665,7 +734,7 @@ LwKrb5MoveCCacheToUserPath(
                     KRB5_File_Cache,
                     &pszCachePath);
     BAIL_ON_LW_ERROR(dwError);
-
+    
     if (strncasecmp(pszCachePath, "FILE:", sizeof("FILE:")-1)) {
         dwError = LW_ERROR_INTERNAL;
         BAIL_ON_LW_ERROR(dwError);
@@ -680,11 +749,7 @@ LwKrb5MoveCCacheToUserPath(
     /* Let the user read and write to their cache file (before this, only
      * root was allowed to read and write the file).
      */
-    dwError = LwChangeOwnerAndPermissions(
-                pszCachePathReal,
-                uid,
-                gid,
-                S_IRWXU);
+    dwError = LwChangeOwner(pszCachePathReal, uid, gid);
     BAIL_ON_LW_ERROR(dwError);
 
 cleanup:
@@ -720,202 +785,6 @@ LwKrb5Shutdown(
     }
 }
 
-DWORD
-LwKrb5GetMachineCreds(
-    PSTR* ppszUsername,
-    PSTR* ppszPassword,
-    PSTR* ppszDomainDnsName,
-    PSTR* ppszHostDnsDomain
-    )
-{
-    DWORD dwError = 0;
-    PSTR  pszUsername = NULL;
-    PSTR  pszPassword = NULL;
-    PSTR  pszDomainDnsName = NULL;
-    PSTR  pszHostDnsDomain = NULL;
-    PLWPS_PASSWORD_INFO pMachineAcctInfo = NULL;
-    HANDLE hPasswordStore = (HANDLE)NULL;
-
-    dwError = LwpsOpenPasswordStore(
-                    LWPS_PASSWORD_STORE_DEFAULT,
-                    &hPasswordStore);
-    BAIL_ON_LW_ERROR(dwError);
-
-    dwError = LwpsGetPasswordByCurrentHostName(
-                    hPasswordStore,
-                    &pMachineAcctInfo);
-    if (dwError)
-    {
-        LW_LOG_ERROR("Unable to read machine password for hostname");
-        BAIL_ON_LW_ERROR(dwError);
-    }
-
-    if (ppszUsername)
-    {
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszMachineAccount,
-            &pszUsername);
-        BAIL_ON_LW_ERROR(dwError);
-
-        if (LW_IS_NULL_OR_EMPTY_STR(pszUsername))
-        {
-            dwError = LW_ERROR_INVALID_ACCOUNT;
-            BAIL_ON_LW_ERROR(dwError);
-        }
-    }
-
-    if (ppszPassword)
-    {
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszMachinePassword,
-            &pszPassword);
-        BAIL_ON_LW_ERROR(dwError);
-
-        if (LW_IS_NULL_OR_EMPTY_STR(pszPassword))
-        {
-            dwError = LW_ERROR_INVALID_PASSWORD;
-            BAIL_ON_LW_ERROR(dwError);
-        }
-    }
-
-    if (ppszDomainDnsName)
-    {
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszDnsDomainName,
-            &pszDomainDnsName);
-        BAIL_ON_LW_ERROR(dwError);
-
-        if (LW_IS_NULL_OR_EMPTY_STR(pszDomainDnsName))
-        {
-            dwError = LW_ERROR_INVALID_DOMAIN;
-            BAIL_ON_LW_ERROR(dwError);
-        }
-    }
-
-    if (ppszHostDnsDomain)
-    {
-        dwError = LwWc16sToMbs(
-            pMachineAcctInfo->pwszHostDnsDomain,
-            &pszHostDnsDomain);
-        BAIL_ON_LW_ERROR(dwError);
-    }
-
-    if (ppszUsername)
-    {
-        *ppszUsername = pszUsername;
-    }
-
-    if (ppszPassword)
-    {
-        *ppszPassword = pszPassword;
-    }
-
-    if (ppszDomainDnsName)
-    {
-        *ppszDomainDnsName = pszDomainDnsName;
-    }
-
-    if (ppszHostDnsDomain)
-    {
-        *ppszHostDnsDomain = pszHostDnsDomain;
-    }
-
-cleanup:
-
-    if (pMachineAcctInfo)
-    {
-        LwpsFreePasswordInfo(hPasswordStore, pMachineAcctInfo);
-    }
-
-    if (hPasswordStore != (HANDLE)NULL)
-    {
-       LwpsClosePasswordStore(hPasswordStore);
-    }
-
-    return dwError;
-
-error:
-
-    if (ppszUsername)
-    {
-        *ppszUsername = NULL;
-    }
-
-    if (ppszPassword)
-    {
-        *ppszPassword = NULL;
-    }
-
-    if (ppszDomainDnsName)
-    {
-        *ppszDomainDnsName = NULL;
-    }
-
-    if (ppszHostDnsDomain)
-    {
-        *ppszHostDnsDomain = NULL;
-    }
-
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_FREE_STRING(pszPassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
-
-    goto cleanup;
-}
-
-DWORD
-LwKrb5RefreshMachineTGT(
-    PDWORD pdwGoodUntilTime
-    )
-{
-    DWORD dwError = 0;
-    DWORD dwGoodUntilTime = 0;
-    PSTR  pszUsername = NULL;
-    PSTR  pszPassword = NULL;
-    PSTR  pszDomainDnsName = NULL;
-    PSTR  pszHostDnsDomain = NULL;
-
-    LW_LOG_VERBOSE("Refreshing machine TGT");
-
-    dwError = LwKrb5GetMachineCreds(
-                    &pszUsername,
-                    &pszPassword,
-                    &pszDomainDnsName,
-                    &pszHostDnsDomain);
-    BAIL_ON_LW_ERROR(dwError);
-
-    dwError = LwSetupMachineSession(
-                    pszUsername,
-                    pszPassword,
-                    pszDomainDnsName,
-                    pszHostDnsDomain,
-                    &dwGoodUntilTime);
-    BAIL_ON_LW_ERROR(dwError);
-
-    if (pdwGoodUntilTime != NULL)
-    {
-        *pdwGoodUntilTime = dwGoodUntilTime;
-    }
-
-cleanup:
-
-    LW_SAFE_FREE_STRING(pszUsername);
-    LW_SAFE_FREE_STRING(pszPassword);
-    LW_SAFE_FREE_STRING(pszDomainDnsName);
-    LW_SAFE_FREE_STRING(pszHostDnsDomain);
-
-    return dwError;
-
-error:
-
-    if (pdwGoodUntilTime != NULL)
-    {
-        *pdwGoodUntilTime = 0;
-    }
-    goto cleanup;
-}
-
 DWORD
 LwTranslateKrb5Error(
     krb5_context ctx,
@@ -925,9 +794,7 @@ LwTranslateKrb5Error(
     DWORD dwLine
     )
 {
-    DWORD dwError = LW_ERROR_SUCCESS;
     PCSTR pszKrb5Error = NULL;
-    unsigned int i = 0;
 
     if (ctx)
     {
@@ -951,6 +818,22 @@ LwTranslateKrb5Error(
                 krbError);
     }
 
+    if (pszKrb5Error)
+    {
+        krb5_free_error_message(ctx, pszKrb5Error);
+    }
+
+    return LwMapKrb5ErrorToLwError(krbError);
+}
+
+DWORD
+LwMapKrb5ErrorToLwError(
+    krb5_error_code krbError
+    )
+{
+    DWORD dwError = LW_ERROR_SUCCESS;
+    unsigned int i = 0;
+
     switch (krbError)
     {
         case ENOENT:
@@ -973,10 +856,6 @@ LwTranslateKrb5Error(
             break;
     }
 
-    if (pszKrb5Error)
-    {
-        krb5_free_error_message(ctx, pszKrb5Error);
-    }
     return dwError;
 }
 
@@ -1082,10 +961,10 @@ LwKrb5VerifyPac(
 
     for (i = 0; i < pPacData->dwBufferCount; i++)
     {
-	switch (pPacData->buffers[i].dwType)
-	{
-	    case PAC_TYPE_LOGON_INFO:
-	        pchLogonInfoStart = (char *)pPacData + pPacData->buffers[i].qwOffset;
+    	switch (pPacData->buffers[i].dwType)
+    	{
+    	    case PAC_TYPE_LOGON_INFO:
+    	        pchLogonInfoStart = (char *)pPacData + pPacData->buffers[i].qwOffset;
                 sLogonInfoLen = pPacData->buffers[i].dwSize;
                 break;
             case PAC_TYPE_SRV_CHECKSUM:
@@ -1107,12 +986,12 @@ LwKrb5VerifyPac(
                 break;
             case PAC_TYPE_KDC_CHECKSUM:
                 /* The checksum is calculated with the signatures zeroed out. */
-		memset(pchPacCopy + pPacData->buffers[i].qwOffset +
-	               (size_t)&((PAC_SIGNATURE_DATA *)0)->pchSignature,
-		       0,
-		       pPacData->buffers[i].dwSize -
-		           (size_t)&((PAC_SIGNATURE_DATA *)0)->pchSignature);
-		break;
+    		memset(pchPacCopy + pPacData->buffers[i].qwOffset +
+    	               (size_t)&((PAC_SIGNATURE_DATA *)0)->pchSignature,
+    		       0,
+    		       pPacData->buffers[i].dwSize -
+    		           (size_t)&((PAC_SIGNATURE_DATA *)0)->pchSignature);
+    		break;
             case PAC_TYPE_LOGON_NAME:
                 pLogonName = (PAC_LOGON_NAME *)((char *)pPacData +
                              pPacData->buffers[i].qwOffset);
@@ -1143,7 +1022,7 @@ LwKrb5VerifyPac(
                 break;
             default:
                 break;
-	}
+    	}
     }
 
     if (pServerSig == NULL)
@@ -1204,20 +1083,20 @@ LwKrb5VerifyPac(
     // Strip off the domain name
     if (strchr(pszClientPrincipal, '@') != NULL)
     {
-        strchr(pszClientPrincipal, '@')[0] = '\0';
+        strchr(pszClientPrincipal, '@')[0] = '\0'; 
     }
 
     dwError = LwWc16snToMbs(
         pLogonName->pwszName,
         &pszLogonName,
         pLogonName->wAccountNameLen / 2);
-    BAIL_ON_LW_ERROR(dwError);
+    BAIL_ON_LW_ERROR(dwError);    
 
     if (strcasecmp(pszClientPrincipal, pszLogonName))
     {
         // The pac belongs to a different user
         dwError = LW_ERROR_INVALID_LOGIN_ID;
-        BAIL_ON_LW_ERROR(dwError);
+        BAIL_ON_LW_ERROR(dwError);    
     }
 
     dwError = LwAllocateMemory(
@@ -1250,8 +1129,8 @@ LwKrb5FindPac(
     krb5_context ctx,
     const krb5_ticket *pTgsTicket,
     const krb5_keyblock *serviceKey,
-    char** ppchLogonInfo,
-    size_t* psLogonInfo
+    OUT PVOID* ppchLogonInfo,
+    OUT size_t* psLogonInfo
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
@@ -1270,9 +1149,9 @@ LwKrb5FindPac(
     ber_tag_t seqTag, context0Tag, context1Tag;
     char* pchLogonInfo = NULL;
     size_t sLogonInfo = 0;
-
+    
     ber = ber_alloc_t(0);
-
+    
     if (pTgsTicket && pTgsTicket->enc_part2)
     {
         ppCur = pTgsTicket->enc_part2->authorization_data;
@@ -1367,19 +1246,18 @@ error:
 }
 
 DWORD
-LwSetupUserLoginSession(
-    uid_t uid,
-    gid_t gid,
-    PCSTR pszUsername,
-    PCSTR pszPassword,
-    BOOLEAN bUpdateUserCache,
-    PCSTR pszServicePrincipal,
-    PCSTR pszServiceRealm,
-    PCSTR pszServicePassword,
-    char** ppchLogonInfo,
-    size_t* psLogonInfo,
-    PDWORD pdwGoodUntilTime,
-    DWORD dwFlags
+LwKrb5InitializeUserLoginCredentials(
+    IN PCSTR pszUserPrincipalName,
+    IN PCSTR pszPassword,
+    IN uid_t uid,
+    IN gid_t gid,
+    IN LW_KRB5_LOGIN_FLAGS Flags,
+    IN PCSTR pszServicePrincipal,
+    IN PCSTR pszServiceRealm,
+    IN PCSTR pszServicePassword,
+    OUT PVOID* ppNdrPacInfo,
+    OUT size_t* pNdrPacInfoSize,
+    OUT PDWORD pdwGoodUntilTime
     )
 {
     DWORD dwError = 0;
@@ -1402,8 +1280,9 @@ LwSetupUserLoginSession(
     BOOLEAN bInLock = FALSE;
     PCSTR pszTempCacheName = NULL;
     PSTR pszTempCachePath = NULL;
-    char* pchLogonInfo = NULL;
-    size_t sLogonInfo = 0;
+    PVOID pNdrPacInfo = NULL;
+    size_t ndrPacInfoSize = 0;
+    DWORD dwGoodUntilTime = 0;
 
     ret = krb5_init_context(&ctx);
     BAIL_ON_KRB_ERROR(ctx, ret);
@@ -1412,30 +1291,28 @@ LwSetupUserLoginSession(
      * be owned by root and only accessible by root.
      */
     ret = krb5_cc_new_unique(
-            ctx,
+            ctx, 
             "FILE",
             "hint",
             &cc);
     BAIL_ON_KRB_ERROR(ctx, ret);
 
 
-    if (dwFlags & LW_USER_LOGIN_SESSION_FLAG_SMART_CARD)
+    if (Flags & LW_KRB5_LOGIN_FLAG_SMART_CARD)
     {
         dwError = LwKrb5GetTgtWithSmartCard(
-                pszUsername,
+                pszUserPrincipalName,
                 pszPassword,
                 krb5_cc_get_name(ctx, cc),
-                pdwGoodUntilTime
-                );
+                &dwGoodUntilTime);
     }
     else
     {
         dwError = LwKrb5GetTgt(
-                pszUsername,
+                pszUserPrincipalName,
                 pszPassword,
                 krb5_cc_get_name(ctx, cc),
-                pdwGoodUntilTime
-                );
+                &dwGoodUntilTime);
     }
 
     BAIL_ON_LW_ERROR(dwError);
@@ -1445,7 +1322,7 @@ LwSetupUserLoginSession(
 
     ret = krb5_cc_get_principal(ctx, cc, &credsRequest.client);
     BAIL_ON_KRB_ERROR(ctx, ret);
-
+ 
     /* Get a TGS for our service using the tgt in the cache */
     ret = krb5_get_credentials(
             ctx,
@@ -1462,7 +1339,7 @@ LwSetupUserLoginSession(
     if (ret != 0) {
         pTgsCreds = NULL;
     }
-
+    
     BAIL_ON_KRB_ERROR(ctx, ret);
 
     //No need to store the tgs in the cc. Kerberos does that automatically
@@ -1505,7 +1382,7 @@ LwSetupUserLoginSession(
     ret = krb5_c_string_to_key(
             ctx,
             pTgsTicket->enc_part.enctype,
-	    &machinePassword,
+            &machinePassword,
             &salt,
             &serviceKey);
     BAIL_ON_KRB_ERROR(ctx, ret);
@@ -1524,7 +1401,7 @@ LwSetupUserLoginSession(
         ret = krb5_auth_con_free(ctx, authContext);
         BAIL_ON_KRB_ERROR(ctx, ret);
     }
-
+    
     ret = krb5_auth_con_init(ctx, &authContext);
     BAIL_ON_KRB_ERROR(ctx, ret);
 
@@ -1562,18 +1439,18 @@ LwSetupUserLoginSession(
             pTgsCreds->server,
             NULL, /* we're not using the keytab */
             &flags,
-	    &pDecryptedTgs);
+            &pDecryptedTgs);
     BAIL_ON_KRB_ERROR(ctx, ret);
 
     dwError = LwKrb5FindPac(
         ctx,
         pDecryptedTgs,
         &serviceKey,
-        &pchLogonInfo,
-        &sLogonInfo);
+        &pNdrPacInfo,
+        &ndrPacInfoSize);
     BAIL_ON_LW_ERROR(dwError);
 
-    if (bUpdateUserCache)
+    if (Flags & LW_KRB5_LOGIN_FLAG_UPDATE_CACHE)
     {
         /* 1. Copy old credentials from the existing user creds cache to
          *      the temporary cache.
@@ -1624,35 +1501,39 @@ LwSetupUserLoginSession(
         }
     }
 
-    *ppchLogonInfo = pchLogonInfo;
-    *psLogonInfo = sLogonInfo;
+error:
+    if (dwError)
+    {
+        LW_SAFE_FREE_MEMORY(pNdrPacInfo);
+        ndrPacInfoSize = 0;
+        dwGoodUntilTime = 0;
+    }
 
-cleanup:
     if (ctx)
     {
         // This function skips fields which are NULL
         krb5_free_cred_contents(ctx, &credsRequest);
-
+    
         if (pTgsCreds != NULL)
         {
             krb5_free_creds(ctx, pTgsCreds);
         }
-
+        
         if (pTgsTicket != NULL)
         {
             krb5_free_ticket(ctx, pTgsTicket);
         }
-
+        
         if (pDecryptedTgs != NULL)
         {
             krb5_free_ticket(ctx, pDecryptedTgs);
         }
-
+        
         if (authContext != NULL)
         {
             krb5_auth_con_free(ctx, authContext);
         }
-
+        
         krb5_free_data_contents(ctx, &apReqPacket);
         krb5_free_data_contents(ctx, &salt);
         krb5_free_keyblock_contents(ctx, &serviceKey);
@@ -1669,14 +1550,114 @@ cleanup:
     }
     LW_SAFE_FREE_STRING(pszTempCachePath);
 
+    *ppNdrPacInfo = pNdrPacInfo;
+    *pNdrPacInfoSize = ndrPacInfoSize;
+    *pdwGoodUntilTime = dwGoodUntilTime;
+
     return dwError;
+}
+
+DWORD
+LwKrb5CheckInitiatorCreds(
+    IN PCSTR pszTargetPrincipalName,
+    OUT PBOOLEAN pbNeedCredentials
+    )
+{
+    DWORD dwError = 0;
+    BOOLEAN bNeedCredentials = FALSE;
+    OM_uint32 majorStatus = 0;
+    OM_uint32 minorStatus = 0;
+    gss_ctx_id_t gssContext = GSS_C_NO_CONTEXT;
+    gss_buffer_desc importName  = GSS_C_EMPTY_BUFFER;
+    gss_buffer_desc inputBufferDesc  = GSS_C_EMPTY_BUFFER;
+    gss_buffer_desc outputBufferDesc = GSS_C_EMPTY_BUFFER;
+    OM_uint32 outputFlags = 0;
+    gss_name_t targetName = GSS_C_NO_NAME;
+
+    // discard const
+    importName.value = (PSTR) pszTargetPrincipalName;
+    importName.length = strlen(pszTargetPrincipalName);
+
+    // discard const from GSS_KRB5_NT_PRINCIPAL_NAME
+    majorStatus = gss_import_name(&minorStatus,
+                                  &importName,
+                                  (gss_OID) GSS_KRB5_NT_PRINCIPAL_NAME,
+                                  &targetName);
+    LW_GSS_LOG_IF_NOT_COMPLETE("gss_import_name", majorStatus, minorStatus);
+    BAIL_ON_GSS_ERROR(dwError, majorStatus, minorStatus);
+
+    // discard const from GSS_KRB5_NT_PRINCIPAL_NAME
+    majorStatus = gss_init_sec_context(&minorStatus,
+                                       NULL,
+                                       &gssContext,
+                                       targetName,
+                                       (gss_OID) gss_mech_krb5,
+                                       GSS_C_REPLAY_FLAG | GSS_C_MUTUAL_FLAG,
+                                       0,
+                                       NULL,
+                                       &inputBufferDesc,
+                                       NULL,
+                                       &outputBufferDesc,
+                                       &outputFlags,
+                                       NULL);
+    LW_GSS_LOG_IF_NOT_COMPLETE_OR_CONTINUE("gss_init_sec_context", majorStatus, minorStatus);
+
+    // Need to cast below for correct comparison.  Note that
+    // some compilers do not catch the mismatch.
+    if (((majorStatus == GSS_S_FAILURE) &&
+         ((minorStatus == (OM_uint32) KRB5KRB_AP_ERR_TKT_EXPIRED) ||
+          (minorStatus == (OM_uint32) KRB5KDC_ERR_NEVER_VALID) ||
+          (minorStatus == (OM_uint32) KRB5KDC_ERR_TGT_REVOKED))) ||
+        ((majorStatus == GSS_S_CRED_UNAVAIL) &&
+         (minorStatus == KG_EMPTY_CCACHE)))
+    {
+        // Need (new) Kerberos credentials because there are no
+        // credentials or the credentials are expired or otherwise
+        // invalid.
+
+        bNeedCredentials = TRUE;
+        goto error;
+    }
+
+    if (majorStatus == GSS_S_FAILURE)
+    {
+        switch (minorStatus)
+        {
+            case (OM_uint32) KRB5KRB_AP_ERR_SKEW:
+                dwError = ERROR_TIME_SKEW;
+                BAIL_ON_LW_ERROR(dwError);
+                break;
+            default:
+                BAIL_ON_GSS_ERROR(dwError, majorStatus, minorStatus);
+                break;
+        }
+    }
+
+    if ((majorStatus != GSS_S_COMPLETE) &&
+        (majorStatus != GSS_S_CONTINUE_NEEDED))
+    {
+        BAIL_ON_GSS_ERROR(dwError, majorStatus, minorStatus);
+    }
 
 error:
+    if (targetName)
+    {
+        gss_release_name(&minorStatus, &targetName);
+    }
 
-    LW_SAFE_FREE_MEMORY(pchLogonInfo);
-    *ppchLogonInfo = NULL;
+    if (outputBufferDesc.value)
+    {
+        majorStatus = gss_release_buffer(&minorStatus, &outputBufferDesc);
+    }
 
-    goto cleanup;
+    if (gssContext)
+    {
+        majorStatus = gss_delete_sec_context(&minorStatus, &gssContext, GSS_C_NO_BUFFER);
+    }
+
+    *pbNeedCredentials = bNeedCredentials;
+
+    return dwError;
 }
 
 /*
