@@ -11,7 +11,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -47,12 +47,17 @@
  *          Brian Dunstan (bdunstan@likewisesoftware.com)
  */
 #include "includes.h"
+#include <gssapi/gssapi_krb5.h>
 
 /* used by inet_addr, not defined on Solaris anywhere!? */
 #ifndef INADDR_NONE
 #define INADDR_NONE ((in_addr_t) -1)
 #endif
 
+#ifndef SASL_CB_SERVERFQDN
+#define SASL_CB_SERVERFQDN 0xc0004001
+#endif
+
 DWORD
 LwCLdapOpenDirectory(
     IN PCSTR pszServerName,
@@ -66,7 +71,7 @@ LwCLdapOpenDirectory(
     PSTR pszURL = NULL;
 
     LW_BAIL_ON_INVALID_STRING(pszServerName);
-
+    
     dwError = LwAllocateStringPrintf(&pszURL, "cldap://%s",
                                         pszServerName);
     BAIL_ON_LW_ERROR(dwError);
@@ -116,7 +121,11 @@ LwLdapPingTcp(
     struct timeval timeout;
     fd_set fds;
     int socketError;
-    SOCKLEN_T socketErrorLength = 0;
+#ifdef GETSOCKNAME_TAKES_SOCKLEN_T
+    socklen_t socketErrorLength = 0;
+#else
+    int socketErrorLength = 0;
+#endif
 
     addr.s_addr = inet_addr(pszHostAddress);
     if (addr.s_addr == INADDR_NONE)
@@ -238,7 +247,7 @@ LwLdapOpenDirectoryServerSingleAttempt(
     int rc = LDAP_VERSION3;
     DWORD dwPort = 389;
     struct timeval timeout = {0};
-    DWORD dwSecurity = GSS_C_MUTUAL_FLAG | GSS_C_REPLAY_FLAG;
+    BOOLEAN bLdapSeal = FALSE;
 
     timeout.tv_sec = dwTimeoutSec;
 
@@ -284,26 +293,9 @@ LwLdapOpenDirectoryServerSingleAttempt(
         BAIL_ON_LDAP_ERROR(dwError);
     }
 
-    dwSecurity |= GSS_C_INTEG_FLAG;
-
     if (dwFlags & LW_LDAP_OPT_SIGN_AND_SEAL)
     {
-        dwSecurity |= GSS_C_CONF_FLAG;
-    }
-
-    dwError = ldap_set_option(ld, LDAP_OPT_SSPI_FLAGS, (void*)&dwSecurity);
-    if (dwError) {
-        LW_LOG_ERROR("Failed to set LDAP GSS-API option to"
-                      " sign and/or seal");
-        BAIL_ON_LDAP_ERROR(dwError);
-    }
-
-    dwError = ldap_set_option(ld, LDAP_OPT_X_GSSAPI_ALLOW_REMOTE_PRINCIPAL,
-                              LDAP_OPT_ON);
-    if (dwError) {
-        LW_LOG_ERROR("Failed to set LDAP GSS-API option to allow"
-                      " remote principals");
-        BAIL_ON_LDAP_ERROR(dwError);
+        bLdapSeal = TRUE;
     }
 
     dwError = LwAllocateMemory(sizeof(*pDirectory), OUT_PPVOID(&pDirectory));
@@ -318,7 +310,10 @@ LwLdapOpenDirectoryServerSingleAttempt(
     }
     else
     {
-        dwError = LwLdapBindDirectory((HANDLE)pDirectory, pszServerName);
+        dwError = LwLdapBindDirectory(
+                      (HANDLE)pDirectory,
+                      pszServerName,
+                      bLdapSeal);
     }
     // The above functions return -1 when a connection times out.
     if (dwError == (DWORD)-1)
@@ -460,202 +455,128 @@ error:
 DWORD
 LwLdapBindDirectory(
     HANDLE hDirectory,
-    PCSTR pszServerName
+    PCSTR pszServerName,
+    BOOLEAN bSeal 
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
-    DWORD dwMajorStatus = 0;
-    DWORD dwMinorStatus = 0;
+    PLW_LDAP_DIRECTORY_CONTEXT pDirectory = (PLW_LDAP_DIRECTORY_CONTEXT)hDirectory;
+    PSTR pszTargetPrinciaplName = NULL;
+    BOOLEAN bNeedCredentials = FALSE;
 
-    CtxtHandle GSSContext = {0};
-    PCtxtHandle pGSSContext = &GSSContext;
-
-    PSTR pszTargetName = NULL;
-
-    PLW_LDAP_DIRECTORY_CONTEXT pDirectory = NULL;
-
-    gss_buffer_desc input_name  = GSS_C_EMPTY_BUFFER;
-    gss_buffer_desc input_desc  = GSS_C_EMPTY_BUFFER;
-    gss_buffer_desc output_desc = GSS_C_EMPTY_BUFFER;
-
-    gss_buffer_desc token = GSS_C_EMPTY_BUFFER;
-
-//    PCtxtHandle pContextHandle = NULL;
-    OM_uint32 ret_flags = 0;
-
-    gss_name_t targ_name = GSS_C_NO_NAME;
-
-    input_desc.value = NULL;
-    input_desc.length = 0;
-
-    //Leave the realm empty so that kerberos referrals are turned on.
-    dwError = LwAllocateStringPrintf(&pszTargetName,"ldap/%s@", pszServerName);
+    // Leave the realm empty so that kerberos referrals are turned on.
+    dwError = LwAllocateStringPrintf(&pszTargetPrinciaplName, "ldap/%s@", pszServerName);
     BAIL_ON_LW_ERROR(dwError);
 
-    pDirectory = (PLW_LDAP_DIRECTORY_CONTEXT)hDirectory;
+    dwError = LwKrb5CheckInitiatorCreds(pszTargetPrinciaplName, &bNeedCredentials);
+    BAIL_ON_LW_ERROR(dwError);
 
-    input_name.value = pszTargetName;
-    input_name.length = strlen(pszTargetName);
-
-    dwMajorStatus = gss_import_name((OM_uint32 *)&dwMinorStatus,
-                                    &input_name,
-                                    (gss_OID)GSS_KRB5_NT_PRINCIPAL_NAME,
-                                    &targ_name);
-    display_status("gss_import_name", dwMajorStatus, dwMinorStatus);
-    BAIL_ON_SEC_ERROR(dwMajorStatus);
-
-    token.value = "{1 3 6 1 5 5 2}";
-    token.length = strlen(token.value);
-
-    memset(pGSSContext, 0, sizeof(CtxtHandle));
-    *pGSSContext = GSS_C_NO_CONTEXT;
-
-    dwMajorStatus = gss_init_sec_context((OM_uint32 *)&dwMinorStatus,
-                                         NULL,
-                                         pGSSContext,
-                                         targ_name,
-                                         (gss_OID)gss_mech_krb5,
-                                         GSS_C_REPLAY_FLAG | GSS_C_MUTUAL_FLAG,
-                                         0,
-                                         NULL,
-                                         &input_desc,
-                                         NULL,
-                                         &output_desc,
-                                         &ret_flags,
-                                         NULL);
-
-    display_status("gss_init_context", dwMajorStatus, dwMinorStatus);
-    if (
-        (dwMajorStatus == GSS_S_FAILURE &&
-        (dwMinorStatus == (DWORD)KRB5KRB_AP_ERR_TKT_EXPIRED ||
-         dwMinorStatus == (DWORD)KRB5KDC_ERR_NEVER_VALID ||
-         dwMinorStatus == (DWORD)KRB5KDC_ERR_TGT_REVOKED)) ||
-        (dwMajorStatus == GSS_S_CRED_UNAVAIL &&
-        dwMinorStatus == 0x25ea10c /* This is KG_EMPTY_CCACHE
-                                    * inside of gssapi, but that symbol
-                                    * is not exposed externally. This
-                                    * code means that the credentials
-                                    * cache does not have a TGT inside
-                                    * of it.
-                                    */
-        )
-	)
+    if (bNeedCredentials)
     {
-        /* The kerberos ticket expired or is about to expire (The
-         * machine password sync thread didn't do its job).
-         */
-        LW_LOG_INFO("Renewing machine tgt outside of password sync thread");
-
-        dwError = LwKrb5RefreshMachineTGT(NULL);
+        dwError = SEC_E_NO_CREDENTIALS;
         BAIL_ON_LW_ERROR(dwError);
     }
 
-    if (dwMajorStatus == GSS_S_FAILURE)
+    dwError = LwLdapBindDirectorySasl(pDirectory->ld, pszServerName, bSeal);
+    BAIL_ON_LW_ERROR(dwError);
+
+error:
+    LW_SAFE_FREE_STRING(pszTargetPrinciaplName);
+
+    return dwError;
+}
+
+static int
+LwLdapGssSpnegoInteract(
+    LDAP *ld,
+    unsigned flags,
+    void *defaults,
+    void *in
+){
+    sasl_interact_t *interact = in;
+
+    while (interact->id != SASL_CB_LIST_END)
     {
-        switch (dwMinorStatus)
+        switch (interact->id)
         {
-            case KRB5KRB_AP_ERR_SKEW:
-                dwError = ERROR_TIME_SKEW;
-                BAIL_ON_LW_ERROR(dwError);
+            case SASL_CB_SERVERFQDN:
+                // Supply the server's FQDN which is
+                // required for Kerberos authentication
+
+                interact->result = defaults;
+                interact->len = strlen(interact->result);
+                interact++;
                 break;
+
             default:
-                BAIL_ON_SEC_ERROR(dwMajorStatus);
-                break;
+                interact->result = "";
+                interact->len = strlen(interact->result);
+                interact++;
         }
     }
 
-    if (dwMajorStatus != 0 &&
-        dwMajorStatus != GSS_S_CONTINUE_NEEDED)
+    return LDAP_SUCCESS;
+}
+
+DWORD
+LwLdapBindDirectorySasl(
+    LDAP *ld,
+    PCSTR pszServerName,
+    BOOLEAN bSeal
+    )
+{
+    DWORD dwError = LW_ERROR_SUCCESS;
+
+    // Do not attempt to canonicalize the server
+    // name which isn't necessary since we used
+    // the server's FQDN or address.
+
+    dwError = ldap_set_option(
+                  ld,
+                  LDAP_OPT_X_SASL_NOCANON,
+                  LDAP_OPT_ON);
+    BAIL_ON_LDAP_ERROR(dwError);
+
+    // ssf=1 is sign, ssf>1 is seal.  By default
+    // it will use the maximum available ssf level
+    // so setting minssf isn't strictly necessary.
+    // Setting minssf guarantees an error if it
+    // cannot provide the minimum level.
+
+    if (bSeal)
     {
-        BAIL_ON_SEC_ERROR(dwMajorStatus);
+        dwError = ldap_set_option(
+                      ld,
+                      LDAP_OPT_X_SASL_SECPROPS,
+                      (void *)"minssf=2");
+        BAIL_ON_LDAP_ERROR(dwError);
+    }
+    else
+    {
+        dwError = ldap_set_option(
+                      ld,
+                      LDAP_OPT_X_SASL_SECPROPS,
+                      (void *)"minssf=1,maxssf=1");
+        BAIL_ON_LDAP_ERROR(dwError);
     }
 
-
-    dwError = ldap_gssapi_bind_s(pDirectory->ld, NULL, NULL);
+    dwError = ldap_sasl_interactive_bind_s(
+                  ld,
+                  NULL,
+                  "GSS-SPNEGO",
+                  NULL,
+                  NULL,
+                  LDAP_SASL_QUIET,
+                  LwLdapGssSpnegoInteract,
+                  (void *)pszServerName);
     if (dwError != 0) {
-        LW_LOG_ERROR("ldap_gssapi_bind_s failed with error code %d", dwError);
+        LW_LOG_ERROR("ldap_sasl_interactive_bind_s failed with error code %d", dwError);
         BAIL_ON_LDAP_ERROR(dwError);
     }
 
 error:
 
-    if (targ_name) {
-        gss_release_name((OM_uint32*)&dwMinorStatus, &targ_name);
-    }
-
-    if (output_desc.value) {
-	gss_release_buffer((OM_uint32 *)&dwMinorStatus, &output_desc);
-    }
-
-    if (*pGSSContext != GSS_C_NO_CONTEXT) {
-        gss_delete_sec_context((OM_uint32*)&dwMinorStatus, pGSSContext, GSS_C_NO_BUFFER);
-    }
-
-    LW_SAFE_FREE_STRING(pszTargetName);
-
-    return(dwError);
-
-}
-
-void display_status(char *msg, OM_uint32 maj_stat, OM_uint32 min_stat)
-{
-    // BAIL_ON_SEC_ERROR shows this code
-    // display_status_1(msg, maj_stat, GSS_C_GSS_CODE);
-    display_status_1(msg, min_stat, GSS_C_MECH_CODE);
-}
-
-void display_status_1(char *m, OM_uint32 code, int type)
-{
-    OM_uint32 maj_stat, min_stat;
-    gss_buffer_desc msg;
-    OM_uint32 msg_ctx;
-
-    if ( code == 0 )
-    {
-        return;
-    }
-
-    msg_ctx = 0;
-    while (1) {
-        maj_stat = gss_display_status(&min_stat, code,
-                                      type, GSS_C_NULL_OID,
-                                      &msg_ctx, &msg);
-
-	switch(code)
-	{
-#ifdef WIN32
-	case SEC_E_OK:
-	case SEC_I_CONTINUE_NEEDED:
-#else
-        case GSS_S_COMPLETE:
-        case GSS_S_CONTINUE_NEEDED:
-#endif
-            LW_LOG_DEBUG("GSS-API error calling %s: %d (%s)", m, code, (char *)msg.value);
-	    break;
-	default:
-            LW_LOG_DEBUG("GSS-API error calling %s: %d (%s)", m, code, (char *)msg.value);
-	}
-
-        (void) gss_release_buffer(&min_stat, &msg);
-
-        if (!msg_ctx)
-            break;
-    }
-    if (code == (OM_uint32)KRB5_FCC_NOFILE)
-    {
-        PSTR pszOrigCachePath = NULL;
-
-        LW_LOG_DEBUG("KRB5CCNAME is set to %s",
-                LW_SAFE_LOG_STRING(getenv("KRB5CCNAME")));
-
-        if (!LwKrb5SetDefaultCachePath(NULL, &pszOrigCachePath))
-        {
-            LW_LOG_DEBUG("gss krb5 ccache is set to %s",
-                    LW_SAFE_LOG_STRING(pszOrigCachePath));
-            LwKrb5SetDefaultCachePath(pszOrigCachePath, NULL);
-            LW_SAFE_FREE_MEMORY(pszOrigCachePath);
-        }
-    }
+    return dwError;
 }
 
 void
@@ -1453,7 +1374,7 @@ error:
 }
 
 DWORD
-LwLdapGetInt64(
+LwLdapGetInt64( 
     IN HANDLE hDirectory,
     IN LDAPMessage* pMessage,
     IN PCSTR pszFieldName,
@@ -1463,12 +1384,12 @@ LwLdapGetInt64(
     DWORD dwError = 0;
     PSTR pszValue = NULL;
     PSTR pszEndPtr = NULL;
-
+ 
     dwError = LwLdapGetString(hDirectory, pMessage, pszFieldName, &pszValue);
     BAIL_ON_LW_ERROR(dwError);
-
+ 
     if (pszValue)
-    {
+    {     
 #if SIZEOF_LONG == 8
         *pqwValue = strtol(pszValue, &pszEndPtr, 10);
 #else
@@ -1478,9 +1399,9 @@ LwLdapGetInt64(
         {
             dwError = LW_ERROR_DATA_ERROR;
             BAIL_ON_LW_ERROR(dwError);
-        }
-    }
-    else
+        }   
+    }       
+    else    
     {
         dwError = LW_ERROR_INVALID_LDAP_ATTR_VALUE;
         // This error occurs very frequently (every time an unenabled user
@@ -1695,6 +1616,70 @@ error:
     goto cleanup;
 }
 
+DWORD
+LwLdapPutString(
+    HANDLE hDirectory,
+    PCSTR  pszDN,
+    PCSTR  pszFieldName,
+    PSTR   pszValue
+    )
+{
+    DWORD dwError = 0;
+    PLW_LDAP_DIRECTORY_CONTEXT pDirectory = NULL;
+    PSTR modvals[2];
+    LDAPMod  mod;
+    LDAPMod* mods[2];
+
+    pDirectory = (PLW_LDAP_DIRECTORY_CONTEXT)hDirectory;
+
+    modvals[0] = pszValue;
+    modvals[1] = NULL;
+    
+    mod.mod_op = LDAP_MOD_REPLACE;
+    mod.mod_type = (PSTR)pszFieldName;
+    mod.mod_values = modvals;
+    
+    mods[0] = &mod;
+    mods[1] = NULL;
+    
+    dwError = ldap_modify_s(pDirectory->ld, pszDN, mods);
+    if (dwError)
+    {
+        LW_LOG_ERROR("Failed to update LDAP object attribute string with error: %d, errno: %d", dwError, errno);
+    }
+    BAIL_ON_LW_ERROR(dwError);
+    
+error:
+
+    return dwError;
+}
+
+DWORD
+LwLdapPutUInt32(
+    HANDLE hDirectory,
+    PCSTR  pszDN,
+    PCSTR  pszFieldName,
+    DWORD  dwValue
+    )
+{
+    DWORD dwError = 0;
+    char szValue[256];
+
+    memset(szValue, 0, sizeof(szValue));
+    sprintf(szValue, "%u", dwValue);
+    
+    dwError = LwLdapPutString(hDirectory, pszDN, pszFieldName, szValue);
+    if (dwError)
+    {
+        LW_LOG_ERROR("Failed to update LDAP object attribute integer with error: %d, errno: %d", dwError, errno);
+    }
+    BAIL_ON_LW_ERROR(dwError);
+
+error:
+
+    return dwError;
+}
+
 /* Escapes a string according to the directions given in RFC 2254. */
 DWORD
 LwLdapEscapeString(
@@ -1973,7 +1958,6 @@ LwLdapDirectoryExtendedDNSearch(
     )
 {
     DWORD dwError = LW_ERROR_SUCCESS;
-    PLW_LDAP_DIRECTORY_CONTEXT pDirectory = NULL;
     CHAR ExtDNCriticality = 'T';
     LDAPControl *pExtDNControl = NULL;
     LDAPControl *ppInputControls[2] = { NULL, NULL };
@@ -1981,7 +1965,6 @@ LwLdapDirectoryExtendedDNSearch(
     struct berval value = {0};
 
 
-    pDirectory = (PLW_LDAP_DIRECTORY_CONTEXT)hDirectory;
     // Setup the extended DN control, in order to be windows 2000 compatible,
     // Do not specify control value, hence, the return result will always be in hexadecimal string format.
     value.bv_len = 0;
