@@ -51,6 +51,13 @@ LwInterlockedCompareExchange(
     LW_IN LW_LONG lCompareValue
     );
 
+LW_PVOID
+LwInterlockedCompareExchangePointer(
+    LW_IN LW_OUT LW_PVOID volatile *ppDestination,
+    LW_IN LW_PVOID pNewPointer,
+    LW_IN LW_PVOID pComparePointer
+    );
+
 LW_LONG
 LwInterlockedRead(
     LW_IN LW_LONG volatile *plSource
@@ -70,7 +77,8 @@ LwInterlockedDecrement(
 
 #define InterlockedCompareExchange(Desination, NewValue, CompareValue) \
     LwInterlockedCompareExchange(Desination, NewValue, CompareValue)
-
+#define InterlockedCompareExchangePointer(Destination, Exchange, Comparand) \
+    LwInterlockedCompareExchangePointer(Destination, Exchange, Comparand)
 #define InterlockedRead(Source)             LwInterlockedRead(Source)
 #define InterlockedIncrement(Destination)   LwInterlockedIncrement(Destination)
 #define InterlockedDecrement(Destination)   LwInterlockedDecrement(Destination)
