@@ -53,7 +53,10 @@
 #include <lw/security-api.h>
 #include <lw/dscache.h>
 #include <lw/threadpool.h>
+#include <lw/affinity.h>
 #include <lw/mapsecurity.h>
 #include <lw/safeint.h>
+#include <lw/hash.h>
+#include <lw/svcm.h>
 
 #endif
