@@ -39,7 +39,7 @@
 #ifndef __LWBASE_SECURITY_API_H__
 #define __LWBASE_SECURITY_API_H__
 
-#include "security-types.h"
+#include <lw/security-types.h>
 #include <lw/ntstatus.h>
 #include <lw/rtlstring.h>
 
@@ -169,6 +169,125 @@ RtlDuplicateSid(
     IN PSID OriginalSid
     );
 
+NTSTATUS
+RtlConvertLittleEndianToSid(
+    IN PVOID Buffer,
+    IN ULONG Length,
+    OUT PSID Sid,           /* The pBuffer MAY be used here */
+    IN OUT PULONG SidSize
+    );
+
+NTSTATUS
+RtlConvertSidToLittleEndian(
+    IN PSID Sid,
+    OUT PVOID Buffer,        /* The pSid MAY be used here */
+    IN ULONG Length,
+    OUT OPTIONAL PULONG UsedLength
+    );
+
+//
+// LUID Functions
+//
+
+LUID
+RtlConvertLongToLuid(
+    IN LONG Long
+    );
+///<
+/// Convert LONG type to LUID.
+///
+/// Assign long integer value to LowPart of the LUID.
+///
+/// @param[in] Long - long integer value
+///
+/// @return LUID
+///
+
+LUID
+RtlConvertUlongToLuid(
+    IN ULONG Ulong
+    );
+///<
+/// Convert ULONG type to LUID.
+///
+/// Assign unsigned long integer value to LowPart of the LUID.
+///
+/// @param[in] Ulong - unsigned long integer value
+///
+/// @return LUID
+///
+
+BOOLEAN
+RtlEqualLuid(
+    IN PLUID Luid1,
+    IN PLUID Luid2
+    );
+///<
+/// Compare two LUIDs.
+///
+/// Returns TRUE if both LUIDs represent the same value.
+///
+/// @param[in] Luid1 - pointer to LUID variable
+/// @param[in] Luid2 - pointer to LUID variable
+///
+/// @return BOOLEAN
+///
+
+
+//
+// PRIVILEGE_SET Functions
+//
+
+ULONG
+RtlLengthRequiredPrivilegeSet(
+    IN ULONG PrivilegeCount
+    );
+///<
+/// Get length required for a PRIVILEGE_SET.
+///
+/// Get number of bytes required to store a PRIVILEGE_SET with
+/// the specified number of privileges.
+///
+/// @param[in] PrivilegeCount - Number of privileges
+///
+/// @return Number of bytes required to store the PRIVILEGE_SET.
+///
+
+ULONG
+RtlLengthPrivilegeSet(
+    IN PPRIVILEGE_SET PrivilegeSet
+    );
+///<
+/// Get length of a PRIVILEGE_SET.
+///
+/// Get number of bytes required to store the specified PRIVILEGE_SET.
+///
+/// @param[in] PrivilegeSet - A privilege set
+///
+/// @return Number of bytes required to store the PRIVILEGE_SET.
+///
+
+NTSTATUS
+RtlCopyPrivilegeSet(
+    IN ULONG DestinationPrivilegeSetLength,
+    OUT PPRIVILEGE_SET DestinationPrivilegeSet,
+    IN PPRIVILEGE_SET SourcePrivilegeSet
+    );
+///<
+/// Copy a privilege set.
+///
+/// Copy a privilege set to allocated buffer of specified length.
+///
+/// @param[in] DestinationPrivilegeSetLength - Length of the destination PRIVILEGE_SET
+/// @param[out] DestinationPrivilegeSet - The PRIVILEGE_SET to copy to
+/// @param[in] SourcePrivilegeSet - The PRIVILEGE_SET to copy from
+///
+/// @return NTSTATUS
+///   @arg STATUS_SUCCESS on success
+///   @arg STATUS_BUFFER_TOO_SMALL if the destination buffer length is too small
+///
+
+
 //
 // ACL Functions
 //
@@ -288,6 +407,11 @@ RtlLengthSecurityDescriptor(
     IN PSECURITY_DESCRIPTOR_ABSOLUTE SecurityDescriptor
     );
 
+ULONG
+RtlLengthSecurityDescriptorRelative(
+    IN PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor
+    );
+
 NTSTATUS
 RtlGetSecurityDescriptorControl(
     IN PSECURITY_DESCRIPTOR_ABSOLUTE SecurityDescriptor,
@@ -387,6 +511,12 @@ RtlAbsoluteToSelfRelativeSD(
     IN OUT PULONG BufferLength
     );
 
+NTSTATUS
+RtlAbsoluteToSelfRelativeSDSwab(
+    IN OUT PSECURITY_DESCRIPTOR_RELATIVE SelfRelativeSecurityDescriptor,
+    IN ULONG BufferLength
+    );
+
 NTSTATUS
 RtlSelfRelativeToAbsoluteSD(
     IN PSECURITY_DESCRIPTOR_RELATIVE SelfRelativeSecurityDescriptor,
@@ -418,7 +548,7 @@ RtlSetSecurityDescriptorInfo(
     OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE NewObjectSecurityDescriptor,
     IN OUT PULONG NewObjectSecurityDescriptorLength,
     IN PGENERIC_MAPPING GenericMapping
-    );
+    ); 
 
 NTSTATUS
 RtlCreatePrivateObjectSecurityEx(
@@ -607,6 +737,41 @@ RtlCreateWellKnownSid(
     IN OUT PULONG SidSize
     );
 
+NTSTATUS
+RtlAllocateWellKnownSid(
+    IN WELL_KNOWN_SID_TYPE WellKnownSidType,
+    IN OPTIONAL PSID DomainOrComputerSid,
+    OUT PSID* Sid
+    );
+
+//
+// SDDL Functions
+//
+
+NTSTATUS
+RtlAllocateSecurityDescriptorFromSddlCString(
+    OUT PSECURITY_DESCRIPTOR_RELATIVE* ppSecurityDescriptor,
+    OUT OPTIONAL PULONG pSecurityDescriptorLength,
+    IN PCSTR pszStringSecurityDescriptor,
+    IN ULONG SddlRevision
+    );
+
+
+NTSTATUS
+RtlAllocateSddlCStringFromSecurityDescriptor(
+    OUT PSTR* ppszStringSecurityDescriptor,
+    IN PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
+    IN ULONG SddlRevision,
+    IN SECURITY_INFORMATION SecurityInformation
+    );
+
+NTSTATUS
+RtlGetSecurityInformationFromSddlCString(
+    IN PCSTR pszStringSecurityDescriptor,
+    OUT SECURITY_INFORMATION* pSecInfo
+    );
+
+
 LW_END_EXTERN_C
 
 #endif /* __LWBASE_SECURITY_API_H__ */
