@@ -137,6 +137,10 @@ typedef ULONG ACCESS_MASK, *PACCESS_MASK;
     0 )
 
 #define VALID_DACL_ACCESS_MASK ( \
+    GENERIC_ALL | \
+    GENERIC_EXECUTE | \
+    GENERIC_WRITE | \
+    GENERIC_READ | \
     STANDARD_RIGHTS_ALL | \
     SPECIFIC_RIGHTS_ALL | \
     0 )
@@ -150,6 +154,100 @@ typedef ULONG ACCESS_MASK, *PACCESS_MASK;
     VALID_SACL_ACCESS_MASK | \
     0 )
 
+
+//
+// Specific Access Rights - File
+//
+
+#define FILE_READ_DATA              0x00000001 // File/Pipe
+#define FILE_LIST_DIRECTORY         0x00000001 // Directory
+#define FILE_WRITE_DATA             0x00000002 // File/Pipe
+#define FILE_ADD_FILE               0x00000002 // Directory
+#define FILE_APPEND_DATA            0x00000004 // File
+#define FILE_ADD_SUBDIRECTORY       0x00000004 // Directory
+#define FILE_CREATE_PIPE_INSTANCE   0x00000004 // Pipe
+#define FILE_READ_EA                0x00000008 // File/Directory
+#define FILE_WRITE_EA               0x00000010 // File/Directory
+#define FILE_EXECUTE                0x00000020 // File
+#define FILE_TRAVERSE               0x00000020 // Directory
+#define FILE_DELETE_CHILD           0x00000040 // Directory
+#define FILE_READ_ATTRIBUTES        0x00000080 // File/Pipe/Directory
+#define FILE_WRITE_ATTRIBUTES       0x00000100 // File/Pipe/Directory
+
+#define FILE_ALL_ACCESS ( \
+    SYNCHRONIZE | \
+    STANDARD_RIGHTS_REQUIRED | \
+    0x000001FF | \
+    0 )
+
+#define FILE_GENERIC_READ ( \
+    SYNCHRONIZE | \
+    STANDARD_RIGHTS_READ | \
+    FILE_READ_ATTRIBUTES | \
+    FILE_READ_DATA | \
+    FILE_READ_EA | \
+    0 )
+
+#define FILE_GENERIC_WRITE ( \
+    SYNCHRONIZE | \
+    STANDARD_RIGHTS_WRITE | \
+    FILE_WRITE_ATTRIBUTES | \
+    FILE_WRITE_DATA | \
+    FILE_WRITE_EA | \
+    FILE_APPEND_DATA | \
+    0 )
+
+#define FILE_GENERIC_EXECUTE ( \
+    SYNCHRONIZE | \
+    STANDARD_RIGHTS_EXECUTE | \
+    FILE_READ_ATTRIBUTES | \
+    FILE_EXECUTE | \
+    0 )
+
+//
+// Specific Access Rights - Registry
+//
+
+#define KEY_QUERY_VALUE         0x0001 //Required to query the values of a registry key
+#define KEY_SET_VALUE           0x0002 //Required to create, delete, or set a registry value
+#define KEY_CREATE_SUB_KEY      0x0004 //Required to create, delete, or rename a subkey of a registry key
+#define KEY_ENUMERATE_SUB_KEYS  0x0008 //Required to enumerate the subkeys of a registry key
+#define KEY_NOTIFY              0x0010 //Required to request change notifications for a registry key or for subkeys of a registry key.
+#define KEY_CREATE_LINK         0x0020 //Reserved for system use
+
+
+#define KEY_ALL_ACCESS ( \
+        (~SYNCHRONIZE) & \
+        (STANDARD_RIGHTS_REQUIRED | \
+        KEY_QUERY_VALUE |\
+        KEY_SET_VALUE |\
+        KEY_CREATE_SUB_KEY |\
+        KEY_ENUMERATE_SUB_KEYS |\
+        KEY_NOTIFY |\
+        KEY_CREATE_LINK) \
+        )
+
+#define KEY_READ ( \
+        (~SYNCHRONIZE) & \
+        (STANDARD_RIGHTS_READ | \
+        KEY_QUERY_VALUE |\
+        KEY_ENUMERATE_SUB_KEYS |\
+        KEY_NOTIFY) \
+        )
+
+
+#define KEY_WRITE ( \
+        (~SYNCHRONIZE) & \
+        (STANDARD_RIGHTS_WRITE | \
+        KEY_SET_VALUE |\
+        KEY_CREATE_SUB_KEY) \
+        )
+
+#define KEY_EXECUTE ( \
+        (~SYNCHRONIZE) & \
+        (KEY_READ)\
+        )
+
 //
 // GENERIC_MAPPING - Used to map the GENERIC_{ALL,EXECUTE,WRITE,READ}
 // bits to specific rights.
@@ -445,6 +543,13 @@ typedef ULONG SID_NAME_USE, *PSID_NAME_USE;
 #define SID_TYPE_LABEL              10
 #endif
 
+
+//
+// SDDL Version information
+//
+#define SDDL_REVISION_1     1
+#define SDDL_REVISION       SDDL_REVISION_1
+
 //
 // SID Attributes
 //
@@ -528,6 +633,7 @@ typedef struct _ACE_HEADER {
 #define SYSTEM_ALARM_CALLBACK_OBJECT_ACE_TYPE   16 // SYSTEM_ALARM_CALLBACK_OBJECT_ACE
 #define SYSTEM_MANDATORY_LABEL_ACE_TYPE         17 // SYSTEM_MANDATORY_LABEL_ACE
 
+
 // Min/Max
 #define ACCESS_MIN_MS_ACE_TYPE          ACCESS_ALLOWED_ACE_TYPE
 #define ACCESS_MAX_MS_V2_ACE_TYPE       SYSTEM_ALARM_ACE_TYPE               // Max Windows NT
@@ -983,6 +1089,46 @@ typedef ULONG SECURITY_INFORMATION, *PSECURITY_INFORMATION;
 #define SEF_MACL_NO_READ_UP                     0X00000200  // unused
 #define SEF_AVOID_OWNER_RESTRICTION             0x00001000
 
+//
+// Privileges
+//
+
+typedef struct _LUID_AND_ATTRIBUTES {
+    LUID Luid;
+    ULONG Attributes;
+} LUID_AND_ATTRIBUTES, *PLUID_AND_ATTRIBUTES;
+
+#define SE_PRIVILEGE_ENABLED_BY_DEFAULT   0x00000001
+#define SE_PRIVILEGE_ENABLED              0x00000002
+#define SE_PRIVILEGE_REMOVED              0x00000004
+
+#define _PRIVILEGE_SET_MAX_PRIVILEGE_COUNT 1000
+
+#define PRIVILEGE_SET_ALL_NECESSARY       0x00000001
+
+typedef struct _PRIVILEGE_SET {
+#ifdef _DCE_IDL_
+    [range(0,_PRIVILEGE_SET_MAX_PRIVILEGE_COUNT)]
+#endif
+    ULONG PrivilegeCount;
+    ULONG Control;
+#ifdef _DCE_IDL_
+    [size_is(PrivilegeCount)]
+#endif
+    LUID_AND_ATTRIBUTES Privilege[];
+} PRIVILEGE_SET, *PPRIVILEGE_SET;
+
+#define PRIVILEGE_SET_MIN_SIZE \
+    (LW_FIELD_OFFSET(PRIVILEGE_SET, Privilege))
+
+#define _PRIVILEGE_SET_GET_SIZE_REQUIRED(PrivilegeCount) \
+    (PRIVILEGE_SET_MIN_SIZE + \
+     (LW_FIELD_SIZE(PRIVILEGE_SET, Privilege[0]) * (PrivilegeCount)))
+
+#define _PRIVILEGE_SET_MAX_SIZE \
+    _SID_GET_SIZE_REQUIRED(_PRIVILEGE_SET_MAX_PRIVILEGE_COUNT)
+
+
 //
 // Access Token
 //
