@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -41,6 +41,7 @@
 
 #include <lw/types.h>
 #include <lw/attrs.h>
+#include <signal.h>
 
 LW_BEGIN_EXTERN_C
 
@@ -112,7 +113,7 @@ typedef enum _LW_TASK_EVENT_MASK
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_COMPLETE     = 0x00,
+    LW_TASK_EVENT_COMPLETE     = 0x000,
     /**
      * A special bit which indicates that a task has been
      * run for the first time when passed as WakeMask
@@ -120,8 +121,8 @@ typedef enum _LW_TASK_EVENT_MASK
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_INIT         = 0x01,
-
+    LW_TASK_EVENT_INIT         = 0x001,
+    
     /**
      * Indicates that the task was explicitly woken by
      * an external caller, such as through #LwRtlWakeTask()
@@ -129,7 +130,7 @@ typedef enum _LW_TASK_EVENT_MASK
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_EXPLICIT     = 0x02,
+    LW_TASK_EVENT_EXPLICIT     = 0x002,
 
     /**
      * Indicates that the task has been cancelled.
@@ -139,28 +140,28 @@ typedef enum _LW_TASK_EVENT_MASK
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_CANCEL       = 0x04,
-
+    LW_TASK_EVENT_CANCEL       = 0x004,
+    
     /**
      * Indicates that the last set timeout has expired.
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_TIME         = 0x08,
+    LW_TASK_EVENT_TIME         = 0x008,
 
     /**
      * Indicates that a file descriptor has become readable.
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_FD_READABLE  = 0x10,
+    LW_TASK_EVENT_FD_READABLE  = 0x010,
 
     /**
      * Indicates that a file descriptor has become writable.
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_FD_WRITABLE  = 0x20,
+    LW_TASK_EVENT_FD_WRITABLE  = 0x020,
 
     /**
      * Indicates that an exception event occurred on a file
@@ -168,31 +169,53 @@ typedef enum _LW_TASK_EVENT_MASK
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_FD_EXCEPTION = 0x40,
-
+    LW_TASK_EVENT_FD_EXCEPTION = 0x040,
+    
     /**
      * Indicates that the task should be immediately run
      * again after other tasks have had a chance to run.
      *
      * @hideinitializer
      */
-    LW_TASK_EVENT_YIELD        = 0x80
+    LW_TASK_EVENT_YIELD        = 0x080,
+
+    /**
+     * Indicates that one or more UNIX signals which the task
+     * subscribes to have arrived.
+     *
+     * @hideinitializer
+     */
+    LW_TASK_EVENT_UNIX_SIGNAL  = 0x100
 } LW_TASK_EVENT_MASK, *PLW_TASK_EVENT_MASK;
 
 /**
- * @brief Work item flags
+ * @brief Scheduling flags
  *
- * Controls how work items are run.
+ * Controls how work items are scheduled.
  */
-typedef enum LW_WORK_ITEM_FLAGS
+typedef enum LW_SCHEDULE_FLAGS
 {
     /**
-     * Indicates that a work item should be
-     * given its own dedicated thread in which to
-     * run.
+     * Indicates the the work item should be placed at the front
+     * of the work queue.
+     *
+     * @hideinitializer
      */
-    LW_WORK_ITEM_DEDICATED         = 0x00
-} LW_WORK_ITEM_FLAGS;
+    LW_SCHEDULE_HIGH_PRIORITY         = 0x00010000
+} LW_SCHEDULE_FLAGS;
+
+#ifndef DOXYGEN
+#define LW_WORK_ITEM_FLAGS LW_SCHEDULE_FLAGS
+#define LW_WORK_ITEM_HIGH_PRIOTIRY LW_SCHEDULE_HIGH_PRIORITY
+#endif
+
+/**
+ * @brief Work item structure
+ *
+ * Opaque structure representing a work item
+ */
+typedef struct _LW_WORK_ITEM LW_WORK_ITEM, *PLW_WORK_ITEM;
+
 
 /**
  * @brief Thread pool option
@@ -216,8 +239,8 @@ typedef enum LW_THREAD_POOL_OPTION
      */
     LW_THREAD_POOL_OPTION_TASK_THREADS,
     /**
-     * (LONG) Number of work threads to create.  A negative number indicates
-     * a multiple of the number of CPUs present on the system.
+     * (LONG) Maximum number of work threads to create.  A negative number
+     * indicates a multiple of the number of CPUs present on the system.
      * (Default: -4)
      */
     LW_THREAD_POOL_OPTION_WORK_THREADS,
@@ -232,7 +255,13 @@ typedef enum LW_THREAD_POOL_OPTION
      * the system default.
      * (Default: 0)
      */
-    LW_THREAD_POOL_OPTION_WORK_THREAD_STACK_SIZE
+    LW_THREAD_POOL_OPTION_WORK_THREAD_STACK_SIZE,
+    /**
+     * (ULONG) Time in seconds before an idle work thread exits.  0 indicates
+     * no timeout.
+     * (Default: 30)
+     */
+    LW_THREAD_POOL_OPTION_WORK_THREAD_TIMEOUT
 } LW_THREAD_POOL_OPTION;
 
 /**
@@ -243,7 +272,7 @@ typedef enum LW_THREAD_POOL_OPTION
  * the pWaitMask parameter to the mask of conditions under
  * which it should next wake up, and the pllTime parameter to the
  * next timeout.
- *
+ * 
  * If a non-zero time is set and the task is woken before
  * the time expires, on the next call to the function the pllTime
  * parameter will contain how much time was remaining.  Note that
@@ -268,11 +297,10 @@ typedef enum LW_THREAD_POOL_OPTION
  *
  * - #LW_TASK_EVENT_YIELD<br>
  *   If this flag is set in the wait mask, it indicates that the task is
- *   not yet finished doing work but is yielding to allow other tasks
- *   to run.  When the task is run again, the wake mask will contain
- *   #LW_TASK_EVENT_YIELD and all other bits that were set in the wait
- *   mask, modulo the special flags above.  Additional events may occur
- *   during a yield and will be added to the wake mask.
+ *   not yet finished doing work and should be run again immediately
+ *   after giving other tasks a chance to run.  When the task is run again,
+ *   the wake mask will contain #LW_TASK_EVENT_YIELD in addition to any
+ *   other events that occurred in the interim.
  *
  * - #LW_TASK_EVENT_COMPLETE<br>
  *   If the wait mask is set to this value, the task will be considered
@@ -300,17 +328,28 @@ LW_VOID
 /**
  * @brief Work item function
  *
- * A function which is queued and dispatched by
- * #LwRtlQueueWorkItem().  Unlike tasks, work item
- * functions are synchronous and may block.
+ * A function which is called to run a work item
+ * created with #LwRtlCreateWorkItem().  A work
+ * item function -- unlike a task function -- may
+ * safely block.
  *
+ * @param[in] pWorkItem the work item
  * @param[in] pContext the user context pointer
  */
 typedef
 LW_VOID
 (*LW_WORK_ITEM_FUNCTION)(
+    PLW_WORK_ITEM pWorkItem,
+    PVOID pContext
+    );
+
+#ifndef DOXYGEN
+typedef
+LW_VOID
+(*LW_WORK_ITEM_FUNCTION_COMPAT)(
     LW_PVOID pContext
     );
+#endif
 
 /**
  * @brief Create a new task
@@ -326,7 +365,7 @@ LW_VOID
  * the #LW_TASK_FUNCTION itself), it should be released with
  * #LwRtlReleaseTask().  The task will not be completely freed
  * until it has been released and the #LW_TASK_FUNCTION has
- * indicated completion by setting an empty wake mask.
+ * indicated completion by setting a wake mask of #LW_TASK_EVENT_COMPLETE.
  *
  * @param[in] pPool the thread pool
  * @param[out] ppTask the created task
@@ -334,7 +373,8 @@ LW_VOID
  * @param[in] pfnFunc the task wakeup function
  * @param[in] pData the user context pointer passed to pfnFunc
  * @retval #LW_STATUS_SUCCESS success
- * @retval #LW_INSUFFICIENT_RESOURCES out of memory
+ * @retval #LW_STATUS_INSUFFICIENT_RESOURCES out of memory
+ * @retval #LW_STATUS_CANCELLED the specified task group was cancelled
  */
 LW_NTSTATUS
 LwRtlCreateTask(
@@ -355,7 +395,7 @@ LwRtlCreateTask(
  * @param[in] pPool the thread pool
  * @param[out] ppGroup the created task
  * @retval #LW_STATUS_SUCCESS success
- * @retval #LW_INSUFFICIENT_RESOURCES out of memory
+ * @retval #LW_STATUS_INSUFFICIENT_RESOURCES out of memory
  */
 LW_NTSTATUS
 LwRtlCreateTaskGroup(
@@ -449,6 +489,56 @@ LwRtlQueryTaskFd(
     LW_OUT PLW_TASK_EVENT_MASK pMask
     );
 
+/**
+ * @brief Configure UNIX signal wakeup events
+ *
+ * Configures whether the given task will be notified
+ * when the specified UNIX signal is delivered to the process.
+ *
+ * Note that if a task is configured to receive any UNIX signals,
+ * it must wait for the #LW_TASK_EVENT_UNIX_SIGNAL event and reap
+ * any delivered signals with #LwRtlNextTaskUnixSignal().  Failure
+ * to do so can cause the main signal dispatch loop to hang.
+ *
+ * @warning The result of calling this function from outside of an
+ * #LW_TASK_FUNCTION is undefined.
+ *
+ * @param[in,out] pTask the task
+ * @param[in] Sig the signal number
+ * @param[in] bSubscribe whether the task should be notified
+ * @retval #LW_STATUS_SUCCESS success
+ * @retval #LW_STATUS_INVALID_PARAMETER the provided signal was invalid
+ * @retval #LW_STATUS_INSUFFICIENT_RESOURCES out of memory
+ */
+LW_NTSTATUS
+LwRtlSetTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_IN int Sig,
+    LW_IN LW_BOOLEAN bSubscribe
+    );
+
+/**
+ * @brief Get next UNIX signal delivered to task
+ *
+ * Gets the next UNIX signal which was delivered to the specified task.
+ * Because #LW_TASK_EVENT_UNIX_SIGNAL events are edge-triggered, you should
+ * continue calling this function as long as it returns TRUE in order to
+ * ensure that no signals are missed.
+ *
+ * @warning The result of calling this function from outside of an
+ * #LW_TASK_FUNCTION is undefined.
+ *
+ * @param[in,out] pTask the task
+ * @param[out] pInfo information about the UNIX signal
+ * @retval TRUE a signal was pending, and information about it was placed in pInfo
+ * @retval FALSE no signal was pending
+ */
+LW_BOOLEAN
+LwRtlNextTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_OUT siginfo_t* pInfo
+    );
+
 /**
  * @brief Manually wake task
  *
@@ -479,8 +569,8 @@ LwRtlCancelTask(
 /**
  * @brief Wait for task completion
  *
- * Waits for the specified task to complete by indicating an empty
- * wakeup trigger mask.
+ * Blocks until the specified task completes by setting
+ * #LW_TASK_EVENT_COMPLETE as its wait mask.
  *
  * @param[in] pTask the task
  */
@@ -505,8 +595,9 @@ LwRtlWakeTaskGroup(
 /**
  * @brief Cancel task group
  *
- * Equivalent to calling #LwRtlCancelTask() on all tasks in the
- * given task group.
+ * All tasks in the task group are cancelled as if by #LwRtlCancelTask().
+ * In addition, any subsequent attempts to create a task within the group
+ * will fail with #LW_STATUS_CANCELLED.
  *
  * @param[in] pGroup the task group
  */
@@ -529,25 +620,82 @@ LwRtlWaitTaskGroup(
     );
 
 /**
- * @brief Queue synchronous work item
+ * @brief Create work item
  *
- * Schedules a work item to be run in another thread.  Unlike tasks,
- * work items are not asynchronous and may block arbitrarily.  This
- * functionality is provided as a convenience for tasks which may need
- * to make calls into blocking code.
+ * Creates a work item which can be scheduled to run
+ * with #LwRtlScheduleWorkItem().  Work items -- unlike tasks --
+ * may block.
  *
- * @param[in] pPool the thread pool
- * @param[in] pfnFunc a work item function to run
- * @param[in] pContext the user context pointer to pass to the work item function
- * @param[in] Flags control flags for the work item
+ * @param[in] pPool thread pool which will own the work item
+ * @param[out] ppWorkItem set to the created work item
+ * @param[in] pfnFunc work item function to run
+ * @param[in] pContext user context to pass to the callback
+ * @retval #LW_STATUS_SUCCESS success
+ * @retval #LW_STATUS_INSUFFICIENT_RESOURCES out of memory
  */
 LW_NTSTATUS
+LwRtlCreateWorkItem(
+    LW_IN PLW_THREAD_POOL pPool,
+    LW_OUT PLW_WORK_ITEM* ppWorkItem,
+    LW_IN LW_WORK_ITEM_FUNCTION pfnFunc,
+    LW_IN PVOID pContext
+    );
+
+/**
+ * @brief Free work item
+ *
+ * Frees a work item.  Once a work item has been scheduled,
+ * this function may only be called safely from the work item
+ * function.
+ *
+ * @param[in,out] ppWorkItem if *ppWorkItem is not NULL, it is freed and set to NULL
+ */
+LW_VOID
+LwRtlFreeWorkItem(
+    LW_IN LW_OUT PLW_WORK_ITEM* ppWorkItem
+    );
+
+/**
+ * @brief Schedule blocking work item
+ *
+ * Schedules a work item to be run with the specified scheduling flags.
+ *
+ * @param[in] pWorkItem the work item
+ * @param[in] Flags scheduling control flags
+ */
+LW_VOID
+LwRtlScheduleWorkItem(
+    LW_IN PLW_WORK_ITEM pWorkItem,
+    LW_IN LW_SCHEDULE_FLAGS Flags
+    );
+
+/**
+ * @brief Wait for scheduled work items to complete
+ *
+ * Blocks until all scheduled work items have finished running.
+ *
+ * @warning It is up to the user to ensure that no further work
+ * items will be scheduled.  Otherwise, this function may return
+ * even if work items are still scheduled, or block forever.
+ *
+ * @warning Calling this function from a work item function
+ * will cause a deadlock
+ */
+LW_VOID
+LwRtlWaitWorkItems(
+    LW_IN PLW_THREAD_POOL pPool
+    );
+
+#ifndef DOXYGEN
+/* Deprecated compat function */
+LW_NTSTATUS
 LwRtlQueueWorkItem(
     LW_IN PLW_THREAD_POOL pPool,
-    LW_IN LW_WORK_ITEM_FUNCTION pfnFunc,
+    LW_IN LW_WORK_ITEM_FUNCTION_COMPAT pfnFunc,
     LW_IN LW_PVOID pContext,
     LW_IN LW_WORK_ITEM_FLAGS Flags
     );
+#endif
 
 /**
  * @brief Create thread pool attributes structure
@@ -627,6 +775,49 @@ LwRtlFreeThreadPool(
     LW_IN LW_OUT PLW_THREAD_POOL* ppPool
     );
 
+/**
+ * @brief Main signal loop
+ *
+ * Runs the threadpool main signal loop.  You must call this function from
+ * your program's main thread if you wish to use UNIX signal events with
+ * your threadpool tasks.
+ *
+ * This function will not return until #LwRtlExitMain() is called.
+ *
+ * @return the status passed to #LwRtlExitMain()
+ */
+LW_NTSTATUS
+LwRtlMain(
+    LW_VOID
+    );
+
+/**
+ * @brief Exit main loop
+ *
+ * Causes #LwRtlMain() to return with the specified status code
+ *
+ * @param[in] Status the status that #LwRtlMain() should return
+ */
+LW_VOID
+LwRtlExitMain(
+    LW_NTSTATUS Status
+    );
+
+/**
+ * @brief Block all signals in the current thread
+ *
+ * Blocks all signals in the current thread.  It is recommended that
+ * your application call this function from the main thread before creating
+ * any other threads.  This guarantees that all signals will be properly
+ * delivered to #LwRtlMain().
+ *
+ * @retval LW_STATUS_SUCCESS success
+ */
+LW_NTSTATUS
+LwRtlBlockSignals(
+    VOID
+    );
+
 /*@}*/
 
 LW_END_EXTERN_C
