@@ -34,6 +34,7 @@
  *
  * Authors: Brian Koropoff (bkoropoff@likewise.com)
  *          Danilo Almeida (dalmeida@likewise.com)
+ *          Rafal Szczesniak (rafal@likewise.com)
  *
  */
 
@@ -167,6 +168,16 @@ typedef struct _LW_GUID {
 #endif
 } LW_GUID, *LW_PGUID;
 
+
+//
+// Locally unique identifier (LUID).
+//
+
+typedef struct _LW_LUID {
+    LW_ULONG LowPart;
+    LW_LONG HighPart;
+} LW_LUID, *LW_PLUID;
+
 #define LW_TRUE  1
 #define LW_FALSE 0
 
@@ -227,8 +238,7 @@ typedef struct _LW_GUID {
 #define LW_PTR_OFFSET(BasePointer, Pointer) LwRtlPointerToOffset(BasePointer, Pointer)
 #define LW_PTR_ADD(Pointer, Offset) LwRtlOffsetToPointer(Pointer, Offset)
 
-#define LW_FIELD_OFFSET(Type, Field) \
-    ((size_t)(&(((Type*)(0))->Field)))
+#define LW_FIELD_OFFSET(Type, Field) offsetof(Type, Field)
 
 #define LW_FIELD_SIZE(Type, Field) \
     (sizeof(((Type*)(0))->Field))
@@ -326,6 +336,8 @@ typedef LW_TCHAR    TCHAR;
 
 typedef LW_GUID     GUID;
 typedef LW_PGUID    PGUID;
+typedef LW_LUID     LUID;
+typedef LW_PLUID    PLUID;
 
 #ifndef TRUE
 #define TRUE LW_TRUE
@@ -436,6 +448,29 @@ typedef struct _LW_ANSI_STRING {
 #define LwRtlDecimalDigitValue(Character) \
     ((Character) - '0')
 
+#define LW_RTL_MAKE_CUSTOM_FREE(FreeFunction, PointerToPointer) \
+    do { \
+        if (*(PointerToPointer)) \
+        { \
+            (FreeFunction)(*(PointerToPointer)); \
+            *(PointerToPointer) = NULL; \
+        } \
+    } while (0)
+
+#define LW_RTL_MAKE_CUSTOM_FREE_SECURE_STRING(FreeFunction, PointerToPointer, CharType) \
+    do { \
+        if (*(PointerToPointer)) \
+        { \
+            CharType* pOverwrite; \
+            for (pOverwrite = *(PointerToPointer); *pOverwrite; pOverwrite++) \
+            { \
+                *pOverwrite = 0; \
+            } \
+            (FreeFunction)(*(PointerToPointer)); \
+            *(PointerToPointer) = NULL; \
+        } \
+    } while (0)
+
 #ifndef LW_STRICT_NAMESPACE
 
 typedef LW_UNICODE_STRING UNICODE_STRING;
@@ -450,7 +485,7 @@ typedef LW_PANSI_STRING PANSI_STRING;
 #define ANSI_STRING_MAX_CHARS LW_ANSI_STRING_MAX_CHARS
 #define ANSI_STRING_MAX_BYTES LW_ANSI_STRING_MAX_BYTES
 
-#define RTL_STRING_IS_NULL_OR_EMPTY(String) LW_RTL_STRING_IS_NULL_OR_EMPTY(String)
+#define RTL_STRING_IS_NULL_OR_EMPTY(String) LW_RTL_STRING_IS_NULL_OR_EMPTY(String) 
 #define RtlCStringIsNullOrEmpty(String) LwRtlCStringIsNullOrEmpty(String)
 
 #define RTL_CONSTANT_STRING(StringLiteral) LW_RTL_CONSTANT_STRING(StringLiteral)
