@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -199,6 +199,8 @@ DumpSecurityDescriptor(
                 break;
         }
     }
+
+    RTL_FREE(&sidString);
 }
 
 static
@@ -249,6 +251,9 @@ DumpTokenInfo(
             MU_ASSERT_STATUS_SUCCESS(status);
 
             MU_INFO("User = %s", sidString);
+
+            RTL_FREE(&sidString);
+
             break;
         }
         case TokenGroups:
@@ -261,14 +266,21 @@ DumpTokenInfo(
 
             for (i = 0; i < tokenGroups->GroupCount; i++)
             {
+                RTL_FREE(&sidString);
+
                 status = RtlAllocateCStringFromSid(&sidString, tokenGroups->Groups[i].Sid);
                 MU_ASSERT_STATUS_SUCCESS(status);
 
                 MU_INFO("Groups[%u] = (0x%08x, %s)", i, tokenGroups->Groups[i].Attributes, sidString);
             }
+
+            RTL_FREE(&sidString);
+
             break;
         }
     }
+
+    RTL_FREE(&tokenInfo);
 }
 
 static
@@ -380,11 +392,14 @@ MU_TEST(Security, 0003_AccessCheck)
 
     DumpToken(token);
 
-    status = RtlValidRelativeSecurityDescriptor(
+    if (!RtlValidRelativeSecurityDescriptor(
                     relativeSd,
                     relativeSdLength,
-                    0);
-    MU_ASSERT_STATUS_SUCCESS(status);
+                    0))
+    {
+        status = STATUS_INVALID_ACL;
+        MU_ASSERT_STATUS_SUCCESS(status);
+    }
 
     status = RtlSelfRelativeToAbsoluteSD(
                     relativeSd,
@@ -408,17 +423,29 @@ MU_TEST(Security, 0003_AccessCheck)
     status = RTL_ALLOCATE(&sd, VOID, sdSize);
     MU_ASSERT_STATUS_SUCCESS(status);
 
-    status = RTL_ALLOCATE(&dacl, VOID, daclSize);
-    MU_ASSERT_STATUS_SUCCESS(status);
+    if (daclSize)
+    {
+        status = RTL_ALLOCATE(&dacl, VOID, daclSize);
+        MU_ASSERT_STATUS_SUCCESS(status);
+    }
 
-    status = RTL_ALLOCATE(&sacl, VOID, saclSize);
-    MU_ASSERT_STATUS_SUCCESS(status);
+    if (saclSize)
+    {
+        status = RTL_ALLOCATE(&sacl, VOID, saclSize);
+        MU_ASSERT_STATUS_SUCCESS(status);
+    }
 
-    status = RTL_ALLOCATE(&owner, VOID, ownerSize);
-    MU_ASSERT_STATUS_SUCCESS(status);
+    if (ownerSize)
+    {
+        status = RTL_ALLOCATE(&owner, VOID, ownerSize);
+        MU_ASSERT_STATUS_SUCCESS(status);
+    }
 
-    status = RTL_ALLOCATE(&primaryGroup, VOID, primaryGroupSize);
-    MU_ASSERT_STATUS_SUCCESS(status);
+    if (primaryGroupSize)
+    {
+        status = RTL_ALLOCATE(&primaryGroup, VOID, primaryGroupSize);
+        MU_ASSERT_STATUS_SUCCESS(status);
+    }
 
     status = RtlSelfRelativeToAbsoluteSD(
                     relativeSd,
