@@ -8,7 +8,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -58,7 +58,7 @@ TimeNow(
     }
     else
     {
-        *pllNow =
+        *pllNow = 
             tv.tv_sec * 1000000000ll +
             tv.tv_usec * 1000ll;
 
@@ -72,7 +72,8 @@ pthread_cond_t gEvent = PTHREAD_COND_INITIALIZER;
 
 MU_FIXTURE_SETUP(Task)
 {
-    MU_ASSERT_STATUS_SUCCESS(LwRtlCreateThreadPool(&gpPool, NULL));
+    MU_ASSERT_STATUS_SUCCESS(LwRtlBlockSignals());
+    MU_ASSERT_STATUS_SUCCESS(LwRtlCreateThreadPool(&gpPool, NULL));   
 }
 
 MU_FIXTURE_TEARDOWN(Task)
@@ -83,6 +84,44 @@ MU_FIXTURE_TEARDOWN(Task)
 static
 VOID
 BasicWorkItem(
+    PLW_WORK_ITEM pItem,
+    PVOID pContext
+    )
+{
+    BOOLEAN volatile *pbValue = pContext;
+
+    pthread_mutex_lock(&gLock);
+    *pbValue = TRUE;
+    pthread_cond_signal(&gEvent);
+    pthread_mutex_unlock(&gLock);
+
+    LwRtlFreeWorkItem(&pItem);
+}
+
+MU_TEST(Task, BasicWorkItem)
+{
+    BOOLEAN volatile bValue = FALSE;
+    PLW_WORK_ITEM pItem = NULL;
+
+    MU_ASSERT_STATUS_SUCCESS(LwRtlCreateWorkItem(
+        gpPool,
+        &pItem,
+        BasicWorkItem,
+        (PVOID) &bValue));
+
+    LwRtlScheduleWorkItem(pItem, 0);
+
+    pthread_mutex_lock(&gLock);
+    while (!bValue)
+    {
+        pthread_cond_wait(&gEvent, &gLock);
+    }
+    pthread_mutex_unlock(&gLock);
+}
+
+static
+VOID
+CompatWorkItem(
     PVOID pContext
     )
 {
@@ -94,13 +133,13 @@ BasicWorkItem(
     pthread_mutex_unlock(&gLock);
 }
 
-MU_TEST(Task, BasicWorkItem)
+MU_TEST(Task, CompatWorkItem)
 {
     BOOLEAN volatile bValue = FALSE;
 
     MU_ASSERT_STATUS_SUCCESS(LwRtlQueueWorkItem(
                                  gpPool,
-                                 BasicWorkItem,
+                                 CompatWorkItem,
                                  (PVOID) &bValue,
                                  0));
 
@@ -147,7 +186,7 @@ MU_TEST(Task, Timer)
                                  gpPool,
                                  &pTask,
                                  NULL,
-                                 Timer,
+                                 Timer, 
                                  (PVOID) &value));
 
     LwRtlWakeTask(pTask);
@@ -221,6 +260,72 @@ MU_TEST(Task, Yield)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, value2, gTarget);
 }
 
+static
+VOID
+OnOff(
+    PLW_TASK pTask,
+    PVOID pContext,
+    LW_TASK_EVENT_MASK WakeMask,
+    PLW_TASK_EVENT_MASK pWaitMask,
+    PLONG64 pllTime
+    )
+{
+    int* fds = pContext;
+    char c = 0;
+
+    if (WakeMask & LW_TASK_EVENT_INIT)
+    {
+        /* Wait for readability of pipe.
+         * Write a byte so we wake up immediately.
+         */
+        MU_ASSERT_STATUS_SUCCESS(LwRtlSetTaskFd(pTask, fds[0], LW_TASK_EVENT_FD_READABLE));
+        MU_ASSERT(write(fds[1], &c, sizeof(c)) == sizeof(c));
+        *pWaitMask = LW_TASK_EVENT_FD_READABLE;
+    }
+    else if (WakeMask & LW_TASK_EVENT_FD_READABLE)
+    {
+        /* Read the byte we wrote */
+        MU_ASSERT(read(fds[0], &c, sizeof(c)) == sizeof(c));
+        /* Write another byte so the read end is readable again */
+        MU_ASSERT(write(fds[1], &c, sizeof(c)) == sizeof(c));
+        /* Wait for a 1-nanosecond timeout instead.
+         * Internally, the threadpoll will need to disable any readability
+         * event it configured.
+         */
+        *pllTime = 1;
+        *pWaitMask = LW_TASK_EVENT_TIME;
+    }
+    else if (WakeMask & LW_TASK_EVENT_TIME)
+    {
+        /* Ensure that we did not receive a readability event */
+        MU_ASSERT(!(WakeMask & LW_TASK_EVENT_FD_READABLE));
+        /* Test complete */
+        *pWaitMask = LW_TASK_EVENT_COMPLETE;
+    }
+}
+
+MU_TEST(Task, EventOnEventOff)
+{
+    PLW_TASK pTask = NULL;
+    int fds[2] = {-1, -1};
+
+    MU_ASSERT(pipe(fds) == 0);
+
+    MU_ASSERT_STATUS_SUCCESS(LwRtlCreateTask(
+                                 gpPool,
+                                 &pTask,
+                                 NULL,
+                                 OnOff,
+                                 (PVOID) &fds));
+
+    LwRtlWakeTask(pTask);
+    LwRtlWaitTask(pTask);
+    LwRtlReleaseTask(&pTask);
+
+    MU_ASSERT(close(fds[0]) == 0);
+    MU_ASSERT(close(fds[1]) == 0);
+}
+
 #define BUFFER_SIZE (64 * 1024)
 #define SEND_SEGMENTS 1
 #define NUM_ITERATIONS 32
@@ -249,3 +354,57 @@ MU_TEST(Task, Transceive)
             ullTime / 1000000000.0,
             (ullTotal / 131072.0) / (ullTime / 1000000000.0));
 }
+
+static
+VOID
+WaitSigTerm(
+    PLW_TASK pTask,
+    PVOID pContext,
+    LW_TASK_EVENT_MASK WakeMask,
+    PLW_TASK_EVENT_MASK pWaitMask,
+    PLONG64 pllTime
+    )
+{
+    siginfo_t info;
+
+    if (WakeMask & LW_TASK_EVENT_INIT)
+    {
+        MU_ASSERT_STATUS_SUCCESS(LwRtlSetTaskUnixSignal(pTask, SIGTERM, TRUE));
+        *pWaitMask = LW_TASK_EVENT_UNIX_SIGNAL;
+        kill(getpid(), SIGTERM);
+    }
+    else if (WakeMask & LW_TASK_EVENT_UNIX_SIGNAL)
+    {
+        while (LwRtlNextTaskUnixSignal(pTask, &info))
+        {
+            switch(info.si_signo)
+            {
+            case SIGTERM:
+                LwRtlExitMain(STATUS_SUCCESS);
+                *pWaitMask = LW_TASK_EVENT_COMPLETE;
+                return;
+                break;
+            default:
+                break;
+            }
+        }
+
+        *pWaitMask = LW_TASK_EVENT_UNIX_SIGNAL;
+    }
+}
+
+MU_TEST(Task, ExitOnSignal)
+{
+    PLW_TASK pTask = NULL;
+
+    MU_ASSERT_STATUS_SUCCESS(LwRtlCreateTask(
+                                 gpPool,
+                                 &pTask,
+                                 NULL,
+                                 WaitSigTerm, 
+                                 NULL));
+
+    LwRtlWakeTask(pTask);
+
+    MU_ASSERT_STATUS_SUCCESS(LwRtlMain());
+}
