@@ -31,6 +31,11 @@
 #ifdef HAVE_PTHREAD_H
 #include <pthread.h>
 #endif
+
+#ifdef HAVE_PTHREAD_NP_H
+#include <pthread_np.h>
+#endif
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,6 +48,7 @@
 #include <fcntl.h>
 
 #include <string.h>
+#include <ctype.h>
 #include <assert.h>
 
 #ifdef HAVE_STRINGS_H
@@ -53,6 +59,20 @@
 #include <sys/varargs.h>
 #endif
 
+#ifdef HAVE_STDARG_H
+#include <stdarg.h>
+#endif
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
+#ifdef HAVE_SYS_CPUSET_H
+#include <sys/cpuset.h>
+#endif
+
+#include <sys/resource.h>
+
 #include <wc16str.h>
 #ifdef HAVE_UUID_UUID_H
 #include <uuid/uuid.h>
