@@ -41,12 +41,12 @@
  * Authors: Danilo Almeida (dalmeida@likewise.com)
  */
 
+#include "config.h"
 #include <lw/base.h>
 #include <lw/mapsecurity-plugin.h>
 #include <lw/security-api.h>
 #include <lw/rtlgoto.h>
 #include <lw/safeint.h>
-#include "config.h"
 #include <pthread.h>
 #include <stdlib.h>
 #ifdef HAVE_DLFCN_H
@@ -278,7 +278,6 @@ LwMapSecurityCreateContext(
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    int EE = 0;
     PLW_MAP_SECURITY_CONTEXT pContext = NULL;
     BOOLEAN bInLock = FALSE;
 
@@ -289,7 +288,7 @@ LwMapSecurityCreateContext(
         status = LwMapSecurityCreateContextInternal(
                                   &gLwMapSecurityState.Context,
                                   gLwMapSecurityState.pCreateContextCallback);
-        GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+        GOTO_CLEANUP_ON_STATUS(status);
     }
 
     pContext = gLwMapSecurityState.Context;
@@ -313,7 +312,7 @@ LwMapSecurityFreeContextInLock(
     PLW_MAP_SECURITY_CONTEXT context = *Context;
 
     // We must be coming into this block with the MapSecurityState
-    // mutex locked
+    // mutex locked 
     if (context)
     {
         if (context != gLwMapSecurityState.Context)
@@ -330,7 +329,7 @@ LwMapSecurityFreeContextInLock(
             // Logic error.  Bail out.  May leak memory.
             goto cleanup;
         }
-
+            
         if ((0 == gLwMapSecurityState.RefCount) &&
             (0 == gLwMapSecurityState.InitCount))
         {
@@ -380,34 +379,24 @@ LwMapSecurityCreateContextInternal(
     )
 {
     NTSTATUS status = 0;
-    int EE = 0;
-    PCSTR pszError = NULL;
     PLW_MAP_SECURITY_CONTEXT pContext = NULL;
     LWMSP_CREATE_CONTEXT_CALLBACK pCreateContextCallback = NULL;
 
     status = RTL_ALLOCATE(&pContext, LW_MAP_SECURITY_CONTEXT, sizeof(*pContext));
-    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     if (!pCallback)
     {
         status = RtlCStringDuplicate(&pContext->LibraryPath, LW_MAP_SECURITY_PLUGIN_PATH);
-        GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+        GOTO_CLEANUP_ON_STATUS(status);
 
         dlerror();
 
         pContext->LibraryHandle = dlopen(pContext->LibraryPath, RTLD_NOW | RTLD_GLOBAL);
         if (!pContext->LibraryHandle)
         {
-#if ENABLE_LOGGING
-            int error = errno;
-#endif
-            pszError = dlerror();
-
-            LOG_ERROR("Failed to load %s (%s (%d))", pContext->LibraryPath,
-                      SAFE_LOG_STRING(pszError), error);
-
             status = STATUS_UNSUCCESSFUL;
-            GOTO_CLEANUP_EE(EE);
+            GOTO_CLEANUP();
         }
 
         dlerror();
@@ -416,14 +405,7 @@ LwMapSecurityCreateContextInternal(
                                             LWMSP_CREATE_CONTEXT_FUNCTION_NAME);
         if (!pCreateContextCallback)
         {
-            pszError = dlerror();
-
-            LOG_ERROR("Failed to load " LWMSP_CREATE_CONTEXT_FUNCTION_NAME
-                      " function from %s (%s)",
-                      pContext->LibraryPath, SAFE_LOG_STRING(pszError));
-
-            status = STATUS_UNSUCCESSFUL;
-            GOTO_CLEANUP_EE(EE);
+            GOTO_CLEANUP();
         }
     }
     else
@@ -431,16 +413,14 @@ LwMapSecurityCreateContextInternal(
         pCreateContextCallback = pCallback;
         if (!pCreateContextCallback)
         {
-            LOG_ERROR("Couldn't find any context create callback function.");
-
             status = STATUS_UNSUCCESSFUL;
-            GOTO_CLEANUP_EE(EE);
+            GOTO_CLEANUP();
         }
     }
 
     status = pCreateContextCallback(&pContext->PluginContext,
                                     &pContext->PluginInterface);
-    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    GOTO_CLEANUP_ON_STATUS(status);
 
 cleanup:
     if (!NT_SUCCESS(status))
@@ -569,7 +549,7 @@ LwMapSecurityGetSidFromId(
             BYTE Buffer[SID_MAX_SIZE];
         } sidBuffer;
         ULONG ulSidSize = sizeof(sidBuffer);
-
+        
         if (IsUser)
         {
             status = RtlCreateWellKnownSid(WinLocalSystemSid,
@@ -764,7 +744,7 @@ LwMapSecurityCreateExtendedAccessToken(
                     &size);
     GOTO_CLEANUP_ON_STATUS(status);
 
-    sidCount++;
+    sidCount++;    
 
     // Only Add "NT AUTHORITY\Authenticated Users" for non-Guest
 
@@ -826,6 +806,7 @@ LwMapSecurityCreateAccessTokenFromUidGid(
     PACCESS_TOKEN accessToken = NULL;
     PSID userSid = NULL;
     PSID groupSid = NULL;
+    PACL freeDefaultDacl = NULL;
     PACCESS_TOKEN_CREATE_INFORMATION createInformation = NULL;
 
     if (0 == Uid)
@@ -842,6 +823,7 @@ LwMapSecurityCreateAccessTokenFromUidGid(
         TOKEN_PRIMARY_GROUP tokenPrimaryGroup = { 0 };
         TOKEN_DEFAULT_DACL tokenDefaultDacl = { 0 };
         TOKEN_UNIX tokenUnix = { 0 };
+        ULONG ulDaclSize = 0;
 
         /* Force gid to be 0 to avoid deadlocking call back into
            lsass when it runs with non-zero gid (Solaris, HP-UX) */
@@ -865,6 +847,33 @@ LwMapSecurityCreateAccessTokenFromUidGid(
         tokenUnix.Gid = Gid;
         tokenUnix.Umask = 0;
 
+        ulDaclSize = ACL_HEADER_SIZE +
+            sizeof(ACCESS_ALLOWED_ACE) +
+            RtlLengthSid(userSid) +
+            sizeof(ULONG);
+
+        status = LW_RTL_ALLOCATE(
+                     &tokenDefaultDacl.DefaultDacl,
+                     VOID,
+                     ulDaclSize);
+        GOTO_CLEANUP_ON_STATUS(status);
+
+        freeDefaultDacl = tokenDefaultDacl.DefaultDacl;
+
+        status = RtlCreateAcl(
+                     tokenDefaultDacl.DefaultDacl,
+                     ulDaclSize,
+                     ACL_REVISION);
+        GOTO_CLEANUP_ON_STATUS(status);
+
+        status = RtlAddAccessAllowedAceEx(
+                     tokenDefaultDacl.DefaultDacl,
+                     ACL_REVISION,
+                     0,
+                     GENERIC_ALL,
+                     userSid);
+        GOTO_CLEANUP_ON_STATUS(status);
+
         status = LwMapSecurityCreateExtendedAccessToken(
                         &accessToken,
                         &tokenUser,
@@ -906,6 +915,7 @@ cleanup:
 
     LwMapSecurityFreeSid(Context, &userSid);
     LwMapSecurityFreeSid(Context, &groupSid);
+    RTL_FREE(&freeDefaultDacl);
     LwMapSecurityFreeAccessTokenCreateInformation(Context, &createInformation);
 
     *AccessToken = accessToken;
@@ -963,7 +973,7 @@ LwMapSecurityCreateAccessTokenFromGssContext(
     IN LW_MAP_SECURITY_GSS_CONTEXT GssContext
     )
 {
-       NTSTATUS status = STATUS_SUCCESS;
+    NTSTATUS status = STATUS_SUCCESS;
     PACCESS_TOKEN accessToken = NULL;
     PACCESS_TOKEN_CREATE_INFORMATION createInformation = NULL;
 
@@ -1020,7 +1030,7 @@ LwMapSecurityCreateAccessTokenFromAnsiStringUsername(
                     &accessToken,
                     &unicodeUsername);
     GOTO_CLEANUP_ON_STATUS(status);
-
+    
 cleanup:
     if (!NT_SUCCESS(status))
     {
@@ -1113,6 +1123,71 @@ cleanup:
     return status;
 }
 
+NTSTATUS
+LwMapSecurityCreateAccessTokenFromNtlmLogon(
+    IN PLW_MAP_SECURITY_CONTEXT Context,
+    OUT PACCESS_TOKEN* ppAccessToken,
+    IN PLW_MAP_SECURITY_NTLM_LOGON_INFO pNtlmInfo,
+    OUT PLW_MAP_SECURITY_NTLM_LOGON_RESULT* ppNtlmResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PACCESS_TOKEN pAccessToken = NULL;
+    PLW_MAP_SECURITY_NTLM_LOGON_RESULT pNtlmResult = NULL;
+    PACCESS_TOKEN_CREATE_INFORMATION pCreateInformation = NULL;
+
+    status = Context->PluginInterface->GetAccessTokenCreateInformationFromNtlmLogon(
+                    Context->PluginContext,
+                    &pCreateInformation,
+                    pNtlmInfo,
+                    &pNtlmResult);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = LwMapSecurityCreateExtendedAccessToken(
+                    &pAccessToken,
+                    pCreateInformation->User,
+                    pCreateInformation->Groups,
+                    pCreateInformation->Owner,
+                    pCreateInformation->PrimaryGroup,
+                    pCreateInformation->DefaultDacl,
+                    pCreateInformation->Unix);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+cleanup:
+
+    if (!NT_SUCCESS(status))
+    {
+        if (pAccessToken)
+        {
+            RtlReleaseAccessToken(&pAccessToken);
+        }
+        if (pNtlmResult)
+        {
+            LwMapSecurityFreeNtlmLogonResult(Context, &pNtlmResult);
+        }
+    }
+
+    LwMapSecurityFreeAccessTokenCreateInformation(Context, &pCreateInformation);
+
+    *ppAccessToken = pAccessToken;
+    *ppNtlmResult = pNtlmResult;
+
+    return status;
+}
+
+VOID
+LwMapSecurityFreeNtlmLogonResult(
+    IN PLW_MAP_SECURITY_CONTEXT                 Context,
+    IN OUT PLW_MAP_SECURITY_NTLM_LOGON_RESULT*  ppNtlmResult
+    )
+{
+    if (*ppNtlmResult)
+    {
+        Context->PluginInterface->FreeNtlmLogonResult(
+                    Context->PluginContext,
+                    ppNtlmResult);
+    }
+}
 
 /*
 local variables:
