@@ -63,7 +63,7 @@
 
 #define RB_PARENT(x)         ((x)->pParent)
 #define RB_SET_PARENT(x, y)  { (x)->pParent = (y); }
-
+    
 typedef enum
 {
     RBTreeNodeColor_Red = 0,
@@ -298,7 +298,7 @@ LwRtlRBTreeFindNode(
     if (pRBTree == NULL) {
 	    return NULL;
     }
-
+    
     pIter = pRBTree->pRoot;
 
     while (pIter && !RB_IS_NIL(pRBTree, pIter))
@@ -332,11 +332,10 @@ LwRtlRBTreeAdd(
 {
     NTSTATUS ntStatus = 0;
     PLWRTL_RB_TREE_NODE pTreeNode = NULL;
-    BOOLEAN bFree = FALSE;
     PLWRTL_RB_TREE_NODE pUncle = NULL;
     PLWRTL_RB_TREE_NODE pParent = NULL;
     PLWRTL_RB_TREE_NODE pGrandParent = NULL;
-
+    
     if (!pKey)
     {
         ntStatus = STATUS_INVALID_PARAMETER_2;
@@ -355,14 +354,10 @@ LwRtlRBTreeAdd(
     pTreeNode->pLeft = pRBTree->pSentinel;
     pTreeNode->pParent = NULL;
 
-    bFree = TRUE;
-
     LwRtlRBTreeInsert(pRBTree, pTreeNode);
 
     RB_COLOR_RED(pTreeNode);
 
-    bFree = FALSE;
-
     while ((pTreeNode != pRBTree->pRoot) && RB_IS_RED(pTreeNode->pParent))
     {
         pParent = RB_PARENT(pTreeNode);
@@ -402,7 +397,7 @@ LwRtlRBTreeAdd(
         else
         {
             if (!RB_IS_NIL(pRBTree, pGrandParent))
-            {
+            {                
                 pUncle = pGrandParent->pLeft;
 
                 if (RB_IS_RED(pUncle))
@@ -414,7 +409,7 @@ LwRtlRBTreeAdd(
                     continue;
                 }
             }
-
+            
             if (pTreeNode == pParent->pLeft)
             {
                 pTreeNode = pParent;
@@ -422,7 +417,7 @@ LwRtlRBTreeAdd(
 
                 pParent = RB_PARENT(pTreeNode);
             }
-
+            
             RB_COLOR_BLACK(pParent);
             if (!RB_IS_NIL(pRBTree, pGrandParent))
             {
@@ -440,7 +435,7 @@ cleanup:
 
 error:
 
-    if (bFree && pTreeNode) {
+    if (pTreeNode) {
         LwRtlRBTreeFreeNode(pRBTree, pTreeNode);
     }
 
@@ -470,7 +465,7 @@ LwRtlRBTreeInsert(
             pCurrent = pCurrent->pRight;
         }
     }
-
+    
     RB_SET_PARENT(pTreeNode, pParent);
 
     if (RB_IS_NIL(pRBTree, RB_PARENT(pTreeNode)))
@@ -510,7 +505,7 @@ LwRtlRBTreeRotateLeft(
     {
         pRBTree->pRoot = pNode;
     }
-    else
+    else 
     {
         if (pTreeNode == pTreeNode->pParent->pLeft)
         {
@@ -546,7 +541,7 @@ LwRtlRBTreeRotateRight(
     {
         pRBTree->pRoot = pNode;
     }
-    else
+    else 
     {
         if (pTreeNode == pTreeNode->pParent->pRight)
         {
@@ -850,7 +845,7 @@ LwRtlRBTreeRemoveNode(
     {
         pRBTree->pRoot = pTmp;
     }
-    else
+    else 
     {
         if (pSuccessor == pSuccessor->pParent->pLeft)
         {
@@ -911,12 +906,10 @@ LwRtlRBTreeFixColors(
 {
     PLWRTL_RB_TREE_NODE pUncle = NULL;
     PLWRTL_RB_TREE_NODE pParent = NULL;
-    PLWRTL_RB_TREE_NODE pGrandParent = NULL;
 
     while ((pRBTree->pRoot != pTreeNode) && RB_IS_BLACK(pTreeNode))
     {
         pParent = RB_PARENT(pTreeNode);
-        pGrandParent = RB_PARENT(pParent);
 
         if (pTreeNode == pParent->pLeft)
         {
@@ -942,11 +935,11 @@ LwRtlRBTreeFixColors(
             {
                 RB_COLOR_BLACK(pUncle->pLeft);
                 RB_COLOR_RED(pUncle);
-
+                
                 LwRtlRBTreeRotateRight(pRBTree, pUncle);
                 pUncle = pParent->pRight;
             }
-
+            
             if (!RB_IS_NIL(pRBTree, pUncle))
             {
                 RB_COPY_COLOR(pUncle, pParent);
@@ -956,7 +949,7 @@ LwRtlRBTreeFixColors(
             {
                 RB_COLOR_BLACK(pUncle->pRight);
                 LwRtlRBTreeRotateLeft(pRBTree, pParent);
-            }
+            }            
             pTreeNode = pRBTree->pRoot;
         }
         else
@@ -986,7 +979,7 @@ LwRtlRBTreeFixColors(
                 LwRtlRBTreeRotateLeft(pRBTree, pUncle);
                 pUncle = pParent->pLeft;
             }
-
+            
             if (!RB_IS_NIL(pRBTree, pUncle))
             {
                 RB_COPY_COLOR(pUncle, pParent);
