@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -96,7 +96,7 @@ cleanup:
 
     return NT_SUCCESS(status) ? TRUE : FALSE;
 }
-
+    
 static
 NTSTATUS
 RtlpVerifyAceEx(
@@ -172,7 +172,7 @@ RtlpValidAce(
 {
     return NT_SUCCESS(RtlpVerifyAceEx(Ace, FALSE));
 }
-
+    
 USHORT
 RtlLengthAccessAllowedAce(
     IN PSID Sid
@@ -191,6 +191,15 @@ RtlLengthAccessDeniedAce(
             RtlLengthSid(Sid));
 }
 
+USHORT
+RtlLengthAccessAuditAce(
+    IN PSID Sid
+    )
+{
+    return (LW_FIELD_OFFSET(SYSTEM_AUDIT_ACE, SidStart) +
+            RtlLengthSid(Sid));
+}
+
 NTSTATUS
 RtlInitializeAccessAllowedAce(
     OUT PACCESS_ALLOWED_ACE Ace,
@@ -410,7 +419,7 @@ RtlpCheckEnoughAclBuffer(
         status = STATUS_INTEGER_OVERFLOW;
         GOTO_CLEANUP();
     }
-
+    
     if (AclSize < sizeNeeded)
     {
         status = STATUS_BUFFER_TOO_SMALL;
@@ -1034,7 +1043,7 @@ RtlpValidAceList(
         GOTO_CLEANUP();
     }
 
-    // Validate ACE list
+    // Validate ACE list                    
     while (aceOffset < AceListLength)
     {
         PACE_HEADER aceHeader = (PACE_HEADER) LW_PTR_ADD(AceList, aceOffset);
@@ -1368,7 +1377,7 @@ RtlAddAccessAllowedAceEx(
             if (aceHeader->AceType == ACCESS_ALLOWED_ACE_TYPE)
             {
                 if (RtlpCompareAceFlagsForInsert(
-                        AceFlags,
+                        AceFlags, 
                         aceHeader->AceFlags) <= 0)
                 {
                     break;
@@ -1379,7 +1388,7 @@ RtlAddAccessAllowedAceEx(
         aceLocation = (PACCESS_ALLOWED_ACE)aceHeader;
     }
 
-    // Covers Acl->AceCount == 0 and the case where we are adding
+    // Covers Acl->AceCount == 0 and the case where we are adding 
     // to the end of the list
 
     if (usLoop == Acl->AceCount)
@@ -1471,7 +1480,7 @@ RtlAddAccessDeniedAceEx(
             if (aceHeader->AceType == ACCESS_DENIED_ACE_TYPE)
             {
                 if (RtlpCompareAceFlagsForInsert(
-                        AceFlags,
+                        AceFlags, 
                         aceHeader->AceFlags) <= 0)
                 {
                     break;
@@ -1482,7 +1491,7 @@ RtlAddAccessDeniedAceEx(
         aceLocation = (PACCESS_DENIED_ACE)aceHeader;
     }
 
-    // Covers Acl->AceCount == 0 and the case where we are adding
+    // Covers Acl->AceCount == 0 and the case where we are adding 
     // to the end of the list
 
     if (usLoop == Acl->AceCount)
