@@ -46,6 +46,22 @@
 // ACE Functions
 //
 
+// TODO-Use #define for this
+USHORT
+RtlLengthAccessAuditAce(
+    IN PSID Sid
+    );
+///<
+/// Get size required for an SYSTEM_AUDIT_ACE.
+///
+/// This function gets the number of bytes required for an
+/// system audit ACE (SYSTEM_AUDIT_ACE) given a particular SID.
+///
+/// @param[in] Sid - SID for ACE.
+///
+/// @return Number of bytes required or 0 if Sid is invalid.
+///
+
 NTSTATUS
 RtlInitializeAccessAllowedAce(
     OUT PACCESS_ALLOWED_ACE Ace,
