@@ -284,7 +284,7 @@ RtlpObjectSetOwner(
                      &bDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
 
-        if (pCreatorSecDescOwner
+        if (pCreatorSecDescOwner 
             && RtlIsSidMemberOfToken(pUserToken, pCreatorSecDescOwner))
         {
             status = RtlDuplicateSid(&pOwner, pCreatorSecDescOwner);
@@ -300,7 +300,7 @@ RtlpObjectSetOwner(
 
     // Check for Defaulting the owner from the parent Sec Desc
 
-    if (AutoInheritFlags & SEF_DEFAULT_OWNER_FROM_PARENT)
+    if (pParentSecDesc && (AutoInheritFlags & SEF_DEFAULT_OWNER_FROM_PARENT))
     {
         status = RtlGetOwnerSecurityDescriptor(
                      pParentSecDesc,
@@ -347,8 +347,8 @@ cleanup:
     {
         LW_RTL_FREE(&pOwner);
     }
-
-
+    
+        
     return status;
 }
 
@@ -401,8 +401,7 @@ RtlpObjectSetGroup(
     }
 
     // Check for Defaulting the owner from the parent Sec Desc
-
-    if (AutoInheritFlags & SEF_DEFAULT_GROUP_FROM_PARENT)
+    if (pParentSecDesc && (AutoInheritFlags & SEF_DEFAULT_GROUP_FROM_PARENT))
     {
         status = RtlGetGroupSecurityDescriptor(
                      pParentSecDesc,
@@ -500,11 +499,13 @@ RtlpObjectSetDacl(
     PTOKEN_DEFAULT_DACL pTokenDefaultDaclInformation = NULL;
     ULONG ulTokenDefaultDaclLength = 0;
     ULONG ulTokenDefaultDaclSize = 0;
-
+    SECURITY_DESCRIPTOR_CONTROL DaclControlSet = 0;
+    SECURITY_DESCRIPTOR_CONTROL DaclControlChange = (SE_DACL_PROTECTED|
+                                                     SE_DACL_AUTO_INHERITED);
 
     // Pull the DACLs so we have something to work with
 
-    if (pParentSecDesc)
+    if (pParentSecDesc && (AutoInheritFlags & SEF_DACL_AUTO_INHERIT))
     {
         status = RtlGetDaclSecurityDescriptor(
                      pParentSecDesc,
@@ -565,7 +566,7 @@ RtlpObjectSetDacl(
         {
             LW_RTL_FREE(&pFinalDacl);
         }
-
+        
         status = RtlQueryAccessTokenInformation(
                      pUserToken,
                      TokenDefaultDacl,
@@ -606,11 +607,27 @@ RtlpObjectSetDacl(
                      pGenericMap);
         GOTO_CLEANUP_ON_STATUS(status);
 
-        bFinalIsDaclDefaulted = TRUE;
+        // Final DACL is defaulted only if there is no parent DACL
+
+        bFinalIsDaclDefaulted = bParentIsDaclPresent ? FALSE : TRUE;
     }
 
     // Finally set the new DACL in the outgoing Seccurity Descriptor
 
+    if (CreatorSecDescControl & SE_DACL_PROTECTED)
+    {
+        DaclControlSet |= SE_DACL_PROTECTED;
+    } else if (AutoInheritFlags & SEF_DACL_AUTO_INHERIT)
+    {
+        DaclControlSet |= SE_DACL_AUTO_INHERITED;
+    }
+
+    status = RtlSetSecurityDescriptorControl(
+                 pSecurityDescriptor,
+                 DaclControlChange,
+                 DaclControlSet);
+    GOTO_CLEANUP_ON_STATUS(status);
+
     status = RtlSetDaclSecurityDescriptor(
                  pSecurityDescriptor,
                  pFinalDacl ? TRUE : FALSE,
@@ -624,8 +641,8 @@ cleanup:
     if (!NT_SUCCESS(status))
     {
         LW_RTL_FREE(&pFinalDacl);
-    }
-
+    }    
+            
     LW_RTL_FREE(&pBuffer);
 
     return status;
@@ -790,7 +807,7 @@ RtlpObjectDaclAssignSecurity(
                 GOTO_CLEANUP_ON_STATUS(status);
             }
             break;
-
+        
             case ACCESS_DENIED_ACE_TYPE:
             {
                 aceDeny = (PACCESS_DENIED_ACE)aceHeader;
@@ -822,12 +839,12 @@ RtlpObjectDaclAssignSecurity(
                 // ignore
                 break;
         }
-
+        
     }
 
     *ppObjectDacl = pDacl;
 
-    status = STATUS_SUCCESS;
+    status = STATUS_SUCCESS;    
 
 cleanup:
     if (!NT_SUCCESS(status))
@@ -891,14 +908,14 @@ RtlpObjectInheritSecurity(
     // Inheritance Algorithm:
     //
     // (a) If defaulted creator DACL and no parent, just use defaulted
-    //     creator DACL,
+    //     creator DACL, 
     //
     // else
     //
     // (b) Add in all creator DACL entries
     // (c) Add in inheritable ACEs from parent
     //      i.   Skip if is a container and is NOT a container inherit
-    //           ace, or if is NOT a container and is NOT a object
+    //           ace, or if is NOT a container and is NOT a object 
     //           (non-container) inherit ace
     //      ii.  For containers, add propagatable inheritance entries
     //      iii. Add inherited ace, applying generic map
@@ -1125,7 +1142,7 @@ RtlpObjectInheritSecurity(
             else if (RtlEqualSid(pAceSid, &CreatorGroup.sid))
             {
                 pAceSid = pTokenPrimaryGroupInfo->PrimaryGroup;
-            }
+            }                
 
             status = RtlAddAccessAllowedAceEx(
                          pDacl,
@@ -1148,7 +1165,7 @@ RtlpObjectInheritSecurity(
             else if (RtlEqualSid(pAceSid, &CreatorGroup.sid))
             {
                 pAceSid = pTokenPrimaryGroupInfo->PrimaryGroup;
-            }
+            }                
 
             status = RtlAddAccessDeniedAceEx(
                          pDacl,
@@ -1177,7 +1194,7 @@ cleanup:
     {
         LW_RTL_FREE(&pDacl);
     }
-
+    
     return status;
 }
 
