@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -158,7 +158,7 @@ RtlValidSecurityDescriptor(
     {
         GOTO_CLEANUP();
     }
-
+        
     if (SecurityDescriptor->Dacl &&
         !RtlValidAcl(SecurityDescriptor->Dacl, NULL))
     {
@@ -416,6 +416,48 @@ RtlLengthSecurityDescriptor(
     return size;
 }
 
+ULONG
+RtlLengthSecurityDescriptorRelative(
+    IN PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor
+    )
+{
+    ULONG size = SECURITY_DESCRIPTOR_RELATIVE_MIN_SIZE;
+
+    if (SecurityDescriptor->Owner)
+    {
+        PSID pOwnerlittleEndianSid = (PSID)LwRtlOffsetToPointer(SecurityDescriptor,
+                                                                LW_LTOH32(SecurityDescriptor->Owner));
+
+        size += RtlLengthRequiredSid(LW_LTOH8(pOwnerlittleEndianSid->SubAuthorityCount));
+    }
+
+    if (SecurityDescriptor->Group)
+    {
+        PSID pGrouplittleEndianSid = (PSID)LwRtlOffsetToPointer(SecurityDescriptor,
+                                                                 LW_LTOH32(SecurityDescriptor->Group));
+
+        size += RtlLengthRequiredSid(LW_LTOH8(pGrouplittleEndianSid->SubAuthorityCount));
+    }
+
+    if (SecurityDescriptor->Dacl)
+    {
+        PACL plittleEndianDacl = (PACL)LwRtlOffsetToPointer(SecurityDescriptor,
+                                                             LW_LTOH32(SecurityDescriptor->Dacl));
+
+        size +=  LW_LTOH16(plittleEndianDacl->AclSize);
+    }
+
+    if (SecurityDescriptor->Sacl)
+    {
+        PACL plittleEndianSacl = (PACL)LwRtlOffsetToPointer(SecurityDescriptor,
+                                                            LW_LTOH32(SecurityDescriptor->Sacl));
+
+        size += LW_LTOH16(plittleEndianSacl->AclSize);
+    }
+
+    return size;
+}
+
 NTSTATUS
 RtlGetSecurityDescriptorControl(
     IN PSECURITY_DESCRIPTOR_ABSOLUTE SecurityDescriptor,
@@ -952,6 +994,190 @@ cleanup:
     return status;
 }
 
+
+/* 
+ * Byte swap a UINT32 that is not guaranteed to be word-boundary aligned.
+ * Advance cursor by the number of bytes swapped afterwards.
+ */
+static VOID
+RtlHTOL32Stream(
+    IN OUT PUINT8 *ppCursor,
+    OUT PUINT32 pValue
+    )
+{
+    PUINT8 cursor = *ppCursor;
+    UINT32 i32 = 0;
+
+    /* Control */
+    memcpy(&i32, cursor, sizeof(i32));
+    if (pValue)
+    {
+        *pValue = i32;
+    }
+    i32 = LW_HTOL32(i32);
+    memcpy(cursor, &i32, sizeof(i32));
+    cursor += sizeof(i32);
+    *ppCursor = cursor;
+}
+
+
+/* Byte swap a Sid that is not guaranteed to be word-boundary aligned */
+static VOID
+RtlEncodeLittleEndianSidSwab(
+    IN PUINT8 cursor,
+    OUT PDWORD pLen
+    )
+{
+    DWORD i = 0;
+    UINT8 subAuthorityCount = 0;
+    DWORD sidSize = 0;
+
+    /* SID List for Owner; Revision and SubAuthorityCount are both 1 bytes */
+    cursor++; // Revision
+
+    subAuthorityCount = *cursor;
+    cursor++; // SubAuthorityCount
+
+    cursor += 6; //IdentifierAuthority
+
+    sidSize = _SID_GET_SIZE_REQUIRED(subAuthorityCount);
+    for (i=0; i<subAuthorityCount; i++)
+    {
+        RtlHTOL32Stream(&cursor, NULL);
+    }
+    if (pLen)
+    {
+        *pLen = sidSize;
+    }
+}
+
+
+static VOID
+RtlpEncodeLittleEndianAclSwab(
+    IN PUINT8 cursor,
+    OUT PDWORD pLen
+    )
+{
+    UINT16 aceCount = 0;
+    DWORD i = 0;
+    PACL littleEndianAcl = (PACL) cursor;
+    PUINT8 startCursor = cursor;
+
+    // This value is in host byte order, so get before swapping.
+    aceCount = littleEndianAcl->AceCount;
+
+    littleEndianAcl->AclSize = LW_HTOL16(littleEndianAcl->AclSize);
+    littleEndianAcl->AceCount = LW_HTOL16(littleEndianAcl->AceCount);
+    littleEndianAcl->Sbz2 = LW_HTOL16(littleEndianAcl->Sbz2);
+
+    cursor += sizeof(ACL);
+    for (i=0; i<aceCount; i++)
+    {
+        PACE_HEADER aceHeader = (PACE_HEADER) cursor;
+        PACCESS_ALLOWED_ACE ace = (PACCESS_ALLOWED_ACE) cursor;
+        UINT16 aceSize = 0;
+
+        aceSize = aceHeader->AceSize;
+        aceHeader->AceSize = LW_HTOL16(aceHeader->AceSize);
+        ace->Mask = LW_HTOL32(ace->Mask);
+
+        RtlEncodeLittleEndianSidSwab((PUINT8) &ace->SidStart, NULL);
+        cursor += aceSize;
+    }
+
+    *pLen = cursor - startCursor;
+}
+
+
+/*
+ * 6.0 -> 6.1 conversion routine to fix byte swapping problem
+ * on big endian systems. The data is supposed to be stored in
+ * little endian, but a bug in lwbase affecting BE systems
+ * effectively disabled all swapping functions, so the data is 
+ * stored in native byte order.
+ * Since this bug is fixed in 6.1, the byte order is wrong, and needs
+ * to be adjusted, which is the purpose of this function.
+ */
+NTSTATUS
+RtlAbsoluteToSelfRelativeSDSwab(
+    IN OUT PSECURITY_DESCRIPTOR_RELATIVE SelfRelativeSecurityDescriptor,
+    IN ULONG BufferLength
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PUINT8 cursor = NULL;
+    ULONG itemLen = 0;
+    DWORD offset = 0;
+    PSECURITY_DESCRIPTOR_RELATIVE selfRelativeSD = NULL;
+
+    if (!SelfRelativeSecurityDescriptor || BufferLength == 0)
+    {
+        status = STATUS_INVALID_PARAMETER;
+        GOTO_CLEANUP();
+    }
+
+    selfRelativeSD = SelfRelativeSecurityDescriptor;
+    selfRelativeSD->Control = LW_HTOL16(selfRelativeSD->Control);
+
+    /* Offset SID list for Owner */
+    offset = SECURITY_DESCRIPTOR_RELATIVE_MIN_SIZE;
+    cursor = ((PUINT8) SelfRelativeSecurityDescriptor) + offset;
+
+    /* SID List for Owner */
+    itemLen = 0;
+    if (offset < BufferLength)
+    {
+        RtlEncodeLittleEndianSidSwab(cursor, &itemLen);
+    }
+    if (itemLen > 0)
+    {
+        selfRelativeSD->Owner = LW_HTOL32(offset);
+    }
+    offset += itemLen;
+    cursor += itemLen;
+
+    /* SID List for Group; Group may not be present */
+    itemLen = 0;
+    if (offset < BufferLength && selfRelativeSD->Group)
+    {
+        RtlEncodeLittleEndianSidSwab(cursor, &itemLen);
+    }
+    if (itemLen > 0)
+    {
+        selfRelativeSD->Group = LW_HTOL32(offset);
+    }
+    offset += itemLen;
+    cursor += itemLen;
+
+    /* Dacl list */
+    itemLen = 0;
+    if (offset < BufferLength)
+    {
+        RtlpEncodeLittleEndianAclSwab(cursor, &itemLen);
+    }
+    if (itemLen > 0)
+    {
+        selfRelativeSD->Dacl = LW_HTOL32(offset);
+    }
+    offset += itemLen;
+    cursor += itemLen;
+
+    /* Sacl list */
+    itemLen = 0;
+    if (offset < BufferLength)
+    {
+        RtlpEncodeLittleEndianAclSwab(cursor, &itemLen);
+    }
+    if (itemLen > 0)
+    {
+        selfRelativeSD->Sacl = LW_HTOL32(offset);
+    }
+
+    /* ...and Bob's your uncle */
+cleanup:
+    return status;
+}
+
 NTSTATUS
 RtlSelfRelativeToAbsoluteSD(
     IN PSECURITY_DESCRIPTOR_RELATIVE SelfRelativeSecurityDescriptor,
@@ -1187,7 +1413,7 @@ RtlQuerySecurityDescriptorInfo(
     IN PSECURITY_DESCRIPTOR_RELATIVE ObjectSecurityDescriptor
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
+    NTSTATUS status = STATUS_SUCCESS;    
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDescAbs = NULL;
 
     // Sanity checks
@@ -1196,10 +1422,10 @@ RtlQuerySecurityDescriptorInfo(
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_ON_STATUS(status);
-    }
+    }    
 
     status = RtlpCreateAbsSecDescFromRelative(
-                 &pSecDescAbs,
+                 &pSecDescAbs, 
                  ObjectSecurityDescriptor);
     GOTO_CLEANUP_ON_STATUS(status);
 
@@ -1209,7 +1435,7 @@ RtlQuerySecurityDescriptorInfo(
     {
         PSID pOwnerSid = NULL;
         BOOLEAN bOwnerDefaulted = FALSE;
-
+        
         status = RtlGetOwnerSecurityDescriptor(
                      pSecDescAbs,
                      &pOwnerSid,
@@ -1224,7 +1450,7 @@ RtlQuerySecurityDescriptorInfo(
                      FALSE);
         GOTO_CLEANUP_ON_STATUS(status);
     }
-
+        
     if (!(SecurityInformation & GROUP_SECURITY_INFORMATION))
     {
         PSID pGroupSid = NULL;
@@ -1235,12 +1461,12 @@ RtlQuerySecurityDescriptorInfo(
                      &pGroupSid,
                      &bGroupDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
-
+        
         LW_RTL_FREE(&pGroupSid);
 
         status = RtlSetGroupSecurityDescriptor(pSecDescAbs, NULL, FALSE);
         GOTO_CLEANUP_ON_STATUS(status);
-    }
+    }        
 
     if (!(SecurityInformation & DACL_SECURITY_INFORMATION))
     {
@@ -1262,15 +1488,15 @@ RtlQuerySecurityDescriptorInfo(
         LW_RTL_FREE(&pDacl);
 
         status = RtlSetDaclSecurityDescriptor(pSecDescAbs, FALSE, NULL, FALSE);
-        GOTO_CLEANUP_ON_STATUS(status);
-
+        GOTO_CLEANUP_ON_STATUS(status);        
+    
         status = RtlSetSecurityDescriptorControl(
                      pSecDescAbs,
                      DaclControlChange,
                      0);
-        GOTO_CLEANUP_ON_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);        
     }
-
+    
     if (!(SecurityInformation & SACL_SECURITY_INFORMATION))
     {
         PACL pSacl = NULL;
@@ -1285,12 +1511,12 @@ RtlQuerySecurityDescriptorInfo(
                      &bSaclPresent,
                      &pSacl,
                      &bSaclDefaulted);
-        GOTO_CLEANUP_ON_STATUS(status);
-
+        GOTO_CLEANUP_ON_STATUS(status);        
+    
         LW_RTL_FREE(&pSacl);
 
         status = RtlSetSaclSecurityDescriptor(pSecDescAbs, FALSE, NULL, FALSE);
-        GOTO_CLEANUP_ON_STATUS(status);
+        GOTO_CLEANUP_ON_STATUS(status);        
 
         status = RtlSetSecurityDescriptorControl(
                      pSecDescAbs,
@@ -1305,8 +1531,8 @@ RtlQuerySecurityDescriptorInfo(
                  pSecDescAbs,
                  SecurityDescriptor,
                  Length);
-    GOTO_CLEANUP_ON_STATUS(status);
-
+    GOTO_CLEANUP_ON_STATUS(status);    
+    
 
 cleanup:
     if (pSecDescAbs)
@@ -1327,40 +1553,40 @@ RtlSetSecurityDescriptorInfo(
     IN PGENERIC_MAPPING GenericMapping
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
+    NTSTATUS status = STATUS_SUCCESS;    
     PSECURITY_DESCRIPTOR_ABSOLUTE pObjSecDescAbs = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pInputSecDescAbs = NULL;
     SECURITY_DESCRIPTOR_CONTROL InputSecDescControl = 0;
-    UCHAR Revision = 0;
-
+    UCHAR Revision = 0;    
+    
     // Sanity checks
 
     if (SecurityInformation == 0)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_ON_STATUS(status);
-    }
+    }    
 
     // Convert to an Absolute SecDesc
 
     status = RtlpCreateAbsSecDescFromRelative(
-                 &pObjSecDescAbs,
+                 &pObjSecDescAbs, 
                  ObjectSecurityDescriptor);
     GOTO_CLEANUP_ON_STATUS(status);
-
+        
     status = RtlpCreateAbsSecDescFromRelative(
-                 &pInputSecDescAbs,
+                 &pInputSecDescAbs, 
                  InputSecurityDescriptor);
     GOTO_CLEANUP_ON_STATUS(status);
 
     // Merge
-
+        
     status = RtlGetSecurityDescriptorControl(
                  pInputSecDescAbs,
                  &InputSecDescControl,
                  &Revision);
     GOTO_CLEANUP_ON_STATUS(status);
-
+        
     // Owner
 
     if (SecurityInformation & OWNER_SECURITY_INFORMATION)
@@ -1395,7 +1621,7 @@ RtlSetSecurityDescriptorInfo(
                      pInputOwnerSid,
                      bInputOwnerDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
-
+        
     }
 
     // Group
@@ -1420,7 +1646,7 @@ RtlSetSecurityDescriptorInfo(
                      &pInputGroupSid,
                      &bInputGroupDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
-
+        
         status = RtlSetGroupSecurityDescriptor(
                      pInputSecDescAbs,
                      NULL,
@@ -1432,7 +1658,7 @@ RtlSetSecurityDescriptorInfo(
                      pInputGroupSid,
                      bInputGroupDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
-    }
+    }        
 
     // Dacl
 
@@ -1487,14 +1713,14 @@ RtlSetSecurityDescriptorInfo(
         GOTO_CLEANUP_ON_STATUS(status);
 
         DaclControlSet = InputSecDescControl & DaclControlChange;
-
+        
         status = RtlSetSecurityDescriptorControl(
                      pObjSecDescAbs,
                      DaclControlChange,
                      DaclControlSet);
         GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     // Sacl
 
     if (SecurityInformation & SACL_SECURITY_INFORMATION)
@@ -1525,14 +1751,14 @@ RtlSetSecurityDescriptorInfo(
                      &pInputSacl,
                      &bInputSaclDefaulted);
         GOTO_CLEANUP_ON_STATUS(status);
-
+    
         status = RtlSetSaclSecurityDescriptor(
                      pInputSecDescAbs,
                      FALSE,
                      NULL,
                      FALSE);
         GOTO_CLEANUP_ON_STATUS(status);
-
+    
         status = RtlSetSecurityDescriptorControl(
                      pObjSecDescAbs,
                      SaclControlChange,
@@ -1561,8 +1787,8 @@ RtlSetSecurityDescriptorInfo(
                  pObjSecDescAbs,
                  NewObjectSecurityDescriptor,
                  NewObjectSecurityDescriptorLength);
-    GOTO_CLEANUP_ON_STATUS(status);
-
+    GOTO_CLEANUP_ON_STATUS(status);    
+    
 
 cleanup:
     if (pObjSecDescAbs)
@@ -1674,7 +1900,7 @@ cleanup:
         LW_RTL_FREE(&pSacl);
         LW_RTL_FREE(&pDacl);
     }
-
+            
     return status;
 }
 
@@ -1683,7 +1909,6 @@ RtlpFreeAbsoluteSecurityDescriptor(
     IN OUT PSECURITY_DESCRIPTOR_ABSOLUTE *ppSecDesc
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
     PSID pOwner = NULL;
     PSID pGroup = NULL;
     PACL pDacl = NULL;
@@ -1699,11 +1924,11 @@ RtlpFreeAbsoluteSecurityDescriptor(
 
     pSecDesc = *ppSecDesc;
 
-    status = RtlGetOwnerSecurityDescriptor(pSecDesc, &pOwner, &bDefaulted);
-    status = RtlGetGroupSecurityDescriptor(pSecDesc, &pGroup, &bDefaulted);
+    (void) RtlGetOwnerSecurityDescriptor(pSecDesc, &pOwner, &bDefaulted);
+    (void) RtlGetGroupSecurityDescriptor(pSecDesc, &pGroup, &bDefaulted);
 
-    status = RtlGetDaclSecurityDescriptor(pSecDesc, &bPresent, &pDacl, &bDefaulted);
-    status = RtlGetSaclSecurityDescriptor(pSecDesc, &bPresent, &pSacl, &bDefaulted);
+    (void) RtlGetDaclSecurityDescriptor(pSecDesc, &bPresent, &pDacl, &bDefaulted);
+    (void) RtlGetSaclSecurityDescriptor(pSecDesc, &bPresent, &pSacl, &bDefaulted);
 
     RTL_FREE(&pSecDesc);
     RTL_FREE(&pOwner);
