@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -216,13 +216,13 @@ RtlGetRidSid(
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     if (Sid->SubAuthorityCount == 0)
     {
         status = STATUS_INVALID_SUB_AUTHORITY;
         GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     *Rid = Sid->SubAuthority[Sid->SubAuthorityCount-1];
 
 cleanup:
@@ -484,7 +484,7 @@ cleanup:
     }
 
     return status;
-
+    
 }
 
 NTSTATUS
@@ -543,7 +543,7 @@ RtlAllocateCStringFromSid(
     {
         count += snprintf(result + count,
                           size - count,
-                          "-%u",
+                          "-%u", 
                           Sid->SubAuthority[i]);
     }
 
@@ -991,7 +991,7 @@ RtlCreateWellKnownSid(
             GOTO_CLEANUP_ON_STATUS(status);
             sidBuffer.Sid.SubAuthority[0] = SECURITY_CREATOR_OWNER_RID;
             break;
-        }
+        }   
         case WinCreatorGroupSid:
         {
             // S-1-3-1
@@ -1000,7 +1000,7 @@ RtlCreateWellKnownSid(
             GOTO_CLEANUP_ON_STATUS(status);
             sidBuffer.Sid.SubAuthority[0] = SECURITY_CREATOR_GROUP_RID;
             break;
-        }
+        }   
         case WinAuthenticatedUserSid:
         {
             // S-1-5-11
@@ -1059,18 +1059,18 @@ RtlCreateWellKnownSid(
         case WinAccountDomainUsersSid:
         case WinAccountDomainGuestsSid:
         {
-            SID_IDENTIFIER_AUTHORITY identifierAuthority = { SECURITY_NT_AUTHORITY };
-            status = RtlInitializeSid(&sidBuffer.Sid,
-                                      &identifierAuthority,
-                                      DomainOrComputerSid->SubAuthorityCount + 1);
-            GOTO_CLEANUP_ON_STATUS(status);
-
             if (DomainOrComputerSid == NULL)
             {
                 status = STATUS_INVALID_PARAMETER;
                 GOTO_CLEANUP_ON_STATUS(status);
             }
 
+            SID_IDENTIFIER_AUTHORITY identifierAuthority = { SECURITY_NT_AUTHORITY };
+            status = RtlInitializeSid(&sidBuffer.Sid,
+                                      &identifierAuthority,
+                                      DomainOrComputerSid->SubAuthorityCount + 1);
+            GOTO_CLEANUP_ON_STATUS(status);
+
             for (i = 0; i < DomainOrComputerSid->SubAuthorityCount; i++)
             {
                 sidBuffer.Sid.SubAuthority[i] = DomainOrComputerSid->SubAuthority[i];
@@ -1179,6 +1179,120 @@ cleanup:
     return status;
 }
 
+NTSTATUS
+RtlAllocateWellKnownSid(
+    IN WELL_KNOWN_SID_TYPE WellKnownSidType,
+    IN OPTIONAL PSID DomainOrComputerSid,
+    OUT PSID* Sid
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PSID sid = NULL;
+    union {
+        SID Sid;
+        UCHAR Buffer[SID_MAX_SIZE];
+    } sidBuffer = { .Buffer = { 0 } };
+    ULONG size = sizeof(sidBuffer);
+
+    status = RtlCreateWellKnownSid(
+                    WellKnownSidType,
+                    DomainOrComputerSid,
+                    &sidBuffer.Sid,
+                    &size);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    status = RtlDuplicateSid(&sid, &sidBuffer.Sid);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+cleanup:
+    if (!NT_SUCCESS(status))
+    {
+        RTL_FREE(&sid);
+    }
+
+    *Sid = sid;
+
+    return status;
+}
+
+NTSTATUS
+RtlConvertLittleEndianToSid(
+    IN PVOID Buffer,
+    IN ULONG Length,
+    OUT PSID Sid,
+    IN OUT PULONG SidSize
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PSID srcSid = NULL;
+    UCHAR i = 0;
+
+    if (*SidSize < Length)
+    {
+        *SidSize = Length;     // Required length
+        status = STATUS_BUFFER_TOO_SMALL;
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    srcSid = (PSID)Buffer;
+
+    Sid->Revision = LW_LTOH8(srcSid->Revision);
+    Sid->SubAuthorityCount = LW_LTOH8(srcSid->SubAuthorityCount);
+    Sid->IdentifierAuthority = srcSid->IdentifierAuthority;   // Byte array
+    for (i = 0; i < Sid->SubAuthorityCount; ++i)
+    {
+        Sid->SubAuthority[i] = LW_LTOH32(srcSid->SubAuthority[i]);
+    }
+    
+    *SidSize = RtlLengthRequiredSid(Sid->SubAuthorityCount);
+
+cleanup:
+
+    return status;
+}
+
+NTSTATUS
+RtlConvertSidToLittleEndian(
+    IN PSID Sid,
+    OUT PVOID Buffer,
+    IN ULONG Length,
+    OUT OPTIONAL PULONG UsedLength
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    ULONG sidSize = RtlLengthRequiredSid(Sid->SubAuthorityCount);
+    ULONG usedLength = 0;
+    PSID dstSid = NULL;
+    UCHAR i = 0;
+
+    if (sidSize > Length)
+    {
+        status = STATUS_BUFFER_TOO_SMALL;
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
+    dstSid = (PSID)Buffer;
+
+    dstSid->Revision = LW_HTOL8(Sid->Revision);
+    dstSid->SubAuthorityCount = LW_HTOL8(Sid->SubAuthorityCount);
+    dstSid->IdentifierAuthority = Sid->IdentifierAuthority;    // Byte array
+    for (i = 0; i < Sid->SubAuthorityCount; ++i)
+    {
+        dstSid->SubAuthority[i] = LW_HTOL32(Sid->SubAuthority[i]);
+    }
+
+    usedLength = sidSize;
+
+cleanup:
+
+    if (UsedLength)
+    {
+        *UsedLength = usedLength;
+    }
+
+    return status;
+}
+
 
 /*
 local variables:
