@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -195,7 +195,7 @@ RtlCreateAccessToken(
         status = RtlSafeAddULONG(&requiredSize, requiredSize, size);
         GOTO_CLEANUP_ON_STATUS(status);
     }
-
+    
     status = RTL_ALLOCATE(&token, ACCESS_TOKEN, requiredSize);
     GOTO_CLEANUP_ON_STATUS(status);
 
@@ -414,7 +414,7 @@ RtlQueryAccessTokenInformation(
         {
             PTOKEN_GROUPS tokenInfo = (PTOKEN_GROUPS) TokenInformation;
             location = LW_PTR_ADD(TokenInformation,
-                                  (sizeof(TOKEN_GROUPS) +
+                                  (sizeof(TOKEN_GROUPS) + 
                                    (sizeof(AccessToken->Groups[0]) * AccessToken->GroupCount)));
             tokenInfo->GroupCount = AccessToken->GroupCount;
             for (i = 0; i < AccessToken->GroupCount; i++)
@@ -475,7 +475,11 @@ RtlQueryAccessTokenInformation(
     status = STATUS_SUCCESS;
 
 cleanup:
-    *ReturnedLength = requiredSize;
+
+    if (ReturnedLength)
+    {
+        *ReturnedLength = requiredSize;
+    }
 
     return status;
 }
@@ -592,7 +596,7 @@ RtlAccessCheck(
         GOTO_CLEANUP();
     }
 
-    if ((SecurityDescriptor->Owner == NULL) ||
+    if ((SecurityDescriptor->Owner == NULL) || 
         (SecurityDescriptor->Group == NULL))
     {
         status = STATUS_INVALID_SECURITY_DESCR;
@@ -673,7 +677,7 @@ RtlAccessCheck(
             {
                 desiredAccess |= (READ_CONTROL | WRITE_DAC);
             }
-
+                
             SetFlag(grantedAccess, (READ_CONTROL | WRITE_DAC) & desiredAccess);
             ClearFlag(desiredAccess, grantedAccess);
         }
@@ -868,7 +872,7 @@ RtlAccessTokenRelativeSize(
     ulRelativeSize += RtlLengthSid(pToken->User.Sid);
     Align32(&ulRelativeSize);
 
-    if (pToken->Groups);
+    if (pToken->Groups)
     {
         ulRelativeSize += sizeof(SID_AND_ATTRIBUTES_SELF_RELATIVE) * pToken->GroupCount;
         Align32(&ulRelativeSize);
@@ -924,29 +928,29 @@ RtlAccessTokenToSelfRelativeAccessToken(
         }
 
         pBuffer = (PBYTE) pRelative;
-
+        
         pRelative->Flags = pToken->Flags;
         pRelative->User.Attributes = pToken->User.Attributes;
         pRelative->GroupCount = pToken->GroupCount;
         pRelative->Uid = pToken->Uid;
         pRelative->Gid = pToken->Gid;
         pRelative->Umask = pToken->Umask;
-
+        
         ulOffset += sizeof(*pRelative);
         Align32(&ulOffset);
-
+        
         pRelative->User.SidOffset = ulOffset;
         memcpy(pBuffer + ulOffset, pToken->User.Sid, RtlLengthSid(pToken->User.Sid));
         ulOffset += RtlLengthSid(pToken->User.Sid);
         Align32(&ulOffset);
-
+        
         if (pToken->Groups)
         {
             pRelative->GroupsOffset = ulOffset;
             pGroups = (PSID_AND_ATTRIBUTES_SELF_RELATIVE) (pBuffer + ulOffset);
             ulOffset += sizeof(SID_AND_ATTRIBUTES_SELF_RELATIVE) * pToken->GroupCount;
             Align32(&ulOffset);
-
+            
             for (i = 0; i < pToken->GroupCount; i++)
             {
                 pGroups[i].Attributes = pToken->Groups[i].Attributes;
@@ -960,7 +964,7 @@ RtlAccessTokenToSelfRelativeAccessToken(
         {
             pRelative->GroupsOffset = 0;
         }
-
+        
         if (pToken->Owner)
         {
             pRelative->OwnerOffset = ulOffset;
@@ -972,7 +976,7 @@ RtlAccessTokenToSelfRelativeAccessToken(
         {
             pRelative->OwnerOffset = 0;
         }
-
+        
         if (pToken->PrimaryGroup)
         {
             pRelative->PrimaryGroupOffset = ulOffset;
@@ -984,7 +988,7 @@ RtlAccessTokenToSelfRelativeAccessToken(
         {
             pRelative->PrimaryGroupOffset = 0;
         }
-
+        
         if (pToken->DefaultDacl)
         {
             pRelative->DefaultDaclOffset = ulOffset;
@@ -996,7 +1000,7 @@ RtlAccessTokenToSelfRelativeAccessToken(
         {
             pRelative->DefaultDaclOffset = 0;
         }
-
+        
 
         assert(ulOffset == ulRelativeSize);
     }
@@ -1037,7 +1041,6 @@ RtlSelfRelativeAccessTokenToAccessToken(
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    PACCESS_TOKEN pToken = NULL;
     ULONG ulOffset = 0;
     PBYTE pBuffer = (PBYTE) pRelative;
     PSID pSid = NULL;
@@ -1045,29 +1048,30 @@ RtlSelfRelativeAccessTokenToAccessToken(
     ULONG ulSize = 0;
     ULONG ulRealSize = 0;
     ULONG i = 0;
-
-    status = RTL_ALLOCATE(&pToken, ACCESS_TOKEN, sizeof(*pToken));
-    GOTO_CLEANUP_ON_STATUS(status);
-
-    pToken->ReferenceCount = 1;
+    TOKEN_USER User = {{0}};
+    TOKEN_OWNER Owner = {0};
+    TOKEN_PRIMARY_GROUP PrimaryGroup = {0};
+    TOKEN_UNIX Unix = {0};
+    PTOKEN_GROUPS pTokenGroups = NULL;
+    TOKEN_DEFAULT_DACL DefaultDacl = {0};
 
     status = CheckOffset(0, sizeof(*pRelative), ulRelativeSize);
     GOTO_CLEANUP_ON_STATUS(status);
 
-    pToken->Flags = pRelative->Flags;
-    pToken->User.Attributes = pRelative->User.Attributes;
-    pToken->GroupCount = pRelative->GroupCount;
-    pToken->Uid = pRelative->Uid;
-    pToken->Gid = pRelative->Gid;
-    pToken->Umask = pRelative->Umask;
+    if (pRelative->Flags & ACCESS_TOKEN_FLAG_UNIX_PRESENT)
+    {
+        Unix.Uid = pRelative->Uid;
+        Unix.Gid = pRelative->Gid;
+        Unix.Umask = pRelative->Umask;
+    }
 
+    User.User.Attributes = pRelative->User.Attributes;
     ulOffset = pRelative->User.SidOffset;
     pSid = (PSID) (pBuffer + ulOffset);
     status = RtlValidateSelfRelativeSid(pSid, ulOffset, ulRelativeSize);
     GOTO_CLEANUP_ON_STATUS(status);
-
-    status = RtlDuplicateSid(&pToken->User.Sid, pSid);
-    GOTO_CLEANUP_ON_STATUS(status);
+    
+    User.User.Sid = pSid;
 
     ulOffset = pRelative->GroupsOffset;
     if (ulOffset)
@@ -1084,25 +1088,32 @@ RtlSelfRelativeAccessTokenToAccessToken(
             pRelative->GroupCount);
         GOTO_CLEANUP_ON_STATUS(status);
 
+        status = LwRtlSafeAddULONG(
+            &ulRealSize,
+            ulRealSize,
+            sizeof(TOKEN_GROUPS));
+        GOTO_CLEANUP_ON_STATUS(status);
+
         status = CheckOffset(ulOffset, ulSize, ulRelativeSize);
         GOTO_CLEANUP_ON_STATUS(status);
 
         pGroups = (PSID_AND_ATTRIBUTES_SELF_RELATIVE) (pBuffer + ulOffset);
-
-        status = RTL_ALLOCATE(&pToken->Groups, SID_AND_ATTRIBUTES, ulRealSize);
+        
+        status = RTL_ALLOCATE(&pTokenGroups, TOKEN_GROUPS, ulRealSize);
         GOTO_CLEANUP_ON_STATUS(status);
+        
+        pTokenGroups->GroupCount = pRelative->GroupCount;
 
         for (i = 0; i < pRelative->GroupCount; i++)
         {
-            pToken->Groups[i].Attributes = pGroups[i].Attributes;
-
+            pTokenGroups->Groups[i].Attributes = pGroups[i].Attributes;
+            
             ulOffset = pGroups[i].SidOffset;
             pSid = (PSID) (pBuffer + ulOffset);
             status = RtlValidateSelfRelativeSid(pSid, ulOffset, ulRelativeSize);
             GOTO_CLEANUP_ON_STATUS(status);
-
-            status = RtlDuplicateSid(&pToken->Groups[i].Sid, pSid);
-            GOTO_CLEANUP_ON_STATUS(status);
+            
+            pTokenGroups->Groups[i].Sid = pSid;
         }
     }
 
@@ -1112,9 +1123,8 @@ RtlSelfRelativeAccessTokenToAccessToken(
         pSid = (PSID) (pBuffer + ulOffset);
         status = RtlValidateSelfRelativeSid(pSid, ulOffset, ulRelativeSize);
         GOTO_CLEANUP_ON_STATUS(status);
-
-        status = RtlDuplicateSid(&pToken->Owner, pSid);
-        GOTO_CLEANUP_ON_STATUS(status);
+    
+        Owner.Owner = pSid;
     }
 
     ulOffset = pRelative->PrimaryGroupOffset;
@@ -1123,18 +1133,27 @@ RtlSelfRelativeAccessTokenToAccessToken(
         pSid = (PSID) (pBuffer + ulOffset);
         status = RtlValidateSelfRelativeSid(pSid, ulOffset, ulRelativeSize);
         GOTO_CLEANUP_ON_STATUS(status);
-
-        status = RtlDuplicateSid(&pToken->PrimaryGroup, pSid);
-        GOTO_CLEANUP_ON_STATUS(status);
+        
+        PrimaryGroup.PrimaryGroup = pSid;
     }
 
-    *ppToken = pToken;
+    status = RtlCreateAccessToken(
+        ppToken,
+        &User,
+        pTokenGroups,
+        &Owner,
+        &PrimaryGroup,
+        &DefaultDacl,
+        pRelative->Flags & ACCESS_TOKEN_FLAG_UNIX_PRESENT ? &Unix : NULL);
+    GOTO_CLEANUP_ON_STATUS(status);
 
 cleanup:
 
+    RTL_FREE(&pTokenGroups);
+
     if (!NT_SUCCESS(status))
     {
-        RtlReleaseAccessToken(&pToken);
+        *ppToken = NULL;
     }
 
     return status;
