@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -51,6 +51,7 @@ TaskDelete(
     PEPOLL_TASK pTask
     )
 {
+    RTL_FREE(&pTask->pUnixSignal);
     RtlMemoryFree(pTask);
 }
 
@@ -134,7 +135,7 @@ RunTask(
         &llNewTime);
 
     /* Clear event arguments except sticky bits */
-    pTask->EventArgs &= LW_TASK_EVENT_CANCEL;
+    pTask->EventArgs &= STICKY_EVENTS;
 
     /* If the function gave us a valid time, update the task deadline */
     if (llNewTime != 0)
@@ -161,45 +162,45 @@ UpdateEventWait(
     __uint32_t events = 0;
     struct epoll_event event;
 
-    if (pTask->EventWait != pTask->EventLastWait)
+    if ((pTask->EventWait & FD_EVENTS) != (pTask->EventLastWait & FD_EVENTS) && pTask->Fd >= 0)
     {
         if (pTask->EventWait & LW_TASK_EVENT_FD_READABLE)
         {
             events |= EPOLLIN;
         }
-
+        
         if (pTask->EventWait & LW_TASK_EVENT_FD_WRITABLE)
         {
             events |= EPOLLOUT;
         }
-
+        
         if (pTask->EventWait & LW_TASK_EVENT_FD_EXCEPTION)
         {
             events |= EPOLLERR;
         }
+        
 
-        if (events)
+        memset(&event, 0, sizeof(event));
+
+        event.events = events | EPOLLET;
+        event.data.ptr = pTask;
+
+        if (epoll_ctl(EpollFd, EPOLL_CTL_MOD, pTask->Fd, &event) < 0)
         {
-            event.events = events | EPOLLET;
-            event.data.ptr = pTask;
-
-            if (epoll_ctl(EpollFd, EPOLL_CTL_MOD, pTask->Fd, &event) < 0)
-            {
-                status = LwErrnoToNtStatus(errno);
-                GOTO_ERROR_ON_STATUS(status);
-            }
+            status = LwErrnoToNtStatus(errno);
+            GOTO_ERROR_ON_STATUS(status);
         }
-
-        pTask->EventLastWait = pTask->EventWait;
     }
 
+    pTask->EventLastWait = pTask->EventWait;
+
 error:
 
     return status;
 }
 
 /*
- * Updates the set events on tasks from epoll results and
+ * Updates the event args on tasks from epoll results and
  * schedules them to run.
  */
 static
@@ -233,12 +234,12 @@ ScheduleWaitingTasks(
         {
             pTask->EventArgs |= LW_TASK_EVENT_FD_READABLE;
         }
-
+        
         if (pEvents[i].events & EPOLLOUT)
         {
             pTask->EventArgs |= LW_TASK_EVENT_FD_WRITABLE;
         }
-
+      
         if ((pEvents[i].events & EPOLLERR) &&
             (pTask->EventWait & LW_TASK_EVENT_FD_EXCEPTION))
         {
@@ -252,7 +253,7 @@ ScheduleWaitingTasks(
         {
             pTask->EventArgs |= LW_TASK_EVENT_TIME;
         }
-
+        
         /* Schedule task to run if it has been triggered */
         if (pTask->EventWait & pTask->EventArgs)
         {
@@ -278,7 +279,7 @@ ScheduleTimedTasks(
     {
         pNext = pRing->pNext;
         pTask = LW_STRUCT_FROM_FIELD(pRing, EPOLL_TASK, QueueRing);
-
+        
         /* No more tasks in the queue are past the deadline
            since the queue is sorted */
         if (pTask->llDeadline > llNow)
@@ -288,7 +289,7 @@ ScheduleTimedTasks(
 
         RingRemove(&pTask->QueueRing);
         RingEnqueue(pRunnable, &pTask->QueueRing);
-
+        
         pTask->EventArgs |= LW_TASK_EVENT_TIME;
     }
 }
@@ -307,7 +308,7 @@ InsertTimedQueue(
     for (pRing = pTimed->pNext; pRing != pTimed; pRing = pRing->pNext)
     {
         pTask = LW_STRUCT_FROM_FIELD(pRing, EPOLL_TASK, QueueRing);
-
+     
         if (pTask->llDeadline > pInsert->llDeadline)
             break;
     }
@@ -334,7 +335,7 @@ Poll(
 
     do
     {
-        if (llNextDeadline != 0)
+        if (llNextDeadline >= 0)
         {
             /* Convert to timeout in milliseconds */
             timeout = (llNextDeadline - *pllNow) / 1000000ll;
@@ -386,7 +387,7 @@ ProcessRunnable(
     PLW_TASK_GROUP pGroup = NULL;
     PRING pRing = NULL;
     PRING pNext = NULL;
-
+    
     /* We are guaranteed to run each task at least once.  If tasks remain
        on the runnable list by yielding, we will continue to run them
        all in a round robin until our ticks are depleted. */
@@ -395,44 +396,41 @@ ProcessRunnable(
         for (pRing = pRunnable->pNext; pRing != pRunnable; pRing = pNext)
         {
             pNext = pRing->pNext;
-
+            
             pTask = LW_STRUCT_FROM_FIELD(pRing, EPOLL_TASK, QueueRing);
-
+            
             RunTask(pTask, llNow);
 
             if (ulTicks)
             {
                 ulTicks--;
             }
-
-            if (pTask->EventWait & LW_TASK_EVENT_YIELD)
+            
+            if (pTask->EventWait != LW_TASK_EVENT_COMPLETE)
             {
-                /* Task is yielding.  Set its trigger arguments
-                   to whatever it gave us (modulo INIT and CANCEL).
-                   Leave it on the runnable list for the next iteration. */
-                pTask->EventArgs =
-                    (pTask->EventWait & ~LW_TASK_EVENT_INIT) |
-                    (pTask->EventArgs & LW_TASK_EVENT_CANCEL);
-            }
-            else if (pTask->EventWait != LW_TASK_EVENT_COMPLETE)
-            {
-                /* Task is still waiting to be runnable, put in back in epoll set */
+                /* Task is still waiting to be runnable, update events in epoll set */
                 status = UpdateEventWait(
                     pTask,
                     pThread->EpollFd
                     );
                 GOTO_ERROR_ON_STATUS(status);
-
-                RingRemove(&pTask->QueueRing);
-
-                if (pTask->EventWait & LW_TASK_EVENT_TIME)
+                
+                if (pTask->EventWait & LW_TASK_EVENT_YIELD)
+                {
+                    /* Task is yielding.  Set YIELD in its trigger arguments and
+                       and leave it on the runnable list for the next iteration */
+                    pTask->EventArgs |= LW_TASK_EVENT_YIELD;
+                }                
+                else if (pTask->EventWait & LW_TASK_EVENT_TIME)
                 {
                     /* If the task is waiting for a timeout, insert it into the timed queue */
+                    RingRemove(&pTask->QueueRing);
                     InsertTimedQueue(pTimed, pTask);
                 }
                 else
                 {
                     /* Otherwise, put it in the generic waiting queue */
+                    RingRemove(&pTask->QueueRing);
                     RingEnqueue(pWaiting, &pTask->QueueRing);
                 }
             }
@@ -448,10 +446,16 @@ ProcessRunnable(
                     GOTO_ERROR_ON_STATUS(status);
                 }
 
+                /* Unsubscribe task from any UNIX signals */
+                if (pTask->pUnixSignal)
+                {
+                    RegisterTaskUnixSignal(pTask, 0, FALSE);
+                }
+
                 LOCK_POOL(pThread->pPool);
                 pThread->ulLoad--;
                 UNLOCK_POOL(pThread->pPool);
-
+                
                 pGroup = pTask->pGroup;
 
                 /* If task was in a task group, remove it and notify anyone waiting
@@ -464,7 +468,7 @@ ProcessRunnable(
                     pthread_cond_broadcast(&pGroup->Event);
                     UNLOCK_GROUP(pGroup);
                 }
-
+                
                 LOCK_THREAD(pThread);
                 if (--pTask->ulRefCount)
                 {
@@ -503,25 +507,25 @@ ScheduleSignalled(
     PLW_TASK pTask = NULL;
     char c = 0;
     int res = 0;
-
+    
     LOCK_THREAD(pThread);
 
     if (pThread->bSignalled)
     {
         pThread->bSignalled = FALSE;
-
+        
         res = read(pThread->SignalFds[0], &c, sizeof(c));
         assert(res == sizeof(c));
-
+        
         /* Add all signalled tasks to the runnable list */
         for (pRing = pThread->Tasks.pNext; pRing != &pThread->Tasks; pRing = pNext)
         {
             pNext = pRing->pNext;
             pTask = LW_STRUCT_FROM_FIELD(pRing, EPOLL_TASK, SignalRing);
-
+            
             RingRemove(&pTask->SignalRing);
             RingRemove(&pTask->QueueRing);
-
+            
             if (pTask->EventSignal != TASK_COMPLETE_MASK)
             {
                 RingEnqueue(pRunnable, &pTask->QueueRing);
@@ -530,7 +534,7 @@ ScheduleSignalled(
                 pTask->EventSignal = 0;
             }
         }
-
+        
         if (pThread->bShutdown && !*pbShutdown)
         {
             *pbShutdown = pThread->bShutdown;
@@ -539,7 +543,7 @@ ScheduleSignalled(
 
     UNLOCK_THREAD(pThread);
 }
-
+    
 
 static
 NTSTATUS
@@ -619,7 +623,7 @@ EventLoop(
         else if (!RingIsEmpty(&waiting) || !bShutdown)
         {
             /* There are waiting tasks or we are not shutting down, so poll indefinitely */
-            llNextDeadline = 0;
+            llNextDeadline = -1;
         }
         else
         {
@@ -643,7 +647,7 @@ error:
 
     return status;
 }
-
+    
 static
 PVOID
 EventThread(
@@ -681,6 +685,7 @@ LwRtlCreateTask(
     RingInit(&pTask->GroupRing);
     RingInit(&pTask->QueueRing);
     RingInit(&pTask->SignalRing);
+
     pTask->pGroup = pGroup;
     pTask->ulRefCount = 2;
     pTask->pfnFunc = pfnFunc;
@@ -701,24 +706,42 @@ LwRtlCreateTask(
         }
     }
 
-    pThread->ulLoad++;
-
-    UNLOCK_POOL(pPool);
-
     pTask->pThread = pThread;
 
     if (pGroup)
     {
         LOCK_GROUP(pGroup);
+        if (pGroup->bCancelled)
+        {
+            UNLOCK_GROUP(pGroup);
+            UNLOCK_POOL(pPool);
+            status = STATUS_CANCELLED;
+            GOTO_ERROR_ON_STATUS(status);
+        }
         RingInsertBefore(&pGroup->Tasks, &pTask->GroupRing);
         UNLOCK_GROUP(pGroup);
     }
 
+    pThread->ulLoad++;
+
+    UNLOCK_POOL(pPool);
+
     *ppTask = pTask;
 
+cleanup:
+
+    return status;
+
 error:
 
-    return status;
+    if (pTask)
+    {
+        TaskDelete(pTask);
+    }
+
+    *ppTask = NULL;
+
+    goto cleanup;
 }
 
 NTSTATUS
@@ -738,20 +761,29 @@ LwRtlCreateTaskGroup(
     status = LW_RTL_ALLOCATE_AUTO(&pGroup);
     GOTO_ERROR_ON_STATUS(status);
 
+    pGroup->pPool = pPool;
+    RingInit(&pGroup->Tasks);
+
     status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL));
     GOTO_ERROR_ON_STATUS(status);
+    pGroup->bLockInit = TRUE;
 
     status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL));
     GOTO_ERROR_ON_STATUS(status);
-
-    RingInit(&pGroup->Tasks);
-    pGroup->pPool = pPool;
+    pGroup->bEventInit = TRUE;
 
     *ppGroup = pGroup;
 
-error:
+cleanup:
 
     return status;
+
+error:
+
+    LwRtlFreeTaskGroup(&pGroup);
+    *ppGroup = NULL;
+
+    goto cleanup;
 }
 
 VOID
@@ -771,16 +803,29 @@ LwRtlReleaseTask(
             RingRemove(&pTask->SignalRing);
         }
         UNLOCK_THREAD(pTask->pThread);
-
+        
         if (ulRefCount == 0)
         {
             TaskDelete(pTask);
         }
-
+        
         *ppTask = NULL;
     }
 }
 
+VOID
+RetainTask(
+    PLW_TASK pTask
+    )
+{
+    if (pTask)
+    {
+        LOCK_THREAD(pTask->pThread);
+        ++pTask->ulRefCount;
+        UNLOCK_THREAD(pTask->pThread);
+    }
+}
+
 VOID
 LwRtlFreeTaskGroup(
     PLW_TASK_GROUP* ppGroup
@@ -790,8 +835,16 @@ LwRtlFreeTaskGroup(
 
     if (pGroup)
     {
-        pthread_mutex_destroy(&pGroup->Lock);
-        pthread_cond_destroy(&pGroup->Event);
+        if (pGroup->bLockInit)
+        {
+            pthread_mutex_destroy(&pGroup->Lock);
+        }
+
+        if (pGroup->bEventInit)
+        {
+            pthread_cond_destroy(&pGroup->Event);
+        }
+
         RtlMemoryFree(pGroup);
 
         *ppGroup = NULL;
@@ -829,7 +882,7 @@ LwRtlSetTaskFd(
             pTask->Fd = -1;
         }
     }
-    else
+    else 
     {
         if (pTask->Fd >= 0)
         {
@@ -839,6 +892,7 @@ LwRtlSetTaskFd(
         }
 
         pTask->Fd = Fd;
+        pTask->EventLastWait = 0;
 
         if (epoll_ctl(pTask->pThread->EpollFd, EPOLL_CTL_ADD, pTask->Fd, &event) < 0)
         {
@@ -901,6 +955,90 @@ LwRtlWakeTask(
     UNLOCK_THREAD(pTask->pThread);
 }
 
+LW_NTSTATUS
+LwRtlSetTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_IN int Sig,
+    LW_IN LW_BOOLEAN bSubscribe
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+
+    if (bSubscribe && !pTask->pUnixSignal)
+    {
+        status = LW_RTL_ALLOCATE_AUTO(&pTask->pUnixSignal);
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
+    status = RegisterTaskUnixSignal(pTask, Sig, bSubscribe);
+    GOTO_ERROR_ON_STATUS(status);
+
+error:
+
+    return status;
+}
+
+void
+NotifyTaskUnixSignal(
+    PLW_TASK pTask,
+    siginfo_t* pInfo
+    )
+{
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->EventSignal != TASK_COMPLETE_MASK)
+    {
+        while (pTask->pUnixSignal->si_signo)
+        {
+            pthread_cond_wait(&pTask->pThread->Event, &pTask->pThread->Lock);
+            if (pTask->EventSignal == TASK_COMPLETE_MASK)
+            {
+                goto cleanup;
+            }
+        }
+
+        *pTask->pUnixSignal = *pInfo;
+        pTask->EventSignal |= LW_TASK_EVENT_UNIX_SIGNAL;
+        RingRemove(&pTask->SignalRing);
+        RingEnqueue(&pTask->pThread->Tasks, &pTask->SignalRing);
+        SignalThread(pTask->pThread);
+    }
+
+cleanup:
+
+    UNLOCK_THREAD(pTask->pThread);
+}
+
+LW_BOOLEAN
+LwRtlNextTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_OUT siginfo_t* pInfo
+    )
+{
+    BOOLEAN bResult = FALSE;
+
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->pUnixSignal == NULL || pTask->pUnixSignal->si_signo == 0)
+    {
+        bResult = FALSE;
+    }
+    else
+    {
+        if (pInfo)
+        {
+            *pInfo = *pTask->pUnixSignal;
+        }
+        pTask->pUnixSignal->si_signo = 0;
+        pthread_cond_broadcast(&pTask->pThread->Event);
+        bResult = TRUE;
+    }
+
+    UNLOCK_THREAD(pTask->pThread);
+
+    return bResult;
+}
+
 VOID
 LwRtlCancelTask(
     PLW_TASK pTask
@@ -971,6 +1109,9 @@ LwRtlCancelTaskGroup(
     PLW_TASK pTask = NULL;
 
     LOCK_GROUP(pGroup);
+
+    pGroup->bCancelled = TRUE;
+
     LockAllThreads(pGroup->pPool);
 
     for (ring = pGroup->Tasks.pNext; ring != &pGroup->Tasks; ring = ring->pNext)
@@ -1010,7 +1151,7 @@ LwRtlWaitTaskGroup(
         for (pRing = pGroup->Tasks.pNext; !bStillAlive && pRing != &pGroup->Tasks; pRing = pRing->pNext)
         {
             pTask = LW_STRUCT_FROM_FIELD(pRing, EPOLL_TASK, GroupRing);
-
+         
             if (pTask->EventSignal != TASK_COMPLETE_MASK)
             {
                 bStillAlive = TRUE;
@@ -1034,12 +1175,11 @@ InitEventThread(
     PEPOLL_POOL pPool,
     PLW_THREAD_POOL_ATTRIBUTES pAttrs,
     PEPOLL_THREAD pThread,
-    LONG lCpu
+    ULONG ulCpu
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
     struct epoll_event event;
-    cpu_set_t cpuSet;
     pthread_attr_t threadAttr;
     BOOLEAN bThreadAttrInit = FALSE;
 
@@ -1047,8 +1187,6 @@ InitEventThread(
     GOTO_ERROR_ON_STATUS(status);
     bThreadAttrInit = TRUE;
 
-    CPU_ZERO(&cpuSet);
-
     pThread->pPool = pPool;
 
     status = LwErrnoToNtStatus(pthread_mutex_init(&pThread->Lock, NULL));
@@ -1074,10 +1212,12 @@ InitEventThread(
 
     SetCloseOnExec(pThread->EpollFd);
 
+    memset(&event, 0, sizeof(event));
+
     /* Add signal fd to epoll set */
     event.events = EPOLLIN;
     event.data.ptr = NULL;
-
+    
     if (epoll_ctl(pThread->EpollFd, EPOLL_CTL_ADD, pThread->SignalFds[0], &event) < 0)
     {
         status = LwErrnoToNtStatus(errno);
@@ -1086,17 +1226,8 @@ InitEventThread(
 
     RingInit(&pThread->Tasks);
 
-    if (lCpu >= 0)
-    {
-        CPU_SET((int) lCpu, &cpuSet);
-
-        status = LwErrnoToNtStatus(
-            pthread_attr_setaffinity_np(
-                &threadAttr,
-                sizeof(cpuSet),
-                &cpuSet));
-        GOTO_ERROR_ON_STATUS(status);
-    }
+    status = LwRtlSetAffinityThreadAttribute(&threadAttr, ulCpu);
+    GOTO_ERROR_ON_STATUS(status);
 
     if (pAttrs && pAttrs->ulTaskThreadStackSize)
     {
@@ -1161,19 +1292,14 @@ LwRtlCreateThreadPool(
 
     status = LW_RTL_ALLOCATE_AUTO(&pPool);
     GOTO_ERROR_ON_STATUS(status);
-
+    
     status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
     status = LwErrnoToNtStatus(pthread_cond_init(&pPool->Event, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
-    numCpus = sysconf(_SC_NPROCESSORS_ONLN);
-
-    if (numCpus < 0)
-    {
-        numCpus = 1;
-    }
+    numCpus = LwRtlGetCpuCount();
 
     if (GetDelegateAttr(pAttrs))
     {
@@ -1183,17 +1309,17 @@ LwRtlCreateThreadPool(
     else
     {
         pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
-
+        
         if (pPool->ulEventThreadCount)
         {
             status = LW_RTL_ALLOCATE_ARRAY_AUTO(
                 &pPool->pEventThreads,
                 pPool->ulEventThreadCount);
             GOTO_ERROR_ON_STATUS(status);
-
+            
             for (i = 0; i < pPool->ulEventThreadCount; i++)
             {
-                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
+                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], i % numCpus);
                 GOTO_ERROR_ON_STATUS(status);
             }
         }
@@ -1201,7 +1327,7 @@ LwRtlCreateThreadPool(
 
     status = InitWorkThreads(&pPool->WorkThreads, pAttrs, numCpus);
     GOTO_ERROR_ON_STATUS(status);
-
+    
     *ppPool = pPool;
 
 cleanup:
@@ -1230,7 +1356,7 @@ LwRtlFreeThreadPool(
         pPool->bShutdown = TRUE;
         pthread_cond_broadcast(&pPool->Event);
         UNLOCK_POOL(pPool);
-
+        
         if (pPool->pEventThreads)
         {
             for (i = 0; i < pPool->ulEventThreadCount; i++)
@@ -1243,17 +1369,19 @@ LwRtlFreeThreadPool(
                 pthread_join(pThread->Thread, NULL);
                 DestroyEventThread(pThread);
             }
-
+            
             RtlMemoryFree(pPool->pEventThreads);
         }
-
+        
         if (pPool->pDelegate)
         {
             ReleaseDelegatePool(&pPool->pDelegate);
         }
 
-        pthread_mutex_destroy(&pPool->Lock);
+        pthread_cond_destroy(&pPool->Event);
 
+        pthread_mutex_destroy(&pPool->Lock);
+        
         DestroyWorkThreads(&pPool->WorkThreads);
 
         RtlMemoryFree(pPool);
@@ -1262,16 +1390,38 @@ LwRtlFreeThreadPool(
     }
 }
 
-NTSTATUS
-LwRtlQueueWorkItem(
-    PLW_THREAD_POOL pPool,
-    LW_WORK_ITEM_FUNCTION pfnFunc,
-    PVOID pContext,
-    LW_WORK_ITEM_FLAGS Flags
+LW_NTSTATUS
+LwRtlCreateWorkItem(
+    LW_IN PLW_THREAD_POOL pPool,
+    LW_OUT PLW_WORK_ITEM* ppWorkItem,
+    LW_IN LW_WORK_ITEM_FUNCTION pfnFunc,
+    LW_IN PVOID pContext
     )
 {
-    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
+    return CreateWorkItem(&pPool->WorkThreads, ppWorkItem, pfnFunc, pContext);
 }
 
+LW_VOID
+LwRtlFreeWorkItem(
+    LW_IN LW_OUT PLW_WORK_ITEM* ppWorkItem
+    )
+{
+    FreeWorkItem(ppWorkItem);
+}
 
+LW_VOID
+LwRtlScheduleWorkItem(
+    LW_IN PLW_WORK_ITEM pWorkItem,
+    LW_IN LW_SCHEDULE_FLAGS Flags
+    )
+{
+    ScheduleWorkItem(NULL, pWorkItem, Flags);
+}
 
+LW_VOID
+LwRtlWaitWorkItems(
+    LW_IN PLW_THREAD_POOL pPool
+    )
+{
+    WaitWorkItems(&pPool->WorkThreads);
+}
