@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -88,6 +88,8 @@ typedef struct _LW_TASK
     PVOID pFuncContext;
     /* File descriptor for fd-based events (owned by thread) */
     int Fd;
+    /* Pending UNIX signal (protected by thread lock) */
+    siginfo_t* pUnixSignal;
     /* Link to siblings in task group (protected by group lock) */
     RING GroupRing;
     /* Link to siblings in scheduler queue (owned by thread) */
@@ -102,6 +104,9 @@ typedef struct _LW_TASK_GROUP
     RING Tasks;
     pthread_mutex_t Lock;
     pthread_cond_t Event;
+    unsigned bCancelled:1;
+    unsigned bLockInit:1;
+    unsigned bEventInit:1;
 } EPOLL_TASK_GROUP, *PEPOLL_TASK_GROUP;
 
 typedef struct _LW_THREAD_POOL
