@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -39,14 +39,9 @@
 #include "includes.h"
 #include "threadpool-kqueue.h"
 
-/* FIXME: Make these tweakable on the thread pool itself */
-
+/* FIXME: Make these configurable via thread pool attributes */
 /* Maximum events to read */
 #define MAX_EVENTS 500
-/* Number of event-processing threads */
-#define EVENT_THREAD_COUNT 8
-/* Number of work item-processing threads */
-#define WORK_THREAD_COUNT 8
 /* Maximum number of ticks (task function invocations) to
    process each iteration of the event loop */
 #define MAX_TICKS 1000
@@ -62,16 +57,15 @@ ClearCommands(
 
 static
 NTSTATUS
-AddCommand(
-    PKQUEUE_COMMANDS pCommands,
-    struct kevent** ppEvent
+ExpandCommands(
+    PKQUEUE_COMMANDS pCommands
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
     ULONG ulNewCapacity = 0;
     struct kevent* pNewCommands = NULL;
 
-    if (pCommands->ulCommandCount + 1 > pCommands->ulCommandCapacity)
+    if (pCommands->ulCommandMax + COMMANDS_PER_FD > pCommands->ulCommandCapacity)
     {
         ulNewCapacity = pCommands->ulCommandCapacity;
 
@@ -80,7 +74,10 @@ AddCommand(
             ulNewCapacity = 8;
         }
 
-        ulNewCapacity *= 2;
+        while (pCommands->ulCommandMax > ulNewCapacity)
+        {
+            ulNewCapacity *= 2;
+        }
 
         pNewCommands = LwRtlMemoryRealloc(
             pCommands->pCommands,
@@ -95,19 +92,41 @@ AddCommand(
         pCommands->pCommands = pNewCommands;
     }
 
-    *ppEvent = &pCommands->pCommands[pCommands->ulCommandCount++];
+    pCommands->ulCommandMax += COMMANDS_PER_FD;
 
 error:
 
     return status;
 }
 
+static
+VOID
+ShrinkCommands(
+    PKQUEUE_COMMANDS pCommands
+    )
+{
+    pCommands->ulCommandMax -= COMMANDS_PER_FD;
+}
+
+static
+VOID
+AddCommand(
+    PKQUEUE_COMMANDS pCommands,
+    struct kevent** ppEvent
+    )
+{
+    assert(pCommands->ulCommandCount < pCommands->ulCommandCapacity);
+
+    *ppEvent = &pCommands->pCommands[pCommands->ulCommandCount++];
+}
+
 static
 VOID
 TaskDelete(
     PKQUEUE_TASK pTask
     )
 {
+    RTL_FREE(&pTask->pUnixSignal);
     RtlMemoryFree(pTask);
 }
 
@@ -191,7 +210,7 @@ RunTask(
         &llNewTime);
 
     /* Clear event arguments except sticky bits */
-    pTask->EventArgs &= LW_TASK_EVENT_CANCEL;
+    pTask->EventArgs &= STICKY_EVENTS;
 
     /* If the function gave us a valid time, update the task deadline */
     if (llNewTime != 0)
@@ -208,53 +227,74 @@ RunTask(
  * Updates the kqueue command set with the events a task is waiting on.
  */
 static
-NTSTATUS
+VOID
 UpdateEventWait(
     PKQUEUE_COMMANDS pCommands,
     PKQUEUE_TASK pTask
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
     struct kevent* pEvent = NULL;
 
-    if (pTask->EventWait != pTask->EventLastWait)
+    if (pTask->Fd >= 0)
     {
-        if (pTask->EventWait & LW_TASK_EVENT_FD_READABLE)
+        if ((pTask->EventWait & LW_TASK_EVENT_FD_READABLE) && 
+            !(pTask->EventLastWait & LW_TASK_EVENT_FD_READABLE))
         {
-            status = AddCommand(pCommands, &pEvent);
-            GOTO_ERROR_ON_STATUS(status);
+            AddCommand(pCommands, &pEvent);
             EV_SET(
                 pEvent,
                 pTask->Fd,
                 EVFILT_READ,
-                EV_ADD | EV_CLEAR,
+                EV_ENABLE,
                 0,
                 0,
                 pTask);
         }
-
-        if (pTask->EventWait & LW_TASK_EVENT_FD_WRITABLE)
+        else if ((pTask->EventLastWait & LW_TASK_EVENT_FD_READABLE) &&
+            !(pTask->EventWait & LW_TASK_EVENT_FD_READABLE))
         {
-            status = AddCommand(pCommands, &pEvent);
-            GOTO_ERROR_ON_STATUS(status);
+            AddCommand(pCommands, &pEvent);
+            EV_SET(
+                pEvent,
+                pTask->Fd,
+                EVFILT_READ,
+                EV_DISABLE,
+                0,
+                0,
+                pTask);
+        }
+        
+        if ((pTask->EventWait & LW_TASK_EVENT_FD_WRITABLE) &&
+            !(pTask->EventLastWait & LW_TASK_EVENT_FD_WRITABLE))
+        {
+            AddCommand(pCommands, &pEvent);
             EV_SET(
                 pEvent,
                 pTask->Fd,
                 EVFILT_WRITE,
-                EV_ADD | EV_CLEAR,
+                EV_ENABLE,
                 0,
                 0,
                 pTask);
         }
-
+        else if ((pTask->EventLastWait & LW_TASK_EVENT_FD_WRITABLE) &&
+            !(pTask->EventWait & LW_TASK_EVENT_FD_WRITABLE))
+        {
+            AddCommand(pCommands, &pEvent);
+            EV_SET(
+                pEvent,
+                pTask->Fd,
+                EVFILT_WRITE,
+                EV_DISABLE,
+                0,
+                0,
+                pTask);
+        }
+        
         /* FIXME: error? */
 
         pTask->EventLastWait = pTask->EventWait;
     }
-
-error:
-
-    return status;
 }
 
 /*
@@ -292,13 +332,13 @@ ScheduleWaitingTasks(
         {
             pTask->EventArgs |= LW_TASK_EVENT_FD_READABLE;
         }
-
+        
         if (pEvents[i].filter == EVFILT_WRITE &&
             (pTask->EventWait & LW_TASK_EVENT_FD_WRITABLE))
         {
             pTask->EventArgs |= LW_TASK_EVENT_FD_WRITABLE;
         }
-
+      
         /* FIXME: errors? */
 
         /* If the task's deadline has expired, set the time event bit */
@@ -308,7 +348,7 @@ ScheduleWaitingTasks(
         {
             pTask->EventArgs |= LW_TASK_EVENT_TIME;
         }
-
+        
         /* Schedule task to run if it has been triggered */
         if (pTask->EventWait & pTask->EventArgs)
         {
@@ -334,7 +374,7 @@ ScheduleTimedTasks(
     {
         pNext = pRing->pNext;
         pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, QueueRing);
-
+        
         /* No more tasks in the queue are past the deadline
            since the queue is sorted */
         if (pTask->llDeadline > llNow)
@@ -344,7 +384,7 @@ ScheduleTimedTasks(
 
         RingRemove(&pTask->QueueRing);
         RingEnqueue(pRunnable, &pTask->QueueRing);
-
+        
         pTask->EventArgs |= LW_TASK_EVENT_TIME;
     }
 }
@@ -363,7 +403,7 @@ InsertTimedQueue(
     for (pRing = pTimed->pNext; pRing != pTimed; pRing = pRing->pNext)
     {
         pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, QueueRing);
-
+     
         if (pTask->llDeadline > pInsert->llDeadline)
             break;
     }
@@ -392,7 +432,7 @@ Poll(
 
     do
     {
-        if (llNextDeadline != 0)
+        if (llNextDeadline >= 0)
         {
             /* Convert to timeout structure */
             llTimeDiff = llNextDeadline - *pllNow;
@@ -415,7 +455,7 @@ Poll(
             pCommands->ulCommandCount,
             pEvents,
             maxEvents,
-            llNextDeadline != 0 ? &timeout : NULL);
+            llNextDeadline >= 0 ? &timeout : NULL);
 
         if (ready < 0 && errno == EINTR)
         {
@@ -442,7 +482,7 @@ error:
 }
 
 static
-NTSTATUS
+VOID
 ProcessRunnable(
     PKQUEUE_THREAD pThread,
     PKQUEUE_COMMANDS pCommands,
@@ -452,14 +492,12 @@ ProcessRunnable(
     LONG64 llNow
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
     ULONG ulTicks = MAX_TICKS;
     PLW_TASK pTask = NULL;
     PLW_TASK_GROUP pGroup = NULL;
     PRING pRing = NULL;
     PRING pNext = NULL;
-    struct kevent* pEvent = NULL;
-
+    
     /* We are guaranteed to run each task at least once.  If tasks remain
        on the runnable list by yielding, we will continue to run them
        all in a round robin until our ticks are depleted. */
@@ -468,42 +506,42 @@ ProcessRunnable(
         for (pRing = pRunnable->pNext; pRing != pRunnable; pRing = pNext)
         {
             pNext = pRing->pNext;
-
+            
             pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, QueueRing);
-
+            
             RunTask(pTask, llNow);
 
             if (ulTicks)
             {
                 ulTicks--;
             }
-
-            if (pTask->EventWait & LW_TASK_EVENT_YIELD)
+            
+            if (pTask->EventWait != LW_TASK_EVENT_COMPLETE)
             {
-                /* Task is yielding.  Set its trigger arguments
-                   to whatever it gave us (modulo INIT and CANCEL).
-                   Leave it on the runnable list for the next iteration. */
-                pTask->EventArgs =
-                    (pTask->EventWait & ~LW_TASK_EVENT_INIT) |
-                    (pTask->EventArgs & LW_TASK_EVENT_CANCEL);
-            }
-            else if (pTask->EventWait != LW_TASK_EVENT_COMPLETE)
-            {
-                /* Task is still waiting to be runnable, put in back in kqueue set */
-                status = UpdateEventWait(pCommands, pTask);
-                GOTO_ERROR_ON_STATUS(status);
-
-                RingRemove(&pTask->QueueRing);
-
-                if (pTask->EventWait & LW_TASK_EVENT_TIME)
+                if (pTask->EventWait & LW_TASK_EVENT_YIELD)
                 {
-                    /* If the task is waiting for a timeout, insert it into the timed queue */
-                    InsertTimedQueue(pTimed, pTask);
-                }
-                else
+                    /* Task is yielding.  Set the YIELD flag and
+                       leave it on the runnable list for the next iteration. */
+                    pTask->EventArgs |= LW_TASK_EVENT_YIELD;
+                }   
+                else 
                 {
-                    /* Otherwise, put it in the generic waiting queue */
-                    RingEnqueue(pWaiting, &pTask->QueueRing);
+                    /* Task is still waiting on events, update kqueue */
+                    UpdateEventWait(pCommands, pTask);
+
+                    if (pTask->EventWait & LW_TASK_EVENT_TIME)
+                    {
+                        /* If the task is waiting for a timeout, 
+                           insert it into the timed queue */
+                        RingRemove(&pTask->QueueRing);
+                        InsertTimedQueue(pTimed, pTask);
+                    }
+                    else
+                    {
+                        /* Otherwise, put it in the generic waiting queue */
+                        RingRemove(&pTask->QueueRing);
+                        RingEnqueue(pWaiting, &pTask->QueueRing);
+                    }
                 }
             }
             else
@@ -514,39 +552,19 @@ ProcessRunnable(
                 /* Remove any associated events from the kqueue */
                 if (pTask->Fd >= 0)
                 {
-                    if (pTask->EventLastWait & LW_TASK_EVENT_FD_READABLE)
-                    {
-                        status = AddCommand(pCommands, &pEvent);
-                        GOTO_ERROR_ON_STATUS(status);
-                        EV_SET(
-                            pEvent,
-                            pTask->Fd,
-                            EVFILT_READ,
-                            EV_DELETE,
-                            0,
-                            0,
-                            0);
-                    }
+                    (void) LwRtlSetTaskFd(pTask, pTask->Fd, 0);
+                }
 
-                    if (pTask->EventLastWait & LW_TASK_EVENT_FD_WRITABLE)
-                    {
-                        status = AddCommand(pCommands, &pEvent);
-                        GOTO_ERROR_ON_STATUS(status);
-                        EV_SET(
-                            pEvent,
-                            pTask->Fd,
-                            EVFILT_WRITE,
-                            EV_DELETE,
-                            0,
-                            0,
-                            0);
-                    }
+                /* Unsubscribe task from any UNIX signals */
+                if (pTask->pUnixSignal)
+                {
+                    RegisterTaskUnixSignal(pTask, 0, FALSE);
                 }
 
                 LOCK_POOL(pThread->pPool);
                 pThread->ulLoad--;
                 UNLOCK_POOL(pThread->pPool);
-
+                
                 pGroup = pTask->pGroup;
 
                 /* If task was in a task group, remove it and notify anyone waiting
@@ -559,7 +577,7 @@ ProcessRunnable(
                     pthread_cond_broadcast(&pGroup->Event);
                     UNLOCK_GROUP(pGroup);
                 }
-
+                
                 LOCK_THREAD(pThread);
                 if (--pTask->ulRefCount)
                 {
@@ -580,9 +598,16 @@ ProcessRunnable(
         }
     }
 
-error:
+    /* Update kevent commands for yielding tasks */
+    for (pRing = pRunnable->pNext; pRing != pRunnable; pRing = pRing->pNext)
+    {
+        pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, QueueRing);
 
-    return status;
+        if (pTask->EventArgs & LW_TASK_EVENT_YIELD)
+        {
+            UpdateEventWait(pCommands, pTask);
+        }
+    }
 }
 
 static
@@ -598,25 +623,25 @@ ScheduleSignalled(
     PLW_TASK pTask = NULL;
     char c = 0;
     int res = 0;
-
+    
     LOCK_THREAD(pThread);
 
     if (pThread->bSignalled)
     {
         pThread->bSignalled = FALSE;
-
+        
         res = read(pThread->SignalFds[0], &c, sizeof(c));
         assert(res == sizeof(c));
-
+        
         /* Add all signalled tasks to the runnable list */
         for (pRing = pThread->Tasks.pNext; pRing != &pThread->Tasks; pRing = pNext)
         {
             pNext = pRing->pNext;
             pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, SignalRing);
-
+            
             RingRemove(&pTask->SignalRing);
             RingRemove(&pTask->QueueRing);
-
+            
             if (pTask->EventSignal != TASK_COMPLETE_MASK)
             {
                 RingEnqueue(pRunnable, &pTask->QueueRing);
@@ -625,7 +650,7 @@ ScheduleSignalled(
                 pTask->EventSignal = 0;
             }
         }
-
+        
         if (pThread->bShutdown && !*pbShutdown)
         {
             *pbShutdown = pThread->bShutdown;
@@ -634,7 +659,7 @@ ScheduleSignalled(
 
     UNLOCK_THREAD(pThread);
 }
-
+    
 
 static
 NTSTATUS
@@ -647,13 +672,12 @@ EventLoop(
     RING runnable;
     RING waiting;
     CLOCK clock = {0};
-    LONG64 llNow;
-    LONG64 llNextDeadline;
+    LONG64 llNow = 0;
+    LONG64 llNextDeadline = 0;
     struct kevent events[MAX_EVENTS];
     int ready = 0;
     BOOLEAN bShutdown = FALSE;
     BOOLEAN bSignalled = FALSE;
-    KQUEUE_COMMANDS commands = {0};
 
     RingInit(&runnable);
     RingInit(&timed);
@@ -665,7 +689,7 @@ EventLoop(
         status = ClockGetMonotonicTime(&clock, &llNow);
         GOTO_ERROR_ON_STATUS(status);
 
-        /* Schedule any timed tasks that are have reached their deadline */
+        /* Schedule any timed tasks that have reached their deadlines */
         ScheduleTimedTasks(
             &timed,
             llNow,
@@ -691,14 +715,13 @@ EventLoop(
         }
 
         /* Process runnable tasks */
-        status = ProcessRunnable(
+        ProcessRunnable(
             pThread,
-            &commands,
+            &pThread->Commands,
             &runnable,
             &timed,
             &waiting,
             llNow);
-        GOTO_ERROR_ON_STATUS(status);
 
         if (!RingIsEmpty(&runnable))
         {
@@ -716,7 +739,7 @@ EventLoop(
         else if (!RingIsEmpty(&waiting) || !bShutdown)
         {
             /* There are waiting tasks or we are not shutting down, so poll indefinitely */
-            llNextDeadline = 0;
+            llNextDeadline = -1;
         }
         else
         {
@@ -729,7 +752,7 @@ EventLoop(
             &clock,
             &llNow,
             pThread->KqueueFd,
-            &commands,
+            &pThread->Commands,
             events,
             MAX_EVENTS,
             llNextDeadline,
@@ -739,11 +762,9 @@ EventLoop(
 
 error:
 
-    RTL_FREE(&commands.pCommands);
-
     return status;
 }
-
+    
 static
 PVOID
 EventThread(
@@ -781,6 +802,7 @@ LwRtlCreateTask(
     RingInit(&pTask->GroupRing);
     RingInit(&pTask->QueueRing);
     RingInit(&pTask->SignalRing);
+
     pTask->pGroup = pGroup;
     pTask->ulRefCount = 2;
     pTask->pfnFunc = pfnFunc;
@@ -801,24 +823,42 @@ LwRtlCreateTask(
         }
     }
 
-    pThread->ulLoad++;
-
-    UNLOCK_POOL(pPool);
-
     pTask->pThread = pThread;
 
     if (pGroup)
     {
         LOCK_GROUP(pGroup);
+        if (pGroup->bCancelled)
+        {
+            UNLOCK_GROUP(pGroup);
+            UNLOCK_POOL(pPool);
+            status = STATUS_CANCELLED;
+            GOTO_ERROR_ON_STATUS(status);
+        }
         RingInsertBefore(&pGroup->Tasks, &pTask->GroupRing);
         UNLOCK_GROUP(pGroup);
     }
 
+    pThread->ulLoad++;
+
+    UNLOCK_POOL(pPool);
+
     *ppTask = pTask;
 
+cleanup:
+
+    return status;
+
 error:
 
-    return status;
+    if (pTask)
+    {
+        TaskDelete(pTask);
+    }
+
+    *ppTask = NULL;
+
+    goto cleanup;
 }
 
 NTSTATUS
@@ -838,20 +878,29 @@ LwRtlCreateTaskGroup(
     status = LW_RTL_ALLOCATE_AUTO(&pGroup);
     GOTO_ERROR_ON_STATUS(status);
 
-    status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL));
-    GOTO_ERROR_ON_STATUS(status);
-
-    status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL));
-    GOTO_ERROR_ON_STATUS(status);
-
     RingInit(&pGroup->Tasks);
     pGroup->pPool = pPool;
 
+    status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL));
+    GOTO_ERROR_ON_STATUS(status);
+    pGroup->bLockInit = TRUE;
+
+    status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL));
+    GOTO_ERROR_ON_STATUS(status);
+    pGroup->bEventInit = TRUE;
+
     *ppGroup = pGroup;
 
+cleanup:
+
+    return status;
+
 error:
 
-    return status;
+    LwRtlFreeTaskGroup(&pGroup);
+    *ppGroup = NULL;
+
+    goto cleanup;
 }
 
 VOID
@@ -871,16 +920,29 @@ LwRtlReleaseTask(
             RingRemove(&pTask->SignalRing);
         }
         UNLOCK_THREAD(pTask->pThread);
-
+        
         if (ulRefCount == 0)
         {
             TaskDelete(pTask);
         }
-
+        
         *ppTask = NULL;
     }
 }
 
+VOID
+RetainTask(
+    PLW_TASK pTask
+    )
+{
+    if (pTask)
+    {
+        LOCK_THREAD(pTask->pThread);
+        ++pTask->ulRefCount;
+        UNLOCK_THREAD(pTask->pThread);
+    }
+}
+
 VOID
 LwRtlFreeTaskGroup(
     PLW_TASK_GROUP* ppGroup
@@ -890,8 +952,16 @@ LwRtlFreeTaskGroup(
 
     if (pGroup)
     {
-        pthread_mutex_destroy(&pGroup->Lock);
-        pthread_cond_destroy(&pGroup->Event);
+        if (pGroup->bLockInit)
+        {
+            pthread_mutex_destroy(&pGroup->Lock);
+        }
+
+        if (pGroup->bEventInit)
+        {
+            pthread_cond_destroy(&pGroup->Event);
+        }
+
         RtlMemoryFree(pGroup);
 
         *ppGroup = NULL;
@@ -906,6 +976,8 @@ LwRtlSetTaskFd(
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
+    struct kevent commands[2];
+    int count = 0;
 
     if (Fd < 0)
     {
@@ -917,10 +989,41 @@ LwRtlSetTaskFd(
     {
         if (Mask == 0)
         {
+            if (pTask->bReadEventAdded)
+            {
+                EV_SET(
+                    &commands[count++],
+                    pTask->Fd,
+                    EVFILT_READ,
+                    EV_DELETE,
+                    0,
+                    0,
+                    NULL);
+            }
+
+            if (pTask->bWriteEventAdded)
+            {
+                EV_SET(
+                    &commands[count++],
+                    pTask->Fd,
+                    EVFILT_WRITE,
+                    EV_DELETE,
+                    0,
+                    0,
+                    NULL);
+            }
+
+            if (kevent(pTask->pThread->KqueueFd, commands, count, NULL, 0, NULL) < 0)
+            {
+                status = LwErrnoToNtStatus(errno);
+                GOTO_ERROR_ON_STATUS(status);
+            }
+
             pTask->Fd = -1;
+            ShrinkCommands(&pTask->pThread->Commands);
         }
     }
-    else
+    else 
     {
         if (pTask->Fd >= 0)
         {
@@ -929,7 +1032,45 @@ LwRtlSetTaskFd(
             GOTO_ERROR_ON_STATUS(status);
         }
 
+        status = ExpandCommands(&pTask->pThread->Commands);
+        GOTO_ERROR_ON_STATUS(status);
+
+        /* Pre-add disabled events to kqueue to guarantee memory availability */
+
+        if (Mask & LW_TASK_EVENT_FD_READABLE)
+        {
+            EV_SET(
+                &commands[count++],
+                Fd,
+                EVFILT_READ,
+                EV_ADD | EV_CLEAR | EV_DISABLE,
+                0,
+                0,
+                pTask);
+        }
+
+        if (Mask & LW_TASK_EVENT_FD_WRITABLE)
+        {
+            EV_SET(
+                &commands[count++],
+                Fd,
+                EVFILT_WRITE,
+                EV_ADD | EV_CLEAR | EV_DISABLE,
+                0,
+                0,
+                pTask);
+        }
+
+        if (kevent(pTask->pThread->KqueueFd, commands, count, NULL, 0, NULL) < 0)
+        {
+            status = LwErrnoToNtStatus(errno);
+            GOTO_ERROR_ON_STATUS(status);
+        }
+
         pTask->Fd = Fd;
+        pTask->EventLastWait = 0;
+        pTask->bReadEventAdded = Mask & LW_TASK_EVENT_FD_READABLE ? TRUE : FALSE;
+        pTask->bWriteEventAdded = Mask & LW_TASK_EVENT_FD_WRITABLE ? TRUE : FALSE;
     }
 
 error:
@@ -986,6 +1127,90 @@ LwRtlWakeTask(
     UNLOCK_THREAD(pTask->pThread);
 }
 
+LW_NTSTATUS
+LwRtlSetTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_IN int Sig,
+    LW_IN LW_BOOLEAN bSubscribe
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+
+    if (bSubscribe && !pTask->pUnixSignal)
+    {
+        status = LW_RTL_ALLOCATE_AUTO(&pTask->pUnixSignal);
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
+    status = RegisterTaskUnixSignal(pTask, Sig, bSubscribe);
+    GOTO_ERROR_ON_STATUS(status);
+
+error:
+
+    return status;
+}
+
+void
+NotifyTaskUnixSignal(
+    PLW_TASK pTask,
+    siginfo_t* pInfo
+    )
+{
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->EventSignal != TASK_COMPLETE_MASK)
+    {
+        while (pTask->pUnixSignal->si_signo)
+        {
+            pthread_cond_wait(&pTask->pThread->Event, &pTask->pThread->Lock);
+            if (pTask->EventSignal == TASK_COMPLETE_MASK)
+            {
+                goto cleanup;
+            }
+        }
+
+        *pTask->pUnixSignal = *pInfo;
+        pTask->EventSignal |= LW_TASK_EVENT_UNIX_SIGNAL;
+        RingRemove(&pTask->SignalRing);
+        RingEnqueue(&pTask->pThread->Tasks, &pTask->SignalRing);
+        SignalThread(pTask->pThread);
+    }
+
+cleanup:
+
+    UNLOCK_THREAD(pTask->pThread);
+}
+
+LW_BOOLEAN
+LwRtlNextTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_OUT siginfo_t* pInfo
+    )
+{
+    BOOLEAN bResult = FALSE;
+
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->pUnixSignal == NULL || pTask->pUnixSignal->si_signo == 0)
+    {
+        bResult = FALSE;
+    }
+    else
+    {
+        if (pInfo)
+        {
+            *pInfo = *pTask->pUnixSignal;
+        }
+        pTask->pUnixSignal->si_signo = 0;
+        pthread_cond_broadcast(&pTask->pThread->Event);
+        bResult = TRUE;
+    }
+
+    UNLOCK_THREAD(pTask->pThread);
+
+    return bResult;
+}
+
 VOID
 LwRtlCancelTask(
     PLW_TASK pTask
@@ -1056,6 +1281,9 @@ LwRtlCancelTaskGroup(
     PLW_TASK pTask = NULL;
 
     LOCK_GROUP(pGroup);
+
+    pGroup->bCancelled = TRUE;
+
     LockAllThreads(pGroup->pPool);
 
     for (ring = pGroup->Tasks.pNext; ring != &pGroup->Tasks; ring = ring->pNext)
@@ -1095,7 +1323,7 @@ LwRtlWaitTaskGroup(
         for (pRing = pGroup->Tasks.pNext; !bStillAlive && pRing != &pGroup->Tasks; pRing = pRing->pNext)
         {
             pTask = LW_STRUCT_FROM_FIELD(pRing, KQUEUE_TASK, GroupRing);
-
+         
             if (pTask->EventSignal != TASK_COMPLETE_MASK)
             {
                 bStillAlive = TRUE;
@@ -1119,11 +1347,10 @@ InitEventThread(
     PKQUEUE_POOL pPool,
     PLW_THREAD_POOL_ATTRIBUTES pAttrs,
     PKQUEUE_THREAD pThread,
-    LONG lCpu
+    ULONG ulCpu
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    cpuset_t cpuSet;
     struct kevent event;
     pthread_attr_t threadAttr;
     BOOLEAN bThreadAttrInit = FALSE;
@@ -1132,8 +1359,6 @@ InitEventThread(
     GOTO_ERROR_ON_STATUS(status);
     bThreadAttrInit = TRUE;
 
-    CPU_ZERO(&cpuSet);
-
     pThread->pPool = pPool;
 
     status = LwErrnoToNtStatus(pthread_mutex_init(&pThread->Lock, NULL));
@@ -1161,8 +1386,7 @@ InitEventThread(
 
     /* Add signal fd to kqueue set */
     EV_SET(&event, pThread->SignalFds[0], EVFILT_READ, EV_ADD, 0, 0, NULL);
-
-
+   
     if (kevent(pThread->KqueueFd, &event, 1, NULL, 0, NULL) < 0)
     {
         status = LwErrnoToNtStatus(errno);
@@ -1171,17 +1395,8 @@ InitEventThread(
 
     RingInit(&pThread->Tasks);
 
-    if (lCpu >= 0)
-    {
-        CPU_SET((int) lCpu, &cpuSet);
-
-        status = LwErrnoToNtStatus(
-            pthread_attr_setaffinity_np(
-                &threadAttr,
-                sizeof(cpuSet),
-                &cpuSet));
-        GOTO_ERROR_ON_STATUS(status);
-    }
+    status = LwRtlSetAffinityThreadAttribute(&threadAttr, ulCpu);
+    GOTO_ERROR_ON_STATUS(status);
 
     if (pAttrs && pAttrs->ulTaskThreadStackSize)
     {
@@ -1231,6 +1446,8 @@ DestroyEventThread(
     {
         close(pThread->SignalFds[1]);
     }
+
+    RTL_FREE(&pThread->Commands.pCommands);
 }
 
 NTSTATUS
@@ -1246,19 +1463,14 @@ LwRtlCreateThreadPool(
 
     status = LW_RTL_ALLOCATE_AUTO(&pPool);
     GOTO_ERROR_ON_STATUS(status);
-
+    
     status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
     status = LwErrnoToNtStatus(pthread_cond_init(&pPool->Event, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
-    numCpus = sysconf(_SC_NPROCESSORS_ONLN);
-
-    if (numCpus < 0)
-    {
-        numCpus = 1;
-    }
+    numCpus = LwRtlGetCpuCount();
 
     if (GetDelegateAttr(pAttrs))
     {
@@ -1268,17 +1480,17 @@ LwRtlCreateThreadPool(
     else
     {
         pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
-
+        
         if (pPool->ulEventThreadCount)
         {
             status = LW_RTL_ALLOCATE_ARRAY_AUTO(
                 &pPool->pEventThreads,
                 pPool->ulEventThreadCount);
             GOTO_ERROR_ON_STATUS(status);
-
+            
             for (i = 0; i < pPool->ulEventThreadCount; i++)
             {
-                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
+                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], i % numCpus);
                 GOTO_ERROR_ON_STATUS(status);
             }
         }
@@ -1315,7 +1527,7 @@ LwRtlFreeThreadPool(
         pPool->bShutdown = TRUE;
         pthread_cond_broadcast(&pPool->Event);
         UNLOCK_POOL(pPool);
-
+        
         if (pPool->pEventThreads)
         {
             for (i = 0; i < pPool->ulEventThreadCount; i++)
@@ -1328,17 +1540,19 @@ LwRtlFreeThreadPool(
                 pthread_join(pThread->Thread, NULL);
                 DestroyEventThread(pThread);
             }
-
+            
             RtlMemoryFree(pPool->pEventThreads);
         }
-
+                
         if (pPool->pDelegate)
         {
             ReleaseDelegatePool(&pPool->pDelegate);
         }
 
-        pthread_mutex_destroy(&pPool->Lock);
+        pthread_cond_destroy(&pPool->Event);
 
+        pthread_mutex_destroy(&pPool->Lock);
+        
         DestroyWorkThreads(&pPool->WorkThreads);
 
         RtlMemoryFree(pPool);
@@ -1347,15 +1561,39 @@ LwRtlFreeThreadPool(
     }
 }
 
-NTSTATUS
-LwRtlQueueWorkItem(
-    PLW_THREAD_POOL pPool,
-    LW_WORK_ITEM_FUNCTION pfnFunc,
-    PVOID pContext,
-    LW_WORK_ITEM_FLAGS Flags
+LW_NTSTATUS
+LwRtlCreateWorkItem(
+    LW_IN PLW_THREAD_POOL pPool,
+    LW_OUT PLW_WORK_ITEM* ppWorkItem,
+    LW_IN LW_WORK_ITEM_FUNCTION pfnFunc,
+    LW_IN PVOID pContext
     )
 {
-    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
+    return CreateWorkItem(&pPool->WorkThreads, ppWorkItem, pfnFunc, pContext);
 }
 
+LW_VOID
+LwRtlFreeWorkItem(
+    LW_IN LW_OUT PLW_WORK_ITEM* ppWorkItem
+    )
+{
+    FreeWorkItem(ppWorkItem);
+}
+
+LW_VOID
+LwRtlScheduleWorkItem(
+    LW_IN PLW_WORK_ITEM pWorkItem,
+    LW_IN LW_SCHEDULE_FLAGS Flags
+    )
+{
+    ScheduleWorkItem(NULL, pWorkItem, Flags);
+}
+
+LW_VOID
+LwRtlWaitWorkItems(
+    LW_IN PLW_THREAD_POOL pPool
+    )
+{
+    WaitWorkItems(&pPool->WorkThreads);
+}
 
