@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -41,23 +41,28 @@
 
 #include <lw/threadpool.h>
 #include <lw/rtlgoto.h>
-#include <pthread.h>
-#include <pthread_np.h>
 #include <sys/types.h>
 #include <sys/event.h>
 #include <sys/time.h>
 #include <sys/param.h>
-#include <sys/cpuset.h>
 
 #include "threadpool-common.h"
 
 #define TASK_COMPLETE_MASK 0xFFFFFFFF
+#define COMMANDS_PER_FD 2
 
 typedef struct _KQUEUE_COMMANDS
 {
   struct kevent* pCommands;
+  /* Command slots used */
   ULONG ulCommandCount;
+  /* Command slots allocated */
   ULONG ulCommandCapacity;
+  /*
+   * Maximum command slots we could possibly need
+   * based on the number of tasks with active fd events.
+   */
+  ULONG ulCommandMax;
 } KQUEUE_COMMANDS, *PKQUEUE_COMMANDS;
 
 typedef struct _KQUEUE_THREAD
@@ -66,8 +71,10 @@ typedef struct _KQUEUE_THREAD
     pthread_t Thread;
     pthread_mutex_t Lock;
     pthread_cond_t Event;
+    /* Self pipe for explicitly waking the thread */
     int SignalFds[2];
     int KqueueFd;
+    KQUEUE_COMMANDS Commands;
     RING Tasks;
     /* Thread load (protected by thread pool lock) */
     ULONG volatile ulLoad;
@@ -83,6 +90,9 @@ typedef struct _LW_TASK
     PLW_TASK_GROUP pGroup;
     /* Ref count (protected by thread lock) */
     ULONG volatile ulRefCount;
+    /* Events physically registered in kqueue */
+    unsigned bReadEventAdded:1;
+    unsigned bWriteEventAdded:1;
     /* Events task is waiting for (owned by thread) */
     LW_TASK_EVENT_MASK EventWait;
     /* Last set of events task waited for (owned by thread) */
@@ -94,12 +104,13 @@ typedef struct _LW_TASK
     LW_TASK_EVENT_MASK volatile EventSignal;
     /* Absolute time of next time wake event (owned by thread) */
     LONG64 llDeadline;
-    /* kevent() commands */
     /* Callback function and context (immutable) */
     LW_TASK_FUNCTION pfnFunc;
     PVOID pFuncContext;
     /* File descriptor for fd-based events (owned by thread) */
     int Fd;
+    /* Last UNIX signal event */
+    siginfo_t* pUnixSignal;
     /* Link to siblings in task group (protected by group lock) */
     RING GroupRing;
     /* Link to siblings in scheduler queue (owned by thread) */
@@ -114,6 +125,9 @@ typedef struct _LW_TASK_GROUP
     RING Tasks;
     pthread_mutex_t Lock;
     pthread_cond_t Event;
+    unsigned bCancelled:1;
+    unsigned bLockInit:1;
+    unsigned bEventInit:1;
 } KQUEUE_TASK_GROUP, *PKQUEUE_TASK_GROUP;
 
 typedef struct _LW_THREAD_POOL
