@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -49,6 +49,7 @@ TaskDelete(
     PSELECT_TASK pTask
     )
 {
+    RTL_FREE(&pTask->pUnixSignal);
     RtlMemoryFree(pTask);
 }
 
@@ -193,14 +194,14 @@ UpdateTriggerSet(
             pTask->TriggerSet |= LW_TASK_EVENT_FD_READABLE;
             pTask->FdSetMask |= LW_TASK_EVENT_FD_READABLE;
         }
-
+        
         if (FD_ISSET(pTask->Fd, pWriteSet))
         {
             FD_CLR(pTask->Fd, pWriteSet);
             pTask->TriggerSet |= LW_TASK_EVENT_FD_WRITABLE;
             pTask->FdSetMask |= LW_TASK_EVENT_FD_WRITABLE;
         }
-
+        
         if (FD_ISSET(pTask->Fd, pExceptSet))
         {
             FD_CLR(pTask->Fd, pExceptSet);
@@ -255,9 +256,9 @@ Sleep(
                 timeout.tv_sec = 0;
                 timeout.tv_usec = 0;
             }
-
+            
             ready = select(nfds, pReadSet, pWriteSet, pExceptSet, &timeout);
-
+            
             if (ready < 0 && errno == EINTR)
             {
                 /* Update current time so the next timeout calculation is correct */
@@ -330,7 +331,7 @@ EventLoop(
 
         /* Get current time for this iteration */
         GOTO_ERROR_ON_STATUS(status = ClockGetMonotonicTime(&clock, &llNow));
-
+        
         /* Figure out which tasks are runnable */
         for (pRing = tasks.pNext; pRing != &tasks; pRing = pNext)
         {
@@ -345,7 +346,7 @@ EventLoop(
                 &writeSet,
                 &exceptSet,
                 llNow);
-
+            
             /* Schedule tasks to run if they have been triggered or were yielding */
             if ((pTask->TriggerWait & LW_TASK_EVENT_YIELD) ||
                 ((pTask->TriggerWait | LW_TASK_EVENT_EXPLICIT) & pTask->TriggerSet))
@@ -377,19 +378,12 @@ EventLoop(
         for (pRing = runnable.pNext; pRing != &runnable; pRing = pNext)
         {
             pNext = pRing->pNext;
-
+            
             pTask = LW_STRUCT_FROM_FIELD(pRing, SELECT_TASK, EventRing);
-
+            
             GOTO_ERROR_ON_STATUS(status = TaskProcessTrigger(pTask, llNow));
-
-            if (pTask->TriggerWait & LW_TASK_EVENT_YIELD)
-            {
-                /* Task is yielding temporarily.  Set its trigger arguments
-                   to whatever it gave us (minus INIT). Leave it on the runnable
-                   list */
-                pTask->TriggerArgs = pTask->TriggerWait & ~LW_TASK_EVENT_INIT;
-            }
-            else if (pTask->TriggerWait != 0)
+            
+            if (pTask->TriggerWait != 0)
             {
                 /* Task is still waiting to be runnable, update select parameters
                    and put it back in the task list */
@@ -400,19 +394,34 @@ EventLoop(
                     &writeSet,
                     &exceptSet,
                     &llNextDeadline);
-
-                RingRemove(&pTask->EventRing);
-                RingInsertBefore(&tasks, &pTask->EventRing);
+            
+                if (pTask->TriggerWait & LW_TASK_EVENT_YIELD)
+                {
+                    /* Task is yielding temporarily.  Set the yield flag on
+                       its trigger arguments.   Leave it on the runnable list */
+                    pTask->TriggerArgs |= LW_TASK_EVENT_YIELD;
+                }
+                else
+                {    
+                    RingRemove(&pTask->EventRing);
+                    RingInsertBefore(&tasks, &pTask->EventRing);
+                }
             }
             else
             {
                 /* Task is complete, notify and remove from task group
                    if it is in one */
-
+                
                 RingRemove(&pTask->EventRing);
+                
+                /* Unregister task from global signal loop */
+                if (pTask->pUnixSignal)
+                {
+                    RegisterTaskUnixSignal(pTask, 0, FALSE);
+                }
 
                 pGroup = pTask->pGroup;
-
+                
                 if (pGroup)
                 {
                     LOCK_GROUP(pGroup);
@@ -421,7 +430,7 @@ EventLoop(
                     pthread_cond_broadcast(&pGroup->Event);
                     UNLOCK_GROUP(pGroup);
                 }
-
+                
                 LOCK_THREAD(pThread);
                 if (--pTask->ulRefCount)
                 {
@@ -478,7 +487,7 @@ EventLoop(
                               llNextDeadline,
                               &ready));
         }
-
+        
         LOCK_THREAD(pThread);
 
         /* Check for a signal to the thread */
@@ -486,17 +495,17 @@ EventLoop(
         {
             FD_CLR(pThread->SignalFds[0], &readSet);
             pThread->bSignalled = FALSE;
-
+            
             res = read(pThread->SignalFds[0], &c, sizeof(c));
             assert(res == sizeof(c));
-
+            
             /* Move all tasks in queue into local task list */
             RingMove(&pThread->Tasks, &tasks);
 
             if (pThread->bShutdown && !bShutdown)
             {
                 bShutdown = pThread->bShutdown;
-
+                
                 /* Cancel all outstanding tasks */
                 for (pRing = tasks.pNext; pRing != &tasks; pRing = pRing->pNext)
                 {
@@ -513,7 +522,7 @@ error:
 
     return status;
 }
-
+    
 static
 PVOID
 EventThread(
@@ -557,23 +566,34 @@ LwRtlCreateTask(
     pTask->llDeadline = 0;
 
     LOCK_POOL(pPool);
-    pTask->pThread = &pPool->pEventThreads[pPool->ulNextEventThread];
-    pPool->ulNextEventThread = (pPool->ulNextEventThread + 1) % pPool->ulEventThreadCount;
-    UNLOCK_POOL(pPool);
-
     if (pGroup)
     {
         LOCK_GROUP(pGroup);
+        if (pGroup->bCancelled)
+        {
+            UNLOCK_GROUP(pGroup);
+            UNLOCK_POOL(pPool);
+            status = STATUS_CANCELLED;
+            GOTO_ERROR_ON_STATUS(status);
+        }
         RingInsertBefore(&pGroup->Tasks, &pTask->GroupRing);
-        UNLOCK_GROUP(pGroup);
     }
 
+    pTask->pThread = &pPool->pEventThreads[pPool->ulNextEventThread];
+    pPool->ulNextEventThread = (pPool->ulNextEventThread + 1) % pPool->ulEventThreadCount;
+    UNLOCK_POOL(pPool);
+
     LOCK_THREAD(pTask->pThread);
     RingInsertBefore(&pTask->pThread->Tasks, &pTask->EventRing);
     /* It's not necessary to signal the thread about the new task here
        since it won't be run anyway */
     UNLOCK_THREAD(pTask->pThread);
 
+    if (pGroup)
+    {
+        UNLOCK_GROUP(pGroup);
+    }
+
     *ppTask = pTask;
 
 error:
@@ -596,17 +616,27 @@ LwRtlCreateTaskGroup(
     }
 
     GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pGroup));
-    GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL)));
-    GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL)));
 
     RingInit(&pGroup->Tasks);
     pGroup->pPool = pPool;
 
+    GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL)));
+    pGroup->bLockInit = TRUE;
+    GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL)));
+    pGroup->bEventInit = TRUE;
+
     *ppGroup = pGroup;
 
+cleanup:
+
+    return status;
+
 error:
 
-    return status;
+    LwRtlFreeTaskGroup(&pGroup);
+    *ppGroup = NULL;
+
+    goto cleanup;
 }
 
 VOID
@@ -622,16 +652,30 @@ LwRtlReleaseTask(
         LOCK_THREAD(pTask->pThread);
         ulRefCount = --pTask->ulRefCount;
         UNLOCK_THREAD(pTask->pThread);
-
+        
         if (ulRefCount == 0)
         {
             TaskDelete(pTask);
         }
-
+        
         *ppTask = NULL;
     }
 }
 
+VOID
+RetainTask(
+    PLW_TASK pTask
+    )
+{
+    if (pTask)
+    {
+        LOCK_THREAD(pTask->pThread);
+        ++pTask->ulRefCount;
+        UNLOCK_THREAD(pTask->pThread);
+    }
+}
+
+
 VOID
 LwRtlFreeTaskGroup(
     PLW_TASK_GROUP* ppGroup
@@ -641,8 +685,16 @@ LwRtlFreeTaskGroup(
 
     if (pGroup)
     {
-        pthread_mutex_destroy(&pGroup->Lock);
-        pthread_cond_destroy(&pGroup->Event);
+        if (pGroup->bLockInit)
+        {
+            pthread_mutex_destroy(&pGroup->Lock);
+        }
+
+        if (pGroup->bEventInit)
+        {
+            pthread_cond_destroy(&pGroup->Event);
+        }
+
         RtlMemoryFree(pGroup);
 
         *ppGroup = NULL;
@@ -677,7 +729,7 @@ LwRtlSetTaskFd(
             pTask->FdWaitMask = Mask;
         }
     }
-    else
+    else 
     {
         if (pTask->Fd >= 0)
         {
@@ -735,6 +787,88 @@ LwRtlWakeTask(
     UNLOCK_THREAD(pTask->pThread);
 }
 
+LW_NTSTATUS
+LwRtlSetTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_IN int Sig,
+    LW_IN LW_BOOLEAN bSubscribe
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+
+    if (bSubscribe && !pTask->pUnixSignal)
+    {
+        status = LW_RTL_ALLOCATE_AUTO(&pTask->pUnixSignal);
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
+    status = RegisterTaskUnixSignal(pTask, Sig, bSubscribe);
+    GOTO_ERROR_ON_STATUS(status);
+
+error:
+
+    return status;
+}
+
+void
+NotifyTaskUnixSignal(
+    PLW_TASK pTask,
+    siginfo_t* pInfo
+    )
+{
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->TriggerSet != TASK_COMPLETE_MASK)
+    {
+        while (pTask->pUnixSignal->si_signo)
+        {
+            pthread_cond_wait(&pTask->pThread->Event, &pTask->pThread->Lock);
+            if (pTask->TriggerSet == TASK_COMPLETE_MASK)
+            {
+                goto cleanup;
+            }
+        }
+
+        *pTask->pUnixSignal = *pInfo;
+        pTask->TriggerSet |= LW_TASK_EVENT_UNIX_SIGNAL;
+        SignalThread(pTask->pThread);
+    }
+
+cleanup:
+
+    UNLOCK_THREAD(pTask->pThread);
+}
+
+LW_BOOLEAN
+LwRtlNextTaskUnixSignal(
+    LW_IN PLW_TASK pTask,
+    LW_OUT siginfo_t* pInfo
+    )
+{
+    BOOLEAN bResult = FALSE;
+
+    LOCK_THREAD(pTask->pThread);
+
+    if (pTask->pUnixSignal == NULL || pTask->pUnixSignal->si_signo == 0)
+    {
+        bResult = FALSE;
+    }
+    else
+    {
+        if (pInfo)
+        {
+            *pInfo = *pTask->pUnixSignal;
+        }
+        pTask->pUnixSignal->si_signo = 0;
+        pthread_cond_broadcast(&pTask->pThread->Event);
+        bResult = TRUE;
+    }
+
+    UNLOCK_THREAD(pTask->pThread);
+
+    return bResult;
+}
+
 VOID
 LwRtlCancelTask(
     PLW_TASK pTask
@@ -794,6 +928,8 @@ LwRtlCancelTaskGroup(
 
     LOCK_GROUP(group);
 
+    group->bCancelled = TRUE;
+
     for (i = 0; i < group->pPool->ulEventThreadCount; i++)
     {
         LOCK_THREAD(&group->pPool->pEventThreads[i]);
@@ -803,7 +939,7 @@ LwRtlCancelTaskGroup(
     {
         pTask = LW_STRUCT_FROM_FIELD(ring, SELECT_TASK, GroupRing);
 
-        pTask->TriggerSet |= LW_TASK_EVENT_EXPLICIT | LW_TASK_EVENT_CANCEL;
+        pTask->TriggerSet |= LW_TASK_EVENT_EXPLICIT | LW_TASK_EVENT_CANCEL;        
     }
 
     for (i = 0; i < group->pPool->ulEventThreadCount; i++)
@@ -833,13 +969,13 @@ LwRtlWaitTaskGroup(
         for (pRing = group->Tasks.pNext; !bStillAlive && pRing != &group->Tasks; pRing = pRing->pNext)
         {
             pTask = LW_STRUCT_FROM_FIELD(pRing, SELECT_TASK, GroupRing);
-
+         
             LOCK_THREAD(pTask->pThread);
             if (pTask->TriggerSet != TASK_COMPLETE_MASK)
             {
                 bStillAlive = TRUE;
             }
-            UNLOCK_THREAD(pTask->pThread);
+            UNLOCK_THREAD(pTask->pThread);         
         }
 
         if (bStillAlive)
@@ -913,6 +1049,35 @@ SelectThreadDestroy(
     pthread_cond_destroy(&pThread->Event);
 }
 
+static
+NTSTATUS
+GetFdLimit(
+    PULONG pulLimit
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    struct rlimit limit = {0};
+
+    if (getrlimit(RLIMIT_NOFILE, &limit) != 0)
+    {
+        status = LwErrnoToNtStatus(errno);
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
+    if (limit.rlim_cur == RLIM_INFINITY)
+    {
+        *pulLimit = (ULONG) 0xFFFFFFFF;
+    }
+    else
+    {
+        *pulLimit = (ULONG) limit.rlim_cur;
+    }
+
+error:
+
+    return status;
+}
+
 NTSTATUS
 LwRtlCreateThreadPool(
     PLW_THREAD_POOL* ppPool,
@@ -923,6 +1088,7 @@ LwRtlCreateThreadPool(
     PLW_THREAD_POOL pPool = NULL;
     int i = 0;
     int numCpus = 0;
+    ULONG ulFdLimit = 0;
 
 #if defined(_SC_NPROCESSORS_ONLN)
     numCpus = sysconf(_SC_NPROCESSORS_ONLN);
@@ -934,10 +1100,10 @@ LwRtlCreateThreadPool(
 #else
     numCpus = 1;
 #endif
-
+   
 
     GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pPool));
-
+    
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL)));
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pPool->Event, NULL)));
 
@@ -951,6 +1117,20 @@ LwRtlCreateThreadPool(
         pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
         pPool->ulNextEventThread = 0;
 
+        status = GetFdLimit(&ulFdLimit);
+        GOTO_ERROR_ON_STATUS(status);
+
+        /*
+         * Each thread needs 2 fds for the notification pipe.
+         * This check will limit us to 1/4 of the process fd limit.
+         * This is particularly important on Solaris which has a very
+         * low default limit of 256.
+         */
+        if (pPool->ulEventThreadCount > ulFdLimit / 8)
+        {
+            pPool->ulEventThreadCount = ulFdLimit / 8;
+        }
+
         if (pPool->ulEventThreadCount)
         {
             GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_ARRAY_AUTO(
@@ -988,7 +1168,7 @@ LwRtlFreeThreadPool(
         pPool->bShutdown = TRUE;
         pthread_cond_broadcast(&pPool->Event);
         UNLOCK_POOL(pPool);
-
+        
         if (pPool->pEventThreads)
         {
             for (i = 0; i < pPool->ulEventThreadCount; i++)
@@ -1001,17 +1181,18 @@ LwRtlFreeThreadPool(
                 pthread_join(pThread->Thread, NULL);
                 SelectThreadDestroy(pThread);
             }
-
+            
             RtlMemoryFree(pPool->pEventThreads);
         }
-
+        
         if (pPool->pDelegate)
         {
             ReleaseDelegatePool(&pPool->pDelegate);
         }
 
+        pthread_cond_destroy(&pPool->Event);
         pthread_mutex_destroy(&pPool->Lock);
-
+        
         DestroyWorkThreads(&pPool->WorkThreads);
 
         RtlMemoryFree(pPool);
@@ -1020,15 +1201,38 @@ LwRtlFreeThreadPool(
     }
 }
 
-NTSTATUS
-LwRtlQueueWorkItem(
-    PLW_THREAD_POOL pPool,
-    LW_WORK_ITEM_FUNCTION pfnFunc,
-    PVOID pContext,
-    LW_WORK_ITEM_FLAGS Flags
+LW_NTSTATUS
+LwRtlCreateWorkItem(
+    LW_IN PLW_THREAD_POOL pPool,
+    LW_OUT PLW_WORK_ITEM* ppWorkItem,
+    LW_IN LW_WORK_ITEM_FUNCTION pfnFunc,
+    LW_IN PVOID pContext
     )
 {
-    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
+    return CreateWorkItem(&pPool->WorkThreads, ppWorkItem, pfnFunc, pContext);
 }
 
+LW_VOID
+LwRtlFreeWorkItem(
+    LW_IN LW_OUT PLW_WORK_ITEM* ppWorkItem
+    )
+{
+    FreeWorkItem(ppWorkItem);
+}
 
+LW_VOID
+LwRtlScheduleWorkItem(
+    LW_IN PLW_WORK_ITEM pWorkItem,
+    LW_IN LW_SCHEDULE_FLAGS Flags
+    )
+{
+    ScheduleWorkItem(NULL, pWorkItem, Flags);
+}
+
+LW_VOID
+LwRtlWaitWorkItems(
+    LW_IN PLW_THREAD_POOL pPool
+    )
+{
+    WaitWorkItems(&pPool->WorkThreads);
+}
