@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -79,6 +79,8 @@ typedef struct _LW_TASK
     PVOID pFuncContext;
     /* File descriptor for fd-based events */
     int Fd;
+    /* Latest signal event */
+    siginfo_t* pUnixSignal;
     /* Wait mask for fd */
     LW_TASK_EVENT_MASK FdWaitMask;
     /* Set mask for fd */
@@ -95,6 +97,9 @@ typedef struct _LW_TASK_GROUP
     RING Tasks;
     pthread_mutex_t Lock;
     pthread_cond_t Event;
+    unsigned bCancelled:1;
+    unsigned bLockInit:1;
+    unsigned bEventInit:1;
 } SELECT_TASK_GROUP, *PSELECT_TASK_GROUP;
 
 typedef struct _LW_THREAD_POOL
