@@ -1,22 +1,48 @@
-/*++
-    Linux DNS client library implementation
-    Copyright (C) 2006 Krishna Ganugapati
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+/*
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        dnsdefines.h
+ *
+ * Abstract:
+ *
+ *        Likewise Dynamic DNS Updates (LWDNS)
+ * 
+ * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
+ *          Sriram Nambakam (snambakam@likewisesoftware.com)
+ */
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-++*/
 #ifndef __DNS_DEFINES_H__
 #define __DNS_DEFINES_H__
 
@@ -68,10 +94,11 @@ extern PFN_LWDNS_LOG_MESSAGE gpfnLWDNSLogger;
 #define _LWDNS_LOG_MESSAGE(Level, Format, ...) \
     DNSLogMessage(gpfnLWDNSLogger, Level, Format, ## __VA_ARGS__)
 
-#define _LWDNS_LOG_WITH_THREAD(Level, Format, ...) \
-    _LWDNS_LOG_MESSAGE(Level,                      \
-                     "0x%x:" Format,               \
-                     (unsigned int)pthread_self(), \
+
+#define _LWDNS_LOG_WITH_THREAD(Level, Format, ...)	\
+  _LWDNS_LOG_MESSAGE(Level,				\
+                     "0x%lx:" Format,			\
+                     (unsigned long)pthread_self(),	\
                      ## __VA_ARGS__)
 
 #define _LWDNS_LOG_WITH_DEBUG(Level, Format, ...) \
@@ -182,6 +209,8 @@ extern PFN_LWDNS_LOG_MESSAGE gpfnLWDNSLogger;
 #define IsNullOrEmptyString(str) \
     (!(str) || !*(str))
 
+#define IsEmptyString(str) (!*(str))
+
 #endif
 
 #define LWDNS_SAFE_FREE_STRING(str) \
@@ -200,14 +229,6 @@ extern PFN_LWDNS_LOG_MESSAGE gpfnLWDNSLogger;
            }                        \
         } while(0);
 
-#define BAIL_ON_INVALID_STRING(str)            \
-    do {                                       \
-        if (IsNullOrEmptyString(str)) {        \
-            dwError = ERROR_INVALID_PARAMETER; \
-            BAIL_ON_LWDNS_ERROR(dwError);      \
-        }                                      \
-    } while(0)
-
 /* DNS Class Types */
 
 #define DNS_CLASS_IN        1
@@ -234,6 +255,8 @@ extern PFN_LWDNS_LOG_MESSAGE gpfnLWDNSLogger;
 
 #define QTYPE_SOA              6 // SOA
 
+#define QTYPE_PTR              12 // PTR
+
 #define QTYPE_ANY              255
 
 #define    QTYPE_TKEY              249
@@ -305,8 +328,12 @@ TXT             16 text strings
 #ifndef WIN32
 #define SOCKET_ERROR             -1
 #define INVALID_SOCKET            -1
+#ifndef TRUE
 #define TRUE                 1
+#endif
+#ifndef FALSE
 #define FALSE                 0
+#endif
 
 
 #endif
