@@ -2,6 +2,32 @@
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
  * -*- mode: c, c-basic-offset: 4 -*- */
 
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
+
 /*
  * Copyright (C) Likewise Software. All rights reserved.
  *
@@ -22,178 +48,12 @@
 #define __LWDNS_H__
 
 #ifndef _WIN32
-
-#if HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
-
-#ifndef DWORD_DEFINED
-#define DWORD_DEFINED 1
-
-typedef uint32_t        DWORD, *PDWORD;
-
-#endif
-
-#ifndef INT_DEFINED
-#define INT_DEFINED 1
-
-typedef int             INT, *PINT;
-
-#endif
-
-#ifndef UINT64_DEFINED
-#define UINT64_DEFINED 1
-
-typedef uint64_t        UINT64, *PUINT64;
-
-#endif
-
-#ifndef UINT32_DEFINED
-#define UINT32_DEFINED 1
-
-typedef uint32_t        UINT32, *PUINT32;
-
-#endif
-
-#ifndef UINT16_DEFINED
-#define UINT16_DEFINED 1
-
-typedef uint16_t        UINT16, *PUINT16;
-
-#endif
-
-#ifndef WORD_DEFINED
-#define WORD_DEFINED 1
-
-typedef uint16_t WORD, *PWORD;
-
-#endif
-
-#ifndef USHORT_DEFINED
-#define USHORT_DEFINED 1
-
-typedef unsigned short  USHORT, *PUSHORT;
-
-#endif
-
-#ifndef ULONG_DEFINED
-#define ULONG_DEFINED 1
-
-typedef unsigned long   ULONG, *PULONG;
-
-#endif
-
-#ifndef ULONGLONG_DEFINED
-#define ULONGLONG_DEFINED 1
-
-typedef unsigned long long ULONGLONG, *PULONGLONG;
-
-#endif
-
-#ifndef UINT8_DEFINED
-#define UINT8_DEFINED 1
-
-typedef uint8_t         UINT8, *PUINT8;
-
-#endif
-
-#ifndef BYTE_DEFINED
-#define BYTE_DEFINED
-
-typedef uint8_t BYTE, *PBYTE;
-
-#endif
-
-#ifndef UCHAR_DEFINED
-#define UCHAR_DEFINED 1
-
-typedef uint8_t UCHAR, *PUCHAR;
-
-#endif
-
-#ifndef HANDLE_DEFINED
-#define HANDLE_DEFINED 1
-
-typedef unsigned long   HANDLE, *PHANDLE;
-
-#endif
-
-#ifndef CHAR_DEFINED
-#define CHAR_DEFINED 1
-
-typedef char            CHAR;
-
-#endif
-
-#ifndef PSTR_DEFINED
-#define PSTR_DEFINED 1
-
-typedef char *          PSTR;
-
-#endif
-
-#ifndef PCSTR_DEFINED
-#define PCSTR_DEFINED 1
-
-typedef const char *    PCSTR;
-
-#endif
-
-#ifndef VOID_DEFINED
-#define VOID_DEFINED 1
-
-typedef void            VOID, *PVOID;
-
-#endif
-
-#ifndef PCVOID_DEFINED
-#define PCVOID_DEFINED 1
-
-typedef const void      *PCVOID;
-
-#endif
-
-#ifndef BOOLEAN_DEFINED
-#define BOOLEAN_DEFINED 1
-
-typedef int             BOOLEAN, *PBOOLEAN;
-
-#endif
-
-#ifndef SOCKADDR_IN_DEFINED
-#define SOCKADDR_IN_DEFINED 1
+#include <lw/types.h>
+#include <lw/attrs.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 
 typedef struct sockaddr_in SOCKADDR_IN, *PSOCKADDR_IN;
-
-#endif
-
-#ifndef TRUE
-#define TRUE  1
-#endif
-
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef OPTIONAL
-#define OPTIONAL
-#endif
-
-#ifndef IN
-#define IN
-#endif
-
-#ifndef OUT
-#define OUT
-#endif
-
-#endif
-
-#ifndef WIN32
-
-#define ERROR_INVALID_PARAMETER EINVAL
-#define ERROR_OUTOFMEMORY       ENOMEM
-
 #endif
 
 #define LWDNS_ERROR_SUCCESS             0
@@ -224,7 +84,9 @@ typedef struct sockaddr_in SOCKADDR_IN, *PSOCKADDR_IN;
 #define LWDNS_ERROR_UPDATE_FAILED       0xE018 // 57368
 #define LWDNS_ERROR_NO_INTERFACES       0xE019 // 57369
 #define LWDNS_ERROR_INVALID_IP_ADDRESS  0xE01A // 57370
-#define LWDNS_ERROR_SENTINEL            0xE01B // 57371
+#define LWDNS_ERROR_STRING_CONV_FAILED  0xE01B // 57371
+#define LWDNS_ERROR_INVALID_PARAMETER   0xE01C // 57372
+#define LWDNS_ERROR_SENTINEL            0xE01D // 57373
 
 #define LWDNS_ERROR_MASK(_e_)           (_e_ & 0xE000)
 
@@ -379,14 +241,20 @@ DNSOpen(
     PHANDLE phDNSServer
     );
 
+DWORD
+DNSUpdatePtrSecure(
+    PSOCKADDR_IN pAddr,
+    PCSTR  pszHostNameFQDN
+    );
+
 DWORD
 DNSUpdateSecure(
     HANDLE hDNSServer,
     PCSTR  pszServerName,
     PCSTR  pszDomainName,
     PCSTR  pszHostname,
-    PSOCKADDR_IN pAddrArray,
-    DWORD  dwNumAddrs
+    DWORD  dwNumAddrs,
+    PSOCKADDR_IN pAddrArray
     );
 
 DWORD
