@@ -1,3 +1,33 @@
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
+
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
+
 #include "includes.h"
 
 static
@@ -385,14 +415,13 @@ DNSGetInfoUsingIoctl(
         memset(szItfName, 0, sizeof(szItfName));
         memcpy(szItfName, pInterfaceRecord->ifr_name, IFNAMSIZ);
         
-        LWDNS_LOG_VERBOSE("Considering network interface [%s]",
-                          IsNullOrEmptyString(szItfName) ? "" : szItfName);
+        LWDNS_LOG_VERBOSE("Considering network interface [%s]", szItfName);
         
         pSA = (struct sockaddr*)&pInterfaceRecord->ifr_addr;
 
         if (pSA->sa_family != AF_INET)
         {
-           LWDNS_LOG_VERBOSE("Skipping network interface [%s] [family:%d] because it is not AF_INET family", IsNullOrEmptyString(szItfName) ? "" : szItfName, pSA->sa_family);
+           LWDNS_LOG_VERBOSE("Skipping network interface [%s] [family:%d] because it is not AF_INET family", szItfName, pSA->sa_family);
            continue;
         }
         
@@ -408,15 +437,13 @@ DNSGetInfoUsingIoctl(
         
         if (dwFlags & IFF_LOOPBACK)
         {
-            LWDNS_LOG_VERBOSE("Skipping loopback network interface [%s]",
-                              IsNullOrEmptyString(szItfName) ? "" : szItfName);
+            LWDNS_LOG_VERBOSE("Skipping loopback network interface [%s]", szItfName);
             continue;
         }
         
         if (!(dwFlags & IFF_UP))
         {
-            LWDNS_LOG_VERBOSE("Skipping in-active network interface [%s]",
-                              IsNullOrEmptyString(szItfName) ? "" : szItfName);
+            LWDNS_LOG_VERBOSE("Skipping in-active network interface [%s]", szItfName);
             continue;
         }
 
@@ -465,7 +492,7 @@ DNSGetInfoUsingIoctl(
         pszIpAddress = inet_ntoa(((struct sockaddr_in*)&pInterfaceInfo->ipAddr)->sin_addr);
         
         LWDNS_LOG_VERBOSE("Added network interface [Name:%s; Address:%s] to list",
-                          (IsNullOrEmptyString(szItfName) ? "" : pInterfaceInfo->pszName),
+                          (IsNullOrEmptyString(pInterfaceInfo->pszName) ? "" : pInterfaceInfo->pszName),
                           (IsNullOrEmptyString(pszIpAddress) ? "" : pszIpAddress));
         
         pInterfaceInfo = NULL;
