@@ -1,22 +1,47 @@
-/*++
-    Linux DNS client library implementation
-    Copyright (C) 2006 Krishna Ganugapati
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-++*/
+/*
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        dnsrecord.c
+ *
+ * Abstract:
+ *
+ *        Likewise Dynamic DNS Updates (LWDNS)
+ * 
+ * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
+ *          Sriram Nambakam (snambakam@likewisesoftware.com)
+ */
 
 #include "includes.h"
 
@@ -70,6 +95,63 @@ error:
     goto cleanup;
 }
 
+DWORD
+DNSCreatePtrRecord(
+    PCSTR pszName,
+    WORD  wClass,
+    PCSTR pszDest,
+    PDNS_RR_RECORD * ppDNSRecord
+    )
+{
+    DWORD dwError = 0;
+    PDNS_RR_RECORD pDNSRRRecord = NULL;
+    PDNS_DOMAIN_NAME pDomainName = NULL;
+
+    dwError = DNSDomainNameFromString(
+                    pszName,
+                    &pDomainName);
+    BAIL_ON_LWDNS_ERROR(dwError);
+
+    dwError = DNSAllocateMemory(
+                    sizeof(DNS_RR_RECORD),
+                    (PVOID *)&pDNSRRRecord);
+    BAIL_ON_LWDNS_ERROR(dwError);
+
+    pDNSRRRecord->RRHeader.dwTTL = DNS_ONE_HOUR_IN_SECS;
+    pDNSRRRecord->RRHeader.wClass = wClass;
+    pDNSRRRecord->RRHeader.wType = QTYPE_PTR;
+    pDNSRRRecord->RRHeader.pDomainName = pDomainName;
+    pDomainName = NULL;
+    pDNSRRRecord->RRHeader.wRDataSize = 0;
+
+    dwError = DNSDomainNameFromString(
+                    pszDest,
+                    &pDomainName);
+    BAIL_ON_LWDNS_ERROR(dwError);
+    pDNSRRRecord->pRDataDomain = pDomainName;
+    pDomainName = NULL;
+
+    *ppDNSRecord = pDNSRRRecord;
+    
+cleanup:
+    if (pDomainName)
+    {
+        DNSFreeDomainName(pDomainName);
+    }
+
+    return dwError;
+
+error:
+    if (pDNSRRRecord)
+    {
+        DNSFreeRecord(pDNSRRRecord);
+    }
+
+    *ppDNSRecord = NULL;
+    
+    goto cleanup;
+}
+
 DWORD
 DNSCreateARecord(
     PCSTR pszHost,
@@ -833,6 +915,11 @@ DNSFreeRecord(
     {
         DNSFreeMemory(pRecord->pRData);
     }
+
+    if (pRecord->pRDataDomain)
+    {
+        DNSFreeDomainName(pRecord->pRDataDomain);
+    }
     
     if (pRecord->RRHeader.pDomainName)
     {
