@@ -1,22 +1,47 @@
-/*++
-    Linux DNS client library implementation
-    Copyright (C) 2006 Krishna Ganugapati
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-++*/
+/*
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        dnsrequest.c
+ *
+ * Abstract:
+ *
+ *        Likewise Dynamic DNS Updates (LWDNS)
+ * 
+ * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
+ *          Sriram Nambakam (snambakam@likewisesoftware.com)
+ */
 
 #include "includes.h"
 
@@ -639,7 +664,6 @@ DNSMarshallRRHeader(
     DWORD dwRead = 0;
     WORD wnType = 0;
     WORD wnClass = 0;
-    WORD wnRDataSize = 0;
     DWORD dwnTTL = 0;
 
     dwError = DNSMarshallDomainName(
@@ -671,14 +695,6 @@ DNSMarshallRRHeader(
                 &dwRead);
     BAIL_ON_LWDNS_ERROR(dwError);
 
-    wnRDataSize = htons(pDNSRecord->RRHeader.wRDataSize);
-    dwError = DNSMarshallBuffer(
-                hSendBuffer,
-                (PBYTE)&wnRDataSize,
-                sizeof(WORD),
-                &dwRead);
-    BAIL_ON_LWDNS_ERROR(dwError);
-
 error:
 
     return(dwError);
@@ -690,11 +706,55 @@ DNSMarshallRData(
     PDNS_RR_RECORD pDNSRecord
     )
 {
+    DWORD dwError = 0;
     DWORD dwWritten = 0;
+    WORD wnRDataSize = 0;
+    PDNS_SENDBUFFER_CONTEXT pDNSContext = NULL;
+    DWORD dwSizeOffset = 0;
+    DWORD dwDataStartOffset = 0;
+    DWORD  dwRead = 0;
 
-    return DNSMarshallBuffer(
+    pDNSContext = (PDNS_SENDBUFFER_CONTEXT)hSendBuffer;
+
+    wnRDataSize = htons(pDNSRecord->RRHeader.wRDataSize);
+    // Remember where the rdata size is written in the buffer so it can be
+    // fixed up later
+    dwSizeOffset = pDNSContext->dwBufferOffset;
+    dwError = DNSMarshallBuffer(
                 hSendBuffer,
-                pDNSRecord->pRData,
-                pDNSRecord->RRHeader.wRDataSize,
-                &dwWritten);
+                (PBYTE)&wnRDataSize,
+                sizeof(WORD),
+                &dwRead);
+    BAIL_ON_LWDNS_ERROR(dwError);
+
+    dwDataStartOffset = pDNSContext->dwBufferOffset;
+
+    if (pDNSRecord->pRDataDomain)
+    {
+        dwError = DNSMarshallDomainName(
+                    hSendBuffer,
+                    pDNSRecord->pRDataDomain);
+        BAIL_ON_LWDNS_ERROR(dwError);
+
+        // Fix up the length
+        wnRDataSize = htons(pDNSContext->dwBufferOffset - dwDataStartOffset);
+        memcpy(&pDNSContext->pSendBuffer[dwSizeOffset],
+               &wnRDataSize,
+               sizeof(wnRDataSize));
+    }
+    else
+    {
+        dwError = DNSMarshallBuffer(
+                    hSendBuffer,
+                    pDNSRecord->pRData,
+                    pDNSRecord->RRHeader.wRDataSize,
+                    &dwWritten);
+        BAIL_ON_LWDNS_ERROR(dwError);
+    }
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
 }
