@@ -1,22 +1,47 @@
-/*++
-    Linux DNS client library implementation
-    Copyright (C) 2006 Krishna Ganugapati
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-++*/
+/*
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        dnssock.c
+ *
+ * Abstract:
+ *
+ *        Likewise Dynamic DNS Updates (LWDNS)
+ * 
+ * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
+ *          Sriram Nambakam (snambakam@likewisesoftware.com)
+ */
 
 #include "includes.h"
 
@@ -30,7 +55,11 @@ DNSOpen(
     DWORD  dwError = 0;
     HANDLE hDNSServer = (HANDLE)NULL;
 
-    BAIL_ON_INVALID_STRING(pszNameServer);
+    if (IsNullOrEmptyString(pszNameServer))
+    {
+        dwError = LWDNS_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LWDNS_ERROR(dwError);
+    }
 
     switch (dwType)
     {
@@ -54,7 +83,7 @@ DNSOpen(
 
         default:
 
-            dwError = ERROR_INVALID_PARAMETER;
+            dwError = LWDNS_ERROR_INVALID_PARAMETER;
     }
     BAIL_ON_LWDNS_ERROR(dwError);
 
@@ -918,103 +947,6 @@ error:
     goto cleanup;
 }
 
-DWORD
-DNSUnmarshallDomainNameAtOffset(
-    HANDLE hRecvBuffer,
-    WORD wOffset,
-    PDNS_DOMAIN_NAME * ppDomainName
-    )
-{
-    DWORD dwError = 0;
-    PDNS_DOMAIN_LABEL pLabel = NULL;
-    PDNS_DOMAIN_LABEL pLabelList = NULL;
-    PDNS_DOMAIN_NAME pDomainName = NULL;
-    PSTR  pszLabel = NULL;
-    BYTE  uLen = 0;
-    DWORD dwCurrent = 0;
-    PDNS_RECEIVEBUFFER_CONTEXT pRecvContext = NULL;
-
-    pRecvContext = (PDNS_RECEIVEBUFFER_CONTEXT)hRecvBuffer;
-    dwCurrent = wOffset;
-
-    while(1)
-    {
-        CHAR szLabel[65];
-        
-        memcpy(&uLen,
-               pRecvContext->pRecvBuffer+dwCurrent,
-               sizeof(char));
-        
-        dwCurrent++;
-
-        if (uLen == 0) {
-            break;
-        }
-
-        memset(szLabel, 0, sizeof(szLabel));
-        memcpy(szLabel, pRecvContext->pRecvBuffer+dwCurrent, uLen);
-        dwCurrent += uLen;
-
-        dwError = DNSAllocateString(
-                    szLabel, 
-                    &pszLabel);
-        BAIL_ON_LWDNS_ERROR(dwError);
-
-        dwError = DNSAllocateMemory(
-                    sizeof(DNS_DOMAIN_LABEL),
-                    (PVOID *)&pLabel);
-        BAIL_ON_LWDNS_ERROR(dwError);
-
-        pLabel->pszLabel = pszLabel;
-        pszLabel = NULL;
-        
-        dwError = DNSAppendLabel(
-                        pLabelList,
-                        pLabel,
-                        &pLabelList);
-        BAIL_ON_LWDNS_ERROR(dwError);
-        
-        pLabel = NULL;
-    }
-
-    dwError = DNSAllocateMemory(
-                sizeof(DNS_DOMAIN_NAME),
-                (PVOID *)&pDomainName);
-    BAIL_ON_LWDNS_ERROR(dwError);
-    
-    pDomainName->pLabelList = pLabelList;
-    pLabelList = NULL;
-
-    *ppDomainName = pDomainName;
-    
-cleanup:
-
-    return dwError;
-
-error:
-
-    *ppDomainName = NULL;
-    
-    LWDNS_SAFE_FREE_STRING(pszLabel);
-    
-    if (pLabel)
-    {
-        DNSFreeLabel(pLabel);
-    }
-    
-    if (pLabelList)
-    {
-        DNSFreeLabelList(pLabelList);
-    }
-    
-    if (pDomainName)
-    {
-        DNSFreeDomainName(pDomainName);
-    }
-    
-    goto cleanup;
-}
-
 DWORD
 DNSReceiveBufferMoveBackIndex(
     HANDLE hRecvBuffer,
