@@ -1,22 +1,47 @@
-/*++
-	Linux DNS client library implementation
-	Copyright (C) 2006 Krishna Ganugapati
+/* Editor Settings: expandtabs and use 4 spaces for indentation
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
-	This library is free software; you can redistribute it and/or
-	modify it under the terms of the GNU Lesser General Public
-	License as published by the Free Software Foundation; either
-	version 2.1 of the License, or (at your option) any later version.
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
-	This library is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	Lesser General Public License for more details.
-
-	You should have received a copy of the GNU Lesser General Public
-	License along with this library; if not, write to the Free Software
-	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-++*/
+/*
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        dnsstruct.h
+ *
+ * Abstract:
+ *
+ *        Likewise Dynamic DNS Updates (LWDNS)
+ * 
+ * Authors: Krishna Ganugapati (krishnag@likewisesoftware.com)
+ *          Sriram Nambakam (snambakam@likewisesoftware.com)
+ */
 
 #ifndef __DNSSTRUCT_H__
 #define __DNSSTRUCT_H__
@@ -65,7 +90,6 @@ typedef struct _DNS_RR_HEADER {
 	WORD wClass;
 	DWORD dwTTL;
 	WORD  wRDataSize;
-	PBYTE pRData;
 }DNS_RR_HEADER, *PDNS_RR_HEADER;
 
 
@@ -104,7 +128,9 @@ typedef struct _DNS_RR_RECORD {
 		DNS_TKEY_OFFSETS TKey;
 		DNS_TSIG_OFFSETS TSig;
 	}Offsets;
+	// either pRData or pRDataDomain may be set
 	PBYTE pRData;
+	PDNS_DOMAIN_NAME pRDataDomain;
 }DNS_RR_RECORD, *PDNS_RR_RECORD;
 
 
@@ -123,11 +149,28 @@ typedef struct _DNS_REQUEST {
 
 
 typedef struct _DNS_UPDATE_REQUEST {
+	// Called Transaction ID in wireshark
+	// Called ID in RFC2136
+	// This is a number randomly chosen by the client to identify which
+	// reply belongs to the request incase the request was sent over UDP.
 	WORD wIdentification;
+	// Composed of QR, Opcode, Z, and RCODE in RFC2136
+	// Called Flags in wireshark
 	WORD wParameter;
+	// Called ZOCOUNT in RFC2136
+	// Number of number zones in the update request (ppZoneRRSet)
+	// The only value that makes sense is 1
 	WORD wZones;
+	// Called PRCOUNT in RFC2136
+	// Number of prerequisites in update request (ppPRRRSet)
 	WORD wPRs;
+	// Called UPCOUNT in RFC2136
+	// Number of updates (adds or deletes) in update request
+	// (ppUpdateRRSet)
 	WORD wUpdates;
+	// Called ADCOUNT in RFC2136
+	// Number of additional records (only a signature record is used here)
+	// in update request (ppAdditionalRRSet)
 	WORD wAdditionals;
 	PDNS_ZONE_RECORD * ppZoneRRSet;
 	PDNS_RR_RECORD * ppPRRRSet;
