@@ -2,6 +2,31 @@
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
  * -*- mode: c, c-basic-offset: 4 -*- */
 
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
 
 #include  "includes.h"
 
@@ -173,7 +198,7 @@ DNSSendPtrSecureUpdate(
     DWORD dwError = 0;
     PDNS_UPDATE_REQUEST pDNSUpdateRequest = NULL;
     PDNS_UPDATE_RESPONSE pDNSUpdateResponse = NULL;
-
+    
     LWDNS_LOG_INFO("Attempting DNS Update (secure)");
 
     dwError = DNSUpdateCreatePtrRUpdateRequest(
@@ -339,6 +364,7 @@ DNSGetPtrNameForAddr(
 {
     DWORD dwError = 0;
     PSTR pszRecordName = NULL;
+    DWORD dwIPV4Addr = ntohl(pAddr->sin_addr.s_addr);
 
     if (pAddr->sin_family != AF_INET)
     {
@@ -349,10 +375,10 @@ DNSGetPtrNameForAddr(
     dwError = LwRtlCStringAllocatePrintf(
                     &pszRecordName,
                     "%d.%d.%d.%d.in-addr.arpa",
-                    (pAddr->sin_addr.s_addr >> 24) & 255,
-                    (pAddr->sin_addr.s_addr >> 16) & 255,
-                    (pAddr->sin_addr.s_addr >>  8) & 255,
-                    (pAddr->sin_addr.s_addr >>  0) & 255
+                    (dwIPV4Addr >>  0) & 255,
+                    (dwIPV4Addr >>  8) & 255,
+                    (dwIPV4Addr >> 16) & 255,
+                    (dwIPV4Addr >> 24) & 255
                 );
     if (dwError)
     {
@@ -367,7 +393,48 @@ cleanup:
 
 error:
     *ppszRecordName = NULL;
-    LWDNS_SAFE_FREE_STRING(pszRecordName);
+    LwRtlCStringFree(&pszRecordName);
+
+    goto cleanup;
+}
+
+DWORD
+DNSGetPtrZoneForAddr(
+    PSTR* ppszZoneName,
+    PSOCKADDR_IN pAddr
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszZoneName = NULL;
+    DWORD dwIPV4Addr = ntohl(pAddr->sin_addr.s_addr);
+
+    if (pAddr->sin_family != AF_INET)
+    {
+        dwError = LWDNS_ERROR_INVALID_IP_ADDRESS;
+        BAIL_ON_LWDNS_ERROR(dwError);
+    }
+
+    dwError = LwRtlCStringAllocatePrintf(
+                    &pszZoneName,
+                    "%d.%d.%d.in-addr.arpa",
+                    (dwIPV4Addr >>  8) & 255,
+                    (dwIPV4Addr >> 16) & 255,
+                    (dwIPV4Addr >> 24) & 255
+                );
+    if (dwError)
+    {
+        dwError = ENOMEM;
+        BAIL_ON_LWDNS_ERROR(dwError);
+    }
+
+    *ppszZoneName = pszZoneName;
+
+cleanup:
+    return dwError;
+
+error:
+    *ppszZoneName = NULL;
+    LwRtlCStringFree(&pszZoneName);
 
     goto cleanup;
 }
@@ -384,17 +451,19 @@ DNSUpdatePtrSecure(
     DWORD   dwNumNSInfos = 0;
     BOOLEAN bDNSUpdated = FALSE;
     PSTR pszRecordName = NULL;
+    PSTR pszPtrZone = NULL;
     DWORD   iNS = 0;
     HANDLE hDNSServer = (HANDLE)NULL;
     PCSTR pszAddress = NULL;
 
-    dwError = DNSGetPtrNameForAddr(
-                    &pszRecordName,
-                    pAddr);
+    dwError = DNSGetPtrZoneForAddr(&pszPtrZone, pAddr);
+    BAIL_ON_LWDNS_ERROR(dwError);
+
+    dwError = DNSGetPtrNameForAddr(&pszRecordName, pAddr);
     BAIL_ON_LWDNS_ERROR(dwError);
 
     dwError = DNSGetNameServers(
-                    pszRecordName,
+                    pszPtrZone,
                     &pszZone,
                     &pNameServerInfos,
                     &dwNumNSInfos);
@@ -444,7 +513,7 @@ DNSUpdatePtrSecure(
                     pszNameServer,
                     dwError);
             dwError = 0;
-
+            
             continue;
         }
 
@@ -459,6 +528,7 @@ DNSUpdatePtrSecure(
 
 cleanup:
     LWDNS_SAFE_FREE_STRING(pszZone);
+    LWDNS_SAFE_FREE_STRING(pszPtrZone);
     if (pNameServerInfos)
     {
         DNSFreeNameServerInfoArray(
@@ -543,15 +613,18 @@ DNSUpdateSecure(
                     pDNSSecureUpdateResponse,
                     &dwResponseCode);
         BAIL_ON_LWDNS_ERROR(dwError);
+
+        dwError = DNSMapRCode(dwResponseCode);
+        BAIL_ON_LWDNS_ERROR(dwError);
     }
     else
     {
+        dwError = DNSMapRCode(dwResponseCode);
+        BAIL_ON_LWDNS_ERROR(dwError);
+
         LWDNS_LOG_INFO("DNS Update (in-secure) succeeded");
     }
 
-    dwError = DNSMapRCode(dwResponseCode);
-    BAIL_ON_LWDNS_ERROR(dwError);
-
 cleanup:
 
     if (*pGSSContext != GSS_C_NO_CONTEXT)
@@ -789,7 +862,7 @@ DNSSendSecureUpdate(
     DWORD dwError = 0;
     PDNS_UPDATE_REQUEST pDNSUpdateRequest = NULL;
     PDNS_UPDATE_RESPONSE pDNSUpdateResponse = NULL;
-
+    
     LWDNS_LOG_INFO("Attempting DNS Update (secure)");
 
     dwError = DNSUpdateCreateARUpdateRequest(
