@@ -2,7 +2,34 @@
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
  * -*- mode: c, c-basic-offset: 4 -*- */
 
+/*
+ * Copyright Likewise Software
+ * All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or (at
+ * your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.  You should have received a copy of the GNU General
+ * Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
+
 #include "includes.h"
+#include <lsa/ad.h>
 
 typedef struct _ARGS {
     BOOLEAN bShowArguments;
@@ -497,9 +524,9 @@ cleanup:
     pArgs->dwAddressCount = dwAddressCount;
     pArgs->LogLevel = LogLevel;
     pArgs->pfnLogger = pfnLogger;
-
+    
     return dwError;
-
+    
 error:
 
     bShowArguments = FALSE;
@@ -584,7 +611,6 @@ GetDnsSuffixByHostname(
     PSTR pszHostDnsSuffix = NULL;
     struct hostent* pHost = NULL;
     PCSTR pszDot = NULL;
-    PCSTR pszFoundFqdn = NULL;
     PCSTR pszFoundDomain = NULL;
 
     pHost = gethostbyname(pszHostname);
@@ -602,7 +628,6 @@ GetDnsSuffixByHostname(
     pszDot = strchr(pHost->h_name, '.');
     if (pszDot)
     {
-        pszFoundFqdn = pHost->h_name;
         pszFoundDomain = pszDot + 1;
     }
     else
@@ -613,7 +638,6 @@ GetDnsSuffixByHostname(
             pszDot = strchr(pHost->h_aliases[i], '.');
             if (pszDot)
             {
-                pszFoundFqdn = pHost->h_aliases[i];
                 pszFoundDomain = pszDot + 1;
                 break;
             }
@@ -628,7 +652,7 @@ GetDnsSuffixByHostname(
 
     dwError = DNSAllocateString(pszFoundDomain, &pszHostDnsSuffix);
     BAIL_ON_LWDNS_ERROR(dwError);
-
+    
 cleanup:
     *ppszHostDnsSuffix = pszHostDnsSuffix;
 
@@ -701,41 +725,6 @@ error:
     goto cleanup;
 }
 
-static
-DWORD
-AllocateStringFromWC16String(
-    OUT PSTR* ppszOutputString,
-    IN PCWSTR pwszInputString
-    )
-{
-    DWORD dwError = 0;
-    PSTR pszResult = NULL;
-
-    pszResult = awc16stombs(pwszInputString);
-    if (!pszResult)
-    {
-        dwError = LWDNS_ERROR_STRING_CONV_FAILED;
-    }
-
-    *ppszOutputString = pszResult;
-
-    return dwError;
-}
-
-static
-VOID
-FreeStringFromWC16String(
-    IN OUT PSTR* ppszAllocatedString
-    )
-{
-    PSTR pszString = *ppszAllocatedString;
-    if (pszString)
-    {
-        free(pszString);
-        *ppszAllocatedString = NULL;
-    }
-}
-
 static
 DWORD
 SetupCredentials(
@@ -745,39 +734,39 @@ SetupCredentials(
 {
     DWORD dwError = 0;
     PSTR pszHostDnsSuffixResult = NULL;
-    PSTR pszHostDnsSuffix = NULL;
-    PSTR pszMachineAccountName = NULL;
-    PSTR pszDnsDomainName = NULL;
-    HANDLE hPasswordStore = NULL;
-    PLWPS_PASSWORD_INFO pPasswordInfo = NULL;
+    PCSTR pszHostDnsSuffix = NULL;
+    HANDLE hLsaConnection = NULL;
+    PLSA_MACHINE_PASSWORD_INFO_A pPasswordInfo = NULL;
 
-    dwError = LwpsOpenPasswordStore(
-                  LWPS_PASSWORD_STORE_DEFAULT,
-                  &hPasswordStore);
+    dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LWDNS_ERROR(dwError);
 
-    dwError = LwpsGetPasswordByHostName(
-                  hPasswordStore,
-                  pszHostname,
-                  &pPasswordInfo);
+    dwError = LsaAdGetMachinePasswordInfo(
+                    hLsaConnection,
+                    NULL,
+                    &pPasswordInfo);
     BAIL_ON_LWDNS_ERROR(dwError);
 
-    dwError = AllocateStringFromWC16String(
-                    &pszHostDnsSuffix,
-                    pPasswordInfo->pwszHostDnsDomain);
-    BAIL_ON_LWDNS_ERROR(dwError);
+    for (pszHostDnsSuffix = pPasswordInfo->Account.Fqdn;
+         *pszHostDnsSuffix;
+         pszHostDnsSuffix++)
+    {
+        if (*pszHostDnsSuffix == '.')
+        {
+            pszHostDnsSuffix++;
+            break;
+        }
+    }
 
-    dwError = AllocateStringFromWC16String(
-                    &pszMachineAccountName,
-                    pPasswordInfo->pwszMachineAccount);
-    BAIL_ON_LWDNS_ERROR(dwError);
+    if (!*pszHostDnsSuffix)
+    {
+        dwError = LWDNS_ERROR_UNEXPECTED;
+        BAIL_ON_LWDNS_ERROR(dwError);
+    }
 
-    dwError = AllocateStringFromWC16String(
-                    &pszDnsDomainName,
-                    pPasswordInfo->pwszDnsDomainName);
-    BAIL_ON_LWDNS_ERROR(dwError);
-
-    dwError = DNSKrb5Init(pszMachineAccountName, pszDnsDomainName);
+    dwError = DNSKrb5Init(
+                    pPasswordInfo->Account.SamAccountName,
+                    pPasswordInfo->Account.DnsDomainName);
     BAIL_ON_LWDNS_ERROR(dwError);
 
     dwError = DNSAllocateString(
@@ -786,18 +775,14 @@ SetupCredentials(
     BAIL_ON_LWDNS_ERROR(dwError);
 
 cleanup:
-    FreeStringFromWC16String(&pszHostDnsSuffix);
-    FreeStringFromWC16String(&pszMachineAccountName);
-    FreeStringFromWC16String(&pszDnsDomainName);
-
     if (pPasswordInfo)
     {
-        LwpsFreePasswordInfo(hPasswordStore, pPasswordInfo);
+        LsaAdFreeMachinePasswordInfo(pPasswordInfo);
     }
 
-    if (hPasswordStore)
+    if (hLsaConnection)
     {
-        LwpsClosePasswordStore(hPasswordStore);
+        LsaCloseServer(hLsaConnection);
     }
 
     *ppszHostDnsSuffix = pszHostDnsSuffixResult;
