@@ -70,125 +70,133 @@ NtpCtxCallComplete(
     void* data
     )
 {
-    PIO_CLIENT_ASYNC_CANCEL_CONTEXT pCancelContext = data;
+    PIO_CLIENT_ASYNC_CONTEXT pContext = data;
     NTSTATUS status = NtIpcLWMsgStatusToNtStatus(callStatus);
 
     if (status == STATUS_SUCCESS &&
-        pCancelContext->out.tag != pCancelContext->responseType)
+        pContext->out.tag != pContext->responseType)
     {
         status = STATUS_INTERNAL_ERROR;
     }
 
-    status = pCancelContext->pfnComplete(
-        status,
-        &pCancelContext->out.data,
-        pCancelContext->pData);
-    pCancelContext->pfnUserCallback(pCancelContext->pUserCallbackContext);
+    pContext->pfnComplete(
+        pContext,
+        status);
+    pContext->pControl->Callback(pContext->pControl->CallbackContext);
 
-    lwmsg_call_destroy_params(pCancelContext->pCall, &pCancelContext->out);
-    lwmsg_call_release(pCancelContext->pCall);
+    LwNtDereferenceAsyncCancelContext(&pContext);
+}
 
-    LwNtDereferenceAsyncCancelContext(&pCancelContext);
+static
+VOID
+NtpFreeClientAsyncContext(
+    PIO_CLIENT_ASYNC_CONTEXT pContext
+    )
+{
+    if (pContext)
+    {
+        if (pContext->pCall)
+        {
+            lwmsg_call_destroy_params(pContext->pCall, &pContext->out);
+            lwmsg_call_release(pContext->pCall);
+        }
+
+        RTL_FREE(&pContext);
+    }
+}
+
+static
+NTSTATUS
+NtpAllocAsyncContext(
+    OUT PVOID* ppContext,
+    size_t Size
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PIO_CLIENT_ASYNC_CONTEXT pContext = NULL;
+    static const LWMsgParams init = LWMSG_PARAMS_INITIALIZER;
+
+    status = RTL_ALLOCATE(&pContext, IO_CLIENT_ASYNC_CONTEXT, Size);
+    BAIL_ON_NT_STATUS(status);
+
+    pContext->in = init;
+    pContext->out = init;
+
+    *ppContext = (PVOID) pContext;
+
+error:
+
+    if (status)
+    {
+        *ppContext = NULL;
+    }
+
+    return status;
 }
 
 static
 NTSTATUS
 NtpCtxCallAsync(
-    IN PIO_CONTEXT pContext,
+    IN PIO_CLIENT_ASYNC_CONTEXT pContext,
     IN LWMsgTag RequestType,
     IN PVOID pRequest,
     IN LWMsgTag ResponseType,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK pControl,
-    IN IO_CLIENT_ASYNC_COMPLETE_FUNCTION pfnComplete,
-    IN PVOID pData
+    IN IO_CLIENT_ASYNC_COMPLETE_FUNCTION pfnComplete
     )
 {
-    NTSTATUS status = 0;
-    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
-    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
-    PIO_CLIENT_ASYNC_CANCEL_CONTEXT pCancelContext = NULL;
-    LWMsgCall* pCall = NULL;
+    NTSTATUS status = STATUS_SUCCESS;
 
-    status = LwIoContextAcquireCall(pContext, &pCall);
+    pContext->in.tag = RequestType;
+    pContext->in.data = pRequest;
+
+    status = LwIoConnectionAcquireCall(&pContext->pCall);
     BAIL_ON_NT_STATUS(status);
 
     if (pControl)
     {
-        status = RTL_ALLOCATE(&pCancelContext, IO_CLIENT_ASYNC_CANCEL_CONTEXT, sizeof(*pCancelContext));
-        BAIL_ON_NT_STATUS(status);
-
-        pCancelContext->lRefcount = 2;
-        pCancelContext->pCall = pCall;
-        pCall = NULL;
-        pCancelContext->pfnComplete = pfnComplete;
-        pCancelContext->pData = pData;
-        pCancelContext->pfnUserCallback = pControl->Callback;
-        pCancelContext->pUserCallbackContext = pControl->CallbackContext;
-
-        pCancelContext->responseType = ResponseType;
-        pCancelContext->in.tag = RequestType;
-        pCancelContext->in.data = pRequest;
-        pCancelContext->out.tag = LWMSG_TAG_INVALID;
-        pCancelContext->out.data = NULL;
+        pContext->lRefcount = 2;
+        pContext->pfnComplete = pfnComplete;
+        pContext->pControl = pControl;
+        pContext->responseType = ResponseType;
 
         status = NtIpcLWMsgStatusToNtStatus(lwmsg_call_dispatch(
-                                                pCancelContext->pCall,
-                                                &pCancelContext->in,
-                                                &pCancelContext->out,
-                                                NtpCtxCallComplete,
-                                                pControl));
-        switch (status)
-        {
-        case STATUS_PENDING:
-            pControl->AsyncCancelContext = pCancelContext;
-            pCancelContext = NULL;
-            break;
-        default:
-            if (status == STATUS_SUCCESS &&
-                pCancelContext->out.tag != pCancelContext->responseType)
-            {
-                status = STATUS_INTERNAL_ERROR;
-            }
-            status = pfnComplete(
-                status,
-                &pCancelContext->out.data,
-                pData);
-            BAIL_ON_NT_STATUS(status);
-        }
+            pContext->pCall,
+            &pContext->in,
+            &pContext->out,
+            NtpCtxCallComplete,
+            pContext));
     }
     else
     {
-        in.tag = RequestType;
-        in.data = pRequest;
+        status = NtIpcLWMsgStatusToNtStatus(lwmsg_call_dispatch(
+            pContext->pCall,
+            &pContext->in,
+            &pContext->out,
+            NULL,
+            NULL));
+    }
 
-        status = NtIpcLWMsgStatusToNtStatus(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
-        if (status == STATUS_SUCCESS && out.tag != ResponseType)
+    switch (status)
+    {
+    case STATUS_PENDING:
+        /* We should only get STATUS_PENDING from async calls */
+        assert(pControl);
+        pControl->AsyncCancelContext = pContext;
+        BAIL_ON_NT_STATUS(status);
+    case STATUS_SUCCESS:
+        if (pContext->out.tag != ResponseType)
         {
             status = STATUS_INTERNAL_ERROR;
+            BAIL_ON_NT_STATUS(status);
         }
-        status = pfnComplete(status, out.data, pData);
+        break;
+    default:
         BAIL_ON_NT_STATUS(status);
     }
 
 cleanup:
 
-    if (pCall)
-    {
-        lwmsg_call_destroy_params(pCall, &out);
-        lwmsg_call_release(pCall);
-    }
-
-    if (pCancelContext)
-    {
-        if (pCancelContext->pCall)
-        {
-            lwmsg_call_destroy_params(pCancelContext->pCall, &out);
-            lwmsg_call_release(pCancelContext->pCall);
-        }
-
-        RTL_FREE(&pCancelContext);
-    }
-
     return status;
 
 error:
@@ -212,7 +220,7 @@ NtpCtxCall(
 
     in.tag = RequestType;
     in.data = pRequest;
-
+    
     status = NtIpcLWMsgStatusToNtStatus(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_NT_STATUS(status);
 
@@ -221,15 +229,15 @@ NtpCtxCall(
         status = STATUS_INTERNAL_ERROR;
         BAIL_ON_NT_STATUS(status);
     }
-
+    
     *ppResponse = out.data;
-
+    
 cleanup:
 
     return status;
 
 error:
-
+    
     *ppResponse = NULL;
 
     NtpCtxFreeResponse(pCall, out.tag, out.data);
@@ -281,10 +289,29 @@ NtpCtxGetBufferResult(
 // Core I/O Operations
 //
 
+typedef struct CREATEPIPE_CONTEXT
+{
+    IO_ASYNC_CONTROL_BLOCK AsyncControl;
+    PIO_ASYNC_CONTROL_BLOCK pChain;
+    PIO_ECP_LIST pEcpList;
+} CREATEPIPE_CONTEXT, *PCREATEPIPE_CONTEXT;
+
+static
+VOID
+LwNtCreateNamedPipeComplete(
+    PVOID pParam
+    )
+{
+    PCREATEPIPE_CONTEXT pContext = (PCREATEPIPE_CONTEXT) pParam;
+
+    pContext->pChain->Callback(pContext->pChain->CallbackContext);
+
+    IoRtlEcpListFree(&pContext->pEcpList);
+    RTL_FREE(&pContext);
+}
+
 NTSTATUS
-LwNtCtxCreateNamedPipeFile(
-    IN PIO_CONTEXT pConnection,
-    IN LW_PIO_CREDS pSecurityToken,
+LwNtCreateNamedPipeFile(
     OUT PIO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -306,39 +333,51 @@ LwNtCtxCreateNamedPipeFile(
 {
     NTSTATUS status = 0;
     int EE = 0;
-    IO_FILE_HANDLE fileHandle = NULL;
-    IO_STATUS_BLOCK ioStatusBlock = { 0 };
     PIO_ECP_LIST ecpList = NULL;
-    IO_ECP_NAMED_PIPE pipeParams = { 0 };
+    PIO_ECP_NAMED_PIPE pPipeParams = NULL;
+    PCREATEPIPE_CONTEXT pContext = NULL;
 
-    status = IoRtlEcpListAllocate(&ecpList);
-    ioStatusBlock.Status = status;
+    status = RTL_ALLOCATE(&pPipeParams, IO_ECP_NAMED_PIPE, sizeof(*pPipeParams));
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pipeParams.NamedPipeType = NamedPipeType;
-    pipeParams.ReadMode = ReadMode;
-    pipeParams.CompletionMode = CompletionMode;
-    pipeParams.MaximumInstances = MaximumInstances;
-    pipeParams.InboundQuota = InboundQuota;
-    pipeParams.OutboundQuota = OutboundQuota;
+    status = IoRtlEcpListAllocate(&ecpList);
+    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    
+    pPipeParams->NamedPipeType = NamedPipeType;
+    pPipeParams->ReadMode = ReadMode;
+    pPipeParams->CompletionMode = CompletionMode;
+    pPipeParams->MaximumInstances = MaximumInstances;
+    pPipeParams->InboundQuota = InboundQuota;
+    pPipeParams->OutboundQuota = OutboundQuota;
     if (DefaultTimeout)
     {
-        pipeParams.DefaultTimeout = *DefaultTimeout;
-        pipeParams.HaveDefaultTimeout = TRUE;
+        pPipeParams->DefaultTimeout = *DefaultTimeout;
+        pPipeParams->HaveDefaultTimeout = TRUE;
     }
 
     status = IoRtlEcpListInsert(ecpList,
                                 IO_ECP_TYPE_NAMED_PIPE,
-                                &pipeParams,
-                                sizeof(pipeParams),
-                                NULL);
-    ioStatusBlock.Status = status;
+                                pPipeParams,
+                                sizeof(*pPipeParams),
+                                LwRtlMemoryFree);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    pPipeParams = NULL;
 
-    status = NtCtxCreateFile(
-                    pConnection,
-                    pSecurityToken,
-                    &fileHandle,
+    if (AsyncControlBlock)
+    {
+        status = RTL_ALLOCATE(&pContext, CREATEPIPE_CONTEXT, sizeof(*pContext));
+        GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+
+        pContext->pEcpList = ecpList;
+        pContext->pChain = AsyncControlBlock;
+        pContext->AsyncControl.Callback = LwNtCreateNamedPipeComplete;
+        pContext->AsyncControl.CallbackContext = pContext;
+
+        AsyncControlBlock = &pContext->AsyncControl;
+    }
+
+    status = NtCreateFile(
+                    FileHandle,
                     AsyncControlBlock,
                     IoStatusBlock,
                     FileName,
@@ -352,27 +391,87 @@ LwNtCtxCreateNamedPipeFile(
                     CreateOptions,
                     NULL,
                     0,
-                    ecpList);
+                    ecpList,
+                    NULL);
+
+    if (pContext)
+    {
+        pContext->pChain->AsyncCancelContext = pContext->AsyncControl.AsyncCancelContext;
+    }
+
 cleanup:
 
-    IoRtlEcpListFree(&ecpList);
+    if (status != STATUS_PENDING)
+    {
+        if (pContext)
+        {
+            IoRtlEcpListFree(&pContext->pEcpList);
+            RTL_FREE(&pContext);
+        }
+        else
+        {
+            IoRtlEcpListFree(&ecpList);
+        }
 
-    *FileHandle = fileHandle;
-    *IoStatusBlock = ioStatusBlock;
+        IoStatusBlock->Status = status;
+
+        RTL_FREE(&pPipeParams);
+    }
 
     LOG_LEAVE_IF_STATUS_EE(status, EE);
     return status;
 }
 
+typedef struct CREATEFILE_CONTEXT
+{
+    IO_CLIENT_ASYNC_CONTEXT Base;
+    NT_IPC_MESSAGE_CREATE_FILE Request;
+    OUT PIO_STATUS_BLOCK IoStatusBlock;
+    OUT PIO_FILE_HANDLE FileHandle;
+} CREATEFILE_CONTEXT, *PCREATEFILE_CONTEXT;
+
+static
+VOID
+LwNtCreateFileComplete(
+    PIO_CLIENT_ASYNC_CONTEXT pBase,
+    NTSTATUS status
+    )
+{
+    PCREATEFILE_CONTEXT pContext = (PCREATEFILE_CONTEXT) pBase;
+    PNT_IPC_MESSAGE_CREATE_FILE_RESULT pResponse = pBase->out.data;
+
+    if (status == STATUS_SUCCESS)
+    {
+        *pContext->FileHandle = pResponse->FileHandle;
+        pResponse->FileHandle = NULL;
+        pContext->IoStatusBlock->Status = pResponse->Status;
+        pContext->IoStatusBlock->CreateResult = pResponse->CreateResult;
+
+        status = pContext->IoStatusBlock->Status;
+    }
+    else
+    {
+        pContext->IoStatusBlock->Status = status;
+    }
+
+    if (pContext->Request.pSecurityToken)
+    {
+        LwIoDeleteCreds(pContext->Request.pSecurityToken);
+    }
+
+    if (pContext->Request.EcpList)
+    {
+        RTL_FREE(&pContext->Request.EcpList);
+    } 
+}
+
 NTSTATUS
-LwNtCtxCreateFile(
-    IN PIO_CONTEXT pConnection,
-    IN LW_PIO_CREDS pSecurityToken,
+LwNtCreateFile(
     OUT PIO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     IN PIO_FILE_NAME FileName,
-    IN OPTIONAL PVOID SecurityDescriptor, // TBD
+    IN OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor,
     IN OPTIONAL PVOID SecurityQualityOfService, // TBD
     IN ACCESS_MASK DesiredAccess,
     IN OPTIONAL LONG64 AllocationSize,
@@ -382,196 +481,210 @@ LwNtCtxCreateFile(
     IN FILE_CREATE_OPTIONS CreateOptions,
     IN OPTIONAL PVOID EaBuffer, // PFILE_FULL_EA_INFORMATION
     IN ULONG EaLength,
-    IN OPTIONAL PIO_ECP_LIST EcpList
+    IN OPTIONAL PIO_ECP_LIST EcpList,
+    IN OPTIONAL LW_PIO_CREDS pCreds
     )
 {
     NTSTATUS status = 0;
     int EE = 0;
-    LWMsgCall* pCall = NULL;
-    const LWMsgTag requestType = NT_IPC_MESSAGE_TYPE_CREATE_FILE;
-    const LWMsgTag responseType = NT_IPC_MESSAGE_TYPE_CREATE_FILE_RESULT;
-    NT_IPC_MESSAGE_CREATE_FILE request = { 0 };
-    PNT_IPC_MESSAGE_CREATE_FILE_RESULT pResponse = NULL;
-    PVOID pReply = NULL;
-    IO_FILE_HANDLE fileHandle = NULL;
-    IO_STATUS_BLOCK ioStatusBlock = { 0 };
     PIO_CREDS pActiveCreds = NULL;
-    PIO_CREDS pResolvedSecurityToken = NULL;
+    PCREATEFILE_CONTEXT pCreateContext = NULL;
 
-    if (AsyncControlBlock)
-    {
-        status = STATUS_INVALID_PARAMETER;
-        GOTO_CLEANUP_EE(EE);
-    }
-
-    if (!pSecurityToken)
+    if (!pCreds)
     {
         status = LwIoGetActiveCreds(FileName->FileName, &pActiveCreds);
         GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-        pSecurityToken = pActiveCreds;
+        pCreds = pActiveCreds;
     }
 
-    status = LwIoResolveCreds(pSecurityToken, &pResolvedSecurityToken);
+    status = NtpAllocAsyncContext(OUT_PPVOID(&pCreateContext), sizeof(*pCreateContext));
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    pCreateContext->IoStatusBlock = IoStatusBlock;
+    pCreateContext->FileHandle = FileHandle;
+
+    status = LwIoResolveCreds(pCreds, &pCreateContext->Request.pSecurityToken);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    request.pSecurityToken = pResolvedSecurityToken;
-    request.FileName = *FileName;
-    request.DesiredAccess = DesiredAccess;
-    request.AllocationSize = AllocationSize;
-    request.FileAttributes = FileAttributes;
-    request.ShareAccess = ShareAccess;
-    request.CreateDisposition = CreateDisposition;
-    request.CreateOptions = CreateOptions;
-    request.EaBuffer = EaBuffer;
-    request.EaLength = EaLength;
-    // TODO -- SDs, etc.
-    request.EcpCount = IoRtlEcpListGetCount(EcpList);
-    if (request.EcpCount)
+    pCreateContext->Request.FileName = *FileName;
+    pCreateContext->Request.DesiredAccess = DesiredAccess;
+    pCreateContext->Request.AllocationSize = AllocationSize;
+    pCreateContext->Request.FileAttributes = FileAttributes;
+    pCreateContext->Request.ShareAccess = ShareAccess;
+    pCreateContext->Request.CreateDisposition = CreateDisposition;
+    pCreateContext->Request.CreateOptions = CreateOptions;
+    pCreateContext->Request.EaBuffer = EaBuffer;
+    pCreateContext->Request.EaLength = EaLength;
+    if (SecurityDescriptor)
+    {
+        pCreateContext->Request.SecurityDescriptor = SecurityDescriptor;
+        pCreateContext->Request.SecDescLength =
+                    RtlLengthSecurityDescriptorRelative(SecurityDescriptor);
+    }
+    pCreateContext->Request.EcpCount = IoRtlEcpListGetCount(EcpList);
+    if (pCreateContext->Request.EcpCount)
     {
         PCSTR pszType = NULL;
         ULONG ecpIndex = 0;
 
-        status = RTL_ALLOCATE(&request.EcpList, NT_IPC_HELPER_ECP, sizeof(*request.EcpList) * request.EcpCount);
-        ioStatusBlock.Status = status;
+        status = RTL_ALLOCATE(
+            &pCreateContext->Request.EcpList,
+            NT_IPC_HELPER_ECP,
+            sizeof(*pCreateContext->Request.EcpList) * pCreateContext->Request.EcpCount);
         GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-        while (ecpIndex < request.EcpCount)
+        while (ecpIndex < pCreateContext->Request.EcpCount)
         {
             status = IoRtlEcpListGetNext(
                             EcpList,
                             pszType,
-                            &request.EcpList[ecpIndex].pszType,
-                            &request.EcpList[ecpIndex].pData,
-                            &request.EcpList[ecpIndex].Size);
-            ioStatusBlock.Status = status;
+                            &pCreateContext->Request.EcpList[ecpIndex].pszType,
+                            &pCreateContext->Request.EcpList[ecpIndex].pData,
+                            &pCreateContext->Request.EcpList[ecpIndex].Size);
             GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-            pszType = request.EcpList[ecpIndex].pszType;
+            pszType = pCreateContext->Request.EcpList[ecpIndex].pszType;
             ecpIndex++;
         }
 
-        assert(ecpIndex == request.EcpCount);
+        assert(ecpIndex == pCreateContext->Request.EcpCount);
         status = STATUS_SUCCESS;
     }
 
-    status = NtpCtxCall(pCall,
-                        requestType,
-                        &request,
-                        responseType,
-                        &pReply);
-    ioStatusBlock.Status = status;
+    status = NtpCtxCallAsync(
+        &pCreateContext->Base,
+        NT_IPC_MESSAGE_TYPE_CREATE_FILE,
+        &pCreateContext->Request,
+        NT_IPC_MESSAGE_TYPE_CREATE_FILE_RESULT,
+        AsyncControlBlock,
+        LwNtCreateFileComplete);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pResponse = (PNT_IPC_MESSAGE_CREATE_FILE_RESULT) pReply;
-
-    fileHandle = pResponse->FileHandle;
-    ioStatusBlock.Status = pResponse->Status;
-    ioStatusBlock.CreateResult = pResponse->CreateResult;
-
-    status = ioStatusBlock.Status;
-
-    /* Unlink file handle from response message so it does not get freed */
-    pResponse->FileHandle = NULL;
-
 cleanup:
 
-    if (pResolvedSecurityToken)
-    {
-        LwIoDeleteCreds(pResolvedSecurityToken);
-    }
-
     if (pActiveCreds)
     {
         LwIoDeleteCreds(pActiveCreds);
     }
 
-    if (status)
+    if (status != STATUS_PENDING)
     {
-        // TODO !!!! -- ASK BRIAN ABOUT FileHandle and failures...
-        assert(!fileHandle);
-        assert(!pResponse || !pResponse->FileHandle);
+        if (pCreateContext)
+        {
+            LwNtCreateFileComplete(&pCreateContext->Base, status);
+            status = IoStatusBlock->Status;
+            NtpFreeClientAsyncContext(&pCreateContext->Base);
+        }
+        else
+        {
+            IoStatusBlock->Status = status;
+        }
     }
 
-    RTL_FREE(&request.EcpList);
-
-    if (pCall)
-    {
-        NtpCtxFreeResponse(pCall, responseType, pResponse);
-        lwmsg_call_release(pCall);
-    }
-
-    *FileHandle = fileHandle;
-    *IoStatusBlock = ioStatusBlock;
-
     LOG_LEAVE_IF_STATUS_EE(status, EE);
-
+    
     return status;
 }
 
+typedef struct CLOSEFILE_CONTEXT
+{
+    IO_CLIENT_ASYNC_CONTEXT Base;
+    LWMsgSession* pSession;
+    NT_IPC_MESSAGE_GENERIC_FILE Request;
+    PIO_STATUS_BLOCK IoStatusBlock;
+} CLOSEFILE_CONTEXT, *PCLOSEFILE_CONTEXT;
+
+static
+VOID
+LwNtCloseFileComplete(
+    PIO_CLIENT_ASYNC_CONTEXT pBase,
+    NTSTATUS status
+    )
+{
+    PCLOSEFILE_CONTEXT pContext = (PCLOSEFILE_CONTEXT) pBase;
+    PNT_IPC_MESSAGE_GENERIC_FILE_IO_RESULT pResponse = pBase->out.data;
+
+    if (status == STATUS_SUCCESS)
+    {
+        status = pResponse->Status;
+    }
+
+    pContext->IoStatusBlock->Status = status;
+    lwmsg_session_release_handle(pContext->pSession, pContext->Request.FileHandle);
+}
+
 NTSTATUS
-LwNtCtxCloseFile(
-    IN PIO_CONTEXT pConnection,
-    IN IO_FILE_HANDLE FileHandle
+LwNtAsyncCloseFile(
+    IN IO_FILE_HANDLE FileHandle,
+    IN OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
+    OUT PIO_STATUS_BLOCK IoStatusBlock
     )
 {
     NTSTATUS status = 0;
     int EE = 0;
-    const LWMsgTag requestType = NT_IPC_MESSAGE_TYPE_CLOSE_FILE;
-    const LWMsgTag responseType = NT_IPC_MESSAGE_TYPE_CLOSE_FILE_RESULT;
-    NT_IPC_MESSAGE_GENERIC_FILE request = { 0 };
-    PNT_IPC_MESSAGE_GENERIC_FILE_IO_RESULT pResponse = NULL;
-    PVOID pReply = NULL;
-    // In case we add IO_STATUS_BLOCK to close later, which we may want/need.
-    IO_STATUS_BLOCK ioStatusBlock = { 0 };
-    LWMsgCall* pCall = NULL;
+    PCLOSEFILE_CONTEXT pContext = NULL;
+    IO_CONNECTION connection = {0};
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoAcquireConnection(&connection);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    request.FileHandle = FileHandle;
-
-    status = NtpCtxCall(pCall,
-                        requestType,
-                        &request,
-                        responseType,
-                        &pReply);
-    ioStatusBlock.Status = status;
+    status = NtpAllocAsyncContext(OUT_PPVOID(&pContext), sizeof(*pContext));
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pResponse = (PNT_IPC_MESSAGE_GENERIC_FILE_IO_RESULT) pReply;
+    pContext->IoStatusBlock = IoStatusBlock;
+    pContext->pSession = connection.pSession;
+    pContext->Request.FileHandle = FileHandle;
 
-    status = NtpCtxGetIoResult(&ioStatusBlock, pResponse);
-    assert(0 == ioStatusBlock.BytesTransferred);
-    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    status = NtpCtxCallAsync(
+        &pContext->Base,
+        NT_IPC_MESSAGE_TYPE_CLOSE_FILE,
+        &pContext->Request,
+        NT_IPC_MESSAGE_TYPE_CLOSE_FILE_RESULT,
+        AsyncControlBlock,
+        LwNtCloseFileComplete);
 
-cleanup:
+ cleanup:
 
-    /* Release file handle regardless of result */
-    lwmsg_session_release_handle(pConnection->pSession, FileHandle);
+     if (status != STATUS_PENDING)
+     {
+         if (pContext)
+         {
+             LwNtCloseFileComplete(&pContext->Base, status);
+             status = IoStatusBlock->Status;
+             NtpFreeClientAsyncContext(&pContext->Base);
+         }
+         else
+         {
+             IoStatusBlock->Status = status;
+             lwmsg_session_release_handle(connection.pSession, FileHandle);
+         }
+     }
 
-    if (pCall)
-    {
-        NtpCtxFreeResponse(pCall, responseType, pResponse);
-        lwmsg_call_release(pCall);
-    }
+     LwIoReleaseConnection(&connection);
 
-    LOG_LEAVE_IF_STATUS_EE(status, EE);
-    return status;
+     LOG_LEAVE_IF_STATUS_EE(status, EE);
+     return status;
 }
 
 NTSTATUS
-LwNtCtxReadFile(
-    IN PIO_CONTEXT pConnection,
+LwNtCloseFile(
+    IN IO_FILE_HANDLE FileHandle
+    )
+{
+    IO_STATUS_BLOCK ioStatusBlock = { 0 };
+
+    return LwNtAsyncCloseFile(FileHandle, NULL, &ioStatusBlock);
+}
+
+NTSTATUS
+LwNtReadFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     OUT PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -585,7 +698,7 @@ LwNtCtxReadFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -627,14 +740,13 @@ cleanup:
 }
 
 NTSTATUS
-LwNtCtxWriteFile(
-    IN PIO_CONTEXT pConnection,
+LwNtWriteFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     IN PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -648,7 +760,7 @@ LwNtCtxWriteFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -692,8 +804,7 @@ cleanup:
 }
 
 NTSTATUS 
-LwNtCtxDeviceIoControlFile(
-    IN PIO_CONTEXT pConnection,
+LwNtDeviceIoControlFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -714,7 +825,7 @@ LwNtCtxDeviceIoControlFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -758,43 +869,38 @@ cleanup:
 
 typedef struct FSCONTROL_CONTEXT
 {
-    NT_IPC_MESSAGE_GENERIC_CONTROL_FILE request;
+    IO_CLIENT_ASYNC_CONTEXT Base;
+    NT_IPC_MESSAGE_GENERIC_CONTROL_FILE Request;
     OUT PIO_STATUS_BLOCK IoStatusBlock;
     OUT PVOID OutputBuffer;
     IN ULONG OutputBufferLength;
 } FSCONTROL_CONTEXT, *PFSCONTROL_CONTEXT;
 
 static
-NTSTATUS
-LwNtCtxFsControlFileComplete(
-    NTSTATUS status,
-    PVOID pOut,
-    PVOID pData
+VOID
+LwNtFsControlFileComplete(
+    PIO_CLIENT_ASYNC_CONTEXT pBase,
+    NTSTATUS status
     )
 {
-    PFSCONTROL_CONTEXT pContext = pData;
+    PFSCONTROL_CONTEXT pContext = (PFSCONTROL_CONTEXT) pBase;
 
     if (status == STATUS_SUCCESS)
     {
-        NtpCtxGetBufferResult(
+        (void) NtpCtxGetBufferResult(
             pContext->IoStatusBlock,
             pContext->OutputBuffer,
             pContext->OutputBufferLength,
-            pOut);
+            pBase->out.data);
     }
     else
     {
         pContext->IoStatusBlock->Status = status;
     }
-
-    RTL_FREE(&pContext);
-
-    return status;
 }
 
 NTSTATUS
-LwNtCtxFsControlFile(
-    IN PIO_CONTEXT pConnection,
+LwNtFsControlFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -809,38 +915,50 @@ LwNtCtxFsControlFile(
     int EE = 0;
     PFSCONTROL_CONTEXT pControlContext = NULL;
 
-    status = RTL_ALLOCATE(&pControlContext, FSCONTROL_CONTEXT, sizeof(*pControlContext));
+    status = NtpAllocAsyncContext(OUT_PPVOID(&pControlContext), sizeof(*pControlContext));
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pControlContext->request.FileHandle = FileHandle;
-    pControlContext->request.ControlCode = FsControlCode;
-    pControlContext->request.InputBuffer = InputBuffer;
-    pControlContext->request.InputBufferLength = InputBufferLength;
-    pControlContext->request.OutputBufferLength = OutputBufferLength;
+    pControlContext->Request.FileHandle = FileHandle;
+    pControlContext->Request.ControlCode = FsControlCode;
+    pControlContext->Request.InputBuffer = InputBuffer;
+    pControlContext->Request.InputBufferLength = InputBufferLength;
+    pControlContext->Request.OutputBufferLength = OutputBufferLength;
     pControlContext->IoStatusBlock = IoStatusBlock;
     pControlContext->OutputBuffer = OutputBuffer;
     pControlContext->OutputBufferLength = OutputBufferLength;
 
     status = NtpCtxCallAsync(
-        pConnection,
+        &pControlContext->Base,
         NT_IPC_MESSAGE_TYPE_FS_CONTROL_FILE,
-        &pControlContext->request,
+        &pControlContext->Request,
         NT_IPC_MESSAGE_TYPE_FS_CONTROL_FILE_RESULT,
         AsyncControlBlock,
-        LwNtCtxFsControlFileComplete,
-        pControlContext);
+        LwNtFsControlFileComplete);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
 cleanup:
 
+    if (status != STATUS_PENDING)
+    {
+        if (pControlContext)
+        {
+            LwNtFsControlFileComplete(&pControlContext->Base, status);
+            status = IoStatusBlock->Status;
+            NtpFreeClientAsyncContext(&pControlContext->Base);
+        }
+        else
+        {
+            IoStatusBlock->Status = status;
+        }
+    }
+
     LOG_LEAVE_IF_STATUS_EE(status, EE);
 
     return status;
 }
 
 NTSTATUS
-LwNtCtxFlushBuffersFile(
-    IN PIO_CONTEXT pConnection,
+LwNtFlushBuffersFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock
@@ -856,7 +974,7 @@ LwNtCtxFlushBuffersFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -895,8 +1013,7 @@ cleanup:
 }
 
 NTSTATUS
-LwNtCtxQueryInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtQueryInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -915,7 +1032,7 @@ LwNtCtxQueryInformationFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -956,8 +1073,7 @@ cleanup:
 }
 
 NTSTATUS 
-LwNtCtxSetInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtSetInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -976,7 +1092,7 @@ LwNtCtxSetInformationFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1023,7 +1139,7 @@ cleanup:
 
 #if 0
 NTSTATUS
-LwNtCtxQueryFullAttributesFile(
+LwNtQueryFullAttributesFile(
     IN PIO_CONTEXT pConnection,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1033,8 +1149,7 @@ LwNtCtxQueryFullAttributesFile(
 #endif
 
 NTSTATUS 
-LwNtCtxQueryDirectoryFile(
-    IN PIO_CONTEXT pConnection,
+LwNtQueryDirectoryFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1056,7 +1171,7 @@ LwNtCtxQueryDirectoryFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1099,9 +1214,8 @@ cleanup:
     return status;
 }
 
-NTSTATUS
-LwNtCtxReadDirectoryChangeFile(
-    IN PIO_CONTEXT pConnection,
+NTSTATUS 
+LwNtReadDirectoryChangeFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1122,7 +1236,7 @@ LwNtCtxReadDirectoryChangeFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1165,8 +1279,7 @@ cleanup:
 }
 
 NTSTATUS
-LwNtCtxQueryVolumeInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtQueryVolumeInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1185,7 +1298,7 @@ LwNtCtxQueryVolumeInformationFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1226,8 +1339,7 @@ cleanup:
 }
 
 NTSTATUS
-LwNtCtxSetVolumeInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtSetVolumeInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1237,8 +1349,7 @@ LwNtCtxSetVolumeInformationFile(
     );
 
 NTSTATUS 
-LwNtCtxLockFile(
-    IN PIO_CONTEXT pConnection,
+LwNtLockFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1252,14 +1363,14 @@ LwNtCtxLockFile(
     NTSTATUS status = 0;
     int EE = 0;
     const LWMsgTag requestType = NT_IPC_MESSAGE_TYPE_LOCK_FILE;
-    const LWMsgTag responseType = NT_IPC_MESSAGE_TYPE_LOCK_FILE_RESULT;
+    const LWMsgTag responseType = NT_IPC_MESSAGE_TYPE_LOCK_FILE_RESULT;    
     NT_IPC_MESSAGE_LOCK_FILE request = { 0 };
     PNT_IPC_MESSAGE_GENERIC_FILE_IO_RESULT pResponse = NULL;
     PVOID pReply = NULL;
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1273,7 +1384,7 @@ LwNtCtxLockFile(
     request.Length = Length;
     request.Key = Key;
     request.FailImmediately = FailImmediately;
-    request.ExclusiveLock = ExclusiveLock;
+    request.ExclusiveLock = ExclusiveLock;    
 
     status = NtpCtxCall(pCall,
                         requestType,
@@ -1304,8 +1415,7 @@ cleanup:
 
 
 NTSTATUS 
-LwNtCtxUnlockFile(
-    IN PIO_CONTEXT pConnection,
+LwNtUnlockFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1324,7 +1434,7 @@ LwNtCtxUnlockFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1374,7 +1484,7 @@ cleanup:
 
 #if 0
 NTSTATUS
-LwNtCtxRemoveDirectoryFile(
+LwNtRemoveDirectoryFile(
     IN PIO_CONTEXT pConnection,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1386,7 +1496,7 @@ LwNtCtxRemoveDirectoryFile(
 }
 
 NTSTATUS
-LwNtCtxDeleteFile(
+LwNtDeleteFile(
     IN PIO_CONTEXT pConnection,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1398,7 +1508,7 @@ LwNtCtxDeleteFile(
 }
 
 NTSTATUS
-LwNtCtxLinkFile(
+LwNtLinkFile(
     IN PIO_CONTEXT pConnection,
     IN PIO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
@@ -1407,7 +1517,7 @@ LwNtCtxLinkFile(
     );
 
 NTSTATUS
-LwNtCtxRenameFile(
+LwNtRenameFile(
     IN PIO_CONTEXT pConnection,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1421,8 +1531,7 @@ LwNtCtxRenameFile(
 //
 
 NTSTATUS
-LwNtCtxQueryQuotaInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtQueryQuotaInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1436,8 +1545,7 @@ LwNtCtxQueryQuotaInformationFile(
     );
 
 NTSTATUS
-LwNtCtxSetQuotaInformationFile(
-    IN PIO_CONTEXT pConnection,
+LwNtSetQuotaInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1446,8 +1554,7 @@ LwNtCtxSetQuotaInformationFile(
     );
 
 NTSTATUS
-LwNtCtxQuerySecurityFile(
-    IN PIO_CONTEXT pConnection,
+LwNtQuerySecurityFile(
     IN IO_FILE_HANDLE Handle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1466,7 +1573,7 @@ LwNtCtxQuerySecurityFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1507,8 +1614,7 @@ cleanup:
 }
 
 NTSTATUS
-LwNtCtxSetSecurityFile(
-    IN PIO_CONTEXT pConnection,
+LwNtSetSecurityFile(
     IN IO_FILE_HANDLE Handle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
@@ -1527,7 +1633,7 @@ LwNtCtxSetSecurityFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     LWMsgCall* pCall = NULL;
 
-    status = LwIoContextAcquireCall(pConnection, &pCall);
+    status = LwIoConnectionAcquireCall(&pCall);
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
     if (AsyncControlBlock)
@@ -1586,7 +1692,8 @@ LwNtDereferenceAsyncCancelContext(
 {
     if (LwInterlockedDecrement(&(*ppAsyncCancelContext)->lRefcount) == 0)
     {
-        RTL_FREE(ppAsyncCancelContext);
+        NtpFreeClientAsyncContext(*ppAsyncCancelContext);
+        *ppAsyncCancelContext = NULL;
     }
 }
 
