@@ -31,11 +31,11 @@
 #ifndef __STRUCTS_H__
 #define __STRUCTS_H__
 
-struct __LW_IO_CONTEXT
+typedef struct __LW_IO_CONNECTION
 {
-    LWMsgClient* pClient;
+    LWMsgPeer* pClient;
     LWMsgSession* pSession;
-};
+} IO_CONNECTION, *PIO_CONNECTION;
 
 typedef struct __LW_IO_THREAD_STATE
 {
@@ -49,14 +49,12 @@ typedef struct IO_PATH_CREDS
     LW_LIST_LINKS link;
 } IO_PATH_CREDS, *PIO_PATH_CREDS;
 
-
-typedef NTSTATUS
+typedef VOID
 (*IO_CLIENT_ASYNC_COMPLETE_FUNCTION) (
-    NTSTATUS status,
-    PVOID pOut,
-    PVOID pData
+    struct _IO_ASYNC_CANCEL_CONTEXT* pContext,
+    NTSTATUS status
     );
-
+    
 typedef struct _IO_ASYNC_CANCEL_CONTEXT
 {
     LONG volatile lRefcount;
@@ -65,9 +63,7 @@ typedef struct _IO_ASYNC_CANCEL_CONTEXT
     LWMsgParams out;
     LWMsgTag responseType;
     IO_CLIENT_ASYNC_COMPLETE_FUNCTION pfnComplete;
-    PIO_ASYNC_COMPLETE_CALLBACK pfnUserCallback;
-    PVOID pUserCallbackContext;
-    PVOID pData;
-} IO_CLIENT_ASYNC_CANCEL_CONTEXT, *PIO_CLIENT_ASYNC_CANCEL_CONTEXT;
+    PIO_ASYNC_CONTROL_BLOCK pControl;
+} IO_CLIENT_ASYNC_CONTEXT, *PIO_CLIENT_ASYNC_CONTEXT;
 
 #endif /* __STRUCTS_H__ */
