@@ -95,7 +95,7 @@ LwIoFindPathCreds(
     size_t pathLength = 0;
     size_t prefixLength = 0;
     PIO_PATH_CREDS pCreds = NULL;
-    PLW_LIST_LINKS pLink = NULL;
+    PLW_LIST_LINKS pLink = NULL;  
 
     Status = LwIoNormalizePath(pwszPath, &pwszNormal);
     BAIL_ON_NT_STATUS(Status);
@@ -105,7 +105,7 @@ LwIoFindPathCreds(
     while ((pLink = LwListTraverse(&gPathCreds, pLink)))
     {
         pCreds = LW_STRUCT_FROM_FIELD(pLink, IO_PATH_CREDS, link);
-
+        
         prefixLength = LwRtlWC16StringNumChars(pCreds->pwszPathPrefix);
 
         if ((bPrecise && LwRtlWC16StringIsEqual(pwszNormal, pCreds->pwszPathPrefix, TRUE)) ||
@@ -141,15 +141,14 @@ LwIoDeletePathCreds(
 {
     if (pPathCreds)
     {
-        if (pPathCreds->pwszPathPrefix)
-        {
-            LwRtlMemoryFree(pPathCreds->pwszPathPrefix);
-        }
+        RTL_FREE(&pPathCreds->pwszPathPrefix);
 
         if (pPathCreds->pCreds)
         {
             LwIoDeleteCreds(pPathCreds->pCreds);
         }
+
+        RTL_FREE(&pPathCreds);
     }
 }
 
@@ -166,7 +165,7 @@ LwIoThreadStateDestruct(
         {
             LwIoDeleteCreds(pState->pCreds);
         }
-
+        
         LwIoFreeMemory(pState);
     }
 }
@@ -191,6 +190,12 @@ LwIoCreateDefaultKrb5Creds(
     krb5Error = krb5_init_context(&pKrb5Context);
     if (krb5Error)
     {
+        if (krb5Error == KRB5_CONFIG_BADFORMAT)
+        {
+            /* If krb5.conf is corrupted, give up */
+            goto cleanup;
+        }
+
         Status = STATUS_INSUFFICIENT_RESOURCES;
         BAIL_ON_NT_STATUS(Status);
     }
@@ -303,7 +308,7 @@ LwIoThreadInit(
 {
     NTSTATUS Status = 0;
     BOOL bInLock = FALSE;
-
+    
     LWIO_LOCK_MUTEX(bInLock, &gLock);
 
     if (!gpLwIoProtocol)
@@ -328,7 +333,7 @@ LwIoThreadInit(
     {
         Status = NtIpcLWMsgStatusToNtStatus(lwmsg_peer_new(NULL, gpLwIoProtocol, &gpClient));
         BAIL_ON_NT_STATUS(Status);
-
+        
         Status = NtIpcLWMsgStatusToNtStatus(lwmsg_peer_add_connect_endpoint(
                                                 gpClient,
                                                 LWMSG_ENDPOINT_LOCAL,
@@ -440,9 +445,9 @@ LwIoSetPathCreds(
     PIO_PATH_CREDS pPathCreds = NULL;
     PIO_CREDS pCredCopy = NULL;
     BOOL bInLock = FALSE;
-
+    
     LWIO_LOCK_MUTEX(bInLock, &gLock);
-
+    
     Status = LwIoFindPathCreds(pwszPathPrefix, TRUE, &pPathCreds);
     BAIL_ON_NT_STATUS(Status);
 
@@ -450,12 +455,12 @@ LwIoSetPathCreds(
     {
         Status = LwIoCopyCreds(pCreds, &pCredCopy);
         BAIL_ON_NT_STATUS(Status);
-
+        
         if (pPathCreds->pCreds)
         {
             LwIoDeleteCreds(pPathCreds->pCreds);
         }
-
+        
         pPathCreds->pCreds = pCredCopy;
         pCredCopy = NULL;
         pPathCreds = NULL;
@@ -464,12 +469,12 @@ LwIoSetPathCreds(
     {
         Status = RTL_ALLOCATE(&pPathCreds, IO_PATH_CREDS, sizeof(IO_PATH_CREDS));
         BAIL_ON_NT_STATUS(Status);
-
+        
         LwListInit(&pPathCreds->link);
-
+        
         Status = LwIoNormalizePath(pwszPathPrefix, &pPathCreds->pwszPathPrefix);
         BAIL_ON_NT_STATUS(Status);
-
+        
         Status = LwIoCopyCreds(pCreds, &pPathCreds->pCreds);
         BAIL_ON_NT_STATUS(Status);
 
@@ -531,7 +536,7 @@ LwIoGetActiveCreds(
         Status = LwIoCopyCreds(gpProcessCreds, &pCreds);
         BAIL_ON_NT_STATUS(Status);
     }
-
+    
     *ppToken = pCreds;
 
 error:
@@ -542,68 +547,67 @@ error:
 }
 
 NTSTATUS
-LwIoAcquireContext(
-    OUT PIO_CONTEXT pContext
+LwIoAcquireConnection(
+    OUT PIO_CONNECTION pConnection
     )
 {
     NTSTATUS Status = STATUS_SUCCESS;
 
     Status = LwIoThreadInit();
     BAIL_ON_NT_STATUS(Status);
-
-    pContext->pClient = gpClient;
-    pContext->pSession = gpSession;
+    
+    pConnection->pClient = gpClient;
+    pConnection->pSession = gpSession;
 
 error:
-
+    
     return Status;
 }
 
 NTSTATUS
-LwIoReleaseContext(
-    IN OUT PIO_CONTEXT pContext
+LwIoReleaseConnection(
+    IN OUT PIO_CONNECTION pConnection
     )
 {
     NTSTATUS Status = STATUS_SUCCESS;
-
-    memset(pContext, 0, sizeof(*pContext));
-
+ 
+    memset(pConnection, 0, sizeof(*pConnection));
+   
     return Status;
 }
 
-NTSTATUS
-LwIoOpenContextShared(
-    PIO_CONTEXT* ppContext
-    )
+#if 0
+static
+__attribute__((destructor))
+VOID
+__LwIoDestruct()
 {
-    PIO_CONTEXT pContext = NULL;
-    NTSTATUS Status = STATUS_SUCCESS;
+    BOOLEAN bInLock = FALSE;
 
-    Status = LwIoThreadInit();
-    BAIL_ON_NT_STATUS(Status);
+    LWIO_LOCK_MUTEX(bInLock, &gLock);
 
-    Status = LwIoAllocateMemory(
-        sizeof(*pContext),
-        OUT_PPVOID(&pContext));
-    BAIL_ON_NT_STATUS(Status);
-
-    pContext->pClient = gpClient;
-    pContext->pSession = gpSession;
-
-    *ppContext = pContext;
-
-cleanup:
-
-    return Status;
-
-error:
-
-    if (pContext)
+    if (gpClient)
     {
-        LwIoCloseContext(pContext);
+        lwmsg_peer_delete(gpClient);
+        gpClient = NULL;
+        gpSession = NULL;
     }
 
-    *ppContext = NULL;
+    if (!gpProcessCreds)
+    {
+        LwIoDeleteCreds(gpProcessCreds);
+        gpProcessCreds = NULL;
+    }
 
-    goto cleanup;
+    if (gbStateKeyInit)
+    {
+        pthread_key_delete(gStateKey);
+        gbStateKeyInit = FALSE;
+    }
+
+    LWIO_UNLOCK_MUTEX(bInLock, &gLock);
+
+    LwIoShutdown();
 }
+#endif
+
