@@ -37,54 +37,6 @@
 
 // TODO-Create a header with device names.
 
-//
-// Specific Access Rights - File
-//
-
-#define FILE_READ_DATA              0x00000001 // File/Pipe
-#define FILE_LIST_DIRECTORY         0x00000001 // Directory
-#define FILE_WRITE_DATA             0x00000002 // File/Pipe
-#define FILE_ADD_FILE               0x00000002 // Directory
-#define FILE_APPEND_DATA            0x00000004 // File
-#define FILE_ADD_SUBDIRECTORY       0x00000004 // Directory
-#define FILE_CREATE_PIPE_INSTANCE   0x00000004 // Pipe
-#define FILE_READ_EA                0x00000008 // File/Directory
-#define FILE_WRITE_EA               0x00000010 // File/Directory
-#define FILE_EXECUTE                0x00000020 // File
-#define FILE_TRAVERSE               0x00000020 // Directory
-#define FILE_DELETE_CHILD           0x00000040 // Directory
-#define FILE_READ_ATTRIBUTES        0x00000080 // File/Pipe/Directory
-#define FILE_WRITE_ATTRIBUTES       0x00000100 // File/Pipe/Directory
-
-#define FILE_ALL_ACCESS ( \
-    SYNCHRONIZE | \
-    STANDARD_RIGHTS_REQUIRED | \
-    0x000001FF | \
-    0 )
-
-#define FILE_GENERIC_READ ( \
-    SYNCHRONIZE | \
-    STANDARD_RIGHTS_READ | \
-    FILE_READ_ATTRIBUTES | \
-    FILE_READ_DATA | \
-    FILE_READ_EA | \
-    0 )
-
-#define FILE_GENERIC_WRITE ( \
-    SYNCHRONIZE | \
-    STANDARD_RIGHTS_WRITE | \
-    FILE_WRITE_ATTRIBUTES | \
-    FILE_WRITE_DATA | \
-    FILE_WRITE_EA | \
-    FILE_APPEND_DATA | \
-    0 )
-
-#define FILE_GENERIC_EXECUTE ( \
-    SYNCHRONIZE | \
-    STANDARD_RIGHTS_EXECUTE | \
-    FILE_READ_ATTRIBUTES | \
-    FILE_EXECUTE | \
-    0 )
 
 //
 // Share Flags
@@ -357,8 +309,6 @@ typedef ULONG NAMED_PIPE_END;
 //
 // Core Types
 //
-
-typedef struct __LW_IO_CONTEXT LW_IO_CONTEXT, *LW_PIO_CONTEXT;
 typedef struct __LW_IO_CREDS LW_IO_CREDS, *LW_PIO_CREDS;
 
 struct _IO_FILE_OBJECT;
@@ -465,13 +415,29 @@ typedef ULONG FILE_INFORMATION_CLASS, *PFILE_INFORMATION_CLASS;
 #define FileTrackingInformation           36 // unused
 #define FileIdBothDirectoryInformation    37 // DIR: FILE_ID_BOTH_DIR_INFORMATION
 #define FileIdFullDirectoryInformation    38 // DIR: FILE_ID_FULL_DIR_INFORMATION
-#define FileValidDataLengthInformation    39 // unused
+#define FileValidDataLengthInformation    39 // SET: FILE_VALID_DATA_LENGTH_INFORMATION
 #define FileShortNameInformation          40 // SET: FILE_NAME_INFORMATION (set 8.3 name)
-#define FileFsAttributeInformation        41 // VOL: FILE_FS_ATTRIBUTE_INFORMATION
-#define FileFsVolumeInformation           42 // VOL: FILE_FS_VOLUME_INFORMATION
-#define FileFsSizeInformation             43 // VOL: FILE_FS_SIZE_INFORMATION
-#define FileFsDeviceInformation           44 // VOL: FILE_FS_DEVICE_INFORMATION
-#define FileMaximumInformation            45 // SENTINEL
+#define FileMaximumInformation            41 // SENTINEL
+
+// LW-specific info classes
+
+#define _LW_FILE_INFORMATION_CLASS_BASE   0x80000000
+
+#define _MAKE_LW_FILE_INFORMATION_CLASS(Code) \
+    (_LW_FILE_INFORMATION_CLASS_BASE | (Code))
+
+#define IS_LW_FILE_INFORMATION_CLASS(Code) \
+    (_LW_FILE_INFORMATION_CLASS_BASE & (Code))
+
+#define IS_VALID_FILE_INFORMATION_CLASS(Code) \
+    ((IS_LW_FILE_INFORMATION_CLASS(Code)) ? \
+     ((Code) < LwFileMaximumInformation && (Code) > _LW_FILE_INFORMATION_CLASS_BASE) : \
+     ((Code) < FileMaximumInformation))
+
+#define LwFilePosixInformation            _MAKE_LW_FILE_INFORMATION_CLASS(1)
+#define LwFilePosixDirectoryInformation   _MAKE_LW_FILE_INFORMATION_CLASS(2)
+#define LwFileMaximumInformation          _MAKE_LW_FILE_INFORMATION_CLASS(3)
+
 
 //
 // Notes:
@@ -615,6 +581,11 @@ typedef struct _FILE_ATTRIBUTE_TAG_INFORMATION {
     ULONG ReparseTag;
 } FILE_ATTRIBUTE_TAG_INFORMATION, *PFILE_ATTRIBUTE_TAG_INFORMATION;
 
+// SET: FileValidDataLengthInformation
+typedef struct _FILE_VALID_DATA_LENGTH_INFORMATION {
+    LONG64 ValidDataLength;
+} FILE_VALID_DATA_LENGTH_INFORMATION, *PFILE_VALID_DATA_LENGTH_INFORMATION;
+
 //
 // Directory Information
 //
@@ -816,6 +787,33 @@ typedef struct _FILE_LINK_INFORMATION {
     WCHAR FileName[1];
 } FILE_LINK_INFORMATION, *PFILE_LINK_INFORMATION;
 
+// QUERY: LwFilePosixInformation
+typedef ULONG LW_UNIX_MODE, *PLW_UNIX_MODE;
+
+typedef struct _LW_FILE_POSIX_INFORMATION {
+    ULONG64 UnixAccessTime;         // Seconds since 1970
+    ULONG64 UnixModificationTime;   // Seconds since 1970
+    ULONG64 UnixChangeTime;         // Seconds since 1970
+    FILE_ATTRIBUTES FileAttributes;
+    ULONG64 EndOfFile;
+    ULONG64 AllocationSize;
+    ULONG UnixNumberOfLinks;
+    LW_UNIX_MODE UnixMode;
+    ULONG Uid;
+    ULONG Gid;
+    ULONG64 VolumeId;
+    ULONG64 InodeNumber;
+    ULONG64 GenerationNumber;
+} LW_FILE_POSIX_INFORMATION, *PLW_FILE_POSIX_INFORMATION;
+
+// DIR: FilePosixDirectoryInformation
+typedef struct _LW_FILE_POSIX_DIR_INFORMATION {
+    ULONG NextEntryOffset;
+    LW_FILE_POSIX_INFORMATION PosixInformation;
+    ULONG FileNameLength;
+    CHAR FileName[1];
+} LW_FILE_POSIX_DIR_INFORMATION, *PLW_FILE_POSIX_DIR_INFORMATION;
+
 // TODO-EA
 
 // IRP_TYPE_QUERY_EA (IN)
@@ -851,6 +849,45 @@ typedef struct _FILE_QUOTA_INFORMATION {
 
 typedef ULONG FS_INFORMATION_CLASS, *PFS_INFORMATION_CLASS;
 
+#define FileFsVolumeInformation           1  // QUERY: FILE_FS_VOLUME_INFORMATION
+#define FileFsLabelInformation            2  // SET: FILE_FS_VOLUME_INFORMATION
+#define FileFsSizeInformation             3  // QUERY: FILE_FS_SIZE_INFORMATION
+#define FileFsDeviceInformation           4  // QUERY: FILE_FS_DEVICE_INFORMATION
+#define FileFsAttributeInformation        5  // QUERY: FILE_FS_ATTRIBUTE_INFORMATION
+#define FileFsControlInformation          6  // QUERY/SET: FILE_FS_CONTROL_INFORMATION
+#define FileFsFullSizeInformation         7  // QUERY: FILE_FS_FULL_SIZE_INFORMATION
+#define FileFsObjectIdInformation         8  // QUERY/SET: FILE_FS_OBJECTID_INFORMATION
+#define FileFsDriverPathInformation       9  // unused
+#define FileFsVolumeFlagsInformation      10 // unused
+#define FileFsMaximumInformation          11 // SENTINEL
+
+
+typedef ULONG FILE_VC_FLAGS, *PFILE_VC_FLAGS;
+
+// Quota Tracking/Enforcement Setting
+#define FILE_VC_QUOTA_NONE                       0x00000000
+#define FILE_VC_QUOTA_TRACK                      0x00000001
+#define FILE_VC_QUOTA_ENFORCE                    0x00000002
+#define FILE_VC_QUOTA_MASK                       0x00000003
+
+// Content Indexing Setting
+#define FILE_VC_CONTENT_INDEX_DISABLED           0x00000008
+
+// Quota Logging Settings
+#define FILE_VC_LOG_QUOTA_THRESHOLD              0x00000010
+#define FILE_VC_LOG_QUOTA_LIMIT                  0x00000020
+#define FILE_VC_LOG_VOLUME_THRESHOLD             0x00000040
+#define FILE_VC_LOG_VOLUME_LIMIT                 0x00000080
+
+// Quota Transient State (is this settable?)
+#define FILE_VC_QUOTAS_INCOMPLETE                0x00000100
+#define FILE_VC_QUOTAS_REBUILDING                0x00000200
+
+#define FILE_VC_VALID_MASK                       0x000003FB
+
+
+typedef ULONG IO_FILE_FS_ATTRIBUTES, *PIO_FILE_FS_ATTRIBUTES;
+
 #define FILE_CASE_SENSITIVE_SEARCH               0x00000001
 #define FILE_CASE_PRESERVED_NAMES                0x00000002
 #define FILE_UNICODE_ON_DISK                     0x00000004
@@ -868,37 +905,85 @@ typedef ULONG FS_INFORMATION_CLASS, *PFS_INFORMATION_CLASS;
 #define FILE_READ_ONLY_VOLUME                    0x00080000
 
 
+typedef ULONG IO_DEVICE_CHARACTERISTICS, *PIO_DEVICE_CHARACTERISTICS;
+
+#define FILE_REMOVABLE_MEDIA                     0x00000001
+#define FILE_READ_ONLY_DEVICE                    0x00000002
+#define FILE_FLOPPY_DISKETTE                     0x00000004
+#define FILE_WRITE_ONCE_MEDIA                    0x00000008
+#define FILE_REMOTE_DEVICE                       0x00000010
+#define FILE_DEVICE_IS_MOUNTED                   0x00000020
+#define FILE_VIRTUAL_VOLUME                      0x00000040
+#define FILE_AUTOGENERATED_DEVICE_NAME           0x00000080
+#define FILE_DEVICE_SECURE_OPEN                  0x00000100
+#define FILE_CHARACTERISTIC_PNP_DEVICE           0x00000800
+#define FILE_CHARACTERISTIC_TS_DEVICE            0x00001000
+#define FILE_CHARACTERISTIC_WEBDAV_DEVICE        0x00002000
+
+
 // VOL: FileFsAttributeInformation
 typedef struct _FILE_FS_ATTRIBUTE_INFORMATION {
-    ULONG  FileSystemAttributes;
-    LONG   MaximumComponentNameLength;
-    ULONG  FileSystemNameLength;
-    WCHAR  FileSystemName[1];
+    IO_FILE_FS_ATTRIBUTES FileSystemAttributes;
+    LONG MaximumComponentNameLength;
+    ULONG FileSystemNameLength;
+    WCHAR FileSystemName[1];
 } FILE_FS_ATTRIBUTE_INFORMATION, *PFILE_FS_ATTRIBUTE_INFORMATION;
 
 // VOL: FileFsVolumeInformation
 typedef struct _FILE_FS_VOLUME_INFORMATION {
-    LONG64  VolumeCreationTime;
-    ULONG   VolumeSerialNumber;
-    ULONG   VolumeLabelLength;
+    LONG64 VolumeCreationTime;
+    ULONG VolumeSerialNumber;
+    ULONG VolumeLabelLength;
     BOOLEAN SupportsObjects;
-    WCHAR   VolumeLabel[1];
+    WCHAR VolumeLabel[1];
 } FILE_FS_VOLUME_INFORMATION, *PFILE_FS_VOLUME_INFORMATION;
 
+// VOL: FileFsLabelInformation
+typedef struct _FILE_FS_LABEL_INFORMATION {
+    ULONG VolumeLabelLength;
+    WCHAR VolumeLabel[1];
+} FILE_FS_LABEL_INFORMATION, *PFILE_FS_LABEL_INFORMATION;
+
 // VOL: FileFsSizeInformation
 typedef struct _FILE_FS_SIZE_INFORMATION {
-    LONG64  TotalAllocationUnits;
-    LONG64  AvailableAllocationUnits;
-    ULONG   SectorsPerAllocationUnit;
-    ULONG   BytesPerSector;
+    LONG64 TotalAllocationUnits;
+    LONG64 AvailableAllocationUnits;
+    ULONG SectorsPerAllocationUnit;
+    ULONG BytesPerSector;
 } FILE_FS_SIZE_INFORMATION, *PFILE_FS_SIZE_INFORMATION;
 
 // VOL: FileFsDeviceInformation
 typedef struct _FILE_FS_DEVICE_INFORMATION {
     DEVICE_TYPE DeviceType;
-    ULONG       DeviceCharacteristics;
+    IO_DEVICE_CHARACTERISTICS DeviceCharacteristics;
 } FILE_FS_DEVICE_INFORMATION, *PFILE_FS_DEVICE_INFORMATION;
 
+// VOL: FileFsControlInformation
+typedef struct _FILE_FS_CONTROL_INFORMATION {
+    LONG64 FreeSpaceStartFiltering;    // must be ignored
+    LONG64 FreeSpaceThreshold;         // must be ignored
+    LONG64 FreeSpaceStopFiltering;     // must be ignored
+    LONG64 DefaultQuotaThreshold;
+    LONG64 DefaultQuotaLimit;
+    FILE_VC_FLAGS FileSystemControlFlags;
+} FILE_FS_CONTROL_INFORMATION, *PFILE_FS_CONTROL_INFORMATION;
+
+// VOL: FileFsFullSizeInformation
+typedef struct _FILE_FS_FULL_SIZE_INFORMATION {
+    LONG64 TotalAllocationUnits;
+    LONG64 CallerAvailableAllocationUnits;
+    LONG64 ActualAvailableAllocationUnits;
+    ULONG SectorsPerAllocationUnit;
+    ULONG BytesPerSector;
+} FILE_FS_FULL_SIZE_INFORMATION, *PFILE_FS_FULL_SIZE_INFORMATION;
+
+// VOL: FileFsFullSizeInformation
+typedef struct _FILE_FS_OBJECTID_INFORMATION {
+    UCHAR ObjectId[16];
+    UCHAR ExtendedInfo[48];
+} FILE_FS_OBJECTID_INFORMATION, *PFILE_FS_OBJECTID_INFORMATION;
+
+
 //
 // Extra Create Parameter Support
 //
@@ -907,9 +992,9 @@ typedef struct _IO_ECP_LIST *PIO_ECP_LIST;
 
 // TODO-Move named pipe ECP stuff to internal header.
 
-#define IO_ECP_TYPE_NAMED_PIPE "Likewise.IO.NamedPipe"
-#define IO_ECP_TYPE_SESSION_KEY "Likewise.IO.SessionKey"
-#define IO_ECP_TYPE_PEER_ADDRESS "Likewise.IO.PeerAddress"
+#define IO_ECP_TYPE_NAMED_PIPE      "Likewise.IO.NamedPipe"
+#define IO_ECP_TYPE_SESSION_KEY     "Likewise.IO.SessionKey"
+#define IO_ECP_TYPE_PEER_ADDRESS    "Likewise.IO.PeerAddress"
 
 typedef struct _IO_ECP_NAMED_PIPE {
     FILE_PIPE_TYPE_MASK NamedPipeType;
@@ -969,8 +1054,6 @@ typedef struct _IO_ECP_NAMED_PIPE {
 
 #ifndef LW_STRICT_NAMESPACE
 
-typedef LW_IO_CONTEXT IO_CONTEXT;
-typedef LW_PIO_CONTEXT PIO_CONTEXT;
 typedef LW_IO_CREDS IO_CREDS;
 typedef LW_PIO_CREDS PIO_CREDS;
 
