@@ -191,7 +191,7 @@ cpp_quote("#endif")
 #endif
 
 #ifndef SHARE_INFO_502_I_DEFINED
-#define SHAER_INFO_502_I_DEFINED 1
+#define SHARE_INFO_502_I_DEFINED 1
 
 #ifdef _DCE_IDL_
 cpp_quote("#ifndef SHARE_INFO_502_I_DEFINED")
@@ -415,6 +415,7 @@ typedef union {
 
 #endif
 
+
 #endif /* _LMSHARE_H_ */
 
 
@@ -426,4 +427,3 @@ indent-tabs-mode: nil
 tab-width: 4
 end:
 */
-
