@@ -53,22 +53,29 @@
 #define IO_DEVICE_FUNC_CODE_LIST_OPEN_FILES 0x00008010
 #define IO_DEVICE_FUNC_CODE_STATISTICS      0x00008020
 
+#define IO_DEVICE_FUNC_CODE_GET_DFS         0x00008030
+#define IO_DEVICE_FUNC_CODE_SET_DFS         0x00008040
+
 #define IO_DEVICE_TRANSFER_TYPE_BUFFERED    0x00000000
 #define IO_DEVICE_TRANSFER_TYPE_IN_DIRECT   0x00000001
 #define IO_DEVICE_TRANSFER_TYPE_OUT_DIRECT  0x00000002
 #define IO_DEVICE_TRANSFER_TYPE_NEITHER     0x00000003
 
 // TODO: Assign standard codes
-#define SRV_DEVCTL_ADD_SHARE      1
-#define SRV_DEVCTL_DELETE_SHARE   2
-#define SRV_DEVCTL_ENUM_SHARE     3
-#define SRV_DEVCTL_SET_SHARE_INFO 4
-#define SRV_DEVCTL_GET_SHARE_INFO 5
-#define SRV_DEVCTL_ENUM_SESSIONS  6
-#define SRV_DEVCTL_DELETE_SESSION 7
-#define SRV_DEVCTL_ENUM_FILES     8
-#define SRV_DEVCTL_GET_FILE_INFO  9
-#define SRV_DEVCTL_CLOSE_FILE     10
+
+#define SRV_DEVCTL_ADD_SHARE         1
+#define SRV_DEVCTL_DELETE_SHARE      2
+#define SRV_DEVCTL_ENUM_SHARE        3
+#define SRV_DEVCTL_SET_SHARE_INFO    4
+#define SRV_DEVCTL_GET_SHARE_INFO    5
+#define SRV_DEVCTL_ENUM_SESSIONS     6
+#define SRV_DEVCTL_DELETE_SESSION    7
+#define SRV_DEVCTL_ENUM_FILES        8
+#define SRV_DEVCTL_GET_FILE_INFO     9
+#define SRV_DEVCTL_CLOSE_FILE       10
+#define SRV_DEVCTL_ENUM_CONNECTION  11
+
+#define RDR_DEVCTL_SET_DOMAIN_HINTS  1
 
 #define IO_DEVICE_CTL_OPEN_FILE_INFO   ( IO_DEVICE_TYPE_DISK_FILE_SYSTEM     | \
                                          IO_DEVICE_REQ_ACCESS_READ_DATA      | \
@@ -76,6 +83,7 @@
                                          IO_DEVICE_FUNC_CODE_LIST_OPEN_FILES | \
                                          IO_DEVICE_TRANSFER_TYPE_NEITHER       \
                                        )
+
 #define IO_DEVICE_CTL_STATISTICS ( IO_DEVICE_TYPE_NETWORK_FILE_SYSTEM | \
                                          IO_DEVICE_REQ_ACCESS_READ_DATA     | \
                                          IO_DEVICE_REQ_ACCESS_WRITE_DATA    | \
@@ -84,6 +92,21 @@
                                          IO_DEVICE_TRANSFER_TYPE_NEITHER      \
                                        )
 
+#define IO_DEVICE_CTL_GET_DFS_REFERRAL ( IO_DEVICE_TYPE_DISK_FILE_SYSTEM     | \
+                                         IO_DEVICE_REQ_ACCESS_READ_DATA      | \
+                                         IO_DEVICE_CUSTOM_CONTROL_CODE       | \
+                                         IO_DEVICE_FUNC_CODE_GET_DFS         | \
+                                         IO_DEVICE_TRANSFER_TYPE_NEITHER       \
+                                       )
+
+#define IO_DEVICE_CTL_SET_DFS_REFERRAL ( IO_DEVICE_TYPE_DISK_FILE_SYSTEM     | \
+                                         IO_DEVICE_REQ_ACCESS_READ_DATA      | \
+                                         IO_DEVICE_CUSTOM_CONTROL_CODE       | \
+                                         IO_DEVICE_FUNC_CODE_SET_DFS         | \
+                                         IO_DEVICE_TRANSFER_TYPE_NEITHER       \
+                                       )
+
+
 /* Device IoControl structures */
 
 typedef struct _IO_OPEN_FILE_INFO_0
@@ -108,7 +131,7 @@ typedef struct _IO_OPEN_FILE_INFO_100
 typedef struct _IO_OPEN_FILE_INFO_INPUT_BUFFER
 {
     DWORD Level;
-
+    
 } IO_OPEN_FILE_INFO_INPUT_BUFFER, *PIO_OPEN_FILE_INFO_INPUT_BUFFER;
 
 typedef struct _IO_STATISTICS_INFO_0
