@@ -51,6 +51,7 @@
 #define __SMBUTILS_H__
 
 #include <lwio/lwio.h>
+#include <lwio/utils.h>
 
 #ifndef _SMB_ENDIAN_SWAP16
 
@@ -187,7 +188,7 @@
            }                      \
         } while(0);
 
-#define SMB_SAFE_CLEAR_FREE_STRING(str)       \
+#define LWIO_SAFE_CLEAR_FREE_STRING(str)       \
         do {                                  \
            if (str) {                         \
               if (*str) {                     \
@@ -201,7 +202,7 @@
 #define LWIO_SAFE_FREE_MEMORY(mem) \
         do {                      \
            if (mem) {             \
-              SMBFreeMemory(mem); \
+              LwIoFreeMemory(mem); \
               (mem) = NULL;       \
            }                      \
         } while(0);
@@ -268,99 +269,6 @@
        bInLock = FALSE; \
     }
 
-/*
- * Logging
- */
-
-#if defined(LW_ENABLE_THREADS)
-
-extern pthread_mutex_t gSMBLogLock;
-
-#define LWIO_LOCK_LOGGER   pthread_mutex_lock(&gSMBLogLock)
-#define LWIO_UNLOCK_LOGGER pthread_mutex_unlock(&gSMBLogLock)
-
-#define _LWIO_LOG_PREFIX_THREAD(Format) \
-    "0x%lx:" Format, ((unsigned long)pthread_self())
-
-#else
-
-#define LWIO_LOCK_LOGGER
-#define LWIO_UNLOCK_LOGGER
-
-#define _LWIO_LOG_PREFIX_THREAD(Format) \
-    Format
-
-#endif
-
-#define _LWIO_LOG_PREFIX_LOCATION(Format, Function, File, Line) \
-    _LWIO_LOG_PREFIX_THREAD("[%s() %s:%d] " Format), \
-    (Function), \
-    (File), \
-    (Line)
-
-#define _LWIO_LOG_WITH_THREAD(Level, Format, ...) \
-    _LWIO_LOG_MESSAGE(Level, \
-                      _LWIO_LOG_PREFIX_THREAD(Format), \
-                      ## __VA_ARGS__)
-
-#define _LWIO_LOG_WITH_LOCATION(Level, Format, Function, File, Line, ...) \
-    _LWIO_LOG_MESSAGE(Level, \
-                      _LWIO_LOG_PREFIX_LOCATION(Format, Function, File, Line), \
-                      ## __VA_ARGS__)
-
-#define _LWIO_LOG_WITH_DEBUG(Level, Format, ...) \
-    _LWIO_LOG_WITH_LOCATION(Level, Format, \
-                            __FUNCTION__, __FILE__, __LINE__, \
-                            ## __VA_ARGS__)
-
-extern HANDLE              ghSMBLog;
-extern LWIO_LOG_LEVEL      gSMBMaxLogLevel;
-extern PFN_LWIO_LOG_MESSAGE gpfnSMBLogger;
-
-#define _LWIO_LOG_MESSAGE(Level, Format, ...) \
-    SMBLogMessage(gpfnSMBLogger, ghSMBLog, Level, Format, ## __VA_ARGS__)
-
-#define _LWIO_LOG_IF(Level, Format, ...)                     \
-    do {                                                    \
-        LWIO_LOCK_LOGGER;                                    \
-        if (gpfnSMBLogger && (gSMBMaxLogLevel >= (Level)))  \
-        {                                                   \
-            if (gSMBMaxLogLevel >= LWIO_LOG_LEVEL_DEBUG)     \
-            {                                               \
-                _LWIO_LOG_WITH_DEBUG(Level, Format, ## __VA_ARGS__); \
-            }                                               \
-            else                                            \
-            {                                               \
-                _LWIO_LOG_WITH_THREAD(Level, Format, ## __VA_ARGS__); \
-            }                                               \
-        }                                                   \
-        LWIO_UNLOCK_LOGGER;                                  \
-    } while (0)
-
-#define SMB_SAFE_LOG_STRING(x) \
-    ( (x) ? (x) : "<null>" )
-
-#define LWIO_LOG_ALWAYS(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_ALWAYS, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_ERROR(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_ERROR, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_WARNING(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_WARNING, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_INFO(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_INFO, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_VERBOSE(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_VERBOSE, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_DEBUG(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_DEBUG, szFmt, ## __VA_ARGS__)
-
-#define LWIO_LOG_TRACE(szFmt, ...) \
-    _LWIO_LOG_IF(LWIO_LOG_LEVEL_TRACE, szFmt, ## __VA_ARGS__)
-
 VOID
 LwIoAssertionFailed(
     IN PCSTR Expression,
@@ -423,11 +331,11 @@ LwIoAssertionFailedFormat(
            BAIL_ON_LWIO_ERROR(dwError);            \
         }
 
-typedef struct __SMB_BIT_VECTOR
+typedef struct __LWIO_BIT_VECTOR
 {
-    DWORD  dwNumBits;
-    PDWORD pVector;
-} SMB_BIT_VECTOR, *PSMB_BIT_VECTOR;
+    ULONG  ulNumBits;
+    PULONG pVector;
+} LWIO_BIT_VECTOR, *PLWIO_BIT_VECTOR;
 
 typedef struct __SMB_HASH_ENTRY SMB_HASH_ENTRY, *PSMB_HASH_ENTRY;
 
@@ -515,7 +423,7 @@ typedef struct __SMB_HANDLE_MANAGER
 {
     SMBHANDLE       dwHandleMax;
     PSMB_HASH_TABLE pHandleTable;
-    PSMB_BIT_VECTOR pFreeHandleIndex;
+    PLWIO_BIT_VECTOR pFreeHandleIndex;
 } SMB_HANDLE_MANAGER, *PSMB_HANDLE_MANAGER;
 
 #if !defined(HAVE_STRTOLL)
@@ -563,6 +471,17 @@ LwIoAllocateMemory(
     LW_PVOID * ppMemory
     );
 
+LW_NTSTATUS
+LwIoAllocateBuffer(
+    size_t     Size,
+    LW_PVOID * ppMemory
+    );
+
+VOID
+LwIoFreeBuffer(
+    LW_PVOID pMemory
+    );
+
 LW_NTSTATUS
 LwIoReallocMemory(
     LW_PVOID pMemory,
@@ -585,24 +504,6 @@ LwIoFreeMemory(
         }                                       \
     } while (0)
 
-DWORD
-SMBAllocateMemory(
-    DWORD dwSize,
-    PVOID * ppMemory
-    );
-
-DWORD
-SMBReallocMemory(
-    PVOID  pMemory,
-    PVOID * ppNewMemory,
-    DWORD dwSize
-    );
-
-void
-SMBFreeMemory(
-    PVOID pMemory
-    );
-
 DWORD
 SMBAllocateString(
     PCSTR pszInputString,
@@ -855,6 +756,12 @@ SMBEnqueue(
     PVOID      pItem
     );
 
+DWORD
+SMBEnqueueFront(
+    PLWIO_QUEUE pQueue,
+    PVOID      pItem
+    );
+
 PVOID
 SMBDequeue(
     PLWIO_QUEUE pQueue
@@ -877,47 +784,47 @@ LWIOQueueFree(
     PLWIO_QUEUE pQueue
     );
 
-DWORD
-SMBBitVectorCreate(
-    DWORD dwNumBits,
-    PSMB_BIT_VECTOR* ppBitVector
+NTSTATUS
+LwioBitVectorCreate(
+    ULONG             ulNumBits,
+    PLWIO_BIT_VECTOR* ppBitVector
     );
 
 BOOLEAN
-SMBBitVectorIsSet(
-    PSMB_BIT_VECTOR pBitVector,
-    DWORD           iBit
+LwioBitVectorIsSet(
+    PLWIO_BIT_VECTOR pBitVector,
+    ULONG            iBit
     );
 
-DWORD
-SMBBitVectorSetBit(
-    PSMB_BIT_VECTOR pBitVector,
-    DWORD           iBit
+NTSTATUS
+LwioBitVectorSetBit(
+    PLWIO_BIT_VECTOR pBitVector,
+    ULONG            iBit
     );
 
-DWORD
-SMBBitVectorUnsetBit(
-    PSMB_BIT_VECTOR pBitVector,
-    DWORD           iBit
+NTSTATUS
+LwioBitVectorUnsetBit(
+    PLWIO_BIT_VECTOR pBitVector,
+    ULONG            iBit
     );
 
-DWORD
-SMBBitVectorFirstUnsetBit(
-    PSMB_BIT_VECTOR pBitVector,
-    PDWORD  pdwUnsetBit
+NTSTATUS
+LwioBitVectorFirstUnsetBit(
+    PLWIO_BIT_VECTOR pBitVector,
+    PULONG           pulUnsetBit
     );
 
 VOID
-SMBBitVectorReset(
-    PSMB_BIT_VECTOR pBitVector
+LwioBitVectorReset(
+    PLWIO_BIT_VECTOR pBitVector
     );
 
 VOID
-SMBBitVectorFree(
-    PSMB_BIT_VECTOR pBitVector
+LwioBitVectorFree(
+    PLWIO_BIT_VECTOR pBitVector
     );
 
-DWORD
+NTSTATUS
 SMBHashCreate(
         size_t sTableSize,
         SMB_HASH_KEY_COMPARE fnComparator,
@@ -925,18 +832,18 @@ SMBHashCreate(
         SMB_HASH_FREE_ENTRY fnFree, //optional
         SMB_HASH_TABLE** ppResult);
 
-void
+VOID
 SMBHashSafeFree(
         SMB_HASH_TABLE** ppResult);
 
-DWORD
+NTSTATUS
 SMBHashSetValue(
         SMB_HASH_TABLE *pTable,
         PVOID  pKey,
         PVOID  pValue);
 
-//Returns ENOENT if pKey is not in the table
-DWORD
+//Returns STATUS_NOT_FOUND if pKey is not in the table
+NTSTATUS
 SMBHashGetValue(
         SMB_HASH_TABLE *pTable,
         PCVOID  pKey,
@@ -949,12 +856,12 @@ SMBHashExists(
     );
 
 //Invalidates all iterators
-DWORD
+NTSTATUS
 SMBHashResize(
         SMB_HASH_TABLE *pTable,
         size_t sTableSize);
 
-DWORD
+NTSTATUS
 SMBHashGetIterator(
         SMB_HASH_TABLE *pTable,
         SMB_HASH_ITERATOR *pIterator);
@@ -965,7 +872,7 @@ SMBHashNext(
         SMB_HASH_ITERATOR *pIterator
         );
 
-DWORD
+NTSTATUS
 SMBHashRemoveKey(
         SMB_HASH_TABLE *pTable,
         PCVOID  pKey);
@@ -994,41 +901,6 @@ size_t
 SMBHashCaselessWc16String(
     PCVOID str);
 
-DWORD
-SMBHandleManagerCreate(
-    SMBHANDLE dwHandleMax,
-    PSMB_HANDLE_MANAGER* ppHandleManager
-    );
-
-DWORD
-SMBHandleManagerAddItem(
-    PSMB_HANDLE_MANAGER pHandleMgr,
-    SMBHandleType       hType,
-    PVOID               pItem,
-    PSMBHANDLE          phItem
-    );
-
-DWORD
-SMBHandleManagerGetItem(
-    PSMB_HANDLE_MANAGER pHandleMgr,
-    SMBHANDLE           handleId,
-    SMBHandleType*      phType,
-    PVOID*              ppItem
-    );
-
-DWORD
-SMBHandleManagerDeleteItem(
-    PSMB_HANDLE_MANAGER pHandleMgr,
-    SMBHANDLE           hItem,
-    SMBHandleType*      phType,
-    PVOID*              ppItem
-    );
-
-VOID
-SMBHandleManagerFree(
-    PSMB_HANDLE_MANAGER pHandleManager
-    );
-
 DWORD
 SMBRemoveFile(
     PCSTR pszPath
@@ -1088,14 +960,6 @@ SMBRemoveDirectory(
     PSTR pszPath
     );
 
-DWORD
-SMBCopyDirectory(
-    PCSTR pszSourceDirPath,
-    uid_t ownerUid,
-    gid_t ownerGid,
-    PCSTR pszDestDirPath
-    );
-
 DWORD
 SMBCheckDirectoryExists(
     PCSTR pszPath,
@@ -1129,31 +993,12 @@ SMBCopyFileWithOriginalPerms(
     PCSTR pszDstPath
     );
 
-DWORD
-SMBBackupFile(
-    PCSTR pszPath
-    );
-
-DWORD
-SMBGetSymlinkTarget(
-   PCSTR pszPath,
-   PSTR* ppszTargetPath
-   );
-
 DWORD
 SMBCreateSymlink(
    PCSTR pszOldPath,
    PCSTR pszNewPath
    );
 
-DWORD
-SMBGetMatchingFilePathsInFolder(
-    PCSTR pszDirPath,
-    PCSTR pszFileNameRegExp,
-    PSTR** pppszHostFilePaths,
-    PDWORD pdwNPaths
-    );
-
 DWORD
 SMBGetPrefixDirPath(
     PSTR* ppszPath
@@ -1170,25 +1015,30 @@ LwioGetHostInfo(
     );
 
 DWORD
-SMBInitLogging(
+LwioInitLogging(
     PCSTR         pszProgramName,
     LWIO_LOG_TARGET  logTarget,
     LWIO_LOG_LEVEL   maxAllowedLogLevel,
     PCSTR         pszPath
     );
 
+PCSTR
+LwioLogLevelGetLabel(
+    LWIO_LOG_LEVEL logLevel
+    );
+
 DWORD
-SMBLogGetInfo(
+LwioLogGetInfo(
     PLWIO_LOG_INFO* ppLogInfo
     );
 
 DWORD
-SMBLogSetInfo(
+LwioLogSetInfo(
     PLWIO_LOG_INFO pLogInfo
     );
 
 VOID
-SMBLogMessage(
+LwioLogMessage(
     PFN_LWIO_LOG_MESSAGE pfnLogger,
     HANDLE hLog,
     LWIO_LOG_LEVEL logLevel,
@@ -1204,13 +1054,13 @@ lsmb_vsyslog(
     );
 
 DWORD
-SMBShutdownLogging(
-    VOID
+LwioValidateLogLevel(
+    DWORD dwLogLevel
     );
 
 DWORD
-SMBValidateLogLevel(
-    DWORD dwLogLevel
+LwioShutdownLogging(
+    VOID
     );
 
 #endif /* __SMBUTILS_H__ */
