@@ -95,7 +95,7 @@ typedef enum _NT_IPC_MESSAGE_TYPE
     NT_IPC_MESSAGE_TYPE_QUERY_VOLUME_INFORMATION_FILE,
     NT_IPC_MESSAGE_TYPE_QUERY_VOLUME_INFORMATION_FILE_RESULT, // NT_IPC_MESSAGE_GENERIC_FILE_BUFFER_RESULT
     NT_IPC_MESSAGE_TYPE_LOCK_FILE,
-    NT_IPC_MESSAGE_TYPE_LOCK_FILE_RESULT,
+    NT_IPC_MESSAGE_TYPE_LOCK_FILE_RESULT, 
     NT_IPC_MESSAGE_TYPE_UNLOCK_FILE,
     NT_IPC_MESSAGE_TYPE_UNLOCK_FILE_RESULT,
     NT_IPC_MESSAGE_TYPE_QUERY_SECURITY_FILE,
@@ -169,9 +169,10 @@ typedef struct _NT_IPC_MESSAGE_CREATE_FILE {
     IN FILE_CREATE_OPTIONS CreateOptions;
     IN OPTIONAL PVOID EaBuffer; // FILE_FULL_EA_INFORMATION
     IN ULONG EaLength;
-    // TODO -- Add stuff for SDs, etc.
+    IN OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor;
+    IN ULONG SecDescLength;
 #if 0
-    IN OPTIONAL PVOID SecurityDescriptor; // TBD
+    // TODO -- Add stuff for QOS.
     IN OPTIONAL PVOID SecurityQualityOfService; // TBD
 #endif
     IN OPTIONAL PNT_IPC_HELPER_ECP EcpList;
@@ -209,7 +210,7 @@ typedef struct _NT_IPC_MESSAGE_CREATE_FILE_RESULT {
 typedef struct _NT_IPC_MESSAGE_READ_FILE {
     IN IO_FILE_HANDLE FileHandle;
     IN ULONG Length;
-    IN OPTIONAL PLONG64 ByteOffset;
+    IN OPTIONAL PULONG64 ByteOffset;
     IN OPTIONAL PULONG Key;
 } NT_IPC_MESSAGE_READ_FILE, *PNT_IPC_MESSAGE_READ_FILE;
 
@@ -227,7 +228,7 @@ typedef struct _NT_IPC_MESSAGE_WRITE_FILE {
     IN IO_FILE_HANDLE FileHandle;
     IN PVOID Buffer;
     IN ULONG Length;
-    IN OPTIONAL PLONG64 ByteOffset;
+    IN OPTIONAL PULONG64 ByteOffset;
     IN OPTIONAL PULONG Key;
 } NT_IPC_MESSAGE_WRITE_FILE, *PNT_IPC_MESSAGE_WRITE_FILE;
 
@@ -360,11 +361,11 @@ typedef struct _NT_IPC_MESSAGE_QUERY_VOLUME_INFORMATION_FILE {
 
 typedef struct _NT_IPC_MESSAGE_LOCK_FILE {
     IN IO_FILE_HANDLE FileHandle;
-    IN LONG64 ByteOffset;
-    IN LONG64 Length;
+    IN ULONG64 ByteOffset;
+    IN ULONG64 Length;
     IN ULONG Key;
     BOOLEAN FailImmediately;
-    BOOLEAN ExclusiveLock;
+    BOOLEAN ExclusiveLock;	
 } NT_IPC_MESSAGE_LOCK_FILE, *PNT_IPC_MESSAGE_LOCK_FILE;
 
 //
@@ -379,8 +380,8 @@ typedef struct _NT_IPC_MESSAGE_LOCK_FILE {
 
 typedef struct _NT_IPC_MESSAGE_UNLOCK_FILE {
     IN IO_FILE_HANDLE FileHandle;
-    IN LONG64 ByteOffset;
-    IN LONG64 Length;
+    IN ULONG64 ByteOffset;
+    IN ULONG64 Length;
     IN ULONG Key;
 } NT_IPC_MESSAGE_UNLOCK_FILE, *PNT_IPC_MESSAGE_UNLOCK_FILE;
 
@@ -437,7 +438,7 @@ NtIpcReleaseFileHandle(
 #endif /* __NT_IPC_MSG_H__ */
 
 
-
+ 
 /*
 local variables:
 mode: c
@@ -446,4 +447,4 @@ indent-tabs-mode: nil
 tab-width: 4
 end:
 */
-
+ 
