@@ -180,7 +180,9 @@ LWMsgTypeSpec gNtIpcTypeSpecMessageCreateFile[] =
     LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_CREATE_FILE, CreateOptions),
     LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_CREATE_FILE, EaLength),
     _LWMSG_MEMBER_BUFFER(NT_IPC_MESSAGE_CREATE_FILE, EaBuffer, EaLength),
-    // TODO -- Add stuff for SDs, etc.
+    LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_CREATE_FILE, SecDescLength),
+    _LWMSG_MEMBER_BUFFER(NT_IPC_MESSAGE_CREATE_FILE, SecurityDescriptor, SecDescLength),
+    // TODO -- Add stuff for QOS etc.
     LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_CREATE_FILE, EcpCount),
     _LWMSG_MEMBER_OPTIONAL_ARRAY(NT_IPC_MESSAGE_CREATE_FILE, EcpList, EcpCount, gNtIpcTypeSpecHelperEcp),
     LWMSG_STRUCT_END,
@@ -328,7 +330,6 @@ LWMsgTypeSpec gNtIpcTypeSpecMessageSetSecurityFile[] =
     _LWMSG_MEMBER_IO_FILE_HANDLE_IN(NT_IPC_MESSAGE_SET_SECURITY_FILE, FileHandle),
     LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_SET_SECURITY_FILE, SecurityInformation),
     LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_SET_SECURITY_FILE, Length),
-    LWMSG_MEMBER_UINT32(NT_IPC_MESSAGE_SET_SECURITY_FILE, Length),
     _LWMSG_MEMBER_BUFFER(NT_IPC_MESSAGE_SET_SECURITY_FILE, SecurityDescriptor, Length),
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
@@ -367,7 +368,7 @@ LWMsgProtocolSpec gNtIpcProtocolSpec[] =
     LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_UNLOCK_FILE_RESULT,            gNtIpcTypeSpecMessageGenericFileIoResult),
     LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_QUERY_SECURITY_FILE,           gNtIpcTypeSpecMessageQuerySecurityFile),
     LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_QUERY_SECURITY_FILE_RESULT,    gNtIpcTypeSpecMessageGenericFileBufferResult),
-    LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_SET_SECURITY_FILE,             gNtIpcTypeSpecMessageQuerySecurityFile),
+    LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_SET_SECURITY_FILE,             gNtIpcTypeSpecMessageSetSecurityFile),
     LWMSG_MESSAGE(NT_IPC_MESSAGE_TYPE_SET_SECURITY_FILE_RESULT,      gNtIpcTypeSpecMessageGenericFileIoResult),
     LWMSG_PROTOCOL_END
 };
