@@ -33,7 +33,7 @@
  *
  * Module Name:
  *
- *        ipc_protocol.c
+ *        lwshareinfo.c
  *
  * Abstract:
  *
@@ -265,12 +265,12 @@ LwShareInfoMarshalAddParameters(
             &pBuffer,
             &szBufferSize));
     BAIL_ON_NT_STATUS(Status);
-
+    
     *ppBuffer = pBuffer;
     *pulBufferSize = (ULONG) szBufferSize;
 
 cleanup:
-
+    
     LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
@@ -278,7 +278,7 @@ cleanup:
 error:
     *ppBuffer = NULL;
     *pulBufferSize = 0;
-
+    
     if (pBuffer)
     {
         RtlMemoryFree(pBuffer);
@@ -309,11 +309,11 @@ LwShareInfoUnmarshalAddParameters(
             ulBufferSize,
             OUT_PPVOID(&pParams)));
     BAIL_ON_NT_STATUS(Status);
-
+    
     *ppParams = pParams;
 
 cleanup:
-
+    
     LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
@@ -321,7 +321,7 @@ cleanup:
 error:
 
     *ppParams = NULL;
-
+    
     if (pParams)
     {
         lwmsg_data_free_graph(pDataContext, gShareInfoAddParamsSpec, pParams);
@@ -354,12 +354,12 @@ LwShareInfoMarshalDeleteParameters(
             &pBuffer,
             &ulBufferSize));
     BAIL_ON_NT_STATUS(Status);
-
+    
     *ppBuffer = pBuffer;
     *pulBufferSize = (ULONG) ulBufferSize;
 
 cleanup:
-
+    
     LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
@@ -367,7 +367,7 @@ cleanup:
 error:
     *ppBuffer = NULL;
     *pulBufferSize = 0;
-
+    
     if (pBuffer)
     {
         RtlMemoryFree(pBuffer);
@@ -399,11 +399,11 @@ LwShareInfoUnmarshalDeleteParameters(
             ulBufferSize,
             OUT_PPVOID(&pParams)));
     BAIL_ON_NT_STATUS(Status);
-
+    
     *ppParams = pParams;
 
 cleanup:
-
+    
     LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
@@ -411,7 +411,7 @@ cleanup:
 error:
 
     *ppParams = NULL;
-
+    
     if (pParams)
     {
         lwmsg_data_free_graph(pDataContext, gShareInfoDeleteParamsSpec, pParams);
@@ -489,11 +489,11 @@ LwShareInfoUnmarshalEnumParameters(
             ulBufferSize,
             OUT_PPVOID(&pParams)));
     BAIL_ON_NT_STATUS(Status);
-
+    
     *ppParams = pParams;
 
 cleanup:
-
+    
     LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
@@ -501,7 +501,7 @@ cleanup:
 error:
 
     *ppParams = NULL;
-
+    
     if (pParams)
     {
         lwmsg_data_free_graph(pDataContext, gShareInfoEnumParamsSpec, pParams);
@@ -510,6 +510,21 @@ error:
     goto cleanup;
 }
 
+VOID
+LwShareInfoFree(
+    ULONG Level,
+    ULONG Count,
+    PVOID pInfo
+    )
+{
+    SHARE_INFO_ENUM_PARAMS params = {0};
+
+    params.dwInfoLevel = Level;
+    params.dwNumEntries = Count;
+    params.info.p0 = pInfo;
+
+    lwmsg_data_destroy_graph_cleanup(NULL, gShareInfoEnumParamsSpec, &params);
+}
 
 LW_NTSTATUS
 LwShareInfoMarshalSetParameters(
@@ -579,7 +594,7 @@ LwShareInfoUnmarshalSetParameters(
             ulBufferSize,
             OUT_PPVOID(&pParams)));
     BAIL_ON_NT_STATUS(ntStatus);
-
+    
     *ppParams = pParams;
 
 cleanup:
@@ -670,7 +685,7 @@ LwShareInfoUnmarshalGetParameters(
             ulBufferSize,
             OUT_PPVOID(&pParams)));
     BAIL_ON_NT_STATUS(ntStatus);
-
+    
     *ppParams = pParams;
 
 cleanup:
