@@ -34,5 +34,9 @@
 #define SRV_ECP_TYPE_CLIENT_ADDRESS "Likewise.SRV.ClientAddress"
 #define SRV_ECP_TYPE_MAX_ACCESS     "Likewise.SRV.MaximalAccess"
 #define SRV_ECP_TYPE_ABE            "Likewise.SRV.AccessBasedEnumeration"
+#define SRV_ECP_TYPE_OEM_SESSION    "Likewise.SRV.OEMSession"
+#define SRV_ECP_TYPE_NET_OPEN_INFO   "Likewise.SRV.NetworkOpenInfo"
+#define SRV_ECP_TYPE_PIPE_INFO       "Likewise.SRV.FilePipeInfo"
+#define SRV_ECP_TYPE_PIPE_LOCAL_INFO "Likewise.SRV.FilePipeLocalInfo"
 
 #endif /* __SRV_ECP_H__ */
