@@ -43,7 +43,8 @@
  */
 
 #include "iop.h"
-#include "ioapi.h"
+#include <lwio/ioapi.h>
+
 
 // Need to add a way to cancel operation from outside IRP layer.
 // Probably requires something in IO_ASYNC_CONTROL_BLOCK.
@@ -113,11 +114,8 @@ IoCreateFile(
     IRP_TYPE irpType = IRP_TYPE_CREATE;
     PIO_FILE_OBJECT pResultFileObject = NULL;
 
-    if (!FileHandle || !IoStatusBlock)
-    {
-        status = STATUS_INVALID_PARAMETER;
-        GOTO_CLEANUP_ON_STATUS_EE(status, EE);
-    }
+    LWIO_ASSERT(FileHandle);
+    LWIO_ASSERT(IoStatusBlock);
 
     if (SecurityQualityOfService)
     {
@@ -205,6 +203,7 @@ cleanup:
     RtlWC16StringFree(&pszFileName);
     IopIrpDereference(&pIrp);
     IopFileObjectDereference(&pFileObject);
+    IopDeviceDereference(&pDevice);
 
     // Ensure that status is in status block even on early bail.
     if (status && (STATUS_PENDING != status))
@@ -228,7 +227,15 @@ IoCancelFile(
     IN IO_FILE_HANDLE FileHandle
     )
 {
-    IopIrpCancelFileObject(FileHandle);
+    IopIrpCancelFileObject(FileHandle, FALSE);
+}
+
+VOID
+IoCancelForRundownFile(
+    IN IO_FILE_HANDLE FileHandle
+    )
+{
+    IopIrpCancelFileObject(FileHandle, TRUE);
 }
 
 NTSTATUS
@@ -250,7 +257,7 @@ IoAsyncCloseFile(
     OUT PIO_STATUS_BLOCK IoStatusBlock
     )
 {
-    return IopFileObjectRundown(
+    return IopFileObjectRundownEx(
                     FileHandle,
                     AsyncControlBlock ? AsyncControlBlock->Callback : NULL,
                     AsyncControlBlock ? AsyncControlBlock->CallbackContext : NULL,
@@ -286,7 +293,7 @@ IopReadWriteFile(
     IN BOOLEAN bIsPagingIo,
     IN OUT PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -296,7 +303,9 @@ IopReadWriteFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     IRP_TYPE irpType = bIsWrite ? IRP_TYPE_WRITE : IRP_TYPE_READ;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -349,7 +358,7 @@ IoReadFile(
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     OUT PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -372,7 +381,7 @@ IoWriteFile(
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     IN PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -395,7 +404,7 @@ IoPagingReadFile(
     OUT PIO_STATUS_BLOCK IoStatusBlock,
     OUT PVOID Buffer,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key
     )
 {
@@ -431,7 +440,7 @@ IopPrepareZctReadWriteFile(
     IN BOOLEAN bIsWrite,
     IN OUT PLW_ZCT_VECTOR Zct,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key,
     OUT PVOID* CompletionContext
     )
@@ -446,7 +455,10 @@ IopPrepareZctReadWriteFile(
     PVOID completionContext = NULL;
     PIRP pCompletionIrp = NULL;
 
-    if (!FileHandle || !IoStatusBlock || !Zct)
+    LWIO_ASSERT(IoStatusBlock);
+    LWIO_ASSERT(CompletionContext);
+
+    if (!FileHandle || !Zct)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -557,7 +569,9 @@ IopCompleteZctReadWriteFile(
     BOOLEAN bIsPagingIo = IsSetFlag(IoFlags, IO_FLAG_PAGING_IO);
     IRP_TYPE irpType = bIsWrite ? IRP_TYPE_WRITE : IRP_TYPE_READ;
 
-    if (!FileHandle || !IoStatusBlock || !CompletionContext)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle || !CompletionContext)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -596,6 +610,13 @@ IopCompleteZctReadWriteFile(
 cleanup:
     IopIrpDereference(&pIrp);
 
+    // ZCT read completion on an already run-down file is ok
+    if ((STATUS_CANCELLED == status) && !bIsWrite)
+    {
+        status = STATUS_SUCCESS;
+        ioStatusBlock.Status = status;
+    }
+
     if (STATUS_PENDING != status)
     {
         *IoStatusBlock = ioStatusBlock;
@@ -613,7 +634,7 @@ IoPrepareZctReadFile(
     IN IO_FLAGS IoFlags,
     IN OUT PLW_ZCT_VECTOR Zct,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key,
     OUT PVOID* CompletionContext
     )
@@ -658,7 +679,7 @@ IoPrepareZctWriteFile(
     IN IO_FLAGS IoFlags,
     IN OUT PLW_ZCT_VECTOR Zct,
     IN ULONG Length,
-    IN OPTIONAL PLONG64 ByteOffset,
+    IN OPTIONAL PULONG64 ByteOffset,
     IN OPTIONAL PULONG Key,
     OUT PVOID* CompletionContext
     )
@@ -716,7 +737,9 @@ IopControlFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     IRP_TYPE irpType = bIsFsControl? IRP_TYPE_FS_CONTROL : IRP_TYPE_DEVICE_IO_CONTROL;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -813,7 +836,9 @@ IoFlushBuffersFile(
     PIRP pIrp = NULL;
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -862,7 +887,9 @@ IopQuerySetInformationFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     IRP_TYPE irpType = bIsSet ? IRP_TYPE_SET_INFORMATION : IRP_TYPE_QUERY_INFORMATION;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -976,7 +1003,9 @@ IoQueryDirectoryFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     PIO_MATCH_FILE_SPEC fileSpec = NULL;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1053,7 +1082,9 @@ IoReadDirectoryChangeFile(
     PIRP pIrp = NULL;
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1109,7 +1140,9 @@ IopQuerySetVolumeInformationFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     IRP_TYPE irpType = bIsSet ? IRP_TYPE_SET_VOLUME_INFORMATION : IRP_TYPE_QUERY_VOLUME_INFORMATION;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1190,8 +1223,8 @@ IoLockFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
-    IN LONG64 ByteOffset,
-    IN LONG64 Length,
+    IN ULONG64 ByteOffset,
+    IN ULONG64 Length,
     IN ULONG Key,
     IN BOOLEAN FailImmediately,
     IN BOOLEAN ExclusiveLock
@@ -1202,7 +1235,9 @@ IoLockFile(
     PIRP pIrp = NULL;
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1212,7 +1247,7 @@ IoLockFile(
     ioStatusBlock.Status = status;
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pIrp->Args.LockControl.LockControl = IO_LOCK_CONTROL_LOCK;
+    pIrp->Args.LockControl.LockControl = IO_LOCK_CONTROL_LOCK;    
     pIrp->Args.LockControl.ByteOffset = ByteOffset;
     pIrp->Args.LockControl.Length = Length;
     pIrp->Args.LockControl.Key = Key;
@@ -1248,8 +1283,8 @@ IoUnlockFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
-    IN LONG64 ByteOffset,
-    IN LONG64 Length,
+    IN ULONG64 ByteOffset,
+    IN ULONG64 Length,
     IN ULONG Key
     )
 {
@@ -1258,7 +1293,9 @@ IoUnlockFile(
     PIRP pIrp = NULL;
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1353,16 +1390,60 @@ IoQueryQuotaInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
-    OUT PVOID Buffer,
+    OUT PFILE_QUOTA_INFORMATION Buffer,
     IN ULONG Length,
     IN BOOLEAN ReturnSingleEntry,
-    IN OPTIONAL PVOID SidList,
+    IN OPTIONAL PFILE_GET_QUOTA_INFORMATION SidList,
     IN ULONG SidListLength,
-    IN OPTIONAL PSID StartSid,
+    IN OPTIONAL PFILE_GET_QUOTA_INFORMATION StartSid,
     IN BOOLEAN RestartScan
     )
 {
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    NTSTATUS status = 0;
+    int EE = 0;
+    PIRP pIrp = NULL;
+    IO_STATUS_BLOCK ioStatusBlock = { 0 };
+    IRP_TYPE irpType = IRP_TYPE_QUERY_QUOTA;
+
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
+    {
+        status = STATUS_INVALID_PARAMETER;
+        GOTO_CLEANUP_EE(EE);
+    }
+
+    status = IopIrpCreate(&pIrp, irpType, FileHandle);
+    ioStatusBlock.Status = status;
+    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+
+    pIrp->Args.QueryQuota.Buffer = Buffer;
+    pIrp->Args.QueryQuota.Length = Length;
+    pIrp->Args.QueryQuota.ReturnSingleEntry = ReturnSingleEntry;
+    pIrp->Args.QueryQuota.SidList = SidList;
+    pIrp->Args.QueryQuota.SidListLength = SidListLength;
+    pIrp->Args.QueryQuota.StartSid = StartSid;
+    pIrp->Args.QueryQuota.RestartScan = RestartScan;
+
+    status = IopIrpDispatch(
+                    pIrp,
+                    AsyncControlBlock,
+                    IoStatusBlock);
+    if (STATUS_PENDING != status)
+    {
+        ioStatusBlock = pIrp->IoStatusBlock;
+        LWIO_ASSERT(ioStatusBlock.BytesTransferred <= Length);
+    }
+
+cleanup:
+    IopIrpDereference(&pIrp);
+
+    if (STATUS_PENDING != status)
+    {
+        *IoStatusBlock = ioStatusBlock;
+    }
+
+    LOG_LEAVE_IF_STATUS_EE(status, EE);
     return status;
 }
 
@@ -1375,11 +1456,50 @@ IoSetQuotaInformationFile(
     IN IO_FILE_HANDLE FileHandle,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
     OUT PIO_STATUS_BLOCK IoStatusBlock,
-    IN PVOID Buffer,
+    IN PFILE_QUOTA_INFORMATION Buffer,
     IN ULONG Length
     )
 {
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    NTSTATUS status = 0;
+    int EE = 0;
+    PIRP pIrp = NULL;
+    IO_STATUS_BLOCK ioStatusBlock = { 0 };
+    IRP_TYPE irpType = IRP_TYPE_SET_QUOTA;
+
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
+    {
+        status = STATUS_INVALID_PARAMETER;
+        GOTO_CLEANUP_EE(EE);
+    }
+
+    status = IopIrpCreate(&pIrp, irpType, FileHandle);
+    ioStatusBlock.Status = status;
+    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+
+    pIrp->Args.SetQuota.Buffer = Buffer;
+    pIrp->Args.SetQuota.Length = Length;
+    
+    status = IopIrpDispatch(
+                    pIrp,
+                    AsyncControlBlock,
+                    IoStatusBlock);
+    if (STATUS_PENDING != status)
+    {
+        ioStatusBlock = pIrp->IoStatusBlock;
+        LWIO_ASSERT(ioStatusBlock.BytesTransferred == 0);
+    }
+
+cleanup:
+    IopIrpDereference(&pIrp);
+
+    if (STATUS_PENDING != status)
+    {
+        *IoStatusBlock = ioStatusBlock;
+    }
+
+    LOG_LEAVE_IF_STATUS_EE(status, EE);
     return status;
 }
 
@@ -1401,7 +1521,9 @@ IopQuerySetSecurityFile(
     IO_STATUS_BLOCK ioStatusBlock = { 0 };
     IRP_TYPE irpType = bIsSet ? IRP_TYPE_SET_SECURITY : IRP_TYPE_QUERY_SECURITY;
 
-    if (!FileHandle || !IoStatusBlock)
+    LWIO_ASSERT(IoStatusBlock);
+
+    if (!FileHandle)
     {
         status = STATUS_INVALID_PARAMETER;
         GOTO_CLEANUP_EE(EE);
@@ -1411,7 +1533,7 @@ IopQuerySetSecurityFile(
     ioStatusBlock.Status = status;
     GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
-    pIrp->Args.QuerySetSecurity.SecurityInformation = SecurityInformation;
+    pIrp->Args.QuerySetSecurity.SecurityInformation = SecurityInformation;    
     pIrp->Args.QuerySetSecurity.SecurityDescriptor = SecurityDescriptor;
     pIrp->Args.QuerySetSecurity.Length = Length;
 
@@ -1480,7 +1602,7 @@ IoSetSecurityFile(
 
 
 
-
+ 
 /*
 local variables:
 mode: c
@@ -1489,4 +1611,4 @@ indent-tabs-mode: nil
 tab-width: 4
 end:
 */
-
+ 
