@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -163,26 +163,33 @@ IopIrpAttach(
     LWIO_ASSERT(Type != IRP_TYPE_UNINITIALIZED);
 
     IopFileObjectLock(pFileObject);
+
     // TODO-Add FILE_OBJECT_FLAG_CLOSED
-    if ((Type != IRP_TYPE_CLOSE) && IsSetFlag(pFileObject->Flags, FILE_OBJECT_FLAG_RUNDOWN))
+    if ((Type != IRP_TYPE_CLOSE) &&
+        (IsSetFlag(pFileObject->Flags, FILE_OBJECT_FLAG_CANCELLED) ||
+         IsSetFlag(pFileObject->Flags, FILE_OBJECT_FLAG_RUNDOWN)))
     {
         status = STATUS_CANCELLED;
+        GOTO_CLEANUP_EE(EE);
     }
     else
     {
         LwListInsertTail(&pFileObject->IrpList,
                          &irpInternal->FileObjectLinks);
     }
-    IopFileObjectUnlock(pFileObject);
-    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
 
+    // These are immutable from here until the IRP is freed.
     pIrp->Type = Type;
     pIrp->FileHandle = pFileObject;
     IopFileObjectReference(pFileObject);
+    // The file object reference keeps an implicit reference to the
+    // device and driver.
     pIrp->DeviceHandle = pFileObject->pDevice;
     pIrp->DriverHandle = pFileObject->pDevice->Driver;
 
 cleanup:
+    IopFileObjectUnlock(pFileObject);
+
     IO_LOG_LEAVE_ON_STATUS_EE(status, EE);
     return status;
 }
@@ -428,7 +435,7 @@ IopIrpReference(
 {
     PIRP_INTERNAL irpInternal = IopIrpGetInternal(pIrp);
     InterlockedIncrement(&irpInternal->ReferenceCount);
-}
+}   
 
 VOID
 IopIrpDereference(
@@ -590,11 +597,10 @@ NTSTATUS
 IopIrpDispatch(
     IN PIRP pIrp,
     IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK AsyncControlBlock,
-    IN PIO_STATUS_BLOCK pIoStatusBlock
+    OUT PIO_STATUS_BLOCK pIoStatusBlock
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    int EE = 0;
     BOOLEAN isAsyncCall = FALSE;
     LW_RTL_EVENT event = LW_RTL_EVENT_ZERO_INITIALIZER;
     PIRP pExtraIrpReference = NULL;
@@ -628,7 +634,7 @@ IopIrpDispatch(
         LWIO_ASSERT(!irpInternal->Completion.IsAsyncCall);
 
         status = LwRtlInitializeEvent(&event);
-        GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+        GOTO_CLEANUP_ON_STATUS(status);
 
         irpInternal->Completion.Sync.Event = &event;
     }
@@ -640,7 +646,7 @@ IopIrpDispatch(
     // it gets subtracted.
 
     status = IopFileObjectAddDispatched(pIrp->FileHandle, pIrp->Type);
-    GOTO_CLEANUP_ON_STATUS_EE(status, EE);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     SetFlag(irpInternal->Flags, IRP_FLAG_DISPATCHED);
 
@@ -684,6 +690,9 @@ IopIrpDispatch(
 
     LWIO_ASSERT((STATUS_PENDING == status) || (pIrp->IoStatusBlock.Status == status));
 
+cleanup:
+    LwRtlCleanupEvent(&event);
+
     if (STATUS_PENDING == status)
     {
         LWIO_ASSERT(isAsyncCall);
@@ -701,17 +710,11 @@ IopIrpDispatch(
 
             IopIrpDereference(&pExtraIrpReference);
         }
-    }
 
-cleanup:
-    if (STATUS_PENDING != status)
-    {
         pIrp->IoStatusBlock.Status = status;
         *pIoStatusBlock = pIrp->IoStatusBlock;
     }
 
-    LwRtlCleanupEvent(&event);
-
     LWIO_ASSERT(IS_BOTH_OR_NEITHER(pExtraIrpReference, (STATUS_PENDING == status)));
     LWIO_ASSERT((STATUS_PENDING != status) || isAsyncCall);
     LWIO_ASSERT(IsValidStatusForIrpType(status, irpType));
@@ -720,9 +723,11 @@ cleanup:
 
 VOID
 IopIrpCancelFileObject(
-    IN PIO_FILE_OBJECT pFileObject
+    IN PIO_FILE_OBJECT pFileObject,
+    IN BOOLEAN IsForRundown
     )
 {
+    BOOLEAN isLocked = FALSE;
     PLW_LIST_LINKS pLinks = NULL;
     PIRP_INTERNAL irpInternal = NULL;
     LW_LIST_LINKS cancelList = { 0 };
@@ -732,6 +737,18 @@ IopIrpCancelFileObject(
 
     // Gather IRPs we want to cancel while holding FO lock.
     IopFileObjectLock(pFileObject);
+    isLocked = TRUE;
+
+    if (IsSetFlag(pFileObject->Flags, FILE_OBJECT_FLAG_CANCELLED))
+    {
+        GOTO_CLEANUP();
+    }
+
+    if (IsForRundown)
+    {
+        SetFlag(pFileObject->Flags, FILE_OBJECT_FLAG_CANCELLED);
+    }
+
     // gather list of IRPs
     for (pLinks = pFileObject->IrpList.Next;
          pLinks != &pFileObject->IrpList;
@@ -749,6 +766,7 @@ IopIrpCancelFileObject(
         }
     }
     IopFileObjectUnlock(pFileObject);
+    isLocked = FALSE;
 
     // Iterate over list, calling IopIrpCancel as appropriate.
     while (!LwListIsEmpty(&cancelList))
@@ -760,6 +778,12 @@ IopIrpCancelFileObject(
         IopIrpCancel(pIrp);
         IopIrpDereference(&pIrp);
     }
+
+cleanup:
+    if (isLocked)
+    {
+        IopFileObjectUnlock(pFileObject);
+    }
 }
 
 VOID
