@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.  You should have received a copy of the GNU General
- * Public License along with this program.  If not, see
+ * Public License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  *
  * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
@@ -84,7 +84,7 @@ LwioSrvRefreshConfig(
 
     BAIL_ON_NT_STATUS(ntStatus);
 
-    ntStatus = IoMgrRefresh();
+    ntStatus = IoMgrRefreshConfig();
     BAIL_ON_NT_STATUS(ntStatus);
 
 cleanup:
@@ -131,7 +131,7 @@ LwioSrvReadRegistry(
         pReg,
         "MaxOpenFileDescriptors",
         TRUE,
-        1,
+        0,
         0xFFFFFFFF,
         &pConfig->MaxOpenFileDescriptors);
 
