@@ -55,21 +55,15 @@ extern PSMBSERVERINFO gpServerInfo;
 extern pthread_t gSignalHandlerThread;
 extern pthread_t* gpSignalHandlerThread;
 
-#define DAEMON_NAME "lwiod"
-#define PID_DIR "/var/run"
-#define PID_FILE PID_DIR "/" DAEMON_NAME ".pid"
-
-#define PID_FILE_CONTENTS_SIZE ((9 * 2) + 2)
-
 #define LWIO_LOCK_SERVERINFO(bInLock)                  \
         if (!bInLock) {                               \
-           pthread_mutex_lock(&gpServerInfo->lock);   \
+           pthread_mutex_lock(gpServerInfo->pLock);   \
            bInLock = TRUE;                            \
         }
 
 #define LWIO_UNLOCK_SERVERINFO(bInLock)                \
         if (bInLock) {                                \
-           pthread_mutex_unlock(&gpServerInfo->lock); \
+           pthread_mutex_unlock(gpServerInfo->pLock); \
            bInLock = FALSE;                           \
         }
 
