@@ -34,6 +34,67 @@
 #include "ntlogmacros.h"
 #include "ioinit.h"
 
+static
+NTSTATUS
+LwIoGetPeerIdentity(
+    IN LWMsgCall* pCall,
+    OUT uid_t* pUid,
+    OUT gid_t* pGid
+    )
+{
+    NTSTATUS ntStatus = 0;
+    LWMsgSession* pSession = lwmsg_call_get_session(pCall);
+    LWMsgSecurityToken* token = lwmsg_session_get_peer_security_token(pSession);
+    uid_t uid = (uid_t) -1;
+    gid_t gid = (gid_t) -1;
+
+    if (token == NULL || strcmp(lwmsg_security_token_get_type(token), "local"))
+    {
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_LWIO_ERROR(ntStatus);
+    }
+
+    ntStatus = NtIpcLWMsgStatusToNtStatus(lwmsg_local_token_get_eid(token, &uid, &gid));
+    BAIL_ON_LWIO_ERROR(ntStatus);
+
+error:
+
+     if (pUid)
+     {
+         *pUid = uid;
+     }
+
+     if (pGid)
+     {
+         *pGid = gid;
+     }
+
+    return ntStatus;
+}
+
+static
+NTSTATUS
+LwIoVerifyRootAccess(
+    IN LWMsgCall* pCall
+    )
+{
+    NTSTATUS ntStatus = 0;
+    uid_t uid = 0;
+
+    ntStatus = LwIoGetPeerIdentity(pCall, &uid, NULL);
+    BAIL_ON_LWIO_ERROR(ntStatus);
+
+    if (uid != 0)
+    {
+        ntStatus = STATUS_ACCESS_DENIED;
+        BAIL_ON_LWIO_ERROR(ntStatus);
+    }
+
+error:
+
+    return ntStatus;
+}
+
 static
 LWMsgStatus
 LwIoDaemonIpcRefreshConfiguration(
@@ -43,19 +104,20 @@ LwIoDaemonIpcRefreshConfiguration(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
     NTSTATUS ntStatus = STATUS_SUCCESS;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
 
-    dwError = SMBAllocateMemory(
+    ntStatus = LwIoVerifyRootAccess(pCall);
+    BAIL_ON_LWIO_ERROR(ntStatus);
+
+    ntStatus = LwIoAllocateMemory(
                     sizeof(LWIO_STATUS_REPLY),
                     (PVOID*)&pStatusResponse);
-    BAIL_ON_LWIO_ERROR(dwError);
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
     ntStatus = LwioSrvRefreshConfig(&gLwioServerConfig);
 
-    /* Transmit refresh error to client but do not fail out of dispatch loop */
     if (ntStatus)
     {
         pStatusResponse->dwError = LwNtStatusToWin32Error(ntStatus);
@@ -86,27 +148,28 @@ LwIoDaemonIpcSetLogInfo(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
 
-    dwError = SMBAllocateMemory(
+    ntStatus = LwIoVerifyRootAccess(pCall);
+    BAIL_ON_LWIO_ERROR(ntStatus);
+
+    ntStatus = LwIoAllocateMemory(
                     sizeof(LWIO_STATUS_REPLY),
                     (PVOID*)&pStatusResponse);
-    BAIL_ON_LWIO_ERROR(dwError);
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    BAIL_ON_INVALID_POINTER(pIn->data);
-
-    dwError = SMBLogSetInfo_r((PLWIO_LOG_INFO)pIn->data);
+    ntStatus = LwioLogSetInfo_r((PLWIO_LOG_INFO)pIn->data);
 
     /* Transmit failure to client but do not bail out of dispatch loop */
-    if (dwError)
+    if (ntStatus)
     {
-        pStatusResponse->dwError = dwError;
+        pStatusResponse->dwError = ntStatus;
         pOut->tag = LWIO_SET_LOG_INFO_FAILED;
         pOut->data = pStatusResponse;
 
-        dwError = 0;
+        ntStatus = 0;
         goto cleanup;
     }
 
@@ -131,26 +194,26 @@ LwIoDaemonIpcGetLogInfo(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
     PLWIO_LOG_INFO pLogInfo = NULL;
 
-    dwError = SMBAllocateMemory(
+    ntStatus = LwIoAllocateMemory(
                     sizeof(LWIO_STATUS_REPLY),
                     (PVOID*)&pStatusResponse);
-    BAIL_ON_LWIO_ERROR(dwError);
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    dwError = SMBLogGetInfo_r(&pLogInfo);
+    ntStatus = LwioLogGetInfo_r(&pLogInfo);
 
-    if (dwError)
+    if (ntStatus)
     {
-        pStatusResponse->dwError = dwError;
+        pStatusResponse->dwError = ntStatus;
         pOut->tag = LWIO_GET_LOG_INFO_FAILED;
         pOut->data = pStatusResponse;
         pStatusResponse = NULL;
 
-        dwError = 0;
+        ntStatus = 0;
         goto cleanup;
     }
 
@@ -170,41 +233,44 @@ error:
 
 static
 LWMsgStatus
-LwIoDaemonIpcGetDriverStatus(
+LwIoDaemonIpcQueryStateDriver(
     IN LWMsgCall* pCall,
     IN const LWMsgParams* pIn,
     OUT LWMsgParams* pOut,
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PWSTR pwszDriverName = pIn->data;
-    PLWIO_DRIVER_STATUS pStatus = NULL;
+    PLWIO_DRIVER_STATE pState = NULL;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
 
-    dwError = SMBAllocateMemory(sizeof(*pStatus), OUT_PPVOID(&pStatus));
-    BAIL_ON_LWIO_ERROR(dwError);
+    ntStatus = LwIoAllocateMemory(sizeof(*pState), OUT_PPVOID(&pState));
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    if (dwError)
+    ntStatus = LwIoAllocateMemory(sizeof(*pStatusResponse), OUT_PPVOID(&pStatusResponse));
+    BAIL_ON_LWIO_ERROR(ntStatus);
+
+    ntStatus = IoMgrQueryStateDriver(pwszDriverName, pState);
+    if (ntStatus)
     {
-        pStatusResponse->dwError = dwError;
-        pOut->tag = LWIO_GET_DRIVER_STATUS_FAILED;
+        pStatusResponse->dwError = ntStatus;
+        pOut->tag = LWIO_QUERY_STATE_DRIVER_FAILED;
         pOut->data = pStatusResponse;
         pStatusResponse = NULL;
 
-        dwError = 0;
+        ntStatus = 0;
         goto cleanup;
     }
 
-    *pStatus = IoMgrGetDriverStatus(pwszDriverName);
-    pOut->tag = LWIO_GET_DRIVER_STATUS_SUCCESS;
-    pOut->data = pStatus;
-    pStatus = NULL;
+    pOut->tag = LWIO_QUERY_STATE_DRIVER_SUCCESS;
+    pOut->data = pState;
+    pState = NULL;
 
 cleanup:
 
-    LWIO_SAFE_FREE_MEMORY(pStatus);
+    LWIO_SAFE_FREE_MEMORY(pState);
     LWIO_SAFE_FREE_MEMORY(pStatusResponse);
 
     return status;
@@ -223,18 +289,21 @@ LwIoDaemonIpcLoadDriver(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PWSTR pwszDriverName = pIn->data;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
 
-    dwError = SMBAllocateMemory(sizeof(*pStatusResponse), OUT_PPVOID(&pStatusResponse));
-    BAIL_ON_LWIO_ERROR(dwError);
+    ntStatus = LwIoVerifyRootAccess(pCall);
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    dwError = IoMgrLoadDriver(pwszDriverName);
+    ntStatus = LwIoAllocateMemory(sizeof(*pStatusResponse), OUT_PPVOID(&pStatusResponse));
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    pStatusResponse->dwError = dwError;
-    pOut->tag = dwError ? LWIO_LOAD_DRIVER_SUCCESS : LWIO_LOAD_DRIVER_FAILED;
+    ntStatus = IoMgrLoadDriver(pwszDriverName);
+    
+    pStatusResponse->dwError = ntStatus;
+    pOut->tag = ntStatus ? LWIO_LOAD_DRIVER_SUCCESS : LWIO_LOAD_DRIVER_FAILED;
     pOut->data = pStatusResponse;
     pStatusResponse = NULL;
 
@@ -258,18 +327,21 @@ LwIoDaemonIpcUnloadDriver(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PWSTR pwszDriverName = pIn->data;
     PLWIO_STATUS_REPLY pStatusResponse = NULL;
 
-    dwError = SMBAllocateMemory(sizeof(*pStatusResponse), OUT_PPVOID(&pStatusResponse));
-    BAIL_ON_LWIO_ERROR(dwError);
+    ntStatus = LwIoVerifyRootAccess(pCall);
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    dwError = IoMgrUnloadDriver(pwszDriverName);
+    ntStatus = LwIoAllocateMemory(sizeof(*pStatusResponse), OUT_PPVOID(&pStatusResponse));
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
-    pStatusResponse->dwError = dwError;
-    pOut->tag = dwError ? LWIO_UNLOAD_DRIVER_SUCCESS : LWIO_UNLOAD_DRIVER_FAILED;
+    ntStatus = IoMgrUnloadDriver(pwszDriverName);
+    
+    pStatusResponse->dwError = ntStatus;
+    pOut->tag = ntStatus ? LWIO_UNLOAD_DRIVER_SUCCESS : LWIO_UNLOAD_DRIVER_FAILED;
     pOut->data = pStatusResponse;
     pStatusResponse = NULL;
 
@@ -293,12 +365,12 @@ LwIoDaemonIpcGetPid(
     IN void* pData
     )
 {
-    DWORD dwError = 0;
+    NTSTATUS ntStatus = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     pid_t* pPid = NULL;
 
-    dwError = SMBAllocateMemory(sizeof(*pPid), OUT_PPVOID(&pPid));
-    BAIL_ON_LWIO_ERROR(dwError);
+    ntStatus = LwIoAllocateMemory(sizeof(*pPid), OUT_PPVOID(&pPid));
+    BAIL_ON_LWIO_ERROR(ntStatus);
 
     *pPid = getpid();
 
@@ -322,7 +394,7 @@ static LWMsgDispatchSpec gLwIoDaemonIpcDispatchSpec[] =
     LWMSG_DISPATCH_BLOCK(LWIO_REFRESH_CONFIG, LwIoDaemonIpcRefreshConfiguration),
     LWMSG_DISPATCH_NONBLOCK(LWIO_SET_LOG_INFO, LwIoDaemonIpcSetLogInfo),
     LWMSG_DISPATCH_NONBLOCK(LWIO_GET_LOG_INFO, LwIoDaemonIpcGetLogInfo),
-    LWMSG_DISPATCH_BLOCK(LWIO_GET_DRIVER_STATUS, LwIoDaemonIpcGetDriverStatus),
+    LWMSG_DISPATCH_BLOCK(LWIO_QUERY_STATE_DRIVER, LwIoDaemonIpcQueryStateDriver),
     LWMSG_DISPATCH_BLOCK(LWIO_LOAD_DRIVER, LwIoDaemonIpcLoadDriver),
     LWMSG_DISPATCH_BLOCK(LWIO_UNLOAD_DRIVER, LwIoDaemonIpcUnloadDriver),
     LWMSG_DISPATCH_NONBLOCK(LWIO_GET_PID, LwIoDaemonIpcGetPid),
