@@ -57,7 +57,7 @@ LwioSrvSetDefaults(
 
 static
 DWORD
-SMBSrvParseArgs(
+LwioSrvParseArgs(
     int argc,
     PSTR argv[],
     PSMBSERVERINFO pSMBServerInfo
@@ -75,12 +75,6 @@ ShowUsage(
     PCSTR pszProgramName
     );
 
-static
-VOID
-SMBSrvExitHandler(
-    VOID
-    );
-
 static
 DWORD
 SMBSrvInitialize(
@@ -105,32 +99,6 @@ SMBSrvStartAsDaemon(
     VOID
     );
 
-static
-DWORD
-SMBSrvGetProcessExitCode(
-    PDWORD pdwExitCode
-    );
-
-static
-VOID
-SMBSrvSetProcessExitCode(
-    DWORD dwExitCode
-    );
-
-#ifdef ENABLE_PIDFILE
-static
-VOID
-SMBSrvCreatePIDFile(
-    VOID
-    );
-
-static
-pid_t
-SMBSrvGetPidFromPidFile(
-    VOID
-    );
-#endif
-
 static
 DWORD
 SMBSrvExecute(
@@ -149,6 +117,12 @@ SMBSrvInterruptHandler(
     int sig
     );
 
+static
+VOID
+SMBSrvInterruptDummyHandler(
+    int sig
+    );
+
 static
 VOID
 SMBSrvBlockSignals(
@@ -216,20 +190,15 @@ lwiod_main(
     dwError = LwNtStatusToWin32Error(ntStatus);
     BAIL_ON_LWIO_ERROR(dwError);
 
-    ntStatus = LwioSrvRefreshConfig(&gLwioServerConfig);
-    dwError = LwNtStatusToWin32Error(ntStatus);
-    BAIL_ON_LWIO_ERROR(dwError);
+    pthread_mutex_init(&gServerInfo.lock, NULL);
+    gServerInfo.pLock = &gServerInfo.lock;
 
-    ntStatus = LwioSrvSetDefaults(&gLwioServerConfig);
-    dwError = LwNtStatusToWin32Error(ntStatus);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    dwError = SMBSrvParseArgs(argc,
+    dwError = LwioSrvParseArgs(argc,
                               argv,
                               &gServerInfo);
     BAIL_ON_LWIO_ERROR(dwError);
 
-    dwError = SMBInitLogging_r(
+    dwError = LwioInitLogging_r(
                     SMBGetProgramName(argv[0]),
                     gServerInfo.logTarget,
                     gServerInfo.maxAllowedLogLevel,
@@ -238,11 +207,6 @@ lwiod_main(
 
     LWIO_LOG_VERBOSE("Logging started");
 
-    if (atexit(SMBSrvExitHandler) < 0) {
-       dwError = errno;
-       BAIL_ON_LWIO_ERROR(dwError);
-    }
-
     if (SMBSrvShouldStartAsDaemon()) {
        dwError = SMBSrvStartAsDaemon();
        BAIL_ON_LWIO_ERROR(dwError);
@@ -251,10 +215,6 @@ lwiod_main(
     dwError = LWNetExtendEnvironmentForKrb5Affinity(TRUE);
     BAIL_ON_LWIO_ERROR(dwError);
 
-#ifdef ENABLE_PIDFILE
-    SMBSrvCreatePIDFile();
-#endif
-
     SMBSrvBlockSignals();
 
     dwError = LwDsCacheAddPidException(getpid());
@@ -264,6 +224,14 @@ lwiod_main(
         BAIL_ON_LWIO_ERROR(dwError);
     }
 
+    ntStatus = LwioSrvRefreshConfig(&gLwioServerConfig);
+    dwError = LwNtStatusToWin32Error(ntStatus);
+    BAIL_ON_LWIO_ERROR(dwError);
+
+    ntStatus = LwioSrvSetDefaults(&gLwioServerConfig);
+    dwError = LwNtStatusToWin32Error(ntStatus);
+    BAIL_ON_LWIO_ERROR(dwError);
+
     dwError = SMBSrvInitialize();
     BAIL_ON_LWIO_ERROR(dwError);
 
@@ -272,23 +240,27 @@ lwiod_main(
 
 cleanup:
 
-    LWIO_LOG_VERBOSE("SMB main cleaning up");
+    LWIO_LOG_VERBOSE("LWIO main cleaning up");
 
     IoCleanup();
 
-    LWIO_LOG_INFO("SMB Service exiting...");
+    LWIO_LOG_INFO("LWIO Service exiting...");
 
     LwDsCacheRemovePidException(getpid());
 
-    SMBSrvSetProcessExitCode(dwError);
+    LwioShutdownLogging_r();
 
-    SMBShutdownLogging_r();
+    if (gServerInfo.pLock)
+    {
+        pthread_mutex_destroy(&gServerInfo.lock);
+        gServerInfo.pLock = NULL;
+    }
 
     return dwError;
 
 error:
 
-    LWIO_LOG_ERROR("SMB Process exiting due to error [Code:%d]", dwError);
+    LWIO_LOG_ERROR("LWIO Process exiting due to error [Code:%d]", dwError);
 
     goto cleanup;
 }
@@ -303,23 +275,39 @@ LwioSrvSetDefaults(
     struct rlimit rlim = {0};
     int err = 0;
     LWIO_CONFIG defaultConfig;
+    rlim_t maxOpenFileDescriptors = 0;
 
     gpServerInfo->maxAllowedLogLevel = LWIO_LOG_LEVEL_ERROR;
 
     *(gpServerInfo->szLogFilePath) = '\0';
 
-    strcpy(gpServerInfo->szCachePath, CACHEDIR);
-    strcpy(gpServerInfo->szPrefixPath, PREFIXDIR);
+    strncpy(gpServerInfo->szCachePath, CACHEDIR, PATH_MAX);
+    gpServerInfo->szCachePath[PATH_MAX] = '\0';
+
+    strncpy(gpServerInfo->szPrefixPath, PREFIXDIR, PATH_MAX);
+    gpServerInfo->szPrefixPath[PATH_MAX] = '\0';
 
     setlocale(LC_ALL, "");
 
     // Enforce configuration settings
 
     ntStatus = LwioSrvInitializeConfig(&defaultConfig);
-    BAIL_ON_NT_STATUS(ntStatus);
+    BAIL_ON_NT_STATUS(ntStatus);    
 
-    rlim.rlim_cur = pConfig->MaxOpenFileDescriptors;
-    rlim.rlim_max = pConfig->MaxOpenFileDescriptors;
+    if (pConfig->MaxOpenFileDescriptors > 0)
+    {
+            maxOpenFileDescriptors = pConfig->MaxOpenFileDescriptors;
+            LWIO_LOG_VERBOSE("Setting maxOpenFileDescriptors to %d\n",
+                maxOpenFileDescriptors);
+    }
+    else
+    {
+            maxOpenFileDescriptors = RLIM_INFINITY;
+            LWIO_LOG_VERBOSE("Setting maxOpenFileDescriptors to unlimited\n");
+    }
+
+    rlim.rlim_cur = maxOpenFileDescriptors;
+    rlim.rlim_max = maxOpenFileDescriptors;
 
     if (setrlimit(RLIMIT_NOFILE, &rlim) < 0)
     {
@@ -361,12 +349,12 @@ cleanup:
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 static
 DWORD
-SMBSrvParseArgs(
+LwioSrvParseArgs(
     int argc,
     PSTR argv[],
     PSMBSERVERINFO pSMBServerInfo
@@ -434,7 +422,8 @@ SMBSrvParseArgs(
       case PARSE_MODE_LOGFILE:
 
         {
-          strcpy(pSMBServerInfo->szLogFilePath, pArg);
+          strncpy(pSMBServerInfo->szLogFilePath, pArg, PATH_MAX);
+          pSMBServerInfo->szLogFilePath[PATH_MAX] = '\0';
 
           SMBStripWhitespace(pSMBServerInfo->szLogFilePath, TRUE, TRUE);
 
@@ -557,47 +546,6 @@ ShowUsage(
            "          [--loglevel {error, warning, info, verbose, debug, trace}]\n", pszProgramName);
 }
 
-static
-VOID
-SMBSrvExitHandler(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    DWORD dwExitCode = 0;
-    CHAR  szErrCodeFilePath[PATH_MAX+1];
-    BOOLEAN  bFileExists = 0;
-    FILE* fp = NULL;
-
-    sprintf(szErrCodeFilePath, "%s/lsasd.err", CACHEDIR);
-
-    dwError = SMBCheckFileExists(szErrCodeFilePath, &bFileExists);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    if (bFileExists) {
-        dwError = SMBRemoveFile(szErrCodeFilePath);
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    dwError = SMBSrvGetProcessExitCode(&dwExitCode);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    if (dwExitCode) {
-       fp = fopen(szErrCodeFilePath, "w");
-       if (fp == NULL) {
-          dwError = errno;
-          BAIL_ON_LWIO_ERROR(dwError);
-       }
-       fprintf(fp, "%d\n", dwExitCode);
-    }
-
-error:
-
-    if (fp != NULL) {
-       fclose(fp);
-    }
-}
-
 static
 DWORD
 SMBSrvInitialize(
@@ -605,26 +553,19 @@ SMBSrvInitialize(
     )
 {
     DWORD dwError = 0;
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-
 
     dwError = SMBInitCacheFolders();
     BAIL_ON_LWIO_ERROR(dwError);
 
 #ifdef ENABLE_STATIC_DRIVERS
-    dwError = IoInitialize("", gStaticDrivers);
+    dwError = IoInitialize(gStaticDrivers);
 #else
-    dwError = IoInitialize("", NULL);
+    dwError = IoInitialize(NULL);
 #endif
     BAIL_ON_LWIO_ERROR(dwError);
 
 error:
 
-    if(ntStatus)
-    {
-        dwError = LwNtStatusToWin32Error(ntStatus);
-    }
-
     return dwError;
 }
 
@@ -740,195 +681,24 @@ SMBSrvStartAsDaemon(
     return (dwError);
 }
 
-static
-DWORD
-SMBSrvGetProcessExitCode(
-    PDWORD pdwExitCode
-    )
-{
-    DWORD dwError = 0;
-    BOOLEAN bInLock = FALSE;
-
-    LWIO_LOCK_SERVERINFO(bInLock);
-
-    *pdwExitCode = gpServerInfo->dwExitCode;
-
-    LWIO_UNLOCK_SERVERINFO(bInLock);
-
-    return dwError;
-}
-
-static
-VOID
-SMBSrvSetProcessExitCode(
-    DWORD dwExitCode
-    )
-{
-    BOOLEAN bInLock = FALSE;
-
-    LWIO_LOCK_SERVERINFO(bInLock);
-
-    gpServerInfo->dwExitCode = dwExitCode;
-
-    LWIO_UNLOCK_SERVERINFO(bInLock);
-}
-
-#ifdef ENABLE_PIDFILE
-static
-VOID
-SMBSrvCreatePIDFile(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-    struct stat myStat = {0};
-    struct stat runningStat = {0};
-
-    pid = SMBSrvGetPidFromPidFile();
-    if (pid > 0)
-    {
-        dwError = SMBSrvGetExecutableStatByPid(
-                    getpid(),
-                    &myStat);
-        if (dwError != 0)
-        {
-            fprintf(stderr, "Unable to stat the executable of this program. Make sure this program was invoked with an absolute path.\n");
-            result = -1;
-            goto error;
-        }
-
-        dwError = SMBSrvGetExecutableStatByPid(
-                    pid,
-                    &runningStat);
-        if (dwError == ENOENT || dwError == ESRCH)
-        {
-            runningStat.st_dev = -1;
-            runningStat.st_ino = -1;
-            dwError = 0;
-        }
-        else if(dwError != 0)
-        {
-            fprintf(stderr, "Unable to stat the executable of pid %ld\n", (long)pid);
-            result = -1;
-            goto error;
-        }
-
-        if (runningStat.st_dev == myStat.st_dev &&
-                runningStat.st_ino == myStat.st_ino)
-        {
-            fprintf(stderr, "Daemon already running as %d\n", (int) pid);
-            result = -1;
-            goto error;
-        }
-        else
-        {
-            fprintf(
-                    stderr,
-                    "Warning: the pid file already exists and contains pid %d, but this pid is not owned by this program. Most likely, this daemon shutdown uncleanly on its last run.\n",
-                    (int) pid);
-            if (remove(PID_FILE) < 0)
-            {
-                fprintf(stderr, "Unable to clear existing pid file\n");
-                result = -1;
-                goto error;
-            }
-        }
-    }
-
-    fd = open(PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0) {
-        fprintf(stderr, "Could not create pid file: %s\n", strerror(errno));
-        result = 1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) write(fd, contents, len);
-    if ( result != (int) len ) {
-        fprintf(stderr, "Could not write to pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    if (result < 0) {
-        exit(1);
-    }
-}
-
-static
-pid_t
-SMBSrvGetPidFromPidFile(
-    VOID
-    )
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(PID_FILE, O_RDONLY, 0644);
-    if (fd < 0) {
-        goto error;
-    }
-
-    result = read(fd, contents, sizeof(contents)-1);
-    if (result <= 0) {
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result <= 0) {
-        result = -1;
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH) {
-        unlink(PID_FILE);
-        pid = 0;
-    }
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    return pid;
-}
-#endif
-
 LWMsgBool
 LwIoDaemonLogIpc (
     LWMsgLogLevel level,
     const char* pszMessage,
+    const char* pszFunction,
     const char* pszFilename,
     unsigned int line,
     void* pData
     )
 {
-    LWIO_LOG_LEVEL ioLevel;
-    LWMsgBool result;
+    LWIO_LOG_LEVEL ioLevel = LWIO_LOG_LEVEL_DEBUG;
+    LWMsgBool result = LWMSG_FALSE;
 
     switch (level)
     {
+    case LWMSG_LOGLEVEL_ALWAYS:
+        ioLevel = LWIO_LOG_LEVEL_ALWAYS;
+        break;
     case LWMSG_LOGLEVEL_ERROR:
         ioLevel = LWIO_LOG_LEVEL_ERROR;
         break;
@@ -945,18 +715,21 @@ LwIoDaemonLogIpc (
         ioLevel = LWIO_LOG_LEVEL_DEBUG;
         break;
     case LWMSG_LOGLEVEL_TRACE:
-    default:
         ioLevel = LWIO_LOG_LEVEL_TRACE;
         break;
     }
-
-    LWIO_LOCK_LOGGER;
+    
     if (pszMessage)
     {
-        if (gSMBMaxLogLevel >= ioLevel)
+        if (gLwioMaxLogLevel >= ioLevel)
         {
-            SMBLogMessage(gpfnSMBLogger, ghSMBLog, ioLevel, "[IPC] %s", pszMessage);
-            result = LWMSG_TRUE;
+            LWIO_LOCK_LOGGER;
+            if (gLwioMaxLogLevel >= ioLevel)
+            {
+                LwioLogMessage(gpfnLwioLogger, ghLwioLog, ioLevel, "[IPC] %s", pszMessage);
+                result = LWMSG_TRUE;
+            }
+            LWIO_UNLOCK_LOGGER;
         }
         else
         {
@@ -965,9 +738,8 @@ LwIoDaemonLogIpc (
     }
     else
     {
-        result = (gSMBMaxLogLevel >= ioLevel);
+        result = (gLwioMaxLogLevel >= ioLevel);
     }
-    LWIO_UNLOCK_LOGGER;
 
     return result;
 }
@@ -1040,7 +812,7 @@ SMBSrvExecute(
     if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notifyFd = atoi(pszSmNotify);
-
+        
         do
         {
             ret = write(notifyFd, &notifyCode, sizeof(notifyCode));
@@ -1061,10 +833,7 @@ SMBSrvExecute(
 
 cleanup:
 
-    // Calling exit() here is a temporary work-around until I/O cancellation
-    // is completely plumbed in.
     LWIO_LOG_VERBOSE("Exiting with dwError = %d", dwError);
-    exit(dwError);
 
     if (pServer)
     {
@@ -1078,6 +847,11 @@ cleanup:
         lwmsg_server_delete(pServer);
     }
 
+    if (pContext)
+    {
+        lwmsg_context_delete(pContext);
+    }
+
     return dwError;
 
 error:
@@ -1113,6 +887,26 @@ SMBHandleSignals(
         BAIL_ON_LWIO_ERROR(dwError);
     }
 
+    // Register dummy handlers for the rest of the signals
+    // caught by sigwait();
+    memset(&action, 0, sizeof(action));
+    action.sa_handler = SMBSrvInterruptDummyHandler;
+    if (sigaction(SIGTERM, &action, NULL) != 0)
+    {
+        dwError = errno;
+        BAIL_ON_LWIO_ERROR(dwError);
+    }
+    if (sigaction(SIGPIPE, &action, NULL) != 0)
+    {
+        dwError = errno;
+        BAIL_ON_LWIO_ERROR(dwError);
+    }
+    if (sigaction(SIGHUP, &action, NULL) != 0)
+    {
+        dwError = errno;
+        BAIL_ON_LWIO_ERROR(dwError);
+    }
+
     // Unblock SIGINT
     sigemptyset(&catch_signal_mask);
     sigaddset(&catch_signal_mask, SIGINT);
@@ -1168,6 +962,17 @@ SMBSrvInterruptHandler(
     }
 }
 
+
+static
+VOID
+SMBSrvInterruptDummyHandler(
+    int sig
+    )
+{
+    return;
+}
+
+
 static
 VOID
 SMBSrvBlockSignals(
