@@ -53,6 +53,37 @@
 
 static uchar8_t smbMagic[4] = { 0xFF, 'S', 'M', 'B' };
 
+static
+NTSTATUS
+SMBPacketAllocatePooled(
+    IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    OUT PSMB_PACKET* ppPacket
+    );
+
+static
+VOID
+SMBPacketReleasePooled(
+    IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    IN OUT PSMB_PACKET        pPacket
+    );
+
+static
+NTSTATUS
+SMBPacketBufferAllocatePooled(
+    IN  PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    IN  size_t                 len,
+    OUT uint8_t**              ppBuffer,
+    OUT size_t*                pAllocatedLen
+    );
+
+static
+VOID
+SMBPacketBufferFreePooled(
+    IN  PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    OUT uint8_t*               pBuffer,
+    IN  size_t                 bufferLen
+    );
+
 BOOLEAN
 SMBIsAndXCommand(
     uint8_t command
@@ -88,7 +119,7 @@ SMBPacketCreateAllocator(
     NTSTATUS ntStatus = 0;
     PLWIO_PACKET_ALLOCATOR pPacketAllocator = NULL;
 
-    ntStatus = SMBAllocateMemory(
+    ntStatus = LwIoAllocateMemory(
                     sizeof(LWIO_PACKET_ALLOCATOR),
                     (PVOID*)&pPacketAllocator);
     BAIL_ON_NT_STATUS(ntStatus);
@@ -115,6 +146,42 @@ SMBPacketAllocate(
     IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
     OUT PSMB_PACKET* ppPacket
     )
+{
+    NTSTATUS ntStatus = 0;
+    PSMB_PACKET pPacket = NULL;
+
+    if (pPacketAllocator)
+    {
+        ntStatus = SMBPacketAllocatePooled(pPacketAllocator, &pPacket);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+    else
+    {
+        ntStatus = LwIoAllocateMemory(sizeof(SMB_PACKET), (PVOID *) &pPacket);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        InterlockedIncrement(&pPacket->refCount);
+    }
+
+    *ppPacket = pPacket;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    *ppPacket = NULL;
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SMBPacketAllocatePooled(
+    IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    OUT PSMB_PACKET* ppPacket
+    )
 {
     NTSTATUS ntStatus = 0;
     BOOLEAN bInLock = FALSE;
@@ -138,7 +205,7 @@ SMBPacketAllocate(
     {
         LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
 
-        ntStatus = SMBAllocateMemory(sizeof(SMB_PACKET), (PVOID *) &pPacket);
+        ntStatus = LwIoAllocateMemory(sizeof(SMB_PACKET), (PVOID *) &pPacket);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
@@ -164,6 +231,31 @@ SMBPacketRelease(
     IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
     IN OUT PSMB_PACKET        pPacket
     )
+{
+    if (pPacketAllocator)
+    {
+        SMBPacketReleasePooled(pPacketAllocator, pPacket);
+    }
+    else if (InterlockedDecrement(&pPacket->refCount) == 0)
+    {
+        if (pPacket->pRawBuffer)
+        {
+            SMBPacketBufferFree(
+                    pPacketAllocator,
+                    pPacket->pRawBuffer,
+                    pPacket->bufferLen);
+        }
+
+        LwIoFreeMemory(pPacket);
+    }
+}
+
+static
+VOID
+SMBPacketReleasePooled(
+    IN PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    IN OUT PSMB_PACKET        pPacket
+    )
 {
     if (InterlockedDecrement(&pPacket->refCount) == 0)
     {
@@ -195,7 +287,7 @@ SMBPacketRelease(
         {
             LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
 
-            SMBFreeMemory(pPacket);
+            LwIoFreeMemory(pPacket);
         }
 
         LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
@@ -209,6 +301,51 @@ SMBPacketBufferAllocate(
     OUT uint8_t**              ppBuffer,
     OUT size_t*                pAllocatedLen
     )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    PBYTE    pBuffer = NULL;
+    size_t   allocatedLen = len;
+
+    if (pPacketAllocator)
+    {
+        ntStatus = SMBPacketBufferAllocatePooled(
+                        pPacketAllocator,
+                        len,
+                        &pBuffer,
+                        &allocatedLen);
+    }
+    else
+    {
+        //
+        // Note: The resultant buffer will not be cleared
+        //
+        ntStatus = LwIoAllocateBuffer(len, (PVOID *) &pBuffer);
+    }
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    *ppBuffer = pBuffer;
+    *pAllocatedLen = allocatedLen;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    *ppBuffer = NULL;
+    *pAllocatedLen = 0;
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SMBPacketBufferAllocatePooled(
+    IN  PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    IN  size_t                 len,
+    OUT uint8_t**              ppBuffer,
+    OUT size_t*                pAllocatedLen
+    )
 {
     NTSTATUS ntStatus = 0;
     uint8_t* pBuffer = NULL;
@@ -237,7 +374,7 @@ SMBPacketBufferAllocate(
 
         LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
 
-        memset(pBuffer, 0, allocatedLen);
+        // memset(pBuffer, 0, allocatedLen);
     }
     else
     {
@@ -245,7 +382,7 @@ SMBPacketBufferAllocate(
 
         LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
 
-        ntStatus = SMBAllocateMemory(allocatedLen, (PVOID *) &pBuffer);
+        ntStatus = LwIoAllocateBuffer(allocatedLen, (PVOID *) &pBuffer);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
@@ -272,6 +409,24 @@ SMBPacketBufferFree(
     OUT uint8_t*               pBuffer,
     IN  size_t                 bufferLen
     )
+{
+    if (pPacketAllocator)
+    {
+        SMBPacketBufferFreePooled(pPacketAllocator, pBuffer, bufferLen);
+    }
+    else
+    {
+        LwIoFreeBuffer(pBuffer);
+    }
+}
+
+static
+VOID
+SMBPacketBufferFreePooled(
+    IN  PLWIO_PACKET_ALLOCATOR pPacketAllocator,
+    OUT uint8_t*               pBuffer,
+    IN  size_t                 bufferLen
+    )
 {
     BOOLEAN bInLock = FALSE;
 
@@ -292,7 +447,7 @@ SMBPacketBufferFree(
     {
         LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
 
-        SMBFreeMemory(pBuffer);
+        LwIoFreeBuffer(pBuffer);
     }
 
     LWIO_UNLOCK_MUTEX(bInLock, &pPacketAllocator->mutex);
@@ -330,7 +485,7 @@ SMBPacketFreeAllocator(
         SMBStackFree(pPacketAllocator->pFreePacketStack);
     }
 
-    SMBFreeMemory(pPacketAllocator);
+    LwIoFreeMemory(pPacketAllocator);
 }
 
 static
