@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -47,20 +47,6 @@ typedef struct _SEC_WINNT_AUTH_IDENTITY
 #define GSS_MECH_NTLM       "\x2b\x06\x01\x04\x01\x82\x37\x02\x02\x0a"
 #define GSS_MECH_NTLM_LEN   10
 
-static
-DWORD
-SMBGetServerCanonicalName(
-    PCSTR pszServerName,
-    PSTR* ppszNormal
-    );
-
-static
-DWORD
-SMBGetServerDomain(
-    PCSTR pszServerName,
-    PSTR* ppszDomain
-    );
-
 static
 void
 smb_display_status(
@@ -104,7 +90,7 @@ SMBKrb5SetDefaultCachePath(
         }
     }
 
-    LWIO_LOG_DEBUG("Cache path set to [%s]", SMB_SAFE_LOG_STRING(pszCachePath));
+    LWIO_LOG_DEBUG("Cache path set to [%s]", LWIO_SAFE_LOG_STRING(pszCachePath));
 
 cleanup:
 
@@ -169,7 +155,6 @@ SMBGSSContextBuild(
     PSMB_GSS_SEC_CONTEXT pContext = NULL;
     PSTR pszTargetName = NULL;
     PSTR pszServerName = NULL;
-    PSTR pszDomainName = NULL;
     PSTR pszUsername = NULL;
     PSTR pszDomain = NULL;
     PSTR pszPassword = NULL;
@@ -195,16 +180,16 @@ SMBGSSContextBuild(
 
     dwError = LwRtlCStringAllocateFromWC16String(&pszServerName, pwszServerName);
     BAIL_ON_LWIO_ERROR(dwError);
+    
+    LWIO_LOG_DEBUG("Build GSS Context for server [%s]", LWIO_SAFE_LOG_STRING(pszServerName));
 
-    LWIO_LOG_DEBUG("Build GSS Context for server [%s]", SMB_SAFE_LOG_STRING(pszServerName));
-
-    dwError = SMBAllocateMemory(
+    dwError = LwIoAllocateMemory(
         sizeof(SMB_GSS_SEC_CONTEXT),
         (PVOID*)&pContext);
     BAIL_ON_LWIO_ERROR(dwError);
 
     pContext->state = SMB_GSS_SEC_CONTEXT_STATE_INITIAL;
-
+    
     if (pCreds)
     {
         switch (pCreds->type)
@@ -214,11 +199,6 @@ SMBGSSContextBuild(
             BAIL_ON_LWIO_ERROR(dwError);
             break;
         case IO_CREDS_TYPE_KRB5_TGT:
-            dwError = SMBGetServerDomain(pszServerName, &pszDomainName);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            SMBStrToUpper(pszDomainName);
-
             sCopyServerChars = strlen(pszServerName);
             if (sCopyServerChars > 0 &&
                     pszServerName[sCopyServerChars - 1] == '.')
@@ -226,7 +206,7 @@ SMBGSSContextBuild(
                 // Strip the trailing dot
                 sCopyServerChars --;
             }
-
+            
             if (sCopyServerChars > INT_MAX)
             {
                 dwError = STATUS_INTEGER_OVERFLOW;
@@ -235,15 +215,14 @@ SMBGSSContextBuild(
 
             dwError = SMBAllocateStringPrintf(
                             &pszTargetName,
-                            "cifs/%.*s@%s",
+                            "cifs/%.*s@",
                             (int)sCopyServerChars,
-                            pszServerName,
-                            pszDomainName);
+                            pszServerName);
             BAIL_ON_LWIO_ERROR(dwError);
 
             inputNameBuffer.value = pszTargetName;
             inputNameBuffer.length = strlen(pszTargetName) + 1;
-
+            
             dwMajorStatus = gss_import_name(
                 (OM_uint32 *)&dwMinorStatus,
                 &inputNameBuffer,
@@ -253,12 +232,41 @@ SMBGSSContextBuild(
             smb_display_status("gss_import_name", dwMajorStatus, dwMinorStatus);
 
             BAIL_ON_SEC_ERROR(dwMajorStatus);
+
+            dwError = LwRtlCStringAllocateFromWC16String(
+                &pszUsername,
+                pCreds->payload.krb5Tgt.pwszClientPrincipal);
+            BAIL_ON_NT_STATUS(dwError);
+
+            usernameBuffer.value = pszUsername;
+            usernameBuffer.length = strlen(pszUsername) + 1;
+
+            dwMajorStatus = gss_import_name(
+                (OM_uint32 *)&dwMinorStatus,
+                &usernameBuffer,
+                GSS_C_NT_USER_NAME,
+                &pUsername);
+            BAIL_ON_SEC_ERROR(dwMajorStatus);
+
+            desiredMechs.count = 1;
+            desiredMechs.elements = (gss_OID) gss_mech_krb5;
+
+            dwMajorStatus = gss_acquire_cred(
+                (OM_uint32 *)&dwMinorStatus,
+                pUsername,
+                0,
+                &desiredMechs,
+                GSS_C_INITIATE,
+                &pContext->credHandle,
+                &actualMechs,
+                &timeRec);
+            BAIL_ON_SEC_ERROR(dwMajorStatus);
             break;
 
         case IO_CREDS_TYPE_PLAIN:
             inputNameBuffer.value = (void*) "unset";
             inputNameBuffer.length = strlen("unset");
-
+            
             dwMajorStatus = gss_import_name(
                 (OM_uint32 *)&dwMinorStatus,
                 &inputNameBuffer,
@@ -266,17 +274,17 @@ SMBGSSContextBuild(
                 &pContext->target_name);
 
             smb_display_status("gss_import_name", dwMajorStatus, dwMinorStatus);
-
+            
             BAIL_ON_SEC_ERROR(dwMajorStatus);
-
+            
             if (pCreds->payload.plain.pwszUsername)
             {
                 dwError = LwRtlCStringAllocateFromWC16String(&pszUsername, pCreds->payload.plain.pwszUsername);
                 BAIL_ON_LWIO_ERROR(dwError);
-
+                
                 usernameBuffer.value = pszUsername;
                 usernameBuffer.length = strlen(pszUsername);
-
+                
                 // If "" is passed in, that means to use anonymous
                 // authentication. gss_import_name fails on "" though
                 if (usernameBuffer.length)
@@ -289,10 +297,10 @@ SMBGSSContextBuild(
                     BAIL_ON_SEC_ERROR(dwMajorStatus);
                 }
             }
-
+            
             desiredMechs.count = 1;
             desiredMechs.elements = (gss_OID) &gssNtlmOidDesc;
-
+            
             dwMajorStatus = gss_acquire_cred(
                 (OM_uint32 *)&dwMinorStatus,
                 pUsername,
@@ -310,10 +318,10 @@ SMBGSSContextBuild(
             {
                 dwError = LwRtlCStringAllocateFromWC16String(&pszDomain, pCreds->payload.plain.pwszDomain);
                 BAIL_ON_LWIO_ERROR(dwError);
-
+                
                 dwError = LwRtlCStringAllocateFromWC16String(&pszPassword, pCreds->payload.plain.pwszPassword);
                 BAIL_ON_LWIO_ERROR(dwError);
-
+                
                 authData.User = pszUsername;
                 authData.UserLength = strlen(pszUsername);
                 authData.Domain = pszDomain;
@@ -321,10 +329,10 @@ SMBGSSContextBuild(
                 authData.Password = pszPassword;
                 authData.PasswordLength = strlen(pszPassword);
                 authData.Flags = 0;
-
+                
                 authDataBuffer.value = &authData;
                 authDataBuffer.length = sizeof(authData);
-
+                
                 dwMajorStatus = gssspi_set_cred_option(
                     (OM_uint32 *)&dwMinorStatus,
                     pContext->credHandle,
@@ -335,34 +343,33 @@ SMBGSSContextBuild(
             break;
         }
     }
-
-    dwError = SMBAllocateMemory(
+    
+    dwError = LwIoAllocateMemory(
         sizeof(CtxtHandle),
         (PVOID*)&pContext->pGSSContext);
     BAIL_ON_LWIO_ERROR(dwError);
-
+    
     *pContext->pGSSContext = GSS_C_NO_CONTEXT;
-
+    
     *phSMBGSSContext = (HANDLE)pContext;
-
+    
 cleanup:
-
+    
     if (pUsername != NULL)
     {
         gss_release_name((OM_uint32 *)&dwMinorStatus, &pUsername);
     }
 
     LWIO_SAFE_FREE_STRING(pszTargetName);
-    LWIO_SAFE_FREE_STRING(pszDomainName);
     LWIO_SAFE_FREE_STRING(pszServerName);
     LWIO_SAFE_FREE_STRING(pszUsername);
     LWIO_SAFE_FREE_STRING(pszDomain);
     LWIO_SAFE_FREE_STRING(pszPassword);
-
+    
     return dwError;
-
+    
 sec_error:
-
+    
     dwError = LWIO_ERROR_GSS;
 
 error:
@@ -478,7 +485,7 @@ SMBGSSContextNegotiate(
 
     if (output_desc.length)
     {
-        dwError = SMBAllocateMemory(
+        dwError = LwIoAllocateMemory(
                         output_desc.length,
                         (PVOID*)&pSecurityBlob);
         BAIL_ON_LWIO_ERROR(dwError);
@@ -604,7 +611,7 @@ SMBGSSContextFree(
                             pContext->pGSSContext,
                             GSS_C_NO_BUFFER);
 
-            SMBFreeMemory(pContext->pGSSContext);
+            LwIoFreeMemory(pContext->pGSSContext);
         }
 
         if (pContext->credHandle)
@@ -612,99 +619,10 @@ SMBGSSContextFree(
             gss_release_cred(&dwMinorStatus, &pContext->credHandle);
         }
 
-        SMBFreeMemory(pContext);
+        LwIoFreeMemory(pContext);
     }
 }
 
-static
-DWORD
-SMBGetServerCanonicalName(
-    PCSTR pszServerName,
-    PSTR* ppszNormal
-    )
-{
-    DWORD dwError = 0;
-    struct addrinfo hints;
-    struct addrinfo* pAddrInfo = NULL;
-
-    memset(&hints, 0, sizeof(hints));
-
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_socktype = 0;
-    hints.ai_protocol = 0;
-    hints.ai_flags = AI_CANONNAME;
-
-    if (getaddrinfo(pszServerName, NULL, &hints, &pAddrInfo))
-    {
-        dwError = LWIO_ERROR_HOST_NOT_FOUND;
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    dwError = SMBAllocateString(pAddrInfo->ai_canonname, ppszNormal);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-error:
-
-    if (pAddrInfo)
-    {
-        freeaddrinfo(pAddrInfo);
-    }
-
-    return dwError;
-}
-
-static
-DWORD
-SMBGetServerDomain(
-    PCSTR pszServerName,
-    PSTR* ppszDomain
-    )
-{
-    DWORD dwError = 0;
-    char* pDot = NULL;
-    PSTR pszDomain = NULL;
-    PSTR pszNormal = NULL;
-    size_t sDomain = 0;
-
-    pDot = strchr(pszServerName, '.');
-
-    if (pDot)
-    {
-        dwError = SMBAllocateString(pszServerName, &pszNormal);
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-    else
-    {
-        dwError = SMBGetServerCanonicalName(pszServerName, &pszNormal);
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    dwError = SMBAllocateString(strchr(pszNormal, '.') + 1, &pszDomain);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    sDomain = strlen(pszDomain);
-    if (sDomain > 0 && pszDomain[sDomain - 1] == '.')
-    {
-        // Strip the trailing dot off
-        pszDomain[sDomain - 1] = 0;
-    }
-
-    *ppszDomain = pszDomain;
-
-cleanup:
-
-    LWIO_SAFE_FREE_MEMORY(pszNormal);
-
-    return dwError;
-
-error:
-
-    *ppszDomain = NULL;
-    LWIO_SAFE_FREE_MEMORY(pszDomain);
-
-    goto cleanup;
-}
-
 NTSTATUS
 SMBCredTokenToKrb5CredCache(
     PIO_CREDS pCredToken,
@@ -726,7 +644,7 @@ SMBCredTokenToKrb5CredCache(
     Status = SMBAllocateStringPrintf(
         &pszCachePath,
         "MEMORY:%lu",
-        (unsigned long) (size_t) pCredToken);
+        (unsigned long) (size_t) ppszCachePath);
     BAIL_ON_NT_STATUS(Status);
 
     krb5Error = krb5_init_context(&pContext);
@@ -742,7 +660,7 @@ SMBCredTokenToKrb5CredCache(
         Status = STATUS_UNSUCCESSFUL;
         BAIL_ON_NT_STATUS(Status);
     }
-
+    
     /* Convert cred token back into krb5 structure */
 
     /* Convert client and server principal names */
@@ -860,7 +778,7 @@ smb_display_status_1(
     int       type
     )
 {
-    OM_uint32 maj_stat, min_stat;
+    OM_uint32 min_stat;
     gss_buffer_desc msg;
     OM_uint32 msg_ctx;
 
@@ -872,9 +790,9 @@ smb_display_status_1(
     msg_ctx = 0;
     while (1)
     {
-        maj_stat = gss_display_status(&min_stat, code,
-                                      type, GSS_C_NULL_OID,
-                                      &msg_ctx, &msg);
+        (void) gss_display_status(&min_stat, code,
+                                  type, GSS_C_NULL_OID,
+                                  &msg_ctx, &msg);
 
         switch(code)
         {
@@ -887,14 +805,14 @@ smb_display_status_1(
 #endif
                 LWIO_LOG_VERBOSE("GSS-API error calling %s: %d (%s)\n",
                         pszId, code,
-                        SMB_SAFE_LOG_STRING((char *)msg.value));
+                        LWIO_SAFE_LOG_STRING((char *)msg.value));
                 break;
 
             default:
 
                 LWIO_LOG_ERROR("GSS-API error calling %s: %d (%s)\n",
                         pszId, code,
-                        SMB_SAFE_LOG_STRING((char *)msg.value));
+                        LWIO_SAFE_LOG_STRING((char *)msg.value));
         }
 
         (void) gss_release_buffer(&min_stat, &msg);
