@@ -355,6 +355,7 @@ WireMarshallNtTransactionResponse(
     }
 
     pResponseHeader = (PNT_TRANSACTION_SECONDARY_RESPONSE_HEADER)pDataCursor;
+    memset(pResponseHeader, 0, sizeof(*pResponseHeader));
 
     pDataCursor += sizeof(NT_TRANSACTION_SECONDARY_RESPONSE_HEADER);
     ulOffset += sizeof(NT_TRANSACTION_SECONDARY_RESPONSE_HEADER);
@@ -413,6 +414,7 @@ WireMarshallNtTransactionResponse(
     pResponseHeader->ulParameterCount = ulParamLength;
     pResponseHeader->ulTotalParameterCount = ulParamLength;
     pResponseHeader->ulParameterOffset = ulOffset;
+    pResponseHeader->ulParameterDisplacement = 0;
 
     if (pParams)
     {
@@ -451,6 +453,7 @@ WireMarshallNtTransactionResponse(
     pResponseHeader->ulDataCount = ulDataLen;
     pResponseHeader->ulTotalDataCount = ulDataLen;
     pResponseHeader->ulDataOffset = ulOffset;
+    pResponseHeader->ulDataDisplacement = 0;
 
     if (pData)
     {
