@@ -881,6 +881,7 @@ WireMarshallTransaction2Response(
     }
 
     pResponseHeader = (PTRANSACTION_SECONDARY_RESPONSE_HEADER)pDataCursor;
+    memset(pResponseHeader, 0, sizeof(*pResponseHeader));
 
     pDataCursor += sizeof(TRANSACTION_SECONDARY_RESPONSE_HEADER);
     ulOffset += sizeof(TRANSACTION_SECONDARY_RESPONSE_HEADER);
@@ -939,6 +940,7 @@ WireMarshallTransaction2Response(
     pResponseHeader->parameterCount = usParamLength;
     pResponseHeader->totalParameterCount = usParamLength;
     pResponseHeader->parameterOffset = (USHORT)ulOffset;
+    pResponseHeader->parameterDisplacement = 0;
 
     if (pParams)
     {
@@ -977,6 +979,7 @@ WireMarshallTransaction2Response(
     pResponseHeader->dataCount = usDataLen;
     pResponseHeader->totalDataCount = usDataLen;
     pResponseHeader->dataOffset = (USHORT)ulOffset;
+    pResponseHeader->dataDisplacement = 0;
 
     if (pData)
     {
