@@ -37,7 +37,7 @@ struct
     pthread_cond_t Event;
     ULONG ulFailureCount;
     BOOLEAN bContinueOnError;
-
+    
 } gState =
 {
     .ulThreadCount = 100,
@@ -65,7 +65,7 @@ LoadThread(
     static const CHAR szPayload[] = PAYLOAD;
     CHAR szCompare[sizeof(szPayload)];
     IO_STATUS_BLOCK ioStatus = {0};
-    LONG64 llOffset = 0;
+    ULONG64 offset = 0;
     CHAR szHostname[256] = {0};
     LW_PIO_CREDS pCreds = NULL;
 
@@ -149,14 +149,15 @@ LoadThread(
                 0,                     /* Create options */
                 NULL,                  /* EA buffer */
                 0,                     /* EA length */
-                NULL);                 /* ECP list */
+                NULL,                  /* ECP list */
+                NULL);
             if (status != STATUS_SUCCESS)
             {
                 gState.ulFailureCount++;
 
                 fprintf(stderr, "Error Opening File: %s (%x).  Failure Count == %d \n",
-                        LwNtStatusToName(status),
-                        status,
+                        LwNtStatusToName(status), 
+                        status,  
                         gState.ulFailureCount);
 
                 if (!gState.bContinueOnError)
@@ -165,7 +166,7 @@ LoadThread(
                 }
             }
         }
-
+      
         /* Pass 2 -- write payload into each file */
 
         printf("[%u] Writing to files...\n", pThread->ulNumber);
@@ -174,7 +175,7 @@ LoadThread(
         {
             pFile = &pFiles[ulFile];
 
-            llOffset = 0;
+            offset = 0;
 
             if (pFile->hHandle)
             {
@@ -184,7 +185,7 @@ LoadThread(
                     &ioStatus, /* IO status block */
                     (PVOID) szPayload, /* Buffer */
                     sizeof(szPayload), /* Buffer size */
-                    &llOffset, /* File offset */
+                    &offset, /* File offset */
                     NULL); /* Key */
                 GOTO_ERROR_ON_STATUS(status);
             }
@@ -223,7 +224,8 @@ LoadThread(
                     FILE_DELETE_ON_CLOSE,  /* Create options */
                     NULL,                  /* EA buffer */
                     0,                     /* EA length */
-                    NULL);                 /* ECP list */
+                    NULL,                  /* ECP list */
+                    NULL);
                 GOTO_ERROR_ON_STATUS(status);
             }
         }
@@ -234,7 +236,7 @@ LoadThread(
 
         for (ulFile = 0; ulFile < gState.ulConnectionsPerThread; ulFile++)
         {
-            llOffset = 0;
+            offset = 0;
 
             if (pFile->hHandle)
             {
@@ -244,7 +246,7 @@ LoadThread(
                     &ioStatus, /* IO status block */
                     szCompare, /* Buffer */
                     sizeof(szCompare), /* Buffer size */
-                    &llOffset, /* File offset */
+                    &offset, /* File offset */
                     NULL); /* Key */
                 GOTO_ERROR_ON_STATUS(status);
 
@@ -260,7 +262,7 @@ LoadThread(
         /* Pass 5 -- close handles */
 
         printf("[%u] Closing files...\n", pThread->ulNumber);
-
+                                                                 
         for (ulFile = 0; ulFile < gState.ulConnectionsPerThread; ulFile++)
         {
             pFile = &pFiles[ulFile];
@@ -325,7 +327,7 @@ PromptPassword(
             status = LwErrnoToNtStatus(errno);
             GOTO_ERROR_ON_STATUS(status);
         }
-
+        
         if (szBuffer[index] == '\n')
         {
             szBuffer[index] = '\0';
@@ -388,7 +390,7 @@ Run(
     gState.bStart = TRUE;
     pthread_cond_broadcast(&gState.Event);
     pthread_mutex_unlock(&gState.Lock);
-
+        
 
     for (ulThread = 0; ulThread < gState.ulThreadCount; ulThread++)
     {
@@ -518,7 +520,7 @@ ParseArgs(
             }
 
             gState.pszServer = ppszArgv[i];
-            gState.pszShare = ppszArgv[++i];
+            gState.pszShare = ppszArgv[++i]; 
 
             return;
         }
