@@ -282,7 +282,7 @@ LwioCopyFileFromLocalToLocal(
     BAIL_ON_NT_STATUS(status);
 
     //Get UID GID and mode
-    status = LwioGetLocalFileOwnerAndPerms(
+    status = LwioGetLocalFileOwnerAndPerms( 
                  pszSrcPath,
                  &uid,
                  &gid,
@@ -322,7 +322,7 @@ LwioCopyFileFromLocalToLocal(
 
    } while(1);
 
-   status = LwioChangeLocalFileOwnerAndPerms(
+   status = LwioChangeLocalFileOwnerAndPerms( 
                  pszDestPath,
                  uid,
                  gid,
@@ -331,8 +331,14 @@ LwioCopyFileFromLocalToLocal(
 
 cleanup:
 
-    close(hSrcFile);
-    close(hDestFile);
+    if (hSrcFile >= 0)
+    {
+        close(hSrcFile);
+    }
+    if (hDestFile >= 0)
+    {
+        close(hDestFile);
+    }
 
     return (status);
 
@@ -484,7 +490,10 @@ cleanup:
         LwNtCloseFile(hRemoteFile);
     }
 
-    close(hLocalFile);
+    if (hLocalFile >= 0)
+    {
+        close(hLocalFile);
+    }
 
     return (status);
 
@@ -823,7 +832,10 @@ cleanup:
         LwNtCloseFile(hRemoteFile);
     }
 
-    close(hLocalFile);
+    if (hLocalFile >= 0)
+    {
+        close(hLocalFile);
+    }
 
     return (status);
 
