@@ -410,7 +410,7 @@ SMBCreateDirectoryRecursive(
 
     if (pszToken != NULL) {
 
-        dwError = SMBAllocateMemory(strlen(pszCurDirPath)+strlen(pszToken)+2,
+        dwError = LwIoAllocateMemory(strlen(pszCurDirPath)+strlen(pszToken)+2,
                                    (PVOID*)&pszDirPath);
         BAIL_ON_LWIO_ERROR(dwError);
 
@@ -450,7 +450,7 @@ SMBCreateDirectoryRecursive(
         BAIL_ON_LWIO_ERROR(dwError);
     }
     if (pszDirPath) {
-        SMBFreeMemory(pszDirPath);
+        LwIoFreeMemory(pszDirPath);
     }
 
     return dwError;
@@ -462,7 +462,7 @@ error:
     }
 
     if (pszDirPath) {
-        SMBFreeMemory(pszDirPath);
+        LwIoFreeMemory(pszDirPath);
     }
 
     return dwError;
@@ -520,12 +520,12 @@ error:
 
         SMBChangeDirectory(pszCurDirPath);
 
-        SMBFreeMemory(pszCurDirPath);
+        LwIoFreeMemory(pszCurDirPath);
 
     }
 
     if (pszTmpPath) {
-        SMBFreeMemory(pszTmpPath);
+        LwIoFreeMemory(pszTmpPath);
     }
 
     return dwError;
@@ -572,7 +572,7 @@ SMBCopyFileWithPerms(
     CHAR szBuf[1024+1];
     int  iFd = -1;
     int  oFd = -1;
-    DWORD dwBytesRead = 0;
+    LONG lBytesRead = 0;
 
     if (IsNullOrEmptyString(pszSrcPath) ||
         IsNullOrEmptyString(pszDstPath)) {
@@ -580,7 +580,7 @@ SMBCopyFileWithPerms(
         BAIL_ON_LWIO_ERROR(dwError);
     }
 
-    dwError = SMBAllocateMemory(strlen(pszDstPath)+strlen(pszTmpSuffix)+2,
+    dwError = LwIoAllocateMemory(strlen(pszDstPath)+strlen(pszTmpSuffix)+2,
                                (PVOID*)&pszTmpPath);
     BAIL_ON_LWIO_ERROR(dwError);
 
@@ -600,7 +600,7 @@ SMBCopyFileWithPerms(
     bRemoveFile = TRUE;
 
     while (1) {
-        if ((dwBytesRead = read(iFd, szBuf, 1024)) < 0) {
+        if ((lBytesRead = read(iFd, szBuf, 1024)) < 0) {
 
             if (errno == EINTR)
                 continue;
@@ -609,10 +609,10 @@ SMBCopyFileWithPerms(
             BAIL_ON_LWIO_ERROR(dwError);
         }
 
-        if (dwBytesRead == 0)
+        if (lBytesRead == 0)
             break;
 
-        if (write(oFd, szBuf, dwBytesRead) != dwBytesRead) {
+        if (write(oFd, szBuf, lBytesRead) != lBytesRead) {
 
             if (errno == EINTR)
                 continue;
@@ -679,88 +679,6 @@ error:
     return dwError;
 }
 
-DWORD
-SMBBackupFile(
-    PCSTR pszPath
-    )
-{
-    DWORD dwError = 0;
-    BOOLEAN bExists = FALSE;
-    CHAR    szBackupPath[PATH_MAX];
-
-    dwError = SMBCheckFileExists(pszPath, &bExists);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    if (!bExists)
-    {
-        /* Do not need to backup, since the file does not yet exist. */
-        goto done;
-    }
-
-    sprintf(szBackupPath, "%s.likewise_lsass.orig", pszPath);
-
-    dwError = SMBCheckFileExists(szBackupPath, &bExists);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    if (bExists)
-    {
-       sprintf(szBackupPath, "%s.likewise_lsass.bak", pszPath);
-    }
-
-    dwError = SMBCopyFileWithOriginalPerms(pszPath, szBackupPath);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-done:
-error:
-
-    return dwError;
-}
-
-DWORD
-SMBGetSymlinkTarget(
-   PCSTR pszPath,
-   PSTR* ppszTargetPath
-   )
-{
-    DWORD dwError = 0;
-    CHAR szBuf[PATH_MAX+1];
-    PSTR pszTargetPath = NULL;
-
-    memset(szBuf, 0, sizeof(szBuf));
-
-    while (1) {
-
-       if (readlink(pszPath, szBuf, PATH_MAX) < 0) {
-          if (errno == EINTR)
-             continue;
-
-          dwError = errno;
-          BAIL_ON_LWIO_ERROR(dwError);
-       }
-
-       break;
-    }
-
-    dwError = SMBAllocateString(
-                    szBuf,
-                    &pszTargetPath);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    *ppszTargetPath = pszTargetPath;
-
-cleanup:
-
-    return dwError;
-
-error:
-
-    *ppszTargetPath = NULL;
-
-    LWIO_SAFE_FREE_STRING(pszTargetPath);
-
-    goto cleanup;
-}
-
 DWORD
 SMBCreateSymlink(
    PCSTR pszOldPath,
@@ -770,266 +688,3 @@ SMBCreateSymlink(
     return ((symlink(pszOldPath, pszNewPath) < 0) ? errno : 0);
 }
 
-DWORD
-SMBCopyDirectory(
-    PCSTR pszSourceDirPath,
-    uid_t ownerUid,
-    gid_t ownerGid,
-    PCSTR pszDestDirPath
-    )
-{
-    DWORD dwError = 0;
-    DIR*  pDir = NULL;
-    struct dirent* pDirEntry = NULL;
-    struct stat statbuf = {0};
-    CHAR  szSrcPath[PATH_MAX+1];
-    CHAR  szDstPath[PATH_MAX+1];
-    PSTR  pszTargetPath = NULL;
-
-    if (NULL == (pDir = opendir(pszSourceDirPath))) {
-       dwError = errno;
-       BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    while (NULL != (pDirEntry = readdir(pDir)))
-    {
-        if (!strcmp(pDirEntry->d_name, ".") ||
-            !strcmp(pDirEntry->d_name, "..")||
-            !strcmp(pDirEntry->d_name, "lost+found"))
-            continue;
-
-        memset(&statbuf, 0, sizeof(statbuf));
-
-        sprintf(szSrcPath, "%s/%s", pszSourceDirPath, pDirEntry->d_name);
-
-        if (lstat(szSrcPath, &statbuf) < 0) {
-            dwError = errno;
-            BAIL_ON_LWIO_ERROR(dwError);
-        }
-
-        sprintf(szDstPath, "%s/%s", pszDestDirPath, pDirEntry->d_name);
-
-        if (S_ISDIR(statbuf.st_mode)) {
-
-            dwError = SMBCreateDirectory(
-                            szDstPath,
-                            statbuf.st_mode);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBChangeOwner(
-                            szDstPath,
-                            ownerUid,
-                            ownerGid);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBCopyDirectory(
-                            szSrcPath,
-                            ownerUid,
-                            ownerGid,
-                            szDstPath);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-        } else if (S_ISREG(statbuf.st_mode)) {
-
-            dwError = SMBCopyFileWithOriginalPerms(
-                            szSrcPath,
-                            szDstPath);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBChangeOwner(
-                            szDstPath,
-                            ownerUid,
-                            ownerGid);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-        } else if (S_ISLNK(statbuf.st_mode)) {
-
-            dwError = SMBGetSymlinkTarget(
-                            szSrcPath,
-                            &pszTargetPath);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBCreateSymlink(
-                            pszTargetPath,
-                            szDstPath);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBChangeOwner(
-                            szDstPath,
-                            ownerUid,
-                            ownerGid);
-            BAIL_ON_LWIO_ERROR(dwError);
-        }
-    }
-
-cleanup:
-
-    if (pDir) {
-        closedir(pDir);
-    }
-
-    LWIO_SAFE_FREE_STRING(pszTargetPath);
-
-    return dwError;
-
-error:
-
-    goto cleanup;
-}
-
-DWORD
-SMBGetMatchingFilePathsInFolder(
-    PCSTR pszDirPath,
-    PCSTR pszFileNameRegExp,
-    PSTR** pppszHostFilePaths,
-    PDWORD pdwNPaths
-    )
-{
-    typedef struct __PATHNODE
-    {
-        PSTR pszPath;
-        struct __PATHNODE *pNext;
-    } PATHNODE, *PPATHNODE;
-
-    DWORD dwError = 0;
-    DIR* pDir = NULL;
-    struct dirent*  pDirEntry = NULL;
-    regex_t rx;
-    BOOLEAN rxAllocated = FALSE;
-    regmatch_t* pResult = NULL;
-    size_t nMatch = 1;
-    DWORD  dwNPaths = 0;
-    DWORD  iPath = 0;
-    PSTR*  ppszHostFilePaths = NULL;
-    CHAR   szBuf[PATH_MAX+1];
-    struct stat statbuf;
-    PPATHNODE pPathList = NULL;
-    PPATHNODE pPathNode = NULL;
-    BOOLEAN bDirExists = FALSE;
-
-    dwError = SMBCheckDirectoryExists(pszDirPath, &bDirExists);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    if(!bDirExists) {
-        dwError = ENOENT;
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    if (regcomp(&rx, pszFileNameRegExp, REG_EXTENDED) != 0) {
-        dwError = LWIO_ERROR_REGEX_COMPILE_FAILED;
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-    rxAllocated = TRUE;
-
-    dwError = SMBAllocateMemory(sizeof(regmatch_t), (PVOID*)&pResult);
-    BAIL_ON_LWIO_ERROR(dwError);
-
-    pDir = opendir(pszDirPath);
-    if (!pDir) {
-        dwError = errno;
-        BAIL_ON_LWIO_ERROR(dwError);
-    }
-
-    while ((pDirEntry = readdir(pDir)) != NULL) {
-
-        int copied = snprintf(
-                            szBuf,
-                            sizeof(szBuf),
-                            "%s/%s",
-                            pszDirPath,
-                            pDirEntry->d_name);
-        if (copied >= sizeof(szBuf))
-        {
-            //Skip pathnames that are too long
-            continue;
-        }
-        memset(&statbuf, 0, sizeof(struct stat));
-        if (lstat(szBuf, &statbuf) < 0) {
-            if(errno == ENOENT)
-            {
-                //This occurs when there is a symbolic link pointing to a
-                //location that doesn't exist, because stat looks at the final
-                //file, not the link. Since this file doesn't exist anyway,
-                //just skip it.
-                continue;
-            }
-            dwError = errno;
-            BAIL_ON_LWIO_ERROR(dwError);
-        }
-
-        /*
-         * For now, we are searching only for regular files
-         * This may be enhanced in the future to support additional
-         * file system entry types
-         */
-        if (((statbuf.st_mode & S_IFMT) == S_IFREG) &&
-            (regexec(&rx, pDirEntry->d_name, nMatch, pResult, 0) == 0)) {
-            dwNPaths++;
-
-            dwError = SMBAllocateMemory(sizeof(PATHNODE), (PVOID*)&pPathNode);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            dwError = SMBAllocateString(szBuf, &pPathNode->pszPath);
-            BAIL_ON_LWIO_ERROR(dwError);
-
-            pPathNode->pNext = pPathList;
-            pPathList = pPathNode;
-            pPathNode = NULL;
-        }
-    }
-
-    if (pPathList) {
-        dwError = SMBAllocateMemory(sizeof(PSTR)*dwNPaths,
-                                    (PVOID*)&ppszHostFilePaths);
-        BAIL_ON_LWIO_ERROR(dwError);
-        /*
-         *  The linked list is in reverse.
-         *  Assign values in reverse to get it right
-         */
-        iPath = dwNPaths-1;
-        pPathNode = pPathList;
-        while (pPathNode) {
-            *(ppszHostFilePaths+iPath) = pPathNode->pszPath;
-            pPathNode->pszPath = NULL;
-            pPathNode = pPathNode->pNext;
-            iPath--;
-        }
-    }
-
-    *pppszHostFilePaths = ppszHostFilePaths;
-    *pdwNPaths = dwNPaths;
-
-cleanup:
-
-    if (pPathNode) {
-        LWIO_SAFE_FREE_STRING(pPathNode->pszPath);
-        SMBFreeMemory(pPathNode);
-    }
-
-    while(pPathList) {
-        pPathNode = pPathList;
-        pPathList = pPathList->pNext;
-        LWIO_SAFE_FREE_STRING(pPathNode->pszPath);
-        SMBFreeMemory(pPathNode);
-    }
-
-    if (rxAllocated) {
-        regfree(&rx);
-    }
-
-    LWIO_SAFE_FREE_MEMORY(pResult);
-
-    if (pDir) {
-        closedir(pDir);
-    }
-
-    return dwError;
-
-error:
-
-    if (ppszHostFilePaths) {
-       SMBFreeStringArray(ppszHostFilePaths, dwNPaths);
-    }
-
-    goto cleanup;
-}
