@@ -51,7 +51,7 @@
  */
 typedef struct LWMsgAssocClass
 {
-    /**
+    /** 
      * @ingroup assoc_impl
      * @brief Constructor method
      *
@@ -166,7 +166,7 @@ typedef struct LWMsgAssocClass
     /**
      * @ingroup assoc_impl
      * @brief Set timeout
-     *
+     * 
      * This method sets a timeout that should be used for subsequent operations
      *
      * @param[in] assoc the association
@@ -186,13 +186,13 @@ typedef struct LWMsgAssocClass
         );
 
     LWMsgStatus
-    (*connect)(
+    (*connect_peer)(
         LWMsgAssoc* assoc,
         LWMsgSession* session
         );
 
     LWMsgStatus
-    (*accept)(
+    (*accept_peer)(
         LWMsgAssoc* assoc,
         LWMsgSessionManager* manager,
         LWMsgSession** session
@@ -221,7 +221,7 @@ struct LWMsgAssoc
  * @brief Create a new association
  *
  * Creates a new association with the specified implementation and protocol.
- *
+ * 
  * @param[in] context an optional context
  * @param[in] prot the protocol understood by the association
  * @param[in] aclass the implementation structure for the new association
