@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -65,7 +65,7 @@
  *
  */
 typedef struct LWMsgCallClass
-{
+{ 
     /**
      * @brief Release call handle (virtual)
      *
@@ -151,7 +151,7 @@ typedef struct LWMsgCallClass
     (*get_session)(
         LWMsgCall* call
         );
-
+    
     /**
      * @brief Get callback handle (virtual)
      *
@@ -184,6 +184,8 @@ struct LWMsgCall
      * the call handle operations
      */
     LWMsgCallClass* vtbl;
+    unsigned is_outgoing:1;
+    void* user_data;
 };
 
 /**
