@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -187,13 +187,6 @@ lwmsg_data_free_graph_internal(
     unsigned char* object
     );
 
-void
-lwmsg_data_free_graph_cleanup(
-    const LWMsgContext* context,
-    LWMsgTypeSpec* type,
-    void* root
-    );
-
 /**
  * @brief Print textual representation of a data graph
  *
