@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -85,6 +85,17 @@ typedef struct LWMsgParams
  */
 typedef struct LWMsgCall LWMsgCall;
 
+/**
+ * @brief Call direction
+ *
+ * The direction of a call.
+ */
+typedef enum LWMsgCallDirection
+{
+    LWMSG_CALL_INCOMING,
+    LWMSG_CALL_OUTGOING
+} LWMsgCallDirection;
+
 /**
  * @brief Call completion callback
  *
@@ -130,7 +141,7 @@ typedef void
  * safely use this function on a #LWMsgParams structure that
  * has been correctly initialized (e.g. with #LWMSG_PARAMS_INITIALIZER)
  * to free any data it might contain.
- *
+ * 
  * @param[in] call the call handle
  * @param[in,out] params the parameters structure
  * @lwmsg_status
@@ -271,6 +282,48 @@ lwmsg_call_acquire_callback(
     LWMsgCall** callback
     );
 
+/**
+ * @brief Get call direction
+ *
+ * Returns whether a call is incoming or outgoing.
+ *
+ * @param[in] call the call handle
+ * @return the call type
+ */
+LWMsgCallDirection
+lwmsg_call_get_direction(
+    LWMsgCall* call
+    );
+
+/**
+ * @brief Set user data
+ *
+ * Sets an arbitrary user data pointer on a call handle,
+ * which may be used for any purpose.
+ *
+ * @param[in,out] call the call handle
+ * @param[in] data the data pointer
+ */
+void
+lwmsg_call_set_user_data(
+    LWMsgCall* call,
+    void* data
+    );
+
+/**
+ * @brief Get user data
+ *
+ * Gets the user data pointer previously set with
+ * #lwmsg_call_set_user_data().
+ *
+ * @param[in] call the call handle
+ * @return the user data pointer
+ */
+void*
+lwmsg_call_get_user_data(
+    LWMsgCall* call
+    );
+
 /**
  * @brief Static initializer for #LWMsgParams
  *
