@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -64,7 +64,7 @@
  * specifications will automatically be promoted to pointers to accomodate
  * this restriction:
  * - Structures
- *
+ *  
  * All operations take a data context which controls tunable
  * parameters of the data model:
  *
@@ -82,7 +82,7 @@
 
 /**
  * @brief Data context
- *
+ * 
  * An opqaue data context which facilitates all
  * data model operations.
  */
@@ -126,7 +126,7 @@ lwmsg_data_context_delete(
 
 /**
  * @brief Get last error message
- *
+ * 
  * Get a descriptive diagnostic message for the last
  * error which occured on the specified data context.
  * The returned string will remain valid until another
@@ -249,6 +249,50 @@ lwmsg_data_destroy_graph(
     void* root
     );
 
+/**
+ * @brief Free in-memory data graph (guaranteed success)
+ *
+ * Like #lwmsg_data_free_graph(), but guarantees success:
+ *
+ * - Does not require an #LWMsgDataContext to be allocated
+ *   (which could itself fail).  Instead, an optional #LWMsgContext
+ *   can be passed in to specify the memory manager.
+ * - Does not return an error
+ *
+ * @warning This function has undefined behavior if the passed
+ * data graph is malformed.
+ *
+ * @param[in] context an optional context
+ * @param[in] type the type of the root node of the graph
+ * @param[in,out] root the root of the graph
+ */
+void
+lwmsg_data_free_graph_cleanup(
+    const LWMsgContext* context,
+    LWMsgTypeSpec* type,
+    void* root
+    );
+
+/**
+ * @brief Destroy in-memory data graph (guaranteed success)
+ *
+ * Like #lwmsg_data_destroy_graph(), but guarantees success
+ * in the same way as #lwmsg_data_free_graph_cleanup().
+ *
+ * @warning This function has undefined behavior if the passed
+ * data graph is malformed.
+ *
+ * @param[in] context an optional context
+ * @param[in] type the type of the root node of the graph
+ * @param[in,out] root the root of the graph
+ */
+void
+lwmsg_data_destroy_graph_cleanup(
+    const LWMsgContext* context,
+    LWMsgTypeSpec* type,
+    void* root
+    );
+
 /**
  * @brief Marshal a data structure
  *
