@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -201,7 +201,7 @@ typedef struct LWMsgPeer LWMsgPeer;
  * same memory manager as the #LWMsgPeer which dispatched the call.
  * By default, this is plain malloc().  The caller assumes ownership of all
  * such memory and the responsibility of freeing it.
- *
+ * 
  * @param[in,out] call the call handle
  * @param[in] in the input parameters
  * @param[out] out the output parameters
@@ -241,6 +241,24 @@ void
     void* data
     );
 
+/**
+ * @brief Call trace function
+ *
+ * A callback function which allows tracing when a call begins or
+ * ends.
+ *
+ * @param[in] call the call handle
+ * @param[in] params the input or output parameters of the call
+ * @param[in] data a user data pointer
+ */
+typedef
+void
+(*LWMsgPeerTraceFunction) (
+    LWMsgCall* call,
+    const LWMsgParams* params,
+    void* data
+    );
+
 /**
  * @brief Create a new peer object
  *
@@ -384,7 +402,7 @@ lwmsg_peer_add_listen_fd(
     LWMsgEndpointType type,
     int fd
     );
-
+    
 /**
  * @brief Add listening endpoint
  *
@@ -525,6 +543,31 @@ lwmsg_peer_set_exception_function(
     void* except_data
     );
 
+/**
+ * @brief Set trace functions
+ *
+ * Sets functions which will be invoked whenever a call begins or ends.
+ * To determine the direction of a call, use #lwmsg_call_get_direction().
+ * To store extra data on the call handle, use #lwmsg_call_set_user_data().
+ * This mechanism can be use for logging, statistics gathering, etc.
+ *
+ * @param[in,out] peer the peer handle
+ * @param[in] begin trace begin function
+ * @param[in] end trace end function
+ * @param[in] data user data pointer to pass to trace functions
+ * @lwmsg_status
+ * @lwmsg_success
+ * @lwmsg_code{INVALID_STATE, the peer is already active}
+ * @lwmsg_endstatus
+ */
+LWMsgStatus
+lwmsg_peer_set_trace_functions(
+    LWMsgPeer* peer,
+    LWMsgPeerTraceFunction begin,
+    LWMsgPeerTraceFunction end,
+    void* data
+    );
+
 /**
  * @brief Add connection endpoint
  *
@@ -554,7 +597,7 @@ lwmsg_peer_add_connect_endpoint(
  * session is already established, this function will perform no action
  * but will still return the established session.  If an outgoing session
  * is in the process of being established, this function will block until
- * it either succeeds or fails.  The session returned is guaranteed to
+ * it either succeeds or fails.  The session returned is guaranteed to 
  * remain valid until #lwmsg_peer_disconnect() is used.
  *
  * @param[in,out] peer the peer handle
@@ -594,7 +637,7 @@ lwmsg_peer_disconnect(
  *
  * Acquire a call handle which can be used to make an outgoing call.  Peer
  * call handles fully support asynchronous calls.  The handle may be reused
- * after each call completes.  If the peer has not been connected with
+ * after each call completes.  If the peer has not been connected with 
  * #lwmsg_peer_connect(), this function will do so implicitly.  The acquired
  * call handle should be released with #lwmsg_call_release() when no longer needed.
  *
