@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -134,6 +134,9 @@ typedef struct PeerAssocTask
     LWMsgMessage outgoing_message;
     unsigned incoming:1;
     unsigned outgoing:1;
+    unsigned send_blocked:1;
+    unsigned recv_blocked:1;
+    unsigned recv_partial:1;
     unsigned destroy_outgoing:1;
     LWMsgCookie volatile next_cookie;
     unsigned int volatile refcount;
@@ -161,7 +164,7 @@ typedef struct PeerEndpoint
 } PeerEndpoint;
 
 struct LWMsgPeer
-{
+{    
     LWMsgErrorContext error;
     const LWMsgContext* context;
     LWMsgProtocol* protocol;
@@ -177,7 +180,7 @@ struct LWMsgPeer
         LWMsgTime idle;
     } timeout;
     void* dispatch_data;
-
+    
     LWMsgSessionConstructFunction session_construct;
     LWMsgSessionDestructFunction session_destruct;
     void* session_construct_data;
@@ -185,6 +188,10 @@ struct LWMsgPeer
     LWMsgPeerExceptionFunction except;
     void* except_data;
 
+    LWMsgPeerTraceFunction trace_begin;
+    LWMsgPeerTraceFunction trace_end;
+    void* trace_data;
+
     struct
     {
         LWMsgDispatchSpec** vector;
@@ -208,7 +215,9 @@ struct LWMsgPeer
     size_t num_clients;
 
     pthread_mutex_t lock;
+    unsigned lock_init:1;
     pthread_cond_t event;
+    unsigned event_init:1;
     PeerState state;
     LWMsgStatus status;
 };
