@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -37,6 +37,7 @@
  *
  */
 
+#include "config.h"
 #include "archive-private.h"
 #include "util-private.h"
 #include "protocol-private.h"
@@ -66,7 +67,7 @@ lwmsg_archive_write_fd(
         {
             written = write(archive->fd, cursor, remaining);
         } while (written < 0 && (errno == EINTR || errno == EAGAIN));
-
+        
         if (written < 0)
         {
             BAIL_ON_ERROR(status = lwmsg_error_raise_errno(&archive->base.context.error, errno));
@@ -103,7 +104,7 @@ lwmsg_archive_read_fd(
         {
             count = read(archive->fd, cursor, remaining);
         } while (count < 0 && (errno == EINTR || errno == EAGAIN));
-
+        
         if (count < 0)
         {
             BAIL_ON_ERROR(status = lwmsg_error_raise_errno(&archive->base.context.error, errno));
@@ -121,7 +122,7 @@ lwmsg_archive_read_fd(
                 break;
             }
         }
-
+        
         remaining -= count;
         cursor += count;
         archive->offset += count;
@@ -131,7 +132,7 @@ lwmsg_archive_read_fd(
     *bytes_read = total;
 
 error:
-
+    
     return status;
 }
 
@@ -223,7 +224,7 @@ lwmsg_archive_write_schema_fd(
     LWMsgBuffer buffer = {0};
     unsigned char data[2048];
     uint32_t length = 0;
-
+    
     BAIL_ON_ERROR(status = lwmsg_protocol_get_protocol_rep(archive->base.prot, &rep));
 
     buffer.base = data;
@@ -234,10 +235,10 @@ lwmsg_archive_write_schema_fd(
 
     /* Leave room to write the schema length in later */
     BAIL_ON_ERROR(status = lwmsg_archive_seek_fd(archive, length_offset + sizeof(length)));
-
+    
     /* Write the marshaled protocol schema */
     BAIL_ON_ERROR(status = lwmsg_data_marshal(archive->data_context, type, rep, &buffer));
-
+    
     end_offset = archive->offset;
 
     /* Go back and write the schema length */
@@ -329,7 +330,7 @@ lwmsg_archive_write_message_fd(
     BAIL_ON_ERROR(status = lwmsg_data_marshal(archive->data_context, type, message->data, &buffer));
 
     end_offset = archive->offset;
-
+    
     /* Go back and write the header */
     BAIL_ON_ERROR(status = lwmsg_archive_seek_fd(archive, header_offset));
     lwmsg_archive_populate_message_header(archive, message, end_offset - header_offset - sizeof(header), &header);
@@ -408,12 +409,12 @@ lwmsg_archive_read_schema_fd(
     uint32_t format_flags = LWMSG_SWAP32(header->format_flags, LWMSG_NATIVE_ENDIAN, archive->byte_order);
     uint32_t length = 0;
     size_t count = 0;
-    LWMsgProtocolRep* rep = NULL;
+    LWMsgProtocolRep* rep = NULL;      
 
     if (format_flags & ARCHIVE_FORMAT_FLAG_SCHEMA)
     {
         BAIL_ON_ERROR(status = lwmsg_archive_read_fd(archive, &length, sizeof(length), &count));
-
+        
         if (count < sizeof(length))
         {
             BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
@@ -426,7 +427,7 @@ lwmsg_archive_read_schema_fd(
             readinfo info;
             LWMsgBuffer buffer = {0};
             LWMsgTypeSpec* type = lwmsg_protocol_rep_spec;
-
+            
             info.archive = archive;
             info.remaining = length;
             buffer.base = info.data;
@@ -434,7 +435,7 @@ lwmsg_archive_read_schema_fd(
             buffer.cursor = buffer.base;
             buffer.wrap = lwmsg_archive_read_message_wrap_fd;
             buffer.data = &info;
-
+            
             /* Unmarshal the schema */
             BAIL_ON_ERROR(status = lwmsg_data_unmarshal(
                               archive->data_context,
@@ -488,7 +489,7 @@ lwmsg_archive_read_header_fd(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
     }
-
+    
     if (header.version_major > ARCHIVE_VERSION_MAJOR ||
         header.version_minor > ARCHIVE_VERSION_MINOR)
     {
@@ -580,7 +581,7 @@ lwmsg_archive_read_message_fd(
 
     BAIL_ON_ERROR(status = lwmsg_archive_read_message_header(archive, message, &message_size));
     BAIL_ON_ERROR(status = lwmsg_protocol_get_message_type(archive->base.prot, message->tag, &type));
-
+    
     info.archive = archive;
     info.remaining = message_size;
     buffer.base = info.data;
