@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -109,7 +109,7 @@ lwmsg_archive_reset(
     BAIL_ON_ERROR(status = LWMSG_STATUS_UNSUPPORTED);
 
 error:
-
+    
     return status;
 }
 
@@ -170,14 +170,14 @@ lwmsg_archive_get_session(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-
+    
     BAIL_ON_ERROR(status = LWMSG_STATUS_UNSUPPORTED);
-
+    
 error:
 
     return status;
 }
-
+    
 static LWMsgAssocState
 lwmsg_archive_get_state(
     LWMsgAssoc* assoc
@@ -252,7 +252,7 @@ lwmsg_archive_connect(
     )
 {
     LWMsgArchive* archive = ARCHIVE_PRIVATE(assoc);
-
+ 
     return lwmsg_archive_open(archive, LWMSG_ARCHIVE_WRITE);
 }
 
@@ -267,7 +267,7 @@ static LWMsgAssocClass archive_class =
     .get_session = lwmsg_archive_get_session,
     .get_state = lwmsg_archive_get_state,
     .set_timeout = lwmsg_archive_set_timeout,
-    .connect = lwmsg_archive_connect,
+    .connect_peer = lwmsg_archive_connect,
     .set_nonblock = lwmsg_archive_set_nonblock,
     .finish = lwmsg_archive_finish
 };
@@ -349,7 +349,7 @@ lwmsg_archive_set_file(
     }
 
     archive->file = strdup(file);
-
+    
     if (!archive->file)
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
@@ -416,7 +416,7 @@ lwmsg_archive_open(
     }
 
     archive->disp = disp;
-
+ 
     switch (archive->disp & (LWMSG_ARCHIVE_READ | LWMSG_ARCHIVE_WRITE))
     {
     case LWMSG_ARCHIVE_READ:
