@@ -37,6 +37,7 @@
  *
  */
 
+#include "config.h"
 #include <lwmsg/type.h>
 #include <lwmsg/assoc.h>
 #include "convert-private.h"
@@ -82,7 +83,7 @@ lwmsg_assoc_marshal_handle(
     BAIL_ON_ERROR(status = assoc->aclass->get_session(assoc, &session));
 
     pointer = *(void**) object;
-
+    
     if (pointer != NULL)
     {
         status = lwmsg_session_handle_pointer_to_id(
@@ -113,7 +114,7 @@ lwmsg_assoc_marshal_handle(
                         (const char*) data,
                         type);
         }
-
+        
         /* Confirm that handle origin is correct according to type attributes */
         if ((attrs->custom & LWMSG_ASSOC_HANDLE_LOCAL_FOR_RECEIVER && transmit->type != LWMSG_HANDLE_REMOTE) ||
             (attrs->custom & LWMSG_ASSOC_HANDLE_LOCAL_FOR_SENDER && transmit->type != LWMSG_HANDLE_LOCAL))
@@ -175,9 +176,9 @@ lwmsg_assoc_unmarshal_handle(
     LWMsgHandleType location = transmit->type;
 
     BAIL_ON_ERROR(status = lwmsg_context_get_data(context, "assoc", (void**) (void*) &assoc));
-
+    
     BAIL_ON_ERROR(status = assoc->aclass->get_session(assoc, &session));
-
+       
     if (location != LWMSG_HANDLE_NULL)
     {
         /* Invert sense of handle location */
@@ -240,9 +241,9 @@ lwmsg_assoc_unmarshal_handle(
             }
             break;
         default:
-            BAIL_ON_ERROR(status);
+            BAIL_ON_ERROR(status);    
         }
-
+        
         /* Set pointer on unmarshalled object */
         *(void**) object = pointer;
     }
@@ -274,12 +275,12 @@ lwmsg_assoc_free_handle(
     LWMsgAssoc* assoc = NULL;
     void* pointer = NULL;
     LWMsgSession* session = NULL;
-
+    
     BAIL_ON_ERROR(status = lwmsg_context_get_data(
                       lwmsg_data_context_get_context(context),
                       "assoc",
                       (void**) (void*) &assoc));
-
+    
     BAIL_ON_ERROR(status = assoc->aclass->get_session(assoc, &session));
 
     pointer = *(void**) object;
@@ -296,7 +297,7 @@ error:
 
 static LWMsgStatus
 lwmsg_assoc_print_handle(
-    LWMsgDataContext* context,
+    LWMsgDataContext* context, 
     LWMsgTypeAttrs* attrs,
     void* object,
     void* data,
@@ -326,7 +327,7 @@ lwmsg_assoc_print_handle(
             &type,
             &location,
             &handle);
-
+        
         switch (status)
         {
         case LWMSG_STATUS_NOT_FOUND:
