@@ -429,7 +429,7 @@ lwmsg_assoc_connect(
     LWMsgSession* session
     )
 {
-    return assoc->aclass->connect(assoc, session);
+    return assoc->aclass->connect_peer(assoc, session);
 }
 
 LWMsgStatus
@@ -439,7 +439,7 @@ lwmsg_assoc_accept(
     LWMsgSession** session
     )
 {
-    return assoc->aclass->accept(assoc, manager, session);
+    return assoc->aclass->accept_peer(assoc, manager, session);
 }
 
 LWMsgStatus
@@ -484,14 +484,14 @@ realloc_wrap(
         {
             new_length = length * 2;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_context_realloc(
                           context,
                           buffer->base,
                           length,
                           new_length,
                           (void**) (void*) &new_buffer));
-
+        
         buffer->base = new_buffer;
         buffer->end = new_buffer + new_length;
         buffer->cursor = new_buffer + offset;
@@ -518,14 +518,14 @@ lwmsg_assoc_print_message_alloc(
 
     buffer.wrap = realloc_wrap;
     buffer.data = (void*) &assoc->context;
-
+    
     BAIL_ON_ERROR(status = lwmsg_data_context_new(&assoc->context, &context));
     BAIL_ON_ERROR(status = lwmsg_protocol_get_message_name(assoc->prot, message->tag, &tag_name));
     BAIL_ON_ERROR(status = lwmsg_protocol_get_message_type(assoc->prot, message->tag, &type));
-
+    
     if (type)
     {
-        BAIL_ON_ERROR(status = lwmsg_buffer_print(&buffer, "%s:\n    ", tag_name));
+        BAIL_ON_ERROR(status = lwmsg_buffer_print(&buffer, "%s: ", tag_name));
         BAIL_ON_ERROR(status = lwmsg_data_print_graph(
                           context,
                           type,
@@ -539,7 +539,7 @@ lwmsg_assoc_print_message_alloc(
     {
         BAIL_ON_ERROR(status = lwmsg_buffer_print(&buffer, "%s", tag_name));
     }
-
+    
     *result = (char*) buffer.base;
 
 cleanup:
