@@ -36,6 +36,8 @@
  * Authors: Brian Koropoff (bkoropoff@likewisesoftware.com)
  *
  */
+
+#include "config.h"
 #include <lwmsg/type.h>
 #include <lwmsg/data.h>
 #include "convert-private.h"
@@ -47,7 +49,7 @@
 
 static LWMsgStatus
 lwmsg_connection_marshal_fd(
-    LWMsgDataContext* context,
+    LWMsgDataContext* context, 
     LWMsgTypeAttrs* attrs,
     void* object,
     void* transmit_object,
