@@ -36,6 +36,8 @@
  * Authors: Brian Koropoff (bkoropoff@likewisesoftware.com)
  *
  */
+
+#include "config.h"
 #include "assoc-private.h"
 #include "connection-private.h"
 #include "util-private.h"
@@ -155,7 +157,7 @@ lwmsg_connection_state_finish_close(
     ConnectionState* state,
     ConnectionEvent* event
     );
-
+ 
 static inline
 LWMsgStatus
 lwmsg_connection_state_begin_reset(
@@ -203,7 +205,7 @@ lwmsg_connection_run(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     ConnectionPrivate* priv = CONNECTION_PRIVATE(assoc);
     ConnectionState state = priv->state;
-
+    
     /* Run state machine until there is nothing to do */
     while (event != CONNECTION_EVENT_NONE)
     {
@@ -275,7 +277,7 @@ done:
     return status;
 
 error:
-
+    
     switch (status)
     {
     case LWMSG_STATUS_SUCCESS:
@@ -312,7 +314,7 @@ lwmsg_connection_state_start(
         {
             /* Set up pre-connected fd */
             BAIL_ON_ERROR(status = lwmsg_connection_connect_existing(assoc));
-
+            
             /* Go straight to the connect */
             *state = CONNECTION_STATE_BEGIN_SEND_CONNECT;
             *event = CONNECTION_EVENT_FINISH;
@@ -335,7 +337,7 @@ lwmsg_connection_state_start(
         {
             /* Set up pre-connected fd */
             BAIL_ON_ERROR(status = lwmsg_connection_connect_existing(assoc));
-
+            
             /* Go straight to the accept */
             *state = CONNECTION_STATE_BEGIN_RECV_CONNECT;
             *event = CONNECTION_EVENT_FINISH;
@@ -373,7 +375,7 @@ lwmsg_connection_state_begin_connect_socket(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
     BAIL_ON_ERROR(status = lwmsg_connection_begin_connect_socket(assoc));
-
+    
 done:
 
     if (status == LWMSG_STATUS_PENDING)
@@ -402,7 +404,7 @@ lwmsg_connection_state_finish_connect_socket(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-
+    
     switch (*event)
     {
     case CONNECTION_EVENT_FINISH:
@@ -704,7 +706,7 @@ lwmsg_connection_state_finish_recv_accept(
         }
 
         priv->active_session = priv->params.connect.session;
-
+                      
         *state = CONNECTION_STATE_ESTABLISHED;
         *event = CONNECTION_EVENT_NONE;
         break;
@@ -777,13 +779,11 @@ lwmsg_connection_state_established(
             BAIL_ON_ERROR(status = LWMSG_STATUS_BUSY);
         }
 
-        *event = CONNECTION_EVENT_FINISH;
+        *event = CONNECTION_EVENT_NONE;
         priv->outgoing = priv->params.message;
         status = lwmsg_connection_begin_send_message(assoc);
         switch (status)
         {
-        case LWMSG_STATUS_SUCCESS:
-            break;
         case LWMSG_STATUS_INVALID_HANDLE:
         case LWMSG_STATUS_MALFORMED:
         case LWMSG_STATUS_OVERFLOW:
@@ -794,6 +794,8 @@ lwmsg_connection_state_established(
         default:
             BAIL_ON_ERROR(status);
         }
+        BAIL_ON_ERROR(status = lwmsg_connection_finish_send_message(assoc));
+        priv->outgoing = NULL;
         break;
     case CONNECTION_EVENT_RECV:
         if (priv->incoming)
@@ -801,9 +803,11 @@ lwmsg_connection_state_established(
             BAIL_ON_ERROR(status = LWMSG_STATUS_BUSY);
         }
 
-        *event = CONNECTION_EVENT_FINISH;
+        *event = CONNECTION_EVENT_NONE;
         priv->incoming = priv->params.message;
         BAIL_ON_ERROR(status = lwmsg_connection_begin_recv_message(assoc));
+        BAIL_ON_ERROR(status = lwmsg_connection_finish_recv_message(assoc));
+        priv->incoming = NULL;
         break;
     case CONNECTION_EVENT_CLOSE:
         *state = CONNECTION_STATE_BEGIN_CLOSE;
@@ -873,7 +877,7 @@ lwmsg_connection_state_finish_close(
         if (priv->fd != -1)
         {
             status = lwmsg_connection_finish_send_shutdown(assoc);
-
+            
             switch (status)
             {
             case LWMSG_STATUS_PEER_CLOSE:
@@ -887,9 +891,9 @@ lwmsg_connection_state_finish_close(
                 break;
             }
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_connection_finish_close(assoc));
-
+        
         *state = CONNECTION_STATE_CLOSED;
         *event = CONNECTION_EVENT_NONE;
         break;
@@ -906,7 +910,7 @@ error:
     goto done;
 }
 
-
+ 
 static inline
 LWMsgStatus
 lwmsg_connection_state_begin_reset(
@@ -961,9 +965,9 @@ lwmsg_connection_state_finish_reset(
             BAIL_ON_ERROR(status);
             break;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_connection_finish_close(assoc));
-
+        
         *state = CONNECTION_STATE_START;
         *event = CONNECTION_EVENT_NONE;
         break;
@@ -1021,7 +1025,7 @@ lwmsg_connection_state_error(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-
+    
     switch(*event)
     {
     case CONNECTION_EVENT_CLOSE:
@@ -1054,7 +1058,7 @@ lwmsg_connection_finish_close(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     ConnectionPrivate* priv = CONNECTION_PRIVATE(assoc);
-
+    
     if (priv->fd != -1)
     {
         close(priv->fd);
