@@ -36,6 +36,8 @@
  * Authors: Brian Koropoff (bkoropoff@likewisesoftware.com)
  *
  */
+
+#include "config.h"
 #include "assoc-private.h"
 #include "connection-private.h"
 #include "util-private.h"
@@ -59,7 +61,7 @@ lwmsg_connection_construct(
     priv->packet_size = 8192;
     priv->fd = -1;
     priv->mode = LWMSG_CONNECTION_MODE_NONE;
-
+    
     memset(&priv->timeout.establish, 0xFF, sizeof(priv->timeout.establish));
     memset(&priv->timeout.message, 0xFF, sizeof(priv->timeout.message));
 
@@ -104,7 +106,7 @@ lwmsg_connection_destruct(
     {
         lwmsg_session_manager_delete(priv->default_manager);
     }
-
+    
     if (priv->marshal_context)
     {
         lwmsg_data_context_delete(priv->marshal_context);
@@ -136,7 +138,7 @@ lwmsg_connection_reset(
     BAIL_ON_ERROR(status = lwmsg_connection_run(assoc, CONNECTION_EVENT_RESET));
 
 error:
-
+    
     return status;
 }
 
@@ -184,12 +186,12 @@ lwmsg_connection_finish(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     ConnectionPrivate* priv = CONNECTION_PRIVATE(assoc);
-
+    
     if (message)
     {
         priv->params.message = NULL;
     }
-
+    
     BAIL_ON_ERROR(status = lwmsg_connection_run(assoc, CONNECTION_EVENT_FINISH));
 
     if (message)
@@ -413,7 +415,7 @@ lwmsg_connection_accept(
 
     priv->params.accept.manager = manager;
     priv->params.accept.session = session;
-
+    
     BAIL_ON_ERROR(status = lwmsg_connection_run(assoc, CONNECTION_EVENT_ACCEPT));
 
 error:
@@ -432,8 +434,8 @@ static LWMsgAssocClass connection_class =
     .get_session = lwmsg_connection_get_session,
     .get_state = lwmsg_connection_get_state,
     .set_timeout = lwmsg_connection_set_timeout,
-    .connect = lwmsg_connection_connect,
-    .accept = lwmsg_connection_accept,
+    .connect_peer = lwmsg_connection_connect,
+    .accept_peer = lwmsg_connection_accept,
     .set_nonblock = lwmsg_connection_set_nonblock,
     .finish = lwmsg_connection_finish
 };
