@@ -302,6 +302,7 @@ void
 lwmsg_context_log(
     const LWMsgContext* context,
     LWMsgLogLevel level,
+    const char* function,
     const char* message,
     const char* filename,
     unsigned int line
@@ -314,7 +315,7 @@ lwmsg_context_log(
 
     if (logfn)
     {
-        logfn(level, message, filename, line, logfndata);
+        logfn(level, message, function, filename, line, logfndata);
     }
 }
 
@@ -322,6 +323,7 @@ void
 lwmsg_context_log_printf(
     const LWMsgContext* context,
     LWMsgLogLevel level,
+    const char* function,
     const char* filename,
     unsigned int line,
     const char* format,
@@ -343,7 +345,7 @@ lwmsg_context_log_printf(
 
         if (message)
         {
-            logfn(level, message, filename, line, logfndata);
+            logfn(level, message, function, filename, line, logfndata);
             free(message);
         }
     }
@@ -362,7 +364,7 @@ lwmsg_context_alloc(
     void* data = NULL;
 
     lwmsg_context_get_memory_functions(context, &fn_alloc, NULL, &fn_realloc, &data);
-
+    
     if (fn_alloc)
     {
         BAIL_ON_ERROR(status = fn_alloc(size, object, data));
@@ -397,7 +399,7 @@ lwmsg_context_free(
     void* data = NULL;
 
     lwmsg_context_get_memory_functions(context, NULL, &fn_free, NULL, &data);
-
+    
     fn_free(object, data);
 }
 
@@ -415,9 +417,9 @@ lwmsg_context_realloc(
     LWMsgFreeFunction fn_free = NULL;
     LWMsgReallocFunction fn_realloc = NULL;
     void* data = NULL;
-
+    
     lwmsg_context_get_memory_functions(context, &fn_alloc, &fn_free, &fn_realloc, &data);
-
+    
     if (fn_realloc)
     {
         BAIL_ON_ERROR(status = fn_realloc(old_object, old_size, new_size, new_object, data));
@@ -457,7 +459,7 @@ lwmsg_context_would_log(
 
     if (logfn)
     {
-        return logfn(level, NULL, NULL, 0, logfndata);
+        return logfn(level, NULL, NULL, NULL, 0, logfndata);
     }
     else
     {
@@ -486,7 +488,7 @@ lwmsg_memlist_alloc(
                       list->parent_context,
                       size,
                       (void**) (void*) &ring));
-
+    
     lwmsg_ring_init(ring);
     lwmsg_ring_enqueue(&list->blocks, ring);
 
@@ -541,7 +543,7 @@ lwmsg_memlist_realloc (
     lwmsg_ring_enqueue(&list->blocks, new_ring);
 
     *new_object = ((unsigned char*) new_ring) + MEMLIST_HEADER_SIZE;
-
+    
 error:
 
     return status;
