@@ -35,12 +35,13 @@
  * Authors: Brian Koropoff (bkoropoff@likewisesoftware.com)
  *
  */
+
+#include "config.h"
 #include <lwmsg/type.h>
 #include <lwmsg/status.h>
 #include "util-private.h"
 #include "convert-private.h"
 
-#include <config.h>
 #include <stdlib.h>
 #include <string.h>
 
