@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -94,7 +94,7 @@ lwmsg_data_extract_active_arm(
                       iter,
                       dominating_struct,
                       &tag));
-
+    
     for (lwmsg_type_enter(iter, active_iter);
          lwmsg_type_valid(active_iter);
          lwmsg_type_next(active_iter))
@@ -106,13 +106,13 @@ lwmsg_data_extract_active_arm(
     }
 
     BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
-
+    
 done:
 
     return status;
 
 error:
-
+        
     goto done;
 }
 
@@ -153,7 +153,7 @@ lwmsg_data_calculate_indirect_metrics(
     unsigned char* element = NULL;
     int is_zero;
     LWMsgTypeIter inner;
-
+    
     lwmsg_type_enter(iter, &inner);
 
     switch (iter->info.kind_indirect.term)
@@ -634,3 +634,20 @@ lwmsg_data_destroy_graph(
 
     return lwmsg_data_free_graph_internal(context, &iter, root);
 }
+
+void
+lwmsg_data_destroy_graph_cleanup(
+    const LWMsgContext* context,
+    LWMsgTypeSpec* type,
+    void* root
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgDataContext dcontext;
+
+    memset(&dcontext, 0, sizeof(dcontext));
+    dcontext.context = context;
+
+    status = lwmsg_data_destroy_graph(&dcontext, type, root);
+    LWMSG_ASSERT(status == LWMSG_STATUS_SUCCESS);
+}
