@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -35,12 +35,13 @@
  * Authors: Brian Koropoff (bkoropoff@likewisesoftware.com)
  *
  */
+
+#include "config.h"
 #include "data-private.h"
 #include "util-private.h"
 #include "type-private.h"
 #include "buffer-private.h"
 #include "convert-private.h"
-#include "config.h"
 
 #include <string.h>
 
@@ -115,7 +116,7 @@ lwmsg_data_marshal_indirect_prologue(
         {
             element = object;
             is_zero = 0;
-
+            
             /* We have to calculate the count by searching for the zero element */
             for (*count = 0;;*count += 1)
             {
@@ -124,26 +125,26 @@ lwmsg_data_marshal_indirect_prologue(
                                   inner_iter,
                                   element,
                                   &is_zero));
-
+                
                 if (is_zero)
                 {
                     break;
                 }
-
+                
                 element += inner_iter->size;
             }
-
+            
             /* The length is implicitly written into the output to
                facilitate easy unmarshalling */
             BAIL_ON_ERROR(status = lwmsg_convert_integer(
                               count,
-                              sizeof(count),
+                              sizeof(*count),
                               LWMSG_NATIVE_ENDIAN,
                               implicit_length,
                               4,
                               context->byte_order,
                               LWMSG_UNSIGNED));
-
+            
             BAIL_ON_ERROR(status = lwmsg_buffer_write(buffer, implicit_length, 4));
         }
         break;
@@ -176,7 +177,7 @@ lwmsg_data_marshal_indirect(
                       buffer,
                       &inner_iter,
                       &count));
-
+    
     /* Enforce nullability.  Even if the pointer is marked
        as non-null, allow it to be null if there are no
        elements to marshal */
@@ -194,8 +195,8 @@ lwmsg_data_marshal_indirect(
            packed and unpacked, we can write the entire array directly into
            the output.  This is important for character strings */
         BAIL_ON_ERROR(status = lwmsg_buffer_write(
-                          buffer,
-                          object,
+                          buffer, 
+                          object, 
                           count));
     }
     else if (count)
@@ -208,8 +209,8 @@ lwmsg_data_marshal_indirect(
             BAIL_ON_ERROR(status = lwmsg_data_marshal_internal(
                               context,
                               state,
-                              &inner_iter,
-                              element,
+                              &inner_iter, 
+                              element, 
                               buffer));
             element += inner_iter.size;
         }
@@ -222,7 +223,7 @@ error:
 
 static LWMsgStatus
 lwmsg_data_marshal_integer(
-    LWMsgDataContext* context,
+    LWMsgDataContext* context, 
     LWMsgMarshalState* state,
     LWMsgTypeIter* iter,
     unsigned char* object,
@@ -254,7 +255,7 @@ lwmsg_data_marshal_integer(
                                                  out_size,
                                                  context->byte_order,
                                                  iter->info.kind_integer.sign));
-
+    
     BAIL_ON_ERROR(status = lwmsg_buffer_write(buffer, out, out_size));
 
 
@@ -359,7 +360,7 @@ lwmsg_data_marshal_custom(
                       &transmit_iter,
                       transmit_object,
                       buffer));
-
+                      
 
 error:
 
@@ -701,7 +702,7 @@ lwmsg_data_marshal_flat(
     mbuf.cursor = buffer;
     mbuf.wrap = NULL;
 
-    return lwmsg_data_marshal(context, type, object, &mbuf);
+    return lwmsg_data_marshal(context, type, object, &mbuf);    
 }
 
 static
@@ -717,7 +718,7 @@ lwmsg_data_marshal_alloc_wrap(
     size_t newlen = oldlen == 0 ? 256 : oldlen * 2;
     void* newmem = NULL;
     size_t offset = 0;
-
+    
     BAIL_ON_ERROR(status = lwmsg_context_realloc(context->context, buffer->base, oldlen, newlen, &newmem));
 
     offset = buffer->cursor - buffer->base;
@@ -741,7 +742,7 @@ lwmsg_data_marshal_flat_alloc(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     LWMsgBuffer mbuf = {0};
-
+    
     mbuf.base = NULL;
     mbuf.cursor = NULL;
     mbuf.end = NULL;
