@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -36,6 +36,7 @@
  *
  */
 
+#include "config.h"
 #include "data-private.h"
 #include "util-private.h"
 #include "type-private.h"
@@ -139,7 +140,7 @@ newline(
                       info->buffer,
                       nl,
                       sizeof(nl)));
-
+    
     info->newline = LWMSG_TRUE;
 
 error:
@@ -147,6 +148,22 @@ error:
     return status;
 }
 
+static
+LWMsgStatus
+newline_if(
+    PrintInfo* info
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    if (!info->newline)
+    {
+        status = newline(info);
+    }
+
+    return status;
+}
+
 static
 LWMsgStatus
 lwmsg_data_print_integer(
@@ -282,19 +299,19 @@ lwmsg_data_print_graph_visit_member(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PrintInfo* info = (PrintInfo*) data;
-
+    
     if (iter->meta.member_name)
     {
         BAIL_ON_ERROR(status = print(info, "%s = ", iter->meta.member_name));
     }
 
-    BAIL_ON_ERROR(status =
+    BAIL_ON_ERROR(status = 
                   lwmsg_data_print_graph_visit(
                       iter,
                       object,
                       data));
 
-    BAIL_ON_ERROR(status = newline(info));
+    BAIL_ON_ERROR(status = newline_if(info));
 
 error:
 
@@ -327,6 +344,14 @@ lwmsg_data_print_string(
         encoding = "UCS-2LE";
 #endif
     }
+    else if (!strcmp(encoding, "utf-16"))
+    {
+#ifdef WORDS_BIGENDIAN
+        encoding = "UTF-16BE";
+#else
+        encoding = "UTF-16LE";
+#endif
+    }
 
     if (iter->kind == LWMSG_KIND_POINTER)
     {
@@ -343,7 +368,10 @@ lwmsg_data_print_string(
                       &element_count,
                       &element_size));
 
-    input_length = element_count * element_size;
+    BAIL_ON_ERROR(status = lwmsg_multiply_unsigned(
+        element_count,
+        element_size,
+        &input_length));
 
     if (!strcmp(iter->info.kind_indirect.encoding, ""))
     {
@@ -379,7 +407,7 @@ lwmsg_data_print_string(
             (void**) (void*) &output_string,
             encoding,
             "");
-
+        
         if (output_length == -1)
         {
             BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
@@ -434,8 +462,7 @@ lwmsg_data_print_hex_ascii(
 
     BAIL_ON_ERROR(status = lwmsg_multiply_unsigned(element_count, element_size, &input_length));
 
-    BAIL_ON_ERROR(status = print(info, "<hex+ascii>"));
-    BAIL_ON_ERROR(status = newline(info));
+    BAIL_ON_ERROR(status = newline_if(info));
     BAIL_ON_ERROR(status = print(info, "{"));
     info->depth += 4;
     BAIL_ON_ERROR(status = newline(info));
@@ -450,7 +477,7 @@ lwmsg_data_print_hex_ascii(
             {
                 BAIL_ON_ERROR(status = print(info, " "));
             }
-
+            
             if (index >= input_length)
             {
                 BAIL_ON_ERROR(status = print(info, "  "));
@@ -511,7 +538,7 @@ lwmsg_data_print_in_encoding(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-
+    
     if (!strcmp(iter->info.kind_indirect.encoding, "hex+ascii"))
     {
         BAIL_ON_ERROR(status = lwmsg_data_print_hex_ascii(iter, object, info));
@@ -522,12 +549,12 @@ lwmsg_data_print_in_encoding(
         BAIL_ON_ERROR(status = lwmsg_data_print_string(iter, object, info));
         BAIL_ON_ERROR(status = print(info, "\""));
     }
-
+    
 error:
 
     return status;
 }
-
+    
 static
 LWMsgStatus
 lwmsg_data_print_custom(
@@ -606,6 +633,7 @@ lwmsg_data_print_graph_visit(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PrintInfo* info = (PrintInfo*) data;
     LWMsgObjectID id = 0;
+    LWMsgTypeIter arm;
 
     if (iter->attrs.flags & LWMSG_TYPE_FLAG_SENSITIVE)
     {
@@ -616,16 +644,22 @@ lwmsg_data_print_graph_visit(
         switch (iter->kind)
         {
         case LWMSG_KIND_UNION:
+            BAIL_ON_ERROR(status = lwmsg_data_extract_active_arm(
+                iter,
+                iter->dom_object,
+                &arm));
+            if (arm.meta.member_name)
+            {
+                BAIL_ON_ERROR(status = print(info, "<arm:%s>", arm.meta.member_name));
+            }
+            BAIL_ON_ERROR(status = lwmsg_data_visit_graph(
+                                          &arm,
+                                          object,
+                                          lwmsg_data_print_graph_visit,
+                                          data));
+            break;
         case LWMSG_KIND_STRUCT:
-            if (iter->meta.type_name)
-            {
-                BAIL_ON_ERROR(status = print(info, "<%s>", iter->meta.type_name));
-            }
-            else
-            {
-                BAIL_ON_ERROR(status = print(info, iter->kind == LWMSG_KIND_STRUCT ? "<struct>" : "<union>"));
-            }
-            BAIL_ON_ERROR(status = newline(info));
+            BAIL_ON_ERROR(status = newline_if(info));
             BAIL_ON_ERROR(status = print(info, "{"));
             BAIL_ON_ERROR(status = newline(info));
             info->depth += 4;
@@ -651,10 +685,6 @@ lwmsg_data_print_graph_visit(
                 {
                     /* Print nothing */
                 }
-                else if (iter->kind == LWMSG_KIND_ARRAY)
-                {
-                    BAIL_ON_ERROR(status = print(info, "<array> "));
-                }
                 else if (iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE)
                 {
                     status = lwmsg_data_object_map_find_object(
@@ -669,13 +699,11 @@ lwmsg_data_print_graph_visit(
                                           *(void**) object,
                                           iter,
                                           &id));
-
                         BAIL_ON_ERROR(status = print(info, "<alias:%lu> ", (unsigned long) id));
                     }
                     else
                     {
                         BAIL_ON_ERROR(status);
-
                         BAIL_ON_ERROR(status = print(info, "<alias:%lu>", (unsigned long) id));
                         /* Skip printing pointee */
                         goto error;
@@ -702,7 +730,7 @@ lwmsg_data_print_graph_visit(
                 /* General case */
                 else
                 {
-                    BAIL_ON_ERROR(status = newline(info));
+                    BAIL_ON_ERROR(status = newline_if(info));
                     BAIL_ON_ERROR(status = print(info, "{"));
                     BAIL_ON_ERROR(status = newline(info));
                     info->depth += 4;
@@ -738,9 +766,9 @@ lwmsg_data_print_graph_visit(
             break;
         }
     }
-
+        
 error:
-
+    
     return status;
 }
 
@@ -765,7 +793,7 @@ lwmsg_data_print_graph(
     info.buffer = buffer;
 
     lwmsg_type_iterate_promoted(type, &iter);
-
+    
     BAIL_ON_ERROR(status = lwmsg_data_visit_graph(
                       &iter,
                       (unsigned char*) &object,
@@ -803,14 +831,14 @@ realloc_wrap(
         {
             new_length = length * 2;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_context_realloc(
                           context,
                           buffer->base,
                           length,
                           new_length,
                           (void**) (void*) &new_buffer));
-
+        
         buffer->base = new_buffer;
         buffer->end = new_buffer + new_length;
         buffer->cursor = new_buffer + offset;
@@ -844,9 +872,9 @@ lwmsg_data_print_graph_alloc(
                       &buffer));
 
     BAIL_ON_ERROR(status = lwmsg_buffer_write(&buffer, &nul, 1));
-
+    
     *result = (char*) buffer.base;
-
+   
 cleanup:
 
     return status;
