@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -68,7 +68,7 @@ lwmsg_peer_call_activate_outgoing(
     lwmsg_ring_remove(&call->queue_ring);
     lwmsg_ring_enqueue(&call->task->active_outgoing_calls, &call->queue_ring);
 }
-
+    
 
 static
 void
@@ -97,7 +97,7 @@ lwmsg_peer_call_worker(
         pthread_mutex_lock(&call->task->call_lock);
         call->state |= PEER_CALL_DISPATCHED | PEER_CALL_COMPLETED;
         lwmsg_peer_call_activate_incoming(call);
-        lwmsg_task_wake(call->task->event_task);
+        lwmsg_task_wake(call->task->event_task); 
         pthread_mutex_unlock(&call->task->call_lock);
         break;
     case LWMSG_DISPATCH_TYPE_BLOCK:
@@ -110,12 +110,12 @@ lwmsg_peer_call_worker(
 
         pthread_mutex_lock(&call->task->call_lock);
         call->state |= PEER_CALL_DISPATCHED;
-
+        
         if (call->state & PEER_CALL_COMPLETED)
         {
             /* The call was already completed */
             lwmsg_peer_call_activate_incoming(call);
-            lwmsg_task_wake(call->task->event_task);
+            lwmsg_task_wake(call->task->event_task); 
         }
         else if ((call->state & PEER_CALL_CANCELLED) &&
                  (call->state & PEER_CALL_PENDED))
@@ -162,7 +162,7 @@ lwmsg_peer_call_dispatch_incoming(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_UNIMPLEMENTED);
     }
-
+    
     switch (call->params.incoming.spec->type)
     {
     case LWMSG_DISPATCH_TYPE_OLD:
@@ -178,6 +178,15 @@ lwmsg_peer_call_dispatch_incoming(
         call->params.incoming.in.data = incoming_message->data;
         call->params.incoming.out.tag = LWMSG_TAG_INVALID;
         call->params.incoming.out.data = NULL;
+
+        if (call->task->peer->trace_begin)
+        {
+            call->task->peer->trace_begin(
+                LWMSG_CALL(call),
+                &call->params.incoming.in,
+                call->task->peer->trace_data);
+        }
+
         if (call->params.incoming.spec->type == LWMSG_DISPATCH_TYPE_BLOCK)
         {
             BAIL_ON_ERROR(status = lwmsg_task_dispatch_work_item(
@@ -188,11 +197,14 @@ lwmsg_peer_call_dispatch_incoming(
         }
         else
         {
+            /* Drop lock for duration of call */
+            pthread_mutex_unlock(&call->task->call_lock);
             status = ((LWMsgPeerCallFunction) call->params.incoming.spec->data) (
                 LWMSG_CALL(call),
                 &call->params.incoming.in,
                 &call->params.incoming.out,
                 call->params.incoming.dispatch_data);
+            pthread_mutex_lock(&call->task->call_lock);
 
             call->state |= PEER_CALL_DISPATCHED;
 
@@ -219,9 +231,9 @@ lwmsg_peer_call_dispatch_incoming(
         status = LWMSG_STATUS_INTERNAL;
         break;
     }
-
+    
 error:
-
+    
     return status;
 }
 
@@ -265,6 +277,15 @@ lwmsg_peer_call_dispatch_outgoing(
     lwmsg_peer_call_activate_outgoing(pcall);
     lwmsg_task_wake(pcall->task->event_task);
 
+    /* Trace call start */
+    if (pcall->task->peer->trace_begin)
+    {
+        pcall->task->peer->trace_begin(
+            LWMSG_CALL(call),
+            pcall->params.outgoing.in,
+            pcall->task->peer->trace_data);
+    }
+
     if (!complete)
     {
         while (!(pcall->state & PEER_CALL_COMPLETED))
@@ -333,7 +354,7 @@ lwmsg_peer_call_complete_incoming(
     }
 
     pthread_mutex_unlock(&pcall->task->call_lock);
-
+    
     return status;
 }
 
@@ -345,6 +366,8 @@ lwmsg_peer_call_complete_outgoing(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
+    lwmsg_hash_remove_entry(&call->task->outgoing_calls, call);
+
     if (call->state & PEER_CALL_RELEASED)
     {
         lwmsg_assoc_destroy_message(call->task->assoc, incoming_message);
@@ -359,6 +382,15 @@ lwmsg_peer_call_complete_outgoing(
 
         lwmsg_message_init(incoming_message);
 
+        /* Trace call completion */
+        if (call->task->peer->trace_end)
+        {
+            call->task->peer->trace_end(
+                LWMSG_CALL(call),
+                call->params.outgoing.out,
+                call->task->peer->trace_data);
+        }
+
         if (call->params.outgoing.complete)
         {
             call->params.outgoing.complete(
@@ -371,7 +403,7 @@ lwmsg_peer_call_complete_outgoing(
             pthread_cond_broadcast(&call->task->call_event);
         }
     }
-
+    
     return status;
 }
 
@@ -385,14 +417,14 @@ lwmsg_peer_call_cancel_incoming(
     if (!(call->state & PEER_CALL_CANCELLED))
     {
         call->state |= PEER_CALL_CANCELLED;
-
+        
         /* If the dispatch function is not finished running,
            we don't call the cancel callback right now --
            lwmsg_peer_call_dispatch_incoming() will handle it.
-
+           
            If the call is already completed, we silently
            ignore the cancel request and return success */
-
+        
         if ((call->state & PEER_CALL_DISPATCHED) &&
             !(call->state & PEER_CALL_COMPLETED) &&
             call->params.incoming.cancel != NULL)
@@ -493,7 +525,7 @@ lwmsg_peer_call_destroy_params(
 
     message.tag = params->tag;
     message.data = params->data;
-
+    
     BAIL_ON_ERROR(status = lwmsg_assoc_destroy_message(pcall->task->assoc, &message));
 
     params->tag = LWMSG_TAG_INVALID;
@@ -517,6 +549,8 @@ lwmsg_peer_call_acquire_callback(
 
     BAIL_ON_ERROR(status = lwmsg_peer_call_new(pcall->task, &my_callback));
 
+    my_callback->base.is_outgoing = LWMSG_TRUE;
+
     *callback = LWMSG_CALL(my_callback);
 
 error:
