@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -49,21 +49,23 @@
 #include <sys/stat.h>
 
 static
-LWMsgStatus
+void
 lwmsg_peer_task_run(
+    LWMsgTask* _task,
     void* data,
     LWMsgTaskTrigger trigger,
     LWMsgTaskTrigger* next_trigger,
-    LWMsgTime* next_wakeup
+    LWMsgTaskTime* next_timeout
     );
 
 static
-LWMsgStatus
+void
 lwmsg_peer_task_run_listen(
+    LWMsgTask* _task,
     void* data,
     LWMsgTaskTrigger trigger,
     LWMsgTaskTrigger* next_trigger,
-    LWMsgTime* next_timeout
+    LWMsgTaskTime* next_timeout
     );
 
 static
@@ -169,7 +171,7 @@ lwmsg_peer_log_message(
         {
             data_direction = ">>";
         }
-
+    
         if (message->status != LWMSG_STATUS_SUCCESS)
         {
             status_text = lwmsg_string_without_prefix(
@@ -264,7 +266,7 @@ lwmsg_peer_log_accept(
     if (lwmsg_context_would_log(peer->context, LWMSG_LOGLEVEL_VERBOSE))
     {
         lwmsg_peer_session_string_for_assoc(assoc, buffer);
-
+            
         LWMSG_LOG_VERBOSE(peer->context, "(session:%s) Accepted association 0x%lx",
                           buffer, (unsigned long) (size_t) assoc);
     }
@@ -284,7 +286,7 @@ lwmsg_peer_log_connect(
     if (lwmsg_context_would_log(peer->context, LWMSG_LOGLEVEL_VERBOSE))
     {
         lwmsg_peer_session_string_for_assoc(assoc, buffer);
-
+            
         LWMSG_LOG_VERBOSE(peer->context, "(session:%s) Connected association 0x%lx",
                           buffer, (unsigned long) (size_t) assoc);
     }
@@ -293,23 +295,27 @@ lwmsg_peer_log_connect(
 }
 
 static
-LWMsgTaskTrigger
-lwmsg_peer_task_assoc_trigger(
-    LWMsgAssoc* assoc
+void
+lwmsg_peer_task_update_blocked(
+    PeerAssocTask* task
     )
 {
-    switch (lwmsg_assoc_get_state(assoc))
+    switch (lwmsg_assoc_get_state(task->assoc))
     {
-    case LWMSG_ASSOC_STATE_IDLE:
-        return LWMSG_TASK_TRIGGER_FD_READABLE;
     case LWMSG_ASSOC_STATE_BLOCKED_SEND:
-        return LWMSG_TASK_TRIGGER_FD_WRITABLE;
+        task->send_blocked = LWMSG_TRUE;
+        task->recv_blocked = LWMSG_FALSE;
+        break;
     case LWMSG_ASSOC_STATE_BLOCKED_RECV:
-        return LWMSG_TASK_TRIGGER_FD_READABLE;
+        task->recv_blocked = LWMSG_TRUE;
+        task->send_blocked = LWMSG_FALSE;
+        break;
     case LWMSG_ASSOC_STATE_BLOCKED_SEND_RECV:
-        return LWMSG_TASK_TRIGGER_FD_READABLE | LWMSG_TASK_TRIGGER_FD_WRITABLE;
+        task->send_blocked = task->recv_blocked = LWMSG_TRUE;
+        break;
     default:
-        return 0;
+        task->send_blocked = task->recv_blocked = LWMSG_FALSE;
+        break;
     }
 }
 
@@ -517,7 +523,7 @@ error:
             {
                 pthread_mutex_destroy(&my_task->call_lock);
             }
-
+            
             if (free_cond)
             {
                 pthread_cond_destroy(&my_task->call_event);
@@ -602,38 +608,36 @@ lwmsg_peer_task_setup_listen(
     {
         BAIL_ON_ERROR(status = lwmsg_peer_task_create_local_socket(task));
     }
-
+    
     /* Get socket flags */
     if ((opts = fcntl(task->fd, F_GETFL, 0)) < 0)
     {
         BAIL_ON_ERROR(status = lwmsg_error_map_errno(errno));
     }
-
+    
     /* Set non-blocking flag */
     opts |= O_NONBLOCK;
-
+    
     /* Set socket flags */
     if (fcntl(task->fd, F_SETFL, opts) < 0)
     {
         BAIL_ON_ERROR(status = lwmsg_error_map_errno(errno));
     }
-
+    
     if (listen(task->fd, task->peer->max_backlog))
     {
         BAIL_ON_ERROR(status = lwmsg_error_map_errno(errno));
     }
-
+    
     if (task->endpoint)
     {
         LWMSG_LOG_INFO(task->peer->context, "Listening on endpoint %s", task->endpoint);
     }
     else
     {
-        LWMSG_LOG_INFO(task->peer->context, "Listening on fd %i", task->fd);
+        LWMSG_LOG_INFO(task->peer->context, "Listening on fd %i", task->fd);  
     }
 
-    lwmsg_task_set_trigger_fd(task->event_task, task->fd);
-
 error:
 
     return status;
@@ -651,9 +655,9 @@ lwmsg_peer_listen_task_new(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerListenTask* my_task = NULL;
-
+    
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&my_task));
-
+    
     BAIL_ON_ERROR(status = lwmsg_task_new(
                       peer->task_manager,
                       peer->listen_tasks,
@@ -666,13 +670,13 @@ lwmsg_peer_listen_task_new(
     if (endpoint)
     {
         my_task->endpoint = strdup(endpoint);
-
+    
         if (!my_task->endpoint)
         {
             BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
         }
     }
-
+    
     my_task->fd = fd;
     my_task->type = type;
     my_task->perms = perms;
@@ -701,7 +705,7 @@ lwmsg_peer_assoc_task_new_accept(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerAssocTask* my_task = NULL;
-
+    
     BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, PEER_TASK_BEGIN_ACCEPT, &my_task));
 
     my_task->assoc = assoc;
@@ -744,7 +748,7 @@ lwmsg_peer_assoc_task_new_connect(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerAssocTask* my_task = NULL;
-
+    
     BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, PEER_TASK_BEGIN_CONNECT, &my_task));
 
     my_task->assoc = assoc;
@@ -830,8 +834,8 @@ lwmsg_peer_task_subject_to_timeout(
     {
         handle_count = lwmsg_session_get_handle_count(task->session);
         num_clients = lwmsg_peer_get_num_clients(peer);
-
-        return handle_count == 0 && num_clients == peer->max_clients &&
+        
+        return handle_count == 0 && num_clients == peer->max_clients && 
             lwmsg_hash_get_count(&task->incoming_calls) == 0;
     }
     else
@@ -854,15 +858,12 @@ lwmsg_peer_task_set_timeout(
     )
 {
     *next_timeout = *timeout;
-    *next_trigger = lwmsg_peer_task_assoc_trigger(task->assoc);
 
     if (lwmsg_time_is_positive(timeout) &&
         lwmsg_peer_task_subject_to_timeout(peer, task, trigger))
     {
         *next_trigger |= LWMSG_TASK_TRIGGER_TIME;
     }
-
-    lwmsg_task_set_trigger_fd(task->event_task, CONNECTION_PRIVATE(task->assoc)->fd);
 }
 
 static
@@ -931,12 +932,13 @@ lwmsg_peer_task_handle_call_error(
 }
 
 static
-LWMsgStatus
+void
 lwmsg_peer_task_run_listen(
+    LWMsgTask* _task,
     void* data,
     LWMsgTaskTrigger trigger,
     LWMsgTaskTrigger* next_trigger,
-    LWMsgTime* next_timeout
+    LWMsgTaskTime* next_timeout
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
@@ -946,6 +948,11 @@ lwmsg_peer_task_run_listen(
     PeerAssocTask* client_task = NULL;
     LWMsgBool slot = LWMSG_FALSE;
     int err = 0;
+    
+    if (trigger & LWMSG_TASK_TRIGGER_INIT)
+    {
+        BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, task->fd));
+    }
 
     while (status == LWMSG_STATUS_SUCCESS)
     {
@@ -953,20 +960,20 @@ lwmsg_peer_task_run_listen(
         client_task = NULL;
         slot = LWMSG_FALSE;
         client_fd = -1;
-
+        
         if (trigger & LWMSG_TASK_TRIGGER_CANCEL)
         {
             lwmsg_peer_listen_task_delete_self(task);
             *next_trigger = 0;
             goto done;
         }
-
+        
         if ((slot = lwmsg_peer_acquire_client_slot(task->peer)))
         {
             do
             {
                 client_fd = accept(task->fd, NULL, NULL);
-
+                
                 if (client_fd < 0)
                 {
                     switch (errno)
@@ -989,9 +996,9 @@ lwmsg_peer_task_run_listen(
                     }
                 }
             } while (client_fd < 0);
-
+            
             BAIL_ON_ERROR(status = lwmsg_set_close_on_exec(client_fd));
-
+            
             /* Create new connection with client fd, put it into task, schedule task */
             BAIL_ON_ERROR(status = lwmsg_connection_new(task->peer->context, task->peer->protocol, &assoc));
             BAIL_ON_ERROR(status = lwmsg_connection_set_fd(assoc, LWMSG_CONNECTION_MODE_LOCAL, client_fd));
@@ -999,7 +1006,7 @@ lwmsg_peer_task_run_listen(
             BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new_accept(task->peer, assoc, &client_task));
             assoc = NULL;
             slot = LWMSG_FALSE;
-
+            
             /* Release a reference so the task will be freed immediately
                when it is cancelled */
             lwmsg_peer_task_unref(client_task);
@@ -1021,20 +1028,20 @@ done:
         lwmsg_peer_release_client_slot(task->peer);
     }
 
-    return LWMSG_STATUS_SUCCESS;
+    return;
 
 error:
-
+    
     if (client_fd >= 0 && !assoc)
     {
         close(client_fd);
     }
-
+    
     if (assoc)
     {
         lwmsg_assoc_delete(assoc);
     }
-
+    
     if (client_task)
     {
         lwmsg_peer_task_cancel_and_unref(client_task);
@@ -1061,7 +1068,7 @@ error:
         *next_trigger = 0;
         break;
     }
-
+    
     goto done;
 }
 
@@ -1077,6 +1084,9 @@ lwmsg_peer_task_run_accept(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
+    /* We already have the fd when accepting a connection, so set it up for events now */
+    BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, CONNECTION_PRIVATE(task->assoc)->fd));
+
     status = lwmsg_assoc_accept(task->assoc, peer->session_manager, &task->session);
 
     switch (status)
@@ -1126,9 +1136,15 @@ lwmsg_peer_task_run_connect(
         task->type = PEER_TASK_DISPATCH;
         BAIL_ON_ERROR(status = lwmsg_peer_log_connect(peer, task->assoc));
         lwmsg_peer_task_notify_status(task, LWMSG_STATUS_SUCCESS);
+        /* Set up the fd the assoc just created for events */
+        BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, CONNECTION_PRIVATE(task->assoc)->fd));
         break;
     case LWMSG_STATUS_PENDING:
         task->type = PEER_TASK_FINISH_CONNECT;
+        /* Even if the connect pended, the fd is now available.  Set it up for events */
+        BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, CONNECTION_PRIVATE(task->assoc)->fd));
+        /* Restore status code */
+        status = LWMSG_STATUS_PENDING;
         lwmsg_peer_task_set_timeout(peer, task, &peer->timeout.establish, trigger, next_trigger, next_timeout);
         break;
     default:
@@ -1162,7 +1178,7 @@ lwmsg_peer_task_rundown(
     LWMsgHashIter iter = {0};
     PeerCall* call = NULL;
     LWMsgMessage cancel = LWMSG_MESSAGE_INITIALIZER;
-    LWMsgMessage message = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage message = LWMSG_MESSAGE_INITIALIZER;    
 
     pthread_mutex_lock(&task->call_lock);
 
@@ -1171,9 +1187,16 @@ lwmsg_peer_task_rundown(
     {
         if (call->state & PEER_CALL_COMPLETED)
         {
+            /* Destroy in call parameters */
+            message.tag = call->params.incoming.in.tag;
+            message.data = call->params.incoming.in.data;
+            lwmsg_assoc_destroy_message(task->assoc, &message);
+
+            /* Destroy out call parameters */
             message.tag = call->params.incoming.out.tag;
             message.data = call->params.incoming.out.data;
             lwmsg_assoc_destroy_message(task->assoc, &message);
+
             lwmsg_hash_remove_entry(&task->incoming_calls, call);
             lwmsg_peer_call_delete(call);
         }
@@ -1195,7 +1218,6 @@ lwmsg_peer_task_rundown(
         {
             cancel.status = LWMSG_STATUS_CANCELLED;
         }
-        lwmsg_hash_remove_entry(&task->outgoing_calls, call);
         lwmsg_peer_call_complete_outgoing(call, &cancel);
     }
     lwmsg_hash_iter_end(&task->outgoing_calls, &iter);
@@ -1225,11 +1247,28 @@ lwmsg_peer_task_run_shutdown(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    int fd = CONNECTION_PRIVATE(task->assoc)->fd;
 
-    /* Make sure all calls are run down before we close the association because it
-       will destroy the session and thus any handles that are open */
+    /* Before we close the assoc, we need to free any data it allocated */
+
+    /* First, make sure any calls are canceled, completed, and freed */
     BAIL_ON_ERROR(status = lwmsg_peer_task_rundown(peer, task, trigger, next_trigger, next_timeout));
 
+    /* Destroy any incoming message we never got around to dispatching */
+    lwmsg_assoc_destroy_message(task->assoc, &task->incoming_message);
+
+    /* Destroy any outgoing message we never got around to sending */
+    if (task->destroy_outgoing)
+    {
+        lwmsg_assoc_destroy_message(task->assoc, &task->outgoing_message);
+    }
+
+    /* We are going to close the fd, so unset it now */
+    if (fd >= 0)
+    {
+        lwmsg_task_unset_trigger_fd(task->event_task, fd);
+    }
+
     switch (task->type)
     {
     case PEER_TASK_BEGIN_CLOSE:
@@ -1241,13 +1280,18 @@ lwmsg_peer_task_run_shutdown(
     default:
         BAIL_ON_ERROR(status = LWMSG_STATUS_INTERNAL);
     }
-
+    
     switch (status)
     {
     case LWMSG_STATUS_SUCCESS:
         task->type = PEER_TASK_DROP;
         break;
     case LWMSG_STATUS_PENDING:
+        /* Oops, we still need to wait for fd events, so set it on the task again */
+        if (fd >= 0)
+        {
+            BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, fd));
+        }
         task->type = PEER_TASK_FINISH_CLOSE;
         lwmsg_peer_task_set_timeout(peer, task, &peer->timeout.establish, trigger, next_trigger, next_timeout);
         break;
@@ -1299,7 +1343,6 @@ lwmsg_peer_task_complete_call(
 
     if (call)
     {
-        lwmsg_hash_remove_entry(&task->outgoing_calls, call);
         lwmsg_peer_log_message(task->peer, task->assoc, &task->incoming_message, LWMSG_TRUE);
         lwmsg_peer_call_complete_outgoing(call, &task->incoming_message);
     }
@@ -1357,6 +1400,10 @@ lwmsg_peer_task_dispatch_incoming_message(
 
     /* Create a call structure to track call */
     BAIL_ON_ERROR(status = lwmsg_peer_call_new(task, &call));
+    call->base.is_outgoing = LWMSG_FALSE;
+    call->params.incoming.in.tag = LWMSG_TAG_INVALID;
+    call->params.incoming.out.tag = LWMSG_TAG_INVALID;
+
     /* The call structure holds a reference to the task */
     lwmsg_peer_task_ref(task);
 
@@ -1386,7 +1433,7 @@ lwmsg_peer_task_dispatch_incoming_message(
         lwmsg_assoc_destroy_message(task->assoc, &task->incoming_message);
         goto error;
     }
-
+    
     /* Make sure the call is supported */
     spec = peer->dispatch.vector[tag];
     if (spec == NULL)
@@ -1437,6 +1484,7 @@ lwmsg_peer_task_run_finish(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    int fd = CONNECTION_PRIVATE(task->assoc)->fd;
 
     /* When the peer runs out of available client slots, it will
        wake all of its tasks so that it can begin enforcing timeouts
@@ -1464,10 +1512,24 @@ lwmsg_peer_task_run_finish(
         task->type = PEER_TASK_DROP;
     }
     /* Is the task unblocked? */
-    else if (trigger & lwmsg_peer_task_assoc_trigger(task->assoc))
+    else if (!task->recv_blocked || !task->send_blocked)
     {
-        status = lwmsg_assoc_finish(task->assoc, NULL);
+        switch (task->type)
+        {
+        case PEER_TASK_FINISH_CLOSE:
+        case PEER_TASK_FINISH_RESET:
+            /* We are going to close the fd, so unset it on task */
+            if (fd >= 0)
+            {
+                lwmsg_task_unset_trigger_fd(task->event_task, fd);
+            }
+            break;
+        default:
+            break;
+        }
 
+        status = lwmsg_assoc_finish(task->assoc, NULL);
+        
         switch (status)
         {
         case LWMSG_STATUS_SUCCESS:
@@ -1491,6 +1553,19 @@ lwmsg_peer_task_run_finish(
             }
             break;
         case LWMSG_STATUS_PENDING:
+            switch (task->type)
+            {
+            case PEER_TASK_FINISH_CLOSE:
+            case PEER_TASK_FINISH_RESET:
+                /* Nevermind, set the fd on the task again */
+                if (fd >= 0)
+                {
+                    BAIL_ON_ERROR(status = lwmsg_task_set_trigger_fd(task->event_task, fd));
+                }
+                break;
+            default:
+                break;
+            }
             break;
         default:
             BAIL_ON_ERROR(status = lwmsg_peer_task_handle_assoc_error(
@@ -1515,15 +1590,14 @@ static
 LWMsgStatus
 lwmsg_peer_task_finish_transceive(
     LWMsgPeer* peer,
-    PeerAssocTask* task,
-    LWMsgTaskTrigger* trigger
+    PeerAssocTask* task
     )
 {
     LWMsgStatus status = LWMSG_STATUS_PENDING;
     LWMsgMessage* message = NULL;
 
-    while ((*trigger & LWMSG_TASK_TRIGGER_FD_READABLE && task->incoming) ||
-           (*trigger & LWMSG_TASK_TRIGGER_FD_WRITABLE && task->outgoing))
+    while ((!task->recv_blocked && task->incoming) ||
+           (!task->send_blocked && task->outgoing))
     {
         status = lwmsg_assoc_finish(task->assoc, &message);
         switch (status)
@@ -1532,6 +1606,7 @@ lwmsg_peer_task_finish_transceive(
             if (message == &task->incoming_message)
             {
                 task->incoming = LWMSG_FALSE;
+                task->recv_partial = LWMSG_FALSE;
                 BAIL_ON_ERROR(status = lwmsg_peer_task_dispatch_incoming_message(task));
             }
             else if (message == &task->outgoing_message)
@@ -1546,13 +1621,13 @@ lwmsg_peer_task_finish_transceive(
             }
             break;
         case LWMSG_STATUS_PENDING:
-            *trigger &= ~(LWMSG_TASK_TRIGGER_FD_READABLE | LWMSG_TASK_TRIGGER_FD_WRITABLE);
+            lwmsg_peer_task_update_blocked(task);
             break;
         default:
             BAIL_ON_ERROR(status);
         }
     }
-
+    
 error:
 
     return status;
@@ -1584,13 +1659,13 @@ lwmsg_peer_task_dispatch_calls(
         if (!(call->state & PEER_CALL_DISPATCHED))
         {
             call->state |= PEER_CALL_DISPATCHED;
-
+            
             lwmsg_message_init(&task->outgoing_message);
             task->outgoing_message.tag = call->params.outgoing.in->tag;
             task->outgoing_message.data = call->params.outgoing.in->data;
             task->outgoing_message.cookie = call->cookie;
             task->outgoing_message.status = call->status;
-
+            
             lwmsg_peer_log_message(task->peer, task->assoc, &task->outgoing_message, LWMSG_TRUE);
             status = lwmsg_assoc_send_message(task->assoc, &task->outgoing_message);
             switch (status)
@@ -1601,6 +1676,16 @@ lwmsg_peer_task_dispatch_calls(
                 task->outgoing = LWMSG_TRUE;
                 BAIL_ON_ERROR(status);
                 break;
+            case LWMSG_STATUS_MALFORMED:
+            case LWMSG_STATUS_INVALID_HANDLE:
+                /* The association should still be usable, so just fail the call */
+                lwmsg_message_init(&message);
+                message.status = status;
+                message.flags = LWMSG_MESSAGE_FLAG_REPLY | LWMSG_MESSAGE_FLAG_SYNTHETIC;
+                message.cookie = call->cookie;
+                lwmsg_peer_call_complete_outgoing(call, &message);
+                status = LWMSG_STATUS_SUCCESS;
+                break;
             default:
                 BAIL_ON_ERROR(status);
             }
@@ -1611,12 +1696,12 @@ lwmsg_peer_task_dispatch_calls(
                  call->status != LWMSG_STATUS_CANCELLED)
         {
             call->status = LWMSG_STATUS_CANCELLED;
-
+            
             lwmsg_message_init(&task->outgoing_message);
             task->outgoing_message.flags = LWMSG_MESSAGE_FLAG_CONTROL;
             task->outgoing_message.status = LWMSG_STATUS_CANCELLED;
             task->outgoing_message.cookie = call->cookie;
-
+            
             lwmsg_peer_log_message(task->peer, task->assoc, &task->outgoing_message, LWMSG_TRUE);
             status = lwmsg_assoc_send_message(task->assoc, &task->outgoing_message);
             switch (status)
@@ -1646,24 +1731,33 @@ lwmsg_peer_task_dispatch_calls(
         if ((call->state & PEER_CALL_COMPLETED) &&
             (call->state & PEER_CALL_DISPATCHED))
         {
+            /* Trace call completion */
+            if (call->task->peer->trace_end)
+            {
+                call->task->peer->trace_end(
+                    LWMSG_CALL(call),
+                    &call->params.incoming.out,
+                    call->task->peer->trace_data);
+            }
+
             lwmsg_message_init(&task->outgoing_message);
             task->outgoing_message.flags = LWMSG_MESSAGE_FLAG_REPLY;
             task->outgoing_message.tag = call->params.incoming.out.tag;
             task->outgoing_message.data = call->params.incoming.out.data;
             task->outgoing_message.cookie = call->cookie;
             task->outgoing_message.status = call->status;
-
+            
             cookie = call->cookie;
-
+            
             /* It's now safe to destroy the incoming call parameters */
             message.tag = call->params.incoming.in.tag;
             message.data = call->params.incoming.in.data;
             lwmsg_assoc_destroy_message(task->assoc, &message);
-
+            
             /* Delete the call structure */
             lwmsg_hash_remove_entry(&task->incoming_calls, call);
             lwmsg_peer_call_delete(call);
-
+            
             do
             {
                 status = lwmsg_assoc_send_message(task->assoc, &task->outgoing_message);
@@ -1707,7 +1801,7 @@ lwmsg_peer_task_dispatch_calls(
 error:
 
     return status;
-}
+}   
 
 static
 LWMsgStatus
@@ -1745,7 +1839,7 @@ lwmsg_peer_task_run_dispatch(
     else
     {
         /* Try to finish any outstanding sends/receives */
-        finish_status = lwmsg_peer_task_finish_transceive(peer, task, trigger);
+        finish_status = lwmsg_peer_task_finish_transceive(peer, task);
         switch (finish_status)
         {
         case LWMSG_STATUS_SUCCESS:
@@ -1756,13 +1850,14 @@ lwmsg_peer_task_run_dispatch(
         }
 
         /* Try to receive and dispatch the next message */
-        if (!task->incoming && *trigger & LWMSG_TASK_TRIGGER_FD_READABLE)
+        if (!task->incoming && !task->recv_blocked)
         {
             recv_status = lwmsg_assoc_recv_message(task->assoc, &task->incoming_message);
-
+            
             switch (recv_status)
             {
             case LWMSG_STATUS_SUCCESS:
+                task->recv_partial = LWMSG_FALSE;
                 BAIL_ON_ERROR(status = lwmsg_peer_task_dispatch_incoming_message(task));
                 break;
             case LWMSG_STATUS_PENDING:
@@ -1774,8 +1869,8 @@ lwmsg_peer_task_run_dispatch(
         }
 
         /* Try to send any outgoing call requests or incoming call replies */
-        if (!task->outgoing)
-        {
+        if (!task->outgoing && !task->send_blocked)
+        {           
             send_status = lwmsg_peer_task_dispatch_calls(task);
             switch (send_status)
             {
@@ -1794,7 +1889,7 @@ lwmsg_peer_task_run_dispatch(
             send_status == LWMSG_STATUS_PENDING &&
             recv_status == LWMSG_STATUS_PENDING)
         {
-            if (!task->incoming && !task->outgoing)
+            if ((!task->incoming || !task->recv_partial) && !task->outgoing)
             {
                 timeout = &peer->timeout.idle;
             }
@@ -1823,7 +1918,7 @@ error:
             status);
         break;
     }
-
+    
     return status;
 }
 
@@ -1838,9 +1933,15 @@ lwmsg_peer_task_run_drop(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    int fd = CONNECTION_PRIVATE(task->assoc)->fd;
 
     BAIL_ON_ERROR(status = lwmsg_peer_task_rundown(peer, task, trigger, next_trigger, next_timeout));
 
+    if (fd >= 0)
+    {
+        lwmsg_task_unset_trigger_fd(task->event_task, fd);
+    }
+
     lwmsg_peer_task_unref(task);
     BAIL_ON_ERROR(status = LWMSG_STATUS_CANCELLED);
 
@@ -1850,90 +1951,127 @@ error:
 }
 
 static
-LWMsgStatus
+void
 lwmsg_peer_task_run(
+    LWMsgTask* _task,
     void* data,
     LWMsgTaskTrigger trigger,
     LWMsgTaskTrigger* next_trigger,
-    LWMsgTime* next_timeout
+    LWMsgTaskTime* next_timeout
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerAssocTask* task = (PeerAssocTask*) data;
     LWMsgPeer* peer = task->peer;
+    LWMsgTime my_timeout = {-1, -1};
 
-    while (status == LWMSG_STATUS_SUCCESS)
+    if (*next_timeout >= 0)
     {
-        switch (task->type)
-        {
-        case PEER_TASK_BEGIN_ACCEPT:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_accept(
-                              peer,
-                              task,
-                              trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        case PEER_TASK_BEGIN_CONNECT:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_connect(
-                              peer,
-                              task,
-                              trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        case PEER_TASK_DISPATCH:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_dispatch(
-                              peer,
-                              task,
-                              &trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        case PEER_TASK_BEGIN_CLOSE:
-        case PEER_TASK_BEGIN_RESET:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_shutdown(
-                              peer,
-                              task,
-                              trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        case PEER_TASK_FINISH_ACCEPT:
-        case PEER_TASK_FINISH_CONNECT:
-        case PEER_TASK_FINISH_CLOSE:
-        case PEER_TASK_FINISH_RESET:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_finish(
-                              peer,
-                              task,
-                              trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        case PEER_TASK_DROP:
-            BAIL_ON_ERROR(status = lwmsg_peer_task_run_drop(
-                              peer,
-                              task,
-                              trigger,
-                              next_trigger,
-                              next_timeout));
-            break;
-        default:
-            break;
-        }
+        my_timeout.seconds = *next_timeout / 1000000000ll;
+        my_timeout.microseconds = (*next_timeout % 1000000000ll) / 1000;
+    }
+
+    if (trigger & LWMSG_TASK_TRIGGER_FD_READABLE)
+    {
+        task->recv_blocked = LWMSG_FALSE;
+        task->recv_partial = LWMSG_TRUE;
+    }
+
+    if (trigger & LWMSG_TASK_TRIGGER_FD_WRITABLE)
+    {
+        task->send_blocked = LWMSG_FALSE;
+    }
+
+    *next_trigger = LWMSG_TASK_TRIGGER_EXPLICIT;
+
+    switch (task->type)
+    {
+    case PEER_TASK_BEGIN_ACCEPT:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_accept(
+            peer,
+            task,
+            trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    case PEER_TASK_BEGIN_CONNECT:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_connect(
+            peer,
+            task,
+            trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    case PEER_TASK_DISPATCH:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_dispatch(
+            peer,
+            task,
+            &trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    case PEER_TASK_BEGIN_CLOSE:
+    case PEER_TASK_BEGIN_RESET:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_shutdown(
+            peer,
+            task,
+            trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    case PEER_TASK_FINISH_ACCEPT:
+    case PEER_TASK_FINISH_CONNECT:
+    case PEER_TASK_FINISH_CLOSE:
+    case PEER_TASK_FINISH_RESET:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_finish(
+            peer,
+            task,
+            trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    case PEER_TASK_DROP:
+        BAIL_ON_ERROR(status = lwmsg_peer_task_run_drop(
+            peer,
+            task,
+            trigger,
+            next_trigger,
+            &my_timeout));
+        break;
+    default:
+        break;
     }
 
 error:
 
     switch (status)
     {
+    case LWMSG_STATUS_SUCCESS:
+    case LWMSG_STATUS_PENDING:
+        lwmsg_peer_task_update_blocked(task);
+
+        if (task->send_blocked)
+        {
+            *next_trigger |= LWMSG_TASK_TRIGGER_FD_WRITABLE;
+        }
+
+        if (task->recv_blocked)
+        {
+            *next_trigger |= LWMSG_TASK_TRIGGER_FD_READABLE;
+        }
+
+        if (status == LWMSG_STATUS_SUCCESS)
+        {
+            *next_trigger |= LWMSG_TASK_TRIGGER_YIELD;
+        }
+        break;
     case LWMSG_STATUS_CANCELLED:
         *next_trigger = 0;
-        break;
-    case LWMSG_STATUS_PENDING:
+        break;            
+
         break;
     default:
-        LWMSG_LOG_ERROR(peer->context, "Caught error: %s", lwmsg_error_name(status));
+        LWMSG_LOG_ERROR(peer->context, "Caught error: %s (%i)", lwmsg_error_name(status), status);
         if (peer->except)
         {
             peer->except(peer, status, peer->except_data);
@@ -1941,7 +2079,7 @@ error:
         break;
     }
 
-    /* Don't cause the task manager to bail out
-       under any circumstances */
-    return LWMSG_STATUS_SUCCESS;
+    *next_timeout = lwmsg_time_is_positive(&my_timeout) ?
+        my_timeout.seconds * 1000000000ll + my_timeout.microseconds * 1000
+        : 0;
 }
