@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -91,10 +91,11 @@ lwmsg_peer_new(
     peer->context = context;
 
     BAIL_ON_ERROR(status = lwmsg_error_map_errno(pthread_mutex_init(&peer->lock, NULL)));
+    peer->lock_init = LWMSG_TRUE;
     BAIL_ON_ERROR(status = lwmsg_error_map_errno(pthread_cond_init(&peer->event, NULL)));
+    peer->event_init = LWMSG_TRUE;
 
     BAIL_ON_ERROR(status = lwmsg_task_acquire_manager(&peer->task_manager));
-    BAIL_ON_ERROR(status = lwmsg_task_group_new(peer->task_manager, &peer->listen_tasks));
     BAIL_ON_ERROR(status = lwmsg_task_group_new(peer->task_manager, &peer->connect_tasks));
 
     peer->max_clients = 100;
@@ -103,9 +104,18 @@ lwmsg_peer_new(
 
     *out_peer = peer;
 
-error:
+done:
 
     return status;
+
+error:
+
+    if (peer)
+    {
+        lwmsg_peer_delete(peer);
+    }
+
+    goto done;
 }
 
 static
@@ -172,9 +182,16 @@ lwmsg_peer_delete(
     {
         lwmsg_session_manager_delete(peer->session_manager);
     }
+    
+    if (peer->lock_init)
+    {
+        pthread_mutex_destroy(&peer->lock);
+    }
 
-    pthread_mutex_destroy(&peer->lock);
-    pthread_cond_destroy(&peer->event);
+    if (peer->event_init)
+    {
+        pthread_cond_destroy(&peer->event);
+    }
 
     if (peer->dispatch.vector)
     {
@@ -210,7 +227,7 @@ lwmsg_peer_set_timeout(
         PEER_RAISE_ERROR(peer, status = LWMSG_STATUS_INVALID_PARAMETER,
                            "Invalid (negative) timeout value");
     }
-
+    
     switch (type)
     {
     case LWMSG_TIMEOUT_MESSAGE:
@@ -311,7 +328,7 @@ lwmsg_peer_add_dispatch_spec(
             max_message_tag = table[i].tag;
         }
     }
-
+    
     if (peer->dispatch.vector_length < max_message_tag + 1)
     {
         new_vector = realloc(peer->dispatch.vector, sizeof(*new_vector) * (max_message_tag + 1));
@@ -320,9 +337,9 @@ lwmsg_peer_add_dispatch_spec(
         {
             BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
         }
-
+        
         /* Zero out new elements of vector */
-        memset(new_vector + peer->dispatch.vector_length, 0,
+        memset(new_vector + peer->dispatch.vector_length, 0, 
                (max_message_tag + 1 - peer->dispatch.vector_length) * sizeof(*new_vector));
 
         peer->dispatch.vector_length = max_message_tag + 1;
@@ -354,7 +371,7 @@ lwmsg_peer_add_listen_fd(
 
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&peer_endpoint));
     lwmsg_ring_init(&peer_endpoint->ring);
-
+    
     peer_endpoint->type = type;
     peer_endpoint->fd = fd;
 
@@ -397,7 +414,7 @@ lwmsg_peer_add_listen_endpoint(
 
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&peer_endpoint));
     lwmsg_ring_init(&peer_endpoint->ring);
-
+    
     peer_endpoint->type = type;
     peer_endpoint->endpoint = strdup(endpoint);
     peer_endpoint->permissions = permissions;
@@ -454,6 +471,8 @@ lwmsg_peer_startup(
                           &peer->session_manager));
     }
 
+    BAIL_ON_ERROR(status = lwmsg_task_group_new(peer->task_manager, &peer->listen_tasks));
+
     for (ring = peer->listen_endpoints.next; ring != &peer->listen_endpoints; ring = ring->next)
     {
         endpoint = LWMSG_OBJECT_FROM_MEMBER(ring, PeerEndpoint, ring);
@@ -475,7 +494,7 @@ lwmsg_peer_startup(
 
     if (lwmsg_context_would_log(peer->context, LWMSG_LOGLEVEL_TRACE))
     {
-        BAIL_ON_ERROR(status = lwmsg_protocol_print_alloc(peer->protocol, &message));
+        BAIL_ON_ERROR(status = lwmsg_protocol_print_alloc(peer->protocol, 4, &message));
         LWMSG_LOG_TRACE(peer->context, "Listen protocol:\n%s", message);
     }
 
@@ -490,7 +509,13 @@ done:
 
 error:
 
-    lwmsg_task_group_cancel(peer->listen_tasks);
+    if (peer->listen_tasks)
+    {
+        lwmsg_task_group_cancel(peer->listen_tasks);
+        lwmsg_task_group_wait(peer->listen_tasks);
+        lwmsg_task_group_delete(peer->listen_tasks);
+        peer->listen_tasks = NULL;
+    }
 
     goto done;
 }
@@ -509,7 +534,7 @@ lwmsg_peer_start_listen(
     {
         peer->state = PEER_STATE_STARTING;
         PEER_UNLOCK(peer, locked);
-
+        
         BAIL_ON_ERROR(status = lwmsg_peer_startup(peer));
 
         PEER_LOCK(peer, locked);
@@ -524,7 +549,7 @@ lwmsg_peer_start_listen(
             pthread_cond_wait(&peer->event, &peer->lock);
         }
     }
-
+    
     if (peer->state == PEER_STATE_ERROR)
     {
         BAIL_ON_ERROR(status = peer->status);
@@ -558,6 +583,8 @@ lwmsg_peer_shutdown(
 
     lwmsg_task_group_cancel(peer->listen_tasks);
     lwmsg_task_group_wait(peer->listen_tasks);
+    lwmsg_task_group_delete(peer->listen_tasks);
+    peer->listen_tasks = NULL;
 
     LWMSG_LOG_INFO(peer->context, "Listener shut down");
 
@@ -745,6 +772,33 @@ lwmsg_peer_set_exception_function(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    
+    lwmsg_peer_lock(peer);
+
+    if (peer->state != PEER_STATE_STOPPED)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_STATE);
+    }
+    
+    peer->except = except;
+    peer->except_data = except_data;
+
+error:
+
+    lwmsg_peer_unlock(peer);
+
+    return status;
+}
+
+LWMsgStatus
+lwmsg_peer_set_trace_functions(
+    LWMsgPeer* peer,
+    LWMsgPeerTraceFunction begin,
+    LWMsgPeerTraceFunction end,
+    void* data
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
     lwmsg_peer_lock(peer);
 
@@ -753,8 +807,9 @@ lwmsg_peer_set_exception_function(
         BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_STATE);
     }
 
-    peer->except = except;
-    peer->except_data = except_data;
+    peer->trace_begin = begin;
+    peer->trace_end = end;
+    peer->trace_data = data;
 
 error:
 
@@ -776,7 +831,7 @@ lwmsg_peer_add_connect_endpoint(
 
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&peer_endpoint));
     lwmsg_ring_init(&peer_endpoint->ring);
-
+    
     peer_endpoint->type = type;
     peer_endpoint->endpoint = strdup(endpoint);
 
@@ -892,7 +947,7 @@ lwmsg_peer_try_connect(
                           peer->session_construct_data,
                           &peer->session_manager));
     }
-
+    
     if (!peer->connect_session)
     {
         BAIL_ON_ERROR(status = lwmsg_session_create(
@@ -991,7 +1046,7 @@ lwmsg_peer_connect(
     LWMsgBool locked = LWMSG_FALSE;
 
     BAIL_ON_ERROR(status = lwmsg_peer_connect_in_lock(peer, &locked));
-
+    
     if (session)
     {
         *session = peer->connect_session;
@@ -1077,12 +1132,12 @@ lwmsg_peer_acquire_call(
         task = peer->connect_task;
         task->refcount++;
         PEER_UNLOCK(peer, locked);
-
+        
         /* Check that the connect task is in a good state */
         pthread_mutex_lock(&task->call_lock);
         status = task->status;
         pthread_mutex_unlock(&task->call_lock);
-
+        
         switch (status)
         {
         case LWMSG_STATUS_PEER_CLOSE:
@@ -1100,8 +1155,9 @@ lwmsg_peer_acquire_call(
             BAIL_ON_ERROR(status);
         }
     } while (status == LWMSG_STATUS_AGAIN);
-
+            
     BAIL_ON_ERROR(status = lwmsg_peer_call_new(task, &pcall));
+    pcall->base.is_outgoing = LWMSG_TRUE;
     task = NULL;
 
     *call = LWMSG_CALL(pcall);
