@@ -36,6 +36,7 @@
  *
  */
 
+#include "config.h"
 #include "protocol-private.h"
 #include "util-private.h"
 #include "type-private.h"
@@ -107,7 +108,7 @@ void
 lwmsg_protocol_delete(LWMsgProtocol* prot)
 {
     lwmsg_memlist_destroy(&prot->specmem);
-
+    
     free(prot->types);
     free(prot);
 }
@@ -189,7 +190,7 @@ lwmsg_protocol_add_protocol_spec(LWMsgProtocol* prot, LWMsgProtocolSpec* spec)
         prot->types = new_types;
         prot->num_types = num_types;
     }
-
+    
     for (i = 0; spec[i].tag != -1; i++)
     {
         /* A NULL typespec indicates a message with an empty payload */
@@ -302,7 +303,7 @@ lwmsg_protocol_add_protocol_rep(
                               &specmap,
                               rep->messages[i].type,
                               &buffer));
-
+            
             spec[next].tag = rep->messages[i].tag;
             spec[next].type = buffer->buffer;
             BAIL_ON_ERROR(status = lwmsg_strdup(
@@ -315,7 +316,7 @@ lwmsg_protocol_add_protocol_rep(
         {
             BAIL_ON_ERROR(status);
             /* Known tag -- check for assignability with existing type */
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_rep_from_spec(
                               prot->context,
                               existing_type,
@@ -335,7 +336,7 @@ lwmsg_protocol_add_protocol_rep(
     BAIL_ON_ERROR(status = lwmsg_protocol_add_protocol_spec(
                           prot,
                           spec));
-
+    
 error:
 
     lwmsg_type_spec_map_destroy(&specmap);
@@ -356,14 +357,10 @@ lwmsg_protocol_is_protocol_rep_compatible(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    LWMsgTypeSpecMap specmap = {0};
-    struct LWMsgProtocolSpec *spec = NULL;
     LWMsgTypeSpec* existing_type = NULL;
     LWMsgTypeRep* existing_rep = NULL;
     size_t i = 0;
 
-    BAIL_ON_ERROR(status = LWMSG_CONTEXT_ALLOC_ARRAY(specmap.context, rep->message_count + 1, &spec));
-
     for (i = 0; i < rep->message_count; i++)
     {
         status = lwmsg_protocol_get_message_type(prot, rep->messages[i].tag, &existing_type);
@@ -375,7 +372,7 @@ lwmsg_protocol_is_protocol_rep_compatible(
         {
             BAIL_ON_ERROR(status);
             /* Known tag -- check for assignability with existing type */
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_rep_from_spec(
                               prot->context,
                               existing_type,
@@ -387,7 +384,7 @@ lwmsg_protocol_is_protocol_rep_compatible(
             existing_rep = NULL;
         }
     }
-
+    
 error:
 
     if (existing_rep)
@@ -425,7 +422,7 @@ lwmsg_protocol_print(
 
     for (i = 0; i < rep->message_count; i++)
     {
-        for (j = 0; i < indent; j++)
+        for (j = 0; j < indent; j++)
         {
             BAIL_ON_ERROR(status = lwmsg_buffer_print(buffer, " "));
         }
@@ -436,7 +433,7 @@ lwmsg_protocol_print(
             {
                 BAIL_ON_ERROR(status = lwmsg_buffer_print(
                                   buffer,
-                                  "Tag %s (%i):\n",
+                                  "%s (%i): ",
                                   rep->messages[i].name,
                                   rep->messages[i].tag));
             }
@@ -444,10 +441,10 @@ lwmsg_protocol_print(
             {
                 BAIL_ON_ERROR(status = lwmsg_buffer_print(
                                   buffer,
-                                  "Tag %i:\n",
+                                  "%i: ",
                                   rep->messages[i].tag));
             }
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_print_rep(
                               rep->messages[i].type,
                               indent + 4,
@@ -459,7 +456,7 @@ lwmsg_protocol_print(
             {
                 BAIL_ON_ERROR(status = lwmsg_buffer_print(
                                   buffer,
-                                  "Tag %s (%i)",
+                                  "%s (%i)",
                                   rep->messages[i].name,
                                   rep->messages[i].tag));
             }
@@ -467,12 +464,15 @@ lwmsg_protocol_print(
             {
                 BAIL_ON_ERROR(status = lwmsg_buffer_print(
                                   buffer,
-                                  "Tag %i",
+                                  "%i",
                                   rep->messages[i].tag));
-            }
+            }  
+        }
+        
+        if (i < rep->message_count - 1)
+        {
+            BAIL_ON_ERROR(status = lwmsg_buffer_print(buffer, "\n"));
         }
-
-        BAIL_ON_ERROR(status = lwmsg_buffer_print(buffer, "\n\n"));
     }
 
 cleanup:
@@ -513,14 +513,14 @@ realloc_wrap(
         {
             new_length = length * 2;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_context_realloc(
                           context,
                           buffer->base,
                           length,
                           new_length,
                           (void**) (void*) &new_buffer));
-
+        
         buffer->base = new_buffer;
         buffer->end = new_buffer + new_length;
         buffer->cursor = new_buffer + offset;
@@ -534,6 +534,7 @@ error:
 LWMsgStatus
 lwmsg_protocol_print_alloc(
     LWMsgProtocol* prot,
+    unsigned int indent,
     char** text
     )
 {
@@ -546,7 +547,7 @@ lwmsg_protocol_print_alloc(
 
     BAIL_ON_ERROR(status = lwmsg_protocol_print(
                       prot,
-                      0,
+                      indent,
                       &buffer));
 
     BAIL_ON_ERROR(status = lwmsg_buffer_write(&buffer, &nul, 1));
