@@ -134,7 +134,7 @@ newline(
                       info->buffer,
                       nl,
                       sizeof(nl)));
-
+    
     info->newline = LWMSG_TRUE;
 
 error:
@@ -142,6 +142,22 @@ error:
     return status;
 }
 
+static
+LWMsgStatus
+newline_if(
+    PrintInfo* info
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    if (!info->newline)
+    {
+        status = newline(info);
+    }
+
+    return status;
+}
+
 static
 LWMsgStatus
 print_type_name(
@@ -286,6 +302,10 @@ lwmsg_type_print_enum(
     size_t i = 0;
     LWMsgVariantRep* var = NULL;
 
+    if (!rep->info.enum_rep.definition->seen)
+    {
+        BAIL_ON_ERROR(status = newline_if(info));
+    }
     BAIL_ON_ERROR(status = print(
                       info, "%sint%u enum ",
                       rep->info.enum_rep.definition->sign == LWMSG_UNSIGNED ? "u" : "",
@@ -376,6 +396,10 @@ lwmsg_type_print_struct(
     LWMsgTypeRep* old_rep = NULL;
 
 
+    if (!rep->info.struct_rep.definition->seen)
+    {
+        BAIL_ON_ERROR(status = newline_if(info));
+    }
     BAIL_ON_ERROR(status = print(info, "struct "));
     BAIL_ON_ERROR(status = print_type_name(info, rep));
 
@@ -424,6 +448,10 @@ lwmsg_type_print_union(
     size_t i = 0;
     LWMsgTypeRep* discrim = NULL;
 
+    if (!rep->info.union_rep.definition->seen)
+    {
+        BAIL_ON_ERROR(status = newline_if(info));
+    }
     BAIL_ON_ERROR(status = print(info, "union "));
     BAIL_ON_ERROR(status = print_type_name(info, rep));
 
@@ -660,18 +688,18 @@ lwmsg_type_print_rep(
 
     memset(&info, 0, sizeof(info));
 
-    info.newline = LWMSG_TRUE;
+    info.newline = LWMSG_FALSE;
     info.depth = indent;
     info.buffer = buffer;
 
     BAIL_ON_ERROR(status = lwmsg_type_print_internal(
                       rep,
                       &info));
-
+    
 error:
-
+    
     lwmsg_data_object_map_destroy(&info.map);
-
+    
     return status;
 }
 
@@ -699,14 +727,14 @@ realloc_wrap(
         {
             new_length = length * 2;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_context_realloc(
                           context,
                           buffer->base,
                           length,
                           new_length,
                           (void**) (void*) &new_buffer));
-
+        
         buffer->base = new_buffer;
         buffer->end = new_buffer + new_length;
         buffer->cursor = new_buffer + offset;
@@ -731,21 +759,21 @@ lwmsg_type_print_spec_alloc(
 
     buffer.wrap = realloc_wrap;
     buffer.data = (void*) context;
-
+    
     BAIL_ON_ERROR(status = lwmsg_type_rep_from_spec(NULL, type, &rep));
 
     BAIL_ON_ERROR(status = lwmsg_type_print_rep(rep, 0, &buffer));
-
+    
     BAIL_ON_ERROR(status = lwmsg_buffer_write(&buffer, &nul, 1));
 
     *result = (char*) buffer.base;
-
+    
 cleanup:
 
     lwmsg_type_free_rep(NULL, rep);
-
+    
     return status;
-
+    
 error:
 
     *result = NULL;
