@@ -36,6 +36,7 @@
  *
  */
 
+#include "config.h"
 #include "type-private.h"
 #include "util-private.h"
 #include "data-private.h"
@@ -369,10 +370,11 @@ lwmsg_type_rep_map_find_spec(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     struct SpecKey key = {kind, spec};
+    LWMsgTypeRepMapEntry* entry = NULL;
 
     BAIL_ON_ERROR(status = lwmsg_type_rep_map_init(map));
 
-    LWMsgTypeRepMapEntry* entry = lwmsg_hash_find_key(&map->hash_by_spec, &key);
+    entry = lwmsg_hash_find_key(&map->hash_by_spec, &key);
 
     if (!entry)
     {
@@ -405,10 +407,7 @@ lwmsg_type_rep_map_insert(
     entry->spec.spec = spec;
     entry->rep = rep;
 
-    if (spec)
-    {
-        lwmsg_hash_insert_entry(&map->hash_by_spec, entry);
-    }
+    lwmsg_hash_insert_entry(&map->hash_by_spec, entry);
 
 done:
 
@@ -861,9 +860,9 @@ lwmsg_type_rep_from_spec_internal(
     if (iter->kind == LWMSG_KIND_CUSTOM)
     {
         LWMsgTypeIter transmit_iter;
-
+        
         lwmsg_type_iterate(iter->info.kind_custom.typeclass->transmit_type, &transmit_iter);
-
+        
         return lwmsg_type_rep_from_spec_internal(map, &transmit_iter, rep);
     }
 
@@ -1032,16 +1031,17 @@ lwmsg_type_spec_map_find(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgTypeSpecBuffer* entry = NULL;
 
     BAIL_ON_ERROR(status = lwmsg_type_spec_map_init(map));
-
-    LWMsgTypeSpecBuffer* entry = lwmsg_hash_find_key(&map->hash_by_rep, rep);
+    
+    entry = lwmsg_hash_find_key(&map->hash_by_rep, rep);
 
     if (!entry)
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
     }
-
+    
     *buffer = entry;
 
 error:
@@ -1076,7 +1076,7 @@ lwmsg_type_spec_map_destroy(
         while ((entry = lwmsg_hash_iter_next(&map->hash_by_rep, &iter)))
         {
             lwmsg_hash_remove_entry(&map->hash_by_rep, entry);
-
+            
             for (ring = entry->backlinks.next;
                  ring != &entry->backlinks;
                  ring = next)
@@ -1116,7 +1116,7 @@ lwmsg_type_spec_buffer_new(
 
     my_buffer->rep = rep;
     my_buffer->buffer_capacity = 8;
-
+    
     BAIL_ON_ERROR(status = LWMSG_CONTEXT_ALLOC_ARRAY(
                       map->context,
                       my_buffer->buffer_capacity,
@@ -1247,12 +1247,12 @@ lwmsg_type_spec_buffer_link(
     if (to->buffer_capacity)
     {
         BAIL_ON_ERROR(status = LWMSG_ALLOC(&link));
-
+        
         lwmsg_ring_init(&link->ring);
-
+        
         link->buffer = from;
         link->offset = from->buffer_size;
-
+        
         lwmsg_ring_enqueue(&to->backlinks, &link->ring);
 
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
@@ -1353,7 +1353,7 @@ lwmsg_type_spec_from_spec(
     size_t cmd = 0;
 
     cmd = LWMSG_CMD_TYPESPEC;
-
+    
     if (state->member_offset >= 0)
     {
         cmd |= LWMSG_FLAG_MEMBER;
@@ -1402,7 +1402,7 @@ lwmsg_type_spec_from_spec(
     buffer->type_size = spec->type_size;
 
 error:
-
+    
     return status;
 }
 
@@ -1464,7 +1464,7 @@ lwmsg_type_spec_from_integer(
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           buffer,
-                          rep->info.integer_rep.definition->width));
+                          rep->info.integer_rep.definition->width)); 
     }
 
     spec[0] = rep->info.integer_rep.definition->width;
@@ -1490,9 +1490,9 @@ lwmsg_type_spec_from_integer(
     }
 
     buffer->type_size = rep->info.integer_rep.definition->width;
-
+  
 error:
-
+    
     return status;
 }
 
@@ -1524,21 +1524,21 @@ lwmsg_type_spec_from_enum(
                           state->map,
                           def,
                           &defbuf));
-
+        
         lwmsg_type_spec_map_insert(state->map, defbuf);
-
+        
         cmd = LWMSG_CMD_ENUM;
-
+        
         if (def->name)
         {
             cmd |= LWMSG_FLAG_META;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
                           cmd));
-
+        
         if (cmd & LWMSG_FLAG_META)
         {
             BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_string(
@@ -1546,7 +1546,7 @@ lwmsg_type_spec_from_enum(
                               defbuf,
                               def->name));
         }
-
+        
         spec[0] = def->width;
         spec[1] = def->width;
         spec[2] = def->sign;
@@ -1560,17 +1560,17 @@ lwmsg_type_spec_from_enum(
         for (i = 0; i < def->variant_count; i++)
         {
             cmd = def->variants[i].is_mask ? LWMSG_CMD_ENUM_MASK : LWMSG_CMD_ENUM_VALUE;
-
+            
             if (def->variants[i].name)
             {
                 cmd |= LWMSG_FLAG_META;
             }
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                               state->map,
                               defbuf,
                               cmd));
-
+            
             if (cmd & LWMSG_FLAG_META)
             {
                 BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_string(
@@ -1584,7 +1584,7 @@ lwmsg_type_spec_from_enum(
                               defbuf,
                               (size_t) def->variants[i].value));
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
@@ -1635,7 +1635,7 @@ lwmsg_type_spec_from_struct(
                           state->map,
                           def,
                           &defbuf));
-
+        
         lwmsg_type_spec_map_insert(state->map, defbuf);
 
         BAIL_ON_ERROR(status = LWMSG_ALLOC_ARRAY(def->field_count, &defbuf->member_metrics));
@@ -1644,19 +1644,19 @@ lwmsg_type_spec_from_struct(
         my_state.member_offset = 0;
         my_state.member_name = NULL;
         my_state.dominating_struct = defbuf;
-
+     
         cmd = LWMSG_CMD_STRUCT;
-
+        
         if (def->name)
         {
             cmd |= LWMSG_FLAG_META;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
                           cmd));
-
+        
         if (cmd & LWMSG_FLAG_META)
         {
             BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_string(
@@ -1673,21 +1673,21 @@ lwmsg_type_spec_from_struct(
                           state->map,
                           defbuf,
                           0));
-
+        
         for (i = 0; i < def->field_count; i++)
         {
             my_state.member_name = def->fields[i].name;
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_spec_from_rep_into(
                               &my_state,
                               def->fields[i].type,
                               defbuf));
-
+            
             defbuf->member_metrics[i].offset = my_state.member_offset;
             defbuf->member_metrics[i].size = my_state.member_size;
             my_state.member_offset += my_state.member_size;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
@@ -1747,21 +1747,21 @@ lwmsg_type_spec_from_union(
                           state->map,
                           def,
                           &defbuf));
-
+        
         lwmsg_type_spec_map_insert(state->map, defbuf);
-
+        
         cmd = LWMSG_CMD_UNION;
-
+        
         if (def->name)
         {
             cmd |= LWMSG_FLAG_META;
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
                           cmd));
-
+        
         if (cmd & LWMSG_FLAG_META)
         {
             BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_string(
@@ -1778,11 +1778,11 @@ lwmsg_type_spec_from_union(
                           state->map,
                           defbuf,
                           0));
-
+        
         for (i = 0; i < def->arm_count; i++)
         {
             my_state.member_name = def->arms[i].name;
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_spec_from_rep_into(
                               &my_state,
                               def->arms[i].type,
@@ -1795,14 +1795,14 @@ lwmsg_type_spec_from_union(
 
             spec[0] = LWMSG_CMD_TAG;
             spec[1] = def->arms[i].tag;
-
+            
             BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append(
                               state->map,
                               defbuf,
                               spec,
                               2));
         }
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                           state->map,
                           defbuf,
@@ -1824,7 +1824,7 @@ lwmsg_type_spec_from_union(
     spec[0] = LWMSG_CMD_DISCRIM;
     spec[1] = state->dominating_struct->member_metrics[rep->info.union_rep.discrim_member_index].offset;
     spec[2] = state->dominating_struct->member_metrics[rep->info.union_rep.discrim_member_index].size;
-
+    
     BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append(
                       state->map,
                       buffer,
@@ -1856,7 +1856,7 @@ lwmsg_type_spec_from_pointer(
     buffer->type_size = sizeof(void*);
 
     cmd = LWMSG_CMD_POINTER;
-
+    
     if (state->member_offset >= 0)
     {
         cmd |= LWMSG_FLAG_MEMBER;
@@ -1879,7 +1879,7 @@ lwmsg_type_spec_from_pointer(
                           buffer,
                           state->member_name));
     }
-
+    
     if (cmd & LWMSG_FLAG_MEMBER)
     {
         align_to_width(&state->member_offset, sizeof(void*));
@@ -1901,7 +1901,7 @@ lwmsg_type_spec_from_pointer(
                       &my_state,
                       pointeebuf,
                       buffer));
-
+    
     BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append_value(
                       state->map,
                       buffer,
@@ -1938,7 +1938,7 @@ lwmsg_type_spec_from_pointer(
         spec[1] = LWMSG_TERM_MEMBER;
         spec[2] = state->dominating_struct->member_metrics[rep->info.pointer_rep.length_member_index].offset;
         spec[3] = state->dominating_struct->member_metrics[rep->info.pointer_rep.length_member_index].size;
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append(
                           state->map,
                           buffer,
@@ -1980,7 +1980,7 @@ lwmsg_type_spec_from_array(
     size_t spec[4];
 
     cmd = LWMSG_CMD_ARRAY;
-
+    
     if (state->member_offset >= 0)
     {
         cmd |= LWMSG_FLAG_MEMBER;
@@ -2003,7 +2003,7 @@ lwmsg_type_spec_from_array(
                           buffer,
                           state->member_name));
     }
-
+    
     if (cmd & LWMSG_FLAG_MEMBER)
     {
         align_to_width(&state->member_offset, sizeof(void*));
@@ -2062,7 +2062,7 @@ lwmsg_type_spec_from_array(
         spec[1] = LWMSG_TERM_MEMBER;
         spec[2] = state->dominating_struct->member_metrics[rep->info.pointer_rep.length_member_index].offset;
         spec[3] = state->dominating_struct->member_metrics[rep->info.pointer_rep.length_member_index].size;
-
+        
         BAIL_ON_ERROR(status = lwmsg_type_spec_buffer_append(
                           state->map,
                           buffer,
@@ -2124,9 +2124,9 @@ lwmsg_type_spec_from_void(
     }
 
     buffer->type_size = 0;
-
+  
 error:
-
+    
     return status;
 }
 
@@ -2139,7 +2139,7 @@ lwmsg_type_spec_from_rep_into(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-
+    
     switch (rep->kind)
     {
     case LWMSG_KIND_INTEGER:
@@ -2237,10 +2237,10 @@ lwmsg_type_spec_from_rep_internal(
                           LWMSG_CMD_END));
 
         lwmsg_type_spec_buffer_finalize(map, entry);
-
+        
         *buffer = entry;
     }
-
+    
 error:
 
     return status;
@@ -2355,10 +2355,11 @@ lwmsg_type_assign_map_find(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     LWMsgTypeAssignPair pair = {left, right};
+    LWMsgTypeAssignEntry* entry = NULL;
 
     BAIL_ON_ERROR(status = lwmsg_type_assign_map_init(map));
-
-    LWMsgTypeAssignEntry* entry = lwmsg_hash_find_key(&map->hash, &pair);
+    
+    entry = lwmsg_hash_find_key(&map->hash, &pair);
 
     if (!entry)
     {
@@ -2380,7 +2381,7 @@ lwmsg_type_assign_map_insert(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     LWMsgTypeAssignEntry* entry = NULL;
-
+    
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&entry));
 
     lwmsg_ring_init(&entry->ring);
@@ -2403,7 +2404,7 @@ lwmsg_type_assign_map_remove(
     )
 {
     LWMsgTypeAssignPair pair = {left, right};
-
+    
     lwmsg_hash_remove_key(&map->hash, &pair);
 }
 
@@ -2421,7 +2422,7 @@ lwmsg_type_assign_map_destroy(
         lwmsg_hash_iter_begin(&map->hash, &iter);
         while ((entry = lwmsg_hash_iter_next(&map->hash, &iter)))
         {
-            lwmsg_hash_remove_entry(&map->hash, entry);
+            lwmsg_hash_remove_entry(&map->hash, entry);           
             free(entry);
         }
         lwmsg_hash_iter_end(&map->hash, &iter);
@@ -2627,7 +2628,7 @@ lwmsg_type_rep_is_assignable_pointer(
                       map,
                       left->info.pointer_rep.pointee_type,
                       right->info.pointer_rep.pointee_type));
-
+    
 error:
 
     return status;
@@ -2657,7 +2658,7 @@ lwmsg_type_rep_is_assignable_array(
                       map,
                       left->info.array_rep.element_type,
                       right->info.array_rep.element_type));
-
+    
 error:
 
     return status;
@@ -2677,7 +2678,7 @@ lwmsg_type_rep_is_assignable_internal(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
     }
-
+  
     if ((status = lwmsg_type_assign_map_find(map, left, right)) != LWMSG_STATUS_NOT_FOUND)
     {
         goto error;
@@ -2685,7 +2686,7 @@ lwmsg_type_rep_is_assignable_internal(
 
     /* Provisionally insert the pair into the map */
     BAIL_ON_ERROR(status = lwmsg_type_assign_map_insert(map, left, right));
-
+    
     switch (left->kind)
     {
     case LWMSG_KIND_INTEGER:
