@@ -117,15 +117,23 @@ char* lwmsg_formatv(const char* fmt, va_list ap)
         if (!str_new)
         {
             free(str);
+            va_end(my_ap);
             return NULL;
         }
 
         str = str_new;
 
         if (vsnprintf(str, len+1, fmt, my_ap) < len)
+        {
+            free(str);
+            va_end(my_ap);
             return NULL;
+        }
         else
+        {
+            va_end(my_ap);
             return str;
+        }
     }
 }
 
@@ -148,24 +156,46 @@ lwmsg_convert_string_alloc(
     const char* output_type
     )
 {
-    size_t cblen;
-    char *buffer;
+    ssize_t cblen = 0;
+    char *buffer = NULL;
+
     if (input == NULL)
-        return -1;
+    {
+        goto error;
+    }
+
     cblen = lwmsg_convert_string_buffer(input, input_len, NULL, 0, input_type, output_type);
+
+    if (cblen < 0)
+    {
+        goto error;
+    }
+
     buffer = malloc(cblen);
     if (buffer == NULL)
-        return -1;
+    {
+        goto error;
+    }
+
     if (lwmsg_convert_string_buffer(input, input_len, buffer, cblen, input_type, output_type) != cblen)
     {
-        free(buffer);
-        return -1;
+        goto error;
     }
-    else
+
+    *output = buffer;
+
+done:
+
+    return cblen;
+
+error:
+
+    cblen = -1;
+    if (buffer)
     {
-        *output = buffer;
-        return cblen;
+        free(buffer);
     }
+    goto done;
 }
 
 ssize_t
@@ -208,7 +238,7 @@ lwmsg_convert_string_buffer(
         return cblen;
     }
     converted = iconv(handle, (ICONV_IN_TYPE) &inbuf, &cbin, &outbuf, &cbout);
-
+    
     iconv_close(handle);
     if(converted == (size_t)-1 && cbout != 0)
         return -1;
@@ -230,7 +260,7 @@ lwmsg_add_unsigned(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_OVERFLOW);
     }
-
+    
     *result = sum;
 
 error:
@@ -251,9 +281,9 @@ lwmsg_multiply_unsigned(
     )
 {
     /* Multiplication with overflow checking:
-
+       
        Operands: x and y
-
+       
        Let        r = 2^((8 * sizeof size_t) / 2)
        Let        x = ra + b
        Let        y = rc + d
@@ -322,9 +352,9 @@ lwmsg_multiply_unsigned(
 
     /* Never overflows */
     bd = b * d;
-
+    
     product = rad + bd;
-
+    
     if (product < rad)
     {
         /* Addition overflowed */
@@ -332,7 +362,7 @@ lwmsg_multiply_unsigned(
     }
 
     *result = product;
-
+    
 error:
 
     return status;
@@ -350,16 +380,16 @@ lwmsg_strerror(
     char* my_message = NULL;
     char buffer[512];
     const char* text = NULL;
-
+    
     text = strerror_r(err, buffer, sizeof(buffer));
-
+    
     if (!text)
     {
         text = "UNKNOWN";
     }
 
     my_message = strdup(text);
-
+    
     if (!my_message)
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
@@ -488,14 +518,14 @@ lwmsg_set_block_sigpipe(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 #ifdef SO_NOSIGPIPE
     int on = 1;
-
+    
     if (setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &on, sizeof(on)) < 0)
     {
         BAIL_ON_ERROR(status = lwmsg_error_map_errno(errno));
     }
 
 error:
-#endif
+#endif   
     return status;
 }
 
@@ -533,7 +563,7 @@ lwmsg_begin_ignore_sigpipe(
     {
         *unblock = LWMSG_FALSE;
     }
-
+    
 error:
 #endif
     return ret;
@@ -564,7 +594,7 @@ lwmsg_end_ignore_sigpipe(
         {
             sigemptyset(&sigpipemask);
             sigaddset(&sigpipemask, SIGPIPE);
-
+            
             do
             {
                 ret = sigtimedwait(&sigpipemask, &info, &timeout);
@@ -572,16 +602,16 @@ lwmsg_end_ignore_sigpipe(
 
             ret = 0;
         }
-
+        
         if (pthread_sigmask(SIG_SETMASK, original, NULL))
         {
             ret = -1;
             goto error;
         }
-
+        
         *unblock = LWMSG_FALSE;
     }
-
+        
 error:
 #endif
     return ret;
@@ -655,7 +685,7 @@ lwmsg_sendmsg_timeout(
         FD_SET(sock, &fds);
 
         ret = select(sock + 1, NULL, &fds, NULL, &timeout);
-
+            
         if (ret < 0)
         {
             return ret;
@@ -698,7 +728,7 @@ lwmsg_recvmsg_timeout(
 #ifdef MSG_NOSIGNAL
     flags |= MSG_NOSIGNAL;
 #endif
-
+    
     if (time && lwmsg_time_is_positive(time))
     {
         timeout.tv_sec = time->seconds;
@@ -730,7 +760,7 @@ lwmsg_sendmsg_timeout(
     int err = 0;
     sigset_t original;
     LWMsgBool unblock = LWMSG_FALSE;
-
+    
 #ifdef MSG_NOSIGNAL
     flags |= MSG_NOSIGNAL;
 #endif
@@ -859,7 +889,7 @@ lwmsg_hash_find_key(
     for (ring = bucket->next; ring != bucket; ring = ring->next)
     {
         entry = lwmsg_hash_entry_from_ring(table, ring);
-
+        
         if (table->equal(key, table->get_key(entry)))
         {
             return entry;
@@ -882,9 +912,9 @@ lwmsg_hash_remove_key(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
     }
-
+    
     lwmsg_ring_remove(lwmsg_hash_ring_from_entry(table, entry));
-
+    
     table->count--;
 
 error:
@@ -905,9 +935,9 @@ lwmsg_hash_remove_entry(
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
     }
-
+    
     lwmsg_ring_remove(lwmsg_hash_ring_from_entry(table, entry));
-
+    
     table->count--;
 
 error:
