@@ -255,17 +255,17 @@ add_thread(void* _data)
 
         in.tag = COUNTER_ADD;
         in.data = &add;
-
+        
         MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
 
         MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, COUNTER_ADD_SUCCESS);
         reply = (CounterReply*) out.data;
-
+        
         MU_VERBOSE("(0x%lx) counter: %i -> %i",
                    (unsigned long) (pthread_self()), 
                    reply->counter,
                    reply->counter+1);
-
+        
         lwmsg_call_destroy_params(call, &out);
         lwmsg_call_release(call);
     }
@@ -317,7 +317,7 @@ MU_TEST(stress, parallel)
     in.data = &request;
 
     MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
-
+    
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, COUNTER_OPEN_SUCCESS);
     lwmsg_call_release(call);
 
@@ -348,7 +348,7 @@ MU_TEST(stress, parallel)
     in.data = data.handle;
 
     MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
-
+    
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, COUNTER_READ_SUCCESS);
     reply = out.data;
 
@@ -356,13 +356,13 @@ MU_TEST(stress, parallel)
 
     lwmsg_call_destroy_params(call, &out);
     lwmsg_call_release(call);
-
+    
     MU_TRY(lwmsg_peer_acquire_call(client, &call));
     in.tag = COUNTER_CLOSE;
     in.data = data.handle;
 
     MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
-
+    
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, COUNTER_CLOSE_SUCCESS);
 
     lwmsg_call_destroy_params(call, &out);
@@ -391,7 +391,7 @@ MU_TEST(stress, parallel_print_protocol)
     MU_TRY(lwmsg_protocol_new(context, &protocol));
     MU_TRY(lwmsg_protocol_add_protocol_spec(protocol, counterprotocol_spec));
 
-    MU_TRY(lwmsg_protocol_print_alloc(protocol, &text));
+    MU_TRY(lwmsg_protocol_print_alloc(protocol, 4, &text));
 
     MU_VERBOSE("\n%s", text);
 }
@@ -432,7 +432,7 @@ MU_TEST(client_server, handle_invalidation)
     in.data = &request;
 
     MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
-
+    
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, COUNTER_OPEN_SUCCESS);
     lwmsg_call_release(call);
 
@@ -445,7 +445,7 @@ MU_TEST(client_server, handle_invalidation)
     MU_TRY(lwmsg_peer_acquire_call(client, &call));
 
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, lwmsg_session_get_handle_location(session, handle, &locality), LWMSG_STATUS_INVALID_HANDLE);
-
+    
     MU_TRY(lwmsg_session_release_handle(session, handle));
 
     MU_TRY(lwmsg_peer_disconnect(client));
@@ -559,7 +559,7 @@ Multicall(
     InvokeRequest invoke;
     LWMsgParams invoke_in = LWMSG_PARAMS_INITIALIZER;
     LWMsgParams invoke_out = LWMSG_PARAMS_INITIALIZER;
-
+    
     MU_TRY(lwmsg_call_acquire_callback(call, &callback));
 
     rep = malloc(sizeof(*rep));
@@ -572,7 +572,7 @@ Multicall(
         invoke.value = req->values[i];
         invoke_in.tag = INVOKE_REQUEST;
         invoke_in.data = &invoke;
-
+        
         MU_TRY(lwmsg_call_dispatch(callback, &invoke_in, &invoke_out, NULL, NULL));
         MU_ASSERT_EQUAL(MU_TYPE_INTEGER, invoke_out.tag, INVOKE_REPLY);
 
@@ -586,7 +586,7 @@ Multicall(
 
     return LWMSG_STATUS_SUCCESS;
 }
-
+    
 static
 LWMsgStatus
 Invoke(
@@ -706,7 +706,7 @@ MU_TEST(client_server, callback)
 
     MU_TRY(lwmsg_peer_start_listen(server));
     MU_TRY(lwmsg_peer_connect(client, &session));
-
+    
     req.func = times_two;
     req.num_values = sizeof(values) / sizeof(*values);
     req.values = values;
@@ -714,7 +714,7 @@ MU_TEST(client_server, callback)
     in.data = &req;
 
     MU_TRY(lwmsg_peer_acquire_call(client, &call));
-
+    
     MU_TRY(lwmsg_session_register_handle(session, "IntFunction", req.func, NULL));
 
     MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
@@ -866,3 +866,103 @@ MU_TEST(client_server, bogus_reply_fail_ping_succeed)
     MU_TRY(lwmsg_peer_disconnect(client));
     MU_TRY(lwmsg_peer_stop_listen(server));
 }
+
+#define NUM_ASSOCS 128
+
+MU_TEST(client_server, client_limit_timeout)
+{
+    LWMsgAssoc* assocs[NUM_ASSOCS] = {0};
+    LWMsgTime idle = {1, 0};
+    int i = 0;
+
+    /* This test shouldn't take more than 5 seconds */
+    MU_TIMEOUT(5000);
+
+    MU_TRY(lwmsg_peer_set_timeout(server, LWMSG_TIMEOUT_IDLE, &idle));
+    MU_TRY(lwmsg_peer_set_max_listen_clients(server, NUM_ASSOCS / 2));
+    MU_TRY(lwmsg_peer_start_listen(server));
+
+    for (i = 0; i < NUM_ASSOCS; i++)
+    {
+        MU_TRY(lwmsg_connection_new(context, protocol, &assocs[i]));
+        MU_TRY(lwmsg_connection_set_endpoint(assocs[i], LWMSG_ENDPOINT_LOCAL, TEST_ENDPOINT));
+        MU_TRY(lwmsg_assoc_connect(assocs[i], NULL));
+    }
+}
+
+typedef struct trace_info
+{
+    LWMsgBool outgoing_begin, outgoing_end;
+    LWMsgBool incoming_begin, incoming_end;
+} trace_info;
+
+static
+void
+trace_begin(
+    LWMsgCall* call,
+    const LWMsgParams* params,
+    void* data
+    )
+{
+    trace_info* info = data;
+
+    if (lwmsg_call_get_direction(call) == LWMSG_CALL_INCOMING)
+    {
+        MU_VERBOSE("Begin in call %i\n", params->tag);
+        info->incoming_begin = LWMSG_TRUE;
+    }
+    else
+    {
+        MU_VERBOSE("Begin out call %i\n", params->tag);
+        info->outgoing_begin = LWMSG_TRUE;
+    }
+}
+
+static
+void
+trace_end(
+    LWMsgCall* call,
+    const LWMsgParams* params,
+    void* data
+    )
+{
+    trace_info* info = data;
+
+    if (lwmsg_call_get_direction(call) == LWMSG_CALL_INCOMING)
+    {
+        MU_VERBOSE("End in call %i\n", params->tag);
+        info->incoming_end = LWMSG_TRUE;
+    }
+    else
+    {
+        MU_VERBOSE("End out call %i\n", params->tag);
+        info->outgoing_end = LWMSG_TRUE;
+    }
+}
+
+MU_TEST(client_server, tracing)
+{
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* call = NULL;
+    trace_info info = {0};
+
+    in.tag = PING_REQUEST;
+    in.data = NULL;
+
+    MU_TRY(lwmsg_peer_set_trace_functions(server, trace_begin, trace_end, &info));
+    MU_TRY(lwmsg_peer_set_trace_functions(client, trace_begin, trace_end, &info));
+
+    MU_TRY(lwmsg_peer_start_listen(server));
+    MU_TRY(lwmsg_peer_connect(client, NULL));
+    MU_TRY(lwmsg_peer_acquire_call(client, &call));
+    MU_TRY(lwmsg_call_dispatch(call, &in, &out, NULL, NULL));
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out.tag, PING_REPLY);
+    MU_ASSERT(info.incoming_begin);
+    MU_ASSERT(info.incoming_end);
+    MU_ASSERT(info.outgoing_begin);
+    MU_ASSERT(info.outgoing_end);
+
+    MU_TRY(lwmsg_peer_disconnect(client));
+    MU_TRY(lwmsg_peer_stop_listen(server));
+}
