@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -73,7 +73,7 @@ lwmsg_test_assoc_thread(
     )
 {
     StartInfo* info = data;
-
+    
     if (info->connect)
     {
         MU_TRY_ASSOC(info->assoc, lwmsg_assoc_connect(info->assoc, NULL));
@@ -100,7 +100,7 @@ lwmsg_test_assoc_pair(
     pthread_t thread1, thread2;
     int sockets[2];
     static LWMsgProtocol* protocol = NULL;
-
+    
     MU_TRY(lwmsg_protocol_new(NULL, &protocol));
     MU_TRY_PROTOCOL(protocol, lwmsg_protocol_add_protocol_spec(protocol, pspec));
 
@@ -112,16 +112,16 @@ lwmsg_test_assoc_pair(
     MU_TRY(lwmsg_connection_new(NULL,
                protocol,
                &s1.assoc));
-
+    
     MU_TRY(lwmsg_connection_set_fd(
                s1.assoc,
                LWMSG_CONNECTION_MODE_PAIR,
                sockets[0]));
-
+    
     MU_TRY(lwmsg_connection_new(NULL,
                protocol,
                &s2.assoc));
-
+    
     MU_TRY(lwmsg_connection_set_fd(
                s2.assoc,
                LWMSG_CONNECTION_MODE_PAIR,
@@ -146,6 +146,7 @@ LWMsgBool
 lwmsg_test_log_function(
     LWMsgLogLevel level,
     const char* message,
+    const char* function,
     const char* filename,
     unsigned int line,
     void* data
@@ -158,6 +159,7 @@ lwmsg_test_log_function(
     {
         switch (level)
         {
+        case LWMSG_LOGLEVEL_ALWAYS:
         case LWMSG_LOGLEVEL_ERROR:
             /* MoonUnit does not have an error loglevel because errors
                are considered a test failure.  Log a warning
@@ -181,8 +183,8 @@ lwmsg_test_log_function(
             mu_level = MU_LEVEL_TRACE;
             break;
         }
-
-        Mu_Interface_Event(filename, line, mu_level, is_error ? "ERROR: %s" : "%s", message);
+    
+        Mu_Interface_Event(filename, line, mu_level, is_error ? "ERROR: %s():%s" : "%s():%s", function, message);
     }
 
     return LWMSG_TRUE;
