@@ -7,7 +7,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -79,8 +79,15 @@ basic_work_item(
 MU_TEST(task, basic_work_item)
 {
     LWMsgBool volatile value = LWMSG_FALSE;
+    union
+    {
+        LWMsgBool volatile *bvalue;
+        void* vvalue;
+    } pun;
 
-    MU_TRY(lwmsg_task_dispatch_work_item(manager, basic_work_item, (void*) &value));
+    pun.bvalue = &value;
+
+    MU_TRY(lwmsg_task_dispatch_work_item(manager, basic_work_item, pun.vvalue));
 
     pthread_mutex_lock(&lock);
     while (!value)
@@ -93,12 +100,13 @@ MU_TEST(task, basic_work_item)
 static const int target = 5;
 
 static
-LWMsgStatus
+void
 timer_task(
+    LWMsgTask* task,
     void* data,
     LWMsgTaskTrigger trigger,
     LWMsgTaskTrigger* next_trigger,
-    LWMsgTime* next_time
+    LWMsgTaskTime* next_time
     )
 {
     int volatile * value = data;
@@ -116,15 +124,12 @@ timer_task(
     {
         (*value)++;
         *next_trigger = LWMSG_TASK_TRIGGER_TIME;
-        next_time->seconds = 0;
-        next_time->microseconds = 10000;
+        *next_time = 10000000ll;
     }
     else
     {
         *next_trigger = 0;
     }
-
-    return LWMSG_STATUS_SUCCESS;
 }
 
 MU_TEST(task, timer_task)
