@@ -6,10 +6,10 @@
  * Copyright (C) Centeris Corporation 2004-2007
  * Copyright (C) Likewise Software    2007-2008
  * All rights reserved.
- *
+ * 
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
+ * it under the terms of the GNU Lesser General Public License as 
+ * published by the Free Software Foundation; either version 2.1 of 
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
@@ -17,8 +17,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program.  If not, see
+ * You should have received a copy of the GNU Lesser General Public 
+ * License along with this program.  If not, see 
  * <http://www.gnu.org/licenses/>.
  */
 
@@ -56,7 +56,7 @@ NetConnectSamr(
     const DWORD dwDefDomainAccess = DOMAIN_ACCESS_ENUM_ACCOUNTS |
                                     DOMAIN_ACCESS_OPEN_ACCOUNT |
                                     DOMAIN_ACCESS_LOOKUP_INFO_2;
-
+			     
     const DWORD dwDefBuiltinAccess = DOMAIN_ACCESS_ENUM_ACCOUNTS |
                                   DOMAIN_ACCESS_OPEN_ACCOUNT |
                                   DOMAIN_ACCESS_LOOKUP_INFO_2;
@@ -267,7 +267,7 @@ NetConnectSamr(
             }
         }
         while (ntStatus == STATUS_MORE_ENTRIES);
-
+        
 domain_name_found:
         ntStatus = SamrLookupDomain(hSamrBinding,
                                     hConn,
@@ -333,7 +333,7 @@ cleanup:
         ntStatus != STATUS_SUCCESS)
     {
         err = LwNtStatusToWin32Error(ntStatus);
-    }
+    }       
 
     return err;
 
@@ -443,7 +443,7 @@ cleanup:
         ntStatus != STATUS_SUCCESS)
     {
         err = LwNtStatusToWin32Error(ntStatus);
-    }
+    }       
 
     return err;
 
@@ -593,7 +593,7 @@ cleanup:
         ntStatus != STATUS_SUCCESS)
     {
         err = LwNtStatusToWin32Error(ntStatus);
-    }
+    }       
 
     return err;
 
@@ -614,7 +614,7 @@ NetDisconnectSamr(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    WINERROR err = ERROR_SUCCESS;
+    WINERROR __attribute__((unused)) err = ERROR_SUCCESS;
     SAMR_BINDING hSamrBinding = NULL;
     PNET_CONN pConn = NULL;
 
@@ -689,7 +689,7 @@ NetDisconnectLsa(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    WINERROR err = ERROR_SUCCESS;
+    WINERROR __attribute__((unused)) err = ERROR_SUCCESS;
     LSA_BINDING hLsaBinding = NULL;
     PNET_CONN pConn = NULL;
 
@@ -738,7 +738,7 @@ NetDisconnectWkssvc(
     PNET_CONN  *ppConn
     )
 {
-    WINERROR err = ERROR_SUCCESS;
+    WINERROR __attribute__((unused)) err = ERROR_SUCCESS;
     WKSS_BINDING hWkssBinding = NULL;
     PNET_CONN pConn = NULL;
 
