@@ -12,7 +12,7 @@
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
  * General Public License for more details.  You should have received a copy
  * of the GNU Lesser General Public License along with this program.  If
@@ -478,10 +478,11 @@ NetAllocBufferAcbFlagsFromUserFlags(
 
 DWORD
 NetAllocBufferWC16String(
-    PVOID  *ppCursor,
-    PDWORD  pdwSpaceLeft,
-    PCWSTR  pwszSource,
-    PDWORD  pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    PCWSTR                pwszSource,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -573,10 +574,11 @@ error:
 
 DWORD
 NetAllocBufferWC16StringFromUnicodeString(
-    PVOID          *ppCursor,
-    PDWORD          pdwSpaceLeft,
-    UNICODE_STRING *pSource,
-    PDWORD          pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    UNICODE_STRING       *pSource,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -671,13 +673,14 @@ error:
 
 DWORD
 NetAllocBufferUnicodeStringFromWC16String(
-    PVOID  *ppCursor,
-    PDWORD  pdwSpaceLeft,
-    PCWSTR  pwszSource,
-    PDWORD  pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    PCWSTR                pwszSource,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
-    const WCHAR wszNullStr[] = { '\0' };
+    const WCHAR wszNullStr[] = {'\0'};
     DWORD err = ERROR_SUCCESS;
     PVOID pCursor = NULL;
     DWORD dwStrSize = 0;
@@ -731,9 +734,10 @@ NetAllocBufferUnicodeStringFromWC16String(
         err = NetAllocBufferWC16String(&pCursor,
                                        &dwSpaceLeft,
                                        pwszSource,
-                                       &dwSize);
+                                       &dwSize,
+                                       eValidation);
         BAIL_ON_WIN_ERROR(err);
-
+                                        
         *ppCursor     = pCursor;
         *pdwSpaceLeft = dwSpaceLeft;
     }
@@ -770,10 +774,11 @@ error:
 
 DWORD
 NetAllocBufferLogonHours(
-    PVOID      *ppCursor,
-    PDWORD      pdwSpaceLeft,
-    LogonHours *pHours,
-    PDWORD      pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    LogonHours           *pHours,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -800,6 +805,9 @@ NetAllocBufferLogonHours(
     /* Logon hours is a 21-byte bit string */
     dwSize += sizeof(UINT8) * 21;
 
+    /* Align the size */
+    dwSize += sizeof(PVOID) - dwSize % sizeof(PVOID);
+
     if (pCursor)
     {
         if (dwSize > dwSpaceLeft)
@@ -863,10 +871,11 @@ error:
  */
 DWORD
 NetAllocBufferSamrLogonHoursFromNetLogonHours(
-    PVOID      *ppCursor,
-    PDWORD      pdwSpaceLeft,
-    PDWORD      pdwHours,
-    PDWORD      pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    PDWORD                pdwHours,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -903,9 +912,10 @@ NetAllocBufferSamrLogonHoursFromNetLogonHours(
                                      &dwSpaceLeft,
                                      pbUnits,
                                      (DWORD)wUnitsPerWeek,
-                                     &dwSize);
+                                     &dwSize,
+                                     eValidation);
         BAIL_ON_WIN_ERROR(err);
-
+                                      
         *ppCursor     = pCursor;
         *pdwSpaceLeft = dwSpaceLeft;
     }
@@ -939,11 +949,12 @@ error:
 
 DWORD
 NetAllocBufferSid(
-    PVOID      *ppCursor,
-    PDWORD      pdwSpaceLeft,
-    PSID        pSourceSid,
-    DWORD       dwSourceSidLength,
-    PDWORD      pdwSize
+    PVOID               *ppCursor,
+    PDWORD               pdwSpaceLeft,
+    PSID                 pSourceSid,
+    DWORD                dwSourceSidLength,
+    PDWORD               pdwSize,
+    NET_VALIDATION_LEVEL eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -1042,11 +1053,12 @@ error:
 
 DWORD
 NetAllocBufferByteBlob(
-    PVOID      *ppCursor,
-    PDWORD      pdwSpaceLeft,
-    PBYTE       pbBlob,
-    DWORD       dwBlobSize,
-    PDWORD      pdwSize
+    PVOID               *ppCursor,
+    PDWORD               pdwSpaceLeft,
+    PBYTE                pbBlob,
+    DWORD                dwBlobSize,
+    PDWORD               pdwSize,
+    NET_VALIDATION_LEVEL eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -1132,11 +1144,12 @@ error:
 
 DWORD
 NetAllocBufferFixedBlob(
-    PVOID      *ppCursor,
-    PDWORD      pdwSpaceLeft,
-    PBYTE       pbBlob,
-    DWORD       dwBlobSize,
-    PDWORD      pdwSize
+    PVOID                *ppCursor,
+    PDWORD                pdwSpaceLeft,
+    PBYTE                 pbBlob,
+    DWORD                 dwBlobSize,
+    PDWORD                pdwSize,
+    NET_VALIDATION_LEVEL  eValidation
     )
 {
     DWORD err = ERROR_SUCCESS;
@@ -1207,7 +1220,7 @@ NetAllocBufferNT4Name(
     PVOID   *ppCursor,
     PDWORD   pdwSpaceLeft,
     PWSTR    pwszDomainName,
-    PWSTR    pwszAccountName,
+    PWSTR    pwszAccountName,    
     PDWORD   pdwSize
     )
 {
