@@ -981,7 +981,8 @@ NetSessionEnumW(
     PDWORD  pdwResumeHandle    /* IN OUT OPTIONAL */
     )
 {
-    NET_API_STATUS  status = 0;
+    NET_API_STATUS status = 0;
+    NET_API_STATUS ret = 0;
     PSRVSVC_CONTEXT pContext = NULL;
     PBYTE pBuffer        = NULL;
     DWORD dwEntriesRead  = 0;
@@ -994,6 +995,11 @@ NetSessionEnumW(
         BAIL_ON_NETAPI_ERROR(status);
     }
 
+    if (pdwResumeHandle)
+    {
+        dwResumeHandle = *pdwResumeHandle;
+    }
+
     status = SrvSvcCreateContext(pwszServername, &pContext);
     BAIL_ON_NETAPI_ERROR(status);
 
@@ -1008,20 +1014,34 @@ NetSessionEnumW(
                     &dwEntriesRead,
                     &dwTotalEntries,
                     &dwResumeHandle);
+    if (status == ERROR_MORE_DATA)
+    {
+        ret = status;
+        status = ERROR_SUCCESS;
+    }
     BAIL_ON_NETAPI_ERROR(status);
 
     *ppBuffer = pBuffer;
     *pdwEntriesRead = dwEntriesRead;
     *pdwTotalEntries = dwTotalEntries;
-    *pdwResumeHandle = dwResumeHandle;
+
+    if (pdwResumeHandle)
+    {
+        *pdwResumeHandle = dwResumeHandle;
+    }
 
 cleanup:
-
     if (pContext)
     {
         SrvSvcCloseContext(pContext);
     }
 
+    if (status == ERROR_SUCCESS &&
+        ret != ERROR_SUCCESS)
+    {
+        status = ret;
+    }
+
     return status;
 
 error:
