@@ -12,6 +12,8 @@
 #include <dce/smb.h>
 #include <dce/lrpc.h>
 #include <dce/schannel.h>
+#include <ntlm/gssntlm.h>
+#include <ntlm/sspintlm.h>
 #include <wc16str.h>
 #include <lw/base.h>
 #include <lw/ntstatus.h>
@@ -20,12 +22,13 @@
 #include <lwstr.h>
 #include <lwsid.h>
 #include <lwio/lwio.h>
-#include <lwps/lwps.h>
+#include <lwio/lmsession.h>
 #include <lwnet.h>
 #include <openssl/rc4.h>
 #include <openssl/md5.h>
 #include <openssl/rand.h>
 
+#include <lw/rpc/common.h>
 #include <lw/rpc/samr.h>
 #include <lw/rpc/lsa.h>
 #include <lw/rpc/netlogon.h>
@@ -35,3 +38,4 @@
 
 #include "netapitest.h"
 #include "params.h"
+#include "common.h"
