@@ -75,7 +75,7 @@ RegOpenServer(
 	)
 {
     return RegNtStatusToWin32Error(
-		NtRegOpenServer(phConnection));
+    		NtRegOpenServer(phConnection));
 }
 
 NTSTATUS
@@ -84,14 +84,14 @@ NtRegOpenServerInitialize(
     )
 {
     NTSTATUS status = 0;
-
+    
     pthread_mutex_lock(&gLock);
 
     if (!gContext.pProtocol)
     {
         status = MAP_LWMSG_ERROR(lwmsg_protocol_new(NULL, &gContext.pProtocol));
         BAIL_ON_NT_STATUS(status);
-
+        
         status = MAP_LWMSG_ERROR(lwmsg_protocol_add_protocol_spec(gContext.pProtocol, RegIPCGetProtocolSpec()));
         BAIL_ON_NT_STATUS(status);
     }
@@ -100,7 +100,7 @@ NtRegOpenServerInitialize(
     {
         status = MAP_LWMSG_ERROR(lwmsg_peer_new(NULL, gContext.pProtocol, &gContext.pClient));
         BAIL_ON_NT_STATUS(status);
-
+        
         status = MAP_LWMSG_ERROR(lwmsg_peer_add_connect_endpoint(
                                      gContext.pClient,
                                      LWMSG_CONNECTION_MODE_LOCAL,
@@ -111,9 +111,9 @@ NtRegOpenServerInitialize(
     if (!gContext.pSession)
     {
         status = MAP_LWMSG_ERROR(lwmsg_peer_connect(gContext.pClient, &gContext.pSession));
-        BAIL_ON_NT_STATUS(status);
+        BAIL_ON_NT_STATUS(status);  
     }
-
+        
 cleanup:
 
     pthread_mutex_unlock(&gLock);
@@ -263,7 +263,7 @@ RegTransactEnumRootKeysW(
             break;
 
         default:
-		status = STATUS_INVALID_PARAMETER;
+        	status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
     }
 
@@ -295,7 +295,7 @@ RegTransactCreateKeyExW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_CREATE_KEY_EX_REQ CreateKeyExReq;
+    REG_IPC_CREATE_KEY_EX_REQ CreateKeyExReq = {0};
     PREG_IPC_CREATE_KEY_EX_RESPONSE pCreateKeyExResp = NULL;
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
@@ -312,31 +312,31 @@ RegTransactCreateKeyExW(
 
     if (pSecurityDescriptor)
     {
-	ulSecDescLen = 1024;
+    	ulSecDescLen = 1024;
 
-	do
-	{
-	    status = NtRegReallocMemory(pSecDescRel,
-								    (PVOID*)&pSecDescRel,
-									ulSecDescLen);
-		BAIL_ON_NT_STATUS(status);
+    	do
+    	{
+    	    status = NtRegReallocMemory(pSecDescRel,
+    		   						    (PVOID*)&pSecDescRel,
+    									ulSecDescLen);
+    		BAIL_ON_NT_STATUS(status);
 
-	    memset(pSecDescRel, 0, ulSecDescLen);
+    	    memset(pSecDescRel, 0, ulSecDescLen);
 
-		status = RtlAbsoluteToSelfRelativeSD(pSecurityDescriptor,
-											 pSecDescRel,
-											 &ulSecDescLen);
-		if (STATUS_BUFFER_TOO_SMALL  == status)
-		{
-		    ulSecDescLen *= 2;
-		}
-		else
-		{
-			BAIL_ON_NT_STATUS(status);
-		}
-	}
-	while((status != STATUS_SUCCESS) &&
-		  (ulSecDescLen <= SECURITY_DESCRIPTOR_RELATIVE_MAX_SIZE));
+    		status = RtlAbsoluteToSelfRelativeSD(pSecurityDescriptor,
+    									  		 pSecDescRel,
+    											 &ulSecDescLen);
+    		if (STATUS_BUFFER_TOO_SMALL  == status)
+    		{
+    		    ulSecDescLen *= 2;
+    		}
+    		else
+    		{
+    			BAIL_ON_NT_STATUS(status);
+    		}
+    	}
+    	while((status != STATUS_SUCCESS) &&
+    		  (ulSecDescLen <= SECURITY_DESCRIPTOR_RELATIVE_MAX_SIZE));
     }
 
     CreateKeyExReq.hKey = hKey;
@@ -372,7 +372,7 @@ RegTransactCreateKeyExW(
             BAIL_ON_NT_STATUS(status);
             break;
         default:
-		status = STATUS_INVALID_PARAMETER;
+        	status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
     }
 
@@ -402,7 +402,7 @@ RegTransactOpenKeyExW(
     )
 {
     NTSTATUS status = 0;
-    REG_IPC_OPEN_KEY_EX_REQ OpenKeyExReq;
+    REG_IPC_OPEN_KEY_EX_REQ OpenKeyExReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
     PREG_IPC_OPEN_KEY_EX_RESPONSE pOpenKeyExResp = NULL;
@@ -439,7 +439,7 @@ RegTransactOpenKeyExW(
             BAIL_ON_NT_STATUS(status);
             break;
         default:
-		status = STATUS_INVALID_PARAMETER;
+        	status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
     }
 
@@ -463,7 +463,7 @@ RegTransactCloseKey(
     )
 {
     NTSTATUS status = 0;
-    REG_IPC_CLOSE_KEY_REQ CloseKeyReq;
+    REG_IPC_CLOSE_KEY_REQ CloseKeyReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -523,7 +523,7 @@ RegTransactDeleteKeyW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_DELETE_KEY_REQ DeleteKeyReq;
+    REG_IPC_DELETE_KEY_REQ DeleteKeyReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -590,7 +590,7 @@ RegTransactQueryInfoKeyW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_QUERY_INFO_KEY_REQ QueryInfoKeyReq;
+    REG_IPC_QUERY_INFO_KEY_REQ QueryInfoKeyReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
     PREG_IPC_QUERY_INFO_KEY_RESPONSE pQueryInfoKeyResp = NULL;
@@ -638,7 +638,7 @@ RegTransactQueryInfoKeyW(
             }
             if (pcbSecurityDescriptor)
             {
-		*pcbSecurityDescriptor = pQueryInfoKeyResp->cSecurityDescriptor;
+            	*pcbSecurityDescriptor = pQueryInfoKeyResp->cSecurityDescriptor;
             }
 
             break;
@@ -680,7 +680,7 @@ RegTransactEnumKeyExW(
 {
 	NTSTATUS status = 0;
 
-    REG_IPC_ENUM_KEY_EX_REQ EnumKeyExReq;
+    REG_IPC_ENUM_KEY_EX_REQ EnumKeyExReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
     PREG_IPC_ENUM_KEY_EX_RESPONSE pEnumKeyExResp = NULL;
@@ -716,7 +716,7 @@ RegTransactEnumKeyExW(
                 memcpy(pClass, pEnumKeyExResp->pClass, (pEnumKeyExResp->cClass+1)*sizeof(*pClass));
                 if (pcClass)
                 {
-			*pcClass = pEnumKeyExResp->cClass;
+                	*pcClass = pEnumKeyExResp->cClass;
                 }
             }
 
@@ -758,7 +758,7 @@ RegTransactGetValueW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_GET_VALUE_REQ GetValueReq;
+    REG_IPC_GET_VALUE_REQ GetValueReq = {0};
     PREG_IPC_GET_VALUE_RESPONSE pGetValueResp = NULL;
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
@@ -774,7 +774,7 @@ RegTransactGetValueW(
     GetValueReq.pSubKey = pSubKey;
     GetValueReq.pValue = pValue;
     GetValueReq.Flags = Flags;
-    GetValueReq.cbData = *pcbData;
+    GetValueReq.cbData = pcbData ? *pcbData : 0;
 
     in.tag = REG_Q_GET_VALUEW;
     in.data = &GetValueReq;
@@ -789,7 +789,7 @@ RegTransactGetValueW(
 
             if (pdwType)
             {
-		*pdwType = pGetValueResp->dwType;
+            	*pdwType = pGetValueResp->dwType;
             }
 
             if (pvData)
@@ -809,6 +809,7 @@ RegTransactGetValueW(
             status = pStatus->status;
             BAIL_ON_NT_STATUS(status);
             break;
+
         default:
             status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
@@ -836,7 +837,7 @@ RegTransactDeleteKeyValueW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_DELETE_KEY_VALUE_REQ DeleteKeyValueReq;
+    REG_IPC_DELETE_KEY_VALUE_REQ DeleteKeyValueReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -893,7 +894,7 @@ RegTransactDeleteTreeW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_DELETE_TREE_REQ DeleteTreeReq;
+    REG_IPC_DELETE_TREE_REQ DeleteTreeReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -950,7 +951,7 @@ RegTransactDeleteValueW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_DELETE_VALUE_REQ DeleteValueReq;
+    REG_IPC_DELETE_VALUE_REQ DeleteValueReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -1012,7 +1013,7 @@ RegTransactEnumValueW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_ENUM_VALUE_REQ EnumValueReq;
+    REG_IPC_ENUM_VALUE_REQ EnumValueReq = {0};
     PREG_IPC_ENUM_VALUE_RESPONSE pEnumValueResp = NULL;
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
@@ -1096,7 +1097,7 @@ RegTransactQueryMultipleValues(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_QUERY_MULTIPLE_VALUES_REQ QueryMultipleValuesReq;
+    REG_IPC_QUERY_MULTIPLE_VALUES_REQ QueryMultipleValuesReq = {0};
     PREG_IPC_QUERY_MULTIPLE_VALUES_RESPONSE pRegResp = NULL;
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
@@ -1193,7 +1194,7 @@ RegTransactSetValueExW(
     )
 {
 	NTSTATUS status = 0;
-    REG_IPC_SET_VALUE_EX_REQ SetValueExReq;
+    REG_IPC_SET_VALUE_EX_REQ SetValueExReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -1255,7 +1256,7 @@ RegTransactSetKeySecurity(
 	)
 {
 	NTSTATUS status = 0;
-    REG_IPC_SET_KEY_SECURITY_REQ SetKeySecurityReq;
+    REG_IPC_SET_KEY_SECURITY_REQ SetKeySecurityReq = {0};
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
 
@@ -1311,12 +1312,12 @@ RegTransactGetKeySecurity(
 	IN HANDLE hConnection,
 	IN HKEY hKey,
 	IN SECURITY_INFORMATION SecurityInformation,
-	OUT PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor,
+	OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor,
 	IN OUT PULONG lpcbSecurityDescriptor
 	)
 {
 	NTSTATUS status = 0;
-    REG_IPC_GET_KEY_SECURITY_REQ GetKeySecurityReq;
+    REG_IPC_GET_KEY_SECURITY_REQ GetKeySecurityReq = {0};
     PREG_IPC_GET_KEY_SECURITY_RES pGetKeySecurityResp = NULL;
     // Do not free pStatus
     PREG_IPC_STATUS pStatus = NULL;
@@ -1331,6 +1332,7 @@ RegTransactGetKeySecurity(
     GetKeySecurityReq.hKey = hKey;
     GetKeySecurityReq.SecurityInformation = SecurityInformation;
     GetKeySecurityReq.Length = *lpcbSecurityDescriptor;
+    GetKeySecurityReq.bRetSecurityDescriptor = SecurityDescriptor ? TRUE : FALSE;
 
     in.tag = REG_Q_GET_KEY_SECURITY;
     in.data = &GetKeySecurityReq;
@@ -1341,10 +1343,16 @@ RegTransactGetKeySecurity(
     switch (out.tag)
     {
         case REG_R_GET_KEY_SECURITY:
-		pGetKeySecurityResp = (PREG_IPC_GET_KEY_SECURITY_RES) out.data;
+        	pGetKeySecurityResp = (PREG_IPC_GET_KEY_SECURITY_RES) out.data;
 
-		*lpcbSecurityDescriptor = pGetKeySecurityResp->Length;
-		memcpy(SecurityDescriptor, pGetKeySecurityResp->SecurityDescriptor, pGetKeySecurityResp->Length);
+        	*lpcbSecurityDescriptor = pGetKeySecurityResp->Length;
+
+        	if (SecurityDescriptor)
+        	{
+        	    memcpy(SecurityDescriptor,
+        	           pGetKeySecurityResp->SecurityDescriptor,
+        	           pGetKeySecurityResp->Length);
+        	}
 
             break;
 
@@ -1372,6 +1380,211 @@ error:
     goto cleanup;
 }
 
+
+NTSTATUS
+RegTransactSetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    )
+{
+    NTSTATUS status = 0;
+    REG_IPC_SET_VALUE_ATTRS_REQ SetValueAttrsReq = {0};
+    // Do not free pStatus
+    PREG_IPC_STATUS pStatus = NULL;
+
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    status = RegIpcAcquireCall(hRegConnection, &pCall);
+    BAIL_ON_NT_STATUS(status);
+
+    SetValueAttrsReq.hKey = hKey;
+    SetValueAttrsReq.pSubKey = pSubKey;
+    SetValueAttrsReq.pValueName = pValueName;
+    SetValueAttrsReq.pValueAttributes = pValueAttributes;
+
+    in.tag = REG_Q_SET_VALUEW_ATTRIBUTES;
+    in.data = &SetValueAttrsReq;
+
+    status = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
+    BAIL_ON_NT_STATUS(status);
+
+    switch (out.tag)
+    {
+        case REG_R_SET_VALUEW_ATTRIBUTES:
+            break;
+
+        case REG_R_ERROR:
+            pStatus = (PREG_IPC_STATUS) out.data;
+            status = pStatus->status;
+            BAIL_ON_NT_STATUS(status);
+            break;
+
+        default:
+            status = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(status);
+    }
+
+cleanup:
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+
+NTSTATUS
+RegTransactGetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    )
+{
+    NTSTATUS status = 0;
+    REG_IPC_GET_VALUE_ATTRS_REQ GetValueAttrsReq = {0};
+    PREG_IPC_GET_VALUE_ATTRS_RESPONSE pGetValueAttrsResp = NULL;
+    // Do not free pStatus
+    PREG_IPC_STATUS pStatus = NULL;
+
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    if (!ppCurrentValue && !ppValueAttributes)
+    {
+        status = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    status = RegIpcAcquireCall(hRegConnection, &pCall);
+    BAIL_ON_NT_STATUS(status);
+
+    GetValueAttrsReq.hKey = hKey;
+    GetValueAttrsReq.pSubKey = pwszSubKey;
+    GetValueAttrsReq.pValueName = pwszValueName;
+    GetValueAttrsReq.bRetCurrentValue = ppCurrentValue ? TRUE : FALSE;
+    GetValueAttrsReq.bRetValueAttributes = ppValueAttributes ? TRUE : FALSE;
+
+    in.tag = REG_Q_GET_VALUEW_ATTRIBUTES;
+    in.data = &GetValueAttrsReq;
+
+    status = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
+    BAIL_ON_NT_STATUS(status);
+
+    switch (out.tag)
+    {
+        case REG_R_GET_VALUEW_ATTRIBUTES:
+            pGetValueAttrsResp = (PREG_IPC_GET_VALUE_ATTRS_RESPONSE) out.data;
+
+            if (ppCurrentValue)
+            {
+                *ppCurrentValue = pGetValueAttrsResp->pCurrentValue;
+                pGetValueAttrsResp->pCurrentValue = NULL;
+            }
+
+            if (ppValueAttributes)
+            {
+                *ppValueAttributes = pGetValueAttrsResp->pValueAttributes;
+                pGetValueAttrsResp->pValueAttributes = NULL;
+            }
+
+            break;
+
+        case REG_R_ERROR:
+            pStatus = (PREG_IPC_STATUS) out.data;
+            status = pStatus->status;
+            BAIL_ON_NT_STATUS(status);
+            break;
+
+        default:
+            status = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(status);
+    }
+
+cleanup:
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+NTSTATUS
+RegTransactDeleteValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName
+    )
+{
+    NTSTATUS status = 0;
+    REG_IPC_DELETE_VALUE_ATTRS_REQ DeleteValueAttrsReq = {0};
+    // Do not free pStatus
+    PREG_IPC_STATUS pStatus = NULL;
+
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    status = RegIpcAcquireCall(hRegConnection, &pCall);
+    BAIL_ON_NT_STATUS(status);
+
+    DeleteValueAttrsReq.hKey = hKey;
+    DeleteValueAttrsReq.pSubKey = pwszSubKey;
+    DeleteValueAttrsReq.pValueName = pwszValueName;
+
+    in.tag = REG_Q_DELETE_VALUEW_ATTRIBUTES;
+    in.data = &DeleteValueAttrsReq;
+
+    status = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
+    BAIL_ON_NT_STATUS(status);
+
+    switch (out.tag)
+    {
+        case REG_R_DELETE_VALUEW_ATTRIBUTES:
+            break;
+
+        case REG_R_ERROR:
+            pStatus = (PREG_IPC_STATUS) out.data;
+            status = pStatus->status;
+            BAIL_ON_NT_STATUS(status);
+            break;
+
+        default:
+            status = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(status);
+    }
+
+cleanup:
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    return status;
+
+error:
+    goto cleanup;
+}
 /*
 local variables:
 mode: c
