@@ -202,12 +202,12 @@ RegTransactSetKeySecurity(
 
 NTSTATUS
 RegTransactGetKeySecurity(
-	IN HANDLE hNtRegConnection,
-	IN HKEY hKey,
-	IN SECURITY_INFORMATION SecurityInformation,
-	OUT PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor,
-	IN OUT PULONG lpcbSecurityDescriptor
-	);
+    IN HANDLE hConnection,
+    IN HKEY hKey,
+    IN SECURITY_INFORMATION SecurityInformation,
+    OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE SecurityDescriptor,
+    IN OUT PULONG lpcbSecurityDescriptor
+    );
 
 NTSTATUS
 RegTransactQueryMultipleValues(
@@ -219,5 +219,32 @@ RegTransactQueryMultipleValues(
     IN OUT OPTIONAL PDWORD pdwTotsize
     );
 
+NTSTATUS
+RegTransactSetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    );
+
+NTSTATUS
+RegTransactGetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    );
+
+NTSTATUS
+RegTransactDeleteValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName
+    );
+
 #endif /* __CLIENTIPC_P_H__ */
 
