@@ -48,7 +48,7 @@
  */
 #include "client.h"
 
-/*
+/* 
  * This is a work-around for the fact we don't exactly know the
  * multibyte size for these paramters. The only way we can determine
  * this correctly is to actually enumerate the key/values and
@@ -87,7 +87,7 @@ NtRegEnumRootKeysA(
     for (iCount = 0; iCount < dwNumRootKeys; iCount++)
     {
         status = LwRtlCStringAllocateFromWC16String(&ppszRootKeyNames[iCount],
-			                                     ppwszRootKeyNames[iCount]);
+        		                                     ppwszRootKeyNames[iCount]);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -148,8 +148,8 @@ NtRegCreateKeyExA(
 
     if (pszSubKey)
     {
-	status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-			                                     pszSubKey);
+    	status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
+    			                                     pszSubKey);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -197,8 +197,8 @@ NtRegCreateKeyExW(
 
     if (wc16slen(pSubKey) > MAX_KEY_LENGTH)
     {
-	status = STATUS_INVALID_BLOCK_LENGTH;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_BLOCK_LENGTH;
+    	BAIL_ON_NT_STATUS(status);
     }
 
 	status = RegTransactCreateKeyExW(
@@ -249,7 +249,7 @@ NtRegDeleteKeyA(
     BAIL_ON_NT_INVALID_POINTER(pszSubKey);
 
     status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-		                                     pszSubKey);
+    		                                     pszSubKey);
     BAIL_ON_NT_STATUS(status);
 
     status = RegTransactDeleteKeyW(
@@ -298,14 +298,14 @@ NtRegDeleteKeyValueA(
     if (pszSubKey)
     {
         status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-			                                     pszSubKey);
+        		                                     pszSubKey);
         BAIL_ON_NT_STATUS(status);
     }
 
     if (pszValueName)
     {
         status = LwRtlWC16StringAllocateFromCString(&pwszValueName,
-			                                     pszValueName);
+        		                                     pszValueName);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -357,7 +357,7 @@ NtRegDeleteTreeA(
     if (pszSubKey)
     {
         status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-			                                     pszSubKey);
+        		                                     pszSubKey);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -404,7 +404,7 @@ NtRegDeleteValueA(
     BAIL_ON_NT_INVALID_STRING(pszValueName);
 
     status = LwRtlWC16StringAllocateFromCString(&pwszValueName,
-		                                    pszValueName);
+    		                                    pszValueName);
     BAIL_ON_NT_STATUS(status);
 
     status = RegTransactDeleteValueW(
@@ -471,8 +471,8 @@ NtRegEnumKeyExA(
     {
         if (*pcClass == 0)
         {
-		status = STATUS_BUFFER_TOO_SMALL;
-		BAIL_ON_NT_STATUS(status);
+    		status = STATUS_BUFFER_TOO_SMALL;
+    		BAIL_ON_NT_STATUS(status);
         }
 
         status = LW_RTL_ALLOCATE(&pwszClass, WCHAR, sizeof(*pwszClass)*(*pcClass));
@@ -558,7 +558,7 @@ NtRegEnumValueA(
     )
 {
 	NTSTATUS status = 0;
-    DWORD dwType = REG_UNKNOWN;
+    DWORD dwType = REG_NONE;
     PWSTR pwszValueName = NULL;
     PSTR pszTempValueName = NULL;
     PVOID pTempData = NULL;
@@ -574,8 +574,8 @@ NtRegEnumValueA(
 
     if (pData && !pcbData)
     {
-	status = STATUS_INVALID_PARAMETER;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_PARAMETER;
+    	BAIL_ON_NT_STATUS(status);
     }
     dwValueNameLen = *pcchValueName;
 
@@ -584,19 +584,19 @@ NtRegEnumValueA(
 
     if (pData && pcbData)
     {
-	status = RtlSafeMultiplyULONG(&cTempData, *pcbData, sizeof(WCHAR));
-	BAIL_ON_NT_STATUS(status);
+    	status = RtlSafeMultiplyULONG(&cTempData, *pcbData, sizeof(WCHAR));
+    	BAIL_ON_NT_STATUS(status);
 
-	if (cTempData > MAX_VALUE_LENGTH)
-	{
-		cTempData = MAX_VALUE_LENGTH;
-	}
+    	if (cTempData > MAX_VALUE_LENGTH)
+    	{
+    		cTempData = MAX_VALUE_LENGTH;
+    	}
 
-	if (cTempData)
-	{
+    	if (cTempData)
+    	{
             status = LW_RTL_ALLOCATE(&pTempData, VOID, cTempData);
             BAIL_ON_NT_STATUS(status);
-	}
+    	}
     }
 
 	status = RegTransactEnumValueW(
@@ -621,9 +621,10 @@ NtRegEnumValueA(
     {
         cTempData *= sizeof(WCHAR);
         dwValueNameLen = *pcchValueName;
+        LWREG_SAFE_FREE_MEMORY(pTempData);
         status = LW_RTL_ALLOCATE(&pTempData, BYTE, cTempData);
         BAIL_ON_NT_STATUS(status);
-	status = RegTransactEnumValueW(
+    	status = RegTransactEnumValueW(
                     hRegConnection,
                     hKey,
                     dwIndex,
@@ -666,12 +667,12 @@ NtRegEnumValueA(
 
     if (pData)
     {
-	memcpy(pData, pValue, cTempData);
+    	memcpy(pData, pValue, cTempData);
     }
 
 done:
     status = LwRtlCStringAllocateFromWC16String(&pszTempValueName,
-		                                     (PCWSTR)pwszValueName);
+    		                                     (PCWSTR)pwszValueName);
     BAIL_ON_NT_STATUS(status);
 
     memcpy((PBYTE)pszValueName, (PBYTE)pszTempValueName, (strlen(pszTempValueName)+1));
@@ -684,7 +685,7 @@ done:
 
     if (pcbData)
     {
-	*pcbData = cTempData;
+    	*pcbData = cTempData;
     }
 
 cleanup:
@@ -698,7 +699,7 @@ cleanup:
 error:
     if (pdwType)
     {
-        *pdwType = REG_UNKNOWN;
+        *pdwType = REG_NONE;
     }
     if (pcbData)
     {
@@ -750,7 +751,7 @@ NtRegGetValueA(
     )
 {
 	NTSTATUS status = 0;
-    DWORD dwType = REG_UNKNOWN;
+    DWORD dwType = REG_NONE;
     PWSTR pwszSubKey = NULL;
     PWSTR pwszValueName = NULL;
     PVOID pTempData = NULL;
@@ -759,30 +760,30 @@ NtRegGetValueA(
 
     if (pvData && !pcbData)
     {
-	status = STATUS_INVALID_PARAMETER;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_PARAMETER;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     if (pszSubKey)
     {
-	status = LwRtlWC16StringAllocateFromCString(&pwszSubKey, pszSubKey);
-	BAIL_ON_NT_STATUS(status);
+    	status = LwRtlWC16StringAllocateFromCString(&pwszSubKey, pszSubKey);
+    	BAIL_ON_NT_STATUS(status);
     }
 
     if (pszValueName)
     {
-	status = LwRtlWC16StringAllocateFromCString(&pwszValueName, pszValueName);
-	BAIL_ON_NT_STATUS(status);
+    	status = LwRtlWC16StringAllocateFromCString(&pwszValueName, pszValueName);
+    	BAIL_ON_NT_STATUS(status);
     }
 
     if (pvData && pcbData)
     {
-	cTempData = (*pcbData)*sizeof(WCHAR);
+    	cTempData = (*pcbData)*sizeof(WCHAR);
 
-	if (cTempData > MAX_VALUE_LENGTH)
-	{
-		cTempData = MAX_VALUE_LENGTH;
-	}
+    	if (cTempData > MAX_VALUE_LENGTH)
+    	{
+    		cTempData = MAX_VALUE_LENGTH;
+    	}
 
 	    status = LW_RTL_ALLOCATE(&pTempData, VOID, cTempData*sizeof(WCHAR));
 	    BAIL_ON_NT_STATUS(status);
@@ -806,7 +807,7 @@ NtRegGetValueA(
         LWREG_SAFE_FREE_MEMORY(pTempData);
         status = LW_RTL_ALLOCATE(&pTempData, BYTE, cTempData);
         BAIL_ON_NT_STATUS(status);
-	status = RegTransactGetValueW(
+    	status = RegTransactGetValueW(
                     hRegConnection,
                     hKey,
                     pwszSubKey,
@@ -848,7 +849,7 @@ NtRegGetValueA(
 
     if (pvData)
     {
-	memcpy(pvData, pValue, cTempData);
+    	memcpy(pvData, pValue, cTempData);
     }
 
 done:
@@ -859,7 +860,7 @@ done:
 
     if (pcbData)
     {
-	*pcbData = cTempData;
+    	*pcbData = cTempData;
     }
 
 cleanup:
@@ -873,7 +874,7 @@ cleanup:
 error:
     if (pdwType)
     {
-        *pdwType = REG_UNKNOWN;
+        *pdwType = REG_NONE;
     }
 
     if (pcbData)
@@ -926,7 +927,7 @@ NtRegOpenKeyExA(
     if (pszSubKey)
     {
         status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-			                                     pszSubKey);
+        		                                     pszSubKey);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -1001,8 +1002,8 @@ NtRegQueryInfoKeyA(
     {
         if (*pcClass == 0)
         {
-		status = STATUS_BUFFER_TOO_SMALL;
-		BAIL_ON_NT_STATUS(status);
+    		status = STATUS_BUFFER_TOO_SMALL;
+    		BAIL_ON_NT_STATUS(status);
         }
 
 	    status = LW_RTL_ALLOCATE(&pwszClass, WCHAR, sizeof(*pwszClass)*(*pcClass));
@@ -1051,7 +1052,7 @@ NtRegQueryInfoKeyA(
 		                        &cbData);
 		BAIL_ON_NT_STATUS(status);
 
-	if (REG_MULTI_SZ == dwType || REG_SZ == dwType)
+    	if (REG_MULTI_SZ == dwType || REG_SZ == dwType)
         {
             cbData *= LW_WC2MBS_FACTOR_MAX;
         }
@@ -1211,7 +1212,7 @@ NtRegSetValueExA(
     if (pszValueName)
     {
         status = LwRtlWC16StringAllocateFromCString(&pwszValueName,
-			                                     pszValueName);
+        		                                     pszValueName);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -1235,11 +1236,11 @@ NtRegSetValueExA(
                                                           &pOutData,
                                                           &cbOutDataLen);
                 BAIL_ON_NT_STATUS(status);
-
+ 
                 bIsStrType = TRUE;
         }
         else if (REG_SZ == dwType)
-	{
+    	{
             /* Verify correct null termination of input data */
             if (strlen((char *) pData) != (cbData-1))
             {
@@ -1247,13 +1248,13 @@ NtRegSetValueExA(
                 BAIL_ON_NT_STATUS(status);
             }
 
-		status = LwRtlWC16StringAllocateFromCString((PWSTR*)&pOutData,
-				                                     (PCSTR)pData);
-		BAIL_ON_NT_STATUS(status);
+    		status = LwRtlWC16StringAllocateFromCString((PWSTR*)&pOutData,
+    				                                     (PCSTR)pData);
+    		BAIL_ON_NT_STATUS(status);
 
-                cbOutDataLen = (mbstrlen((const char*) pData)+1) * sizeof(WCHAR);
-		bIsStrType = TRUE;
-	}
+            cbOutDataLen = (mbstrlen((const char*) pData)+1) * sizeof(WCHAR);
+    		bIsStrType = TRUE;
+    	}
     }
 
     status = RegTransactSetValueExW(
@@ -1320,12 +1321,12 @@ NtRegSetKeySecurity(
 	)
 {
     return RegTransactSetKeySecurity(
-	    hNtRegConnection,
-	    hKey,
-	    SecurityInformation,
-	    pSecDescRel,
-	    pSecDescLen
-	    );
+    	    hNtRegConnection,
+    	    hKey,
+    	    SecurityInformation,
+    	    pSecDescRel,
+    	    pSecDescLen
+    	    );
 }
 
 REG_API
@@ -1334,16 +1335,119 @@ NtRegGetKeySecurity(
     IN HANDLE hNtRegConnection,
     IN HKEY hKey,
     IN SECURITY_INFORMATION SecurityInformation,
-    OUT PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel,
+    OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel,
     IN OUT PULONG pulSecDescLen
     )
 {
 	return RegTransactGetKeySecurity(
-		    hNtRegConnection,
-		    hKey,
-		    SecurityInformation,
-		    pSecDescRel,
-		    pulSecDescLen
-		    );
+	    	    hNtRegConnection,
+	    	    hKey,
+	    	    SecurityInformation,
+	    	    pSecDescRel,
+	    	    pulSecDescLen
+	    	    );
+}
+
+
+NTSTATUS
+NtRegSetValueAttributesA(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCSTR pszSubKey,
+    IN PCSTR pszValueName,
+    IN PLWREG_VALUE_ATTRIBUTES_A pValueAttributes
+    )
+{
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+NTSTATUS
+NtRegSetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    )
+{
+    NTSTATUS status = 0;
+
+    if (!RegValidValueAttributes(pValueAttributes))
+    {
+        status = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    status = RegTransactSetValueAttributesW(
+            hRegConnection,
+            hKey,
+            pSubKey,
+            pValueName,
+            pValueAttributes);
+    BAIL_ON_NT_STATUS(status);
+
+error:
+
+    return status;
+}
+
+NTSTATUS
+NtRegGetValueAttributesA(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCSTR pszSubKey,
+    IN PCSTR pszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES_A* ppValueAttributes
+    )
+{
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+NTSTATUS
+NtRegGetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    )
+{
+    return RegTransactGetValueAttributesW(
+        hRegConnection,
+        hKey,
+        pwszSubKey,
+        pwszValueName,
+        ppCurrentValue,
+        ppValueAttributes
+        );
+}
+
+NTSTATUS
+NtRegDeleteValueAttributesA(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCSTR pszSubKey,
+    IN PCSTR pszValueName
+    )
+{
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+NTSTATUS
+NtRegDeleteValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName
+    )
+{
+    return RegTransactDeleteValueAttributesW(
+            hRegConnection,
+            hKey,
+            pwszSubKey,
+            pwszValueName
+            );
 }
 
