@@ -53,84 +53,11 @@
 #include <lw/types.h>
 #include <lw/attrs.h>
 #include <lw/security-types.h>
-/*
- * Logging
- */
-typedef enum
-{
-    REG_LOG_LEVEL_ALWAYS = 0,
-    REG_LOG_LEVEL_ERROR,
-    REG_LOG_LEVEL_WARNING,
-    REG_LOG_LEVEL_INFO,
-    REG_LOG_LEVEL_VERBOSE,
-    REG_LOG_LEVEL_DEBUG,
-    REG_LOG_LEVEL_TRACE
-} RegLogLevel;
-
-typedef enum
-{
-    REG_LOG_TARGET_DISABLED = 0,
-    REG_LOG_TARGET_CONSOLE,
-    REG_LOG_TARGET_FILE,
-    REG_LOG_TARGET_SYSLOG
-} RegLogTarget;
-
-typedef VOID (*PFN_REG_LOG_MESSAGE)(
-                    HANDLE      hLog,
-                    RegLogLevel logLevel,
-                    PCSTR       pszFormat,
-                    va_list     msgList
-                    );
-
-typedef struct __REG_LOG_INFO {
-    RegLogLevel  maxAllowedLogLevel;
-    RegLogTarget logTarget;
-    PSTR         pszPath;
-} REG_LOG_INFO, *PREG_LOG_INFO;
 
 typedef struct __REG_KEY_HANDLE *HKEY, **PHKEY;
 
 typedef ACCESS_MASK REGSAM;
 
-#define KEY_QUERY_VALUE         0x0001 //Required to query the values of a registry key
-#define KEY_SET_VALUE           0x0002 //Required to create, delete, or set a registry value
-#define KEY_CREATE_SUB_KEY      0x0004 //Required to create, delete, or rename a subkey of a registry key
-#define KEY_ENUMERATE_SUB_KEYS  0x0008 //Required to enumerate the subkeys of a registry key
-#define KEY_NOTIFY              0x0010 //Required to request change notifications for a registry key or for subkeys of a registry key.
-#define KEY_CREATE_LINK         0x0020 //Reserved for system use
-
-
-#define KEY_ALL_ACCESS ( \
-	    (~SYNCHRONIZE) & \
-	    (STANDARD_RIGHTS_REQUIRED | \
-	    KEY_QUERY_VALUE |\
-		KEY_SET_VALUE |\
-		KEY_CREATE_SUB_KEY |\
-		KEY_ENUMERATE_SUB_KEYS |\
-		KEY_NOTIFY |\
-		KEY_CREATE_LINK) \
-		)
-
-#define KEY_READ ( \
-		(~SYNCHRONIZE) & \
-	    (STANDARD_RIGHTS_READ | \
-	    KEY_QUERY_VALUE |\
-	    KEY_ENUMERATE_SUB_KEYS |\
-	    KEY_NOTIFY) \
-		)
-
-
-#define KEY_WRITE ( \
-		(~SYNCHRONIZE) & \
-	    (STANDARD_RIGHTS_WRITE | \
-	    KEY_SET_VALUE |\
-	    KEY_CREATE_SUB_KEY) \
-		)
-
-#define KEY_EXECUTE ( \
-		(~SYNCHRONIZE) & \
-        (KEY_READ)\
-        )
 
 typedef DWORD REG_DATA_TYPE;
 typedef DWORD *PREG_DATA_TYPE;
@@ -154,7 +81,8 @@ typedef DWORD *PREG_DATA_TYPE;
 #define REG_KEY                            21 // represent the reg entry is a Key
 #define REG_KEY_DEFAULT                    22 // Default "@" entry
 #define REG_PLAIN_TEXT                     23 // A string without "" around it
-#define REG_UNKNOWN                        24 // Unknown data type
+#define REG_SZA                            24 // Alias for REG_MULTI_SZ
+#define REG_ATTRIBUTES                     25 // Registry schema type
 
 
 typedef DWORD REG_DATA_TYPE_FLAGS;
@@ -203,6 +131,67 @@ typedef struct _SECURITY_ATTRIBUTES {
     BOOL   bInheritHandle;
 }SECURITY_ATTRIBUTES, *PSECURITY_ATTRIBUTES;
 
+//
+// Value attribute
+//
+typedef enum _LWREG_VALUE_RANGE_TYPE
+{
+    LWREG_VALUE_RANGE_TYPE_NONE = 0,
+    LWREG_VALUE_RANGE_TYPE_BOOLEAN,
+    LWREG_VALUE_RANGE_TYPE_ENUM,
+    LWREG_VALUE_RANGE_TYPE_INTEGER,
+} LWREG_VALUE_RANGE_TYPE;
+
+typedef enum _LWREG_VALUE_HINT
+{
+    LWREG_VALUE_HINT_NONE = 0,
+    LWREG_VALUE_HINT_SECONDS,
+    LWREG_VALUE_HINT_PATH,
+    LWREG_VALUE_HINT_ACCOUNT
+} LWREG_VALUE_HINT;
+
+typedef struct _LWREG_RANGE_INTEGER
+{
+    DWORD Min;
+    DWORD Max;
+} LWREG_RANGE_INTEGER;
+
+typedef struct _LWREG_VALUE_ATTRIBUTES_A
+{
+    REG_DATA_TYPE ValueType;
+    PVOID pDefaultValue;
+    DWORD DefaultValueLen;
+    PSTR  pszDocString;
+    LWREG_VALUE_RANGE_TYPE RangeType;
+    LWREG_VALUE_HINT Hint;
+    union _LWREG_RANGE_A{
+        LWREG_RANGE_INTEGER RangeInteger;
+        PSTR* ppszRangeEnumStrings;
+    } Range;
+} LWREG_VALUE_ATTRIBUTES_A, *PLWREG_VALUE_ATTRIBUTES_A;
+
+
+typedef struct _LWREG_VALUE_ATTRIBUTES
+{
+    REG_DATA_TYPE ValueType;
+    PVOID pDefaultValue;
+    DWORD DefaultValueLen;
+    PWSTR pwszDocString;
+    LWREG_VALUE_RANGE_TYPE RangeType;
+    LWREG_VALUE_HINT Hint;
+    union _LWREG_RANGE{
+        LWREG_RANGE_INTEGER RangeInteger;
+        PWSTR* ppwszRangeEnumStrings;
+    } Range;
+} LWREG_VALUE_ATTRIBUTES, *PLWREG_VALUE_ATTRIBUTES;
+
+typedef struct _LWREG_CURRENT_VALUEINFO{
+    DWORD dwType;
+    PVOID pvData;
+    DWORD cbData;
+}LWREG_CURRENT_VALUEINFO, *PLWREG_CURRENT_VALUEINFO;
+
+
 void
 RegFreeMultiStrsA(
     PSTR* ppszStrings
@@ -226,7 +215,10 @@ RegCopyValueBytes(
     IN OUT OPTIONAL PDWORD pcbData
     );
 
-#endif /* __REG_H__ */
+BOOLEAN
+RegValidValueAttributes(
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    );
 
 
 /*Range 40700 - 41200 is reserved for registry specific error*/
@@ -243,6 +235,8 @@ RegCopyValueBytes(
 #define LWREG_ERROR_NO_SECURITY_ON_KEY                        40710
 #define LWREG_ERROR_INVALID_SECURITY_DESCR                    40711
 #define LWREG_ERROR_INVALID_ACCESS_TOKEN                      40712
+#define LWREG_ERROR_DELETE_DEFAULT_VALUE_NOT_ALLOWED          40713
+
 
 #define LWREG_ERROR_INVALID_CACHE_PATH                        40720 //LWERROR VALUE 40001
 #define LWREG_ERROR_INVALID_PREFIX_PATH                       40721 //40003
@@ -252,6 +246,11 @@ RegCopyValueBytes(
 #define LWREG_ERROR_NOT_HANDLED                               40725 //40017
 #define LWREG_ERROR_UNEXPECTED_TOKEN                          40726 //40062
 #define LWREG_ERROR_UNKNOWN                                   40727 //40188
+#define LWREG_ERROR_SYNTAX                                    40728 
+#define LWREG_ERROR_PARSE                                     40729
+
+#endif /* __REG_H__ */
+
 
 /*
 local variables:
