@@ -122,7 +122,7 @@ RegUtilSetValue(
     IN PCSTR pszKeyName,
     IN PCSTR pszValueName,
     IN REG_DATA_TYPE valueType,
-    IN PCVOID pData,
+    IN PVOID pData,
     IN DWORD dwDataLen
     );
 
@@ -136,6 +136,16 @@ RegUtilGetValues(
     OUT PDWORD pdwValueArrayLen
     );
 
+DWORD
+RegUtilGetKeyObjectCounts(
+    IN OPTIONAL HANDLE hReg,
+    IN PSTR pszRootKeyName,
+    IN PSTR pszSubKeyPath,
+    IN PSTR keyName,
+    OUT OPTIONAL PDWORD pdwSubKeysCount,
+    OUT OPTIONAL PDWORD pdwValuesCount
+    );
+
 DWORD
 RegUtilDeleteValue(
     IN OPTIONAL HANDLE hReg,
