@@ -224,6 +224,39 @@ typedef struct __REG_HASH_ITERATOR
 /*
  * Logging
  */
+
+typedef enum
+{
+    REG_LOG_LEVEL_ALWAYS = 0,
+    REG_LOG_LEVEL_ERROR,
+    REG_LOG_LEVEL_WARNING,
+    REG_LOG_LEVEL_INFO,
+    REG_LOG_LEVEL_VERBOSE,
+    REG_LOG_LEVEL_DEBUG,
+    REG_LOG_LEVEL_TRACE
+} RegLogLevel;
+
+typedef enum
+{
+    REG_LOG_TARGET_DISABLED = 0,
+    REG_LOG_TARGET_CONSOLE,
+    REG_LOG_TARGET_FILE,
+    REG_LOG_TARGET_SYSLOG
+} RegLogTarget;
+
+typedef struct __REG_LOG_INFO {
+    RegLogLevel  maxAllowedLogLevel;
+    RegLogTarget logTarget;
+    PSTR         pszPath;
+} REG_LOG_INFO, *PREG_LOG_INFO;
+
+typedef VOID (*PFN_REG_LOG_MESSAGE)(
+                    HANDLE      hLog,
+                    RegLogLevel logLevel,
+                    PCSTR       pszFormat,
+                    va_list     msgList
+                    );
+
 #if defined(LW_ENABLE_THREADS)
 
 extern pthread_mutex_t gLogLock;
@@ -793,6 +826,16 @@ NtRegReallocMemory(
     IN DWORD dwSize
     );
 
+void
+RegSafeFreeValueAttributes(
+    PLWREG_VALUE_ATTRIBUTES* ppValueAttrs
+    );
+
+void
+RegSafeFreeCurrentValueInfo(
+    PLWREG_CURRENT_VALUEINFO* ppValueInfo
+    );
+
 void
 RegFreeString(
     PSTR pszString
@@ -810,6 +853,11 @@ RegFreeWC16StringArray(
     DWORD dwCount
     );
 
+void
+RegFreeWC16StringArrayWithNullTerminator(
+    PWSTR * ppwStringArray
+    );
+
 void
 RegFreeValueByteArray(
     PBYTE* ppValues,
@@ -955,4 +1003,27 @@ RegGetErrorMessageForLoggingEvent(
     LW_PSTR* ppszErrorMsg
     );
 
+
+DWORD
+RegWC16StringArraysAllocateFromCStringArraysWithNullTerminator(
+    IN PSTR* ppszStrings,
+    OUT PWSTR** pppwszStrings
+    );
+
+DWORD
+RegCopyValueAToW(
+    IN REG_DATA_TYPE dwType,
+    IN PVOID pData,
+    IN DWORD cbData,
+    OUT PVOID* ppOutData,
+    OUT PDWORD pcbOutDataLen
+    );
+
+DWORD
+RegConvertValueAttributesAToW(
+    LWREG_VALUE_ATTRIBUTES_A attrA,
+    PLWREG_VALUE_ATTRIBUTES* ppAttrW
+    );
+
+
 #endif /* __REG_UTILS_H__ */
