@@ -567,12 +567,8 @@ static LWMsgTypeSpec gRegSetKeyValueSpec[] =
     LWMSG_MEMBER_PWSTR(REG_IPC_SET_KEY_VALUE_REQ, pValueName),
 
     LWMSG_MEMBER_UINT32(REG_IPC_SET_KEY_VALUE_REQ, dwType),
-
     LWMSG_MEMBER_UINT32(REG_IPC_SET_KEY_VALUE_REQ, cbData),
-
-    LWMSG_MEMBER_POINTER_BEGIN(REG_IPC_SET_KEY_VALUE_REQ, pData),
-    LWMSG_UINT8(BYTE),
-    LWMSG_POINTER_END,
+    LWMSG_MEMBER_POINTER(REG_IPC_SET_KEY_VALUE_REQ, pData, LWMSG_UINT8(BYTE)),
     LWMSG_ATTR_LENGTH_MEMBER(REG_IPC_SET_KEY_VALUE_REQ, cbData),
 
     LWMSG_STRUCT_END,
@@ -644,6 +640,8 @@ static LWMsgTypeSpec gRegGetKeySecuritySpec[] =
 
     LWMSG_MEMBER_UINT32(REG_IPC_GET_KEY_SECURITY_REQ, Length),
 
+    LWMSG_MEMBER_UINT8(REG_IPC_GET_KEY_SECURITY_REQ, bRetSecurityDescriptor),
+
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
@@ -663,6 +661,154 @@ static LWMsgTypeSpec gRegGetKeySecurityResp[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gRegValueRangeInteger[] =
+{
+        LWMSG_STRUCT_BEGIN(LWREG_RANGE_INTEGER),
+
+        LWMSG_MEMBER_UINT32(LWREG_RANGE_INTEGER, Min),
+        LWMSG_MEMBER_UINT32(LWREG_RANGE_INTEGER, Max),
+
+        LWMSG_STRUCT_END,
+        LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec gRegValueAttributesSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LWREG_VALUE_ATTRIBUTES),
+
+    LWMSG_MEMBER_UINT32(LWREG_VALUE_ATTRIBUTES, ValueType),
+
+    LWMSG_MEMBER_UINT32(LWREG_VALUE_ATTRIBUTES, DefaultValueLen),
+    LWMSG_MEMBER_PBYTE(LWREG_VALUE_ATTRIBUTES, pDefaultValue),
+    LWMSG_ATTR_LENGTH_MEMBER(LWREG_VALUE_ATTRIBUTES, DefaultValueLen),
+
+    LWMSG_MEMBER_PWSTR(LWREG_VALUE_ATTRIBUTES, pwszDocString),
+
+    LWMSG_MEMBER_UINT8(LWREG_VALUE_ATTRIBUTES, RangeType),
+    LWMSG_MEMBER_UINT8(LWREG_VALUE_ATTRIBUTES, Hint),
+
+    // range union
+    LWMSG_MEMBER_UNION_BEGIN(LWREG_VALUE_ATTRIBUTES, Range),
+    LWMSG_MEMBER_TYPESPEC(union _LWREG_RANGE, RangeInteger, gRegValueRangeInteger),
+    LWMSG_ATTR_TAG(LWREG_VALUE_RANGE_TYPE_INTEGER),
+    LWMSG_MEMBER_POINTER_BEGIN(union _LWREG_RANGE, ppwszRangeEnumStrings),
+    LWMSG_PWSTR,
+    LWMSG_POINTER_END,
+    LWMSG_ATTR_ZERO_TERMINATED,
+    LWMSG_ATTR_TAG(LWREG_VALUE_RANGE_TYPE_ENUM),
+    LWMSG_MEMBER_VOID(union _LWREG_RANGE, bool_empty), // can be any arbitrary name
+    LWMSG_ATTR_TAG(LWREG_VALUE_RANGE_TYPE_BOOLEAN),
+    LWMSG_MEMBER_VOID(union _LWREG_RANGE, empty), // can be any arbitrary name
+    LWMSG_ATTR_TAG(LWREG_VALUE_RANGE_TYPE_NONE),
+    LWMSG_UNION_END,
+    LWMSG_ATTR_DISCRIM(LWREG_VALUE_ATTRIBUTES, RangeType),
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec gRegSetValueAttrsSpec[] =
+{
+    // HKEY hKey
+    // PCWSTR pSubKey
+    // PCWSTR pValueName
+    // PLWREG_VALUE_ATTRIBUTES pValueAttributes
+
+    LWMSG_STRUCT_BEGIN(REG_IPC_SET_VALUE_ATTRS_REQ),
+
+    LWMSG_MEMBER_HANDLE(REG_IPC_SET_VALUE_ATTRS_REQ, hKey, HKEY),
+    LWMSG_ATTR_HANDLE_LOCAL_FOR_RECEIVER,
+
+    LWMSG_MEMBER_PWSTR(REG_IPC_SET_VALUE_ATTRS_REQ, pSubKey),
+    LWMSG_MEMBER_PWSTR(REG_IPC_SET_VALUE_ATTRS_REQ, pValueName),
+    LWMSG_MEMBER_POINTER_BEGIN(REG_IPC_SET_VALUE_ATTRS_REQ, pValueAttributes),
+    LWMSG_TYPESPEC(gRegValueAttributesSpec),
+    LWMSG_POINTER_END,
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+
+static LWMsgTypeSpec gRegDeleteValueAttrsSpec[] =
+{
+    // HKEY hKey
+    // PCWSTR pSubKey
+    // PCWSTR pValueName
+
+    LWMSG_STRUCT_BEGIN(REG_IPC_DELETE_VALUE_ATTRS_REQ),
+
+    LWMSG_MEMBER_HANDLE(REG_IPC_DELETE_VALUE_ATTRS_REQ, hKey, HKEY),
+    LWMSG_ATTR_HANDLE_LOCAL_FOR_RECEIVER,
+
+    LWMSG_MEMBER_PWSTR(REG_IPC_DELETE_VALUE_ATTRS_REQ, pSubKey),
+    LWMSG_MEMBER_PWSTR(REG_IPC_DELETE_VALUE_ATTRS_REQ, pValueName),
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+
+static LWMsgTypeSpec gRegGetValueAttrsSpec[] =
+{
+    // HKEY hKey
+    // PCWSTR pSubKey
+    // PCWSTR pValueName
+    // BOOLEAN bRetCurrentValue;
+    // BOOLEAN bRetValueAttributes
+
+    LWMSG_STRUCT_BEGIN(REG_IPC_GET_VALUE_ATTRS_REQ),
+
+    LWMSG_MEMBER_HANDLE(REG_IPC_GET_VALUE_ATTRS_REQ, hKey, HKEY),
+    LWMSG_ATTR_HANDLE_LOCAL_FOR_RECEIVER,
+
+    LWMSG_MEMBER_PWSTR(REG_IPC_GET_VALUE_ATTRS_REQ, pSubKey),
+    LWMSG_MEMBER_PWSTR(REG_IPC_GET_VALUE_ATTRS_REQ, pValueName),
+    LWMSG_MEMBER_UINT8(REG_IPC_GET_VALUE_ATTRS_REQ, bRetCurrentValue),
+    LWMSG_MEMBER_UINT8(REG_IPC_GET_VALUE_ATTRS_REQ, bRetValueAttributes),
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+
+static LWMsgTypeSpec gRegCurrentValueInfo[] =
+{
+    // DWORD dwType
+    // PVOID pvData
+    // DWORD cbData
+
+    LWMSG_STRUCT_BEGIN(LWREG_CURRENT_VALUEINFO),
+
+    LWMSG_MEMBER_UINT32(LWREG_CURRENT_VALUEINFO, dwType),
+
+    LWMSG_MEMBER_UINT32(LWREG_CURRENT_VALUEINFO, cbData),
+    LWMSG_MEMBER_PBYTE(LWREG_CURRENT_VALUEINFO, pvData),
+    LWMSG_ATTR_LENGTH_MEMBER(LWREG_CURRENT_VALUEINFO, cbData),
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+
+static LWMsgTypeSpec gRegGetValueAttrsResp[] =
+{
+    // PLWREG_CURRENT_VALUEINFO pCurrentValue
+    // PLWREG_VALUE_ATTRIBUTES pValueAttributes
+
+    LWMSG_STRUCT_BEGIN(REG_IPC_GET_VALUE_ATTRS_RESPONSE),
+
+    LWMSG_MEMBER_POINTER_BEGIN(REG_IPC_GET_VALUE_ATTRS_RESPONSE, pCurrentValue),
+    LWMSG_TYPESPEC(gRegCurrentValueInfo),
+    LWMSG_POINTER_END,
+
+    LWMSG_MEMBER_POINTER_BEGIN(REG_IPC_GET_VALUE_ATTRS_RESPONSE, pValueAttributes),
+    LWMSG_TYPESPEC(gRegValueAttributesSpec),
+    LWMSG_POINTER_END,
+
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
 
 
 /******************************************************************************/
@@ -707,6 +853,14 @@ static LWMsgProtocolSpec gRegIPCSpec[] =
     LWMSG_MESSAGE(REG_R_SET_KEY_SECURITY, NULL),
     LWMSG_MESSAGE(REG_Q_GET_KEY_SECURITY, gRegGetKeySecuritySpec),
     LWMSG_MESSAGE(REG_R_GET_KEY_SECURITY, gRegGetKeySecurityResp),
+    /*Value Attributes Operation APIs*/
+    LWMSG_MESSAGE(REG_Q_SET_VALUEW_ATTRIBUTES, gRegSetValueAttrsSpec),
+    LWMSG_MESSAGE(REG_R_SET_VALUEW_ATTRIBUTES, NULL),
+    LWMSG_MESSAGE(REG_Q_GET_VALUEW_ATTRIBUTES, gRegGetValueAttrsSpec),
+    LWMSG_MESSAGE(REG_R_GET_VALUEW_ATTRIBUTES, gRegGetValueAttrsResp),
+    LWMSG_MESSAGE(REG_Q_DELETE_VALUEW_ATTRIBUTES, gRegDeleteValueAttrsSpec),
+    LWMSG_MESSAGE(REG_R_DELETE_VALUEW_ATTRIBUTES, NULL),
+
     LWMSG_PROTOCOL_END
 };
 
@@ -725,9 +879,6 @@ RegMapLwmsgStatus(
 {
     switch (status)
     {
-    default:
-        return STATUS_INTERNAL_ERROR;
-
     case LWMSG_STATUS_SUCCESS:
         return STATUS_SUCCESS;
 
@@ -774,6 +925,12 @@ RegMapLwmsgStatus(
 
     case LWMSG_STATUS_SESSION_LOST:
         return LwErrnoToNtStatus(EPIPE);
+
+    case LWMSG_STATUS_NOT_FOUND:
+        return STATUS_NOT_FOUND;
+
+    default:
+        return STATUS_INTERNAL_ERROR;
     }
 }
 
