@@ -65,7 +65,10 @@ RegLexOpen(
         RegLexParseOpenBracket;
     pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX(']')]  =
         RegLexParseCloseBracket;
+    pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('{')]  = RegLexParseOpenBrace;
+    pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('}')]  = RegLexParseCloseBrace;
     pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('"')]  = RegLexParseQuote;
+    pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('-')]  = RegLexParseDash;
     pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('@')]  = RegLexParseAt;
     pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX('=')]  = RegLexParseEquals;
     pLexHandle->parseFuncArray[REGLEX_CHAR_INDEX(',')]  = RegLexParseComma;
@@ -82,6 +85,7 @@ cleanup:
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pLexHandle);
     goto cleanup;
 }
 
@@ -104,7 +108,8 @@ RegLexClose(
 
 DWORD RegLexTokenToString(
     REGLEX_TOKEN token,
-    PSTR tokenStr)
+    PSTR tokenStr,
+    DWORD dwTokenStrLen)
 {
     static char *tokenStrs[] = {
         "REGLEX_FIRST",
@@ -132,14 +137,20 @@ DWORD RegLexTokenToString(
         "REGLEX_REG_KEY",
         "REGLEX_REG_NAME_DEFAULT",
         "REGLEX_REG_STRING_ARRAY",                       /* sza:      */
+        "REGLEX_ATTRIBUTES",                             /* { attributes } */
+        "REGLEX_ATTRIBUTES_START",                       /* { attributes start */
+        "REGLEX_ATTRIBUTES_END",                         /* attributes end } */
+        "REGLEX_REG_INTEGER_RANGE",                      /* integer:M - N */
+        "REGLEX_DASH",                                   /* Integer range values separator */
     };
     if (token < (sizeof(tokenStrs)/sizeof(char *)))
     {
-        strcpy(tokenStr, tokenStrs[token]);
+        tokenStr[0] = '\0';
+        strncat(tokenStr, tokenStrs[token], dwTokenStrLen-1);
     }
     else
     {
-        sprintf(tokenStr, "ERROR: No Such Token %d", token);
+        snprintf(tokenStr, dwTokenStrLen, "ERROR: No Such Token %d", token);
     }
 
     return 0;
@@ -191,10 +202,10 @@ DWORD RegLexAppendChar(
     CHAR inC)
 {
     DWORD dwError = 0;
-    BAIL_ON_INVALID_HANDLE(lexHandle);
     PVOID pNewMemory = NULL;
     DWORD newValueSize = 0;
 
+    BAIL_ON_INVALID_HANDLE(lexHandle);
     if (lexHandle->curToken.valueCursor >= lexHandle->curToken.valueSize)
     {
         newValueSize = lexHandle->curToken.valueSize * 2;
@@ -217,6 +228,7 @@ cleanup:
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pNewMemory);
     goto cleanup;
 }
 
@@ -232,6 +244,7 @@ RegLexParseQuote(
     if (lexHandle->curToken.token == REGLEX_PLAIN_TEXT)
     {
         lexHandle->isToken = TRUE;
+        lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         dwError = RegIOUnGetChar(ioHandle, NULL);
         return dwError;
     }
@@ -277,6 +290,7 @@ RegLexParseOpenBracket(
     if (lexHandle->curToken.token == REGLEX_PLAIN_TEXT)
     {
         lexHandle->isToken = TRUE;
+        lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         dwError = RegIOUnGetChar(ioHandle, NULL);
         return dwError;
     }
@@ -287,6 +301,7 @@ RegLexParseOpenBracket(
         {
             /* This is a problem, can't have [ then another [ */
             dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         }
         else
         {
@@ -318,6 +333,7 @@ RegLexParseCloseBracket(
         {
             /* This is a problem, can't have ] without a previous [ */
             dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         }
         else
         {
@@ -334,6 +350,78 @@ RegLexParseCloseBracket(
     return dwError;
 }
 
+DWORD
+RegLexParseOpenBrace(
+    PREGLEX_ITEM lexHandle,
+    HANDLE ioHandle,
+    CHAR inC)
+{
+    DWORD dwError = 0;
+
+    if (lexHandle->state != REGLEX_STATE_IN_QUOTE &&
+        lexHandle->state != REGLEX_STATE_IN_KEY)
+    {
+        lexHandle->curToken.token = REGLEX_ATTRIBUTES_BEGIN;
+        if (lexHandle->eValueNameType == REGLEX_VALUENAME_ATTRIBUTES)
+        {
+            /* This is a problem, can't have { then another { */
+            dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+        }
+        else if (!lexHandle->curToken.pszValue ||
+                 !lexHandle->curToken.pszValue[0])
+        {
+            dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+        }
+        else
+        {
+            lexHandle->curToken.token = REGLEX_ATTRIBUTES_BEGIN;
+            lexHandle->eValueNameType = REGLEX_VALUENAME_ATTRIBUTES;
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+            lexHandle->curToken.valueCursor = 0;
+        }
+    }
+    else
+    {
+        RegLexAppendChar(lexHandle, inC);
+    }
+    return dwError;
+}
+
+DWORD
+RegLexParseCloseBrace(
+    PREGLEX_ITEM lexHandle,
+    HANDLE ioHandle,
+    CHAR inC)
+{
+    DWORD dwError = 0;
+
+    if (lexHandle->state != REGLEX_STATE_IN_QUOTE &&
+        lexHandle->state != REGLEX_STATE_IN_KEY)
+    {
+        if (lexHandle->eValueNameType != REGLEX_VALUENAME_ATTRIBUTES)
+        {
+            /* This is a problem, can't have } without a previous { */
+            dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+        }
+        else
+        {
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+            lexHandle->curToken.token = REGLEX_ATTRIBUTES_END;
+            lexHandle->state = REGLEX_STATE_INIT;
+        }
+    }
+    else
+    {
+        RegLexAppendChar(lexHandle, inC);
+    }
+    return dwError;
+}
+
 DWORD
 RegLexParseAt(
     PREGLEX_ITEM lexHandle,
@@ -341,6 +429,8 @@ RegLexParseAt(
     CHAR inC)
 {
     DWORD dwError = 0;
+    BOOLEAN eof = FALSE;
+    BOOLEAN bHasSecurity = FALSE;
 
     if (lexHandle->state != REGLEX_STATE_IN_QUOTE)
     {
@@ -351,10 +441,31 @@ RegLexParseAt(
         lexHandle->state = REGLEX_STATE_INIT;
         lexHandle->curToken.valueCursor = 0;
         RegLexAppendChar(lexHandle, inC);
-    }
-    else
-    {
-        RegLexAppendChar(lexHandle, inC);
+
+        /* Handle the case of @frob, i.e. "@security" */
+        dwError = RegIOGetChar(ioHandle, &inC, &eof);
+        while (dwError == 0 && !eof && isalpha((int) inC))
+        {
+            RegLexAppendChar(lexHandle, inC);
+            dwError = RegIOGetChar(ioHandle, &inC, &eof);
+            bHasSecurity = TRUE;
+        }
+        if (eof)
+        {
+            return dwError;
+        }
+        dwError = RegIOUnGetChar(ioHandle, NULL);
+        if (bHasSecurity)
+        {
+            if (strcmp(lexHandle->curToken.pszValue, "@security") == 0)
+            {
+                lexHandle->eValueNameType = REGLEX_VALUENAME_SECURITY;
+            }
+            else
+            {
+                dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            }
+        }
     }
     return dwError;
 }
@@ -371,9 +482,54 @@ RegLexParseEquals(
     if (lexHandle->state != REGLEX_STATE_IN_QUOTE &&
         lexHandle->state != REGLEX_STATE_IN_KEY)
     {
-        lexHandle->isToken = TRUE;
-        lexHandle->curToken.token = REGLEX_EQUALS;
-        lexHandle->curToken.valueCursor = 0;
+        if (lexHandle->curToken.valueCursor > 0)
+        {
+            dwError = RegIOUnGetChar(ioHandle, NULL);
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+        }
+        else
+        {
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+            lexHandle->curToken.token = REGLEX_EQUALS;
+            lexHandle->curToken.valueCursor = 0;
+        }
+    }
+    else
+    {
+        RegLexAppendChar(lexHandle, inC);
+    }
+    return dwError;
+}
+
+
+DWORD
+RegLexParseDash(
+    PREGLEX_ITEM lexHandle,
+    HANDLE ioHandle,
+    CHAR inC)
+{
+    DWORD dwError = 0;
+
+    if (lexHandle->state != REGLEX_STATE_IN_QUOTE &&
+        lexHandle->state != REGLEX_STATE_IN_KEY &&
+        lexHandle->eValueNameType != REGLEX_VALUENAME_SECURITY)
+    {
+        if (lexHandle->curToken.valueCursor > 0)
+        {
+            dwError = RegIOUnGetChar(ioHandle, NULL);
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+        }
+        else
+        {
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
+            lexHandle->curToken.token = REGLEX_DASH;
+            lexHandle->curToken.valueCursor = 0;
+            RegLexAppendChar(lexHandle, inC);
+        }
     }
     else
     {
@@ -397,12 +553,14 @@ RegLexParseComma(
         if (lexHandle->curToken.valueCursor > 0 && lexHandle->curToken.valueCursor <=2)
         {
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             lexHandle->curToken.token = REGLEX_HEXPAIR;
         }
         else
         {
             /* Syntax error: Hex pair is only 2 characters */
             dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         }
     }
     else
@@ -531,7 +689,9 @@ RegLexParseBinary(
             lexHandle->curToken.valueCursor = 0;
         }
         else if (strcasecmp(lexHandle->curToken.pszValue, "sza") == 0 ||
-                 strcasecmp(lexHandle->curToken.pszValue, "REG_STRING_ARRAY") == 0)
+                 strcasecmp(lexHandle->curToken.pszValue, "REG_STRING_ARRAY") == 0 ||
+                 (strcasecmp(lexHandle->curToken.pszValue, "string") == 0 &&
+                  lexHandle->eValueNameType == REGLEX_VALUENAME_ATTRIBUTES))
         {
             /* REG_STRING_ARRAY
              * Similar to REG_MULTI_SZ (token type returned will be MULTI_SZ).
@@ -582,6 +742,14 @@ RegLexParseBinary(
             lexHandle->state = REGLEX_STATE_BINHEX_STR;
             lexHandle->curToken.valueCursor = 0;
         }
+        else if (strcasecmp(lexHandle->curToken.pszValue, "integer") == 0)
+        {
+            /* integer:m-n range  */
+            lexHandle->isToken = TRUE;
+            lexHandle->curToken.token = REGLEX_REG_INTEGER_RANGE;
+            lexHandle->state = REGLEX_STATE_INTEGER_RANGE;
+            lexHandle->curToken.valueCursor = 0;
+        }
     }
 
     if (lexHandle->isToken)
@@ -601,7 +769,8 @@ RegLexParseColon(
     DWORD dwError = 0;
 
     if (lexHandle->state == REGLEX_STATE_IN_QUOTE ||
-        lexHandle->state == REGLEX_STATE_IN_KEY)
+        lexHandle->state == REGLEX_STATE_IN_KEY || 
+        lexHandle->eValueNameType == REGLEX_VALUENAME_SECURITY)
     {
         RegLexAppendChar(lexHandle, inC);
         return dwError;
@@ -674,6 +843,7 @@ RegLexParseNewline(
         }
 
         lexHandle->isToken = TRUE;
+        lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         lexHandle->curToken.token = REGLEX_HEXPAIR_END;
         lexHandle->state = REGLEX_STATE_INIT;
         return dwError;
@@ -681,6 +851,7 @@ RegLexParseNewline(
     else if (lexHandle->tokenDataType == REGLEX_REG_STRING_ARRAY)
     {
         lexHandle->isToken = TRUE;
+        lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         lexHandle->tokenDataType = REGLEX_FIRST;
         return dwError;
     }
@@ -689,16 +860,19 @@ RegLexParseNewline(
         if (lexHandle->curToken.valueCursor == 8)
         {
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             lexHandle->curToken.token = REGLEX_REG_DWORD;
             return dwError;
         }
         else
         {
             dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             return dwError;
         }
     }
-    else if (lexHandle->state == REGLEX_STATE_INIT &&
+    else if ((lexHandle->state == REGLEX_STATE_INIT ||
+              lexHandle->state == REGLEX_STATE_INTEGER_RANGE) &&
              lexHandle->curToken.valueCursor > 0)
     {
         /*
@@ -706,6 +880,7 @@ RegLexParseNewline(
          * has been found before the current end-of-line.
          */
         lexHandle->isToken = TRUE;
+        lexHandle->curToken.lineNum = lexHandle->parseLineNum - 1;
         lexHandle->curToken.token = REGLEX_PLAIN_TEXT;
         return dwError;
     }
@@ -767,6 +942,7 @@ RegLexParseWhitespace(
             lexHandle->curToken.valueCursor <= 2)
         {
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             lexHandle->curToken.token = REGLEX_HEXPAIR;
         }
     }
@@ -775,11 +951,13 @@ RegLexParseWhitespace(
         dwError = RegLexParseBinary(lexHandle);
         if (dwError || lexHandle->isToken)
         {
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             return dwError;
         }
         if (lexHandle->curToken.token != REGLEX_FIRST)
         {
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
         }
     }
     return dwError;
@@ -806,6 +984,7 @@ RegLexParseDefaultState(
         {
             /* Eat white spaces in binhex strings */
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             lexHandle->curToken.token = REGLEX_HEXPAIR;
             return dwError;
         }
@@ -813,12 +992,14 @@ RegLexParseDefaultState(
         if (!isxdigit((int)inC))
         {
             dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             return dwError;
         }
         if (lexHandle->curToken.valueCursor == 2)
         {
             dwError = RegIOUnGetChar(ioHandle, &inC);
             lexHandle->isToken = TRUE;
+            lexHandle->curToken.lineNum = lexHandle->parseLineNum;
             lexHandle->curToken.token = REGLEX_HEXPAIR;
             return dwError;
         }
@@ -856,8 +1037,7 @@ RegLexUnGetToken(PREGLEX_ITEM lexHandle)
     {
         if (lexHandle->prevToken.pszValue)
         {
-            RegMemoryFree(lexHandle->prevToken.pszValue);
-            lexHandle->prevToken.pszValue = NULL;
+            LWREG_SAFE_FREE_MEMORY(lexHandle->prevToken.pszValue);
         }
     }
     lexHandle->prevToken = lexHandle->curToken;
@@ -881,7 +1061,9 @@ RegLexResetToken(
     lexHandle->state = REGLEX_FIRST;
     lexHandle->tokenDataType = REGLEX_FIRST;
     lexHandle->isToken = FALSE;
+    LWREG_SAFE_FREE_MEMORY(lexHandle->curToken.pszValue);
     memset(&lexHandle->curToken, 0, sizeof(lexHandle->curToken));
+    lexHandle->prevToken.pszValue = NULL;
 
 cleanup:
     return dwError;
@@ -939,6 +1121,7 @@ RegLexGetToken(
 
     if (!lexHandle->curToken.pszValue)
     {   /* Extra byte for string termination */
+        LWREG_SAFE_FREE_MEMORY(lexHandle->curToken.pszValue);
         dwError = RegAllocateMemory(REGLEX_DEFAULT_SZ_LEN + 1,
                                     (LW_PVOID) &lexHandle->curToken.pszValue);
         BAIL_ON_REG_ERROR(dwError);
@@ -963,12 +1146,11 @@ RegLexGetToken(
             {
                 if (lexHandle->state == REGLEX_STATE_IN_QUOTE)
                 {
-                    dwError = LWREG_ERROR_INVALID_CONTEXT;
+                    dwError = LWREG_ERROR_UNEXPECTED_TOKEN;
                 }
                 else if (lexHandle->state == REGLEX_STATE_IN_KEY)
                 {
                     lexHandle->isToken = TRUE;
-                    lexHandle->curToken.lineNum = lexHandle->parseLineNum;
                     lexHandle->curToken.token = REGLEX_REG_KEY;
                     lexHandle->state = REGLEX_STATE_INIT;
                     *pRetToken = lexHandle->curToken.token;
