@@ -316,7 +316,7 @@ cleanup:
     {
         for (iCount=0; iCount<dwNumRootKeys; iCount++)
         {
-		LWREG_SAFE_FREE_MEMORY(ppwszRootKeyNames[iCount]);
+        	LWREG_SAFE_FREE_MEMORY(ppwszRootKeyNames[iCount]);
         }
         ppwszRootKeyNames = NULL;
     }
@@ -551,13 +551,13 @@ RegSrvIpcEnumKeyExW(
 
     if (pReq->cName)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pKeyName, WCHAR, pReq->cName*sizeof(*pKeyName));
+    	status = LW_RTL_ALLOCATE((PVOID*)&pKeyName, WCHAR, pReq->cName*sizeof(*pKeyName));
         BAIL_ON_NT_STATUS(status);
     }
 
     if (pReq->cClass)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pClassName, WCHAR, pReq->cClass*sizeof(*pClassName));
+    	status = LW_RTL_ALLOCATE((PVOID*)&pClassName, WCHAR, pReq->cClass*sizeof(*pClassName));
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -689,7 +689,7 @@ RegSrvIpcGetValueW(
 
     if (pReq->cbData)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pData, BYTE, pReq->cbData*sizeof(*pData));
+    	status = LW_RTL_ALLOCATE((PVOID*)&pData, BYTE, pReq->cbData*sizeof(*pData));
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -866,17 +866,17 @@ RegSrvIpcEnumValueW(
     PWSTR pValueName = NULL;
     PBYTE pValue = NULL;
     PREG_IPC_STATUS pStatus = NULL;
-    REG_DATA_TYPE type = REG_UNKNOWN;
+    REG_DATA_TYPE type = REG_NONE;
 
     if (pReq->cName)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pValueName, WCHAR, pReq->cName*sizeof(*pValueName));
+    	status = LW_RTL_ALLOCATE((PVOID*)&pValueName, WCHAR, pReq->cName*sizeof(*pValueName));
         BAIL_ON_NT_STATUS(status);
     }
 
     if (pReq->cValue)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pValue, BYTE, pReq->cValue*sizeof(*pValue));
+    	status = LW_RTL_ALLOCATE((PVOID*)&pValue, BYTE, pReq->cValue*sizeof(*pValue));
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -1036,10 +1036,10 @@ RegSrvIpcSetKeySecurity(
     PREG_IPC_STATUS pStatus = NULL;
 
     status = RegSrvSetKeySecurity(RegSrvIpcGetSessionData(pCall),
-		                      pReq->hKey,
-		                      pReq->SecurityInformation,
-		                      pReq->SecurityDescriptor,
-		                      pReq->Length);
+    		                      pReq->hKey,
+    		                      pReq->SecurityInformation,
+    		                      pReq->SecurityDescriptor,
+    		                      pReq->Length);
     if (!status)
     {
         pOut->tag = REG_R_SET_KEY_SECURITY;
@@ -1075,17 +1075,17 @@ RegSrvIpcGetKeySecurity(
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
     PREG_IPC_STATUS pStatus = NULL;
 
-    if (pReq->Length)
+    if (pReq->bRetSecurityDescriptor && pReq->Length)
     {
-	status = LW_RTL_ALLOCATE((PVOID*)&pSecDescRel, VOID, pReq->Length);
+    	status = LW_RTL_ALLOCATE((PVOID*)&pSecDescRel, VOID, pReq->Length);
         BAIL_ON_NT_STATUS(status);
     }
 
     status = RegSrvGetKeySecurity(RegSrvIpcGetSessionData(pCall),
-		                      pReq->hKey,
-		                      pReq->SecurityInformation,
-		                      pSecDescRel,
-		                      &pReq->Length);
+    		                      pReq->hKey,
+    		                      pReq->SecurityInformation,
+    		                      pSecDescRel,
+    		                      &pReq->Length);
 	if (!status)
 	{
 		status = LW_RTL_ALLOCATE((PVOID*)&pRegResp, REG_IPC_GET_KEY_SECURITY_RES, sizeof(*pRegResp));
@@ -1115,3 +1115,146 @@ error:
     goto cleanup;
 }
 
+LWMsgStatus
+RegSrvIpcSetValueAttibutesW(
+    LWMsgCall* pCall,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
+    void* data
+    )
+{
+    NTSTATUS status = 0;
+    PREG_IPC_SET_VALUE_ATTRS_REQ pReq = pIn->data;
+    PREG_IPC_STATUS pStatus = NULL;
+
+    status = RegSrvSetValueAttributesW(
+        RegSrvIpcGetSessionData(pCall),
+        pReq->hKey,
+        pReq->pSubKey,
+        pReq->pValueName,
+        pReq->pValueAttributes);
+
+    if (!status)
+    {
+        pOut->tag = REG_R_SET_VALUEW_ATTRIBUTES;
+        pOut->data = NULL;
+    }
+    else
+    {
+        status = RegSrvIpcCreateError(status, &pStatus);
+        BAIL_ON_NT_STATUS(status);
+
+        pOut->tag = REG_R_ERROR;
+        pOut->data = pStatus;
+    }
+
+cleanup:
+    return MAP_REG_ERROR_IPC(status);
+
+error:
+    goto cleanup;
+}
+
+LWMsgStatus
+RegSrvIpcGetValueAttibutesW(
+    LWMsgCall* pCall,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
+    void* data
+    )
+{
+    NTSTATUS status = 0;
+    PREG_IPC_GET_VALUE_ATTRS_REQ pReq = pIn->data;
+    PREG_IPC_GET_VALUE_ATTRS_RESPONSE pRegResp = NULL;
+    PREG_IPC_STATUS pStatus = NULL;
+    PLWREG_CURRENT_VALUEINFO pCurrentValue = NULL;
+    PLWREG_VALUE_ATTRIBUTES pValueAttributes = NULL;
+
+    if (!pReq->bRetCurrentValue && !pReq->bRetValueAttributes)
+    {
+        status = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    status = RegSrvGetValueAttributesW(
+            RegSrvIpcGetSessionData(pCall),
+            pReq->hKey,
+            pReq->pSubKey,
+            pReq->pValueName,
+            pReq->bRetCurrentValue ? &pCurrentValue :NULL,
+            pReq->bRetValueAttributes ? &pValueAttributes : NULL);
+
+    if (!status)
+    {
+        status = LW_RTL_ALLOCATE((PVOID*)&pRegResp,
+                                 REG_IPC_GET_VALUE_ATTRS_RESPONSE,
+                                 sizeof(*pRegResp));
+        BAIL_ON_NT_STATUS(status);
+
+        pRegResp->pCurrentValue = pCurrentValue;
+        pCurrentValue = NULL;
+        pRegResp->pValueAttributes = pValueAttributes;
+        pValueAttributes = NULL;
+
+        pOut->tag = REG_R_GET_VALUEW_ATTRIBUTES;
+        pOut->data = pRegResp;
+    }
+    else
+    {
+        status = RegSrvIpcCreateError(status, &pStatus);
+        BAIL_ON_NT_STATUS(status);
+
+        pOut->tag = REG_R_ERROR;
+        pOut->data = pStatus;
+    }
+
+cleanup:
+    RegSafeFreeCurrentValueInfo(&pCurrentValue);
+    RegSafeFreeValueAttributes(&pValueAttributes);
+
+    return MAP_REG_ERROR_IPC(status);
+
+error:
+    goto cleanup;
+}
+
+LWMsgStatus
+RegSrvIpcDeleteValueAttibutesW(
+    LWMsgCall* pCall,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
+    void* data
+    )
+{
+    NTSTATUS status = 0;
+    PREG_IPC_DELETE_VALUE_ATTRS_REQ pReq = pIn->data;
+    PREG_IPC_STATUS pStatus = NULL;
+
+    status = RegSrvDeleteValueAttributesW(
+            RegSrvIpcGetSessionData(pCall),
+            pReq->hKey,
+            pReq->pSubKey,
+            pReq->pValueName);
+
+    if (!status)
+    {
+        pOut->tag = REG_R_DELETE_VALUEW_ATTRIBUTES;
+        pOut->data = NULL;
+    }
+    else
+    {
+        status = RegSrvIpcCreateError(status, &pStatus);
+        BAIL_ON_NT_STATUS(status);
+
+        pOut->tag = REG_R_ERROR;
+        pOut->data = pStatus;
+    }
+
+cleanup:
+
+    return MAP_REG_ERROR_IPC(status);
+
+error:
+    goto cleanup;
+}
+
