@@ -86,8 +86,8 @@ RegSrvEnumRootKeysW(
 
     for (iCount = 0; iCount< NUM_ROOTKEY; iCount++)
     {
-	status = LwRtlWC16StringDuplicate(&ppwszRootKeys[iCount], ROOT_KEYS[iCount]);
-	BAIL_ON_NT_STATUS(status);
+    	status = LwRtlWC16StringDuplicate(&ppwszRootKeys[iCount], ROOT_KEYS[iCount]);
+    	BAIL_ON_NT_STATUS(status);
     }
 
     *pdwNumRootKeys = NUM_ROOTKEY;
@@ -375,11 +375,11 @@ RegSrvSetKeySecurity(
     )
 {
     return gpRegProvider->pfnRegSrvSetKeySecurity(
-		Handle,
-		hKey,
-		SecurityInformation,
-		pSecurityDescriptor,
-		ulSecDescLength);
+    		Handle,
+    		hKey,
+    		SecurityInformation,
+    		pSecurityDescriptor,
+    		ulSecDescLength);
 }
 
 NTSTATUS
@@ -387,18 +387,75 @@ RegSrvGetKeySecurity(
     IN HANDLE Handle,
     IN HKEY hKey,
     IN SECURITY_INFORMATION SecurityInformation,
-    OUT PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
+    OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
     IN OUT PULONG pulSecDescLength
     )
 {
     return gpRegProvider->pfnRegSrvGetKeySecurity(
-		Handle,
-		hKey,
-		SecurityInformation,
-		pSecurityDescriptor,
-		pulSecDescLength);
+    		Handle,
+    		hKey,
+    		SecurityInformation,
+    		pSecurityDescriptor,
+    		pulSecDescLength);
 }
 
+//
+// Registry Value Attributes Server APIs
+//
+NTSTATUS
+RegSrvSetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    )
+{
+   return gpRegProvider->pfnRegSrvSetValueAttributes(
+           hRegConnection,
+            hKey,
+            pSubKey,
+            pValueName,
+            pValueAttributes);
+}
+
+NTSTATUS
+RegSrvGetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    )
+{
+   return gpRegProvider->pfnRegSrvGetValueAttributes(
+           hRegConnection,
+            hKey,
+            pwszSubKey,
+            pwszValueName,
+            ppCurrentValue,
+            ppValueAttributes);
+}
+
+NTSTATUS
+RegSrvDeleteValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName
+    )
+{
+    return gpRegProvider->pfnRegSrvDeleteValueAttributes(
+            hRegConnection,
+             hKey,
+             pwszSubKey,
+             pwszValueName);
+}
+
+
+
+
 // Key Context (key handle) helper utility functions
 void
 RegSrvSafeFreeKeyContext(
@@ -415,10 +472,17 @@ RegSrvSafeFreeKeyContext(
         LWREG_SAFE_FREE_MEMORY(pKeyResult->pwszKeyName);
         LWREG_SAFE_FREE_MEMORY(pKeyResult->pwszParentKeyName);
         RegFreeWC16StringArray(pKeyResult->ppwszSubKeyNames, pKeyResult->dwNumCacheSubKeys);
+
         RegFreeWC16StringArray(pKeyResult->ppwszValueNames, pKeyResult->dwNumCacheValues);
         RegFreeValueByteArray(pKeyResult->ppValues, pKeyResult->dwNumCacheValues);
         LWREG_SAFE_FREE_MEMORY(pKeyResult->pdwValueLen);
         LWREG_SAFE_FREE_MEMORY(pKeyResult->pTypes);
+
+        RegFreeWC16StringArray(pKeyResult->ppwszDefaultValueNames, pKeyResult->dwNumCacheDefaultValues);
+        RegFreeValueByteArray(pKeyResult->ppDefaultValues, pKeyResult->dwNumCacheDefaultValues);
+        LWREG_SAFE_FREE_MEMORY(pKeyResult->pdwDefaultValueLen);
+        LWREG_SAFE_FREE_MEMORY(pKeyResult->pDefaultTypes);
+
         LWREG_SAFE_FREE_MEMORY(pKeyResult->pSecurityDescriptor);
 
         memset(pKeyResult, 0, sizeof(*pKeyResult));
@@ -579,8 +643,8 @@ RegSrvGetKeySecurityDescriptor_inlock(
 
     if (ulSecDescRelLen < pKeyResult->ulSecDescLength)
     {
-	status = STATUS_BUFFER_TOO_SMALL;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_BUFFER_TOO_SMALL;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     memcpy(pSecurityDescriptor, pKeyResult->pSecurityDescriptor, pKeyResult->ulSecDescLength);
@@ -694,10 +758,21 @@ RegSrvResetValueInfo(
 
     pKeyResult->ppwszValueNames = NULL;
     pKeyResult->ppValues = NULL;
-
     pKeyResult->dwNumCacheValues = 0;
     pKeyResult->dwNumValues = 0;
 
+    pKeyResult->bHasDefaultValueInfo = FALSE;
+
+    RegFreeWC16StringArray(pKeyResult->ppwszDefaultValueNames, pKeyResult->dwNumCacheDefaultValues);
+    RegFreeValueByteArray(pKeyResult->ppDefaultValues, pKeyResult->dwNumCacheDefaultValues);
+    LWREG_SAFE_FREE_MEMORY(pKeyResult->pdwDefaultValueLen);
+    LWREG_SAFE_FREE_MEMORY(pKeyResult->pDefaultTypes);
+
+    pKeyResult->ppwszDefaultValueNames = NULL;
+    pKeyResult->ppDefaultValues = NULL;
+    pKeyResult->dwNumCacheDefaultValues = 0;
+    pKeyResult->dwNumCacheValues = 0;
+
     LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
 }
 
@@ -718,6 +793,24 @@ RegSrvHasValueInfo(
     return bHasValueInfo;
 }
 
+BOOLEAN
+RegSrvHasDefaultValueInfo(
+    IN PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    BOOLEAN bInLock = FALSE;
+    BOOLEAN bHasDefaultValueInfo = FALSE;
+
+    LWREG_LOCK_RWMUTEX_SHARED(bInLock, &pKeyResult->mutex);
+
+    bHasDefaultValueInfo = pKeyResult->bHasDefaultValueInfo;
+
+    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
+
+    return bHasDefaultValueInfo;
+}
+
+
 DWORD
 RegSrvValueNum(
     IN PREG_KEY_CONTEXT pKeyResult
@@ -735,6 +828,23 @@ RegSrvValueNum(
     return dwValueCount;
 }
 
+DWORD
+RegSrvDefaultValueNum(
+    IN PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    BOOLEAN bInLock = FALSE;
+    DWORD dwValueCount = 0;
+
+    LWREG_LOCK_RWMUTEX_SHARED(bInLock, &pKeyResult->mutex);
+
+    dwValueCount = pKeyResult->dwNumDefaultValues;
+
+    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
+
+    return dwValueCount;
+}
+
 size_t
 RegSrvMaxValueNameLen(
     IN PREG_KEY_CONTEXT pKeyResult
@@ -768,57 +878,3 @@ RegSrvMaxValueLen(
 
     return sMaxValueLen;
 }
-
-PCWSTR
-RegSrvValueName(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex
-    )
-{
-    BOOLEAN bInLock = FALSE;
-    PCWSTR pwszValueName = NULL;
-
-    LWREG_LOCK_RWMUTEX_SHARED(bInLock, &pKeyResult->mutex);
-
-    pwszValueName = pKeyResult->ppwszValueNames[dwIndex];
-
-    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
-
-    return pwszValueName;
-}
-
-void
-RegSrvValueContent(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex,
-    PBYTE* ppValue,
-    PDWORD pdwValueLen
-    )
-{
-    BOOLEAN bInLock = FALSE;
-
-    LWREG_LOCK_RWMUTEX_SHARED(bInLock, &pKeyResult->mutex);
-
-    *ppValue = pKeyResult->ppValues[dwIndex];
-    *pdwValueLen = pKeyResult->pdwValueLen[dwIndex];
-
-    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
-}
-
-REG_DATA_TYPE
-RegSrvValueType(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex
-    )
-{
-    BOOLEAN bInLock = FALSE;
-    REG_DATA_TYPE type = REG_UNKNOWN;
-
-    LWREG_LOCK_RWMUTEX_SHARED(bInLock, &pKeyResult->mutex);
-
-    type = pKeyResult->pTypes[dwIndex];
-
-    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
-
-    return type;
-}
