@@ -81,16 +81,34 @@ typedef struct __REG_KEY_CONTEXT
     PWSTR* ppwszSubKeyNames;
     BOOLEAN bHasSubKeyInfo;
 
-    DWORD dwNumValues;
-    DWORD dwNumCacheValues;
     size_t sMaxValueNameLen;
     size_t sMaxValueLen;
+
+    // keep track of the number of values that have been set by user
+    DWORD dwNumValues;
+    DWORD dwNumCacheValues;
+
     PREG_DATA_TYPE pTypes;
     PWSTR* ppwszValueNames;
     PBYTE* ppValues;
     PDWORD pdwValueLen;
+
     BOOLEAN bHasValueInfo;
 
+    // keep track of the number of values that have not been set by user
+    // but are defined in schema as value attributes
+    DWORD dwNumDefaultValues;
+    DWORD dwNumCacheDefaultValues;
+
+    PREG_DATA_TYPE pDefaultTypes;
+    PWSTR* ppwszDefaultValueNames;
+    PBYTE* ppDefaultValues;
+    PDWORD pdwDefaultValueLen;
+
+    BOOLEAN bHasDefaultValueInfo;
+
+
+
 } REG_KEY_CONTEXT, *PREG_KEY_CONTEXT;
 
 
@@ -350,12 +368,38 @@ RegSrvGetKeySecurity(
     IN HANDLE Handle,
     IN HKEY hKey,
     IN SECURITY_INFORMATION SecurityInformation,
-    OUT PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
+    OUT OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
     IN OUT PULONG pulSecDescLength
     );
+//
+// Registry value attribute APIs
+//
+NTSTATUS
+RegSrvSetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    );
 
+NTSTATUS
+RegSrvGetValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    );
 
-
+NTSTATUS
+RegSrvDeleteValueAttributesW(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pwszValueName
+    );
 
 // Key context (key handle) utility functions
 BOOLEAN
@@ -447,11 +491,21 @@ RegSrvHasValueInfo(
     IN PREG_KEY_CONTEXT pKeyResult
     );
 
+BOOLEAN
+RegSrvHasDefaultValueInfo(
+    IN PREG_KEY_CONTEXT pKeyResult
+    );
+
 DWORD
 RegSrvValueNum(
     IN PREG_KEY_CONTEXT pKeyResult
     );
 
+DWORD
+RegSrvDefaultValueNum(
+    IN PREG_KEY_CONTEXT pKeyResult
+    );
+
 size_t
 RegSrvMaxValueNameLen(
     IN PREG_KEY_CONTEXT pKeyResult
@@ -462,26 +516,6 @@ RegSrvMaxValueLen(
     IN PREG_KEY_CONTEXT pKeyResult
     );
 
-PCWSTR
-RegSrvValueName(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex
-    );
-
-void
-RegSrvValueContent(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex,
-    PBYTE* ppValue,
-    PDWORD pdwValueLen
-    );
-
-REG_DATA_TYPE
-RegSrvValueType(
-    IN PREG_KEY_CONTEXT pKeyResult,
-    DWORD dwIndex
-    );
-
 //Registry ACL check
 
 NTSTATUS
