@@ -92,8 +92,6 @@ main(
        BAIL_ON_REG_ERROR(dwError);
     }
 
-    RegSrvCreatePIDFile();
-
     dwError = RegBlockSelectedSignals();
     BAIL_ON_REG_ERROR(dwError);
 
@@ -106,7 +104,7 @@ main(
     if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notifyFd = atoi(pszSmNotify);
-
+        
         do
         {
             ret = write(notifyFd, &notifyCode, sizeof(notifyCode));
@@ -166,8 +164,13 @@ RegSrvSetDefaults(
 
     *(gpServerInfo->szLogFilePath) = '\0';
 
-    strcpy(gpServerInfo->szCachePath, CACHEDIR);
-    strcpy(gpServerInfo->szPrefixPath, PREFIXDIR);
+    strncpy(gpServerInfo->szCachePath,
+            CACHEDIR,
+            sizeof(gpServerInfo->szCachePath)-1);
+
+    strncpy(gpServerInfo->szPrefixPath,
+            PREFIXDIR,
+            sizeof(gpServerInfo->szPrefixPath)-1);
 
     setlocale(LC_ALL, "");
 
@@ -674,107 +677,6 @@ RegSrvGetPrefixPath(
     goto cleanup;
 }
 
-VOID
-RegSrvCreatePIDFile(
-    VOID
-    )
-{
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-
-    pid = RegSrvGetPidFromPidFile();
-    if (pid > 0)
-    {
-        fprintf(stderr, "Daemon already running as %d\n", (int) pid);
-        result = -1;
-        goto error;
-    }
-
-    fd = open(PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0)
-    {
-        fprintf(stderr, "Could not create pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) write(fd, contents, len);
-    if ( result != (int) len )
-    {
-        fprintf(stderr, "Could not write to pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
- error:
-    if (fd != -1)
-    {
-        close(fd);
-    }
-
-    if (result < 0)
-    {
-        exit(1);
-    }
-}
-
-pid_t
-RegSrvGetPidFromPidFile(
-    VOID
-    )
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(PID_FILE, O_RDONLY, 0644);
-    if (fd < 0)
-    {
-        goto error;
-    }
-
-    result = read(fd, contents, sizeof(contents)-1);
-    if (result <= 0)
-    {
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result <= 0)
-    {
-        result = -1;
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH)
-    {
-        unlink(PID_FILE);
-        pid = 0;
-    }
-
- error:
-    if (fd != -1)
-    {
-        close(fd);
-    }
-
-    return pid;
-}
-
 DWORD
 RegBlockSelectedSignals(
     VOID
