@@ -48,9 +48,12 @@
 #ifndef __SQLCACHE_CREATE_H__
 #define __SQLCACHE_CREATE_H__
 
-#define REG_DB_TABLE_NAME_KEYS        "regkeys1"
-#define REG_DB_TABLE_NAME_VALUES      "regvalues1"
-#define REG_DB_TABLE_NAME_ACLS        "regacl1"
+#define REG_DB_TABLE_NAME_KEYS          "regkeys1"
+#define REG_DB_TABLE_NAME_VALUES        "regvalues1"
+#define REG_DB_TABLE_NAME_ACLS          "regacl1"
+
+#define REG_DB_TABLE_NAME_SCHEMA_VALUES "regschemavalues1"
+
 
 #define REG_DB_TABLE_NAME_CACHE_TAGS  "regcachetags"
 #define REG_DB_TABLE_NAME_ENTRIES     "regentry1"
@@ -97,6 +100,18 @@
 		"    );\n" \
 		_REG_DB_SQL_CREATE_INDEX(REG_DB_TABLE_NAME_ACLS, "CacheId") \
 		"\n" \
+    _REG_DB_SQL_CREATE_TABLE(REG_DB_TABLE_NAME_SCHEMA_VALUES) "(\n" \
+        "    LastUpdated integer,\n" \
+        "    ParentId integer,\n" \
+        "    ValueName text COLLATE NOCASE,\n" \
+        "    Type integer,\n" \
+        "    DefaultValue blob,\n" \
+        "    Document text COLLATE NOCASE,\n" \
+        "    RangeType integer,\n" \
+        "    Hint integer,\n" \
+        "    Range blob,\n" \
+        "    UNIQUE (ParentId, ValueName)\n" \
+        "    );\n" \
 		""
 
 #define REG_DB_INSERT_REG_KEY "INSERT INTO " REG_DB_TABLE_NAME_KEYS " (" \
@@ -142,7 +157,7 @@
             REG_DB_TABLE_NAME_KEYS ".AclIndex " \
             "from " REG_DB_TABLE_NAME_KEYS " " \
             "where " REG_DB_TABLE_NAME_KEYS ".KeyName = ?1 " \
-			"AND " REG_DB_TABLE_NAME_KEYS ".ParentId = ?2 " \
+            		"AND " REG_DB_TABLE_NAME_KEYS ".ParentId = ?2 " \
 
 #define REG_DB_QUERY_KEY_ACL_INDEX_BY_KEYID  "select " \
 	        REG_DB_TABLE_NAME_KEYS ".AclIndex " \
@@ -225,7 +240,7 @@
 
 #define REG_DB_DELETE_KEYVALUE  "delete from "  REG_DB_TABLE_NAME_VALUES " " \
             "where " REG_DB_TABLE_NAME_VALUES ".ParentId = ?1" \
-			"AND " REG_DB_TABLE_NAME_VALUES ".ValueName = ?2" \
+            		"AND " REG_DB_TABLE_NAME_VALUES ".ValueName = ?2" \
 
 
 #define REG_DB_DELETE_ACL  "delete from "  REG_DB_TABLE_NAME_ACLS " " \
