@@ -65,11 +65,11 @@ RegDbOpenKey_inlock(
     status = SqliteCacheGetDbKeyInfo(pwszFullKeyPath, &pRegKey);
     if (!status)
     {
-	goto cleanup;
+    	goto cleanup;
     }
     else if (STATUS_OBJECT_NAME_NOT_FOUND == status)
     {
-	status = 0;
+    	status = 0;
     }
     BAIL_ON_NT_STATUS(status);
 
@@ -78,20 +78,20 @@ RegDbOpenKey_inlock(
         LWREG_SAFE_FREE_MEMORY(pwszKeyToken);
         RegDbSafeFreeEntryKey(&pRegKey);
 
-	status = SqliteGetKeyToken(pwszCurrentKeyPath, wch, &pwszKeyToken);
+    	status = SqliteGetKeyToken(pwszCurrentKeyPath, wch, &pwszKeyToken);
         BAIL_ON_NT_STATUS(status);
 
         status = RegDbOpenKeyName_inlock(hDb,
-			                  !pwszKeyToken ? pwszCurrentKeyPath : pwszKeyToken,
-			                  &qwParentId,
-			                  &pRegKey);
+        		                  !pwszKeyToken ? pwszCurrentKeyPath : pwszKeyToken,
+        		                  &qwParentId,
+        		                  &pRegKey);
         BAIL_ON_NT_STATUS(status);
 
         pwszCurrentKeyPath = RegStrchr(pwszCurrentKeyPath, wch);
 
         if (pwszCurrentKeyPath)
         {
-		pwszCurrentKeyPath++;
+        	pwszCurrentKeyPath++;
         }
     }
 
@@ -100,11 +100,11 @@ RegDbOpenKey_inlock(
 
     if (pRegKey->qwAclIndex != -1)
     {
-	status = RegDbGetKeyAclByAclIndex_inlock(hDb,
-			                             pRegKey->qwAclIndex,
-			                             &pRegKey->pSecDescRel,
-			                             &pRegKey->ulSecDescLength);
-	BAIL_ON_NT_STATUS(status);
+    	status = RegDbGetKeyAclByAclIndex_inlock(hDb,
+    	    		                             pRegKey->qwAclIndex,
+    	    		                             &pRegKey->pSecDescRel,
+    	    		                             &pRegKey->ulSecDescLength);
+    	BAIL_ON_NT_STATUS(status);
     }
 
     REG_LOG_VERBOSE("Registry::sqldb.c RegDbOpenKey_inlock() finished\n");
@@ -126,11 +126,11 @@ cleanup:
 
     if (!status && ppRegKey)
     {
-	*ppRegKey = pRegKey;
+    	*ppRegKey = pRegKey;
     }
     else
     {
-	RegDbSafeFreeEntryKey(&pRegKey);
+    	RegDbSafeFreeEntryKey(&pRegKey);
     }
 
     LWREG_SAFE_FREE_MEMORY(pwszKeyToken);
@@ -141,7 +141,7 @@ error:
 
     if(ppRegKey)
     {
-	*ppRegKey = NULL;
+    	*ppRegKey = NULL;
     }
 
     RegDbSafeFreeEntryKey(&pRegKey);
@@ -179,14 +179,14 @@ RegDbOpenKeyName_inlock(
         nGotColumns = sqlite3_column_count(pstQuery);
         if (nGotColumns != nExpectedCols)
         {
-		status = STATUS_DATA_ERROR;
+        	status = STATUS_DATA_ERROR;
             BAIL_ON_NT_STATUS(status);
         }
 
         if (sResultCount >= 1)
         {
             //Duplicate keys are found
-		status = STATUS_DUPLICATE_NAME;
+        	status = STATUS_DUPLICATE_NAME;
             BAIL_ON_NT_STATUS(status);
         }
 
@@ -211,7 +211,7 @@ RegDbOpenKeyName_inlock(
     if (status == SQLITE_DONE)
     {
         // No more results found
-	status = STATUS_SUCCESS;
+    	status = STATUS_SUCCESS;
     }
     BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
 
@@ -220,7 +220,7 @@ RegDbOpenKeyName_inlock(
 
     if (!sResultCount)
     {
-	status = STATUS_OBJECT_NAME_NOT_FOUND;
+    	status = STATUS_OBJECT_NAME_NOT_FOUND;
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -244,7 +244,7 @@ error:
 }
 
 NTSTATUS
-RegDbDeleteKey_inlock(
+RegDbDeleteKeyWithNoSubKeys_inlock(
     IN REG_DB_HANDLE hDb,
     IN int64_t qwId,
     IN int64_t qwAclId,
@@ -295,6 +295,22 @@ RegDbDeleteKey_inlock(
     status = (DWORD)sqlite3_reset(pstQuery);
     BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
 
+    // Delete the key's values associated attributes
+    pstQuery = pConn->pstDeleteAllValueAttributes;
+
+    status = RegSqliteBindInt64(pstQuery, 1, qwId);
+    BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+
+    status = (DWORD)sqlite3_step(pstQuery);
+    if (status == SQLITE_DONE)
+    {
+        status = STATUS_SUCCESS;
+    }
+    BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+
+    status = (DWORD)sqlite3_reset(pstQuery);
+    BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+
     // Delete the keys' ACL if there is no more reference to it
     if (qwAclId != -1)
     {
@@ -303,8 +319,8 @@ RegDbDeleteKey_inlock(
 
         if (!sCount)
         {
-	    status = RegDbDeleteAcl_inlock(hDb, qwAclId);
-	    BAIL_ON_NT_STATUS(status);
+    	    status = RegDbDeleteAcl_inlock(hDb, qwAclId);
+    	    BAIL_ON_NT_STATUS(status);
         }
     }
 
@@ -321,7 +337,6 @@ cleanup:
     goto cleanup;
 }
 
-
 NTSTATUS
 RegDbDeleteAcl_inlock(
     IN REG_DB_HANDLE hDb,
@@ -380,8 +395,8 @@ RegDbQueryInfoKeyCount_inlock(
 
     if (qwId <= 0)
     {
-	status = STATUS_INTERNAL_ERROR;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INTERNAL_ERROR;
+    	BAIL_ON_NT_STATUS(status);
    }
 
     switch (queryType)
@@ -436,10 +451,6 @@ RegDbQueryInfoKeyCount_inlock(
                 BAIL_ON_NT_STATUS(status);
 
                 break;
-
-            default:
-                status = STATUS_INVALID_PARAMETER;
-                BAIL_ON_NT_STATUS(status);
         }
 
         sResultCount++;
@@ -503,8 +514,8 @@ RegDbQueryInfoKey_inlock(
 
     if (qwId <= 0)
     {
-	status = STATUS_INTERNAL_ERROR;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INTERNAL_ERROR;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     pstQuery = pConn->pstQuerySubKeys;
@@ -668,8 +679,8 @@ RegDbQueryAclRefCountWOCurrKey_inlock(
 
     if (qwSdId <= 0)
     {
-	status = STATUS_INTERNAL_ERROR;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INTERNAL_ERROR;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     pstQuery = pConn->pstQueryAclRefCount;
@@ -1119,6 +1130,506 @@ error:
     goto cleanup;
 }
 
+NTSTATUS
+RegDbGetKeyValue_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwParentKeyId,
+    IN PCWSTR pwszValueName,
+    IN REG_DATA_TYPE valueType,
+    IN OPTIONAL PBOOLEAN pbIsWrongType,
+    OUT OPTIONAL PREG_DB_VALUE* ppRegEntry
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PREG_DB_CONNECTION pConn = (PREG_DB_CONNECTION)hDb;
+    // do not free
+    sqlite3_stmt *pstQuery = NULL;
+    size_t sResultCount = 0;
+    const int nExpectedCols = 5;
+    int iColumnPos = 0;
+    int nGotColumns = 0;
+    PREG_DB_VALUE pRegEntry = NULL;
+
+
+    if (valueType == REG_NONE)
+    {
+        pstQuery = pConn->pstQueryKeyValue;
+
+        status = (NTSTATUS)RegSqliteBindStringW(pstQuery, 1, pwszValueName);
+        BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+
+        status = (NTSTATUS)RegSqliteBindInt64(pstQuery, 2, qwParentKeyId);
+        BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+    }
+    else
+    {
+        if (pbIsWrongType && !*pbIsWrongType)
+        {
+            pstQuery = pConn->pstQueryKeyValueWithType;
+        }
+        else
+        {
+            pstQuery = pConn->pstQueryKeyValueWithWrongType;
+        }
+
+        status = RegSqliteBindStringW(pstQuery, 1, pwszValueName);
+        BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+
+        status = RegSqliteBindInt64(pstQuery, 2, qwParentKeyId);
+        BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+
+        status = RegSqliteBindInt32(pstQuery, 3, (int)valueType);
+        BAIL_ON_SQLITE3_ERROR_STMT(status, pstQuery);
+    }
+
+    while ((status = (DWORD)sqlite3_step(pstQuery)) == SQLITE_ROW)
+    {
+        nGotColumns = sqlite3_column_count(pstQuery);
+        if (nGotColumns != nExpectedCols)
+        {
+            status = STATUS_DATA_ERROR;
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (sResultCount >= 1)
+        {
+            // Duplicate key value records are found
+            status = STATUS_DUPLICATE_NAME;
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        status = LW_RTL_ALLOCATE((PVOID*)&pRegEntry, REG_DB_VALUE, sizeof(*pRegEntry));
+        BAIL_ON_NT_STATUS(status);
+
+        iColumnPos = 0;
+
+        status = RegDbUnpackRegValueInfo(pstQuery,
+                                         &iColumnPos,
+                                         pRegEntry);
+        BAIL_ON_NT_STATUS(status);
+
+        sResultCount++;
+    }
+
+    if (status == SQLITE_DONE)
+    {
+        // No more results found
+        status = STATUS_SUCCESS;
+    }
+    BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+
+    status = (DWORD)sqlite3_reset(pstQuery);
+    BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+
+    if (!sResultCount)
+    {
+        status = STATUS_OBJECT_NAME_NOT_FOUND;
+        BAIL_ON_NT_STATUS(status);
+    }
+
+cleanup:
+    if (!status && ppRegEntry)
+    {
+        *ppRegEntry = pRegEntry;
+    }
+    else
+    {
+        RegDbSafeFreeEntryValue(&pRegEntry);
+        if (ppRegEntry)
+            *ppRegEntry = NULL;
+    }
+
+    return status;
+
+error:
+    if (pstQuery != NULL)
+    {
+        sqlite3_reset(pstQuery);
+    }
+
+    goto cleanup;
+}
+
+NTSTATUS
+RegDbUpdateRegValues_inlock(
+    IN HANDLE hDb,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_VALUE* ppValues
+    )
+{
+    NTSTATUS status = 0;
+    PREG_DB_CONNECTION pConn = (PREG_DB_CONNECTION)hDb;
+    sqlite3_stmt *pstQueryEntry = NULL;
+    int iColumnPos = 1;
+    PREG_DB_VALUE pEntry = NULL;
+    DWORD dwIndex = 0;
+    BOOLEAN bGotNow = FALSE;
+    time_t now = 0;
+
+
+    for (dwIndex = 0; dwIndex < dwEntryCount; dwIndex++)
+    {
+        pEntry = ppValues[dwIndex];
+
+        if (pEntry == NULL)
+        {
+            continue;
+        }
+
+        pstQueryEntry = pConn->pstUpdateRegValue;
+
+        if (!bGotNow)
+        {
+            status = RegGetCurrentTimeSeconds(&now);
+            BAIL_ON_NT_STATUS(status);
+
+            bGotNow = TRUE;
+        }
+
+        iColumnPos = 1;
+
+        status = sqlite3_reset(pstQueryEntry);
+        BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+
+        status = RegSqliteBindInt64(
+                    pstQueryEntry,
+                    iColumnPos,
+                    now);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindBlob(
+                   pstQueryEntry,
+                   iColumnPos,
+                   pEntry->pValue,
+                   pEntry->dwValueLen);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindInt64(
+                    pstQueryEntry,
+                    iColumnPos,
+                    pEntry->qwParentId);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindStringW(
+                    pstQueryEntry,
+                    iColumnPos,
+                    pEntry->pwszValueName);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = (DWORD)sqlite3_step(pstQueryEntry);
+        if (status == SQLITE_DONE)
+        {
+            status = 0;
+        }
+        BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+    }
+
+    REG_LOG_VERBOSE("Registry::sqldb.c RegDbUpdateRegValues_inlock() finished\n");
+
+    status = sqlite3_reset(pstQueryEntry);
+    BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+
+cleanup:
+
+    return status;
+
+ error:
+
+    goto cleanup;
+}
+
+NTSTATUS
+RegDbStoreRegKeys_inlock(
+    IN HANDLE hDB,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_KEY* ppKeys
+    )
+{
+    NTSTATUS status = 0;
+    PREG_DB_CONNECTION pConn = (PREG_DB_CONNECTION)hDB;
+    sqlite3_stmt *pstCreateKey = NULL;
+    sqlite3_stmt *pstCreateKeyAcl = NULL;
+    int iColumnPos = 1;
+    // Do not free pEntry
+    PREG_DB_KEY pEntry = NULL;
+    DWORD dwIndex = 0;
+    BOOLEAN bGotNow = FALSE;
+    time_t now = 0;
+
+    for (dwIndex = 0; dwIndex < dwEntryCount; dwIndex++)
+    {
+        pEntry = ppKeys[dwIndex];
+
+        if (pEntry == NULL)
+        {
+            continue;
+        }
+
+        // Creating a new key qwDbId should be -1
+        if (pEntry->version.qwDbId != -1)
+        {
+            REG_LOG_DEBUG("Registry::sqldb.c RegDbStoreRegKeys() qwDbId is -1\n");
+            continue;
+        }
+
+        status = RegDbGetKeyAclIndexByKeyAcl_inlock(
+                                            hDB,
+                                            pEntry->pSecDescRel,
+                                            pEntry->ulSecDescLength,
+                                            &pEntry->qwAclIndex);
+        BAIL_ON_NT_STATUS(status);
+
+        if (pEntry->qwAclIndex == -1)
+        {
+            pstCreateKeyAcl = pConn->pstCreateRegAcl;
+
+            status = RegSqliteBindBlob(
+                       pstCreateKeyAcl,
+                       1,
+                       (BYTE*)pEntry->pSecDescRel,
+                       pEntry->ulSecDescLength);
+            BAIL_ON_SQLITE3_ERROR_STMT(status, pstCreateKeyAcl);
+
+            status = (DWORD)sqlite3_step(pstCreateKeyAcl);
+            if (status == SQLITE_DONE)
+            {
+                status = 0;
+            }
+            BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+
+            pEntry->qwAclIndex = sqlite3_last_insert_rowid(pConn->pDb);
+
+            status = sqlite3_reset(pstCreateKeyAcl);
+            BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+        }
+
+        pstCreateKey = pConn->pstCreateRegKey;
+
+        if (!bGotNow)
+        {
+            status = RegGetCurrentTimeSeconds(&now);
+            BAIL_ON_NT_STATUS(status);
+
+            bGotNow = TRUE;
+        }
+
+        iColumnPos = 1;
+
+        status = RegSqliteBindInt64(
+                    pstCreateKey,
+                    iColumnPos,
+                    pEntry->qwParentId);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindStringW(
+                    pstCreateKey,
+                    iColumnPos,
+                    pEntry->pwszKeyName);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindInt64(
+                    pstCreateKey,
+                    iColumnPos,
+                    pEntry->qwAclIndex);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindInt64(
+                    pstCreateKey,
+                    iColumnPos,
+                    now);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = (DWORD)sqlite3_step(pstCreateKey);
+        if (status == SQLITE_DONE)
+        {
+            status = 0;
+        }
+        BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+
+        status = sqlite3_reset(pstCreateKey);
+        BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+    }
+
+    REG_LOG_VERBOSE("Registry::sqldb.c RegDbStoreRegKeys_inlock() finished\n");
+
+cleanup:
+
+    return status;
+
+ error:
+
+    goto cleanup;
+}
+
+NTSTATUS
+RegDbStoreRegValues_inlock(
+    IN HANDLE hDb,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_VALUE* ppValues
+    )
+{
+    NTSTATUS status = 0;
+    PREG_DB_CONNECTION pConn = (PREG_DB_CONNECTION)hDb;
+    sqlite3_stmt *pstQueryEntry = NULL;
+    int iColumnPos = 1;
+    PREG_DB_VALUE pEntry = NULL;
+    DWORD dwIndex = 0;
+    BOOLEAN bGotNow = FALSE;
+    time_t now = 0;
+
+
+    for (dwIndex = 0; dwIndex < dwEntryCount; dwIndex++)
+    {
+        pEntry = ppValues[dwIndex];
+
+        if (pEntry == NULL)
+        {
+            continue;
+        }
+
+        pstQueryEntry = pConn->pstCreateRegValue;
+
+        if (!bGotNow)
+        {
+            status = RegGetCurrentTimeSeconds(&now);
+            BAIL_ON_NT_STATUS(status);
+
+            bGotNow = TRUE;
+        }
+
+        iColumnPos = 1;
+
+        status = sqlite3_reset(pstQueryEntry);
+        BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+
+        status = RegSqliteBindInt64(
+                    pstQueryEntry,
+                    iColumnPos,
+                    pEntry->qwParentId);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindStringW(
+                    pstQueryEntry,
+                    iColumnPos,
+                    pEntry->pwszValueName);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindInt32(
+                    pstQueryEntry,
+                    iColumnPos,
+                    pEntry->type);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindBlob(
+                   pstQueryEntry,
+                   iColumnPos,
+                   pEntry->pValue,
+                   pEntry->dwValueLen);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = RegSqliteBindInt64(
+                    pstQueryEntry,
+                    iColumnPos,
+                    now);
+        BAIL_ON_NT_STATUS(status);
+        iColumnPos++;
+
+        status = (DWORD)sqlite3_step(pstQueryEntry);
+        if (status == SQLITE_DONE)
+        {
+            status = 0;
+        }
+        BAIL_ON_SQLITE3_ERROR_DB(status, pConn->pDb);
+    }
+
+    REG_LOG_VERBOSE("Registry::sqldb.c RegDbStoreRegValues_inlock() finished\n");
+
+    status = sqlite3_reset(pstQueryEntry);
+    BAIL_ON_SQLITE3_ERROR(status, sqlite3_errmsg(pConn->pDb));
+
+cleanup:
+
+    return status;
+
+ error:
+
+    goto cleanup;
+}
+
+NTSTATUS
+RegDbCreateKeyValue_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwParentKeyId,
+    IN PCWSTR pwszValueName,
+    IN PBYTE pValue,
+    IN DWORD dwValueLen,
+    IN REG_DATA_TYPE valueType,
+    OUT OPTIONAL PREG_DB_VALUE* ppRegEntry
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PREG_DB_VALUE pRegEntry = NULL;
+
+    status = LW_RTL_ALLOCATE((PVOID*)&pRegEntry, REG_DB_VALUE, sizeof(*pRegEntry));
+    BAIL_ON_NT_STATUS(status);
+
+    memset(pRegEntry, 0, sizeof(*pRegEntry));
+
+    status = LwRtlWC16StringDuplicate(&pRegEntry->pwszValueName, pwszValueName);
+    BAIL_ON_NT_STATUS(status);
+
+    if (dwValueLen)
+    {
+        status = LW_RTL_ALLOCATE((PVOID*)&pRegEntry->pValue, BYTE, sizeof(*pRegEntry->pValue)*dwValueLen);
+        BAIL_ON_NT_STATUS(status);
+
+        memset(pRegEntry->pValue, 0, sizeof(*pRegEntry->pValue)*dwValueLen);
+
+        memcpy(pRegEntry->pValue, pValue, dwValueLen);
+    }
+
+    pRegEntry->dwValueLen = dwValueLen;
+    pRegEntry->type = valueType;
+    pRegEntry->qwParentId = qwParentKeyId;
+
+    status = RegDbStoreRegValues_inlock(
+                 hDb,
+                 1,
+                 &pRegEntry);
+    BAIL_ON_NT_STATUS(status);
+
+    if (ppRegEntry)
+    {
+        *ppRegEntry = pRegEntry;
+    }
+
+cleanup:
+    if (!ppRegEntry)
+    {
+        RegDbSafeFreeEntryValue(&pRegEntry);
+    }
+
+    return status;
+
+error:
+    RegDbSafeFreeEntryValue(&pRegEntry);
+    *ppRegEntry = NULL;
+
+    goto cleanup;
+}
+
 
 
 void
@@ -1179,11 +1690,11 @@ RegDbDuplicateDbKeyEntry(
     pRegDupKey->pSecDescRel = NULL;
 
     status = LwRtlWC16StringDuplicate(&pRegDupKey->pwszKeyName,
-		                          pRegKey->pwszKeyName);
+    		                          pRegKey->pwszKeyName);
     BAIL_ON_NT_STATUS(status);
 
     status = LwRtlWC16StringDuplicate(&pRegDupKey->pwszFullKeyName,
-		                          pRegKey->pwszFullKeyName);
+    		                          pRegKey->pwszFullKeyName);
     BAIL_ON_NT_STATUS(status);
 
     status = LW_RTL_ALLOCATE((PVOID*)&pRegDupKey->pSecDescRel, VOID, pRegKey->ulSecDescLength);
@@ -1191,7 +1702,7 @@ RegDbDuplicateDbKeyEntry(
 
     memcpy(pRegDupKey->pSecDescRel, pRegKey->pSecDescRel, pRegKey->ulSecDescLength);
 
-done:
+done: 
     *ppRegKey = pRegDupKey;
 
 cleanup:
@@ -1267,8 +1778,8 @@ RegDbSafeRecordSubKeysInfo_inlock(
     if (sCacheCount)
     {
         status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppwszSubKeyNames,
-		                 PWSTR,
-		                     sizeof(*(pKeyResult->ppwszSubKeyNames)) * sCacheCount);
+    	    	                 PWSTR,
+    		                     sizeof(*(pKeyResult->ppwszSubKeyNames)) * sCacheCount);
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -1279,7 +1790,7 @@ RegDbSafeRecordSubKeysInfo_inlock(
 			sSubKeyLen = RtlWC16StringNumChars(ppRegEntries[iCount]->pwszKeyName);
 
 	        status = LwRtlWC16StringDuplicate(&pKeyResult->ppwszSubKeyNames[iCount],
-				                      ppRegEntries[iCount]->pwszKeyName);
+	              		                      ppRegEntries[iCount]->pwszKeyName);
 	        BAIL_ON_NT_STATUS(status);
 	    }
 
@@ -1357,31 +1868,31 @@ RegDbSafeRecordValuesInfo_inlock(
     }
 
     status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppwszValueNames, PWSTR,
-		                  sizeof(*(pKeyResult->ppwszValueNames))* sCacheCount);
+    		                  sizeof(*(pKeyResult->ppwszValueNames))* sCacheCount);
     BAIL_ON_NT_STATUS(status);
 
     status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppValues, PBYTE,
-		                  sizeof(*(pKeyResult->ppValues))* sCacheCount);
+    		                  sizeof(*(pKeyResult->ppValues))* sCacheCount);
     BAIL_ON_NT_STATUS(status);
 
     status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->pTypes, REG_DATA_TYPE,
-		                  sizeof(*(pKeyResult->pTypes))* sCacheCount);
+    		                  sizeof(*(pKeyResult->pTypes))* sCacheCount);
     BAIL_ON_NT_STATUS(status);
 
     status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->pdwValueLen, DWORD,
-		                  sizeof(*(pKeyResult->pdwValueLen))* sCacheCount);
+    		                  sizeof(*(pKeyResult->pdwValueLen))* sCacheCount);
     BAIL_ON_NT_STATUS(status);
 
     for (iCount = 0; iCount < (DWORD)sCacheCount; iCount++)
     {
         status = LwRtlWC16StringDuplicate(&pKeyResult->ppwszValueNames[iCount],
-			                          ppRegEntries[iCount]->pwszValueName);
+        		                          ppRegEntries[iCount]->pwszValueName);
         BAIL_ON_NT_STATUS(status);
 
         if (ppRegEntries[iCount]->dwValueLen)
         {
             status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppValues[iCount], BYTE,
-			                sizeof(*(pKeyResult->ppValues[iCount]))* ppRegEntries[iCount]->dwValueLen);
+            		                sizeof(*(pKeyResult->ppValues[iCount]))* ppRegEntries[iCount]->dwValueLen);
             BAIL_ON_NT_STATUS(status);
 
             memcpy(pKeyResult->ppValues[iCount], ppRegEntries[iCount]->pValue, ppRegEntries[iCount]->dwValueLen);
@@ -1447,3 +1958,130 @@ cleanup:
 error:
     goto cleanup;
 }
+
+
+NTSTATUS
+RegDbSafeRecordDefaultValuesInfo_inlock(
+    IN size_t sCount,
+    IN size_t sCacheCount,
+    IN PREG_DB_VALUE_ATTRIBUTES* ppRegEntries,
+    IN OUT PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    int iCount = 0;
+    size_t sValueNameLen = 0;
+
+    BAIL_ON_NT_INVALID_POINTER(pKeyResult);
+
+    //Remove previous subKey information if there is any
+    RegFreeWC16StringArray(pKeyResult->ppwszDefaultValueNames, pKeyResult->dwNumCacheDefaultValues);
+    RegFreeValueByteArray(pKeyResult->ppDefaultValues, pKeyResult->dwNumCacheDefaultValues);
+    LWREG_SAFE_FREE_MEMORY(pKeyResult->pDefaultTypes);
+    LWREG_SAFE_FREE_MEMORY(pKeyResult->pdwDefaultValueLen);
+
+    if (!sCacheCount)
+    {
+        goto cleanup;
+    }
+
+    status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppwszDefaultValueNames, PWSTR,
+                              sizeof(*(pKeyResult->ppwszDefaultValueNames))* sCacheCount);
+    BAIL_ON_NT_STATUS(status);
+
+    status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppDefaultValues, PBYTE,
+                              sizeof(*(pKeyResult->ppDefaultValues))* sCacheCount);
+    BAIL_ON_NT_STATUS(status);
+
+    status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->pDefaultTypes, REG_DATA_TYPE,
+                              sizeof(*(pKeyResult->pDefaultTypes))* sCacheCount);
+    BAIL_ON_NT_STATUS(status);
+
+    status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->pdwDefaultValueLen, DWORD,
+                              sizeof(*(pKeyResult->pdwDefaultValueLen))* sCacheCount);
+    BAIL_ON_NT_STATUS(status);
+
+    for (iCount = 0; iCount < (DWORD)sCacheCount; iCount++)
+    {
+        status = LwRtlWC16StringDuplicate(&pKeyResult->ppwszDefaultValueNames[iCount],
+                                          ppRegEntries[iCount]->pwszValueName);
+        BAIL_ON_NT_STATUS(status);
+
+        if (!ppRegEntries[iCount]->pValueAttributes)
+        {
+            status = STATUS_INTERNAL_ERROR;
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (ppRegEntries[iCount]->pValueAttributes->DefaultValueLen)
+        {
+            status = LW_RTL_ALLOCATE((PVOID*)&pKeyResult->ppDefaultValues[iCount], BYTE,
+               sizeof(*(pKeyResult->ppDefaultValues[iCount])) * ppRegEntries[iCount]->pValueAttributes->DefaultValueLen);
+            BAIL_ON_NT_STATUS(status);
+
+            memcpy(pKeyResult->ppDefaultValues[iCount],
+                   ppRegEntries[iCount]->pValueAttributes->pDefaultValue,
+                   ppRegEntries[iCount]->pValueAttributes->DefaultValueLen);
+        }
+
+        pKeyResult->pdwDefaultValueLen[iCount] = ppRegEntries[iCount]->pValueAttributes->DefaultValueLen;
+        pKeyResult->pDefaultTypes[iCount] = ppRegEntries[iCount]->pValueAttributes->ValueType;
+
+        if (pKeyResult->sMaxValueLen < (size_t)ppRegEntries[iCount]->pValueAttributes->DefaultValueLen)
+        {
+            pKeyResult->sMaxValueLen = (size_t)ppRegEntries[iCount]->pValueAttributes->DefaultValueLen;
+        }
+
+        if (pKeyResult->ppwszDefaultValueNames[iCount])
+        {
+            sValueNameLen = RtlWC16StringNumChars(pKeyResult->ppwszDefaultValueNames[iCount]);
+        }
+
+        if (pKeyResult->sMaxValueNameLen < sValueNameLen)
+            pKeyResult->sMaxValueNameLen = sValueNameLen;
+
+        sValueNameLen = 0;
+    }
+
+cleanup:
+    pKeyResult->dwNumDefaultValues = (DWORD)sCount;
+    pKeyResult->dwNumCacheDefaultValues = sCacheCount;
+
+    pKeyResult->bHasDefaultValueInfo = TRUE;
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+NTSTATUS
+RegDbSafeRecordDefaultValuesInfo(
+    IN size_t sCount,
+    IN size_t sCacheCount,
+    IN PREG_DB_VALUE_ATTRIBUTES* ppRegEntries,
+    IN OUT PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    BOOLEAN bInLock = FALSE;
+
+    BAIL_ON_NT_INVALID_POINTER(pKeyResult);
+
+    LWREG_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pKeyResult->mutex);
+
+    status = RegDbSafeRecordDefaultValuesInfo_inlock(sCount,
+                                                     sCacheCount,
+                                                     ppRegEntries,
+                                                     pKeyResult);
+    BAIL_ON_NT_STATUS(status);
+
+cleanup:
+    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
