@@ -33,20 +33,22 @@
  *
  * Module Name:
  *
- *        sqlcache_p.h
+ *        sqldb_p.h
  *
  * Abstract:
  *
- *        Likewise Security and Authentication Subsystem (LSASS)
+ *        Likewise Registry
  *
  *        Private functions in sqlite3 Caching backend
  *
- * Authors: Kyle Stemen (kstemen@likewisesoftware.com)
+ * Authors: Wei Fu (wfu@likewise.com)
  *
  */
+
 #ifndef __SQLCACHE_P_H__
 #define __SQLCACHE_P_H__
 
+
 #define REG_DB_FREE_UNUSED_CACHEIDS   \
     "delete from " REG_DB_TABLE_NAME_CACHE_TAGS " where CacheId NOT IN " \
         "CacheId NOT IN ( select CacheId from " REG_DB_TABLE_NAME_ENTRIES " );\n"
@@ -56,6 +58,8 @@ typedef struct _REG_DB_CONNECTION
     sqlite3 *pDb;
     pthread_rwlock_t lock;
 
+    // registry user view related sql statement
+
     sqlite3_stmt *pstCreateRegKey;
     sqlite3_stmt *pstCreateRegValue;
     sqlite3_stmt *pstCreateRegAcl;
@@ -82,6 +86,17 @@ typedef struct _REG_DB_CONNECTION
     sqlite3_stmt *pstQueryAclByOffset;
     sqlite3_stmt *pstUpdateRegAclByCacheId;
 
+    // registry schema view related sql statement
+    sqlite3_stmt *pstCreateRegValueAttributes;
+    sqlite3_stmt *pstQueryValueAttributes;
+    sqlite3_stmt *pstQueryValueAttributesWithType;
+    sqlite3_stmt *pstQueryValueAttributesWithWrongType;
+    sqlite3_stmt *pstUpdateValueAttributes;
+    sqlite3_stmt *pstDeleteValueAttributes;
+    sqlite3_stmt *pstDeleteAllValueAttributes;
+    sqlite3_stmt *pstQueryDefaultValues;
+    sqlite3_stmt *pstQueryDefaultValuesCount;
+
 
 } REG_DB_CONNECTION, *PREG_DB_CONNECTION;
 
@@ -218,16 +233,25 @@ RegDbSafeRecordValuesInfo(
     );
 
 NTSTATUS
-RegDbOpen(
-    IN PCSTR pszDbPath,
-    OUT PREG_DB_HANDLE phDb
+RegDbSafeRecordDefaultValuesInfo_inlock(
+    IN size_t sCount,
+    IN size_t sCacheCount,
+    IN PREG_DB_VALUE_ATTRIBUTES* ppRegEntries,
+    IN OUT PREG_KEY_CONTEXT pKeyResult
     );
 
 NTSTATUS
-RegDbStoreRegKeys(
-    IN HANDLE hDB,
-    IN DWORD dwEntryCount,
-    IN PREG_DB_KEY* ppKeys
+RegDbSafeRecordDefaultValuesInfo(
+    IN size_t sCount,
+    IN size_t sCacheCount,
+    IN PREG_DB_VALUE_ATTRIBUTES* ppRegEntries,
+    IN OUT PREG_KEY_CONTEXT pKeyResult
+    );
+
+NTSTATUS
+RegDbOpen(
+    IN PCSTR pszDbPath,
+    OUT PREG_DB_HANDLE phDb
     );
 
 NTSTATUS
@@ -248,8 +272,8 @@ NTSTATUS
 RegDbCreateKey(
     IN REG_DB_HANDLE hDb,
     IN PCWSTR pwszFullKeyName,
-    IN PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
-    IN ULONG SDLength,
+    IN PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel,
+    IN ULONG ulSecDescLength,
     OUT PREG_DB_KEY* ppRegKey
     );
 
@@ -283,6 +307,14 @@ RegDbDeleteKey_inlock(
     IN PCWSTR pwszFullKeyName
     );
 
+NTSTATUS
+RegDbDeleteKeyWithNoSubKeys_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwId,
+    IN int64_t qwAclId,
+    IN PCWSTR pwszFullKeyName
+    );
+
 NTSTATUS
 RegDbQueryInfoKey(
     IN REG_DB_HANDLE hDb,
@@ -478,4 +510,53 @@ RegDbUpdateKeyAclIndex_inlock(
 	IN int64_t qwKeySdId
 	);
 
+NTSTATUS
+RegDbGetKeyValue_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwParentKeyId,
+    IN PCWSTR pwszValueName,
+    IN REG_DATA_TYPE valueType,
+    IN OPTIONAL PBOOLEAN pbIsWrongType,
+    OUT OPTIONAL PREG_DB_VALUE* ppRegEntry
+    );
+
+NTSTATUS
+RegDbDeleteKeyValue_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwParentKeyId,
+    IN PCWSTR pwszValueName
+    );
+
+NTSTATUS
+RegDbUpdateRegValues_inlock(
+    IN HANDLE hDB,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_VALUE* ppValues
+    );
+
+NTSTATUS
+RegDbStoreRegKeys_inlock(
+    IN HANDLE hDB,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_KEY* ppKeys
+    );
+
+NTSTATUS
+RegDbStoreRegValues_inlock(
+    IN HANDLE hDB,
+    IN DWORD dwEntryCount,
+    IN PREG_DB_VALUE* ppValues
+    );
+
+NTSTATUS
+RegDbCreateKeyValue_inlock(
+    IN REG_DB_HANDLE hDb,
+    IN int64_t qwParentKeyId,
+    IN PCWSTR pwszValueName,
+    IN PBYTE pValue,
+    IN DWORD dwValueLen,
+    IN REG_DATA_TYPE valueType,
+    OUT OPTIONAL PREG_DB_VALUE* ppRegEntry
+    );
+
 #endif /* __SQLCACHE_P_H__ */
