@@ -48,13 +48,6 @@
 
 #include "includes.h"
 
-#define REG_EMPTY_VALUE_NAME_W {0}
-
-static
-REG_DATA_TYPE
-GetRegDataType(
-    REG_DATA_TYPE_FLAGS Flags
-    );
 
 static
 NTSTATUS
@@ -126,19 +119,19 @@ SqliteProvider_Initialize(
 
     for (iCount = 0; iCount < NUM_ROOTKEY; iCount++)
     {
-	dwError = RegNtStatusToWin32Error(
-			                   SqliteCreateKeyInternal(NULL,
-							                   NULL,
-							                   ppwszRootKeyNames[iCount],
-							                   0,
-							                   pSecDescRel,
-							                   ulSecDescLength,
-							                   NULL,
-							                   NULL));
-	if (LWREG_ERROR_KEYNAME_EXIST == dwError)
-	{
-		dwError = 0;
-	}
+    	dwError = RegNtStatusToWin32Error(
+    			                   SqliteCreateKeyInternal(NULL,
+    			                		                   NULL,
+    			                		                   ppwszRootKeyNames[iCount],
+    			                		                   0,
+    			                		                   pSecDescRel,
+    			                		                   ulSecDescLength,
+    			                		                   NULL,
+    			                		                   NULL));
+    	if (LWREG_ERROR_KEYNAME_EXIST == dwError)
+    	{
+    		dwError = 0;
+    	}
         BAIL_ON_REG_ERROR(dwError);
     }
 
@@ -203,8 +196,8 @@ SqliteCreateKeyEx(
     // Check key length
     if (wc16slen(pSubKey) > MAX_KEY_LENGTH)
     {
-	status = STATUS_INVALID_BLOCK_LENGTH;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_BLOCK_LENGTH;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     // check whether key is valid
@@ -216,7 +209,7 @@ SqliteCreateKeyEx(
 
 	// Check whether create subkey is allowed on parentKey
 	status = RegSrvAccessCheckKeyHandle(pKeyHandle,
-			                            KEY_CREATE_SUB_KEY);
+ 			                            KEY_CREATE_SUB_KEY);
 	BAIL_ON_NT_STATUS(status);
 
 	status = LwRtlWC16StringAllocatePrintfW(
@@ -227,12 +220,12 @@ SqliteCreateKeyEx(
 	BAIL_ON_NT_STATUS(status);
 
     status =  SqliteCreateKeyInternal(
-		   Handle,
-		   pKeyCtx,
-		   pwszKeyNameWithSubKey,
-		   AccessDesired,
-		   pSecDescRel,
-		   ulSecDescLength,
+    		   Handle,
+    		   pKeyCtx,
+    		   pwszKeyNameWithSubKey,
+    		   AccessDesired,
+    		   pSecDescRel,
+    		   ulSecDescLength,
                (PREG_KEY_HANDLE*)phkResult,
                pdwDisposition);
     BAIL_ON_NT_STATUS(status);
@@ -265,9 +258,9 @@ SqliteOpenKeyEx(
 
     if (pKeyHandle)
     {
-	pKey = pKeyHandle->pKey;
+    	pKey = pKeyHandle->pKey;
 
-	if (LW_IS_NULL_OR_EMPTY_STR(pKey->pwszKeyName))
+    	if (LW_IS_NULL_OR_EMPTY_STR(pKey->pwszKeyName))
         {
             status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
@@ -287,7 +280,7 @@ SqliteOpenKeyEx(
     }
     else
     {
-	pwszFullKeyName =  pwszSubKey;
+    	pwszFullKeyName =  pwszSubKey;
     }
 
 	status = SqliteOpenKeyInternal(Handle,
@@ -398,10 +391,13 @@ SqliteQueryInfoKey(
     size_t sNumSubKeys = 0;
     size_t sNumValues = 0;
     DWORD dwOffset = 0;
+    size_t sNumDefaultValues = 0;
+    DWORD dwDefaultOffset = 0;
     BOOLEAN bInLock = FALSE;
     PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
     PREG_KEY_CONTEXT pKeyCtx = NULL;
 
+
     BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
     BAIL_ON_INVALID_RESERVED_POINTER(pdwReserved);
 
@@ -420,6 +416,9 @@ SqliteQueryInfoKey(
     status = SqliteCacheKeyValuesInfo_inlock(pKeyCtx);
     BAIL_ON_NT_STATUS(status);
 
+    status = SqliteCacheKeyDefaultValuesInfo_inlock(pKeyCtx);
+    BAIL_ON_NT_STATUS(status);
+
     if (pKeyCtx->dwNumSubKeys > pKeyCtx->dwNumCacheSubKeys)
     {
         dwOffset = pKeyCtx->dwNumCacheSubKeys;
@@ -450,9 +449,25 @@ SqliteQueryInfoKey(
         } while (sNumValues);
     }
 
+    if (pKeyCtx->dwNumDefaultValues > pKeyCtx->dwNumCacheDefaultValues)
+    {
+        dwDefaultOffset = pKeyCtx->dwNumCacheDefaultValues;
+        do
+        {
+            status = SqliteCacheUpdateDefaultValuesInfo_inlock(
+                              dwDefaultOffset,
+                              pKeyCtx,
+                              &sNumDefaultValues);
+            BAIL_ON_NT_STATUS(status);
+
+            dwDefaultOffset+= (DWORD)sNumDefaultValues;
+        } while (sNumDefaultValues);
+    }
+
     status = SqliteCacheKeySecurityDescriptor_inlock(pKeyCtx);
     BAIL_ON_NT_STATUS(status);
 
+
     if (pcSubKeys)
     {
         *pcSubKeys = pKeyCtx->dwNumSubKeys;
@@ -463,11 +478,11 @@ SqliteQueryInfoKey(
     }
     if (pcValues)
     {
-        *pcValues = pKeyCtx->dwNumValues;
+        *pcValues = pKeyCtx->dwNumValues + pKeyCtx->dwNumDefaultValues;
     }
     if (pcMaxValueNameLen)
     {
-        *pcMaxValueNameLen = pKeyCtx-> sMaxValueNameLen;
+        *pcMaxValueNameLen = pKeyCtx->sMaxValueNameLen;
     }
     if (pcMaxValueLen)
     {
@@ -559,26 +574,26 @@ SqliteEnumKeyEx(
 
     if (dwIndex >= pKeyCtx->dwNumSubKeys)
     {
-	status = STATUS_NO_MORE_MATCHES;
+    	status = STATUS_NO_MORE_ENTRIES;
         BAIL_ON_NT_STATUS(status);
     }
 
     if (dwIndex < pKeyCtx->dwNumCacheSubKeys)
     {
-	pSubKeyName = pKeyCtx->ppwszSubKeyNames[dwIndex];
+    	pSubKeyName = pKeyCtx->ppwszSubKeyNames[dwIndex];
     }
     else
     {
-	status = RegDbQueryInfoKey(ghCacheConnection,
-			                   pKeyCtx->pwszKeyName,
-			                   pKeyCtx->qwId,
+    	status = RegDbQueryInfoKey(ghCacheConnection,
+    			                   pKeyCtx->pwszKeyName,
+    			                   pKeyCtx->qwId,
                                    1,
                                    dwIndex,
                                    &sNumSubKeys,
                                    &ppRegEntries);
         BAIL_ON_NT_STATUS(status);
 
-        if (sNumSubKeys != 1)
+        if (sNumSubKeys > 1)
         {
             status = STATUS_INTERNAL_ERROR;
             BAIL_ON_NT_STATUS(status);
@@ -589,7 +604,7 @@ SqliteEnumKeyEx(
 
     if (pSubKeyName)
     {
-	sSubKeyLen = RtlWC16StringNumChars(pSubKeyName);
+    	sSubKeyLen = RtlWC16StringNumChars(pSubKeyName);
     }
 
     if (*pcName < sSubKeyLen+1)
@@ -665,26 +680,26 @@ SqliteEnumKeyEx_inDblock(
 
     if (dwIndex >= pKeyCtx->dwNumSubKeys)
     {
-	status = STATUS_NO_MORE_MATCHES;
+    	status = STATUS_NO_MORE_ENTRIES;
         BAIL_ON_NT_STATUS(status);
     }
 
     if (dwIndex < pKeyCtx->dwNumCacheSubKeys)
     {
-	pSubKeyName = pKeyCtx->ppwszSubKeyNames[dwIndex];
+    	pSubKeyName = pKeyCtx->ppwszSubKeyNames[dwIndex];
     }
     else
     {
-	status = RegDbQueryInfoKey_inlock(ghCacheConnection,
-			                   pKeyCtx->pwszKeyName,
-			                   pKeyCtx->qwId,
+    	status = RegDbQueryInfoKey_inlock(ghCacheConnection,
+    			                   pKeyCtx->pwszKeyName,
+    			                   pKeyCtx->qwId,
                                    1,
                                    dwIndex,
                                    &sNumSubKeys,
                                    &ppRegEntries);
         BAIL_ON_NT_STATUS(status);
 
-        if (sNumSubKeys != 1)
+        if (sNumSubKeys > 1)
         {
             status = STATUS_INTERNAL_ERROR;
             BAIL_ON_NT_STATUS(status);
@@ -695,7 +710,7 @@ SqliteEnumKeyEx_inDblock(
 
     if (pSubKeyName)
     {
-	sSubKeyLen = RtlWC16StringNumChars(pSubKeyName);
+    	sSubKeyLen = RtlWC16StringNumChars(pSubKeyName);
     }
 
     if (*pcName < sSubKeyLen+1)
@@ -734,7 +749,6 @@ SqliteSetValueEx(
 {
     NTSTATUS status = STATUS_SUCCESS;
     PWSTR   pwszValueName = NULL;
-    PWSTR   pwcValue = NULL;
     PWSTR*  ppwszOutMultiSz = NULL;
     BOOLEAN bIsWrongType = TRUE;
     wchar16_t wszEmptyValueName[] = REG_EMPTY_VALUE_NAME_W;
@@ -752,16 +766,16 @@ SqliteSetValueEx(
 
     if (MAX_VALUE_LENGTH < cbData)
     {
-	status = STATUS_INVALID_BLOCK_LENGTH;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_BLOCK_LENGTH;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     status = LwRtlWC16StringDuplicate(&pwszValueName, !pValueName ? wszEmptyValueName : pValueName);
     BAIL_ON_NT_STATUS(status);
 
     status = RegDbGetKeyValue(ghCacheConnection,
-		                  pKeyCtx->qwId,
-		                  (PCWSTR)pwszValueName,
+    		                  pKeyCtx->qwId,
+    		                  (PCWSTR)pwszValueName,
                               (REG_DATA_TYPE)dwType,
                               &bIsWrongType,
                               NULL);
@@ -778,7 +792,7 @@ SqliteSetValueEx(
 
     if (!cbData)
     {
-	goto done;
+    	goto done;
     }
 
     switch (dwType)
@@ -789,11 +803,11 @@ SqliteSetValueEx(
 
         case REG_MULTI_SZ:
         case REG_SZ:
-		if (cbData == 1)
-		{
+        	if (cbData == 1)
+        	{
                 status = STATUS_INTERNAL_ERROR;
                 BAIL_ON_NT_STATUS(status);
-		}
+        	}
 
             if (pData[cbData-1] != '\0' || pData[cbData-2] != '\0' )
             {
@@ -804,13 +818,13 @@ SqliteSetValueEx(
             break;
 
         default:
-		status = STATUS_NOT_SUPPORTED;
+        	status = STATUS_NOT_SUPPORTED;
             BAIL_ON_NT_STATUS(status);
     }
 
 done:
     status = RegDbSetKeyValue(ghCacheConnection,
-		                  pKeyCtx->qwId,
+    		                  pKeyCtx->qwId,
 							  (PCWSTR)pwszValueName,
 						       (const PBYTE)pData,
 						       cbData,
@@ -823,7 +837,7 @@ done:
 cleanup:
 
     LWREG_SAFE_FREE_MEMORY(pwszValueName);
-    LWREG_SAFE_FREE_MEMORY(pwcValue);
+
     if (ppwszOutMultiSz)
     {
         RegFreeMultiStrsW(ppwszOutMultiSz);
@@ -847,76 +861,62 @@ SqliteGetValue(
     IN OUT OPTIONAL PDWORD pcbData
     )
 {
-	NTSTATUS status = STATUS_SUCCESS;
-    PWSTR pwszValueName = NULL;
-    PREG_DB_VALUE pRegEntry = NULL;
-    BOOLEAN bIsWrongType = FALSE;
-    wchar16_t wszEmptyValueName[] = REG_EMPTY_VALUE_NAME_W;
-    PWSTR pwszKeyNameWithSubKey = NULL;
-    PCWSTR pwszFullKeyName = NULL;
-    PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
-    PREG_KEY_CONTEXT pKeyCtx = NULL;
-    PREG_KEY_HANDLE pKeyHandleInUse = NULL;
-    PREG_KEY_CONTEXT pKeyCtxInUse = NULL;
+    NTSTATUS status = STATUS_SUCCESS;
+    PLWREG_CURRENT_VALUEINFO pCurrentValue = NULL;
+    PLWREG_VALUE_ATTRIBUTES pValueAttributes = NULL;
+    // Do not free
+    PBYTE pValue = NULL;
+    DWORD cbValue = 0;
+    REG_DATA_TYPE dwType = REG_NONE;
 
-    BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
-    pKeyCtx = pKeyHandle->pKey;
-    BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtx);
+    status = SqliteGetValueAttributes_Internal(
+                                      Handle,
+                                      hKey,
+                                      pSubKey,
+                                      pValueName,
+                                      GetRegDataType(Flags),
+                                      FALSE,
+                                      &pCurrentValue,
+                                      &pValueAttributes);
+    BAIL_ON_NT_STATUS(status);
 
-    if (pSubKey)
+    // Value does not exist
+    if (!pCurrentValue && !pValueAttributes)
     {
-        status = LwRtlWC16StringAllocatePrintfW(
-                        &pwszKeyNameWithSubKey,
-                        L"%ws\\%ws",
-                        pKeyCtx->pwszKeyName,
-                        pSubKey);
+        status = STATUS_OBJECT_NAME_NOT_FOUND;
         BAIL_ON_NT_STATUS(status);
     }
 
-    pwszFullKeyName = pSubKey ? pwszKeyNameWithSubKey : pKeyCtx->pwszKeyName;
+    // Value is not set by client, hence using value attributes
+    if (!pCurrentValue && pValueAttributes)
+    {
+        pValue = pValueAttributes->pDefaultValue;
+        cbValue = pValueAttributes->DefaultValueLen;
+        dwType = pValueAttributes->ValueType;
+    }
+    else if (pCurrentValue)
+    {
+        pValue = pCurrentValue->pvData;
+        cbValue = pCurrentValue->cbData;
+        dwType = pCurrentValue->dwType;
+    }
 
-	status = SqliteOpenKeyInternal(Handle,
-			                       pwszFullKeyName,
-			                       KEY_QUERY_VALUE,
-								   &pKeyHandleInUse);
-	BAIL_ON_NT_STATUS(status);
-
-	// ACL check
-    status = RegSrvAccessCheckKeyHandle(pKeyHandleInUse, KEY_QUERY_VALUE);
-    BAIL_ON_NT_STATUS(status);
-
-	pKeyCtxInUse = pKeyHandleInUse->pKey;
-	BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtxInUse);
-
-    status = LwRtlWC16StringDuplicate(&pwszValueName, !pValueName ? wszEmptyValueName : pValueName);
-    BAIL_ON_NT_STATUS(status);
-
-    status = RegDbGetKeyValue(ghCacheConnection,
-		                  pKeyCtxInUse->qwId,
-		                  pwszValueName,
-                              GetRegDataType(Flags),
-                              &bIsWrongType,
-                              &pRegEntry);
-    BAIL_ON_NT_STATUS(status);
-
-    status = RegCopyValueBytes(pRegEntry->pValue,
-		                   pRegEntry->dwValueLen,
+    status = RegCopyValueBytes(pValue,
+                               cbValue,
                                pData,
                                pcbData);
     BAIL_ON_NT_STATUS(status);
 
-    *pdwType = (DWORD)pRegEntry->type;
+    *pdwType = dwType;
 
 cleanup:
-    SqliteSafeFreeKeyHandle(pKeyHandleInUse);
-    LWREG_SAFE_FREE_MEMORY(pwszValueName);
-    LWREG_SAFE_FREE_MEMORY(pwszKeyNameWithSubKey);
-    RegDbSafeFreeEntryValue(&pRegEntry);
+    RegSafeFreeCurrentValueInfo(&pCurrentValue);
+    RegSafeFreeValueAttributes(&pValueAttributes);
 
     return status;
 
 error:
-   *pdwType = 0;
+    *pdwType = 0;
 
     goto cleanup;
 }
@@ -937,7 +937,7 @@ SqliteDeleteKeyValue(
     PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
     PREG_KEY_CONTEXT pKeyCtx = NULL;
     PREG_KEY_HANDLE pKeyHandleInUse = NULL;
-    PREG_KEY_CONTEXT pKeyCtxInUse = NULL;
+
 
     BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
     pKeyCtx = pKeyHandle->pKey;
@@ -964,29 +964,16 @@ SqliteDeleteKeyValue(
 
 	// ACL check
     status = RegSrvAccessCheckKeyHandle(pKeyHandleInUse, KEY_SET_VALUE);
-    BAIL_ON_NT_STATUS(status);
-
-	pKeyCtxInUse = pKeyHandleInUse->pKey;
-	BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtxInUse);
+    BAIL_ON_NT_STATUS(status);;
 
     status = LwRtlWC16StringDuplicate(&pwszValueName, !pValueName ? wszEmptyValueName : pValueName);
     BAIL_ON_NT_STATUS(status);
 
-    status = RegDbGetKeyValue(ghCacheConnection,
-		                  pKeyCtxInUse->qwId,
-		                  pwszValueName,
-                              REG_UNKNOWN,
-                              NULL,
-                              NULL);
+    status = SqliteDeleteValue(Handle,
+                               (HKEY)pKeyHandleInUse,
+                               pwszValueName);
     BAIL_ON_NT_STATUS(status);
 
-    status = RegDbDeleteKeyValue(ghCacheConnection,
-		                     pKeyCtxInUse->qwId,
-		                     pwszValueName);
-    BAIL_ON_NT_STATUS(status);
-
-    SqliteCacheResetKeyValueInfo(pKeyCtxInUse->pwszKeyName);
-
 cleanup:
     SqliteSafeFreeKeyHandle(pKeyHandleInUse);
     LWREG_SAFE_FREE_MEMORY(pwszValueName);
@@ -1010,6 +997,10 @@ SqliteDeleteValue(
     wchar16_t wszEmptyValueName[] = REG_EMPTY_VALUE_NAME_W;
     PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
     PREG_KEY_CONTEXT pKeyCtx = NULL;
+    BOOLEAN bInLock = FALSE;
+    PREG_DB_CONNECTION pConn = (PREG_DB_CONNECTION)ghCacheConnection;
+    PSTR pszError = NULL;
+
 
     BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
 	// ACL check
@@ -1022,27 +1013,82 @@ SqliteDeleteValue(
     status = LwRtlWC16StringDuplicate(&pwszValueName, !pValueName ? wszEmptyValueName : pValueName);
     BAIL_ON_NT_STATUS(status);
 
-    status = RegDbGetKeyValue(ghCacheConnection,
-		                  pKeyCtx->qwId,
-		                  pwszValueName,
-                              REG_UNKNOWN,
-                              NULL,
-                              NULL);
+    ENTER_SQLITE_LOCK(&pConn->lock, bInLock);
+
+    status = sqlite3_exec(
+                    pConn->pDb,
+                    "begin;",
+                    NULL,
+                    NULL,
+                    &pszError);
+    BAIL_ON_SQLITE3_ERROR(status, pszError);
+
+    status = RegDbGetKeyValue_inlock((REG_DB_HANDLE)pConn,
+    		                         pKeyCtx->qwId,
+    		                         pwszValueName,
+                                     REG_NONE,
+                                     NULL,
+                                     NULL);
+    // (1) if value is user-set delete the user set value
+    if (!status)
+    {
+        status = RegDbDeleteKeyValue_inlock((REG_DB_HANDLE)pConn,
+                                            pKeyCtx->qwId,
+                                            pwszValueName);
+        BAIL_ON_NT_STATUS(status);
+    }
+    // (2) if value is not user-set
+    else if (STATUS_OBJECT_NAME_NOT_FOUND == status)
+    {
+        status = RegDbGetValueAttributes_inlock((REG_DB_HANDLE)pConn,
+                                                pKeyCtx->qwId,
+                                                pwszValueName,
+                                                REG_NONE,
+                                                NULL,
+                                                NULL);
+        // (2).1 value is not user-set but has schema info
+        if (!status)
+        {
+            status = STATUS_CANNOT_DELETE;
+        }
+        // (2).2 value is not user-set and has no schema info
+        // bail with status that is generated
+        BAIL_ON_NT_STATUS(status);
+    }
     BAIL_ON_NT_STATUS(status);
 
-    status = RegDbDeleteKeyValue(ghCacheConnection,
-		                     pKeyCtx->qwId,
-		                     pwszValueName);
-    BAIL_ON_NT_STATUS(status);
+    status = sqlite3_exec(
+                    pConn->pDb,
+                    "end",
+                    NULL,
+                    NULL,
+                    &pszError);
+    BAIL_ON_SQLITE3_ERROR(status, pszError);
+
+    REG_LOG_VERBOSE("Registry::sqldb.c SqliteDeleteValue() finished\n");
+
+    // To persist the locking order. On success reset key value info
+    LEAVE_SQLITE_LOCK(&pConn->lock, bInLock);
 
     SqliteCacheResetKeyValueInfo(pKeyCtx->pwszKeyName);
 
 cleanup:
+    LEAVE_SQLITE_LOCK(&pConn->lock, bInLock);
     LWREG_SAFE_FREE_MEMORY(pwszValueName);
 
     return status;
 
 error:
+    if (pszError)
+    {
+        sqlite3_free(pszError);
+    }
+    sqlite3_exec(pConn->pDb,
+                 "rollback",
+                 NULL,
+                 NULL,
+                 NULL);
+
     goto cleanup;
 }
 
@@ -1061,7 +1107,7 @@ SqliteEnumValue(
 {
 	NTSTATUS status = STATUS_SUCCESS;
     size_t sValueNameLen = 0;
-    REG_DATA_TYPE valueType = REG_UNKNOWN;
+    REG_DATA_TYPE valueType = REG_NONE;
     BOOLEAN bInLock = FALSE;
     size_t sNumValues = 0;
     PREG_DB_VALUE* ppRegEntries = NULL;
@@ -1072,6 +1118,11 @@ SqliteEnumValue(
     DWORD dwValueLen = 0;
     PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
     PREG_KEY_CONTEXT pKeyCtx = NULL;
+    DWORD dwTotalValue = 0;
+    size_t sNumDefaultValues = 0;
+    PREG_DB_VALUE_ATTRIBUTES* ppRegDefaultEntries = NULL;
+    DWORD dwDefaultIndex = 0;
+
 
     BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
     BAIL_ON_INVALID_RESERVED_POINTER(pdwReserved);
@@ -1092,49 +1143,101 @@ SqliteEnumValue(
     status = SqliteCacheKeyValuesInfo_inlock(pKeyCtx);
     BAIL_ON_NT_STATUS(status);
 
-    if (!pKeyCtx->dwNumValues)
+    status = SqliteCacheKeyDefaultValuesInfo_inlock(pKeyCtx);
+    BAIL_ON_NT_STATUS(status);
+
+
+    dwTotalValue = pKeyCtx->dwNumValues + pKeyCtx->dwNumDefaultValues;
+    if (dwTotalValue == 0)
     {
         goto cleanup;
     }
 
-    if (dwIndex >= pKeyCtx->dwNumValues)
+    if (dwIndex >= dwTotalValue)
     {
-	status = STATUS_NO_MORE_MATCHES;
+    	status = STATUS_NO_MORE_ENTRIES;
         BAIL_ON_NT_STATUS(status);
     }
 
+    // (1) Enumrate user specified values that has been cached
+    if (dwIndex < pKeyCtx->dwNumCacheValues)
+    {
+        pValName = pKeyCtx->ppwszValueNames[dwIndex];
+        valueType = pKeyCtx->pTypes[dwIndex];
+        dwValueLen = pKeyCtx->pdwValueLen[dwIndex];
+        pValueContent = pKeyCtx->ppValues[dwIndex];
+
+        goto done;
+    }
+
+    // (2) Enumrate user specified values that has been not been cached
+    // until there are no more user specified values.
     if (dwIndex >= pKeyCtx->dwNumCacheValues)
     {
-	status = RegDbQueryInfoKeyValue(ghCacheConnection,
-			                        pKeyCtx->qwId,
+    	status = RegDbQueryInfoKeyValue(ghCacheConnection,
+    			                        pKeyCtx->qwId,
                                         1,
                                         dwIndex,
                                         &sNumValues,
                                         &ppRegEntries);
+    	// sNumValues == 0 means no more entry
+    	if (!status  && !sNumValues)
+    	{
+    	    // (3) Enumrate default values that has been cached
+    	    dwDefaultIndex = dwIndex-pKeyCtx->dwNumValues;
+
+    	    if (dwDefaultIndex < pKeyCtx->dwNumCacheDefaultValues)
+    	    {
+    	        pValName = pKeyCtx->ppwszDefaultValueNames[dwDefaultIndex];
+    	        valueType = pKeyCtx->pDefaultTypes[dwDefaultIndex];
+    	        dwValueLen = pKeyCtx->pdwDefaultValueLen[dwDefaultIndex];
+    	        pValueContent = pKeyCtx->ppDefaultValues[dwDefaultIndex];
+
+    	        goto done;
+    	    }
+    	    // (4) Enumrate default values that has not been cached
+    	    else if (dwDefaultIndex >= pKeyCtx->dwNumCacheDefaultValues)
+    	    {
+    	        status = RegDbQueryDefaultValues(ghCacheConnection,
+    	                                         pKeyCtx->qwId,
+    	                                         1,
+    	                                         dwDefaultIndex,
+    	                                         &sNumDefaultValues,
+    	                                         &ppRegDefaultEntries);
+    	        BAIL_ON_NT_STATUS(status);
+
+    	        if (sNumDefaultValues > 1)
+    	        {
+    	            status = STATUS_INTERNAL_ERROR;
+    	            BAIL_ON_NT_STATUS(status);
+    	        }
+
+    	        pValName = ppRegDefaultEntries[0]->pwszValueName;
+    	        valueType = ppRegDefaultEntries[0]->pValueAttributes->ValueType;
+    	        dwValueLen = ppRegDefaultEntries[0]->pValueAttributes->DefaultValueLen;
+    	        pValueContent = ppRegDefaultEntries[0]->pValueAttributes->pDefaultValue;
+
+    	        goto done;
+    	    }
+    	}
         BAIL_ON_NT_STATUS(status);
 
-        if (sNumValues != 1)
+        if (sNumValues > 1)
         {
             status = STATUS_INTERNAL_ERROR;
             BAIL_ON_NT_STATUS(status);
         }
 
-	pValName = ppRegEntries[0]->pwszValueName;
+    	pValName = ppRegEntries[0]->pwszValueName;
         valueType = ppRegEntries[0]->type;
         dwValueLen = ppRegEntries[0]->dwValueLen;
         pValueContent = ppRegEntries[0]->pValue;
     }
-    else
-    {
-	pValName = pKeyCtx->ppwszValueNames[dwIndex];
-        valueType = pKeyCtx->pTypes[dwIndex];
-        dwValueLen = pKeyCtx->pdwValueLen[dwIndex];
-        pValueContent = pKeyCtx->ppValues[dwIndex];
-    }
 
+done:
     if (pValName)
     {
-	sValueNameLen = RtlWC16StringNumChars(pValName);
+    	sValueNameLen = RtlWC16StringNumChars(pValName);
     }
 
     if (*pcchValueName < sValueNameLen+1)
@@ -1149,7 +1252,7 @@ SqliteEnumValue(
     if (pcbData)
     {
         status = RegCopyValueBytes(pValueContent,
-			                   dwValueLen,
+        		                   dwValueLen,
                                    pData,
                                    pcbData);
         BAIL_ON_NT_STATUS(status);
@@ -1164,6 +1267,8 @@ cleanup:
     LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyCtx->mutex);
 
     RegDbSafeFreeEntryValueList(sNumValues,&ppRegEntries);
+    RegDbSafeFreeEntryValueAttributesList(sNumDefaultValues,
+                                          &ppRegDefaultEntries);
 
     return status;
 
@@ -1172,7 +1277,7 @@ error:
 
     if (pType)
     {
-        *pType = REG_UNKNOWN;
+        *pType = REG_NONE;
     }
 
     goto cleanup;
@@ -1215,7 +1320,7 @@ SqliteDeleteTree(
     ENTER_SQLITE_LOCK(&pConn->lock, bInDbLock);
 
     status = sqlite3_exec(
-		        pConn->pDb,
+    		        pConn->pDb,
                     "begin;",
                     NULL,
                     NULL,
@@ -1233,7 +1338,7 @@ SqliteDeleteTree(
         BAIL_ON_NT_STATUS(status);
 
         status = SqliteDeleteTreeInternal_inlock_inDblock(
-			                           Handle,
+        		                           Handle,
                                            hCurrKey);
         BAIL_ON_NT_STATUS(status);
 
@@ -1254,7 +1359,7 @@ SqliteDeleteTree(
     }
 
     status = sqlite3_exec(
-		        pConn->pDb,
+    		        pConn->pDb,
                     "end",
                     NULL,
                     NULL,
@@ -1281,9 +1386,9 @@ error:
         sqlite3_free(pszError);
     }
     sqlite3_exec(pConn->pDb,
-				 "rollback",
-				 NULL,
-			     NULL,
+	 			 "rollback",
+	 			 NULL,
+	 		     NULL,
 			     NULL);
 
     goto cleanup;
@@ -1324,12 +1429,12 @@ SqliteQueryMultipleValues(
 
     for (iCount = 0; iCount < num_vals; iCount++)
     {
-	BAIL_ON_NT_INVALID_POINTER(pVal_list[iCount].ve_valuename);
+    	BAIL_ON_NT_INVALID_POINTER(pVal_list[iCount].ve_valuename);
 
         status = RegDbGetKeyValue(ghCacheConnection,
-			                  pKeyCtx->qwId,
-			                  pVal_list[iCount].ve_valuename,
-                                  REG_UNKNOWN,
+        		                  pKeyCtx->qwId,
+        		                  pVal_list[iCount].ve_valuename,
+                                  REG_NONE,
                                   NULL,
                                   &ppRegEntries[iCount]);
         BAIL_ON_NT_STATUS(status);
@@ -1373,7 +1478,6 @@ error:
     goto cleanup;
 }
 
-static
 REG_DATA_TYPE
 GetRegDataType(
     REG_DATA_TYPE_FLAGS Flags
@@ -1397,7 +1501,7 @@ GetRegDataType(
             break;
 
         default:
-            dataType = REG_UNKNOWN;
+            dataType = REG_NONE;
     }
 
     return dataType;
@@ -1604,3 +1708,92 @@ cleanup:
 error:
     goto cleanup;
 }
+
+#if 0
+// The following code maintains 'RegGetValue' previous behavior
+NTSTATUS
+SqliteGetValue(
+    IN HANDLE Handle,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN OPTIONAL PCWSTR pValueName,
+    IN OPTIONAL REG_DATA_TYPE_FLAGS Flags,
+    OUT PDWORD pdwType,
+    OUT OPTIONAL PBYTE pData,
+    IN OUT OPTIONAL PDWORD pcbData
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    PWSTR pwszValueName = NULL;
+    PREG_DB_VALUE pRegEntry = NULL;
+    BOOLEAN bIsWrongType = FALSE;
+    wchar16_t wszEmptyValueName[] = REG_EMPTY_VALUE_NAME_W;
+    PWSTR pwszKeyNameWithSubKey = NULL;
+    PCWSTR pwszFullKeyName = NULL;
+    PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
+    PREG_KEY_CONTEXT pKeyCtx = NULL;
+    PREG_KEY_HANDLE pKeyHandleInUse = NULL;
+    PREG_KEY_CONTEXT pKeyCtxInUse = NULL;
+
+    BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
+    pKeyCtx = pKeyHandle->pKey;
+    BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtx);
+
+    if (pSubKey)
+    {
+        status = LwRtlWC16StringAllocatePrintfW(
+                        &pwszKeyNameWithSubKey,
+                        L"%ws\\%ws",
+                        pKeyCtx->pwszKeyName,
+                        pSubKey);
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    pwszFullKeyName = pSubKey ? pwszKeyNameWithSubKey : pKeyCtx->pwszKeyName;
+
+    status = SqliteOpenKeyInternal(Handle,
+                                   pwszFullKeyName,
+                                   KEY_QUERY_VALUE,
+                                   &pKeyHandleInUse);
+    BAIL_ON_NT_STATUS(status);
+
+    // ACL check
+    status = RegSrvAccessCheckKeyHandle(pKeyHandleInUse, KEY_QUERY_VALUE);
+    BAIL_ON_NT_STATUS(status);
+
+    pKeyCtxInUse = pKeyHandleInUse->pKey;
+    BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtxInUse);
+
+    status = LwRtlWC16StringDuplicate(&pwszValueName, !pValueName ? wszEmptyValueName : pValueName);
+    BAIL_ON_NT_STATUS(status);
+
+    status = RegDbGetKeyValue(ghCacheConnection,
+                              pKeyCtxInUse->qwId,
+                              pwszValueName,
+                              GetRegDataType(Flags),
+                              &bIsWrongType,
+                              &pRegEntry);
+    BAIL_ON_NT_STATUS(status);
+
+    status = RegCopyValueBytes(pRegEntry->pValue,
+                               pRegEntry->dwValueLen,
+                               pData,
+                               pcbData);
+    BAIL_ON_NT_STATUS(status);
+
+    *pdwType = (DWORD)pRegEntry->type;
+
+cleanup:
+    SqliteSafeFreeKeyHandle(pKeyHandleInUse);
+    LWREG_SAFE_FREE_MEMORY(pwszValueName);
+    LWREG_SAFE_FREE_MEMORY(pwszKeyNameWithSubKey);
+    RegDbSafeFreeEntryValue(&pRegEntry);
+
+    return status;
+
+error:
+   *pdwType = 0;
+
+    goto cleanup;
+}
+#endif
