@@ -205,6 +205,58 @@ SqliteGetKeySecurity(
     IN OUT PULONG pulSecDescRelLen
     );
 
+//
+// Attribute related APIs
+//
+NTSTATUS
+SqliteSetValueAttributes(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pSubKey,
+    IN PCWSTR pValueName,
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    );
+
+NTSTATUS
+SqliteGetValueAttributes_Internal(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pValueName,
+    IN OPTIONAL REG_DATA_TYPE dwType,
+    // Whether bail or not in case there is no value attributes
+    IN BOOLEAN bDoBail,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    );
+
+NTSTATUS
+SqliteGetValueAttributes(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pValueName,
+    OUT OPTIONAL PLWREG_CURRENT_VALUEINFO* ppCurrentValue,
+    OUT OPTIONAL PLWREG_VALUE_ATTRIBUTES* ppValueAttributes
+    );
+
+NTSTATUS
+SqliteDeleteValueAttributes(
+    IN HANDLE hRegConnection,
+    IN HKEY hKey,
+    IN OPTIONAL PCWSTR pwszSubKey,
+    IN PCWSTR pValueName
+    );
+
+NTSTATUS
+SqliteQueryInfoDefaultValues(
+    IN REG_DB_HANDLE hDb,
+    IN PREG_KEY_CONTEXT pKey,
+    OUT OPTIONAL PDWORD pcDefaultValues,
+    OUT OPTIONAL PDWORD pcMaxDefaultValueNameLen,
+    OUT OPTIONAL PDWORD pcMaxDefaultValueLen
+    );
+
 /* Obsolete API */
 NTSTATUS
 SqliteQueryMultipleValues(
