@@ -65,7 +65,7 @@ SqliteGetKeyToken(
     if (pwszFound)
     {
         status = LW_RTL_ALLOCATE((PVOID*)&pwszOutputString, wchar16_t,
-			                  sizeof(*pwszOutputString)* (pwszFound - pwszInputString +1));
+        		                  sizeof(*pwszOutputString)* (pwszFound - pwszInputString +1));
         BAIL_ON_NT_STATUS(status);
 
         memcpy(pwszOutputString, pwszInputString,(pwszFound - pwszInputString) * sizeof(*pwszOutputString));
@@ -101,7 +101,7 @@ SqliteGetParentKeyName(
     if (pwszFound)
     {
         status = LW_RTL_ALLOCATE((PVOID*)&pwszOutputString, wchar16_t,
-			                  sizeof(*pwszOutputString)* (pwszFound - pwszInputString +1));
+        		                  sizeof(*pwszOutputString)* (pwszFound - pwszInputString +1));
         BAIL_ON_NT_STATUS(status);
 
         memcpy(pwszOutputString, pwszInputString,(pwszFound - pwszInputString) * sizeof(*pwszOutputString));
@@ -347,7 +347,7 @@ done:
 
     if (pdwDisposition)
     {
-	*pdwDisposition =  dwDisposition;
+    	*pdwDisposition =  dwDisposition;
     }
 
 cleanup:
@@ -400,9 +400,9 @@ SqliteOpenKeyInternal(
 
     // pServerState->pToken should be created at this point
     status = SqliteOpenKeyInternal_inlock(handle,
-		                              pwszFullKeyName,
-		                              AccessDesired,
-		                              ppKeyHandle);
+    		                              pwszFullKeyName,
+    		                              AccessDesired,
+    		                              ppKeyHandle);
     BAIL_ON_NT_STATUS(status);
 
 cleanup:
@@ -535,7 +535,6 @@ SqliteDeleteKeyInternal_inlock(
     )
 {
 	NTSTATUS status = STATUS_SUCCESS;
-    size_t sSubkeyCount = 0;
     PWSTR pwszParentKeyName = NULL;
     PREG_KEY_HANDLE pKeyHandle = NULL;
     // Do not free
@@ -545,8 +544,8 @@ SqliteDeleteKeyInternal_inlock(
     BAIL_ON_NT_STATUS(status);
 
     status = SqliteOpenKeyInternal_inlock(handle,
-		                              pwszKeyName,
-		                              0,
+    		                              pwszKeyName,
+    		                              0,
                                           &pKeyHandle);
     BAIL_ON_NT_STATUS(status);
 
@@ -554,33 +553,16 @@ SqliteDeleteKeyInternal_inlock(
     pKeyCtx = pKeyHandle->pKey;
     BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtx);
 
-    // Delete key from DB
-    // Make sure this key does not have subkey before go ahead and delete it
-    // Also need to delete the all of this subkey's values
-    status = RegDbQueryInfoKeyCount(ghCacheConnection,
-		                        pKeyCtx->qwId,
-                                    QuerySubKeys,
-                                    &sSubkeyCount);
+    // Delete all the values of this key
+    status = RegDbDeleteKey(ghCacheConnection, pKeyCtx->qwId, pKeyCtx->qwSdId, pwszKeyName);
     BAIL_ON_NT_STATUS(status);
 
-    if (sSubkeyCount == 0)
-    {
-        // Delete all the values of this key
-        status = RegDbDeleteKey(ghCacheConnection, pKeyCtx->qwId, pKeyCtx->qwSdId, pwszKeyName);
-        BAIL_ON_NT_STATUS(status);
+    status = SqliteGetParentKeyName(pwszKeyName, '\\',&pwszParentKeyName);
+    BAIL_ON_NT_STATUS(status);
 
-        status = SqliteGetParentKeyName(pwszKeyName, '\\',&pwszParentKeyName);
-        BAIL_ON_NT_STATUS(status);
-
-        if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
-        {
-		SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
-        }
-    }
-    else
+    if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
     {
-	status = STATUS_KEY_HAS_CHILDREN;
-        BAIL_ON_NT_STATUS(status);
+        SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
     }
 
 cleanup:
@@ -602,7 +584,6 @@ SqliteDeleteKeyInternal_inlock_inDblock(
     )
 {
 	NTSTATUS status = STATUS_SUCCESS;
-    size_t sSubkeyCount = 0;
     PWSTR pwszParentKeyName = NULL;
     PREG_KEY_HANDLE pKeyHandle = NULL;
     // Do not free
@@ -612,8 +593,8 @@ SqliteDeleteKeyInternal_inlock_inDblock(
     BAIL_ON_NT_STATUS(status);
 
     status = SqliteOpenKeyInternal_inlock_inDblock(handle,
-		                                pwszKeyName,
-		                                0,
+    		                                pwszKeyName,
+    		                                0,
                                             &pKeyHandle);
     BAIL_ON_NT_STATUS(status);
 
@@ -621,33 +602,16 @@ SqliteDeleteKeyInternal_inlock_inDblock(
     pKeyCtx = pKeyHandle->pKey;
     BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtx);
 
-    // Delete key from DB
-    // Make sure this key does not have subkey before go ahead and delete it
-    // Also need to delete the all of this subkey's values
-    status = RegDbQueryInfoKeyCount_inlock(ghCacheConnection,
-		                        pKeyCtx->qwId,
-                                    QuerySubKeys,
-                                    &sSubkeyCount);
+    // Delete all the values of this key
+    status = RegDbDeleteKey_inlock(ghCacheConnection, pKeyCtx->qwId, pKeyCtx->qwSdId, pwszKeyName);
     BAIL_ON_NT_STATUS(status);
 
-    if (sSubkeyCount == 0)
-    {
-        // Delete all the values of this key
-        status = RegDbDeleteKey_inlock(ghCacheConnection, pKeyCtx->qwId, pKeyCtx->qwSdId, pwszKeyName);
-        BAIL_ON_NT_STATUS(status);
+    status = SqliteGetParentKeyName(pwszKeyName, '\\',&pwszParentKeyName);
+    BAIL_ON_NT_STATUS(status);
 
-        status = SqliteGetParentKeyName(pwszKeyName, '\\',&pwszParentKeyName);
-        BAIL_ON_NT_STATUS(status);
-
-        if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
-        {
-		SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
-        }
-    }
-    else
+    if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
     {
-	status = STATUS_KEY_HAS_CHILDREN;
-        BAIL_ON_NT_STATUS(status);
+        SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
     }
 
 cleanup:
