@@ -76,7 +76,7 @@ SqliteCacheLocateActiveKey_inlock(
     DWORD status = 0;
 
     status = RegHashGetValue(gActiveKeyList.pKeyList,
-		                 pwszKeyName,
+    		                 pwszKeyName,
                               (PVOID*)&pKeyResult);
     if (!status && pKeyResult)
     {
@@ -156,8 +156,8 @@ SqliteCacheDeleteActiveKey_inlock(
     NTSTATUS status = 0;
 
     status = RegHashGetValue(gActiveKeyList.pKeyList,
-		                 pwszKeyName,
-		                 (PVOID*)&pKeyResult);
+    		                 pwszKeyName,
+    		                 (PVOID*)&pKeyResult);
     if (STATUS_OBJECT_NAME_NOT_FOUND == status)
     {
         return;
@@ -167,9 +167,9 @@ SqliteCacheDeleteActiveKey_inlock(
 
     for (iCount = 0; (pHashEntry = RegHashNext(&hashIterator)) != NULL; iCount++)
     {
-	if (LwRtlWC16StringIsEqual((PCWSTR)pHashEntry->pKey, pwszKeyName, FALSE))
-	{
-		RegHashRemoveKey(gActiveKeyList.pKeyList, pHashEntry->pKey);
+    	if (LwRtlWC16StringIsEqual((PCWSTR)pHashEntry->pKey, pwszKeyName, FALSE))
+    	{
+    		RegHashRemoveKey(gActiveKeyList.pKeyList, pHashEntry->pKey);
 
             break;
         }
@@ -321,7 +321,7 @@ SqliteCacheFreeKeyCtxHashEntry(
 
     if (pKeyResult)
     {
-	RegSrvSafeFreeKeyContext(pKeyResult);
+    	RegSrvSafeFreeKeyContext(pKeyResult);
     }
 }
 
@@ -342,7 +342,7 @@ SqliteCacheSubKeysInfo_inlock(
     }
 
     status = RegDbQueryInfoKeyCount(ghCacheConnection,
-		                        pKeyResult->qwId,
+    		                        pKeyResult->qwId,
                                     QuerySubKeys,
                                     &sNumSubKeys);
     BAIL_ON_NT_STATUS(status);
@@ -352,8 +352,8 @@ SqliteCacheSubKeysInfo_inlock(
                       : sNumSubKeys;
 
     status = RegDbQueryInfoKey(ghCacheConnection,
-		                   pKeyResult->pwszKeyName,
-		                   pKeyResult->qwId,
+    		                   pKeyResult->pwszKeyName,
+    		                   pKeyResult->qwId,
                                sNumCacheSubKeys,
                                0,
                                &sNumCacheSubKeys,
@@ -392,7 +392,7 @@ SqliteCacheSubKeysInfo_inlock_inDblock(
     }
 
     status = RegDbQueryInfoKeyCount_inlock(ghCacheConnection,
-		                        pKeyResult->qwId,
+    		                        pKeyResult->qwId,
                                     QuerySubKeys,
                                     &sNumSubKeys);
     BAIL_ON_NT_STATUS(status);
@@ -402,8 +402,8 @@ SqliteCacheSubKeysInfo_inlock_inDblock(
                       : sNumSubKeys;
 
     status = RegDbQueryInfoKey_inlock(ghCacheConnection,
-		                   pKeyResult->pwszKeyName,
-		                   pKeyResult->qwId,
+    		                   pKeyResult->pwszKeyName,
+    		                   pKeyResult->qwId,
                                sNumCacheSubKeys,
                                0,
                                &sNumCacheSubKeys,
@@ -460,16 +460,16 @@ SqliteCacheKeySecurityDescriptor_inlock(
     ULONG ulSecDescRelLen = 0;
 
     if (pKeyResult->bHasSdInfo)
-	goto cleanup;
+    	goto cleanup;
 
     status = RegDbGetKeyAclByKeyId(ghCacheConnection,
 			                       pKeyResult->qwId,
 			                       &pKeyResult->qwSdId,
 			                       &pSecurityDescriptor,
-		                           &ulSecDescRelLen);
+	   	                           &ulSecDescRelLen);
 	BAIL_ON_NT_STATUS(status);
 
-    status = RegSrvSetKeySecurityDescriptor_inlock(pKeyResult,
+    status = RegSrvSetKeySecurityDescriptor_inlock(pKeyResult, 
                                                    pSecurityDescriptor,
                                                    ulSecDescRelLen);
     BAIL_ON_NT_STATUS(status);
@@ -523,8 +523,8 @@ SqliteCacheUpdateSubKeysInfo_inlock(
     size_t sSubKeyLen = 0;
 
     status = RegDbQueryInfoKey(ghCacheConnection,
-		                   pKeyResult->pwszKeyName,
-		                   pKeyResult->qwId,
+    		                   pKeyResult->pwszKeyName,
+    		                   pKeyResult->qwId,
                                dwDefaultCacheSize,
                                dwOffSet,
                                &sNumSubKeys,
@@ -601,7 +601,7 @@ SqliteCacheKeyValuesInfo_inlock(
     }
 
     status = RegDbQueryInfoKeyCount(ghCacheConnection,
-		                        pKeyResult->qwId,
+    		                        pKeyResult->qwId,
                                     QueryValues,
                                     &sNumValues);
     BAIL_ON_NT_STATUS(status);
@@ -611,7 +611,7 @@ SqliteCacheKeyValuesInfo_inlock(
                       : sNumValues;
 
     status = RegDbQueryInfoKeyValue(ghCacheConnection,
-		                        pKeyResult->qwId,
+    		                        pKeyResult->qwId,
                                     sNumCacheValues,
                                     0,
                                     &sNumCacheValues,
@@ -674,7 +674,7 @@ SqliteCacheUpdateValuesInfo_inlock(
 
 
     status = RegDbQueryInfoKeyValue(ghCacheConnection,
-		                        pKeyResult->qwId,
+    		                        pKeyResult->qwId,
                                     dwDefaultCacheSize,
                                     dwOffSet,
                                     &sNumValues,
@@ -693,7 +693,7 @@ SqliteCacheUpdateValuesInfo_inlock(
 			pKeyResult->sMaxValueNameLen = sValueNameLen;
 
         status = RegCopyValueBytes(ppRegEntries[iCount]->pValue,
-			                   ppRegEntries[iCount]->dwValueLen,
+        		                   ppRegEntries[iCount]->dwValueLen,
                                    NULL,
                                    &dwValueLen);
         BAIL_ON_NT_STATUS(status);
@@ -753,7 +753,7 @@ SqliteCacheFreeDbKeyHashEntry(
 
     if (pRegKey)
     {
-	RegDbSafeFreeEntryKey(&pRegKey);
+    	RegDbSafeFreeEntryKey(&pRegKey);
     }
 }
 
@@ -772,7 +772,7 @@ SqliteCacheGetDbKeyInfo(
     LWREG_LOCK_MUTEX(bInLock, &gRegDbKeyList.mutex);
 
     status = RegHashGetValue(gRegDbKeyList.pKeyList,
-		                 pwszKeyName,
+    		                 pwszKeyName,
                              (PVOID*)&pRegKeyRef);
     BAIL_ON_NT_STATUS(status);
 
@@ -869,8 +869,8 @@ SqliteCacheDeleteDbKeyInfo_inlock(
     int iCount = 0;
 
     status = RegHashGetValue(gRegDbKeyList.pKeyList,
-		                 (PCVOID)pwszKeyName,
-		                 (PVOID*)&pRegKey);
+    		                 (PCVOID)pwszKeyName,
+    		                 (PVOID*)&pRegKey);
     if (STATUS_OBJECT_NAME_NOT_FOUND == status)
     {
         return;
@@ -880,9 +880,9 @@ SqliteCacheDeleteDbKeyInfo_inlock(
 
     for (iCount = 0; (pHashEntry = RegHashNext(&hashIterator)) != NULL; iCount++)
     {
-	if (LwRtlWC16StringIsEqual((PCWSTR)pHashEntry->pKey, pwszKeyName, FALSE))
-	{
-		RegHashRemoveKey(gRegDbKeyList.pKeyList, pHashEntry->pKey);
+    	if (LwRtlWC16StringIsEqual((PCWSTR)pHashEntry->pKey, pwszKeyName, FALSE))
+    	{
+    		RegHashRemoveKey(gRegDbKeyList.pKeyList, pHashEntry->pKey);
 
             break;
         }
@@ -912,6 +912,171 @@ SqliteReleaseDbKeyInfo_inlock(
 {
     if (pRegKey)
     {
-	SqliteCacheDeleteDbKeyInfo_inlock(pRegKey->pwszFullKeyName);
+    	SqliteCacheDeleteDbKeyInfo_inlock(pRegKey->pwszFullKeyName);
     }
 }
+
+
+NTSTATUS
+SqliteCacheKeyDefaultValuesInfo_inlock(
+    IN OUT PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    size_t sNumDefaultValues = 0;
+    size_t sNumCacheDefaultValues = 0;
+    PREG_DB_VALUE_ATTRIBUTES* ppRegEntries = NULL;
+
+    if (pKeyResult->bHasDefaultValueInfo)
+    {
+        goto cleanup;
+    }
+
+    status = RegDbQueryDefaultValuesCount(ghCacheConnection,
+                                    pKeyResult->qwId,
+                                    &sNumDefaultValues);
+    BAIL_ON_NT_STATUS(status);
+
+    sNumCacheDefaultValues = (sNumDefaultValues > (size_t)dwDefaultCacheSize)
+                      ? dwDefaultCacheSize
+                      : sNumDefaultValues;
+
+    status = RegDbQueryDefaultValues(ghCacheConnection,
+                                    pKeyResult->qwId,
+                                    sNumCacheDefaultValues,
+                                    0,
+                                    &sNumCacheDefaultValues,
+                                    &ppRegEntries);
+    BAIL_ON_NT_STATUS(status);
+
+    status = RegDbSafeRecordDefaultValuesInfo_inlock(
+                            sNumDefaultValues,
+                            sNumCacheDefaultValues,
+                            ppRegEntries,
+                            pKeyResult);
+    BAIL_ON_NT_STATUS(status);
+
+cleanup:
+    RegDbSafeFreeEntryValueAttributesList(sNumCacheDefaultValues,
+                                          &ppRegEntries);
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+NTSTATUS
+SqliteCacheKeyDefaultValuesInfo(
+    IN OUT PREG_KEY_CONTEXT pKeyResult
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    BOOLEAN bInLock = FALSE;
+
+    BAIL_ON_NT_INVALID_POINTER(pKeyResult);
+
+    LWREG_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pKeyResult->mutex);
+
+    status = SqliteCacheKeyDefaultValuesInfo_inlock(pKeyResult);
+    BAIL_ON_NT_STATUS(status);
+
+cleanup:
+    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+NTSTATUS
+SqliteCacheUpdateDefaultValuesInfo_inlock(
+    DWORD dwOffSet,
+    IN OUT PREG_KEY_CONTEXT pKeyResult,
+    OUT size_t* psNumValues
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    size_t sNumValues = 0;
+    PREG_DB_VALUE_ATTRIBUTES* ppRegEntries = NULL;
+    int iCount = 0;
+    size_t sValueNameLen = 0;
+    DWORD dwValueLen = 0;
+
+
+    status = RegDbQueryDefaultValues(ghCacheConnection,
+                                    pKeyResult->qwId,
+                                    dwDefaultCacheSize,
+                                    dwOffSet,
+                                    &sNumValues,
+                                    &ppRegEntries);
+    BAIL_ON_NT_STATUS(status);
+
+    for (iCount = 0; iCount < (int)sNumValues; iCount++)
+    {
+        if (!ppRegEntries[iCount]->pValueAttributes)
+        {
+            status = STATUS_INTERNAL_ERROR;
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (ppRegEntries[iCount]->pwszValueName)
+        {
+            sValueNameLen = RtlWC16StringNumChars(ppRegEntries[iCount]->pwszValueName);
+        }
+
+        if (pKeyResult->sMaxValueNameLen < sValueNameLen)
+            pKeyResult->sMaxValueNameLen = sValueNameLen;
+
+        status = RegCopyValueBytes(ppRegEntries[iCount]->pValueAttributes->pDefaultValue,
+                                   ppRegEntries[iCount]->pValueAttributes->DefaultValueLen,
+                                   NULL,
+                                   &dwValueLen);
+        BAIL_ON_NT_STATUS(status);
+
+        if (pKeyResult->sMaxValueLen < (size_t)dwValueLen)
+            pKeyResult->sMaxValueLen = (size_t)dwValueLen;
+
+        sValueNameLen = 0;
+        dwValueLen = 0;
+    }
+
+cleanup:
+    *psNumValues = sNumValues;
+    RegDbSafeFreeEntryValueAttributesList(sNumValues,
+                                          &ppRegEntries);
+
+    return status;
+
+error:
+    goto cleanup;
+}
+
+NTSTATUS
+SqliteCacheUpdateDefaultValuesInfo(
+    DWORD dwOffSet,
+    IN OUT PREG_KEY_CONTEXT pKeyResult,
+    OUT size_t* psNumValues
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    BOOLEAN bInLock = FALSE;
+
+    BAIL_ON_NT_INVALID_POINTER(pKeyResult);
+
+    LWREG_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pKeyResult->mutex);
+
+    status = SqliteCacheUpdateDefaultValuesInfo_inlock(dwOffSet,
+                                                       pKeyResult,
+                                                       psNumValues);
+    BAIL_ON_NT_STATUS(status);
+
+cleanup:
+    LWREG_UNLOCK_RWMUTEX(bInLock, &pKeyResult->mutex);
+
+    return status;
+
+error:
+    goto cleanup;
+}
