@@ -144,6 +144,7 @@ typedef enum _REGSHELL_CMDLINE_STATE_E
     REGSHELL_CMDLINE_STATE_ADDVALUE_KEYNAME,
     REGSHELL_CMDLINE_STATE_ADDVALUE_VALUENAME,
     REGSHELL_CMDLINE_STATE_ADDVALUE_TYPE,
+    REGSHELL_CMDLINE_STATE_ADDVALUE_OPTION,
     REGSHELL_CMDLINE_STATE_ADDVALUE_VALUE,
     REGSHELL_CMDLINE_STATE_ADDVALUE_STOP,
 
@@ -174,23 +175,26 @@ static REGSHELL_CMD_ID shellCmds[] = {
     { "import", REGSHELL_CMD_IMPORT           },
     { "export", REGSHELL_CMD_EXPORT           },
     { "upgrade", REGSHELL_CMD_UPGRADE         },
+    { "cleanup", REGSHELL_CMD_CLEANUP         },
 };
 
 
 DWORD
 RegShellCmdEnumToString(
     REGSHELL_CMD_E cmdEnum,
-    PCHAR cmdString)
+    PCHAR cmdString,
+    DWORD dwCmdStringLen)
 {
     DWORD dwError = 0;
 
     if (cmdEnum<sizeof(shellCmds)/sizeof(shellCmds[0]))
     {
-        strcpy(cmdString, shellCmds[cmdEnum].pszCommand);
+        cmdString[0] = '\0';
+        strncat(cmdString, shellCmds[cmdEnum].pszCommand, dwCmdStringLen-1);
     }
     else
     {
-        sprintf(cmdString, "Invalid Command: %d", cmdEnum);
+        snprintf(cmdString, dwCmdStringLen, "Invalid Command: %d", cmdEnum);
         dwError = LWREG_ERROR_INVALID_CONTEXT;
     }
     return dwError;
@@ -249,6 +253,7 @@ cleanup:
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pNewCmdItem);
     goto cleanup;
 }
 
@@ -290,7 +295,7 @@ error:
  *   REGSHELL_CMD_CHDIR
  *
  * Outputs:
- * pParseState->pszFullRootKeyName
+ * pParseState->pszFullRootKeyName 
  *    "HKEY_THIS_MACHINE"
  *
  * pCmdItem
@@ -314,7 +319,7 @@ RegShellCmdParseKeyName(
     PSTR pszKeyName = NULL; /* Working copy of original pszInKeyName */
     PSTR pszRootKeyName = NULL;
     PSTR pszSubKey = NULL;
-
+    
     BAIL_ON_INVALID_POINTER(pParseState);
     BAIL_ON_INVALID_POINTER(pszInKeyName);
 
@@ -329,7 +334,7 @@ RegShellCmdParseKeyName(
     }
     dwError = RegCStringDuplicate(&pszKeyName, pszKeyName);
     BAIL_ON_REG_ERROR(dwError);
-
+    
     pszCursor = pszKeyName + strlen(pszKeyName) - 1;
     if (*pszCursor == ']')
     {
@@ -342,7 +347,7 @@ RegShellCmdParseKeyName(
         *pszCursor-- = '\0';
     }
 
-    /*
+    /* 
      * Find \ separator, and if found, determine if stuff before \
      * is a valid root key. If no \ is found, test the stuff passed in
      * for being a root key anyway.
@@ -373,6 +378,8 @@ RegShellCmdParseKeyName(
     {
         /* Valid root key found */
         RegCloseKey(pParseState->hReg, hRootKey);
+    
+        LWREG_SAFE_FREE_STRING(pParseState->pszFullRootKeyName);
         pParseState->pszFullRootKeyName = pszRootKeyName;
         pszRootKeyName = NULL;
         bFoundRootKey = TRUE;
@@ -383,7 +390,7 @@ RegShellCmdParseKeyName(
 
     if (!bFoundRootKey)
     {
-        /*
+        /* 
          * Path provided does not start with a root key.
          * Assume this is a subkey, and use it as such.
          */
@@ -391,6 +398,7 @@ RegShellCmdParseKeyName(
         BAIL_ON_REG_ERROR(dwError);
 
         /* Use default HKEY_THIS_MACHINE root key */
+        LWREG_SAFE_FREE_STRING(pParseState->pszFullRootKeyName);
         dwError = RegCStringDuplicate(
                       &pParseState->pszFullRootKeyName,
                       HKEY_THIS_MACHINE);
@@ -426,7 +434,7 @@ RegShellParseStringType(
     DWORD i = 0;
     DWORD dwError = 0;
     DWORD backendType = 0;
-    REG_DATA_TYPE type = REG_UNKNOWN;
+    REG_DATA_TYPE type = REG_NONE;
 
     static REGSHELL_REG_TYPE_ENTRY typeStrs[] = {
         { "REG_DWORD", REG_DWORD, RRF_RT_REG_DWORD             },
@@ -535,8 +543,8 @@ RegShellImportDwordString(
     {
         if (!isspace((int) *pszPtr))
         {
-            /*
-             * Trap an invalid character was present in the string
+            /* 
+             * Trap an invalid character was present in the string 
              * number value, ignore trailing white space.
              */
             dwError = RegMapErrnoToLwRegError(EINVAL);
@@ -593,11 +601,12 @@ RegShellImportDwordString(
     *pdwValue = (DWORD) ulValue;
     *ppBinaryValue = (PUCHAR) pdwValue;
     *pBinaryValueLen = sizeof(DWORD);
-
+    
 cleanup:
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pdwValue);
     goto cleanup;
 }
 
@@ -812,15 +821,16 @@ RegShellImportBinaryString(
         *ppBinaryValue = binaryValue;
         *pBinaryValueLen = binaryValueLen;
     }
+    else
+    {
+        LWREG_SAFE_FREE_MEMORY(binaryValue);
+    }
 
 cleanup:
     return dwError;
 
 error:
-    if (binaryValue)
-    {
-        RegMemoryFree(binaryValue);
-    }
+    LWREG_SAFE_FREE_MEMORY(binaryValue);
     goto cleanup;
 }
 
@@ -858,6 +868,7 @@ RegShellCmdParseValueName(
                       cmd,
                       argv[2],
                       &pCmdItem);
+        BAIL_ON_REG_ERROR(dwError);
         argIndx = 3;
     }
     else
@@ -865,7 +876,9 @@ RegShellCmdParseValueName(
         dwError = RegShellCmdParseCommand(
                       cmd,
                       &pCmdItem);
+        BAIL_ON_REG_ERROR(dwError);
     }
+    BAIL_ON_INVALID_POINTER(pCmdItem);
 
     pszValue = argv[argIndx++];
 
@@ -883,14 +896,11 @@ RegShellCmdParseValueName(
     }
     else
     {
-        if (pCmdItem)
+        pszKeyName = pCmdItem->keyName;
+        if (pszKeyName && pszKeyName[0] == '\\')
         {
-            pszKeyName = pCmdItem->keyName;
-            if (pszKeyName && pszKeyName[0] == '\\')
-            {
-                pszKeyName++;
-                bFullPath = TRUE;
-            }
+            pszKeyName++;
+            bFullPath = TRUE;
         }
 
         dwError = RegShellUtilGetValue(
@@ -904,14 +914,14 @@ RegShellCmdParseValueName(
                       NULL);
         BAIL_ON_REG_ERROR(dwError);
     }
-    argCount = argc - argIndx;
-    if (argCount < 0)
+    if (argIndx >= argc)
     {
         dwError = LWREG_ERROR_INVALID_CONTEXT;
         goto error;
     }
     BAIL_ON_REG_ERROR(dwError);
 
+    argCount = argc - argIndx;
     dwError = RegCStringDuplicate(&pCmdItem->valueName, pszValue);
     BAIL_ON_REG_ERROR(dwError);
 
@@ -1053,7 +1063,7 @@ RegShellDumpCmdItem(
 
     BAIL_ON_INVALID_POINTER(rsItem);
 
-    RegShellCmdEnumToString(rsItem->command, tokenName);
+    RegShellCmdEnumToString(rsItem->command, tokenName, sizeof(tokenName));
     printf("DumpCmd: command=%s\n", tokenName);
     if (rsItem->keyName)
     {
@@ -1063,7 +1073,7 @@ RegShellDumpCmdItem(
     {
         printf("DumpCmd: valueName=%s\n", rsItem->valueName);
     }
-    if (rsItem->type != REG_UNKNOWN)
+    if (rsItem->type != REG_NONE)
     {
         RegExportBinaryTypeToString(
             rsItem->type,
@@ -1134,6 +1144,63 @@ error:
 }
 
 
+DWORD
+RegShellPszIsValidKey(
+    PREGSHELL_PARSE_STATE pParseState,
+    PSTR pszKey,
+    PBOOLEAN pbIsValid)
+{
+
+    DWORD dwError = 0;
+    PSTR pszRootKey = NULL;
+    PSTR pszSubKey = NULL;
+    PSTR pszPtr = NULL;
+    
+    BAIL_ON_INVALID_POINTER(pParseState);
+    BAIL_ON_INVALID_POINTER(pszKey);
+    BAIL_ON_INVALID_POINTER(pbIsValid);
+
+    if (*pszKey == '[' && strchr(pszKey, '\\'))
+    {
+        /*  Assume stuff between [ and \ is root key */
+        dwError = RegCStringDuplicate(
+                      (LW_PVOID) &pszRootKey, &pszKey[1]);
+        BAIL_ON_REG_ERROR(dwError);
+        pszPtr = strchr(pszRootKey, '\\');
+        *pszPtr++ = '\0';
+        
+        dwError = RegCStringDuplicate(
+                      (LW_PVOID) &pszSubKey, pszPtr);
+        BAIL_ON_REG_ERROR(dwError);
+        pszPtr = strrchr(pszSubKey, ']');
+        if (pszPtr)
+        {
+            *pszPtr = '\0';
+        }
+    }
+    else
+    {
+        /* Assume value is subkey */
+        dwError = RegCStringDuplicate(
+                      (LW_PVOID) &pszSubKey, pszKey);
+        BAIL_ON_REG_ERROR(dwError);
+    }
+    dwError = RegShellIsValidKey(
+                  pParseState->hReg,
+                  pszRootKey,
+                  pszSubKey);
+    *pbIsValid = (dwError == 0) ? TRUE : FALSE;
+    
+cleanup:
+    LWREG_SAFE_FREE_STRING(pszRootKey);
+    LWREG_SAFE_FREE_STRING(pszSubKey);
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
 DWORD
 RegShellCmdParse(
     PREGSHELL_PARSE_STATE pParseState,
@@ -1144,7 +1211,12 @@ RegShellCmdParse(
     REGSHELL_CMD_E cmd = 0;
     DWORD dwError = 0;
     PSTR pszCommand = NULL;
+    PSTR pszSubKeyName = "";
     PREGSHELL_CMD_ITEM pCmdItem = NULL;
+    DWORD dwArgc = 2;
+    DWORD dwNewArgc = 0;
+    DWORD dwExportFormat = 0;
+    BOOLEAN bIsValidKey = FALSE;
 
     BAIL_ON_INVALID_POINTER(argv);
 
@@ -1173,35 +1245,73 @@ RegShellCmdParse(
                 dwError = RegShellCmdParseCommand(cmd, &pCmdItem);
             }
             break;
+
         case REGSHELL_CMD_EXPORT:
-            dwError = RegShellCmdParseCommand(cmd, &pCmdItem);
+            if (argc > 2)
+            {
+                if (!strcmp("--legacy", argv[dwArgc]))
+                {
+                    dwExportFormat = REGSHELL_EXPORT_LEGACY;
+                    dwArgc++;
+                }
+                else if (!strcmp("--values", argv[dwArgc]))
+                {
+                    dwExportFormat = REGSHELL_EXPORT_VALUES;
+                    dwArgc++;
+                }
+            }
+
+            dwError = RegShellPszIsValidKey(
+                          pParseState,
+                          argv[dwArgc],
+                          &bIsValidKey);
+            if (dwError == 0 && bIsValidKey && dwArgc < argc)
+            {
+                pszSubKeyName = argv[dwArgc++];
+            }
+
+            dwError = RegShellCmdParseKeyName(
+                          pParseState,
+                          cmd,
+                          pszSubKeyName,
+                          &pCmdItem);
             BAIL_ON_REG_ERROR(dwError);
-            if (argc > 3)
+
+            dwError = RegAllocateMemory(
+                          sizeof(PSTR) * argc,
+                          (PVOID*)&pCmdItem->args);
+            BAIL_ON_REG_ERROR(dwError);
+
+            if (dwExportFormat == REGSHELL_EXPORT_LEGACY)
             {
-                dwError = RegShellCmdParseKeyName(
-                              pParseState,
-                              cmd,
-                              argv[2],
-                              &pCmdItem);
-                BAIL_ON_REG_ERROR(dwError);
-                dwError = RegAllocateMemory(
-                              sizeof(PSTR) * 2,
-                              (PVOID*)&pCmdItem->args);
-                BAIL_ON_REG_ERROR(dwError);
                 dwError = RegCStringDuplicate(
-                              (LW_PVOID) &pCmdItem->args[0], argv[3]);
+                              (LW_PVOID) &pCmdItem->args[dwNewArgc++],
+                              "--legacy");
+                BAIL_ON_REG_ERROR(dwError);
+            }
+            else if (dwExportFormat == REGSHELL_EXPORT_VALUES)
+            {
+                dwError = RegCStringDuplicate(
+                              (LW_PVOID) &pCmdItem->args[dwNewArgc++],
+                              "--values");
+                BAIL_ON_REG_ERROR(dwError);
+            }
+
+            if (dwArgc < argc)
+            {
+                dwError = RegCStringDuplicate(
+                              (LW_PVOID) &pCmdItem->args[dwNewArgc++],
+                              argv[dwArgc++]);
                 BAIL_ON_REG_ERROR(dwError);
-                pCmdItem->argsCount = 1;
             }
             else
             {
-                dwError = RegAllocateMemory(
-                              sizeof(PSTR) * 2,
-                              (PVOID*)&pCmdItem->args);
-                BAIL_ON_REG_ERROR(dwError);
                 dwError = RegCStringDuplicate(
-                              (LW_PVOID) &pCmdItem->args[0], argv[2]);
+                              (LW_PVOID) &pCmdItem->args[dwNewArgc++],
+                              "-");
+                BAIL_ON_REG_ERROR(dwError);
             }
+            pCmdItem->argsCount = dwNewArgc;
             break;
 
         case REGSHELL_CMD_LIST_KEYS:
@@ -1225,6 +1335,7 @@ RegShellCmdParse(
 
         case REGSHELL_CMD_IMPORT:
         case REGSHELL_CMD_UPGRADE:
+        case REGSHELL_CMD_CLEANUP:
         case REGSHELL_CMD_SET_HIVE:
             if (argc != 3)
             {
@@ -1309,31 +1420,47 @@ cleanup:
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pCmdItem);
     goto cleanup;
 }
 
-
 DWORD
 RegShellAllocKey(
+    PREGSHELL_PARSE_STATE pParseState,
     PSTR pszKeyName,
     PSTR *pszNewKey)
 {
-    DWORD dwError;
+    DWORD dwError = 0;
+    DWORD dwLen = 0;
+    PSTR pszRetKey = NULL;
 
-    dwError = RegAllocateMemory(strlen(pszKeyName) + 3, (PVOID*)pszNewKey);
+    dwLen = strlen(pszKeyName) + 3;  // 3: [ ] and \0
+    dwError = RegAllocateMemory(dwLen, (PVOID) &pszRetKey);
     BAIL_ON_REG_ERROR(dwError);
 
-    strcpy(*pszNewKey, "[");
-    strcat(*pszNewKey, pszKeyName);
-    strcat(*pszNewKey, "]");
+    pszRetKey[0] = '\0';
+    if (!pParseState->bBracketPrefix)
+    {
+        strncat(pszRetKey, "[", dwLen--);
+        strncat(pszRetKey, pszKeyName, dwLen);
+        dwLen -= strlen(pszKeyName);
+        strncat(pszRetKey, "]", dwLen--);
+    }
+    else
+    {
+        strncat(pszRetKey, pszKeyName, dwLen);
+    }
 
 cleanup:
+    *pszNewKey = pszRetKey;
     return dwError;
 
 error:
+    LWREG_SAFE_FREE_MEMORY(pszRetKey);
     goto cleanup;
 }
 
+
 DWORD
 RegShellCmdlineParseToArgv(
     PREGSHELL_PARSE_STATE pParseState,
@@ -1358,7 +1485,7 @@ RegShellCmdlineParseToArgv(
     PSTR pszKeyName = NULL;
     REGLEX_TOKEN token = 0;
     REGSHELL_CMDLINE_STATE_E state = 0;
-    REG_DATA_TYPE valueType = REG_UNKNOWN;
+    REG_DATA_TYPE valueType = REG_NONE;
     REGSHELL_CMD_E cmdEnum = 0;
     PREGSHELL_CMD_ITEM pCmdItem = NULL;
 
@@ -1513,7 +1640,9 @@ RegShellCmdlineParseToArgv(
                     dwArgc = 2;
                     state = REGSHELL_CMDLINE_STATE_IMPORT;
                 }
-                else if (cmdEnum == REGSHELL_CMD_IMPORT)
+                else if (cmdEnum == REGSHELL_CMD_IMPORT ||
+                         cmdEnum == REGSHELL_CMD_UPGRADE ||
+                         cmdEnum == REGSHELL_CMD_CLEANUP)
                 {
                     dwAllocSize = 4;
                     dwArgc = 2;
@@ -1525,12 +1654,6 @@ RegShellCmdlineParseToArgv(
                     dwArgc = 2;
                     state = REGSHELL_CMDLINE_STATE_ADDVALUE;
                 }
-                else if (cmdEnum == REGSHELL_CMD_UPGRADE)
-                {
-                    dwAllocSize = 4;
-                    dwArgc = 2;
-                    state = REGSHELL_CMDLINE_STATE_IMPORT;
-                }
                 else
                 {
                     dwError = LWREG_ERROR_INVALID_CONTEXT;
@@ -1580,7 +1703,7 @@ RegShellCmdlineParseToArgv(
                 RegLexGetAttribute(pParseState->lexHandle,
                                    &attrSize,
                                    &pszAttr);
-                RegShellAllocKey(pszAttr, &pszArgv[dwArgc-1]);
+                RegShellAllocKey(pParseState, pszAttr, &pszArgv[dwArgc-1]);
                 BAIL_ON_REG_ERROR(dwError);
                 state = REGSHELL_CMDLINE_STATE_LIST_KEYS_STOP;
                 break;
@@ -1612,6 +1735,7 @@ RegShellCmdlineParseToArgv(
                 else
                 {
                     if (token == REGLEX_REG_KEY ||
+                        token == REGLEX_KEY_PREFIX ||
                         token == REGLEX_PLAIN_TEXT)
                     {
                         state = REGSHELL_CMDLINE_STATE_CD_REGKEY;
@@ -1625,7 +1749,7 @@ RegShellCmdlineParseToArgv(
 
 
             case REGSHELL_CMDLINE_STATE_CD_REGKEY:
-                dwError = RegShellAllocKey(pszAttr, &pszArgv[2]);
+                dwError = RegShellAllocKey(pParseState, pszAttr, &pszArgv[2]);
                 BAIL_ON_REG_ERROR(dwError);
                 dwArgc += 1;
 
@@ -1639,7 +1763,7 @@ RegShellCmdlineParseToArgv(
                                          &eof);
                 BAIL_ON_REG_ERROR(dwError);
                 stop = eof;
-                if (!eof)
+                if (!eof && !pParseState->bBracketPrefix)
                 {
                     dwError = LWREG_ERROR_INVALID_CONTEXT;
                 }
@@ -1671,7 +1795,7 @@ RegShellCmdlineParseToArgv(
                 break;
 
             case REGSHELL_CMDLINE_STATE_ADDKEY_REGKEY:
-                dwError = RegShellAllocKey(pszAttr, &pszArgv[2]);
+                dwError = RegShellAllocKey(pParseState, pszAttr, &pszArgv[2]);
                 BAIL_ON_REG_ERROR(dwError);
                 state = REGSHELL_CMDLINE_STATE_ADDKEY_STOP;
                 break;
@@ -1729,12 +1853,72 @@ RegShellCmdlineParseToArgv(
                 {
                     state = REGSHELL_CMDLINE_STATE_ADDVALUE_VALUENAME;
                 }
+                else if (token == REGLEX_DASH)
+                {
+                    dwError = RegLexGetToken(pParseState->ioHandle,
+                                             pParseState->lexHandle,
+                                             &token,
+                                             &eof);
+                    BAIL_ON_REG_ERROR(dwError);
+                    if (token != REGLEX_DASH)
+                    {
+                        state = REGSHELL_CMDLINE_STATE_ADDVALUE_VALUENAME;
+                        RegLexUnGetToken(pParseState->lexHandle);
+                    }
+                    else if (eof)
+                    {
+                        state = REGSHELL_CMDLINE_STATE_ADDVALUE_VALUENAME;
+                    }
+                    else
+                    {
+                        state = REGSHELL_CMDLINE_STATE_ADDVALUE_OPTION;
+                    }
+                }
                 else
                 {
                     dwError = LWREG_ERROR_INVALID_CONTEXT;
                 }
                 break;
 
+            case REGSHELL_CMDLINE_STATE_ADDVALUE_OPTION:
+                /* Handle export --legacy option */
+                dwError = RegLexGetToken(pParseState->ioHandle,
+                                         pParseState->lexHandle,
+                                         &token,
+                                         &eof);
+                if (dwError || eof || token != REGLEX_PLAIN_TEXT)
+                {
+                    dwError = LWREG_ERROR_INVALID_CONTEXT;
+                    BAIL_ON_REG_ERROR(dwError);
+                }
+
+                RegLexGetAttribute(pParseState->lexHandle,
+                                   &attrSize,
+                                   &pszAttr);
+                if (cmdEnum == REGSHELL_CMD_EXPORT && 
+                    !strcmp("legacy", pszAttr))
+                {
+                    dwError = RegCStringDuplicate(
+                                  &pszArgv[dwArgc++],
+                                  "--legacy");
+                    BAIL_ON_REG_ERROR(dwError);
+                }
+                else if (cmdEnum == REGSHELL_CMD_EXPORT && 
+                    !strcmp("values", pszAttr))
+                {
+                    dwError = RegCStringDuplicate(
+                                  &pszArgv[dwArgc++],
+                                  "--values");
+                    BAIL_ON_REG_ERROR(dwError);
+                }
+                else
+                {
+                    dwError = LWREG_ERROR_INVALID_CONTEXT;
+                    BAIL_ON_REG_ERROR(dwError);
+                }
+                state = REGSHELL_CMDLINE_STATE_ADDVALUE;
+                break;
+                         
                 case REGSHELL_CMDLINE_STATE_ADDVALUE_KEYNAME:
                     if (pszPrevAttr)
                     {
@@ -1755,7 +1939,7 @@ RegShellCmdlineParseToArgv(
                     BAIL_ON_REG_ERROR(dwError);
                     if (pCmdItem->keyName)
                     {
-                        dwError = RegShellAllocKey(pCmdItem->keyName, &pszArgv[dwArgc++]);
+                        dwError = RegShellAllocKey(pParseState, pCmdItem->keyName, &pszArgv[dwArgc++]);
                         BAIL_ON_REG_ERROR(dwError);
                     }
 
@@ -1770,10 +1954,15 @@ RegShellCmdlineParseToArgv(
                     }
                     else if (token == REGLEX_REG_SZ ||
                              token == REGLEX_PLAIN_TEXT ||
-                         token == REGLEX_KEY_NAME_DEFAULT)
+                             token == REGLEX_KEY_NAME_DEFAULT)
                     {
                         state = REGSHELL_CMDLINE_STATE_ADDVALUE_VALUENAME;
                     }
+                    else if (token == REGLEX_DASH)
+                    {
+                        state = REGSHELL_CMDLINE_STATE_ADDVALUE_VALUE;
+                        RegLexUnGetToken(pParseState->lexHandle);
+                    }
                     else
                     {
                         dwError = LWREG_ERROR_INVALID_CONTEXT;
@@ -1852,7 +2041,7 @@ RegShellCmdlineParseToArgv(
                                   pszAttr,
                                   &valueType,
                                   NULL);
-                    if (valueType == REG_UNKNOWN)
+                    if (valueType == REG_NONE)
                     {
                         dwError = LWREG_ERROR_INVALID_CONTEXT;
                         BAIL_ON_REG_ERROR(dwError);
@@ -1869,7 +2058,7 @@ RegShellCmdlineParseToArgv(
                      * the next call to RegLexGetToken(), as it consumes
                      * the first hex pair if found.
                      */
-                    if (valueType == REG_BINARY || REG_DWORD)
+                    if (valueType == REG_BINARY || valueType == REG_DWORD)
                     {
                         dwError = RegIOBufferGetData(
                                       pParseState->ioHandle,
@@ -1917,7 +2106,8 @@ RegShellCmdlineParseToArgv(
                         RegLexResetToken(pParseState->lexHandle);
                     }
                     else if (token == REGLEX_REG_SZ ||
-                             token == REGLEX_PLAIN_TEXT)
+                             token == REGLEX_PLAIN_TEXT ||
+                             token == REGLEX_DASH)
                     {
                         RegLexGetAttribute(pParseState->lexHandle,
                                            &attrSize,
@@ -1930,7 +2120,7 @@ RegShellCmdlineParseToArgv(
                             dwError = RegReallocMemory(
                                           pszArgv,
                                           (LW_PVOID) &pszArgvRealloc,
-                                          dwAllocSize * sizeof(PSTR *));
+                                          dwAllocSize * sizeof(*pszArgv));
                             BAIL_ON_REG_ERROR(dwError);
                             pszArgv = pszArgvRealloc;
                             pszArgvRealloc = NULL;
@@ -2078,8 +2268,9 @@ RegShellUsage(
         "       delete_value [[KeyName]] \"ValueName\"\n"
         "       set_hive HIVE_NAME\n"
         "       import file.reg | -\n"
-        "       export [[keyName]] file.reg | - \n"
+        "       export [--legacy | --values] [[keyName]] [file.reg | -]\n"
         "       upgrade file.reg | -\n"
+        "       cleanup file.reg | -\n"
         "       exit | quit | ^D\n"
         "       history\n"
         "\n"
