@@ -105,7 +105,7 @@ RegUtilDeleteKey(
                pszSubKeyPath,
                keyName);
 }
-
+     
 
 DWORD
 RegUtilDeleteTree(
@@ -183,6 +183,25 @@ RegUtilGetValues(
 }
 
 
+DWORD
+RegUtilGetKeyObjectCounts(
+    IN OPTIONAL HANDLE hReg,
+    IN PSTR pszRootKeyName,
+    IN PSTR pszSubKeyPath,
+    IN PSTR keyName,
+    OUT OPTIONAL PDWORD pdwSubKeysCount,
+    OUT OPTIONAL PDWORD pdwValuesCount)
+{
+    return RegShellUtilGetKeyObjectCounts(
+               hReg,
+               pszRootKeyName,
+               pszSubKeyPath,
+               keyName,
+               pdwSubKeysCount,
+               pdwValuesCount);
+}
+
+
 DWORD
 RegUtilDeleteValue(
     IN OPTIONAL HANDLE hReg,
