@@ -212,7 +212,6 @@ RegShellIsValidKey(
     PWSTR pSubKey = NULL;
 
     BAIL_ON_INVALID_HANDLE(hReg);
-    BAIL_ON_INVALID_POINTER(pszKey);
 
     if (!pszRootKeyName)
     {
@@ -221,20 +220,23 @@ RegShellIsValidKey(
     dwError = RegOpenKeyExA(hReg, NULL, pszRootKeyName, 0, KEY_READ, &pRootKey);
     BAIL_ON_REG_ERROR(dwError);
 
-	dwError = RegWC16StringAllocateFromCString(&pSubKey, pszKey);
-	BAIL_ON_REG_ERROR(dwError);
+    if (pszKey)
+    {
+        dwError = RegWC16StringAllocateFromCString(&pSubKey, pszKey);
+        BAIL_ON_REG_ERROR(dwError);
 
-    dwError = RegOpenKeyExW(
-        hReg,
-        pRootKey,
-        pSubKey,
-        0,
-        KEY_READ,
-        &pFullKey);
-    BAIL_ON_REG_ERROR(dwError);
+        dwError = RegOpenKeyExW(
+            hReg,
+            pRootKey,
+            pSubKey,
+            0,
+            KEY_READ,
+            &pFullKey);
+        BAIL_ON_REG_ERROR(dwError);
+    }
 
 cleanup:
-
+    
     LWREG_SAFE_FREE_MEMORY(pSubKey);
     if (pFullKey)
     {
@@ -329,7 +331,7 @@ RegShellUtilAddKeySecDesc(
         pszToken = strtok_r (NULL, pszDelim, &pszStrtokState);
 
         if (LW_IS_NULL_OR_EMPTY_STR(pszToken) &&
-		REG_OPENED_EXISTING_KEY == dwDisposition && bDoBail)
+        	REG_OPENED_EXISTING_KEY == dwDisposition && bDoBail)
         {
             dwError = LWREG_ERROR_KEYNAME_EXIST;
             BAIL_ON_REG_ERROR(dwError);
@@ -337,7 +339,6 @@ RegShellUtilAddKeySecDesc(
     }
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pCurrentKey)
     {
         RegCloseKey(hReg, pCurrentKey);
@@ -346,6 +347,7 @@ cleanup:
     {
         RegCloseKey(hReg, pNextKey);
     }
+    RegCloseServer(hRegLocal);
     LWREG_SAFE_FREE_STRING(pszFullPath);
     LWREG_SAFE_FREE_STRING(pszSubKey);
     LWREG_SAFE_FREE_MEMORY(pwszSubKey);
@@ -435,7 +437,7 @@ RegShellUtilDeleteKey(
         BAIL_ON_REG_ERROR(dwError);
         LWREG_SAFE_FREE_MEMORY(pwszSubKey);
     }
-    else
+    else 
     {
         pCurrentKey = pRootKey;
         pRootKey = NULL;
@@ -448,7 +450,6 @@ RegShellUtilDeleteKey(
     BAIL_ON_REG_ERROR(dwError);
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pCurrentKey)
     {
         RegCloseKey(hReg, pCurrentKey);
@@ -457,12 +458,13 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    RegCloseServer(hRegLocal);
     LWREG_SAFE_FREE_MEMORY(pwszSubKey);
-
+                                       
     LWREG_SAFE_FREE_STRING(pszFullPath);
     LWREG_SAFE_FREE_STRING(pszParentPath);
     LWREG_SAFE_FREE_STRING(pszSubKey);
-
+                                       
     return dwError;
 
 error:
@@ -513,8 +515,8 @@ RegShellUtilDeleteTree(
 
     if (pszParentPath && pszParentPath[1])
     {
-	dwError = RegWC16StringAllocateFromCString(&pwszSubKey, pszParentPath+1);
-	BAIL_ON_REG_ERROR(dwError);
+    	dwError = RegWC16StringAllocateFromCString(&pwszSubKey, pszParentPath+1);
+    	BAIL_ON_REG_ERROR(dwError);
 
         dwError = RegOpenKeyExW(
                       hReg,
@@ -539,7 +541,6 @@ RegShellUtilDeleteTree(
     BAIL_ON_REG_ERROR(dwError);
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pCurrentKey)
     {
         RegCloseKey(hReg, pCurrentKey);
@@ -548,6 +549,7 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    RegCloseServer(hRegLocal);
     LWREG_SAFE_FREE_MEMORY(pwszSubKey);
     LWREG_SAFE_FREE_STRING(pszFullPath);
     LWREG_SAFE_FREE_STRING(pszParentPath);
@@ -610,8 +612,10 @@ RegShellUtilGetKeys(
 
     if (pszParentPath && pszParentPath[1])
     {
-	dwError = RegWC16StringAllocateFromCString(&pwszSubKey, pszParentPath+1);
-	BAIL_ON_REG_ERROR(dwError);
+    	dwError = RegWC16StringAllocateFromCString(
+                      &pwszSubKey,
+                      pszParentPath+1);
+    	BAIL_ON_REG_ERROR(dwError);
 
         dwError = RegOpenKeyExW(
                       hReg,
@@ -636,7 +640,9 @@ RegShellUtilGetKeys(
         pFullKey = pRootKey;
         pRootKey = NULL;
 
-        dwError = RegWC16StringAllocateFromCString(&pwszParentKeyName, pszRootKeyName);
+        dwError = RegWC16StringAllocateFromCString(
+                      &pwszParentKeyName,
+                      pszRootKeyName);
         BAIL_ON_REG_ERROR(dwError);
     }
 
@@ -658,10 +664,12 @@ RegShellUtilGetKeys(
 
     if (!dwSubKeyCount)
     {
-	goto done;
+    	goto done;
     }
 
-    dwError = RegAllocateMemory(sizeof(*subKeys) * dwSubKeyCount, (PVOID*)&subKeys);
+    dwError = RegAllocateMemory(
+                  sizeof(*subKeys) * dwSubKeyCount,
+                  (PVOID*)&subKeys);
     BAIL_ON_REG_ERROR(dwError);
 
 #ifdef _LW_DEBUG
@@ -672,7 +680,9 @@ RegShellUtilGetKeys(
     {
         dwSubKeyLen = dwMaxSubKeyLen+1;
 
-        dwError = RegAllocateMemory(sizeof(*pszKeyName) * dwSubKeyLen, (PVOID*)&pszKeyName);
+        dwError = RegAllocateMemory(
+                      sizeof(*pszKeyName) * dwSubKeyLen,
+                      (PVOID*)&pszKeyName);
         BAIL_ON_REG_ERROR(dwError);
 
         dwError = RegEnumKeyExA((HANDLE)hReg,
@@ -703,7 +713,6 @@ done:
     *pdwRetSubKeyCount = dwSubKeyCount;
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pFullKey)
     {
         RegCloseKey(hReg, pFullKey);
@@ -712,6 +721,7 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    RegCloseServer(hRegLocal);
     LWREG_SAFE_FREE_STRING(pszParentPath);
     LWREG_SAFE_FREE_STRING(pszKeyName);
     LWREG_SAFE_FREE_MEMORY(pwszParentKeyName);
@@ -855,7 +865,6 @@ RegShellUtilSetValue(
     BAIL_ON_REG_ERROR(dwError);
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pFullKey)
     {
         RegCloseKey(hReg, pFullKey);
@@ -864,6 +873,7 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    RegCloseServer(hRegLocal);
     if (type != REG_BINARY)
     {
         LWREG_SAFE_FREE_MEMORY(pData);
@@ -891,7 +901,7 @@ RegShellUtilValueArrayFree(
         LWREG_SAFE_FREE_MEMORY(pValueArray[i].pValueName);
         LWREG_SAFE_FREE_MEMORY(pValueArray[i].pData);
     }
-
+    
     LWREG_SAFE_FREE_MEMORY(pValueArray);
 cleanup:
     return dwError;
@@ -915,6 +925,7 @@ RegShellUtilGetValues(
     DWORD dwError = 0;
     DWORD indx = 0;
     PSTR pszValueName = NULL;
+    PWSTR pwszValueName = NULL;
     DWORD dwValueNameLen = 0;
     DWORD regType = 0;
     PBYTE pData = NULL;
@@ -928,6 +939,7 @@ RegShellUtilGetValues(
     PREGSHELL_UTIL_VALUE pValueArray = NULL;
     DWORD dwMaxValueNameLen = 0;
     DWORD dwMaxValueLen = 0;
+    PLWREG_CURRENT_VALUEINFO pCurrentValue = NULL;
 
     if (!hReg)
     {
@@ -953,8 +965,10 @@ RegShellUtilGetValues(
 
     if (pszParentPath && strcmp(pszParentPath, "\\") != 0)
     {
-	dwError = RegWC16StringAllocateFromCString(&pwszParentPath, pszParentPath+1);
-	BAIL_ON_REG_ERROR(dwError);
+    	dwError = RegWC16StringAllocateFromCString(
+                      &pwszParentPath,
+                      pszParentPath+1);
+    	BAIL_ON_REG_ERROR(dwError);
 
         dwError = RegOpenKeyExW(
                       hReg,
@@ -990,10 +1004,12 @@ RegShellUtilGetValues(
 
     if (!dwValuesCount)
     {
-	goto done;
+    	goto done;
     }
 
-    dwError = RegAllocateMemory(sizeof(*pValueArray) * dwValuesCount, (PVOID*)&pValueArray);
+    dwError = RegAllocateMemory(
+                  sizeof(*pValueArray) * dwValuesCount,
+                  (PVOID*)&pValueArray);
     BAIL_ON_REG_ERROR(dwError);
 
     /*
@@ -1010,7 +1026,9 @@ RegShellUtilGetValues(
          */
         dwValueNameLen = dwMaxValueNameLen;
 
-        dwError = RegAllocateMemory(sizeof(*pszValueName) * dwValueNameLen, (PVOID*)&pszValueName);
+        dwError = RegAllocateMemory(
+                      sizeof(*pszValueName) * dwValueNameLen,
+                      (PVOID*)&pszValueName);
         BAIL_ON_REG_ERROR(dwError);
 
         dwDataLen = dwMaxValueLen;
@@ -1033,8 +1051,31 @@ RegShellUtilGetValues(
                       &dwDataLen);
         BAIL_ON_REG_ERROR(dwError);
 
-	dwError = RegWC16StringAllocateFromCString(&pValueArray[indx].pValueName, pszValueName);
-	BAIL_ON_REG_ERROR(dwError);
+    	dwError = RegWC16StringAllocateFromCString(
+                      &pwszValueName,
+                      pszValueName);
+        BAIL_ON_REG_ERROR(dwError);
+
+        dwError = LwRegGetValueAttributesW(
+                      hReg,
+                      pFullKey,
+                      NULL,
+                      pwszValueName,
+                      &pCurrentValue,
+                      NULL);
+        /*
+         * Don't care if this fails.
+         * Manually added new values don't have attributes to back them.
+         */
+        if (pCurrentValue)
+        {
+            pValueArray[indx].bValueSet = TRUE;
+        }
+
+    	dwError = RegWC16StringAllocateFromCString(
+                      &pValueArray[indx].pValueName,
+                      pszValueName);
+    	BAIL_ON_REG_ERROR(dwError);
         LWREG_SAFE_FREE_STRING(pszValueName);
 
         pValueArray[indx].type = regType;
@@ -1044,12 +1085,10 @@ RegShellUtilGetValues(
     }
 
  done:
-
     *valueArray = pValueArray;
     *pdwValueArrayLen = indx;
 
 cleanup:
-    RegCloseServer(hRegLocal);
     if (pFullKey)
     {
         RegCloseKey(hReg, pFullKey);
@@ -1058,10 +1097,14 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    RegCloseServer(hRegLocal);
     LWREG_SAFE_FREE_STRING(pszValueName);
     LWREG_SAFE_FREE_STRING(pszParentPath);
     LWREG_SAFE_FREE_MEMORY(pSubKey);
+    LWREG_SAFE_FREE_MEMORY(pwszValueName);
     LWREG_SAFE_FREE_MEMORY(pData);
+    RegSafeFreeCurrentValueInfo(&pCurrentValue);
+
     return dwError;
 
 error:
@@ -1076,6 +1119,112 @@ error:
 }
 
 
+DWORD
+RegShellUtilGetKeyObjectCounts(
+    HANDLE hReg,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
+    PDWORD pdwSubKeysCount,
+    PDWORD pdwValuesCount
+    )
+{
+    HANDLE hRegLocal = NULL;
+    DWORD dwError = 0;
+    HKEY pRootKey = NULL;
+    HKEY pFullKey = NULL;
+    PSTR pszParentPath = NULL;
+    PWSTR pwszParentPath = NULL;
+    DWORD dwSubKeysCount = 0;
+    DWORD dwValuesCount = 0;
+
+    if (!hReg)
+    {
+        dwError = RegOpenServer(&hRegLocal);
+        BAIL_ON_REG_ERROR(dwError);
+        hReg = hRegLocal;
+    }
+
+
+    if (!pszRootKeyName)
+    {
+        return 0;
+    }
+    dwError = RegShellCanonicalizePath(pszDefaultKey,
+                                       keyName,
+                                       &pszParentPath,
+                                       NULL,
+                                       NULL);
+    BAIL_ON_REG_ERROR(dwError);
+
+    dwError = RegOpenKeyExA(hReg, NULL, pszRootKeyName, 0, KEY_READ, &pRootKey);
+    BAIL_ON_REG_ERROR(dwError);
+
+    if (pszParentPath && strcmp(pszParentPath, "\\") != 0)
+    {
+    	dwError = RegWC16StringAllocateFromCString(&pwszParentPath, pszParentPath+1);
+    	BAIL_ON_REG_ERROR(dwError);
+
+        dwError = RegOpenKeyExW(
+                      hReg,
+                      pRootKey,
+                      pwszParentPath,
+                      0,
+                      KEY_READ,
+                      &pFullKey);
+        BAIL_ON_REG_ERROR(dwError);
+        LWREG_SAFE_FREE_MEMORY(pwszParentPath);
+    }
+    else
+    {
+        pFullKey = pRootKey;
+        pRootKey = NULL;
+    }
+
+    dwError = RegQueryInfoKeyA(
+                  hReg,
+                  pFullKey,
+                  NULL,
+                  NULL,
+                  NULL,
+                  &dwSubKeysCount,
+                  NULL,
+                  NULL,
+                  &dwValuesCount,
+                  NULL,
+                  NULL,
+                  NULL,
+                  NULL);
+    BAIL_ON_REG_ERROR(dwError);
+
+    if (pdwSubKeysCount)
+    {
+        *pdwSubKeysCount = dwSubKeysCount;
+    }
+    if (pdwValuesCount)
+    {
+        *pdwValuesCount = dwValuesCount;
+    }
+
+cleanup:
+    if (pFullKey)
+    {
+        RegCloseKey(hReg, pFullKey);
+    }
+    if (pRootKey)
+    {
+        RegCloseKey(hReg, pRootKey);
+    }
+    RegCloseServer(hRegLocal);
+    LWREG_SAFE_FREE_STRING(pszParentPath);
+    return dwError;
+
+error:
+    LWREG_SAFE_FREE_MEMORY(pwszParentPath);
+    goto cleanup;
+}
+
+
 DWORD
 RegShellUtilDeleteValue(
     HANDLE hReg,
@@ -1418,13 +1567,12 @@ RegShellUtilGetValue(
                 ;
             *pdwValueLen = dwIndex;
             *pppszMultiValue = ppszMultiStrArray;
-
+            
             break;
     }
 
 cleanup:
     LWREG_SAFE_FREE_MEMORY(pRetData);
-    RegCloseServer(hRegLocal);
     if (hFullKeyName)
     {
         RegCloseKey(hReg, hFullKeyName);
@@ -1437,6 +1585,7 @@ cleanup:
     {
         RegCloseKey(hReg, hRootKey);
     }
+    RegCloseServer(hRegLocal);
     return dwError;
 error:
     goto cleanup;
@@ -1462,9 +1611,9 @@ RegShellUtilEscapeString(
     /* Count number of \ found in string to escape */
     for (i=0; pszValue[i]; i++)
     {
-        if (pszValue[i] == '\\' || pszValue[i] == '\n' ||
-            pszValue[i] == '\r' || pszValue[i] == '"' ||
-            pszValue[i] == '\t' || pszValue[i] == '\a' ||
+        if (pszValue[i] == '\\' || pszValue[i] == '\n' || 
+            pszValue[i] == '\r' || pszValue[i] == '"' || 
+            pszValue[i] == '\t' || pszValue[i] == '\a' || 
             pszValue[i] == '\v' || pszValue[i] == '\f')
         {
             dwEscapeValueLen++;
@@ -1533,3 +1682,725 @@ cleanup:
 error:
     goto cleanup;
 }
+
+DWORD
+RegExportBinaryTypeToString(
+    REG_DATA_TYPE token,
+    PSTR tokenStr,
+    BOOLEAN dumpFormat
+    )
+{
+    DWORD dwError = 0;
+    static char *typeStrs[][2] = {
+        { "hex(0):", "REG_NONE" },
+        { "REG_SZ", "REG_SZ"},
+        { "hex(2):", "REG_EXPAND_SZ" },
+        { "hex:", "REG_BINARY" },
+        { "dword:", "REG_DWORD" },
+        { "dwordbe:", "REG_DWORD_BIG_ENDIAN" },
+        { "link:", "REG_LINK" },
+        { "hex(7):", "REG_MULTI_SZ" },
+        { "hex(8):", "REG_RESOURCE_LIST" },
+        { "hex(9):", "REG_FULL_RESOURCE_DESCRIPTOR" },
+        { "hex(a):", "REG_RESOURCE_REQUIREMENTS_LIST" },
+        { "hex(b):", "REG_QUADWORD" },
+        { "unknown12:", "REG_UNKNOWN12" },
+        { "unknown13:", "REG_UNKNOWN13" },
+        { "unknown14:", "REG_UNKNOWN14" },
+        { "unknown15:", "REG_UNKNOWN15" },
+        { "unknown16:", "REG_UNKNOWN16" },
+        { "unknown17:", "REG_UNKNOWN17" },
+        { "unknown18:", "REG_UNKNOWN18" },
+        { "unknown19:", "REG_UNKNOWN19" },
+        { "unknown20:", "REG_UNKNOWN20" },
+        { "REG_KEY", "REG_KEY" },
+        { "REG_KEY_DEFAULT", "REG_KEY_DEFAULT" },
+        { "REG_PLAIN_TEXT", "REG_PLAIN_TEXT" },
+        { "REG_UNKNOWN", "REG_UNKNOWN" },
+        { "sza:", "REG_STRING_ARRAY" }, /* Maps to REG_MULTI_SZ */
+        { "REG_ATTRIBUTES", "REG_ATTRIBUTES" },
+    };
+
+    BAIL_ON_INVALID_POINTER(tokenStr);
+
+    if (token < ((sizeof(typeStrs)/sizeof(char *))/2))
+    {
+        if (dumpFormat)
+        {
+            strcpy(tokenStr, typeStrs[token][0]);
+        }
+        else
+        {
+            strcpy(tokenStr, typeStrs[token][1]);
+        }
+    }
+    else
+    {
+        sprintf(tokenStr, "ERROR: No Such Token %d", token);
+    }
+
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+DWORD
+RegExportAttributeEntries(
+    PREG_PARSE_ITEM pItem,
+    PSTR *ppszDumpString,
+    PDWORD pdwDumpStringLen
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszDumpString = NULL;
+    PSTR pszString = NULL;
+    DWORD dwAttrStringLen = 0;
+    PSTR pszIndentChar = " ";
+    PWSTR *ppwszRangeEnumStrings = NULL;
+    DWORD dwIndentLevel = 4;
+
+    BAIL_ON_INVALID_POINTER(pItem);
+    BAIL_ON_INVALID_POINTER(ppszDumpString);
+    BAIL_ON_INVALID_POINTER(pdwDumpStringLen);
+
+    dwError = RtlCStringAllocateAppendPrintf(
+                  &pszDumpString, "\"%s\" = {\n",
+                  pItem->valueName);
+    BAIL_ON_REG_ERROR(dwError);
+
+    if (pItem->value)
+    {
+        dwError = RegExportEntry(
+                      NULL,
+                      NULL,
+                      REG_SZ,    /* Type of value name "value" */
+                      "value",
+                      pItem->regAttr.ValueType,
+                      pItem->value,
+                      pItem->valueLen,
+                      &pszString,
+                      &dwAttrStringLen);
+        BAIL_ON_REG_ERROR(dwError);
+
+        if (pszString && dwAttrStringLen)
+        {
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*s%s\n",
+                          dwIndentLevel,
+                          pszIndentChar,
+                          pszString);
+            BAIL_ON_REG_ERROR(dwError);
+            LWREG_SAFE_FREE_STRING(pszString);
+        }
+    }
+
+    if (pItem->regAttr.pDefaultValue)
+    {
+        dwError = RegExportEntry(
+                      NULL,
+                      NULL,
+                      REG_SZ,    /* Type of value name "value" */
+                      "default",
+                      pItem->regAttr.ValueType,
+                      pItem->regAttr.pDefaultValue,
+                      pItem->regAttr.DefaultValueLen,
+                      &pszString,
+                      &dwAttrStringLen);
+        BAIL_ON_REG_ERROR(dwError);
+
+        dwError = RtlCStringAllocateAppendPrintf(
+                      &pszDumpString, "%*s%s\n",
+                      dwIndentLevel,
+                      pszIndentChar,
+                      pszString);
+        BAIL_ON_REG_ERROR(dwError);
+        LWREG_SAFE_FREE_STRING(pszString);
+    }
+
+    if (pItem->regAttr.pwszDocString)
+    {
+        dwError = RegCStringAllocateFromWC16String(
+                      &pszString,
+                      pItem->regAttr.pwszDocString);
+        BAIL_ON_REG_ERROR(dwError);
+
+        dwError = RtlCStringAllocateAppendPrintf(
+                      &pszDumpString, "%*sdoc=\"%s\"\n",
+                      dwIndentLevel,
+                      pszIndentChar,
+                      pszString);
+        BAIL_ON_REG_ERROR(dwError);
+        LWREG_SAFE_FREE_STRING(pszString);
+    }
+
+    switch (pItem->regAttr.RangeType)
+    {
+        case LWREG_VALUE_RANGE_TYPE_BOOLEAN:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*srange=boolean\n",
+                          dwIndentLevel,
+                          pszIndentChar);
+            BAIL_ON_REG_ERROR(dwError);
+
+            break;
+        case LWREG_VALUE_RANGE_TYPE_ENUM:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*srange=string:",
+                          dwIndentLevel,
+                          pszIndentChar);
+            BAIL_ON_REG_ERROR(dwError);
+            for (ppwszRangeEnumStrings =
+                     pItem->regAttr.Range.ppwszRangeEnumStrings;
+                 *ppwszRangeEnumStrings;
+                 ppwszRangeEnumStrings++)
+            {
+                dwError = RegCStringAllocateFromWC16String(
+                              &pszString,
+                              *ppwszRangeEnumStrings);
+                BAIL_ON_REG_ERROR(dwError);
+
+                dwError = RtlCStringAllocateAppendPrintf(
+                              &pszDumpString, "\"%s\"",
+                              pszString);
+                BAIL_ON_REG_ERROR(dwError);
+                LWREG_SAFE_FREE_STRING(pszString);
+                if (ppwszRangeEnumStrings[1])
+                {
+                    dwError = RtlCStringAllocateAppendPrintf(
+                                  &pszDumpString, " ");
+                    BAIL_ON_REG_ERROR(dwError);
+                }
+            }
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "\n");
+            BAIL_ON_REG_ERROR(dwError);
+            break;
+        case LWREG_VALUE_RANGE_TYPE_INTEGER:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*srange=integer:%d-%d\n",
+                          dwIndentLevel,
+                          pszIndentChar,
+                          pItem->regAttr.Range.RangeInteger.Min,
+                          pItem->regAttr.Range.RangeInteger.Max);
+            BAIL_ON_REG_ERROR(dwError);
+            break;
+        default:
+            if (pItem->regAttr.RangeType > 0)
+            {
+                dwError = RtlCStringAllocateAppendPrintf(
+                              &pszDumpString, "%*srange=invalid value (%d)\n",
+                              dwIndentLevel,
+                              pszIndentChar,
+                              pItem->regAttr.RangeType);
+                BAIL_ON_REG_ERROR(dwError);
+            }
+            break;
+    }
+
+    switch (pItem->regAttr.Hint)
+    {
+        case LWREG_VALUE_HINT_SECONDS:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*shint=seconds\n",
+                          dwIndentLevel,
+                          pszIndentChar);
+            BAIL_ON_REG_ERROR(dwError);
+            break;
+
+        case LWREG_VALUE_HINT_PATH:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*shint=path\n",
+                          dwIndentLevel,
+                          pszIndentChar);
+            BAIL_ON_REG_ERROR(dwError);
+            break;
+
+        case LWREG_VALUE_HINT_ACCOUNT:
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString, "%*shint=account\n",
+                          dwIndentLevel,
+                          pszIndentChar);
+            BAIL_ON_REG_ERROR(dwError);
+            break;
+
+        default:
+            if (pItem->regAttr.Hint > 0)
+            {
+
+                dwError = RtlCStringAllocateAppendPrintf(
+                              &pszDumpString, "%*shint=invalid hint (%d)\n",
+                              dwIndentLevel,
+                              pszIndentChar,
+                              pItem->regAttr.Hint);
+                BAIL_ON_REG_ERROR(dwError);
+            }
+    }
+
+
+    dwError = RtlCStringAllocateAppendPrintf(
+                  &pszDumpString, "}");
+cleanup:
+    *ppszDumpString = pszDumpString;
+    *pdwDumpStringLen = strlen(pszDumpString);
+    return dwError;
+
+error:
+    LWREG_SAFE_FREE_STRING(pszString);
+    LWREG_SAFE_FREE_STRING(pszDumpString);
+    goto cleanup;
+}
+
+
+DWORD
+RegExportAttributes(
+    PREG_PARSE_ITEM pItem,
+    PSTR *ppszDumpString,
+    PDWORD pdwDumpStringLen
+    )
+{
+    DWORD dwError = 0;
+    BAIL_ON_INVALID_POINTER(pItem);
+    BAIL_ON_INVALID_POINTER(ppszDumpString);
+    BAIL_ON_INVALID_POINTER(pdwDumpStringLen);
+
+
+    if (pItem->type == REG_ATTRIBUTES)
+    {
+        dwError = RegExportAttributeEntries(
+                      pItem,
+                      ppszDumpString,
+                      pdwDumpStringLen);
+        BAIL_ON_REG_ERROR(dwError);
+    }
+    else
+    {
+        dwError = RegExportEntry(
+                      pItem->keyName,
+                      NULL,
+                      pItem->valueType,
+                      pItem->valueName,
+                      pItem->type,
+                      pItem->value,
+                      pItem->valueLen,
+                      ppszDumpString,
+                      pdwDumpStringLen);
+        BAIL_ON_REG_ERROR(dwError);
+    }
+
+cleanup:
+    return dwError;
+error:
+    goto cleanup;
+}
+
+
+/* Dump multi_sz in sza:"plain text" " format" */
+DWORD
+RegExportMultiStringArray(
+    PCSTR pszValueName,
+    PVOID pValue,
+    DWORD dwValueLen,
+    PSTR *ppszDumpString,
+    PDWORD pdwDumpStringLen
+    )
+{
+    DWORD dwError = 0;
+    DWORD dwPwszLen= 0;
+    PWSTR pwszValue = NULL;
+    PSTR pszString = NULL;
+    PSTR pszDumpString = NULL;
+    PSTR pszEscapedValue = NULL;
+    DWORD dwEscapedValueLen = 0;
+
+    BAIL_ON_INVALID_POINTER(pszValueName);
+    BAIL_ON_INVALID_POINTER(pValue);
+
+    pwszValue = (PWSTR) pValue;
+    dwError = RegCStringAllocatePrintf(
+                  &pszDumpString,
+                  "\"%s\"=sza:",
+                  pszValueName);
+    BAIL_ON_REG_ERROR(dwError);
+
+    while (*pwszValue)
+    {
+        dwError = RegCStringAllocateFromWC16String(
+                      &pszString,
+                      pwszValue);
+        BAIL_ON_REG_ERROR(dwError);
+
+        LWREG_SAFE_FREE_STRING(pszEscapedValue);
+        dwError = RegShellUtilEscapeString(
+                      pszString,
+                      &pszEscapedValue,
+                      &dwEscapedValueLen);
+        BAIL_ON_REG_ERROR(dwError);
+
+
+        dwError = RtlCStringAllocateAppendPrintf(
+                      &pszDumpString,
+                      "\"%s\"",
+                      pszEscapedValue);
+        BAIL_ON_REG_ERROR(dwError);
+        LWREG_SAFE_FREE_STRING(pszString);
+
+        dwPwszLen = wc16slen(pwszValue);
+        pwszValue += dwPwszLen + 1;
+        if (*pwszValue)
+        {
+            dwError = RtlCStringAllocateAppendPrintf(
+                          &pszDumpString,
+                          " \\\n    ");
+            BAIL_ON_REG_ERROR(dwError);
+        }
+    }
+
+cleanup:
+    LWREG_SAFE_FREE_STRING(pszEscapedValue);
+    *ppszDumpString = pszDumpString;
+    *pdwDumpStringLen = strlen(pszDumpString);
+    return dwError;
+error:
+    LWREG_SAFE_FREE_STRING(pszString);
+    LWREG_SAFE_FREE_STRING(pszDumpString);
+    goto cleanup;
+}
+
+
+
+DWORD
+RegExportEntry(
+    PCSTR keyName,
+    PCSTR pszSddlCString,
+    REG_DATA_TYPE valueType,
+    PCSTR valueName,
+    REG_DATA_TYPE type,
+    LW_PVOID value,
+    DWORD valueLen,
+    PSTR *dumpString,
+    PDWORD dumpStringLen
+    )
+{
+    DWORD dwError = 0;
+    switch (type)
+    {
+        case REG_BINARY:
+        case REG_NONE:
+        case REG_EXPAND_SZ:
+        case REG_RESOURCE_LIST:
+        case REG_FULL_RESOURCE_DESCRIPTOR:
+        case REG_RESOURCE_REQUIREMENTS_LIST:
+        case REG_QWORD:
+            dwError = RegExportBinaryData(valueType,
+                                          valueName,
+                                          type,
+                                          value,
+                                          valueLen,
+                                          dumpString,
+                                          dumpStringLen);
+            break;
+        case REG_MULTI_SZ:
+            dwError = RegExportMultiStringArray(
+                          valueName,
+                          value,
+                          valueLen,
+                          dumpString,
+                          dumpStringLen);
+            break;
+        case REG_DWORD:
+            dwError = RegExportDword(valueType,
+                                     valueName,
+                                     *((PDWORD) value),
+                                     dumpString,
+                                     dumpStringLen);
+            break;
+
+        case REG_KEY:
+            dwError = RegExportRegKey(keyName,
+                                      pszSddlCString,
+                                      dumpString,
+                                      dumpStringLen);
+            break;
+
+        case REG_SZ:
+            dwError = RegExportString(valueType,
+                                      valueName,
+                                      value,
+                                      dumpString,
+                                      dumpStringLen);
+            break;
+
+        case REG_PLAIN_TEXT:
+        default:
+            dwError = RegExportPlainText((PCHAR) value,
+                                        dumpString,
+                                        dumpStringLen);
+    }
+    return dwError;
+}
+
+
+DWORD
+RegExportDword(
+    REG_DATA_TYPE valueType,
+    PCSTR valueName,
+    DWORD value,
+    PSTR *dumpString,
+    PDWORD dumpStringLen
+    )
+{
+    DWORD bufLen = 0;
+    PSTR dumpBuf = NULL;
+    DWORD dwError = 0;
+
+    BAIL_ON_INVALID_POINTER(valueName);
+    BAIL_ON_INVALID_POINTER(dumpString);
+    BAIL_ON_INVALID_POINTER(dumpStringLen);
+
+    /*
+     *  "name"=1234ABCD\r\n\0
+     *  14: *  ""=1234ABCD\r\n\0
+     */
+    bufLen = strlen(valueName) + 20;
+
+    dwError = RegAllocateMemory(sizeof(*dumpBuf) * bufLen, (PVOID*)&dumpBuf);
+    BAIL_ON_REG_ERROR(dwError);
+
+    if (valueType == REG_KEY_DEFAULT)
+    {
+        *dumpStringLen = sprintf(dumpBuf, "@=dword:%08x",
+                                 value);
+    }
+    else
+    {
+        *dumpStringLen = sprintf(dumpBuf, "\"%s\"=dword:%08x",
+                                 valueName,
+                                 value);
+    }
+
+    *dumpString = dumpBuf;
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+
+}
+
+
+DWORD
+RegExportRegKey(
+    PCSTR keyName,
+    PCSTR pszSddlCString,
+    PSTR *dumpString,
+    PDWORD dumpStringLen
+    )
+{
+    DWORD bufLen = 0;
+    PSTR dumpBuf = NULL;
+    DWORD dwError = 0;
+    char szTmpSec[] = "\r\n@security = ";
+
+    BAIL_ON_INVALID_POINTER(keyName);
+    BAIL_ON_INVALID_POINTER(dumpString);
+    BAIL_ON_INVALID_POINTER(dumpStringLen);
+
+    /*
+     *  [key_name]\r\n\0
+     *  5:  []\r\n\0
+     */
+    bufLen = strlen(keyName) + 5 +(LwRtlCStringIsNullOrEmpty(pszSddlCString) ? 0 : (strlen(szTmpSec) + strlen(pszSddlCString)));
+
+
+    dwError = RegAllocateMemory(sizeof(*dumpBuf) * bufLen, (PVOID*)&dumpBuf);
+    BAIL_ON_REG_ERROR(dwError);
+
+    *dumpStringLen = LwRtlCStringIsNullOrEmpty(pszSddlCString)
+                   ? sprintf(dumpBuf, "[%s]", keyName)
+                   : sprintf(dumpBuf, "[%s]\r\n@security = %s", keyName, pszSddlCString);
+    *dumpString = dumpBuf;
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
+DWORD
+RegExportString(
+    REG_DATA_TYPE valueType,
+    PCSTR valueName,
+    PCSTR value,
+    PSTR *dumpString,
+    PDWORD retDumpStringLen
+    )
+{
+    DWORD bufLen = 0;
+    PSTR dumpBuf = NULL;
+    PSTR valueEscName = NULL;
+    DWORD dwError = 0;
+    DWORD dumpStringLen = 0;
+    DWORD dwEscapeStringLen = 0;
+
+    BAIL_ON_INVALID_POINTER(valueName);
+    BAIL_ON_INVALID_POINTER(dumpString);
+    BAIL_ON_INVALID_POINTER(retDumpStringLen);
+
+    dwError = RegShellUtilEscapeString(
+                  value,
+                  &valueEscName,
+                  &dwEscapeStringLen);
+    BAIL_ON_REG_ERROR(dwError);
+
+    bufLen = strlen(valueName) + dwEscapeStringLen + 8;
+
+    dwError = RegAllocateMemory(sizeof(*dumpBuf) * bufLen, (PVOID*)&dumpBuf);
+    BAIL_ON_REG_ERROR(dwError);
+
+    if (valueType == REG_KEY_DEFAULT)
+    {
+        dumpStringLen = sprintf(dumpBuf, "%s=\"%s\"",
+                            valueName,
+                            valueEscName);
+    }
+    else
+    {
+        dumpStringLen = sprintf(dumpBuf, "\"%s\"=\"%s\"",
+                            valueName,
+                            valueEscName);
+    }
+    LWREG_SAFE_FREE_MEMORY(valueEscName);
+    *retDumpStringLen = dumpStringLen;
+    *dumpString = dumpBuf;
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
+DWORD
+RegExportPlainText(
+    PCHAR value,
+    PSTR *dumpString,
+    PDWORD dumpStringLen
+    )
+{
+    DWORD bufLen = 0;
+    PSTR dumpBuf = NULL;
+    DWORD dwError = 0;
+
+    BAIL_ON_INVALID_POINTER(dumpString);
+    BAIL_ON_INVALID_POINTER(dumpStringLen);
+
+    bufLen = strlen(value) + 8;
+
+    dwError = RegAllocateMemory(sizeof(*dumpBuf) * bufLen, (PVOID*)&dumpBuf);
+    BAIL_ON_REG_ERROR(dwError);
+
+    *dumpStringLen = sprintf(dumpBuf, "%s", (PCHAR) value);
+    *dumpString = dumpBuf;
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
+DWORD
+RegExportBinaryData(
+    REG_DATA_TYPE valueType,
+    PCSTR valueName,
+    REG_DATA_TYPE type,
+    UCHAR *value,
+    DWORD valueLen,
+    PSTR *dumpString,
+    PDWORD dumpStringLen
+    )
+{
+    DWORD bufLen = 0;
+    DWORD formatLines = 0;
+    DWORD indx = 0;
+    DWORD dwError = 0;
+    DWORD linePos = 0;
+    PSTR dumpBuf = NULL;
+    PSTR fmtCursor = NULL;
+    UCHAR *pValue = NULL;
+    BOOLEAN firstHex = FALSE;
+
+    CHAR typeName[128];
+
+    if (type == 0 && valueLen == 0)
+    {
+        *dumpString = NULL;
+        *dumpStringLen = 0;
+        return 0;
+    }
+
+    RegExportBinaryTypeToString(type, typeName, TRUE);
+    /* 5 extra for " "= \\n characters on first line */
+    bufLen = strlen(valueName) + strlen(typeName) + 6;
+
+    /* 4 extra characters per line: Prefix "  " spaces, suffix \ and \r\n */
+    formatLines = valueLen / 25 + 1;
+    bufLen += valueLen * 3 + (formatLines * 5) + 1;
+
+    dwError = RegAllocateMemory(sizeof(*dumpBuf) * bufLen, (PVOID*)&dumpBuf);
+    BAIL_ON_REG_ERROR(dwError);
+
+    /* Format binary prefix */
+    fmtCursor = dumpBuf;
+    if (valueType == REG_KEY_DEFAULT)
+    {
+        fmtCursor += sprintf(fmtCursor, "@=%s", typeName);
+    }
+    else
+    {
+        fmtCursor += sprintf(fmtCursor, "\"%s\"=%s",
+                             valueName, typeName);
+    }
+
+    pValue = (UCHAR *) value;
+    linePos = fmtCursor - dumpBuf;
+    indx = 0;
+    while (indx < valueLen)
+    {
+        while(((linePos + 3)<REGEXPORT_LINE_WIDTH && indx<valueLen) ||
+               !firstHex)
+        {
+            firstHex = TRUE;
+            fmtCursor += sprintf(fmtCursor, "%02x,", pValue[indx]);
+            linePos += 3;
+            indx++;
+        }
+        if (indx < valueLen)
+        {
+            fmtCursor += sprintf(fmtCursor, "\\\r\n  ");
+            linePos = 2;
+        }
+        else
+        {
+            fmtCursor[-1] = '\0';
+            linePos = 0;
+        }
+
+    }
+
+    *dumpString = dumpBuf;
+    *dumpStringLen = fmtCursor - dumpBuf;
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
