@@ -1,6 +1,7 @@
 #ifndef _RSUTILS_H
 #define _RSUTILS_H
 
+
 #define REGEXPORT_LINE_WIDTH 80
 
 typedef struct _REGSHELL_UTIL_VALUE
@@ -9,13 +10,17 @@ typedef struct _REGSHELL_UTIL_VALUE
     PWSTR pValueName;
     LW_PVOID pData;
     DWORD dwDataLen;
+    BOOLEAN bValueSet;
 } REGSHELL_UTIL_VALUE, *PREGSHELL_UTIL_VALUE;
 
 
 typedef enum _REGSHELL_UTIL_IMPORT_MODE
 {
     REGSHELL_UTIL_IMPORT_OVERWRITE = 0,
+    REGSHELL_UTIL_IMPORT_OVERWRITE_VERBOSE,
     REGSHELL_UTIL_IMPORT_UPGRADE,
+    REGSHELL_UTIL_IMPORT_UPGRADE_VERBOSE,
+    REGSHELL_UTIL_IMPORT_CLEANUP,
 } REGSHELL_UTIL_IMPORT_MODE;
 
 typedef struct _REGSHELL_UTIL_IMPORT_CONTEXT
@@ -109,6 +114,16 @@ RegShellUtilGetValues(
     PDWORD pdwValueArrayLen
     );
 
+DWORD
+RegShellUtilGetKeyObjectCounts(
+    HANDLE hReg,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
+    PDWORD pdwSubKeysCount,
+    PDWORD pdwValuesCount
+    );
+
 DWORD
 RegShellUtilDeleteValue(
     HANDLE hReg,
@@ -130,28 +145,6 @@ RegShellUtilGetValue(
     OUT OPTIONAL PDWORD pdwValueLen
     );
 
-DWORD
-RegShellUtilImportCallback(
-    PREG_PARSE_ITEM pItem,
-    HANDLE hUserCtx
-    );
-
-DWORD
-RegShellUtilImportDebugCallback(
-    PREG_PARSE_ITEM pItem,
-    HANDLE userContext
-    );
-
-DWORD
-RegShellUtilExport(
-    HANDLE hReg,
-    FILE* fp,
-    HKEY hKey,
-    PCSTR pszKeyName,
-    DWORD dwNumSubKeys,
-    DWORD dwMaxSubKeyLen
-    );
-
 DWORD
 RegShellUtilEscapeString(
     PCSTR pszValue,
@@ -159,6 +152,12 @@ RegShellUtilEscapeString(
     PDWORD pdwEscapeValueLen
     );
 
+DWORD
+RegShellUtilValueArrayFree(
+    PREGSHELL_UTIL_VALUE pValueArray,
+    DWORD dwValueArrayLen
+    );
+
 DWORD
 RegExportBinaryTypeToString(
     REG_DATA_TYPE token,
@@ -166,16 +165,25 @@ RegExportBinaryTypeToString(
     BOOLEAN dumpFormat
     );
 
+DWORD
+RegExportAttributes(
+    PREG_PARSE_ITEM pItem,
+    PSTR *ppszDumpString,
+    PDWORD pdwDumpStringLen
+    );
+
 DWORD
 RegExportEntry(
     PCSTR keyName,
+    PCSTR pszSddlCString,
     REG_DATA_TYPE valueType,
     PCSTR valueName,
     REG_DATA_TYPE type,
     LW_PVOID value,
     DWORD valueLen,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
 RegExportDword(
@@ -183,13 +191,16 @@ RegExportDword(
     PCSTR valueName,
     DWORD value,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
 RegExportRegKey(
     PCSTR keyName,
+    PCSTR pszSddlCString,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
 RegExportString(
@@ -197,7 +208,8 @@ RegExportString(
     PCSTR valueName,
     PCSTR value,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
 RegExportBinaryData(
@@ -207,17 +219,21 @@ RegExportBinaryData(
     UCHAR *value,
     DWORD valueLen,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
 RegExportPlainText(
     PCHAR value,
     PSTR *dumpString,
-    PDWORD dumpStringLen);
+    PDWORD dumpStringLen
+    );
 
 DWORD
-RegShellUtilValueArrayFree(
-    PREGSHELL_UTIL_VALUE pValueArray,
-    DWORD dwValueArrayLen);
+RegExportAttributeEntries(
+    PREG_PARSE_ITEM pItem,
+    PSTR *ppszDumpString,
+    PDWORD pdwDumpStringLen
+    );
 
 #endif
