@@ -56,10 +56,10 @@ RegMultiStrsToByteArrayW(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegMultiStrsToByteArrayW(ppwszInMultiSz,
-				                   ppOutBuf,
-				                   pOutBufLen)
-				                  );
+    		NtRegMultiStrsToByteArrayW(ppwszInMultiSz,
+    				                   ppOutBuf,
+    				                   pOutBufLen)
+    				                  );
 }
 
 NTSTATUS
@@ -86,7 +86,7 @@ NtRegMultiStrsToByteArrayW(
 
         if (ppwszInMultiSz[idx])
         {
-		len = RtlWC16StringNumChars(ppwszInMultiSz[idx]);
+        	len = RtlWC16StringNumChars(ppwszInMultiSz[idx]);
         }
 
         OutBufLen +=  (len + 1) * sizeof(WCHAR);
@@ -95,7 +95,7 @@ NtRegMultiStrsToByteArrayW(
     OutBufLen += sizeof(WCHAR); // double null at end
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&pOutBuf, BYTE,
-		                  sizeof(*pOutBuf) * OutBufLen);
+    		                  sizeof(*pOutBuf) * OutBufLen);
     BAIL_ON_REG_ERROR(dwError);
 
     for (idx=0, pCursor = pOutBuf; ppwszInMultiSz[idx]; idx++)
@@ -104,7 +104,7 @@ NtRegMultiStrsToByteArrayW(
 
         if (ppwszInMultiSz[idx])
         {
-		len = RtlWC16StringNumChars(ppwszInMultiSz[idx]);
+        	len = RtlWC16StringNumChars(ppwszInMultiSz[idx]);
         }
 
         len++; // accommodate null
@@ -150,10 +150,10 @@ RegMultiStrsToByteArrayA(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegMultiStrsToByteArrayA(ppszInMultiSz,
-				                   ppOutBuf,
-				                   pOutBufLen)
-				                  );
+    		NtRegMultiStrsToByteArrayA(ppszInMultiSz,
+    				                   ppOutBuf,
+    				                   pOutBufLen)
+    				                  );
 }
 
 NTSTATUS
@@ -182,7 +182,7 @@ NtRegMultiStrsToByteArrayA(
     OutBufLen++; // double null at end
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&pOutBuf, BYTE,
-		                  sizeof(*pOutBuf) * OutBufLen);
+    		                  sizeof(*pOutBuf) * OutBufLen);
     BAIL_ON_REG_ERROR(dwError);
 
     for (idx=0, pCursor = pOutBuf; ppszInMultiSz[idx]; idx++)
@@ -230,10 +230,10 @@ RegByteArrayToMultiStrsW(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegByteArrayToMultiStrsW(pInBuf,
-				                   bufLen,
-				                   pppwszStrings)
-				                  );
+    		NtRegByteArrayToMultiStrsW(pInBuf,
+    				                   bufLen,
+    				                   pppwszStrings)
+    				                  );
 
 }
 
@@ -266,7 +266,7 @@ NtRegByteArrayToMultiStrsW(
     {
         if (pwszCursor)
         {
-		len = RtlWC16StringNumChars(pwszCursor);
+        	len = RtlWC16StringNumChars(pwszCursor);
         }
 
         if (len)
@@ -277,7 +277,7 @@ NtRegByteArrayToMultiStrsW(
     } while (len);
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&ppwszStrings, PWSTR,
-		                  sizeof(*ppwszStrings) * (dwNumStrings + 1));
+    		                  sizeof(*ppwszStrings) * (dwNumStrings + 1));
     BAIL_ON_REG_ERROR(dwError);
 
     pwszCursor = (PWSTR)pInBuf;
@@ -293,7 +293,7 @@ NtRegByteArrayToMultiStrsW(
         }
 
         dwError = LW_RTL_ALLOCATE((PVOID*)&ppwszStrings[iStr], WCHAR,
-			                  sizeof(*ppwszStrings[iStr]) * (len + 1));
+        		                  sizeof(*ppwszStrings[iStr]) * (len + 1));
         BAIL_ON_REG_ERROR(dwError);
 
         memcpy( (PBYTE)ppwszStrings[iStr],
@@ -329,10 +329,10 @@ RegByteArrayToMultiStrsA(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegByteArrayToMultiStrsA(pInBuf,
-				                   bufLen,
-				                   pppszStrings)
-				                  );
+    		NtRegByteArrayToMultiStrsA(pInBuf,
+    				                   bufLen,
+    				                   pppszStrings)
+    				                  );
 }
 
 NTSTATUS
@@ -372,7 +372,7 @@ NtRegByteArrayToMultiStrsA(
     } while (sLen);
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&ppszStrings, PSTR,
-		                  sizeof(*ppszStrings) * (dwNumStrings + 1));
+    		                  sizeof(*ppszStrings) * (dwNumStrings + 1));
     BAIL_ON_REG_ERROR(dwError);
 
     pszCursor = (PSTR)pInBuf;
@@ -388,7 +388,7 @@ NtRegByteArrayToMultiStrsA(
         }
 
         dwError = LW_RTL_ALLOCATE((PVOID*)&ppszStrings[iStr], CHAR,
-			                  sizeof(*ppszStrings[iStr]) * (len + 1));
+        		                  sizeof(*ppszStrings[iStr]) * (len + 1));
         BAIL_ON_REG_ERROR(dwError);
 
         memcpy( (PBYTE)ppszStrings[iStr],
@@ -425,11 +425,11 @@ RegConvertByteStreamA2W(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegConvertByteStreamA2W(pData,
-				                  cbData,
-				                  ppOutData,
-				                  pcbOutDataLen)
-				                  );
+    		NtRegConvertByteStreamA2W(pData,
+    				                  cbData,
+    				                  ppOutData,
+    				                  pcbOutDataLen)
+    				                  );
 }
 
 NTSTATUS
@@ -451,7 +451,7 @@ NtRegConvertByteStreamA2W(
     cbOutDataLen = cbData * sizeof(WCHAR);
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&pOutData, BYTE,
-		                  sizeof(*pOutData) * cbOutDataLen);
+    		                  sizeof(*pOutData) * cbOutDataLen);
     BAIL_ON_REG_ERROR(dwError);
 
     pszCursor = (PCSTR)pData;
@@ -467,8 +467,8 @@ NtRegConvertByteStreamA2W(
             pwszValue = NULL;
         }
 
-	dwError = LwRtlWC16StringAllocateFromCString(&pwszValue, pszCursor);
-	BAIL_ON_REG_ERROR(dwError);
+    	dwError = LwRtlWC16StringAllocateFromCString(&pwszValue, pszCursor);
+    	BAIL_ON_REG_ERROR(dwError);
 
         cbWcLen = wc16slen(pwszValue);
         memcpy((PBYTE)pwszCursor,
@@ -514,11 +514,11 @@ RegConvertByteStreamW2A(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegConvertByteStreamW2A(pData,
-				                  cbData,
-				                  ppOutData,
-				                  pcbOutDataLen)
-				                  );
+    		NtRegConvertByteStreamW2A(pData,
+    				                  cbData,
+    				                  ppOutData,
+    				                  pcbOutDataLen)
+    				                  );
 }
 
 NTSTATUS
@@ -541,7 +541,7 @@ NtRegConvertByteStreamW2A(
     cbOutDataLen = (cbData / sizeof(WCHAR)) * 4;
 
     dwError = LW_RTL_ALLOCATE((PVOID*)&pOutData, BYTE,
-		                  sizeof(*pOutData) * cbOutDataLen);
+    		                  sizeof(*pOutData) * cbOutDataLen);
     BAIL_ON_REG_ERROR(dwError);
 
     pwszCursor = (PWSTR)pData;
@@ -553,7 +553,7 @@ NtRegConvertByteStreamW2A(
 
         if (pwszCursor)
         {
-		len = RtlWC16StringNumChars(pwszCursor);
+        	len = RtlWC16StringNumChars(pwszCursor);
         }
 
         if (pszValue)
@@ -562,10 +562,10 @@ NtRegConvertByteStreamW2A(
             pszValue = NULL;
         }
 
-	dwError = LwRtlCStringAllocateFromWC16String(&pszValue, pwszCursor);
+    	dwError = LwRtlCStringAllocateFromWC16String(&pszValue, pwszCursor);
         BAIL_ON_REG_ERROR(dwError);
 
-        cbMbsLen = strlen(pszValue);
+        cbMbsLen = strlen(pszValue); 
         memcpy(pszCursor, pszValue, cbMbsLen + 1);
 
         pszCursor  += cbMbsLen + 1;
@@ -610,8 +610,8 @@ RegCopyValueBytes(
 
     if (pData && !pcbData)
     {
-	status = STATUS_INVALID_PARAMETER;
-	BAIL_ON_NT_STATUS(status);
+    	status = STATUS_INVALID_PARAMETER;
+    	BAIL_ON_NT_STATUS(status);
     }
 
     if(pData && dwValueLen > *pcbData)
@@ -667,3 +667,153 @@ RegFreeMultiStrsW(
 
     LwRtlMemoryFree(ppwszStrings);
 }
+
+
+typedef struct _REG_HINT_ENTRY
+{
+   PSTR pszHintName;
+   LWREG_VALUE_HINT dwHintValue;
+} REG_HINT_ENTRY;
+
+static REG_HINT_ENTRY gHints[] =
+{
+   {"seconds", 1},
+   {"path",    2},
+   {"account", 3},
+   {NULL,      0}
+
+};
+
+
+DWORD
+RegFindHintByName(PSTR pszHint)
+{
+   DWORD dwI = 0;
+
+   if (!pszHint)
+   {
+       return 0;
+   }
+
+   for (dwI=0; gHints[dwI].pszHintName; dwI++)
+   {
+       if (strcmp(pszHint, gHints[dwI].pszHintName) == 0)
+       {
+           return gHints[dwI].dwHintValue;
+       }
+    }
+
+   return 0;
+}
+
+PSTR
+RegFindHintByValue(DWORD dwHint)
+{
+   DWORD dwI = 0;
+
+   for (dwI=0; gHints[dwI].pszHintName; dwI++)
+   {
+       if (dwHint == gHints[dwI].dwHintValue)
+       {
+           return gHints[dwI].pszHintName;
+       }
+    }
+
+   return NULL;
+}
+
+BOOLEAN
+RegValidValueAttributes(
+    IN PLWREG_VALUE_ATTRIBUTES pValueAttributes
+    )
+{
+    BOOLEAN bIsValid = TRUE;
+    DWORD dwDefaultValue = 0;
+
+    switch (pValueAttributes->ValueType)
+    {
+        case REG_DWORD:
+
+            // Check range information consistency
+            if (pValueAttributes->RangeType == LWREG_VALUE_RANGE_TYPE_ENUM  ||
+                pValueAttributes->RangeType > LWREG_VALUE_RANGE_TYPE_INTEGER )
+            {
+                bIsValid = FALSE;
+                goto cleanup;
+            }
+            else if (pValueAttributes->RangeType == LWREG_VALUE_RANGE_TYPE_BOOLEAN &&
+                    pValueAttributes->pDefaultValue)
+            {
+                dwDefaultValue = *(PDWORD) pValueAttributes->pDefaultValue;
+
+                if (dwDefaultValue != 0 && dwDefaultValue != 1)
+                {
+                    bIsValid = FALSE;
+                    goto cleanup;
+                }
+            }
+            else if (pValueAttributes->RangeType == LWREG_VALUE_RANGE_TYPE_INTEGER)
+            {
+                if (pValueAttributes->Range.RangeInteger.Max < pValueAttributes->Range.RangeInteger.Min)
+                {
+                    bIsValid = FALSE;
+                    goto cleanup;
+                }
+
+                if (pValueAttributes->pDefaultValue)
+                {
+                    dwDefaultValue = *(PDWORD) pValueAttributes->pDefaultValue;
+
+                    if (dwDefaultValue < pValueAttributes->Range.RangeInteger.Min ||
+                        dwDefaultValue > pValueAttributes->Range.RangeInteger.Max)
+                    {
+                        bIsValid = FALSE;
+                        goto cleanup;
+                    }
+                }
+            }
+
+            // Check hint information consistency
+            if (pValueAttributes->Hint >= LWREG_VALUE_HINT_PATH)
+            {
+                bIsValid = FALSE;
+                goto cleanup;
+            }
+
+            break;
+
+        case REG_SZ:
+        case REG_MULTI_SZ:
+
+            // Check range information consistency
+            if (pValueAttributes->RangeType == LWREG_VALUE_RANGE_TYPE_BOOLEAN ||
+                pValueAttributes->RangeType >= LWREG_VALUE_RANGE_TYPE_INTEGER)
+            {
+                bIsValid = FALSE;
+                goto cleanup;
+            }
+            else if (pValueAttributes->RangeType == LWREG_VALUE_RANGE_TYPE_ENUM)
+            {
+                if (!pValueAttributes->Range.ppwszRangeEnumStrings)
+                {
+                    bIsValid = FALSE;
+                    goto cleanup;
+                }
+            }
+
+            // Check hint information consistency
+            if (pValueAttributes->Hint == LWREG_VALUE_HINT_SECONDS)
+            {
+                bIsValid = FALSE;
+                goto cleanup;
+            }
+
+            break;
+
+        default:
+            goto cleanup;
+    }
+
+cleanup:
+    return bIsValid;
+}
