@@ -79,7 +79,7 @@ RegInitializeStringBuffer(
 
     if (sCapacity > DWORD_MAX - 1)
     {
-	status = STATUS_INVALID_PARAMETER;
+    	status = STATUS_INVALID_PARAMETER;
         BAIL_ON_NT_STATUS(status);
     }
 
@@ -118,13 +118,13 @@ RegAppendStringBuffer(
 
         if (sNewCapacity > DWORD_MAX - 1)
         {
-		status = STATUS_INVALID_PARAMETER;
+        	status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
         }
 
         if (sNewCapacity < pBuffer->sCapacity)
         {
-		status = STATUS_INVALID_PARAMETER;
+        	status = STATUS_INVALID_PARAMETER;
             BAIL_ON_NT_STATUS(status);
         }
 
@@ -137,11 +137,14 @@ RegAppendStringBuffer(
         pBuffer->sCapacity = sNewCapacity;
     }
 
-    memcpy(
-        pBuffer->pszBuffer + pBuffer->sLen,
-        pszAppend,
-        sAppendLen);
-    pBuffer->sLen += sAppendLen;
+    if (pszAppend != NULL)
+    {
+        memcpy(
+            pBuffer->pszBuffer + pBuffer->sLen,
+            pszAppend,
+            sAppendLen);
+        pBuffer->sLen += sAppendLen;
+    }
     pBuffer->pszBuffer[pBuffer->sLen] = '\0';
 
 cleanup:
@@ -169,6 +172,57 @@ RegFreeMemory(
 	LwRtlMemoryFree(pMemory);
 }
 
+void
+RegSafeFreeValueAttributes(
+    PLWREG_VALUE_ATTRIBUTES* ppValueAttrs
+    )
+{
+    PLWREG_VALUE_ATTRIBUTES pValueAttrs = NULL;
+
+    if ((ppValueAttrs == NULL) || (*ppValueAttrs == NULL)) {
+        return;
+    }
+
+    pValueAttrs = *ppValueAttrs;
+    RTL_FREE(&pValueAttrs->pDefaultValue);
+    RTL_FREE(&pValueAttrs->pwszDocString);
+
+    if (LWREG_VALUE_RANGE_TYPE_ENUM == pValueAttrs->RangeType)
+    {
+        if (pValueAttrs->Range.ppwszRangeEnumStrings)
+        {
+            RegFreeWC16StringArrayWithNullTerminator(pValueAttrs->Range.ppwszRangeEnumStrings);
+        }
+        pValueAttrs->Range.ppwszRangeEnumStrings = NULL;
+    }
+
+    RTL_FREE(&pValueAttrs);
+
+    *ppValueAttrs = NULL;
+
+    return;
+}
+
+void
+RegSafeFreeCurrentValueInfo(
+    PLWREG_CURRENT_VALUEINFO* ppValueInfo
+    )
+{
+    PLWREG_CURRENT_VALUEINFO pValueInfo = NULL;
+
+    if ((ppValueInfo == NULL) || (*ppValueInfo == NULL)) {
+        return;
+    }
+
+    pValueInfo = *ppValueInfo;
+    RTL_FREE(&pValueInfo->pvData);
+    RTL_FREE(&pValueInfo);
+
+    *ppValueInfo = NULL;
+
+    return;
+}
+
 DWORD
 RegReallocMemory(
     IN PVOID pMemory,
@@ -177,10 +231,10 @@ RegReallocMemory(
     )
 {
     return RegNtStatusToWin32Error(
-		NtRegReallocMemory(pMemory,
-				         ppNewMemory,
-				         dwSize)
-				         );
+    		NtRegReallocMemory(pMemory,
+    				         ppNewMemory,
+    				         dwSize)
+    				         );
 
 }
 
@@ -201,7 +255,7 @@ NtRegReallocMemory(
        pNewMemory = realloc(pMemory, dwSize);
     }
     if (!pNewMemory){
-	status = STATUS_INSUFFICIENT_RESOURCES;
+    	status = STATUS_INSUFFICIENT_RESOURCES;
        *ppNewMemory = NULL;
     }else {
        *ppNewMemory = pNewMemory;
@@ -232,7 +286,7 @@ RegFreeStringArray(
         {
             if (ppStringArray[i])
             {
-		LwRtlMemoryFree(ppStringArray[i]);
+            	LwRtlMemoryFree(ppStringArray[i]);
             }
         }
 
@@ -242,6 +296,26 @@ RegFreeStringArray(
     return;
 }
 
+void
+RegFreeWC16StringArrayWithNullTerminator(
+    PWSTR * ppwStringArray
+    )
+{
+    DWORD i = 0;
+
+    if ( ppwStringArray )
+    {
+        while (ppwStringArray[i])
+        {
+            LwRtlMemoryFree(ppwStringArray[i++]);
+        }
+
+        LwRtlMemoryFree(ppwStringArray);
+    }
+
+    return;
+}
+
 void
 RegFreeWC16StringArray(
     PWSTR * ppwStringArray,
@@ -256,7 +330,7 @@ RegFreeWC16StringArray(
         {
             if (ppwStringArray[i])
             {
-		LwRtlMemoryFree(ppwStringArray[i]);
+                LwRtlMemoryFree(ppwStringArray[i]);
             }
         }
 
@@ -280,7 +354,7 @@ RegFreeValueByteArray(
         {
             if (ppValues[i])
             {
-		LwRtlMemoryFree(ppValues[i]);
+            	LwRtlMemoryFree(ppValues[i]);
             }
         }
 
@@ -357,7 +431,7 @@ RegHexStrToByteArray(
     }
 
     status = LW_RTL_ALLOCATE((PVOID*)&pucByteArray, UCHAR,
-			          sizeof(*pucByteArray)* dwByteArrayLength);
+            		          sizeof(*pucByteArray)* dwByteArrayLength);
     BAIL_ON_NT_STATUS(status);
 
     for (i = 0; i < dwByteArrayLength; i++)
@@ -405,7 +479,7 @@ RegByteArrayToHexStr(
     PSTR pszHexString = NULL;
 
     status = LW_RTL_ALLOCATE((PVOID*)&pszHexString, CHAR,
-			          sizeof(*pszHexString)* (dwByteArrayLength*2 + 1));
+            		          sizeof(*pszHexString)* (dwByteArrayLength*2 + 1));
     BAIL_ON_NT_STATUS(status);
 
     for (i = 0; i < dwByteArrayLength; i++)
@@ -490,8 +564,8 @@ RegAllocateMemory(
 	if (Size)
     {
 		dwError = RegNtStatusToWin32Error(
-				RTL_ALLOCATE(ppMemory, VOID, Size)
-				);
+		    		RTL_ALLOCATE(ppMemory, VOID, Size)
+		    		);
     }
 	else
 	{
@@ -625,6 +699,287 @@ RegStrchr(
 	return NULL;
 }
 
+
+DWORD
+RegWC16StringArraysAllocateFromCStringArraysWithNullTerminator(
+    IN PSTR* ppszStrings,
+    OUT PWSTR** pppwszStrings
+    )
+{
+    DWORD dwError = 0;
+    PWSTR* ppwszStrings = NULL;
+    int i = 0;
+
+    if (!ppszStrings || !ppszStrings[0])
+    {
+        goto cleanup;
+    }
+
+    while (ppszStrings[i++]);
+
+    dwError = RegAllocateMemory(sizeof(*ppwszStrings)*i, (PVOID*)&ppwszStrings);
+    BAIL_ON_REG_ERROR(dwError);
+
+    i = 0;
+
+    while (ppszStrings[i])
+    {
+        dwError = RegWC16StringAllocateFromCString(
+                               &ppwszStrings[i],
+                               ppszStrings[i]);
+        BAIL_ON_REG_ERROR(dwError);
+
+        i++;
+    }
+
+    *pppwszStrings = ppwszStrings;
+
+cleanup:
+    return dwError;
+
+error:
+    if (ppwszStrings)
+    {
+        RegFreeWC16StringArrayWithNullTerminator(ppwszStrings);
+    }
+    *pppwszStrings = NULL;
+
+    goto cleanup;
+}
+
+DWORD
+RegCopyValueAToW(
+    IN REG_DATA_TYPE dwType,
+    IN PVOID pData,
+    IN DWORD cbData,
+    OUT PVOID* ppOutData,
+    OUT PDWORD pcbOutDataLen
+    )
+{
+    DWORD dwError = 0;
+    PVOID pOutData = NULL;
+    DWORD cbOutDataLen = 0;
+    BOOLEAN bIsStrType = FALSE;
+
+    if (dwType == REG_MULTI_SZ)
+    {
+        if (!pData)
+        {
+            pData = (PBYTE) "";
+        }
+        if (cbData == 0)
+        {
+            cbData = 1;
+        }
+    }
+
+    if (pData)
+    {
+        if (REG_MULTI_SZ == dwType)
+        {
+            dwError = RegConvertByteStreamA2W((PBYTE)pData,
+                                              cbData,
+                                              (PBYTE*)&pOutData,
+                                              &cbOutDataLen);
+            BAIL_ON_REG_ERROR(dwError);
+
+            bIsStrType = TRUE;
+        }
+        else if (REG_SZ == dwType)
+        {
+            /* Verify correct null termination of input data */
+            if (strlen((char *) pData) != (cbData-1))
+            {
+                dwError = ERROR_INVALID_PARAMETER;
+                BAIL_ON_REG_ERROR(dwError);
+            }
+
+            dwError = RegWC16StringAllocateFromCString((PWSTR*)&pOutData, pData);
+            BAIL_ON_REG_ERROR(dwError);
+
+            cbOutDataLen = (mbstrlen((const char*) pData)+1) * sizeof(WCHAR);
+            bIsStrType = TRUE;
+        }
+    }
+
+    if (!bIsStrType && cbData)
+    {
+        dwError = RegAllocateMemory(cbData, &pOutData);
+        BAIL_ON_REG_ERROR(dwError);
+
+        if (pData)
+        {
+            memcpy(pOutData, pData, cbData);
+            cbOutDataLen = cbData;
+        }
+    }
+
+    *ppOutData = pOutData;
+    *pcbOutDataLen = cbOutDataLen;
+
+cleanup:
+    return dwError;
+
+error:
+    LWREG_SAFE_FREE_MEMORY(pOutData);
+
+    *ppOutData = NULL;
+    *pcbOutDataLen = 0;
+
+    goto cleanup;
+}
+
+DWORD
+RegConvertValueAttributesAToW(
+    LWREG_VALUE_ATTRIBUTES_A attrA,
+    PLWREG_VALUE_ATTRIBUTES* ppAttrW
+    )
+{
+    DWORD dwError = 0;
+    PLWREG_VALUE_ATTRIBUTES pAttr = NULL;
+
+    dwError = RegAllocateMemory(sizeof(*pAttr),(PVOID*)&pAttr);
+    BAIL_ON_REG_ERROR(dwError);
+
+    pAttr->DefaultValueLen = attrA.DefaultValueLen;
+    pAttr->Hint = attrA.Hint;
+    pAttr->RangeType = attrA.RangeType;
+
+    switch (pAttr->RangeType)
+    {
+        case LWREG_VALUE_RANGE_TYPE_ENUM:
+
+            dwError = RegWC16StringArraysAllocateFromCStringArraysWithNullTerminator(
+                            attrA.Range.ppszRangeEnumStrings,
+                            &pAttr->Range.ppwszRangeEnumStrings);
+            BAIL_ON_REG_ERROR(dwError);
+
+            break;
+
+        case LWREG_VALUE_RANGE_TYPE_INTEGER:
+             pAttr->Range.RangeInteger.Min = attrA.Range.RangeInteger.Min;
+             pAttr->Range.RangeInteger.Max = attrA.Range.RangeInteger.Max;
+
+             break;
+
+        case LWREG_VALUE_RANGE_TYPE_BOOLEAN:
+             pAttr->Range.RangeInteger.Min = 0;
+             pAttr->Range.RangeInteger.Max = 1;
+
+             break;
+
+        default:
+             dwError = ERROR_INVALID_PARAMETER;
+             BAIL_ON_REG_ERROR(dwError);
+      }
+
+    if (attrA.pszDocString)
+    {
+        dwError = RegWC16StringAllocateFromCString(
+                             &pAttr->pwszDocString,
+                             attrA.pszDocString
+                               );
+        BAIL_ON_REG_ERROR(dwError);
+    }
+
+    pAttr->ValueType = attrA.ValueType;
+
+    dwError = RegCopyValueAToW(pAttr->ValueType,
+                               attrA.pDefaultValue,
+                               attrA.DefaultValueLen,
+                               &pAttr->pDefaultValue,
+                               &pAttr->DefaultValueLen);
+    BAIL_ON_REG_ERROR(dwError);
+
+    *ppAttrW = pAttr;
+
+cleanup:
+    return dwError;
+
+error:
+
+    RegSafeFreeValueAttributes(&pAttr);
+
+    goto cleanup;
+}
+
+#if 0
+DWORD
+RegDuplicateValueAttributes(
+    LWREG_VALUE_ATTRIBUTES pAttr,
+    PLWREG_VALUE_ATTRIBUTES* ppAttr
+    )
+{
+    DWORD dwError = 0;
+    PLWREG_VALUE_ATTRIBUTES pAttr = NULL;
+
+    dwError = RegAllocateMemory(sizeof(*pAttr),(PVOID*)&pAttr);
+    BAIL_ON_REG_ERROR(dwError);
+
+    pAttr->DefaultValueLen = pAttr.DefaultValueLen;
+    pAttr->Hint = attrA.Hint;
+    pAttr->RangeType = attrA.RangeType;
+
+    switch (pAttr->RangeType)
+    {
+        case LWREG_VALUE_RANGE_TYPE_ENUM:
+
+            dwError = RegWC16StringArraysAllocateFromCStringArraysWithNullTerminator(
+                            attrA.Range.ppszRangeEnumStrings,
+                            &pAttr->Range.ppwszRangeEnumStrings);
+            BAIL_ON_REG_ERROR(dwError);
+
+            break;
+
+        case LWREG_VALUE_RANGE_TYPE_INTEGER:
+             pAttr->Range.RangeInteger.Min = attrA.Range.RangeInteger.Min;
+             pAttr->Range.RangeInteger.Max = attrA.Range.RangeInteger.Max;
+
+             break;
+
+        case LWREG_VALUE_RANGE_TYPE_BOOLEAN:
+             pAttr->Range.RangeInteger.Min = 0;
+             pAttr->Range.RangeInteger.Max = 1;
+
+             break;
+
+        default:
+             dwError = ERROR_INVALID_PARAMETER;
+             BAIL_ON_REG_ERROR(dwError);
+      }
+
+    if (attrA.pszDocString)
+    {
+        dwError = RegWC16StringAllocateFromCString(
+                             &pAttr->pwszDocString,
+                             attrA.pszDocString
+                               );
+        BAIL_ON_REG_ERROR(dwError);
+    }
+
+    pAttr->ValueType = attrA.ValueType;
+
+    dwError = RegCopyValueAToW(pAttr->ValueType,
+                               attrA.pDefaultValue,
+                               attrA.DefaultValueLen,
+                               &pAttr->pDefaultValue,
+                               &pAttr->DefaultValueLen);
+    BAIL_ON_REG_ERROR(dwError);
+
+    *ppAttrW = pAttr;
+
+cleanup:
+    return dwError;
+
+error:
+
+    RegSafeFreeValueAttributes(&pAttr);
+
+    goto cleanup;
+}
+#endif
+
+
 static
 NTSTATUS
 RegHexCharToByte(
@@ -649,7 +1004,7 @@ RegHexCharToByte(
     }
     else
     {
-	status = STATUS_INVALID_PARAMETER;
+    	status = STATUS_INVALID_PARAMETER;
        BAIL_ON_NT_STATUS(status);
     }
 
