@@ -230,12 +230,12 @@ LwSmConfigureSignals(
         if (sigaddset(&set, blockSignals[i]) < 0)
         {
             dwError = LwMapErrnoToLwError(errno);
-            BAIL_ON_ERROR(dwError);
+            BAIL_ON_ERROR(dwError); 
         }
     }
 
     dwError = LwMapErrnoToLwError(pthread_sigmask(SIG_SETMASK, &set, NULL));
-    BAIL_ON_ERROR(dwError);
+    BAIL_ON_ERROR(dwError); 
 
     intAction.sa_handler = LwSmHandleSigint;
     intAction.sa_flags = 0;
@@ -283,7 +283,7 @@ LwSmWaitSignals(
         if (sigaddset(&set, waitSignals[i]) < 0)
         {
             dwError = LwMapErrnoToLwError(errno);
-            BAIL_ON_ERROR(dwError);
+            BAIL_ON_ERROR(dwError); 
         }
     }
 
@@ -735,11 +735,11 @@ LwSmRestart(
     BAIL_ON_ERROR(dwError);
 
     dwError = LwSmAcquireServiceHandle(pwszServiceName, &hHandle);
-    BAIL_ON_ERROR(dwError);
+    BAIL_ON_ERROR(dwError); 
 
     dwError = LwSmQueryServiceReverseDependencyClosure(hHandle, &ppwszReverseDeps);
     BAIL_ON_ERROR(dwError);
-
+ 
     count = LwSmStringListLength(ppwszReverseDeps);
 
     dwError = LwAllocateMemory(sizeof(*pStatus) * count, OUT_PPVOID(&pStatus));
@@ -795,7 +795,7 @@ LwSmRestart(
             {
                 dwError = LwWc16sToMbs(ppwszDependencies[i], &pszTemp);
                 BAIL_ON_ERROR(dwError);
-
+                
                 printf("Starting service dependency: %s\n", pszTemp);
                 LW_SAFE_FREE_MEMORY(pszTemp);
             }
@@ -881,7 +881,7 @@ error:
 
 static
 DWORD
-LwSmRefresh(
+LwSmDoRefresh(
     int argc,
     char** pArgv
     )
@@ -890,22 +890,35 @@ LwSmRefresh(
     PWSTR pwszServiceName = NULL;
     LW_SERVICE_HANDLE hHandle = NULL;
     
-    dwError = LwMbsToWc16s(pArgv[1], &pwszServiceName);
-    BAIL_ON_ERROR(dwError);
-    
-    dwError = LwSmAcquireServiceHandle(pwszServiceName, &hHandle);
-    BAIL_ON_ERROR(dwError);
-    
-    if (!gState.bQuiet)
+    if (argc > 1)
     {
-        printf("Refreshing service: %s\n", pArgv[1]);
+        dwError = LwMbsToWc16s(pArgv[1], &pwszServiceName);
+        BAIL_ON_ERROR(dwError);
+    
+        dwError = LwSmAcquireServiceHandle(pwszServiceName, &hHandle);
+        BAIL_ON_ERROR(dwError);
+
+        if (!gState.bQuiet)
+        {
+            printf("Refreshing service: %s\n", pArgv[1]);
+        }
+
+        dwError = LwSmRefreshService(hHandle);
+        BAIL_ON_ERROR(dwError);
+    }
+    else
+    {
+        if (!gState.bQuiet)
+        {
+            printf("Refreshing service manager\n");
+        }
+
+        dwError = LwSmRefresh();
+        BAIL_ON_ERROR(dwError);
     }
 
-    dwError = LwSmRefreshService(hHandle);
-    BAIL_ON_ERROR(dwError);
-    
 cleanup:
-    
+
     LW_SAFE_FREE_MEMORY(pwszServiceName);
 
     if (hHandle)
@@ -1002,6 +1015,15 @@ LwSmInfo(
 
     printf("\n");
 
+    if (pInfo->dwFdLimit)
+    {
+        printf("File descriptor limit: %lu\n", (unsigned long) pInfo->dwFdLimit);
+    }
+    else
+    {
+        printf("File descriptor limit: inherit\n");
+    }
+
 cleanup:
     
     LW_SAFE_FREE_MEMORY(pwszServiceName);
@@ -1075,6 +1097,131 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+LwSmAutostart(
+    int argc,
+    char** pArgv
+    )
+{
+    DWORD dwError = 0;
+    PWSTR *ppwszAllServices = NULL;
+    LW_SERVICE_HANDLE hHandle = NULL;
+    PLW_SERVICE_INFO pInfo = NULL;
+    size_t i = 0;
+    PWSTR *ppwszDependencies = NULL;
+    LW_SERVICE_HANDLE hDepHandle = NULL;
+    LW_SERVICE_STATUS status = {0};
+    PSTR pszTemp = NULL;
+    size_t j = 0;
+
+    dwError = LwSmEnumerateServices(&ppwszAllServices);
+    BAIL_ON_ERROR(dwError);
+
+    for (i = 0; ppwszAllServices[i]; i++)
+    {
+        dwError = LwSmAcquireServiceHandle(ppwszAllServices[i], &hHandle);
+        BAIL_ON_ERROR(dwError);
+
+        dwError = LwSmQueryServiceInfo(hHandle, &pInfo);
+        BAIL_ON_ERROR(dwError);
+
+        if (pInfo->bAutostart)
+        {
+            dwError = LwSmQueryServiceDependencyClosure(
+                            hHandle,
+                            &ppwszDependencies);
+            BAIL_ON_ERROR(dwError);
+
+            for (j = 0; ppwszDependencies[j]; j++)
+            {
+                dwError = LwSmAcquireServiceHandle(
+                                ppwszDependencies[j],
+                                &hDepHandle);
+                BAIL_ON_ERROR(dwError);
+
+                dwError = LwSmQueryServiceStatus(hDepHandle, &status);
+                BAIL_ON_ERROR(dwError);
+
+                if (status.state != LW_SERVICE_STATE_RUNNING)
+                {
+                    if (!gState.bQuiet)
+                    {
+                        dwError = LwWc16sToMbs(ppwszDependencies[j], &pszTemp);
+                        BAIL_ON_ERROR(dwError);
+
+                        printf("Starting service dependency: %s\n", pszTemp);
+                        LW_SAFE_FREE_MEMORY(pszTemp);
+                    }
+
+                    dwError = LwSmStartService(hDepHandle);
+                    BAIL_ON_ERROR(dwError);
+                }
+
+                dwError = LwSmReleaseServiceHandle(hDepHandle);
+                hDepHandle = NULL;
+                BAIL_ON_ERROR(dwError);
+            }
+
+            if (ppwszDependencies)
+            {
+                LwSmFreeServiceNameList(ppwszDependencies);
+                ppwszDependencies = NULL;
+            }
+
+            if (!gState.bQuiet)
+            {
+                dwError = LwWc16sToMbs(ppwszAllServices[i], &pszTemp);
+                BAIL_ON_ERROR(dwError);
+
+                printf("Starting service: %s\n", pszTemp);
+                LW_SAFE_FREE_MEMORY(pszTemp);
+            }
+
+            dwError = LwSmStartService(hHandle);
+            BAIL_ON_ERROR(dwError);
+        }
+
+        dwError = LwSmReleaseServiceHandle(hHandle);
+        hHandle = NULL;
+        BAIL_ON_ERROR(dwError);
+    }
+
+cleanup:
+
+    if (hHandle)
+    {
+        LwSmReleaseServiceHandle(hHandle);
+        hHandle = NULL;
+    }
+
+    if (ppwszAllServices)
+    {
+        LwSmFreeStringList(ppwszAllServices);
+        ppwszAllServices = NULL;
+    }
+
+    if (ppwszDependencies)
+    {
+        LwSmFreeServiceNameList(ppwszDependencies);
+        ppwszDependencies = NULL;
+    }
+
+    if (hDepHandle)
+    {
+        LwSmReleaseServiceHandle(hDepHandle);
+        hDepHandle = NULL;
+    }
+
+    LW_SAFE_FREE_MEMORY(pszTemp);
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
 static
 PVOID
 LwSmWaitThread(
@@ -1084,7 +1231,7 @@ LwSmWaitThread(
     DWORD dwError = 0;
     LW_SERVICE_HANDLE hHandle = pData;
     LW_SERVICE_STATUS status = {0};
-
+    
     while (status.state != LW_SERVICE_STATE_STOPPED &&
            status.state != LW_SERVICE_STATE_DEAD)
     {
@@ -1127,10 +1274,10 @@ LwSmProxy(
 
     dwError = LwMbsToWc16s(pArgv[1], &pwszServiceName);
     BAIL_ON_ERROR(dwError);
-
+    
     dwError = LwSmAcquireServiceHandle(pwszServiceName, &hHandle);
     BAIL_ON_ERROR(dwError);
-
+    
     dwError = LwMapErrnoToLwError(pthread_create(
                                       &waitThread,
                                       NULL,
@@ -1140,7 +1287,7 @@ LwSmProxy(
 
     dwError = LwMapErrnoToLwError(pthread_detach(waitThread));
     BAIL_ON_ERROR(dwError);
-
+    
     if (!gState.bQuiet)
     {
         printf("Proxying for service: %s\n", pArgv[1]);
@@ -1196,7 +1343,7 @@ LwSmFindServiceWithPid(
     {
         dwError = LwSmAcquireServiceHandle(ppwszServiceNames[i], &hHandle);
         BAIL_ON_ERROR(dwError);
-
+    
         dwError = LwSmQueryServiceStatus(hHandle, &status);
         BAIL_ON_ERROR(dwError);
 
@@ -1213,7 +1360,9 @@ LwSmFindServiceWithPid(
         }
 
         LwSmReleaseServiceHandle(hHandle);
+        hHandle = NULL;
         LwSmFreeServiceInfo(pInfo);
+        pInfo = NULL;
     }
 
     dwError = LW_ERROR_INVALID_PARAMETER;
@@ -1260,17 +1409,19 @@ LwSmGdb(
 
     dwError = LwMbsToWc16s(pArgv[1], &pwszServiceName);
     BAIL_ON_ERROR(dwError);
-
+    
     dwError = LwSmAcquireServiceHandle(pwszServiceName, &hHandle);
     BAIL_ON_ERROR(dwError);
-
+    
     dwError = LwSmQueryServiceStatus(hHandle, &status);
     BAIL_ON_ERROR(dwError);
 
     if (status.state != LW_SERVICE_STATE_RUNNING)
     {
-        printf("Service is not running\n");
-        dwError = LW_ERROR_INVALID_PARAMETER;
+        dwError = LwSmStart(argc, pArgv);
+        BAIL_ON_ERROR(dwError);
+
+        dwError = LwSmQueryServiceStatus(hHandle, &status);
         BAIL_ON_ERROR(dwError);
     }
 
@@ -1281,7 +1432,9 @@ LwSmGdb(
         pInfo->type != LW_SERVICE_TYPE_LEGACY_EXECUTABLE)
     {
         LwSmReleaseServiceHandle(hHandle);
+        hHandle = NULL;
         LwSmFreeServiceInfo(pInfo);
+        pInfo = NULL;
 
         dwError = LwSmFindServiceWithPid(status.pid, &hHandle);
         BAIL_ON_ERROR(dwError);
@@ -1299,7 +1452,7 @@ LwSmGdb(
 
     dwError = LwAllocateStringPrintf(&pszPid, "%lu", (unsigned long) status.pid);
     BAIL_ON_ERROR(dwError);
-
+    
     if (execlp("gdb", "gdb", pszExecutablePath, pszPid, NULL) < 0)
     {
         dwError = LwMapErrnoToLwError(errno);
@@ -1348,7 +1501,7 @@ LwSmSetLog(
     else if (!strcasecmp(pArgv[1], "file"))
     {
         if (argc < 3)
-        {
+        {     
             dwError = LW_ERROR_INVALID_PARAMETER;
             BAIL_ON_ERROR(dwError);
         }
@@ -1480,6 +1633,7 @@ LwSmUsage(
     printf("Usage: %s [ options ... ] <command> ...\n\n", pArgv[0]);
     printf("Service commands:\n"
            "    list                       List all known services and their status\n"
+           "    autostart                  Start all services configured for autostart\n"
            "    start-only <service>       Start a service\n"
            "    start <service>            Start a service and all dependencies\n"
            "    stop-only <service>        Stop a service\n"
@@ -1491,6 +1645,8 @@ LwSmUsage(
            "    status <service>           Get the status of a service\n"
            "    gdb <service>              Attach gdb to the specified running service\n\n");
     printf("Maintenance commands:\n"
+           "    refresh                    Refresh service manager configuration\n"
+           "    shutdown                   Shutdown service manager\n"
            "    set-log <type> [<target>]  Set logging destination\n"
            "                               Valid types:\n"
            "                                   file (target is filename)\n"
@@ -1575,7 +1731,7 @@ main(
         }
         else if (!strcmp(pArgv[i], "refresh"))
         {
-            dwError = LwSmRefresh(argc-i, pArgv+i);
+            dwError = LwSmDoRefresh(argc-i, pArgv+i);
             goto error;
         }
         else if (!strcmp(pArgv[i], "restart"))
@@ -1613,6 +1769,16 @@ main(
             dwError = LwSmCmdGetLogLevel(argc-i, pArgv+i);
             goto error;
         }
+        else if (!strcmp(pArgv[i], "autostart"))
+        {
+            dwError = LwSmAutostart(argc-i, pArgv+i);
+            goto error;
+        }
+        else if (!strcmp(pArgv[i], "shutdown"))
+        {
+            dwError = LwSmShutdown();
+            goto error;
+        }
         else
         {
             dwError = LW_ERROR_INVALID_PARAMETER;
