@@ -42,6 +42,8 @@
 #include "config.h"
 #include <lwsm/lwsm.h>
 
+#define CONTROL_LOCK CACHEDIR "/.lwsmd-lock"
+
 #define BAIL_ON_ERROR(_e_)  \
     do                      \
     {                       \
@@ -73,7 +75,15 @@
         }                                       \
     } while (0)
 
+#ifndef BROKEN_ONCE_INIT
 #if defined(sun) || defined(_AIX)
+#define BROKEN_ONCE_INIT 1
+#else
+#define BROKEN_ONCE_INIT 0
+#endif
+#endif
+
+#if BROKEN_ONCE_INIT
 #  define ONCE_INIT {PTHREAD_ONCE_INIT}
 #else
 #  define ONCE_INIT PTHREAD_ONCE_INIT
@@ -119,6 +129,10 @@ typedef enum _SM_IPC_TAG
     SM_IPC_SET_LOG_LEVEL_RES,
     SM_IPC_GET_LOG_LEVEL_REQ,
     SM_IPC_GET_LOG_LEVEL_RES,
+    SM_IPC_REFRESH_REQ,
+    SM_IPC_REFRESH_RES,
+    SM_IPC_SHUTDOWN_REQ,
+    SM_IPC_SHUTDOWN_RES
 } SM_IPC_TAG;
 
 typedef struct _SM_IPC_WAIT_STATE_CHANGE_REQ
