@@ -131,18 +131,18 @@ typedef enum _LW_SERVICE_HOME
  */
 typedef enum _LW_SERVICE_INFO_MASK
 {
-    /**
+    /** 
      * @brief Update name
      * @hideinitializer
      */
     LW_SERVICE_INFO_MASK_NAME         = 0x01,
     /**
-     * @brief Update description
+     * @brief Update description 
      * @hideinitializer
      */
     LW_SERVICE_INFO_MASK_DESCRIPTION  = 0x02,
     /**
-     * @brief Update type
+     * @brief Update type 
      * @hideinitializer
      */
     LW_SERVICE_INFO_MASK_TYPE         = 0x04,
@@ -171,11 +171,16 @@ typedef enum _LW_SERVICE_INFO_MASK
      * @hideinitializer
      */
     LW_SERVICE_INFO_MASK_AUTOSTART    = 0x80,
+    /**
+     * @brief Update fd limit
+     * @hideinitializer
+     */
+    LW_SERVICE_INFO_MASK_FD_LIMIT     = 0x100,
     /**
      * @brief Update all flags
      * @hideinitializer
      */
-    LW_SERVICE_INFO_MASK_ALL          = 0xFF
+    LW_SERVICE_INFO_MASK_ALL          = 0x1FF
 } LW_SERVICE_INFO_MASK, *PLW_SERVICE_INFO_MASK;
 
 /**
@@ -202,6 +207,8 @@ typedef struct _LW_SERVICE_INFO
     LW_PWSTR* ppwszDependencies;
     /** @brief Is this service automatically started? */
     LW_BOOL bAutostart;
+    /** @brief Desired file descriptor limit for the process */
+    LW_DWORD dwFdLimit;
 } LW_SERVICE_INFO, *PLW_SERVICE_INFO;
 
 typedef const LW_SERVICE_INFO* PCLW_SERVICE_INFO;
@@ -529,6 +536,32 @@ LwSmSetLogLevel(
     LW_SM_LOG_LEVEL level
     );
 
+/**
+ * @brief Refresh service manager
+ *
+ * Causes the service manager to reread its own configuration.
+ * To refresh the configuration of a particular service, use
+ * #LwSmRefreshService().
+ *
+ * @retval LW_ERROR_SUCCESS success
+ */
+DWORD
+LwSmRefresh(
+    VOID
+    );
+
+/**
+ * @brief Shutdown
+ *
+ * Causes the service manager to shut down completely.
+ *
+ * @retval LW_ERROR_SUCCESS success
+ */
+DWORD
+LwSmShutdown(
+    VOID
+    );
+
 VOID
 LwSmFreeLogTarget(
     PSTR pszTarget
@@ -548,6 +581,7 @@ LwSmFreeLogTarget(
 
 #define LW_SERVICE_LOADER_INTERFACE_VERSION 1
 
+struct _LW_SERVICE_LOADER_PLUGIN;
 typedef struct _LW_SERVICE_OBJECT LW_SERVICE_OBJECT, *PLW_SERVICE_OBJECT;
 
 typedef struct _LW_SERVICE_LOADER_VTBL
@@ -558,6 +592,7 @@ typedef struct _LW_SERVICE_LOADER_VTBL
     DWORD (*pfnRefresh)(PLW_SERVICE_OBJECT pObject);
     DWORD (*pfnConstruct)(PLW_SERVICE_OBJECT pObject, PCLW_SERVICE_INFO pInfo, PVOID* ppData);
     VOID  (*pfnDestruct)(PLW_SERVICE_OBJECT pObject);
+    VOID  (*pfnShutdown)(const struct _LW_SERVICE_LOADER_PLUGIN* pPlugin);
 } LW_SERVICE_LOADER_VTBL, *PLW_SERVICE_LOADER_VTBL;
 
 typedef const struct _LW_SERVICE_LOADER_PLUGIN
