@@ -42,6 +42,7 @@
 #include "common.h"
 
 #include <pthread.h>
+#include <time.h>
 
 struct _LW_SERVICE_OBJECT
 {
@@ -74,6 +75,10 @@ typedef struct _SM_TABLE_ENTRY
     BOOL volatile bDepsMarked;
     /* Has pInfo been changed since the service was last constructed? */
     BOOL volatile bDirty;
+    /* How many automatic restarts have we attempted? */
+    DWORD RestartAttempts;
+    /* When did we begin the last restart period? */
+    time_t LastRestartPeriod;
     /* Lock controlling access to entry */
     pthread_mutex_t lock;
     pthread_mutex_t* pLock;
@@ -297,6 +302,16 @@ LwSmTableGetEntryReverseDependencyClosure(
     PWSTR** pppwszServiceList
     );
 
+DWORD
+LwSmTableInit(
+    VOID
+    );
+
+VOID
+LwSmTableShutdown(
+    VOID
+    );
+
 DWORD
 LwSmRegistryEnumServices(
     HANDLE hReg,
@@ -317,7 +332,7 @@ LwSmBootstrap(
     );
 
 DWORD
-LwSmAutostartServices(
+LwSmPopulateTable(
     VOID
     );
 
@@ -326,6 +341,11 @@ LwSmLoaderInitialize(
     PSM_LOADER_CALLS pCalls
     );
 
+VOID
+LwSmLoaderShutdown(
+    VOID
+    );
+
 DWORD
 LwSmLoaderGetVtbl(
     PCWSTR pwszLoaderName,
