@@ -38,6 +38,19 @@
 
 #include "includes.h"
 
+#ifdef LW_SERVICE_CONTAINERS
+static SM_BOOTSTRAP_SERVICE gRegistryService =
+{
+    .pszName = "lwreg",
+    .type = LW_SERVICE_TYPE_MODULE,
+    .pszPath = LIBDIR "/lw-svcm/lwreg" MOD_EXT,
+    .ppszArgs =
+    {
+        "--syslog",
+        NULL
+    }
+};
+#else
 static SM_BOOTSTRAP_SERVICE gRegistryService =
 {
     .pszName = "lwreg",
@@ -50,6 +63,7 @@ static SM_BOOTSTRAP_SERVICE gRegistryService =
         NULL
     }
 };
+#endif
 
 static PSM_BOOTSTRAP_SERVICE gBootstrapServices[] =
 {
@@ -70,6 +84,8 @@ LwSmBootstrap(
     size_t j = 0;
     size_t len = 0;
 
+    SM_LOG_VERBOSE("Bootstrapping");
+
     for (i = 0; gBootstrapServices[i]; i++)
     {
         pService = gBootstrapServices[i];
@@ -103,12 +119,12 @@ LwSmBootstrap(
             BAIL_ON_ERROR(dwError);
         }
 
-	/* Create empty Env list. */
+        /* Create empty Env list. */
         dwError = LwAllocateMemory(
             1 * sizeof(*pInfo->ppwszEnv), OUT_PPVOID(&pInfo->ppwszEnv));
         BAIL_ON_ERROR(dwError);
 
-	pInfo->ppwszEnv[0] = NULL;
+        pInfo->ppwszEnv[0] = NULL;
 
         dwError = LwAllocateMemory(
             1 * sizeof(*pInfo->ppwszDependencies),
