@@ -50,35 +50,36 @@ typedef struct _SM_PROCESS_TABLE
 {
     pthread_mutex_t lock;
     pthread_mutex_t* pLock;
-    SM_LINK execs;
-    pthread_t thread;
-    BOOLEAN bThreadStarted;
-    BOOLEAN bThreadStop;
+    PLW_THREAD_POOL pPool;
 } SM_PROCESS_TABLE;
 
 typedef struct _SM_EXECUTABLE
 {
+    PLW_TASK pTask;
     LW_SERVICE_STATE state;
     pid_t pid;
+    int notifyFd;
     LW_SERVICE_TYPE type;
     PWSTR pwszPath;
     PWSTR* ppwszArgs;
     PWSTR* ppwszEnv;
+    DWORD dwFdLimit;
     PLW_SERVICE_OBJECT pObject;
-    SM_LINK link;
-    SM_LINK threadLink;
 } SM_EXECUTABLE, *PSM_EXECUTABLE;
 
 static SM_PROCESS_TABLE gProcTable =
 {
     .lock = PTHREAD_MUTEX_INITIALIZER,
     .pLock = &gProcTable.lock,
-    .execs = {&gProcTable.execs, &gProcTable.execs},
-    .thread = (pthread_t) -1,
-    .bThreadStarted = FALSE,
-    .bThreadStop = FALSE
+    .pPool = NULL
 };
 
+static
+VOID
+LwSmExecutableFree(
+    PSM_EXECUTABLE pExec
+    );
+
 static
 DWORD
 LwSmExecProgram(
@@ -87,9 +88,13 @@ LwSmExecProgram(
     );
 
 static
-PVOID
-LwSmExecutableThread(
-    PVOID pData
+VOID
+LwSmExecutableTask(
+    PLW_TASK pTask,
+    PVOID pContext,
+    LW_TASK_EVENT_MASK WakeMask,
+    PLW_TASK_EVENT_MASK pWaitMask,
+    PLONG64 pllTime
     );
 
 static
@@ -102,8 +107,6 @@ LwSmForkProcess(
     pid_t pid = -1;
     struct timespec ts = {1, 0};
     int notifyPipe[2] = {-1, -1};
-    char c = 0;
-    int ret = 0;
 
     if (pExec->type == LW_SERVICE_TYPE_EXECUTABLE)
     {
@@ -141,19 +144,14 @@ LwSmForkProcess(
             close(notifyPipe[1]);
             notifyPipe[1] = -1;
 
-            do
-            {
-                ret = read(notifyPipe[0], &c, sizeof(c));
-            } while (ret < 0 && errno == EINTR);
-
-            if (ret != sizeof(c) || c != 0)
-            {
-                dwError = LW_ERROR_SERVICE_UNRESPONSIVE;
-                BAIL_ON_ERROR(dwError);
-            }
+            /* Save read side of pipe as notification fd */
+            pExec->notifyFd = notifyPipe[0];
+            notifyPipe[0] = -1;
         }
         else
         {
+            pExec->notifyFd = -1;
+
             /* Wait for process to start up */
             if (nanosleep(&ts, NULL) != 0)
             {
@@ -161,8 +159,6 @@ LwSmForkProcess(
                 BAIL_ON_ERROR(dwError);
             }
         }
-
-        pExec->state = LW_SERVICE_STATE_RUNNING;
     }
 
 cleanup:
@@ -207,17 +203,25 @@ LwSmExecutableStart(
 
     pExec->pid = -1;
     pExec->state = LW_SERVICE_STATE_STARTING;
-
+        
     LwSmNotifyServiceObjectStateChange(pObject, pExec->state);
 
-    /* Take an additional reference to the table entry
-       because our child monitoring thread will need it */
-    LwSmRetainServiceObject(pObject);
-
-    /* Wake up the process manager thread to start the program */
-    dwError = LwMapErrnoToLwError(pthread_kill(gProcTable.thread, SIGCHLD));
+    dwError = LwSmForkProcess(pExec);
     BAIL_ON_ERROR(dwError);
 
+    /* Create a task to track the process */
+    dwError = LwNtStatusToWin32Error(
+        LwRtlCreateTask(
+            gProcTable.pPool,
+            &pExec->pTask,
+            NULL,
+            LwSmExecutableTask,
+            pExec));
+    BAIL_ON_ERROR(dwError);
+
+    /* Wake up the task */
+    LwRtlWakeTask(pExec->pTask);
+        
 cleanup:
 
     UNLOCK(bLocked, &gProcTable.lock);
@@ -229,6 +233,27 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+LwSmSetLimits(
+    PSM_EXECUTABLE pExec
+    )
+{
+    DWORD dwError = 0;
+    struct rlimit limit = {0};
+
+    if (pExec->dwFdLimit)
+    {
+        limit.rlim_cur = pExec->dwFdLimit;
+        limit.rlim_max = pExec->dwFdLimit;
+
+        /* Ignore errors */
+        (void) setrlimit(RLIMIT_NOFILE, &limit);
+    }
+
+    return dwError;
+}
+
 static
 DWORD
 LwSmExecProgram(
@@ -248,9 +273,19 @@ LwSmExecProgram(
 
     sigemptyset(&set);
 
+    /* Put ourselves in our own process group to dissociate from
+       the parent's controlling terminal, if any */
+    dwError = LwMapErrnoToLwError(setpgid(getpid(), getpid()));
+    BAIL_ON_ERROR(dwError);
+    
+    /* Reset the signal mask */
     dwError = LwMapErrnoToLwError(pthread_sigmask(SIG_SETMASK, &set, NULL));
     BAIL_ON_ERROR(dwError);
 
+    /* Set any applicable limits */
+    dwError = LwSmSetLimits(pExec);
+    BAIL_ON_ERROR(dwError);
+
     dwError = LwWc16sToMbs(pExec->pwszPath, &pszPath);
     BAIL_ON_ERROR(dwError);
 
@@ -330,22 +365,33 @@ LwSmExecutableStop(
         BAIL_ON_ERROR(dwError);
         break;
     case LW_SERVICE_STATE_RUNNING:
+        pExec->state = LW_SERVICE_STATE_STOPPING;       
+        LwSmNotifyServiceObjectStateChange(pObject, pExec->state);
+        
         if (kill(pExec->pid, SIGTERM) < 0)
         {
             dwError = LwMapErrnoToLwError(errno);
             BAIL_ON_ERROR(dwError);
         }
-        pExec->state = LW_SERVICE_STATE_STOPPING;
-        LwSmNotifyServiceObjectStateChange(pObject, pExec->state);
-
-        /* The background thread will notice when the
+        
+        /* The associated task will notice when the
            child process finally exits and update the
-           status to LW_SERVICE_STOPPED */
+           status to LW_SERVICE_STOPPED.  We can release the
+           task handle now since we will no longer need
+           to use it */
+        LwRtlReleaseTask(&pExec->pTask);
         break;
     case LW_SERVICE_STATE_DEAD:
         /* Go directly to stopped state */
         pExec->state = LW_SERVICE_STATE_STOPPED;
         LwSmNotifyServiceObjectStateChange(pObject, pExec->state);
+        if (pExec->pTask)
+        {
+            /* Cancel task in case it's still alive for some reason */
+            LwRtlCancelTask(pExec->pTask);
+            /* Release task handle */
+            LwRtlReleaseTask(&pExec->pTask);
+        }
         break;
     }
 
@@ -433,6 +479,8 @@ LwSmExecutableConstruct(
     dwError = LwAllocateMemory(sizeof(*pExec), OUT_PPVOID(&pExec));
     BAIL_ON_ERROR(dwError);
 
+    pExec->notifyFd = -1;
+
     dwError = LwSmCopyString(pInfo->pwszPath, &pExec->pwszPath);
     BAIL_ON_ERROR(dwError);
 
@@ -446,27 +494,12 @@ LwSmExecutableConstruct(
     pExec->type = pInfo->type;
     pExec->state = LW_SERVICE_STATE_STOPPED;
     pExec->pObject = pObject;
-    LwSmLinkInit(&pExec->link);
-    LwSmLinkInit(&pExec->threadLink);
+    pExec->dwFdLimit = pInfo->dwFdLimit;
 
     *ppData = pExec;
 
     LOCK(bLocked, &gProcTable.lock);
 
-    LwSmLinkInsertBefore(&gProcTable.execs, &pExec->link);
-
-    if (!gProcTable.bThreadStarted)
-    {
-        dwError = LwMapErrnoToLwError(pthread_create(
-                                          &gProcTable.thread,
-                                          NULL,
-                                          LwSmExecutableThread,
-                                          NULL));
-        BAIL_ON_ERROR(dwError);
-
-        gProcTable.bThreadStarted = TRUE;
-    }
-
 cleanup:
 
     UNLOCK(bLocked, &gProcTable.lock);
@@ -475,117 +508,175 @@ cleanup:
 
 error:
 
+    if (pExec)
+    {
+        LwSmExecutableFree(pExec);
+    }
+
     goto cleanup;
 }
 
+static
+VOID
+LwSmExecutableFree(
+    PSM_EXECUTABLE pExec
+    )
+{
+    if (pExec->notifyFd >= 0)
+    {
+        close(pExec->notifyFd);
+    }
+
+    if (pExec->pTask)
+    {
+        LwRtlCancelTask(pExec->pTask);
+        LwRtlReleaseTask(&pExec->pTask);
+    }
+
+    LW_SAFE_FREE_MEMORY(pExec->pwszPath);
+    LwSmFreeStringList(pExec->ppwszArgs);
+    LwSmFreeStringList(pExec->ppwszEnv);
+    LwFreeMemory(pExec);
+}
+
+
 static
 VOID
 LwSmExecutableDestruct(
     PLW_SERVICE_OBJECT pObject
     )
 {
-    return;
+    LwSmExecutableFree(LwSmGetServiceObjectData(pObject));
 }
 
 static
-PVOID
-LwSmExecutableThread(
-    PVOID pData
+VOID
+LwSmExecutableTask(
+    PLW_TASK pTask,
+    PVOID pContext,
+    LW_TASK_EVENT_MASK WakeMask,
+    PLW_TASK_EVENT_MASK pWaitMask,
+    PLONG64 pllTime
     )
 {
     BOOLEAN bLocked = FALSE;
-    PSM_LINK pLink = NULL;
-    PSM_LINK pNext = NULL;
-    PSM_EXECUTABLE pExec = NULL;
-    PLW_SERVICE_OBJECT pObject = NULL;
+    PSM_EXECUTABLE pExec = pContext;
     pid_t pid = -1;
-    int status = 0;
-    SM_LINK changed;
-    sigset_t set;
-    int sig = 0;
+    NTSTATUS status = STATUS_SUCCESS;
+    char c = 0;
+    int ret = 0;
+    siginfo_t info;
 
-    sigemptyset(&set);
-    sigaddset(&set, SIGCHLD);
-    
-    LwSmLinkInit(&changed);
-    
-    for (;;)
+    LOCK(bLocked, &gProcTable.lock);
+
+    if (WakeMask & LW_TASK_EVENT_INIT)
     {
-        LwSmLinkRemove(&changed);
-        
-        LOCK(bLocked, &gProcTable.lock);
-        
-        if (gProcTable.bThreadStop)
+        status = LwRtlSetTaskUnixSignal(pTask, SIGCHLD, TRUE);
+        BAIL_ON_ERROR(status);
+
+        *pWaitMask = LW_TASK_EVENT_UNIX_SIGNAL;
+
+        if (pExec->notifyFd >= 0)
         {
-            break;
+            status = LwRtlSetTaskFd(pTask, pExec->notifyFd, LW_TASK_EVENT_FD_READABLE);
+            BAIL_ON_ERROR(status);
+
+            *pWaitMask |= LW_TASK_EVENT_FD_READABLE;
         }
-        
-        pLink = NULL;
-
-        while ((pLink = SM_LINK_ITERATE(&gProcTable.execs, pLink)))
+    }
+    else if (WakeMask & LW_TASK_EVENT_CANCEL)
+    {
+        *pWaitMask = LW_TASK_EVENT_COMPLETE;
+        goto cleanup;
+    }
+    else if (WakeMask & LW_TASK_EVENT_FD_READABLE)
+    {
+        do
         {
-            pExec = STRUCT_FROM_MEMBER(pLink, SM_EXECUTABLE, link);
+            ret = read(pExec->notifyFd, &c, sizeof(c));
+        } while (ret < 0 && errno == EINTR);
 
-            if (pExec->pid != -1)
+        if (ret != sizeof(c) || c != 0)
+        {
+            goto error;
+        }
+
+        status = LwRtlSetTaskFd(pTask, pExec->notifyFd, 0);
+        BAIL_ON_ERROR(status);
+
+        close(pExec->notifyFd);
+        pExec->notifyFd = -1;
+
+        pExec->state = LW_SERVICE_STATE_RUNNING;
+        LwSmNotifyServiceObjectStateChange(pExec->pObject, pExec->state);
+
+        *pWaitMask &= ~LW_TASK_EVENT_FD_READABLE;
+    }
+    else if (WakeMask & LW_TASK_EVENT_UNIX_SIGNAL)
+    {
+        while (LwRtlNextTaskUnixSignal(pTask, &info))
+        {
+            if (info.si_signo == SIGCHLD)
             {
-                pid = waitpid(pExec->pid, &status, WNOHANG);
+                pid = waitpid(pExec->pid, &ret, WNOHANG);
                 
                 if (pid == pExec->pid)
                 {
+                    pExec->pid = -1;
+
                     switch (pExec->state)
                     {
                     case LW_SERVICE_STATE_STOPPING:
                         pExec->state = LW_SERVICE_STATE_STOPPED;
-                        break;
+                        LwSmNotifyServiceObjectStateChange(pExec->pObject, pExec->state);
+                        *pWaitMask = LW_TASK_EVENT_COMPLETE;
+                        goto cleanup;
                     default:
-                        pExec->state = LW_SERVICE_STATE_DEAD;
-                        break;
+                        /* Uh-oh */
+                        goto error;
                     }
-                    
-                    pExec->pid = -1;
-                    
-                    LwSmLinkRemove(&pExec->threadLink);
-                    LwSmLinkInsertBefore(&changed, &pExec->threadLink);
-                }
-            }
-            else
-            {
-                switch (pExec->state)
-                {
-                case LW_SERVICE_STATE_STARTING:
-                    if (LwSmForkProcess(pExec) == 0)
-                    {
-                          LwSmLinkRemove(&pExec->threadLink);
-                          LwSmLinkInsertBefore(&changed, &pExec->threadLink);
-                    }
-                    break;
-                default:
-                    break;
                 }
             }
         }
-
-        UNLOCK(bLocked, &gProcTable.lock);
-
-        for (pLink = changed.pNext; pLink != &changed; pLink = pNext)
-        {
-            pNext = pLink->pNext;
-            pExec = STRUCT_FROM_MEMBER(pLink, SM_EXECUTABLE, threadLink);
-            pObject = pExec->pObject;
-            
-            LwSmLinkRemove(pLink);
-            
-            LwSmNotifyServiceObjectStateChange(pObject, pExec->state);
-            LwSmReleaseServiceObject(pObject);
-        }
-
-        do
-        {
-            sigwait(&set, &sig);
-        } while (sig != SIGCHLD);
     }
 
-    return NULL;
+cleanup:
+
+    UNLOCK(bLocked, &gProcTable.lock);
+    
+    return;
+
+error:
+
+    pExec->state = LW_SERVICE_STATE_DEAD;
+    LwSmNotifyServiceObjectStateChange(pExec->pObject, pExec->state);
+  
+    if (pExec->pid >= 0)
+    {
+        kill(pExec->pid, SIGKILL);
+        waitpid(pExec->pid, &ret, 0);
+        pExec->pid = -1;
+    }
+
+    if (pExec->notifyFd >= 0)
+    {
+        close(pExec->notifyFd);
+        pExec->notifyFd = -1;
+    }
+
+    *pWaitMask = LW_TASK_EVENT_COMPLETE;
+
+    goto cleanup;
+}
+
+static
+VOID
+LwSmExecutableShutdown(
+    PLW_SERVICE_LOADER_PLUGIN pPlugin
+    )
+{
+    LwRtlFreeThreadPool(&gProcTable.pPool);
+    pthread_mutex_destroy(gProcTable.pLock);
 }
 
 static
@@ -596,7 +687,8 @@ LW_SERVICE_LOADER_VTBL gExecutableVtbl =
     .pfnGetStatus = LwSmExecutableGetStatus,
     .pfnRefresh = LwSmExecutableRefresh,
     .pfnConstruct = LwSmExecutableConstruct,
-    .pfnDestruct = LwSmExecutableDestruct
+    .pfnDestruct = LwSmExecutableDestruct,
+    .pfnShutdown = LwSmExecutableShutdown
 };
 
 static
@@ -615,7 +707,14 @@ ServiceLoaderInit(
     PLW_SERVICE_LOADER_PLUGIN* ppPlugin
     )
 {
-    *ppPlugin = &gPlugin;
+    DWORD dwError = 0;
 
-    return LW_ERROR_SUCCESS;
+    dwError = LwNtStatusToWin32Error(LwRtlCreateThreadPool(&gProcTable.pPool, NULL));
+    BAIL_ON_ERROR(dwError);
+
+    *ppPlugin = &gPlugin;
+    
+error:
+
+    return dwError;
 }
