@@ -140,6 +140,11 @@ LwSmLoaderInitialize(
 
 error:
 
+    if (pLoaderDir)
+    {
+        closedir(pLoaderDir);
+    }
+    
     LW_SAFE_FREE_MEMORY(pszPath);
 
     return dwError;
@@ -181,9 +186,23 @@ LwSmLoaderLoadPlugin(
 
     LwSmLinkInsertBefore(&gPluginList, &pPlugin->link);
 
-error:
+cleanup:
 
     return dwError;
+
+error:
+
+    if (pPlugin)
+    {
+        if (pPlugin->hLibrary)
+        {
+            dlclose(pPlugin->hLibrary);
+        }
+
+        LwFreeMemory(pPlugin);
+    }
+
+    goto cleanup;
 }
 
 DWORD
@@ -216,5 +235,37 @@ LwSmLoaderGetVtbl(
 
 error:
 
+    LW_SAFE_FREE_MEMORY(pszLoaderName);
+
     return dwError;
 }
+
+VOID
+LwSmLoaderShutdown(
+    VOID
+    )
+{
+    PSM_LINK pLink = NULL;
+    PSM_LINK pNext = NULL;
+    PSM_PLUGIN pPlugin = NULL;
+
+    for (pLink = LwSmLinkBegin(&gPluginList);
+         LwSmLinkValid(&gPluginList, pLink);
+         pLink = pNext)
+    {
+        pNext = LwSmLinkNext(pLink);
+        pPlugin = STRUCT_FROM_MEMBER(pLink, SM_PLUGIN, link);
+
+        if (pPlugin->pPlugin && pPlugin->pPlugin->pVtbl->pfnShutdown)
+        {
+            pPlugin->pPlugin->pVtbl->pfnShutdown(pPlugin->pPlugin);
+        }
+
+        if (pPlugin->hLibrary)
+        {
+            dlclose(pPlugin->hLibrary);
+        }
+
+        LwFreeMemory(pPlugin);
+    }
+}
