@@ -40,17 +40,20 @@
 
 static struct
 {
+    PLW_THREAD_POOL pPool;
     LWMsgContext* pIpcContext;
     LWMsgProtocol* pIpcProtocol;
     LWMsgServer* pIpcServer;
     BOOLEAN bStartAsDaemon;
+    BOOLEAN bNotified;
     int notifyPipe[2];
     LW_SM_LOG_LEVEL logLevel;
     PCSTR pszLogFilePath;
     BOOLEAN bSyslog;
-    BOOLEAN bDisableAutostart;
+    int ControlLock;
 } gState = 
 {
+    .pPool = NULL,
     .pIpcContext = NULL,
     .pIpcProtocol = NULL,
     .pIpcServer = NULL,
@@ -59,7 +62,7 @@ static struct
     .logLevel = LW_SM_LOG_LEVEL_WARNING,
     .pszLogFilePath = NULL,
     .bSyslog = FALSE,
-    .bDisableAutostart = FALSE
+    .ControlLock = -1
 };
 
 static
@@ -77,13 +80,7 @@ LwSmDaemonize(
 
 static
 DWORD
-LwSmConfigureSignals(
-    VOID
-    );
-
-static
-DWORD
-LwSmWaitSignals(
+LwSmMain(
     VOID
     );
 
@@ -105,12 +102,6 @@ LwSmStopIpcServer(
     VOID
     );
 
-static
-DWORD
-LwSmPopulateTable(
-    VOID
-    );
-
 static
 DWORD
 LwSmShutdownServices(
@@ -129,6 +120,12 @@ LwSmNotify(
     int status
     );
 
+static
+DWORD
+LwSmControlLock(
+    VOID
+    );
+
 int
 main(
     int argc,
@@ -136,16 +133,15 @@ main(
     )
 {
     DWORD dwError = 0;
-    BOOLEAN bNotified = FALSE;
 
     /* Parse command line */
     dwError = LwSmParseArguments(argc, ppszArgv);
     BAIL_ON_ERROR(dwError);
 
-    /* Block signals */
-    dwError = LwSmConfigureSignals();
+    /* Block all signals */
+    dwError = LwNtStatusToWin32Error(LwRtlBlockSignals());
     BAIL_ON_ERROR(dwError);
-    
+
     /* Fork into background if running as a daemon */
     if (gState.bStartAsDaemon)
     {
@@ -153,6 +149,16 @@ main(
         BAIL_ON_ERROR(dwError);
     }
 
+    dwError = LwSmControlLock();
+    BAIL_ON_ERROR(dwError);
+
+    dwError = LWNetExtendEnvironmentForKrb5Affinity(FALSE);
+    BAIL_ON_ERROR(dwError);
+
+    /* Mac OS X - avoid potential circular calls into directory services */
+    dwError = LwDsCacheAddPidException(getpid());
+    BAIL_ON_ERROR(dwError);
+
     /* Set up logging */
     dwError = LwSmConfigureLogging(ppszArgv[0]);
     BAIL_ON_ERROR(dwError);
@@ -161,51 +167,12 @@ main(
     dwError = LwSmLoaderInitialize(&gTableCalls);
     BAIL_ON_ERROR(dwError);
 
-    /* Bootstrap ourselves by adding and starting any
-       services we need to run (e.g. registry) */
-    dwError = LwSmBootstrap();
+    /* Initialize the service table subsystem */
+    dwError = LwSmTableInit();
     BAIL_ON_ERROR(dwError);
 
-    /* Read configuration and populate service table */
-    dwError = LwSmPopulateTable();
-    BAIL_ON_ERROR(dwError);
-
-    /* Start the 'Autostart' services */
-    if (!gState.bDisableAutostart)
-    {
-        dwError = LwSmAutostartServices();
-        BAIL_ON_ERROR(dwError);
-    }
-
-    /* Start IPC server */
-    dwError = LwSmStartIpcServer();
-    BAIL_ON_ERROR(dwError);
-
-    /* If we are starting as a daemon, indicate that we
-       are ready to the parent process.  This ensures that
-       the parent does not exit until we are actually accepting
-       IPC connections */
-    if (gState.bStartAsDaemon)
-    {
-        dwError = LwSmNotify(0);
-        BAIL_ON_ERROR(dwError);
-        bNotified = TRUE;
-    }
-
-    /* Loop waiting for signals */
-    dwError = LwSmWaitSignals();
-    BAIL_ON_ERROR(dwError);
-
-    /* Stop IPC server */
-    dwError = LwSmStopIpcServer();
-    BAIL_ON_ERROR(dwError);
-
-    /* Shut down all running services as we exit */
-    dwError = LwSmShutdownServices();
-    BAIL_ON_ERROR(dwError);
-
-    /* Shut down logging */
-    dwError = LwSmSetLogger(NULL, NULL);
+    /* Enter main loop */
+    dwError = LwSmMain();
     BAIL_ON_ERROR(dwError);
 
 error:
@@ -213,11 +180,23 @@ error:
     /* If we are starting as a daemon and have not
        notified the parent process yet, notify it
        of an error now */
-    if (gState.bStartAsDaemon && !bNotified)
+    if (gState.bStartAsDaemon && !gState.bNotified)
     {
         LwSmNotify(1);
     }
 
+    /* Shut down service table */
+    LwSmTableShutdown();
+
+    /* Shut down loaders */
+    LwSmLoaderShutdown();
+
+    /* Shut down logging */
+    LwSmSetLogger(NULL, NULL);
+
+    /* Remove DS cache exception */
+    LwDsCacheRemovePidException(getpid());
+
     return dwError ? 1 : 0;
 }
 
@@ -244,20 +223,20 @@ LwSmParseArguments(
             gState.bSyslog = TRUE;
             gState.logLevel = LW_SM_LOG_LEVEL_INFO;
         }
-        else if (!strcmp(ppszArgv[i], "--log-level"))
+        else if (!strcmp(ppszArgv[i], "--loglevel"))
         {
             if (++i >= argc)
             {
                 dwError = LW_ERROR_INVALID_PARAMETER;
                 BAIL_ON_ERROR(dwError);
             }
-
+            
             dwError = LwSmLogLevelNameToLogLevel(
                 ppszArgv[i],
                 &gState.logLevel);
             BAIL_ON_ERROR(dwError);
         }
-        else if (!strcmp(ppszArgv[i], "--log-file"))
+        else if (!strcmp(ppszArgv[i], "--logfile"))
         {
             if (++i >= argc)
             {
@@ -267,10 +246,55 @@ LwSmParseArguments(
 
             gState.pszLogFilePath = ppszArgv[i];
         }
-        else if (!strcmp(ppszArgv[i], "--disable-autostart"))
+    }
+
+error:
+
+    return dwError;
+}
+
+static
+DWORD
+LwSmControlLock(
+    VOID
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+    struct flock lock = {0};
+
+    lock.l_type = F_WRLCK;
+    lock.l_whence = SEEK_SET;
+    lock.l_start = 0;
+    lock.l_len = 0;
+    lock.l_pid = getpid();
+
+    if ((gState.ControlLock = open(CONTROL_LOCK, O_WRONLY | O_CREAT | O_TRUNC,
+                    0200)) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_ERROR(dwError);
+    }
+
+    if (fcntl(gState.ControlLock, F_SETFD, FD_CLOEXEC) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_ERROR(dwError);
+    }
+
+    if (fcntl(gState.ControlLock, F_SETLK, &lock) < 0)
+    {
+        switch(errno)
         {
-            gState.bDisableAutostart = TRUE;
+        case EACCES:
+        case EAGAIN:
+            dwError = ERROR_SERVICE_ALREADY_RUNNING;
+            break;
+        default:
+            dwError = LwErrnoToWin32Error(errno);
+            break;
         }
+
+        BAIL_ON_ERROR(dwError);
     }
 
 error:
@@ -425,145 +449,208 @@ error:
 
 static
 VOID
-LwSmHandleSigint(
-    int sig
+Startup(
+    PVOID pUnused
     )
 {
-    raise(SIGTERM);
+    DWORD dwError = 0;
+
+    SM_LOG_INFO("Likewise Service Manager starting up");
+
+    /* Bootstrap ourselves by adding and starting any
+       services we need to run (e.g. registry) */
+    dwError = LwSmBootstrap();
+    BAIL_ON_ERROR(dwError);
+    
+    /* Read configuration and populate service table */
+    dwError = LwSmPopulateTable();
+    BAIL_ON_ERROR(dwError);
+    
+    /* Start IPC server */
+    dwError = LwSmStartIpcServer();
+    BAIL_ON_ERROR(dwError);
+
+    /* If we are starting as a daemon, indicate that we
+       are ready to the parent process.  This ensures that
+       the parent does not exit until we are actually accepting
+       IPC connections */
+    if (gState.bStartAsDaemon)
+    {
+        dwError = LwSmNotify(0);
+        BAIL_ON_ERROR(dwError);
+        gState.bNotified = TRUE;
+    }
+
+    SM_LOG_INFO("Likewise Service Manager startup complete");
+
+    return;
+
+error:
+
+    LwRtlExitMain(STATUS_UNSUCCESSFUL);
 }
 
 static
 VOID
-LwSmHandleSig(
-    int sig
-    )
-{
-    return;
-}
-
-static
-DWORD
-LwSmConfigureSignals(
-    VOID
+RefreshConfig(
+    PVOID pUnused
     )
 {
     DWORD dwError = 0;
-    sigset_t set;
-    static int blockSignals[] =
-    {
-        SIGTERM,
-        SIGCHLD,
-        SIGHUP,
-        SIGPIPE,
-        -1
-    };
-    int i = 0;
-    struct sigaction action;
 
-    memset(&action, 0, sizeof(action));
+    dwError = LwSmPopulateTable();
+    BAIL_ON_ERROR(dwError);
 
-    if (sigemptyset(&set) < 0)
+    return;
+
+error:
+
+    LwRtlExitMain(STATUS_UNSUCCESSFUL);
+}
+
+static
+VOID
+Shutdown(
+    PVOID pUnused
+    )
+{
+    DWORD dwError = 0;
+
+    /* Shut down all running services  */
+    dwError = LwSmShutdownServices();
+    BAIL_ON_ERROR(dwError);
+    
+    /* Stop IPC server */
+    dwError = LwSmStopIpcServer();
+    BAIL_ON_ERROR(dwError);
+
+    /* Exit from main loop */
+    LwRtlExitMain(STATUS_SUCCESS);
+
+    return;
+
+error:
+    
+    LwRtlExitMain(STATUS_UNSUCCESSFUL);
+}
+ 
+
+static
+VOID
+MainTask(
+    PLW_TASK pTask,
+    PVOID pContext,
+    LW_TASK_EVENT_MASK WakeMask,
+    PLW_TASK_EVENT_MASK pWaitMask,
+    PLONG64 pllTime
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    siginfo_t info;
+
+    if (WakeMask & LW_TASK_EVENT_CANCEL)
     {
-        dwError = LwMapErrnoToLwError(errno);
-        BAIL_ON_ERROR(dwError);
+        *pWaitMask = LW_TASK_EVENT_COMPLETE;
+        goto cleanup;
     }
-
-    for (i = 0; blockSignals[i] != -1; i++)
+    else if (WakeMask & LW_TASK_EVENT_INIT)
     {
-        if (sigaddset(&set, blockSignals[i]) < 0)
-        {
-            dwError = LwMapErrnoToLwError(errno);
-            BAIL_ON_ERROR(dwError); 
-        }
+        status = LwRtlSetTaskUnixSignal(pTask, SIGTERM, TRUE);
+        BAIL_ON_ERROR(status);
 
-	action.sa_handler = LwSmHandleSig;
-	action.sa_flags = 0;
+        status = LwRtlSetTaskUnixSignal(pTask, SIGINT, TRUE);
+        BAIL_ON_ERROR(status);
 
-	if (sigaction(blockSignals[i], &action, NULL) < 0)
-        {
-             dwError = LwMapErrnoToLwError(errno);
-             BAIL_ON_ERROR(dwError);
-        }
+        status = LwRtlSetTaskUnixSignal(pTask, SIGHUP, TRUE);
+        BAIL_ON_ERROR(status);
+        
+        status = LwRtlQueueWorkItem(gState.pPool, Startup, NULL, 0);
+        BAIL_ON_ERROR(status);
+
+        *pWaitMask = LW_TASK_EVENT_UNIX_SIGNAL;
     }
-
-    dwError = LwMapErrnoToLwError(pthread_sigmask(SIG_SETMASK, &set, NULL));
-    BAIL_ON_ERROR(dwError); 
-
-    action.sa_handler = LwSmHandleSigint;
-    action.sa_flags = 0;
-
-    if (sigaction(SIGINT, &action, NULL) < 0)
+    else if (WakeMask & LW_TASK_EVENT_UNIX_SIGNAL)
     {
-        dwError = LwMapErrnoToLwError(errno);
-        BAIL_ON_ERROR(dwError);
+        while (LwRtlNextTaskUnixSignal(pTask, &info))
+        {
+            switch(info.si_signo)
+            {
+            case SIGTERM:
+            case SIGINT:
+                SM_LOG_VERBOSE(info.si_signo == SIGINT ?
+                    "Shutting down on SIGINT" :
+                    "Shutting down on SIGTERM");
+                /* Shutting down stops all running services, which is a blocking operation */
+                status = LwRtlQueueWorkItem(gState.pPool, Shutdown, NULL, 0);
+                BAIL_ON_ERROR(status);
+                *pWaitMask = LW_TASK_EVENT_COMPLETE;
+                goto cleanup;
+            case SIGHUP:
+                SM_LOG_VERBOSE("Refreshing configuration on SIGHUP");
+                /* Refreshing config reads from the registry, which is a blocking operation */
+                status = LwRtlQueueWorkItem(gState.pPool, RefreshConfig, NULL, 0);
+                BAIL_ON_ERROR(status);
+                break;
+            default:
+                break;
+            }
+        }
+        
+        *pWaitMask = LW_TASK_EVENT_UNIX_SIGNAL;
     }
 
 cleanup:
 
-    return dwError;
+    return;
 
 error:
 
+    if (status)
+    {
+        LwRtlExitMain(status);
+        *pWaitMask = LW_TASK_EVENT_COMPLETE;
+    }
+
     goto cleanup;
 }
 
 static
 DWORD
-LwSmWaitSignals(
+LwSmMain(
     VOID
     )
 {
     DWORD dwError = 0;
-    sigset_t set;
-    static int waitSignals[] =
-    {
-        SIGTERM,
-        SIGHUP,
-        -1
-    };
-    int sig = -1;
-    int i = 0;
+    PLW_TASK pTask = NULL;
 
-    if (sigemptyset(&set) < 0)
-    {
-        dwError = LwMapErrnoToLwError(errno);
-        BAIL_ON_ERROR(dwError);
-    }
+    dwError = LwNtStatusToWin32Error(LwRtlCreateThreadPool(&gState.pPool, NULL));
+    BAIL_ON_ERROR(dwError);
 
-    for (i = 0; waitSignals[i] != -1; i++)
-    {
-        if (sigaddset(&set, waitSignals[i]) < 0)
-        {
-            dwError = LwMapErrnoToLwError(errno);
-            BAIL_ON_ERROR(dwError); 
-        }
-    }
+    dwError = LwNtStatusToWin32Error(LwRtlCreateTask(
+                                         gState.pPool,
+                                         &pTask,
+                                         NULL,
+                                         MainTask,
+                                         NULL));
+    BAIL_ON_ERROR(dwError);
 
-    for (;;)
-    {
-        if (sigwait(&set, &sig) < 0)
-        {
-            dwError = LwMapErrnoToLwError(errno);
-            BAIL_ON_ERROR(dwError);
-        }
+    LwRtlWakeTask(pTask);
 
-        switch (sig)
-        {
-        case SIGTERM:
-            SM_LOG_ALWAYS("Shutting down on SIGTERM");
-            goto cleanup;
-        case SIGHUP:
-            dwError = LwSmPopulateTable();
-            BAIL_ON_ERROR(dwError);
-            break;
-        default:
-            break;
-        }
-    }
+    dwError = LwNtStatusToWin32Error(LwRtlMain());
+    BAIL_ON_ERROR(dwError);
 
 cleanup:
 
-    return dwError;
+     if (pTask)
+     {
+         LwRtlCancelTask(pTask);
+         LwRtlWaitTask(pTask);
+         LwRtlReleaseTask(&pTask);
+     }
+
+     LwRtlFreeThreadPool(&gState.pPool);
+
+     return dwError;
 
 error:
 
@@ -606,16 +693,20 @@ LWMsgBool
 LwSmLogIpc (
     LWMsgLogLevel level,
     const char* pszMessage,
+    const char* pszFunction,
     const char* pszFilename,
     unsigned int line,
     void* pData
     )
 {
-    LW_SM_LOG_LEVEL smLevel;
-    LWMsgBool result;
+    LW_SM_LOG_LEVEL smLevel = LW_SM_LOG_LEVEL_DEBUG;
+    LWMsgBool result = LWMSG_FALSE;
 
     switch (level)
     {
+    case LWMSG_LOGLEVEL_ALWAYS:
+        smLevel = LW_SM_LOG_LEVEL_ALWAYS;
+        break;
     case LWMSG_LOGLEVEL_ERROR:
         smLevel = LW_SM_LOG_LEVEL_ERROR;
         break;
@@ -632,7 +723,6 @@ LwSmLogIpc (
         smLevel = LW_SM_LOG_LEVEL_DEBUG;
         break;
     case LWMSG_LOGLEVEL_TRACE:
-    default:
         smLevel = LW_SM_LOG_LEVEL_TRACE;
         break;
     }
@@ -643,7 +733,7 @@ LwSmLogIpc (
         {
             LwSmLogMessage(
                 smLevel,
-                __FUNCTION__,
+                pszFunction,
                 pszFilename,
                 line,
                 pszMessage);
@@ -666,6 +756,8 @@ LwSmStartIpcServer(
 {
     DWORD dwError = 0;
 
+    SM_LOG_VERBOSE("Starting IPC server");
+
     dwError = MAP_LWMSG_STATUS(lwmsg_context_new(NULL, &gState.pIpcContext));
     BAIL_ON_ERROR(dwError);
 
@@ -673,7 +765,7 @@ LwSmStartIpcServer(
         gState.pIpcContext,
         LwSmLogIpc,
         NULL);
-
+    
     dwError = MAP_LWMSG_STATUS(lwmsg_protocol_new(gState.pIpcContext, &gState.pIpcProtocol));
     BAIL_ON_ERROR(dwError);
 
@@ -682,25 +774,25 @@ LwSmStartIpcServer(
                                    LwSmIpcGetProtocolSpec()));
     BAIL_ON_ERROR(dwError);
 
-    dwError = MAP_LWMSG_STATUS(lwmsg_server_new(
+    dwError = MAP_LWMSG_STATUS(lwmsg_peer_new(
                                    gState.pIpcContext,
                                    gState.pIpcProtocol,
                                    &gState.pIpcServer));
     BAIL_ON_ERROR(dwError);
 
-    dwError = MAP_LWMSG_STATUS(lwmsg_server_add_dispatch_spec(
+    dwError = MAP_LWMSG_STATUS(lwmsg_peer_add_dispatch_spec(
                                    gState.pIpcServer,
                                    LwSmGetDispatchSpec()));
     BAIL_ON_ERROR(dwError);
 
-    dwError = MAP_LWMSG_STATUS(lwmsg_server_set_endpoint(
+    dwError = MAP_LWMSG_STATUS(lwmsg_peer_add_listen_endpoint(
                                    gState.pIpcServer,
                                    LWMSG_CONNECTION_MODE_LOCAL,
                                    SM_ENDPOINT,
                                    0666));
     BAIL_ON_ERROR(dwError);
 
-    dwError = MAP_LWMSG_STATUS(lwmsg_server_start(gState.pIpcServer));
+    dwError = MAP_LWMSG_STATUS(lwmsg_peer_start_listen(gState.pIpcServer));
     BAIL_ON_ERROR(dwError);
 
 cleanup:
@@ -750,93 +842,6 @@ error:
     goto cleanup;
 }
 
-static
-DWORD
-LwSmPopulateTable(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    HANDLE hReg = NULL;
-    PWSTR* ppwszNames = NULL;
-    PWSTR pwszName = NULL;
-    PLW_SERVICE_INFO pInfo = NULL;
-    PSM_TABLE_ENTRY pEntry = NULL;
-    size_t i = 0;
-
-    SM_LOG_VERBOSE("Populating service table");
-
-    dwError = RegOpenServer(&hReg);
-    BAIL_ON_ERROR(dwError);
-
-    dwError = LwSmRegistryEnumServices(hReg, &ppwszNames);
-    switch (dwError)
-    {
-    case LWREG_ERROR_NO_SUCH_KEY_OR_VALUE:
-        /* No services in registry */
-        dwError = 0;
-        goto cleanup;
-    }
-    BAIL_ON_ERROR(dwError);
-
-    for (i = 0; ppwszNames[i]; i++)
-    {
-        pwszName = ppwszNames[i];
-
-        dwError = LwSmRegistryReadServiceInfo(hReg, pwszName, &pInfo);
-        switch (dwError)
-        {
-        case LWREG_ERROR_NO_SUCH_KEY_OR_VALUE:
-            dwError = 0;
-            continue;
-        default:
-            break;
-        }
-        BAIL_ON_ERROR(dwError);
-
-        dwError = LwSmTableGetEntry(pwszName, &pEntry);
-        if (!dwError)
-        {
-            dwError = LwSmTableUpdateEntry(pEntry, pInfo, LW_SERVICE_INFO_MASK_ALL);
-            BAIL_ON_ERROR(dwError);
-        }
-        else if (dwError == LW_ERROR_NO_SUCH_SERVICE)
-        {
-            dwError = LwSmTableAddEntry(pInfo, &pEntry);
-            BAIL_ON_ERROR(dwError);
-        }
-        else
-        {
-            BAIL_ON_ERROR(dwError);
-        }
-
-        LwSmTableReleaseEntry(pEntry);
-        pEntry = NULL;
-
-        LwSmCommonFreeServiceInfo(pInfo);
-        pInfo = NULL;
-    }
-
-cleanup:
-
-    if (pInfo)
-    {
-        LwSmCommonFreeServiceInfo(pInfo);
-        pInfo = NULL;
-    }
-
-    if (hReg)
-    {
-        RegCloseServer(hReg);
-    }
-
-    return dwError;
-
-error:
-
-    goto cleanup;
-}
-
 static
 DWORD
 LwSmShutdownServices(
@@ -859,7 +864,12 @@ LwSmShutdownServices(
         BAIL_ON_ERROR(dwError);
 
         dwError = LwSmShutdownService(pEntry);
-        BAIL_ON_ERROR(dwError);
+        if (dwError)
+        {
+            /* Ignore errors and try to shut down everything we can */
+            SM_LOG_WARNING("Could not shut down service: %u\n", (unsigned int) dwError);
+            dwError = 0;
+        }
 
         LwSmTableReleaseEntry(pEntry);
         pEntry = NULL;
@@ -931,3 +941,4 @@ error:
 
     goto cleanup;
 }
+
