@@ -200,6 +200,8 @@ LwSmRegistryReadServiceInfo(
         {'E', 'n', 'v', 'i', 'r', 'o', 'n', 'm', 'e', 'n', 't', 0};
     static const WCHAR wszAutostart[] =
         {'A', 'u', 't', 'o', 's', 't', 'a', 'r', 't', 0};
+    static const WCHAR wszFdLimit[] =
+            {'F', 'd', 'L', 'i', 'm', 'i', 't', 0};
 
     dwError = LwWc16sToMbs(pwszName, &pszName);
     BAIL_ON_ERROR(dwError);
@@ -290,6 +292,19 @@ LwSmRegistryReadServiceInfo(
 
     pInfo->bAutostart = dwAutostart ? TRUE : FALSE;
 
+    dwError = LwSmRegistryReadDword(
+        hReg,
+        pRootKey,
+        pwszParentKey,
+        wszFdLimit,
+        &pInfo->dwFdLimit);
+    if (dwError == LWREG_ERROR_NO_SUCH_KEY_OR_VALUE)
+    {
+        dwError = 0;
+        pInfo->dwFdLimit = 0;
+    }
+    BAIL_ON_ERROR(dwError);
+
     *ppInfo = pInfo;
 
 cleanup:
@@ -427,7 +442,8 @@ LwSmRegistryReadStringList(
 
         *pwszSpace = (WCHAR) '\0';
 
-        LwAllocateWc16String(&ppwszValues[i], pwszCursor);
+        dwError = LwAllocateWc16String(&ppwszValues[i], pwszCursor);
+        BAIL_ON_ERROR(dwError);
 
         pwszCursor = pwszSpace + 1;
     }
@@ -436,6 +452,8 @@ LwSmRegistryReadStringList(
 
 cleanup:
 
+    LW_SAFE_FREE_MEMORY(pwszValuesString);
+
     return dwError;
 
 error:
