@@ -38,6 +38,9 @@
 
 #include "includes.h"
 
+#define RESTART_PERIOD 30
+#define RESTART_LIMIT 2
+
 static
 DWORD
 LwSmTablePollEntry(
@@ -63,6 +66,8 @@ LwSmTableFreeEntry(
     PSM_TABLE_ENTRY pEntry
     );
 
+static PLW_THREAD_POOL gpPool = NULL;
+
 static SM_TABLE gServiceTable = 
 {
     .lock = PTHREAD_MUTEX_INITIALIZER,
@@ -75,6 +80,7 @@ static PCSTR gLoaderTable[] =
     [LW_SERVICE_TYPE_LEGACY_EXECUTABLE] = "executable",
     [LW_SERVICE_TYPE_EXECUTABLE] = "executable",
     [LW_SERVICE_TYPE_DRIVER] = "driver",
+    [LW_SERVICE_TYPE_MODULE] = "svcm",
     [LW_SERVICE_TYPE_STUB] = "stub"
 };
 
@@ -181,6 +187,7 @@ LwSmTableReconstructEntry(
     )
 {
     DWORD dwError = 0;
+    PWSTR pwszLoaderName = NULL;
 
     if (pEntry->object.pData)
     {
@@ -188,6 +195,12 @@ LwSmTableReconstructEntry(
         pEntry->object.pData = NULL;
     }
 
+    dwError = LwMbsToWc16s(gLoaderTable[pEntry->pInfo->type], &pwszLoaderName);
+    BAIL_ON_ERROR(dwError);
+
+    dwError = LwSmLoaderGetVtbl(pwszLoaderName, &pEntry->pVtbl);
+    BAIL_ON_ERROR(dwError);
+
     dwError = pEntry->pVtbl->pfnConstruct(&pEntry->object, pEntry->pInfo, &pEntry->object.pData);
     BAIL_ON_ERROR(dwError);
 
@@ -195,6 +208,8 @@ LwSmTableReconstructEntry(
 
 error:
 
+    LW_SAFE_FREE_MEMORY(pwszLoaderName);
+
     return dwError;
 }
 
@@ -207,7 +222,6 @@ LwSmTableAddEntry(
     DWORD dwError = 0;
     BOOL bLocked = TRUE;
     PSM_TABLE_ENTRY pEntry = NULL;
-    PWSTR pwszLoaderName = NULL;
 
     dwError = LwAllocateMemory(sizeof(*pEntry), OUT_PPVOID(&pEntry));
     BAIL_ON_ERROR(dwError);
@@ -227,12 +241,6 @@ LwSmTableAddEntry(
     BAIL_ON_ERROR(dwError);
     pEntry->pEvent = &pEntry->event;
 
-    dwError = LwMbsToWc16s(gLoaderTable[pEntry->pInfo->type], &pwszLoaderName);
-    BAIL_ON_ERROR(dwError);
-
-    dwError = LwSmLoaderGetVtbl(pwszLoaderName, &pEntry->pVtbl);
-    BAIL_ON_ERROR(dwError);
-
     dwError = LwSmTableReconstructEntry(pEntry);
     BAIL_ON_ERROR(dwError);
 
@@ -248,8 +256,6 @@ LwSmTableAddEntry(
 
 cleanup:
 
-    LW_SAFE_FREE_MEMORY(pwszLoaderName);
-
     return dwError;
 
 error:
@@ -313,6 +319,7 @@ LwSmTableUpdateEntry(
 
     pUpdate->type = mask & LW_SERVICE_INFO_MASK_TYPE ? pInfo->type : pEntry->pInfo->type;
     pUpdate->bAutostart = mask & LW_SERVICE_INFO_MASK_AUTOSTART ? pInfo->bAutostart : pEntry->pInfo->bAutostart;
+    pUpdate->dwFdLimit = mask & LW_SERVICE_INFO_MASK_AUTOSTART ? pInfo->dwFdLimit : pEntry->pInfo->dwFdLimit;
 
     /* Atomically replace previous info structure */
     LwSmCommonFreeServiceInfo(pEntry->pInfo);
@@ -398,6 +405,8 @@ LwSmTableFreeEntry(
     {
         pthread_cond_destroy(pEntry->pEvent);
     }
+
+    LwFreeMemory(pEntry);
 }
 
 DWORD
@@ -443,7 +452,7 @@ LwSmTableStartEntry(
 
                 dwError = LwWc16sToMbs(pEntry->pInfo->pwszName, &pszServiceName);
                 BAIL_ON_ERROR(dwError);
-
+                
                 SM_LOG_INFO("Starting service: %s", pszServiceName);
 
                 if (pEntry->bDirty)
@@ -621,7 +630,7 @@ LwSmTableStopEntry(
 
                 dwError = LwWc16sToMbs(pEntry->pInfo->pwszName, &pszServiceName);
                 BAIL_ON_ERROR(dwError);
-
+                
                 SM_LOG_INFO("Stopping service: %s", pszServiceName);
 
                 UNLOCK(bLocked, pEntry->pLock);
@@ -773,7 +782,7 @@ LwSmTablePollEntry(
         switch (dwError)
         {
         case LW_ERROR_SUCCESS:
-            pEntry->bDepsMarked = TRUE;
+            pEntry->bDepsMarked = TRUE; 
             break;
         case LW_ERROR_SERVICE_DEPENDENCY_UNMET:
             /* This means we're in an inconsistent state where
@@ -841,17 +850,119 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+LwSmTableStartRecursive(
+    PSM_TABLE_ENTRY pEntry
+    )
+{
+    DWORD error = 0;
+    PSM_TABLE_ENTRY pOtherEntry = NULL;
+    PWSTR* ppServices = NULL;
+    DWORD index = 0;
+
+    /* Start all dependencies */
+    error = LwSmTableGetEntryDependencyClosure(pEntry, &ppServices);
+    BAIL_ON_ERROR(error);
+
+    for (index = 0; ppServices[index]; index++)
+    {
+        error = LwSmTableGetEntry(ppServices[index], &pOtherEntry);
+        BAIL_ON_ERROR(error);
+
+        error = LwSmTableStartEntry(pOtherEntry);
+        BAIL_ON_ERROR(error);
+
+        LwSmTableReleaseEntry(pOtherEntry);
+        pOtherEntry = NULL;
+    }
+
+    error = LwSmTableStartEntry(pEntry);
+    BAIL_ON_ERROR(error);
+
+error:
+
+    if (ppServices)
+    {
+        LwSmFreeStringList(ppServices);
+    }
+
+    if (pOtherEntry)
+    {
+        LwSmTableReleaseEntry(pOtherEntry);
+    }
+
+    return error;
+}
+
+static
+VOID
+LwSmTableWatchdog(
+    PVOID pContext
+    )
+{
+    DWORD error = 0;
+    PSM_TABLE_ENTRY pEntry = pContext;
+    PSM_TABLE_ENTRY pOtherEntry = NULL;
+    LW_SERVICE_STATUS status = {0};
+    PWSTR* ppServices = NULL;
+    DWORD index = 0;
+
+    error = LwSmTableStartRecursive(pEntry);
+    BAIL_ON_ERROR(error);
+
+    /* See if any reverse dependencies changed state without announcing it */
+    error = LwSmTableGetEntryReverseDependencyClosure(pEntry, &ppServices);
+    BAIL_ON_ERROR(error);
+
+    for (index = 0; ppServices[index]; index++)
+    {
+        error = LwSmTableGetEntry(ppServices[index], &pOtherEntry);
+        BAIL_ON_ERROR(error);
+
+        error = LwSmTableGetEntryStatus(pOtherEntry, &status);
+        BAIL_ON_ERROR(error);
+
+        if (status.state == LW_SERVICE_STATE_DEAD)
+        {
+            (void) LwSmTableStartRecursive(pOtherEntry);
+        }
+
+        LwSmTableReleaseEntry(pOtherEntry);
+        pOtherEntry = NULL;
+    }
+
+error:
+
+    if (ppServices)
+    {
+        LwSmFreeStringList(ppServices);
+    }
+
+    if (pOtherEntry)
+    {
+        LwSmTableReleaseEntry(pOtherEntry);
+    }
+
+    LwSmTableReleaseEntry(pEntry);
+
+    return;
+}
+
 VOID
 LwSmTableNotifyEntryStateChanged(
     PSM_TABLE_ENTRY pEntry,
     LW_SERVICE_STATE state
     )
 {
+    DWORD error = ERROR_SUCCESS;
     BOOLEAN bLocked = FALSE;
     PSM_LINK pLink = NULL;
     PSM_LINK pNext = NULL;
     PSM_ENTRY_NOTIFY pNotify = NULL;
-
+    PSTR pServiceName = NULL;
+    time_t now = 0;
+    
     LOCK(bLocked, pEntry->pLock);
 
     for (pLink = LwSmLinkBegin(&pEntry->waiters);
@@ -870,7 +981,58 @@ LwSmTableNotifyEntryStateChanged(
 
     pthread_cond_broadcast(pEntry->pEvent);
 
+    if (state == LW_SERVICE_STATE_DEAD)
+    {
+        now = time(NULL);
+        if (now == (time_t) -1)
+        {
+            error = LwErrnoToWin32Error(errno);
+            BAIL_ON_ERROR(error);
+        }
+
+        error = LwWc16sToMbs(pEntry->pInfo->pwszName, &pServiceName);
+        BAIL_ON_ERROR(error);
+
+        if ((now - pEntry->LastRestartPeriod) > RESTART_PERIOD)
+        {
+            pEntry->RestartAttempts = 0;
+            pEntry->LastRestartPeriod = now;
+        }
+
+        if (pEntry->RestartAttempts < RESTART_LIMIT)
+        {
+            pEntry->RestartAttempts++;
+            pEntry->dwRefCount++;
+
+            SM_LOG_WARNING(
+                "Restarting dead service: %s (attempt %u/%u)",
+                pServiceName,
+                (unsigned int) pEntry->RestartAttempts,
+                (unsigned int) RESTART_LIMIT);
+
+
+            error = LwNtStatusToWin32Error(LwRtlQueueWorkItem(gpPool, LwSmTableWatchdog, pEntry, 0));
+            if (error)
+            {
+                pEntry->dwRefCount--;
+            }
+            BAIL_ON_ERROR(error);
+        }
+        else
+        {
+            SM_LOG_ERROR(
+                "Service died: %s (restarted %u times in %lu seconds)",
+                pServiceName,
+                (unsigned int) pEntry->RestartAttempts,
+                (unsigned long) (now - pEntry->LastRestartPeriod));
+        }
+    }
+
+error:
+
     UNLOCK(bLocked, pEntry->pLock);
+
+    LW_SAFE_FREE_MEMORY(pServiceName);
 }
 
 DWORD
@@ -919,16 +1081,16 @@ LwSmTableRegisterEntryNotify(
     {
         dwError = LwAllocateMemory(sizeof(*pNotify), OUT_PPVOID(&pNotify));
         BAIL_ON_ERROR(dwError);
-
+        
         LwSmLinkInit(&pNotify->link);
         pNotify->pfnNotifyEntryStateChange = pfnNotifyEntryStateChange;
         pNotify->pData = pData;
-
+        
         LwSmLinkInsertBefore(&pEntry->waiters, &pNotify->link);
     }
-
+    
 cleanup:
-
+    
     UNLOCK(bLocked, pEntry->pLock);
 
     return dwError;
@@ -952,7 +1114,7 @@ LwSmTableUnregisterEntryNotify(
     PSM_LINK pLink = NULL;
     PSM_LINK pNext = NULL;
     PSM_ENTRY_NOTIFY pNotify = NULL;
-
+    
     LOCK(bLocked, pEntry->pLock);
 
     for (pLink = LwSmLinkBegin(&pEntry->waiters);
@@ -1147,7 +1309,7 @@ cleanup:
 
     if (pDepInfo)
     {
-        LwSmCommonFreeServiceInfo(pInfo);
+        LwSmCommonFreeServiceInfo(pDepInfo);
     }
 
     if (pDepEntry)
@@ -1250,6 +1412,48 @@ LwSmTableNotifyServiceObjectStateChange(
     return LwSmTableNotifyEntryStateChanged(pEntry, newState);
 }
 
+DWORD
+LwSmTableInit(
+    VOID
+    )
+{
+    DWORD error = ERROR_SUCCESS;
+
+    error = LwNtStatusToWin32Error(LwRtlCreateThreadPool(&gpPool, NULL));
+    BAIL_ON_ERROR(error);
+
+error:
+
+    return error;
+}
+
+VOID
+LwSmTableShutdown(
+    VOID
+    )
+{
+    BOOL bLocked = FALSE;
+    PSM_LINK pLink = NULL;
+    PSM_LINK pNext = NULL;
+    PSM_TABLE_ENTRY pEntry = NULL;
+
+    LOCK(bLocked, gServiceTable.pLock);
+
+    for (pLink = LwSmLinkBegin(&gServiceTable.entries);
+         LwSmLinkValid(&gServiceTable.entries, pLink);
+         pLink = pNext)
+    {
+        pNext = LwSmLinkNext(pLink);
+        pEntry = STRUCT_FROM_MEMBER(pLink, SM_TABLE_ENTRY, link);
+
+        LwSmTableFreeEntry(pEntry);
+    }
+
+    UNLOCK(bLocked, gServiceTable.pLock);
+    pthread_mutex_destroy(gServiceTable.pLock);
+    LwRtlFreeThreadPool(&gpPool);
+}
+
 SM_LOADER_CALLS gTableCalls =
 {
     .pfnGetServiceObjectData = LwSmTableGetServiceObjectData,
