@@ -31,5 +31,5 @@
 #include <lwerror.h>
 #include <lwstr.h>
 #include <lwmem.h>
-#include <lw/base.h>
 
+#include <errno.h>
