@@ -36,6 +36,4 @@
 #include <lwerror.h>
 #include <lwstr.h>
 #include <lwmem.h>
-#include <lw/base.h>
-#include <reg/reg.h>
 
