@@ -47,7 +47,6 @@ UnquoteString(
     )
 {
     DWORD dwError = 0;
-    int   nRead   = 0;
     PSTR pszOutput = pszInput;
 
     while (pszInput && *pszInput != 0)
@@ -73,7 +72,7 @@ UnquoteString(
                     pszInput += 2;
                     break;
                 case 'x':
-                    nRead = sscanf(pszInput + 2, "%2hhX", pszOutput);
+                    sscanf(pszInput + 2, "%2hhX", pszOutput);
                     pszInput += 4;
                     break;
                 default:
@@ -172,7 +171,7 @@ RegComp(
     )
 {
     DWORD dwError = regcomp(pRegex, pszExpr, REG_EXTENDED);
-    
+
     if (dwError)
     {
         dwError = LwMapErrnoToLwError(EINVAL);
