@@ -49,7 +49,15 @@
 
 static LWNET_CLIENT_CONNECTION_CONTEXT gContext = {0};
 static volatile LONG glLibraryRefCount = 0;
+#ifndef BROKEN_ONCE_INIT
 #if defined(__LWI_SOLARIS__) || defined (__LWI_AIX__)
+#define BROKEN_ONCE_INIT 1
+#else
+#define BROKEN_ONCE_INIT 0
+#endif
+#endif
+
+#if BROKEN_ONCE_INIT
 static pthread_once_t gOnceControl = {PTHREAD_ONCE_INIT};
 #else
 static pthread_once_t gOnceControl = PTHREAD_ONCE_INIT;
@@ -73,7 +81,7 @@ LWNetOpenServerOnce(
     BAIL_ON_LWNET_ERROR(dwError);
 
     dwError = MAP_LWMSG_ERROR(lwmsg_client_set_endpoint(
-                                  gContext.pClient,
+                                  gContext.pClient, 
                                   LWMSG_CONNECTION_MODE_LOCAL,
                                   LWNET_CACHE_DIR "/" LWNET_SERVER_FILENAME));
     BAIL_ON_LWNET_ERROR(dwError);
@@ -91,13 +99,13 @@ error:
         lwmsg_client_delete(gContext.pClient);
         gContext.pClient = NULL;
     }
-
+    
     if (gContext.pProtocol)
     {
         lwmsg_protocol_delete(gContext.pProtocol);
         gContext.pProtocol = NULL;
     }
-
+    
     goto cleanup;
 }
 
@@ -111,7 +119,7 @@ LWNetOpenServer(
     BAIL_ON_INVALID_POINTER(phConnection);
 
     pthread_once(&gOnceControl, LWNetOpenServerOnce);
-
+    
     dwError = gdwOnceError;
     BAIL_ON_LWNET_ERROR(dwError);
 
@@ -123,11 +131,11 @@ cleanup:
 
 error:
 
-    if (phConnection)
+    if (phConnection) 
     {
         *phConnection = NULL;
     }
-
+    
     goto cleanup;
 }
 
@@ -184,7 +192,7 @@ LWNetAcquireCall(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_client_acquire_call(pContext->pClient, ppCall));
     BAIL_ON_LWNET_ERROR(dwError);
-
+        
 error:
 
     return dwError;
@@ -227,7 +235,7 @@ LWNetTransactGetDCName(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_GET_DC_NAME:
@@ -260,7 +268,7 @@ error:
     {
         *ppDCInfo = NULL;
     }
-
+                    
     goto cleanup;
 }
 
@@ -294,7 +302,7 @@ LWNetTransactGetDCList(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_GET_DC_LIST:
@@ -329,7 +337,7 @@ error:
 
     *ppDcList = NULL;
     *pdwDcCount = 0;
-
+                    
     goto cleanup;
 
 }
@@ -357,7 +365,7 @@ LWNetTransactGetDCTime(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_GET_DC_TIME:
@@ -412,7 +420,7 @@ LWNetTransactGetDomainController(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_GET_DOMAIN_CONTROLLER:
@@ -446,61 +454,6 @@ error:
     goto cleanup;
 }
 
-DWORD
-LWNetTransactGetCurrentDomain(
-    HANDLE hConnection,
-    PSTR* ppszDomainFQDN
-    )
-{
-    DWORD dwError = 0;
-    PLWNET_IPC_ERROR pError = NULL;
-    PLWNET_IPC_STRING pCurRes = NULL;
-
-    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
-    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
-    LWMsgCall* pCall = NULL;
-
-    dwError = LWNetAcquireCall(hConnection, &pCall);
-    BAIL_ON_LWNET_ERROR(dwError);
-
-    in.tag = LWNET_Q_GET_CURRENT_DOMAIN;
-    in.data = NULL;
-
-    dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
-    BAIL_ON_LWNET_ERROR(dwError);
-
-    switch (out.tag)
-    {
-    case LWNET_R_GET_CURRENT_DOMAIN:
-        pCurRes = out.data;
-        *ppszDomainFQDN = pCurRes->pszString;
-        pCurRes->pszString = NULL;
-        break;
-    case LWNET_R_ERROR:
-        pError = (PLWNET_IPC_ERROR) out.data;
-        dwError = pError->dwError;
-        BAIL_ON_LWNET_ERROR(dwError);
-        break;
-    default:
-        dwError = LW_ERROR_INTERNAL;
-        BAIL_ON_LWNET_ERROR(dwError);
-    }
-
-cleanup:
-
-    if (pCall)
-    {
-        lwmsg_call_destroy_params(pCall, &out);
-        lwmsg_call_release(pCall);
-    }
-
-    return dwError;
-
-error:
-
-    goto cleanup;
-}
-
 DWORD
 LWNetTransactSetLogLevel(
     IN HANDLE hConnection,
@@ -525,7 +478,7 @@ LWNetTransactSetLogLevel(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_SET_LOG_LEVEL:
@@ -579,7 +532,7 @@ LWNetTransactGetLogInfo(
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LWNET_ERROR(dwError);
-
+    
     switch (out.tag)
     {
     case LWNET_R_GET_LOG_INFO:
@@ -613,7 +566,7 @@ cleanup:
     *ppszLogPath = pszLogPath;
 
     return dwError;
-
+    
 error:
     LogLevel = 0;
     LogTarget = 0;
@@ -621,3 +574,75 @@ error:
 
     goto cleanup;
 }
+
+
+DWORD
+LWNetTransactResolveName(
+    LW_IN HANDLE hConnection,
+    LW_IN LW_PCWSTR pcwszHostName,
+    LW_OUT LW_PWSTR *ppwszCanonName,
+    LW_OUT PLWNET_RESOLVE_ADDR **pppAddressList,
+    LW_OUT PDWORD pdwAddressListLen)
+{
+    DWORD dwError = 0;
+    PLWNET_IPC_ERROR pError = NULL;
+    LWNET_RESOLVE_NAME_ADDRESS inHostName = {0};
+    PLWNET_RESOLVE_ADDR *ppAddressList = NULL;
+    DWORD dwAddressListLen = 0;
+
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    
+    dwError = LWNetAcquireCall(hConnection, &pCall);
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    inHostName.pwszHostName = (PWSTR) pcwszHostName;
+
+    in.tag = LWNET_Q_RESOLVE_NAME;
+    in.data = &inHostName;
+
+    dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    switch (out.tag)
+    {
+    case LWNET_R_RESOLVE_NAME:
+    {
+        PLWNET_RESOLVE_NAME_ADDRESS_RESPONSE pResponse = 
+            (PLWNET_RESOLVE_NAME_ADDRESS_RESPONSE) out.data;
+        ppAddressList = pResponse->ppAddressList;
+        dwAddressListLen = pResponse->dwAddressListLen;
+
+        *ppwszCanonName = pResponse->pwszCanonName;
+        // Returned to caller, NULL to prevent freeing by lwmsg
+        pResponse->ppAddressList = NULL;
+        pResponse->pwszCanonName = NULL;
+        break;
+    }
+    case LWNET_R_ERROR:
+        pError = (PLWNET_IPC_ERROR) out.data;
+        dwError = pError->dwError;
+        BAIL_ON_LWNET_ERROR(dwError);
+        break;
+    default:
+        dwError = LW_ERROR_INTERNAL;
+        BAIL_ON_LWNET_ERROR(dwError);
+    }
+
+cleanup:
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    *pppAddressList = ppAddressList;
+    *pdwAddressListLen = dwAddressListLen;
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
