@@ -50,31 +50,88 @@
 #ifndef __IPC_CLIENT_P_H__
 #define __IPC_CLIENT_P_H__
 
-typedef struct __LWNET_CLIENT_CONNECTION_CONTEXT {
-    int    fd;
+#include <lwmsg/lwmsg.h>
+
+typedef struct __LWNET_CLIENT_CONNECTION_CONTEXT
+{
+    LWMsgProtocol* pProtocol;
+    LWMsgClient* pClient;
 } LWNET_CLIENT_CONNECTION_CONTEXT, *PLWNET_CLIENT_CONNECTION_CONTEXT;
 
+DWORD
+LWNetAcquireCall(
+    HANDLE hConnection,
+    LWMsgCall** ppCall
+    );
+
 DWORD
 LWNetOpenServer(
     PHANDLE phConnection
     );
 
-DWORD
-LWNetSendMessage(
-    HANDLE hConnection,
-    PLWNETMESSAGE pMessage
-    );
-
-DWORD
-LWNetGetNextMessage(
-    HANDLE hConnection,
-    PLWNETMESSAGE* ppMessage
-    );
-
 DWORD
 LWNetCloseServer(
     HANDLE hConnection
     );
 
-#endif /* __IPC_CLIENT_P_H__ */
+DWORD
+LWNetTransactGetDCName(
+    HANDLE hConnection,
+    PCSTR pszServerFQDN,
+    PCSTR pszDomainFQDN,
+    PCSTR pszSiteName,
+    PCSTR pszPrimaryDomain,
+    DWORD dwFlags,
+    DWORD dwBlackListCount,
+    PSTR* ppszAddressBlackList,
+    PLWNET_DC_INFO* ppDCInfo
+    );
 
+DWORD
+LWNetTransactGetDCList(
+    IN HANDLE hConnection,
+    IN PCSTR pszDomainFQDN,
+    IN PCSTR pszSiteName,
+    IN DWORD dwFlags,
+    OUT PLWNET_DC_ADDRESS* ppDcList,
+    OUT LW_PDWORD pdwDcCount
+    );
+
+DWORD
+LWNetTransactGetDCTime(
+    HANDLE hConnection,
+    PCSTR pszDomainFQDN,
+    PLWNET_UNIX_TIME_T pDCTime
+    );
+
+DWORD
+LWNetTransactGetDomainController(
+    HANDLE hConnection,
+    PCSTR pszDomainFQDN,
+    PSTR* ppszDomainControllerFQDN
+    );
+
+DWORD
+LWNetTransactSetLogLevel(
+    IN HANDLE hConnection,
+    IN LWNET_LOG_LEVEL LogLevel
+    );
+
+DWORD
+LWNetTransactGetLogInfo(
+    IN HANDLE hConnection,
+    OUT PLWNET_LOG_LEVEL pLogLevel,
+    OUT PLWNET_LOG_TARGET pLogTarget,
+    OUT PSTR* ppszLogPath
+    );
+
+DWORD
+LWNetTransactResolveName(
+    LW_IN HANDLE hConnection,
+    LW_IN LW_PCWSTR pcwszHostName,
+    LW_OUT LW_PWSTR *pwszCanonName,
+    LW_OUT PLWNET_RESOLVE_ADDR **pppAddressList,
+    LW_OUT PDWORD pdwAddressListLen
+    );
+
+#endif /* __IPC_CLIENT_P_H__ */
