@@ -48,6 +48,8 @@
 #ifndef __LWNET_DEF_H__
 #define __LWNET_DEF_H__
 
+#include <lw/winerror.h>
+
 #ifndef WIN32
 #define PATH_SEPARATOR_STR "/"
 #else
@@ -148,12 +150,6 @@
 
 #endif
 
-#if defined(HAVE_SOCKLEN_T) && defined(GETSOCKNAME_TAKES_SOCKLEN_T)
-#    define SOCKLEN_T socklen_t
-#else
-#    define SOCKLEN_T int
-#endif
-
 #define LWNET_MODE_BITS_URWX_GRX_ORX (S_IRWXU | S_IRGRP | S_IXGRP |S_IROTH | S_IXOTH)
 #define LWNET_MODE_BITS_URW_GR_OR    (S_IRUSR | S_IWUSR | S_IRGRP |S_IROTH)
 
@@ -166,21 +162,23 @@
 #define LWNET_MICROSECONDS_IN_SECOND ((LWNET_MICROSECONDS_IN_MILLISECOND)*(LWNET_MILLISECONDS_IN_SECOND))
 #define LWNET_NANOSECONDS_IN_MILLISECOND (1000 * LWNET_MICROSECONDS_IN_MILLISECOND);
 
+#define LWNET_SYSLOG_MAX_LEVEL LOG_INFO
+
 #define BAIL_ON_INVALID_STRING(pszParam)             \
         if (IsNullOrEmptyString(pszParam)) {         \
-           dwError = LWNET_ERROR_INVALID_PARAMETER; \
+           dwError = ERROR_INVALID_PARAMETER; \
            BAIL_ON_LWNET_ERROR(dwError);            \
         }
 
 #define BAIL_ON_INVALID_HANDLE(hParam)               \
         if (hParam == (HANDLE)NULL) {                \
-           dwError = LWNET_ERROR_INVALID_PARAMETER; \
+           dwError = ERROR_INVALID_PARAMETER; \
            BAIL_ON_LWNET_ERROR(dwError);            \
         }
 
 #define BAIL_ON_INVALID_POINTER(p)                   \
         if (NULL == p) {                             \
-           dwError = LWNET_ERROR_INVALID_PARAMETER; \
+           dwError = ERROR_INVALID_PARAMETER; \
            BAIL_ON_LWNET_ERROR(dwError);            \
         }
 
