@@ -194,6 +194,27 @@ typedef struct _LWNET_DC_ADDRESS {
 } LWNET_DC_ADDRESS, *PLWNET_DC_ADDRESS;
 
 
+typedef enum LWNET_ADDR_TYPE
+{
+    LWNET_IP_ADDR_V4 = 1,
+    LWNET_IP_ADDR_V6
+} LWNET_ADDR_TYPE;
+
+
+typedef union LWNET_ADDR
+{
+    BYTE Ip4Addr[4];
+    BYTE Ip6Addr[16];
+} LWNET_ADDR, *PLWNET_ADDR;
+
+
+typedef struct LWNET_RESOLVE_ADDR
+{
+    LWNET_ADDR_TYPE AddressType;
+    LWNET_ADDR Address;
+} LWNET_RESOLVE_ADDR, *PLWNET_RESOLVE_ADDR;
+
+
 LW_BEGIN_EXTERN_C
 
 LWNET_API
@@ -255,12 +276,6 @@ LWNetGetDCTime(
     LW_OUT PLWNET_UNIX_TIME_T pDCTime
     );
 
-LWNET_API
-LW_DWORD
-LWNetGetCurrentDomain(
-    LW_OUT LW_PSTR* ppszDomainFQDN
-    );
-
 LWNET_API
 LW_DWORD
 LWNetExtendEnvironmentForKrb5Affinity(
@@ -337,6 +352,23 @@ LWNetGetLogInfo(
     LW_OUT LW_PSTR* ppszLogPath
     );
 
+LWNET_API
+LW_DWORD
+LWNetResolveName(
+    LW_IN LW_PCWSTR pcwszHostName,
+    LW_OUT LW_OPTIONAL LW_PWSTR *ppwszCanonName,
+    LW_OUT PLWNET_RESOLVE_ADDR **pppAddressList,
+    LW_OUT PDWORD pdwAddressListLen
+    );
+
+LWNET_API
+LW_DWORD
+LWNetResolveNameFree(
+    LW_IN LW_OPTIONAL LW_PWSTR pwszCanonName,
+    LW_IN PLWNET_RESOLVE_ADDR *ppAddressList,
+    LW_IN DWORD dwAddressListLen
+    );
+
 LW_END_EXTERN_C
 
 
