@@ -1,6 +1,6 @@
-/* Editor Settings: expandtabs and use 4 spaces for indentation
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4 -*-
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ * Editor Settings: expandtabs and use 4 spaces for indentation */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -263,9 +263,9 @@ LWNetSrvGetDCName(
         if ((dwBlackListCount == 0) && LWNetSrvIsAffinitizableRequestFlags(dwDsFlags))
         {
             // We need to update the cache with this entry.
-
+            
             bUpdateCache = TRUE;
-
+            
             dwError = LWNetGetSystemTime(&now);
             BAIL_ON_LWNET_ERROR(dwError);
 
@@ -274,13 +274,10 @@ LWNetSrvGetDCName(
             isBackoffToWritableDc = bFailedFindWritable;
             lastBackoffToWritableDc = isBackoffToWritableDc ? now : 0;
 
-            // Will only affinitize for KDC/LDAP (i.e., not PDC/GC) and if the site is the same.
-            if (!(dwDsFlags & (DS_PDC_REQUIRED | DS_GC_SERVER_REQUIRED)) &&
-                (IsNullOrEmptyString(pszSiteName) ||
-                 !strcasecmp(pDcInfo->pszDCSiteName, pszSiteName)))
-            {
-                bUpdateKrb5Affinity = TRUE;
-            }
+            bUpdateKrb5Affinity = LWNetIsUpdateKrb5AffinityEnabled(
+                                      dwDsFlags,
+                                      pszSiteName, 
+                                      pDcInfo);
         }
     }
 
@@ -573,7 +570,7 @@ LWNetSrvGetDCTimeFromDC(
 #ifdef HAVE_TIMEGM
     ttDcTimeUTC = timegm(&dcTime);
 #else
-    // We need to convert a universal time in a time structure to
+    // We need to convert a universal time in a time structure to 
     // seconds since the epoch but the only function that is available
     // assumes it's a local time and applies the timezone offset.  We
     // have to determine what the offset is for today's date which could
