@@ -255,13 +255,14 @@ LWNetCacheDbOpen(
     )
 {
     DWORD dwError = 0;
+    LONG lError = 0;
     LWNET_CACHE_DB_HANDLE dbHandle = NULL;
 
     dwError = LWNetAllocateMemory(sizeof(*dbHandle), (PVOID *)&dbHandle);
     BAIL_ON_LWNET_ERROR(dwError);
 
-    // TODO-dalmeida-2008/06/30 -- Convert error code
-    dwError = pthread_rwlock_init(&dbHandle->Lock, NULL);
+    lError = pthread_rwlock_init(&dbHandle->Lock, NULL);
+    dwError = LwMapErrnoToLwError(lError);
     BAIL_ON_LWNET_ERROR(dwError);
 
     dbHandle->pLock = &dbHandle->Lock;
@@ -377,7 +378,7 @@ LWNetCacheDbRegistryReadString(
                       NULL,
                       pszRetString,
                       &dwValueLen);
-
+ 
     }
     *ppszRetString = pszRetString;
 
@@ -468,7 +469,7 @@ LWNetCacheDbRegistryInitValueArray(
 {
     DWORD dwError = 0;
     PLWNET_CACHE_REGISTRY_VALUE pValueArray = NULL;
-    LWNET_CACHE_REGISTRY_VALUE valueArray[] =
+    LWNET_CACHE_REGISTRY_VALUE valueArray[] = 
     {
         {
             "DnsDomainName",
@@ -652,6 +653,7 @@ LWNetCacheDbRegistryReadValues(
     }
 
 cleanup:
+    LWNET_SAFE_FREE_MEMORY(valueArray);
     return dwError;
 
 error:
@@ -851,7 +853,7 @@ LWNetCacheDbRegistryWriteValues(
                   &valueArray,
                   &dwArrayEntries);
     BAIL_ON_LWNET_ERROR(dwError);
-
+              
     for (index=0; index<dwArrayEntries; index++)
     {
         dwError = LWNetCacheDbRegistryWriteValue(
@@ -895,7 +897,7 @@ LWNetCacheDbWriteToRegistry(
                       HKEY_THIS_MACHINE,
                       LWNET_NETLOGON_REGISTRY_KEY,
                       LWNET_CACHE_REGISTRY_KEY);
-
+                      
     dwError = RegUtilAddKey(
                   hReg,
                   HKEY_THIS_MACHINE,
@@ -913,7 +915,7 @@ LWNetCacheDbWriteToRegistry(
         pCacheDbEntry = (PLWNET_CACHE_DB_ENTRY) pCacheList->pItem;
 
         dwError = LwAllocateStringPrintf(
-                       &pszNewCacheKey,
+                       &pszNewCacheKey, 
                       "%s%s%s-%d",
                        pCacheDbEntry->pszDnsDomainName ?
                            pCacheDbEntry->pszDnsDomainName : "",
@@ -922,7 +924,7 @@ LWNetCacheDbWriteToRegistry(
                            pCacheDbEntry->pszSiteName : "",
                        (int) pCacheDbEntry->QueryType);
         BAIL_ON_LWNET_ERROR(dwError);
-
+        
         dwError = RegUtilAddKey(
                       hReg,
                       HKEY_THIS_MACHINE,
@@ -1101,32 +1103,32 @@ LWNetCacheDbQuery(
                   pEntry->DcInfo.pszNetBIOSDomainName,
                   &pDcInfo->pszNetBIOSDomainName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pEntry->DcInfo.pszFullyQualifiedDomainName,
                   &pDcInfo->pszFullyQualifiedDomainName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pEntry->DcInfo.pszDnsForestName,
                   &pDcInfo->pszDnsForestName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pEntry->DcInfo.pszDCSiteName,
                   &pDcInfo->pszDCSiteName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pEntry->DcInfo.pszClientSiteName,
                   &pDcInfo->pszClientSiteName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pEntry->DcInfo.pszNetBIOSHostName,
                   &pDcInfo->pszNetBIOSHostName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     if (pEntry->DcInfo.pszUserName)
     {
         dwError = LWNetAllocateString(
@@ -1232,32 +1234,32 @@ LWNetCacheDbUpdate(
                   pDcInfo->pszNetBIOSDomainName,
                   &pNewEntry->DcInfo.pszNetBIOSDomainName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pDcInfo->pszFullyQualifiedDomainName,
                   &pNewEntry->DcInfo.pszFullyQualifiedDomainName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pDcInfo->pszDnsForestName,
                   &pNewEntry->DcInfo.pszDnsForestName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pDcInfo->pszDCSiteName,
                   &pNewEntry->DcInfo.pszDCSiteName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pDcInfo->pszClientSiteName,
                   &pNewEntry->DcInfo.pszClientSiteName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
                   pDcInfo->pszNetBIOSHostName,
                   &pNewEntry->DcInfo.pszNetBIOSHostName);
     BAIL_ON_LWNET_ERROR(dwError);
-
+           
     dwError = LWNetAllocateString(
 	          pDcInfo->pszUserName ? pDcInfo->pszUserName : "",
 		  &pNewEntry->DcInfo.pszUserName);
@@ -1380,7 +1382,7 @@ LWNetCacheDbExport(
     DWORD i = 0;
 
     RW_LOCK_ACQUIRE_READ(DbHandle->pLock);
-
+    
     for (pListEntry = DbHandle->pCacheList;
          pListEntry;
          pListEntry = pListEntry->pNext)
