@@ -1,6 +1,6 @@
-/* Editor Settings: expandtabs and use 4 spaces for indentation
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4 -*-
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ * Editor Settings: expandtabs and use 4 spaces for indentation */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -400,10 +400,29 @@ LWNetSrvPingCLdapPoll(
 {
     DWORD dwError = 0;
     int sret = 0;
+    LWNET_UNIX_MS_TIME_T CurrentTime = 0;
+    LWNET_UNIX_MS_TIME_T StopTime = 0;
+    DWORD dwRemainder = 0;
+
+    dwError = LWNetGetSystemTimeInMs(&StopTime);
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    StopTime += dwTimeoutMilliseconds;
 
     do
     {
-        sret = poll(Readfds, dwFdCount, dwTimeoutMilliseconds);
+        dwError = LWNetGetSystemTimeInMs(&CurrentTime);
+        BAIL_ON_LWNET_ERROR(dwError);
+
+        if (CurrentTime < StopTime)
+        {
+            dwRemainder = StopTime - CurrentTime;
+            sret = poll(Readfds, dwFdCount, dwRemainder);
+        }
+        else
+        {
+            sret = 0;
+        }
     } while (sret < 0 && errno == EINTR);
 
     if (sret < 0)
@@ -483,7 +502,7 @@ LWNetSrvPingCLdapPackArrays(
     // Eliminate the invalid connections while preserving
     // the order so that the newest connections remain at
     // the end of the list.
-    for (dwTargetConn = 0, dwSourceConn = 0 ;
+    for (dwTargetConn = 0, dwSourceConn = 0 ; 
          dwSourceConn < dwConnectionCount ;
          dwSourceConn++)
     {
@@ -538,7 +557,7 @@ LWNetSrvPingCLdapArray(
     //     2. polling for responses
     //     3. processing responses
     //
-    // When starting CLDAP searches, a portion of the
+    // When starting CLDAP searches, a portion of the 
     // available connections will be started followed
     // by a short poll until the maximum number of
     // connections has been used.  Once the maximum
@@ -585,7 +604,7 @@ LWNetSrvPingCLdapArray(
 
     StopTime = CurrentTime + (dwActualTimeoutSeconds * 1000);
 
-    while (!pDcInfo)
+    for (;;)
     {
         LWNetSrvPingCLdapNewConnections(
             pszDnsDomainName,
@@ -627,7 +646,7 @@ LWNetSrvPingCLdapArray(
                 dwSingleConnTimeoutMilliseconds > CurrentTime)
             {
                 // Use the remaining time for the oldest connection
-                dwTimeoutMilliseconds = pConnections[0].StartTime +
+                dwTimeoutMilliseconds = pConnections[0].StartTime + 
                                         dwSingleConnTimeoutMilliseconds -
                                         CurrentTime;
                 dwTimeoutMilliseconds = CT_MIN(dwSingleConnTimeoutMilliseconds, StopTime - CurrentTime);
@@ -671,12 +690,6 @@ LWNetSrvPingCLdapArray(
                                 Readfds);
     }
 
-    if (!pDcInfo)
-    {
-        dwError = NERR_DCNotFound;
-        BAIL_ON_LWNET_ERROR(dwError);
-    }
-
 error:
     if (pConnections)
     {
@@ -767,7 +780,6 @@ LWNetSrvGetDCNameDiscoverInternal(
     OUT OPTIONAL PDWORD pdwServerCount,
     OUT PBOOLEAN bFailedFindWritable
     );
-
 DWORD
 LWNetSrvGetDCNameDiscover(
     IN PCSTR pszDnsDomainName,
@@ -817,7 +829,7 @@ LWNetSrvGetDCNameDiscover(
                   ppServerArray,
                   pdwServerCount,
                   pbFailedFindWritable);
-    BAIL_ON_LWNET_ERROR(dwError);
+    BAIL_ON_LWNET_ERROR(dwError);    
 
 cleanup:
 
@@ -825,7 +837,7 @@ cleanup:
 
 error:
 
-    goto cleanup;
+    goto cleanup;    
 }
 
 static
@@ -889,7 +901,7 @@ LWNetSrvGetDCNameDiscoverInternal(
     {
         dwError = LWNetFindServersInDomain(
             pServerArray,
-            dwServerCount,
+            dwServerCount, 
             pszPrimaryDomain,
             &pServersInPrimaryDomain,
             &dwServersInPrimaryDomainCount);
@@ -947,6 +959,30 @@ LWNetSrvGetDCNameDiscoverInternal(
     // If we got the correct site already, we are done.
     if (LWNetSrvIsInSameSite(pDcInfo))
     {
+        if (LWNetIsUpdateKrb5AffinityEnabled(dwDsFlags, pszSiteName, pDcInfo) &&
+            !pszSiteName &&
+            pDcInfo->pszClientSiteName)
+        {
+            dwError = pfnDCListQuery(
+                          pszDnsDomainName,
+                          pDcInfo->pszClientSiteName,
+                          dwDsFlags,
+                          &pSiteServerArray,
+                          &dwSiteServerCount);
+            if (dwError == 0)
+            {
+                // Use the site-specific DC.
+                LWNET_SAFE_FREE_MEMORY(pServerArray);
+                dwServerCount = 0;
+
+                pServerArray = pSiteServerArray;
+                dwServerCount = dwSiteServerCount;
+
+                pSiteServerArray = NULL;
+                dwSiteServerCount = 0;
+            }
+        }
+
         dwError = 0;
         goto cleanup;
     }
@@ -1045,7 +1081,7 @@ LWNetFindServersInDomain(
     {
         dwError = LWNetAllocateMemory(dwServersInDomainCount * sizeof(*pServerArray), OUT_PPVOID(&pServersInDomain));
         BAIL_ON_LWNET_ERROR(dwError);
-
+        
         for (i = 0; i < dwServerCount; i++)
         {
             if (LWNetServerIsInDomain(&pServerArray[i], pszDomainName))
@@ -1054,7 +1090,7 @@ LWNetFindServersInDomain(
             }
         }
     }
-
+        
     *ppServersInDomain = pServersInDomain;
     *pdwServersInDomainCount = dwServersInDomainCount;
 
@@ -1333,12 +1369,24 @@ error:
     return dwError;
 }
 
+BOOLEAN
+LWNetIsUpdateKrb5AffinityEnabled(
+    IN DWORD DsFlags,
+    IN PCSTR SiteName,
+    IN PLWNET_DC_INFO pDcInfo
+    )
+{
+    BOOLEAN enabled = FALSE;
 
-/*
-local variables:
-mode: c
-c-basic-offset: 4
-indent-tabs-mode: nil
-tab-width: 4
-end:
-*/
+    // Will only affinitize for KDC/LDAP (i.e., not PDC/GC), if the site
+    // is the same
+
+    if (!(DsFlags & (DS_PDC_REQUIRED | DS_GC_SERVER_REQUIRED)) &&
+        (IsNullOrEmptyString(SiteName) || 
+         (strcasecmp(SiteName, pDcInfo->pszClientSiteName) == 0)))
+    {
+        enabled = TRUE;
+    }
+
+    return enabled;
+}
