@@ -96,10 +96,10 @@ LWNetSrvIpcGetDomainController(
     );
 
 DWORD
-LWNetSrvIpcGetCurrentDomain(
+LWNetSrvIpcResolveName(
     LWMsgCall* pCall,
-    const LWMsgParams* pRequest,
-    LWMsgParams* pResponse,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
     void* data
     );
 
