@@ -87,11 +87,6 @@ main(
     dwError = LWNetStartupPreCheck();
     BAIL_ON_LWNET_ERROR(dwError);
 
-#ifdef ENABLE_PIDFILE
-    // ISSUE-2008/07/03-dalmeida -- Should return/check for errors
-    LWNetSrvCreatePIDFile();
-#endif
-
     dwError = LWNetBlockSelectedSignals();
     BAIL_ON_LWNET_ERROR(dwError);
 
@@ -635,97 +630,6 @@ LWNetSrvGetPrefixPath(
     goto cleanup;
 }
 
-VOID
-LWNetSrvCreatePIDFile(
-    VOID
-    )
-{
-    int result = -1;
-    pid_t pid;
-    char contents[PID_FILE_CONTENTS_SIZE];
-    size_t len;
-    int fd = -1;
-
-    pid = LWNetSrvGetPidFromPidFile();
-    if (pid > 0) {
-        fprintf(stderr, "Daemon already running as %d\n", (int) pid);
-        result = -1;
-        goto error;
-    }
-
-    fd = open(PID_FILE, O_CREAT | O_WRONLY | O_EXCL, 0644);
-    if (fd < 0) {
-        fprintf(stderr, "Could not create pid file: %s\n", strerror(errno));
-        result = 1;
-        goto error;
-    }
-
-    pid = getpid();
-    snprintf(contents, sizeof(contents)-1, "%d\n", (int) pid);
-    contents[sizeof(contents)-1] = 0;
-    len = strlen(contents);
-
-    result = (int) write(fd, contents, len);
-    if ( result != (int) len ) {
-        fprintf(stderr, "Could not write to pid file: %s\n", strerror(errno));
-        result = -1;
-        goto error;
-    }
-
-    result = 0;
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    if (result < 0) {
-        exit(1);
-    }
-}
-
-pid_t
-LWNetSrvGetPidFromPidFile(
-    VOID
-    )
-{
-    pid_t pid = 0;
-    int fd = -1;
-    int result;
-    char contents[PID_FILE_CONTENTS_SIZE];
-
-    fd = open(PID_FILE, O_RDONLY, 0644);
-    if (fd < 0) {
-        goto error;
-    }
-
-    result = read(fd, contents, sizeof(contents)-1);
-    if (result <= 0) {
-        goto error;
-    }
-    contents[result-1] = 0;
-
-    result = atoi(contents);
-    if (result <= 0) {
-        result = -1;
-        goto error;
-    }
-
-    pid = (pid_t) result;
-    result = kill(pid, 0);
-    if (result != 0 || errno == ESRCH) {
-        unlink(PID_FILE);
-        pid = 0;
-    }
-
- error:
-    if (fd != -1) {
-        close(fd);
-    }
-
-    return pid;
-}
-
 DWORD
 LWNetSrvInitLogging(
     PCSTR pszProgramName
