@@ -856,7 +856,11 @@ LWNetDnsBuildSRVRecordList(
                                          &pSRVRecord);
         if (dwError)
         {
-            LWNET_LOG_ERROR("Failed to build SRV record information");
+            // Already logged on ERROR_NOT_FOUND
+            if (dwError != ERROR_NOT_FOUND)
+            {
+                LWNET_LOG_ERROR("Failed to build SRV record information");
+            }
             // Skip
             dwError = 0;
         }
@@ -994,6 +998,7 @@ LWNetDnsGetAddressForServer(
 
     if (IsNullOrEmptyString(pszAddress))
     {
+        LWNET_LOG_WARNING("Unable to get IP address for '%s'", pszHostname);
         dwError = ERROR_NOT_FOUND;
         BAIL_ON_LWNET_ERROR(dwError);
     }
@@ -1182,7 +1187,7 @@ LWNetDnsQueryWithBuffer(
         dwError = ERROR_NOT_FOUND;
         BAIL_ON_LWNET_ERROR(dwError);
     }
-
+    
     if (dwBufferSize < CT_MIN(sizeof(DNS_RESPONSE_HEADER), MAX_DNS_UDP_BUFFER))
     {
         dwError = ERROR_INVALID_PARAMETER;
@@ -1208,7 +1213,7 @@ LWNetDnsQueryWithBuffer(
 #endif
     if (responseSize < 0)
     {
-        LWNET_LOG_ERROR("DNS lookup for '%s' failed with errno %d, h_errno = %d", pszQuestion, errno, h_errno);
+        LWNET_LOG_VERBOSE("DNS lookup for '%s' failed with errno %d, h_errno = %d", pszQuestion, errno, h_errno);
         dwError = DNS_ERROR_BAD_PACKET;
         BAIL_ON_LWNET_ERROR(dwError);
     }
