@@ -1,7 +1,7 @@
-/* $OpenLDAP$ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/gssapi.c,v 1.1.2.3 2009/02/17 21:02:51 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2009 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Author: Stefan Metzmacher <metze@sernet.de>
@@ -72,11 +72,11 @@ gsserrstr(
 			   mech, &msg_ctx, &minor_msg);
 
 	snprintf(buf, buf_len, "gss_rc[%d:%*s] mech[%*s] minor[%u:%*s]",
-		 gss_rc, gss_msg.length,
+		 gss_rc, (int)gss_msg.length,
 		 (const char *)(gss_msg.value?gss_msg.value:""),
-		 mech_msg.length,
+		 (int)mech_msg.length,
 		 (const char *)(mech_msg.value?mech_msg.value:""),
-		 minor_status, minor_msg.length,
+		 minor_status, (int)minor_msg.length,
 		 (const char *)(minor_msg.value?minor_msg.value:""));
 
 	gss_release_buffer(&min2, &mech_msg);
@@ -199,6 +199,7 @@ sb_sasl_gssapi_encode(
 	if ( conf_req_flag && conf_state == 0 ) {
 		ber_log_printf( LDAP_DEBUG_ANY, p->sbiod->sbiod_sb->sb_debug,
 				"sb_sasl_gssapi_encode: GSS_C_CONF_FLAG was ignored by our gss_wrap()\n" );
+		gss_release_buffer(&minor_status, &wrapped);
 		return -1;
 	}
 
@@ -209,8 +210,9 @@ sb_sasl_gssapi_encode(
 		ber_pvt_sb_grow_buffer( dst, pkt_len ) < 0 )
 	{
 		ber_log_printf( LDAP_DEBUG_ANY, p->sbiod->sbiod_sb->sb_debug,
-				"sb_sasl_gssapi_encode: failed to grow the buffer to %u bytes\n",
+				"sb_sasl_gssapi_encode: failed to grow the buffer to %lu bytes\n",
 				pkt_len );
+		gss_release_buffer(&minor_status, &wrapped);
 		return -1;
 	}
 
@@ -240,13 +242,14 @@ sb_sasl_gssapi_decode(
 	gss_ctx_id_t gss_ctx = (gss_ctx_id_t)p->ops_private;
 	int gss_rc;
 	OM_uint32 minor_status;
-	gss_buffer_desc unwrapped, wrapped;
+	gss_buffer_desc unwrapped = { 0 }, wrapped;
 	gss_OID ctx_mech = GSS_C_NO_OID;
 	OM_uint32 ctx_flags = 0;
 	int conf_req_flag = 0;
 	int conf_state;
 	unsigned char *b;
 	ber_len_t pkt_len;
+	ber_int_t result = 0;
 
 	wrapped.value	= src->buf_base + 4;
 	wrapped.length	= src->buf_end - 4;
@@ -273,13 +276,15 @@ sb_sasl_gssapi_decode(
 		ber_log_printf( LDAP_DEBUG_ANY, p->sbiod->sbiod_sb->sb_debug,
 				"sb_sasl_gssapi_decode: failed to decode packet: %s\n",
 				gsserrstr( msg, sizeof(msg), ctx_mech, gss_rc, minor_status ) );
-		return -1;
+		result = -1;
+		goto cleanup;
 	}
 
 	if ( conf_req_flag && conf_state == 0 ) {
 		ber_log_printf( LDAP_DEBUG_ANY, p->sbiod->sbiod_sb->sb_debug,
 				"sb_sasl_gssapi_encode: GSS_C_CONF_FLAG was ignored by our peer\n" );
-		return -1;
+		result = -1;
+		goto cleanup;
 	}
 
 	/* Grow the packet buffer if neccessary */
@@ -287,9 +292,10 @@ sb_sasl_gssapi_decode(
 		ber_pvt_sb_grow_buffer( dst, unwrapped.length ) < 0 )
 	{
 		ber_log_printf( LDAP_DEBUG_ANY, p->sbiod->sbiod_sb->sb_debug,
-				"sb_sasl_gssapi_decode: failed to grow the buffer to %u bytes\n",
+				"sb_sasl_gssapi_decode: failed to grow the buffer to %lu bytes\n",
 				pkt_len );
-		return -1;
+		result = -1;
+		goto cleanup;
 	}
 
 	dst->buf_end = unwrapped.length;
@@ -299,9 +305,9 @@ sb_sasl_gssapi_decode(
 	/* copy the wrapped blob to the right location */
 	memcpy(b, unwrapped.value, unwrapped.length);
 
+cleanup:
 	gss_release_buffer(&minor_status, &unwrapped);
-
-	return 0;
+	return result;
 }
 
 static void
@@ -351,8 +357,6 @@ map_gsserr2ldap(
 	int gss_rc,
 	OM_uint32 minor_status )
 {
-	OM_uint32 min2;
-	OM_uint32 msg_ctx = 0;
 	char msg[256];
 
 	Debug( LDAP_DEBUG_ANY, "%s\n",
@@ -597,7 +601,7 @@ guess_service_principal(
 	}
 
 	ret = snprintf(svc_principal, svc_principal_size, principal_fmt, name);
-	if (ret < 0 || ret >= svc_principal_size) {
+	if (ret < 0 || (size_t)(ret+1) > svc_principal_size) {
 		ld->ld_errno = LDAP_LOCAL_ERROR;
 		return ld->ld_errno;
 	}
@@ -701,6 +705,7 @@ ldap_int_gss_spnego_bind_s( LDAP *ld )
 	gss_OID req_mech = GSS_C_NO_OID;
 	gss_OID ret_mech = GSS_C_NO_OID;
 	gss_ctx_id_t gss_ctx = GSS_C_NO_CONTEXT;
+	gss_cred_id_t gss_cred = GSS_C_NO_CREDENTIAL;
 	gss_name_t principal = GSS_C_NO_NAME;
 	OM_uint32 req_flags;
 	OM_uint32 ret_flags;
@@ -715,7 +720,7 @@ ldap_int_gss_spnego_bind_s( LDAP *ld )
 	rc = ldap_gssapi_get_rootdse_infos ( ld, &mechlist,
 					     &ldapServiceName, &dnsHostName);
 	if ( rc != LDAP_SUCCESS ) {
-		return rc;
+		goto rc_error;
 	}
 
 	/* check that the server supports GSS-SPNEGO */
@@ -753,8 +758,16 @@ ldap_int_gss_spnego_bind_s( LDAP *ld )
 	 */
 	input_token.value = NULL;
 	input_token.length = 0;
+
+	/*
+	 * Set credentials handle if it's been set (e.g. for gss-ntlm authentication)
+	 */
+	if (ld->ld_options.gssapi_cred_handle) {
+		gss_cred = (gss_cred_id_t)ld->ld_options.gssapi_cred_handle;
+	}
+
 	gss_rc = gss_init_sec_context(&minor_status,
-				      GSS_C_NO_CREDENTIAL,
+				      gss_cred,
 				      &gss_ctx,
 				      principal,
 				      req_mech,
@@ -791,7 +804,7 @@ ldap_int_gss_spnego_bind_s( LDAP *ld )
 		}
 
 		gss_rc = gss_init_sec_context(&minor_status,
-					      GSS_C_NO_CREDENTIAL,
+					      gss_cred,
 					      &gss_ctx,
 					      principal,
 					      req_mech,
@@ -943,6 +956,14 @@ ldap_int_gssapi_get_option( LDAP *ld, int option, void *arg )
 		}
 		break;
 
+	case LDAP_OPT_X_GSSAPI_CREDENTIAL_HANDLE:
+		if ( ld->ld_options.gssapi_cred_handle ) {
+			* (void**)arg = ld->ld_options.gssapi_cred_handle;
+		} else {
+			* (void**)arg = GSS_C_NO_CREDENTIAL;
+		}
+		break;
+
 	case LDAP_OPT_X_GSSAPI_DO_NOT_FREE_CONTEXT:
 		if ( ld->ld_options.ldo_gssapi_options & LDAP_GSSAPI_OPT_DO_NOT_FREE_GSS_CONTEXT ) {
 			* (int *) arg = (int)-1;
@@ -1008,6 +1029,12 @@ ldap_int_gssapi_set_option( LDAP *ld, int option, void *arg )
 		}
 		break;
 
+	case LDAP_OPT_X_GSSAPI_CREDENTIAL_HANDLE:
+		if ( arg != LDAP_OPT_OFF) {
+			ld->ld_options.gssapi_cred_handle = arg;
+		}
+		break;
+
 	case LDAP_OPT_X_GSSAPI_DO_NOT_FREE_CONTEXT:
 		if ( arg != LDAP_OPT_OFF ) {
 			ld->ld_options.ldo_gssapi_options |= LDAP_GSSAPI_OPT_DO_NOT_FREE_GSS_CONTEXT;
@@ -1036,7 +1063,9 @@ ldap_gssapi_bind(
 	LDAP *ld,
 	LDAP_CONST char *dn,
 	LDAP_CONST char *creds )
-{ return LDAP_NOT_SUPPORTED; }
+{
+	return LDAP_NOT_SUPPORTED;
+}
 
 int
 ldap_gssapi_bind_s(
